package ACL;

use strict;
use warnings;

#use Data::Dumper;

my %acl;
my %cfg;
my @rawacl;
my @cfggrp;    # array of groups from acl.cfg
my @groups;
my @grplist;

my $basedir = $ENV{INPUTDIR};
$basedir ||= "..";

my $cfgdir = "$basedir/etc/web_config";
if ( defined $ENV{TMPDIR_LPAR} ) {
  my $tmpdir = $ENV{TMPDIR_LPAR};
}

my $aclAdminGroup = $ENV{ACL_ADMIN_GROUP} ||= "admins";
my $aclGrpListVar = $ENV{ACL_GRPLIST_VARNAME};

if ( !defined $aclGrpListVar || $aclGrpListVar eq "" ) {
  $aclGrpListVar = 0;
}

my $useacl = 0;

use lib "../bin";
use Users;

# print STDERR "ACL: $useacl\n";

%cfg = Users::getConfig();

# $user ||= "tester";

#$gids = "CN=lpar2rrd-admins,CN=Users,DC=xorux,DC=com; CN=lpar2rrd-marketing,CN=Users,DC=xorux,DC=com";
sub init {
  my $uid = shift;
  if ( $uid ) {
    $ENV{XORUX_ACCESS_CONTROL} = 1;
  } else {
    $uid = $ENV{REMOTE_USER};
  }
  if ( $aclGrpListVar && defined $ENV{$aclGrpListVar} && $uid ) {
    $useacl = 1;
  }
  elsif ( !$ENV{DISABLE_ACL} && $ENV{XORUX_ACCESS_CONTROL} && $uid ) {
    $useacl = 2;
  }
  if ( $useacl == 2 ) {
    if ( $cfg{users}{$uid}{groups} ) {
      @grplist = @{ $cfg{users}{$uid}{groups} };
    }

  }
  elsif ( $useacl == 1 ) {
    my $gids = $ENV{$aclGrpListVar};
    if ( $gids =~ "=" ) {
      @groups = split( /;/, $gids );
      chomp @groups;
      foreach my $group (@groups) {
        if ($group) {
          my $grpcn = ( split( /[=,]/, $group ) )[1];
          push @grplist, $grpcn;
        }
      } ## end foreach my $group (@groups)
      if ( $ENV{AUTHENTICATE_PRIMARYGROUPID} && $ENV{AUTHENTICATE_PRIMARYGROUPID} == 513 ) {  # add primary group if exists (Domain Users)
        push @grplist, "Domain Users";
      }
    }
    elsif ( $gids =~ "|" ) {
      @groups = split( /\|/, $gids );
      chomp @groups;
      foreach my $group (@groups) {
        if ($group) {

          #	my $grpcn = (split(/\//, $group))[-1];
          push @grplist, $group;
        }
      } ## end foreach my $group (@groups)
    } ## end elsif ( $gids =~ "|" )
  } ## end elsif ( $useacl == 1 )
}

sub getAdminGroup {
  my $username = shift;
  init($username);
  return $aclAdminGroup;
}

sub getGroups {
  my $username = shift;
  init($username);
  return @grplist;
}

sub getGroupsHtml {
  my $username = shift;
  init($username);

  #my $list = join "],[", @grplist;
  #return "[" . $list . "]";
  my $list = "<b>Group membership</b></br>" . join "</br>", sort @grplist;
  return $list;
} ## end sub getGroupsHtml


sub getCfgGroups {
  my $username = shift;
  init($username);
  return @cfggrp;
}

sub getAcl {
  my $username = shift;
  init($username);
  return \%acl;
}

sub getRawAcl {
  my $username = shift;
  init($username);
  return @rawacl;
}

sub getSections {
  my $username = shift;
  init($username);
  my @sections = qw/power vmware solo cgroup/;
  my %ret;
  foreach my $sec (@sections) {
    $ret{$sec} = 0;
  }

  foreach my $group (@grplist) {
    foreach my $sec (@sections) {
      if ( $cfg{groups}{$group}{ACL}{sections}{$sec} ) {
        $ret{$sec} = 1;
      }
    }
  }
  return %ret;
}

sub getServersOld {
  my $username = shift;
  init($username);
  my @list;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{servers} ) {
      foreach ( @{ $cfg{groups}{$group}{ACL}{servers} } ) {
        if ( $_ eq "*" ) {
          @list = ('*');
          return @list;
        }
        push @list, $_;
      } ## end foreach ( @{ $cfg{groups}{$group...}})
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)
  return uniq(@list);
} ## end sub getServersOld

sub getServers {
  my $username = shift;
  init($username);
  my @list;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{lpars} ) {
      while ( my ( $srv, $value ) = each( %{ $cfg{groups}{$group}{ACL}{lpars} } ) ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{power} || $srv eq "*" ) {
          @list = ('*');
          return @list;
        }
        foreach my $lpar ( @{$value} ) {
          if ( $lpar eq "*" ) {
            push @list, $srv;
          }
        }
      } ## end while ( my ( $srv, $value...))
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)

  #print STDERR "SERVERS: ". uniq(@list) . "\n";
  return uniq(@list);
} ## end sub getServers

sub getLpars {
  my $username = shift;
  init($username);
  my %hash;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{lpars} ) {
      while ( my ( $key, $value ) = each( %{ $cfg{groups}{$group}{ACL}{lpars} } ) ) {
        foreach my $lpar ( @{$value} ) {
          $hash{$key}{$lpar} = 1;
        }
      }
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)

  return %hash;
} ## end sub getLpars

sub getPools {
  my $username = shift;
  init($username);
  my %hash;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{pools} ) {
      while ( my ( $key, $value ) = each( %{ $cfg{groups}{$group}{ACL}{pools} } ) ) {
        foreach my $lpar ( @{$value} ) {
          $hash{$key}{$lpar} = 1;
        }
      }
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)

  return %hash;
} ## end sub getLpars

sub getCustoms {
  my $username = shift;
  init($username);
  my @custgrps;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{cgroups} ) {
      foreach ( @{ $cfg{groups}{$group}{ACL}{cgroups} } ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{cgroup} || $_ eq "*" ) {
          @custgrps = ('*');
          return @custgrps;
        }
        push @custgrps, $_;
      } ## end foreach ( @{ $cfg{groups}{$group...}})
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)
  return uniq(@custgrps);
} ## end sub getCustoms

sub isAdmin {
  my $username = shift;
  init($username);
  if ( grep /^$aclAdminGroup$/, @grplist ) {
    return 1;
  }
}

sub isReadOnly {
  my $username = shift;
  init($username);
  if ( grep /^ReadOnly$/, @grplist ) {
    return 1;
  }
}

sub canShow {
  # my ( $type, $cgrp, $srv, $lpar ) = @_;
  my ( $platform, $subsys, $source, $item, $username ) = @_;
  init($username);

  if (isReadOnly) {
    return 8; # maybe it will be usefull to know it's RO ACL, return specific value (8)
  }

  $source = urldecode($source);
  $item  = urldecode($item);

  foreach my $group (@grplist) {
    if ( $subsys eq "CUSTOM" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{cgroup} || ( $source
        && ( grep /^$source$|\*/, @{ $cfg{groups}{$group}{ACL}{cgroups} } ) ) )
      {
        return 3;
      }
      else {
        return 0;
      }
    }
    elsif ( $platform eq "POWER" ) {
      if ( $subsys eq "SERVER" ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{power} || ($source && $cfg{groups}{$group}{ACL}{lpars}{"*"} ) ) {
          return 2;
        } else {
          return 0;
        }
      }
      if ( $subsys eq "LPAR" ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{power} || ($source && $cfg{groups}{$group}{ACL}{lpars}{"*"} ) ) {
          return 2;
        }
        elsif ( exists $cfg{groups}{$group}{ACL}{lpars}{$source} &&
          ( grep( /^$item$/, @{ $cfg{groups}{$group}{ACL}{lpars}{$source} } ) || grep( /^\*$/, @{ $cfg{groups}{$group}{ACL}{lpars}{$source} } ) )
        )
        {
          return 2;
        }
        else {
          return 0;
        } ## end if ( exists $cfg{groups...})
      }
      elsif ( $subsys eq "POOL" ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{power} || ($source && $cfg{groups}{$group}{ACL}{pools}{"*"} ) ) {
          return 2;
        }
        elsif ( exists $cfg{groups}{$group}{ACL}{pools}{$source} &&
          ( grep( /^$item$/, @{ $cfg{groups}{$group}{ACL}{pools}{$source} } ) || grep( /^\*$/, @{ $cfg{groups}{$group}{ACL}{pools}{$source} } ) )
        )
        {
          return 2;
        }
        else {
          return 0;
        }
      } ## end if ( exists $cfg{groups...})
    }
    elsif ( $platform  eq "VMWARE" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{vmware} ) {
        return 2;
      }
      elsif( $subsys eq "VM" ) {
        if ( $source && $cfg{groups}{$group}{ACL}{vms}{"*"} ) {
          return 2;
        }
        elsif ( exists $cfg{groups}{$group}{ACL}{vms}{$source} &&
          ( grep( /^$item$/, @{ $cfg{groups}{$group}{ACL}{vms}{$source} } ) || grep( /^\*$/, @{ $cfg{groups}{$group}{ACL}{vms}{$source} } ) )
        )
        {
          return 2;
        }
      } ## end if ( exists $cfg{groups...})
      elsif ( $subsys eq "CLUSTER" ) {
        if ( $cfg{groups}{$group}{ACL}{vms}{$source} && grep( /^\*$/, @{ $cfg{groups}{$group}{ACL}{vms}{$source} } ) ) {
          return 2;
        }
      }
      elsif ( $subsys eq "ESXI" ) {
      }
      elsif ( $subsys eq "RESPOOL" ) {
      }
      elsif ( $subsys eq "DATASTORE" ) {
      }
      return 0;
    }
    elsif ( $platform eq "UNMANAGED" ) {
      if ( $subsys eq "SERVER" ) {
        if ( $cfg{groups}{$group}{ACL}{sections}{solo} || ( $source && $cfg{groups}{$group}{ACL}{solo}{"*"} ) ) {
          return 2;
        }
        elsif ( exists $cfg{groups}{$group}{ACL}{solo}{$source} &&
          ( grep( /^$item$/, @{ $cfg{groups}{$group}{ACL}{solo}{$source} } ) || grep( /^\*$/, @{ $cfg{groups}{$group}{ACL}{solo}{$source} } ) )
        )
        {
          return 2;
        }
        else {
          return 0;
        }
      } ## end if ( exists $cfg{groups...})
    } ## end elsif ( $type eq "U" )
    elsif ( $platform eq "OVIRT" ) {
      return 2;
    }
    elsif ( $platform eq "XENSERVER" ) {
      return 2;
    }
  } ## end foreach my $group (@grplist)
  return 0;
} ## end sub canShow

sub getVMs {
  my $username = shift;
  init($username);
  my %hash;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{vmware} ) {
      while ( my ( $key, $value ) = each( %{ $cfg{groups}{$group}{ACL}{vmware} } ) ) {
        foreach my $lpar ( @{$value} ) {
          $hash{$key}{$lpar} = 1;
        }
      }
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)
  return %hash;
} ## end sub getVMs

sub getSolos {
  my $username = shift;
  init($username);
  my %hash;
  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{solo} ) {
      while ( my ( $key, $value ) = each( %{ $cfg{groups}{$group}{ACL}{solo} } ) ) {
        foreach my $lpar ( @{$value} ) {
          $hash{$key}{$lpar} = 1;
        }
      }
    } ## end if ( exists $cfg{groups...})
  } ## end foreach my $group (@grplist)
  return %hash;
} ## end sub getSolos

sub hasItems {
  my ( $type, $srv, $username ) = @_;
  init($username);

  if (isReadOnly) {
    return 8; # maybe it will be usefull to know it's RO ACL, return specific value (8)
  }

  # print STDERR "PARS: $cgrp : $srv : $lpar\n";

  foreach my $group (@grplist) {
    if ( $type eq "P" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{power}
        || exists $cfg{groups}{$group}{ACL}{lpars}{"*"}
        || exists $cfg{groups}{$group}{ACL}{lpars}{$srv} )
      {
        return 2;
      }
    }
    if ( $type eq "O" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{power}
        || exists $cfg{groups}{$group}{ACL}{pools}{"*"}
        || exists $cfg{groups}{$group}{ACL}{pools}{$srv} )
      {
        return 2;
      }
    }
    elsif ( $type eq "V" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{vmware}
        || exists $cfg{groups}{$group}{ACL}{vms}{"*"}
        || exists $cfg{groups}{$group}{ACL}{vms}{$srv} )
      {
        return 2;
      }
    }
    elsif ( $type eq "U" ) {
      if ( $cfg{groups}{$group}{ACL}{sections}{solo}
        || exists $cfg{groups}{$group}{ACL}{solo}{"*"}
        || exists $cfg{groups}{$group}{ACL}{solo}{$srv} )
      {
        return 2;
      }
    }
  } ## end foreach my $group (@grplist)
  return 0;
} ## end sub hasItems

sub canShowOld {
  my ( undef, $cgrp, $srv, $lpar ) = @_;

  foreach my $group (@grplist) {
    if ( exists $cfg{groups}{$group}{ACL}{lpars}{$srv} ) {
      if ( grep( /^$lpar$/, @{ $cfg{groups}{$group}{ACL}{lpars}{$srv} } ) ) {
        return 1;
      }
    }
    elsif ( $srv
      && ( grep /^$srv$|\*/, @{ $cfg{groups}{$group}{ACL}{servers} } ) )
    {
      return 2;
    }
    elsif ( $cgrp
      && ( grep /^$cgrp$|\*/, @{ $cfg{groups}{$group}{ACL}{cgroups} } ) )
    {
      return 3;
    }
  } ## end foreach my $group (@grplist)
  return 0;
} ## end sub canShowOld

sub useACL {
  my $username = shift;
  init($username);
  return $useacl;
}

sub collons {
  return s/===double-col===/:/g;
}

sub pipes {
  return s/===pipe===/\|/g;
}

sub uniq {
  my %seen;
  return grep { !$seen{$_}++ } @_;
}

sub urlencode {
  my $s = shift;

  # $s =~ s/ /+/g;
  $s =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

  # $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  if ($s) {
    $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  }
  return $s;
} ## end sub urldecode

1;
