package HostCfg;

# LPAR2RRD user management module
# vim: autoindent tabstop=2 shiftwidth=2 expandtab softtabstop=2 filetype=perl
use strict;
use warnings;
use POSIX;
use Data::Dumper;
use MIME::Base64 qw( decode_base64 encode_base64 );

my $basedir = $ENV{INPUTDIR};
$basedir ||= "..";

my $cfgdir     = "$basedir/etc/web_config";
my $realcfgdir = "$basedir/etc";

use JSON qw(decode_json encode_json);

my $json = JSON->new->utf8->pretty;

my %alerts;       # hash holding alerting configuration
my %emailgrps;    # hash holding e-mail groups
my %config;       # hash holding configuration
my $rawcfg;
my @rawcfg;
my $oldcfg    = 0;
my $now       = time();
my $isotime   = strftime( '%Y-%m-%dT%H:%M:%SZ', gmtime($now) );
my $URL       = "";
my $isimported  = 0;
my $vmwlibdir = "$basedir/vmware-lib/apps";
my $credstore  = "$basedir/.vmware/credstore/vicredentials.xml";
my $vmwarelist = $ENV{VMWARE_LIST}      ||= "";
my $perl       = $ENV{PERL};
#if ($ENV{REQUEST_SCHEME} . $ENV{SERVER_NAME} . $ENV{SERVER_PORT}) {
#  $URL = "$ENV{REQUEST_SCHEME}://$ENV{SERVER_NAME}";
#}

#if ($ENV{VM_IMAGE}) {
#  $adminName = "monitor";
#  $adminPass = "\$apr1\$UZuWgWzB\$gk6cSafcM9F2Bl0Jl94ZB.";
#}

sub readCfg {
  if ( !open( CFG, "<", "$cfgdir/hosts.json" ) ) {
    if ( !open( CFG, ">", "$cfgdir/hosts.json" ) ) {
      die "Cannot open file:$cfgdir/hosts.json $!\n";
    }
    else {    # create empty cfg file
      # prepare Power hosts from etc/lpar2rrd.cfg
      use IO::Socket::INET;
      my %imported;
      my $hmclist = $ENV{HMC_LIST}      ||= "";
      if ( -e $hmclist && -f _ && -r _ ) {
        $hmclist = `cat $hmclist`;
      } elsif ( -e "$basedir/etc/$hmclist" && -f _ && -r _ ) {
        $hmclist = `cat $basedir/etc/$hmclist`;
      }
      my $hmcuser = $ENV{HMC_USER}      ||= "lpar2rrd";
      my $sshkey  = $ENV{SSH} ||= "";
      if ( $hmclist && $hmclist ne "hmc1 sdmc1 ivm1" ) {
        foreach my $hmc ( split " ", $hmclist ) {
          my $sock = IO::Socket::INET->new(
            PeerAddr => $hmc,
            PeerPort => 22,
            Proto    => 'tcp',
            Timeout  => 5
          );

          if ($sock) {
            %{ $imported{$hmc} } =
            ( "proto" => "https",
              "username" => "$hmcuser",
              "host" => "$hmc",
              "ssh_port" => "22",
              "password" => "",
              "auth_ssh" => \1,
              "api_port" => "12443",
              "auth_api" => \0,
              "ssh_key_id" => "$sshkey"
            );

            close($sock);
            $isimported = 1;
          } ## end if ($sock)
        }
      }
      my $impstringpower = JSON->new->utf8(0)->pretty(1)->encode( \%imported );

      my $impstringvmware = "{}";
      if ( -e "$vmwlibdir/credstore_admin.pl" ) {
        $impstringvmware = JSON->new->utf8(0)->pretty(1)->encode( importVmware() );
      }

      print CFG <<_MARKER_;
{
   "platforms" : {
      "IBM Power Systems": {
         "order" : 1,
         "ssh" : false,
         "api" : true,
         "aliases" : $impstringpower
      },
      "VMware": {
         "order" : 2,
         "ssh" : false,
         "api" : true,
         "aliases" : $impstringvmware
      },
      "XenServer": {
         "order" : 3,
         "ssh" : true,
         "api" : true,
         "aliases" : {}
      },
      "Hyper-V": {
         "order" : 4,
         "ssh" : false,
         "api" : true,
         "aliases" : {}
      },
      "KVM": {
         "order" : 5,
         "ssh" : true,
         "api" : false,
         "aliases" : {}
      },
      "RHV (oVirt)": {
         "order" : 6,
         "ssh" : false,
         "api" : true,
         "aliases" : {}
      }
   }
}
_MARKER_

      close CFG;
      open( CFG, "<", "$cfgdir/hosts.json" );

    }
  }
  {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
    if ($rawcfg) {
      %config = %{ decode_json($rawcfg) };
      if ( ( keys %{ $config{platforms}{VMware}{aliases} } < 1 ) && $vmwarelist ) {
        $config{platforms}{VMware}{aliases} = importVmware();
        close CFG;
        open( CFG, ">", "$cfgdir/hosts.json" );
        print CFG JSON->new->utf8(0)->pretty(1)->encode( \%config );
        close CFG;
        open( CFG, "<", "$cfgdir/hosts.json" );
        $rawcfg = <CFG>;
      }
    }
    close CFG;
  }
  chmod 0666,"$cfgdir/hosts.json";
  # print Dumper \%config;
} ## end sub readCfg

sub getGroups {

  # return sorted array of defined groups
  &readCfg;
  return sort keys %emailgrps;
} ## end sub getGroups

sub getConfig {

  # return configuration hash
  &readCfg;
  return %config;
} ## end sub getConfig

sub getRawConfig {
  &readCfg;
  return ($rawcfg, $isimported);
}

sub getGroupMembers {

  # param: group_name
  # return sorted array of defined alerts
  my $groupName = shift;
  return $emailgrps{$groupName};
} ## end sub getGroupMembers

sub getHostConnections {

  # params: platformName
  # return: hash of hosts connections
  my $platform = shift;
  &readCfg;
  my %hostConns;
  if ($config{aliases}) {
    foreach my $key ( keys %{ $config{aliases} } ) {
      my $host = $config{aliases}{$key};
      if ( ( !$platform ) || ( $host->{platform} eq $platform ) ) {
        $hostConns{$key} = $host;
        if ( $hostConns{$key}{password} ) {
          $hostConns{$key}{password} = unobscure_password($hostConns{$key}{password});
        }
      }
    }
  } else {
    if ( $config{platforms}{$platform}{aliases} ) {
      %hostConns = %{ $config{platforms}{$platform}{aliases} };
      foreach my $key ( keys %hostConns ) {
        if ( $hostConns{$key}{password} ) {
          $hostConns{$key}{password} = unobscure_password($hostConns{$key}{password});
        }
      }
    }
  }
  if ( $platform eq "VMware" ) {
    my @items;
    foreach my $alias ( sort keys %hostConns ) {
      push @items, "$alias|$hostConns{$alias}{host}|$hostConns{$alias}{username}";
    }
    return join( " ", @items );
  }
  else {
    return \%hostConns;
  }
} ## end sub getFullName

sub getSSHKeys {
  my @keys;
  if ($ENV{VM_IMAGE}) {
    my @pubkeys = glob "/home/lpar2rrd/.ssh/*.pub";
    foreach my $pub (@pubkeys) {
      $pub =~ s{\.[^.]+$}{};
      if ( -f $pub ) {
        push @keys, $pub;
      }
    }
  }
  return @keys;
} ## end sub getEmail

sub getEmail {

  # params: group_name, group_member
  # return full name of a member
  my ( $groupName, $groupMem ) = @_;
  return $alerts{groups}{$groupName}{$groupMem}{email};
} ## end sub getEmail

sub getUserDetails {

  # params: group_name, group_member
  # return hash of member details
  my ( $groupName, $groupMem ) = @_;
  return $alerts{groups}{$groupName}{$groupMem};
} ## end sub getUserDetails

sub unobscure_password {
  my $string    = shift;
  my $unobscure = decode_base64($string);
  $unobscure = unpack( chr( ord("a") + 19 + print "" ), $unobscure );
  return $unobscure;
} ## end sub unobscure_password

sub obscure_password {
  my $string = shift;
  my $obscure = encode_base64( pack( "u", $string ), "" );
  return $obscure;
}

# prepare VMware hosts from etc/webconfig/vmware.cfg
sub importVmware {
  my %imported = ();
  if ( $vmwarelist ) {
    my @vhosts = split ( " ", $vmwarelist );
    foreach my $vcfg ( @vhosts ) {
      my ( $alias, $host, $username ) = split( /\|/, $vcfg );
      my $passresp = `$perl $basedir/vmware-lib/apps/credstore_admin.pl get --server '$host' --username '$username' 2>&1`;
      if ( $passresp =~ /^Password: .*/ ) {
        my (undef, $password) = split ( " ", $passresp );
        %{ $imported{$alias} } =
        ( "proto" => "https",
          "username" => "$username",
          "host" => "$host",
          "password" => obscure_password($password),
          "auth_ssh" => \0,
          "api_port" => "443",
          "auth_api" => \1,
        );
      }
    }
  }
  return ( \%imported );
}

1;
