package LoadDataModuleVMWare;    #
use strict;
use Date::Parse;
use RRDp;
use File::Copy;
use Xorux_lib;
use Math::BigInt;

my $rrdtool = $ENV{RRDTOOL};

# standard data retentions
my $one_minute_sample     = 86400;
my $five_mins_sample      = 25920;
my $one_hour_sample       = 4320;
my $five_hours_sample     = 1734;
my $one_day_sample        = 1080;
my $one_minute_sample_mem = 86400;
my $five_mins_sample_mem  = 25920;
my $one_hour_sample_mem   = 4320;
my $five_hours_sample_mem = 1734;
my $one_day_sample_mem    = 1080;

my $JUMP_DETECT = 900;

my $rename_once = 0;

my $et_VirtualMachine         = "VirtualMachine";
my $et_HostSystem             = "HostSystem";
my $et_Datastore              = "Datastore";
my $et_ResourcePool           = "ResourcePool";
my $et_Datacenter             = "Datacenter";
my $et_ClusterComputeResource = "ClusterComputeResource";

my $all_vmware_VMs = "vmware_VMs";
my $no_inserted    = 66;

sub load_data {    # from VMWARE
  my ( $managedname, $host, $wrkdir, $input, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, $trans_tmp, $last_file, $no_time, $vm_uuid, $hmc_user, $lpar_uuid, $entity_type, $frk ) = @_;
  my @lpar_trans  = @{$trans_tmp};
  my $counter     = 0;
  my $counter_tot = 0;
  my $counter_ins = 0;
  my $last_rec    = "";
  my $rrd         = "";
  my $time        = "";
  my $t           = "";
  my $answer      = "";
  my $utime       = time() + 3600;
  $rename_once = 0;
  my $keep_virtual = 0;

  if ( $step > 60 ) {
    $JUMP_DETECT = 10800;    # 3 hours
  }
  if ( $type_sam =~ m/d/ ) {
    $JUMP_DETECT = 178000    # 2 days + for daily graphs/feed
  }

  # print "updating RRD   : $host:$managedname:data:$wrkdir/$managedname/$host/$lpar_uuid.rr$type_sam $frk\n" if $DEBUG;

  my @lines;
  my $update_line = $$input;

  ( $t, undef ) = split( ",", $update_line );    #first record time to update

  #  print "LoadDataModulVMWare \$t $t \$update_line $update_line\n";

  my @rrd_exist = "";

  my @lpar_name = "";
  my @lpar_time = "";
  my @lpar_jump = "";

  my $last_time  = "";
  my $error_once = 0;
  my $last_time_print = 0;
  my $line = $update_line;

  #print "$line";
  chomp($line);
  $time = "";
  my $lpar = $lpar_uuid;

  $rrd = "$wrkdir/$managedname/$host/$lpar.rr$type_sam";

  if ( $type_sam eq "v" ) {    # datastore/vm_uuid.rrv for VM IOPS in this datastore
    my $dir = "$wrkdir/$managedname/$host/$lpar";
    if ( !-d $dir ) {
      print "create dir     : datastore $dir\n";
      mkdir($dir) || main::error( "Cannot create dir  $dir : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    $rrd = "$dir/$vm_uuid.rr$type_sam";
  } ## end if ( $type_sam eq "v" )

  print "updating RRD   : $rrd $frk\n" if $DEBUG;

  #  create rrd db if necessary
  #  find out if rrd db exist, and place info to the array to do not check its existency every time
  my $rrd_exist_ok  = 0;
  my $rrd_exist_row = 0;
  foreach my $row (@rrd_exist) {
    $rrd_exist_row++;
    if ( $row =~ m/^$lpar$/ ) {
      $rrd_exist_ok = 1;
      last;
    }
  } ## end foreach my $row (@rrd_exist)
  if ( $rrd_exist_ok == 0 ) {

    #print " create_rrd_vmware_vm($rrd,$t,$counter_tot,$step,$type_sam,$DEBUG,$host,$managedname,$no_time,$act_time,$vm_uuid,$hmc_user,$wrkdir,$lpar,$t,$keep_virtual,$entity_type);\n";
    my $ret = create_rrd_vmware_vm( $rrd, $t, $counter_tot, $step, $type_sam, $DEBUG, $host, $managedname, $no_time, $act_time, $vm_uuid, $hmc_user, $wrkdir, $lpar, $t, $keep_virtual, $entity_type );
    if ( $ret == 1 ) {

      # this lpar has been recently renamed, ignore it then
      return 1;    # when vmware then no next;
    }
    if ( $ret == 2 ) {
      return 1;    # RRD creation problem, skip whole load
    }
    $rrd_exist[$rrd_exist_row] = $lpar;
  } ## end if ( $rrd_exist_ok == ...)

  #  print "find out last record and save it\n";
  # it has to be done due to better stability, when updating older record than the latest one update and process crashes
  my $l_count = 0;
  my $found   = -1;

  my $ltime;
  my $lpar_meta = quotemeta($lpar);
  foreach my $l (@lpar_name) {
    if ( $l =~ m/^$lpar_meta$/ ) {
      $found = $l_count;
      last;
    }
    $l_count++;
  } ## end foreach my $l (@lpar_name)
  if ( $found > -1 ) {
    $ltime = $lpar_time[$found];
    print "this is not expected for vmware \$ltime $ltime\n";
    $l_count = $found;
  }
  else {
    # find out last record in the db
    # as this makes it slowly to test it each time then it is done
    # once per a lpar for whole load and saved into the array

    # construction against crashing daemon Perl code when RRDTool error appears
    # this does not work well in old RRDTOool: $RRDp::error_mode = 'catch';
    # construction is not too costly as it runs once per each load
    eval {
      RRDp::cmd qq(last "$rrd" );
      $last_rec = RRDp::read;
    };
    if ($@) {
      rrd_error( $@, $rrd );
      if ( $rrd_exist[$rrd_exist_row] =~ m/^$lpar$/ ) {
        $rrd_exist[$rrd_exist_row] = "";
      }
      next;
    } ## end if ($@)
    chomp($$last_rec);
    $lpar_name[$l_count] = $lpar;
    $lpar_time[$l_count] = $$last_rec;
    $lpar_jump[$l_count] = 0;
    $ltime               = $$last_rec;

    #  print "last record from rrdfile $rrd \$ltime $ltime\n";
  } ## end else [ if ( $found > -1 ) ]

  my $max_cycles = 0;
  if ( defined $update_line ) {
    $max_cycles = $update_line =~ tr/ //;
    $max_cycles++;    # cycle must go at least one
  }

  # print "\$update_line ,$update_line,\n";
  # $max_cycles prevents infinite the cycle
  while ( defined $t && defined $ltime && $t <= $ltime && $max_cycles > 0 ) {    # try to skip one data piece
    my $skipped = "";
    if ( defined $update_line && $update_line ne "" ) {
      ( $skipped, $update_line ) = split( " ", $update_line, 2 );
    }
    if ( defined $update_line && $update_line ne "" ) {
      ( $t, undef ) = split( ",", $update_line );
    }
    if ( !defined $t || $t eq "" ) {
      $t = $ltime;
    }
    print "data skipped   : $skipped\n";
    $max_cycles--;
  } ## end while ( defined $t && defined...)

  if ( defined $t ) {
    # print "               : \$t $t > \$ltime $ltime (first data time to update > last rec) VMWare $frk\n";
  }

  if ( defined $t && defined $ltime && $t > $ltime ) {

    #  print "$lpar: $time $t $update_line \n";
    $update_line =~ s/ $//g;
    my $result = rindex( $update_line, " " );
    my $fragment = substr $update_line, $result + 1;    #get time of last record in update line
    ( $last_time, undef ) = split( ",", $fragment );
    ( my $sec, my $min, my $hour, my $mday, my $mon, my $year ) = localtime($last_time);
    $mon++;
    $year += 1900;
    $last_time = "$mon/$mday/$year $hour:$min:$sec";    # change day <-> mon for US date format
    $last_time_print = "$hour:$min:$sec"; # time is enough as it is never older one hour
    $update_line =~ s/,/:/g;

    RRDp::cmd qq(update "$rrd" $update_line);

    if ( $counter_ins == 0 ) {
      $counter_ins++;                                   # just to force update last.txt
    }
    $answer = RRDp::read;
    if ( !$$answer eq '' && $$answer =~ m/ERROR/ ) {
      main::error(" $host:$managedname : $rrd : $t: : $line : $$answer");
      if ( $$answer =~ m/is not an RRD file/ ) {
        ( my $err, my $file, my $txt ) = split( /'/, $$answer );
        main::error("Removing as it seems to be corrupted: $file");
        unlink("$file") || main::error( "Cannot rm $file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
    } ## end if ( !$$answer eq '' &&...)
  } ## end if ( defined $t && defined...)

  if ( $counter_ins > 0 ) {
    print "inserted       : $host:$managedname:lpar $counter_ins record(s) $frk\n" if $DEBUG;
  }
  else {
    return $no_inserted;    # no success
  }

  # write down timestamp of last record
  if ( $type_sam !~ "d" && $counter_ins > 0 ) {
    my $line = 0;
    if ( -f "$wrkdir/$managedname/$host/$last_file" ) {
      open( FHLT, "< $wrkdir/$managedname/$host/$last_file" ) || main::error( " Can't open $wrkdir/$managedname/$host/$last_file : $!" . __FILE__ . ":" . __LINE__ );
      $line = <FHLT>;
      close(FHLT);
    }
    if ( !defined $line || index($line,"/")<0 ) {    # for sure
      $line = 0;
    }
    chomp($line);
    my $old_time = str2time($line);
    my $new_time = str2time($last_time);
    # print "upd last_file  : $host:$managedname $last_time : $wrkdir/$managedname/$host/$last_file $new_time instead of $old_time $frk\n" if $DEBUG;

    # update only when it is newer ! e.g. datastore has 2 rrd files and so on
    if ( $new_time > $old_time && $last_file ne "" ) {
      print "upd last_file  : $host: $last_time_print : $wrkdir/$managedname/$host/$last_file $new_time instead of $old_time $frk\n" if $DEBUG;
      open( FHLT, "> $wrkdir/$managedname/$host/$last_file" ) || main::error( " Can't open $wrkdir/$managedname/$host/$last_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      print FHLT "$last_time";
      close(FHLT);
    }
  } ## end if ( $type_sam !~ "d" ...)

  return 99;    # symbol of success
} ## end sub load_data

sub create_rrd_vmware_vm {
  my $rrd          = shift;
  my $start_time   = shift;
  my $counter_tot  = shift;
  my $step         = shift;
  my $type_sam     = shift;
  my $DEBUG        = shift;
  my $host         = shift;
  my $managedname  = shift;
  my $no_time      = shift;
  my $act_time     = shift;
  my $vm_uuid      = shift;      # for VM rrd under datastore for IOPS
  my $hmc_user     = shift;
  my $wrkdir       = shift;
  my $lpar         = shift;
  my $time_rec     = shift;
  my $keep_virtual = shift;
  my $entity_type  = shift;
  my $step_new     = $step;
  my $no_time_new  = $no_time;

  #  $start_time = 1430000000; # something old enough, do not use  passed $start_time!!!!
  $start_time = time - ( 24 * 3600 );    # for vmware good

  my $ret = recent_rename( $host, $managedname, $lpar, $wrkdir, $DEBUG, $time_rec, $act_time );
  if ( $ret == 1 ) {

    # this lpar has been recently renamed, do not create rrdtool files
    return 1;
  }

  if ( -f "$rrd" ) {
    if ( -s "$rrd" == 0 ) {              # it was only signal file with no data
      `rm -f "$rrd"`;
    }
    else {
      return 0;
    }
  } ## end if ( -f "$rrd" )
  load_retentions( $step, $act_time );    # load data retentions
                                          # print "0000 $one_minute_sample - $five_mins_sample - $one_hour_sample - $five_hours_sample - $one_day_sample\n";
  print "create_rrd_vmware_vm: $host:$managedname $rrd ; STEP=$step_new \n" if $DEBUG;
  touch("create_rrd_vmware_vm $rrd");

  # print "++ $rrd $start_time $step_new - $no_time_new \n";

  if ( $entity_type eq $et_ClusterComputeResource ) {
    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:CPU_usage_MHz:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage_Proc:GAUGE:$no_time_new:0:U"
        "DS:CPU_reserved_MHz:GAUGE:$no_time_new:0:U"
        "DS:CPU_total_MHz:GAUGE:$no_time_new:0:U"
        "DS:Cluster_eff_CPU_MHz:GAUGE:$no_time_new:0:U"
        "DS:Cluster_eff_mem_MB:GAUGE:$no_time_new:0:U"
        "DS:Memory_total_MB:GAUGE:$no_time_new:0:U"
        "DS:Memory_shared_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_zero_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_baloon_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_consumed_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_overhead_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_active_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_granted_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_compress_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_reserved_MB:GAUGE:$no_time_new:0:U"
        "DS:Memory_swap_KB:GAUGE:$no_time_new:0:U"
        "DS:Mem_cmprssion_KBps:GAUGE:$no_time_new:0:U"
        "DS:Mem_dcompress_KBps:GAUGE:$no_time_new:0:U"
        "DS:Memory_usage_Proc:GAUGE:$no_time_new:0:U"
        "DS:Power_cup_Watt:GAUGE:$no_time_new:0:U"
        "DS:Power_usage_Watt:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"

        );
  } ## end if ( $entity_type eq $et_ClusterComputeResource)
  elsif ( $entity_type eq $et_ResourcePool ) {
    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:CPU_usage_MHz:GAUGE:$no_time_new:0:U"
        "DS:Memory_shared_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_zero_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_baloon_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_consumed_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_overhead_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_active_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_granted_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_compress_KB:GAUGE:$no_time_new:0:U"
        "DS:Memory_swap_KB:GAUGE:$no_time_new:0:U"
        "DS:Mem_cmprssion_KBps:GAUGE:$no_time_new:0:U"
        "DS:Mem_dcompress_KBps:GAUGE:$no_time_new:0:U"
        "DS:CPU_limit:GAUGE:$no_time_new:0:U"
        "DS:CPU_reservation:GAUGE:$no_time_new:0:U"
        "DS:Memory_limit:GAUGE:$no_time_new:0:U"
        "DS:Memory_reservation:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage_Proc:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_ResourcePool)
  elsif ( $entity_type eq $et_HostSystem ) {
    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:CPU_Alloc:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage:GAUGE:$no_time_new:0:U"
        "DS:host_hz:GAUGE:$no_time_new:0:U"
        "DS:Memory_active:GAUGE:$no_time_new:0:U"
        "DS:Memory_granted:GAUGE:$no_time_new:0:U"
        "DS:Memory_baloon:GAUGE:$no_time_new:0:U"
        "DS:Disk_usage:GAUGE:$no_time_new:0:U"
        "DS:Disk_read:GAUGE:$no_time_new:0:U"
        "DS:Disk_write:GAUGE:$no_time_new:0:U"
        "DS:Network_usage:GAUGE:$no_time_new:0:U"
        "DS:Network_received:GAUGE:$no_time_new:0:U"
        "DS:Network_transmitted:GAUGE:$no_time_new:0:U"
        "DS:Memory_swapin:GAUGE:$no_time_new:0:U"
        "DS:Memory_swapout:GAUGE:$no_time_new:0:U"
        "DS:Memory_compres:GAUGE:$no_time_new:0:U"
        "DS:Memory_decompres:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage_Proc:GAUGE:$no_time_new:0:U"
        "DS:Memory_Host_Size:GAUGE:$no_time_new:0:U"
        "DS:CPU_ready_ms:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_HostSystem)
  elsif ( $entity_type eq $et_Datastore && $rrd =~ /rrs$/ ) {    # irregular update / 30 minutes

    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:freeSpace:GAUGE:$no_time_new:0:U"
        "DS:Disk_used:GAUGE:$no_time_new:0:U"
        "DS:Disk_provisioned:GAUGE:$no_time_new:0:U"
        "DS:Disk_capacity:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_Datastore...)
  elsif ( $entity_type eq $et_Datastore && $rrd =~ /rrt$/ ) {    # regular update

    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:Datastore_read:GAUGE:$no_time_new:0:U"
        "DS:Datastore_write:GAUGE:$no_time_new:0:U"
        "DS:Datastore_ReadAvg:GAUGE:$no_time_new:0:U"
        "DS:Datastore_WriteAvg:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_Datastore...)
  elsif ( $entity_type eq $et_Datastore && $rrd =~ /rru$/ ) {    # regular update

    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:Dstore_readLatency:GAUGE:$no_time_new:0:U"
        "DS:Dstore_writeLatency:GAUGE:$no_time_new:0:U"
        "DS:Dstore_maxDepth:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_Datastore...)
  elsif ( $entity_type eq $et_Datastore && $rrd =~ /rrv$/ ) {    # regular update into datastore/vm_uuid.rrv IOPS

    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:vm_iops_read:GAUGE:$no_time_new:0:U"
        "DS:vm_iops_write:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end elsif ( $entity_type eq $et_Datastore...)
  else {                                                         # it is VM
    RRDp::cmd qq(create "$rrd"  --start "$start_time"  --step "$step_new"
        "DS:CPU_Alloc:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage:GAUGE:$no_time_new:0:U"
        "DS:host_hz:GAUGE:$no_time_new:0:U"
        "DS:Memory_active:GAUGE:$no_time_new:0:U"
        "DS:Memory_granted:GAUGE:$no_time_new:0:U"
        "DS:Memory_baloon:GAUGE:$no_time_new:0:U"
        "DS:Disk_usage:GAUGE:$no_time_new:0:U"
        "DS:Disk_read:GAUGE:$no_time_new:0:U"
        "DS:Disk_write:GAUGE:$no_time_new:0:U"
        "DS:Network_usage:GAUGE:$no_time_new:0:U"
        "DS:Network_received:GAUGE:$no_time_new:0:U"
        "DS:Network_transmitted:GAUGE:$no_time_new:0:U"
        "DS:Memory_swapin:GAUGE:$no_time_new:0:U"
        "DS:Memory_swapout:GAUGE:$no_time_new:0:U"
        "DS:Memory_compres:GAUGE:$no_time_new:0:U"
        "DS:Memory_decompres:GAUGE:$no_time_new:0:U"
        "DS:CPU_usage_Proc:GAUGE:$no_time_new:0:U"
        "DS:vCPU:GAUGE:$no_time_new:0:U"
        "DS:CPU_ready_ms:GAUGE:$no_time_new:0:U"
        "RRA:AVERAGE:0.5:1:$one_minute_sample"
        "RRA:AVERAGE:0.5:5:$five_mins_sample"
        "RRA:AVERAGE:0.5:60:$one_hour_sample"
        "RRA:AVERAGE:0.5:300:$five_hours_sample"
        "RRA:AVERAGE:0.5:1440:$one_day_sample"
        );
  } ## end else [ if ( $entity_type eq $et_ClusterComputeResource)]

  if ( !Xorux_lib::create_check("file: $rrd, $one_minute_sample, $five_mins_sample, $one_hour_sample, $five_hours_sample, $one_day_sample") ) {
    main::error( "unable to create $rrd : at " . __FILE__ . ": line " . __LINE__ );
    RRDp::end;
    RRDp::start "$rrdtool";
    return 2;
  } ## end if ( !Xorux_lib::create_check...)

  my $vmware_signal_file = "$wrkdir/$managedname/$host/vmware.txt";
  if ( !-f "$vmware_signal_file" ) {
    `touch "$vmware_signal_file"`;    # it must be there
    touch("$vmware_signal_file");     # say install_html.sh that there was any change
  }
  return 0;
} ## end sub create_rrd_vmware_vm

sub touch {
  my $text = shift;

  my $version    = "$ENV{version}";
  my $basedir    = $ENV{INPUTDIR};
  my $new_change = "$basedir/tmp/$version-vmware";
  my $host       = $ENV{HMC};
  my $DEBUG      = $ENV{DEBUG};

  if ( !-f $new_change ) {
    `touch $new_change`;    # say install_html.sh that there was any change
    if ( $text eq '' ) {
      print "touch          : $host $new_change\n" if $DEBUG;
    }
    else {
      print "touch          : $host $new_change : $text\n" if $DEBUG;
    }
  } ## end if ( !-f $new_change )

  return 0;
} ## end sub touch

sub load_retentions {
  my $step     = shift;
  my $act_time = shift;
  my $basedir  = $ENV{INPUTDIR};

  # standards
  $one_minute_sample = 86400;
  $five_mins_sample  = 25920;
  $one_hour_sample   = 4320;
  $five_hours_sample = 1734;
  $one_day_sample    = 1080;

  $one_minute_sample_mem = 86400;
  $five_mins_sample_mem  = 25920;
  $one_hour_sample_mem   = 4320;
  $five_hours_sample_mem = 1734;
  $one_day_sample_mem    = 1080;

  if ( !-f "$basedir/etc/retention.cfg" ) {

    # standard retentions in place
    return 0;
  }

  # extra retentions are specifiled in $basedir/etc/retention.cfg
  open( FH, "< $basedir/etc/retention.cfg" ) || main::error("Can't read from: $basedir/etc/retention.cfg: $!");

  my @lines = <FH>;
  foreach my $line (@lines) {
    chomp($line);

    # MEM
    if ( $line =~ m/^1min/ && $line =~ m/MEM/ ) {
      ( my $trash, $one_minute_sample_mem ) = split( /:/, $line );
      next;
    }
    if ( $line =~ m/^5min/ && $line =~ m/MEM/ ) {
      ( my $trash, $five_mins_sample_mem ) = split( /:/, $line );
      next;
    }
    if ( $line =~ m/^60min/ && $line =~ m/MEM/ ) {
      ( my $trash, $one_hour_sample_mem ) = split( /:/, $line );
      next;
    }
    if ( $line =~ m/^300min/ && $line =~ m/MEM/ ) {
      ( my $trash, $five_hours_sample_mem ) = split( /:/, $line );
      next;
    }
    if ( $line =~ m/^1440min/ && $line =~ m/MEM/ ) {
      ( my $trash, $one_day_sample_mem ) = split( /:/, $line );
      next;
    }

    # CPU
    if ( $line =~ m/^1min/ ) {
      ( my $trash, $one_minute_sample ) = split( /:/, $line );
    }
    if ( $line =~ m/^5min/ ) {
      ( my $trash, $five_mins_sample ) = split( /:/, $line );
    }
    if ( $line =~ m/^60min/ ) {
      ( my $trash, $one_hour_sample ) = split( /:/, $line );
    }
    if ( $line =~ m/^300min/ ) {
      ( my $trash, $five_hours_sample ) = split( /:/, $line );
    }
    if ( $line =~ m/^1440min/ ) {
      ( my $trash, $one_day_sample ) = split( /:/, $line );
    }
  } ## end foreach my $line (@lines)

  close(FH);

  $step              = $step / 60;
  $one_minute_sample = $one_minute_sample / $step;
  $five_mins_sample  = $five_mins_sample / $step;
  $one_hour_sample   = $one_hour_sample / $step;
  $five_hours_sample = $five_hours_sample / $step;
  $one_day_sample    = $one_day_sample / $step;

  $one_minute_sample_mem = $one_minute_sample_mem / $step;
  $five_mins_sample_mem  = $five_mins_sample_mem / $step;
  $one_hour_sample_mem   = $one_hour_sample_mem / $step;
  $five_hours_sample_mem = $five_hours_sample_mem / $step;
  $one_day_sample_mem    = $one_day_sample_mem / $step;
  return 1;
} ## end sub load_retentions

sub recent_rename {
  my $host          = shift;
  my $managedname   = shift;
  my $lpar          = shift;
  my $wrkdir        = shift;
  my $DEBUG         = shift;
  my $time_rec      = shift;
  my $act_time      = shift;
  my $RENAME_PERIOD = 600;     # if there is detected same lpar name 10 minutes after a rename then it is ignored

  my $rename_tmp = "$wrkdir/$managedname/$host/rename.tmp";
  if ( !-f "$rename_tmp" ) {
    return 0;
  }

  #print "001 $rename_tmp $lpar\n";

  open( FR, "< $rename_tmp" ) || main::error( " Can't open $rename_tmp : $!" . __FILE__ . ":" . __LINE__ ) && return 0;

  my @lines = <FR>;
  close(FR);
  foreach my $line (@lines) {
    chomp($line);
    ( my $time, my $lpar_act ) = split( /:/, $line );

    #print "002 $line $time - $lpar_act - $lpar - ($time + $RENAME_PERIOD) > $time_rec\n";
    if ( $lpar_act eq '' ) {
      next;    # some trash
    }
    if ( $lpar_act =~ m/^$lpar$/ && ( $time + $RENAME_PERIOD ) > $time_rec ) {
      print "rename ignore  : $host:$managedname:$lpar : rename period ignore : ($time + $RENAME_PERIOD) > $time_rec\n" if $DEBUG;
      return 1;
    }
  } ## end foreach my $line (@lines)

  return 0;
} ## end sub recent_rename

sub rrd_error {
  my $err_text = shift;
  my $rrd_file = shift;
  my $basedir  = $ENV{INPUTDIR};
  my $tmpdir   = "$basedir/tmp";
  if ( defined $ENV{TMPDIR_LPAR} ) {
    $tmpdir = $ENV{TMPDIR_LPAR};
  }

  chomp($err_text);

  if ( !-f "$rrd_file" ) {
    return 0;
  }

  if ( $err_text =~ m/ERROR:/ ) {

    # copy of the corrupted file into "save" place and remove the original one
    copy( "$rrd_file", "$tmpdir/" ) || main::error("Cannot: cp $rrd_file $tmpdir/: $!");
    unlink("$rrd_file") || main::error("Cannot rm $rrd_file : $!");
    main::error("$err_text, moving it into: $tmpdir/");
  } ## end if ( $err_text =~ m/ERROR:/)
  else {
    main::error("$err_text");
  }
  return 0;
} ## end sub rrd_error

return 1;
