package Users;

# LPAR2RRD user management module
# vim: autoindent tabstop=2 shiftwidth=2 expandtab softtabstop=2 filetype=perl
use strict;
use warnings;
use POSIX;
use Data::Dumper;

my $basedir = $ENV{INPUTDIR};
$basedir ||= "..";

my $cfgdir     = "$basedir/etc/web_config";
my $realcfgdir = "$basedir/etc";

use JSON qw(decode_json encode_json);

my $json = JSON->new->utf8->pretty;

my %alerts;       # hash holding alerting configuration
my %emailgrps;    # hash holding e-mail groups
my %config;       # hash holding configuration
my $rawcfg;
my @rawcfg;
my $oldcfg    = 0;
my $now       = time();
my $isotime   = strftime( '%Y-%m-%dT%H:%M:%SZ', gmtime($now) );
my $adminName = $ENV{REMOTE_USER} ||= "admin";
my $adminPass = "\$apr1\$CSoXefyw\$wGe9K7Ld5ClOEozE4zC.T1";

#if ($ENV{VM_IMAGE}) {
#  $adminName = "monitor";
#  $adminPass = "\$apr1\$UZuWgWzB\$gk6cSafcM9F2Bl0Jl94ZB.";
#}

sub readCfg {
  my $aclAdminGroup = $ENV{ACL_ADMIN_GROUP} ||= "admins";
  if ( !open( CFG, "$cfgdir/users.json" ) ) {
    if ( !open( CFG, ">$cfgdir/users.json" ) ) {
      die "Cannot open file: $!\n";
    }
    else {    # create empty cfg file
      print CFG <<_MARKER_;
{
  "ACLimported" : false,
  "groups" : {
    "$aclAdminGroup": {
      "description": "LPAR2RRD Administrators",
      "ACL": {
        "lpars": {},
        "pools": {},
        "cgroups": ["*"],
        "vms": {},
        "solo": {}
      }
    },
    "ReadOnly": {
      "description": "Member can see everything but nothing can change."
    }
  },
  "users" : {
    "$adminName": {
      "name":            "LPAR2RRD Administrator",
      "email":           "",
      "htpassword":      "$adminPass",
      "active":          1,
      "created":         "$isotime",
      "updated":         "$isotime",
      "last_login":      "",
      "groups": [
        "$aclAdminGroup"
      ],
      "config": {
        "locale":        "en-US",
        "timezone":      "",
        "menu_width":    150,
        "db_height":     50,
        "db_width":      120,
        "db_items": [
        ]
      }
    }
  }
}
_MARKER_

      close CFG;
      open( CFG, "$cfgdir/users.json" );
    } ## end else [ if ( !open( CFG, ">$cfgdir/users.json"...))]
  } ## end if ( !open( CFG, "$cfgdir/users.json"...))
  {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
    if ($rawcfg) {
      %config = %{ decode_json($rawcfg) };
    }
  }
  close CFG;
  if ( !$config{ACLimported} || $config{ACLimported} eq "false" ) {
    if ( open( ACL, "<", "$cfgdir/acl.cfg" ) ) {
      my %acl;
      while ( my $line = <ACL> ) {
        chomp $line;
        next unless s/^([^|]*?)\|([^|]*?)\|([^|]*?)\|(.*)//, $line;
        my @val = split( /\|/, $line );
        for (@val) {
          &pipes($_);
        }
        my ( $group, $cgrp, $srv, $lpar ) = @val;

        my @servers = ( split /,/, $srv );
        chomp @servers;
        if (@servers) {
          $acl{$group}{servers} = [@servers];
        }

        my @customs = ( split /,/, $cgrp );
        chomp @customs;
        if (@customs) {
          $acl{$group}{cgroups} = [@customs];
        }

        if ($lpar) {
          for my $field ( split /,/, $lpar ) {
            my ( $key, $value ) = ( split /=>/, $field );
            if ($value) {
              $acl{$group}{lpars}{$key}{$value} = 1;
            }
          } ## end for my $field ( split /,/...)
        } ## end if ($lpar)
      } ## end while ( my $line = <ACL> )
      close ACL;
      foreach my $grp ( keys %acl ) {
        $config{groups}{$grp}{ACL}{lpars} = ();
        foreach my $srv ( @{ $acl{$grp}{servers} } ) {
          $config{groups}{$grp}{ACL}{lpars}{$srv} = [];
          push @{ $config{groups}{$grp}{ACL}{lpars}{$srv} }, "*";
        }
        $config{groups}{$grp}{ACL}{cgroups} = $acl{$grp}{cgroups} ||= [];
        foreach my $srv ( keys %{ $acl{$grp}{lpars} } ) {
          $config{groups}{$grp}{ACL}{lpars}{$srv} = [ keys %{ $acl{$grp}{lpars}{$srv} } ];
        }
      } ## end foreach my $grp ( keys %acl)

      # print STDERR Dumper \%config;
      $config{ACLimported} = "true";
      if ( open( CFG, ">$cfgdir/users.json" ) ) {
        print CFG $json->encode( \%config );
        close CFG;
      }

    } ## end if ( open( ACL, "<", "$cfgdir/acl.cfg"...))
  } ## end if ( !$config{ACLimported...})
  if ( !$config{groups}{ReadOnly} ) {
    $config{groups}{ReadOnly}{description} = "Member can see everything but nothing can change.";
    $rawcfg = $json->encode( \%config );
  }

  # print Dumper \%config;
} ## end sub readCfg

sub getGroups {

  # return sorted array of defined groups
  &readCfg;
  return sort keys %emailgrps;
} ## end sub getGroups

sub getDefinedAlerts {

  # return sorted array of defined alerts
  &readCfg;
  return sort keys %alerts;
} ## end sub getDefinedAlerts

sub getAlerts {

  # return sorted array of defined alerts
  &readCfg;
  return %alerts;
} ## end sub getAlerts

sub getConfig {

  # return configuration hash
  &readCfg;
  return %config;
} ## end sub getConfig

sub getRawConfig {
  &readCfg;
  return $rawcfg;
}

sub getHTPasswords {
  &readCfg;
  my $result = "";
  foreach my $user ( sort keys %{ $config{users} } ) {
    if ( $config{users}{$user}{active} && $config{users}{$user}{htpassword} ) {
      $result .= "$user:$config{users}{$user}{htpassword}\n";
    }
  }
  return $result;
} ## end sub getHTPasswords

sub getAlertDetails {

  # params: alert_name
  # return hash of alert details
  my ( $storage, $type, $name, $metric ) = shift;
  return $alerts{$storage}{$type}{$name}{$metric};
} ## end sub getAlertDetails

sub getGroupMembers {

  # param: group_name
  # return sorted array of defined alerts
  my $groupName = shift;
  return $emailgrps{$groupName};
} ## end sub getGroupMembers

sub getFullName {

  # params: group_name, group_member
  # return full name of a member
  my ( $groupName, $groupMem ) = @_;
  return $alerts{groups}{$groupName}{$groupMem}{fullname};
} ## end sub getFullName

sub getEmail {

  # params: group_name, group_member
  # return full name of a member
  my ( $groupName, $groupMem ) = @_;
  return $alerts{groups}{$groupName}{$groupMem}{email};
} ## end sub getEmail

sub getUserDetails {

  # params: group_name, group_member
  # return hash of member details
  my ( $groupName, $groupMem ) = @_;
  return $alerts{groups}{$groupName}{$groupMem};
} ## end sub getUserDetails

sub doublecoma {
  return s/===========doublecoma=========/:/g;
}

sub pipes {
  return s/===pipe===/\|/g;
}

1;
