use strict;
use warnings;
use POSIX;
use Time::Local;
use Data::Dumper;
use JSON;

use Xorux_lib qw(read_json write_json uuid_big_endian_format);
use OVirtDataWrapper;

#`. /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg`;

###### RUN SCRIPT WITHOUT ARGUMENTS (PRINT ALL SERVER):
######
###### .  /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl
######

###### RUN SCRIPT WITH ARGUMENTS (PRINT ONLY ONE SERVER):
######
###### . /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl name_server
######   (for example - .  /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl Power750_02)
######

###### RUN SCRIPT FOR VMWARE ALL
######
###### .  /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl VMWARE
######

###### RUN SCRIPT FOR HITACHI ALL
######
###### .  /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl HITACHI
######

###### RUN SCRIPT FOR HYPERV ALL
######
###### .  /home/lpar2rrd/lpar2rrd/etc/lpar2rrd.cfg ; $PERL ./bin/find_active_lpar.pl HYPERV
######


######################### PRINT HEADER IF PROGRAM STARTED FROM BROWSER
if ( defined $ENV{FROMWEB} && $ENV{FROMWEB} eq "1" ) {
  print "Content-type: text/plain\n\n";
}

# flush after every write
$| = 1;

defined $ENV{INPUTDIR} || error( " Not defined INPUTDIR, probably not read etc/lpar2rrd.cfg " . __FILE__ . ":" . __LINE__ ) && exit 1;
my $basedir              = $ENV{INPUTDIR};
my $actual_unix_time     = time;
my $last_ten_days        = 10 * 86400;                            ### ten days back
my $actual_last_ten_days = $actual_unix_time - $last_ten_days;    ### ten days back with actual unix time-

my $active_days = 30;
my $year_back = time - 31536000;
if ( defined $ENV{VMWARE_ACTIVE_DAYS} && $ENV{VMWARE_ACTIVE_DAYS} > 1 && $ENV{VMWARE_ACTIVE_DAYS} < 3650 ) {
  $active_days = $ENV{VMWARE_ACTIVE_DAYS};
}
my $last_30_days        = $active_days * 86400;                   ### 30 days back
my $actual_last_30_days = $actual_unix_time - $last_30_days;      ### 30 days back with actual unix time-
my $wrkdir              = "$basedir/data";
my $no_hmc_dir          = "$wrkdir/Linux/no_hmc";
my $agents_uuid_file    = "$no_hmc_dir/linux_uuid_name.json";

opendir( DIR, "$wrkdir" ) || error( " directory does not exists : $wrkdir " . __FILE__ . ":" . __LINE__ ) && exit 1;
my @wrkdir_all = grep !/^\.\.?$/, readdir(DIR);
closedir(DIR);

my $list_of_vm = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
my $vm_dir     = "$wrkdir/vmware_VMs";
my @vm_list;
if ( -e $list_of_vm ) {    ####### ALL VM servers in @vm_list
  open( FC, "< $list_of_vm" ) || error( "Cannot read $list_of_vm: $!" . __FILE__ . ":" . __LINE__ );
  @vm_list = <FC>;
  close(FC);
}

my $s               = ":";
my $deleted         = "R:";
my $actived         = "L:";
my %hash_lpar       = ();
my %hash_wpar       = ();
my %hash_wlm        = ();
my %hash_vm         = ();
my %hash_timestamp1 = ();
my $wpar_menu       = "W:";
my $wlm_menu        = "WLM:";
my $server_list_cfg = "$basedir/etc/alias.cfg";
my @lines_dualhmc1 = "";

################ look for hitachi lpars uuid
my @hitachi_uuid_txt = <$wrkdir/Hitachi/*/lpar_uuids.txt>;
my %hitachi_lpar_uuids;
foreach my $file (@hitachi_uuid_txt) {
  if ( open( FH, " < $file" )) {
    while ( my $line = <FH> ) {
      my ($uuid, undef, undef) = split /,/, $line;
      $hitachi_lpar_uuids{$uuid} = 1;
    }
    close FH;
  }
  else {
    error( "Cannot open $file: $!" . __FILE__ . ":" . __LINE__ );
    next;
  }
}
################ look for ovirt lpars uuid
my %ovirt_uuids;

if ( -f "$wrkdir/oVirt/metadata.json" ) {
  %ovirt_uuids = %{ OVirtDataWrapper::get_conf_section( 'mapping' ) };
}

################ check alias.cfg for VM,WPAR,LPAR
my @alias_vmware;
my @alias_servers;
my @alias_servers_wpar;
if ( -e $server_list_cfg ) {
  open( FC, "< $server_list_cfg" ) || error( "Cannot read $server_list_cfg: $!" . __FILE__ . ":" . __LINE__ );
  my @server_list_config = <FC>;
  close(FC);
  @alias_vmware       = grep {/^VM:/} @server_list_config;      ### grep only "VM:" and save it in array
  @alias_servers      = grep {/^LPAR:/} @server_list_config;    ### grep only "LPAR:" and save it in array
  @alias_servers_wpar = grep {/^WPAR:/} @server_list_config;    ### grep only "WPAR:" and save it in array
} ## end if ( -e $server_list_cfg)

################ read arguments (ONLY ONE SERVER!)
my ($param_server) = @ARGV;

if ( defined $param_server ) {                                  ### only for shell if you defined only one server
  if ( $param_server eq "AGENT_UUIDS" ) {
    my $result = uuids();
    exit $result;
  } ## end if ( $param_server eq "AGENT_UUIDS" ...)
  if ( $param_server eq "VMWARE" ) {
    my $result = print_vm_all();
    exit $result;
  } ## end if ( $param_server eq VMWARE ...)
  if ( $param_server eq "HYPERV" ) {
    my $result = print_hyperv_all();
    exit $result;
  } ## end if ( $param_server eq HYPERV ...)
  if ( $param_server eq "HITACHI" ) {
    my $result = print_hitachi_all();
    exit $result;
  } ## end if ( $param_server eq "HITACHI" ...)
  if ( $param_server eq "SOLARIS-L" ) {
    my $result = print_solaris_all();
    exit $result;
  } ## end if ( $param_server eq "SOLARIS-L" ...)
  else {
    @wrkdir_all = ();
    $wrkdir_all[0] = "$wrkdir/$param_server";
    my $result = print_all_server();
    exit $result;
  }
} ## end if ( defined $param_server)
else {
  my $result = print_all_server();
  exit $result;
} ## end else [ if ( defined $param_server)]

################### PRINT ALL SERVERS
sub print_all_server {
  foreach my $server_all (@wrkdir_all) {
    #print "1)server-$server_all\n";
    $server_all = "$wrkdir/$server_all";
    my $server = basename($server_all);
    if ( $server =~ /[vV][mM][wW][aA][rR][eE]/ ) { next; }
    if ( -l $server_all )                        { next; }
    if ( -f "$server_all" )                      { next; }
    if ( $server_all =~ /--HMC--/ )              { next; }
    if ( $server_all =~ /Hitachi$/ )             { next; }
    if ( $server_all =~ /Solaris/ )             { next; }
    chomp $server_all;
    chomp $server;
    opendir( DIR, "$wrkdir/$server" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @hmcdir_all = grep !/^\.\.?$/, readdir(DIR);
    closedir(DIR);
    my $last_update_time = 0;
    #print "@hmcdir_all\n";
    foreach my $hmc_all_base (@hmcdir_all) {
      my $hmc_all = "$wrkdir/$server/$hmc_all_base";
      #print "2)hmc-$hmc_all\n";
      my $hmc     = basename($hmc_all);
      if ( $hmc_all =~ /\.rrx/ ) { next; }
      if ( $hmc =~ /--NMON--/ )  { next; }
      if ( $hmc =~ /no_hmc/ ) {    ####################### SERVER - NO_HMC !!!
        opendir( DIR, "$wrkdir/$server/$hmc" ) || error( "can't opendir $wrkdir/$server/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @lpars_dir_nohmc_all = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);
        foreach my $lpars_dir_base (@lpars_dir_nohmc_all) {    #################### OS AGENT
          my $lpars_dir  = "$wrkdir/$server/$hmc/$lpars_dir_base";
          my $server_unk = $server;
          my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpars_dir_base, "" );
          if ( $lpars_dir =~ /--AS400--$/ ) {                  ################# AS400 ----- NO_HMC
            if ( -f "$lpars_dir/S0200ASPJOB.mmm" ) {
              my $lpar_as = basename($lpars_dir);
              $lpar_as =~ s/--AS400--$//;
              my ($result_lpar_cfg) = grep /:$lpar_as:/, @alias_servers;
              my $lpar_b            = "";
              my $alias_os          = "";
              my $alias             = "";
              if ( defined $result_lpar_cfg && $result_lpar_cfg ne "" ) {
                ( undef, $lpar_b, $alias ) = split( /:/, $result_lpar_cfg );
                chomp $alias;
                $alias_os = " [$alias]";
              }
              if ( $server =~ /--unknown$/ ) {
                $server =~ s/--unknown$//;
                print "$actived" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_as" . "$alias_os" . "$s" . "$lpar_as" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$lpar_as&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
                $server  = "$server--unknown";
                $lpar_as = "$lpar_as--AS400--";
              } ## end if ( $server =~ /--unknown$/)
              else {
                print "$actived" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_as" . "$s" . "$lpar_as" . "$alias_os" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$lpar_as&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
                $lpar_as = "$lpar_as--AS400--";
              }
            } ## end if ( -f "$lpars_dir/S0200ASPJOB.mmm")
          } ## end if ( $lpars_dir =~ /--AS400--$/)
          else {
            if ( -f "$lpars_dir/cpu.mmm" ) {
              #if ($server =~ /--unknown/){next;}
              my $file_cpu       = "$lpars_dir/cpu.mmm";
              my $last_timestamp = ( stat($file_cpu) )[9];
              if ( $last_timestamp > $actual_last_ten_days ) {
                $lpars_dir =~ s/--NMON--$//g;
                my $lpar = basename($lpars_dir);
                if ( !defined $hash_lpar{$lpars_dir} ) {
                  $hash_lpar{$lpars_dir} = "test";
                }
                else {
                  next;
                }

                #server_unk =~ s/--unknown$//;
                my $lpar_name = $lpar;
                $lpar =~ s/&&1/\//g;
                my $server_enc    = urlencode($server_unk);
                my $hmc_enc       = urlencode($hmc);
                my $lpar_name_enc = urlencode($lpar);
                my ($result_lpar_cfg) = grep /:$lpar:/, @alias_servers;
                my $lpar_b            = "";
                my $alias_os          = "";
                my $alias             = "";
                my $uuid_txt = "$wrkdir/$server/$hmc/$lpar/uuid.txt";
                if (-f $uuid_txt && open( FCD, "< $uuid_txt")) {
                  if ($server =~ /Solaris/){next;}
                  my $uuid_old = <FCD>;
                  close(FCD);
                  # e.g. D2051C42-C469-27DA-3FF6-6E508678C004
                  if (! defined $uuid_old) {
                    error( "Bad data in file $wrkdir/$server/$hmc/$lpar/uuid.txt: " . __FILE__ . ":" . __LINE__ );
                  }
                  else {
                    my @uuid_atoms = split("-",$uuid_old);
                    if (defined $uuid_atoms[3] && defined $uuid_atoms[4]) {
                      my $pattern = "-$uuid_atoms[3]-$uuid_atoms[4]";
                      $pattern = lc $pattern;
                      chomp $pattern;
                      # print STDERR "235 $pattern\n";
                      my @matches = grep {/$pattern/} @vm_list;
                      next if (defined $matches[0]);
                      # print STDERR "237 not found match\n";
                    }
                    chomp $uuid_old;
                    if ( defined $hitachi_lpar_uuids{$uuid_old} || defined $ovirt_uuids{ lc $uuid_old } ) {
                      next;
                    }
                  }
                }
                if ( defined $result_lpar_cfg && $result_lpar_cfg ne "" ) {
                  ( undef, $lpar_b, $alias ) = split( /:/, $result_lpar_cfg );
                  chomp $alias;
                  $alias_os = " [$alias]";
                }
                my @ldom_names = "";
                if ( $server_unk =~ /--unknown/ ) {    ###### SERVER(LINUX) ---unknown
                  $server_unk =~ s/--unknown//;
                     print "$actived" . "$hmc" . "$s" . "$server_unk" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_os" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
                }
                else {
                  if ( $hmc =~ /no_hmc/ ) { next; }
                  print "$actived" . "$hmc" . "$s" . "$server_unk" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_os" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
                }
              } ## end if ( $last_timestamp >...)
            } ## end if ( -f "$lpars_dir/cpu.mmm")
          } ## end else [ if ( $lpars_dir =~ /--AS400--$/)]
        } ## end foreach my $lpars_dir_base ...
      } ## end if ( $hmc =~ /no_hmc/ )
      my $server_un = $server;
      if ( $server =~ /--unknown$/ ) {    ######## OS AGENT -- SERVER--unknown
        if ( $hmc =~ /no_hmc/ ) { next; }
        opendir( DIR, "$wrkdir/$server/$hmc" ) || error( "can't opendir $wrkdir/$server/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @lpars_dir_all = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);
        foreach my $lpars_dir_base (@lpars_dir_all) {
          my $lpars_dir = "$wrkdir/$server_un/$hmc/$lpars_dir_base";
          if ( -f "$lpars_dir/cpu.mmm" ) {    ##### OS AGENT DATA
            my $file_cpu       = "$lpars_dir/cpu.mmm";
            my $last_timestamp = ( stat($file_cpu) )[9];
            if ( $last_timestamp > $actual_last_ten_days ) {
              my $lpar = basename($lpars_dir);
              if ( $lpar =~ /--NMON--/ )     { next; }
              if ( $server =~ /--unknown$/ ) { $server =~ s/--unknown$//; }
              my $lpar_name     = $lpar;
              my $agent_type    = find_agent_type( $wrkdir, $server, $hmc, $lpar, "" );
              my $server_enc    = urlencode($server);
              my $hmc_enc       = urlencode($hmc);
              my $lpar_name_enc = urlencode($lpar_name);
              my ($result_lpar_cfg) = grep /:$lpar:/, @alias_servers;
              my $lpar_b            = "";
              my $alias_os          = "";
              my $alias             = "";

              if ( defined $result_lpar_cfg && $result_lpar_cfg ne "" ) {
                ( undef, $lpar_b, $alias ) = split( /:/, $result_lpar_cfg );
                chomp $alias;
                $alias_os = " [$alias]";
              }
              print "$actived" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_os" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc" . "--unknown" . "&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
            } ## end if ( $last_timestamp >...)
          } ## end if ( -f "$lpars_dir/cpu.mmm")
        } ## end foreach my $lpars_dir_base ...
      } ## end if ( $server =~ /--unknown$/)

      #if (!-d $hmc) {next;}
      my $hmc_dir_test = "$wrkdir/$server/$hmc";
      if ( !-d $hmc_dir_test ) { next; }
      my $LAN_interface_file = "$wrkdir/$server/$hmc/LAN_aliases.json";
      my $SAN_interface_file = "$wrkdir/$server/$hmc/SAN_aliases.json";
      my $SAS_interface_file = "$wrkdir/$server/$hmc/SAS_aliases.json";
      my $LAN_interface_aliases;
      my $SAN_interface_aliases;
      my $SAS_interface_aliases;
      if (-e $LAN_interface_file){
        $LAN_interface_aliases = read_json($LAN_interface_file);
      }
      if (-e $SAN_interface_file){
        $SAN_interface_aliases = read_json($SAN_interface_file);
      }
      if (-e $SAS_interface_file){
        $SAS_interface_aliases = read_json($SAS_interface_file);
      }
      opendir( DIR, "$wrkdir/$server/$hmc" ) || error( "can't opendir $wrkdir/$server/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next; #tady if adapters
      my @lpars_dir_all = grep !/^\.\.?$/, readdir(DIR);
      closedir(DIR);
      my $cpu_cfg = "$hmc_all/cpu.cfg";
      my @lines_config;

      if ( -e $cpu_cfg ) {
        open( FH, "< $cpu_cfg" ) || error( "Cannot read $cpu_cfg: $!" . __FILE__ . ":" . __LINE__ ) && next;
        @lines_config = <FH>;
        close(FH);
      }
      foreach my $lpars_dir_base (@lpars_dir_all) {
        if ( $lpars_dir_base eq "adapters" ){
          my $adapter_dir = "$wrkdir/$server/$hmc/$lpars_dir_base";
          opendir( DIR, "$adapter_dir" ) || error( "can't opendir $adapter_dir: $! :" . __FILE__ . ":" . __LINE__ ) && next;
          my @adapters = grep !/^\.\.?$/, readdir(DIR);
          closedir(DIR);
          if (!(@adapters)){ # if there is not any file in adapters folder
            next;
          }
          else{
           my $lan = 0;
           my $hea = 0;
           my $san = 0;
           my $sas = 0;
           my $sri = 0;
           my $tmp_server = $server;
           $server = urlencode($server);
           my @current_sriov_adapters;
           my $to_aggregate;
           foreach my $adapter (@adapters){
            #if ($adapter =~ m/-V[0-9]*-C[0-9]*/){next;} #hash this if you want Virtual Adapters with XXXXX-V*-C*-T* in name
            if ($adapter =~ m/rasm/){
              my $act_alias;
              my $act_lpar;
              #print "SAN: $adapter\n";
              (my $adapter_name, undef) = split(".r", $adapter);
              my $interface_string;
              my $interface_string_url;
              foreach my $alias (keys %{$SAN_interface_aliases}){
                if ($alias =~ /$adapter_name/){
                  $act_alias="$SAN_interface_aliases->{$alias}{alias}";
                  $act_lpar="$SAN_interface_aliases->{$alias}{partition}";
                  $interface_string="$act_lpar $act_alias";
                  $interface_string_url="$act_lpar%20$act_alias";
                }
              }
              if (!defined $interface_string || $interface_string eq ""){
                $interface_string_url = $interface_string = $adapter_name;
              }
              if ($interface_string !~ $adapter_name){
                $interface_string = "$interface_string $adapter_name";
              }
              if (length ($adapter_name) <= 7 || !defined $interface_string || $interface_string eq "") { next; }
              print "Y:$hmc:$tmp_server:$interface_string_url:$interface_string:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$adapter&item=power_san&entitle=0&gui=1&none=none:::P:\n";
              $san = 1;
            }
            elsif ($adapter =~ m/rapm/){
              my $act_alias;
              my $act_lpar;
              (my $adapter_name, undef) = split(".r", $adapter);
              my $interface_string;
              my $interface_string_url;
              foreach my $alias (keys %{$SAS_interface_aliases}){
                if ($alias =~ /$adapter_name/){
                  $act_alias="$SAS_interface_aliases->{$alias}{alias}";
                  $act_lpar="$SAS_interface_aliases->{$alias}{partition}";
                  $interface_string="$act_lpar $act_alias";
                  $interface_string_url="$act_lpar%20$act_alias";
                }
              }
              if (!defined $interface_string || $interface_string eq ""){
                $interface_string_url = $interface_string = $adapter_name;
              }
              if ($interface_string !~ $adapter_name){
                $interface_string = "$interface_string $adapter_name";
              }
              if (length ($adapter_name) <= 7 || !defined $interface_string || $interface_string eq "") { next; }
              ### THIS LOOKS THAT IS THE SAME AS ralm ADAPTERS, SO IT'S LAN NOT SAN
              print "s:$hmc:$tmp_server:$interface_string_url:$interface_string:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$adapter&item=power_sas&entitle=0&gui=1&none=none:::P:\n";
              $sas = 1;
            }
            elsif ($adapter =~ m/ralm/){
              my $act_alias;
              my $act_lpar;
              #print "LAN: $adapter\n";
              (my $adapter_name, undef) = split(".r", $adapter);
              my $interface_string;
              my $interface_string_url;
              foreach my $alias (keys %{$LAN_interface_aliases}){
                if ($alias =~ /$adapter_name/){
                  $act_alias="$LAN_interface_aliases->{$alias}{alias}";
                  $act_lpar="$LAN_interface_aliases->{$alias}{partition}";
                  $interface_string="$act_lpar $act_alias";
                  $interface_string_url="$act_lpar%20$act_alias";
                }
              }
              if (!defined $interface_string || $interface_string eq ""){
                $interface_string_url = $interface_string = $adapter_name;
              }
              if ($interface_string !~ $adapter_name){
                $interface_string = "$interface_string $adapter_name";
              }
              if (length ($adapter_name) <= 7 || !defined $interface_string || $interface_string eq "") { next; }
              print "N:$hmc:$tmp_server:$interface_string_url:$interface_string:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$adapter&item=power_lan&entitle=0&gui=1&none=none:::P:\n";
              $lan = 1;
            }
            elsif ($adapter =~ m/rasr/){
              (my $gui_agg_name, undef) = split("-S", $adapter);
              if (length ($adapter) <= 7 || length ($gui_agg_name) <= 7) { next; }

              # this doesn't work on aix, so linear search nelow is used for now
              #if (!($gui_agg_name ~~ @current_sriov_adapters)){
              #  print "r:$hmc:$tmp_server:$adapter_name:$gui_agg_name:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$adapter&item=power_sri&entitle=0&gui=1&none=none:::P:\n";
              #  push (@current_sriov_adapters, $gui_agg_name);
              #  $sri = 1;
              #}

              my $is_there=0;
              foreach my $csa (@current_sriov_adapters){
                if ($gui_agg_name eq $csa){
                  $is_there=1;
                }
              }
              if (!($is_there)){
                print "r:$hmc:$tmp_server:$gui_agg_name:$gui_agg_name:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$gui_agg_name&item=power_sri&entitle=0&gui=1&none=none:::P:\n";
                push (@current_sriov_adapters, $gui_agg_name);
                $sri = 1;
              }
              push(@{$to_aggregate->{$gui_agg_name}}, $adapter);
            }
            elsif ($adapter =~ m/rahm/){
              (my $adapter_name, undef) = split(".r", $adapter);
              if (length ($adapter_name) <= 7 ) { next; }
              print "HEA:$hmc:$tmp_server:$adapter_name:$adapter_name:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$adapter&item=power_hea&entitle=0&gui=1&none=none:::P:\n";
              $hea = 1;
            }
           }
           write_json("$wrkdir/$server/$hmc/sriov_log_port_list.json", $to_aggregate);
           if ($hea == 1){
            print "HEA:$hmc:$tmp_server:$server:hea-totals:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=hea-totals&item=power_hea&entitle=0&gui=1&none=none:::P:\n";
           }
           if ($lan == 1){
            print "N:$hmc:$tmp_server:$server:lan-totals:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=lan-totals&item=power_lan&entitle=0&gui=1&none=none:::P:\n";
           }
           if ($san == 1){
            print "Y:$hmc:$tmp_server:$server:san-totals:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=san-totals&item=power_san&entitle=0&gui=1&none=none:::P:\n";
           }
           if ($sas == 1){
            print "s:$hmc:$tmp_server:$server:sas-totals:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=sas-totals&item=power_sas&entitle=0&gui=1&none=none:::P:\n";
           }
           if ($sri == 1){
            print "r:$hmc:$tmp_server:$server:sriov-totals:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=sri-totals&item=power_sri&entitle=0&gui=1&none=none:::P:\n";
           }
           $server = $tmp_server;
          }
        }
        my $lpars_dir = "$wrkdir/$server/$hmc/$lpars_dir_base";
        my $lpar      = basename($lpars_dir);
        if ( !-f "$hmc_all/cpu.cfg" ) {    ##### OS AGENT but no cpu.cfg in HMC
          if ( -f $lpars_dir )           { next; }
          if ( $server =~ /--unknown$/ ) { next; }
          if ( -f "$lpars_dir/cpu.mmm" ) {    ###### if exists cpu.mmm in <lpar> dir
            my $file_cpu       = "$lpars_dir/cpu.mmm";
            my $last_timestamp = ( stat($file_cpu) )[9];
            if ( $last_timestamp > $actual_last_ten_days ) {
              $lpars_dir =~ s/--NMON--//;
              if ( !defined $hash_lpar{$lpars_dir} ) {
                $hash_lpar{$lpars_dir} = "test";
              }
              else {
                next;
              }
              $lpar =~ s/--NMON--//;
              my $lpar_name  = $lpar;
              my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar, "" );
              my $server_enc = urlencode($server);
              if ( $server =~ /--unknown/ ) { next; }
              my $hmc_enc       = urlencode($hmc);
              my $lpar_name_enc = urlencode($lpar_name);
              my ($result_lpar_cfg) = grep /:$lpar:/, @alias_servers;
              my $lpar_b            = "";
              my $alias_os          = "";
              my $alias             = "";

              if ( defined $result_lpar_cfg && $result_lpar_cfg ne "" ) {
                ( undef, $lpar_b, $alias ) = split( /:/, $result_lpar_cfg );
                chomp $alias;
                $alias_os = " [$alias]";
              }
              print "$actived" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_os" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
            } ## end if ( $last_timestamp >...)
          } ## end if ( -f "$lpars_dir/cpu.mmm")
        } ## end if ( !-f "$hmc_all/cpu.cfg")
        if ( -f "$hmc_all/vmware.txt" ) { next; }
        my $lpar_dir   = $lpar;
        my $lpar_short = $lpar;
        $lpar_short =~ s/\.r..$//;
        if ( ( $lpar =~ /\.rrm$/ ) || ( $lpar =~ /\.rsh$/ && !-f "$wrkdir/$server/$hmc/$lpar_short.rrm" ) ) {
          my $rrm_file_test = "$wrkdir/$server/$hmc/$lpar";
#         print "$rrm_file_test!!\n";
          $lpar =~ s/.rsh$//g;
          $lpar =~ s/.rrm$//g;
          $lpar =~ s/:/\\:/g;
          my $lpar_name = $lpar;
          if ( $lpar_name =~ /SharedPool\d/ ) { next; }
          $lpar =~ s/&&1/\//g;
          my $result_cpu_cfg = grep /^lpar_name=$lpar,lpar_id/, @lines_config;
          my $health_status = "";
          if   ( $result_cpu_cfg == 0 ) { $health_status = "R:"; }
          else                          { $health_status = "L:"; }
          my $server_enc    = urlencode($server);
          my $hmc_enc       = urlencode($hmc);
          my $lpar_name_enc = urlencode($lpar);
          my $b_lpar        = $lpar;
          my ($lpar_alias) = grep /^LPAR:\Q$lpar\E/, @alias_servers;
          my $lpar_a       = "";
          my $alias        = "";
          my $alias_lpar   = "";
          my $rrm_stat = ( stat("$rrm_file_test") )[9];
          #if ( $rrm_stat > $last_update_time ) {
          #  $last_update_time = $rrm_stat;
          #}

          if ( defined $lpar_alias && $lpar_alias ne "" ) {
            chomp $lpar_alias;
            $lpar_alias =~ s/\\:/===backslash-col===/g;

            #print "$lpar_alias\n";
            ( undef, $lpar_a, $alias ) = split( /:/, $lpar_alias, 3 );
            $lpar_a =~ s/===backslash-col===/\\:/g;
            $alias =~ s/===backslash-col===/\\:/g;

            #print "\n,,$lpar_a,,$alias,,\n";
            chomp $alias;
            $alias =~ s/\:/===double-col===/g;
            $alias_lpar = " [$alias]";
          } ## end if ( defined $lpar_alias...)
          $lpar_a =~ s/replace/\\:/g;
          chomp $alias;
          $lpar =~ s/\\:/:/g;
          my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar, "" );
          $lpar_name =~ s/\\:/:/g;
          if ( $hmc =~ /no_hmc/ )       { next; }
          if ( $server =~ /--unknown/ ) { next; }
          $lpar =~ s/\:/===double-col===/g;
          $lpar_name_enc = urlencode($lpar);
          $lpar_name_enc =~ s/%3A/===double-col===/g;
          $lpar_name_enc =~ s/%3D%3D%3D/===/g;
          if ( $lpar eq "mem-pool" ) { next; }
          if ( $lpar eq "mem" )      { next; }
          if ( $lpar eq "pool" )     { next; }
          if ($rrm_stat > $year_back){
            $hash_timestamp1{$server}{$lpar}{$health_status}{$hmc}=$rrm_stat;
            #print "$health_status" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_lpar" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
          }
        } ## end if ( ( $lpar =~ /\.rrm$/...))
        $lpar_dir =~ s/\.rrd$|\.rrm$|\.rsd$|\.rsm$//;
        if ( !-d "$wrkdir/$server/$hmc/$lpar_dir" ) { next; }
        opendir( DIR, "$wrkdir/$server/$hmc/$lpar_dir" ) || error( "can't opendir $wrkdir/$server/$hmc/$lpar_dir: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @lpars_dir_wpar = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);

        #my @lpars_dir_wpar = <$wrkdir/$server_space/$hmc/$lpar/*>;
        foreach my $lpars_wpar_base (@lpars_dir_wpar) {
          my $lpars_wpar = "$wrkdir/$server/$hmc/$lpar_dir/$lpars_wpar_base";
          my $wpar       = basename($lpars_wpar);
          #print "$lpars_wpar\n";
          if ( !defined $hash_wpar{$lpars_wpar} ) {
            $hash_wpar{$lpars_wpar} = "test";
          }
          else {
            next;
          }
          if ( -f "$lpars_wpar/cpu.mmm" ) {
            chomp $lpars_wpar;
            #print "$lpars_wpar\n";
            my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar, $lpars_wpar );
            my $rrm_stat_w = ( stat("$lpars_wpar/cpu.mmm") )[9];
            my $wpar        = ( split( /\//, $lpars_wpar ) )[-1];
            my $lpar_a      = ( split( /\//, $lpars_wpar ) )[-2];
            my $hmc_wpar    = ( split( /\//, $lpars_wpar ) )[-3];
            my $server_wpar = ( split( /\//, $lpars_wpar ) )[-4];
            my $server_wpar_enc = urlencode($server_wpar);
            my $hmc_wpar_enc    = urlencode($hmc_wpar);
            my $wpar_name_enc   = urlencode($wpar);
            my $result_wpar_cfg = grep /:$wpar:/, @alias_servers_wpar;
            if ( $hmc =~ /no_hmc/ ) { next; }

            if ( $result_wpar_cfg == 0 ) {
              my $lpars_wpar_hash = "$lpars_wpar";
              $lpars_wpar_hash =~ s/$hmc\///g;
              $hash_timestamp1{$server}{$lpars_wpar_hash}{W}{$hmc}=$rrm_stat_w;
              #print "$wpar_menu" . "$hmc_wpar" . "$s" . "$server_wpar" . "$s" . "$wpar_name_enc" . "$s" . "$wpar" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_wpar_enc&server=$server_wpar_enc&lpar=$lpar_a--WPAR--$wpar_name_enc&item=lpar&entitle=0&gui=1&none=none:" . "$lpar_a" . "::P:$agent_type??\n";
            }
            else {
              foreach my $lpar_alias_wpar (@alias_servers_wpar) {    ######## FIND LPAR IN ALIAS.cfg
                $lpar_alias_wpar =~ s/\\:/replace/g;
                ( undef, my $lpar_b, my $alias ) = split( /:/, $lpar_alias_wpar );
                chomp $alias;
                if ( $hmc =~ /no_hmc/ ) { next; }
                $lpar_a =~ s/replace/\\:/g;
                if ( $lpar_b eq $wpar ) {                            ############ WPAR WITH ALIAS
                  my $lpars_wpar_hash = "$lpars_wpar";
                  $lpars_wpar_hash =~ s/$hmc\///g;
                  $hash_timestamp1{$server}{$lpars_wpar_hash}{W}{$hmc}=$rrm_stat_w;
                  #print "$wpar_menu" . "$hmc_wpar" . "$s" . "$server_wpar" . "$s" . "$wpar_name_enc" . "$s" . "$wpar" . " [$alias]" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_wpar_enc&server=$server_wpar_enc&lpar=$lpar_a--WPAR--$wpar_name_enc&item=lpar&entitle=0&gui=1&none=none:" . "$lpar_a" . "::P:$agent_type!!!\n";
                }
              } ## end foreach my $lpar_alias_wpar...
            } ## end else [ if ( $result_wpar_cfg ...)]
          } ## end if ( -f "$lpars_wpar/cpu.mmm")
        } ## end foreach my $lpars_wpar_base...

       wlmstat( $server, $lpar_dir );

      } ## end foreach my $lpars_dir_base ...
    } ## end foreach my $hmc_all_base (@hmcdir_all)
  } ## end foreach my $server_all (@wrkdir_all)
  #print Dumper \%hash_timestamp1;
  foreach my $server_a ( keys %hash_timestamp1 ) {
    #print "$server_a====\n";
    foreach my $lpar_a ( keys %{ $hash_timestamp1{$server_a} } ) {
      #print "$lpar\n";
      my $last_timestamp = 0;
      my $file = "";
      foreach my $health_status_a(keys %{ $hash_timestamp1{$server_a}{$lpar_a} }){
        foreach my $hmc_a ( keys %{ $hash_timestamp1{$server_a}{$lpar_a}{$health_status_a} }) {
          if ($hash_timestamp1{$server_a}{$lpar_a}{$health_status_a}{$hmc_a} > $last_timestamp){
            $file = "$server_a,$lpar_a,$hmc_a,$health_status_a";
            $last_timestamp = $hash_timestamp1{$server_a}{$lpar_a}{$health_status_a}{$hmc_a};
          }
        }
        #push(@lines_dualhmc1,"$file\n");
      }
      push(@lines_dualhmc1,"$file\n");
    }
  }
  ########################################## WPAR and LPAR - dual hmc parsing
  foreach my $line (@lines_dualhmc1){
    chomp $line;
    if (defined $line && $line eq ""){next}
    my ($server,$lpar,$hmc,$health_status) = split(/,/,$line);
    if ($health_status eq "W"){
      my $lpars_wpar = $lpar;
      my $wpar        = ( split( /\//, $lpars_wpar ) )[-1];
      my $lpar_a      = ( split( /\//, $lpars_wpar ) )[-2];
      #my $hmc_wpar    = ( split( /\//, $lpars_wpar ) )[-3];
      my $server_wpar = ( split( /\//, $lpars_wpar ) )[-3];
      #print "$wpar,$lpar_a,$server_wpar\n";
      my $server_wpar_enc = urlencode($server_wpar);
      my $hmc_wpar_enc    = urlencode($hmc);
      my $wpar_name_enc   = urlencode($wpar);
      my $result_wpar_cfg = grep /:$wpar:/, @alias_servers_wpar;
      my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar, $lpars_wpar );
      if ( $result_wpar_cfg == 0 ) {
        print "$wpar_menu" . "$hmc" . "$s" . "$server_wpar" . "$s" . "$wpar_name_enc" . "$s" . "$wpar" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_wpar_enc&server=$server_wpar_enc&lpar=$lpar_a--WPAR--$wpar_name_enc&item=lpar&entitle=0&gui=1&none=none:" . "$lpar_a" . "::P:W\n";
      }
      else {
        foreach my $lpar_alias_wpar (@alias_servers_wpar) {    ######## FIND LPAR IN ALIAS.cfg
          $lpar_alias_wpar =~ s/\\:/replace/g;
          ( undef, my $lpar_b, my $alias ) = split( /:/, $lpar_alias_wpar );
          chomp $alias;
          if ( $hmc =~ /no_hmc/ ) { next; }
          $lpar_a =~ s/replace/\\:/g;
          if ( $lpar_b eq $wpar ) {                            ############ WPAR WITH ALIAS
            print "$wpar_menu" . "$hmc" . "$s" . "$server_wpar" . "$s" . "$wpar_name_enc" . "$s" . "$wpar" . " [$alias]" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_wpar_enc&server=$server_wpar_enc&lpar=$lpar_a--WPAR--$wpar_name_enc&item=lpar&entitle=0&gui=1&none=none:" . "$lpar_a" . "::P:W\n";
          }
        } ## end foreach my $lpar_alias_wpar...
      }
    }else{
      my $server_enc    = urlencode($server);
      my $hmc_enc       = urlencode($hmc);
      my $lpar_name_enc = urlencode($lpar);
      my ($lpar_alias) = grep /^LPAR:\Q$lpar\E/, @alias_servers;
      my $lpar_a       = "";
      my $alias        = "";
      my $alias_lpar   = "";
      if ( defined $lpar_alias && $lpar_alias ne "" ) {
        chomp $lpar_alias;
        $lpar_alias =~ s/\\:/===backslash-col===/g;

        #print "$lpar_alias\n";
        ( undef, $lpar_a, $alias ) = split( /:/, $lpar_alias, 3 );
        $lpar_a =~ s/===backslash-col===/\\:/g;
        $alias =~ s/===backslash-col===/\\:/g;

        #print "\n,,$lpar_a,,$alias,,\n";
        chomp $alias;
        $alias =~ s/\:/===double-col===/g;
        $alias_lpar = " [$alias]";
      } ## end if ( defined $lpar_alias...)
      chomp $alias;
      $lpar =~ s/\\:/:/g;
      my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar, "" );
      $lpar =~ s/\:/===double-col===/g;
      $lpar_name_enc = urlencode($lpar);
      $lpar_name_enc =~ s/%3A/===double-col===/g;
      $lpar_name_enc =~ s/%3D%3D%3D/===/g;
      print "$health_status" . "$hmc" . "$s" . "$server" . "$s" . "$lpar_name_enc" . "$s" . "$lpar" . "$alias_lpar" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_enc&server=$server_enc&lpar=$lpar_name_enc&item=lpar&entitle=0&gui=1&none=none:::P:$agent_type\n";
      #print "$server,$lpar,$hmc,$lpar_alias\n";
    }
  }
  return 0;
} ## end sub print_all_server
#print "@lines_dualhmc1\n";

sub wlmstat {

	my $server   = shift;
	my $lpar_dir = shift;

	if ($server =~ "windows" || $server =~ "Hitachi" || $server =~ "Solaris" || $server =~ "vmware" || $server =~ "oVirt" || $server =~ "XEN" ) { return }
	if ( !-d "$wrkdir/$server" ) { return }
	opendir( HMC, "$wrkdir/$server" ) || error ( "Can't open $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && return;
	my @hmcs = grep !/^\.\.?$/, readdir(HMC);
	closedir(HMC);

	my %sort_hash;
	my @timestamp_arr;

	foreach my $hmc_tmp ( @hmcs ) {
		if ( -d "$wrkdir/$server/$hmc_tmp" ) {
			my $timestamp = ( stat("$wrkdir/$server/$hmc_tmp"))[9];
			$sort_hash{$timestamp}=$hmc_tmp;
			push @timestamp_arr, $timestamp;
		}
	}

	## sort timestamps in array -> $timestamp_arr_s[0] == last modified
	@timestamp_arr = sort { $b <=> $a } @timestamp_arr;

	my $hmc = $sort_hash{$timestamp_arr[0]};

	if ( !-d "$wrkdir/$server/$hmc/$lpar_dir" ) { return }
	opendir( DIR, "$wrkdir/$server/$hmc/$lpar_dir" ) || error( "can't opendir $wrkdir/$server/$hmc/$lpar_dir: $! :" . __FILE__ . ":" . __LINE__ ) && return;
	my @lpars_dir_wlm = grep !/^\.\.?$/, readdir(DIR);
	closedir(DIR);

	foreach my $lpars_wlm_base (@lpars_dir_wlm) {
		my $lpars_wlm = "$wrkdir/$server/$hmc/$lpar_dir/$lpars_wlm_base";
		my $wlm       = basename($lpars_wlm);
		if ( !defined $hash_wlm{$lpars_wlm} ) {
			$hash_wlm{$lpars_wlm} = "test";
		}
		else {
			next;
		}
		if ( -d $lpars_wlm ) { #same as wpar but test if dir has wlm- files
			if ( !-d "$lpars_wlm" ) { next }
			opendir( DIR, "$lpars_wlm" ) || error( "can't opendir $lpars_wlm: $! :" . __FILE__ . ":" . __LINE__ ) && next;
			my @lpars_dir_wlm = grep !/^\.\.?$/, readdir(DIR);
			closedir(DIR);
			@lpars_dir_wlm = sort @lpars_dir_wlm;
			if ( !defined $lpars_dir_wlm[0] ){next;}
			if ( $lpars_dir_wlm[0] !~ "wlm" || $lpars_dir_wlm[-1] !~ "wlm" ){last;}
			chomp $lpars_wlm;
			my $agent_type = find_agent_type( $wrkdir, $server, $hmc, $lpar_dir );
			my $wlm        = ( split( /\//, $lpars_wlm ) )[-1];
			my $lpar_a     = ( split( /\//, $lpars_wlm ) )[-2];
			my $hmc_wlm    = ( split( /\//, $lpars_wlm ) )[-3];
			my $server_wlm = ( split( /\//, $lpars_wlm ) )[-4];
			my $server_wlm_enc = urlencode($server_wlm);
			my $hmc_wlm_enc    = urlencode($hmc_wlm);
			my $wlm_name_enc   = urlencode($wlm);
			#my $result_wlm_cfg = grep /:$wlm:/, @alias_servers_wpar;
			if ( $hmc =~ /no_hmc/ ) { next; }
			print "$wlm_menu" . "$hmc_wlm" . "$s" . "$server_wlm" . "$s" . "$wlm_name_enc" . "$s" . "$wlm" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$hmc_wlm_enc&server=$server_wlm_enc&lpar=$lpar_a--WPAR--$wlm_name_enc&item=lpar&entitle=0&gui=1&none=none:" . "$lpar_a" . "::P:$agent_type\n";
		} ## end if ( -d $lpars_wlm)
	} ## end foreach my $lpars_wpar_base...
} ## end sub wlmstat

sub print_solaris_all {
  my $count_cpu = 0;
  foreach my $server_all (@wrkdir_all) {
    $server_all = "$wrkdir/$server_all";
    print "$server_all\n";
    my $server = basename($server_all);
    if ( -d "$wrkdir/Solaris/"){
      if ($server eq "Solaris"){
        chomp $server_all;
        chomp $server;
        opendir( DIR, "$wrkdir/$server" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @hmcdir_all = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);
        if (!@hmcdir_all){####### if exist only directory Solaris
          if (-d "$wrkdir/Solaris--unknown/"){
            opendir( DIR, "$wrkdir/Solaris--unknown" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
            my @hmcdir_all_1 = grep !/^\.\.?$/, readdir(DIR);
            closedir(DIR);
            foreach my $hmc_all_base_1 (@hmcdir_all_1) {
              my $hmc_all = "$wrkdir/Solaris--unknown/$hmc_all_base_1";
              my $hmc     = basename($hmc_all);
              my $item = "sol-ldom";
              opendir( DIR, "$wrkdir/Solaris--unknown/$hmc" ) || error( "can't opendir $wrkdir/Solaris--unknown/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next;
              my @no_ldom = grep !/^\.\.?$/, readdir(DIR);
              foreach my $no_ldom_file(@no_ldom){
                my $no_ldom     = basename($no_ldom_file);
                print "$actived" . "no_hmc" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=$no_ldom&item=$item&entitle=0&gui=1&none=none:::S:L\n";
              }
            }
          }
        }
        foreach my $hmc_all_base (@hmcdir_all) {
          my $hmc_all = "$wrkdir/$server/$hmc_all_base";
          my $hmc     = basename($hmc_all);
          my $item = "";
          if (-f"$hmc_all/solaris11.txt"){
            $item = "sol11";
          }else{
            $item = "sol10";
          }
          my $item_l = "sol-ldom";
          my $un = "";
          my $ldom_uuid = basename($hmc_all);
          if (-d "$wrkdir/$server/$ldom_uuid/ZONE"){
            opendir( DIR, "$wrkdir/$server/$ldom_uuid/ZONE" ) || error( "can't opendir $wrkdir/$server/$ldom_uuid: $! :" . __FILE__ . ":" . __LINE__ ) && next;
            my @only_zone_path = grep /total|global|system/, readdir(DIR);
            foreach my $only_zone(@only_zone_path){
              $only_zone =~ s/\.mmm//g;
              $un =~ "-$only_zone";
              if ($only_zone eq "total" && $count_cpu == 0){
                print "$actived" . "no_hmc" . "$s" . "$server" . "$s" . "cpuagg-sol" . "$s" . "Total" . "$s" . "/lpar2rrd-cgi/detail.sh?host=no_hmc&server=$server&lpar=cod&item=cpuagg-sol&entitle=0&gui=1&none=none:::S:P\n";
                $count_cpu++;

              }
              print "$actived" . "no_hmc" . "$s" . "$ldom_uuid" . "$s" . "$only_zone" . "$s" . "$only_zone" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$server&server=$ldom_uuid&lpar=$only_zone&item=$item$un&entitle=0&gui=1&none=none:::S:Z\n";
              #print "$actived" . "no_hmc" . "$s" . "$ldom_uuid" . "$s" . "$only_zone" . "$s" . "$only_zone" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$server&server=$ldom_uuid&lpar=$only_zone&item=$item$un$only_zone&entitle=0&gui=1&none=none:::S:Z\n";    old method
            }
          }
          if (-d "$wrkdir/$server/$ldom_uuid/ZONE"){
            opendir( DIR, "$wrkdir/$server/$ldom_uuid/ZONE" ) || error( "can't opendir $wrkdir/$server/$ldom_uuid: $! :" . __FILE__ . ":" . __LINE__ ) && next;
            my @only_zone_path = grep !/^\.\.?$|total|global|system/, readdir(DIR);
            foreach my $only_zone(@only_zone_path){
              $only_zone =~ s/\.mmm//g;
              $un = "-$only_zone";
              print "$actived" . "no_hmc" . "$s" . "$ldom_uuid" . "$s" . "$only_zone" . "$s" . "$only_zone" . "$s" . "/lpar2rrd-cgi/detail.sh?host=$server&server=$ldom_uuid&lpar=$only_zone&item=$item$un&entitle=0&gui=1&none=none:::S:Z\n";
            }
          }
          opendir( DIR, "$wrkdir/$server/$ldom_uuid" ) || error( "can't opendir $wrkdir/$server/$ldom_uuid: $! :" . __FILE__ . ":" . __LINE__ ) && next;
          rewinddir(DIR);
          my @ldom_path1 = grep /_ldom|net|san*/, readdir(DIR);
          closedir(DIR);
          if (@ldom_path1){
            #my $ldom_name = "";
            my $solaris_type = "";
            #print "$wrkdir/$server/$ldom_uuid/$ldom_uuid\n";
            my $string_ldom = "_ldom";
            if (-f "$wrkdir/$server/$ldom_uuid/$ldom_uuid$string_ldom.mmm"){
              $solaris_type = "L";
            }else{
              $solaris_type = "G";
            }
            my $menu_txt3 = "";
            my $menu_txt4 = "";
            my $ldom_file = $ldom_uuid;
            $ldom_file =~ s/\.mmm//g;
            my $ldom_name = "$ldom_file";
            $ldom_name =~ s/_ldom$//g;
            $ldom_name =~ s/\s+//g;
            print "$actived" . "no_hmc" . "$s" . "$ldom_name" . "$s" . "$ldom_name" . "$s" . "$ldom_name" . "$s" . "/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=$ldom_uuid&item=$item_l&entitle=0&gui=1&none=none:::S:$solaris_type\n";
          }
          else{###if exist Solaris directory - but has not some ldom or zones. Only uuid and solaris.txt
            if (-d "$wrkdir/Solaris--unknown/"){
              opendir( DIR, "$wrkdir/Solaris--unknown" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
              my @hmcdir_all_1 = grep !/^\.\.?$/, readdir(DIR);
              closedir(DIR);
              foreach my $hmc_all_base_1 (@hmcdir_all_1) {
                my $hmc_all = "$wrkdir/Solaris--unknown/$hmc_all_base_1";
                my $hmc     = basename($hmc_all);
                my $item = "sol-ldom";
                opendir( DIR, "$wrkdir/Solaris--unknown/$hmc" ) || error( "can't opendir $wrkdir/Solaris--unknown/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next;
                my @no_ldom = grep !/^\.\.?$/, readdir(DIR);
                foreach my $no_ldom_file(@no_ldom){
                  my $no_ldom     = basename($no_ldom_file);
                  if (-d "$wrkdir/Solaris--unknown/$no_ldom") {
                    opendir( DIR, "$wrkdir/Solaris/$no_ldom" ) || error( "can't opendir $wrkdir/Solaris/$no_ldom: $! :" . __FILE__ . ":" . __LINE__ ) && next;
                    my @ldom_path = grep /ldom|san|ZONE/, readdir(DIR);
                    if (@ldom_path){
                      print "$actived" . "no_hmc" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=$no_ldom&item=$item&entitle=0&gui=1&none=none:::S:G\n";
                    }
                  }
                }
              }
            }
          }
        }
      }
    }
    elsif (-d "$wrkdir/Solaris--unknown/"){
      if ($server eq "Solaris--unknown"){
        chomp $server_all;
        chomp $server;
        opendir( DIR, "$wrkdir/$server" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @hmcdir_all = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);
        foreach my $hmc_all_base (@hmcdir_all) {
          my $hmc_all = "$wrkdir/$server/$hmc_all_base";
          my $hmc     = basename($hmc_all);
          my $item = "sol-ldom";
          opendir( DIR, "$wrkdir/$server/$hmc" ) || error( "can't opendir $wrkdir/$server/$hmc: $! :" . __FILE__ . ":" . __LINE__ ) && next;
          my @no_ldom = grep !/^\.\.?$/, readdir(DIR);
          foreach my $no_ldom_file(@no_ldom){
            my $no_ldom     = basename($no_ldom_file);
            print "$actived" . "no_hmc" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "$no_ldom" . "$s" . "/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=$no_ldom&item=$item&entitle=0&gui=1&none=none:::S:G\n";
          }
        }
      }
    }
  }
}
############### VMWARE
sub print_vm_all {
  my $uuid;
  my $name_vm;
  my $list_of_vm = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my $vm_dir     = "$wrkdir/vmware_VMs";
  my ( $code, $linux_uuids ) = -f $agents_uuid_file ? Xorux_lib::read_json( $agents_uuid_file ) : ( 0, undef );

  opendir( DIR, "$vm_dir" ) || error( " directory does not exists : $vm_dir " . __FILE__ . ":" . __LINE__ ) && return 1;
  my @all_rrm_files = grep {/\.rrm$/} readdir(DIR);
  closedir(DIR);
  s/$_/$vm_dir\/$_/ for @all_rrm_files;
  if ( !defined $vm_list[0] ) {
    error( "Not exists $list_of_vm: $!" . __FILE__ . ":" . __LINE__ ) && return 1;
  }
  while ( my $server_all = shift(@wrkdir_all) ) {
    $server_all = "$wrkdir/$server_all";
    if ( !-d $server_all )               { next; }
    if ( -l $server_all )                { next; }
    if ( $server_all =~ /\/vmware_VMs/ ) { next; }
    my $server  = basename($server_all);
    my $hmc_dir = "$wrkdir/$server";
    opendir( DIR, "$hmc_dir" ) || error( " directory does not exists : $hmc_dir " . __FILE__ . ":" . __LINE__ ) && return 1;
    my @hmc_dir_all = grep !/^\.\.?$/, readdir(DIR);
    closedir(DIR);
    s/$_/$hmc_dir\/$_/ for @hmc_dir_all;

    while ( my $hmc_all = shift(@hmc_dir_all) ) {
      next if ( !-f "$hmc_all/vmware.txt" );
      next if ( !-f "$hmc_all/pool.rrm" );     # strange when not present
      my $file_time = ( stat("$hmc_all/pool.rrm"))[9];
      next if $file_time < $actual_last_30_days;

       # case: user moved esxi from one cluster/vcenter to another and later returned -> esxi & VMs were printed under both
       # the dir structure was the same as with two hmcs
       # trick to find the newest pool.rrm and only this to print
       # go through this while only once i.e. last; at the end

      my $newer_hmc = "$hmc_all";
      # print "652 ----------------- \@hmc_dir_all @hmc_dir_all ". scalar @hmc_dir_all ." $file_time\n";
      if (scalar @hmc_dir_all > 0) {
        foreach (@hmc_dir_all) {
          $newer_hmc = $_ if ( stat("$_/pool.rrm"))[9] > $file_time;
        }
        next if "$newer_hmc" ne "$hmc_all";
      }
      my $hmc          = basename($hmc_all);
      my $cluster_name = "$wrkdir/$server/$hmc/my_cluster_name";
      my $cluster_a;
      my $cluster;
      if ( -f $cluster_name ) {
        open( FC, "< $cluster_name" ) || error( "Cannot read $cluster_name: $!" . __FILE__ . ":" . __LINE__ );
        $cluster_a = <FC>;
        close(FC);
      }
      if ( defined $cluster_a && $cluster_a ne '' ) {
        ( $cluster, my $alias_vm ) = split( /\|/, $cluster_a );
        chomp $cluster;
      }
      else {
        $cluster = "";
      }
      $cluster =~ s/\:/===double-col===/g; # goes to menu.txt
      my $vm_list_info_file    = "$wrkdir/$server/$hmc/cpu.csv"; # from here we can test poweredOn or Off
      my @vm_list_info         = "";
      if ( -f "$vm_list_info_file" && open( FC, "< $vm_list_info_file")) {
        @vm_list_info = <FC>;
        close (FC);
      }
      else {
        error( "Cannot read $vm_list_info_file: $!" . __FILE__ . ":" . __LINE__ );
      }

      my $list_server_vm       = "$wrkdir/$server/$hmc/VM_hosting.vmh";
      my $vcenter_name_for_VM  = "";
      my $my_vcenter_name_file = "$wrkdir/$server/$hmc/my_vcenter_name";
      if ( -f "$my_vcenter_name_file" ) {
        open( FC, "< $my_vcenter_name_file" ) || error( "Cannot read $my_vcenter_name_file: $!" . __FILE__ . ":" . __LINE__ );
        my $my_vcenter_name = <FC>;
        chomp $my_vcenter_name;
        close(FC);
        ( undef, $vcenter_name_for_VM, undef ) = split( /\|/, $my_vcenter_name );
        if ( !defined $vcenter_name_for_VM ) {
          $vcenter_name_for_VM = "";
        }
      } ## end if ( -f "$my_vcenter_name_file")
      my @vm_list_server;    ###### START OR END VM
      if ( -e $list_server_vm ) {
        open( FC, "< $list_server_vm" ) || error( "Cannot read $list_server_vm: $!" . __FILE__ . ":" . __LINE__ ) && next;
        @vm_list_server = <FC>;
        close(FC);
      }
      foreach my $vm_servers (@vm_list_server) {
        chomp $vm_servers;
        if ( $vm_servers =~ /end=\d+$/ ) { next; }
        ( my $uuid_server, my $start, my $end ) = split( /:s/, $vm_servers );
        my $uuid_test = "";
        ($uuid_test) = grep /\Q$uuid_server/, @all_rrm_files;
        next if !defined $uuid_test;
        next if $uuid_test eq "";

        ($uuid_test) = grep /\Q$uuid_server/, @vm_list;
        my $uuid    = "";
        my $vm_name = "";
        my $linux_yes = "";
        if ( defined $uuid_test && $uuid_test ne "" ) {
          ( $uuid, $vm_name ) = split( /,/, $uuid_test );
          chomp $vm_name;
          if (-f "$wrkdir/Linux--unknown/no_hmc/$vm_name/uuid.txt") {
            $linux_yes = ":M";
          }
          else{
            chomp $uuid_test;
            #print "$uuid_test!!!\n";
            my (undef,undef,undef,$uuid_old,$vm_name_a) = split (/,/,$uuid_test);
            if ( $code && defined $uuid_old && $uuid_old ne ''
                 && defined $linux_uuids->{ Xorux_lib::uuid_big_endian_format( $uuid_old, '-' ) } ) {
              $linux_yes = ":M";
            }
            #         my ($testV) = grep /\Q$uuid_test/, @linux_uuid_files;
          }
        }
        chomp $start;
        $hmc =~ s/\:/===double-col===/g;
        $start =~ s/\d//g;
        if ( $uuid eq $uuid_server ) {
          my $alias_vmware = "";
          my $alias_test   = "";
          my $vm_name_a    = "";
          my $alias_vmw    = "";
          ($alias_test) = grep /\Q$vm_name/, @alias_vmware;
          if ( defined $alias_test && $alias_test ne "" ) {
            ( undef, $vm_name_a, $alias_vmw ) = split( /:/, $alias_test );
            chomp $alias_vmw;
            $alias_vmware = " [$alias_vmw]";
            chomp $alias_vmware;
          } ## end if ( defined $alias_test...)
          my $last_timestamp = ( stat("$wrkdir/vmware_VMs/$uuid_server.rrm") )[9];
          if ( $last_timestamp > $actual_last_30_days ) {
            # there can be same VM, same name, same uuid, in more vCenters but only one is poweredOn and only this should be printed
            # this means if VM is poweredOff and has actual data file timestamp so this should not be printed
            # actual is not older than 60 minutes
            (my $info_line) = grep /$uuid/,@vm_list_info;
            if ( defined $info_line && $info_line =~ "poweredOff" && ($last_timestamp + 3600) > $actual_unix_time ) {
              print STDERR "ERROR          : poweredOff & actual data file for $info_line: " . __FILE__ . ":" . __LINE__ . "\n" ;
              next;
            }
            $vm_name =~ s/\:/===double-col===/g;
            print "L:$cluster:$server:$uuid_server:$vm_name$alias_vmware:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$uuid_server&item=lpar&entitle=0&gui=1&none=none::$vcenter_name_for_VM:V$linux_yes\n";
          }
        } ## end if ( $uuid eq $uuid_server)
      } ## end foreach my $vm_servers (@vm_list_server)
      last;
    } ## end while ( my $hmc_all = shift...)
  } ## end while ( my $server_all = ...)
  return 0;
} ## end sub print_vm_all

############### URL ENCODING
sub urlencode {
  my $s = shift;

  # $s =~ s/ /+/g;
  $s =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

  # $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;

  # $s =~ s/\+/ /g;
  return $s;
} ## end sub urldecode

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time:ERROR: $text\n";
  return 1;
} ## end sub error

sub basename {
  return ( split "\/", $_[0] )[-1];
}

### Alerting support, each lpar/VM must have defined
# A: AIX or Linux on Power or VIOS without SEA
# B: AIX or Linux on Power or VIOS without SEA without SAN (well, it should not happen as at least san-sissas0.mmm or san-vscsi should be in place)
# C: AIX or Linux on Power without OS agenta --> only HMC data
# L: Linux OS agent or AIX or WPAR without HMC CPU
# M: Linux OS agent or AIX or Solaris or WPAR without HMC CPU & SAN
# I: AS400
# S: Solaris - no HMC CPU & SAN
# V: VIOS with SEA
# U: VIOS with SEA without SAN (well, it should not happen as at least san-sissas0.mmm should be in place)
# W: WPAR (cpu,mem,pg only)
# Y: AIX or Linux on Power with SAN but without SAN resp time  (especially Linuxes without iostat cmd installed)
# X: VMware

sub find_agent_type {
  my ( $wrkdir, $server, $hmc, $lpar, $wpar ) = @_;
  my $type     = "A";
  my $lpar_dir = "$wrkdir/$server/$hmc/$lpar";
  my $server_space = $server;
  if ( $server =~ m/ / ) {
    $server_space = "\"" . $server . "\"";    # it must be here to support space with server names
  }

  my $lpar_space = $lpar;
  if ( $lpar =~ m/ / ) {
    $lpar_space = "\"" . $lpar . "\"";        # it must be here to support space with server names
  }

  if ( ! -d "$lpar_dir" ) {
    return "C";
  }

  if ( $lpar =~ m/--AS400--$/ ) {
    return "I";
  }

  if ( $server =~ /^Solaris--unknown$/ ) {
    return "S";
  }

  if ( !$wpar eq '' ) {
    # WPAR
    return "W";
  }

  my @files = <$wrkdir/$server_space/$hmc/$lpar_space/sea*mmm>;
  foreach my $vios (@files) {
    my @vios_san = <$wrkdir/$server_space/$hmc/$lpar_space/san*mmm>;
    foreach my $vios_san (@vios_san) {
      return "V";
    }
    return "U";
  } ## end foreach my $vios (@files)

  @files = <$wrkdir/$server_space/$hmc/$lpar_space/san-*mmm>;
  foreach my $lpar_act (@files) {
    if ( -f "$lpar_dir.rrm" || -f "$lpar_dir.rrh" ) {
      @files = <$wrkdir/$server_space/$hmc/$lpar_space/san_resp*mmm>;
      foreach my $lpar_act (@files) {
        # AIX with SAN
        return "A";
      }
      # AIX/Linux on Power without installed iostat (no response time)
      return "Y";
    }
    else {
      #Linux (Solaris no as it cannot have SAN)
      @files = <$wrkdir/$server_space/$hmc/$lpar_space/san_resp*mmm>;
      foreach my $lpar_act (@files) {
        return "L";
      }
      # Linux without installed iostat (no response time)
      return "Y";
    }
  } ## end foreach my $lpar_act (@files)

  if ( -f "$lpar_dir/cpu.mmm" ) {
    if ( -f "$lpar_dir.rrm" || -f "$lpar_dir.rrh" ) {

      # AIX without SAN
      return "B";
    }
    else {
      #Linux (Solaris no as it cannot have SAN)
      return "M";
    }
  } ## end if ( -f "$lpar_dir/cpu.mmm")

  # nothing has been identified, expecting AIX without the agent
  return "C";
} ## end sub find_agent_type



sub print_hitachi_all {
  my $hitachi_dir = "$wrkdir/Hitachi";

  opendir( DIR, "$hitachi_dir" ) || error( " directory does not exists : $hitachi_dir " . __FILE__ . ":" . __LINE__ ) && return 1;
  my @server_dir = grep !/^\.\.?$/, readdir(DIR);
  closedir(DIR);

  foreach my $server (@server_dir) {
    if ( !-d "$hitachi_dir/$server" ) { next; }
    my $server_col = $server;
    $server_col =~ s /:/===double-col===/g;
    my $server_enc = urlencode( $server_col );
    my $active_lpar_file = "$hitachi_dir/$server/lpar_uuids.json";
    my ( $code, $active_lpars ) = -f $active_lpar_file ? Xorux_lib::read_json( $active_lpar_file ) : ( 0, undef );

    opendir( DIR, "$hitachi_dir/$server" ) || error( "can't opendir $hitachi_dir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @host_dir = readdir(DIR);
    closedir(DIR);

    my $print_entry =
      (sub {
	 my $menu_type = shift;
	 my $item = shift;
	 my $regex = shift;
	 my @filtered_files = grep /$regex/, @host_dir;

	 if ( scalar @filtered_files && $item =~ /(hitachi-lan)|(hitachi-san)/ ) {
	   my $total_name = $1 ? "lan-totals" : "san-totals";
           my $item_name = "hitachi-$total_name";
	   print "$menu_type:Hitachi:$server_col:$total_name:$total_name:/lpar2rrd-cgi/detail.sh?host=$server_enc&server=Hitachi&lpar=$total_name&item=$item_name&entitle=0&gui=1&none=none:::B\n";
	 }

	 foreach my $file ( @filtered_files ) {
	   my $filename = $file;
	   $filename =~ s/$regex//;

           if ( !-f "$hitachi_dir/$server/$file" || ( $code && !exists $active_lpars->{ $filename } ) ) {
             next;
           }

	   $filename =~ s/:/===double-col===/g;
	   my $filename_enc = urlencode($filename);

	   print "$menu_type:Hitachi:$server_col:$filename_enc:$filename:/lpar2rrd-cgi/detail.sh?host=$server_enc&server=Hitachi&lpar=$filename_enc&item=$item&entitle=0&gui=1&none=none:::B\n";
	 } ## foreach my $h_lpar (@san_dir_all)
       });

    $print_entry->("L", "lpar", qr/\.hlm$/);
    $print_entry->("N", "hitachi-lan", qr/\.hnm$/);
    $print_entry->("Y", "hitachi-san", qr/\.hhm$/);
  } ## end foreach my $server

  return 0;
} ## end sub print_hitachi_all


sub print_hyperv_all {
  my $data_dir = "$wrkdir/windows";
  opendir( DIR, "$data_dir" ) || error( " directory does not exists : $data_dir " . __FILE__ . ":" . __LINE__ ) && return 1;
   my @domain_dir = grep !/^\.\.?$/, readdir(DIR);
  closedir(DIR);

  # prepare text for testing nodes in clusters
  # trick to concatenate files for reading
  @ARGV = <"$wrkdir/windows/cluster_*/node_list.html">;
  #   print "found file \@ARGV ,@ARGV, for windows \n";
  # read all files from @ARGV
  my @node_lists = (<>) if defined $ARGV[0] && $ARGV[0] ne "";
  #   print "@text\n";

  foreach my $domain (@domain_dir) {
    # print "993 \$domain $domain\n";
    if ( !-d "$data_dir/$domain" ) { next; }
    if ( "$domain" !~ /^domain_/) { next; }
    if ( "$domain" =~ /^domain_UnDeFiNeD/) { next; } # sometimes is created when error in coming data

    #my $server_col = $server;
    #$server_col =~ s /:/===double-col===/g;
    #my $server_enc = urlencode( $server_col );

    # prepare info about all VMs in this domain
    my $domain_vm_uuid_file = "$data_dir/$domain/hyperv_VMs/vm_uuid_name.txt";
    if (! -f $domain_vm_uuid_file) { next }; # standalone workstationes without VMs

    my @domain_vm_list = ();
    if (open( FC, "< $domain_vm_uuid_file" )) {
      @domain_vm_list = <FC>;
      close(FC);
    }
    else {
      error( "Cannot read $domain_vm_uuid_file: $!" . __FILE__ . ":" . __LINE__ );
    }

    opendir( DIR, "$data_dir/$domain" ) || error( "can't opendir $data_dir/$domain: $! :" . __FILE__ . ":" . __LINE__ ) && next;
     my @server_dir = grep !/^\.\.?$/, readdir(DIR);
    closedir(DIR);
    # print "1005 \@server_dir @server_dir\n";

    foreach my $server (@server_dir) {
      if ( !-d "$data_dir/$domain/$server" ) { next; }
      if ( "$server" eq "hyperv_VMs") { next; }
      next if ( !-f "$data_dir/$domain/$server/pool.rrm" );     # strange when not present
      my $file_time = ( stat("$data_dir/$domain/$server/pool.rrm"))[9];
      next if $file_time < $actual_last_30_days;
      # print "1009 found $data_dir/$domain/$server\n";

      my $vm_dir     = "$data_dir/$domain/hyperv_VMs";
      opendir( DIR, "$vm_dir" ) || error( " directory does not exists : $vm_dir " . __FILE__ . ":" . __LINE__ ) && next;
       my @all_rrm_files = grep {/\.rrm$/} readdir(DIR);
      closedir(DIR);
      s/$_/$vm_dir\/$_/ for @all_rrm_files;

      my $server_path          = "$data_dir/$domain/$server";
      my $list_server_vm       = "$server_path/VM_hosting.vmh";
      my $vcenter_name_for_VM  = "";

      # find out if this node is in cluster
      my $this_cluster = "";
      my @matches =  grep { /$server/ } @node_lists;
      # print "@matches\n";
      if ((scalar @matches) > 1) {
        error( "server $server is in more ??? clusters @matches  : " . __FILE__ . ":" . __LINE__ );
      }
      if ((scalar @matches) > 0) {
        # <TR> <TD><B>MSNET-HVCL</B></TD> <TD align="center">domain_ad.xorux.com</TD> <TD align="center">HVNODE01</TD> <TD al ...
        $this_cluster = $matches[0];
        (undef, $this_cluster, undef) = split ( "\<B\>", $this_cluster);
        ($this_cluster, undef) = split ( "\<", $this_cluster);
      }

      #my $my_vcenter_name_file = "$wrkdir/$server/$hmc/my_vcenter_name";
      #if ( -f "$my_vcenter_name_file" ) {
      #  open( FC, "< $my_vcenter_name_file" ) || error( "Cannot read $my_vcenter_name_file: $!" . __FILE__ . ":" . __LINE__ );
      #  my $my_vcenter_name = <FC>;
      #  chomp $my_vcenter_name;
      #  close(FC);
      #  ( undef, $vcenter_name_for_VM, undef ) = split( /\|/, $my_vcenter_name );
      #  if ( !defined $vcenter_name_for_VM ) {
      #    $vcenter_name_for_VM = "";
      #  }
      #} ## end if ( -f "$my_vcenter_name_file")

      my @vm_list_server;    ###### START OR END VM
      if ( -f $list_server_vm && open( FC, "< $list_server_vm" )) {
        @vm_list_server = <FC>;
        close(FC);
      }
      else {
        # print "Server $domain/$server may not have any VMs\n";
        next;
      }
      # print "1056 \@all_rrm_files @all_rrm_files\n";
      foreach my $vm_servers (@vm_list_server) {
        chomp $vm_servers;
        # print "1059 \$vm_servers $vm_servers\n";
        if ( $vm_servers =~ /end=\d+$/ ) { next; }
        ( my $uuid_server, my $start, my $end ) = split( /:s/, $vm_servers );
        my $uuid_test = "";
        ($uuid_test) = grep /\Q$uuid_server/, @all_rrm_files;
        next if !defined $uuid_test;
        next if $uuid_test eq "";

        ($uuid_test) = grep /\Q$uuid_server/, @domain_vm_list;
        my $uuid    = "";
        my $vm_name = "";
        #my $linux_yes = "";
        #if ( defined $uuid_test && $uuid_test ne "" ) {
          ( $uuid, $vm_name ) = split( /,/, $uuid_test );
        #  chomp $vm_name;
        #  if (-f "$wrkdir/Linux--unknown/no_hmc/$vm_name/uuid.txt") {
        #    $linux_yes = ":M";
        #  }
        #  else{
        #    chomp $uuid_test;
        #    #print "$uuid_test!!!\n";
        #    my (undef,undef,undef,$uuid_old,$vm_name_a) = split (/,/,$uuid_test);
        #    if (defined $uuid_old && $uuid_old ne '') {
        #      my (undef,undef,undef,$first,$second) = split (/-/,$uuid_old);
        #      my $last_two_uuid = "$first-$second";
        #      my $big = uc($last_two_uuid);
        #      my $test = grep /$big/,@linux_uuid_files;
        #      if ($test == 1) { $linux_yes = ":M"; }
        #    }
        #    #         my ($testV) = grep /\Q$uuid_test/, @linux_uuid_files;
        #  }
        #}

        chomp $start;
        #$hmc =~ s/\:/===double-col===/g;
        $start =~ s/\d//g;
        #if ( $uuid eq $uuid_server ) {
          my $alias_vmware = "";
          my $alias_test   = "";
          my $vm_name_a    = "";
          my $alias_vmw    = "";
          #($alias_test) = grep /\Q$vm_name/, @alias_vmware;
          if ( defined $alias_test && $alias_test ne "" ) {
            ( undef, $vm_name_a, $alias_vmw ) = split( /:/, $alias_test );
            chomp $alias_vmw;
            $alias_vmware = " [$alias_vmw]";
            chomp $alias_vmware;
          } ## end if ( defined $alias_test...)
          my $last_timestamp = ( stat("$data_dir/$domain/hyperv_VMs/$uuid_server.rrm") )[9];
          if ( $last_timestamp > $actual_last_30_days ) {
            # there can be same VM, same name, same uuid, in more vCenters but only one is poweredOn and only this should be printed
            # this means if VM is poweredOff and has actual data file timestamp so this should not be printed
            # actual is not older than 60 minutes
            #(my $info_line) = grep /$uuid/,@vm_list_info;
            #if ( defined $info_line && $info_line =~ "poweredOff" && ($last_timestamp + 3600) > $actual_unix_time ) {
            #  print STDERR "ERROR          : poweredOff & actual data file for $info_line: " . __FILE__ . ":" . __LINE__ . "\n" ;
            #  next;
            #}
            $vm_name =~ s/\:/===double-col===/g;
            my $domain_without = $domain;
            $domain_without =~ s/^domain_//;
            print "L:$domain_without:$server:$uuid_server:$vm_name:/lpar2rrd-cgi/detail.sh?host=$server&server=windows/$domain&lpar=$uuid_server&item=lpar&entitle=0&gui=1&none=none:$this_cluster\::H\n";
            # print "L:$cluster:$server:$uuid_server:$vm_name$alias_vmware:/lpar2rrd-cgi/detail.sh?host=$hmc&server=$server&lpar=$uuid_server&item=lpar&entitle=0&gui=1&none=none::$vcenter_name_for_VM:V$linux_yes\n";
          }
        #} ## end if ( $uuid eq $uuid_server)
      } ## end foreach my $vm_servers (@vm_list_server)
      # last;
    } ## end foreach my $server

# L:cluster_New Cluster:10.22.11.8:500f3258-3b89-fcb5-e6cd-ec0258ee9f69:VyOS:/lpar2rrd-cgi/detail.sh?host=10.22.11.10&server=10.22.11.8&lpar=500f3258-3b89-fcb5-e6cd-ec0258ee9f69&item=lpar&entitle=0&gui=1&none=none::Hosting:V

    # print "$menu_type:Hitachi:$server_col:$total_name:$total_name:/lpar2rrd-cgi/detail.sh?host=$server_enc&server=Hitachi&lpar=$total_name&item=$item_name&entitle=0&gui=1&none=none:::B\n";

    # print "$menu_type:Hitachi:$server_col:$file_name_enc:$file_name:/lpar2rrd-cgi/detail.sh?host=$server_enc&server=Hitachi&lpar=$file_name_enc&item=$item&entitle=0&gui=1&none=none:::B\n";

  } ## end foreach my $domain

  return 0;
} ## end sub print_hyperv_all

################ look for Linux lpars uuid
sub uuids {
  my @linux_uuid_txt = `ls $no_hmc_dir/*/uuid.txt 2>/dev/null`;
  my $uuid_linux;
  my %uuid_to_name;

  foreach my $file (@linux_uuid_txt) {
    if (open( FH, " < $file" )) {
      $uuid_linux = <FH>;
      close FH;
      next if !defined $uuid_linux || $uuid_linux eq "";
      chomp ($uuid_linux,$file);
      $file =~ /\/no_hmc\/(.*)\/uuid\.txt$/;
      $uuid_to_name{ uc $uuid_linux } = $1;
    }
    else {
      error( "Cannot open $file: $!" . __FILE__ . ":" . __LINE__ );
      next;
    }
  }

  if ( -d $no_hmc_dir && scalar keys %uuid_to_name ) {
    Xorux_lib::write_json( $agents_uuid_file, \%uuid_to_name );
  }

  return 0;
}

sub write_json{
  my $path = shift;
  my $config=shift;       # hash holding configuration
  # my $json = JSON->new->utf8;    # this will produce compressed JSON - shorter, better for machine reading
  my $json = JSON->new->utf8->pretty;      # this will produce human readable files
  if (!defined $config || $config eq ""){
    return -1;
  }

  # write to JSON file
  if ( open( CFG, ">$path" ) ) {
    eval{
      print CFG $json->encode($config);
    };
    if (@_){
      print STDERR "Cannot write to $path\n";
      return -1;
    }
    close CFG;
  }
  else {
    error( "Couldn't open $path $!" . __FILE__ . ":" . __LINE__ );
  }
  return 1;
}

sub read_json{
  my $src=shift;
  if (!defined $src || $src eq ""){
    print "Path to json file(\"$src\") not defined, can't read it in function read_json()\n";
    return -1;
  }
  my $read;
  my $rawcfg;       # helper for file reading
  # read from JSON file
  if ( open( CFG, "$src" ) ) {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
    if ($rawcfg) {
      $read = decode_json($rawcfg);
    }
    close CFG;
    return $read;
  }
  else {
    # error handling
    error (__FILE__.":".__LINE__."Cannot read $src as .json file\n");
  }
}

sub reduce_double_hmc {
my $server = shift;
my $lpar   = shift;


}
