#!/usr/bin/perl
# generates JSON data structures
use strict;
use warnings;

use CGI::Carp qw(fatalsToBrowser);
use Storable;

use Data::Dumper;

my $DEBUG    = $ENV{DEBUG}             ||= "";
my $GUIDEBUG = $ENV{GUIDEBUG}          ||= "";
my $DEMO     = $ENV{DEMO}              ||= "";
my $BETA     = $ENV{BETA}              ||= "";
my $WLABEL   = $ENV{WLABEL}   ||= "";
my $TITLE    = $ENV{CUSTOM_PAGE_TITLE} ||= "";
my $errlog   = $ENV{ERRLOG};
my $basedir  = $ENV{INPUTDIR}          ||= "/home/lpar2rrd/lpar2rrd";
my $webdir   = $ENV{WEBDIR};
my $inputdir = $ENV{INPUTDIR};
my $dashb_rrdheight = $ENV{DASHB_RRDHEIGHT};
my $dashb_rrdwidth  = $ENV{DASHB_RRDWIDTH};
my $legend_height   = $ENV{LEGEND_HEIGHT};
my $alturl          = $ENV{WWW_ALTERNATE_URL};
my $alttmp          = $ENV{WWW_ALTERNATE_TMP};
my $uid             = $ENV{REMOTE_USER} ||= "";
my $listByHMC       = $ENV{LIST_BY_HMC};
my $pic_col         = $ENV{PICTURE_COLOR};
my $grpnames        = "";
my $aclCustAll      = 0;
my $isAdmin;
my $isReadonly;
my @aclCust;
my %aclLpars;
my %aclVMs;
my %aclSolos;

my $version;
if ( open( VER, "$basedir/etc/version.txt" ) ) {
  my @versions = <VER>;
  close VER;
  $version = pop(@versions);
  $version =~  s/ .*//;
  chomp $version;
} ## end if ( open( VER, "$basedir/etc/version.txt"...))
else {
  $version = $ENV{version};
}
my $prodname = "LPAR2RRD";

if ($WLABEL) {
  $prodname = $WLABEL;
}

print "Content-type: application/json\n\n";

my $md5module = 1;
eval "use Digest::MD5 qw(md5_hex); 1" or $md5module = 0;

if ( !$md5module ) {    # binary MD5 module not found, use bundled pure Perl one
  use Digest::Perl::MD5 qw(md5_hex);

  # print STDERR "Bundled MD5\n";
}

use JSON qw(encode_json decode_json);

use lib "../bin";

use CustomGroups;
use Alerting;
use Users;
use ACL;

my $useacl        = ACL::useACL;
my $aclAdminGroup = ACL::getAdminGroup;
my %sections      = ACL::getSections;

if ($useacl) {
  @aclCust = (ACL::getCustoms);
  if ( ACL::isAdmin || ( $aclCust[0] && $aclCust[0] eq "*" ) ) {
    $aclCustAll = 1;
  }

  %aclLpars   = ACL::getLpars;
  %aclVMs     = ACL::getVMs;
  %aclSolos   = ACL::getSolos;
  $grpnames   = ACL::getGroupsHtml();
  $isAdmin    = ACL::isAdmin();
  $isReadonly = ACL::isReadOnly();
} ## end if ($useacl)
else {
  $uid = "admin";
  $isAdmin = 1;
}

my @menu;    # array for complete menu json generation
my %idx;     # index file for menu entries
my %pack;    # parsed menu.txt
my %env;     # environment values

my %platform = (
  P => "POWER",
  V => "VMWARE",
  C => "CUSTOM",
  O => "OVIRT",
  X => "XENSERVER",
  H => "HYPERV",
  S => "SOLARIS",
  L => "LINUX",
  U => "UNMANAGED"
);

# set unbuffered stdout
#$| = 1;

#open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);

#print OUT "-- $QUERY_STRING\n" if $DEBUG == 2;

# `echo "QS $QUERY_STRING " >> /tmp/xx32`;
my ( $jsontype, $par1, $par2, $par3, $par4 ) = split( /&/, $QUERY_STRING );

$par1 = urldecode($par1);
$par2 = urldecode($par2);
$par3 = urldecode($par3);
$par4 = urldecode($par4);

if ( $jsontype eq "" ) {
  if (@ARGV) {
    $jsontype = "jsontype=" . $ARGV[0];

    #$basedir  = "..";
  }
  else {
    $jsontype = "jsontype=dump";
  }
} ## end if ( $jsontype eq "" )

$jsontype =~ s/jsontype=//;

if ( $jsontype eq "test" ) {

  #$basedir = "..";
  &test();
  exit;
} ## end if ( $jsontype eq "test")

if ( $jsontype eq "dump" ) {
  &dumpHTML();
  exit;
}

# CGI-BIN HTML header
#if ( !@ARGV ) {
#print "CContent-type: application/json\n\n";
#}

if ( $jsontype eq "menu" ) {
  if ( -r "$basedir/debug/menu.json" ) {
    open(FILE, "<$basedir/debug/menu.json") or
      die "Can't open file for input: $!";
    while (<FILE>) {
      print;
    }
    close(FILE);
  }
  else {
    &mainMenu(0);
    print JSON->new->utf8(0)->encode( \@menu );
  }

  # print Dumper @menu;
  # print encode_json (\@menu);
  exit;
} ## end if ( $jsontype eq "menu")
elsif ( $jsontype eq "menuh" ) {
  &mainMenu(1);

  #print Dumper \@menu;
  print JSON->new->utf8(0)->encode( \@menu );
  exit;
} ## end elsif ( $jsontype eq "menuh")
elsif ( $jsontype eq "defmenu" ) {
  &mainMenu(2);
  exit;
}
elsif ( $jsontype eq "lparsel" ) {
  &lparSelect();
  exit;
}
elsif ( $jsontype eq "aclitems" ) {
  &aclSelect();
  exit;
}
elsif ( $jsontype eq "lparselest" ) {
  &lparSelectEst();
  exit;
}
elsif ( $jsontype eq "hmcsel" ) {
  &hmcSelect();
  exit;
}
elsif ( $jsontype eq "powersel" ) {
  &print_all_models();
  exit;
}
elsif ( $jsontype eq "pools" ) {
  &poolsSelect();
  exit;
}
elsif ( $jsontype eq "estpools" ) {
  &poolsSelectEst();
  exit;
}
elsif ( $jsontype eq "lparnames" ) {
  &readMenu();
  $par1 =~ s/term=//;
  &lparNames($par1);
  exit;
} ## end elsif ( $jsontype eq "lparnames")
elsif ( $jsontype eq "histrep" ) {
  &readMenu();
  $par1 =~ s/hmc=//;
  $par2 =~ s/managedname=//;
  $par3 =~ s/type=//;
  $par4 =~ s/hostname=//;
  &histReport( $par1, $par2, $par3, $par4 );
  exit;
} ## end elsif ( $jsontype eq "histrep")
elsif ( $jsontype eq "env" ) {
  $env{version} = $version;
  &readMenu();
  &sysInfo();
  exit;
} ## end elsif ( $jsontype eq "env")
elsif ( $jsontype eq "pre" ) {
  &readMenu();
  &genPredefined();
  exit;
}
elsif ( $jsontype eq "cust" ) {
  &readMenu();
  &custGroupsSelect();
  exit;
}
elsif ( $jsontype eq "aclgrp" ) {
  &readMenu();
  &aclGroups();
  exit;
}
elsif ( $jsontype eq "fleet" ) {
  &readMenu();
  &genFleet();
  exit;
}
elsif ( $jsontype eq "fleetalrt" ) {
  &readMenu();
  &genFleetAlrt();
  exit;
}
elsif ( $jsontype eq "fleetrpt" ) {
  &readMenu();
  &genFleetReport();
  exit;
}
elsif ( $jsontype eq "fleetree" ) {
  &readMenu();
  &genFleetTree();
  exit;
}
elsif ( $jsontype eq "fleetall" ) {
  &readMenu();
  &genFleetAll();
  exit;
}
elsif ( $jsontype eq "custgrps" ) {
  &genCustGrps();
  exit;
}
elsif ( $jsontype eq "alrttree" ) {
  &readMenu();
  &genAlertTree();
  exit;
}
elsif ( $jsontype eq "alrtgrptree" ) {
  &genAlertGroupTree();
  exit;
}
elsif ( $jsontype eq "alrttimetree" ) {
  &genAlertTimeTree();
  exit;
}
elsif ( $jsontype eq "clusters" ) {
  $par1 =~ s/vc=//;
  &genClusters($par1);
  exit;
}
elsif ( $jsontype eq "datastores" ) {
  $par1 =~ s/vc=//;
  &genDataStores($par1);
  exit;
}
elsif ( $jsontype eq "respools" ) {
  $par1 =~ s/vc=//;
  &genResPools($par1);
  exit;
}
elsif ( $jsontype eq "vms" ) {
  $par1 =~ s/vc=//;
  &genVMs($par1);
  exit;
}
elsif ( $jsontype eq "metrics" ) {
  &getMetrics();
  exit;
}
elsif ( $jsontype eq "about" ) {
  &about();
  exit;
}

sub sysInfo {
  my @files;
  my $ostype;
  my $sideMenuWidth = "";
  my $vmImage;
  if ( $^O eq "aix" ) {
    @files = grep { !/\.xh$/ } <$basedir/agent/*aix*>;
    $ostype = "AIX";
  }
  elsif ( $^O eq "linux" ) {
    @files = grep { !/\.xh$/ } <$basedir/agent/*noarch*>;
    $ostype = "Linux";
  }
  elsif ( $^O eq "linux" ) {
    @files = grep { !/\.xh$/ } <$basedir/agent/*Linux.ppc*>;
    $ostype = "Linux PPC";
  }
  if ( exists $ENV{'SIDE_MENU_WIDTH'} ) {
    $sideMenuWidth = $ENV{'SIDE_MENU_WIDTH'};
  }
  if ( $ENV{'VM_IMAGE'} ) {
    $vmImage = 1;
  }
  my $sshcmd = $ENV{SSH} ||= "";

  no warnings 'uninitialized';
  print "{\n";    # envelope begin
  print "\"version\":\"$env{version}\",\n";
  print "\"free\":\"$env{free}\",\n";
  print "\"entitle\":\"$env{entitle}\",\n";
  print "\"dashb_rrdheight\":\"$dashb_rrdheight\",\n";
  print "\"dashb_rrdwidth\":\"$dashb_rrdwidth\",\n";
  print "\"legend_height\":\"$legend_height\",\n";
  print "\"picture_color\":\"$pic_col\",\n";
  print "\"guidebug\":\"$GUIDEBUG\",\n";
  print "\"demo\":\"$DEMO\",\n";
  print "\"beta\":\"$BETA\",\n";
  print "\"custom_page_title\":\"$TITLE\",\n";
  print "\"rpm\":\"$files[0]\",\n";
  print "\"ostype\":\"$ostype\",\n";
  print "\"basedir\":\"$basedir\",\n";
  print "\"wlabel\":\"$WLABEL\",\n";
  print "\"basename\":". &boolean($env{free} eq "1" ? 1 : 0) . ",\n";
  print "\"listbyhmc\":" . &boolean($listByHMC) . ",\n";
  print "\"hasPower\":" . &boolean( $env{platform}{power} ) . ",\n";
  print "\"hasVMware\":" . &boolean( $env{platform}{vmware} && !$ENV{EXCLUDE_VMWARE} ) . ",\n";
  print "\"useRestAPIExclusion\":" . &boolean( $ENV{USE_RESTAPI_EXCLUSION} ) . ",\n";
  print "\"hasVMwareAgent\":" . &boolean( $env{hasVMwareAgent} ) . ",\n";
  print "\"hasSolaris\":" . &boolean( $env{platform}{solaris} ) . ",\n";
  print "\"hasXen\":" . &boolean( $env{platform}{xen} ) . ",\n";
  print "\"hasOVirt\":" . &boolean( $env{platform}{ovirt} ) . ",\n";
  print "\"hasLinux\":" . &boolean( $env{platform}{linux} ) . ",\n";
  print "\"hasUnmanaged\":" . &boolean( $env{platform}{unmanaged} ) . ",\n";
  print "\"vmImage\":" . &boolean($vmImage) . ",\n";
  print "\"uid\":\"$uid\",\n";
  print "\"gid\":\"$grpnames\",\n";
  print "\"sideMenuWidth\":\"$sideMenuWidth\",\n";
  print "\"isAdmin\":" . &boolean( $isAdmin ) . ",\n";
  print "\"useACL\":" . &boolean( $useacl ) . ",\n";
  print "\"aclAdminGroup\":\"$aclAdminGroup\",\n";
  print "\"sshcmd\":\"$sshcmd\"\n";
  print "}\n";    # envelope end
} ## end sub sysInfo

sub dumpHTML {

  #use Data::Dumper;
  print "Content-type: application/octet-stream\n";
  print("Content-Disposition:attachment;filename=debug.txt\n\n");
  my $buffer;
  if ( $ENV{'CONTENT_LENGTH'} ) {
    read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
    if ($buffer) {
      my @pairs = split( /&/, $buffer );

      my @q = split( /=/, $pairs[1] );
      my $html = urldecode( $q[1] );
      print $html;
    } ## end if ($buffer)
  } ## end if ( $ENV{'CONTENT_LENGTH'...})
  mainMenu(0);

  # print encode_json ( \@menu );

  # print Dumper @menu;

  #use CGI;
  #use CGI('header');
  #print header(-type=>'application/octet-stream',
  #       -attachment=>'debug.txt');
  #my $q = new CGI;
  #print $q->param('tosave');

} ## end sub dumpHTML

sub test {
  print "Content-type: text/plain\n\n";

  use Data::Dumper;
  # my $Dumper = new Data::Dumper;
  # Data::Dumper->import();
  # import Data::Dumper qw(Dumper);

  #print Dumper @aclServ;
  #print Dumper @aclCust;
  #print Dumper \%aclLpars;
  &readMenu();
  print Dumper \%pack;
  print Dumper \%env;
  # my %alerts = Alerting::getAlerts();
  # print Dumper \%alerts;
  print Dumper ACL::getGroups();
  print Dumper ACL::isAdmin();
  #my $d = Data::Dumper->new(*$pack{lstree});
  #print $d->Dump();
  #print $Dumper->Dump( \$pack{lhmc} );
  # print Dumper \$pack{fleet};
  # print Dumper \$pack{lnames};
  #print Dumper $pack{ctree};
  #print Dumper \$pack{times};
  #print Dumper \$pack{vclusters};
  # print Dumper $pack{hyperv};
  #  &mainMenu(0);
  #  print Dumper \@menu;
  # &genHMCs;
  # print encode_json( \$pack{lstree} );
  #	print ($pack{ctreeh} > 0 ? "true" : "false" );
} ## end sub test

sub mainMenu {

  # local *STDOUT;
  # open STDOUT, ">", \$x or die "Can't open STDOUT: $!\n";
  my $showHMC = shift;
  &readMenu( 0, $showHMC );
  my $variant = $env{variant} eq "p" ? "<span style='font-weight: normal'>(free)</span>" : "";
  my $hash = substr( md5_hex("DASHBOARD"), 0, 7 );
  ### Generate JSON
  push @menu, { "title" => "DASHBOARD", "extraClasses" => "boldmenu", "href" => "dashboard.html", "hash" => "$hash" };
  &custs();
  $hash = substr( md5_hex("Reporter"), 0, 7 );
  push @menu, { "title" => "Reporter", "extraClasses" => "boldmenu", "href" => "/lpar2rrd-cgi/reporter.sh?cmd=form", "hash" => "$hash" };

  #if ( !$useacl || $isAdmin ) {
    #$hash = substr( md5_hex("alert"), 0, 7 );
    #push @menu, { "title" => "Alerting", "extraClasses" => "boldmenu", "href" => "/lpar2rrd-cgi/alcfg.sh?cmd=form", "hash" => "$hash" };
  #}
  if ( !$useacl || $isAdmin || $isReadonly ) {
    &globalWoTitle;
  }
  if ( $env{platform}{power} ) {
    if ( scalar keys %{ $env{platform} } > 1 ) {
      $idx{psrv} = push @menu, { "title" => "IBM Power Systems", "folder" => "true", "children" => [] };
      $idx{psrv}--;
      if ( !$useacl || $isAdmin || $isReadonly || $sections{power} ) {
        for ( sort keys %{ $pack{global}{P} } ) {
          my $t  = $pack{global}{P}{$_}{text};
          my $u  = $pack{global}{P}{$_}{url};
          my $id = $pack{global}{P}{$_}{id};
          if ( lc $t eq "custom groups" || lc $t eq "alerting" ) {
            next;
          }
          if ( lc $t eq "cpu workload estimator"
            || lc $t eq "resource configuration advisor"
            || lc $t eq "heatmap" )
          {
            push @{ $menu[ $idx{psrv} ]{children} }, pushmenub( $t, $u, $id );
          } ## end if ( lc $t eq "cpu workload estimator"...)
          else {
            push @{ $menu[ $idx{psrv} ]{children} }, pushmenu( $t, $u, $id );
          }
        } ## end for ( sort keys %{ $pack...})
        &globalHmcTotals();
      } ## end if ( !$useacl || $aclServAll)
    } ## end if ( $env{platform}{vmware} &&...)
    else {
      if ($showHMC) {
        $idx{psrv} = push @menu, { "title" => "HMC", "folder" => "true", "children" => [] };
        $idx{psrv}--;
      }
      else {
        $idx{psrv} = push @menu, { "title" => "SERVER", "folder" => "true", "children" => [] };
        $idx{psrv}--;
      }
    } ## end else [ if ( $env{platform}{vmware} &&...)]
    if ($showHMC) {
      &genHMCs();    # List by HMCs
    }
    else {
      &genServers();    # List by Servers
    }
  } ## end if ( $env{platform}{power} )
  if ( $env{platform}{vmware} && !$ENV{EXCLUDE_VMWARE} ) {
    if ( !$idx{vmware} ) {
      $idx{vmware} = push @menu, { "title" => "VMware", "folder" => "true", "children" => [] };
      $idx{vmware}--;
    }
    if ( scalar keys %{ $env{platform} } > 1 ) {
      if ( !$useacl || $isAdmin || $isReadonly || $sections{vmware} ) {
        for ( sort keys %{ $pack{global}{V} } ) {
          my $t  = $pack{global}{V}{$_}{text};
          my $u  = $pack{global}{V}{$_}{url};
          my $id = $pack{global}{V}{$_}{id};
          {
            if ( lc $t eq "cpu workload estimator"
              || lc $t eq "resource configuration advisor"
              || lc $t eq "heatmap" )
            {
              push @{ $menu[ $idx{vmware} ]{children} }, pushmenub( $t, $u, $id );
            } ## end if ( lc $t eq "cpu workload estimator"...)
            else {
              push @{ $menu[ $idx{vmware} ]{children} }, pushmenu( $t, $u, $id );
            }
          }
        } ## end for ( sort keys %{ $pack...})
      } ## end if ( !$useacl || $aclServAll)

    } ## end if ( $env{platform}{power} )
        # print STDERR "XXXXX\n";
    &genVMware;
  } ## end if ( $env{platform}{vmware} &&...)
  if ( ! $env{platform} ) {
    for ( sort keys %{ $pack{global}{P} } ) {
      my $t  = $pack{global}{P}{$_}{text};
      my $u  = $pack{global}{P}{$_}{url};
      my $id = $pack{global}{P}{$_}{id};
      if ( lc $t eq "custom groups" || lc $t eq "alerting" ) {
        next;
      }
      if ( lc $t eq "cpu workload estimator"
        || lc $t eq "resource configuration advisor"
        || lc $t eq "heatmap" )
      {
        push @menu, pushmenub( $t, $u, $id );
      } ## end if ( lc $t eq "cpu workload estimator"...)
      else {
        push @menu, pushmenu( $t, $u, $id );
      }
    } ## end for ( sort keys %{ $pack...})
  } ## end if ( !$env{platform}{vmware} ...)
  if ( $env{platform}{blade} ) {
    &genHitachi(1);
  }
  if ( $env{platform}{solaris} ) {
    &genSolaris(1);
  }
  if ( $pack{xen} ) {
    $idx{xen} = push @menu, $pack{xen};
    $idx{xen}--;
  }
  if ( $pack{ovirt} ) {
    $idx{ovirt} = push @menu, $pack{ovirt};
    $idx{ovirt}--;
  }
  if ( $env{platform}{hyperv} ) {
    &genHyperV(1);
  }
  if ( $env{platform}{linux} ) {
    &genLinux(1);
  }
  if ( $env{platform}{unmanaged} ) {
    &genStandalone(1);    # List by Servers
  }

  #	&genHMCs ();  # List by HMCs

  # &tail();
  ### End of JSON
} ## end sub mainMenu

sub lparSelect {
  &readMenu();
  ### Generate JSON
  print "[\n";      # envelope begin
  &genLpars();      # List by Servers
  print "\n]\n";    # envelope end
  ### End of JSON
} ## end sub lparSelect

sub lparSelectEst {
  &readMenu();
  ### Generate JSON
  print "[\n";       # envelope begin
  &genLparsEst();    # List by Servers
  print "\n]\n";     # envelope end
  ### End of JSON
} ## end sub lparSelectEst

sub hmcSelect {
  &readMenu();
  ### Generate JSON
  print "[\n";        # envelope begin
  &genHmcSelect();    # List by HMCs
  print "\n]\n";      # envelope end
  ### End of JSON
} ## end sub hmcSelect

sub poolsSelect {
  &readMenu();
  ### Generate JSON
  print "[\n";        # envelope begin
  &genPools();        # generate list of Pools
  print "\n]\n";      # envelope end
  ### End of JSON
} ## end sub poolsSelect

sub poolsSelectEst {
  &readMenu();
  ### Generate JSON
  print "[\n";        # envelope begin
  &genPoolsEst();     # generate list of Pools
  print "\n]\n";      # envelope end
  ### End of JSON
} ## end sub poolsSelectEst

sub custGroupsSelect {
  ### Generate JSON
  print "[\n";        # envelope begin
  &genCusts();        #
  print "\n]\n";      # envelope end
  ### End of JSON
} ## end sub custGroupsSelect

sub aclGroups {
  ### Generate JSON
  print "[\n";        # envelope begin
  if ($useacl) {
    my $delim = '';
    for ( ACL::getCfgGroups() ) {
      if ( $_ ne $aclAdminGroup ) {
        print $delim . "{\"title\":\"$_\"}";
        $delim = ",\n";
      }
    } ## end for ( ACL::getCfgGroups...)
  } ## end if ($useacl)
  print "\n]\n";      # envelope end
  ### End of JSON
} ## end sub aclGroups

sub readMenu {
  my $alt     = shift;
  my $defmenu = shift;
  my $tmppath = "tmp";
  if ($alturl) {
    my @parts = split( /\//, $ENV{HTTP_REFERER} );
    if ( $ENV{HTTP_REFERER}
      && ( $parts[-1] ne $alturl )
      && ( $parts[-2] ne $alturl ) )
    {    # URL doesn't match WWW_ALTERNATE_URL
      $tmppath = $alttmp;    # use menu from WWW_ALTERNATE_TMP
    } ## end if ( $ENV{HTTP_REFERER...})
    if ($alt) {
      $tmppath = $alttmp;
    }
  } ## end if ($alturl)

  my $skel;
  if ( -r "$basedir/debug/menu.txt" ) {
    $skel = file_read("$basedir/debug/menu.txt");
  }
  elsif ( !-r "$basedir/$tmppath/menu.txt" || defined $defmenu && $defmenu == 2 ) {
    $skel = file_read("$basedir/html/menu_default.txt");
  }
  else {
    $skel = file_read("$basedir/$tmppath/menu.txt");
  }

  #if ( !-r $skel ) {

  ## print "[{\"title\":\"SKEL: Cannot open file $skel: $!\"}]";
  #print STDERR "Cannot open file $skel: $!\n";
  #exit;
  #} ## end if ( !-r $skel )
  #open( SKEL, $skel );

  foreach my $line ( split /^/, $skel ) {
    my ( $hmc, $srv, $txt, $url );
    chomp $line;
    my @val = split( ':', $line );
    for (@val) {
      &collons($_);
    }
    {
      "O" eq $val[0] && do {
        $env{free} = ( $val[1] == 1 ) ? 1 : ( $val[2] ? $val[2] : 0 );
        $env{variant} = $val[2] ||= "";
        last;
      };
      "E" eq $val[0] && do {
        $env{entitle} = ( $val[1] == 1 ) ? 1 : 0;
        last;
      };
      "G" eq $val[0] && do {
        my ( $id, $txt, $url, $type ) = ( $val[1], $val[2], $val[3], $val[8] );
        if ( $txt eq "HMC totals" ) { last; }
        if ( $txt eq "FAVOURITES" ) {
          $type = "";
        }
        my $cnt = keys %{ $pack{global}{$type} };
        $pack{global}{$type}{$cnt}{text} = $txt;
        $pack{global}{$type}{$cnt}{url}  = $url;
        $pack{global}{$type}{$cnt}{id}   = $id;
        last;
      };
      "F" eq $val[0] && do {
        my ( $txt, $url ) = ( $val[2], $val[3] );
        push @{ $pack{ftreeh} }, [ $txt, $url ];
        last;
      };
      "C" eq $val[0] && do {
        my ( $txt, $url, $type ) = ( $val[2], $val[3], $val[8] );
        if ( !$useacl || $aclCustAll || ACL::canShow( "", "CUSTOM", $txt ) ) {
          if ( $txt ne "<b>Configuration</b>" ) {
            push @{ $pack{ctree} }, [ $txt, $url, $type ];
            $pack{ctreeh}{$txt} = $url;
          }
        } ## end if ( !$useacl || $aclCustAll...)
        last;
      };
      "H" eq $val[0] && do {
        my ( $id, $txt, $url, $type ) = ( $val[1], $val[2], $val[3], $val[8] );
        $pack{hmc}{$txt}{url} = $url;
        $pack{hmc}{$txt}{id}  = $id;
        if ($type) {
          $pack{hmc}{$txt}{type} = $type;
        }
        last;
      };

      #S:ahmc11:BSRV21:CPUpool-pool:CPU pool:ahmc11/BSRV21/pool/gui-cpu.html::1399967748
      "S" eq $val[0] && do {
        my ( $hmc, $srv, $pool, $txt, $url, $alias, $timestamp, $type ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[6], $val[7], $val[8] );
        my $onList = ACL::canShow( $platform{$type}, "SERVER", $srv, "" );
        $onList = 1;
        if ( !$useacl || $isAdmin || $isReadonly || $onList ) {
          if ( !$timestamp ) {
            $timestamp = 999;
          }
          if ( !exists $pack{times}{$srv}{timestamp}{$hmc} ) {
            $pack{times}{$srv}{timestamp}{$hmc} = $timestamp;
          }
          if ($type eq "X" ) {
            $env{platform}{xen} ||= 1;
            $hmc = $pool;
            $srv = $val[1];
            $pack{xen}{pool}{$srv}{label}{$hmc}{text} = $txt;
            $pack{xen}{pool}{$srv}{label}{$hmc}{url} = $url;
          }
          elsif ( $type eq "V" ) {
            $env{platform}{vmware} ||= 1;
            $pack{vclusters}{$hmc}{$srv} = 1;
            my $cluster = $hmc;
            if ( $hmc =~ /^cluster_/ ) {
              $hmc = ( $url =~ /host=([^&]*)/ )[0];
            }
            if ( !exists $pack{vtree}{$hmc}{cluster}{$cluster}{host}{$srv} ) {
              $pack{vtree}{$hmc}{cluster}{$cluster}{host}{$srv} = 1;
            }
          } ## end if ( $type eq "V" )
          elsif ( $type eq "P" ) {
            $env{platform}{power} ||= 1;
          }
          elsif ( $type eq "B" ) {
            $env{platform}{blade} ||= 1;
          }
          elsif ( $type eq "H" ) {
            $env{platform}{hyperv} ||= 1;
            $pack{hyperv}{dom}{$hmc}{server}{$srv}{totals}{$txt} = $url;
          }
          if ( !$pack{times}{$srv}{"active"} ) {
            $pack{times}{$srv}{"active"} = 1;
          }
          $pack{times}{$srv}{"removed"}{$hmc} = 0;
          if ($type) {
            $pack{times}{$srv}{type} = $type;
          }
          if ($alias) {
            $pack{times}{$srv}{alias} = $alias;
          }
          if ( $pool =~ /^CPUpool/ ) {
            $pool = substr( $pool, 8 );
          }
          elsif ( $pool eq "pagingagg" || $pool eq "mem" ) {
            $pool = "cod";
          }
          elsif ( $pool eq "hea" ) {
            $pool = "hea";
          }
          else {
            $pool = "";
          }
          push @{ $pack{stree}{$srv}{$hmc} }, [ $txt, $url, $pool ];
          if ( $url =~ /item=(sh)?pool/ ) {
            my $type = ( $url =~ /item=([^&]*)/ )[0];
            my $poolname = ( split " : ", $txt )[1];
            $poolname ||= "CPU pool";
            $pack{fleet}{$srv}{subsys}{POOL}{$poolname}{value} = $pool;
            $pack{fleet}{$srv}{subsys}{POOL}{$poolname}{type} = $type;
          }
        } ## end if ( !$useacl || $aclServAll...)
        last;
      };
      "D" eq $val[0] && do {
        my ( $hmc, $srv, $pool, $txt, $url, $timestamp, $type ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8] );
        if ( !$useacl || $isAdmin || $isReadonly ) {
          if ( !$timestamp ) {
            $timestamp = 999;
          }
          if ( $type ne "V" && !exists $pack{times}{$srv}{timestamp}{$hmc} ) {
            $pack{times}{$srv}{timestamp}{$hmc} = $timestamp;
          }
          if ( !$pack{times}{$srv}{active} ) {
            $pack{times}{$srv}{"active"} = 0;
          }
          $pack{times}{$srv}{"removed"}{$hmc} = 1;
          if ($type) {
            $pack{times}{$srv}{type} = $type;
          }
          if ( $type eq "V" ) {
            $env{platform}{vmware} ||= 1;
            $pack{vclusters}{$hmc}{$srv} = 1;
            my $cluster = $hmc;
            if ( $hmc =~ /^cluster_/ ) {
              $hmc = ( $url =~ /host=([^&]*)/ )[0];
            }
            if ( !exists $pack{vtree}{$hmc}{cluster}{$cluster}{host}{$srv} ) {
              $pack{vtree}{$hmc}{cluster}{$cluster}{host}{$srv} = 1;
            }
          } ## end if ( $type eq "V" )
          if ( $pool =~ /^CPUpool/ ) {
            $pool = substr( $pool, 8 );
          }
          else {
            $pool = "";
          }
          push @{ $pack{stree}{$srv}{$hmc} }, [ $txt, $url, $pool ];
        } ## end if ( !$useacl || $aclServAll...)
        last;
      };
      "L" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        my $onList = 0;
        if ( $type eq "P" ) {
          if ( $hmc eq "no_hmc" ) {
            $type = "U";
            $onList = ACL::canShow( $platform{$type}, "SERVER", $srv, $txt );
          }
          else {
            $onList = ACL::canShow( $platform{$type}, "LPAR", $srv, $atxt );
          }
        } ## end if ( $type eq "P" )
        elsif ( $type eq "V" ) {
          if ( $hmc eq "no_hmc" ) {
            $type = "U";
          }
          $onList = ACL::canShow( $platform{$type}, "VM", $hmc, $txt );
        }
        elsif ( $type eq "S" || $type eq "B" || $type eq "X" ) {
          $onList = 1;
          # $hmc = "";
        }
        if ( $type eq "X" ) {
          $hmc = "XEN";
          $srv = "XEN";
        }

        #if (! exists $pack{hmc}{$hmc}) {
        #if ($type) {
        #$pack{hmc}{$hmc}{type} = $type;
        #}
        #}
        if ( !$useacl || $isAdmin || $isReadonly || $onList ) {
          $atxt = urldecode($atxt);
          if ( $type ne "S" && $type ne "H" ) {
            if ( $type ne "P" ) {
              push @{ $pack{lstree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
            }
            push @{ $pack{lhmc}{$hmc}{$srv} }, [ $txt, $url, $atxt ];
          }
          if ( ( $hmc eq "no_hmc" )
              or ( !exists $pack{times}{$srv}{timestamp}{$hmc} ) )
          {
            if ( $srv ne "Linux" ) {
              $pack{times}{$srv}{timestamp}{$hmc} = 999;
            }
          }
          if ( $hmc eq "no_hmc" && !$pack{times}{$srv}{active} ) {
            if ( $srv ne "Linux" && $type eq "U" ) {
              $pack{times}{$srv}{"active"} = 1;
              $env{platform}{unmanaged} ||= 1;
            }
          }
          if ( !exists $pack{times}{$srv}{active} ) {
            $pack{times}{$srv}{"active"} = 1;
          }
          if ( !exists $pack{times}{$srv}{type} ) {
            $pack{times}{$srv}{type} = $type;
          }
          $pack{fleet}{$srv}{platform} ||= $type;
          if ( $type eq "P" ) {
            $pack{lnames}{$type}{$srv}{$txt} = $atxt;
            $pack{lpartree}{$srv}{$atxt}{url} = $url;
            $pack{lpartree}{$srv}{$atxt}{alt} = $txt;
            $env{platform}{power} ||= 1;
            $pack{fleet}{$srv}{subsys}{LPAR}{ urldecode($atxt) }{type} = $mset;
          }
          elsif ( $type eq "V" ) {
            $env{platform}{vmware} ||= 1;
            $pack{lnames}{$type}{$valias}{$txt} = $atxt;
            if ( $mset ) {
              $env{hasVMwareAgent} ||= 1;
              ( my $cls = $hmc ) =~ s/cluster_//;
              $pack{fleet}{$cls}{subsys}{LPAR}{$txt}{type} = $mset;
              # $pack{fleet}{$cls}{subsys}{LPAR}{$txt}{uuid} = $atxt;
              $pack{fleet}{$cls}{platform} ||= $type;
              $pack{fleet}{$cls}{uuid}{$txt} = $atxt;
              $pack{fleet}{$cls}{cluster}{$txt} = $cls;
            } ## end elsif ( $type eq "V" )
          }
          elsif ( $type eq "S" ) {
            $env{platform}{solaris} ||= 1;
            $pack{times}{$srv}{type} ||= $type;
            $pack{times}{$srv}{"active"} ||= 1;
            if ( $mset eq "T" ) {
              push @{ $pack{stree}{$srv}{solaris} }, [ $txt, $url ];
            }
            elsif ( $mset eq "Z" )  {
              $pack{lnames}{$type}{$valias}{$txt} = $atxt;
              push @{ $pack{lstree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
              $pack{soltree}{host}{$srv}{zone}{$txt}{url} = $url;
              if ( $txt ne $atxt ) {
                $pack{soltree}{host}{$srv}{zone}{$txt}{alt} = $atxt;
              }
            }
            elsif ( $mset eq "L" )  {
              $pack{soltree}{host}{$srv}{ldom}{Totals}{url} = $url;
              $pack{stree}{$srv}{soltotal}{txt} = "Totals";
              $pack{stree}{$srv}{soltotal}{url} = $url;
            }
            elsif ( $mset eq "G" )  {
              $pack{soltree}{host}{$srv}{globzone}{Totals}{url} = $url;
              $pack{stree}{$srv}{soltotal}{txt} = "Totals";
              $pack{stree}{$srv}{soltotal}{url} = $url;
            }
            elsif ( $mset eq "P" )  {
              $pack{soltree}{totals} = $url;
            }


            #my $host = ( $url =~ /host=([^&]*)/ )[0];
            #if ($host) {
            #$srv = $host;
            #}
          }
          elsif ( $type eq "U" ) {
            if ( $mset eq "S" ) {
              # $env{platform}{solaris} ||= 1;
              $pack{fleet}{$srv}{subsys}{LPAR}{$txt}{type} = "S";
            }
            elsif ( $srv eq "Linux" ) {
              $env{platform}{linux} ||= 1;
              $pack{fleet}{$srv}{platform} = "L";
              $pack{fleet}{$srv}{subsys}{LPAR}{$txt}{type} = "L";
            }
            else {
              $pack{fleet}{$srv}{subsys}{LPAR}{$txt}{type} = "U";
            }
          }
          elsif ( $type eq "B" ) {
            $env{platform}{blade} ||= 1;
            $pack{fleet}{$srv}{subsys}{LPAR}{$txt}{type} = "S";
          }
          elsif ( $type eq "X" ) {
            $env{platform}{xen} ||= 1;
            $pack{fleet}{$srv}{subsys}{LPAR}{$txt}{type} = "X";
          }
          elsif ( $type eq "H" ) {
            $env{platform}{hyperv} ||= 1;
            if ( $val[6] ) {
              $pack{hyperv}{cluster}{$val[6]}{vm}{$txt} = $url;
            }
            else {
              $pack{hyperv}{dom}{$hmc}{server}{$srv}{vm}{$txt} = $url;
            }
          }
        }
        last;
      };

      #R:ahmc11:BSRV21:BSRV21LPAR5:BSRV21LPAR5:/lpar2rrd-cgi/detail.sh?host=ahmc11&server=BSRV21&lpar=BSRV21LPAR5&item=lpar&entitle=0&gui=1&none=none::
      "R" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url ) = ( $val[1], $val[2], $val[3], $val[4], $val[5] );
        if ( !$useacl || $isAdmin || $isReadonly )
        {
          push @{ $pack{rtree}{$srv} }, [ $txt, $url, $atxt ];
          if ( !exists $pack{times}{$srv}{timestamp}{$hmc} ) {
            $pack{times}{$srv}{timestamp}{$hmc} = 999;
          }

          # push @lnames, $txt;
        } ## end if ( !$useacl || $aclServAll...)
        last;
      };
      "W" eq $val[0] && do {
        my ( $hmc, $srv, $txt, $url, $wparparent, $mset ) = ( $val[1], $val[2], $val[4], $val[5], $val[6], $val[9] );
        if ( !$useacl || $isAdmin || $isReadonly )
        {
          push @{ $pack{wtree}{$srv}{$wparparent} }, [ $txt => $url ];
          $pack{fleet}{$srv}{subsys}{WPAR}{ urldecode( $val[3] ) }{type} = $mset;
        } ## end if ( !$useacl || $aclServAll...)
        last;
      };
      "WLM" eq $val[0] && do {
        my ( $hmc, $srv, $txt, $url, $wparparent, $mset ) = ( $val[1], $val[2], $val[4], $val[5], $val[6], $val[9] );
        if ( !$useacl || $isAdmin || $isReadonly )
        {
          push @{ $pack{wlmtree}{$srv}{$wparparent} }, [ $txt => $url ];
          $pack{fleet}{$srv}{subsys}{WLM}{ urldecode( $val[3] ) }{type} = $mset;
        }
        last;
      };
      "T" eq $val[0] && do {
        my ( $id, $txt, $url ) = ( $val[1], $val[2], $val[3] );

        #if ($txt ne "ACL" || $aclServAll) {
        push @{ $pack{tail} }, [ $txt, $url, $id ];

        #}
        last;
      };
      "V" eq $val[0] && do {
        my ( $vcenter, $txt, $url, $alias ) = ( $val[1], $val[2], $val[3], $val[6] );

        #if ($txt ne "ACL" || $aclServAll) {
        my $uuid = ( $url =~ /server=([^&]*)/ )[0];
        if ( $alias && !exists $pack{vtree}{$vcenter}{alias} ) {
          $pack{vtree}{$vcenter}{alias} = $alias;
        }
        if ( $uuid && !exists $pack{vtree}{$vcenter}{uuid} ) {
          $pack{vtree}{$vcenter}{uuid} = $uuid;
        }
        if ( !exists $pack{vtree}{$vcenter}{menu} ) {
          $pack{vtree}{$vcenter}{menu} = [];
        }
        push @{ $pack{vtree}{$vcenter}{menu} }, [ $txt, $url ];

        # $pack{vtree}{$vcenter}{menu}{$txt} = $url;

        #}
        last;
      };
      "A" eq $val[0] && do {    # VMware clusters
        my ( $vcenter, $cluster, $name, $url, $type ) = ( $val[1], $val[2], $val[3], $val[4], $val[8] );

        #if ($txt ne "ACL" || $aclServAll) {
        if ( $type eq "V" ) {
          $pack{cluster}{$vcenter}{$cluster}{$name} = $url;
        }
        elsif ( $type eq "H" ) {
          $pack{hyperv}{cluster}{$val[2]}{totals}{$val[3]} = $url;
        }

        #}
        last;
      };
      "B" eq $val[0] && do {
        my ( $vcenter, $cluster, $name, $url ) = ( $val[1], $val[2], $val[3], $val[4] );

        #if ($txt ne "ACL" || $aclServAll) {
        if ( $name ne "Resources" ) {
          $pack{respool}{$vcenter}{$cluster}{$name} = $url;
        }
        else {
          $pack{respool}{$vcenter}{$name} = $url;
        }

        #}
        last;
      };
      "Z" eq $val[0] && do {    # VMware datastores
        my ( $vcenter, $dc, $name, $url ) = ( $val[1], $val[2], $val[3], $val[4] );
        if ($vcenter) {
          $pack{datastore}{$vcenter}{$dc}{$name} = $url;
        }
        else {
          $vcenter = ( $url =~ /server=([^&]*)/ )[0];
          $pack{datastore}{$vcenter}{$dc}{$name} = $url;
        }
        last;
      };
      "Q" eq $val[0] && do {    # product version
        $env{version} = $version ? $version : $val[1];
        last;
      };
      "N" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        if ($type eq "X") {
          $hmc = $atxt;
          $srv = $val[1];
        }
        if ( !$useacl || $isAdmin || $isReadonly ) {
          $atxt = urldecode($atxt);
          push @{ $pack{lantree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
        }
        last;
      };
      "Y" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        if ($type eq "X") {
          $hmc = $atxt;
          $srv = $val[1];
        }
        if ( !$useacl || $isAdmin || $isReadonly ) {
          $atxt = urldecode($atxt);
          push @{ $pack{santree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
        }
        last;
      };
      "s" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );

        if ( !$useacl || $isAdmin || $isReadonly ) {
          $atxt = urldecode($atxt);
          push @{ $pack{sastree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
        }
        last;
      };
      "r" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );

        if ( !$useacl || $isAdmin || $isReadonly ) {
          $atxt = urldecode($atxt);
          push @{ $pack{sriovtree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
        }
        last;
      };
      "HEA" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );

        if ( !$useacl || $isAdmin || $isReadonly ) {
          $atxt = urldecode($atxt);
          push @{ $pack{heatree}{$srv}{$hmc} }, [ $txt, $url, $atxt ];
        }
        last;
      };
      "ST" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        $pack{xen}{pool}{$hmc}{total}{$txt}{url} = $url;
        last;
      };
      "NT" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        $pack{xen}{pool}{$hmc}{label}{$atxt}{lan}{txt} = $txt;
        $pack{xen}{pool}{$hmc}{label}{$atxt}{lan}{url} = $url;
        last;
      };
      "YT" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        $pack{xen}{pool}{$hmc}{label}{$atxt}{san}{txt} = $txt;
        $pack{xen}{pool}{$hmc}{label}{$atxt}{san}{url} = $url;
        last;
      };
      "HDT" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        $pack{hyperv}{dom}{$hmc}{server}{$srv}{drive}{total}{$txt} = $url;
        last;
      };
      "HDI" eq $val[0] && do {
        my ( $hmc, $srv, $atxt, $txt, $url, $valias, $type, $mset ) = ( $val[1], $val[2], $val[3], $val[4], $val[5], $val[7], $val[8], $val[9] );
        $pack{hyperv}{dom}{$hmc}{server}{$srv}{drive}{item}{$txt} = $url;
        last;
      };
    };
  } ## end foreach my $line ( split /^/...)

  # close(SKEL);
  my $xenmenu = "$basedir/tmp/menu_xenserver.json";
  if ( -r "$basedir/debug/menu.txt" ) {
    $xenmenu = "$basedir/debug/menu_xenserver.json";
  }
  if( -e $xenmenu && -f _ && -r _ ) {
    my %xmenu = %{ genXen( $xenmenu ) };
    if ( %xmenu ) {
      $env{platform}{xen} = 1;
      $pack{xen} = \%xmenu;
    }
  }

  my $ovirtmenu = "$basedir/tmp/menu_ovirt.json";
  if ( -r "$basedir/debug/menu.txt" ) {
    $ovirtmenu = "$basedir/debug/menu_ovirt.json";
  }
  if( -e $ovirtmenu && -f _ && -r _ ) {
    $env{platform}{ovirt} = 1;
    my %omenu = %{ genOVirt( $ovirtmenu ) };
    $pack{ovirt} = \%omenu;
  }

  # for debugging only - simulate no Power / VMware menu
  # $env{platform}{power} = 0;
  # $env{platform}{vmware} = 0;
  # $env{cached} = 0;
} ## end sub readMenu

### Generate SERVERs submenu (no HMC parents)
sub genServers {
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( $pack{times}{$srv}{"active"}
      && $pack{times}{$srv}{type}
      && $pack{times}{$srv}{type} eq "P"
      && ( !$useacl || $isAdmin || $isReadonly || ACL::hasItems( "P", $srv ) || ACL::hasItems( "O", $srv ) )
      && !( exists $pack{times}{$srv}{timestamp}{no_hmc} && ( scalar keys %{ $pack{times}{$srv}{timestamp} } == 1 ) ) )
    {
      $idx{csrv} = push @{ $menu[ $idx{psrv} ]{children} }, { "title" => "$srv", "folder" => "true", "children" => [] };
      $idx{csrv}--;
      my ($hmc) = sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} } keys %{ $pack{times}{$srv}{timestamp} };

      # warn $srv . ACL::canShow( "POWER", "SERVER", $srv );
        if ( exists $pack{stree}{$srv}->{$hmc} && ( !$useacl || $isAdmin || ACL::canShow( "POWER", "SERVER", $srv ) || ACL::hasItems( "O", $srv ) ) ) {
          &serverMenu( $hmc, $srv );
        }
        if ( exists $pack{lpartree}{$srv} ) {
          $idx{lpars} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children} }, { "title" => "LPAR", "folder" => "true", "children" => [] };
          $idx{lpars}--;
          for my $lpar ( sort keys %{ $pack{lpartree}{$srv} } ) {
            if ( exists $pack{wtree}{$srv}{$lpar} && exists $pack{wlmtree}{$srv}{$lpar} ) {
              $idx{lparchld} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNodewChild( $lpar, $pack{lpartree}{$srv}{$lpar}{url}, $hmc, $srv, 1, $pack{lpartree}{$srv}{$lpar}{alt}, $pack{hmc}{$hmc}{type}, undef, "L" );
              $idx{lparchld}--;
              $idx{wpars} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{lparchld} ]{children} }, { "title" => "WPAR", "folder" => "true", "children" => [] };
              $idx{wpars}--;
              for my $wpar ( sort @{ $pack{wtree}{$srv}{$lpar} } ) {
                push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{lparchld} ]{children}[ $idx{wpars} ]{children} }, pushFullNode( @$wpar[0], @$wpar[1], $hmc, $srv, 1, $lpar . "/" . @$wpar[0], undef, undef, "W" );
              }
              $idx{wlm} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{lparchld} ]{children} }, { "title" => "WLM", "folder" => "true", "children" => [] };
              $idx{wlm}--;
              for my $wpar ( sort @{ $pack{wlmtree}{$srv}{$lpar} } ) {
                push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{lparchld} ]{children}[ $idx{wlm} ]{children} }, pushFullNode( @$wpar[0], @$wpar[1], $hmc, $srv, 1, $lpar . "/" . @$wpar[0], undef, undef, "W" );
              }
            }
            elsif ( exists $pack{wtree}{$srv}{$lpar} ) {
              $idx{wpars} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNodewChild( $lpar, $pack{lpartree}{$srv}{$lpar}{url}, $hmc, $srv, 1, $pack{lpartree}{$srv}{$lpar}{alt}, $pack{hmc}{$hmc}{type}, undef, "L" );
              $idx{wpars}--;
              for my $wpar ( sort @{ $pack{wtree}{$srv}{$lpar} } ) {
                push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{wpars} ]{children} }, pushFullNode( @$wpar[0], @$wpar[1], $hmc, $srv, 1, $lpar . "/" . @$wpar[0], undef, undef, "W" );
              }
            } ## end if ( exists $pack{wtree...})
            elsif ( exists $pack{wlmtree}{$srv}{$lpar} ) {
              $idx{wlm} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNodewChild( $lpar, $pack{lpartree}{$srv}{$lpar}{url}, $hmc, $srv, 1, $pack{lpartree}{$srv}{$lpar}{alt}, $pack{hmc}{$hmc}{type}, undef, "L" );
              $idx{wlm}--;
              for my $wpar ( sort @{ $pack{wlmtree}{$srv}{$lpar} } ) {
                push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{wlm} ]{children} }, pushFullNode( @$wpar[0], @$wpar[1], $hmc, $srv, 1, $lpar . "/" . @$wpar[0], undef, undef, "W" );
              }
            } ## end if ( exists $pack{wtree...})
            else {
              push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( $lpar, $pack{lpartree}{$srv}{$lpar}{url}, $hmc, $srv, 1, $pack{lpartree}{$srv}{$lpar}{alt}, $pack{hmc}{$hmc}{type}, undef, "L" );
            }
          }    # L3 END
          if ( exists $pack{rtree}{$srv} ) {
            $idx{removed} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, { "title" => "Removed", "folder" => "true", "children" => [] };
            $idx{removed}--;
            for my $removed ( sort @{ $pack{rtree}{$srv} } ) {
              push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{removed} ]{children} }, pushFullNode( @$removed[0], @$removed[1], $hmc, $srv, 1, undef, undef, undef, "R" );
            }
          } ## end if ( exists $pack{rtree...})
        } ## end if ( exists $pack{lstree...})
        my $ref = $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children};
        if ( exists $pack{lantree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "lan", $ref, "LAN", "N" );
        }
        if ( exists $pack{santree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "san", $ref, "SAN", "Y" );
        }
        if ( exists $pack{sastree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "sas", $ref, "SAS", "s" );
        }
        if ( exists $pack{sriovtree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "sriov", $ref, "SR-IOV", "r" );
        }
        if ( exists $pack{heatree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "hea", $ref, "HEA", "HEA" );
        }
    } ## end if ( $pack{times}{$srv...})
  } ## end for my $srv ( sort keys...)
  if ( $env{platform}{unmanaged} && scalar keys %{ $env{platform} } == 1 ) {
    &genStandalone(0);    # List by Servers
  }
} ## end sub genServers

### Generate standalone SERVERs submenu
sub genStandalone {
  my $toplevel = shift;
  if ($toplevel) {
    $idx{solo} = push @menu, { title => "Unmanaged", folder => "true", children => [] };
    $idx{solo}--;
  }
  else {
    $idx{solo} = $idx{psrv};
  }

  # for my $srv ( sort keys $pack{lstree} ) {
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( $pack{times}{$srv}{"active"}
      && exists $pack{times}{$srv}{timestamp}{no_hmc}
      && ( ! $pack{times}{$srv}{type} || $pack{times}{$srv}{type} ne "S")
      && scalar keys %{ $pack{times}{$srv}{timestamp} } == 1
      && ( !$useacl || $isAdmin || $isReadonly || ACL::hasItems( "U", $srv ) ) )
    {
      $idx{csrv} = push @{ $menu[ $idx{solo} ]{children} }, { title => $srv, folder => "true", children => [] };
      $idx{csrv}--;
      my $hmc = "no_hmc";
      if ( exists $pack{lstree}{$srv}->{$hmc} ) {
        for my $lpar ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{lstree}{$srv}->{$hmc} } ) {
          if ( !$useacl || $isAdmin || $isReadonly || ACL::canShow( "UNMANAGED", "VM", $srv, @$lpar[0] ) ) {
            push @{ $menu[ $idx{solo} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], $hmc, $srv, 1, @$lpar[2], $pack{hmc}{$hmc}{type}, undef, "U" );
          }
        }    # L3 END
      } ## end if ( exists $pack{lstree...})
    } ## end if ( $pack{times}{$srv...})
  } ## end for my $srv ( sort keys...)
} ## end sub genStandalone

sub genLinux {
  my $toplevel = shift;
  if ($toplevel) {
    $idx{linux} = push @menu, { title => "Linux", folder => "true", children => [] };
    $idx{linux}--;
  }
  else {
    $idx{linux} = $idx{psrv};
  }

  # for my $srv ( sort keys $pack{lstree} ) {
  my $hmc = "no_hmc";
  for my $srv ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{lhmc}{no_hmc}{Linux} } ) {
    if ( !$useacl || $isAdmin || $isReadonly || ACL::hasItems( "U", @$srv[0] ) ) {
      # $idx{csrv} = push @{ $menu[ $idx{linux} ]{children} }, { title => @$srv[0], folder => "true", children => [] };
      # $idx{csrv}--;
      if ( !$useacl || $isAdmin || $isReadonly || ACL::canShow( "LINUX", "SERVER", @$srv[0] ) ) {
        push @{ $menu[ $idx{linux} ]{children} }, pushFullNode( @$srv[0], @$srv[1], $hmc, "Linux", 1, @$srv[2], $pack{hmc}{$hmc}{type}, undef, "U" );
      }
    }
  }
}

### Generate Solaris submenu
sub genSolaris {
  my $toplevel = shift;
  if ($toplevel) {
    $idx{solaris} = push @menu, { title => "Solaris", folder => "true", children => [] };
    $idx{solaris}--;
    if ( $pack{soltree}{totals} ) {
      push @{ $menu[ $idx{solaris} ]{children} }, pushFullNode( "Totals", "/lpar2rrd-cgi/detail.sh?host=no_hmc&server=Solaris&lpar=cod&item=cpuagg-sol&entitle=0&gui=1&none=none", "cpuagg-sol", "Solaris", 1, "", "S", undef, "S" );
    }
  }
  else {
    $idx{solaris} = $idx{psrv};
  }

  # for my $srv ( sort keys $pack{lstree} ) {
  for my $srv ( sort keys %{ $pack{soltree}{host} } ) {
    $idx{csrv} = push @{ $menu[ $idx{solaris} ]{children} }, { title => $srv, folder => "true", children => [] };
    $idx{csrv}--;
    if ( exists $pack{soltree}{host}{$srv}{ldom}{Totals} ) {
      push @{ $menu[ $idx{solaris} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( "Totals", $pack{soltree}{host}{$srv}{ldom}{Totals}{url}, "Totals", $srv, 1, "", "S", undef, "S" );
    }
    if ( exists $pack{soltree}{host}{$srv}{globzone}{Totals} ) {
      push @{ $menu[ $idx{solaris} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( "Totals", $pack{soltree}{host}{$srv}{globzone}{Totals}{url}, "Totals", $srv, 1, "", "S", undef, "S" );
    }
    if ( exists $pack{soltree}{host}{$srv}{zone} ) {
      $idx{lpars} = push @{ $menu[ $idx{solaris} ]{children}[ $idx{csrv} ]{children} }, { "title" => "ZONE", "folder" => "true", "children" => [] };
      $idx{lpars}--;
      for my $lpar ( @{ $pack{lstree}{$srv}{no_hmc} } ) {
        push @{ $menu[ $idx{solaris} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], "Solaris", $srv, 1, @$lpar[2], "S", undef, "S" );
      }    # L3 END
    }
  } ## end for my $srv ( sort keys...)
} ## end sub genSolaris

### Generate Hitachi Blade submenu
sub genHitachi {
  my $toplevel = shift;
  if ($toplevel) {
    $idx{hitachi} = push @menu, { title => "Hitachi", folder => "true", children => [] };
    $idx{hitachi}--;
  }
  else {
    $idx{hitachi} = $idx{psrv};
  }

  # for my $srv ( sort keys $pack{lstree} ) {
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( $pack{times}{$srv}{"active"}
      && $pack{times}{$srv}{type}
      && $pack{times}{$srv}{type} eq "B" )
    {
      $idx{csrv} = push @{ $menu[ $idx{hitachi} ]{children} }, { title => $srv, folder => "true", children => [] };
      $idx{csrv}--;
      my $hmc = "Hitachi";
      # if ( exists $pack{stree}{$srv}->{$hmc} && ( !$useacl || $aclServAll || ACL::canShow("P", "", $srv, "") ) ) {
        &serverMenu( "Hitachi", $srv );
      # }
      if ( exists $pack{lstree}{$srv}->{$hmc} ) {
        $idx{lpars} = push @{ $menu[ $idx{hitachi} ]{children}[ $idx{csrv} ]{children} }, { "title" => "LPAR", "folder" => "true", "children" => [] };
        $idx{lpars}--;
        for my $lpar ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{lstree}{$srv}->{$hmc} } ) {
          push @{ $menu[ $idx{hitachi} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], $hmc, $srv, 1, @$lpar[2], $pack{hmc}{$hmc}{type}, undef, "B" );
        }    # L3 END
      } ## end if ( exists $pack{lstree...})
      my $ref = $menu[ $idx{hitachi} ]{children}[ $idx{csrv} ]{children};
      if ( exists $pack{lantree}{$srv}{$hmc} ) {
        sub_tree_ref( $srv, $hmc, "lan", $ref, "LAN", "N" );
      }
      if ( exists $pack{santree}{$srv}{$hmc} ) {
        sub_tree_ref( $srv, $hmc, "san", $ref, "SAN", "Y" );
      }
      if ( exists $pack{sastree}{$srv}{$hmc} ) {
        sub_tree_ref( $srv, $hmc, "sas", $ref, "SAS", "s" );
      }
      if ( exists $pack{sriovtree}{$srv}{$hmc} ) {
        sub_tree_ref( $srv, $hmc, "sriov", $ref, "SR-IOV", "r" );
      }
    } ## end if ( $pack{times}{$srv...})
  } ## end for my $srv ( sort keys...)
} ## end sub genHitachi

sub sub_tree_ref {
  my ( $srv, $hmc, $subsys, $node, $title, $obj ) = @_;
  $idx{$subsys} = push @{ $node }, { "title" => $title, "folder" => "true", "children" => [] };
  $idx{$subsys}--;
  $idx{$subsys . "totals"} = push @{ $$node[ $idx{$subsys} ]{children} }, { "title" => "Totals" };
  $idx{$subsys . "totals"}--;
  $idx{$subsys . "items"} = push @{ $$node[ $idx{$subsys} ]{children} }, { "title" => "Items", "folder" => "true", "children" => [] };
  $idx{$subsys . "items"}--;
  for my $lpar ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{$subsys . "tree"}{$srv}->{$hmc} } ) {
    if ( @$lpar[0] eq "$subsys-totals" ) {
      @{ $$node[ $idx{$subsys} ]{children} }[ $idx{$subsys . "totals"} ] = pushFullNode( "Totals", @$lpar[1], $hmc, $srv, 1, @$lpar[0], $pack{hmc}{$hmc}{type}, undef, $obj );
    }
    else {
      push @{ $$node[ $idx{$subsys} ]{children}[ $idx{$subsys . "items"} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], $hmc, $srv, 1, @$lpar[2], $pack{hmc}{$hmc}{type}, undef, $obj );
    }
  }
}

### Generate Xen submenu
sub genXen {
  my $xenmenu = shift;
  my %xencfg;
  open( CFG, $xenmenu );
  {
    local $/ = undef;    # required for re-read of encode_json pretty output
    my $rawcfg = <CFG>;
    if ($rawcfg) {
      %xencfg = %{ decode_json($rawcfg) };
    }
  }
  close CFG;
  # print Dumper \%xencfg;
  return \%xencfg;

} ## end sub genXen

### Generate oVirt submenu
sub genOVirt {
  my $ovirtmenu = shift;
  my %ovirtcfg;
  open( CFG, $ovirtmenu );
  {
    local $/ = undef;    # required for re-read of encode_json pretty output
    my $rawcfg = <CFG>;
    if ($rawcfg) {
      %ovirtcfg = %{ decode_json($rawcfg) };
    }
  }
  close CFG;
  # print Dumper \%ovirtcfg;
  return \%ovirtcfg;

} ## end sub genOVirt

sub genHyperV {
  my $toplevel = shift;
  if ($toplevel) {
    $idx{hyperv} = push @menu, { title => "Hyper-V", folder => "true", children => [] };
    $idx{hyperv}--;
  }
  else {
    $idx{hyperv} = $idx{psrv};
  }

  for my $dom ( sort keys %{ $pack{hyperv}{dom} } ) {
    $idx{dom} = push @{ $menu[ $idx{hyperv} ]{children} }, { title => $dom, folder => "true", children => [] };
    $idx{dom}--;
    for my $srv ( sort keys %{ $pack{hyperv}{dom}{$dom}{server} } ) {
      $idx{hsrv} = push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children} }, { title => $srv, folder => "true", children => [] };
      $idx{hsrv}--;
      # if ( exists $pack{stree}{$srv}->{$hmc} && ( !$useacl || $aclServAll || ACL::canShow("P", "", $srv, "") ) ) {
      foreach my $total ( 'CPU', 'Memory', 'Disk', 'Net' ) {
        if ( exists $pack{hyperv}{dom}{$dom}{server}{$srv}{totals}{$total} ) {
          push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children} }, pushFullNode( $total, $pack{hyperv}{dom}{$dom}{server}{$srv}{totals}{$total}, $dom, $srv, 0 );
        }
      }
      # }
      if ( exists $pack{hyperv}{dom}{$dom}{server}{$srv}{vm} ) {
        $idx{lpars} = push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
        $idx{lpars}--;
        for my $lpar ( sort keys %{ $pack{hyperv}{dom}{$dom}{server}{$srv}{vm} } ) {
          push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( $lpar, $pack{hyperv}{dom}{$dom}{server}{$srv}{vm}{$lpar}, $dom, $srv, 1, "", "H", undef, "H" );
        }
      }
      if ( exists $pack{hyperv}{dom}{$dom}{server}{$srv}{drive} ) {
        $idx{drive} = push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children} }, { "title" => "Storage", "folder" => "true", "children" => [] };
        $idx{drive}--;
        for my $lpar ( sort keys %{ $pack{hyperv}{dom}{$dom}{server}{$srv}{drive}{total} } ) {
          push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children}[ $idx{drive} ]{children} }, pushFullNode( $lpar, $pack{hyperv}{dom}{$dom}{server}{$srv}{drive}{total}{$lpar}, $dom, $srv, 1, "", "H", undef, "H" );
        }
        if ( exists $pack{hyperv}{dom}{$dom}{server}{$srv}{drive}{item} ) {
          $idx{dritem} = push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children}[ $idx{drive} ]{children} }, { "title" => "Disk", "folder" => "true", "children" => [] };
          $idx{dritem}--;
          for my $lpar ( sort keys %{ $pack{hyperv}{dom}{$dom}{server}{$srv}{drive}{item} } ) {
            push @{ $menu[ $idx{hyperv} ]{children}[ $idx{dom} ]{children}[ $idx{hsrv} ]{children}[ $idx{drive} ]{children}[ $idx{dritem} ]{children} }, pushFullNode( $lpar, $pack{hyperv}{dom}{$dom}{server}{$srv}{drive}{item}{$lpar}, $dom, $srv, 1, "", "H", undef, "H" );
          }
        }
      }
    }
  } ## end for my $srv ( sort keys...)
  for my $clstr ( sort keys %{ $pack{hyperv}{cluster} } ) {
    $idx{hclstr} = push @{ $menu[ $idx{hyperv} ]{children} }, { title => "Cluster: $clstr", folder => "true", children => [] };
    $idx{hclstr}--;
    for my $ctot ( keys %{ $pack{hyperv}{cluster}{$clstr}{totals} } ) {
      push @{ $menu[ $idx{hyperv} ]{children}[ $idx{hclstr} ]{children} }, pushFullNode( $ctot, $pack{hyperv}{cluster}{$clstr}{totals}{$ctot}, "", $clstr, 0 );
    }
    if ( exists $pack{hyperv}{cluster}{$clstr}{vm} ) {
      $idx{lpars} = push @{ $menu[ $idx{hyperv} ]{children}[ $idx{hclstr} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
      $idx{lpars}--;
      for my $lpar ( sort keys %{ $pack{hyperv}{cluster}{$clstr}{vm} } ) {
        push @{ $menu[ $idx{hyperv} ]{children}[ $idx{hclstr} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( $lpar, $pack{hyperv}{cluster}{$clstr}{vm}{$lpar}, "cluster", $clstr, 1, "", "H", undef, "H" );
      }
    }
  }
} ## end sub genStandalone

sub genVMware {
  for my $vcl ( sort keys %{ $pack{vclusters} } ) {
    if ( $vcl ne "" && $vcl !~ "cluster_" && not exists $pack{vtree}{$vcl} ) {
      $pack{vclusters}{""} = delete $pack{vclusters}{$vcl};

      # $pack{vtree}{$vcl}{$vcl}{url} = "";
    }
  } ## end for my $vcl ( sort keys...)
  if ( $pack{vtree} ) {
    for my $vc ( sort keys %{ $pack{vtree} } ) {
      if ( $vc eq "" ) {
        next;
      }
      my $vcAlias = $pack{vtree}{$vc}{alias} ||= $vc;
      $idx{vc} = push @{ $menu[ $idx{vmware} ]{children} }, { "title" => "$vcAlias", "altname" => $vc, "uuid" => $pack{vtree}{$vc}{uuid}, "folder" => "true", "children" => [] };
      $idx{vc}--;
      my $delim = "";

      if ( !$useacl || $isAdmin || $isReadonly || $sections{vmware} ) {
        $delim = vcTotals($vc);
      }
      if ( exists $pack{respool}{$vc}{Resources} ) {
        push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children} }, pushFullNode( "Unregistered VMs", $pack{respool}{$vc}{Resources}, $vc, $vc, 1 );
      }
      if ( exists $pack{cluster}{$vc} ) {

        # print "{\"title\":\"Cluster\",\"folder\":\"true\",\"children\":[\n";
        foreach my $cl ( sort keys %{ $pack{cluster}{$vc} } ) {
          if ( !$useacl || $isAdmin || $isReadonly || ACL::hasItems( "V", $cl ) ) {

            $cl =~ /cluster_(.*)/;
            my $tcl = "Cluster: $1";
            $idx{cl} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children} }, { "title" => $tcl, "altname" => $cl, "folder" => "true", "children" => [] };
            $idx{cl}--;

            # print $n2 . "{\"title\":\"$cl\",\"folder\":\"true\",\"children\":[\n";
            if ( !$useacl || $isAdmin || $isReadonly || ACL::canShow( "VMWARE", "CLUSTER", "$cl" ) ) {
              clTotals( $vc, $cl );
            }

            # Resource Pools submenu
            if ( !$useacl || $isAdmin || $isReadonly ) {
              if ( exists $pack{respool}{$vc}{$cl} ) {
                $idx{rp} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children} }, { "title" => "Resource Pool", "folder" => "true", "children" => [] };
                $idx{rp}--;
                my $delim = '';
                foreach my $rp ( sort keys %{ $pack{respool}{$vc}{$cl} } ) {
                  if ( $rp ne "Resources" ) {
                    push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children}[ $idx{rp} ]{children} }, pushFullNode( $rp, $pack{respool}{$vc}{$cl}{$rp}, $vc, $cl, 1, undef, undef, undef, "RP" );
                  }
                }
              } ## end if ( exists $pack{respool...})

              # ESXi submenu
              if ( exists $pack{vclusters}{$cl} ) {
                &genESXiServers( $cl, "ESXi", $vc, 0 );
              }
            } ## end if ( !$useacl || $aclServAll)

            # VM submenu
            if ( exists $pack{lhmc}{$cl} ) {
              $idx{lpar} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
              $idx{lpar}--;
              my %allVMs;
              foreach my $srv ( sort keys %{ $pack{lhmc}{$cl} } ) {
                foreach my $vm ( @{ $pack{lhmc}{$cl}{$srv} } ) {
                  $allVMs{ @$vm[0] }{url} = @$vm[1];
                  $allVMs{ @$vm[0] }{alt} = @$vm[2];
                  $allVMs{ @$vm[0] }{srv} = $srv;
                }
              } ## end foreach my $srv ( sort keys...)
              for my $lpar ( sort { lc($a) cmp lc($b) } keys %allVMs ) {
                push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children}[ $idx{lpar} ]{children} }, pushFullNode( $lpar, $allVMs{$lpar}{url}, $vc, $allVMs{$lpar}{srv}, 1, $allVMs{$lpar}{alt}, "V", $cl, "VM" );
              }    # L3 END
                   #if ( exists $pack{rtree}{$cl} ) {
                   #    print $n3
                   #        . "\n{\"title\":\"Removed\",\"folder\":\"true\",\"children\":[\n";
                   #    my $n4 = "";
                   #    #for my $removed ( @{ $pack{rtree}{$hmc}->{$srv} } ) {
                   #    #    print $n4
                   #    #        . &fullNode( @$removed[0], @$removed[1], $vcntr, $srv,
                   #    #        1 );
                   #        $n4 = ",\n";
                   #        #}
                   #    print "]}";
                   #}
            } ## end if ( exists $pack{lhmc...})
          } ## end if ( !$useacl || $aclServAll...)
        } ## end foreach my $cl ( sort keys ...)

        # print "]}\n";
      } ## end if ( exists $pack{cluster...})
      if ( !$useacl || $isAdmin || $isReadonly ) {
        if ( $pack{datastore}{$vc} ) {
          my $dcdelim = "";
          for my $dc ( sort keys %{ $pack{datastore}{$vc} } ) {
            $dc =~ /datastore_(.*)/;
            my $tdc = "Datastores: $1";
            $idx{dc} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children} }, { title => $tdc, altname => $dc, folder => "true", children => [] };
            $idx{dc}--;

            # print $dcdelim . fullNode_wchild($tdc, "", "", "", 0, $dc);
            for my $ds ( sort keys %{ $pack{datastore}{$vc}{$dc} } ) {
              push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{dc} ]{children} }, pushFullNode( $ds, $pack{datastore}{$vc}{$dc}{$ds}, $vc, $dc, 1, undef, undef, undef, "DS" );
            }
          } ## end for my $dc ( sort keys ...)
        } ## end if ( $pack{datastore}{...})
      } ## end if ( !$useacl || $aclServAll)
      if ( exists $pack{vclusters}{$vc} ) {
        &genESXiServers( $vc, "ESXi", $vc, 1 );
      }
    } ## end for my $vc ( sort keys ...)
  } ## end if ( $pack{vtree} )
  if ( exists $pack{vclusters}{""} ) {
    &genESXiServers( "", "ESXi", "", 1 );
  }
} ## end sub genVMware

### Generate ESXi SERVERs submenu
sub genESXiServers {
  my ( $clstr, $title, $vcntr, $listVMs ) = @_;
  my $vcenterWoClusters = ( $vcntr eq $clstr );
  if ( !exists $pack{vclusters}{$clstr} ) {
    return;
  }
  if ($clstr) {
    if ($vcenterWoClusters) {
      $idx{esxi} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children} }, { "title" => $title, "folder" => "true", "children" => [] };
    }
    else {
      $idx{esxi} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children} }, { "title" => $title, "folder" => "true", "children" => [] };
    }
  } ## end if ($clstr)
  else {
    $idx{esxi} = push @{ $menu[ $idx{vmware} ]{children} }, { "title" => $title, "folder" => "true", "children" => [] };
  }
  $idx{esxi}--;

  # for my $srv ( sort keys $pack{lstree} ) {
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( exists $pack{vclusters}{$clstr}{$srv} && $pack{times}{$srv}{"active"} && $pack{times}{$srv}{type} eq "V" ) {
      my $tsrv = $pack{times}{$srv}{"alias"} ||= $srv;
      if ($clstr) {
        if ($vcenterWoClusters) {
          $idx{csrv} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{esxi} ]{children} }, { "title" => $tsrv, "altname" => $srv, "folder" => "true", "children" => [] };
        }
        else {
          $idx{csrv} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children}[ $idx{esxi} ]{children} }, { "title" => $tsrv, "altname" => $srv, "folder" => "true", "children" => [] };
        }
      } ## end if ($clstr)
      else {
        $idx{csrv} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{esxi} ]{children} }, { "title" => $tsrv, "altname" => $srv, "folder" => "true", "children" => [] };
      }
      $idx{csrv}--;
      for my $hmc (
        sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
        keys %{ $pack{times}{$srv}{timestamp} }
        )
      {
        if ($clstr) {
          if ($vcenterWoClusters) {
            &serverMenu( $vcntr, $srv, $clstr, 1, 1 );
          }
          elsif ( exists $pack{stree}{$srv}{$vcntr} ) {
            &serverMenu( $vcntr, $srv, $clstr, 1 );
          }
        } ## end if ($clstr)
        else {
          my $host = ( keys %{ $pack{stree}{$srv} } )[0];
          if ( exists $pack{stree}{$srv}{$host} ) {
            &serverMenu( $host, $srv, undef, 1 );
          }
        } ## end else [ if ($clstr) ]
        if ($listVMs) {
          if ($vcenterWoClusters) {
            $hmc = "";
          }
          if ( exists $pack{lstree}{$srv}->{$hmc} ) {
            if ($clstr) {
              if ($vcenterWoClusters) {
                $idx{lpar} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
              }
              else {
                $idx{lpar} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
              }
            } ## end if ($clstr)
            else {
              $idx{lpar} = push @{ $menu[ $idx{vmware} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children} }, { "title" => "VM", "folder" => "true", "children" => [] };
            }
            $idx{lpar}--;
            for my $lpar ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{lstree}{$srv}->{$hmc} } ) {
              my $thost;
              if ( !$vcntr ) {
                @$lpar[1] =~ /host=([^&]*)/;
                $thost = $1;
              }

              if ($clstr) {
                if ($vcenterWoClusters) {
                  push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children}[ $idx{lpar} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], ( $vcntr ? $vcntr : $thost ), $srv, 1, @$lpar[2], "VM" );
                }

                # push @{ $menu[$idx{vmware}]{children}[$idx{vc}]{children}[$idx{cl}]{children}[$idx{lpar}]{children} }, pushFullNode( @$lpar[0], @$lpar[1], ($vcntr ? $vcntr : $thost), $srv, 1, @$lpar[2] );
              } ## end if ($clstr)
              else {
                push @{ $menu[ $idx{vmware} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children}[ $idx{lpar} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], ( $vcntr ? $vcntr : $thost ), $srv, 1, @$lpar[2], "VM" );
              }

            }    # L3 END
            if ( exists $pack{rtree}{$srv} ) {
              ## print "\n{\"title\":\"Removed\",\"folder\":\"true\",\"children\":[\n";
              for my $removed ( @{ $pack{rtree}{$srv} } ) {
                ## print &fullNode( @$removed[0], @$removed[1], $vcntr, $srv, 1 );
              }
            } ## end if ( exists $pack{rtree...})
          } ## end if ( exists $pack{lstree...})
        } ## end if ($listVMs)
        if ( !$clstr && $pack{datastore}{$srv} ) {
          my $dcdelim = "";
          for my $dc ( sort keys %{ $pack{datastore}{$srv} } ) {
            $dc =~ /datastore_(.*)/;
            my $tdc = "Datastores: $1";
            ## print $dcdelim . "{\"title\":\"$tdc\",\"folder\":\"true\",\"altname\":\"$dc\",\"children\":[\n";

            # print $dcdelim . fullNode_wchild($tdc, "", "", "", 0, $dc);
            for my $ds ( sort keys %{ $pack{datastore}{$srv}{$dc} } ) {
              ## print &fullNode( $ds, $pack{datastore}{$srv}{$dc}{$ds}, $srv, $dc, 1 );
            }
          } ## end for my $dc ( sort keys ...)
        } ## end if ( !$clstr && $pack{...})
        last;
      }    # L2 END
    } ## end if ( exists $pack{vclusters...})
  } ## end for my $srv ( sort keys...)
} ## end sub genESXiServers

sub genPredefined {
  print "[\n";    # envelope begin
  my $delim = "";
  for my $srv ( sort keys %{ $pack{times} } ) {
    for my $hmc (
      sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
      keys %{ $pack{times}{$srv}{timestamp} }
      )
    {
      if ( $pack{times}{$srv}{type} eq "P" ) {
        my $hash = substr( md5_hex( $hmc . $srv . "pool" ), 0, 7 );
        print $delim . "\"" . $hash . "xkdma\"";
        $delim = ",\n";
        # print $delim . "\"" . $hash . "xldma\""; # aggregated
        last;
      } ## end if ( $hmc ne "no_hmc" )
    } ## end for my $hmc ( sort { $pack...})
  } ## end for my $srv ( sort keys...)
  for my $vc ( keys %{ $pack{cluster} } ) {
    for my $cl ( keys %{ $pack{cluster}{$vc} } ) {
      my $url = $pack{cluster}{$vc}{$cl}{'Cluster totals'};
      my $cl_id = ( $url =~ /host=([^&]*)/ )[0];
      my $vc_id = ( $url =~ /server=([^&]*)/ )[0];
      my $hash = substr( md5_hex( $cl_id . $vc_id . "nope" ), 0, 7 );
      print $delim . "\"" . $hash . "amdma\"";
      $delim = ",\n";
    }
  }
  for ( @{ $pack{ctree} } ) {
    my $grp = @$_[0];

    # $egrp = @$_[1];
    my $hash = substr( md5_hex( "nana" . $grp ), 0, 7 );
    print $delim . "\"" . $hash . "xodna\"";
    $delim = ",\n";
  } ## end for ( @{ $pack{ctree} })
  print "\n]\n";    # envelope end
} ## end sub genPredefined

### Generate HMCs submenu
sub genHMCs {

  # print "{\"title\":\"HMC\",\"folder\":\"true\",\"children\":[\n";
  for my $hmc ( sort keys %{ $pack{hmc} } ) {
    if ( $pack{hmc}{$hmc}{type} ne 'P' ) {
      next;
    }
    $idx{hmc} = push @{ $menu[ $idx{psrv} ]{children} }, { "title" => "$hmc", "folder" => "true", "children" => [] };
    $idx{hmc}--;
    if ( exists $pack{hmc}{$hmc} ) {
      if ( !$useacl || $isAdmin || $isReadonly ) {
        hmcTotals($hmc);
      }
    }
    for my $srv ( sort keys %{ $pack{lhmc}{$hmc} } ) {
      if ( !$pack{times}{$srv}{"removed"}{$hmc} ) {
        $idx{csrv} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children} }, { "title" => "$srv", "folder" => "true", "children" => [] };
        $idx{csrv}--;
        if ( exists $pack{stree}{$srv}->{$hmc} && ( !$useacl || $isAdmin || $isReadonly ) ) {
          serverByHmc( $hmc, $srv );
        }
        if ( exists $pack{lhmc}{$hmc}->{$srv} ) {
          $idx{lpars} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children} }, { "title" => "LPAR", "folder" => "true", "children" => [] };
          $idx{lpars}--;
          for my $lpar ( sort { "\L$a->[0]" cmp "\L$b->[0]" } @{ $pack{lhmc}{$hmc}->{$srv} } ) {
            if ( exists $pack{wtree}{$srv}->{$hmc}->{ @$lpar[2] } ) {
              $idx{wpars} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNodewChild( @$lpar[0], @$lpar[1], $hmc, $srv, 1, @$lpar[2], $pack{hmc}{$hmc}{type} );
              $idx{wpars}--;
              for my $wpar ( @{ $pack{wtree}{$srv}->{$hmc}->{ @$lpar[2] } } ) {
                push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{wpars} ]{children} }, pushFullNode( @$wpar[0], @$wpar[1], $hmc, $srv, 1, @$lpar[0] . "/" . @$wpar[0] );
              }
            } ## end if ( exists $pack{wtree...})
            else {
              push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, pushFullNode( @$lpar[0], @$lpar[1], $hmc, $srv, 1, @$lpar[2], $pack{hmc}{$hmc}{type} );
            }
          }    # L3 END
          if ( exists $pack{rtree}{$srv} ) {
            $idx{removed} = push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children} }, { "title" => "Removed", "folder" => "true", "children" => [] };
            $idx{removed}--;
            for my $removed ( @{ $pack{rtree}{$srv} } ) {
              push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children}[ $idx{lpars} ]{children}[ $idx{removed} ]{children} }, pushFullNode( @$removed[0], @$removed[1], $hmc, $srv, 1 );
            }
          } ## end if ( exists $pack{rtree...})
        } ## end if ( exists $pack{lhmc...})
        my $ref = $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children};
        if ( exists $pack{lantree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "lan", $ref, "LAN", "N" );
        }
        if ( exists $pack{santree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "san", $ref, "SAN", "Y" );
        }
        if ( exists $pack{sastree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "sas", $ref, "SAS", "s" );
        }
        if ( exists $pack{sriovtree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "sriov", $ref, "SR-IOV", "r" );
        }
        if ( exists $pack{heatree}{$srv}{$hmc} ) {
          sub_tree_ref( $srv, $hmc, "hea", $ref, "HEA", "HEA" );
        }
      } ## end if ( !$pack{times}{$srv...})
    } ## end for my $srv ( sort keys...)
  } ## end for my $hmc ( sort keys...)
} ## end sub genHMCs

sub globalHmcTotals {
  if ( $pack{lhmc} ) {
    if ( $idx{psrv} ) {
      $idx{hmc} = push @{ $menu[ $idx{psrv} ]{children} }, { "title" => "HMC Totals", "folder" => "true", "children" => [] };
    }
    else {
      $idx{hmc} = push @menu, { "title" => "HMC Totals", "folder" => "true", "children" => [] };
    }
    $idx{hmc}--;
    for my $hmc ( sort keys %{ $pack{hmc} } ) {
      if ( $pack{hmc}{$hmc}{type} eq "P" ) {
        if ( $idx{psrv} ) {
          push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children} }, pushmenu( $hmc, $pack{hmc}{$hmc}{url}, $pack{hmc}{$hmc}{id} );
        }
        else {
          push @{ $menu[ $idx{hmc} ]{children} }, pushmenu( $hmc, $pack{hmc}{$hmc}{url}, $pack{hmc}{$hmc}{id} );
        }
      } ## end if ( $pack{hmc}{$hmc}{...})
    } ## end for my $hmc ( sort keys...)
  } ## end if ( $pack{lhmc} )
} ## end sub globalHmcTotals

### Generate HMC select tree
sub genHmcSelect {
  my $n1 = "";
  for my $hmc ( sort keys %{ $pack{lhmc} } ) {
    if ( ( !$pack{hmc}{$hmc} || $pack{hmc}{$hmc}{type} && $pack{hmc}{$hmc}{type} ne "P" ) || $hmc eq "no_hmc" ) {
      next;
    }
    print $n1 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $srv ( sort keys %{ $pack{lhmc}{$hmc} } ) {
      my $extra = "";
      if ( !$pack{times}{$srv}{"active"} ) {
        $extra = "\"extraClasses\":\"removed\",";
      }
      print $n2 . "{\"title\":\"$srv\",$extra\"folder\":\"true\",\"children\":[\n";
      $n2 = ",";

      # print "{\"title\":\"LPAR\",\"folder\":\"true\",\"children\":[\n";
      my $n3 = "";
      for my $lpar ( @{ $pack{lhmc}{$hmc}->{$srv} } ) {
        my $noalias = @$lpar[0];
        $noalias =~ s/ \[.*\]//g;
        my $value = "$hmc|$srv|$noalias";
        print $n3 . "{\"title\":\"@$lpar[0]\",\"key\":\"$value\"}";
        $n3 = ",";
      }    # L3 END
      if ( exists $pack{rtree}{$srv} ) {
        for my $removed ( @{ $pack{rtree}{$srv} } ) {
          my $value = "$hmc|$srv|@$removed[0]";
          my $lextra = $extra ? "" : "\"extraClasses\":\"removed\",";
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",$lextra\"key\":\"$value\"}";
        }
      } ## end if ( exists $pack{rtree...})
      print "]}";
    }    # L2 END
    print "]}\n";
  } ## end for my $hmc ( sort keys...)
} ## end sub genHmcSelect

### Generate lpar list for select inputs
sub genLpars {
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( ! exists $pack{times}{$srv}{type} || $pack{times}{$srv}{type} ne "P" ) {
      next;
    }
    my $extra = "";
    if ( !$pack{times}{$srv}{"active"} ) {
      $extra = "\"extraClasses\":\"removed\",";
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",$extra\"key\":\"_$srv\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $hmc (
      sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
      keys %{ $pack{times}{$srv}{timestamp} }
      )
    {

      #		print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      #			$n2 = ",";
      my $n3 = "";
      for my $lpar ( sort keys %{ $pack{lpartree}{$srv} } ) {
        my $noalias = $lpar;
        $noalias =~ s/ \[.*\]//g;
        my $value = "$hmc|$srv|$noalias";
        print $n3 . "{\"title\":\"$lpar\",\"key\":\"$value\"}";
        $n3 = ",";
      }    # L3 END
      if ( exists $pack{rtree}{$srv} ) {
        for my $removed ( @{ $pack{rtree}{$srv} } ) {
          my $value = "$hmc|$srv|@$removed[0]";
          my $lextra = $extra ? "" : "\"extraClasses\":\"removed\",";
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",$lextra\"key\":\"$value\"}";
          $n3 = ",";
        } ## end for my $removed ( @{ $pack...})
      } ## end if ( exists $pack{rtree...})
      print "]}";
      last;
    }    # L2 END
  } ## end for my $srv ( sort keys...)
} ## end sub genLpars

### Generate lpar list for CPU Workload Estimator (just POWER servers)
sub genLparsEst {
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( ( exists $pack{times}{$srv}{type} && $pack{times}{$srv}{type} ne "P" ) || exists $pack{lhmc}{no_hmc}{$srv} ) {
      next;
    }
    my $extra = "";
    if ( !$pack{times}{$srv}{"active"} ) {
      $extra = "\"extraClasses\":\"removed\",";
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",$extra\"key\":\"_$srv\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $hmc (
      sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
      keys %{ $pack{times}{$srv}{timestamp} }
      )
    {

      #		print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      #			$n2 = ",";
      my $n3 = "";
      for my $lpar ( sort keys %{ $pack{lpartree}{$srv} } ) {
        my $noalias = $lpar;
        $noalias =~ s/ \[.*\]//g;
        my $value = "$hmc|$srv|$noalias";
        print $n3 . "{\"title\":\"$lpar\",\"key\":\"$value\"}";
        $n3 = ",";
      }    # L3 END
      if ( exists $pack{rtree}{$srv} ) {
        for my $removed ( @{ $pack{rtree}{$srv} } ) {
          my $value = "$hmc|$srv|@$removed[0]";
          my $lextra = $extra ? "" : "\"extraClasses\":\"removed\",";
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",$lextra\"key\":\"$value\"}";
          $n3 = ",";
        } ## end for my $removed ( @{ $pack...})
      } ## end if ( exists $pack{rtree...})
      print "]}";
      last;
    }    # L2 END
  } ## end for my $srv ( sort keys...)
} ## end sub genLparsEst

### Generate lpar list for select inputs
sub aclSelect {
  &readMenu();
  ### Generate JSON
  print "[\n";    # envelope begin
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{times} } ) {
    if ( $pack{times}{$srv}{type} ne "P" ) {
      next;
    }
    my $extra = "";
    if ( !$pack{times}{$srv}{"active"} ) {
      $extra = "\"extraClasses\":\"removed\",";
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",$extra\"key\":\"_$srv\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $hmc (
      sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
      keys %{ $pack{times}{$srv}{timestamp} }
      )
    {

      #		print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      #			$n2 = ",";
      my $n3 = "";
      for my $lpar ( keys %{ $pack{lpartree}{$srv} } ) {
        my $noalias = @$lpar;
        $noalias =~ s/ \[.*\]//g;
        my $value = "$hmc|$srv|$noalias";
        print $n3 . "{\"title\":\"$lpar\",\"key\":\"$value\"}";
        $n3 = ",";
      }    # L3 END
      if ( exists $pack{rtree}{$srv} ) {
        for my $removed ( @{ $pack{rtree}{$srv} } ) {
          my $value = "$hmc|$srv|@$removed[0]";
          my $lextra = $extra ? "" : "\"extraClasses\":\"removed\",";
          print $n3 . "\n" . "{\"title\":\"@$removed[0]\",$lextra\"key\":\"$value\"}";
          $n3 = ",";
        } ## end for my $removed ( @{ $pack...})
      } ## end if ( exists $pack{rtree...})
      print "]}";
      last;
    }    # L2 END
  } ## end for my $srv ( sort keys...)
  print "\n]\n";    # envelope end
} ## end sub aclSelect

### Generate pool list for select inputs
sub genPools {
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{lstree} } ) {
    if ( exists $pack{times}{$srv}{type} && $pack{times}{$srv}{type} eq "V" ) {
      next;
    }
    my $extra = "";
    if ( !$pack{times}{$srv}{"active"} ) {
      $extra = "\"extraClasses\":\"removed\",";
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",$extra\"key\":\"_$srv\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";
    for my $hmc ( sort keys %{ $pack{lstree}{$srv} } ) {

      #		print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      #			$n2 = ",";
      my $n3 = "";
      for my $lpar ( @{ $pack{stree}{$srv}->{$hmc} } ) {
        if ( @$lpar[0] =~ /pool/ ) {
          my $value = "$hmc|$srv|@$lpar[2]";
          print $n3 . "{\"title\":\"@$lpar[0]\",\"key\":\"$value\"}";
          $n3 = ",";
        }
      }    # L3 END
      print "]}";
      last;
    }    # L2 END
  } ## end for my $srv ( sort keys...)
} ## end sub genPools

### Generate pool list for Estimator
sub genPoolsEst {
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{lpartree} } ) {
    if ( exists $pack{times}{$srv}{type} && $pack{times}{$srv}{type} ne "P" ) {
      next;
    }
    my $extra = "";
    if ( !$pack{times}{$srv}{"active"} ) {
      $extra = "\"extraClasses\":\"removed\",";
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",$extra\"key\":\"_$srv\",\"children\":[\n";
    $n1 = ",";
    my $n2 = "";

    # for my $hmc ( sort keys %{ $pack{lstree}{$srv} } ) {
    for my $hmc (
      sort { $pack{times}{$srv}{timestamp}{$b} <=> $pack{times}{$srv}{timestamp}{$a} }
      keys %{ $pack{times}{$srv}{timestamp} }
      )
    {
      #		print $n2 . "{\"title\":\"$hmc\",\"folder\":\"true\",\"children\":[\n";
      # print STDERR "HMC: $hmc\n";
      #			$n2 = ",";
      #$hmc = $hmc[0];
      my $n3 = "";
      for my $lpar ( @{ $pack{stree}{$srv}->{$hmc} } ) {
        if ( @$lpar[0] =~ /pool/ ) {
          if ( !( @$lpar[0] =~ /DefaultPool/ ) ) {
            my $value = "$hmc|$srv|@$lpar[2]";
            print $n3 . "{\"title\":\"@$lpar[0]\",\"key\":\"$value\"}";
            $n3 = ",";
          }
        } ## end if ( @$lpar[0] =~ /pool/)
      }    # L3 END
      print "]}";
      last;
    }    # L2 END
  } ## end for my $srv ( sort keys...)
} ## end sub genPoolsEst

### Generate pool list for non-global historical reports
sub genServerPools {
  my ( $hmc, $srv ) = @_;
  my $n3 = "";
  for my $lpar ( @{ $pack{stree}{$srv}->{$hmc} } ) {
    if ( @$lpar[0] =~ /pool/ ) {
      my $value = "@$lpar[2]";
      if ( $value eq "pool" ) {
        print $n3 . txtkeysel( @$lpar[0], $value );
      }
      else {
        print $n3 . txtkey( @$lpar[0], $value );
      }
      $n3 = ",";
    } ## end if ( @$lpar[0] =~ /pool/)
  } ## end for my $lpar ( @{ $pack...})
  print $n3;
} ## end sub genServerPools

### Generate Custom groups list for select inputs
sub genCusts {
  my $delim = '';
  foreach my $cgroup ( sort keys %{ $pack{ctreeh} } ) {
    if ( $cgroup ne "Custom groups" && $cgroup ne "<b>Configuration</b>" ) {
      my $egrp = urlencode( $cgroup );
      print $delim . "{\"title\":\"$cgroup\",\"obj\":\"C\",\"key\":\"$egrp\"}";
      $delim = ",\n";
    }
  }
} ## end sub genCusts

### Generate VMware Clusters list for select inputs
sub genClusters {
  my $vc = shift;
  &readMenu;
  print "[\n";    # envelope begin
  my $delim = '';
  foreach my $cl ( keys %{ $pack{cluster}{$vc} } ) {
    ( my $cls = $cl ) =~ s/cluster_//;
    print $delim . "{\"title\":\"$cls\",\"key\":\"$cl\"}";
    $delim = ",\n";
  }
  print "\n]\n";    # envelope end
} ## end sub genClusters
### Generate VMware DataStores list for select inputs
sub genDataStores {
  my $vc = shift;
  &readMenu;
  print "[\n";      # envelope begin
  my $delim = '';
  if ( exists $pack{datastore}{$vc} ) {
    foreach my $dc ( keys %{ $pack{datastore}{$vc} } ) {
      ( my $dcs = $dc ) =~ s/datastore_//;
      print $delim . "{\"title\":\"$dcs\",\"folder\":\"true\",\"key\":\"_$dc\",\"children\":[\n";
      $delim = ",\n";
      my $delim1 = '';
      foreach my $ds ( keys %{ $pack{datastore}{$vc}{$dc} } ) {
        my $key = "$dc:$ds";
        print $delim1 . "{\"title\":\"$ds\",\"key\":\"$key\"}";
        $delim1 = ",\n";
      }
      print "]}";
    } ## end foreach my $dc ( keys %{ $pack...})
  } ## end if ( exists $pack{datastore...})
  print "\n]\n";    # envelope end
} ## end sub genDataStores
### Generate VMware resPools list for select inputs
sub genResPools {
  my $vc = shift;
  &readMenu;
  print "[\n";      # envelope begin
  my $delim = '';
  if ( exists $pack{respool}{$vc} ) {
    foreach my $cl ( keys %{ $pack{respool}{$vc} } ) {
      ( my $cls = $cl ) =~ s/cluster_//;
      print $delim . "{\"title\":\"$cls\",\"folder\":\"true\",\"key\":\"_$cl\",\"children\":[\n";
      $delim = ",\n";
      if ( ref( $pack{respool}{$vc}{$cl} ) eq "HASH" ) {
        my $delim1 = '';
        foreach my $rp ( keys %{ $pack{respool}{$vc}{$cl} } ) {
          my $key = "$cl:$rp";
          print $delim1 . "{\"title\":\"$rp\",\"key\":\"$key\"}";
          $delim1 = ",\n";
        }
      } ## end if ( ref( $pack{respool...}))
      print "]}";
    } ## end foreach my $cl ( keys %{ $pack...})
  } ## end if ( exists $pack{respool...})
  print "\n]\n";    # envelope end
} ## end sub genResPools
### Generate VMware VM list for select inputs
sub genVMs {
  my $vc = shift;
  &readMenu;
  print "[\n";      # envelope begin
  my $delim = '';
  foreach my $cl ( keys %{ $pack{cluster}{$vc} } ) {
    if ( exists $pack{lhmc}{$cl} ) {
      ( my $cls = $cl ) =~ s/cluster_//;
      my $n3 = "";
      print $delim . "{\"title\":\"$cls\",\"folder\":\"true\",\"children\":[\n";
      $delim = ",";
      my %allVMs;
      foreach my $srv ( keys %{ $pack{lhmc}{$cl} } ) {
        foreach my $vm ( @{ $pack{lhmc}{$cl}{$srv} } ) {
          $allVMs{ @$vm[0] }{url} = @$vm[1];
          $allVMs{ @$vm[0] }{alt} = @$vm[2];
          $allVMs{ @$vm[0] }{srv} = $srv;
        }
      } ## end foreach my $srv ( keys %{ $pack...})
      for my $lpar ( sort { lc($a) cmp lc($b) } keys %allVMs ) {
        print $n3 . "{\"title\":\"$lpar\",\"key\":\"$allVMs{$lpar}{alt}\"}";

        # print $n3 . &fullNode( $lpar, $allVMs{$lpar}{url}, $vc, $allVMs{$lpar}{srv}, 1, $allVMs{$lpar}{alt}, "V", $cl );
        $n3 = ",";
      }    # L3 END
      print "]}\n";
    } ## end if ( exists $pack{lhmc...})
  } ## end foreach my $cl ( keys %{ $pack...})
  print "\n]\n";    # envelope end
} ## end sub genVMs

sub histReport {
  my ( $hmc, $srv, $type, $hostname ) = @_;
  print "[\n";
  $type ||= "power";

  #print txtkeysel("All CPU pools", "pool") . ",\n";
  my $vmname = "LPAR";
  if ( $type eq "vmware" ) {
    $vmname = "VM";
    if ( $hmc eq "solo_esxi" ) {
      $hmc = "";
    }
    &genServerPools( $hostname, $srv );
  } ## end if ( $type eq "vmware")
  else {
    &genServerPools( $hmc, $srv );
    print txtkeysel( $vmname . "s aggregated", "multiview" ) . ",\n";
  }

  print txtkey( "Memory", "mem" ) . "\n";

  if ( $type eq "vmware" ) {
    print "," . txtkey( "Disk", "disk" ) . "\n";
    print "," . txtkey( "Net",  "net" ) . "\n";
  }

  # if ($type ne "vmware") {
  print ",{\"title\":\"$vmname\",\"folder\":\"true\",\"key\":\"_$hmc-$srv\",\"expanded\":true,\"children\":[\n";
  my $n3 = "";
  if ( $type eq "power" ) {
    for my $lpar ( sort keys %{ $pack{lpartree}{$srv} } ) {
      print $n3 . "{\"title\":\"$lpar\",\"key\":\"$lpar\"}";
      $n3 = ",\n";
    }
  }
  else {
    for my $lpar ( @{ $pack{lstree}{$srv}->{$hmc} } ) {
      my $value = ( $type ne "vmware" ) ? "@$lpar[0]" : "@$lpar[2]";
      print $n3 . "{\"title\":\"@$lpar[0]\",\"key\":\"$value\"}";
      $n3 = ",\n";
    }
  }
  if ( exists $pack{rtree}{$srv} ) {
    for my $removed ( @{ $pack{rtree}{$srv} } ) {
      print $n3 . "\n" . "{\"title\":\"@$removed[0]\",\"extraClasses\":\"removed\",\"key\":\"@$removed[0]\"}";
    }
  }
  print "]}";

  # }

  print "]";

} ## end sub histReport

sub urlencode {
  my $s = shift;

  # $s =~ s/ /+/g;
  $s =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

  # $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  if ($s) {
    $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  }
  return $s;
} ## end sub urldecode

# encode
# $string =~ s/([^^A-Za-z0-9\-_.!~*'()])/ sprintf "%%%0x", ord $1 /eg;
#
# #decode
# $string =~ s/%([A-Fa-f\d]{2})/chr hex $1/eg;

### Global section without Title
sub globalWoTitle {
  my $fsub = "";
  my $csub = "";
  my $hsub = "";
  my $variant = $env{variant} eq "p" ? "<span style='font-weight: normal'>(free)</span>" : "";

  if ( $pack{ftreeh} ) {
    $fsub = 1;
  }
  if ( $pack{ctreeh} ) {
    $csub = 1;
  }

  #for ( sort keys %{ $pack{global}{""} } ) {
  #  my $t = $pack{global}{""}{$_}{text};
  # if ( ( lc $t eq "favourites" ) && $fsub ) {
  if ($fsub) {
    &favs();
  }

  #} ## end for ( sort keys %{ $pack...
  if ( $env{platform}{power} && scalar keys %{ $env{platform} } == 1 ) {
    for ( sort keys %{ $pack{global}{P} } ) {
      my $t  = $pack{global}{P}{$_}{text};
      my $u  = $pack{global}{P}{$_}{url};
      my $id = $pack{global}{P}{$_}{id};
      if ( lc $t eq "custom groups" || lc $t eq "alerting" ) {
        next;
      }
      if ( lc $t eq "cpu workload estimator"
        || lc $t eq "resource configuration advisor" )
      {
        push @menu, pushmenub( $t, $u, $id );
      }
      else {
        push @menu, pushmenu( $t, $u, $id );
      }
    } ## end for ( sort keys %{ $pack...})
    &globalHmcTotals();
  } ## end if ( $env{platform}{power} &&...)
  elsif ( $env{platform}{vmware} && scalar keys %{ $env{platform} } == 1 ) {
    for ( sort keys %{ $pack{global}{V} } ) {
      my $t  = $pack{global}{V}{$_}{text};
      my $u  = $pack{global}{V}{$_}{url};
      my $id = $pack{global}{V}{$_}{id};
      {
        if ( lc $t eq "cpu workload estimator"
          || lc $t eq "resource configuration advisor" )
        {
          push @menu, pushmenub( $t, $u, $id );
        }
        else {
          push @menu, pushmenu( $t, $u, $id );
        }
      }
    } ## end for ( sort keys %{ $pack...})
    $idx{vmware} = push @menu, { "title" => "VMware", "folder" => "true", "children" => [] };
    $idx{vmware}--;
    #  push @{ $menu[ $idx{vmware} ]{children} }, pushFullNode( "<b>Configure</b>", "/lpar2rrd-cgi/vmwcfg.sh?cmd=getlist", "VMware", "VMware", 0 );
  } ## end elsif ( !$env{platform}{power} &&...)
} ## end sub globalWoTitle

### Favourites
sub favs {
  if ( $pack{ftreeh} ) {
    $idx{favs} = push @menu, { title => "FAVOURITES", folder => "true", children => [] };
    $idx{favs}--;
    for my $fav ( @{ $pack{ftreeh} } ) {
      push @{ $menu[ $idx{favs} ]{children} }, pushmenu( @$fav[0], @$fav[1] );
    }
  } ## end if ( $pack{ftreeh} )
} ## end sub favs
### Custom Groups
sub custs {
  if ( $pack{ctreeh} ) {
    $idx{custom} = push @menu, { title => "CUSTOM GROUPS", folder => "true", children => [] };
    $idx{custom}--;
    # my $delim = ",\n";
    my %coll = CustomGroups::getCollections();
    if ( $coll{collection} ) {
      for my $collname ( sort keys %{ $coll{collection} } ) {
        $idx{cfav} = push @{ $menu[ $idx{custom} ]{children} }, { title => $collname, folder => "true", children => [] };
        $idx{cfav}--;
        for my $cgrp ( sort keys %{ $coll{collection}{$collname} } ) {
          if ( $pack{ctreeh}{$cgrp} ) {
            push @{ $menu[ $idx{custom} ]{children}[ $idx{cfav} ]{children} }, pushFullNode( $cgrp, $pack{ctreeh}{$cgrp}, "na", "na", 0, undef, undef, undef, "C" );
          }
        }

      } ## end for my $collname ( sort...)
    } ## end if ( $coll{collection})
    if ( $coll{nocollection} ) {

      # print Dumper \%ctreeh;
      for my $cgrp ( sort keys %{ $coll{nocollection} } ) {
        if ( $pack{ctreeh}{$cgrp} ) {
          push @{ $menu[ $idx{custom} ]{children} }, pushFullNode( $cgrp, $pack{ctreeh}{$cgrp}, "na", "na", 0, undef, undef, undef, "C" );
        }
      }
    } ## end if ( $coll{nocollection...})
  }
} ## end sub custs

### HMC submenu
sub hmcs {
  print "{\"title\":\"HMC totals\",\"folder\":\"true\",\"children\":[\n";
  my $delim = '';
  for ( $pack{hmc} ) {
    print $delim . &txthref( @$_[0], @$_[1] );
    $delim = ",\n";
  }
  print "]}";
} ## end sub hmcs

### single HMC Totals
sub hmcTotals {
  my ($hmc) = @_;
  if ( exists $pack{hmc}{$hmc} ) {
    push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children} }, pushmenu( "Totals", $pack{hmc}{$hmc}{url} );
  }
} ## end sub hmcTotals

### single vCenter Totals
sub vcTotals {
  my ($vc) = @_;
  if ( exists $pack{vtree}{$vc}{menu} ) {
    foreach ( @{ $pack{vtree}{$vc}{menu} } ) {
      push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children} }, pushmenu( @$_[0], @$_[1] );
    }
  }
} ## end sub vcTotals
### single cluster Totals
sub clTotals {
  my ( $vc, $cl ) = @_;
  if ( exists $pack{cluster}{$vc}{$cl} ) {
    foreach my $title ( keys %{ $pack{cluster}{$vc}{$cl} } ) {
      my $url = $pack{cluster}{$vc}{$cl}{$title};
      my $cl_id = ( $url =~ /host=([^&]*)/ )[0];
      my $vc_id = ( $url =~ /server=([^&]*)/ )[0];
      # warn "$vc, $cl, $cl_id";
      push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children} }, pushFullNode( $title, $url, $cl_id, $vc_id, 1, "nope", "V", $cl );
    }
  }
} ## end sub clTotals
### Tail menu section
sub tail {
  my $freeOrFull = ( $env{free} eq 1 ? "free" : "full" );
  push @menu, { "title" => "LPAR2RRD <span style='font-weight: normal'>($env{version} $freeOrFull)</span>", "folder" => "true", "children" => [] };
  my $delim = '';
  for ( @{ $pack{tail} } ) {
    if ( @$_[0] ne "Access Control" ) {
      if ( !$useacl || @$_[0] eq "Documentation" || @$_[0] eq "User management" ) {
        push @{ $menu[$#menu]{children} }, pushmenu( @$_[0], @$_[1] );
      }
    }
  } ## end for ( @{ $pack{tail} } )

  # push @{ $menu[$#menu]{children} }, pushmenu( "Host configuration", "/lpar2rrd-cgi/hosts.sh?cmd=form" );
  # push @{ $menu[$#menu]{children} }, pushmenu( "User management", "/lpar2rrd-cgi/users.sh?cmd=form" );

  # if ($useacl) {
  #    print $delim . &txthref( "Access Control", "/lpar2rrd-cgi/acl.sh" );
  #}
  #    print $delim . &txthref( "Custom Groups config", "/lpar2rrd-cgi/cgrps.sh" );
  if ($GUIDEBUG) {
    print $delim . &txthref( "Load debug content", "debug.txt" );
  }
} ## end sub tail

### Single Server menu
# params: (hmc, srv)
sub serverMenu {
  my ( $h, $s, $c, $v, $vcwocl ) = @_;
  for ( @{ $pack{stree}{$s}->{$h} } ) {
    if ( defined $v ) {
      if ( defined $c ) {
        if ($vcwocl) {
          push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $c, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
        }
        else {
          push @{ $menu[ $idx{vmware} ]{children}[ $idx{vc} ]{children}[ $idx{cl} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $c, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
        }
      } ## end if ( defined $c )
      else {
        push @{ $menu[ $idx{vmware} ]{children}[ $idx{esxi} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
      }
    } ## end if ( defined $v )
    else {
      if ( $h eq "solaris" ) {
        push @{ $menu[ $idx{solaris} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
      }
      elsif ( $h eq "Hitachi" ) {
        push @{ $menu[ $idx{hitachi} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
      }
      elsif ( $pack{times}{$s}{type} eq "X" ) {
        push @{ $menu[ $idx{xen} ]{children}[ $idx{csrv} ]{children}[ $idx{xlabel} ]{children} }, pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
      }
      else {
        my $obj = "";
        if ( @$_[1] =~ "SharedPool" ) {
          $obj = "SP";
        }
        elsif ( @$_[2] eq "pool" ) {
          $obj = "P";
        }
        if ( !$useacl || $isAdmin || ACL::canShow("POWER", "SERVER", $s) || ACL::canShow("POWER", "POOL", $s, @$_[2]) ) {
          my $node = pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h, $obj );
          if ( @$_[0] eq "VIEW" || @$_[0] eq "Historical reports" ) {
            $node->{extraClasses} = "noregroup";
          }
          push @{ $menu[ $idx{psrv} ]{children}[ $idx{csrv} ]{children} }, $node;
        }
      }
    }
  } ## end for ( @{ $pack{stree}{$s...}})

  # print $delim;
} ## end sub serverMenu

### Single Server menu
# params: (hmc, srv)
sub serverByHmc {
  my ( $h, $s, $c ) = @_;
  for ( @{ $pack{stree}{$s}->{$h} } ) {
    if ( defined $c ) {
      push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $c, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
    }
    else {
      push @{ $menu[ $idx{psrv} ]{children}[ $idx{hmc} ]{children}[ $idx{csrv} ]{children} }, pushFullNode( @$_[0], @$_[1], $h, $s, 0, @$_[2], $pack{times}{$s}{type}, $h );
    }
  } ## end for ( @{ $pack{stree}{$s...}})
} ## end sub serverByHmc

sub lparNames {
  my @unique = sort( do {
      my %seen;
      grep { !$seen{$_}++ } keys %{ $pack{lnames}{P} };
      }
  );
  print "[";
  if (@_) {
    @unique = grep( {/@_/i} @unique );
  }
  my $delim = '';
  for (@unique) {

    #		print Dumper $_;
    print $delim . "{\"value\":\"$_\"}";
    $delim = ",\n";
  } ## end for (@unique)
  print "]";
} ## end sub lparNames

sub genFleetTree {
  print "[";
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{fleet} } ) {
    if ( !$pack{times}{$srv}{"active"} ) {
      next;
    }
    print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",\"children\":[\n";
    $n1 = ",\n";
    my $n2 = "";
    for my $type ( sort keys %{ $pack{fleet}{$srv}{subsys} } ) {
      print $n2 . "{\"title\":\"$type\",\"folder\":\"true\",\"children\":[";
      $n2 = ",\n";
      my $n3 = "";

      # my @uni = uniq( @{ $pack{fleet}{$srv}{$type} } );
      foreach my $name ( sort keys %{ $pack{fleet}{$srv}{subsys}{$type} } ) {
        print $n3 . "{\"title\":\"$name\"}";
        $n3 = ",";
      }    # L3 END
      print "]}";
    }    # L2 END
    print "]}";
  } ## end for my $srv ( sort keys...)
  print "]\n";
} ## end sub genFleetTree

sub genFleet {
  print "{";
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{fleet} } ) {
    if ( !$pack{times}{$srv}{"active"} || !$pack{times}{$srv}{type} || $pack{times}{$srv}{type} ne "P" ) {
      next;
    }
    print $n1 . "\"$srv\":{";
    $n1 = ",\n";
    my $n2 = "";
    for my $type ( sort keys %{ $pack{fleet}{$srv}{subsys} } ) {
      print $n2 . "\"$type\":[";
      $n2 = ",\n";
      my $n3 = "";

      # my @uni = uniq( @{ $pack{fleet}{$srv}{$type} } );

      # print Dumper \@uni;
      foreach my $name ( keys %{ $pack{fleet}{$srv}{subsys}{$type} } ) {
        print $n3 . "\"$name\"";
        $n3 = ",";
      }    # L3 END
      print "]";
    }    # L2 END
    print "}";
  } ## end for my $srv ( sort keys...)
  print "}\n";
} ## end sub genFleet

sub genFleetAlrt {
  print "{";
  my $n1 = "";
  foreach my $srv ( sort keys %{ $pack{fleet} } ) {
    if ( $pack{fleet}{$srv}{platform} eq "P" && ( !$pack{times}{$srv}{"active"} || !$pack{times}{$srv}{type} ) ) {
      next;
    }
    if ( $pack{fleet}{$srv}{platform} eq "V" && !exists $pack{fleet}{$srv}{subsys}{LPAR} ) {
      next;
    }
    print $n1 . "\"$srv\":{\"platform\":\"$pack{fleet}{$srv}{platform}\",\"subsys\":{";
    $n1 = ",\n";
    my $n2 = "";
    foreach my $subsys ( sort keys %{ $pack{fleet}{$srv}{subsys} } ) {
      print $n2 . "\"$subsys\":[";
      $n2 = ",\n";
      my $n3 = "";

      # my @uni = uniq( @{ $pack{fleet}{$srv}{$type} } );

      # print Dumper \@uni;
      foreach my $name ( sort keys %{ $pack{fleet}{$srv}{subsys}{$subsys} } ) {
        my $uuid = "";
        my $cluster = "";

        if ( $pack{fleet}{$srv}{uuid}{$name} ) {
          $uuid = $pack{fleet}{$srv}{uuid}{$name};
        }
        if ( $pack{fleet}{$srv}{cluster}{$name} ) {
          $cluster = $pack{fleet}{$srv}{cluster}{$name};
        }
        print $n3 . "[\"$name\",\"$pack{fleet}{$srv}{subsys}{$subsys}{$name}{type}\",\"$uuid\",\"$cluster\"]";
        $n3 = ",";
      }    # L3 END
      print "]";
    }    # L2 END
    print "}}";
  } ## end for my $srv ( sort keys...)
  print "}\n";
} ## end sub genFleetAlrt

sub genFleetReport {
  my %abbr = (
    P => "POWER",
    V => "VMWARE",
    U => "UNMANAGED"
  );
  my %subsystems = (
    CLUSTER       => "Cluster totals",
    RESPOOL       => "Resource pool",
    VM            => "Virtual machine",
    DATASTORE     => "Datastore",
    STORAGEDOMAIN => "Storage Domain",
    ESXI          => "Host (ESXi)",
    HOST          => "Host",
    DISK          => "Disk",
    LDOM          => "LDOM"
  );
  my %fl;
  foreach my $srv ( sort keys %{ $pack{fleet} } ) {
    my $platform = $pack{fleet}{$srv}{platform};
    my $hmc;
    if ( $pack{stree}{$srv} ) {
      $hmc = ( keys %{ $pack{stree}{$srv} } )[0];
    }
    if ( !$platform || $platform ne "P" || ( $platform eq "P" && ( !$pack{times}{$srv}{"active"} || !$pack{times}{$srv}{type} ) ) ) {
      next;
    }
    $fl{POWER}{$srv}{subsys}{SERVER} = [];
    push @{ $fl{POWER}{$srv}{subsys}{SERVER} }, {hmc => $hmc, name => "Memory", value => "pool", type => "pool"};
    foreach my $subsys ( sort keys %{ $pack{fleet}{$srv}{subsys} } ) {
      $fl{POWER}{$srv}{subsys}{$subsys} = [];
      foreach my $name ( sort keys %{ $pack{fleet}{$srv}{subsys}{$subsys} } ) {
        my %item = ( name => $name);
        if ( $pack{fleet}{$srv}{subsys}{$subsys}{$name}{type} ) {
          $item{type} = $pack{fleet}{$srv}{subsys}{$subsys}{$name}{type};
        }
        if ( $pack{fleet}{$srv}{subsys}{$subsys}{$name}{value} ) {
          $item{value} = $pack{fleet}{$srv}{subsys}{$subsys}{$name}{value};
        }
        if ( $pack{fleet}{$srv}{uuid}{$name} ) {
          $item{uuid} = $pack{fleet}{$srv}{uuid}{$name};
        }
        if ( $pack{fleet}{$srv}{cluster}{$name} ) {
          $item{cluster} = $pack{fleet}{$srv}{cluster}{$name};
        }
        if ( $hmc ) {
          $item{hmc} = $hmc;
        }
        push @{ $fl{POWER}{$srv}{subsys}{$subsys} },  \%item;
      }    # L3 END
    }    # L2 END
  } ## end for my $srv ( sort keys...)
  foreach my $srv ( sort keys %{ $pack{vtree} } ) {
    my $srvalias = $pack{vtree}{$srv}{alias};
    my %vcss;
    if ( $pack{vtree}{$srv}{cluster} ) {
      $vcss{CLUSTER} ||= 1;
      foreach my $cname ( sort keys %{ $pack{vtree}{$srv}{cluster} } ) {
        my %item = ();
        ( my $cls = $cname ) =~ s/cluster_//;
        if ( $pack{vtree}{$srv}{cluster}{$cname}{host} ) {
          $vcss{ESXI} ||= 1;
          $item{ESXI} = ();
          foreach my $name ( sort keys %{ $pack{vtree}{$srv}{cluster}{$cname}{host} } ) {
            push @{ $item{ESXI} }, {name => $name};
          }
        }
        if ( $pack{respool}{$srv}{$cname} ) {
          $vcss{RESPOOL} ||= 1;
          $item{RESPOOL} = ();
          foreach my $name ( sort keys %{ $pack{respool}{$srv}{$cname} } ) {
            push @{ $item{RESPOOL} }, {name => $name};
          }
        }
        if ( $pack{lhmc}{$cname} ) {
          $vcss{VM} ||= 1;
          $item{VM} = ();
          foreach my $esxi ( keys %{ $pack{lhmc}{$cname} } ) {
            foreach my $name ( @{ $pack{lhmc}{$cname}{$esxi} } ) {
              push @{ $item{VM} }, {name => @{ $name }[0], uuid => @{ $name }[2]};
            }
          }
        }
        $fl{VMWARE}{$srvalias}{inventory}{CLUSTER}{$cls} = \%item;
      }
    }
    if ( $pack{datastore}{$srv} ) {
      $vcss{DATASTORE} ||= 1;
      foreach my $dname ( sort keys %{ $pack{datastore}{$srv} } ) {
        my %item = ();
        ( my $dc = $dname ) =~ s/datastore_//;
        if ( $pack{datastore}{$srv}{$dname} ) {
          $item{DATASTORE} = ();
          foreach my $name ( sort keys %{ $pack{datastore}{$srv}{$dname} } ) {
            push @{ $item{DATASTORE} }, {name => $name};
          }
        }
        $fl{VMWARE}{$srvalias}{inventory}{DATACENTER}{$dc} = \%item;
      }    # l3 end
    }
    $vcss{subsys} = ();
    foreach my $ss ( qw/CLUSTER RESPOOL VM DATASTORE ESXI/ ) {
      if ( $vcss{$ss} ) {
        push @{ $vcss{subsys} }, { value => $ss, text => $subsystems{$ss} };
      }
    }
    $fl{VMWARE}{$srvalias}{subsys} = $vcss{subsys};
  }

  # build oVirt structures
  my $omenu = "$basedir/data/oVirt/metadata.json";
  if( -e $omenu && -f _ && -r _ ) {
    use OVirtDataWrapper;
    foreach my $datacenter_uuid ( @{ OVirtDataWrapper::get_uuids( 'datacenter' ) } ) {
      my %vcss;
      my $datacenter_label  = OVirtDataWrapper::get_label( 'datacenter', $datacenter_uuid );
      my %item = ();
      foreach my $cluster_uuid ( @{ OVirtDataWrapper::get_arch( $datacenter_uuid, 'datacenter', 'cluster' ) } ) {
        $vcss{CLUSTER} ||= 1;
        my $cluster_label  = OVirtDataWrapper::get_label( 'cluster', $cluster_uuid );
        $item{CLUSTER}{$cluster_uuid}{label} = $cluster_label;
        foreach my $host_uuid ( @{ OVirtDataWrapper::get_arch( $cluster_uuid, 'cluster', 'host' ) } ) {
          $vcss{HOST} ||= 1;
          my $host_label  = OVirtDataWrapper::get_label( 'host', $host_uuid );
          push @{ $item{CLUSTER}{$cluster_uuid}{HOST} }, {name => $host_label, uuid => $host_uuid};
        }
        foreach my $vm_uuid ( @{ OVirtDataWrapper::get_arch( $cluster_uuid, 'cluster', 'vm' ) } ) {
          $vcss{VM} ||= 1;
          my $vm_label  = OVirtDataWrapper::get_label( 'vm', $vm_uuid );
          push @{ $item{CLUSTER}{$cluster_uuid}{VM} }, {name => $vm_label, uuid => $vm_uuid};
        }
      }
      foreach my $storage_domain_uuid ( @{ OVirtDataWrapper::get_arch( $datacenter_uuid, 'datacenter', 'storage_domain' ) } ) {
        $vcss{STORAGEDOMAIN} ||= 1;
        my $storage_domain_label = OVirtDataWrapper::get_label( 'storage_domain', $storage_domain_uuid );
        $item{STORAGEDOMAIN}{$storage_domain_uuid}{label} = $storage_domain_label;
        foreach my $disk_uuid ( @{ OVirtDataWrapper::get_arch( $storage_domain_uuid, 'storage_domain', 'disk' ) } ) {
          $vcss{DISK} ||= 1;
          my $disk_label  = OVirtDataWrapper::get_label( 'disk', $disk_uuid );
          push @{ $item{STORAGEDOMAIN}{$storage_domain_uuid}{DISK} }, {name => $disk_label, uuid => $disk_uuid};
        }
      }
      $fl{OVIRT}{$datacenter_uuid}{inventory} = \%item;
      $vcss{subsys} = ();
      foreach my $ss ( qw/CLUSTER RESPOOL VM STORAGEDOMAIN DISK HOST/ ) {
        if ( $vcss{$ss} ) {
          push @{ $vcss{subsys} }, { value => $ss, text => $subsystems{$ss} };
        }
      }
      $fl{OVIRT}{$datacenter_uuid}{label} = $datacenter_label;
      $fl{OVIRT}{$datacenter_uuid}{subsys} = $vcss{subsys};
    }
  }
  foreach my $srv ( sort keys %{ $pack{soltree}{host} } ) {
    if ( $pack{soltree}{host}{$srv}{ldom} ) {
      push @{ $fl{SOLARIS}{LDOM} }, $srv;
    }
  }
  print JSON->new->utf8(0)->pretty()->encode( \%fl );
} ## end sub genFleetReport

sub tobeerased {
  my %fl;
  my $omenu = "$basedir/data/oVirt/metadata.json";
  if( -e $omenu && -f _ && -r _ ) {
    open( CFG, $omenu );
    {
      local $/ = undef;    # required for re-read of encode_json pretty output
      my $rawcfg = <CFG>;
      if ($rawcfg) {
        my %config = %{ decode_json($rawcfg) };
        my %vcss;
        foreach my $dc ( sort keys %{ $config{architecture}{datacenter} } ) {
          my $dcname = $config{labels}{datacenter}{$dc};
          foreach my $cl ( sort $config{architecture}{datacenter}{$dc}{cluster} ) {
            $vcss{CLUSTER} ||= 1;
            my $clname = $config{labels}{cluster}{$cl};
            my %item = ();
            $item{$clname} = ();
            if ( exists $config{architecture}{cluster}{$cl}{host} ) {
              $vcss{HOST} ||= 1;
              $item{$clname}{HOST} = ();
              foreach my $name ( sort @{ $config{architecture}{cluster}{$cl}{host} } ) {
                my $lname = $config{labels}{host}{$name};
                push @{ $item{$clname}{HOST} }, {name => $lname};
              }
            }
            if ( exists $config{architecture}{cluster}{$cl}{vm} ) {
              $vcss{VM} ||= 1;
              $item{$clname}{VM} = ();
              foreach my $name ( sort @{ $config{architecture}{cluster}{$cl}{vm} } ) {
                my $lname = $config{labels}{host}{$name};
                push @{ $item{$clname}{VM} }, {name => $lname, uuid => $name };
              }
            }
            $fl{OVIRT}{$dcname}{inventory}{CLUSTER} = \%item;
          }
        }
        # $fl{OVIRT} = \%config;
      }
    }
    close CFG;
  }
}
sub genFleetAll {
  print "{";
  my $n1 = "";
  for my $srv ( sort keys %{ $pack{lnames}{P} } ) {
    if ( !$pack{times}{$srv}{"active"} ) {
      next;
    }
    if ( !$pack{fleet}{$srv}{subsys}{LPAR} ) {
      next;
    }
    my @tkeys = keys %{ $pack{fleet}{$srv}{subsys}{LPAR} };
    if ( $pack{fleet}{$srv}{subsys}{LPAR}{$tkeys[0]} =~ /^(M|S|L)$/ ) {
      next;
    }
    print $n1 . "\"$srv\":{";
    $n1 = ",\n";
    my $n2 = "";
    for my $type ( sort keys %{ $pack{fleet}{$srv}{subsys} } ) {
      print $n2 . "\"$type\":[";
      $n2 = ",\n";
      my $n3 = "";

      # print Dumper \@uni;
      foreach my $name ( keys %{ $pack{fleet}{$srv}{subsys}{$type} } ) {
        print $n3 . "\"$name\"";
        $n3 = ",";
      }    # L3 END
      print "]";
    }    # L2 END
    print "}";
  } ## end for my $srv ( sort keys...)
  for my $srv ( sort keys %{ $pack{lnames}{V} } ) {
    print $n1 . "\"$srv\":{";
    $n1 = ",\n";
    print "\"VM\":[";
    my $n3  = "";
    my @uni = uniq( keys %{ $pack{lnames}{V}{$srv} } );

    # print Dumper \@uni;
    foreach my $name (@uni) {
      print $n3 . "\"$name\"";
      $n3 = ",";
    }    # L3 END
    print "]";
    print "}";
  } ## end for my $srv ( sort keys...)
  if ( $pack{ovirt} ) {
    use OVirtDataWrapper;
    foreach my $datacenter_uuid ( @{ OVirtDataWrapper::get_uuids( 'datacenter' ) } ) {
      foreach my $cluster_uuid ( @{ OVirtDataWrapper::get_arch( $datacenter_uuid, 'datacenter', 'cluster' ) } ) {
        my $cluster_label  = OVirtDataWrapper::get_label( 'cluster', $cluster_uuid );
        print $n1 . "\"$cluster_label\":{";
        $n1 = ",\n";
        print "\"OVIRTVM\":[";
        my $n3  = "";
        foreach my $vm_uuid ( @{ OVirtDataWrapper::get_arch( $cluster_uuid, 'cluster', 'vm' ) } ) {
          my $vm_label  = OVirtDataWrapper::get_label( 'vm', $vm_uuid );
          print $n3 . "\"$vm_label\"";
          $n3 = ",";
        }
        print "]";
        print "}";
      }
    }
  }
  if ( $pack{xen} ) {
    for my $srv ( sort @{ $pack{xen}{children} } ) {
      if ( $srv->{children} ) {
        print $n1 . "\"$srv->{title}\":{";
        $n1 = ",\n";
        print "\"XENVM\":[";
        my $n3  = "";
        foreach my $child (@{ $srv->{children} } ) {
          # print Dumper \@uni;
          if ( $child->{title} && $child->{title} eq "VM" ) {
            foreach my $name (@{ $child->{children} }) {
              print $n3 . "\"$name->{title}\"";
              $n3 = ",";
            }    # L3 END
          }
        }
        print "]";
        print "}";
      }
    } ## end for my $srv ( sort keys...)
  }
  if ( $pack{soltree} ) {
    for my $srv ( sort keys %{ $pack{soltree}{host} } ) {
      if ( $pack{soltree}{host}{$srv}{zone} ) {
        print $n1 . "\"$srv\":{";
        $n1 = ",\n";
        my $n2 = "";
        if ( $pack{soltree}{host}{$srv}{zone}{total} ) {
          print "\"SOLARISLDOM\":[\"total\"]";
          $n2 = ",\n";
        }
        print $n2 . "\"SOLARISZONE\":[";
        my $n3  = "";
        foreach my $zone ( sort keys %{ $pack{soltree}{host}{$srv}{zone} } ) {
          if ( $zone ne "global" && $zone ne "total" && $zone ne "system" ) {
            print $n3 . "\"$zone\"";
            $n3 = ",";
          }
        }
        print "]";
        print "}";
      }
    } ## end for my $srv ( sort keys...)
  }
  print "}\n";
} ## end sub genFleetAll

sub genCustGrps {
  my %cgrps = CustomGroups::getGrp();
  my @cgtree;
  for my $cgrp ( sort keys %cgrps ) {
    my $hcg = { title => $cgrp, folder => \1, type => $cgrps{$cgrp}{'type'}, loaded => \1, children => [] };
    my $onList = ACL::canShow( "C", $cgrp );
    if ( $useacl && !$isAdmin && !$onList  ) {
      next;
    }
    if ( $cgrps{$cgrp}{'collection'} ) {
      $hcg->{collection} = $cgrps{$cgrp}{'collection'};
    }
    for my $src ( sort keys %{ $cgrps{$cgrp}{'children'} } ) {
      my $hsrc = { title => $src, folder => \1, children => [] };
      for my $name ( @{ $cgrps{$cgrp}{'children'}{$src} } ) {
        push @{ $hsrc->{children} }, { title => $name };
      }
      push @{ $hcg->{children} }, $hsrc;
    }
    push @cgtree, $hcg;
  }
  print JSON->new->pretty(1)->utf8(0)->encode( \@cgtree );
} ## end sub genCustGrps

sub genAlertTree {
  print "[";
  my $n1     = "";
  my %alerts = Alerting::getAlerts();

  # print Dumper \%alerts;
  foreach my $srv ( sort keys %alerts ) {
    if ( $srv eq "" ) {
      print $n1 . "{\"title\":\"--- ALL SERVERS ---\",\"folder\":\"true\",\"children\":[\n";
    }
    else {
      print $n1 . "{\"title\":\"$srv\",\"folder\":\"true\",\"children\":[\n";
    }
    $n1 = "\n,";
    my $n2 = "";
    foreach my $subsys ( sort keys %{ $alerts{$srv} } ) {
      foreach my $name ( sort keys %{ $alerts{$srv}{$subsys} } ) {
        my $type = "";
        if ( $subsys eq "LPAR" ) {
          if ( $pack{fleet}{$srv}{subsys}{$subsys}{$name} ) {
            $type = $pack{fleet}{$srv}{subsys}{$subsys}{$name}{type};
          }
          else {
            $type = findObjType( undef, $subsys, $name );
          }
        } ## end if ( $subsys eq "LPAR")
        if ( $subsys eq "LPAR" && !$name ) {
          print $n2 . "{\"title\":\"--- ALL LPARS ---\",\"subsys\":\"$subsys\",\"folder\":\"true\",\"children\":[\n";
          $type = "L";
        }
        else {
          print $n2 . "{\"title\":" . JSON->new->allow_nonref->encode( $name ) . ",\"subsys\":\"$subsys\",\"folder\":\"true\",\"children\":[\n";
        }
        $n2 = "\n,";
        my $n3      = "";
        my $ruleidx = 1;
        foreach my $metric ( %{ $alerts{$srv}{$subsys}{$name} } ) {
          foreach my $vals ( @{ $alerts{$srv}{$subsys}{$name}{$metric} } ) {
            my $percent = "false";
            if ( $metric eq "OSCPU" ) {
              $percent = "true";
            }
            if ( $metric eq "CPU" && $vals->{limit} =~ /%/ ) {
              $percent = "true";
              $vals->{limit} =~ s/%//;
            }

            # print $vals->{limit};
            # $my ( $limit, $peak, $repeat, $exclude, $mailgrp ) = (%{\$val{limit}, \$val{limit}, \$val{limit}, \$val{limit}, \$val{limit});
            #print $n3 . "{\"title\":\"Rule #$ruleidx\",\"metrics\":\"$metric\",\"limit\":\"$vals->{limit}\",\"peak\":\"$vals->{peak}\",\"repeat\":\"$vals->{repeat}\",\"exclude\":\"$vals->{exclude}\",\"mailgrp\":\"$vals->{mailgrp}\"}";
            my $fake;
            my $cluster;
            if ( $vals->{uuid} ) {
              if ( $type eq "M" ) {
                $fake = "Linux";
                $cluster = $srv;
              }
              elsif ( $type eq "S" ) {
                $fake = "Solaris";
              }
            }
            no warnings 'uninitialized';
            print $n3 . "{\"title\":\"\",\"metric\":\"$metric\",\"limit\":\"$vals->{limit}\",\"percent\":$percent,\"peak\":\"$vals->{peak}\",\"repeat\":\"$vals->{repeat}\",\"exclude\":\"$vals->{exclude}\",\"mailgrp\":\"$vals->{mailgrp}\",\"type\":\"$type\",\"uuid\":\"$vals->{uuid}\",\"fakeserver\":\"$fake\",\"cluster\":\"$cluster\"}";
            $n3 = "\n,";
            $ruleidx++;
          } ## end foreach my $vals ( @{ $alerts...})
        } ## end foreach my $metric ( %{ $alerts...})
        print "]}\n";
      } ## end foreach my $name ( sort keys...)
    } ## end foreach my $subsys ( sort keys...)
    print "]}\n";
  } ## end foreach my $srv ( sort keys...)
  print "]\n";
} ## end sub genAlertTree

sub genAlertGroupTree {
  print "[";
  my $n1 = "";

  foreach my $grp ( Alerting::getDefinedGroups() ) {
    print $n1 . "{\"title\":\"$grp\",\"folder\":\"true\",\"children\":[\n";
    $n1 = "\n,";
    my $n2 = "";

    # my @members = Alerting::getGroupMembers($grp);
    # print Dumper @members;
    foreach ( @{ Alerting::getGroupMembers($grp) } ) {

      # my $user = Alerting::getUserDetails($grp, $_);
      print $n2 . "{\"title\":\"$_\"}";
      $n2 = "\n,";
    } ## end foreach ( @{ Alerting::getGroupMembers...})
    print "]}\n";
  } ## end foreach my $grp ( Alerting::getDefinedGroups...)
  print "]\n";
} ## end sub genAlertGroupTree

sub getAlertConfig {
  print "[";
  my $n1  = "";
  my %cfg = Alerting::getConfig();

  # print Dumper %cfg;
  while ( my ( $key, $val ) = each %cfg ) {
    print $n1 . "{\"$key\":\"$val\"}";
    $n1 = "\n,";
  }
  print "]\n";
} ## end sub getAlertConfig

sub getMetrics {
  my $cfgdir = "$basedir/etc";
  my $rawcfg;
  if ( !open( CFG, "$cfgdir/metrics.json" ) ) {
    die "Cannot open file: $!\n";
  } else {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
  close CFG;
  print $rawcfg;
  }
}

sub about {

  my %about;

  $about{RRDp_version} = 'n/a';
  $about{RRDs_version} = 'n/a';
  $about{rrdtool_version} = 'n/a';
  $about{perl_version} = 'n/a';
  $about{os_info} = 'n/a';
  $about{tool_version} = "$version";
  $about{edition} = "n/a";

  eval {
    use RRDp;
    $about{RRDp_version} = "$RRDp::VERSION";
  };
  #eval {
    #use RRDs;
    #$about{RRDs_version} = "$RRDs::VERSION";
  #};


  $_ = `rrdtool`;

  if ( /^RRDtool ([1-9]*\.[0-9]*(\.[0-9]*)?)/ ) {
    $about{rrdtool_version} = $1;
  }

  my @perlv = `perl -v`;
  # warn @perlv;

  if ( @perlv ) {
    foreach ( @perlv ) {
      chomp;
      if ( /^This is perl/ ) {
        $about{perl_version} = $_;
      }
    }
  }

  $_ = `uname`;
  chomp;
  $about{os_info} = $_;
  $about{edition} = "free";
  if ( -f "$basedir/bin/premium.sh" ) {
    $about{edition} = "Enterprise";
    my @technology;
    if ( -f "$basedir/html/.p" ) {
      push @technology, "IBM Power Systems";
    }
    if ( -f "$basedir/html/.v" ) {
      push @technology, "VMware";
    }
    if ( -f "$basedir/html/.s" ) {
      push @technology, "Solaris";
    }
    if ( -f "$basedir/html/.x" ) {
      push @technology, "XenServer";
    }
    if ( -f "$basedir/html/.o" ) {
      push @technology, "oVirt/RHV";
    }
    if ( -f "$basedir/html/.h" ) {
      push @technology, "HyperV";
    }
    if ( @technology ) {
      $about{edition} .= " (" . join(", ", @technology) . ")";
    }
  }

  # my %about = (rrdp_version => "$rrdp_version", rrds_version => "$rrds_version", rrdtool_version => "$rrdtool_version");
  print JSON->new->utf8(0)->encode( \%about );
}

sub txthref {
  my $hash = substr( md5_hex( $_[0] . $_[1] ), 0, 7 );
  return "{\"title\":\"$_[0]\",\"href\":\"$_[1]\",\"hash\":\"$hash\"}";
}

sub fullNode {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $parent ) = @_;
  $href ||= "";
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  my $srvType = "";
  if ($type) {
    $srvType = ( $type eq "P" ? "power" : "vmware" );
  }
  my $parentKey = "";
  if ($parent) {
    $parentKey = ",\"parent\":\"$parent\"";
  }
  if ( !$altname ) {
    my $hashstr = $hmc . $srv . $title;
    my $hash = substr( md5_hex($hashstr), 0, 7 );
    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"str\":\"$key\",\"type\":\"$srvType\"$parentKey,\"hashstr\":\"$hashstr\"}";
  }
  else {
    my $hashstr = $hmc . $srv . $altname;
    my $hash = substr( md5_hex($hashstr), 0, 7 );
    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"str\":\"$key\",\"type\":\"$srvType\"$parentKey,\"hashstr\":\"$hashstr\"}";
  }
} ## end sub fullNode

sub fullNode_wchild {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $parent ) = @_;
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  my $srvType;
  if ($type) {
    $srvType = ( $type eq "P" ? "power" : "vmware" );
  }
  my $parentKey = "";
  if ($parent) {
    $parentKey = ",\"parent\":\"$parent\"";
  }

  if ( !$altname ) {
    my $hash = substr( md5_hex( $hmc . $srv . $title ), 0, 7 );
    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"str\":\"$key\",\"type\":\"$srvType\"$parentKey,\"children\":[";
  }
  else {
    my $hash = substr( md5_hex( $hmc . $srv . $altname ), 0, 7 );
    return "{\"title\":\"$title\",\"href\":\"$href\",\"hash\":\"$hash\",\"hmc\":\"$hmc\",\"srv\":\"$srv\",\"altname\":\"$altname\",\"str\":\"$key\",\"type\":\"$srvType\"$parentKey,\"children\":[";
  }
} ## end sub fullNode_wchild

sub pushFullNode {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $parent, $obj ) = @_;
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  my $srvType = "";
  if ($type) {
    $srvType = ( $type eq "P" ? "power" : "vmware" );
  }

  my $hashstr = $hmc . $srv . $title;
  if ($altname) {
    $hashstr = $hmc . $srv . $altname;
  }

  my $hash = substr( md5_hex($hashstr), 0, 7 );
  # $hashstr = undef;
  #my $ret = { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj, hashstr => $hashstr };
  my $ret = { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj };
  while ( my ( $key, $val ) = each %{$ret} ) {
    delete $ret->{$key} if ( not defined $val );
  }
  return $ret;
} ## end sub pushFullNode

sub pushFullNodewChild {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $parent, $obj ) = @_;
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  my $srvType = "";
  if ($type) {
    $srvType = ( $type eq "P" ? "power" : "vmware" );
  }

  my $hashstr = $hmc . $srv . $title;
  if ($altname) {
    $hashstr = $hmc . $srv . $altname;
  }

  my $hash = substr( md5_hex($hashstr), 0, 7 );
  $hashstr = undef;
  #my $ret = { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj, hashstr => $hashstr, children => [] };
  my $ret = { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj, children => [] };
  while ( my ( $key, $val ) = each %{$ret} ) {
    delete $ret->{$key} if ( not defined $val );
  }
  return $ret;
} ## end sub pushFullNodewChild

sub pushFullNodewChil {
  my ( $title, $href, $hmc, $srv, $islpar, $altname, $type, $parent, $obj ) = @_;
  my $key = ( $srv eq "na" ? "" : $srv ) . " " . $title;
  my $srvType = "";
  if ($type) {
    $srvType = ( $type eq "P" ? "power" : "vmware" );
  }
  if ( !$altname ) {
    my $hashstr = $hmc . $srv . $title;
    my $hash = substr( md5_hex($hashstr), 0, 7 );
    if ($parent) {

      #return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, str => $key, type => $srvType, parent => $parent, obj => $obj, hashstr => $hashstr, children => [] };
      return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, str => $key, type => $srvType, parent => $parent, obj => $obj, children => [] };
    }
    else {
      #return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, str => $key, type => $srvType, obj => $obj, hashstr => $hashstr, children => [] };
      return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, str => $key, type => $srvType, obj => $obj, children => [] };
    }
  } ## end if ( !$altname )
  else {
    my $hashstr = $hmc . $srv . $altname;
    my $hash = substr( md5_hex($hashstr), 0, 7 );
    if ($parent) {

      #return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj, hashstr => $hashstr, children => [] };
      return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, parent => $parent, obj => $obj, children => [] };
    }
    else {
      #return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, obj => $obj, hashstr => $hashstr, children => [] };
      return { title => $title, href => $href, hash => $hash, hmc => $hmc, srv => $srv, altname => $altname, str => $key, type => $srvType, obj => $obj, children => [] };
    }
  } ## end else [ if ( !$altname ) ]
} ## end sub pushFullNodewChil

sub txthrefbold {
  my $hash = substr( md5_hex( $_[0] ), 0, 7 );
  return "{\"title\":\"$_[0]\",\"extraClasses\":\"boldmenu\",\"href\":\"$_[1]\",\"hash\":\"$hash\"}";
}

sub pushmenub {
  my ( $txt, $url, $id ) = @_;
  if ( !$id ) {
    $id = $txt;
  }
  my $hash = substr( md5_hex($id), 0, 7 );
  return { "title" => $txt, "extraClasses" => "boldmenu", "href" => $url, "hash" => $hash };
} ## end sub pushmenub

sub pushmenu {
  my ( $txt, $url, $id ) = @_;
  if ( !$id ) {
    $id = $txt;
  }
  my $hash = substr( md5_hex($id), 0, 7 );
  return { "title" => $txt, "href" => $url, "hash" => $hash };
} ## end sub pushmenu

sub txthref_wchld {
  return "{\"title\":\"$_[0]\",\"href\":\"$_[1]\",\"children\":[";
}

sub txtkey {
  return "{\"title\":\"$_[0]\",\"key\":\"$_[1]\"}";
}

sub txtkeysel {
  return "{\"title\":\"$_[0]\",\"selected\":true,\"key\":\"$_[1]\"}";
}

sub collons {
  return s/===double-col===/:/g;
}

sub print_all_models {
  if ( -f "$inputdir/etc/rperf_table.txt" ) {
    open( FALL, "< $inputdir/etc/rperf_table.txt" )
      || error( "$inputdir/etc/rperf_table.txt $! :" . __FILE__ . ":" . __LINE__ ) && return 1;
  }
  else {
    open( FALL, "< $inputdir/etc/free_rperf_table.txt" )
      || error( "$inputdir/etc/free_rperf_table.txt $! :" . __FILE__ . ":" . __LINE__ ) && return 1;
  }
  my @lines = <FALL>;

  close(FALL);

  # remove spaces to be able correctly sort per CPU type
  my $i = 0;
  foreach my $line (@lines) {
    $lines[$i] =~ s/ //g;
    $lines[$i] =~ s/	//g;
    $i++;
  }
  no warnings 'uninitialized';
  my @lines_sort = sort { ( split ':', $a )[1] cmp( split ':', $b )[1] } @lines;

  #  print Dumper @lines_sort;

  # sort
  # 8284-22A : S822 :  P8/6		:  3.89		:  120.8		:
  # P8, P7+, P7, P6+, P6, P5+, P5

  print "[";

  # print_new_server ("P8\\+/",\@lines_sort, 0, 0);
  print_new_server( "P9/",    \@lines_sort, 1, 0 );
  print_new_server( "P8/",    \@lines_sort, 0, 0 );
  print_new_server( "P7\\+/", \@lines_sort, 0, 0 );
  print_new_server( "P7/",    \@lines_sort, 0, 0 );
  print_new_server( "P6\\+/", \@lines_sort, 0, 0 );
  print_new_server( "P6/",    \@lines_sort, 0, 0 );
  print_new_server( "P5\\+/", \@lines_sort, 0, 0 );
  print_new_server( "P5/",    \@lines_sort, 0, 1 );
  print "]";

  return 0;
} ## end sub print_all_models

sub print_new_server {
  my ( $cpu_filter, $lines_sort_tmp, $expanded, $last ) = @_;
  my @lines_sort = @{$lines_sort_tmp};
  my ($class) = split( /\//, $cpu_filter );
  $class =~ s/\\//;
  $class = substr( $class, 1 );
  my $expandedstr = ( $expanded ? '"expanded":true,' : "" );
  if ( exists $lines_sort[0] ) {
    print "{\"title\":\"Power$class\",\"folder\":\"true\",$expandedstr\"children\":[\n";
    my $delim = "";
    foreach my $line (@lines_sort) {
      chomp($line);
      if ( $line =~ m/^#/ || $line =~ m/^$/ ) {
        next;
      }
      if ( $line !~ m/:/ ) {
        next;
      }

      my $rperf = 0;
      my $cpw   = 0;
      ( my $model, my $type, my $cpu, my $ghz, $rperf, my $rperf_st, my $rperf_smt2, my $rperf_stm4, my $rperf_smt8, $cpw, my $fix ) = split( /:/, $line );

      if ( $cpu !~ m/^$cpu_filter/ ) {
        next;
      }
      no warnings "uninitialized";

      # It must have defined status
      if ( $rperf eq '' ) {
        $rperf = 0;
      }
      if ( $cpw eq '' ) {
        $cpw = 0;
      }

      # add spaces
      my $model_space = $model;
      for ( my $k = length($model_space); $k < 9; $k++ ) {
        $model_space .= "&nbsp;";
      }
      my $type_space = $type;
      for ( my $k = length($type_space); $k < 7; $k++ ) {
        $type_space .= "&nbsp;";
      }
      my $cpu_space = $cpu;
      for ( my $k = length($cpu_space); $k < 6; $k++ ) {
        $cpu_space .= "&nbsp;";
      }

      my $value = "$model|$type|$cpu|$ghz|$rperf|$rperf_st|$rperf_smt2|$rperf_stm4|$rperf_smt8|$cpw|$fix";

      print "$delim\{\"title\":\"$model\",\"type\":\"$type\",\"cpu\":\"$cpu\",\"ghz\":\"$ghz\",\"fix\":\"$fix\",\"key\":\"$value\"}";
      $delim = ",\n";
    } ## end foreach my $line (@lines_sort)
    if ($last) {
      print "]}\n";
    }
    else {
      print "]},\n";
    }
  } ## end if ( exists $lines_sort...)

  return 0;
} ## end sub print_new_server

sub uniq {
  my %seen;
  return grep { !$seen{$_}++ } @_;
}

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub boolean {
  my $val = shift;
  if ($val) {
    return "true";
  }
  else {
    return "false";
  }
} ## end sub boolean

sub findObjType {
  my ( $srv, $subsys, $name ) = @_;
  if ($srv) {
    foreach my $obj ( @{ $pack{fleet}{$srv}{subsys}{$subsys} } ) {
      if ( $obj->{k} eq $name ) {
        return $obj->{t};
      }
    }
  } ## end if ($srv)
  else {
    foreach $srv ( keys %{ $pack{fleet} } ) {
      if ( $pack{fleet}{$srv}{subsys}{$subsys}{$name} ) {

        # print "$name\n";
        return $pack{fleet}{$srv}{subsys}{$subsys}{$name}{type};
      }
    } ## end foreach $srv ( keys %{ $pack...})
  } ## end else [ if ($srv) ]
  return "";
} ## end sub findObjType

sub file_read {
  my $file = shift;
  if ( !open IO, $file ) {
    print STDERR "Cannot open $file for input: $!\n";
    exit;
  }
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read
