#!/usr/bin/perl

use strict;
use warnings;
use CGI::Carp qw(fatalsToBrowser);
use MIME::Base64 qw(encode_base64 decode_base64);
use Data::Dumper;
use File::Copy;
use File::Temp qw/ tempfile/;
use JSON;
use File::Basename;
use HostCfg;
use ACL;

my $useacl        = ACL::useACL;
my $aclAdminGroup = ACL::getAdminGroup;
my $basedir  = $ENV{INPUTDIR};
$basedir   ||= "..";
my $bindir   = "$basedir/bin";
my $perl     = $ENV{PERL};
my $ssh_web_ident = $ENV{SSH_WEB_IDENT};
my $webdir   = $ENV{WEBDIR};
my $vmwlibdir = "$basedir/vmware-lib/apps";
my $credstore = "$basedir/.vmware/credstore/vicredentials.xml";

my %platforms = (
  power  => {longname => "IBM Power Systems", pid => 'load_hmc_rest_api.sh.pid', croncmd => "# IBM Power Systems - REST API\n0,20,40 * * * * $basedir/load_hmc_rest_api.sh > $basedir/load_hmc_rest_api.out 2>&1"},
  vmware => {longname => "VMware", pid => 'load_vmware.sh.pid'},
  xen    => {longname => "XenServer", pid => 'load_xenserver.sh.pid', croncmd => "# XEN Server support\n0,20,40 * * * *  $basedir/load_xenserver.sh > $basedir/load_xenserver.out 2>&1"},
  hyperv => {longname => "Hyper-V", pid => 'load_hyperv.sh.pid'},
  kvm    => {longname => "KVM"},
  ovirt  => {longname => "RHV (oVirt)", pid => 'load_ovirt.sh.pid', croncmd => "# oVirt / RHV Server support\n0,20,40 * * * *  $basedir/load_ovirt.sh > $basedir/load_ovirt.out 2>&1"},
  custom => {longname => "Custom Group"},
  common => {longname => "load.sh", pid => 'load.sh.pid', croncmd => "0 * * * *  $basedir/load.sh > $basedir/load.out 2>&1"},
);

# print STDERR "$useacl  $aclAdminGroup\n";

my $json = JSON->new->utf8->pretty;

sub file_write {
  my $file = shift;
  open IO, ">$file" or die "Cannot open $file for output: $!\n";
  print IO @_;
  close IO;
} ## end sub file_write

sub file_write_append {
  my $file = shift;
  if ( -f $file ) {
    open IO, ">>$file" or die "Cannot open $file for output: $!\n";
  }
  else {
    open IO, ">$file" or die "Cannot open $file for output: $!\n";
  }
  print IO @_;
  close IO;
} ## end sub file_write_append

sub file_read {
  my $file = shift;
  open IO, $file or die "Cannot open $file for input: $!\n";
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read

sub trim {
  my $s = shift;
  if ($s) {
    $s =~ s/^\s+|\s+$//g;
  }
  return $s;
} ## end sub trim

# get URL parameters (could be GET or POST) and put them into hash %PAR
my ( $buffer, @pairs, $pair, $name, $value, %PAR );

if ( !defined $ENV{'REQUEST_METHOD'} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";

  #{
  #local $/ = undef;   # required for re-read of encode_json pretty output
  #my $cfg = decode_json(file_read("users.json"));
  #print Dumper $cfg;
  #}
  my %hosts = %{ HostCfg::getHostConnections("IBM Power Systems") };
  print Dumper \%hosts;
  exit;
}

if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
  read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
}
else {
  $buffer = $ENV{'QUERY_STRING'};
}

# Split information into name/value pairs
@pairs = split( /&/, $buffer );
foreach $pair (@pairs) {
  ( $name, $value ) = split( /=/, $pair );
  $value =~ tr/+/ /;
  $value =~ s/%(..)/pack("C", hex($1))/eg;
  if ( exists $PAR{$name} ) {
    if ( ref $PAR{$name} eq "ARRAY" ) {
      push @{ $PAR{$name} }, $value;
    }
    else {
      my $firstVal = $PAR{$name};
      $PAR{$name} = [];
      push @{ $PAR{$name} }, $firstVal;
      push @{ $PAR{$name} }, $value;
    } ## end else [ if ( ref $PAR{$name} eq...
  }
  else {
    $PAR{$name} = $value;
  }
} ## end foreach $pair (@pairs)

# print STDERR Dumper \%PAR;

if ( !defined $PAR{cmd} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";
  exit;
}

if ( $PAR{cmd} eq "test" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";

  my $repname = $PAR{repname};
  my $username = $PAR{user};

  # print Dumper \%ENV;
  # print Dumper \%PAR;
  # print "{ \"status\" : \"success\", \"msg\" : \"<div>You sent request to send report <b>$repname</b><div>\" }";

  my $reportsdir   = "$basedir/reports";

  if ( ! -d $reportsdir ) {
    umask 0000;
    mkdir( "$reportsdir", 0777 ) || print STDERR ( "Cannot mkdir $reportsdir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
  }
  # $ENV{'QUERY_STRING'} = "rep=" . encode_base64url($repname);
  # print STDERR "BASE: $basedir        REP: $repname\n";

  my $olog = "$basedir/reports/reporter-GUI.output.log";
  my $elog = "$basedir/reports/reporter-GUI.error.log";

  my ($oo, $oof) = tempfile( UNLINK => 1 );
  my ($ee, $eef) = tempfile( UNLINK => 1 );

  # print $ft $png;
  # close $ft;

  my $output = `$perl $bindir/reporter.pl "$repname" "$username" 1>>$oof 2>>$eef`;
  `cat $oof >> $olog`;
  `cat $eef >> $elog`;


  # do {
    # my $output = `$perl $bindir/reporter.pl "$repname" 1>>$basedir/reports/reporter-GUI.output.log 2>>$basedir/reports/reporter-GUI.error.log`;
    # local *STDOUT;
    # open STDOUT, ">>", \$oo;
    # local *STDERR;
    # open STDERR, ">>", \$ee;
    # local @ARGV = ($repname);
    # system($^X, "$bindir/reporter.pl", "$repname");
    # eval (require "$bindir/reporter.pl");
    # close STDOUT;
    #warn "OO";
    #warn "EE";
    # close STDERR;
    # do $bindir . '/reporter.pl';
    # `$perl $bindir/reporter.pl "$repname"`;
    # };
  #### pid
  my $ees;
  my $oos;
  {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $ees = <$ee>;
    $oos = <$oo>;
  }
  close $oo;
  close $ee;
  # warn "OO: $oo";
  # warn "EE: $ee";

  my $stat = "true";
  if ($ees) {
    $stat = "false";
  }
  $ees =~ s/\n/\\n/g;
  $ees =~ s/\\:/\\\\:/g;
  $ees =~ s/"/\\"/g;
  my $dnld = my $filename = my $stored = "";
  my $emails = 'false';

  foreach my $line (split /^/, $oos) {
    if ($line =~ /^download/) {
      $dnld = (split " : ", $line)[1];
      $dnld = trim($dnld);
      $filename = fileparse($dnld);
    }
    elsif ($line =~ /^send email to/) {
      $emails = 'true';
    }
    elsif ($line =~ /^stored report/) {
      $stored = (split " : ", $line)[1];
      $stored = trim($stored);
    }
  }
  $oos =~ s/\n/\\n/g;
  $oos =~ s/\\:/\\\\:/g;
  $oos =~ s/"/\\"/g;
  # my $retstr = "\"msg\" : \"<div>Reporter configuration has been succesfuly saved!<br /><br /></div>\", \"debug\" : \"";
  print "{ \"status\" : $stat, \"log\" : \"$oos\", \"error\" : \"$ees\", \"dnld\": \"$dnld\", \"filename\": \"$filename\", \"emails\": $emails, \"stored\": \"$stored\" }";

}
elsif ( $PAR{cmd} eq "modtest" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";
  my $errors  = "";
  my @reqmods = qw(IO::Compress::Zip);

  for my $mod (@reqmods) {
    eval {
      ( my $file = $mod ) =~ s|::|/|g;
      require $file . '.pm';
      $mod->import();
      1;
    } or do {
      $errors .= "$@";
      }
  } ## end for my $mod (@reqmods)

  if ($errors) {
    result( 0, "ZIP module test", $errors );
  }
  else {
    result( 1, "ZIP module test" );

  } ## end else [ if ($errors) ]

} ## end elsif ( $PAR{cmd} eq "modtest")
elsif ( $PAR{cmd} eq "json" ) {    ### Get list of credentials
  my ($cfg, $imported) = HostCfg::getRawConfig($aclAdminGroup);
  #warn $imported;
  if ( $imported ) {
    print "X-Hosts-Imported: true\n";
  }
  print "Content-type: application/json\n\n";
  print $cfg;

}
elsif ( $PAR{cmd} eq "form" ) {

  my $platform = $PAR{platform};
  my $user = "admin";
  if ($ENV{REMOTE_USER}) {
    $user = $ENV{REMOTE_USER}
  }
  print "Content-type: text/html\n\n";

  if ( $useacl && !ACL::isAdmin ) {
      print "<div>You have to be a member of Administrators group to change hosts configuration!</div>";
      exit;
  }
  my @errors;

  if ( $platform eq "ovirt" ) {
    my @reqmods = qw(DBI DBD::Pg);
    for my $mod (@reqmods) {
      eval {
        ( my $file = $mod ) =~ s|::|/|g;
        require $file . '.pm';
        $mod->import();
        1;
      } or do {
        push @errors, "$@";
      }
    }
  }

  if ( $platform eq "vmware" && !-e "$vmwlibdir/credstore_admin.pl" ) {
      print "<p><b>VMware Perl SDK</b> libraries and apps are not installed yet. We are not authorized to redistribute the <b>VMWare Perl SDK</b> due to licensing restrictions.<p>";
      if ( defined $ENV{VI_IMAGE} && $ENV{VI_IMAGE} ) {
        print <<_MARKER_;
    <p>Download <a href='https://code.vmware.com/sdk/vsphere-perl' target="blank"><b>vSphere Perl SDK</b></a> from VMware website to your computer and then upload it to the running appliance via following form:<br>
    It requires free registration at VMware.<br>
    Does not matter if 32bit or 64bit package<br>
    VMware-vSphere-Perl-SDK-&lt;version&gt;.tar.gz, download one of *tar.gz</p>
    <form id="sdk-install" action="/lpar2rrd-cgi/vmwcfg.sh" method="post" enctype="multipart/form-data">
    <p>File to Upload: <input type="file" accept=".tar.gz" name="sdk"></p>
    <p><input type="submit" id="sdk-upload" name="Submit" value="Upload SDK file" /></p>
    <div class="progress">
      <div class="bar"></div >
      <div class="percent">0%</div >
    </div>
    </form>
  <div id="status"></div>
_MARKER_
    } ## end if ( $ENV{VM_IMAGE} )
    else {
      print "<p>Try manuall installation instead: <a href='http://www.lpar2rrd.com/VMware-performance-monitoring-installation.htm'>http://www.lpar2rrd.com/VMware-performance-monitoring-installation.htm</a> --> VMware tab.</p>";
    }
    exit;
  }
  elsif ( $platform eq "ovirt" && @errors ) {
		print "<p style='font-style: italic'><span style='color: red'>Warning:</span> You cannot use Postgres database, your host system cannot find required Perl modules:<p>" . join("</p><p>", @errors) . "</p><p>Install modules as per this <b><a href='http://www.lpar2rrd.com/oVirt-Postgres-support.htm' target='_blank'>document</a></b>.</p>";
  }
  else {
    # TABs
    print <<_MARKER_;
<div style='text-align: center'>
  <div style='display: inline-block'>
    <div style='float: left; margin-right: 10px; outline: none' class="cggrpnames">
        <table id="hosttable" class="cfgtree" data-platform="$platform">
          <thead>
          <tr>
            <th>Alias <button id="addnewhost">New</button></th>
            <th>Hostname / IP</th>
            <th>User name</th>
            <th>Connection Test</th>
            <th>Delete</th>
          </tr>
          </thead>
          <tbody>
          </tbody>
        </table>
      </div>
      <br style="clear: both">
      <div id="cfgcomment" style="max-width: 80em; font-size: smaller; text-align: left;">
_MARKER_
    print cronTest($platform);
    print cronTest("common");

    if ( -e "$webdir/hostcfg-$platform.html" && -f _ && -r _ ) {
      print file_read("$webdir/hostcfg-$platform.html");
    }
    print <<_MARKER_;
      </div>
    <br style="clear: both">
  </div>
</div>

_MARKER_
  }

}
elsif ( $PAR{cmd} eq "saveall" ) {    ### Save configuration as is
  my $basedir = $ENV{INPUTDIR};
  $basedir ||= "..";

  my $cfgdir = "$basedir/etc/web_config";

  my $tmpdir = "$basedir/tmp";
  if ( defined $ENV{TMPDIR_LPAR} ) {
    $tmpdir = $ENV{TMPDIR_LPAR};
  }
  # print "Content-type: application/json\n\n";
  print "Content-type: text/html\n\n";
  if ( $PAR{acl} ) {
    my $retstr  = "";
    my %oldcfg  = HostCfg::getConfig();

    # if ( $ENV{'SERVER_NAME'} eq "demo.stor2rrd.com" ) {
    if ( $ENV{'DEMO'} ) {
      $retstr = "\"msg\" : \"<div>This demo site does not allow saving any changes you do in the admin GUI panel.</div>\"";
      $retstr .= ", \"cfg\" : \"\"";
      print "{ \"status\" : \"fail\", $retstr }";

    }
    # disable for now...
    if ( $useacl && !ACL::isAdmin ) {
      print "{ \"status\" : \"fail\", \"msg\" : \"<div>You have to be a member of Administrators group to change hosts configuration!</div>\" }";
    #
    }
    elsif ( open( CFG, ">$cfgdir/hosts.json" ) ) {
      my $cfg     = $PAR{acl};
      my %new_cfg = %{ decode_json($cfg) };
      # print STDERR Dumper \%new_usr_cfg;
      #if ( $PAR{user} && $new_usr_cfg{users}{$PAR{user}} ) {
      #  $oldcfg{users}{ $PAR{user} } = $new_usr_cfg{users}{$PAR{user}};
      #}
      print CFG $json->encode( \%new_cfg );
      close CFG;
      $cfg =~ s/\n/\\n/g;
      $cfg =~ s/\\:/\\\\:/g;
      $cfg =~ s/"/\\"/g;
      $retstr = "\"msg\" : \"<div>Hosts configuration has been succesfuly saved!<br /><br /></div>\", \"cfg\" : \"";
      print "{ \"status\" : \"success\", $retstr" . "$cfg\" }";
    }
    else {
      print "{ \"status\" : \"fail\", \"msg\" : \"<div>File $cfgdir/hosts.json cannot be written by webserver, check apache user permissions: <span style='color: red'>$!</span></div>\" }";
    }
  }
  else {
    print "{ \"status\" : \"fail\", \"msg\" : \"<div>No data was written to hosts.json</div>\" }";
  }

}
elsif ( $PAR{cmd} eq "grptree" ) {
  my %cfg = ReporterCfg::getConfig();
  print "Content-type: application/json\n\n";
  print "[";
  my $n1 = "";

  # print Dumper \%cfg;
  foreach my $grp ( sort keys %{ $cfg{groups} } ) {
    if ( $grp eq "$aclAdminGroup" ) {
      next;
    }
    print $n1 . "{\"title\":\"$grp\"}\n";
    $n1 = "\n,";
  }
  print "]\n";
}

elsif ( $PAR{cmd} eq "sshkeys" ) {    ### Get SSH keys
  print "Content-type: application/json\n\n";
  my @keys = HostCfg::getSSHKeys();
  print $json->encode( \@keys );
}
elsif ( $PAR{cmd} eq "mailtest" ) {
  print "Content-type: application/json\n\n";
  my $errors  = "";
  my %cfg = ReporterCfg::getConfig($aclAdminGroup);
  my $user = "admin";
  if ($ENV{REMOTE_USER}) {
    $user = $ENV{REMOTE_USER}
  }

  my $group = $PAR{group};

  if (! $cfg{users}{$user}{groups}{$group}) {
    $errors = "No such group.";
  } elsif (! $cfg{users}{$user}{groups}{$group}{emails} ) {
    $errors = "No emails defined.";
  } else {
    for my $mail ( @{ $cfg{users}{$user}{groups}{$group}{emails} } ) {

      my $from = 'reporter@yourdomain.com';
      if ( $cfg{users}{$user}{groups}{$group}{mailfrom} ) {
        $from = $cfg{users}{$user}{groups}{$group}{mailfrom};
      }
      my $subject = 'Test E-mail';
      my $message = 'This is test email sent by Reporter';

      my $mailprog = "/usr/sbin/sendmail";
      if ( ! -f $mailprog ) {
        print STDERR "Alerting: emailing program missing \"$mailprog\", install it " . __FILE__ . ":" . __LINE__ . "\n";
      }
      else {

        open(MAIL, "|$mailprog -t");

        # Email Header
        print MAIL "To: $mail\n";
        print MAIL "From: $from\n";
        print MAIL "Subject: $subject\n\n";
        # Email Body
        print MAIL $message;

        close(MAIL);
        # print "Email Sent Successfully\n";
      }
    }
  }
  if ($errors) {
    result( 0, "E-mail test for group $group failed: ", $errors );
  }
  else {
    result( 1, "E-mail test for group $group was completed, please check your mailbox.");

  } ## end else [ if ($errors) ]
}

elsif ( $PAR{cmd} eq "conntest" ) {

  print "Content-type: application/json\n\n";
  my $host = $PAR{host};
  my $port = $PAR{port};
  use IO::Socket::INET;
  my $sock;

  eval {
    local $SIG{ALRM} = sub { die 'Timed Out'; };
    alarm 10;
    $sock = new IO::Socket::INET(
      PeerAddr => $host,
      PeerPort => $port,
      Proto    => 'tcp',
      Timeout  => 3
    );

    alarm 0;
  };
  alarm 0; # race condition protection
  if ( $@ && $@ =~ /Timed Out/ ) {
    result( 0,  "TCP connection to $host:$port timed out after 10 seconds!");
  }
  elsif ( $sock ) {
    result( 1, "TCP connection to $host:$port is <span class='noerr'>OK</span>.");
  }
  else {
    result( 0,  "TCP connection to $host:$port has failed! Open it on the firewall.");
  }
}

elsif ( $PAR{cmd} eq "apitest" ) {

  print "Content-type: application/json\n\n";
  my $platform = $PAR{platform};
  my $host = $PAR{host};
  my $port = $PAR{port};
  my $proto = $PAR{proto};
  my $username = $PAR{username};
  my $password = HostCfg::unobscure_password($PAR{password});

  if ( $platform eq "XenServer" ) {
    # warn "$host $username $password";
    my $output = `$perl $bindir/xen-test-xapi.pl "$host" "$port" "$proto" "$username" "$password"`;
    print $output;
    exit;
  }
  elsif ( $platform eq "IBM Power Systems"){
    my $managednames = `\$PERL $basedir/bin/hmc-restapi-test.pl "$host"`;
    chomp($managednames);
    if (length($managednames) != 0 && $managednames !~ m/No Servers Found/){
      result (1, "API OK" );
      exit;
    }
    else{
      result (0, "User $username authorization failed on $proto://$host:$port.<br>Check username and password<br>Check if $host version is at least HMC v8 otherwise use HMC CLI (SSH)<br>Troubleshooting: <a href=\"www.lpar2rrd.com/IBM-Power-Systems-REST-API-troubleshooting.htm\">www.lpar2rrd.com/IBM-Power-Systems-REST-API-troubleshooting.htm</a>" );
      exit
    }
  }
  elsif ( $platform eq "VMware" ) {
    my $call_err = "$perl $basedir/vmware-lib/apps/connect.pl --credstore '$credstore' --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1";
    my $conn;
    eval {
      local $SIG{ALRM} = sub { die 'Timed Out'; };
      alarm 30;
      $conn = `$perl $basedir/vmware-lib/apps/connect.pl --credstore '$credstore' --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1`;
      alarm 0;
    };
    alarm 0; # race condition protection
    if ( $@ && $@ =~ /Timed Out/ ) {
      result( 0,  "API login to $host timed out after 30 seconds!");
    }
    elsif ( $conn =~ "Connection Successful" ) {
      result( 1, "API authorization to $host is <span class='noerr'>OK</span>.");
    }
    else {
      result( 0,  "API authorization to $host has failed!\n\n$conn");
    }
  }
  elsif ( $platform eq "RHV (oVirt)" ) {
    require DBI;
    my $driver  = "Pg";
    my $database = "ovirt_engine_history";
    my $dsn = "DBI:$driver:dbname = $database; host = $host; port = $port";
    my $dbh = DBI->connect($dsn, $username, $password, { RaiseError => 0 });
    if ( $dbh ) {
      my $stmt = qq(SELECT * FROM calendar LIMIT 5;);
      my $sth = $dbh->prepare( $stmt );
      my $rv = $sth->execute();
      if ( $rv ) {
        if($rv < 0) {
          result(0, $DBI::errstr);
        }
        else {
          result(1, "Connected database succesfuly");
        }
      }
      else {
        result(0, $DBI::errstr);
      }
    }
    else {
      result(0, $DBI::errstr);
    }
  }
  else {
    print "{ \"success\": false, \"error\" : \"Not implemented, WIP\"}";
  }
}

# SSH authentication (key-based)
elsif ( $PAR{cmd} eq "sshtest" ) {
  print "Content-type: application/json\n\n";
  my $platform = $PAR{platform};
  my $host = $PAR{host};
  my $port = $PAR{port};
  my $username = $PAR{username};
  my $sshkey;
  if ( $platform eq "IBM Power Systems" ) {
    result (0, "Do not use basic SSH test for Power.");
    exit;
    $sshkey = $ssh_web_ident;
  } else {
    $sshkey = $PAR{sshkey};
  }

  # test SSH, only if LPAR2RRD is running in the image, thus access to keys is not an issue
  if ( defined $ENV{VI_IMAGE} && $ENV{VI_IMAGE} == 1 ) {
    # warn "$host $username $sshkey";
    my $response = `$bindir/sshtest.sh "$host" "$username" "$sshkey"`;
    print $response;
    exit;
  } else {
    result(0, "Test can be done only manually via ssh command line<br>\n"
           . " su - &lt;LPAR2RRD user&gt; # usualy: lpar2rrd<br>\n"
           . " cd $basedir<br>\n"
           . " ./bin/sshtest.sh $host $username $sshkey");
  }

  exit;
}

# test commands/environment over SSH
elsif ( $PAR{cmd} eq "sshdatatest" ) {

  print "Content-type: application/json\n\n";
  my $platform = $PAR{platform};
  my $host = $PAR{host};
  my $port = $PAR{port};
  my $username = $PAR{username};
  my $sshkey;
  if ( $platform eq "IBM Power Systems" ) {
    $sshkey = $ssh_web_ident;
  } else {
    $sshkey = $PAR{sshkey};
  }

  # setup the script and alternative help message for your platform
  my ($ssh_cmd, $alt_help);
  if ( $platform eq "IBM Power Systems" ) {
    $ssh_cmd = "sh $bindir/sample_rate.sh $host";
    $alt_help = "<br>Test can be done only manually via ssh command line<br>\nsu - &lt;LPAR2RRD user&gt; # usually: su - lpar2rrd<br>cd $basedir<br>./bin/sample_rate.sh $host<br>\n";
    if ( defined $ENV{VI_IMAGE} && $ENV{VI_IMAGE} == 1 ) {
      my $output = `$ssh_cmd`;
      my %out;
      my @lines = split ("\n", $output);

      for (my $i=0; $i<=(scalar (@lines)); $i++){
        if (defined $lines[$i] && $lines[$i] ne "" && $lines[$i] =~ m/OK/){

          my ($first, $time) = split ("=", $lines[$i]);
          my @first_part = split (":", $first);
          my $status; my $hmc_status; my $sample_rate; my $servername; my $hmc_time; my $host; my $server_time;
          if (defined $first_part[3] && $first_part[3] ne ""){ #SERVER
            ($status, $sample_rate, $servername) = ($first_part[0], $first_part[1], $first_part[2]);
            $out{server}{$servername}{lparsutil_time} = $time;
            $out{server}{$servername}{status} = $status;
            $out{server}{$servername}{sample_rate} = $sample_rate;
          }
          else{ #HMC
            ($hmc_status, $host) = ($first_part[0], $first_part[1]);
            $out{hmc_info}{hmc_time} = $time;
            $out{hmc_info}{host} = $host;
            $out{hmc_info}{status} = $status;
          }

        }
      }
      if (!defined $out{hmc_info}{hmc_time}){
        $output =~ s/\n/<br>/g;
        result (0, "Error: $output");
        exit;
      }
      my $html;
      my $i=0;
      $html = "<table id=\"pwrsrvtest\"><tbody>";
      $html .= "<tr><td>HMC Time</td><td id=$i><span>$out{hmc_info}{hmc_time}</span></td></tr>"; $i++;
      foreach my $servername (keys %{$out{server}}){
        $html .= "<tr>
                 <td>$servername</td>
                 <td id=\"td_$i\">
                    <span>$out{server}{$servername}{status} Sample Rate:$out{server}{$servername}{sample_rate}, $out{server}{$servername}{lparsutil_time}</span>
                 </td>
              </tr>";
        $i++;
      }
      $html .= "</tbody></table><br>Assure that HMC_time is more less same as server_time on its servers.<br>Refer to <a target=\"blank\" href=\"http://www.lpar2rrd.com/HMC-CLI-time.htm\">http://www.lpar2rrd.com/HMC-CLI-time.htm</a> when times are different (30mins+)";
      result (1, $html);
      exit;
    }
    else{
      result(0, $alt_help);
      exit;
    }
  } elsif ( $platform eq "XenServer" ) {
    $ssh_cmd  = "$bindir/xen-test-ssh.sh \"$host\" \"$username\" \"$sshkey\"";
    $alt_help = "Test can be done only manually via ssh command line<br>\n"
           . " su - &lt;LPAR2RRD user&gt; # usualy: lpar2rrd<br>\n"
           . " cd $basedir<br>\n"
           . " ./bin/xen-test-ssh.sh $host $username $sshkey";
  } else {
    result(0, "test not supported for $platform");
    exit;
  }

  # test SSH, only if LPAR2RRD is running in the image, thus access to keys is not an issue
  if ( defined $ENV{VI_IMAGE} && $ENV{VI_IMAGE} == 1 ) {
    # warn "$host $username $sshkey";
    my $output = `$ssh_cmd`;
    print $output;
  } else {
    result(0, $alt_help);
  }

  exit;
}

elsif ( $PAR{cmd} eq "vmwaredatatest" ) {

  print "Content-type: application/json\n\n";
  my $platform = $PAR{platform};
  my $host = $PAR{host};
  my $port = $PAR{port};
  my $proto = $PAR{proto};
  my $username = $PAR{username};
  my $password = HostCfg::unobscure_password($PAR{password});

  my $call_err = "$perl $basedir/vmware-lib/apps/connect.pl --credstore '$credstore' --server '$host' --username '$username' --password '$password' --portnumber '$port' --protocol '$proto' 2>&1";
  my $conn = "";
  require VMware::VIRuntime;

  # easy test if at least one VM and one HostSystem presented
  $ENV{VI_SERVER} = "$host";
  $ENV{VI_CREDSTORE} = "$credstore";
  Opts::parse();
  Opts::validate();
  Util::connect();

  my $obj_views = Vim::find_entity_views( view_type => 'VirtualMachine', properties => ['name'] );
  if ( !defined $obj_views || $obj_views eq "" || !defined @$obj_views[0] ) {
    $conn = "not presented any VirtualMachine - is there Read-only permission for all Vmware folders?";
  }
  $obj_views = Vim::find_entity_views( view_type => 'HostSystem', properties => ['name'] );
  if ( !defined $obj_views || $obj_views eq "" || !defined @$obj_views[0] ) {
    $conn = "not presented any HostSystem - is there Read-only permission for all Vmware folders?";
  }
  Util::disconnect();
  if ( $conn =~ "not presented" ) {
    result( 0, "$conn" );
  }
  else {
    result( 1, "$conn" );
  }
}
elsif ( $PAR{cmd} eq "vmwareremovecreds" ) {    ### remove selected credentials
  print "Content-type: application/json\n\n";
  if ( defined $ENV{DEMO} ) {
    &result( 0, "You cannot remove credentials in live demo" );
    exit;
  }
  my $alias    = $PAR{alias};
  my $username = $PAR{username};
  my $server   = $PAR{server};
  if ( $server && $username ) {    # alias is empty, remove credentials too
    # my $call = "PER5LIB=$perl5lib; $perl $vmwlibdir/connect.pl --credstore $credstore --server $server --username $username --verbose";
    my $call = "$perl $vmwlibdir/credstore_admin.pl --credstore '$credstore' remove --server '$server' --username '$username' 2>&1";

    # print $call . "\n";
    my $conn = `$call`;
    if ( $? == -1 ) {
      &result( 0, "command failed: $!" );
    }
    elsif ( $? == 0 ) {
      chomp $conn;
      if ( $conn =~ "successfully" ) {
          &result( 1, "$conn" );
      }
      else {
        &result( 0, "$conn" );
      }
    }
    else {
      my $arg = sprintf( "command exited with value %d", $? >> 8 );
      &result( 0, "$arg\n$conn" );
    }
  } ## end if ( $server && $username)
  else {
    &result( 0, "You have to specify server and username!" );
  }

} ## end elsif ( $PAR{cmd} eq "remove")
elsif ( $PAR{cmd} eq "vmwareaddcreds" ) {    ### add new connection
  print "Content-type: application/json\n\n";
  my $alias    = $PAR{alias};
  my $username = $PAR{username};
  my $server   = $PAR{server};
  my $password = HostCfg::unobscure_password($PAR{password});
  if ( $alias && $server && $username && $password ) {
    $ENV{VMPASS} = $password;
    my $call = "$perl $vmwlibdir/credstore_admin.pl --credstore '$credstore' add --server '$server' --username '$username' --password \$VMPASS 2>&1";
    my $conn = `$call`;
    $ENV{VMPASS} = "";
    if ( -e $credstore && -f _ && -r _ ) {
      # check other user read permissions
      use Fcntl ':mode';
      my $mode = (stat($credstore))[2];
      my $other_read =  $mode & S_IROTH;
      if ( ! $other_read ) {
        # set 644 if not world readable
        chmod 0644, $credstore;
      }
      $mode = sprintf "%04o", S_IMODE($mode);
    }
    if ( $? == -1 ) {
      &result( 0, "command failed: $!" );
    }
    elsif ( $? == 0 ) {
      chomp $conn;
      if ( $conn =~ "successfully" ) {
          &result( 1, "$conn" );
      }
      else {
        &result( 0, "$conn" );
      }
    }
    else {
      my $arg = sprintf( "command exited with value %d", $? >> 8 );
      &result( 0, "$arg $conn" );
    }
  } ## end if ( $alias && $server...)
  else {
    &result( 0, "You have to specify alias, server, username and password!" );
  }
}

elsif ( $PAR{cmd} eq "powerserverlist" ) {
  print "Content-type: application/json\n\n";
  my $hmc = $PAR{hmc};
  my $srvlist = `\$PERL $basedir/bin/hmc-restapi-test.pl $hmc`;
  if ($srvlist =~ m/expired/ || $srvlist =~ m/No Servers Found/){
    result (0, "No servers found. Maybe not supported HMC version (should be at least v8)");
  }
  else{
    print $srvlist;
  }
}

elsif ( $PAR{cmd} eq "powerserversingletest" ) {
  print "Content-type: application/json\n\n";
  my $platform = $PAR{platform};
  my $hmc = $PAR{hmc};
  my $server = $PAR{server};
  my $testresult = `\$PERL $basedir/bin/hmc-restapi-test.pl $hmc $server`;
  if ( $testresult ne "" ) {
    result( 0, $testresult );
  }
  else {
    result( 1 );
  }
}

# return empty string if file not exists || version number
elsif ( $PAR{cmd} eq "gethmcversion" ) {
  print "Content-type: application/json\n\n";
  my $hmcversion;
  my $hmc = $PAR{hmc};
  if ( $hmc ne "" ) {
    my $verfile = "$basedir/tmp/HMC-version-$hmc.txt";
    if ( -e $verfile && -f _ && -r _ ) {
      if ( open( my $fh, '<', $verfile) ) {
        my $row = <$fh>;
        chomp $row;
        $hmcversion = $row;
        close($fh);
      }
    }
  }
  my %res = ( hmcversion => $hmcversion );
  print encode_json( \%res );
}

sub cronTest {
  my $platform = shift;
  my $pidfile = "$basedir/tmp/$platforms{$platform}{pid}";
  if ( -e $pidfile && -f _ && -r _ ) {
    my $age  = (-M $pidfile) * 86400;
    if ($age > 7200) {    # if file modify time > 2 hours
      if ( $platform  eq "common" ) {
        return "<div><p style='font-style: italic'><span style='color: red'>Error:</span> Main cron job (load.sh) not running, please check it out!</p></div>";
      }
      else {
        return "<div><p style='font-style: italic'><span style='color: red'>Error:</span> Cron job not running, please check it out!</p></div>";
      }
    }
    else {
      return "";
    }
  } else {
    if ( $platform  eq "common" ) {
      return "<div><p style='font-style: italic'><span style='color: red'>Error:</span> Main cron job (load.sh) not found, it has to be configured! Add these lines to lpar2rrd user crontab:<br><pre>$platforms{$platform}{croncmd}</pre></div>";
    }
    else {
      if ( $platform eq "power" ) {
        return "<div><p style='font-style: italic'><span style='color: red'>Error:</span> Cron job not found, it has to be configured! <br>Note this is checked once a 20 minutes only, ignore it if you have just fixed it.<br><br>Add these lines to lpar2rrd user crontab (just if you use HMC REST API access):<br><pre>$platforms{$platform}{croncmd}</pre></div>";
      }
      else {
        return "<div><p style='font-style: italic'><span style='color: red'>Error:</span> Cron job not found, it has to be configured! <br>Note this is checked once a 20 minutes only, ignore it if you have just fixed it.<br><br>Add these lines to lpar2rrd user crontab:<br><pre>$platforms{$platform}{croncmd}</pre></div>";
      }
    }
  }
}

sub jsonTimeToHuman {
  my $str = shift;
  $str =~ tr/TZ/ /;
  return $str;
}

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  $s =~ s/\+/ /g;
  return $s;
}

sub encode_base64url {
  my $e = encode_base64(shift, "");
  $e =~ s/=+\z//;
  $e =~ tr[+/][-_];
  return $e;
}

sub decode_base64url {
  my $s = shift;
  $s =~ tr[-_][+/];
  $s .= '=' while length($s) % 4;
  return decode_base64($s);
}

sub result {
  my ( $status, $msg, $log ) = @_;
  $log ||= "";
  # $status = ( $status ) ? "true" : "false";
  my %res = ( success => $status, error => $msg, log => $log );
  print encode_json( \%res );
} ## end sub result
