#!/bin/ksh

#set -x

PATH=$PATH:/usr/bin:/bin
export PATH
umask 0022

if [ ! -d $WEBDIR ]; then
   echo "WEBDIR does not exist, supply it as the first parametr"
   exit 1
fi
if [ ! -d $INPUTDIR ]; then
   echo "Source does not exist, supply it as the second parametr"
   exit 1
fi
if [ ! -f $PERL ]; then
   echo "perl path is invalid, correct it and re-run the tool"
   exit 1
fi

xormon=0
if [ ! "$XORMON"x = "x" ]; then
  xormon=$XORMON
fi

type_lpar="L"
type_lpar_removed="R"
type_wpar="W"

type_amenu="A" # VMWARE CLUSTER menu
type_bmenu="B" # VMWARE RESOURCEPOOL menu
type_cmenu="C" # custom group menu
type_fmenu="F" # favourites menu
type_gmenu="G" # global menu
type_hmenu="H" # HMC menu
type_zmenu="Z" # datastore menu
type_smenu="S" # server menu
type_dmenu="D" # server menu - already deleted (non active) servers
type_tmenu="T" # tail menu
type_vmenu="V" # VMWARE total menu
type_qmenu="Q" # tool version
type_hdt_menu="HDT" # hyperv disk global
type_hdi_menu="HDI" # hyperv disk item


type_version="O" # free(open)/full version (1/0)
type_ent="E" # ENTITLEMENT_LESS for DTE

type_server_power="P" # power
type_server_vmware="V" # vmware
type_server_kvm="K" # KVM
type_server_hyperv="H" # Hyper-V
type_server_hitachi="B" # Hitachi blade
type_server_xenserver="X" # XenServer
type_server_ovirt="O" # oVirt

CGI_DIR="lpar2rrd-cgi"

gmenu_created=0 # global Power  menu is created only once
vmenu_created=0 # global VMWare menu is created only once
smenu_created=0 # super menu is created only once -favorites and customs groups

# L - lpar
# G - global menu
# S - server menu
# T - tail menu
# H - HMC menu

premium=0
if [ -f "$INPUTDIR/bin/premium.sh" ];  then
  premium=1
fi

ALIAS_CFG="$INPUTDIR/etc/alias.cfg"
ALIAS=0
# 10 days in secs`
TEND=864000

if [ -f "$ALIAS_CFG" ]; then
  ALIAS=1
fi
IVM=0
SDMC=0
if [ "$TMPDIR_LPAR"x = "x" ]; then
  TMPDIR_LPAR="$INPUTDIR/tmp"
fi
INDEX=$INPUTDIR/html

version_run="$version-run"
MENU_OUT=""
VMWARE=0
XEN=0
OVIRT=0

if [ ! "$1"x = "x" -a "$1" = "power" ]; then
  MENU_OUT="$TMPDIR_LPAR/menu_power.txt-tmp"
  MENU_OUT_FINAL=$TMPDIR_LPAR/menu_power.txt
fi
if [ ! "$1"x = "x" -a "$1" = "vmware" ]; then
  MENU_OUT="$TMPDIR_LPAR/menu_vmware.txt-tmp"
  MENU_OUT_FINAL=$TMPDIR_LPAR/menu_vmware.txt
  version_run="$version-vmware"
  VMWARE=1
fi
if [ ! "$1"x = "x" -a "$1" = "xenserver" ]; then
  MENU_XENSERVER_JSON_OUT="$TMPDIR_LPAR/menu_xenserver.json"
  version_run="$version-xenserver"
  XEN=1
fi
if [ ! "$1"x = "x" -a "$1" = "ovirt" ]; then
  MENU_OVIRT_JSON_OUT="$TMPDIR_LPAR/menu_ovirt.json"
  version_run="$version-ovirt"
  OVIRT=1
fi

if [ $XEN -eq 1 -o $OVIRT -eq 1 ]; then
  # new JSON menu: avoid dealing with menu.txt (MENU_OUT)
  :
elif [ "$MENU_OUT"x = "x" ]; then
  echo "install-html.sh must be called with param either 'power', 'vmware', 'xenserver' or 'ovirt', exiting !!!"
  exit
fi

rm -f $MENU_OUT

if [ ! -f "$MENU_OUT_FINAL" ]; then
  if [ "$1" = "vmware" -o "$1" = "power" ]; then
    # force to create tmp/menu.txt when it does not exist
    if [ $DEBUG ]; then echo "menu           : $MENU_OUT_FINAL is missing, force it to create a new one"; fi
    touch "$TMPDIR_LPAR/$version_run"
  fi
fi

if [ ! -f "$MENU_XENSERVER_JSON_OUT" ]; then
  if [ "$1" = "xenserver" ]; then
    # force to create tmp/menu_xenserver.json, if it does not exist
    if [ $DEBUG ]; then echo "menu           : menu_$1.json is missing, force it to create a new one"; fi
    touch "$TMPDIR_LPAR/$version_run"
  fi
fi

if [ ! -f "$MENU_OVIRT_JSON_OUT" ]; then
  if [ "$1" = "ovirt" ]; then
    # force to create tmp/menu_ovirt.json, if it does not exist
    if [ $DEBUG ]; then echo "menu           : menu_$1.json is missing, force it to create a new one"; fi
    touch "$TMPDIR_LPAR/$version_run"
  fi
fi


pwd=`pwd`
CONFIG_DIR=$INPUTDIR/data	# must be absolute path
TIME=`$PERL -e '$v = time(); print "$v\n"'`
dashb="class=\"relpos\"><div><div class=\"favs favoff\"></div>"

if [ "$KEEP_VIRTUAL"x = "x" ]; then # accounting for DHL
  KEEP_VIRTUAL=0
fi
if [ "$ENTITLEMENT_LESS"x = "x" ]; then # DTE Energy
  ENTITLEMENT_LESS=0
fi

if [ ! -d "$WEBDIR/favourites" ]; then
  mkdir "$WEBDIR/favourites"
fi
. $INPUTDIR/bin/fav.sh  # read favorites code


if [ ! -d "$WEBDIR/custom" ]; then
  mkdir "$WEBDIR/custom"
fi


if [ $DEBUG ]; then echo "installing WWW : install-html.sh $1"; fi
if [ $DEBUG ]; then echo "Host identif   : $UNAME "; fi
DATE=`date`
if [ $DEBUG ]; then echo "timestamp      : $DATE "; fi

#LPM=1     # "./load.sh html" sets LPM=0
if [ $premium -eq 1 -a $LPM -eq 1 -a "$1" = "power" ]; then
  if [ -f $TMPDIR_LPAR/$version-LPM -o $UPGRADE -eq 1 ]; then
    . $INPUTDIR/bin/premium.sh
    # prepare LPM, graph will be created one run after
    # find and symlink lpar under LPM
    if [ $DEBUG ]; then echo "LPM search     : rrm"; fi
    RR="rrm"
    RR_NEW="rrl"
    #lpm_support

    if [ -f "$INPUTDIR/bin/premium_lpm_find.pl" ]; then
      listing=`$PERL "$INPUTDIR/bin/premium_lpm_find.pl"`
      if [ $? -eq 0 ]; then
        echo "$listing";
        echo "LPM            : premium_lpm_find.pl: OK"
      else
        echo "LPM            : premium_lpm_find.pl failed with value $?"
      fi
    else
      echo "timestamp      : no LPM support "
    fi
    # find and symlink lpar under LPM for old hourly DBs
    # excluded from 4.81-006
    #if [ $DEBUG ]; then echo "LPM search     : rrh"; fi
    #RR="rrh"
    #RR_NEW="rri"
    #lpm_support

    RR=""
    DATE=`date`
    if [ $DEBUG ]; then echo "timestamp LPM  : $DATE "; fi
    rm -f $TMPDIR_LPAR/$version-LPM
  fi
fi

# copy of index pages and others
cp $INPUTDIR/html/not-implemented-yet.html $INPUTDIR/html/nmonfile.html $INPUTDIR/html/noscript.html $INPUTDIR/html/wipecookies.html $INPUTDIR/html/index.html $INPUTDIR/html/dashboard.html $INPUTDIR/html/cpu_workload_estimator.html $WEBDIR/

#
### subroutines
#

find_out_alias ()
{
      #find out alias in etc/alias.cfg
      x_slash=$1
      item_type=$2 # LPAR or VM

      lpar_menu=$x_slash
      if [ $ALIAS -eq 1 ]; then
        i_slash_colon=`echo "$x_slash"|sed 's/:/\\\\\\\\:/g'` # support colon in lpar name (use back slash for prefixing colons in etc/alias.cfg)
        for alias_row in `egrep "^$item_type:$i_slash_colon:" $ALIAS_CFG |sed 's/ /=====space=====/g'|head -1 2>/dev/null`
        do
          # use 3th - 100th column to allow users use colons in the alias name
          l_alias_tmp=`echo "$alias_row"|sed -e 's/\\\\:/=====colon=====/g' |cut -d":" -f "3-100"|sed -e 's/. $//' -e 's/=====space=====/ /g' -e 's/=====colon=====/:/g'`
          lpar_menu=`echo "$x_slash [$l_alias_tmp]"`
        done
      fi
      echo "$lpar_menu"
}

# WPAR search
wpar_search ()
{
  w_lpar_dir=$1
  w_lpar=$2
  w_lpar_url=$3
  w_hmc=$4
  w_server=$5
  w_hmc_url=$6
  w_server_url=$7
  for w_lpar_item in "$w_lpar_dir"/*
  do
    if [ ! -d "$w_lpar_item" ]; then
      continue
    fi
    if [ `ls -l -- "$w_lpar_item"/*.mmm 2>/dev/null|wc -l` -eq 0 ]; then
      continue
    fi
    wpar=`basename $TWOHYPHENS "$w_lpar_item"`
    if [ $DEBUG -eq 1 ]; then echo "WPAR found     : $wpar : $w_lpar_item"; fi
    wpar_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$wpar"`

    #find out alias in etc/alias.cfg
    if [ $ALIAS -eq 1 ]; then
      wpar_colon=`echo "$wpar"|sed 's/:/\\\\\\\\:/g'` # support colon in lpar name (use back slash for prefixing colons in etc/alias.cfg)
      for alias_row in `egrep "^WPAR:$wpar_colon:" $ALIAS_CFG |sed 's/ /=====space=====/g'|head -1 2>/dev/null`
      do
        # use 3th - 100th column to allow users use colons in the alias name
        w_alias=`echo $alias_row|sed -e 's/\\\\:/=====colon=====/g' |cut -d":" -f "3-100"|sed -e 's/. $//' -e 's/=====space=====/ /g' -e 's/=====colon=====/:/g'`
        wpar=`echo "$wpar [$w_alias]"`
      done
    fi

    menu "$type_wpar" "$w_hmc" "$w_server" "$wpar_url" "$wpar" "/$CGI_DIR/detail.sh?host=$w_hmc_url&server=$w_server_url&lpar=$lpar_url--WPAR--$wpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none" "$w_lpar"
  done
}


# create skeleton of menu
menu () {
  a_type=$1
  a_hmc=`echo "$2"|sed -e 's/:/===double-col===/g' -e 's/\\\\_/ /g'`
  a_server=`echo "$3"|sed -e 's/:/===double-col===/g' -e 's/\\\\_/ /g'`
  a_lpar=`echo "$4"|sed 's/:/===double-col===/g'`
  a_text=`echo "$5"|sed 's/:/===double-col===/g'`
  a_url=`echo "$6"|sed -e 's/:/===double-col===/g' -e 's/ /%20/g'`
  a_lpar_wpar=`echo "$7"|sed 's/:/===double-col===/g'` # lpar name when wpar is passing
  a_last_time=$8

  if [ ! "$LPARS_EXCLUDE"x = "x" ]; then
    if [ `echo "$4"|egrep "$LPARS_EXCLUDE"|wc -l` -eq 1 ]; then
      # keep 2 independent "if" otherwise Solaris has some problem with that
      # excluding some LPARs based on a string : LPARS_EXCLUDE --> etc/.magic
      echo "lpar exclude   : $2:$2:$4 - exclude string: $LPARS_EXCLUDE"
      return 1
    fi
  fi

  if [ "$type_server" = "$type_server_power" -a "$a_type" = "$type_gmenu" -a $gmenu_created -eq 1 ]; then
    return # print global menu once
  fi
  #if [ "$type_server" = "$type_server_kvm" -a "$a_type" = "$type_gmenu" -a $kmenu_created -eq 1 ]; then
  #  return # print global menu once
  #fi
  if [ "$type_server" = "$type_server_vmware" -a  "$a_type" = "$type_gmenu" -a $vmenu_created -eq 1 ]; then
    return # print global menu once
  fi

  echo "$a_type:$a_hmc:$a_server:$a_lpar:$a_text:$a_url:$a_lpar_wpar:$a_last_time:$type_server" >> $MENU_OUT
}

# check if the HMC is alive (at least one server under the HMC has been updated in past 10 days)
check_hmc_alive ()
{

  hmc_once=0
  for pool_file_space in `ls $INPUTDIR/data/*/"$1"/pool.rrm 2>/dev/null|sed 's/ /\%20/g'`
  do
    (( hmc_once = hmc_once + 1 ))
    pool_file=`echo "$pool_file_space"|sed 's/\%20/ /g'`
    access_t=`$PERL -e '$inp=shift;$v = (stat("$inp"))[9]; print "$v\n";' "$pool_file"`
    tot=`expr $TEND + $access_t`
    if [ $tot -gt $TIME ]; then
      return 1
    fi
  done

  # and the same for pool.rrh
  for pool_file_space in `ls $INPUTDIR/data/*/"$1"/pool.rrh 2>/dev/null|sed 's/ /\%20/g'`
  do
    (( hmc_once = hmc_once + 1 ))
    pool_file=`echo "$pool_file_space"|sed 's/\%20/ /g'`
    access_t=`$PERL -e '$inp=shift;$v = (stat("$inp"))[9]; print "$v\n";' "$pool_file"`
    tot=`expr $TEND + $access_t`
    if [ $tot -gt $TIME ]; then
      return 1
    fi
  done

  if [ $hmc_once -eq 0 ]; then
    # something went wrong, it did not list the HMC, then consider it rather as alive
    if [ $DEBUG -eq 1 ]; then echo "HMC test       : $1 HMC has not been listed : $hmc_once"; fi
    return 1
  else
    return 0
  fi
}


# menu GLOBAL-->HMC
menu_hmc  ()
{
 HMCLIST=`for h1 in $WEBDIR/*; do if [ -d "$h1" ]; then echo $h1|sed 's/ /%20/g'; fi done|sort -f|egrep -v ".png|.html|upload|jquery|css|\/no_hmc$|\/favourites$|\/custom$"|xargs -n 1024`

 for hmc1 in $HMCLIST
 do
   hmc1=`echo $hmc1|sed 's/%20/ /g'`
   # directory test is already above
   #if [ ! -d "$hmc1" ]; then
   #  continue
   #fi

   hmc1base=`basename $TWOHYPHENS "$hmc1"`
   ivm_file=`ls $INPUTDIR/data/*/"$hmc1base"/IVM 2>/dev/null `
   if [ ! "$ivm_file"x = "x" ]; then
     # Looks like IVM, exclude it here
     continue
   fi

   # since 4.8
   # sometimes there is empty data/*/hmc1base dir e.g. user removed, or when debugging or beta testing
   empty_dir=`find "$INPUTDIR"/data/*/"$hmc1base"/ -type f 2>/dev/null|wc -l|sed 's/ //g'`
   if [ $empty_dir -eq 0 ]; then
     # Looks like empty, exclude it here
     continue
   fi

   has_one_server=0
   for srv in $hmc1/*
   do
     if [ -f "$srv/config.html" -o -f "$srv/vmware.txt" -o -f "$srv/hyperv.txt" ]; then
       has_one_server=1
       break
     fi
   done

   # find out server type
   type_server=$type_server_power
   vmware_find=`ls $INPUTDIR/data/*/"$hmc1base"/vmware.txt 2>/dev/null|wc -l `
   if [ $vmware_find -gt 0  ]; then
      type_server=$type_server_vmware
   fi
   kvm_find=`ls $INPUTDIR/data/*/"$hmc1base"/kvm.txt 2>/dev/null |wc -l`
   if [  $kvm_find -gt 0 ]; then
       type_server=$type_server_kvm
   fi
   hyperv_find=`ls $INPUTDIR/data/*/"$hmc1base"/hyperv.txt 2>/dev/null|wc -l `
   if [ $hyperv_find -gt 0  ]; then
      type_server=$type_server_hyperv
   fi

   if [ $has_one_server -eq 1 ]; then #  -a ! "$type_server" = "$type_server_vmware" -a ! "$type_server" = "$type_server_kvm" ]; then
   # print only HMC with at least one attached server, it is security test to avoid trash inside
     check_hmc_alive $hmc1base
     if [ $? -eq 1 ]; then
       menu "$type_hmenu" "$hmc1base" "$hmc1base" "/$CGI_DIR/detail.sh?host=$hmc1base&server=nope&lpar=nope&item=hmctotals&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
     else
       if [ $DEBUG -eq 1 ]; then echo "HMC dead       : $hmc1base does not any active server attached, excluding it"; fi
     fi
   fi

 done
}



print_log () {

cat << END
<div  id="tabs"> <ul>
  <li><a href="support_logs.html">Support Logs</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=errcgi&gui=$new_gui">CGI-BIN</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=loadout&gui=$new_gui">Run time</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=errlog&gui=$new_gui">Error</a></li>
END
if [ -f "$INPUTDIR/logs/error.log-vmware" ]; then
  echo " <li><a href=\"/$CGI_DIR/log-cgi.sh?name=errlogvm&gui=$new_gui\">Error VMware</a></li>"
fi

if [ -f /var/log/httpd/error_log ]; then
  # Apache error log
  echo " <li><a href=\"/$CGI_DIR/log-cgi.sh?name=apache&gui=$new_gui\">Apache error</a></li>"
fi
#if [ ! $ENTITLEMENT_LESS -eq 1 ]; then
#  echo "  <li><a href=\"/$CGI_DIR/log-cgi.sh?name=entitle&gui=$new_gui\">Resource Configuration Advisor</a></li>"
#fi

#echo "  <li><a href=\"/$CGI_DIR/log-cgi.sh?name=alhist&gui=$new_gui\">Alert history</a></li>"
#echo "  <li><a href=\"/$CGI_DIR/log-cgi.sh?name=counters&gui=$new_gui\">VMware counters</a></li>"

cat << END
   </ul>
</div>

END

}


print_sys_log () {
hmc_sys="$1"
server_sys="$2"
new_gui=$3

cat << END
<div  id="tabs">
<ul>
  <li class="tabhmc"><a href="$hmc_sys/$server_sys/gui-change-state.html">State change</a></li>
  <li class="tabhmc"><a href="$hmc_sys/$server_sys/gui-change-config.html">Config change</a></li>
</ul>
</div>
END
}


print_cfg () {
new_gui=$1

cat << END
<div  id="tabs"> <ul>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=maincfg&gui=$new_gui">Global</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=favcfg&gui=$new_gui">Favourites</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=custcfg&gui=$new_gui">Custom Groups</a></li>
  <li><a href="/$CGI_DIR/log-cgi.sh?name=alrtcfg&gui=$new_gui">Alerting</a></li>
   </ul>
</div>
END
}


# build hmc based lpar and server cvs files into global one
create_cvs_global () {

first=0
cvs_global_lpar="lpar-config.csv"
if [ `ls $WEBDIR/*/*-lpar-config.csv 2>/dev/null|wc -l` -gt 0 ]; then
  for cvs_file in $WEBDIR/*/*-lpar-config.csv
  do
    if [ $first -eq 0 ]; then
      first=1
      cp "$cvs_file" "$WEBDIR/$cvs_global_lpar"
      continue
    fi
    egrep -v "^HMC;server;lpar_name;" "$cvs_file" >> "$WEBDIR/$cvs_global_lpar"
  done
fi


first=0
cvs_global_server="server-config.csv"
if [ `ls $WEBDIR/*/*-server-config.csv 2>/dev/null|wc -l` -gt 0 ]; then
  for cvs_file in $WEBDIR/*/*-server-config.csv
  do
    if [ $first -eq 0 ]; then
      first=1
      cp "$cvs_file" "$WEBDIR/$cvs_global_server"
      continue
    fi

    egrep -v "^HMC;server;" "$cvs_file" >> "$WEBDIR/$cvs_global_server"
  done
fi
}

print_cfg_global () {

cat << END
<BR>
<font size="-1"><A HREF="lpar-config.csv" target="_blank">CSV LPAR GLOBAL</A> / <A HREF="server-config.csv" target="_blank">CSV Server GLOBAL</A></font>
<CENTER>
<TABLE class="tabsyscfg"> <TR>
END

cd $WEBDIR

tmp_search="gui-config-high-sum.html-tmp"


# header at first
for lcfg in `ls */$tmp_search 2>/dev/null|sort`
do
  hmccfg=`echo $lcfg|cut -d"/" -f1`
  servercfg=`echo $lcfg|cut -d"/" -f2`
  lcfg_path=`echo $lcfg|sed s/-tmp$//`
  echo "<td><center><h3><a href="$lcfg_path">$hmccfg</a></h3></center></td>"
done
echo "</tr><tr>"

# body
for lcfg in `ls */$tmp_search 2>/dev/null|sort`
do
  #hmccfg=`echo $lcfg|cut -d"/" -f1`
  #servercfg=`echo $lcfg|cut -d"/" -f2`
  echo "<TD valign="top">"
  cat $lcfg
  echo "</td>"
done
echo "</tr></table>"
echo "</center><br><br><table><tr><td bgcolor=\"#80FF80\"> <font size=\"-1\"> running</font></td>"
echo "<td bgcolor=\"#FF8080\"> <font size=\"-1\"> not running</font></td>"
echo "<td bgcolor=\"#FFFF80\"> <font size=\"-1\"> CPU pool</font></td></tr></table><br>"
date=`date`
echo "<font size="-1">It is updated once a day, last run: $date"

}


as400_cleaning () {
  # AS400 cleaning
  # It must be here to assure that old AS400 JOB data is regularly deleted
  # remove JOB & DISK data files inactive for more than 8 days (they are not displayed in the GUI anyway)
  if [ $DEBUG ]; then echo "AS400 cleaning : "; fi
  for as400_space in `find $INPUTDIR/data -type d -name \*--AS400-- | sed 's/ /===space===/g'`
  do
    as400=`echo "$as400_space"| sed 's/===space===/ /g'`
    if [ -d "$as400/DSK" ]; then
      #if [ $DEBUG ]; then echo "AS400 cleaning : $as400"; fi
      # must be +8 here!
      find "$as400/DSK" -type f -mtime +8 -exec rm -f {} \;
    fi
  done
}

job_cleaning () {
  # clean all files in /JOB dirs older 8 days
  if [ $DEBUG ]; then echo "clean all /JOB : "; fi
  for job_space in `find $INPUTDIR/data -type d -name JOB | sed 's/ /===space===/g'`
  do
    job=`echo "$job_space"| sed 's/===space===/ /g'`
    if [ -d "$job" ]; then
      # if [ $DEBUG ]; then echo "all JOB cleaning : $job"; fi
      # must be +8 here!
      find "$job/" -type f -mtime +8 -exec rm -f {} \;
    fi
  done
}

cfg_backup ()
{
  cfg_file=$1
  date_act=`date "+%Y-%m-%d_%H:%M"`
  if [ ! -d .web_config ]; then
    mkdir .web_config
  fi
  if [ -f web_config/$cfg_file ]; then
    if [ ! -f .web_config/$cfg_file ]; then
      cp -p web_config/$cfg_file .web_config/$cfg_file
      cp -p web_config/$cfg_file .web_config/$cfg_file-$date_act
    else
      if [ `diff web_config/$cfg_file .web_config/$cfg_file 2>/dev/null| wc -l` -gt 0 ]; then
        cp -p web_config/$cfg_file .web_config/$cfg_file
        cp -p web_config/$cfg_file .web_config/$cfg_file-$date_act
      fi
    fi
  fi
}

sys_cfg () {
# To get some basic system overview for troubleshooting and healthcheck activities
echo "======== id; date; tail -5 $INPUTDIR/etc/version.txt, cat $INPUTDIR/etc/.magic" > $INPUTDIR/logs/sys.log
id >> $INPUTDIR/logs/sys.log
date >> $INPUTDIR/logs/sys.log
tail -5 $INPUTDIR/etc/version.txt >> $INPUTDIR/logs/sys.log
cat $INPUTDIR/etc/.magic >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== df $INPUTDIR/data" >> $INPUTDIR/logs/sys.log
df $INPUTDIR/data >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== uname -a" >> $INPUTDIR/logs/sys.log
uname -a >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ls -l $INPUTDIR/bin" >> $INPUTDIR/logs/sys.log
ls -l $INPUTDIR/bin >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ls -la $INPUTDIR/etc/web_config" >> $INPUTDIR/logs/sys.log
ls -la $INPUTDIR/etc/web_config >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ps -ef |egrep \"apache|httpd|2rrd\" " >> $INPUTDIR/logs/sys.log
ps -ef |egrep "apache|httpd|2rrd" >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ps -ef |egrep 2rrd| wc -l " >> $INPUTDIR/logs/sys.log
ps -ef |egrep "2rrd"| wc -l >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== crontab -l" >> $INPUTDIR/logs/sys.log
crontab -l >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ulimit -a" >> $INPUTDIR/logs/sys.log
ulimit -a >> $INPUTDIR/logs/sys.log 2>/dev/null
echo "" >> $INPUTDIR/logs/sys.log
echo "======== vmstat 3 2" >> $INPUTDIR/logs/sys.log
vmstat 3 2 2>/dev/null >> $INPUTDIR/logs/sys.log
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ls -l $INPUTDIR/data " >> $INPUTDIR/logs/sys.log
ls -l $INPUTDIR/data >> $INPUTDIR/logs/sys.log
if [ `uname -a|grep AIX|wc -l| sed 's/ //g' ` -gt 0 ]; then
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== AIX " >> $INPUTDIR/logs/sys.log
  echo "======== lsattr -El sys0  |grep maxuproc " >> $INPUTDIR/logs/sys.log
  lsattr -El sys0  |grep maxuproc  >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== svmon -G -O unit=MB" >> $INPUTDIR/logs/sys.log
  svmon -G -O unit=MB  >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== lparstat -i | grep Online Virtual " >> $INPUTDIR/logs/sys.log
  lparstat -i | grep "Online Virtual" >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== https : rpm -q perl-Crypt-SSLeay perl-Net_SSLeay.pm openssl " >> $INPUTDIR/logs/sys.log
  ulimit -c 0 # must be there as it migh coredump on some AIXes when running from crontab
  rpm -q perl-Crypt-SSLeay perl-Net_SSLeay.pm openssl >> $INPUTDIR/logs/sys.log 2>>$INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "LIBPATH: $LIBPATH" >> $INPUTDIR/logs/sys.log
  echo "PERL: $PERL" >> $INPUTDIR/logs/sys.log
  echo "PERL5LIB: $PERL5LIB" >> $INPUTDIR/logs/sys.log
  libpath_save=$LIBPATH
  export LIBPATH=/usr/lib:/opt/freeware/lib:$LIBPATH
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== $PERL -MLWP -e \'print \"LWP Version: \$LWP::VERSION\"\' " >> $INPUTDIR/logs/sys.log
  $PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'  >> $INPUTDIR/logs/sys.log
  unset LIBPATH
  export LIBPATH=$libpath_save
  echo "======== $PERL -MLWP -e \'print \"LWP Version: $LWP::VERSION\"\' " >> $INPUTDIR/logs/sys.log
  $PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'  >> $INPUTDIR/logs/sys.log
else
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== free 2>/dev/null" >> $INPUTDIR/logs/sys.log
  free 2>/dev/null >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== ls -l  /var/tmp/systemd-private*/tmp/*2rrd-realt-error.log 2>/dev/null" >> $INPUTDIR/logs/sys.log
  ls -l  /var/tmp/systemd-private*/tmp/*2rrd-realt-error.log 2>/dev/null >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== rpm -q perl-LWP-Protocol-https perl-Crypt-SSLeay perl-Mozilla-CA perl-libwww-perl 2>/dev/null" >> $INPUTDIR/logs/sys.log
  rpm -q perl-LWP-Protocol-https perl-Crypt-SSLeay perl-Mozilla-CA perl-libwww-perl 2>/dev/null >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== dpkg --list liblwp-protocol-https-perl libcrypt-ssleay-perl libio-socket-ssl-perl libmozilla-ldap-perl " >> $INPUTDIR/logs/sys.log
  dpkg --list liblwp-protocol-https-perl libcrypt-ssleay-perl libio-socket-ssl-perl libmozilla-ldap-perl >> $INPUTDIR/logs/sys.log 2>/dev/null
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== $PERL -MLWP -e \'print \"LWP Version: $LWP::VERSION\"\' " >> $INPUTDIR/logs/sys.log
  $PERL -MLWP -e 'print "LWP Version: $LWP::VERSION\n"'  >> $INPUTDIR/logs/sys.log
fi
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ls -l $INPUTDIR/vmware-lib " >> $INPUTDIR/logs/sys.log
ls -l $INPUTDIR/vmware-lib >> $INPUTDIR/logs/sys.log
echo "" >> $INPUTDIR/logs/sys.log
echo "======== $PERL bin/perl_modules_check.pl " >> $INPUTDIR/logs/sys.log
$PERL $INPUTDIR/bin/perl_modules_check.pl >> $INPUTDIR/logs/sys.log
echo "" >> $INPUTDIR/logs/sys.log
echo "======== ls -la $INPUTDIR/etc $INPUTDIR/etc/web_config" >> $INPUTDIR/logs/sys.log
ls -la $INPUTDIR/etc $INPUTDIR/etc/web_config >> $INPUTDIR/logs/sys.log
echo "" >> $INPUTDIR/logs/sys.log
echo "======== cat $INPUTDIR/etc/lpar2rrd.cfg" >> $INPUTDIR/logs/sys.log
cat $INPUTDIR/etc/lpar2rrd.cfg >> $INPUTDIR/logs/sys.log
echo "" >> $INPUTDIR/logs/sys.log
echo "======== cat $INPUTDIR/tmp/HMC-version-*txt" >> $INPUTDIR/logs/sys.log
for hmc_name in $INPUTDIR/tmp/HMC-version-*txt
do
  hmc_version=`cat $hmc_name 2>/dev/null`
  hmc_name_base=`basename $hmc_name .txt`
  echo "$hmc_name_base: $hmc_version" | sed 's/HMC-version-//' >> $INPUTDIR/logs/sys.log
done
if [ -f $INPUTDIR/vmware-lib/apps/connect.pl ]; then
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== VMware: $PERL $INPUTDIR/vmware-lib/apps/connect.pl --version " >> $INPUTDIR/logs/sys.log
  $PERL $INPUTDIR/vmware-lib/apps/connect.pl --version >> $INPUTDIR/logs/sys.log
  echo "" >> $INPUTDIR/logs/sys.log
  echo "======== VMware: ls -l $INPUTDIR/vmware-lib/ " >> $INPUTDIR/logs/sys.log
  ls -l $INPUTDIR/vmware-lib/ >> $INPUTDIR/logs/sys.log
fi
echo "" >> $INPUTDIR/logs/sys.log
echo "END" >> $INPUTDIR/logs/sys.log
}

# checks if managed name is alive: if it does not update
#  pool.rrm for 10 days then it is considered as dead
is_alive() {

# never use same name of variable in function as it the program globally!!!!
hmcA=`echo "$1"|sed 's/\\\\_/ /g'`
managednameA=`echo "$2"|sed 's/\\\\_/ /g'`

if [ `echo "$managednameA"|egrep -- "--unknown$"|wc -l` -eq 1 -o `echo "$hmcA"|egrep "^no_hmc$"|wc -l` -eq 1 ]; then
  # lpars without the HMC, exclude old one (they might appear there if LPAR is installed before the server has allowed util collection
  # always alive so far
  if [ `find $INPUTDIR/data/"$managednameA"/"$hmcA" -type f -mtime -10 2>/dev/null|wc -l` -gt 1 ]; then
    return 0
  else
    return 1
  fi
fi

# webdir always live so far
if [ -f "$INPUTDIR/data/$managednameA/$hmcA/vmware.txt" ]; then
  return 0
fi
if [ -f "$INPUTDIR/data/$managednameA/$hmcA/hyperv.txt" ]; then
  return 0
fi


if [ ! -d $INPUTDIR/data/"$managednameA"/"$hmcA" ]; then
 if [ -d "$WEBDIR/$hmcA/$managednameA" ]; then
   #if [ $DEBUG -eq 1 ]; then echo "rm old system  : rm -rf $WEBDIR/$hmcA/$managednameA (no HMC found)"; fi
   rm -rf $WEBDIR/"$hmcA"/"$managednameA"
 fi
 return 1
fi


if [ -f "$INPUTDIR/data/$managednameA/$hmcA/pool.rrm" ]; then
  access_t=`$PERL -e '$inp=shift;$m=shift;$h=shift;$v = (stat("$inp/data/$m/$h/pool.rrm"))[9]; print "$v\n";' $INPUTDIR "$managednameA" $hmcA`
else if [ -f "$INPUTDIR/data/$managednameA/$hmcA/pool.rrh" ]; then
       access_t=`$PERL -e '$inp=shift;$m=shift;$h=shift;$v = (stat("$inp/data/$m/$h/pool.rrh"))[9]; print "$v\n";' $INPUTDIR "$managednameA" $hmcA`
     else
       access_t=0
     fi
fi

tot=`expr $TEND + $access_t`

if [ $tot -lt $TIME ]; then
  # it is death
  if [ -d "$WEBDIR/$hmcA/$managednameA" ]; then
    #if [ $DEBUG -eq 1 ]; then echo "rm old system  : rm -rf $WEBDIR/$hmcA/$managednameA (is not being updated for >10days)"; fi
    rm -rf $WEBDIR/"$hmcA"/"$managednameA"
  fi
  return 1
fi

return 0
}

# checks if file is alive (if it does not update for 30 days then it is considered as dead)
file_alive() {
# echo file_alive "$1" that is all
if [ -f "$1" ]; then
  # echo lives $1
  if [ `find "$1" -mtime +$active_days 2>/dev/null | wc -l ` -eq 0 ]; then
      return 0 # is alive
  else
      return 1 # is dead
  fi
else
  return 1 # is dead
fi
}

#
### subroutines part end
#

#
# real start of the script`
#


if [ $UPGRADE -eq 0 ]; then
  if [ ! -f $TMPDIR_LPAR/$version_run ]; then
    if [ $DEBUG -eq 1 ]; then echo "Menu           : no menu refresh for $1"; fi
    exit 0
  else
    if [ $DEBUG -eq 1 ]; then echo "Menu           : menu refresh for $1"; fi
  fi
else
  if [ $DEBUG -eq 1 ]; then echo "Menu           : due to an upgrade, re-newing web pages"; fi
  touch $TMPDIR_LPAR/$version
fi

if [ $UPGRADE -eq 1  -o ! -f $WEBDIR/not-implemented-yet.html -o ! -f $WEBDIR/cgcfg-help.html -o ! -f $WEBDIR/nmonfile.html -o ! -f $WEBDIR/noscript.html -o ! -f $WEBDIR/wipecookies.html -o ! -f "$WEBDIR/index.html" -o ! -f "$WEBDIR/gui-help.html" -o ! -f "$WEBDIR/dashboard.html" -o ! -f "$WEBDIR/cpu_workload_estimator.html" ]; then
  cp "$INPUTDIR/html/cpu_workload_estimator.html" "$WEBDIR/"
  cp "$INPUTDIR/html/dashboard.html" "$WEBDIR/"
  cp "$INPUTDIR/html/gui-help.html" "$WEBDIR/"
  cp "$INPUTDIR/html/index.html" "$WEBDIR/"
  cp "$INPUTDIR/html/wipecookies.html" "$WEBDIR/"
  cp "$INPUTDIR/html/noscript.html" "$WEBDIR/"
  cp "$INPUTDIR/html/nmonfile.html" "$WEBDIR/"
  cp "$INPUTDIR/html/test.html" "$WEBDIR/"
  cp "$INPUTDIR/html/favicon.ico" "$WEBDIR/"
  cp "$INPUTDIR/html/robots.txt" "$WEBDIR/"
  cp "$INPUTDIR/html/cgcfg-help.html" "$WEBDIR/"
  cp "$INPUTDIR/html/not-implemented-yet.html" "$WEBDIR/"

  # remove new fancybox as we use the older GNU licensed now
  rm -f "$WEBDIR/jquery/jquery.fancybox.pack.js"
fi


if [ ! -d "$WEBDIR/jquery" -o $UPGRADE -eq 1 -o ! -d "$WEBDIR/css" ]; then
  if [ ! -L "$WEBDIR/jquery" ]; then
    cd $INPUTDIR/html
    tar cf - jquery | (cd $WEBDIR ; tar xf - )
    cd - >/dev/null
  fi
fi

if [ ! -d "$WEBDIR/css" -o $UPGRADE -eq 1 ]; then
  if [ ! -L "$WEBDIR/css" ]; then
    cd $INPUTDIR/html
    tar cf - css | (cd $WEBDIR ; tar xf - )
    cd - >/dev/null
  fi
fi

# No, no, it must be done manually once apache is properly configured
# copy must be done even to cgi-bin dir
#if [ ! -f "$WEBDIR/.htaccess" -a -f "$INPUTDIR/html/.htaccess" ]; then
#  copy only once
#  cp "$INPUTDIR/html/.htaccess" "$WEBDIR/.htaccess"
#fi

# it must run all the time but only once for all technologies
if [ ! "$1"x = "x" -a "$1" = "power" ]; then
  cfg_backup custom_groups.cfg
  cfg_backup acl.cfg
  cfg_backup users.json
  cfg_backup htusers.cfg
  cfg_backup alerting.cfg
  cfg_backup reporter.json
  cfg_backup hosts.json

  sys_cfg
  as400_cleaning
  job_cleaning

  # create pages with physical and logical configuration
  echo "config global  : creating global config"
  G_CFG="$WEBDIR/config-global.htm"
  G_CFG_GUI_BASE="gui-config-global.htm"
  G_CFG_GUI="$WEBDIR/$G_CFG_GUI_BASE"
  print_cfg_global > "$G_CFG_GUI"
  DATE=`date`
  echo "config global  : creating CSV global config $DATE"
  create_cvs_global
fi


if [ "$VMWARE_ACTIVE_DAYS"x = "x" ]; then
  active_days=30
else
  active_days=$VMWARE_ACTIVE_DAYS
fi
# echo "active_days $active_days"


###################################################################################################
# Main section
###################################################################################################

# generate linux_uuid_name.txt for all agent VMs
if [ -d "$INPUTDIR/data/Linux--unknown" ]; then
  `$PERL $INPUTDIR/bin/find_active_lpar.pl "AGENT_UUIDS" 2>>$ERRLOG`
  DATE=`date`
  if [ $? -eq 0 ]; then
    echo "AGENT_UUIDS    : OK at $DATE, find_active_lpar.pl"
  else
    echo "AGENT_UUIDS    : AGENT_UUIDS failed with value $? find_active_lpar.pl"
  fi
fi

# XEN part
if [ $XEN -eq 1 ]; then
  rm -f $MENU_XENSERVER_JSON_OUT

  if [ -d "$INPUTDIR/data/XEN" -o -d "$INPUTDIR/data/XEN_VMs" ]; then
    # write menu json to file, only if it is non-empty
    MENU_JSON=`$PERL -w $INPUTDIR/bin/xen-genmenu.pl 2>>$ERRLOG`

    if [ -n "$MENU_JSON" ]; then
      echo "$MENU_JSON" >> "$MENU_XENSERVER_JSON_OUT"
    fi
  fi

  rm -f $TMPDIR_LPAR/$version_run
  exit
fi

# oVirt part
if [ $OVIRT -eq 1 ]; then
  rm -f $MENU_OVIRT_JSON_OUT

  if [ -d "$INPUTDIR/data/oVirt" -a -f "$INPUTDIR/data/oVirt/metadata.json" ]; then
    MENU_JSON=`$PERL -w $INPUTDIR/bin/ovirt-genmenu.pl 2>>$ERRLOG`

    if [ -n "$MENU_JSON" ]; then
      echo "$MENU_JSON" >> "$MENU_OVIRT_JSON_OUT"
    fi
  fi

  rm -f $TMPDIR_LPAR/$version_run
  exit
fi

#
### separated vmware part creates only vmware lines
#
if [ $VMWARE -eq 1 ]; then
  DATE=`date`
  echo "timestamp      : $DATE install-html-vmware start ----------"
  if [ -f "$INPUTDIR/bin/install-html-vmware.pl" ]; then
    "$PERL" "$INPUTDIR/bin/install-html-vmware.pl" 2>>$ERRLOG
    if [ $? -eq 0 ]; then
      echo "VMware         : $PERL $INPUTDIR/bin/install-html-vmware.pl: OK"
    else
      echo "WMware         : $PERL $INPUTDIR/bin/install-html-vmware.pl failed with value $?"
    fi
    cat "$TMPDIR_LPAR/menu_vmware_pl.txt" >> $MENU_OUT
  else
    . $INPUTDIR/bin/install-html-vmware.sh
  fi
  DATE=`date`
  echo "timestamp      : $DATE install-html-vmware finish ----------"

  # add all VMWARE VMs if exist
  if [ -d "$INPUTDIR/data/vmware_VMs" ]; then
    `$PERL $INPUTDIR/bin/find_active_lpar.pl "VMWARE" >> $MENU_OUT 2>>$ERRLOG`
    if [ $? -eq 0 ]; then
      DATE=`date`
      echo "VMware         : find_active_lpar.pl OK at $DATE"
    else
      echo "VMware         : find_active_lpar.pl failed with value $?"
      DATE=`date`
      if [ $DEBUG ]; then echo "timestamp      : $DATE  finish"; fi
    fi
  fi

  # copy new menu over the prod ove
  if [ -f $MENU_OUT ]; then
    cp $MENU_OUT $MENU_OUT_FINAL
    # create total tmp/menu.txt which is used by heatmap, daily_check etc., start always with IBM Power menu
    if [ -f "$TMPDIR_LPAR/menu_power.txt" ]; then
      cp "$TMPDIR_LPAR/menu_power.txt" $TMPDIR_LPAR/menu.txt
      cat $MENU_OUT >> $TMPDIR_LPAR/menu.txt
    else
      cp $MENU_OUT $TMPDIR_LPAR/menu.txt
    fi
  else
    echo "menu does not exist: file $MENU_OUT, probably no VMware configured"
    exit
  fi

  # data_check Vmotion tab
  echo "lpar_start_counter : in background"
  nohup $PERL -w $BINDIR/lpar_start_counter.pl > $TMPDIR/lpar_start_counter.txt 2>>$ERRLOG &
  #if [ $? -eq 0 ]; then
  #  echo "command lpar_start_counter.pl: OK"
  #else
  #  echo "command lpar_start_counter.pl failed with value $?"
  #fi

  rm -f $TMPDIR_LPAR/$version_run

  RET=`grep ":ERROR:" $MENU_OUT`
  if [ `echo $RET| wc -w` -gt 1 ]; then
    echo "ERROR menu.txt : $RET"
  fi
  exit
fi

#
### power part creates all necessary menu lines except vmware lines
#

if [ $premium -eq 1 ]; then # BINDIR cannot be used here as it is relative
  technology=""
  if [ -f "$INPUTDIR/html/.p" ]; then
    technology="$technology:p"
  fi
  if [ -f "$INPUTDIR/html/.v" ]; then
    technology="$technology:v"
  fi
  if [ -f "$INPUTDIR/html/.s" ]; then
    technology="$technology:s"
  fi
  if [ -f "$INPUTDIR/html/.x" ]; then
    technology="$technology:x"
  fi
  if [ -f "$INPUTDIR/html/.o" ]; then
    technology="$technology:o"
  fi
  if [ -f "$INPUTDIR/html/.h" ]; then
    technology="$technology:h"
  fi
  technology=`echo $technology| sed 's/^://g'`
  menu "$type_version" "0" "$technology" "" "" "" ""
else
  menu "$type_version" "1" "" "" "" "" ""
fi

menu "$type_ent" "$ENTITLEMENT_LESS" "" "" "" "" ""

version_act=$version
if [ -f $INPUTDIR/etc/version.txt ]; then
  # actually installed version include patch level (in $version is not patch level)
  version_act=`cat $INPUTDIR/etc/version.txt|tail -1| sed 's/ .*//'`
fi
menu "$type_qmenu" "$version_act"

#
# Main loop for creating of menu
#


# workaround for sorting
HLIST=`for m in $INPUTDIR/data/*; do if [ -L "$m" ]; then continue; fi; echo "$m"|sed -e 's/ /=====space=====/g'; done|sort -fr|xargs -n 1024`
for dir1 in $HLIST
do

  # workaround for managed names with a space inside
  dir1_space=`echo "$dir1"|sed 's/=====space=====/ /g'`
  #managedname_space=`basename "$dir1"`
  managedname=`basename $TWOHYPHENS "$dir1_space"`
  managedname_space="$managedname"
  # echo "xxxxxxxxxxxxxxxxxxxxxxxxxxx working for server $managedname_space"
  #date
  # exclude sym links
  # exclusion must be done above in HLIST to avoid a problem with server name like model-serial (that match sym link model*serial)!!
  #if [ -L "$dir1" -o -f "$dir1" ]; then
  #  continue
  #fi

  # exclude folder with VMware VMs
  if [ `echo "$managedname"|egrep "^vmware_VMs"|wc -l` -eq 1 ]; then
     continue
  fi
  # exclude folder with Hyper-V VMs
  #if [ `echo "$managedname"|egrep "^hyperv_VMs"|wc -l` -eq 1 ]; then
  #   continue
  #fi
  # exclude directory with all XenServer stuff
  if [ `echo "$managedname"|egrep "^XEN"|wc -l` -eq 1 ]; then
     continue
  fi

  # exclude directory with all oVirt stuff
  if [ `echo "$managedname"|egrep "^oVirt"|wc -l` -eq 1 ]; then
     continue
  fi

  # Exclude excluded systems
  eval 'echo "$MANAGED_SYSTEMS_EXCLUDE" | egrep "^$managedname:|:$managedname:|:$managedname$|^$managedname$" >/dev/null 2>&1'
  if [ $? -eq 0 ]; then
    if [ $DEBUG -eq 1 ]; then echo "Excluding      : $managedname"; fi
    continue
  fi

  # workaround for sorting
  MLIST=`for m in $INPUTDIR/data/"$managedname"/*; do echo "$m"|sed -e 's/ /=====space=====/g' ; done|sort -f|xargs -n 1024`

  # Hitachi
  if [ `echo "$managedname" | egrep "^Hitachi" | wc -l` -eq 1 ]; then
    type_server=$type_server_hitachi

    for dir2 in $MLIST
    do
      dir2_space=`echo "$dir2"|sed 's/=====space=====/ /g'`
      hvm_id=`basename $TWOHYPHENS "$dir2_space"`
      hvm_id_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$hvm_id"`

      last_time=0 # servers with newer timestamp will be defaulty displayed in servers menu
        if [ -f $INPUTDIR/data/"$managedname"/"$hvm_id"/SYS-CPU.hrm ]; then
          last_time=`$RRDTOOL last $INPUTDIR/data/"$managedname"/"$hvm_id"/SYS-CPU.hrm`
        fi

      # system data
      menu "$type_smenu" "Hitachi" "$hvm_id" "CPU" "CPU pool" "/lpar2rrd-cgi/detail.sh?host=$hvm_id_url&server=Hitachi&lpar=pool&item=pool&entitle=0&gui=1&none=none" "" $last_time
      menu "$type_smenu" "Hitachi" "$hvm_id" "MEM" "MEM" "/lpar2rrd-cgi/detail.sh?host=$hvm_id_url&server=Hitachi&lpar=mem&item=memalloc&entitle=0&gui=1&none=none" "" $last_time
    done
    continue
  fi

  #
  ### windows
  #

  if [ `echo "$managedname" | egrep "^windows" | wc -l` -eq 1 ]; then
    type_server=$type_server_hyperv
    #    type_server=$type_server_power

    # hyperv global menu
    menu "$type_gmenu" "heatmapvm" "Heatmap" "heatmap-hyperv.html"
    # menu "$type_gmenu" "advisorvm" "Resource Configuration Advisor" "gui-cpu_max_check_vm.html"
    # menu "$type_gmenu" "ghreportsvm" "Historical reports" "not-implemented-yet.html"
    # menu "$type_gmenu" "datastoretop" "Datastores TOP" "/$CGI_DIR/detail.sh?host=&server=&lpar=&item=dstr-table-top&entitle=0&none=none"
    # menu "$type_gmenu" "gcfgvm" "Configuration" "not-implemented-yet.html"
    menu "$type_gmenu" "gtop10vm" "Top 10 global" "/$CGI_DIR/detail.sh?host=&server=&lpar=cod&item=topten_vm&entitle=0&none=none"

    # echo "mlist ,$MLIST,"
    for dir2 in $MLIST
    do
      dir2_space=`echo "$dir2"|sed 's/=====space=====/ /g'`
      dir2_space_base=`basename $TWOHYPHENS "$dir2_space"` # this is domain name
      dir2_space_base_par=${dir2_space_base#"cluster_"}
      if [ `echo "$dir2_space_base" | egrep "^cluster_" | wc -l` -eq 1 ]; then

        if [ $DEBUG -eq 1 ]; then echo "add to menu $type_amenu  : CLUST TOT $vn_file_nem:$cn_file_nem:$managedname"; fi
        menu "$type_amenu" "" "$dir2_space_base_par" "Cluster totals" "/$CGI_DIR/detail.sh?host=$dir2_space_base&server=windows&lpar=nope&item=cluster&entitle=0&gui=1&none=none"

        continue
      fi
      if [ `echo "$dir2_space_base" | egrep "^domain_UnDeFiNeD" | wc -l` -eq 1 ]; then
        continue # this can be created when error comes in data
      fi

      # data/windows/domain_name/server_name or hyperv_VMs/
      # get server_name
      server_name_list=`for m in $INPUTDIR/data/"$managedname"/"$dir2_space_base"/*; do echo "$m"|sed -e 's/ /=====space=====/g' ; done|sort -f|xargs -n 1024`
      # echo "server_name_list ,$server_name_list,"
      for server_name in $server_name_list
      do
        server_name_space=`echo "$server_name"|sed 's/=====space=====/ /g'`
        if [ `echo "$server_name_space" | egrep "hyperv_VMs$" | wc -l` -eq 1 ]; then
          continue
        fi
        server_name_space_base=`basename $TWOHYPHENS "$server_name_space"`
        domain_id="$dir2_space_base"
        domain_id_par=${domain_id#"domain_"}
        server_id="$server_name_space_base"
        domain_id_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$domain_id"`
        server_id_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$server_id"`

        domain_id_url="windows/$domain_id_url";

        last_time=0 # servers with newer timestamp will be defaulty displayed in servers menu
      #    if [ -f $INPUTDIR/data/"$managedname"/"$hvm_id"/SYS-CPU.hrm ]; then
      #      last_time=`$RRDTOOL last $INPUTDIR/data/"$managedname"/"$hvm_id"/SYS-CPU.hrm`
      #    fi

        # echo "domain_id_par ,$domain_id_par, server_id ,$server_id, domain_id ,$domain_id,"
        # system data
        menu "$type_smenu" "$domain_id_par" "$server_id" "CPU" "CPU" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=pool&item=pool&entitle=0&gui=1&none=none" "" $last_time
        menu "$type_smenu" "$domain_id_par" "$server_id" "Memory" "Memory" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=mem&item=memalloc&entitle=0&gui=1&none=none" "" $last_time
        # menu "$type_smenu" "$domain_id_par" "$server_id" "Disk" "Disk" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=cod&item=vmdiskrw&entitle=0&gui=1&none=none" "" $last_time
        menu "$type_hdt_menu" "$domain_id_par" "$server_id" "Data" "Data" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=cod&item=hdt_data&entitle=0&gui=1&none=none" "" $last_time
        menu "$type_hdt_menu" "$domain_id_par" "$server_id" "IO" "IO" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=cod&item=hdt_io&entitle=0&gui=1&none=none" "" $last_time
        menu "$type_smenu" "$domain_id_par" "$server_id" "Net" "Net" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=cod&item=vmnetrw&entitle=0&gui=1&none=none" "" $last_time
        # working with Local_Fixed_Disk
        volume_list=`for m in "$server_name_space"/Local_Fixed_Disk*; do echo "$m"|sed -e 's/ /=====space=====/g' ; done|sort -f|xargs -n 1024`
        # echo "in $server_name_space Local_Fixed_Disk_list ,$volume_list,"
        if [ `echo "$volume_list" | egrep "Local_Fixed_Disk_" | wc -l` -gt 0 ]; then
          for csi in $volume_list
          do
            cso=${csi/Local_Fixed_Disk_/}
            cs=${cso/.rrm/}
            cs_space=`echo "$cs"|sed 's/=====space=====/ /g'`
            cs_id=`basename $TWOHYPHENS "$cs_space"`

            cs_id_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$cs_id"`
            menu "$type_hdi_menu" "$domain_id_par" "$server_id" "$cs_id" "$cs_id_url" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=$cs_id&item=lfd&entitle=0&gui=1&none=none" "" $last_time
          done
        fi

        # working with Cluster_Storage_Volume
        volume_list=`for m in "$server_name_space"/Cluster_Storage*; do echo "$m"|sed -e 's/ /=====space=====/g' ; done|sort -f|xargs -n 1024`
        # echo "in $server_name_space cluster_shared_volume_list ,$volume_list,"
        if [ `echo "$volume_list" | egrep "Cluster_Storage_" | wc -l` -gt 0 ]; then
          # menu "$type_smenu" "$domain_id_par" "$server_id" "CSV" "CSV" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=cod&item=csv&entitle=0&gui=1&none=none" "" $last_time
          for csi in $volume_list
          do
#            cso=${csi/Cluster_Storage_Volume/CSV}
            cso=${csi/Cluster_Storage_/}
            cs=${cso/.rrm/}
            cs_space=`echo "$cs"|sed 's/=====space=====/ /g'`
            cs_id=`basename $TWOHYPHENS "$cs_space"`
            cs_id_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$cs_id"`
            menu "$type_hdi_menu" "$domain_id_par" "$server_id" "$cs_id" "$cs_id_url" "/lpar2rrd-cgi/detail.sh?host=$server_id_url&server=$domain_id_url&lpar=$cs_id&item=csv&entitle=0&gui=1&none=none" "" $last_time
          done
        fi

      done
    done
    continue
  fi

  #
  ### end of windows section
  #

  vcenter_printed=0
  for dir2 in $MLIST
  do
    # workaround for managed names with a space inside
    dir2_space=`echo "$dir2"|sed 's/=====space=====/ /g'`
    if [ ! -d "$dir2_space" ]; then
      continue # is not HMC, some trash
    fi

    # echo "ooooooooooooooooooooooo working for hmc $dir2_space"
    #date
    hmc=`basename $TWOHYPHENS "$dir2_space"`
    #hmc_space=`basename "$dir2"`
    hmc_space="$hmc"

    server_menu=$type_smenu # must be used $server_menu since now instead of $type_smenu as it says if the server is dead or not
    is_alive `echo "$hmc"|sed 's/ /\\\\\\_/g'`  `echo "$managedname" |sed 's/ /\\\\\\_/g'`
    if [ $? -eq 1 ] ; then
      # avoid systems not being updated
      #continue
      server_menu=$type_dmenu # continue, just sight it as "dead", this brings it into menu.txt and is further used only for global hostorical reporting of already dead servers
    fi

    if [ ! -d "$WEBDIR/$hmc" ]; then
      mkdir "$WEBDIR/$hmc"
    fi
    if [ ! -d "$WEBDIR/$hmc/$managedname" ]; then
      mkdir "$WEBDIR/$hmc/$managedname"
    fi

    if [ -f "$CONFIG_DIR/$managedname/$hmc/config.cfg" ]; then
      cp "$CONFIG_DIR/$managedname/$hmc/config.cfg" "$WEBDIR/$hmc/$managedname/config.html" 2>/dev/null
      if [ $DEBUG -eq 1 ]; then echo "copy save cfg  : $hmc:$managedname"; fi
    fi

    # Check whether it is IVM based box
    IVM=0
    if [ -f "$INPUTDIR/data/$managedname/$hmc/IVM" ];  then
      IVM=1
    fi
    SDMC=0
    if [ -f "$INPUTDIR/data/$managedname/$hmc/SDMC" ];  then
      SDMC=1
    fi

    # find out server type
    type_server=$type_server_power
    if [ -f $INPUTDIR/data/"$managedname"/vmware.txt -o -f $INPUTDIR/data/"$managedname"/"$hmc"/vmware.txt ]; then
      type_server=$type_server_vmware
      continue
    fi
    if [ -f $INPUTDIR/data/"$managedname"/kvm.txt -o -f $INPUTDIR/data/"$managedname"/"$hmc"/kvm.txt ]; then
      type_server=$type_server_kvm
      continue
    fi
    if [ -f $INPUTDIR/data/"$managedname"/hyperv.txt -o -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then
      type_server=$type_server_hyperv
      continue
    fi

    # find what kind of sample rate is being used 3600 or less
    # must be once more as it can vary for different managed systems
    if [ ! -s "$INPUTDIR/data/$managedname/$hmc/pool.rrh" ];  then
      # when pool.rrh has 0 size then delete it, it come from old versions < 4.63
      rm -f "$INPUTDIR/data/$managedname/$hmc/pool.rrh"
    fi
    if [ -f "$INPUTDIR/data/$managedname/$hmc/pool.rrm" ];  then
      if [ -f "$INPUTDIR/data/$managedname/$hmc/pool.rrh" ];  then
        rrh=`$PERL -e '$inp=shift;$m=shift;$h=shift;$v = (stat("$inp/data/$m/$h/pool.rrh"))[9]; print "$v\n";' $INPUTDIR "$managedname" $hmc`
        rrm=`$PERL -e '$inp=shift;$m=shift;$h=shift;$v = (stat("$inp/data/$m/$h/pool.rrm"))[9]; print "$v\n";' $INPUTDIR "$managedname" $hmc`
        if [ $rrh -gt $rrm ]; then
          RR=rrh
        else
          RR=rrm
        fi
      else
         RR=rrm
      fi
    else
      if [ -f "$INPUTDIR/data/$managedname/$hmc/pool.rrh" ];  then
        RR=rrh
      else
        RR=rrm # default, but strange as pool.rrm does not exist
      fi
    fi

    # copy servers log
    if [ $IVM -eq 0 ]; then
      if [ ! -f $INPUTDIR/data/"$managedname"/"$hmc"/vmware.txt -a ! -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then   # not for vmware or hyperv
        if [ -f $INPUTDIR/data/"$managedname"/"$hmc"/gui-change-state.txt ]; then
          cp $INPUTDIR/data/"$managedname"/"$hmc"/gui-change-state.txt $WEBDIR/"$hmc"/"$managedname"/gui-change-state.html
          if [ $DEBUG -eq 1 ]; then echo "copy change sta: $hmc:$managedname"; fi
        fi
        if [ -f $INPUTDIR/data/"$managedname"/"$hmc"/gui-change-config.txt ]; then
          cp $INPUTDIR/data/"$managedname"/"$hmc"/gui-change-config.txt $WEBDIR/"$hmc"/"$managedname"/gui-change-config.html
          if [ $DEBUG -eq 1 ]; then echo "copy change cfg: $hmc:$managedname"; fi
        fi
      fi
    fi

    if [ $gmenu_created -eq 0 -a "$type_server" = "$type_server_power" ]; then
      # super menu
      if [ $smenu_created -eq 0 ]; then
        if [ `grep -v "^#" $INPUTDIR/etc/favourites.cfg 2>/dev/null| wc -l` -gt 0 ]; then
          menu "$type_gmenu" "fav" "FAVOURITES" "favourites/$fav_first/$fav_first/gui-index.html"
        fi
        smenu_created=1 # only once
      fi

      # here create Power global menu
      menu "$type_gmenu" "cgroups" "CUSTOM GROUPS" "custom/$group_first/gui-index.html"
      menu "$type_gmenu" "estimator" "CPU Workload Estimator" "cpu_workload_estimator.html"

      if [ ! $ENTITLEMENT_LESS -eq 1 ]; then
        menu "$type_gmenu" "advisor" "Resource Configuration Advisor" "gui-cpu_max_check.html"
      fi
      menu "$type_gmenu" "heatmaplpar" "Heatmap" "heatmap-power.html"
      menu "$type_gmenu" "alert" "Alerting" "/lpar2rrd-cgi/alcfg.sh?cmd=form"

      if [ $xormon -eq 0 ]; then
        menu "$type_gmenu" "ghreports" "Historical reports" "/lpar2rrd-cgi/histrep.sh?mode=global"
      fi
      menu "$type_gmenu" "gcfg" "Configuration" "/$CGI_DIR/detail.sh?host=&server=&lpar=cod&item=servers&entitle=0&none=none"
      if [ $xormon -eq 0 ]; then
        menu "$type_gmenu" "gtop10" "Top 10 global" "/$CGI_DIR/detail.sh?host=&server=&lpar=cod&item=topten&entitle=0&none=none"
      fi
      menu "$type_gmenu" "nmon" "NMON file grapher" "nmonfile.html"

      if [ ! $ENTITLEMENT_LESS -eq 1 ]; then
        menu "$type_gmenu" "rmc" "RMC check" "gui-rmc.html"
      fi

      menu "$type_gmenu" "hmctot" "HMC totals" ""

      if [ $KEEP_VIRTUAL -eq 1 ]; then # accounting for DHL
        menu "$type_gmenu" "accounting" "Accounting" "/$CGI_DIR/virtual.sh?sort=server&gui=1"
      fi

      gmenu_created=1 # it will not print global menu items
    fi

    # Add others managed systems into the menu
    hmc_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$hmc"`
    hmc_url_hash=`echo "$hmc_url"|sed -e 's/\#/\%23/g'` # for support hashes in the server name
    managedname_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "$managedname"`
    managedname_url_hash=`echo "$managedname_url"|sed -e 's/\#/\%23/g'` # for support hashes in the server name


    # power & not vmware ESXi & not hyperv servers
    if [ `echo "$managedname"|egrep -- "--unknown$"|wc -l` -eq 0  -a `echo "$hmc"|egrep "^no_hmc$"|wc -l` -eq 0 ]; then
      # exclude that for non HMC based lpars
      if [ -f "$INPUTDIR"/data/"$managedname"/"$hmc"/pool.rrm -o -f "$INPUTDIR"/data/"$managedname"/"$hmc"/pool.rrh ]; then
        # exclude servers where is not pool.rrm/h --> but include their lpars below

        # CPU pool
        last_time=0 # servers with newer timestamp will be defaulty displayed in servers menu
        if [ -f $INPUTDIR/data/"$managedname"/"$hmc"/pool.rrh ]; then
          last_time=`$RRDTOOL last $INPUTDIR/data/"$managedname"/"$hmc"/pool.rrh`
        fi
        if [ -f "$INPUTDIR"/data/"$managedname"/"$hmc"/pool.rrm ]; then
          last_time=`$RRDTOOL last $INPUTDIR/data/"$managedname"/"$hmc"/pool.rrm`
        fi

        if [ ! -f "$INPUTDIR"/data/"$managedname"/"$hmc"/vmware.txt -a ! -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then
           menu "$server_menu" "$hmc_space" "$managedname_space" "CPUpool-pool" "CPU pool" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=pool&item=pool&entitle=$ENTITLEMENT_LESS&gui=1&none=none" "" $last_time
        else
           # vmware vcenter server must public its cluster name
           # if not in cluster then public its vcenter name
           cluster_name=""
#           if [ -f $INPUTDIR/data/"$managedname"/"$hmc"/my_cluster_name ]; then
#              cluster_name=`sed 's/|.*//' "$INPUTDIR/data/"$managedname"/"$hmc"/my_cluster_name"`
#           else
#              if [ -f $INPUTDIR/data/"$managedname"/"$hmc"/my_vcenter_name ]; then
#                cluster_name=`sed 's/|.*//' "$INPUTDIR/data/"$managedname"/"$hmc"/my_vcenter_name"`
#              fi
#           fi
#           # vmware server must public its alias name if non vcenter
#           alias_name=""
#           if [ -f "$INPUTDIR/data/$managedname/$hmc/vmware_alias_name" ]; then
#              alias_name=`sed 's/^[^|]*|//' "$INPUTDIR/data/$managedname/$hmc/vmware_alias_name"`
#           fi
#
#           echo "testing update : esxi pool file $INPUTDIR/data/$managedname/$hmc/pool.rrm"
#           file_alive "$INPUTDIR/data/$managedname/$hmc/pool.rrm"
#           if [ $? -eq 1 ] ; then
#             continue
#           else
#             if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : ESXi $managedname:$hmc"; fi
#             menu "$server_menu" "$cluster_name" "$managedname_space" "CPUpool-pool" "CPU pool" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=pool&item=pool&entitle=$ENTITLEMENT_LESS&gui=1&none=none" "$alias_name" $last_time
#             hmc_space=$cluster_name # change it for all menu items
#           fi
        fi
        shared_pools=0
        for sh_pool_space in `ls "$INPUTDIR"/data/"$managedname"/"$hmc"/SharedPool*$RR 2>/dev/null|sed 's/ /\\\\\\_/g'`
        do
          # workaround for managed names with a space inside
          sh_pool=`echo "$sh_pool_space"|sed 's/\\\\_/ /g'`

          if [ ! -f "$sh_pool" ]; then
            continue # on IVMs sometimes happened that it found something although nothing existed there
          fi
          cpupool=`basename $TWOHYPHENS "$sh_pool" .$RR`
          if [ ! "$cpupool" = "SharedPool0" ]; then
            shared_pools=1
            break
          fi
        done

        if [ $shared_pools -eq 1 ]; then
          # not use "-mtime -2" as SharedPools should be visible all the time
          # workaround for sorting
          PLIST=`for m in "$INPUTDIR"/data/"$managedname"/"$hmc"/SharedPool*$RR; do echo "$m" |sed 's/ /\\\\\\\\_/g'; done|sort -f|xargs -n 1024`
          for cpupool_all_space in $PLIST
          do
            cpupool_all=`echo "$cpupool_all_space"|sed 's/\\\\_/ /g'`
            cpupool=`basename $TWOHYPHENS "$cpupool_all" .$RR`
            if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:$cpupool"; fi
            cpupool_sep=`echo "$cpupool"|sed 's/SharedPool/CPU pool /g'`

	        # add cpu pool alias if mapping table exists
            if [ -f "$INPUTDIR/data/$managedname/$hmc/cpu-pools-mapping.txt" ]; then
              pool_id=`echo "$cpupool"|sed 's/SharedPool//g'`
              pool_alias=`egrep "^$pool_id," "$INPUTDIR/data/$managedname/$hmc/cpu-pools-mapping.txt"|sed 's/^\$pool_id,//g'| awk -F, '{print $2 }'`
              if [ "$pool_alias"x = "x" ]; then
                 continue # it is old pool, no more configured, all actually configured pool are in cpu-pools-mapping.txt
              fi
              if [ "$pool_alias" = "$cpupool" ]; then
                menu "$server_menu" "$hmc_space" "$managedname_space" "CPUpool-$cpupool" "$cpupool_sep" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=$cpupool&item=shpool&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
              else
                menu "$server_menu" "$hmc_space" "$managedname_space" "CPUpool-$cpupool" "$cpupool_sep : $pool_alias" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=$cpupool&item=shpool&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
              fi
	        else
                menu "$server_menu" "$hmc_space" "$managedname_space" "CPUpool-$cpupool" "$cpupool_sep" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=$cpupool&item=shpool&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
            fi
          done
        fi

        if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:memory"; fi
        menu "$server_menu" "$hmc_space" "$managedname_space" "mem" "Memory" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=cod&item=memalloc&entitle=$ENTITLEMENT_LESS&gui=1&none=none"


        if [ `ls -l "$INPUTDIR"/data/"$managedname"/"$hmc"/*/pgs.mmm 2>/dev/null| grep -v -- "--NMON--"| wc -l |sed 's/ //g'` -gt 0 ]; then
          # paging aggregated do not support NMON data yet (v4.50)
          if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:Paging agg   "; fi
          menu "$server_menu" "$hmc_space" "$managedname_space" "pagingagg" "Paging aggregated" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=cod&item=pagingagg&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
        fi

        if [ "$type_server" = "$type_server_power" ]; then
          mode="power"
#        else
#          mode="esxi"
#          if [ "$hmc_space" = "" ]; then # if esxi soliter
#            mode="solo_esxi"
#          fi
        fi

        if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:Hist reports "; fi
        menu "$server_menu" "$hmc_space" "$managedname_space" "hreports" "Historical reports" "/lpar2rrd-cgi/histrep.sh?mode=$mode&host=$hmc_url_hash"

        if [ ! -f "$INPUTDIR/data/$managedname/$hmc/vmware.txt" -a ! -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then
          # not for VMware nor Hyper-V
          if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:Top 10       "; fi
          menu "$server_menu" "$hmc_space" "$managedname_space" "top10" "Top 10" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=cod&item=topten&entitle=0&none=none"
        fi


        if [ -f "$INPUTDIR/data/$managedname/$hmc/cod.$RR" ]; then
          # Capacity on Demand if exist data
          if [ ! -f "$INPUTDIR/data/$managedname/$hmc/vmware.txt" -a ! -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then
            # not for VMware nor Hyper-V
            if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:CoD          "; fi
            menu "$server_menu" "$hmc_space" "$managedname_space" "cod" "Capacity on Demand" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=cod&item=codused&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
          fi
        fi
        if [ $REST_API -eq 0 ]; then
          if [ $IVM -eq 0 ]; then
            print_sys_log "$hmc" "$managedname" 1 > $WEBDIR/"$hmc"/"$managedname"/gui-sys_log.html
            if [ -f "$WEBDIR/"/"$hmc"/"$managedname"/gui-sys_log.html -a ! -f "$INPUTDIR/data/$managedname/$hmc/vmware.txt" -a ! -f $INPUTDIR/data/"$managedname"/"$hmc"/hyperv.txt ]; then
              # not for VMware nor Hyper-V
                if [ $xormon -eq 0 -a $REST_API -eq 0 ]; then
                  if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:Server's logs"; fi
                  menu "$server_menu" "$hmc_space" "$managedname_space" "syslog" "Server's logs" "$hmc_url_hash/$managedname_url_hash/gui-sys_log.html"
                fi
            fi
          fi
        fi

        # add VIEWs
        if [ $DEBUG -eq 1 ]; then echo "add to menu $server_menu  : $hmc:$managedname:Views"; fi

        #echo "----------- ------- -      ------- $hmc:$managedname:Views"
        #date

        menu "$server_menu" "$hmc_space" "$managedname_space" "view" "VIEW" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=cod&item=view&entitle=$ENTITLEMENT_LESS&gui=1&none=none"

      fi  # for not pool.rrm.h based server
    fi  # not for HMC managed lpar --> --unknown server suffix

    managedname_print=`echo "$managedname"|sed 's/--unknown$//'`

    # OS agent systems only without the HMC
    #if [ `echo "$managedname"|egrep -- "--unknown$"|wc -l` -eq 1 -o `echo $hmc|egrep "^no_hmc$"|wc -l` -eq 1 ]; then
    #   cd "$INPUTDIR"/data/"$managedname"/"$hmc"
    #   for lpar_os_all in  */cpu.mmm
    #   do
    #     lpar_os=`dirname -- "$lpar_os_all"`
    #     if [ "$lpar_os"  = "." -o  "$lpar_os"  = "*" -o "$lpar_os"  = "-" -o ! -d "$lpar_os" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     if [ -f "$lpar_os.$RR" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     lpar_no_nmon=`echo $lpar_os|sed 's/--NMON--$//'`
    #     if [ `echo $lpar_os|egrep -- "--NMON--$"|wc -l` -eq 1 -a -d "$lpar_no_nmon" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     lpar_slash=`echo "$lpar_no_nmon"|sed 's/\&\&1/\//g'`
    #     lpar_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "dashdashdashdash$lpar_slash"| sed 's/^dashdashdashdash//'` # dashdashdashdash is preventin if lpar name starts with a dash

    #     #find out alias in etc/alias.cfg
    #     lpar_menu=`find_out_alias "$lpar_slash" "LPAR"`

    #     if [ $DEBUG -eq 1 ]; then echo "add to menu $type_lpar  : $hmc:$managedname:$lpar_no_nmon : lpar_url:$lpar_url : OS agent"; fi
    #     menu "$type_lpar" "$hmc_space" "$managedname_print" "$lpar_url" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #     wpar_search "$INPUTDIR"/data/"$managedname"/"$hmc"/"$lpar_os" "$lpar_no_nmon" "$lpar_url" "$hmc" "$managedname_print" "$hmc_url" "$managedname_url"
    #   done

    #   # os agent AS400
    #   for lpar_os_all in  */S0200ASPJOB.mmm
    #   do
    #     lpar_os=`dirname -- "$lpar_os_all"`
    #     if [ "$lpar_os"  = "." -o  "$lpar_os"  = "*" -o "$lpar_os"  = "-" -o ! -d "$lpar_os" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     if [ -f "$lpar_os.$RR" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     lpar_no_as400=`echo $lpar_os|sed 's/--AS400--$//'`
    #     if [ `echo $lpar_os|egrep -- "--AS400--$"|wc -l` -eq 1 -a -d "$lpar_no_as400" ]; then
    #       continue # it will be included in normal LPAR processing
    #     fi

    #     lpar_slash=`echo "$lpar_no_as400"|sed 's/\&\&1/\//g'`
    #     lpar_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "dashdashdashdash$lpar_slash"| sed 's/^dashdashdashdash//'` # dashdashdashdash is preventin if lpar name starts with a dash

    #     #find out alias in etc/alias.cfg
    #     lpar_menu=`find_out_alias "$lpar_slash" "LPAR"`

    #     if [ $DEBUG -eq 1 ]; then echo "add to menu $type_lpar  : $hmc:$managedname:$lpar_no_as400 : lpar_url:$lpar_url : OS agent"; fi
    #     menu "$type_lpar" "$hmc_space" "$managedname_print" "$lpar_url" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #     # wpar_search "$INPUTDIR"/data/"$managedname"/"$hmc"/"$lpar_os" "$lpar_no_nmon" "$lpar_url" "$hmc" "$managedname_print" "$hmc_url" "$managedname_url"
    #   done
    #   cd - >/dev/null
    #fi

    # vmware VMs, take only registered and having data  - 'start=' must be at the end of vmotion line
    #if [ -f "$INPUTDIR/data/$managedname/$hmc/VM_hosting.vmh" -a -f "$INPUTDIR/data/vmware_VMs/vm_uuid_name.txt" ]; then # vmware name instead of uuid
    #  for dir4 in `grep "start=[0-9]*$" $INPUTDIR/data/$managedname/$hmc/VM_hosting.vmh|cut -d":" -f1`
    #  do
    #    if [ -f "$INPUTDIR/data/vmware_VMs/$dir4.rrm" ]; then
    #       lpar_menu=`grep "$dir4" "$INPUTDIR/data/vmware_VMs/vm_uuid_name.txt"|cut -d"," -f2`

           #find out alias in etc/alias.cfg
    #      lpar_menu=`find_out_alias "$lpar_menu" "VM"`

    #     menu "$type_lpar" "$hmc_space" "$managedname_print" "$dir4" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$dir4&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #   fi
    # done
    #  continue
    #fi

    # exclude LPARs removed 7 days ago and more
    # do not exlude it, use cpu.cfg to check what is actual and what is not
    # NMON does not have to be exlcuded here as lpars are searched only per .rrm
    #for dir3 in `find "$INPUTDIR"/data/"$managedname"/"$hmc" |egrep ".$RR$"|sed 's/ /\%20/g'|sort -f`
    #do
    #  #pregarz start space in lpar name
    #  ff=$(echo "$dir3"|tr '@' ' ')
    #  i=`basename $TWOHYPHENS "$ff" .$RR`

    #  if [ "$i" = "mem-pool" -o "$i" = "pool-multi" -o "$i" = "mem" -o "$i" = "pool" -o "$i" = "daily-view" -o "$i" = "weekly-view" -o "$i" = "monthly-view" -o "$i" = "yearly-view" ]; then
    #    continue
    #  fi

    #  # need to exclude shared pools but regex does not work in "if"
    #  echo "$i" | egrep "^SharedPool[0-9]*" >/dev/null 2>&1
    #  if [ $? -eq 0 ]; then
    #    continue
    #  fi
    #  lpar_dir=`dirname -- "$ff"`
    #  lpar_dir_space=`echo "$lpar_dir"|sed 's/\%20/ /g'`
    #  i_space=`echo "$i"|sed 's/\%20/ /g'`
    #  i_slash=`echo "$i_space"|sed 's/\&\&1/\//g'`

    #  #find out alias in etc/alias.cfg
    #  lpar_menu=`find_out_alias "$i_slash" "LPAR"`

    #  # show just active lpar (must be in cpu.cfg file)
    #  lpar_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "dashdashdashdash$i_slash"| sed 's/^dashdashdashdash//'` # dashdashdashdash is preventin if lpar name starts with a dash
    #  if [ -f "$INPUTDIR/data/$managedname/$hmc/cpu.cfg" ]; then
    #    egrep "^lpar_name=$i_slash,lpar_id=" "$INPUTDIR/data/$managedname/$hmc/cpu.cfg" >/dev/null
    #    if [ $? -eq 0 ]; then
    #      if [ $DEBUG -eq 1 ]; then echo "add to menu $type_lpar  : $hmc:$managedname:$i_slash : lpar_url:$lpar_url";
    #      fi
    #      menu "$type_lpar" "$hmc_space" "$managedname_print" "$lpar_url" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #      wpar_search "$lpar_dir_space/$i" "$i_slash" "$lpar_url" "$hmc" "$managedname_print" "$hmc_url" "$managedname_url"
    #    else
    #      # removed lpars (do not exist in cpu.cfg)
    #      if [ $DEBUG -eq 1 ]; then echo "add to menu $type_lpar_removed  : $hmc:$managedname:$i_slash : lpar_url:$lpar_url";
    #      fi
    #      menu "$type_lpar_removed" "$hmc_space" "$managedname_print" "$lpar_url" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #      wpar_search "$lpar_dir_space/$i" "$i_slash" "$lpar_url" "$hmc" "$managedname_print" "$hmc_url" "$managedname_url"
    #    fi
    #  else
    #       # nor cpu.cfg neither vmware.txt exists from some reason, ignore it and display L
    #    if [ $DEBUG -eq 1 ]; then echo "add to menu $type_lpar  : $hmc:$managedname:$i_slash : lpar_url:$lpar_url : no cpu.cfg (no vmware.txt)";
    #    fi
    #    menu "$type_lpar" "$hmc_space" "$managedname_print" "$lpar_url" "$lpar_menu" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #    wpar_search "$lpar_dir_space/$i" "$i_slash" "$lpar_url" "$hmc" "$managedname_print" "$hmc_url" "$managedname_url"
    #  fi
    #done

    #if [ `echo "$managedname"|egrep -- "--unknown$"|wc -l` -eq 0 -a `echo $hmc|egrep "^no_hmc$"|wc -l` -eq 0 ]; then
    #  # NMON & OS agent without *rrm file (OS agent does not usally need sit as it creates .rrh with 0 size automatically
    #  # for situation when server exists but has only dedicated CPU --> no pool.rr*, then it is not --unknown but nmon data exists
    #  cd "$INPUTDIR/data/$managedname/$hmc"
    #  for lpar_dir_tmp in `ls -d $INPUTDIR/data/"$managedname"/"$hmc"/*--NMON-- 2>/dev/null| sed 's/ /====space====/g'|sort -f`
    #  do
    #    lpar_dir_nmon=`echo "$lpar_dir_tmp" | sed 's/====space====/ /g'`
    #    if [ ! -d "$lpar_dir_nmon" ]; then
    #      continue
    #    fi
    #    lpar_dir=`echo "$lpar_dir_nmon" | sed 's/--NMON--$//'`
    #    if [ -f "$lpar_dir.rrm" -o -f "$lpar_dir.rrh" ]; then
    #      continue # it has been already worked out above
    #    fi
    #    lpar_name=`basename $TWOHYPHENS "$lpar_dir"`
    #    i_slash=`echo "$lpar_name"|sed 's/\&\&1/\//g'`
    #    lpar_url=`$PERL -e '$s=shift;$s=~s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;print "$s\n";' "dashdashdashdash$i_slash"| sed 's/^dashdashdashdash//'` # dashdashdashdash is preventin if lpar name starts with a dash
    #    menu "$type_lpar" "$hmc_space" "$managedname_print" "$lpar_url" "$i_slash" "/$CGI_DIR/detail.sh?host=$hmc_url&server=$managedname_url&lpar=$lpar_url&item=lpar&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
    #  done
    #  cd - >/dev/null
    #fi

    if [ $HEA -eq 1 ]; then
      ls $INPUTDIR/data/"$managedname"/*/hea-*port*.db >/dev/null 2>/dev/null    # workaround as 2.56 had a bug and created hea subdir even when no HEA presented
      if [ $? -eq 0 ]; then
        menu "$server_menu" "$hmc_space" "$managedname_space" "hea" "HEA" "/$CGI_DIR/detail.sh?host=$hmc_url_hash&server=$managedname_url_hash&lpar=cod&item=hea&entitle=$ENTITLEMENT_LESS&gui=1&none=none"
      fi
    fi

  done
done

# add all IBM Power LPARs (include X86 Linuxes, NMON, AS400)
DATE=`date`
echo "Power          : find_active_lpar.pl: $DATE starting"
`$PERL $INPUTDIR/bin/find_active_lpar.pl >> $MENU_OUT 2>>$ERRLOG`
DATE=`date`
if [ $? -eq 0 ]; then
  echo "Power          : find_active_lpar.pl: OK at $DATE "
else
  echo "Power          : find_active_lpar.pl failed with value $? at $DATE"
fi

# add all Hitachi if exist
# Hitachi is ok here because it does not have special menu file
if [ -d "$INPUTDIR/data/Hitachi" ]; then
  `$PERL $INPUTDIR/bin/find_active_lpar.pl "HITACHI" >> $MENU_OUT 2>>$ERRLOG`
  if [ $? -eq 0 ]; then
    echo "Hitachi        : find_active_lpar.pl OK"
  else
    echo "Hitachi        : find_active_lpar.pl failed with value $?"
  fi
fi

# add all HYPERV if exist
# HYPERV is ok here because it does not have special menu file
if [ -d "$INPUTDIR/data/windows" ]; then
  `$PERL $INPUTDIR/bin/find_active_lpar.pl "HYPERV" >> $MENU_OUT 2>>$ERRLOG`
  if [ $? -eq 0 ]; then
    echo "Hyper-V        : find_active_lpar.pl : OK"
  else
    echo "Hyper-V        : find_active_lpar.pl failed with value $?"
  fi
fi

# add all SOLARIS if exist
if [ -d "$INPUTDIR/data/Solaris" ]; then
  `$PERL $INPUTDIR/bin/find_active_lpar.pl "SOLARIS-L" >> $MENU_OUT 2>>$ERRLOG`
  if [ $? -eq 0 ]; then
    echo "Solaris        : find_active_lpar.pl - SOLARIS with LDOM - OK"
  else
    echo "Solaris        : find_active_lpar.pl failed with value $?"
  fi
elif [ -d "$INPUTDIR/data/Solaris--unknown" ]; then
  `$PERL $INPUTDIR/bin/find_active_lpar.pl "SOLARIS-L" >> $MENU_OUT 2>>$ERRLOG`
  if [ $? -eq 0 ]; then
    echo "Solaris        : find_active_lpar.pl - SOLARIS without LDOM - OK"
  else
    echo "Solaris        : find_active_lpar.pl failed with value $?"
  fi
fi
print_cfg > $WEBDIR/gui-cfg.html
print_log > $WEBDIR/gui-log.html

# must be here to let created all www/* structure
menu_hmc # it creates Global HMC menu --> it runs only once

menu "$type_tmenu" "doc" "Documentation" "gui-help.html"
menu "$type_tmenu" "dcheck" "Data check" "/$CGI_DIR/detail.sh?host=&server=&lpar=cod&item=data_check&entitle=0&none=none"
#menu "$type_tmenu" "lcfg" "Configuration list" "gui-cfg.html"
# excluded, it is not necessary anymore -PH
menu "$type_tmenu" "logs" "Logs" "gui-log.html"
if [ ! "$VI_IMAGE"x = "x" ]; then
  if [ $VI_IMAGE -eq 1 ]; then
    menu "$type_tmenu" "update" "Product upgrade" "/$CGI_DIR/upgrade.sh?cmd=form"
  fi
fi

favourite  # favourite load

# add custom groups
if [ -f "$INPUTDIR/etc/web_config/custom_groups.cfg" -o -f "$INPUTDIR/etc/custom_groups.cfg" ]; then
  `$PERL $INPUTDIR/bin/custom-genmenu.pl >> $MENU_OUT 2>>$ERRLOG`
  if [ $? -eq 0 ]; then
    echo "Custom         : custom-genmenu.pl: OK"
  else
    echo "Custom         : custom-genmenu.pl failed with value $?"
  fi
fi

menu "$type_tmenu" "ACL" "Access Control" "/$CGI_DIR/acl.sh"

#read

# copy new menu over the prod ove
if [ -f $MENU_OUT ]; then
  cp $MENU_OUT $MENU_OUT_FINAL
  # create total tmp/menu.txt which is used by heatmap, daily_check etc.
  cp $MENU_OUT $TMPDIR_LPAR/menu.txt
  if [ -f "$TMPDIR_LPAR/menu_vmware.txt" ]; then
    cat "$TMPDIR_LPAR/menu_vmware.txt" >> $TMPDIR_LPAR/menu.txt
  fi
elif [ -f $MENU_XENSERVER_JSON_OUT -o -f $MENU_OVIRT_JSON_OUT ]; then
  # new JSON menu: avoid dealing with menu.txt (MENU_OUT)
  :
else
  echo "menu does not exist: file $MENU_OUT"
fi

#  clean out NMON online grapher files
if [ -d "$INPUTDIR/data_all" ]; then
  # clean all temp files older than 2 days
  find /tmp -name ext-nmon-query-\* -mtime +2 -exec rm -f {} 2>/dev/null \;
  find "$INPUTDIR/data_all"  -mtime +2 -type l -exec rm -f {} 2>/dev/null \;
  find "$INPUTDIR/data_all"  -mtime +2 -type f -exec rm -f {} 2>/dev/null \;
  find "$INPUTDIR/data_all"  -mtime +2 -type d -exec rm -rf {} 2>/dev/null \;
fi

# Clean out <defunct> processe
if [ `ps -ef| grep lpar2rrd | grep -v grep | grep "<defunct>"| wc -l |sed 's/ //g'` -gt 0 ]; then
  echo "Defuncts       : killing defuncts"
  kill -9 `ps -ef| grep lpar2rrd | grep -v grep | grep "<defunct>" | awk '{print $2}' |xargs`
  # lpar2rrd user is hardcoded on purpose
fi

# back up GUI config files
cd $INPUTDIR/etc
if [ ! -d .web_config ]; then
  mkdir .web_config
fi
if [ ! -f .web_config/Readme.txt ]; then
  echo "here is backup of all modification in GUI configuration files done through the web" > .web_config/Readme.txt
fi

rm -f $TMPDIR_LPAR/$version_run

RET=`grep ":ERROR:" $MENU_OUT`
if [ `echo $RET| wc -w` -gt 1 ]; then
  echo "ERROR menu.txt : $RET"
fi
exit
