# ovirt-gencong.pl
# generate configuration from data/oVirt/conf.json file and save it in as html page

use 5.008_008;

use strict;
use warnings;

use JSON;
use Data::Dumper;

use Xorux_lib qw(error read_json wrtie_json file_time_diff);
use OVirtDataWrapper;
use OVirtMenu;

defined $ENV{INPUTDIR} || Xorux_lib::error( "INPUTDIR undefined, config etc/lpar2rrd.cfg probably has not been loaded " . __FILE__ . ":" . __LINE__ ) && exit 1;

my $upgrade        = defined $ENV{UPGRADE} ? $ENV{UPGRADE} : 0;
my $version        = "$ENV{version}";
my $inputdir       = $ENV{INPUTDIR};
my $main_data_dir  = "$inputdir/data/oVirt";
my $metadata_file  = "$main_data_dir/conf.json";
my $conf_dir       = "$main_data_dir/configuration";
my $touch_file     = "$inputdir/tmp/oVirt_genconf.touch";
my $run_touch_file = "$inputdir/tmp/$version-ovirt"; # for generating menu
my $generate_time  = localtime();

my $hosts_cfg_file           = "$conf_dir/hosts.html";
my $vms_cfg_file             = "$conf_dir/vms.html";
my $storage_domains_cfg_file = "$conf_dir/storage_domains.html";

my $host_rows = '';
my $vm_rows   = '';
my $sd_rows   = '';

my @host_header = ( 'Datacenter', 'Cluster', 'Host name', 'FQDN or IP', 'Memory [GB]',
                    'Swap [GB]', 'CPU model', 'Number of cores', 'Number of sockets', 'CPU speed [MHz]',
                    'OS', 'Kernel version', 'KVM version', 'Threads per core',
                    'Hardware product name', 'Hardware serial number' );
my @vm_header = ( 'Datacenter', 'Cluster', 'VM name', 'Type', 'Memory [GB]', 'CPU per socket',
                  'Number of sockets', 'OS' );
my @storage_domain_header = ( 'Datacenter', 'Storage domain name', 'Storage domain type', 'Storage type',
                              'Size [GB]', 'Used [GB]', 'Free [GB]' );

unless ( -d $conf_dir ) {
  mkdir( $conf_dir, 0755 ) || Xorux_lib::error( "Cannot mkdir $conf_dir: $!" . __FILE__ . ":" . __LINE__ );
}

if ( !-f $touch_file ) {
  `touch $touch_file`;
  `touch $run_touch_file`; # generate menu_ovirt.json
  print "ovirt-genconf.pl  : first run after install, " . localtime() . "\n";
}
else {
  my $run_time = ( stat( $touch_file ) )[9];
  my ( undef, undef, undef, $actual_day )   = localtime( time() );
  my ( undef, undef, undef, $last_run_day ) = localtime( $run_time );

  if ( $actual_day == $last_run_day && $upgrade == 0 ) {
    print "ovirt-genconf.pl  : already ran today, skip\n";
    exit( 0 );  # run just once a day
  }
  else {
    `touch $touch_file`;
    `touch $run_touch_file`; # generate menu_ovirt.json
    print "ovirt-genconf.pl  : generate configuration, " . localtime() . "\n";
  }
}

opendir( DH, $conf_dir ) || Xorux_lib::error("Could not open '$conf_dir' for reading '$!'\n") && exit;
my @files = grep /.*.json/, readdir DH;
closedir( DH );

foreach my $file ( sort @files ) {
  my ( $code, $ref );

  if ( -f "$conf_dir/$file" ) {
    ( $code, $ref ) = Xorux_lib::read_json( "$conf_dir/$file" );
  }

  if ( $code ) {
    my $data = $ref;

    eval {
      my $act_time = defined $data->{timestamp} ? $data->{timestamp}
        : Xorux_lib::error( "Missing timestamp on configuration $file " . __FILE__ . ":" . __LINE__ );

      gen_hosts_html( $data );
      gen_vms_html( $data );
      gen_storage_domains_html( $data );
    };
    if ( $@ ) {
      Xorux_lib::error( "Error while generating configuration file $file : $@ : " . __FILE__ . ":" . __LINE__ );
    }
  }
}

if ( $host_rows ) {
  open( HOSTH, '>', $hosts_cfg_file ) || Xorux_lib::error( "Couldn't open file $hosts_cfg_file $!" . __FILE__ . ":" . __LINE__ );
  print HOSTH ${ generate_table( \@host_header, \$host_rows ) };
  print HOSTH "<br>\n";
  print HOSTH "It is updated once a day, last run: " . $generate_time;
  close( HOSTH );
}

if ( $vm_rows ) {
  open( VMH, '>', $vms_cfg_file ) || Xorux_lib::error( "Couldn't open file $vms_cfg_file $!" . __FILE__ . ":" . __LINE__ );
  print VMH ${ generate_table( \@vm_header, \$vm_rows ) };
  print VMH "<br>\n";
  print VMH "It is updated once a day, last run: " . $generate_time;
  close( VMH );
}

if ( $sd_rows ) {
  open( SDH, '>', $storage_domains_cfg_file ) || Xorux_lib::error( "Couldn't open file $storage_domains_cfg_file $!" . __FILE__ . ":" . __LINE__ );
  print SDH ${ generate_table( \@storage_domain_header, \$sd_rows ) };
  print SDH "<br>\n";
  print SDH "It is updated once a day, last run: " . $generate_time;
  close( SDH );
}

print "ovirt-genconf.pl  : finish, " . localtime() . "\n";
exit 0;

################################################################################

sub gen_hosts_html {
  my $data            = shift;
  my $datacenter_name = defined $data->{datacenter}{datacenter_name} ? $data->{datacenter}{datacenter_name} : '';

  foreach my $host_uuid ( keys %{ $data->{host} } ) {
    my $host = $data->{host}{ $host_uuid };

    my $cluster_name      = defined $host->{cluster_name} ? $host->{cluster_name} : '';
    my $host_name         = defined $host->{host_name} ? $host->{host_name} : '';
    my $fqdn_or_ip        = defined $host->{fqdn_or_ip} ? $host->{fqdn_or_ip} : '';
    my $memory_size       = defined $host->{memory_size_mb}
      ? sprintf "%.0f", $host->{memory_size_mb} / 1024 : '';
    my $swap_size         = defined $host->{swap_size_mb} ? sprintf "%.0f", $host->{swap_size_mb} / 1024 : '';
    my $cpu_model         = defined $host->{cpu_model} ? $host->{cpu_model} : '';
    my $number_of_cores   = defined $host->{number_of_cores} ? $host->{number_of_cores} : '';
    my $number_of_sockets = defined $host->{number_of_sockets} ? $host->{number_of_sockets} : '';
    my $cpu_speed_mh      = defined $host->{cpu_speed_mh} ? $host->{cpu_speed_mh} : '';
    my $host_os           = defined $host->{host_os} ? $host->{host_os} : '';
    my $kernel_version    = defined $host->{kernel_version} ? $host->{kernel_version} : '';
    my $kvm_version       = defined $host->{kvm_version} ? $host->{kvm_version} : '';
    my $threads_per_core  = defined $host->{threads_per_core} ? $host->{threads_per_core} : '';
    my $hw_product_name   = defined $host->{hardware_product_name} ? $host->{hardware_product_name} : '';
    my $hw_serial_number  = defined $host->{hardware_serial_number} ? $host->{hardware_serial_number} : '';

    my $host_url  = OVirtMenu::get_url( { type => 'host', host => $host_uuid } );
    my $host_link = "<a href=\"$host_url\"><b>$host_name</b></a>";

    $host_rows .= ${ generate_row( $datacenter_name, $cluster_name, $host_link, $fqdn_or_ip,
                                   $memory_size, $swap_size, $cpu_model, $number_of_cores,
                                   $number_of_sockets, $cpu_speed_mh, $host_os, $kernel_version, $kvm_version,
                                   $threads_per_core, $hw_product_name, $hw_serial_number ) };
  }

  return 1;
} ## end sub gen_hosts_html

sub gen_vms_html {
  my $data            = shift;
  my $datacenter_name = defined $data->{datacenter}{datacenter_name} ? $data->{datacenter}{datacenter_name} : '';

  foreach my $vm_uuid ( keys %{ $data->{vm} } ) {
    my $vm = $data->{vm}{ $vm_uuid };

    my $cluster_name      = defined $vm->{cluster_name} ? $vm->{cluster_name} : '';
    my $vm_name           = defined $vm->{vm_name} ? $vm->{vm_name} : '';
    my $vm_type           = defined $vm->{vm_type} ? $vm->{vm_type} : '';
    my $memory_size       = defined $vm->{memory_size_mb} ? sprintf "%.0f", $vm->{memory_size_mb} / 1024 : '';
    my $cpu_per_socket    = defined $vm->{cpu_per_socket} ? $vm->{cpu_per_socket} : '';
    my $number_of_sockets = defined $vm->{number_of_sockets} ? $vm->{number_of_sockets} : '';
    my $operating_system  = defined $vm->{operating_system} ? $vm->{operating_system} : '';

    my $vm_url  = OVirtMenu::get_url( { type => 'vm', vm => $vm_uuid } );
    my $vm_link = "<a href=\"$vm_url\"><b>$vm_name</b></a>";


    $vm_rows .= ${ generate_row( $datacenter_name, $cluster_name, $vm_link, $vm_type, $memory_size,
                                 $cpu_per_socket, $number_of_sockets, $operating_system ) };
  }

  return 1;
} ## end sub gen_vms_html

sub gen_storage_domains_html {
  my $data            = shift;
  my $datacenter_name = defined $data->{datacenter}{datacenter_name} ? $data->{datacenter}{datacenter_name} : '';

  foreach my $sd_uuid ( keys %{ $data->{storage_domain} } ) {
    my $sd = $data->{storage_domain}{ $sd_uuid };

    my $storage_domain_name = defined $sd->{storage_domain_name} ? $sd->{storage_domain_name} : '';
    my $storage_domain_type = defined $sd->{storage_domain_type} ? $sd->{storage_domain_type} : '';
    my $storage_type        = defined $sd->{storage_type} ? $sd->{storage_type} : '';
    my $total_disk_size_gb  = defined $sd->{total_disk_size_gb} ? $sd->{total_disk_size_gb} : '';
    my $used_disk_size_gb   = defined $sd->{used_disk_size_gb} ? $sd->{used_disk_size_gb} : '';
    my $free_disk_size_gb   = defined $sd->{free_disk_size_gb} ? $sd->{free_disk_size_gb} : '';

    my $storage_domain_url  = OVirtMenu::get_url( { type => 'storage_domain', storage_domain => $sd_uuid } );
    my $storage_domain_link = "<a href=\"$storage_domain_url\"><b>$storage_domain_name</b></a>";

    $sd_rows .= ${ generate_row( $datacenter_name, $storage_domain_link, $storage_domain_type, $storage_type,
                                 $total_disk_size_gb, $used_disk_size_gb, $free_disk_size_gb ) };
  }

  return 1;
} ## end sub gen_storage_domains_html

################################################################################

sub generate_row {
  my @values = @_;
  my $rows   = '';

  $rows .= "<tr>\n";
  foreach my $value ( @values ) {
    $value = '' unless defined $value;
    $rows .=  "<td style=\"text-align:left; color:black;\" nowrap=\"\">$value</td>\n";
  }
  $rows .=  "</tr>\n";

  return \$rows;
} ## end sub generate_row

sub generate_table {
  my @header   = @{ shift @_ };
  my $rows     = ${ shift @_ };
  my $headline = shift;
  my $acc      = "";

  if ( $rows eq '' ) {
    return \$acc;
  }

  $acc .= "<center>\n";
  $acc .= defined $headline ? "<br><br><b>$headline:</b>\n" : "<br>\n";
  $acc .= "<table class =\"tabconfig tablesorter tablesorter-ice tablesorter7f0583dfe15c8 hasFilters\">\n";
  $acc .= "<thead>\n";
  $acc .= "<tr>\n";

  foreach my $column ( @header ) {
    $acc .= "<th class = \"sortable\" style=\"text-align:center; color:black;\" nowrap=\"\">$column</th>\n";
  }

  $acc .= "</tr>\n";
  $acc .= "</thead>\n";
  $acc .= "<tbody>\n";
  $acc .= $rows;
  $acc .= "</tbody>\n";
  $acc .= "</table>\n";
  $acc .= "</center>\n";

  return \$acc;
} ## end sub generate_table
