#!/usr/bin/perl

use lib "../bin";

use strict;
use warnings;
use Time::Local;
use Data::Dumper;
use POSIX qw(mktime strftime);
use JSON qw(encode_json decode_json);
use MIME::Base64 qw(encode_base64 decode_base64);

my $basedir      = $ENV{INPUTDIR};
my $wrkdir       = "$basedir/data";
my $tmpdir       = "$basedir/tmp";
my $bindir       = "$basedir/bin";
my $reportsdir   = "$basedir/reports";
my $reporter_cfg = "$basedir/etc/web_config/reporter.json";

$ENV{'REQUEST_METHOD'} = "GET";
my $perl  = $ENV{'PERL'}           ||= "perl";
my $debug = $ENV{'DEBUG_REPORTER'} ||= 9;
umask 0000;

my $act_time2debug = localtime();
print "reporter START : $act_time2debug\n" if $debug == 9;

if ( ! -f $reporter_cfg ) {
  print "reporter       : $reporter_cfg does not exists! Exiting...\n" if $debug == 9;
  exit 1;
}
if ( ! -d $reportsdir ) {
  mkdir( "$reportsdir", 0777 ) || error( "Cannot mkdir $reportsdir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
}

my ($user_name, $report_name, $startup_from) = @ARGV;

#
# Configuration
#
print "load cfg       : $reporter_cfg\n" if $debug == 9;
open( CFG, "<$reporter_cfg" ) || error( "Couldn't open file $reporter_cfg $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
my %cfg = %{ decode_json(join("", <CFG>)) };
close(CFG);

#
# Global variables
#
my $product    = "LPAR2RRD";
my $product_lc = "";
if ( defined $ENV{WLABEL} && $ENV{WLABEL} ne 'Dark' ) { $product = $ENV{WLABEL}; }
$product_lc = lc $product;

print "product        : $product\n" if $debug == 9;
my $uname_n    = `uname -n`;
chomp $uname_n;
my $mail_from_default = "$product_lc-reporter\@$uname_n.com";

#
# find all lpars
#
my %power_inventory;
get_power_inventory();

#
# find all VMs
#
my %vmware_inventory;
get_vmware_inventory();

#
# load oVirt structure
#
my %ovirt_inventory;
get_ovirt_inventory();

#
# load Solaris structure
#
my %solaris_inventory;
get_solaris_inventory();

#
# Progress bar variables
#
my %status;
my $pid     = $$;  # this process ID
my $done    = 0;
my $count   = 0;
my $API_RUN = 0;

#
# alias.cfg
#
my %aliases;
#my $alias_cfg_file = "$basedir/etc/alias.cfg";
#my @alias_cfg;
#if ( -f $alias_cfg_file ) {
#  open( CFG, "<$alias_cfg_file" ) || error( "Couldn't open file $alias_cfg_file $!" . __FILE__ . ":" . __LINE__ ) && exit;
#  my @alias_cfg = <CFG>;
#  close(CFG);
#
#  foreach my $alias_line (@alias_cfg) {
#    chomp $alias_line;
#    if ( $alias_line =~ m/^#/ ) { next; }
#    if ( $alias_line =~ m/^$/ ) { next; }
#
#    my ($al_type, $al_host, $al_id, $alias) = split(/:/, $alias_line);
#    if ( defined $al_type && $al_type ne '' && defined $al_host && $al_host ne '' && defined $al_id && $al_id ne '' && defined $alias && $alias ne '' ) {
#      if ( $al_type eq "SANPORT" ) {
#        $al_id =~ s/^port//; # just to be sure, bcs there can be configured port id like "port1"
#      }
#      $aliases{$al_type}{$al_host}{$alias}{'ID'} = $al_id;
#    }
#  }
#}


#
# Reports
#
if ( defined $report_name && $report_name ne '' && defined $user_name && $user_name ne '' ) {
  print "user           : $user_name\n" if $debug == 9;
  print "report name    : $report_name\n" if $debug == 9;

  # if the reporter is running from the GUI, there have to be the following var, because vars reporter_name and user_name are required also for manual startup by wrapper
  # status file (required for progress bar) is not created, when reporter is not running from the GUI
  if ( defined $startup_from && $startup_from eq "RUNFROMGUI" ) {
    $API_RUN = 1; # reporter is executed from the GUI
  }


  #
  # Report main variables
  #
  my $format   = "";
  my $sunix    = "";
  my $eunix    = "";
  my $sdate    = "";
  my $edate    = "";
  my $time     = "";

  # format IMG/PDF/CSV
  if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'} && ($cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'} eq "IMG" || $cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'} eq "PDF" || $cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'} eq "CSV" ) ) {
    $format = $cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'};
  }
  else {
    error( "Report : $report_name : Unknown format of output file! Exiting... ") && exit 1;
  }

  # report time range
  if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'mode'} && $cfg{'users'}{$user_name}{'reports'}{$report_name}{'mode'} eq "timerange" ) {
    if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'sunix'} && isdigit($cfg{'users'}{$user_name}{'reports'}{$report_name}{'sunix'}) && exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'eunix'} && isdigit($cfg{'users'}{$user_name}{'reports'}{$report_name}{'eunix'}) ) {
      $sunix = $cfg{'users'}{$user_name}{'reports'}{$report_name}{'sunix'};
      $eunix = $cfg{'users'}{$user_name}{'reports'}{$report_name}{'eunix'};

      if ( defined $sunix && isdigit($sunix) && $sunix > 0 && defined $eunix && isdigit($eunix) && $eunix > 0 ) {
        my ( $sec_s, $min_s, $hour_s, $day_s, $month_s, $year_s, $wday_s, $yday_s, $isdst_s ) = localtime($sunix);
        $sdate = sprintf( "%02d:%02d:%02d %02d.%02d.%4d", $hour_s, $min_s, $sec_s, $day_s, $month_s + 1, $year_s + 1900 );

        my ( $sec_e, $min_e, $hour_e, $day_e, $month_e, $year_e, $wday_e, $yday_e, $isdst_e ) = localtime($eunix);
        $edate = sprintf( "%02d:%02d:%02d %02d.%02d.%4d", $hour_e, $min_e, $sec_e, $day_e, $month_e + 1, $year_e + 1900 );
      }
      else {
        error( "Report : $report_name : Something is wrong with setting time range! sunix=\"$sunix\", eunix=\"$eunix\"! Exiting... ") && exit 1;
      }
      $time = "a";
    }
    else {
      error( "Report : $report_name : Unknown time range! sunix=\"$cfg{'users'}{$user_name}{'reports'}{$report_name}{'sunix'}\", eunix=\"$cfg{'users'}{$user_name}{'reports'}{$report_name}{'eunix'}\"! Exiting... ") && exit 1;
    }
  }
  else { # recurrence
    if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} && $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} ne '' && exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'range'} && $cfg{'users'}{$user_name}{'reports'}{$report_name}{'range'} ne '' ) {
      ($sunix, $eunix) = set_report_time_range("$cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'}","$cfg{'users'}{$user_name}{'reports'}{$report_name}{'range'}");

      if ( defined $sunix && isdigit($sunix) && $sunix > 0 && defined $eunix && isdigit($eunix) && $eunix > 0 ) {
        my ( $sec_s, $min_s, $hour_s, $day_s, $month_s, $year_s, $wday_s, $yday_s, $isdst_s ) = localtime($sunix);
        $sdate = sprintf( "%02d:%02d:%02d %02d.%02d.%4d", $hour_s, $min_s, $sec_s, $day_s, $month_s + 1, $year_s + 1900 );

        my ( $sec_e, $min_e, $hour_e, $day_e, $month_e, $year_e, $wday_e, $yday_e, $isdst_e ) = localtime($eunix);
        $edate = sprintf( "%02d:%02d:%02d %02d.%02d.%4d", $hour_e, $min_e, $sec_e, $day_e, $month_e + 1, $year_e + 1900 );
      }
      else {
        error( "Report : $report_name : Something is wrong with setting time range! freq=\"$cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'}\",range=\"$cfg{'users'}{$user_name}{'reports'}{$report_name}{'range'}\"! Exiting... ") && exit 1;
      }

      if ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} eq "daily" ) {
        $time = "d";
      }
      elsif ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} eq "weekly" ) {
        $time = "w";
      }
      elsif ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} eq "monthly" ) {
        $time = "m";
      }
      elsif ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} eq "yearly" ) {
        $time = "y";
      }
      else {
        error( "Report : $report_name : Unknown freq value! There should be daily/weekly/monthly/yearly! freq=\"$cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'}\"! Exiting... ") && exit 1;
      }
      print "time range     : $cfg{'users'}{$user_name}{'reports'}{$report_name}{'freq'} $cfg{'users'}{$user_name}{'reports'}{$report_name}{'range'}\n" if $debug == 9;
    }
  }

  print "format         : $format\n" if $debug == 9;
  print "start time     : $sunix, $sdate\n" if $debug == 9;
  print "end time       : $eunix, $edate\n" if $debug == 9;


  #
  # Make report
  #
  if ( $format eq "IMG" && $product eq "LPAR2RRD" ) {
    make_img_report_lpar("$report_name","$user_name","$sunix","$eunix","$time");
  }
  if ( $format eq "PDF" && $product eq "LPAR2RRD" ) {
    make_pdf_report_lpar("$report_name","$user_name","$sunix","$eunix","$sdate","$edate","$time");
  }
  if ( $format eq "CSV" && $product eq "LPAR2RRD" ) {
    make_csv_report_lpar("$report_name","$user_name","$sunix","$eunix","$sdate","$edate","$time");
  }
}
else {
  # only first run after the midnight
  my $reporter_run = "$reportsdir/reporter-run";
  if ( !-f $reporter_run ) {
    `touch $reporter_run`;
  }
  else {
    my $run_time = ( stat("$reporter_run") )[9];
    ( my $sec, my $min, my $h, my $aday, my $m, my $y, my $wday, my $yday, my $isdst ) = localtime( time() );
    ( $sec, $min, $h, my $png_day, $m, $y, $wday, $yday, $isdst ) = localtime($run_time);
    if ( $aday == $png_day ) {
      print "reporter       : not this time $aday == $png_day\n";
      print "reporter END   : $act_time2debug\n" if $debug == 9;
      exit(0);
    }
    else {
      `touch $reporter_run`;
    }
  }

  if ( -f "$bindir/premium.pl" ) {
    require "$bindir/premium.pl";
  }
  else {
    require "$bindir/standard.pl";
  }

  set_reports( $product, $debug, \%cfg );
}


$act_time2debug = localtime();
print "reporter END   : $act_time2debug\n" if $debug == 9;

# progress bar end
%status = (status => "done", count => $count, done => $done);
write_file( "/tmp/lrep-$pid.status", encode_json( \%status ) );

exit 0;

sub get_power_inventory {
  my $menu_txt = "$tmpdir/menu.txt";
  if ( -f $menu_txt ) {
    open( MENU, "<$menu_txt" ) || error( "Couldn't open file $menu_txt $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    my @menu = <MENU>;
    chomp @menu;
    close(MENU);

    my @list_of_lpars = grep {/^L:.*:.*:.*:.*:.*:.*:P:.*$/} @menu; # "P" -> only POWER section
    # L:hmc:P02DR__9117-MMC-SN44K8102:p770-demo:p770-demo:/lpar2rrd-cgi/detail.sh?host=hmc&server=P02DR__9117-MMC-SN44K8102&lpar=p770-demo&item=lpar&entitle=0&gui=1&none=none:::P:A
    foreach my $line (@list_of_lpars) {
      my (undef, $hmc, $server, $lpar, undef, undef, undef, undef, undef, $lpar_type ) = split(/:/, $line);

      if ( $hmc eq "no_hmc" ) { next; } # skip lpars from unmanaged linux systems
      # test if server is not deleted
      # D:hmc:p710 space test:CPUpool-pool:CPU pool:/lpar2rrd-cgi/detail.sh?host=hmc&server=p710%20space%20test&lpar=pool&item=pool&entitle=0&gui=1&none=none::1522309746:P
      my $deleted_num = grep {/^D:$hmc:$server:.*:.*:.*:.*:P$/} @menu;
      if ( $deleted_num && isdigit($deleted_num) && $deleted_num > 0 ) { next; }

      $hmc    = urldecodel("$hmc");
      $server = urldecodel("$server");
      $lpar   = urldecodel("$lpar");

      $hmc    =~ s/===double-col===/:/g;
      $server =~ s/===double-col===/:/g;
      $lpar   =~ s/===double-col===/:/g;

      $power_inventory{LPARS}{$lpar}{MENU_TYPE}                   = $lpar_type;
      $power_inventory{LPARS}{$lpar}{SERVER}                      = $server;
      $power_inventory{LPARS}{$lpar}{HMC}                         = $hmc;

      $power_inventory{SERVERS}{$server}{LPARS}{$lpar}{MENU_TYPE} = $lpar_type;
      $power_inventory{SERVERS}{$server}{LPARS}{$lpar}{HMC}       = $hmc;
    }

    my @list_of_pools = grep {/^S:.*:.*:.*:.*:.*:.*:.*:P$/} @menu; # "P" -> only POWER section
    # S:hmc:p710 space test:CPUpool-pool:CPU pool:/lpar2rrd-cgi/detail.sh?host=hmc&server=p710%20space%20test&lpar=pool&item=pool&entitle=0&gui=1&none=none::1543187114:P
    # S:hmc:p710 space test:CPUpool-SharedPool0:CPU pool 0 ===double-col=== DefaultPool:/lpar2rrd-cgi/detail.sh?host=hmc&server=p710%20space%20test&lpar=SharedPool0&item=shpool&entitle=0&gui=1&none=none:::P
    foreach my $line (@list_of_pools) {
      my (undef, $hmc, $server, $pool_id, $pool_name, undef, undef, undef, undef ) = split(/:/, $line);

      if ( $pool_id !~ m/^CPUpool-/ ) { next; }

      $hmc       = urldecodel("$hmc");
      $server    = urldecodel("$server");
      $pool_id   = urldecodel("$pool_id");
      $pool_name = urldecodel("$pool_name");

      $hmc       =~ s/===double-col===/:/g;
      $server    =~ s/===double-col===/:/g;
      $pool_id   =~ s/===double-col===/:/g;
      $pool_id   =~ s/^CPUpool-//;
      $pool_name =~ s/===double-col===/:/g;

      if ( $pool_name =~ m/\s+:\s+/ ) { (undef, $pool_name) = split(/\s+:\s+/, $pool_name); }

      $power_inventory{SERVERS}{$server}{POOLS}{$pool_name}{POOL_ID} = $pool_id;
      $power_inventory{SERVERS}{$server}{POOLS}{$pool_name}{HMC}     = $hmc;
    }
  }

  return 1;
}

sub get_solaris_inventory {
  my $menu_txt = "$tmpdir/menu.txt";
  if ( -f $menu_txt ) {
    open( MENU, "<$menu_txt" ) || error( "Couldn't open file $menu_txt $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    my @menu = <MENU>;
    chomp @menu;
    close(MENU);

    # LDOMs
    my @list_of_ldoms = grep {/^L:no_hmc:.*:.*:.*:.*:.*:.*:S:L$/} @menu;
    # L:no_hmc:t8-1mvbc-l5:t8-1mvbc-l5:t8-1mvbc-l5:/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=t8-1mvbc-l5&item=sol-ldom&entitle=0&gui=1&none=none:::S:L
    foreach my $line (@list_of_ldoms) {
      my (undef, undef, $ldom, undef, undef, undef, undef, undef, undef, undef ) = split(/:/, $line);

      $ldom = urldecodel("$ldom");
      $ldom =~ s/===double-col===/:/g;

      $solaris_inventory{LDOM}{$ldom}{NAME} = $ldom;
    }

    # Global zones
    my @list_of_gz = grep {/^L:no_hmc:.*:.*:.*:.*:.*:.*:S:G$/} @menu;
    # L:no_hmc:kvm-solaris:kvm-solaris:kvm-solaris:/lpar2rrd-cgi/detail.sh?host=0&server=Solaris--unknown&lpar=kvm-solaris&item=sol-ldom&entitle=0&gui=1&none=none:::S:G
    foreach my $line (@list_of_gz) {
      my (undef, undef, $gz, undef, undef, undef, undef, undef, undef, undef ) = split(/:/, $line);

      $gz = urldecodel("$gz");
      $gz =~ s/===double-col===/:/g;

      $solaris_inventory{GLOBAL_ZONE}{$gz}{NAME} = $gz;
    }

    # Zones
    my @list_of_z = grep {/^L:no_hmc:.*:.*:.*:.*:.*:.*:S:Z$/} @menu;
    # L:no_hmc:kvm-solaris:test_z2:test_z2:/lpar2rrd-cgi/detail.sh?host=Solaris&server=kvm-solaris&lpar=test_z2&item=sol11-test_z2&entitle=0&gui=1&none=none:::S:Z
    foreach my $line (@list_of_z) {
      my (undef, undef, $host, $zone, undef, undef, undef, undef, undef, undef ) = split(/:/, $line);

      $host = urldecodel("$host");
      $zone = urldecodel("$zone");
      $host =~ s/===double-col===/:/g;
      $zone =~ s/===double-col===/:/g;

      $solaris_inventory{ZONE}{$zone}{NAME} = $zone;
      $solaris_inventory{ZONE}{$zone}{HOST} = $host;
    }
  }

  return 1;
}

sub get_vmware_inventory {
  my $menu_txt = "$tmpdir/menu.txt";
  if ( -f $menu_txt ) {
    open( MENU, "<$menu_txt" ) || error( "Couldn't open file $menu_txt $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    my @menu = <MENU>;
    chomp @menu;
    close(MENU);

    #
    # Vcenter
    #
    my @list_of_vcenters = grep {/^V:.*:Totals:.*:.*:.*:.*:.*:V$/} @menu;
    # V:10.22.11.10:Totals:/lpar2rrd-cgi/detail.sh?host=10.22.11.10&server=vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28&lpar=nope&item=hmctotals&entitle=0&gui=1&none=none:::Hosting::V
    foreach my $line (@list_of_vcenters) {
      my (undef, $vc_ip, undef, $url, undef, undef, $vcenter, undef ) = split(/:/, $line);

      $vc_ip   =~ s/===double-col===/:/g;
      $url     =~ s/===double-col===/:/g;
      $vcenter =~ s/===double-col===/:/g;

      $vmware_inventory{VCENTER}{$vcenter}{VCENTER_IP} = $vc_ip;
    }

    #
    # CLUSTER
    #
    my @list_of_cl = grep {/^A:.*:cluster_.*:Cluster totals:.*:.*:.*:.*:V$/} @menu; # "V" -> only VMware section
    # A:10.22.11.10:cluster_New Cluster:Cluster totals:/lpar2rrd-cgi/detail.sh?host=cluster_domain-c87&server=vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28&lpar=nope&item=cluster&entitle=0&gui=1&none=none::Hosting::V
    foreach my $line (@list_of_cl) {
      my (undef, undef, $cluster, undef, $detail_url, undef, $vcenter, undef ) = split(/:/, $line);

      $cluster    =~ s/^cluster_//;
      $cluster    =~ s/===double-col===/:/g;
      $vcenter    =~ s/===double-col===/:/g;
      $detail_url =~ s/===double-col===/:/g;

      # get url params
      $detail_url =~ s/^.+detail\.sh\?//;
      my @params  = split(/&/, $detail_url);
      foreach my $line (@params) {
        if ( $line =~ m/=/) {
          my ($param, $value) = split(/=/, $line);
          if ( defined $param && defined $value ) {
            $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{URL_PARAM}{$param} = $value;
          }
        }
      }
    }

    #
    # VM
    #
    #my @list_of_vms = grep {/^L:.*:.*:.*:.*:.*:.*:.*:V$/} @menu; # "V" -> only VMware section
    my @list_of_vms = grep {/^L:.*:.*:.*:.*:.*:.*:.*:V/} @menu; # "V" -> only VMware section
    # L:cluster_New Cluster:10.22.11.9:501c1a53-cf7d-07cb-88e4-cf94ca6c5b0e:vm-karel:/lpar2rrd-cgi/detail.sh?host=10.22.11.10&server=10.22.11.9&lpar=501c1a53-cf7d-07cb-88e4-cf94ca6c5b0e&item=lpar&entitle=0&gui=1&none=none::Hosting:V
    foreach my $line (@list_of_vms) {
      my (undef, $cluster, $server, $vm_uuid, $vm_name, $detail_url, undef, $vcenter, undef ) = split(/:/, $line);

      $cluster    =~ s/^cluster_//;
      $cluster    =~ s/===double-col===/:/g;
      $server     =~ s/===double-col===/:/g;
      $vm_uuid    =~ s/===double-col===/:/g;
      $vm_name    =~ s/===double-col===/:/g;
      $vcenter    =~ s/===double-col===/:/g;
      $detail_url =~ s/===double-col===/:/g;

      $vmware_inventory{VM}{$vm_uuid}{VCENTER} = $vcenter;
      $vmware_inventory{VM}{$vm_uuid}{CLUSTER} = $cluster;
      $vmware_inventory{VM}{$vm_uuid}{SERVER}  = $server;
      $vmware_inventory{VM}{$vm_uuid}{VM_NAME} = $vm_name;

      $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{VM}{$vm_uuid}{VM_NAME} = $vm_name;
      $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{VM}{$vm_uuid}{SERVER}  = $server;

      # get url params
      $detail_url =~ s/^.+detail\.sh\?//;
      my @params  = split(/&/, $detail_url);
      foreach my $line (@params) {
        if ( $line =~ m/=/) {
          my ($param, $value) = split(/=/, $line);
          if ( defined $param && defined $value ) {
            $vmware_inventory{VM}{$vm_uuid}{URL_PARAM}{$param} = $value;
            $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{VM}{$vm_uuid}{URL_PARAM}{$param} = $value;
          }
        }
      }
    }

    #
    # ESXi
    #
    my @list_of_esxi = grep {/^S:cluster_.*:.*:CPUpool-pool:CPU pool:.*:.*:.*:V$/} @menu;
    # S:cluster_New Cluster:10.22.11.9:CPUpool-pool:CPU pool:/lpar2rrd-cgi/detail.sh?host=10.22.11.10&server=10.22.11.9&lpar=pool&item=pool&entitle=0&gui=1&none=none::1548025200:V
    foreach my $line (@list_of_esxi) {
      my (undef, $cluster, $esxi, undef, undef, $url, undef ) = split(/:/, $line);

      $cluster    =~ s/^cluster_//;
      $cluster    =~ s/===double-col===/:/g;
      $url        =~ s/===double-col===/:/g;
      $esxi       =~ s/===double-col===/:/g;
      my $vcenter = "";

      # get url params
      $url =~ s/^.+detail\.sh\?//;
      my @params  = split(/&/, $url);
      foreach my $line (@params) {
        if ( $line =~ m/=/) {
          my ($param, $value) = split(/=/, $line);
          if ( defined $param && $param eq "host" && defined $value ) { # Vcenter IP
            foreach my $vc ( sort keys %{ $vmware_inventory{VCENTER} } ) {
              if ( exists $vmware_inventory{VCENTER}{$vc}{VCENTER_IP} && $vmware_inventory{VCENTER}{$vc}{VCENTER_IP} eq $value ) {
                $vcenter = $vc;
              }
            }
            last;
          }
        }
      }
      if ( defined $vcenter && $vcenter ne '' ) {
        $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{ESXI}{$esxi}{NAME} = $esxi;
      }
    }

    #
    # RESPOOL
    #
    my @list_of_rp = grep {/^B:.*:cluster_.*:.*:.*:.*:.*:.*:V$/} @menu;
    # B:10.22.11.10:cluster_New Cluster:Development:/lpar2rrd-cgi/detail.sh?host=cluster_domain-c87&server=vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28&lpar=resgroup-139&item=resourcepool&entitle=0&gui=1&none=none::Hosting::V
    foreach my $line (@list_of_rp) {
      my (undef, undef, $cluster, $rp, $url, undef, $vcenter, undef, undef ) = split(/:/, $line);

      $cluster =~ s/^cluster_//;
      $cluster =~ s/===double-col===/:/g;
      $url     =~ s/===double-col===/:/g;
      $rp      =~ s/===double-col===/:/g;
      $vcenter =~ s/===double-col===/:/g;

      # get url params
      $url =~ s/^.+detail\.sh\?//;
      my @params  = split(/&/, $url);
      foreach my $line (@params) {
        if ( $line =~ m/=/) {
          my ($param, $value) = split(/=/, $line);
          if ( defined $param && defined $value ) {
            $vmware_inventory{VCENTER}{$vcenter}{CLUSTER}{$cluster}{RESPOOL}{$rp}{URL_PARAM}{$param} = $value;
          }
        }
      }
    }

    #
    # DATASTORE
    #
    my @list_of_ds = grep {/^Z:.*:datastore_.*:.*:.*:.*:.*:.*:V$/} @menu;
    # Z:10.22.11.10:datastore_DC:3PAR-phys-xorux-test:/lpar2rrd-cgi/detail.sh?host=datastore_datacenter-2&server=vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28&lpar=590e2b41-3f75d5e4-3f85-18a90577a87c&item=datastore&entitle=0&gui=1&none=none::Hosting::V
    foreach my $line (@list_of_ds) {
      my (undef, undef, $dc, $ds, $url, undef, $vcenter, undef, undef ) = split(/:/, $line);

      $dc      =~ s/^datastore_//;
      $dc      =~ s/===double-col===/:/g;
      $ds      =~ s/===double-col===/:/g;
      $url     =~ s/===double-col===/:/g;
      $vcenter =~ s/===double-col===/:/g;
      $url     =~ s/^.+detail\.sh\?//;

      $vmware_inventory{VCENTER}{$vcenter}{DATASTORE}{$ds}{DATACENTER} = $dc;
      $vmware_inventory{VCENTER}{$vcenter}{DATASTORE}{$ds}{URL}        = $url;

      # get url params
      my @params  = split(/&/, $url);
      foreach my $line (@params) {
        if ( $line =~ m/=/) {
          my ($param, $value) = split(/=/, $line);
          if ( defined $param && defined $value ) {
            $vmware_inventory{VCENTER}{$vcenter}{DATASTORE}{$ds}{URL_PARAM}{$param} = $value;
          }
        }
      }
    }


  }

  return 1;
}

sub get_ovirt_inventory {
  if ( -f "$wrkdir/oVirt/metadata.json" ) {
    use OVirtDataWrapper;

    # Datacenters
    foreach my $datacenter_uuid ( @{ OVirtDataWrapper::get_uuids( 'datacenter' ) } ) {
      my $datacenter_label  = OVirtDataWrapper::get_label( 'datacenter', $datacenter_uuid );
      $ovirt_inventory{DATACENTER}{$datacenter_uuid}{label} = $datacenter_label;
      $ovirt_inventory{TRANSLATE_UUID}{$datacenter_uuid} = $datacenter_label;

      # Clusters
      foreach my $cluster_uuid ( @{ OVirtDataWrapper::get_arch( $datacenter_uuid, 'datacenter', 'cluster' ) } ) {
        my $cluster_label  = OVirtDataWrapper::get_label( 'cluster', $cluster_uuid );
        $ovirt_inventory{DATACENTER}{$datacenter_uuid}{CLUSTER}{$cluster_uuid}{label} = $cluster_label;
        $ovirt_inventory{TRANSLATE_UUID}{$cluster_uuid} = $cluster_label;

        # Hosts
        foreach my $host_uuid ( @{ OVirtDataWrapper::get_arch( $cluster_uuid, 'cluster', 'host' ) } ) {
          my $host_label    = OVirtDataWrapper::get_label( 'host', $host_uuid );
          $ovirt_inventory{DATACENTER}{$datacenter_uuid}{CLUSTER}{$cluster_uuid}{HOST}{$host_uuid}{label} = $host_label;
          $ovirt_inventory{TRANSLATE_UUID}{$host_uuid} = $host_label;

          # LANs
          foreach my $nic_uuid ( @{ OVirtDataWrapper::get_arch( $host_uuid, 'host', 'nic' ) } ) {
            my $nic_label     = OVirtDataWrapper::get_label( 'host_nic', $nic_uuid );
            $ovirt_inventory{DATACENTER}{$datacenter_uuid}{CLUSTER}{$cluster_uuid}{HOST}{$host_uuid}{LAN}{$nic_uuid}{label} = $nic_label;
            $ovirt_inventory{TRANSLATE_UUID}{$nic_uuid} = $nic_label;
          }
        }

        # VMs
        foreach my $vm_uuid ( @{ OVirtDataWrapper::get_arch( $cluster_uuid, 'cluster', 'vm' ) } ) {
          my $vm_label      = OVirtDataWrapper::get_label( 'vm', $vm_uuid );
          $ovirt_inventory{DATACENTER}{$datacenter_uuid}{CLUSTER}{$cluster_uuid}{VM}{$vm_uuid}{label} = $vm_label;
          $ovirt_inventory{TRANSLATE_UUID}{$vm_uuid} = $vm_label;
        }
      }

      # Storage domain
      foreach my $domain_uuid ( @{ OVirtDataWrapper::get_arch( $datacenter_uuid, 'datacenter', 'storage_domain' ) } ) {
        my $domain_label  = OVirtDataWrapper::get_label( 'storage_domain', $domain_uuid );
        $ovirt_inventory{DATACENTER}{$datacenter_uuid}{STORAGEDOMAIN}{$domain_uuid}{label} = $domain_label;
        $ovirt_inventory{TRANSLATE_UUID}{$domain_uuid} = $domain_label;

        # Disks
        foreach my $disk_uuid ( @{ OVirtDataWrapper::get_arch( $domain_uuid, 'storage_domain', 'disk' ) } ) {
          my $disk_label    = OVirtDataWrapper::get_label( 'disk', $disk_uuid );
          $ovirt_inventory{DATACENTER}{$datacenter_uuid}{STORAGEDOMAIN}{$domain_uuid}{DISK}{$disk_uuid}{label} = $disk_label;
          $ovirt_inventory{TRANSLATE_UUID}{$disk_uuid} = $disk_label;
        }
      }
    }
  }

  return 1;
}

sub get_uuids_ovirt {
  my $params = shift;
  my @uuids;

  if ( exists $params->{subsys} && exists $params->{datacenter} ) {
    # CLUSTER
    if ( $params->{subsys} eq "CLUSTER" && exists $ovirt_inventory{DATACENTER}{ $params->{datacenter} }{CLUSTER} ) {
      # datacenter -> cluster
      foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{ $params->{datacenter} }{CLUSTER} } ) {
        push(@uuids, $uuid);
      }
    }
    # VM
    elsif ( $params->{subsys} eq "VM" && exists $params->{host} && ref($params->{host}) eq "ARRAY" ) {
      # datacenter -> cluster -> vm
      foreach my $cluster ( @{ $params->{host} } ) {
        if ( exists $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{VM} ) {
          foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{VM} } ) {
            push(@uuids, $uuid);
          }
        }
      }
    }
    # HOST/LAN-TOTAL
    elsif ( ( $params->{subsys} eq "HOST" || $params->{subsys} eq "LAN-TOTAL" ) && exists $params->{host} && ref($params->{host}) eq "ARRAY" ) {
      # datacenter -> cluster -> host
      foreach my $cluster ( @{ $params->{host} } ) {
        if ( exists $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{HOST} ) {
          foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{HOST} } ) {
            push(@uuids, $uuid);
          }
        }
      }
    }
    # LAN
    elsif ( $params->{subsys} eq "LAN" && exists $params->{host} && ref($params->{host}) eq "ARRAY" && exists $params->{name} && ref($params->{name}) eq "ARRAY" ) {
      # datacenter -> cluster -> host -> name (lan/nic)
      foreach my $cluster ( @{ $params->{host} } ) {
        foreach my $host ( @{ $params->{name} } ) {
          if ( exists $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{HOST}{$host}{LAN} ) {
            foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{$params->{datacenter}}{CLUSTER}{$cluster}{HOST}{$host} } ) {
              push(@uuids, $uuid);
            }
          }
        }
      }
    }
    # STORAGEDOMAIN
    if ( $params->{subsys} eq "STORAGEDOMAIN" && exists $ovirt_inventory{DATACENTER}{ $params->{datacenter} }{STORAGEDOMAIN} ) {
      # datacenter -> storage domain
      foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{ $params->{datacenter} }{STORAGEDOMAIN} } ) {
        push(@uuids, $uuid);
      }
    }
    # DISK
    elsif ( $params->{subsys} eq "DISK" && exists $params->{host} && ref($params->{host}) eq "ARRAY" ) {
      # datacenter -> storage domain -> disk
      foreach my $storagedomain ( @{ $params->{host} } ) {
        if ( exists $ovirt_inventory{DATACENTER}{$params->{datacenter}}{STORAGEDOMAIN}{$storagedomain}{DISK} ) {
          foreach my $uuid ( sort keys %{ $ovirt_inventory{DATACENTER}{$params->{datacenter}}{STORAGEDOMAIN}{$storagedomain}{DISK} } ) {
            push(@uuids, $uuid);
          }
        }
      }
    }
  }

  return @uuids;
}

sub test_acl {
  my $user     = shift;
  my $platform = shift;
  my $subsys   = shift;
  my $host     = shift;
  my $item     = shift;

  use ACL;    # use module ACL.pm

  if ( !ACL::isAdmin($user) ) {
    if ( !ACL::canShow( $platform, $subsys, $host, $item, $user ) ) {
      print "ACL            : not allowed for user $user $platform: $subsys, $host, $item\n" if $debug == 9;
      return 0;
    }
  }
  #print "allowed for user $user $platform: $subsys, $host, $item\n";

  return 1;
}

sub get_list_of_graphs {
  my $user_name   = shift;
  my $report_name = shift;

  #
  # progress bar, inventory for all graphs
  #
  my %graphs;
  $count = 0;
  if ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'items'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'items'}) eq "ARRAY" ) {
    foreach my $params ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'items'} } ) {
      #
      # POWER
      #
      if ( $params->{'metrics'} && ref($params->{'metrics'}) eq "ARRAY" && $params->{group} && $params->{group} eq "POWER" && $params->{'subsys'} ) {
        # 1.
        # host = always all, items = always all
        # ALL lpars/pools/memory for ALL servers
        if ( $params->{'allhosts'} && $params->{'entiresubsys'} ) {
          if ( $power_inventory{SERVERS} ) {
            foreach my $server ( sort keys %{ $power_inventory{SERVERS} } ) {
              #
              # LPARs
              #
              if ( $params->{'subsys'} eq "LPAR" ) {
                if ( $power_inventory{SERVERS}{$server}{LPARS} ) {
                  foreach my $name ( sort keys %{ $power_inventory{SERVERS}{$server}{LPARS} } ) {
                    # ACL
                    if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,$name) ) { next; }

                    if ( $power_inventory{SERVERS}{$server}{LPARS}{$name}{MENU_TYPE} && $power_inventory{SERVERS}{$server}{LPARS}{$name}{HMC} ) {
                      my @available_metrics = find_lpar_metrics("$power_inventory{SERVERS}{$server}{LPARS}{$name}{MENU_TYPE}"); # available metrics for lpar
                      foreach my $metric ( @{ $params->{'metrics'} } ) {
                        # test if metric is available for this lpar
                        my $metric_found = grep {/^$metric$/} @available_metrics;
                        if ( $metric_found == 0 ) { next; } # this metric is not available
                        $count++;

                        $graphs{$count}->{group}  = $params->{group};
                        $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{LPARS}{$name}{HMC};
                        $graphs{$count}->{host}   = $server;
                        $graphs{$count}->{subsys} = $params->{subsys};
                        $graphs{$count}->{metric} = $metric;
                        $graphs{$count}->{name}   = $name;
                        if ( $params->{sample_rate} ) {
                          $graphs{$count}->{sample_rate} = $params->{sample_rate};
                        }
                      }
                    }
                  }
                }
              }
              #
              # POOLs
              #
              if ( $params->{'subsys'} eq "POOL" ) {
                if ( $power_inventory{SERVERS}{$server}{POOLS} ) {
                  foreach my $name ( sort keys %{ $power_inventory{SERVERS}{$server}{POOLS} } ) {
                    if ( exists $power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID} && exists $power_inventory{SERVERS}{$server}{POOLS}{$name}{HMC} ) {
                      # ACL
                      if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,$power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID}) ) { next; }

                      # available metrics for pool or shpool
                      my @available_metrics = find_pool_metrics("$power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID}","$cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'}");
                      foreach my $metric ( @{ $params->{'metrics'} } ) {
                        # test if metric is available for this pool/shpool
                        my $metric_found = grep {/^$metric$/} @available_metrics;
                        if ( $metric_found == 0 ) { next; } # this metric is not available
                        $count++;

                        $graphs{$count}->{group}  = $params->{group};
                        $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{POOLS}{$name}{HMC};
                        $graphs{$count}->{host}   = $server;
                        $graphs{$count}->{subsys} = $params->{subsys};
                        $graphs{$count}->{metric} = $metric;
                        $graphs{$count}->{name}   = $power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID};
                        if ( $params->{sample_rate} ) {
                          $graphs{$count}->{sample_rate} = $params->{sample_rate};
                        }
                      }
                    }
                  }
                }
              }
              #
              # Server memory
              #
              if ( $params->{'subsys'} eq "SERVER" ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,"MEMORY") ) { next; }

                if ( exists $power_inventory{SERVERS}{$server}{POOLS}{'CPU pool'}{HMC} ) {
                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    $count++;

                    $graphs{$count}->{group}  = $params->{group};
                    $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{POOLS}{'CPU pool'}{HMC};
                    $graphs{$count}->{host}   = $server;
                    $graphs{$count}->{subsys} = $params->{subsys};
                    $graphs{$count}->{metric} = $metric;
                    $graphs{$count}->{name}   = "pool";
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
          }
        }
        # 2.
        # host = always all
        # SINGLE LPARs/POOLs/SERVER mem for all servers
        elsif ( $params->{'allhosts'} && ! $params->{'entiresubsys'} && $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
          if ( $power_inventory{SERVERS} ) {
            foreach my $server ( sort keys %{ $power_inventory{SERVERS} } ) {
              #
              # LPARs
              #
              if ( $params->{'subsys'} eq "LPAR" ) {
                foreach my $name ( @{ $params->{'name'} } ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,$name) ) { next; }

                  if ( $power_inventory{SERVERS}{$server}{LPARS}{$name}{MENU_TYPE} && $power_inventory{SERVERS}{$server}{LPARS}{$name}{HMC} ) { # load available metrics for this lpar
                    my @available_metrics = find_lpar_metrics("$power_inventory{SERVERS}{$server}{LPARS}{$name}{MENU_TYPE}"); # available metrics for lpar

                    foreach my $metric ( @{ $params->{'metrics'} } ) {
                      # test if metric is available for this lpar
                      my $metric_found = grep {/^$metric$/} @available_metrics;
                      if ( $metric_found == 0 ) { next; } # this metric is not available
                      $count++;

                      $graphs{$count}->{group}  = $params->{group};
                      $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{LPARS}{$name}{HMC};
                      $graphs{$count}->{host}   = $server;
                      $graphs{$count}->{subsys} = $params->{subsys};
                      $graphs{$count}->{metric} = $metric;
                      $graphs{$count}->{name}   = $name;
                      if ( $params->{sample_rate} ) {
                        $graphs{$count}->{sample_rate} = $params->{sample_rate};
                      }
                    }
                  }
                }
              }
              #
              # POOLs
              #
              if ( $params->{'subsys'} eq "POOL" ) {
                foreach my $name ( @{ $params->{'name'} } ) {
                  if ( exists $power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID} && exists $power_inventory{SERVERS}{$server}{POOLS}{$name}{HMC} ) {
                    # ACL
                    if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,$power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID}) ) { next; }

                    # available metrics for pool or shpool
                    my @available_metrics = find_pool_metrics("$power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID}","$cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'}");
                    foreach my $metric ( @{ $params->{'metrics'} } ) {
                      # test if metric is available for this pool/shpool
                      my $metric_found = grep {/^$metric$/} @available_metrics;
                      if ( $metric_found == 0 ) { next; } # this metric is not available
                      $count++;

                      $graphs{$count}->{group}  = $params->{group};
                      $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{POOLS}{$name}{HMC};
                      $graphs{$count}->{host}   = $server;
                      $graphs{$count}->{subsys} = $params->{subsys};
                      $graphs{$count}->{metric} = $metric;
                      $graphs{$count}->{name}   = $power_inventory{SERVERS}{$server}{POOLS}{$name}{POOL_ID};
                      if ( $params->{sample_rate} ) {
                        $graphs{$count}->{sample_rate} = $params->{sample_rate};
                      }
                    }
                  }
                }
              }
              #
              # SERVER mem
              #
              if ( $params->{'subsys'} eq "SERVER" && exists $power_inventory{SERVERS}{$server}{POOLS}{'CPU pool'}{HMC} ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$server,"MEMORY") ) { next; }

                foreach my $metric ( @{ $params->{'metrics'} } ) {
                  $count++;

                  $graphs{$count}->{group}  = $params->{group};
                  $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$server}{POOLS}{'CPU pool'}{HMC};
                  $graphs{$count}->{host}   = $server;
                  $graphs{$count}->{subsys} = $params->{subsys};
                  $graphs{$count}->{metric} = $metric;
                  $graphs{$count}->{name}   = $params->{name};
                  if ( $params->{sample_rate} ) {
                    $graphs{$count}->{sample_rate} = $params->{sample_rate};
                  }
                }
              }
            }
          }
        }
        # 3.
        # items = always all
        # ALL LPARs/POOLs/SERVER mem for single servers
        elsif ( !$params->{'allhosts'} && $params->{'entiresubsys'} && $params->{'host'} && ref($params->{'host'}) eq "ARRAY" ) {
          foreach my $host ( @{ $params->{'host'} } ) {
            #
            # LPARs
            #
            if ( $params->{'subsys'} eq "LPAR" && $power_inventory{SERVERS}{$host}{LPARS} ) {
              foreach my $lpar_name ( sort keys %{ $power_inventory{SERVERS}{$host}{LPARS} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,$lpar_name) ) { next; }

                if ( $power_inventory{SERVERS}{$host}{LPARS}{$lpar_name}{MENU_TYPE} && $power_inventory{SERVERS}{$host}{LPARS}{$lpar_name}{HMC} ) { # load available metrics for this lpar
                  my @available_metrics = find_lpar_metrics("$power_inventory{SERVERS}{$host}{LPARS}{$lpar_name}{MENU_TYPE}"); # available metrics for lpar

                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    # test if metric is available for this lpar
                    my $metric_found = grep {/^$metric$/} @available_metrics;
                    if ( $metric_found == 0 ) { next; } # this metric is not available
                    $count++;

                    $graphs{$count}->{group}  = $params->{group};
                    $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{LPARS}{$lpar_name}{HMC};
                    $graphs{$count}->{host}   = $host;
                    $graphs{$count}->{subsys} = $params->{subsys};
                    $graphs{$count}->{metric} = $metric;
                    $graphs{$count}->{name}   = $lpar_name;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
            #
            # POOLs
            #
            if ( $params->{'subsys'} eq "POOL" && $power_inventory{SERVERS}{$host}{POOLS} ) {
              foreach my $name ( sort keys %{ $power_inventory{SERVERS}{$host}{POOLS} } ) {
                if ( exists $power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID} && exists $power_inventory{SERVERS}{$host}{POOLS}{$name}{HMC} ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,$power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID}) ) { next; }

                  # available metrics for pool or shpool
                  my @available_metrics = find_pool_metrics("$power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID}","$cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'}");
                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    # test if metric is available for this pool/shpool
                    my $metric_found = grep {/^$metric$/} @available_metrics;
                    if ( $metric_found == 0 ) { next; } # this metric is not available
                    $count++;

                    $graphs{$count}->{group}  = $params->{group};
                    $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{POOLS}{$name}{HMC};
                    $graphs{$count}->{host}   = $host;
                    $graphs{$count}->{subsys} = $params->{subsys};
                    $graphs{$count}->{metric} = $metric;
                    $graphs{$count}->{name}   = $power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID};
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
            #
            # Server memory
            #
            if ( $params->{'subsys'} eq "SERVER" && exists $power_inventory{SERVERS}{$host}{POOLS}{'CPU pool'}{HMC} ) {
              # ACL
              if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,"MEMORY") ) { next; }

              foreach my $metric ( @{ $params->{'metrics'} } ) {
                $count++;

                $graphs{$count}->{group}  = $params->{group};
                $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{POOLS}{'CPU pool'}{HMC};
                $graphs{$count}->{host}   = $host;
                $graphs{$count}->{subsys} = $params->{subsys};
                $graphs{$count}->{metric} = $metric;
                $graphs{$count}->{name}   = "pool";
                if ( $params->{sample_rate} ) {
                  $graphs{$count}->{sample_rate} = $params->{sample_rate};
                }
              }
            }
          }
        }
        # 4.
        # SINGLE LPARs/POOLs/etc. for single servers
        #
        elsif ( !$params->{'allhosts'} && !$params->{'entiresubsys'} && $params->{'host'} && ref($params->{'host'}) eq "ARRAY" && $params->{'name'} && ref($params->{'name'}) eq "ARRAY") {
          foreach my $host ( @{ $params->{'host'} } ) {
            #
            # LPARs
            #
            if ( $params->{'subsys'} eq "LPAR" ) {
              foreach my $name ( @{ $params->{'name'} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,$name) ) { next; }

                if ( $power_inventory{SERVERS}{$host}{LPARS}{$name}{MENU_TYPE} && $power_inventory{SERVERS}{$host}{LPARS}{$name}{HMC} ) { # load available metrics for this lpar
                  my @available_metrics = find_lpar_metrics("$power_inventory{SERVERS}{$host}{LPARS}{$name}{MENU_TYPE}"); # available metrics for lpar

                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    # test if metric is available for this lpar
                    my $metric_found = grep {/^$metric$/} @available_metrics;
                    if ( $metric_found == 0 ) { next; } # this metric is not available
                    $count++;

                    $graphs{$count}->{group}  = $params->{group};
                    $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{LPARS}{$name}{HMC};
                    $graphs{$count}->{host}   = $host;
                    $graphs{$count}->{subsys} = $params->{subsys};
                    $graphs{$count}->{metric} = $metric;
                    $graphs{$count}->{name}   = $name;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
            #
            # POOLs
            #
            if ( $params->{'subsys'} eq "POOL" ) {
              foreach my $name ( @{ $params->{'name'} } ) {
                if ( exists $power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID} && exists $power_inventory{SERVERS}{$host}{POOLS}{$name}{HMC} ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,$power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID}) ) { next; }

                  # available metrics for pool or shpool
                  my @available_metrics = find_pool_metrics("$power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID}","$cfg{'users'}{$user_name}{'reports'}{$report_name}{'format'}");
                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    # test if metric is available for this pool/shpool
                    my $metric_found = grep {/^$metric$/} @available_metrics;
                    if ( $metric_found == 0 ) { next; } # this metric is not available
                    $count++;

                    $graphs{$count}->{group}  = $params->{group};
                    $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{POOLS}{$name}{HMC};
                    $graphs{$count}->{host}   = $host;
                    $graphs{$count}->{subsys} = $params->{subsys};
                    $graphs{$count}->{metric} = $metric;
                    $graphs{$count}->{name}   = $power_inventory{SERVERS}{$host}{POOLS}{$name}{POOL_ID};
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
            #
            # SERVER mem
            #
            if ( $params->{'subsys'} eq "SERVER" && exists $power_inventory{SERVERS}{$host}{POOLS}{'CPU pool'}{HMC} ) {
              # ACL
              if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$host,"MEMORY") ) { next; }

              foreach my $metric ( @{ $params->{'metrics'} } ) {
                $count++;

                $graphs{$count}->{group}  = $params->{group};
                $graphs{$count}->{hmc}    = $power_inventory{SERVERS}{$host}{POOLS}{'CPU pool'}{HMC};
                $graphs{$count}->{host}   = $host;
                $graphs{$count}->{subsys} = $params->{subsys};
                $graphs{$count}->{metric} = $metric;
                $graphs{$count}->{name}   = $params->{name};
                if ( $params->{sample_rate} ) {
                  $graphs{$count}->{sample_rate} = $params->{sample_rate};
                }
              }
            }
          }
        }
      }
      #
      # CUSTOM GROUPS
      #
      if ( $params->{'metrics'} && ref($params->{'metrics'}) eq "ARRAY" && $params->{group} && $params->{group} eq "CUSTOM" && $params->{'subsys'} && $params->{'host'} && ref($params->{'host'}) eq "ARRAY" ) {
        foreach my $host ( @{ $params->{'host'} } ) {
          # ACL
          if ( !test_acl($user_name,$params->{group},$params->{group},$host,"") ) { next; }

          foreach my $metric ( @{ $params->{'metrics'} } ) {
            if ( $params->{subsys} eq "LPAR" || $params->{subsys} eq "POOL" ) {
              my $graph_exists = find_custom_group_metric("$host","$metric");
              #print STDERR "$graph_exists : $params->{host} : $metric\n";
              if ( $graph_exists == 0 ) { next; } # unsupported metric for this custom group, probably lpars without os agent
            }
            $count++;

            $graphs{$count}->{group}  = $params->{group};
            $graphs{$count}->{host}   = $host;
            $graphs{$count}->{subsys} = $params->{subsys};
            $graphs{$count}->{metric} = $metric;
            if ( $params->{sample_rate} ) {
              $graphs{$count}->{sample_rate} = $params->{sample_rate};
            }
          }
        }
      }
      #
      # VMWARE
      #
      if ( $params->{'metrics'} && ref($params->{'metrics'}) eq "ARRAY" && $params->{group} && $params->{group} eq "VMWARE" && $params->{'subsys'} ) {
        #
        # VM
        #
        if ( $params->{'subsys'} eq "VM" ) {
          if ( $params->{'entiresubsys'} && exists $params->{vcenter} && exists $params->{host} && ref($params->{'host'}) eq "ARRAY" ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM} ) {
                foreach my $vm_uuid ( sort keys %{ $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM} } ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},"cluster_$cluster",$vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM}{$vm_uuid}{VM_NAME}) ) { next; }

                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    $count++;

                    $graphs{$count}->{group}   = $params->{group};
                    $graphs{$count}->{host}    = $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM}{$vm_uuid}{URL_PARAM}{host};
                    $graphs{$count}->{server}  = $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM}{$vm_uuid}{URL_PARAM}{server};
                    $graphs{$count}->{vcenter} = $params->{vcenter};
                    $graphs{$count}->{cluster} = $cluster;
                    $graphs{$count}->{vm_uuid} = $vm_uuid;
                    $graphs{$count}->{vm_name} = $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{VM}{$vm_uuid}{VM_NAME};
                    $graphs{$count}->{subsys}  = $params->{subsys};
                    $graphs{$count}->{metric}  = $metric;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
          }
          elsif ( !$params->{'entiresubsys'} && $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
            foreach my $vm_uuid ( @{ $params->{'name'} } ) {
              if ( exists $vmware_inventory{VM}{$vm_uuid} && exists $vmware_inventory{VM}{$vm_uuid}{URL_PARAM}{host} && exists $vmware_inventory{VM}{$vm_uuid}{URL_PARAM}{server} && exists $vmware_inventory{VM}{$vm_uuid}{VCENTER} && exists $vmware_inventory{VM}{$vm_uuid}{CLUSTER} && exists $vmware_inventory{VM}{$vm_uuid}{VM_NAME} ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},"cluster_$vmware_inventory{VM}{$vm_uuid}{CLUSTER}",$vmware_inventory{VM}{$vm_uuid}{VM_NAME}) ) { next; }

                foreach my $metric ( @{ $params->{'metrics'} } ) {
                  $count++;

                  $graphs{$count}->{group}   = $params->{group};
                  $graphs{$count}->{host}    = $vmware_inventory{VM}{$vm_uuid}{URL_PARAM}{host};
                  $graphs{$count}->{server}  = $vmware_inventory{VM}{$vm_uuid}{URL_PARAM}{server};
                  $graphs{$count}->{vcenter} = $vmware_inventory{VM}{$vm_uuid}{VCENTER};
                  $graphs{$count}->{cluster} = $vmware_inventory{VM}{$vm_uuid}{CLUSTER};
                  $graphs{$count}->{vm_uuid} = $vm_uuid;
                  $graphs{$count}->{vm_name} = $vmware_inventory{VM}{$vm_uuid}{VM_NAME};
                  $graphs{$count}->{subsys}  = $params->{subsys};
                  $graphs{$count}->{metric}  = $metric;
                  if ( $params->{sample_rate} ) {
                    $graphs{$count}->{sample_rate} = $params->{sample_rate};
                  }
                }
              }
            }
          }
        }
        #
        # ESXi
        #
        if ( $params->{'subsys'} eq "ESXI" && exists $params->{vcenter} && exists $params->{host} && ref($params->{'host'}) eq "ARRAY" ) {
          if ( $params->{'entiresubsys'} ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{ESXI} && exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{VCENTER_IP} ) {
                foreach my $esxi ( sort keys %{ $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{ESXI} } ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$cluster,$esxi) ) { next; }

                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    $count++;

                    if ( $metric eq "pool" ) {
                      $graphs{$count}->{lpar} = "pool";
                    }
                    elsif ( $metric eq "lparagg" ) {
                      $graphs{$count}->{lpar} = "pool-multi";
                    }
                    else {
                      $graphs{$count}->{lpar} = "cod";
                    }

                    $graphs{$count}->{group}   = $params->{group};
                    $graphs{$count}->{host}    = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{VCENTER_IP};
                    $graphs{$count}->{server}  = $esxi;
                    $graphs{$count}->{vcenter} = $params->{'vcenter'};
                    $graphs{$count}->{cluster} = $cluster;
                    $graphs{$count}->{subsys}  = $params->{subsys};
                    $graphs{$count}->{metric}  = $metric;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
          }
          elsif ( !$params->{'entiresubsys'} && $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              foreach my $esxi ( @{ $params->{'name'} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$cluster,$esxi) ) { next; }

                if ( exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{VCENTER_IP} ) {
                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    $count++;

                    if ( $metric eq "pool" ) {
                      $graphs{$count}->{lpar} = "pool";
                    }
                    elsif ( $metric eq "lparagg" ) {
                      $graphs{$count}->{lpar} = "pool-multi";
                    }
                    else {
                      $graphs{$count}->{lpar} = "cod";
                    }

                    $graphs{$count}->{group}   = $params->{group};
                    $graphs{$count}->{host}    = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{VCENTER_IP};
                    $graphs{$count}->{server}  = $esxi;
                    $graphs{$count}->{vcenter} = $params->{'vcenter'};
                    $graphs{$count}->{cluster} = $cluster;
                    $graphs{$count}->{subsys}  = $params->{subsys};
                    $graphs{$count}->{metric}  = $metric;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
          }
        }
        #
        # RESPOOL
        #
        # B:10.22.11.10:cluster_New Cluster:Development:/lpar2rrd-cgi/detail.sh?host=cluster_domain-c87&server=vmware_eb6102a7-1fa0-4376-acbb-f67e34a2212c_28&lpar=resgroup-139&item=resourcepool&entitle=0&gui=1&none=none::Hosting::V
        if ( $params->{'subsys'} eq "RESPOOL" && exists $params->{vcenter} && exists $params->{host} && ref($params->{'host'}) eq "ARRAY" ) {
          if ( $params->{'entiresubsys'} ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{RESPOOL} ) {
                foreach my $rp ( sort keys %{ $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{RESPOOL} } ) {
                  # ACL
                  if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$cluster,$rp) ) { next; }

                  if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{CLUSTER}{$cluster}{RESPOOL}{$rp}{URL_PARAM} ) {
                    my $url_params = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{CLUSTER}{$cluster}{RESPOOL}{$rp}{URL_PARAM};
                    if ( exists $url_params->{host} && exists $url_params->{server} && exists $url_params->{lpar} ) {
                      foreach my $metric ( @{ $params->{'metrics'} } ) {
                        $count++;

                        $graphs{$count}->{group}   = $params->{group};
                        $graphs{$count}->{subsys}  = $params->{subsys};
                        $graphs{$count}->{host}    = $url_params->{host};
                        $graphs{$count}->{server}  = $url_params->{server};
                        $graphs{$count}->{lpar}    = $url_params->{lpar};
                        $graphs{$count}->{vcenter} = $params->{'vcenter'};
                        $graphs{$count}->{cluster} = $cluster;
                        $graphs{$count}->{respool} = $rp;
                        $graphs{$count}->{metric}  = $metric;
                        if ( $params->{sample_rate} ) {
                          $graphs{$count}->{sample_rate} = $params->{sample_rate};
                        }
                      }
                    }
                  }
                }
              }
            }
          }
          elsif ( !$params->{'entiresubsys'} && $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              foreach my $rp ( @{ $params->{'name'} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$cluster,$rp) ) { next; }

                if ( exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{CLUSTER}{$cluster}{RESPOOL}{$rp}{URL_PARAM} ) {
                  my $url_params = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{CLUSTER}{$cluster}{RESPOOL}{$rp}{URL_PARAM};
                  if ( exists $url_params->{host} && exists $url_params->{server} && exists $url_params->{lpar} ) {
                    foreach my $metric ( @{ $params->{'metrics'} } ) {
                      $count++;

                      $graphs{$count}->{group}   = $params->{group};
                      $graphs{$count}->{subsys}  = $params->{subsys};
                      $graphs{$count}->{host}    = $url_params->{host};
                      $graphs{$count}->{server}  = $url_params->{server};
                      $graphs{$count}->{lpar}    = $url_params->{lpar};
                      $graphs{$count}->{vcenter} = $params->{'vcenter'};
                      $graphs{$count}->{cluster} = $cluster;
                      $graphs{$count}->{respool} = $rp;
                      $graphs{$count}->{metric}  = $metric;
                      if ( $params->{sample_rate} ) {
                        $graphs{$count}->{sample_rate} = $params->{sample_rate};
                      }
                    }
                  }
                }
              }
            }
          }
        }
        #
        # DATASTORE
        #
        if ( $params->{'subsys'} eq "DATASTORE" && exists $params->{vcenter} && exists $params->{host} && ref($params->{'host'}) eq "ARRAY" ) {
          if ( $params->{'entiresubsys'} ) {
            if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE} ) {
              foreach my $ds ( sort keys %{ $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$params->{'vcenter'},$ds) ) { next; }

                if ( exists $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE}{$ds}{URL_PARAM} && exists $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE}{$ds}{DATACENTER} ) {
                  my $url_params = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL_PARAM};
                  if ( exists $url_params->{host} && exists $url_params->{server} && exists $url_params->{lpar} && exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL} ) {
                    my @available_metrics = find_datastore_metrics("$vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL}");
                    foreach my $metric ( @{ $params->{'metrics'} } ) {
                      # test if metric is available for this datastore
                      my $metric_found = grep {/^$metric$/} @available_metrics;
                      if ( $metric_found == 0 ) { next; } # this metric is not available
                      $count++;

                      $graphs{$count}->{group}      = $params->{group};
                      $graphs{$count}->{subsys}     = $params->{subsys};
                      $graphs{$count}->{host}       = $url_params->{host};
                      $graphs{$count}->{server}     = $url_params->{server};
                      $graphs{$count}->{lpar}       = $url_params->{lpar};
                      $graphs{$count}->{vcenter}    = $params->{'vcenter'};
                      $graphs{$count}->{datacenter} = $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE}{$ds}{DATACENTER};
                      $graphs{$count}->{datastore}  = $ds;
                      $graphs{$count}->{metric}     = $metric;
                      if ( $params->{sample_rate} ) {
                        $graphs{$count}->{sample_rate} = $params->{sample_rate};
                      }
                    }
                  }
                }
              }
            }
          }
          elsif ( !$params->{'entiresubsys'} && $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              foreach my $ds ( @{ $params->{'name'} } ) {
                # ACL
                if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE}{$ds}{DATACENTER},$ds) ) { next; }

                if ( exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL_PARAM} ) {
                  my $url_params = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL_PARAM};
                  if ( exists $url_params->{host} && exists $url_params->{server} && exists $url_params->{lpar} ) {
                    my @available_metrics = find_datastore_metrics("$vmware_inventory{VCENTER}{$params->{'vcenter'}}{DATASTORE}{$ds}{URL}");
                    foreach my $metric ( @{ $params->{'metrics'} } ) {
                      # test if metric is available for this datastore
                      my $metric_found = grep {/^$metric$/} @available_metrics;
                      if ( $metric_found == 0 ) { next; } # this metric is not available
                      $count++;

                      $graphs{$count}->{group}     = $params->{group};
                      $graphs{$count}->{subsys}    = $params->{subsys};
                      $graphs{$count}->{host}      = $url_params->{host};
                      $graphs{$count}->{server}    = $url_params->{server};
                      $graphs{$count}->{lpar}      = $url_params->{lpar};
                      $graphs{$count}->{vcenter}   = $params->{'vcenter'};
                      $graphs{$count}->{datacenter} = $vmware_inventory{VCENTER}{$params->{vcenter}}{DATASTORE}{$ds}{DATACENTER};
                      $graphs{$count}->{datastore} = $ds;
                      $graphs{$count}->{metric}    = $metric;
                      if ( $params->{sample_rate} ) {
                        $graphs{$count}->{sample_rate} = $params->{sample_rate};
                      }
                    }
                  }
                }
              }
            }
          }
        }
        #
        # CLUSTER
        #
        if ( $params->{'subsys'} eq "CLUSTER" ) {
          if ( !$params->{'entiresubsys'} && exists $params->{vcenter} && exists $params->{host} && ref($params->{'host'}) eq "ARRAY"  ) {
            foreach my $cluster ( @{ $params->{host} } ) {
              # ACL
              if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$params->{'vcenter'},$cluster) ) { next; }

              if ( exists $vmware_inventory{VCENTER}{$params->{'vcenter'}}{CLUSTER}{$cluster}{URL_PARAM} ) {
                my $url_params = $vmware_inventory{VCENTER}{$params->{'vcenter'}}{CLUSTER}{$cluster}{URL_PARAM};
                if ( exists $url_params->{host} && exists $url_params->{server} && exists $url_params->{lpar} ) {
                  foreach my $metric ( @{ $params->{'metrics'} } ) {
                    $count++;

                    $graphs{$count}->{group}   = $params->{group};
                    $graphs{$count}->{subsys}  = $params->{subsys};
                    $graphs{$count}->{host}    = $url_params->{host};
                    $graphs{$count}->{server}  = $url_params->{server};
                    $graphs{$count}->{lpar}    = $url_params->{lpar};
                    $graphs{$count}->{vcenter} = $params->{'vcenter'};
                    $graphs{$count}->{cluster} = $cluster;
                    $graphs{$count}->{metric}  = $metric;
                    if ( $params->{sample_rate} ) {
                      $graphs{$count}->{sample_rate} = $params->{sample_rate};
                    }
                  }
                }
              }
            }
          }
        }
      }
      #
      # oVirt
      #
      if ( $params->{'metrics'} && ref($params->{'metrics'}) eq "ARRAY" && $params->{group} && $params->{group} eq "OVIRT" && $params->{'subsys'} && $params->{'datacenter'} && exists $params->{'host'} && ref($params->{'host'}) eq "ARRAY"  ) {
        my @uuids;
        if ( $params->{'entiresubsys'} ) { # all items from subsystem
          @uuids = get_uuids_ovirt(\%$params);
        }
        elsif ( $params->{'subsys'} eq "VM" || $params->{'subsys'} eq "DISK" || $params->{'subsys'} eq "HOST" ) { # single items
          if ( exists $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) {
            @uuids = @{ $params->{'name'} };
          }
        }
        elsif ( $params->{'subsys'} eq "CLUSTER" || $params->{'subsys'} eq "STORAGEDOMAIN" ) { # single items
          @uuids = @{ $params->{'host'} };
        }
        foreach my $uuid ( @uuids ) {
          # ACL
          if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$uuid,$uuid) ) { next; }

          foreach my $metric ( @{ $params->{'metrics'} } ) {
            # test if storage domain has got aggreg data and latency
            if ( $params->{'subsys'} eq "STORAGEDOMAIN" && test_storagedomain_metric($uuid,$metric) != 1 ) { next; }

            $count++;

            foreach my $host ( @{ $params->{'host'} } ) {
              $graphs{$count}->{host} = $host;
            }

            $graphs{$count}->{datacenter} = $params->{datacenter};
            $graphs{$count}->{group}      = $params->{group};
            $graphs{$count}->{subsys}     = $params->{subsys};
            $graphs{$count}->{uuid}       = $uuid;
            $graphs{$count}->{metric}     = $metric;
            if ( $params->{sample_rate} ) {
              $graphs{$count}->{sample_rate} = $params->{sample_rate};
            }
          }
        }
      }
      #
      # Solaris
      #
      if ( $params->{'metrics'} && ref($params->{'metrics'}) eq "ARRAY" && $params->{group} && $params->{group} eq "SOLARIS" ) {
        my @names;
        if ( $params->{'entiresubsys'} ) { # all items from subsystem
          foreach my $name ( sort keys %{ $solaris_inventory{LDOM} } ) {
            push(@names, $name)
          }
        }
        elsif ( $params->{'subsys'} eq "LDOM" && exists $params->{'name'} && ref($params->{'name'}) eq "ARRAY" ) { # single items
          @names = @{ $params->{'name'} };
        }
        foreach my $name ( @names ) {
          # ACL
          if ( !test_acl($user_name,$params->{group},$params->{'subsys'},$name,$name) ) { next; }

          foreach my $metric ( @{ $params->{'metrics'} } ) {
            $count++;

            $graphs{$count}->{group}  = $params->{group};
            $graphs{$count}->{subsys} = $params->{subsys};
            $graphs{$count}->{name}   = $name;
            $graphs{$count}->{metric} = $metric;
            if ( $params->{sample_rate} ) {
              $graphs{$count}->{sample_rate} = $params->{sample_rate};
            }
          }
        }
      }
    }
  }

  return %graphs;
}

sub test_storagedomain_metric {
  my $uuid   = shift;
  my $metric = shift;

  if ( $metric eq "ovirt_storage_domain_aggr_data" || $metric eq "ovirt_storage_domain_aggr_latency" ) {
    # test if this storagedomain has got assignet DISKs
    if ( exists $ovirt_inventory{DATACENTER} ) {
      foreach my $dc_uuid ( keys %{ $ovirt_inventory{DATACENTER} } ) {
        if ( exists $ovirt_inventory{DATACENTER}{$dc_uuid}{STORAGEDOMAIN}{$uuid}{DISK} ) { return 1; }
      }
    }

    return 0;
  }
  else {
    return 1;
  }
}

sub find_custom_group_metric {
  my $name = shift;
  my $item = shift;

  if ( $item eq "custom"           && -f "$tmpdir/custom-group-$name-d.cmd" )           { return 1; }
  if ( $item eq "custom_cpu_trend" && -f "$tmpdir/custom-group-$name-cpu_trend-y.cmd" ) { return 1; }
  if ( $item eq "custommem"        && -f "$tmpdir/custom-group-mem-$name-d.cmd" )       { return 1; }
  if ( $item eq "customosmem"      && -f "$tmpdir/custom-group-mem-os-$name-d.cmd" )    { return 1; }
  if ( $item eq "customoslan"      && -f "$tmpdir/custom-group-lan-os-$name-d.cmd" )    { return 1; }
  if ( $item eq "customossan1"     && -f "$tmpdir/custom-group-san1-os-$name-d.cmd" )   { return 1; }
  if ( $item eq "customossan2"     && -f "$tmpdir/custom-group-san2-os-$name-d.cmd" )   { return 1; }

  return 0;
}

sub set_report_time_range {
  my $freq  = shift;
  my $range = shift;

  my $sunix = 0;
  my $eunix = 0;

  my $day_sec  = 60*60*24;
  my $act_time = time();
  my $act_date = localtime();
  my ( $act_sec, $act_min, $act_hour, $act_day, $act_month, $act_year, $act_wday, $act_yday, $act_isdst ) = localtime();

  # previous day/week/month/year
  if ( $range eq "prev" ) {
    if ( $freq eq "daily" ) {
      $eunix = mktime(0,0,0,$act_day,$act_month,$act_year);
      $sunix = $eunix - $day_sec;
    }
    if ( $freq eq "weekly" ) {
      $eunix = mktime(0,0,0,$act_day,$act_month,$act_year);
      $eunix = $eunix - (($act_wday - 1) * $day_sec);
      $sunix = $eunix - (7 * $day_sec);
    }
    if ( $freq eq "monthly" ) {
      $sunix = mktime(0,0,0,1,$act_month-1,$act_year);
      $eunix = mktime(0,0,0,1,$act_month,$act_year);
    }
    if ( $freq eq "yearly" ) {
      $sunix = mktime(0,0,0,1,0,$act_year-1);
      $eunix = mktime(0,0,0,1,0,$act_year);
    }
  }

  # last day/week/month/year
  if ( $range eq "last" ) {
    if ( $freq eq "daily" ) {
      $eunix = $act_time;
      $sunix = $eunix - $day_sec;
    }
    if ( $freq eq "weekly" ) {
      $eunix = $act_time;
      $sunix = $eunix - ($day_sec * 7);
    }
    if ( $freq eq "monthly" ) {
      $sunix = mktime($act_sec,$act_min,$act_hour,$act_day,$act_month-1,$act_year);
      $eunix = $act_time;
    }
    if ( $freq eq "yearly" ) {
      $sunix = mktime($act_sec,$act_min,$act_hour,$act_day,$act_month,$act_year-1);
      $eunix = $act_time;
    }
  }

  return($sunix,$eunix);
}

sub make_csv_report_lpar {
  my $report_name = shift;
  my $user_name   = shift;
  my $sunix       = shift;
  my $eunix       = shift;
  my $sdate       = shift;
  my $edate       = shift;
  my $time        = shift;

  my $report_name_space = $report_name;
  $report_name_space    =~ s/\s/_/g;


  #
  # Report variables
  #
  my $zip_report = 0;
  if ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'zipattach'} ) { $zip_report = 1; }
  print "zipattach      : $zip_report\n" if $debug == 9;

  if ( exists $cfg{'users'}{$user_name}{'csvDelimiter'} ) {
    $ENV{'CSV_SEPARATOR'} = $cfg{'users'}{$user_name}{'csvDelimiter'};
  }


  #
  # Report dir
  #
  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
  my $act_time2name  = sprintf( "%4d%02d%02d_%02d%02d%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );
  my $report_dir     = "$reportsdir/$user_name/$report_name_space";
  my $act_report_dir = "$report_dir/$act_time2name";

  if ( ! -d "$reportsdir/$user_name" ) {
    mkdir( "$reportsdir/$user_name", 0777 ) || error( "Cannot mkdir $reportsdir/$user_name: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }
  if ( ! -d $report_dir ) {
    mkdir( "$report_dir", 0777 ) || error( "Cannot mkdir $report_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }
  if ( ! -d $act_report_dir ) {
    mkdir( "$act_report_dir", 0777 ) || error( "Cannot mkdir $act_report_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }


  #
  # progress bar, inventory for all graphs
  #
  my %graphs = get_list_of_graphs("$user_name","$report_name");


  #
  # Make report / Create PNG
  #
  #print Dumper \%graphs;
  $done = 0;
  foreach my $graph_idx ( sort { $a <=> $b } keys %graphs ) {
    my $params = $graphs{$graph_idx};
    my ($csv_name, $query_string_act) = set_query_string_csv("$sunix","$eunix",\%$params);
    if ($csv_name && $query_string_act) {
      get_csv_stor("$act_report_dir/$csv_name","$query_string_act");
    }
    else {
      error( "Can't create QUERY_STRING for: user=$user_name,report=$report_name,metric=$params->{metric} : " . __FILE__ . ":" . __LINE__ );
      error( "Item parameters: " . encode_json(\%$params));
    }
    $done++;
    %status = (status => "pending", count => $count, done => $done);
    write_file( "/tmp/lrep-$pid.status", encode_json( \%status ) );
  }


  #
  # Zip files
  #
  my $zip_file_name = "$report_name_space-$act_time2name.zip";
  my $zip_file      = "$report_dir/$zip_file_name";
  my $ret           = zip_file("$zip_file","$user_name/$report_name_space/$act_time2name","$reportsdir");
  if ( ! -f $zip_file) {
    error( "Something wrong, zip file does not exists!" . __FILE__ . ":" . __LINE__ ) && return;
  }


  #
  # Send report via email
  #
  my $emails_count = 0;
  my %attachments;
  if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
    foreach ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
      if ( exists $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}) eq "ARRAY" ) {
        foreach ( @{ $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}} ) {
          $emails_count++;
        }
      }
    }
  }
  print "emails found   : $emails_count\n" if $debug == 9;
  if ( $emails_count > 0 ) {
    #
    # Set attachments
    #
    #if ( defined $zip_report && isdigit($zip_report) && $zip_report == 1 ) {
    if ( -f $zip_file ) { # always send a zip in case of csv
      open( ZIP, "<$zip_file" ) || error( "Couldn't open file $zip_file $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
      my $attachment_file = join("", <ZIP>);
      my $attachment_data = encode_base64($attachment_file);
      close(ZIP);

      $attachments{$zip_file_name}{'data'}         = $attachment_data;
      $attachments{$zip_file_name}{'Content-Type'} = "application/zip";

      print "attachment     : Content-Type=\"application/zip\",filename=\"$zip_file_name\"\n" if $debug == 9;
    #}
    }
    else {
      opendir(DIR, "$act_report_dir") || error(" directory does not exists : $act_report_dir".__FILE__.":".__LINE__) && exit 1;
      my @files = grep !/^\.\.?$/, readdir(DIR);
      closedir(DIR);

      foreach my $file (@files) {
        chomp $file;

        chdir("$act_report_dir") || error( "Couldn't change directory to $act_report_dir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;

        open( PNG, "<$file" ) || error( "Couldn't open file $file $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
        my $attachment_file = join("", <PNG>);
        my $attachment_data = encode_base64($attachment_file);
        close(PNG);

        $attachments{$file}{'data'}         = $attachment_data;
        $attachments{$file}{'Content-Type'} = "text/csv";

        print "attachment     : Content-Type=\"text/csv\",filename=\"$file\"\n" if $debug == 9;
      }
    }
    my $subject   = "$product: $report_name";
    my $mailprog  = "/usr/sbin/sendmail";
    my $message   = "Hello,\n\nyour report has been succesfully created!";
    my $msgbody   = "Hello,\n\nyour report has been succesfully created!";
    my $boundary  = "===" . time . "===";
    my $mail_from = "";

    if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
      foreach my $group ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} && $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} ne '' ) {
          $mail_from = $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'};
        }
        else {
          $mail_from = $mail_from_default;
        }
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}) eq "ARRAY" ) {
          foreach my $mail_to ( @{ $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}} ) {
            print "send email to  : group=\"$group\" email=\"$mail_to\" from=\"$mail_from\"\n" if $debug == 9;
            send_email( $mailprog, $mail_to, $mail_from, $subject, $message, $boundary, $msgbody, \%attachments );
          }
        }
      }
    }
  }


  #
  # Print location of output files
  #
  if ( -f $zip_file ) {
    print "download       : $zip_file\n" if $debug == 9;
    print "stored zip     : $zip_file\n" if $debug == 9;
  }
  else {
    error( "Something wrong, zip file does not exists!" . __FILE__ . ":" . __LINE__ );
  }
  if ( -d $act_report_dir ) {
    print "stored report  : $act_report_dir\n" if $debug == 9;
  }
  else {
    error( "Something wrong, report directory does not exists!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub get_csv_stor {
  my $csv          = shift;
  my $query_string = shift;

  $ENV{'QUERY_STRING'} = $query_string;

  chdir("$bindir") || error( "Couldn't change directory to $bindir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  print "create csv     : QUERY_STRING=$ENV{'QUERY_STRING'}\n" if $debug == 9;

  #my $ret = `$perl $bindir/lpar2rrd-rep.pl 2>$reportsdir/error.log-tmp`;
  my $ret = `$perl $bindir/lpar2csv.pl 2>$reportsdir/error.log-tmp`;

  if ( -f "$reportsdir/error.log-tmp" ) {
    my $file_size = ( stat("$reportsdir/error.log-tmp") )[9];
    if ( isdigit($file_size) && $file_size > 0 ) {
      open( ERR, "<$reportsdir/error.log-tmp" ) || error( "Couldn't open file $reportsdir/error.log-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
      my @lines = <ERR>;
      close(ERR);

      foreach my $line (@lines) {
        chomp $line;
        error( "ERROR lpar2csv.pl: $line" );
      }
    }
    unlink("$reportsdir/error.log-tmp");
  }

  # remove html header
  my ( $header, $justcsv ) = ( split "\n\n", $ret, 2 );

  # write png without header
  if ( defined $justcsv && $justcsv ne '' ) {
    open( CSV, ">$csv" ) || error( "Couldn't open file $csv $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    print CSV "$justcsv";
    close(CSV);
  }

  # check if image was created
  if ( ! -f $csv ) {
    error( "CSV was not created! \"$csv\"" ) && return 0;
  }

  # check file size
  my $file_size = ( stat("$csv") )[9];
  if ( isdigit($file_size) && $file_size == 0 ) {
    error( "CSV file size is 0! Something wrong with csv creation. File \"$csv\" removed!" );
    unlink("$csv");
    return 0;
  }
  #print "PNG HEADER: $header\n";

  return 1;
}

sub find_pool_metrics {
  my $pool_id    = shift;
  my $rep_format = shift;

  my @metrics;

  #>->-"POOL" : {
  #>->->-"IMG" : {
  #>->->->-"ITEMS" : {
  #>->->->->-"pool" : ["pool", "trendpool", "pool-max", "trendpool-max", "lparagg"],
  #>->->->->-"shpool" : ["shpool", "trendshpool", "shpool-max", "trendshpool-max", "poolagg"],
  #>->->->->-"all" : ["pool", "trendpool", "pool-max", "trendpool-max", "lparagg", "shpool", "trendshpool", "shpool-max", "trendshpool-max", "poolagg"]
  #>->->->-},
  #>->->->-"AGGREGATES" : {
  #>->->->-}
  #>->->-},
  #>->->-"CSV" : {
  #>->->->-"ITEMS" : {
  #>->->->->-"pool" : ["pool", "pool-max", "lparagg"],
  #>->->->->-"shpool" : ["shpool", "shpool-max"],
  #>->->->->-"all" : ["pool", "pool-max", "lparagg", "shpool", "shpool-max"]
  #>->->->-},
  #>->->->-"AGGREGATES" : {
  #>->->->-}

  if ( defined $pool_id && defined $rep_format ) {
    if ( $pool_id eq "pool" ) { # CPU pool
      if ( $rep_format eq "PDF" || $rep_format eq "IMG" ) {
        @metrics = ("pool", "trendpool", "pool-max", "trendpool-max", "lparagg");
      }
      elsif ($rep_format eq "CSV" ) {
        @metrics = ("pool", "pool-max", "lparagg");
      }
    }
    elsif ( $pool_id =~ m/^SharedPool/ ) { # shpool
      if ( $rep_format eq "PDF" || $rep_format eq "IMG" ) {
        @metrics = ("shpool", "trendshpool", "shpool-max", "trendshpool-max", "poolagg");
      }
      elsif ($rep_format eq "CSV" ) {
        @metrics = ("shpool", "shpool-max");
      }
    }
  }

  return @metrics;
}

sub find_lpar_metrics {
  my $menu_char = shift;

  my @metrics;

  # A: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SAN', 'SAN_IOPS', 'SAN_RESP'],    // AIX or Linux on Power or VIOS without SEA
  # B: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN'],                                   // AIX or Linux on Power or VIOS without SEA without SAN
  # C: ['CPU'], // AIX or Linux on Power without OS agent
  # L: ['OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SAN', 'SAN_IOPS', 'SAN_RESP'],           // Linux OS agent or AIX without HMC CPU
  # M: ['OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN'],                                          // Linux OS agent or AIX or without HMC CPU & SAN
  # // I: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SAN', 'SAN_IOPS'], // AS400 (not implemented yet)
  # S: ['OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN'],                                          // Solaris (no HMC CPU, no SAN)
  # V: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SAN', 'SAN_IOPS', 'SAN_RESP', 'SEA'],    // VIOS with SEA
  # U: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SEA'],                                   // VIOS with SEA without SAN
  # W: ['OSCPU', 'MEM', 'PAGING1', 'PAGING2'],                                                 // WPAR (cpu,mem,pg only)
  # X: ['CPU', 'OSCPU', 'MEM', 'PAGING1', 'PAGING2', 'LAN', 'SAN', 'SAN_IOPS', 'SAN_RESP'],    // VMware VM

  if ( $menu_char eq "A" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2', 'san_resp'); }
  if ( $menu_char eq "B" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan'); }
  if ( $menu_char eq "C" ) { @metrics = ('lpar', 'lparmemalloc', 'trend'); }
  if ( $menu_char eq "L" ) { @metrics = ('oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2', 'san_resp'); }
  if ( $menu_char eq "M" ) { @metrics = ('oscpu', 'mem', 'pg1', 'pg2', 'lan'); }
  #if ( $menu_char eq "I" ) { @metrics = ('lpar', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2'); }
  if ( $menu_char eq "S" ) { @metrics = ('oscpu', 'mem', 'pg1', 'pg2', 'lan'); }
  if ( $menu_char eq "V" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2', 'san_resp', 'sea'); }
  if ( $menu_char eq "U" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'sea'); }
  if ( $menu_char eq "W" ) { @metrics = ('oscpu', 'mem', 'pg1', 'pg2'); }
  if ( $menu_char eq "X" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2', 'san_resp'); }
  if ( $menu_char eq "Y" ) { @metrics = ('lpar', 'lparmemalloc', 'trend', 'oscpu', 'mem', 'pg1', 'pg2', 'lan', 'san1', 'san2'); }

  return @metrics;
}

sub find_datastore_metrics {
  my $query_string = shift;

  $ENV{'QUERY_STRING'} = $query_string;

  chdir("$bindir") || error( "Couldn't change directory to $bindir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  #print "detail-cgi.pl  : QUERY_STRING=$ENV{'QUERY_STRING'}\n" if $debug == 9;

  my $ret = `$perl $bindir/detail-cgi.pl 2>$reportsdir/error.log-tmp`;

  if ( -f "$reportsdir/error.log-tmp" ) {
    my $file_size = ( stat("$reportsdir/error.log-tmp") )[9];
    if ( isdigit($file_size) && $file_size > 0 ) {
      open( ERR, "<$reportsdir/error.log-tmp" ) || error( "Couldn't open file $reportsdir/error.log-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
      my @lines = <ERR>;
      close(ERR);

      foreach my $line (@lines) {
        chomp $line;
        error( "ERROR detail-cgi.pl: $line" );
      }
    }
    unlink("$reportsdir/error.log-tmp");
  }

  my @metrics;

  my @out = ( split "\n", $ret );
  foreach my $line (@out) {
    if ( $line =~ m/<li/ && $line =~ m/<\/li>/ && $line =~ m/#tabs-[0-9]+/ ) {
      if ( $line =~ m/>SPACE</ )    { push(@metrics, "dsmem"); }
      if ( $line =~ m/>DATA</ )     { push(@metrics, "dsrw"); }
      if ( $line =~ m/>IOPS</ )     { push(@metrics, "dsarw"); }
      if ( $line =~ m/>IOPS\/VM</ ) { push(@metrics, "ds-vmiops"); }
      if ( $line =~ m/>Latency</ )  { push(@metrics, "dslat"); }
    }
    if ( $line =~ m/<\/ul>/ ) { last; }
  }

  return @metrics;
}

sub text_item {
  my $item = shift;
  my $type = shift;
  my $text = $item;

  # lpar metrics
  if ( $item eq "lpar" )                { $text = "CPU"; }
  if ( $item eq "trend" )               { $text = "CPU trend"; }
  if ( $item eq "oscpu" )               { $text = "CPU OS"; }
  if ( $item eq "mem" )                 { $text = "Memory"; }
  if ( $item eq "pg1" )                 { $text = "Paging 1"; }
  if ( $item eq "pg2" )                 { $text = "Paging 2"; }
  if ( $item eq "lan" )                 { $text = "LAN"; }
  if ( $item eq "san1" )                { $text = "SAN"; }
  if ( $item eq "san2" )                { $text = "SAN IOPS"; }
  if ( $item eq "san_resp" )            { $text = "SAN RESP"; }
  if ( $item eq "sea" )                 { $text = "SEA"; }
  if ( $item eq "lparmemalloc" )        { $text = "Memory allocation"; }
  # pool metrics
  if ( $item eq "pool" )                { $text = "CPU"; }
  if ( $item eq "trendpool" )           { $text = "CPU trend"; }
  if ( $item eq "pool-max" )            { $text = "CPU max"; }
  if ( $item eq "trendpool-max" )       { $text = "CPU max trend"; }
  if ( $item eq "lparagg" )             { $text = "LPARs aggregated"; }
  if ( $item eq "shpool" )              { $text = "CPU"; }
  if ( $item eq "trendshpool" )         { $text = "CPU trend"; }
  if ( $item eq "shpool-max" )          { $text = "CPU max"; }
  if ( $item eq "trendshpool-max" )     { $text = "CPU max trend"; }
  if ( $item eq "poolagg" )             { $text = "LPARs aggregated"; }
  # memory metrics
  if ( $item eq "memalloc" )            { $text = "Allocation"; }
  if ( $item eq "trendmemalloc" )       { $text = "Allocation trend"; }
  if ( $item eq "memaggreg" )           { $text = "Aggregated"; }
  # custom groups
  if ( $item eq "custom" )              { $text = "CPU"; }
  if ( $item eq "custom_cpu_trend" )    { $text = "CPU trend"; }
  if ( $item eq "custommem" )           { $text = "MEM allocated"; }
  if ( $item eq "customosmem" )         { $text = "Memory"; }
  if ( $item eq "customoslan" )         { $text = "LAN"; }
  if ( $item eq "customossan1" )        { $text = "SAN"; }
  if ( $item eq "customossan2" )        { $text = "SAN IOPS"; }
  if ( $item eq "customvmmemactive" )   { $text = "MEM Active"; }
  if ( $item eq "customvmmemconsumed" ) { $text = "MEM Granted"; }
  if ( $item eq "customdisk" )          { $text = "DISK"; }
  if ( $item eq "customnet" )           { $text = "NET"; }
  # vm
  if ( $item eq "lpar" && defined $type && $type eq "VM" ) { $text = "CPU GHz"; }
  if ( $item eq "vmw-proc" )            { $text = "CPU"; }
  if ( $item eq "vmw-mem" )             { $text = "MEM"; }
  if ( $item eq "vmw-disk" )            { $text = "DISK"; }
  if ( $item eq "vmw-iops" )            { $text = "IOPS"; }
  if ( $item eq "vmw-net" )             { $text = "NET"; }
  if ( $item eq "vmw-swap" )            { $text = "SWAP"; }
  if ( $item eq "vmw-comp" )            { $text = "COMP"; }
  if ( $item eq "vmw-ready" )           { $text = "CPU READY"; }
  # esxi
  if ( $item eq "lparagg"   && defined $type && $type eq "ESXI" ) { $text = "VMs aggregated"; }
  if ( $item eq "memalloc"  && defined $type && $type eq "ESXI" ) { $text = "Memory allocation"; }
  if ( $item eq "memaggreg" && defined $type && $type eq "ESXI" ) { $text = "Memory aggregated"; }
  # respool
  if ( $item eq "rpcpu" )               { $text = "CPU"; }
  if ( $item eq "rplpar" )              { $text = "CPU VMs"; }
  if ( $item eq "rpmem" )               { $text = "MEMORY"; }
  # datastore
  if ( $item eq "dsmem" )               { $text = "SPACE"; }
  if ( $item eq "dsrw" )                { $text = "DATA"; }
  if ( $item eq "dsarw" )               { $text = "IOPS"; }
  if ( $item eq "ds-vmiops" )           { $text = "IOPS VMs"; }
  if ( $item eq "dslat" )               { $text = "Latency"; }
  # cluster
  if ( $item eq "clustcpu" )            { $text = "CPU"; }
  if ( $item eq "clustlpar" )           { $text = "CPU VMs"; }
  if ( $item eq "clustmem" )            { $text = "MEMORY"; }
  if ( $item eq "clustser" )            { $text = "SERVER"; }
  if ( $item eq "clustlpardy" )         { $text = "CPU READY"; }
  # oVirt
  if ( $item eq "ovirt_storage_domains_total_aggr_data" )    { $text = "Data"; }
  if ( $item eq "ovirt_storage_domains_total_aggr_latency" ) { $text = "Latency"; }
  if ( $item eq "ovirt_cluster_aggr_host_cpu_core" )         { $text = "CPU"; }
  if ( $item eq "ovirt_cluster_aggr_host_cpu_percent" )      { $text = "CPU %"; }
  if ( $item eq "ovirt_cluster_aggr_host_mem_used" )         { $text = "MEM used"; }
  if ( $item eq "ovirt_cluster_aggr_host_mem_free" )         { $text = "MEM free"; }
  if ( $item eq "ovirt_cluster_aggr_host_mem_used_percent" ) { $text = "MEM used %"; }
  if ( $item eq "ovirt_cluster_aggr_vm_cpu_core" )           { $text = "VM CPU"; }
  if ( $item eq "ovirt_cluster_aggr_vm_cpu_percent" )        { $text = "VM CPU %"; }
  if ( $item eq "ovirt_cluster_aggr_vm_mem_used" )           { $text = "VM MEM used"; }
  if ( $item eq "ovirt_cluster_aggr_vm_mem_free" )           { $text = "VM MEM free"; }
  if ( $item eq "ovirt_cluster_aggr_vm_mem_used_percent" )   { $text = "VM MEM used %"; }
  if ( $item eq "ovirt_host_cpu_core" )                      { $text = "CPU"; }
  if ( $item eq "ovirt_host_cpu_percent" )                   { $text = "CPU %"; }
  if ( $item eq "ovirt_host_mem" )                           { $text = "MEM"; }
  if ( $item eq "ovirt_host_nic_aggr_net" )                  { $text = "Net"; }
  if ( $item eq "ovirt_host_nic_net" )                       { $text = "Net"; }
  if ( $item eq "ovirt_vm_cpu_core" )                        { $text = "CPU"; }
  if ( $item eq "ovirt_vm_cpu_percent" )                     { $text = "CPU %"; }
  if ( $item eq "ovirt_vm_mem" )                             { $text = "MEM"; }
  if ( $item eq "ovirt_vm_aggr_net" )                        { $text = "Net"; }
  if ( $item eq "ovirt_vm_aggr_data" )                       { $text = "Data"; }
  if ( $item eq "ovirt_vm_aggr_latency" )                    { $text = "Latency"; }
  if ( $item eq "ovirt_storage_domain_space" )               { $text = "Space"; }
  if ( $item eq "ovirt_storage_domain_aggr_data" )           { $text = "Data"; }
  if ( $item eq "ovirt_storage_domain_aggr_latency" )        { $text = "Latency"; }
  if ( $item eq "ovirt_disk_data" )                          { $text = "Data"; }
  if ( $item eq "ovirt_disk_latency" )                       { $text = "Latency"; }
  if ( $item eq "custom_ovirt_vm_cpu_percent" )              { $text = "CPU %"; }
  if ( $item eq "custom_ovirt_vm_cpu_core" )                 { $text = "CPU cores"; }
  if ( $item eq "custom_ovirt_vm_memory_used" )              { $text = "MEM used"; }
  if ( $item eq "custom_ovirt_vm_memory_free" )              { $text = "MEM free"; }
  if ( $item eq "custom-xenvm-cpu-percent" )                 { $text = "CPU %"; }
  if ( $item eq "custom-xenvm-cpu-cores" )                   { $text = "CPU"; }
  if ( $item eq "custom-xenvm-memory-used" )                 { $text = "MEM used"; }
  if ( $item eq "custom-xenvm-memory-free" )                 { $text = "MEM free"; }
  if ( $item eq "custom-xenvm-vbd" )                         { $text = "Data"; }
  if ( $item eq "custom-xenvm-vbd-iops" )                    { $text = "IOPS"; }
  if ( $item eq "custom-xenvm-vbd-latency" )                 { $text = "Latency"; }
  if ( $item eq "custom-xenvm-lan" )                         { $text = "Net"; }
  # solaris
  if ( $item eq "s_ldom_c" )                                 { $text = "CPU"; }
  if ( $item eq "s_ldom_m" )                                 { $text = "MEM"; }

  return $text;
}

sub translate_shpool_id {
  my $server = shift;
  my $id     = shift;
  my $name   = $id;

  $id =~ s/^SharedPool//;

  opendir( DIR, "$wrkdir/$server" ) || error( "can't opendir $wrkdir/$server: $! :" . __FILE__ . ":" . __LINE__ ) && next;
  my @hmc_list = grep !/^\.\.?$/, readdir(DIR);
  closedir(DIR);
  chomp @hmc_list;

  foreach my $hmc (@hmc_list) {
    if ( -f "$wrkdir/$server/$hmc/cpu-pools-mapping.txt" ) {
      open( POOL, "< $wrkdir/$server/$hmc/cpu-pools-mapping.txt" ) || error( "Couldn't open file $wrkdir/$server/$hmc/cpu-pools-mapping.txt $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
      my @cfg = <POOL>;
      close(POOL);
      chomp @cfg;

      foreach my $line (@cfg) {
        my ($id_n, $name_n) = split(",", $line);
        if ( $id eq $id_n ) {
          return $name_n;
        }
      }
    }
  }

  return $name;
}

sub make_pdf_report_lpar {
  my $report_name = shift;
  my $user_name   = shift;
  my $sunix       = shift;
  my $eunix       = shift;
  my $sdate       = shift;
  my $edate       = shift;
  my $time        = shift;

  my $report_name_space = $report_name;
  $report_name_space    =~ s/\s/_/g;


  #
  # Report variables
  #
  my $png_height_orig   = 150;
  my $png_width_orig    = 620;
  my $png_height        = $png_height_orig;
  my $png_width         = $png_width_orig;
  my $detail            = 10;


  #
  # Report dir
  #
  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
  my $act_time2name  = sprintf( "%4d%02d%02d_%02d%02d%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );
  my $report_dir     = "$reportsdir/$user_name/$report_name_space";

  if ( ! -d "$reportsdir/$user_name" ) {
    mkdir( "$reportsdir/$user_name", 0777 ) || error( "Cannot mkdir $reportsdir/$user_name: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }
  if ( ! -d $report_dir ) {
    mkdir( "$report_dir", 0777 ) || error( "Cannot mkdir $report_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }


  #
  # progress bar, inventory for all graphs
  #
  my %graphs = get_list_of_graphs("$user_name","$report_name");


  #
  # Make report / Create PNG
  #
  my $pdf_name = "$report_name_space-$act_time2name.pdf";
  my %pdf_sections;

  if ( -d "$report_dir/PDF-tmp" ) {
    remove_files("$report_dir/PDF-tmp");
  }
  if ( ! -d "$report_dir/PDF-tmp" ) {
    mkdir( "$report_dir/PDF-tmp", 0777 ) || error( "Cannot mkdir $report_dir/PDF-tmp: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }

  #print Dumper \%graphs;
  $done = 0;
  foreach my $graph_idx ( sort { $a <=> $b } keys %graphs ) {
    my $params = $graphs{$graph_idx};
    my ($img_name, $query_string_act, $host, $subsys, $name, $metric) = set_query_string_img("$png_height","$png_width","$detail","$sunix","$eunix","$time",\%$params);
    if ($img_name && $query_string_act) {
      get_img_stor("$report_dir/PDF-tmp/$img_name","$query_string_act");
      $pdf_sections{$host}{$subsys}{$name}{$done}{$metric}{"$report_dir/PDF-tmp/$img_name"} = "$report_dir/PDF-tmp/$img_name";
    }
    else {
      error( "Can't create QUERY_STRING for: user=$user_name,report=$report_name,metric=$params->{metric} : " . __FILE__ . ":" . __LINE__ );
      error( "Item parameters: " . encode_json(\%$params));
    }
    $done++;
    %status = (status => "pending", count => $count, done => $done);
    write_file( "/tmp/lrep-$pid.status", encode_json( \%status ) );
  }

  print "create PDF     : $report_dir/$pdf_name\n" if $debug == 9;
  create_pdf("$pdf_name","$report_dir/$pdf_name","$report_dir/PDF-tmp","$png_height_orig","$png_width_orig","$sdate","$edate","$report_name",\%pdf_sections);

  # remove PDF dir
  if ( -d "$report_dir/PDF-tmp" ) {
    remove_files("$report_dir/PDF-tmp");
  }
  if ( ! -f "$report_dir/$pdf_name" ) {
    error( "Something wrong! PDF does not exists! \"$report_dir/$pdf_name\" $!" . __FILE__ . ":" . __LINE__ ) && return;
  }


  #
  # Send report via email
  #
  my $emails_count = 0;
  if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
    foreach ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
      if ( exists $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}) eq "ARRAY" ) {
        foreach ( @{ $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}} ) {
          $emails_count++;
        }
      }
    }
  }
  print "emails found   : $emails_count\n" if $debug == 9;
  if ( $emails_count > 0 && -f "$report_dir/$pdf_name" ) {
    # send email with attached pdf file
    my $subject   = "$product: $report_name";
    my $mailprog  = "/usr/sbin/sendmail";
    my $message   = "Hello,\n\nyour report has been succesfully created!";
    my $msgbody   = "Hello,\n\nyour report has been succesfully created!";
    my $boundary  = "===" . time . "===";
    my $mail_from = "";

    open( PDF, "<$report_dir/$pdf_name" ) || error( "Couldn't open file $report_dir/$pdf_name $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    my $attachment_file = join("", <PDF>);
    my $attachment_data = encode_base64($attachment_file);
    close(PDF);

    my %attachments;
    $attachments{$pdf_name}{'data'}         = $attachment_data;
    $attachments{$pdf_name}{'Content-Type'} = "application/pdf";

    print "attachment     : Content-Type=\"application/pdf\",filename=\"$pdf_name\"\n" if $debug == 9;

    if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
      foreach my $group ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} && $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} ne '' ) {
          $mail_from = $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'};
        }
        else {
          $mail_from = $mail_from_default;
        }
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}) eq "ARRAY" ) {
          foreach my $mail_to ( @{ $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}} ) {
            print "send email to  : group=\"$group\" email=\"$mail_to\" from=\"$mail_from\"\n" if $debug == 9;
            send_email( $mailprog, $mail_to, $mail_from, $subject, $message, $boundary, $msgbody, \%attachments );
          }
        }
      }
    }
  }


  #
  # Print location of output files
  #
  if ( -f "$report_dir/$pdf_name" ) {
    print "download       : $report_dir/$pdf_name\n" if $debug == 9;
    print "stored report  : $report_dir/$pdf_name\n" if $debug == 9;
  }
  else {
    error( "Something wrong! PDF does not exists! \"$report_dir/$pdf_name\" $!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub create_pdf {
  my $pdf_name    = shift;
  my $pdf_file    = shift;
  my $png_dir     = shift;
  my $png_height  = shift;
  my $png_width   = shift;
  my $sdate       = shift;
  my $edate       = shift;
  my $report_name = shift;
  my $sect_hash   = shift;
  my %sections    = %{$sect_hash};

  #use PDF::API2;
  eval 'use PDF::API2; 1';


  use constant mm   => 25.4 / 72;    # 25.4 mm in an inch, 72 points in an inch
  use constant in   => 1 / 72;       # 72 points in an inch
  use constant pt   => 1;            # 1 point
  use constant A4_x => 210 / mm;     # x points in an A4 page ( 595.2755 )
  use constant A4_y => 297 / mm;     # y points in an A4 page ( 841.8897 )

  use constant US_x => 216 / mm;     # x points in an US letter page ( 612.2834 )
  use constant US_y => 279 / mm;     # y points in an US letter page ( 790.8661 )

  my $format  = "A4";
  my $pagetop = 796;
  my $pagey   = A4_y;
  my $pagex   = A4_x;

  my $act_time    = localtime();
  my $header_text = "$sdate - $edate";
  #my $header_text = $report_name;

  # Create a blank PDF file
  my $pdf = PDF::API2->new();
  $pdf->info(
    'CreationDate' => $act_time,
    'ModDate'      => $act_time,
    'Creator'      => "reporter.pl",
    'Title'        => "$product report",
    'Subject'      => ""
  );

  my $logo_file = "";
  if (defined $basedir && $basedir ne '' ) {
    if ( $product eq "LPAR2RRD" ) {
      if ( -f "$basedir/html/css/images/logo-$product_lc.png") {
        $logo_file = "$basedir/html/css/images/logo-$product_lc.png";
      }
      else {
        error("Logo file is not exist! \"$basedir/html/css/images/logo-$product_lc.png\" ".__FILE__.":".__LINE__) && exit 1;
      }
    }
  }
  else {
    error("basedir is not defined!".__FILE__.":".__LINE__) && exit 1;
  }
  my $logo  = $pdf->image_png("$logo_file");
  my $font  = $pdf->corefont( 'Helvetica',      -encode => "utf8" );
  my $fontb = $pdf->corefont( 'Helvetica-Bold', -encode => "utf8" );

  my $outline_root = $pdf->outlines;

  # add graphs to pdf page
  my $png_idx = 0;
  if (%sections) {
    foreach my $host ( sort keys( %sections ) ) {

      my $sect = $outline_root->outline();
      $sect->title("$host");

      foreach my $subsys ( keys( %{ $sections{$host} } ) ) {

        my $page = "";
        if ( $png_idx == 0 ) {
          $page    = add_new_page($pdf,$format,$pagey,$pagex,$logo,$fontb,$font,$header_text,$host,$report_name); # add report name to header only to 1st page
          $pagetop = 768;
        }
        else {
          $page = add_new_page($pdf,$format,$pagey,$pagex,$logo,$fontb,$font,$header_text,$host);
          $pagetop = 796;
        }
        my $content = $page->gfx();
        my $cntr    = $pagetop;

        # bookmarks
        my $outline = $sect->outline();
        $outline->title("$subsys");
        $outline->dest($page);

        foreach my $name ( sort keys( %{ $sections{$host}{$subsys} } ) ) {
          # bookmarks
          my $outline_last  = $outline->outline();
          my $name2bookmark = $name;
          if ( $name =~ m/^SharedPool[0-9]+$/ ) { # translate shared pool id to human name
            $name2bookmark = translate_shpool_id("$host","$name");
          }

          my $graph_idx = 0;
          foreach my $item_idx ( sort { $a <=> $b } keys( %{ $sections{$host}{$subsys}{$name} } ) ) {
            foreach my $item ( keys( %{ $sections{$host}{$subsys}{$name}{$item_idx} } ) ) {
              foreach my $png ( keys( %{ $sections{$host}{$subsys}{$name}{$item_idx}{$item} } ) ) {
                $png_idx++;
                $graph_idx++;

                if ( ! -f $png ) {
                  error("Graph for PDF not found! \"$png\" :".__FILE__.":".__LINE__) && next;
                }

                my $dpng    = $pdf->image_png($png);
                my $rwidth  = $dpng->width();
                my $rheight = $dpng->height();

                my $zoomfactor = ( 595 - 60 ) / $rwidth;

                # $zoomfactor = 0.500;
                my $height_png = $rheight * $zoomfactor;
                my $width_png  = $rwidth * $zoomfactor;

                if ( $png_height > $pagetop ) {
                  $zoomfactor = ( $pagetop - 30 ) / $rheight;
                  $height_png = $rheight * $zoomfactor;
                  $width_png  = $rwidth * $zoomfactor;

                  # print STDERR "PDF generator: image was too big to fit in the page, shrinking...\n";
                  # next;
                } ## end if ( $height > $pagetop)

                if ( ( $cntr - $height_png ) < 24 ) {
                  $page    = add_new_page($pdf,$format,$pagey,$pagex,$logo,$fontb,$font,$header_text,$host);
                  $pagetop = 796;
                  $content = $page->gfx();
                  $cntr    = $pagetop;
                }

                if ( $graph_idx == 1 ) { # add bookmark for LPAR only at first page
                  # bookmarks
                  #$outline_last = $outline->outline();
                  $outline_last->title("$name2bookmark");
                  $outline_last->dest($page);
                }

                 # bookmarks
                 my $outline_item = $outline_last->outline();
                 my $item_text_human = text_item($item,$subsys);
                 $outline_item->title("$item_text_human");
                 $outline_item->dest($page);

                $content->image( $dpng, 30, $cntr - $height_png, $zoomfactor );

                # close $fh;
                $cntr -= $height_png;
              }
              if ( $png_idx == 0 ) {
                error("Files for PDF not found! :".__FILE__.":".__LINE__);
              }
            }
          }
        }
      }
    }
  }

  # Save the PDF
  $pdf->saveas("$pdf_file");

  return 1;
}

sub add_new_page {
  my $pdf         = shift;
  my $format      = shift;
  my $pagey       = shift;
  my $pagex       = shift;
  my $logo        = shift;
  my $fontb       = shift;
  my $font        = shift;
  my $header_text = shift;
  my $section     = shift;
  my $report_name = shift;

  my $page = $pdf->page();
  $page->mediabox($format);

  my $count    = $pdf->pages();
  my $grey_box = $page->gfx(1);
  if ( defined $report_name && $report_name ne '' ) {
    $grey_box->fillcolor('#0B2F3A');
  }
  else {
    $grey_box->fillcolor('#555');
  }
  $grey_box->strokecolor('#222');
  $grey_box->rect(
    80 * mm,    # left
    $pagey - ( 130 * mm ),    # bottom
    $pagex - ( 140 * mm ),    # width
    70 * mm                 # height
  );

  if ( defined $report_name && $report_name ne '' ) {
    my $grey_box2 = $page->gfx(1);
    $grey_box2->fillcolor('#555');
    $grey_box2->strokecolor('#222');
    $grey_box2->rect(
      80 * mm,    # left
      $pagey - ( 205 * mm ),    # bottom
      $pagex - ( 140 * mm ),    # width
      70 * mm                 # height
    );

    $grey_box2->fill;
  }

  $grey_box->fill;

  my $logo_box = $page->gfx();
  $logo_box->image( $logo, 12, 12, 0.35 );

  my $prod_link = $page->annotation();
  my $prod_url  = "http://www.$product_lc.com";
  my %options   = (
    -border => [ 0,  0,  0 ],
    -rect   => [ 12, 12, 80, 30 ]
  );
  $prod_link->url( $prod_url, %options );

  # Add some text to the page
  my $text = $page->text();
  #$text->font( $fontb, 15 );
  $text->font( $fontb, 10 );
  $text->fillcolor("white");
  if ( defined $report_name && $report_name ne '' ) {
    $text->translate( 365, $pagey - 64 );
  }
  else {
    $text->translate( 365, $pagey - 38 );
  }
  $text->text($header_text);

  # Add some text to the page
  my $text_section = $page->text();
  #$text_section->font( $fontb, 15 );
  $text_section->font( $fontb, 12 );
  $text_section->fillcolor("white");
  if ( defined $report_name && $report_name ne '' ) {
    $text_section->translate( 40, $pagey - 64 );
  }
  else {
    $text_section->translate( 40, $pagey - 38 );
  }
  $text_section->text($section);

  # report name in the first page header
  if ( defined $report_name && $report_name ne '' ) {
    my $text_rep = $page->text();
    $text_rep->font( $fontb, 15 );
    $text_rep->fillcolor("white");
    $text_rep->translate( 40, $pagey - 39 );
    $text_rep->text($report_name);
  }

  my $act_date = localtime();
  my $paragraph = "Generated from $product version $ENV{version} $act_date                                                              Page $count";
  my $footer = $page->text;
  $footer->textstart;
  $footer->lead(7);
  $footer->font( $font, 7 );
  $footer->fillcolor('navy');
  #$footer->translate( 560, 14 );
  $footer->translate( 550, 24 );
  $footer->section( "$paragraph", 400, 16, -align => "right" );

  return $page;
}

sub make_img_report_lpar {
  my $report_name = shift;
  my $user_name   = shift;
  my $sunix       = shift;
  my $eunix       = shift;
  my $time        = shift;

  my $report_name_space = $report_name;
  $report_name_space    =~ s/\s/_/g;


  #
  # Report variables
  #
  my $zip_report = 0;
  my $png_height = 150;
  my $png_width  = 620;
  my $detail     = 10;

  if ( defined $ENV{RRDHEIGHT} && isdigit($ENV{RRDHEIGHT}) && defined $ENV{RRDWIDTH} && isdigit($ENV{RRDWIDTH}) ) {
    $png_height = $ENV{RRDHEIGHT};
    $png_width  = $ENV{RRDWIDTH};
  }
  if ( $cfg{'users'}{$user_name}{'reports'}{$report_name}{'zipattach'} ) { $zip_report        = 1; }
  print "zipattach      : $zip_report\n" if $debug == 9;


  #
  # Report dir
  #
  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime();
  my $act_time2name  = sprintf( "%4d%02d%02d_%02d%02d%02d", $year + 1900, $month + 1, $day, $hour, $min, $sec );
  my $report_dir     = "$reportsdir/$user_name/$report_name_space";
  my $act_report_dir = "$report_dir/$act_time2name";

  if ( ! -d "$reportsdir/$user_name" ) {
    mkdir( "$reportsdir/$user_name", 0777 ) || error( "Cannot mkdir $reportsdir/$user_name: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }
  if ( ! -d $report_dir ) {
    mkdir( "$report_dir", 0777 ) || error( "Cannot mkdir $report_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }
  if ( ! -d $act_report_dir ) {
    mkdir( "$act_report_dir", 0777 ) || error( "Cannot mkdir $act_report_dir: $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  }


  #
  # progress bar, inventory for all graphs
  #
  my %graphs = get_list_of_graphs("$user_name","$report_name");


  #
  # Make report / Create PNG
  #
  #print Dumper \%graphs;
  $done = 0;
  foreach my $graph_idx ( sort { $a <=> $b } keys %graphs ) {
    my $params = $graphs{$graph_idx};
    my ($img_name, $query_string_act) = set_query_string_img("$png_height","$png_width","$detail","$sunix","$eunix","$time",\%$params);
    if ($img_name && $query_string_act) {
      get_img_stor("$act_report_dir/$img_name","$query_string_act");
    }
    else {
      error( "Can't create QUERY_STRING for: user=$user_name,report=$report_name,metric=$params->{metric} : " . __FILE__ . ":" . __LINE__ );
      error( "Item parameters: " . encode_json(\%$params));
    }
    $done++;
    %status = (status => "pending", count => $count, done => $done);
    write_file( "/tmp/lrep-$pid.status", encode_json( \%status ) );
  }


  #
  # Zip file
  #
  my $zip_file_name = "$report_name_space-$act_time2name.zip";
  my $zip_file      = "$report_dir/$zip_file_name";
  my $ret           = zip_file("$zip_file","$user_name/$report_name_space/$act_time2name","$reportsdir");
  if ( ! -f $zip_file) {
    error( "Something wrong, zip file does not exists!" . __FILE__ . ":" . __LINE__ ) && return;
  }


  #
  # Send report via email
  #
  my $emails_count = 0;
  my %attachments;
  if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
    foreach ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
      if ( exists $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}) eq "ARRAY" ) {
        foreach ( @{ $cfg{'users'}{$user_name}{'groups'}{$_}{'emails'}} ) {
          $emails_count++;
        }
      }
    }
  }
  print "emails found   : $emails_count\n" if $debug == 9;
  if ( $emails_count > 0 ) {
    #
    # Set attachments
    #
    if ( defined $zip_report && isdigit($zip_report) && $zip_report == 1 ) {
      if ( -f $zip_file ) {
        open( ZIP, "<$zip_file" ) || error( "Couldn't open file $zip_file $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
        my $attachment_file = join("", <ZIP>);
        my $attachment_data = encode_base64($attachment_file);
        close(ZIP);

        $attachments{$zip_file_name}{'data'}         = $attachment_data;
        $attachments{$zip_file_name}{'Content-Type'} = "application/zip";

        print "attachment     : Content-Type=\"application/zip\",filename=\"$zip_file_name\"\n" if $debug == 9;
      }
    }
    else { # send email with attached single graphs
      opendir(DIR, "$act_report_dir") || error(" directory does not exists : $act_report_dir".__FILE__.":".__LINE__) && exit 1;
      my @files = grep !/^\.\.?$/, readdir(DIR);
      closedir(DIR);

      foreach my $file (@files) {
        chomp $file;

        chdir("$act_report_dir") || error( "Couldn't change directory to $act_report_dir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;

        open( PNG, "<$file" ) || error( "Couldn't open file $file $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
        my $attachment_file = join("", <PNG>);
        my $attachment_data = encode_base64($attachment_file);
        close(PNG);

        $attachments{$file}{'data'}         = $attachment_data;
        $attachments{$file}{'Content-Type'} = "image/png";

        print "attachment     : Content-Type=\"image/png\",filename=\"$file\"\n" if $debug == 9;
      }
    }
    my $subject   = "$product: $report_name";
    my $mailprog  = "/usr/sbin/sendmail";
    my $message   = "Hello,\n\nyour report has been succesfully created!";
    my $msgbody   = "Hello,\n\nyour report has been succesfully created!";
    my $boundary  = "===" . time . "===";
    my $mail_from = "";

    if ( exists $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} && ref($cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'}) eq "ARRAY" ) {
      foreach my $group ( @{ $cfg{'users'}{$user_name}{'reports'}{$report_name}{'recipients'} } ) {
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} && $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'} ne '' ) {
          $mail_from = $cfg{'users'}{$user_name}{'groups'}{$group}{'mailfrom'};
        }
        else {
          $mail_from = $mail_from_default;
        }
        if ( exists $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'} && ref($cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}) eq "ARRAY" ) {
          foreach my $mail_to ( @{ $cfg{'users'}{$user_name}{'groups'}{$group}{'emails'}} ) {
            print "send email to  : group=\"$group\" email=\"$mail_to\" from=\"$mail_from\"\n" if $debug == 9;
            send_email( $mailprog, $mail_to, $mail_from, $subject, $message, $boundary, $msgbody, \%attachments );
          }
        }
      }
    }
  }


  #
  # Print location of output files
  #
  if ( -f $zip_file ) {
    print "download       : $zip_file\n" if $debug == 9;
    print "stored zip     : $zip_file\n" if $debug == 9;
  }
  else {
    error( "Something wrong, zip file does not exists!" . __FILE__ . ":" . __LINE__ );
  }
  if ( -d $act_report_dir ) {
    print "stored report  : $act_report_dir\n" if $debug == 9;
  }
  else {
    error( "Something wrong, report directory does not exists!" . __FILE__ . ":" . __LINE__ );
  }

  return 1;
}

sub set_query_string_img {
  my $png_height = shift;
  my $png_width  = shift;
  my $detail     = shift;
  my $sunix      = shift;
  my $eunix      = shift;
  my $time       = shift;
  my $params     = shift;

  my $query_string_act = "";
  my $img_name         = "";
  my $hmc              = "";
  my $host             = "";
  my $subsys           = "";
  my $name             = "";
  my $name2bookmark    = "";

  my $metric = $params->{'metric'};

  #
  # IBM POWER
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "POWER" && exists $params->{'subsys'} && exists $params->{'hmc'} && exists $params->{'host'} && exists $params->{'name'} ) {
    $hmc           = $params->{'hmc'};
    $host          = $params->{'host'};
    $subsys        = $params->{'subsys'};
    $name          = $params->{'name'};
    $name2bookmark = $params->{'name'};
    $img_name      = "$host\_$subsys\_$name\_$metric.png";
    # LPAR graphs : lpar (hmc cpu), oscpu, mem, pg1, pg2, lan, san1, san2
    if ( $params->{'subsys'} eq "LPAR" ) {
      if ( $metric eq "trend" ) { # lpar cpu (hmc) trend
        #host=hmc&server=P02DR%5F%5F9117-MMC-SN44K8102&lpar=Accept&item=trend&time=y&type_sam=m&detail=0&upper=0.635&entitle=0&none=-1540542675
        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=y&type_sam=m&detail=$detail&height=$png_height&width=$png_width";
      }
      else {
        if ( $metric eq "lpar" ) {
          $img_name = "$host\_$subsys\_$name\_cpu.png";
        }
        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=$time&type_sam=m&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
      }
    }
    # CPU pool, shared pool : pool, lparagg
    if ( $params->{'subsys'} eq "POOL" ) {
      if ( $name   eq "pool" ) {
        $img_name      = "$host\_$subsys\_CPUPOOL_$metric.png";
        $name2bookmark = "CPU POOL";
      }
      if ( $name =~ m/^SharedPool[0-9]+$/ ) { # translate shared pool id to human name
        my $shpool_n = translate_shpool_id("$host","$name");
        $img_name      = "$host\_$subsys\_$shpool_n\_$metric.png";
      }
      if ( $metric =~ m/^trend/ ) {
        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=y&type_sam=m&detail=$detail&height=$png_height&width=$png_width";
      }
      else {
        if ( $metric eq "lparagg" ) { $name = "pool-multi"; }

        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=$time&type_sam=m&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
      }
    }
    # SERVER memory
    if ( $params->{'subsys'} eq "SERVER" ) {
      if ( $metric eq "memalloc" || $metric eq "memaggreg" || $metric eq "trendmemalloc" )  {
        $name          = "cod";
        $name2bookmark = "MEMORY";
        $img_name      = "$host\_$subsys\_MEMORY_$metric.png";
      }
      if ( $metric eq "pagingagg" ) {
        $name          = "cod";
        $name2bookmark = "PAGING";
        $img_name      = "$host\_$subsys\_PAGING_$metric.png";
      }
      if ( $metric eq "trendmemalloc" ) {
        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=y&type_sam=m&detail=$detail&height=$png_height&width=$png_width";
      }
      else {
        $query_string_act = "host=" . urlencode("$hmc") . "&server=" . urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&time=$time&type_sam=m&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
      }
    }
  }

  #
  # CUSTOM GROUPS
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "CUSTOM" && exists $params->{'subsys'} && exists $params->{'host'} ) {
    $host          = "na";
    $hmc           = "na";
    $subsys        = $params->{'subsys'};
    $name          = $params->{'host'};
    $name2bookmark = $params->{'host'};
    $img_name      = "CUSTOM-GROUP_$name\_$metric.png";

    if ( $params->{'subsys'} eq "VM" )          { $hmc = "VMware"; }
    if ( $params->{'subsys'} eq "OVIRTVM" )     { $hmc = "oVirt"; }
    if ( $params->{'subsys'} eq "XENVM" )       { $hmc = "XenServer"; }
    if ( $params->{'subsys'} eq "SOLARISLDOM" ) { $hmc = "Solaris"; }
    if ( $params->{'subsys'} eq "SOLARISZONE" ) { $hmc = "Solaris"; }

    if ( $metric eq "custom_cpu_trend" ) {
      $query_string_act = "host=$hmc&server=$host&lpar=" . urlencode("$name") . "&item=$metric&time=y&type_sam=m&detail=$detail&height=$png_height&width=$png_width";
    }
    else {
      $query_string_act = "host=$hmc&server=$host&lpar=" . urlencode("$name") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
    # add following info to bookmarks
    $host = "CUSTOM GROUPS";
  }

  #
  # VMWARE
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "VMWARE" && exists $params->{'subsys'} ) {
    if ( $params->{'subsys'} eq "VM" ) {
      $host          = "$params->{'vcenter'} - $params->{cluster}";
      $subsys        = $params->{'subsys'};
      $name2bookmark = $params->{'vm_name'};
      $img_name      = "$params->{vcenter}\_$params->{cluster}\_$subsys\_$params->{'vm_name'}\_$metric.png";

      $query_string_act = "host=" . urlencode("$params->{'host'}") . "&server=" . urlencode("$params->{'server'}") . "&lpar=" . urlencode("$params->{'vm_uuid'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
    if ( $params->{'subsys'} eq "ESXI" ) {
      $host          = "$params->{'vcenter'} - $params->{cluster}";
      $subsys        = $params->{'subsys'};
      $name2bookmark = $params->{'server'};
      $img_name      = "$params->{vcenter}\_$params->{cluster}\_$subsys\_$params->{'server'}\_$metric.png";
      my $type_sam   = "na";
      if ( $metric eq "pool" ) { $type_sam = "m"; }

      $query_string_act = "host=" . urlencode("$params->{'host'}") . "&server=" . urlencode("$params->{'server'}") . "&lpar=" . urlencode("$params->{'lpar'}") . "&item=$metric&time=$time&type_sam=$type_sam&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
    if ( $params->{'subsys'} eq "RESPOOL" ) {
      $host          = "$params->{'vcenter'} - $params->{cluster}";
      $subsys        = $params->{'subsys'};
      $name2bookmark = $params->{'respool'};
      $img_name      = "$params->{vcenter}\_$params->{cluster}\_$subsys\_$params->{'respool'}\_$metric.png";

      $query_string_act = "host=" . urlencode("$params->{'host'}") . "&server=" . urlencode("$params->{'server'}") . "&lpar=" . urlencode("$params->{'lpar'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
    if ( $params->{'subsys'} eq "DATASTORE" ) {
      $host          = "$params->{'vcenter'} - $params->{datacenter}";
      $subsys        = $params->{'subsys'};
      $name2bookmark = $params->{'datastore'};
      $img_name      = "$params->{vcenter}\_$subsys\_$params->{'datastore'}\_$metric.png";

      $query_string_act = "host=" . urlencode("$params->{'host'}") . "&server=" . urlencode("$params->{'server'}") . "&lpar=" . urlencode("$params->{'lpar'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
    if ( $params->{'subsys'} eq "CLUSTER" ) {
      $host          = "$params->{'vcenter'} - $params->{cluster}";
      $subsys        = $params->{'subsys'};
      $name2bookmark = $params->{'cluster'};
      $img_name      = "$params->{vcenter}\_$subsys\_$params->{'cluster'}\_$metric.png";

      $query_string_act = "host=" . urlencode("$params->{'host'}") . "&server=" . urlencode("$params->{'server'}") . "&lpar=" . urlencode("$params->{'lpar'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
    }
  }

  #
  # OVIRT
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "OVIRT" && exists $params->{'subsys'} && exists $params->{'datacenter'} && exists $params->{'host'} ) {
    $host          = (exists $ovirt_inventory{TRANSLATE_UUID}{ $params->{'datacenter'} }) ? $ovirt_inventory{TRANSLATE_UUID}{ $params->{'datacenter'} } : $params->{'datacenter'};
    #$subsys        = (exists $ovirt_inventory{TRANSLATE_UUID}{ $params->{'host'} }) ? "$params->{'subsys'} - $ovirt_inventory{TRANSLATE_UUID}{ $params->{'host'} }" : "$params->{'subsys'} - $params->{'host'}";
    $subsys        = $params->{'subsys'};
    $name2bookmark = (exists $ovirt_inventory{TRANSLATE_UUID}{ $params->{'uuid'} }) ? $ovirt_inventory{TRANSLATE_UUID}{ $params->{'uuid'} } : $params->{'uuid'};
    $img_name      = "$host\_$subsys\_$name2bookmark\_$metric.png";

    $query_string_act = "host=oVirt&server=nope&lpar=" . urlencode("$params->{'uuid'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
  }

  #
  # SOLARIS
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "SOLARIS" && exists $params->{'subsys'} && exists $params->{'name'} ) {
    $host            = "Solaris";
    $subsys          = $params->{'subsys'};
    $name2bookmark   = $params->{'name'};
    my $metric_label = text_item($metric);
    $img_name        = "$host\_$subsys\_$name2bookmark\_$metric_label.png";

    #http://10.22.11.103/lpar2rrd-cgi/detail-graph.sh?host=0&server=Solaris--unknown&lpar=t8-1mvbc-l3&item=s_ldom_c&time=d&type_sam=m&detail=1&upper=-1&entitle=0&none=none
    $query_string_act = "host=0&server=Solaris--unknown&lpar=" . urlencode("$params->{'name'}") . "&item=$metric&time=$time&type_sam=na&detail=$detail&sunix=$sunix&eunix=$eunix&entitle=0&height=$png_height&width=$png_width";
  }


  $img_name =~ s/\//&&1/g;

  return("$img_name","$query_string_act","$host","$subsys","$name2bookmark","$metric");
}

sub set_query_string_csv {
  my $sunix      = shift;
  my $eunix      = shift;
  my $params     = shift;

  my $query_string_act = "";
  my $csv_name         = "";
  my $hmc              = "";
  my $host             = "";
  my $subsys           = "";
  my $name             = "";

  my $metric = $params->{'metric'};

  # convert timestamp
  my ( undef, undef, $shour, $sday, $smon, $syear, undef, undef, undef ) = localtime($sunix);
  $syear = $syear + 1900;
  $smon  = $smon + 1;
  my ( undef, undef, $ehour, $eday, $emon, $eyear, undef, undef, undef ) = localtime($eunix);
  $eyear = $eyear + 1900;
  $emon  = $emon + 1;

  # IBM POWER
  if ( exists $params->{'group'} && $params->{'group'} eq "POWER" && exists $params->{'subsys'} && exists $params->{'hmc'} && exists $params->{'host'} && exists $params->{'name'} ) {
    $hmc      = $params->{'hmc'};
    $host     = $params->{'host'};
    $subsys   = $params->{'subsys'};
    $name     = $params->{'name'};
    $csv_name = "$host\_$subsys\_$name\_$metric.csv";
    # LPAR graphs
    if ( $params->{'subsys'} eq "LPAR" ) {
      if ( $metric eq "lpar" ) { $csv_name = "$host\_$subsys\_$name\_cpu.csv"; }
      $query_string_act = "subsys=$params->{'subsys'}&platform=POWER&hmc=" . urlencode("$hmc") . "&server=" .  urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&sunix=$sunix&eunix=$eunix";
    }
    # CPU pool, shared pool, server memory, server paging
    if ( $params->{'subsys'} eq "POOL" ) {
      my $name2csv_file_name = $name;
      if ( $name eq "pool" ) { $name2csv_file_name = "CPUPOOL"; }
      if ( $name =~ m/^SharedPool[0-9]+$/ ) { # translate shared pool id to human name
        $name2csv_file_name = translate_shpool_id("$host","$name");
      }
      $csv_name = "$host\_$subsys\_$name2csv_file_name\_$metric.csv";
      if ( $metric eq "pool" || $metric eq "shpool" ) { $csv_name = "$host\_$subsys\_$name2csv_file_name\_cpu.csv"; }
      if ( $metric eq "pool-max" || $metric eq "shpool-max" ) { $csv_name = "$host\_$subsys\_$name2csv_file_name\_cpu-max.csv"; }
      $query_string_act = "subsys=$params->{'subsys'}&platform=POWER&hmc=" . urlencode("$hmc") . "&server=" .  urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&sunix=$sunix&eunix=$eunix";
    }
    # SERVER memory
    if ( $params->{'subsys'} eq "SERVER" ) {
      if ( $metric eq "memalloc" || $metric eq "memaggreg" ) {
        $csv_name         = "$host\_$subsys\_MEMORY_$metric.csv";
        $query_string_act = "subsys=$params->{'subsys'}&platform=POWER&hmc=" . urlencode("$hmc") . "&server=" .  urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&sunix=$sunix&eunix=$eunix";
      }
    }
  }
  #
  # CUSTOM GROUPS
  #
  if ( exists $params->{'group'} && $params->{'group'} eq "CUSTOM" && exists $params->{'host'} && exists $params->{'subsys'} ) {
    $host     = "na";
    $hmc      = "na";
    $name     = $params->{'host'};
    my $metric2filename = text_item($metric,$params->{'subsys'});
    $metric2filename =~ s/\s/_/g;
    $csv_name = "CUSTOM-GROUP_$name\_$metric2filename.csv";

    $query_string_act = "subsys=$params->{'group'}-$params->{'subsys'}&platform=POWER&hmc=" . urlencode("$hmc") . "&server=" .  urlencode("$host") . "&lpar=" . urlencode("$name") . "&item=$metric&sunix=$sunix&eunix=$eunix";
  }

  # set SAMPLE_RATE if exists
  if ( defined $params->{'sample_rate'} && isdigit($params->{'sample_rate'}) ) {
    $query_string_act .= "&sample_rate=$params->{'sample_rate'}";
  }

  $csv_name =~ s/\//&&1/g;

  return("$csv_name","$query_string_act");
}

sub get_img_stor {
  my $img          = shift;
  my $query_string = shift;

  $ENV{'PICTURE_COLOR'} = "FFF";
  $ENV{'QUERY_STRING'}  = $query_string;

  chdir("$bindir") || error( "Couldn't change directory to $bindir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
  print "create graph   : QUERY_STRING=$ENV{'QUERY_STRING'}\n" if $debug == 9;

  my $ret = `$perl $bindir/detail-graph-cgi.pl 2>$reportsdir/error.log-tmp`;

  if ( -f "$reportsdir/error.log-tmp" ) {
    my $file_size = ( stat("$reportsdir/error.log-tmp") )[9];
    if ( isdigit($file_size) && $file_size > 0 ) {
      open( ERR, "<$reportsdir/error.log-tmp" ) || error( "Couldn't open file $reportsdir/error.log-tmp $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
      my @lines = <ERR>;
      close(ERR);

      foreach my $line (@lines) {
        chomp $line;
        error( "ERROR detail-graph-cgi.pl: $line" );
      }
    }
    unlink("$reportsdir/error.log-tmp");
  }

  # remove html header
  my ( $header, $justpng ) = ( split "\n\n", $ret, 2 );

  # write png without header
  if ( defined $justpng && $justpng ne '' ) {
    open( PNG, ">$img" ) || error( "Couldn't open file $img $!" . __FILE__ . ":" . __LINE__ ) && exit 1;
    print PNG "$justpng";
    close(PNG);
  }

  # check if image was created
  if ( ! -f $img ) {
    error( "Graph was not created! \"$img\"" ) && return 0;
  }

  # check file size
  my $file_size = ( stat("$img") )[9];
  if ( isdigit($file_size) && $file_size == 0 ) {
    error( "Graph file size is 0! Something wrong with graph creation. File \"$img\" removed!" );
    unlink("$img");
    return 0;
  }
  #print "PNG HEADER: $header\n";

  return 1;
}

sub remove_files {
  my $dir = shift;

  if ( -d $dir ) {
    opendir(DIR, "$dir") || error(" directory does not exists : $dir ".__FILE__.":".__LINE__) && exit 1;
    my @files = grep !/^\.\.?$/, readdir(DIR);
    closedir(DIR);

    foreach my $file (@files) {
      chomp $file;

      if ( -f "$dir/$file" ) {
        print "remove file    : $dir/$file\n" if $debug == 9;
        unlink("$dir/$file");
      }
      if ( -d "$dir/$file" ) {
        opendir(DIR, "$dir/$file") || error(" directory does not exists : $dir/$file ".__FILE__.":".__LINE__) && exit 1;
        my @files_2 = grep !/^\.\.?$/, readdir(DIR);
        closedir(DIR);

        foreach my $file_2 (@files_2) {
          chomp $file_2;
          if ( -f "$dir/$file/$file_2" ) {
            print "remove file    : $dir/$file/$file_2\n" if $debug == 9;
            unlink("$dir/$file/$file_2");
          }
        }
        print "remove dir     : $dir/$file\n" if $debug == 9;
        rmdir("$dir/$file");
      }
    }
    print "remove dir     : $dir\n" if $debug == 9;
    rmdir("$dir");
  }

  return 1;
}

sub urlencode {
  my $s = shift;
  $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
}

sub urldecodel {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  return $s;
}

sub send_email {
  my ( $mailprog, $email_to, $email, $subject, $message, $boundary, $msgbody, $attachment_data ) = @_;

  my %attachments = %$attachment_data;

  if ( ! -f $mailprog ) {
    error( "Alerting: emailing program missing \"$mailprog\", install it " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  if ( open MAIL, "|$mailprog -t" ) {
    print MAIL "To: $email_to\n";
    print MAIL "From: $email\n";
    print MAIL "Subject: $subject\n";

    print MAIL "MIME-Version: 1.0\n";
    print MAIL "Content-Type: multipart/mixed; boundary=\"$boundary\"\n\n";
    print MAIL "--$boundary\n";
    print MAIL "Content-Type: text/plain; charset=utf-8;\n\n";
    print MAIL "$msgbody\n\n\n";

    foreach my $filename ( keys(%attachments) ) {
      if ( defined $attachments{$filename}{'data'} && $attachments{$filename}{'data'} ne '' && defined $attachments{$filename}{'Content-Type'} && $attachments{$filename}{'Content-Type'} ne '' ) {
        print MAIL "--$boundary\n";
        print MAIL "Content-Type: $attachments{$filename}{'Content-Type'};";
        print MAIL "name=\"$filename\"\n";
        print MAIL "Content-Transfer-Encoding: base64\n";
        print MAIL "Content-Disposition: attachment;";
        print MAIL "filename=\"$filename\"\n\n";
        print MAIL "$attachments{$filename}{'data'}\n";
      }
    }
    print MAIL "--$boundary--\n";
    close MAIL;
  }

  return 1;
}

sub translate_name_to_id {
  my $host     = shift;
  my $type     = shift;
  my $name     = shift;
  my $st_type  = shift;
  my $cfg_file = shift;
  my $id       = $name;

  if ( -f $cfg_file ) {
    open( CFG, "< $cfg_file" ) || error( "Can't open $cfg_file : $! " . __FILE__ . ":" . __LINE__ ) && exit 1;
    my @cfg = <CFG>;
    close(CFG);

    if ( $type eq "NODE" || $type eq "VOLUME-FOLDER" || $type eq "PORT" || $type eq "NAS" || $type eq "FS" || $type eq "RANK" || $type eq "IO-GROUP" || $type eq "NAS" || $type eq "TIER" || $type eq "HOST-GROUP" || $type eq "CATALYST" || ($type eq "POOL" && $st_type eq "VMAX") ) {
      if ( $name =~ m/\(/ || $name =~ m/\)/ ) { # INFINIBOX has got PORT names like: N3ETH1 (ETH)
        $name =~ s/\(/\\(/g;
        $name =~ s/\)/\\)/g;
      }
      my ($cfg_line) = grep {/:$name$/} @cfg;
      if ( defined $cfg_line && $cfg_line ne '' ) {
        my ($id_new, undef) = split(/:/, $cfg_line);
        if ( -f "$wrkdir/$host/$type/$id_new.rrd" ) {
          $id = $id_new;
        }
        else {
          if ( $type eq "RANK" && $st_type eq "SWIZ" ) {
            my ($rrd_file) = <$wrkdir/$host/$type/$id_new-P*.rrd>;
            if ( defined $rrd_file && $rrd_file ne '' ) {
              $id = $id_new;
            }
          }
        }
      }
    }
    if ( $type eq "DRIVE" || ($type eq "CPU-NODE" && $st_type eq "STOREONCE") ) {
      my ($cfg_line) = grep {/,$name,/} @cfg;
      if ( defined $cfg_line && $cfg_line ne '' ) {
        my ($id_new, undef) = split(/,/, $cfg_line);
        if ( -f "$wrkdir/$host/$type/$id_new.rrd" ) {
          $id = $id_new;
        }
      }
      else {
        ($cfg_line) = grep {/,$name$/} @cfg;
        if ( defined $cfg_line && $cfg_line ne '' ) {
          my ($id_new, undef) = split(/,/, $cfg_line);
          if ( -f "$wrkdir/$host/$type/$id_new.rrd" ) {
            $id = $id_new;
          }
        }
      }
    }

  }

  return $id;
}

sub translate_port_id {
  my $port_name = shift;
  my $phys_cfg  = shift;

  my $port_name_new = $port_name;

  if ( -f $phys_cfg ) {
    open( FHR, "< $phys_cfg" ) || error( "Can't open $phys_cfg : $! " . __FILE__ . ":" . __LINE__ ) && exit 1;
    my @phys_ports = <FHR>;
    close(FHR);

    my $port_line = "";
    ($port_line) = grep {/ : $port_name$/} @phys_ports;
    if ( defined $port_line && $port_line ne '' ) {
      ( $port_name_new, undef ) = split( " : ", $port_line );
    }
  }

  return $port_name_new;
}

sub alias_to_name {
  my $host     = shift;
  my $subsys   = shift;
  my $name     = shift;
  my $name_new = $name;

  if ( exists $aliases{$subsys}{$host}{$name}{'ID'} && $aliases{$subsys}{$host}{$name}{'ID'} ne '' ) {
    $name_new = $aliases{$subsys}{$host}{$name}{'ID'};
    if ( $subsys eq "SANPORT" ) {
      $name_new =~ s/^port//;
    }
    print "alias found    : $subsys:$host:$name_new:$name\n" if $debug == 9;
  }

  return $name_new;
}

sub zip_file {
  my $file_out = shift;
  my $files_in = shift;
  my $dir      = shift;

  chdir("$dir") || error( "Couldn't change directory to $dir $!" . __FILE__ . ":" . __LINE__ ) && exit 1;

  my $ZipError     = "";
  my $module_found = 1;

  eval 'use IO::Compress::Zip qw(zip $ZipError)';

  if ( $@ ) {
    $module_found = 0;
  }

  if ( $module_found == 1 ) {
    print "zip report     : $files_in/* to $file_out\n" if $debug == 9;

    if ( -d $files_in ) {
      my @files = <$files_in/*>;
      zip(\@files => "$file_out") or error("zip failed: $ZipError") && return 0;
    }
    else {
      error("Directory \"$files_in\" not exists!") && return 0;
    }
  }

  return 1;
}

sub is_dir_empty {
  my $dir = shift;

  if ( -f $dir ) {
    opendir(DIR, "$dir") || error ("can't opendir $dir: $! :".__FILE__.":".__LINE__) && return 0;
    my @files = grep !/^\.\.?$/, readdir(DIR);
    closedir(DIR);

    my $count = @files;
    if ( $count > 0 ) {
      return 0;
    }
    else {
      return 1;
    }
  }

  # not exists
  return 3;
}

sub isdigit {
  my $digit = shift;

  if ( !defined($digit) || $digit eq '' ) {
    return 0;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/^\+//;
  $digit_work =~ s/e\+//;
  $digit_work =~ s/e\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  # NOT a number
  return 0;
}

sub file_time_diff {
  my $file = shift;

  my $act_time  = time();
  my $file_time = $act_time;
  my $time_diff = 0;

  if ( -f $file ) {
    $file_time = ( stat($file) )[9];
    $time_diff = $act_time - $file_time;
  }

  return ($time_diff);
} ## end sub file_time_diff

sub message {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "$act_time: $text\n";

  return 1;
}

sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  #print "$act_time: $text : $!\n" if $DEBUG > 2;;

  return 1;
} ## end sub error

sub write_file {
  my $file = shift;

  if ( $API_RUN == 0 && $file eq "\/tmp\/lrep-$pid.status" ) { return 1; } # do not create status file if reports are generated from load.sh

  open IO, ">$file" or die "Cannot open $file for output: $!\n";
  print IO @_;
  close IO;

  return 1;
}
