#!/usr/bin/perl

use strict;
use warnings;
use CGI::Carp qw(fatalsToBrowser);
use MIME::Base64 qw(encode_base64 decode_base64);
use Data::Dumper;
use File::Copy;
use File::Temp qw/ tempfile/;
use File::Path 'rmtree';
use JSON;
use File::Basename;

use lib "../bin";
use ReporterCfg;
use ACL;

my $useacl        = ACL::useACL;
my $aclAdminGroup = ACL::getAdminGroup;
my $basedir  = $ENV{INPUTDIR};
my $bindir   = "$basedir/bin";
my $perl     = $ENV{PERL};

# print STDERR "$useacl  $aclAdminGroup\n";

my $json = JSON->new->utf8->pretty;

sub file_write {
  my $file = shift;
  open IO, ">$file" or die "Cannot open $file for output: $!\n";
  print IO @_;
  close IO;
} ## end sub file_write

sub file_write_append {
  my $file = shift;
  if ( -f $file ) {
    open IO, ">>$file" or die "Cannot open $file for output: $!\n";
  }
  else {
    open IO, ">$file" or die "Cannot open $file for output: $!\n";
  }
  print IO @_;
  close IO;
} ## end sub file_write_append

sub file_read {
  my $file = shift;
  open IO, $file or die "Cannot open $file for input: $!\n";
  my @data = <IO>;
  close IO;
  wantarray ? @data : join( '' => @data );
} ## end sub file_read

sub trim {
  my $s = shift;
  if ($s) {
    $s =~ s/^\s+|\s+$//g;
  }
  return $s;
} ## end sub trim

# get URL parameters (could be GET or POST) and put them into hash %PAR
my ( $buffer, @pairs, $pair, $name, $value, %PAR );

if ( !defined $ENV{'REQUEST_METHOD'} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";

  #{
  #local $/ = undef;   # required for re-read of encode_json pretty output
  #my $cfg = decode_json(file_read("users.json"));
  #print Dumper $cfg;
  #}
  exit;
}

if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
  read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
}
else {
  $buffer = $ENV{'QUERY_STRING'};
}

# Split information into name/value pairs
@pairs = split( /&/, $buffer );
foreach $pair (@pairs) {
  ( $name, $value ) = split( /=/, $pair );
  $value =~ tr/+/ /;
  $value =~ s/%(..)/pack("C", hex($1))/eg;
  if ( exists $PAR{$name} ) {
    if ( ref $PAR{$name} eq "ARRAY" ) {
      push @{ $PAR{$name} }, $value;
    }
    else {
      my $firstVal = $PAR{$name};
      $PAR{$name} = [];
      push @{ $PAR{$name} }, $firstVal;
      push @{ $PAR{$name} }, $value;
    } ## end else [ if ( ref $PAR{$name} eq...
  }
  else {
    $PAR{$name} = $value;
  }
} ## end foreach $pair (@pairs)

# print STDERR Dumper \%PAR;

if ( !defined $PAR{cmd} ) {
  print "Content-type: text/plain\n\n";
  print "No command specified, exiting...";
  exit;
}

if ( $PAR{cmd} eq "generate" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";

  my $repname = $PAR{repname};
  my $username = $PAR{user};

  $| = 1;

  if ( ! defined ( my $pid = fork() ) ) {
    warn "Fork failed (parent PID: $$";
  }
  elsif ( $pid != 0 ) {
    my %res = (success => \1, pid => $pid);
    print encode_json ( \%res );
    exit;
  } else {

    close (STDIN);
    close (STDOUT);
    close (STDERR);

    my $reportsdir   = "$basedir/reports";

    if ( ! -d $reportsdir ) {
      umask 0000;
      mkdir( "$reportsdir", 0777 ) || print STDERR ( "Cannot mkdir $reportsdir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
    }

    my $olog = "$reportsdir/reporter-GUI.output.log";
    my $elog = "$reportsdir/reporter-GUI.error.log";

    my $chpid = $$;

    my $oof = "/tmp/lrep-$chpid.out.log";
    my $eef = "/tmp/lrep-$chpid.err.log";

    {
      local @ARGV = ( $username, $repname, "RUNFROMGUI" );
      open local(*STDOUT), '>', $oof;
      open local(*STDERR), '>', $eef;
      {
        no warnings;
        open local(*STDIN), '<', '/dev/null';
      }
      do "$bindir/reporter.pl";
    }
    exit;
  }
}
elsif ( $PAR{cmd} eq "modtest" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";
  my $errors  = "";
  my @reqmods = qw(IO::Compress::Zip);

  for my $mod (@reqmods) {
    eval {
      ( my $file = $mod ) =~ s|::|/|g;
      require $file . '.pm';
      $mod->import();
      1;
    } or do {
      $errors .= "$@";
      }
  } ## end for my $mod (@reqmods)

  if ($errors) {
    result( 0, "ZIP module test", $errors );
  }
  else {
    result( 1, "ZIP module test" );

  } ## end else [ if ($errors) ]

} ## end elsif ( $PAR{cmd} eq "modtest")
elsif ( $PAR{cmd} eq "json" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";

  # print Dumper \%ENV;
  # print Dumper \%PAR;
  my $cfg = ReporterCfg::getRawConfig($aclAdminGroup);
  print $cfg;

}
elsif ( $PAR{cmd} eq "form" ) {    ### Get list of credentials

  my %cfg = ReporterCfg::getConfig($aclAdminGroup);
  my $user = "admin";
  if ($ENV{REMOTE_USER}) {
    $user = $ENV{REMOTE_USER}
  }
  my $csv_delim = "";
  if ($cfg{users}{$user}{csvDelimiter}) {
    $csv_delim = $cfg{users}{$user}{csvDelimiter};
  }
  print "Content-type: text/html\n\n";

    # TABs
    print <<_MARKER_;
<div id='hiw'><a href='http://www.lpar2rrd.com/reporter.htm' target='_blank'><img src='css/images/help-browser.gif' alt='Reporter help page' title='Reporter help page'></a></div>
<div id='tabs' style='text-align: center;'>
  <ul>
    <li><a href='#tabs-1'>Definitions</a></li>
    <li><a href='#tabs-2'>Groups</a></li>
    <li><a href='#tabs-3'>Options</a></li>
    <li><a href='/lpar2rrd-cgi/reporter.sh?cmd=history'>History</a></li>
  </ul>
<div id='tabs-1' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none' class="cggrpnames">
      <table id="reptable" class="cfgtree">
        <thead>
        <tr>
          <th>Report name <button id="addrep">New</button></th>
          <th>Edit</th>
          <th>Clone</th>
          <th>Run</th>
          <th>Delete</th>
          <th>Format</th>
          <th>Recurrence</th>
          <th>Next run</th>
          <th>Recipient group</th>
        </tr>
        </thead>
        <tbody>
        </tbody>
      </table>
    </div>
    <br style="clear: both">
    <div id="moderr" style="max-width: 80em; font-size: smaller; text-align: left; margin-left: 1em;"></div>
    <div id="freeinfo" style="max-width: 80em; font-size: smaller; text-align: left; margin-left: 1em; display: none">
    <p>You are using the <a href="http://www.lpar2rrd.com/support.htm">free edition</a> of LPAR2RRD.<br>
    It is restricted to only one report with an unlimited number of items at a time.<br>
    Automated reporting is disabled. You can run the report only manually.<br>
    In order to change the output format (PNG/PDF/CSV), delete your current report and create a new one.</p>
    </div>
    <div id="repexamples" style="max-width: 80em; font-size: smaller; text-align: left; margin-left: 1em; display: none">
      <h4 style="margin-bottom: 4px;">DEMO site: examples of above defined reports<br>You can generate it on the fly by selecting "Manual run"</h4>
      <table border="0">
        <tr><td>LPARs CPU weekly pdf<a href="http://www.lpar2rrd.com/userfiles/reports/P8LPARsweeklyCPU-20181022_083133.pdf" target="_blank"><img src="css/images/pdf.png"></a></td></tr>
        <tr><td>CPU POOLs weekly pdf<a href="http://www.lpar2rrd.com/userfiles/reports/P8CPUPOOLweekly-20181022_092909.pdf" target="_blank"><img src="css/images/pdf.png"></a></td></tr>
        <tr><td>LPARs SAN monthly pdf<a href="http://www.lpar2rrd.com/userfiles/reports/P8LPARmonthlySAN-20181022_083847.pdf" target="_blank"><img src="css/images/pdf.png"></a></td></tr>
        <tr><td>CPU POOLs daily CSV<a href="http://www.lpar2rrd.com/userfiles/reports/P8CPUPOOLdailyCSV-20181022_084608.zip" target="_blank"><img src="css/images/zip.png"></a></td></tr>
        <tr><td>VIOS all yearly png<a href="http://www.lpar2rrd.com/userfiles/reports/P8VIOSyearly-20181022_084258.zip" target="_blank"><img src="css/images/zip.png"></a></td></tr>
      </table>
  </div>

  <br style="clear: both">
  <p class="notice">This feature is not supported yet for XenServer, Solaris and MS Windows server/Hyper-V. It will come in future releases. <br>oVirt/RHV is supported partially.</p>
  <pre>
  <div id='aclfile' style='text-align: left; margin: auto; background: #fcfcfc; border: 1px solid #c0ccdf; border-radius: 10px; padding: 15px; display: none; overflow: auto'></div>
  </pre>
</div>
<div id='tabs-2' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none' class="cggrpnames">
      <table id="repgrptable" class="cfgtree">
        <thead>
        <th><button id="addgrp">New group</button></th>
        <th>Description</th>
        <th>E-mails</th>
        <th></th>
        <!--th><button id="cgcfg-help-button" title="Help on usage">?</button></th-->
        </tr>
        </thead>
        <tbody>
        </tbody>
      </table>
  </div>
</div>
<div id='tabs-3' style='display: inline-block'>
  <div style='float: left; margin-right: 10px; outline: none'>
    <fieldset class="cggrpnames">
      <div>
        <label for="csv_delim">CSV delimiter&nbsp;</label>
        <input id="csv_delim" name="CSV_DELIMITER" class="text medium" type="text" size="1" maxlength="1" title="Delimiter used in CSV reports" value="$csv_delim">
      </div>
    </fieldset>
    <div style="text-align: center">
      <button style='font-weight: bold; margin-top: 1em' name='saverepcfg' class='saverepcfg'>Save configuration</button>
    </div>
  </div>
</div>
<style>
#optform  {
  display: table;
}
#optform div {
  display: table-row;
}
#optform label {
  display: table-cell;
}
#optform input {
  display: table-cell;
}
</style>
</div>
_MARKER_

}
elsif ( $PAR{cmd} eq "history" ) {    ### generated reports
  print "Content-type: text/html\n\n";
  my $username   = $useacl ? $ENV{REMOTE_USER} : "admin";
  my $reportsdir = "$basedir/reports/$username";
  if ( -d $reportsdir ) {
  print <<_MARKER_;
<div style='text-align: center;'>
  <div style='display: inline-block; outline: none'>
    <table id='rephistory' class=''>
    <thead>
    <tr><th class='group-text'>Report name</th><th>Filename</th><th class='group-false sorter-metric data-metric-name-abbr="B"'>Size&nbsp;&nbsp;</th><th class='group-date-monthyear'>Generated</th><th class='sorter-false'></th></tr>
    </thead>
    <tbody>
_MARKER_
    my $search = list("$basedir/reports/$username");
    # print Dumper $search;
    find( $search, '\.(pdf|zip)$');
    print "</tbody>\n";
    print "</table>";
    print "</div></div>";
  }
}
elsif ( $PAR{cmd} eq "remrep" ) {    ### remove single report
  my $username   = $useacl ? $ENV{REMOTE_USER} : "admin";
  my $reportsdir = "$basedir/reports/$username";

  print "Content-type: application/json\n\n";
  if ( $ENV{DEMO} ) {
    my %res = (success => \0, log => "You cannot remove reports on DEMO site!");
    print encode_json ( \%res );
    exit;
  }
  my $repname = "$reportsdir/$PAR{repname}";
  my $repfile = "$repname/$PAR{repfile}";
  # warn $repfile;
  if ( -f $repfile ) {
    if ( ! unlink $repfile ) {
      my %res = (success => \0, log => $!);
      print encode_json ( \%res );
      exit;
    }
    my ( $srcdir ) = $repfile =~ /(\d{8}_\d{6})\..*$/;
    $srcdir = "$repname/$srcdir";
    # warn $srcdir;
    if ( -d $srcdir ) {
      if ( ! rmtree $srcdir ) {
        my %res = (success => \0, log => $!);
        print encode_json ( \%res );
        exit;
      }
    }
  }
  my %res = (success => \1);
  print encode_json ( \%res );
}
elsif ( $PAR{cmd} eq "remdir" ) {    ### remove all history of report $PAR{repname}
  my $username   = $useacl ? $ENV{REMOTE_USER} : "admin";
  my $reportsdir = "$basedir/reports/$username";

  print "Content-type: application/json\n\n";
  if ( $ENV{DEMO} ) {
    my %res = (success => \0, log => "You cannot remove reports on DEMO site!");
    print encode_json ( \%res );
    exit;
  }
  my $repname = "$reportsdir/$PAR{repname}";
  if ( -d $repname ) {
    if ( ! rmtree $repname ) {
      my %res = (success => \0, log => $!);
      print encode_json ( \%res );
      exit;
    }
  }
  my %res = (success => \1);
  print encode_json ( \%res );
}
elsif ( $PAR{cmd} eq "saveall" ) {    ### Save configuration as is
  my $basedir = $ENV{INPUTDIR};
  $basedir ||= "..";

  my $cfgdir = "$basedir/etc/web_config";

  my $tmpdir = "$basedir/tmp";
  if ( defined $ENV{TMPDIR_LPAR} ) {
    $tmpdir = $ENV{TMPDIR_LPAR};
  }
  # print "Content-type: application/json\n\n";
  print "Content-type: text/html\n\n";
  if ( $PAR{acl} ) {
    my $retstr  = "";
    my %oldcfg  = ReporterCfg::getConfig();

    # if ( $ENV{'SERVER_NAME'} eq "demo.stor2rrd.com" ) {
    if ( $ENV{'DEMO'} ) {
      $retstr = "\"msg\" : \"<div>This demo site does not allow saving any changes you do in the admin GUI panel.</div>\"";
      $retstr .= ", \"cfg\" : \"\"";
      print "{ \"status\" : \"fail\", $retstr }";

    }
    # disable for now...
    elsif (( $useacl && $ENV{REMOTE_USER} ne $PAR{user} )) {
      print "{ \"status\" : \"fail\", \"msg\" : \"<div>You can only change your own report definitions!</div>\" }";
    #
    }
    elsif ( open( CFG, ">$cfgdir/reporter.json" ) ) {
      my $cfg     = $PAR{acl};
      my %new_usr_cfg = %{ decode_json($cfg) };
      # print STDERR Dumper \%new_usr_cfg;
      if ( $PAR{user} && $new_usr_cfg{users}{$PAR{user}} ) {
        $oldcfg{users}{ $PAR{user} } = $new_usr_cfg{users}{$PAR{user}};
      }
      print CFG $json->encode( \%oldcfg );
      close CFG;
      $cfg =~ s/\n/\\n/g;
      $cfg =~ s/\\:/\\\\:/g;
      $cfg =~ s/"/\\"/g;
      $retstr = "\"msg\" : \"<div>Reporter configuration has been succesfuly saved!<br /><br /></div>\", \"cfg\" : \"";
      print "{ \"status\" : \"success\", $retstr" . "$cfg\" }";
    }
    else {
      print "{ \"status\" : \"fail\", \"msg\" : \"<div>File $cfgdir/reporter.json cannot be written by webserver, check apache user permissions: <span style='color: red'>$!</span></div>\" }";
    }
  }
  else {
    print "{ \"status\" : \"fail\", \"msg\" : \"<div>No data was written to reporter.json</div>\" }";
  }

}
elsif ( $PAR{cmd} eq "grptree" ) {
  my %cfg = ReporterCfg::getConfig();
  print "Content-type: application/json\n\n";
  print "[";
  my $n1 = "";

  # print Dumper \%cfg;
  foreach my $grp ( sort keys %{ $cfg{groups} } ) {
    if ( $grp eq "$aclAdminGroup" ) {
      next;
    }
    print $n1 . "{\"title\":\"$grp\"}\n";
    $n1 = "\n,";
  }
  print "]\n";
}
elsif ( $PAR{cmd} eq "get" ) {    ### Get generated PDF
  print "Content-type: text/html\n";
  my $reportsdir   = "$basedir/reports";
  my ( $basename, $dirname ) = fileparse( $PAR{filename} );
  my $filename = "$PAR{filename}";
  $filename =~ s/\.\.//g;
  # $filename =~ //;
  $filename = "$reportsdir/$filename";

  if ( open( FILE, "<", "$filename" ) ) {
    print "Content-Disposition:attachment;filename=$basename\n\n";
    binmode FILE;
    while (<FILE>) {
      print $_;
    }
    close FILE;
  } ## end if ( open( PDF, "<", "/tmp/pdfgen.$PAR{id}.pdf"...))
  else {
    print "\n";
  }
} ## end elsif ( $PAR{cmd} eq "get")
elsif ( $PAR{cmd} eq "mailtest" ) {
  print "Content-type: application/json\n\n";
  my $errors  = "";
  my %cfg = ReporterCfg::getConfig($aclAdminGroup);
  my $user = "admin";
  if ($ENV{REMOTE_USER}) {
    $user = $ENV{REMOTE_USER}
  }

  my $group = $PAR{group};

  if (! $cfg{users}{$user}{groups}{$group}) {
    $errors = "No such group.";
  } elsif (! $cfg{users}{$user}{groups}{$group}{emails} ) {
    $errors = "No emails defined.";
  } else {
    for my $mail ( @{ $cfg{users}{$user}{groups}{$group}{emails} } ) {

      my $from = 'reporter@yourdomain.com';
      if ( $cfg{users}{$user}{groups}{$group}{mailfrom} ) {
        $from = $cfg{users}{$user}{groups}{$group}{mailfrom};
      }
      my $subject = 'Test E-mail';
      my $message = 'This is test email sent by Reporter';

      my $mailprog = "/usr/sbin/sendmail";
      if ( ! -f $mailprog ) {
        $errors = "Alerting: emailing program missing \"$mailprog\", install it " . __FILE__ . ":" . __LINE__ . "\n";
        print STDERR $errors;
      }
      else {
        open(MAIL, "|$mailprog -t");
        # Email Header
        print MAIL "To: $mail\n";
        print MAIL "From: $from\n";
        print MAIL "Subject: $subject\n\n";
        # Email Body
        print MAIL $message;

        close(MAIL);
        # print "Email Sent Successfully\n";
      }
    }
  }
  if ( $errors ) {
    result( 0, "E-mail test for group $group failed: <br> $errors <br><br>In case of any problem follow <a target='_blank' href='http://www.lpar2rrd.com/mail-troubleshooting.htm'>troubleshooting docu.</a>" );
  }
  else {
    result( 1, "E-mail test for group $group was completed, please check your mailbox. <br><br>In case of any problem follow <a target='_blank' href='http://www.lpar2rrd.com/mail-troubleshooting.htm'>troubleshooting docu.</a>");

  } ## end else [ if ($errors) ]
}

elsif ( $PAR{cmd} eq "stop" ) {
  print "Content-type: application/json\n\n";
  my $chpid = $PAR{pid};

  kill 'HUP', $chpid;
  my %status = (status => "terminated");
  file_write( "/tmp/lrep-$PAR{pid}.status", encode_json( \%status ) );
  print encode_json( \%status );
  # unlink "/tmp/lrep-$chpid.status";
}

elsif ( $PAR{cmd} eq "status" ) {    ### Get list of credentials
  print "Content-type: application/json\n\n";
  my $chpid = $PAR{pid};
  if ( -f "/tmp/lrep-$chpid.status" ) {
    my $stat = file_read("/tmp/lrep-$chpid.status");
    {
      local $/ = undef;   # required for re-read of encode_json pretty output
      my %hstat = %{ decode_json( $stat ) };
      my $chruns = kill( 0, $chpid );   # check if the child is still alive

      if ( $hstat{status} ne "pending" || ! $chruns ) {
        my $reportsdir   = "$basedir/reports";

        if ( ! -d $reportsdir ) {
          umask 0000;
          mkdir( "$reportsdir", 0777 ) || print STDERR ( "Cannot mkdir $reportsdir: $!" . __FILE__ . ":" . __LINE__ ) && exit;
        }

        my $olog = "$reportsdir/reporter-GUI.output.log";
        my $elog = "$reportsdir/reporter-GUI.error.log";

        my $oof = "/tmp/lrep-$chpid.out.log";
        my $eef = "/tmp/lrep-$chpid.err.log";

        `cat $oof >> $olog`;
        `cat $eef >> $elog`;

        my $oos = file_read($oof);
        my $ees = file_read($eef);

        my $dnld = my $filename = my $stored = "";

        foreach my $line (split /^/, $oos) {
          if ($line =~ /^download/) {
            $dnld = (split " : ", $line)[1];
            $filename = fileparse($dnld);
            $dnld = (split "/reports/", $line, 2)[1];
            $dnld = trim($dnld);
            $hstat{dnld} = $dnld;
            $hstat{filename} = $filename;
          }
          elsif ($line =~ /^send email to/) {
            $hstat{emails} = 1;
          }
          elsif ($line =~ /^stored report/) {
            $stored = (split " : ", $line)[1];
            $stored = trim($stored);
            $hstat{stored} = $stored;
          }
        }
        # my $retstr = "\"msg\" : \"<div>Reporter configuration has been succesfuly saved!<br /><br /></div>\", \"debug\" : \"";
        if ($ees) {
          $hstat{status} = "failed";
          $hstat{error} = $ees;
        }
        $hstat{log} = $oos;

        print encode_json( \%hstat );

        unlink "/tmp/lrep-$chpid.status";
        unlink "/tmp/lrep-$chpid.out.log";
        unlink "/tmp/lrep-$chpid.err.log";
      } else {
        print $stat;
      }
    }
  }
  else {
    my %hstat = (status => "waiting");
    print encode_json( \%hstat );
  }
  exit;

}

sub jsonTimeToHuman {
  my $str = shift;
  $str =~ tr/TZ/ /;
  return $str;
}

sub epoch2human {
  # Output: 2015:02:05T19:54:07
  my ( $tm, $tz ) = @_;    # epoch, TZ offset (+0100)
  my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) = localtime($tm);
  my $y   = $year + 1900;
  my $m   = $mon + 1;
  my $mcs = 0;
  my $str = sprintf( "%4d-%02d-%02d %02d:%02d:%02d", $y, $m, $mday, $hour, $min, $sec );
  return ($str);
} ## end sub epoch2human

sub url_encode {
  my $s = shift;
  $s =~ s/ /+/g;
  $s =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
  return $s;
} ## end sub urlencode

sub encode_base64url {
  my $e = encode_base64(shift, "");
  $e =~ s/=+\z//;
  $e =~ tr[+/][-_];
  return $e;
}

sub decode_base64url {
  my $s = shift;
  $s =~ tr[-_][+/];
  $s .= '=' while length($s) % 4;
  return decode_base64($s);
}
sub result {
  my ( $status, $msg, $log ) = @_;
  $log ||= "";
  $msg =~ s/\n/\\n/g;
  $msg =~ s/\\:/\\\\:/g;
  $log =~ s/\n/\\n/g;
  $log =~ s/\\:/\\\\:/g;
  $log =~ s/\t/ /g;
  $status = ($status) ? "true" : "false";
  print "{ \"success\": $status, \"message\" : \"$msg\", \"log\": \"$log\"}";
} ## end sub result

sub list {
  my $base  = shift;
  my $repdir = {};

  # store my base for later
  $repdir->{base} = $base;

  # read in contents of current directory
  opendir (BASE, $base);
  my @entries = grep !/^\.\.?\z/, readdir BASE;
  chomp(@entries);
  closedir(BASE);

  for my $entry (@entries) {

    # if entry is a directory, launch a new File::List to explore it
    # and store a reference to the new object in the dirlist hash
    if (-d "$base/$entry") {
      my $newbase = list("$base/$entry");
      $repdir->{dirlist}{ $entry } = $newbase;
    }

    # if entry is a file, store it's name in the dirlist hash
    elsif ( -f "$base/$entry"){
      $repdir->{dirlist}{ $entry } = 1;
    }
  }
  return $repdir;
}

sub find {

  my $self   = shift;
  my $reg    = shift;

  for my $dir (sort keys %{ $self->{dirlist} } ) {
    for my $subkey (sort keys %{ $self->{dirlist}{$dir}{dirlist} } ) {
      my $path = "$self->{base}/$dir/$subkey";
      if ( ( $path =~ /$reg/i ) ) {
        my $last_modified = epoch2human( ( stat( $path ) )[9] );
        my $size = ( stat( $path ) )[7];
        my $dnld = (split "/reports/", $path, 2)[1];
        $dnld = trim($dnld);
        my $link = "/lpar2rrd-cgi/reporter.sh?cmd=get&filename=" . url_encode($dnld);
        print "<tr><td class='repname'>$dir</td><td class='repfile'><a href='$link'>$subkey</a></td><td>" . kbstring( $size ) . "</td><td>$last_modified</td><td class='remrep'><div class='delete'></div></td></tr>\n";
      }
    }
  }
}

sub kbstring {
  my $value = shift;
  my @prefix = ('', qw(K M G T));
  while (@prefix>1 && $value >= 1000) {
    shift @prefix;
    $value /= 1024;
  }
  return sprintf '%.0f %sB', $value, $prefix[0];
}
