use warnings;
use strict;
use Date::Parse;
use RRDp;
use File::Glob ':glob';
use CGI::Carp qw(fatalsToBrowser);
use JSON;
use Xorux_lib qw(read_json write_json uuid_big_endian_format);

use XenServerDataWrapper;
use XenServerMenu;
use OVirtDataWrapper;
use OVirtMenu;
use HostCfg;

my $DEBUG        = $ENV{DEBUG};
my $errlog       = $ENV{ERRLOG};
my $xport        = $ENV{EXPORT_TO_CSV};
my $webdir       = $ENV{WEBDIR};
my $lpm_env      = $ENV{LPM};
my $basedir      = $ENV{INPUTDIR};
my $wrkdir       = $basedir . "/data";
my $detail_graph = "detail-graph";
my $tmpdir       = "$basedir/tmp";
my $rest_api     = 0;
my $bindir       = $ENV{BINDIR};

my $agents_uuid_file = "$wrkdir/Linux--unknown/no_hmc/linux_uuid_name.json";

if ( defined $ENV{TMPDIR_LPAR} ) {
  $tmpdir = $ENV{TMPDIR_LPAR};
}

if ( defined $ENV{NMON_EXTERNAL} ) {
  $wrkdir .= $ENV{NMON_EXTERNAL};
  $detail_graph = "detail-graph-external";
}
my $cpu_max_filter = 25;                         # max 2.5k peak in % is allowed (in fact it cannot by higher than 2k% now when 1 logical CPU == 0.05 entitlement)
if ( defined $ENV{CPU_MAX_FILTER} ) {
  $cpu_max_filter = $ENV{CPU_MAX_FILTER};
}

if ( -f "$bindir/premium.pl" ) {
    require "$bindir/premium.pl";
}
else {
    require "$bindir/standard.pl";
}

my $base       = 0;
my $detail_yes = 1;
my $detail_no  = 0;
my $gui        = 0;
my $NMON       = "--NMON--";
my $WPAR       = "--WPAR--";
my $AS400      = "--AS400--";
my $false_picture = "";  # if you do not want to show picture but some notice

my $tab_exe = "";    # for preparing tab executive string

# set unbuffered stdout
$| = 1;

# CGI-BIN HTML header
print "Content-type: text/html\n\n";

open( OUT, ">> $errlog" ) if $DEBUG == 2;

#foreach $key (sort keys(%ENV)) {
#   print "$key = $ENV{$key}<p>";
#}
#window.open(href, windowname, 'width=1200,height=450,scrollbars=yes');

# get QUERY_STRING
use Env qw(QUERY_STRING);
timing_debug($QUERY_STRING);

# print STDERR "61 detail_cgi \$QUERY_STRING $QUERY_STRING\n";

################ get URL parameters (could be GET or POST) and return them as hash refference
#                multiple values with same name create an array
sub getURLparams {
  my ( $buffer, @pairs, $pair, $name, $value, %PAR );

  if ( defined $ENV{'REQUEST_METHOD'} ) {
    if ( lc $ENV{'REQUEST_METHOD'} eq "post" ) {
      read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
    }
    else {
      $buffer = $ENV{'QUERY_STRING'};
    }

    # Split information into name/value pairs
    @pairs = split( /&/, $buffer );
    foreach $pair (@pairs) {
      ( $name, $value ) = split( /=/, $pair );
      $value =~ tr/+/ /;
      $value =~ s/%(..)/pack("C", hex($1))/eg;
      if ( exists $PAR{$name} ) {
        if ( ref $PAR{$name} eq "ARRAY" ) {
          push @{ $PAR{$name} }, $value;
        }
        else {
          my $firstVal = $PAR{$name};
          $PAR{$name} = [];
          push @{ $PAR{$name} }, $firstVal;
          push @{ $PAR{$name} }, $value;
        }
      }
      else {
        $PAR{$name} = $value;
      }
    }
    return \%PAR;
  }
  else {
    return 0;
  }
} ## end sub getURLparams

sub urlencode {
  my $s = shift;
  $s =~ s/([^a-zA-Z0-9!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  if ($s) {
    $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  }
  return $s;
} ## end sub urldecode

my %params;   # hash containing URL parameters. Use like this: $params{server}

my $qs = getURLparams;
if ( $qs ) {
  %params = %{ $qs };
   use Data::Dumper;
   #print "<pre style='font-size: small'>QS: ". $ENV{'QUERY_STRING'} . "\n";
   #print STDERR Data::Dumper->Dump( [ \%params ], [ qw(*params) ] );
   #print "</pre>";
}

# assign values from %params for back compatibility
my ( $host, $server, $lpar, $item, $entitle, $sunix, $eunix );
( $host, $server, $lpar, $item, $entitle, $gui, $sunix, $eunix ) = ( $params{host}, $params{server}, $params{lpar}, $params{item}, $params{entitle}, $params{gui}, $params{sunix}, $params{eunix} );

# my $sunix, my $eunix used only for VMWARE datastore top

# initialize if only custom URL params are used (because the older code here depends on these variables)
$host   = '' if not defined $params{host};
$server = '' if not defined $params{server};
$lpar   = '' if not defined $params{lpar};
$item   = '' if not defined $params{item};

$gui     = '' if not defined $gui;
$entitle = '' if not defined $entitle;

if (is_host_rest($host)){
  $rest_api = 1;
}
else{
  $rest_api = 0;
}

if ($params{source}){
  print_html_file("$webdir/$params{source}");
}


sub print_html_file {
  my $file = shift;
  open (my $in, "<", $file) || error ("Cannot read $file ".__FILE__.":".__LINE__."\n") && return -1;
  my @lines = <$in>;
  close($in);
  print "<div>\n";
  my $file_html = $file;
  my $print_html;
  if ( -f "$file_html" ) {
    open( FH, "< $file_html" );
    $print_html = do { local $/; <FH> };
    close(FH);
    print "$print_html";
  } ## end if ( -f "$file_html" )
  print "</div>\n";
}

my $bmax = 10;
#if ( $entitle =~ m/MAX/ ) {
#  $bmax = $entitle;
#}
if (defined $params{MAX}) {
  $bmax = $params{MAX}
}

# if == 1 then restrict views (only CPU and mem)
if ( $entitle eq '' || isdigit($entitle) == 0 ) {
  $entitle = 0;    # when any problem then allow it!
}
my $host_url = urlencode("$host");
my $server_url = urlencode("$server");
my $lpar_url = urlencode("$lpar");


#print STDERR "host: ";print STDERR Dumper $host;print STDERR "\nhost_url: ";print STDERR Dumper $host_url;
#print STDERR "server: ";print STDERR Dumper $server;print STDERR "\nserver_url: ";print STDERR Dumper $server_url;
#print STDERR "lpar: ";print STDERR Dumper $lpar;print STDERR "\nlpar_url: ";print STDERR Dumper $lpar_url;

#$lpar =~ s/\+/ /g;
$lpar =~ s/ \[.*\]//g;    # remove alias info
my $lpar_slash = $lpar;
$lpar_slash =~ s/\//\&\&1/g;       # replace for "/"
$lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace

if ( $gui =~ m/gui=/ ) {
  $gui =~ s/gui=//;
}
else {
  $gui = 0;
}

###   if VMWARE
my $vmware = 0;
my $pth    = "$wrkdir/*/$host/vmware.txt";
$pth =~ s/ /\\ /g;
my $no_name = "";
my @files   = (<$pth$no_name>);    # unsorted, workaround for space in names
$vmware = 1 if ( defined $files[0] && ( $files[0] ne "" ) && index( $files[0], "vmware.txt" ) != -1 );
my $tab_type = "tabhmc";
if ($vmware) {
  $tab_type = "";
}

my $hyperv = 0;
if ( !$vmware ) {                  # then try if hyperv
  # print STDERR "188 detail-cgi.pl ,$wrkdir / $server / $host /hyperv.txt,\n";
  $hyperv = 1 if ( -f "$wrkdir/windows/$host/$server/hyperv.txt" );
  $hyperv = 1 if $server =~ /^windows/;
  if ($hyperv) {
    $tab_type = "";
    if ($server !~ /^windows/) {
      $server     = "windows/$server";
      $server_url = "windows/$server_url";
    }
  }
}

my $hitachi = 0;
if ( $server =~ /^Hitachi$/ ) {
  $hitachi = 1;
}

# flags for platforms that use the new URL format
my $xenserver = my $ovirt = 0;
if ( defined $params{platform} ) {
  # set other flags=0 as an ad-hoc workaround to prevent evaluation of branches for older platform implementations
  $vmware = $hyperv = $hitachi = 0;

  if ( $params{platform} =~ m/^XenServer$/ ) {
    $xenserver = 1;
    $item = 'xenserver';
  } elsif ( $params{platform} =~ m/^oVirt$/ ) {
    $ovirt = 1;
    $item = 'ovirt';
  }
}

# print STDERR "126 detail_cgi \$QUERY_STRING $QUERY_STRING \$vmware $vmware \$hyperv $hyperv \$server $server \$item $item\n";

# notice to upper right corner - VMs aggregated
my $question_mark = "<div id=\"hiw\"><a href=\"http://www.lpar2rrd.com/VMware-GHz-vrs-real_CPU.htm\" target=\"_blank\" class=\"nowrap\"><img src=\"css/images/help-browser.gif\" alt=\"How it works?\" title=\"How it works?\"></img></a></div>";

my $wpar = 0;
if ( $lpar =~ m/$WPAR/ ) {
  $wpar = 1;
}

# print HTML header

# find out $type_sam [m/d/h]
my $type_sam      = type_sam( $host, $server, $lpar, 0, $lpar_slash );
my $type_sam_year = type_sam( $host, $server, $lpar, 0, $lpar_slash );
my $upper         = -1;
my $solaris = "";
if ($item =~ m/sol\d+-*/){
  if ($item =~ /sol11-all/){next;}
  if ($item =~ /sol10-*/){
    print_solaris10 ($host_url,$server_url,$lpar_url,$item,$type_sam,$entitle,$upper);
  }
  elsif ($item=~ /sol11-*/){
    print_solaris ($host_url,$server_url,$lpar_url,$item,$type_sam,$entitle,$upper);
  }
  exit (0);
}
if ($item =~ m/sol-ldom/){
    print_solaris_ldom ($host_url,$server_url,$lpar_url,$item,$type_sam,$entitle,$upper);
}

if ( $item =~ m/^codused$/ ) {

  # CoD
  print_cod( $host_url, $server_url, $lpar_url, $item, $type_sam, $entitle, $upper );
  exit(0);
} ## end if ( $item =~ m/^codused$/)

if ($item eq "cpuagg-sol") {
  print_ldom_agg ($host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}


if ( $item =~ m/^pagingagg$/ ) {

  # Epaging aggregated
  print_pagingagg( $host_url, $server_url, $lpar_url, $item, $type_sam, $bmax, $upper );
  exit(0);
} ## end if ( $item =~ m/^pagingagg$/)

if ( $item =~ m/^pool$/ ) {

  # Server pool
  print_pool( $host_url, $server_url, $lpar_url, $item, $type_sam );
  exit(0);
} ## end if ( $item =~ m/^pool$/)

if ( $item =~ m/^shpool$/ ) {

  # Server pool
  print_shpool( $host_url, $server_url, $lpar_url, $item, $type_sam );
  exit(0);
} ## end if ( $item =~ m/^shpool$/)

if ( $item =~ m/^hea/ ) {
  print_hea( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^memalloc/ ) {
  print_memory( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^hmctotals/ ) {    #formerly in install-html.sh print_multipool
  print_hmctotals( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

# differentiate based on $host (power, vmware, xenserver)
if ( $item =~ m/^custom/ ) {       # formerly in custom.pl & install-html.sh
  if ( $host =~ m/^XenServer/ ) {
    print_custom_xenserver( $host_url, $server_url, $lpar_url, $item, $entitle );
  } elsif ( $host =~ m/^oVirt/ ) {
    print_custom_ovirt( $host_url, $server_url, $lpar_url, $item, $entitle );
  } elsif ($host =~ m/^Solaris/ ){
    print_custom_solaris( $host_url, $server_url, $lpar_url, $item, $entitle );
  }else {
    print_custom( $host_url, $server_url, $lpar_url, $item, $entitle );
  }
  exit(0);
}

if ( $item =~ m/^view/ ) {
  print_view( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^topten$/ ) {
  print_topten_all( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^topten_vm$/ ) {
  print_topten_vm( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^data_check/ ) {
  print_data_check( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^servers/ ) {
  print_data_servers( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^vmdisk/ ) {
  print_vmw_disk( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^vmnet/ ) {
  print_vmw_disk( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^cluster/ ) {
  print_cluster( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^resourcepool/ ) {
  print_resourcepool( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^datastore/ ) {
  print_datastore( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}
if ( $item =~ m/^dstr-table-top/ ) {
  print_datastore_table_top();
  exit(0);
}
if ( $item =~ m/^dstr-top/ ) {
  print_datastore_top( $sunix, $eunix );
  exit(0);
}
if ( $item =~ m/^(hitachi-lan|hitachi-san)$/ ) {
  print_hitachi_adapters( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^(hitachi-lan-totals|hitachi-san-totals)/ ) {
  print_hitachi_adapters_totals( $host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/(power_lan|power_san|power_sas|power_sri|power_hea)/ && !($lpar =~ m/totals/)) {
  print_power_adapters ($host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/(power_lan|power_san|power_sas|power_sri|power_hea)/ && ($lpar =~ m/totals/)) {
  print_power_adapters_agg ($host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^hdt_data/ || $item =~ m/^hdt_io/ ) {
  print_hyperv_disk_total($host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

if ( $item =~ m/^lfd/ || $item =~ m/^csv/) {
  print_hyperv_disk_lfd($host_url, $server_url, $lpar_url, $item, $entitle );
  exit(0);
}

my $tab_number;
my @item_agent      = ();
my @item_agent_tab  = ();
my $item_agent_indx = 0;
my $os_agent        = 0;
my $nmon_agent      = 0;
my $iops            = "IOPS";

# VMWARE
if ($vmware) {

  # TABs
  # basic CPU
  $tab_number = 0;
  print "\n<div  id=\"tabs\"> <ul>\n";

  # my $cpu = 0;

  # if ( -f "$wrkdir/vmware_VMs/$lpar_slash.rrm") {
  $tab_number++;

  # $cpu = $tab_number;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">CPU</a></li>\n";
  $upper = rrd_upper( $wrkdir, "vmware_VMs", "", $lpar, $type_sam, $type_sam_year, $item, $lpar_slash );

  # }
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">CPU GHz</a></li>\n";
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">MEM</a></li>\n";

  # if there are not data (no counters), do not show tabs
  my $is_dusage = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Disk_usage" );
  my $is_dread  = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Disk_read" );
  if ( $is_dusage || $is_dread ) {
    $tab_number++;
    print "  <li class=\"\"><a href=\"#tabs-$tab_number\">DISK</a></li>\n";
  }

  # if there is data for IOPS under any datastore in vCenter/datacenter/datastore/*.rrv
  # print STDERR "282 \$host $host \$server $server \$lpar $lpar\n";
  my $vm_iops = 0;
  if ( open( FH, "< $wrkdir/$server/$host/my_vcenter_name" ) ) {
    my $vcenter_name = <FH>;
    close(FH);
    chomp $vcenter_name;    # e.g. '10.22.11.10|Regina-new'
                            # now look for the appropriate vCenter
    ( undef, $vcenter_name ) = split( /\|/, $vcenter_name );

    # print STDERR "287 \$vcenter_name $vcenter_name\n";
    my $vCenter = "";
    foreach my $center (<$wrkdir/vmware_*>) {
      next if $center =~ "vmware_VMs";
      next if !open( FH, "< $center/vmware_alias_name" );
      my $vmware_alias_name = <FH>;
      close(FH);
      next if $vmware_alias_name !~ /$vcenter_name$/;
      $vCenter = $center;
      last;
    } ## end foreach my $center (<$wrkdir/vmware_*>)

    # print STDERR "299 found \$vCenter $vCenter $vCenter/*/*/$lpar.rrv\n";
    if ( $vCenter ne "" ) {
      my @files = <$vCenter/*/*/$lpar.rrv>;

      #print STDERR "302 \@files @files\n";
      if ( defined $files[0] && $files[0] ne "" ) {
        $tab_number++;
        print "  <li class=\"\"><a href=\"#tabs-$tab_number\">IOPS</a></li>\n";
        $vm_iops = 1;
      }
    } ## end if ( $vCenter ne "" )
  } ## end if ( open( FH, "< $wrkdir/$server/$host/my_vcenter_name"...))

  my $is_nusage    = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Network_usage" );
  my $is_nreceived = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Network_received" );
  if ( $is_nusage || $is_nreceived ) {
    $tab_number++;
    print "  <li class=\"\"><a href=\"#tabs-$tab_number\">LAN</a></li>\n";
  }

  my $is_memswap = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Memory_swapin" );
  if ($is_memswap) {
    $tab_number++;
    print "  <li class=\"\"><a href=\"#tabs-$tab_number\">SWAP</a></li>\n";
  }

  my $is_memcomp = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Memory_compres" );
  if ($is_memcomp) {
    $tab_number++;
    print "  <li class=\"\"><a href=\"#tabs-$tab_number\">COMP</a></li>\n";
  }

  my $is_ready = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "CPU_ready_ms" );

  #    my $is_ready = 0;
  if ($is_ready) {
    $tab_number++;
    print "  <li class=\"\"><a href=\"#tabs-$tab_number\">CPU READY</a></li>\n";
  }

  # show vMotion only if exists
  # since 4.84-5 show always, users have info on which ESXi is VM now
  # my @path_lines = `grep "$lpar" "$wrkdir"/*/*/VM_hosting.vmh`; #vmx
  # print STDERR "...... \$lpar $lpar \$wrkdir $wrkdir \@path_lines @path_lines\n";
  # if (scalar @path_lines > 2) {
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">vMotion</a></li>\n";
  # }

  #
  ### join Linux agent data with vmware VM
  #

  my $lpar_name_agent = "";
  my ( $code, $linux_uuids ) = -f $agents_uuid_file ? Xorux_lib::read_json( $agents_uuid_file ) : ( 0, undef );
  # 1st method: look for the right VM old UUID inside all agents data uuid.txt
  # agents uuids are prepared by find_active_lpar.pl in a txt file

  # get VMs data with the old uuid (not instanceUuid)
  my $vm_uuid_name_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
   # e.g. 501c487b-66db-574a-1578-8bb38694a41f,vm-jindra,vm-jindra,421c05d2-69c4-da27-3ff6-6e508678c004,other-vm-name-can-be-here
   #                                                              ,     this is old UUID               ,
  my @vm_uuid_names     = ();
  if (open( FH, " < $vm_uuid_name_file" )) {
    @vm_uuid_names = <FH>;
    close FH;
  }
  else {
    error( "Cannot open $vm_uuid_name_file: $!" . __FILE__ . ":" . __LINE__ );
  }

  # get proper VM line (for this actual $lpar) from vm_uuid_names according to instanceUuid
  my @matches = grep {/^$lpar/} @vm_uuid_names;
  if (!defined $matches[0] ) {error( "! defined VM uuid in vmware_VMs/vm_uuid_name.txt & exit " . __FILE__ . ":" . __LINE__ ) && exit };
    # will you test  or scalar @matches > 1 ?
  chomp $matches[0];
  (undef, my $lpar_name, undef, my $uuid_old, my $other_lpar_name) = split (",",$matches[0]);

  if ( defined $uuid_old && $uuid_old ne '' ) {
    chomp $uuid_old;
    my $formatted_uuid = Xorux_lib::uuid_big_endian_format( $uuid_old, '-' );

    if ( $code && defined $linux_uuids->{ $formatted_uuid } ) {
      $lpar_name_agent = $linux_uuids->{ $formatted_uuid }
    }
  }

  if (!$lpar_name_agent) {
    # 2nd method: get VM name & try directly the agent data path
    # print STDERR "533 2nd method -f $wrkdir/Linux/no_hmc/$lpar_name/uuid.txt\n";
    if (-f "$wrkdir/Linux/no_hmc/$lpar_name/uuid.txt") {
      $lpar_name_agent = $lpar_name;
      # print STDERR "536 \$lpar_name_agent \$lpar_name_agent for $lpar_name\n";
    }
    # 3rd method: look for the other lpar name
    if (!$lpar_name_agent) {
      if (defined $other_lpar_name) {
        chomp $other_lpar_name;
        if (-f "$wrkdir/Linux/no_hmc/$other_lpar_name/uuid.txt") {
          $lpar_name_agent = $other_lpar_name;
          # print STDERR "544 \$lpar_name_agent $lpar_name_agent for $lpar_name\n";
        }
      }
    }
  }

  if ($lpar_name_agent) {
    @item_agent      = ();
    @item_agent_tab  = ();
    $item_agent_indx = $tab_number;
    $os_agent        = 0;
    $nmon_agent      = 0;
    $iops            = "IOPS";

    build_agents_tabs("Linux","no_hmc",$lpar_name_agent);
  }

  # TABs header end
  print "   </ul> \n";

  $tab_number = 1;
  $item       = "vmw-proc";

  # if ( $cpu > 0 ) {
  # lpar without HMC use only agents --> so no cpu graphs, probably no use for vmware
  print "<div id=\"tabs-$tab_number\"><br><br>\n";
  $tab_number++;
  my $refresh = "";
  $refresh = "<div class=\"refresh\"><A HREF=\"/lpar2rrd-cgi/lpar2rrd-realt.sh?source=$lpar_url&hmc=$host_url&mname=$server_url&new_gui=$gui\"></A></div>\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, "trendvm", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";

  # maybe later
  #    print_lpar_cfg ($host_url,$server_url,$lpar_url,$wrkdir,$server,$host,$lpar_slash,$lpar);
  print "</td></tr></table>\n";
  print "</div>\n\n";

  # }

  my $vmw_tab = $tab_number;
  print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
  $vmw_tab++;
  print "<center>\n";
  $item = "lpar";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";

  print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
  $vmw_tab++;
  print "<center>\n";
  $item = "vmw-mem";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, "trendvmem", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";
  print "</td></tr></table>\n";
  print "</div>\n\n";

  if ( $is_dusage || $is_dread ) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    $item = "vmw-disk";
    if ($is_dread) {
      $item = "vmw-diskrw";
    }
    print "<table align=\"center\" summary=\"Graphs\">\n";
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ( $is_dusage || $is_dread)

  if ($vm_iops) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    print "<table align=\"center\" summary=\"Graphs\">\n";
    $item = "vmw-iops";

    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ($vm_iops)

  if ( $is_nusage || $is_nreceived ) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    print "<table align=\"center\" summary=\"Graphs\">\n";
    $item = "vmw-net";
    if ($is_nreceived) {
      $item = "vmw-netrw";
    }
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ( $is_nusage || $is_nreceived)

  if ($is_memswap) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    print "<table align=\"center\" summary=\"Graphs\">\n";
    $item = "vmw-swap";
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ($is_memswap)

  if ($is_memcomp) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    print "<table align=\"center\" summary=\"Graphs\">\n";
    $item = "vmw-comp";
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ($is_memcomp)

  if ($is_ready) {
    print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
    $vmw_tab++;
    print "<center>\n";
    print "<table align=\"center\" summary=\"Graphs\">\n";
    $item = "vmw-ready";
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>\n";
    print "</center></div>\n\n";
  } ## end if ($is_ready)

  # show vMotion only if exists
  # since 4.84-5 show always, users have info on which ESXi is VM now
  #    if (scalar @path_lines > 2) {
  print "<div id=\"tabs-$vmw_tab\"><br><br>\n";
  $vmw_tab++;
  print "<center>\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  $item = "vmw-vmotion";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";

  #    }

  $tab_number = $vmw_tab;
  if ( $lpar_name_agent ) {
    $tab_number--;

    $host = "no_hmc";
    $server = "Linux";
    $lpar = $lpar_name_agent;

    build_agents_html ("Linux","no_hmc",$lpar_name_agent,$tab_number);
  }

  print "</div><br>\n";
  exit(0);

} ## end if ($vmware)

# HYPER-V
if ($hyperv) {

  # TABs
  # basic CPU
  $tab_number = 0;
  print "\n<div  id=\"tabs\"> <ul>\n";

  # my $cpu = 0;

  # if ( -f "$wrkdir/hyperv_VMs/$lpar_slash.rrm") {
  $tab_number++;

  # $cpu = $tab_number;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">CPU</a></li>\n";
  # $upper = rrd_upper( $wrkdir, "vmware_VMs", "", $lpar, $type_sam, $type_sam_year, $item, $lpar_slash );

  # }
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">MEM</a></li>\n";

  # if there are not data (no counters), do not show tabs
  #my $is_dusage = test_metric_in_rrd("vmware_VMs","","$lpar.rrm",$item,"Disk_usage");
  #my $is_dread  = test_metric_in_rrd("vmware_VMs","","$lpar.rrm",$item,"Disk_read");
  #if ($is_dusage || $is_dread) {
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">DISK</a></li>\n";
  # }
  $tab_number++;
  print "  <li class=\"\"><a href=\"#tabs-$tab_number\">LAN</a></li>\n";
  # vmotion not, it is not ready
  #$tab_number++;
  #print "  <li class=\"\"><a href=\"#tabs-$tab_number\">vMotion</a></li>\n";

  # TABs header end
  print "   </ul> \n";

  $tab_number = 1;
  $item       = "hyp-cpu";

  # if ( $cpu > 0 ) {
  # lpar without HMC use only agents --> so no cpu graphs, probably no use for vmware and no use for hyperv
  print "<div id=\"tabs-$tab_number\"><br><br>\n";
  $tab_number++;
  my $refresh = "";
  $refresh = "<div class=\"refresh\"><A HREF=\"/lpar2rrd-cgi/lpar2rrd-realt.sh?source=$lpar_url&hmc=$host_url&mname=$server_url&new_gui=$gui\"></A></div>\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n";
  print "<tr>\n";

  # maybe later
  #  print_item ($host_url,$server_url,$lpar_url,"trendvm","y",$type_sam_year,$entitle,$detail_no,"norefr","nostar",2,"legend");
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";

  # maybe later
  #    print_lpar_cfg ($host_url,$server_url,$lpar_url,$wrkdir,$server,$host,$lpar_slash,$lpar);
  print "</td></tr></table>\n";
  print "</div>\n\n";

  # }

  print "<div id=\"tabs-$tab_number\"><br><br>\n";
  $tab_number++;
  print "<center>\n";
  $item = "hyp-mem";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n";
  print "<tr>\n";
  # maybe later
  # print_item( $host_url, $server_url, $lpar_url, "trendvmem", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";
  print "</td></tr></table>\n";
  print "</div>\n\n";

  #if ($is_dusage || $is_dread) {
  print "<div id=\"tabs-$tab_number\"><br><br>\n";
  $tab_number++;
  print "<center>\n";
  $item = "hyp-disk";

  #if ($is_dread) {
  #  $item = "vmw-diskrw";
  #}
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";

  #}

  print "<div id=\"tabs-$tab_number\"><br><br>\n";
  $tab_number++;
  print "<center>\n";
  $item = "hyp-net";

  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";

  # show vMotion only if exists
  #    if (scalar @path_lines > 2) {
  #    it is not ready
  #print "<div id=\"tabs-$tab_number\"><br><br>\n";
  #$tab_number++;
  #print "<center>\n";
  #print "<table align=\"center\" summary=\"Graphs\">\n";
  #$item = "hyp-vmotion";
  #print "<tr>\n";
  #print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  #print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  #print "</tr>\n<tr>\n";
  #print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  #print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  #print "</tr></table>\n";
  #print "</center></div>\n\n";

  # }
  print "</div><br>\n";
  exit(0);

} ## end if ($hyperv)


if ( $hitachi ) {
  my $lpar_uuids_file         = "$wrkdir/Hitachi/$host/lpar_uuids.json";
  my ( $code1, $lpar_uuids )  = -f $lpar_uuids_file ? Xorux_lib::read_json( $lpar_uuids_file ) : ( 0, undef );
  my ( $code2, $linux_uuids ) = -f $agents_uuid_file ? Xorux_lib::read_json( $agents_uuid_file ) : ( 0, undef );

  my $lpar_agent_name = $lpar_uuids->{ $lpar }{agent_name} if $code1;
  my $lpar_uuid       = $lpar_uuids->{ $lpar }{uuid} if $code1;

  $tab_number = 1;

  print "<CENTER>";
  print "<div id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-$tab_number\">CPU</a></li>\n";
  $tab_number++;
  print "  <li class=\"$tab_type\"><a href=\"#tabs-$tab_number\">CPU percentage</a></li>\n";
  $tab_number++;

  if ( $lpar_uuid && $lpar_agent_name && $code2 && $linux_uuids->{ $lpar_uuid } ) {
    @item_agent      = ();
    @item_agent_tab  = ();
    $item_agent_indx = $tab_number;
    $os_agent        = 0;
    $nmon_agent      = 0;
    $iops            = "IOPS";

    build_agents_tabs("Linux","no_hmc", $lpar_agent_name );
  }

  print "</ul>\n";

  $tab_number = 1;

  $item = "lpar";
  print "<div id=\"tabs-$tab_number\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";
  $tab_number++;

  $item = "cpu-percentages";
  print "<div id=\"tabs-$tab_number\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";
  $tab_number++;

  if ( $lpar_agent_name ) {
    build_agents_html ( "Linux", "no_hmc", $lpar_agent_name, $tab_number );
  }
  print "</CENTER>";
  print "</div><br>\n";

#  print STDERR Dumper(@item_agent_tab) . " ## " . Dumper(@item_agent) . "\n";

  exit(0);
} ## end if ($hitachi)

if ( $ovirt ) {
  my $mapping;
  $host_url   = 'oVirt';
  $server_url = 'nope';

  if ( $params{type} eq 'vm' ) {
    $lpar_url = $params{vm};
    $mapping  = OVirtDataWrapper::get_mapping( $params{vm} );
  }
  elsif ( $params{type} eq 'host' ) {
    $lpar_url = $params{host};
    $mapping  = OVirtDataWrapper::get_mapping( $params{host} );
  }
  elsif ( $params{type} eq 'host_nic_aggr' ) {
    $lpar_url = $params{host};
  }
  elsif ( $params{type} eq 'host_nic' ) {
    $server_url = $params{host};
    $lpar_url   = $params{nic};
  }
  elsif ( $params{type} eq 'storage_domain' ) {
    $lpar_url = $params{storage_domain};
  }
  elsif ( $params{type} eq 'disk' ) {
    $lpar_url = $params{disk};
  }
  elsif ( $params{type} eq 'disk_aggr' ) {
    $lpar_url = $params{storage_domain};
  }
  elsif ( $params{type} eq 'cluster_aggr' ) {
    $lpar_url = $params{cluster};
  }
  elsif ( $params{type} eq 'storage_domains_total_aggr') {
    $lpar_url = 'nope';
  }

  print "<CENTER>";

  # get tabs
  my @tabs  = @{ OVirtMenu::get_tabs( $params{type} ) };
  my @items = ();

  $tab_number = 1;
  if (@tabs) {
    print "<div id=\"tabs\">\n";
    print "<ul>\n";

    foreach my $tab_header (@tabs) {
      while ( my ( $tab_type, $tab_label ) = each %$tab_header ) {
        if ( $tab_type =~ /^(aggr_data|aggr_latency)$/ && $params{type} eq 'storage_domain'
             && !scalar @{ OVirtDataWrapper::get_arch( $params{storage_domain}, 'storage_domain', 'disk' ) } ) {
          next; # there are no disks on this storage sto skip tabs with disks totals
        }

        print "  <li class=\"$tab_type\"><a href=\"#tabs-$tab_number\">$tab_label</a></li>\n";
        $tab_number++;
        push @items, "ovirt_$params{type}\_$tab_type";
      }
    }

    if ( $mapping ) {
      @item_agent      = ();
      @item_agent_tab  = ();
      $item_agent_indx = $tab_number;
      $os_agent        = 0;
      $nmon_agent      = 0;
      $iops            = "IOPS";

      build_agents_tabs( "Linux", "no_hmc", $mapping );
    }

    print "</ul>\n";
  }

  # print tab content
  sub print_tab_content {
    my ($tab_number, $host_url, $server_url, $lpar_url, $item, $entitle, $detail_yes, $legend) = @_;

    print "<div id=\"tabs-$tab_number\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    print_item( $host_url, $server_url, $lpar_url, $item, "d", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print "</tr></table>";
    print "</div>\n";
  }

  my $conf_dir                 = "$wrkdir/oVirt/configuration";
  my $hosts_cfg_file           = "$conf_dir/hosts.html";
  my $vms_cfg_file             = "$conf_dir/vms.html";
  my $storage_domains_cfg_file = "$conf_dir/storage_domains.html";
  my @conf_files               = ( $hosts_cfg_file, $vms_cfg_file, $storage_domains_cfg_file );

  for ( $tab_number = 1; $tab_number <= $#items + 1; $tab_number++ ) {
    if ( $params{type} eq 'configuration' ) {
      my @file;
      my $conf_file = $conf_files[$tab_number - 1];

      if ( -f $conf_file ) {
        open( CFGH, '<', $conf_file ) || Xorux_lib::error( "Couldn't open file $conf_file $!" . __FILE__ . ":" . __LINE__ );
        @file = <CFGH>;
        close(CFGH);
      }

      print "<div id=\"tabs-$tab_number\">\n";

      if ( scalar @file ) {
        print @file;
      }
      else {
        print "<p>Configuration is generated during first load each day.</p>";
      }

      print "</div>\n";
    }
    else {
      my $ovirt_item = $items[$tab_number - 1];
      my $legend     = $ovirt_item =~ /aggr/ ? 'nolegend' : 'legend';

      print_tab_content( $tab_number, $host_url, $server_url, $lpar_url, $ovirt_item, $entitle,
                           $detail_yes, $legend );
    }
  }

  if ( $mapping ) {
    $server     = 'Linux';
    $host       = 'no_hmc';
    $lpar       = $mapping;
    build_agents_html( "Linux", "no_hmc", $mapping, $tab_number );
  }

  print "</CENTER>";
  print "</div><br>\n";

  exit 0;
} ## end if $ovirt


# XenServer
if ( $xenserver ) {
  # TODO temporary wrapper for compatibility with legacy subroutines interfacing mainly detail-graph-cgi.pl
  $host_url = "XenServer";
  if ( $params{type} =~ m/^pool-aggr$/ || $params{type} =~ m/^pool-storage-aggr$/ ) {
    $server_url = $params{pool};
    $lpar_url = "nope";
  } elsif ( $params{type} =~ m/^vm$/ ) {
    $server_url = "nope";
    $lpar_url = $params{vm};
  } elsif ( $params{type} =~ m/^(host|net|storage)/ ) {
    $server_url = $params{host};
    if ( $params{type} =~ m/^net$/ ) {
      $lpar_url = $params{net};
    } elsif( $params{type} =~ m/^storage$/ ) {
      $lpar_url = $params{storage};
    } else {
      $lpar_url = "nope";
    }
  }

  # print page contents
  print "<CENTER>";

  # get tabs
  my @tabs = @{ XenServerMenu::get_tabs( $params{type} ) };

  if (@tabs) {
    print "<div id=\"tabs\">\n";
    print "<ul>\n";

    my $tab_counter = 1;
    foreach my $tab_header (@tabs) {
      while (my ($tab_type, $tab_label) = each %$tab_header) {
        print "  <li class=\"$tab_type\"><a href=\"#tabs-$tab_counter\">$tab_label</a></li>\n";
        $tab_counter++;
      }
    }

    print "</ul>\n";
  }

  # print tab contents
  sub print_tab_contents {
    my ($tab_number, $host_url, $server_url, $lpar_url, $item, $entitle, $detail_yes, $legend) = @_;

    print "<div id=\"tabs-$tab_number\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    print_item( $host_url, $server_url, $lpar_url, $item, "d", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", "", $entitle, $detail_yes, "norefr", "star", 1, $legend );
    print "</tr></table>";
    print "</div>\n";
  }

  my $legend = ( $params{type} =~ m/aggr/ ) ? "nolegend" : "legend";
  if ( $params{type} =~ m/^host$/ ) {
    my @xen_items = ("xen-host-cpu-cores", "xen-host-cpu-percent", "xen-host-vm-cpu-cores-aggr", "xen-host-vm-cpu-percent-aggr","xen-host-memory", "xen-host-vm-memory-used-aggr", "xen-host-vm-memory-free-aggr");
    for $tab_number (1 .. $#xen_items + 1) {
      # adjust $legend for aggregated VM graphs
      $legend = ( $xen_items[$tab_number - 1] =~ m/aggr/ ) ? "nolegend" : "legend";
      print_tab_contents( $tab_number, $host_url, $server_url, $lpar_url, $xen_items[$tab_number - 1], $entitle, $detail_yes, $legend );
    }

  } elsif ( $params{type} =~ m/^pool-aggr$/ ) {
    my @xen_items = ("xen-host-cpu-cores", "xen-host-cpu-percent", "xen-vm-cpu-cores", "xen-vm-cpu-percent", "xen-host-memory-used", "xen-host-memory-free", "xen-vm-memory-used", "xen-vm-memory-free");
    for $tab_number (1 .. $#xen_items + 1) {
      my $xen_item = $xen_items[$tab_number - 1] . "-aggr";
      print_tab_contents( $tab_number, $host_url, $server_url, $lpar_url, $xen_item, $entitle, $detail_yes, $legend );
    }

  } elsif ( $params{type} =~ m/^net$/ || $params{type} =~ m/^net-aggr$/ ) {
    my @xen_items = ("xen-lan-traffic");
    for $tab_number (1 .. $#xen_items + 1) {
      my $xen_item = $xen_items[$tab_number - 1];
      if ( $params{type} =~ m/^net-aggr$/ ) { $xen_item .= "-aggr"; }
      print_tab_contents( $tab_number, $host_url, $server_url, $lpar_url, $xen_item, $entitle, $detail_yes, $legend );
    }

  } elsif ( $params{type} =~ m/^storage$/ || $params{type} =~ m/^storage-aggr$/ || $params{type} =~ m/^pool-storage-aggr$/ ) {
    my @xen_items = ("xen-disk-vbd", "xen-disk-vbd-iops", "xen-disk-vbd-latency");
    for $tab_number (1 .. $#xen_items + 1) {
      my $xen_item = $xen_items[$tab_number - 1];
      if ( $params{type} =~ m/storage-aggr$/ ) { $xen_item .= "-aggr"; }
      if ( $params{type} =~ m/^pool-storage-aggr$/ ) { $xen_item =~ s/disk/pool/; }
      print_tab_contents( $tab_number, $host_url, $server_url, $lpar_url, $xen_item, $entitle, $detail_yes, $legend );
    }

  } elsif ( $params{type} =~ m/^vm$/ ) {
    my @xen_items = ("xen-vm-cpu-cores", "xen-vm-cpu-percent", "xen-vm-memory", "xen-vm-vbd", "xen-vm-vbd-iops", "xen-vm-vbd-latency", "xen-vm-lan");
    for $tab_number (1 .. $#xen_items + 1) {
      print_tab_contents( $tab_number, $host_url, $server_url, $lpar_url, $xen_items[$tab_number - 1], $entitle, $detail_yes, $legend );
    }
  } elsif ( $params{type} =~ m/^configuration$/ ) {
    my $mapping_host_pool = XenServerDataWrapper::get_conf_section('arch-pool');
    my $mapping_vm_host   = XenServerDataWrapper::get_conf_section('arch-host-vm');
    my $host_config = XenServerDataWrapper::get_conf_section('spec-host');
    my $vm_config   = XenServerDataWrapper::get_conf_section('spec-vm');
    my $config_update_time = localtime( XenServerDataWrapper::get_conf_update_time() );

    my $html_tab_header = sub {
      my @columns = @_;
      my $result = "";

      $result .= "<center>";
      $result .= "<table class=\"tabconfig tablesorter\">";
      $result .= "<thead><tr>";
      foreach my $item (@columns) {
        $result .= "<th class=\"sortable\">" . $item . "</th>";
      }
      $result .= "</tr></thead>";
      $result .= "<tbody>";

      return $result;
    };

    my $html_table_row = sub {
      my @cells = @_;
      my $result = "";

      $result .= "<tr>";
      foreach my $cell (@cells) { $result .= "<td>" . $cell . "</td>"; }
      $result .= "</tr>";

      return $result;
    };

    my $html_tab_footer = "</tbody></table><p>Last update time: " . $config_update_time . "</p></center>";

    # Host tab
    print "<div id=\"tabs-1\">\n";

    print $html_tab_header->( "Pool", "Host", "Address", "Memory [GiB]",
                              "CPU count", "Socket count", "CPU model",
                              "Xen version" );

    my @hosts = @{ XenServerDataWrapper::get_host_list() };
    unless ( scalar @hosts > 0 ) {
      print "<tr><td colspan=\"8\" align=\"center\">no hosts found</td></tr>";
    }
    foreach my $host (@hosts) {
      my $cell_pool = "NA";
      foreach my $pool ( keys %{$mapping_host_pool} ) {
        if ( grep( /^$host$/, @{$mapping_host_pool->{$pool}} ) ) {
          my $pool_label = XenServerDataWrapper::get_label('pool', $pool);
          my $pool_link  = XenServerMenu::get_url({ type => "pool-aggr", pool => $pool });
          $cell_pool = "<a href=\"${pool_link}\">${pool_label}</a>";
        }
      }

      my $host_label = XenServerDataWrapper::get_label('host', $host);
      my $host_link  = XenServerMenu::get_url({ type => "host", host => $host });
      my $cell_host  = "<a href=\"${host_link}\">${host_label}</a>";

      my $address = exists $host_config->{$host}{address} ? $host_config->{$host}{address} : "NA";
      my $memory = exists $host_config->{$host}{memory} ? $host_config->{$host}{memory} : "NA";
      my $cpu_count = exists $host_config->{$host}{cpu_count} ? $host_config->{$host}{cpu_count} : "NA";
      my $socket_count = exists $host_config->{$host}{socket_count} ? $host_config->{$host}{socket_count} : "NA";
      my $cpu_model = exists $host_config->{$host}{cpu_model} ? $host_config->{$host}{cpu_model} : "NA";
      my $xen_version    = exists $host_config->{$host}{version_xen} ? $host_config->{$host}{version_xen} : "NA";

      print $html_table_row->( $cell_pool, $cell_host, $address, $memory,
                               $cpu_count, $socket_count, $cpu_model, $xen_version );
    }

    print $html_tab_footer;
    print "</div>\n";

    # VM tab
    print "<div id=\"tabs-2\">\n";

    print $html_tab_header->( "Pool", "Host", "VM", "Memory [GiB]",
                              "VCPU", "VCPU at startup", "VCPU max",
                              "Operating system" );

    my @vms = @{ XenServerDataWrapper::get_vm_list() };
    unless ( scalar @vms > 0 ) {
      print "<tr><td colspan=\"8\" align=\"center\">no VMs found</td></tr>";
    }
    foreach my $vm (@vms) {
      my $cell_pool = my $cell_host = "NA";
      foreach my $host ( keys %{$mapping_vm_host} ) {
        if ( grep( /^$vm$/, @{$mapping_vm_host->{$host}} ) ) {
          my $host_label = XenServerDataWrapper::get_label('host', $host);
          my $host_link  = XenServerMenu::get_url({ type => "host", host => $host });
          $cell_host = "<a href=\"${host_link}\">${host_label}</a>";

          foreach my $pool ( keys %{$mapping_host_pool} ) {
            if ( grep( /^$host$/, @{$mapping_host_pool->{$pool}} ) ) {
              my $pool_label = XenServerDataWrapper::get_label('pool', $pool);
              my $pool_link  = XenServerMenu::get_url({ type => "pool-aggr", pool => $pool });
              $cell_pool = "<a href=\"${pool_link}\">${pool_label}</a>";
            }
          }
        }
      }

      my $vm_label = XenServerDataWrapper::get_label('vm', $vm);
      my $vm_link  = XenServerMenu::get_url({ type => "vm", vm => $vm });
      my $cell_vm  = "<a href=\"${vm_link}\">${vm_label}</a>";

      my $memory = exists $vm_config->{$vm}{memory} ? $vm_config->{$vm}{memory} : "NA";
      my $cpu_count = exists $vm_config->{$vm}{cpu_count} ? $vm_config->{$vm}{cpu_count} : "NA";
      my $cpu_count_start = exists $vm_config->{$vm}{cpu_count_start} ? $vm_config->{$vm}{cpu_count_start} : "NA";
      my $cpu_count_max   = exists $vm_config->{$vm}{cpu_count_max} ? $vm_config->{$vm}{cpu_count_max} : "NA";

      my $vm_os = exists $vm_config->{$vm}{os} ? $vm_config->{$vm}{os} : "NA";

      print $html_table_row->( $cell_pool, $cell_host, $cell_vm, $memory,
                               $cpu_count, $cpu_count_start, $cpu_count_max, $vm_os );
    }

    print $html_tab_footer;
    print "</div>\n";

    # Storage tab
    print "<div id=\"tabs-3\">\n";

    print $html_tab_header->( "Hosts", "Storage",
                              "Physical utilisation [GB]", "Physical size [GB]", "Virtual allocation [GB]",
                              "Volume", "Physical utilisation [GB]", "Virtual size [GB]", "VM" );

    # architecture > storage > ( sr_host, sr_vdi, vdi_vm )
    my $mapping_storage = XenServerDataWrapper::get_conf_section('arch-storage');
    my $sr_config       = XenServerDataWrapper::get_conf_section('spec-sr');
    my $vdi_config      = XenServerDataWrapper::get_conf_section('spec-vdi');

    my @sr_list = keys %{$sr_config};

    unless ( scalar @sr_list > 0 ) {
      print "<tr><td colspan=\"9\" align=\"center\">no storages found</td></tr>";
    }

    foreach my $sr (@sr_list) {
      # skip certain types (hardware devices, )
      if ( exists $sr_config->{$sr}{type} ) {
        my $sr_type = $sr_config->{$sr}{type};

        if ( $sr_type eq 'udev' || $sr_type eq 'iso' ) { next; }
      }

      # get label
      my $sr_label = XenServerDataWrapper::get_label( "sr", $sr );
      if ( $sr_label eq $sr ) {
        $sr_label = "no label";
      }
      # get the first part of UUID that is used in RRD filenames: for hypertext
      my @sr_uuid = split '-', $sr;
      my $sr_short_uuid = $sr_uuid[0];

      # get params
      my $sr_phys_util  = ( exists $sr_config->{$sr}{physical_utilisation} ) ? $sr_config->{$sr}{physical_utilisation} : "NA";
      my $sr_phys_size  = ( exists $sr_config->{$sr}{physical_size} ) ? $sr_config->{$sr}{physical_size} : "NA";
      my $sr_virt_alloc = ( exists $sr_config->{$sr}{virtual_allocation} ) ? $sr_config->{$sr}{virtual_allocation} : "NA";

      # get a list of hosts connected to this storage, generate hypertext links
      my @sr_hosts   = ( exists $mapping_storage->{sr_host}{$sr} ) ? @{$mapping_storage->{sr_host}{$sr}} : ();
      my $cell_hosts = "";
      my $cell_sr = "";
      my $sr_link_url = "";
      foreach my $sr_host (@sr_hosts) {
        unless ( $sr_host ) { next; }
        my $host_label    = XenServerDataWrapper::get_label( "host", $sr_host );
        my $host_link_url = XenServerMenu::get_url({ type => "host", host => $sr_host });

        $cell_hosts .= "<a href=\"$host_link_url\">$host_label</a><br>";
        $sr_link_url = XenServerMenu::get_url({ type => "storage", host => $sr_host, storage => $sr_short_uuid });
      }
      if ( $sr_link_url ) {
        $cell_sr = "<a href=\"${sr_link_url}\">${sr_label}<br>UUID: $sr</a>";
      } else {
        $cell_hosts = "NA";
        $cell_sr    = $sr_label . "<br>UUID: " . $sr;
      }

      # walk through underlying VDIs (volumes)
      my @sr_vdis  = ( exists $mapping_storage->{sr_vdi}{$sr} ) ? @{$mapping_storage->{sr_vdi}{$sr}} : ();

      unless ( scalar @sr_vdis > 0 ) {
        # print the storage with empty remaining cells
        my $cell_vdi = my $vdi_phys_util = my $vdi_virt_size = my $cell_vm = "NA";
        print $html_table_row->( $cell_hosts, $cell_sr, $sr_phys_util, $sr_phys_size, $sr_virt_alloc,
                                 $cell_vdi, $vdi_phys_util, $vdi_virt_size, $cell_vm );
        next;
      }

      foreach my $vdi (@sr_vdis) {
        unless ( $vdi ) { next; }
        my $vdi_label = XenServerDataWrapper::get_label( "vdi", $vdi );
        if ( $vdi_label eq $vdi ) { $vdi_label = "no label"; }
        my $cell_vdi = $vdi_label . "<br>UUID: " . $vdi;

        # get VDI params
        my $vdi_phys_util = ( exists $vdi_config->{$vdi}{physical_utilisation} ) ? $vdi_config->{$vdi}{physical_utilisation} : "NA";
        my $vdi_virt_size = ( exists $vdi_config->{$vdi}{virtual_size} ) ? $vdi_config->{$vdi}{virtual_size} : "NA";

        # walk through VMs that use this VDI as storage
        my @vm_list = ( exists $mapping_storage->{vdi_vm}{$vdi} ) ? @{$mapping_storage->{vdi_vm}{$vdi}} : ();

        unless ( scalar @vm_list > 0 ) {
          # print the storage with empty remaining cell
          my $cell_vm = "NA";
          print $html_table_row->( $cell_hosts, $cell_sr, $sr_phys_util, $sr_phys_size, $sr_virt_alloc,
                                   $cell_vdi, $vdi_phys_util, $vdi_virt_size, $cell_vm );
        }

        foreach my $vm (@vm_list) {
          unless ( $vm ) { next; }
          my $vm_label    = XenServerDataWrapper::get_label( "vm", $vm );
          my $vm_link_url = XenServerMenu::get_url({ type => "vm", vm => $vm });
          my $cell_vm     = "<a href=\"$vm_link_url\">$vm_label</a>";

          print $html_table_row->( $cell_hosts, $cell_sr, $sr_phys_util, $sr_phys_size, $sr_virt_alloc,
                                   $cell_vdi, $vdi_phys_util, $vdi_virt_size, $cell_vm );
        }
      }
    }

    print $html_tab_footer;
    print "</div>\n";

    # Storage-VM tab
    print "<div id=\"tabs-4\">\n";

    print $html_tab_header->( "Host", "Storage", "Volume", "VM" );

    unless ( scalar @sr_list > 0 ) {
      print "<tr><td colspan=\"4\" align=\"center\">no storages found</td></tr>";
    }

    # reuse config and @sr_list from the previous tab
    foreach my $sr (@sr_list) {
      my $sr_label = XenServerDataWrapper::get_label( "sr", $sr );
      if ( $sr_label eq $sr ) {
        $sr_label = "no label";
      }
      my @sr_uuid = split '-', $sr;
      my $sr_short_uuid = $sr_uuid[0];

      my @sr_hosts = @{$mapping_storage->{sr_host}{$sr}};

      foreach my $sr_host (@sr_hosts) {
        unless ( $sr_host ) { next; }
        my $host_label    = XenServerDataWrapper::get_label( "host", $sr_host );
        my $host_link_url = XenServerMenu::get_url({ type => "host", host => $sr_host });
        my $sr_link_url   = XenServerMenu::get_url({ type => "storage", host => $sr_host, storage => $sr_short_uuid });

        if ( exists $mapping_storage->{sr_vdi}{$sr} ) {
          my @vdi_list = @{$mapping_storage->{sr_vdi}{$sr}};
          foreach my $vdi (@vdi_list) {
            if ( exists $mapping_storage->{vdi_vm}{$vdi} ) {
              my $vdi_label = XenServerDataWrapper::get_label( "vdi", $vdi );
              if ( $vdi_label eq $vdi ) {
                $vdi_label = "no label";
              }

              my @vm_list = @{$mapping_storage->{vdi_vm}{$vdi}};
              foreach my $vm (@vm_list) {
                unless ( $vm ) { next; }
                my $vm_label    = XenServerDataWrapper::get_label( "vm", $vm );
                my $vm_link_url = XenServerMenu::get_url({ type => "vm", vm => $vm });
                print "<tr><td><a href=\"$host_link_url\">$host_label</a></td><td><a href=\"$sr_link_url\">$sr_label ($sr)</a></td><td>$vdi_label ($vdi)</td><td><a href=\"$vm_link_url\">$vm_label</a></td></tr>\n";
              }
            }
          }
        }
      }
    }

    print $html_tab_footer;
    print "</div>\n";
  }

  # closing page contents
  print "</CENTER>";
  print "</div><br>\n";

  exit(0);
} ## end if ($xenserver)


# POWER
# TABs
# basic CPU
$tab_number = 0;
print "\n<div  id=\"tabs\"> <ul>\n";

# basic CPU from the HMC
my $cpu = 0;

if ( -f "$wrkdir/$server/$host/$lpar_slash.rr$type_sam" ) {
  my $filesize = -s "$wrkdir/$server/$host/$lpar_slash.rr$type_sam";

  # OS agent based lpars without the HMC does have only touched .rrh file
  if ( $filesize > 0 ) {
    $tab_number++;
    $cpu = $tab_number;
    print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_number\">CPU</a></li>\n";
    $upper = rrd_upper( $wrkdir, $server, $host, $lpar, $type_sam, $type_sam_year, $item, $lpar_slash );
  } ## end if ( $filesize > 0 )
} ## end if ( -f "$wrkdir/$server/$host/$lpar_slash.rr$type_sam")

my $lpm = is_lpm( $host, $server, $lpar, $lpm_env );
if ( $lpm > 0 ) {
  $tab_number++;
  $lpm = $tab_number;
  print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_number\">LPM</a></li>\n";
}

my $ams = 0;
if ( $entitle == 0 && -f "$wrkdir/$server/$host/$lpar_slash.rm$type_sam" ) {
  $tab_number++;
  $ams = $tab_number;
  print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_number\">AMS</a></li>\n";
}

my $wpar_cpu = 0;
my $wpar_name = find_wpar( "$wrkdir/$server/$host/$lpar_slash", $wpar );
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash/$wpar_name", "cpu" ) ) {
  $tab_number++;
  $wpar_cpu = $tab_number;
  print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_number\">WPAR_CPU</a></li>\n";
}

my $wpar_mem = 0;
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash/$wpar_name", "mem" ) ) {
  $tab_number++;
  $wpar_mem = $tab_number;
  print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_number\">WPAR_MEM</a></li>\n";
}

@item_agent      = "";
@item_agent_tab  = "";
$item_agent_indx = 0;
$os_agent        = 0;
$nmon_agent      = 0;
$iops            = "IOPS";

if ($server !~ /Solaris|Solaris--unknown|Solaris\d+--unknown/){
  build_agents_tabs($server,$host,$lpar_slash);
}

# TABs header end
print "   </ul> \n";
$tab_number = 1;

if ( $cpu > 0 ) {

  # lpar without HMC use only agents --> so no cpu graphs
  print "<div id=\"tabs-$tab_number\"><br><br>\n";

  my $refresh = "";
  $refresh = "<div class=\"refresh\"><A HREF=\"/lpar2rrd-cgi/lpar2rrd-realt.sh?source=$lpar_url&hmc=$host_url&mname=$server_url&new_gui=$gui\"></A></div>\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n";

  # Trend graph
  print "<tr>\n";
  if ( !$vmware && !$hyperv ) {
    print_item( $host_url, $server_url, $lpar_url, "trend", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  }
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";
  if ( !$vmware && !$hyperv ) {
    print_lpar_cfg( $host_url, $server_url, $lpar_url, $wrkdir, $server, $host, $lpar_slash, $lpar );
  }
  print "</td></tr></table>\n";
  print "</div>\n\n";
} ## end if ( $cpu > 0 )

# LPM
print OUT "002 LPM : $host,$server,$lpar,$lpm : $lpm \n" if $DEBUG == 2;
if ( $lpm > 0 ) {
  print "<div id=\"tabs-$lpm\"><br><br>\n";
  print "<center>\n";
  print "\n<table align=\"center\" summary=\"Graphs\">\n";
  $item = "lpm";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_no, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_no, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_no, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "star", 1, "legend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";
} ## end if ( $lpm > 0 )

# AMS
if ( $ams > 0 ) {
  print "<div id=\"tabs-$ams\"><br><br>\n";
  print "<center>\n";
  print OUT "003 AMS : $wrkdir/$server/$host/$lpar_slash.rm$type_sam \n" if $DEBUG == 2;
  print "\n<table align=\"center\" summary=\"Graphs\">\n";
  $item = "ams";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";
} ## end if ( $ams > 0 )

# WPAR-CPU
if ( $wpar_cpu > 0 ) {
  print "<div id=\"tabs-$wpar_cpu\"><br><br>\n";
  print "<center>\n";
  print OUT "003 WPAR-CPU : $wrkdir/$server/$host/$lpar_slash.rm$type_sam \n" if $DEBUG == 2;
  print "\n<table align=\"center\" summary=\"Graphs\">\n";
  $item = "wpar_cpu";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";
} ## end if ( $wpar_cpu > 0 )

# WPAR-MEM
if ( $wpar_mem > 0 ) {
  print "<div id=\"tabs-$wpar_mem\"><br><br>\n";
  print "<center>\n";
  print OUT "003 WPAR-MEM : $wrkdir/$server/$host/$lpar_slash.rm$type_sam \n" if $DEBUG == 2;
  print "\n<table align=\"center\" summary=\"Graphs\">\n";
  $item = "wpar_mem";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>\n";
  print "</center></div>\n\n";
} ## end if ( $wpar_mem > 0 )
build_agents_html( $server_url, $host_url, $lpar_url, 0);

print "</div><br>\n";
exit(0);

sub build_agents_html {
  my $server_url = shift;
  my $host_url   = shift;
  my $lpar_url   = shift;
  my $tab_number = shift;
if ( $os_agent > 0 || $nmon_agent > 0 ) {
  my $lpar_url_save = $lpar_url;
  foreach my $item (@item_agent) {
    if (!defined $item) {
      next;
    }
    if ( $item =~ m/file_sys/) {
      #print STDERR "1366 JCOM: \$vmware $vmware \$item $item path:$wrkdir/$server/$host/$lpar/FS.csv\n";
      my $file = "$wrkdir/$server/$host/$lpar/FS.csv";
      print "<div id =\"tabs-$item_agent_tab[$tab_number]\">
        <center>
        <h4>Filesystem usage</h4>
        <tbody>
        <tr>
          <table class =\"tabconfig tablesorter\"data-sortby=\"5\">
          <thead>
            <tr><th class = \"sortable\">Filesystem&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
            <th class = \"sortable\">Total [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
            <th class = \"sortable\">Used [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
            <th class = \"sortable\">Available [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
            <th class = \"sortable\">Usage [%]&nbsp;&nbsp;</th>
            <th class = \"sortable\">Mounted on&nbsp;&nbsp;</th>
          </tr></thead>\n\n";
      if ( -f "$file" ) {
        open( FH, "< $file" ) || error( "Cannot open $file: $!" . __FILE__ . ":" . __LINE__ );
        my @file_sys_arr = <FH>;
        close(FH);
        foreach my $line (@file_sys_arr)
        {
        chomp ($line);
        (my $filesystem, my $blocks, my $used, my $avaliable, my $usage, my $mounted) = split(" ", $line);
        $filesystem =~ s/=====double-colon=====/:/g;
        $mounted    =~ s/=====double-colon=====/:/g;
        print "<tr><td>$filesystem</td>
          <td>$blocks</td>
          <td>$used</td>
          <td>$avaliable</td>
          <td>$usage</td>
          <td>$mounted</td></tr>";
        }
      } ## end if ( -f "$file" )
      my $last_update = localtime((stat($file))[9]);
      print "<tfoot><tr><td colspan=\"6\">Last update time: $last_update</td></tr></tfoot>";
      print "</tr>
        </tbody>
        </table>
        </center>
        </div>\n";
      $tab_number++;
      next;
    } ## end if ( $item =~ m/file_sys/)

    my $demo_line = "";
    if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) && ($item =~ m/dsk_latency/) ) {
      $demo_line = "<br>
      Demo site does not actually support this data.<br>
      For examples check this: <a href=\"http://www.lpar2rrd.com/as400-ASP_latency-monitoring.htm\" target=\"_blank\">www.lpar2rrd.com/as400-ASP_latency-monitoring.htm</a> <br>
      <br>";
      $false_picture = "<br>$demo_line<br>";
    }

    if (($item =~ m/dsk_latency/) && ((premium() =~ "free") || (! test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$AS400/LTC", ".*", "mmc" )))) {
      # prepare false picture
      $false_picture = "<br>$demo_line<br>

        <table><tr><td>

        You are using free LPAR2RRD IBM i agent edition.<br>

        ASP disk service time and ASP disk wait time are not available in free edition.<br>

        This is one of <a href=\"http://www.lpar2rrd.com/support.htm#benefits\" class=\"ulink\" target=\"_blank\"><b>benefits</b></a> of the Enterprise edition which is distributed to customers under support<br>
        <br>
        For ASP latency examples check this: <a href=\"http://www.lpar2rrd.com/as400-ASP_latency-monitoring.htm\" target=\"_blank\">www.lpar2rrd.com/as400-ASP_latency-monitoring.htm</a> <br>

        </td></tr></table>

        <br><br>";
    }

    my $nmon = "";
    $lpar_url = $lpar_url_save;
    if ( $item =~ m/^nmon-/ ) {
      $nmon = "NMON";
      $lpar_url .= $NMON;
      $item =~ s/^nmon-//;
    }
    $lpar =~ s/--WPAR--/\//g;    # WPAR delimiter replace
    my $legend = "legend";
    if ( $item =~ m/lan/ || $item =~ m/san1/ || $item =~ m/san2/ || $item =~ m/sea/ || $item =~ m/san_resp/ || $item =~ m/size/ || $item =~ m/res/ || $item =~ m/threads/ || $item =~ m/faults/ || $item =~ m/pages/ || $item =~ m/waj/ || $item =~ m/job_cpu/ || $item =~ m/disk_io/ || $item =~ m/cap_/ || $item =~ m/data_as/ || $item =~ m/iops_as/ || $item =~ m/dsk_latency/ || $item =~ m/data_ifcb/ || $item =~ m/disk_busy/ || $item =~ m/jobs/ || $item =~ m/wlm-cpu/ || $item =~ m/wlm-mem/ || $item =~ m/wlm-dkio/ ) {
      $legend = "nolegend";
    }
    if ( $item =~ m/waj/ || $item =~ m/job_cpu/ || $item =~ m/disk_io/ ) {
      $legend = "nolegend higher";    #instead of 'm' and 'y' graphs there is 3x longer legends
    }
    print "<div id=\"tabs-$item_agent_tab[$tab_number]\"><br><br><center>\n";

    if ( $item =~ m/^oscpu$/ ) { print "<a href=\"http://www.lpar2rrd.com/os_cpu_monitoring.html\">How it works</a>\n"; }
    if ( $item =~ m/^job_cpu$/ || $item =~ m/^waj$/ || $item =~ m/^disk_io$/ ) { print "<a href=\"http://www.lpar2rrd.com/wrkactjob.htm\">How it works</a>\n"; }
    if ( $item =~ m/disk_busy/ ) { print "<a href=\"http://www.lpar2rrd.com/as400_help.htm\">How it works</a>\n"; }
    if ( $item =~ m/^jobs$/ ) {    print "<a href=\"http://www.lpar2rrd.com/job-top.htm\">How it works</a>\n";  }

    print OUT "005 OS : $item \n" if $DEBUG == 2;
    print "\n<table align=\"center\" summary=\"Graphs\">\n";
    print "<h4>Disk Busy</h4>" if $item =~ m/disk_busy/;
    my $star_db = "star";
    if ($item =~ m/jobs/) {
      print "<h4>JOB CPU</h4>";
      $star_db = "nostar";
    }
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, $legend );
    $false_picture = "do not print" if $false_picture ne "";
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "$star_db", 1, $legend );
    print "</tr>\n";

    if ( $item !~ m/waj/ && $item !~ m/job_cpu/ && $item !~ m/disk_io/ && $item !~ m/disk_busy/ && $item !~ m/jobs/ ) {    # for this item do not prepare 'm' or 'y' graphs
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, $legend );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, $legend );
      print "</tr>\n";
    } ## end if ( $item !~ m/waj/ &&...)
    if ( $item eq "jobs" ) {
      $item = "jobs_mem";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>JOB MEM</h4></td></tr>\n";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, $legend );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, $legend );
      print "</tr>\n";
    } ## end if ( $item eq "jobs" )
    if ( $item =~ m/disk_busy/ ) {   # AS400 disk: upper graphs are 'busy', lower graphs are IOPS
      $item = "disks";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>Disk IO aggregated</h4></td></tr>\n";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, $legend );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, $legend );
      print "</tr>\n";
    } ## end if ( $item =~ m/disk_busy/)
    if ( $item =~ m/data_ifcb/ ) {
      print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets IFCB</h4></td></tr>\n";
      $item = "paket_ifcb";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n";

      print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets Discarded IFCB</h4></td></tr>\n";
      $item = "dpaket_ifcb";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n";
    } ## end if ( $item =~ m/data_ifcb/)
    if ( $item =~ m/dsk_latency/ && $false_picture eq "") {
      print "<tr><td colspan=\"2\" align=\"center\"><h4>DSK SERVICE</h4></td></tr>\n";
      $item = "dsk_svc_as";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n";

      print "<tr><td colspan=\"2\" align=\"center\"><h4>DSK WAIT</h4></td></tr>\n";
      $item = "dsk_wait_as";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n";
    } ## end if ( $item =~ m/data_ifcb/)
    if ( $item =~ m/cap_used/ ) {
      print "<tr><td colspan=\"2\" align=\"center\"><h4>ASP % used</h4></td></tr>\n";
      $item = "cap_proc";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "$legend" );
      print "</tr>\n";
    } ## end if ( $item =~ m/cap_used/)

    print "</table>\n";

    # summary graphs
    if ( $item =~ m/lan/ || $item  eq "san1" || $item eq "san2" || $item =~ m/sea/ ) {
      my $sitem    = "s" . $item;
      my $text_sum = "Summary of transferred Bytes a Day";
      if ( $item =~ m/san2/ ) { $text_sum = "Summary of $iops a Day" }
      # print STDERR "1394 detail-cgi.pl $item,$sitem,$server_url\n";
      print "<br><table align=\"center\" summary=\"Graphs\">\n";
      print "<tr><td colspan=\"2\"><hr></td></tr>\n";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>$text_sum</h4></td></tr>\n";
      print "<tr>\n";
      # no details, no zoom, no problem
      print_item( $host_url, $server_url, $lpar_url, $sitem, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $sitem, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print "</tr>\n";
      print "</table>\n";
    } ## end if ( $item =~ m/lan/ ||...)
    if ( $item =~ m/lan/ || $item =~ m/sea/ ) {
      my $sitem = "packets_"."$item";
      print "<br><table align=\"center\" summary=\"Graphs\">\n";
      print "<tr><td colspan=\"2\"><hr></td></tr>\n";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets</h4></td></tr>\n";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $sitem, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print_item( $host_url, $server_url, $lpar_url, $sitem, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $sitem, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print_item( $host_url, $server_url, $lpar_url, $sitem, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print "</tr>\n";
      print "</table>\n";
    } ## end if ( $item =~ m/lan/ || $item =~ m/sea/)
    print "</center></div>\n\n";
    $tab_number++;
    $false_picture = "";
  } ## end foreach my $item (@item_agent)
} ## end if ( $os_agent > 0 || ...)
} ## end sub build_agents_html

sub build_agents_tabs {

  my $server = shift;
  my $host = shift;
  my $lpar_slash = shift;
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "cpu" ) ) {
  $item_agent[$item_agent_indx] = "oscpu";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">CPU OS</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && -f "$wrkdir/$server/$host/$lpar_slash/JOB/cputop0.mmc" ) {
  $item_agent[$item_agent_indx] = "jobs";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">JOB</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "mem" ) ) {
  $item_agent[$item_agent_indx] = "mem";
  $tab_number++;
  $os_agent = $tab_number;
  my $name_m = "Memory";
  if ($server =~ /Solaris--unknown|Solaris\d+--unknown/){ $name_m = "Memory OS"; }
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">$name_m</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( test_file_in_directory...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "pgs" ) ) {
  $item_agent[$item_agent_indx] = "pg1";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">Paging 1</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
  if ( $wpar == 0 ) {

    # exclude that from WPARs as it is exactly same as on hosted LPAR
    $item_agent[$item_agent_indx] = "pg2";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">Paging 2</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $wpar == 0 )
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "lan" ) ) {
  $item_agent[$item_agent_indx] = "lan";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">LAN</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "san-" ) ) {
  $item_agent[$item_agent_indx] = "san1";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
  $item_agent[$item_agent_indx] = "san2";
  $tab_number++;
  $os_agent = $tab_number;
  my $iops_type = "IOPS";
  if ( ! -f "$wrkdir/$server/$host/$lpar_slash/cpu.txt" ) {
      # Linux --> there are not IOPS but SAN Frames
      $iops_type = "Frames";
  }
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN $iops_type</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "san_resp" ) ) {
  $item_agent[$item_agent_indx] = "san_resp";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN RESP</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "ame" ) ) {
  $item_agent[$item_agent_indx] = "ame";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">AME</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "sea" ) ) {
  $item_agent[$item_agent_indx] = "sea";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SEA</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "wlm" ) ) {
  #my $is_nusage    = test_metric_in_rrd( "vmware_VMs", "", "$lpar.rrm", $item, "Network_usage" );
  my $wlm_cpu  = "";
  my $wlm_mem  = "";
  my $wlm_dkio = "";
  opendir( DIR, "$wrkdir/$server/$host/$lpar_slash" ) || error( "Error in opening dir \"$wrkdir/$server/$host/$lpar_slash\" $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @wlm_files = readdir(DIR);
  closedir(DIR);
  foreach my $each ( @wlm_files ) {
    next if ($each !~ /wlm-/) ;
    my $lpar_slash_file = $lpar_slash;
    $lpar_slash_file .= "/$each";
    #$wlm_cpu = test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_cpu" );
    #$wlm_mem = test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_mem" );
    #$wlm_cpu = test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_dkio");
    if ( test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_cpu" ) ) {
      $wlm_cpu = 1;
    }
    if ( test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_mem" )) {
      $wlm_mem = 1;
    }
    if ( test_metric_in_rrd ( $server, $host, $lpar_slash_file, "wlm", "wlm_dkio")) {
      $wlm_dkio = 1;
    }
  #my ( $server, $host, $lpar, $item, $ds ) = @_;
  }
  #return $found;
  if ( ( ! $wlm_cpu eq "" && isdigit($wlm_cpu) && $wlm_cpu == 1) || $lpar_slash !~ /\// ) {
    $item_agent[$item_agent_indx] = "wlm-cpu";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">WLM-CPU</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  }
  if ((! $wlm_mem eq "" && isdigit($wlm_mem) && $wlm_mem == 1) || $lpar_slash !~ /\// ) {
    $item_agent[$item_agent_indx] = "wlm-mem";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">WLM-MEM</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  }
  if ((! $wlm_dkio eq "" && isdigit($wlm_dkio) && $wlm_dkio == 1) || $lpar_slash !~ /\// ) {
    $item_agent[$item_agent_indx] = "wlm-dkio";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">WLM-DKIO</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  }
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash", "FS", "csv" ) ) {
  $item_agent[$item_agent_indx] = "file_sys";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">FS</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entile == 0 ...
if ( $entitle == 0 && $item eq "lpar" && test_file_in_directory( "$wrkdir/Solaris/$lpar_slash", "san-c" ) ) {
  $item_agent[$item_agent_indx] = "s_ldom_san1";
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number; #### without ldoms
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 &&  $item eq "lpar" && test_file_in_directory( "$wrkdir/Solaris/$lpar_slash", "san-c" ) ) {
  $item_agent[$item_agent_indx] = "s_ldom_san2"; #### without ldoms
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN IOPS</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)
if ( $entitle == 0 &&  $item eq "lpar" && test_file_in_directory( "$wrkdir/Solaris/$lpar_slash", "san_tresp" ) ) {
  $item_agent[$item_agent_indx] = "s_ldom_san_resp"; #### without ldoms
  $tab_number++;
  $os_agent = $tab_number;
  print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">SAN RESP</a></li>\n";
  $item_agent_tab[$item_agent_indx] = $tab_number;
  $item_agent_indx++;
} ## end if ( $entitle == 0 && ...)

# NMON
if ( -d "$wrkdir/$server/$host/$lpar_slash$NMON" ) {
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "cpu" ) ) {
    $item_agent[$item_agent_indx] = "nmon-oscpu";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">CPU OS-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $entitle == 0 && ...)
  if ( test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "mem" ) ) {
    $item_agent[$item_agent_indx] = "nmon-mem";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">Memory-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( test_file_in_directory...)
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "pgs" ) ) {
    $item_agent[$item_agent_indx] = "nmon-pg1";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">Paging 1-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
    if ( $wpar == 0 ) {

      # exclude that from WPARs as it is exactly same as on hosted LPAR
      $item_agent[$item_agent_indx] = "nmon-pg2";
      $tab_number++;
      $nmon_agent = $tab_number;
      print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">Paging 2-N</a></li>\n";
      $item_agent_tab[$item_agent_indx] = $tab_number;
      $item_agent_indx++;
    } ## end if ( $wpar == 0 )
  } ## end if ( $entitle == 0 && ...)
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "lan" ) ) {
    $item_agent[$item_agent_indx] = "nmon-lan";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">LAN-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $entitle == 0 && ...)
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "san-" ) ) {
    $item_agent[$item_agent_indx] = "nmon-san1";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">SAN-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
    $item_agent[$item_agent_indx] = "nmon-san2";
    $tab_number++;
    $nmon_agent = $tab_number;

    if ( test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "san-host" ) ) {

      # Linux --> there are not IOPS but SAN Frames
      $iops = "Frames";
    }
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">SAN $iops</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $entitle == 0 && ...)
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "ame" ) ) {
    $item_agent[$item_agent_indx] = "nmon-ame";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">AME-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $entitle == 0 && ...)
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$NMON", "sea" ) ) {
    $item_agent[$item_agent_indx] = "nmon-sea";
    $tab_number++;
    $nmon_agent = $tab_number;
    print "  <li class='tabnmon'><a href=\"#tabs-$tab_number\">SEA-N</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;
  } ## end if ( $entitle == 0 && ...)
} ## end if ( -d "$wrkdir/$server/$host/$lpar_slash$NMON")

# NMON end

# AS400 agent tabs
# print STDERR "testing AS400 $wrkdir/$server/$host/$lpar_slash$AS400\n";
if ( -d "$wrkdir/$server/$host/$lpar_slash$AS400" ) {
  if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$AS400", "S0200ASPJOB" ) ) {

    # print STDERR "detail-cgi.pl 573 $wrkdir/$server/$host/$lpar_slash, S0200ASPJOB\n";

    $item_agent[$item_agent_indx] = "job_cpu";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">WRKACTJOB</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "waj";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">CPUTOP</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "disk_io";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">IOTOP</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$AS400/DSK", ".*", "mmc" ) ) {
      $item_agent[$item_agent_indx] = "disk_busy";
      $tab_number++;
      $os_agent = $tab_number;
      print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">DISKTOP</a></li>\n";
      $item_agent_tab[$item_agent_indx] = $tab_number;
      $item_agent_indx++;
    } ## end if ( $entitle == 0 && ...)

    $item_agent[$item_agent_indx] = "S0200ASPJOB";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">JOBS</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    #$item_agent[$item_agent_indx] = "ASP" ;
    #$tab_number++;
    #$os_agent = $tab_number;
    #print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP</a></li>\n";
    #$item_agent_tab[$item_agent_indx] = $tab_number;
    #$item_agent_indx++;

    $item_agent[$item_agent_indx] = "size";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">POOL SIZE</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    # excluded - not interesting metric
    #$item_agent[$item_agent_indx] = "res" ;
    #$tab_number++;
    #$os_agent = $tab_number;
    #print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">POOL RES</a></li>\n";
    #$item_agent_tab[$item_agent_indx] = $tab_number;
    #$item_agent_indx++;

    $item_agent[$item_agent_indx] = "threads";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">THREADS</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "faults";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">FAULTS</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "pages";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">PAGES</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    # excluded - not interesting metric
    #$item_agent[$item_agent_indx] = "ADDR" ;
    #$tab_number++;
    #$os_agent = $tab_number;
    #print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ADDR</a></li>\n";
    #$item_agent_tab[$item_agent_indx] = $tab_number;
    #$item_agent_indx++;

    $item_agent[$item_agent_indx] = "cap_used";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP USED</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "cap_free";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP FREE</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "data_as";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP DATA</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    $item_agent[$item_agent_indx] = "iops_as";
    $tab_number++;
    $os_agent = $tab_number;
    print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP IOPS</a></li>\n";
    $item_agent_tab[$item_agent_indx] = $tab_number;
    $item_agent_indx++;

    if ( $entitle == 0 ) { # && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$AS400/LTC", ".*", "mmc" ) ) left_curly  # always prepare tabs
      $item_agent[$item_agent_indx] = "dsk_latency";
      $tab_number++;
      $os_agent = $tab_number;
      print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">ASP LATENCY</a></li>\n";
      $item_agent_tab[$item_agent_indx] = $tab_number;
      $item_agent_indx++;
    } ## end if ( $entitle == 0 && ...)

    if ( $entitle == 0 && test_file_in_directory( "$wrkdir/$server/$host/$lpar_slash$AS400/IFC", ".*", "mmc" ) ) {
      $item_agent[$item_agent_indx] = "data_ifcb";
      $tab_number++;
      $os_agent = $tab_number;
      print "  <li class='tabagent'><a href=\"#tabs-$tab_number\">LAN</a></li>\n";
      $item_agent_tab[$item_agent_indx] = $tab_number;
      $item_agent_indx++;
    } ## end if ( $entitle == 0 && ...)
  } ## end if ( $entitle == 0 && ...)
} ## end if ( -d "$wrkdir/$server/$host/$lpar_slash$AS400")
} ## end sub build_agents_tabs


sub is_lpm {
  my $host      = shift;
  my $server    = shift;
  my $lpar      = shift;
  my $lpm       = shift;
  my $lpm_count = 0;

  if ( $lpm == 1 ) {
    my $lpm_suff = "rrl";

    # never use function "grab", grrrr ...
    my $server_space = $server;
    if ( $server =~ m/ / ) {    # workaround for server name with a space inside, nothing else works, grrr
      $server_space = "\"" . $server . "\"";
    }
    my $host_space = $host;
    if ( $host =~ m/ / ) {      # workaround for server name with a space inside, nothing else works, grrr
      $host_space = "\"" . $host . "\"";
    }
    my $lpar_space = $lpar;
    if ( $lpar =~ m/ / ) {      # workaround for server name with a space inside, nothing else works, grrr
      $lpar_space = "\"" . $lpar . "\"";
    }
    $lpar_space =~ s/\//&&1/g;

    foreach my $trash (<$wrkdir/$server_space/$host_space/$lpar_space=====*=====*.$lpm_suff>) {
      $lpm_count++;
    }
    $lpm_suff = "rrk";
    foreach my $trash (<$wrkdir/$server_space/$host_space/$lpar_space=====*=====*.$lpm_suff>) {
      $lpm_count++;
    }
    $lpm_suff = "rri";
    foreach my $trash (<$wrkdir/$server_space/$host_space/$lpar_space=====*=====*.$lpm_suff>) {
      $lpm_count++;
    }

    # print STDERR "001 $host,$server,$lpar,$lpm : $lpm_count\n";

    if ( !-f "$wrkdir/$server/$host/lpm-exclude.txt" ) {
      return $lpm_count;
    }

    # if it is a VIO server then do not go for LPM
    # it is already being exlcuded in install-html.sh, this is just for sure
    my $lpm_excl = "$wrkdir/$server/$host/lpm-exclude.txt";
    open( FH, "< $lpm_excl" ) || return $lpm_count;
    my @lpm_excl_vio = <FH>;
    close(FH);

    foreach my $lpm_line (@lpm_excl_vio) {
      chomp($lpm_line);
      if ( "$lpm_line" =~ m/$lpar/ && length($lpm_line) == length($lpar) ) {

        #print STDERR "LPM VIO exclude: $host:$server:$lpar: - $lpm_line\n" if $DEBUG ;
        $lpm_count = 0;
        last;
      } ## end if ( "$lpm_line" =~ m/$lpar/...)
    } ## end foreach my $lpm_line (@lpm_excl_vio)
  } ## end if ( $lpm == 1 )

  #print STDERR "002 $host,$server,$lpar,$lpm : $lpm_count\n";

  return $lpm_count;
} ## end sub is_lpm

sub type_sam {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $yearly     = shift;
  my $lpar_slash = shift;

  if ( $yearly == 1 ) {
    if ( -f "$wrkdir/$server/$host/$lpar_slash.rrd" ) {
      return "d";
    }
    return "m";
  } ## end if ( $yearly == 1 )

  if ( -f "$wrkdir/$server/$host/$lpar_slash.rrm" ) {
    return "m";
  }

  if ( -f "$wrkdir/$server/$host/$lpar_slash.rrh" ) {
    return "h";
  }

  return "m";
} ## end sub type_sam

# find out max value to get fixed upper limit for all 4 graphs
sub rrd_upper {
  my $wrkdir        = shift;
  my $server        = shift;
  my $host          = shift;
  my $lpar          = shift;
  my $type_sam      = shift;
  my $type_sam_year = shift;
  my $item          = shift;
  my $lpar_slash    = shift;
  my $rrd           = "$wrkdir/$server/$host/$lpar_slash.rr$type_sam";

  if ( !-f "$rrd" || $item =~ m/^codused$/ || $item =~ m/^pagingagg$/ ) {

    # Capacity on demad also does not go here
    return 0;
  }

  my $rrdtool = $ENV{RRDTOOL};

  # start RRD via a pipe
  RRDp::start "$rrdtool";

  # print STDERR "------ rrd_upper 2 $wrkdir/$server/$host/$lpar_slash.rr$type_sam\n";

  my $ret   = find_rrd_upper( $rrd, "now-1d" );
  my $ret_w = find_rrd_upper( $rrd, "now-1w" );
  my $ret_m = find_rrd_upper( $rrd, "now-1m" );

  $rrd = "$wrkdir/$server/$host/$lpar_slash.rr$type_sam_year";
  if ( !-f "$rrd" ) {
    $rrd = "$wrkdir/$server/$host/$lpar_slash.rr$type_sam";
  }
  my $ret_y = find_rrd_upper( $rrd, "now-1y" );

  if ( $ret_w > $ret ) { $ret = $ret_w; }
  if ( $ret_m > $ret ) { $ret = $ret_m; }
  if ( $ret_y > $ret ) { $ret = $ret_y; }

  # close RRD pipe
  RRDp::end;

  return $ret;

} ## end sub rrd_upper

sub find_rrd_upper {
  my $rrd        = shift;
  my $start_time = shift;

  $rrd =~ s/:/\\:/g;

  if ($vmware) {
    RRDp::cmd qq(graph "tmp/name.png"
	   "--start" "$start_time"
	   "--end" "now"
       "DEF:utl=$rrd:CPU_usage:AVERAGE"
	   "PRINT:utl:MAX: %3.3lf"
	 );
  } ## end if ($vmware)
  else {
    RRDp::cmd qq(graph "tmp/name.png"
       "--start" "$start_time"
       "--end" "now"
       "DEF:cur=$rrd:curr_proc_units:AVERAGE"
       "DEF:ent=$rrd:entitled_cycles:AVERAGE"
       "DEF:cap_peak=$rrd:capped_cycles:AVERAGE"
       "DEF:uncap=$rrd:uncapped_cycles:AVERAGE"
       "CDEF:cap=cap_peak,ent,/,1,GT,UNKN,cap_peak,IF"
       "CDEF:tot=cap,uncap,+"
       "CDEF:util=tot,ent,/,$cpu_max_filter,GT,UNKN,tot,ent,/,IF"
       "CDEF:utiltot=util,cur,*"
       "PRINT:utiltot:MAX: %3.3lf"
     );
  } ## end else [ if ($vmware) ]
  my $answer = RRDp::read;

  if ( $$answer =~ m/NaN/ || $$answer =~ m/nan/ ) {
    return 0;
  }
  ( my $addr, my $max ) = split( / +/, $$answer );
  chomp($max);

  if ( isdigit($max) == 1 ) {
    return $max;
  }
  return 0;
} ## end sub find_rrd_upper

# CoD  codunreport
sub print_cod {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam, $entitle, $upper ) = @_;

  print "<table align=\"center\" summary=\"Graphs\">\n";
  print OUT "006 CoD : $item \n" if $DEBUG == 2;

  #print "<tr><td colspan=\"2\" align=\"center\"><A NAME=COD></A> <H3>Capacity on Demand</H3> <BR></td></tr>\n";
  $item = "codunreport";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print "</tr></table>";

  return 1;
} ## end sub print_cod

# print Lpar cfg table
sub print_lpar_cfg {
  my ( $host_url, $server_url, $lpar_url, $wrkdir, $server, $host, $lpar_slash, $lpar ) = @_;

  my $cfg = "$wrkdir/$server/$host/cpu.cfg-$lpar_slash";
  print "</center>\n";
  if ( -f "$cfg" ) {
    open( FH, " < $cfg" ) || error( "Cannot open $cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line (<FH>) {
      if ( $line =~ m/logical_serial_num/ ) {    #exclude serial info
        next;
      }
      else {
        print $line;
      }
    } ## end foreach my $line (<FH>)
  } ## end if ( -f "$cfg" )

  #print "LPAR configuration: <A HREF=\"$host_url/$server_url/config.html#$lpar_url\"><b>$lpar</b></A>\n";
  return 1;
} ## end sub print_lpar_cfg

# print a link with detail
sub print_item {
  my ( $host_url, $server_url, $lpar_url, $item, $time_graph, $type_sam, $entitle, $detail, $refresh, $nostar, $colspan, $legend ) = @_;

  return if $false_picture eq "do not print";

  my $legend_class = "";
  if ( $legend =~ m/nolegend/ ) {
    $legend_class = "nolegend";
  }
  my $legend_higher = "legend";
  if ( $legend =~ m/higher/ ) {
    $legend_higher = "legend higher";
  }

  my $favs_favoff = "<div class=\"favs favoff\"></div>";    # dash-board-able
  if ( $nostar =~ m/nostar/ ) {
    $favs_favoff = "";
  }
  if ( $refresh =~ m/norefr/ ) {
    $refresh = "";
  }
  my $colspan_text = "";
  if ( $colspan == 2 ) {
    $colspan_text = "colspan=\"2\"";
  }
  my $xdetail = 9;

  my $u_time = time * (-1);
  if ($rest_api){ $refresh = ""; }
  if ( $detail > 0 ) {
    print "          <td$colspan_text class=\"relpos\">
            <div>";
    if ($false_picture ne "") {
      print "$false_picture";
    }
    else {
      print"  <div class=\"g_title\">
                $favs_favoff
                $refresh
                <div class=\"popdetail\"></div>
                <div class=\"g_text\" data-server=\"$server_url\"data-lpar=\"$lpar_url\" data-item=\"$item\" data-time=\"$time_graph\"><span class=\"tt_span\"></span></div>
              </div>
              <a class=\"detail\" href=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=1&upper=$upper&entitle=$entitle&none=none\">
                <div title=\"Click to show detail\"><img class=\"$legend_class lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=$xdetail&upper=$upper&entitle=$entitle&none=$u_time\" src=\"css/images/sloading.gif\">
                  <div class=\"zoom\" title=\"Click and drag to select range\"></div>
                </div>
              </a>
              <div class=\"$legend_higher\"></div>
              <div class=\"updated\"></div>";
    }
    print " </div>
          </td>\n";
  } ## end if ( $detail > 0 )
  else {
    print "<td $colspan_text align=\"center\"><div><img class=\"$legend_class lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=0&upper=$upper&entitle=$entitle&none=$u_time\" src=\"css/images/sloading.gif\"></div></td>\n";
  }
  return 1;
} ## end sub print_item

# print a link with detail into $tab_exe
sub print_item_tab_exe {
  my ( $host_url, $server_url, $lpar_url, $item, $time_graph, $type_sam, $entitle, $detail, $refresh, $nostar ) = @_;

  my $favs_favoff = "<div class=\"favs favoff\"></div>";    # dash-board-able
  if ( $nostar =~ m/nostar/ ) {
    $favs_favoff = "";
  }
  if ( $refresh =~ m/norefr/ ) {
    $refresh = "";
  }

  my $u_time = time * (-1);
  if ( $detail > 0 ) {
    $tab_exe .= "<td valign=\"top\" class=\"relpos\"><div>$favs_favoff<div class=\"popdetail\"></div>$refresh<a class=\"detail\" href=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=1&upper=$upper&entitle=$entitle&none=none\"><font color=\"#C8C8C8\" title=\"Click to show detail\"><img class=\"lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=0&upper=$upper&entitle=$entitle&none=$u_time&none=$u_time\" src=\"css/images/loading.gif\"><div class=\"zoom\" title=\"Click and drag to select range\"></div></font></a></div></td>\n";
  }
  else {
    $tab_exe .= "<td align=\"center\" valign=\"top\" colspan=\"2\"><font color=\"#C8C8C8\"><img class=\"lazy\" border=\"0\" data-src=\"/lpar2rrd-cgi/$detail_graph.sh?host=$host_url&server=$server_url&lpar=$lpar_url&item=$item&time=$time_graph&type_sam=$type_sam&detail=0&upper=$upper&entitle=$entitle&none=$u_time&none=$u_time\" src=\"css/images/loading.gif\"></td>\n";
  }
  return 1;
} ## end sub print_item_tab_exe

sub test_file_in_directory {

  # same sub in lpar-list-cgi.pl
  # Use a regular expression to find files
  #    beginning by $fpn
  #    ending by .mmm or ending is 3rd param - if used
  #    returns 0 (zero) or first filename found i.e. non zero
  #    special care for san- sea-

  my $dir    = shift;
  my $fpn    = shift;
  my $ending = shift;

  $ending = "mmm" if !defined $ending;

  # searching OS agent file
  my $found = 0;
  if ( !-d $dir ) { return $found; }
  opendir( DIR, $dir ) || error( "Error in opening dir $dir $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  while ( my $file = readdir(DIR) ) {
    if ( $file =~ m/^$fpn.*\.$ending$/ ) {
      $found = "$dir/$file";
      last;
    }
  } ## end while ( my $file = readdir...)
  closedir(DIR);
  return $found;
} ## end sub test_file_in_directory

# paging aggregated
sub print_pagingagg {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam, $bmax, $upper ) = @_;

  # print STDERR "2056 bmax $bmax\n";
  #my $bmax = $entitle; # entitle passes max of paging peak when lpar is displayed then
  $bmax =~ s/MAX=//;
  if ( !isdigit($bmax) ) {
    $bmax = 10;
  }
  if ( $bmax <= 0 ) { $bmax = 10; }    # default

  print "<form method=\"GET\" action=\"/lpar2rrd-cgi/detail.sh\">\n";
  $server_url = urldecode ($server_url);
  # print STDERR "2068 \$host_url $host_url \$server_url $server_url \$lpar_url $lpar_url\n";
  print "<INPUT type=\"hidden\" name=\"host\" value=\"$host_url\">\n";
  print "<INPUT type=\"hidden\" name=\"server\" value=\"$server_url\">\n";
  print "<INPUT type=\"hidden\" name=\"lpar\" value=\"$lpar_url\">\n";
  print "<INPUT type=\"hidden\" name=\"item\" value=\"pagingagg\">\n";
  print "<p style=\"text-align: right\"><font size=-1>\n";
  print "Displayed are only LPARs with MAX above ";
  print "<INPUT TYPE=\"TEXT\" TITLE=\"Write new Max\" onblur=\"this.style.color='#f00'\" size=\"2\" NAME=\"MAX\" VALUE=\"$bmax\">\n";
  print "<INPUT type=\"hidden\" name=\"gui\" value=\"$gui\">\n";
  print "<INPUT TYPE=\"SUBMIT\" NAME=\"peak\" VALUE=\"kB/s\" ALT=\"MAX peak\">\n";
  print "</font></p>\n";
  print "</form>\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print OUT "006 CoD : $item \n" if $DEBUG == 2;
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $bmax, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $bmax, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $bmax, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $bmax, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>\n";
  print "<ul style=\"display: none\"><li class=\"tabagent\"></li></ul>\n";    # to add the data source icon

  return 1;

} ## end sub print_pagingagg

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub print_hea {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  my $cfg             = "$wrkdir/$server/$host/config.cfg";
  my $first           = 0;
  my $act_time        = time();
  my $port_group      = 0;
  my $phys_port_id    = 0;
  my $port            = "";
  my $type_sam        = "";
  my $last_adapter_id = 0;
  my $adapter_count   = 0;
  my $tab_count       = 0;
  my $adapter_id;

  $tab_exe = "";    # must be global

  # could be none or up to 4 adapters (more ?), each up to 4 ports
  # each port has its own database
  opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server/$host" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files_not_sorted = grep( /.*-.*\.db$/, readdir(DIR) );
  closedir(DIR);
  my $file  = "";
  my @files = sort @files_not_sorted;

  print "<div  id=\"tabs\">\n";    # prepare tab declarative
  print "<ul>\n";

  foreach $file (@files) {
    if ( $file =~ "HSCL" || $file =~ "VIOSE0" ) {
      unlink($file);
      next;                        # some error
    }
    $tab_count++;
    chomp($file);
    ( undef, $adapter_id, my $port ) = split( /-/, $file );
    $port =~ s/\.db//;
    my $port_whole = $port;
    $port =~ s/port//g;

    #print "-- $file $port\n";
    if ( $last_adapter_id ne $adapter_id ) {
      $adapter_count++;
      $last_adapter_id = $adapter_id;
    }
    print "  <li class=\"tabhmc\"><a href=\"#tabs-$tab_count\">HEA$adapter_count-$port</a></li>\n";
    my $rrd = "$wrkdir/$server/$host/$file";

    # prepare tab executive
    $tab_exe .= "<div id=\"tabs-$tab_count\">\n";
    $tab_exe .= "<center>";

    $tab_exe .= "<table align=\"center\" summary=\"Graphs\">\n";
    $tab_exe .= "<tr><td colspan=\"2\" align=\"center\"><A NAME=HEA></A> <H3>$adapter_id-$port_whole</H3> <BR></td></tr>\n";

    $tab_exe .= "<tr>";
    $item = $file;
    print_item_tab_exe( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item_tab_exe( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    $tab_exe .= "</tr>\n<tr>\n";
    print_item_tab_exe( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item_tab_exe( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    $tab_exe .= "</tr></table>";

    my $lpar_logical_port_table = "";
    $first = 0;
    if ( -f "$cfg" ) {
      my $port_id = $port;
      my $line    = "";
      $port_id =~ s/port//g;
      my $ret = substr( $port_id, 0, 1 );
      if ( $ret =~ /\D/ ) {
        error( "Error $host: Wrong input data, HEA port : $port" . __FILE__ . ":" . __LINE__ ) && return 0;
        return 1;
      }

      if ( $port_id == 1 ) { $phys_port_id = 0; $port_group = 1; }
      if ( $port_id == 2 ) { $phys_port_id = 1; $port_group = 1; }
      if ( $port_id == 3 ) { $phys_port_id = 0; $port_group = 2; }
      if ( $port_id == 4 ) { $phys_port_id = 1; $port_group = 2; }

      my @hea_ports = `grep $adapter_id "$cfg" |grep phys_port_id=$phys_port_id|grep " port_group=$port_group"|grep " lpar_name="|sort|uniq|awk -F= '{print \$4,"%",\$8}'|sed -e 's/port_group//g' -e 's/drc_index//g'`;

      foreach $line (@hea_ports) {
        chomp($line);
        if ( $first == 0 ) {
          $lpar_logical_port_table .= "<BR><TABLE class=\"tabconfig\">\n";
          $lpar_logical_port_table .= "<TR> <TD><font size=-1><B>LPAR&nbsp;&nbsp;&nbsp;&nbsp;</B></font></TD> <TD align=\"center\"><font size=-1><B>Logical port</font></B></TD></TR>\n";

          $first = 1;
        } ## end if ( $first == 0 )
        ( my $lpar, my $lport ) = split( /%/, $line );
        $lpar =~ s/ state //;
        $lpar_logical_port_table .= "<TR> <TD><font size=-1>$lpar</font></TD> <TD align=\"center\"><font size=-1>$lport</font></TD></TR>\n";

      } ## end foreach $line (@hea_ports)
      if ( $first == 1 ) {
        $lpar_logical_port_table .= "</table><BR>\n";
      }
    } ## end if ( -f "$cfg" )

    $tab_exe .= "$lpar_logical_port_table";
    $tab_exe .= "</div>\n";

  } ## end foreach $file (@files)
  print "</ul>\n";
  print "$tab_exe";

  return 1;
} ## end sub print_hea

sub print_power_adapters{
  #host=vhmc&server=P02DR__9117-MMC-SN44K8102&item=power_lan&entitle=0&gui=1&none=none
  my @array = @_;
  foreach my $item (@array){
    if ($item =~ m/=/){
      #(undef, $item) = split ("=", $item);
    }
  }
  my ($host_url, $server_url, $lpar_url, $item, $entitle) = @_;
  my $legend;
  if ($item =~ m/power_sri/){$legend = "nolegend";} #one item is aggregated graph, so need nolegend in graph
  else {$legend = "legend";}
  print "<CENTER>\n";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li ><a href=\"#tabs-1\">Data</a></li>\n";
  if    ( $item =~ m/lan/ ){    print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ( $item =~ m/hea/ ){    print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ( $item =~ m/sri/ ){    print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ( $item =~ m/san/ ){    print "  <li ><a href=\"#tabs-2\">IOPS</a></li>\n";}
  elsif ( $item =~ m/sas/ ){    print "  <li ><a href=\"#tabs-2\">IOPS</a></li>\n";}
  print "</ul>\n";

  my $item1 = "$item\_data";
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$item1,"d",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print_item ($host_url,$server_url,$lpar_url,$item1,"w",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$item1,"m",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print_item ($host_url,$server_url,$lpar_url,$item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print "</tr>";
  print "</table>";
  print "</div>";

  my $item2 = "$item\_io";
  print "<div id=\"tabs-2\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$item2,"d",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print_item ($host_url,$server_url,$lpar_url,$item2,"w",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$item2,"m",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print_item ($host_url,$server_url,$lpar_url,$item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","nostar",1,"$legend");
  print "</tr>";
  print "</table>";
  print "</div>";

  print "</div>\n";
  return 1;
}

sub print_power_adapters_agg{
  #host=vhmc&server=P02DR__9117-MMC-SN44K8102&item=power_lan&entitle=0&gui=1&none=none
  my @array = @_;
  my ($host_url, $server_url, $lpar_url, $item, $entitle) = @_;
  print "<CENTER>\n";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li ><a href=\"#tabs-1\">Data</a></li>\n";
  if ($item =~ m/power_lan/   ){  print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ($item =~ m/power_sri/){  print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ($item =~ m/power_hea/){  print "  <li ><a href=\"#tabs-2\">Packets</a></li>\n";}
  elsif ($item =~ m/power_san/){  print "  <li ><a href=\"#tabs-2\">IOPS</a></li>\n";}
  elsif ($item =~ m/power_sas/){  print "  <li ><a href=\"#tabs-2\">IOPS</a></li>\n";}
  print "</ul>\n";

  my $item1 = "$item\_data";
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$item1,"d",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$item1,"w",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$item1,"m",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print "</tr>";
  print "</table>";
  print "</div>";

  my $item2 = "$item\_io";
  print "<div id=\"tabs-2\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$item2,"d",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$item2,"w",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$item2,"m",$type_sam,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","nostar",1,"nolegend");
  print "</tr>";
  print "</table>";
  print "</div>";
  print "</div>\n";
  return 1;
}

sub print_solaris
{
  my ($host_url,$server_url,$lpar_url,$item,$type_sam) = @_;
  my $tabs = 1;
  my $zone_name = $item;
  $zone_name =~ s/sol11-//g;
  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU percent</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">Memory</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">Net</a></li>\n";
  $tabs++;
  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/cpu.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU OS</a></li>\n";
    $tabs++;
  }
  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/mem.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Memory OS</a></li>\n";
    $tabs++;
  }
  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/pgs.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Paging 1</a></li>\n";
    $tabs++;
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Paging 2</a></li>\n";
    $tabs++;
  }
  my @lan_path;
  if (-d "$wrkdir/Solaris--unknown/no_hmc/$zone_name/"){
    opendir( DIR, "$wrkdir/Solaris--unknown/no_hmc/$zone_name/" ) || error( "can't opendir $wrkdir/Solaris--unknown/no_hmc/$zone_name/: $! :" . __FILE__ . ":" . __LINE__ );
    @lan_path = grep !/^\.\.?$/, readdir(DIR);
    my $i = 0;
    foreach my $lan_name(@lan_path){
      if ($lan_name =~ /lan/){
        if ($i == 0){
          $i++;
          print "  <li class=\"\"><a href=\"#tabs-$tabs\">LAN</a></li>\n";
          $tabs++;
        }
      }
    }
  }
  my $file_fs = "$wrkdir/Solaris--unknown/no_hmc/$lpar/FS.csv";
  if (-f $file_fs){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">FS</a></li>\n";
    $tabs++;
  } ## end if ( $item =~ m/file_sys/)


  print "</ul>\n";

  my $s_item1 = "";
  #if ($item =~ /sol11-global|system|total/){
  #  $s_item1 = "s_s";
  #}
  #else{
  #  $s_item1 = "s_z";
  #}
  $tabs = 1;
  $s_item1 .= "solaris_zone_cpu";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div>\n";

  my $s_item2 = "";
  #if ($item =~ /sol11-global|system|total/){
  #  $s_item2 = "s_s";
  #}
  #else{
  #  $s_item2 = "s_z";
  #}
  $s_item2 .= "solaris_zone_os_cpu";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div>\n";
  my $s_item3 = "";
  #if ($item =~ /sol11-global|system|total/){
  #  $s_item3 = "s_s";
  #}
  #else{
  #  $s_item3 = "s_z";
  #}
  $s_item3 .= "solaris_zone_mem";
  # $item="pool";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item3,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item3,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item3,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item3,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div>\n";

  my $s_item4 = "";
  #if ($item =~ /sol11-global|system|total/){
  #  $s_item4 = "s_s";
  #}
  #else{
  #  $s_item4 = "s_z";
  #}
  $s_item4 .= "solaris_zone_net";
  # $item="pool";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item4,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item4,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item4,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item4,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div>\n";

  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/cpu.mmm"){
    my $s_item5 = "oscpu";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item5,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item5,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item5,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item5,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }

  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/mem.mmm"){
    my $s_item6 = "mem";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item6,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item6,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item6,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item6,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }
  if (-f "$wrkdir/Solaris--unknown/no_hmc/$zone_name/pgs.mmm"){
    my $s_item7 = "pg1";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
    my $s_item8 = "pg2";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }
  my $j = 0;
  foreach my $lan_name(@lan_path){
    if ($lan_name =~ /lan/){
      if ($j == 0){
        my $s_item9 = "lan";
        $host_url ="no_hmc";
        $server_url ="Solaris--unknown";
        print "<div id=\"tabs-$tabs\">\n";
        #$tabs++;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item9,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item9,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item9,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item9,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr></table>";
        $j++;

        my $text_sum = "Summary of transferred Bytes a Day";
        $s_item9 = "slan";
        print "<table align=\"center\" summary=\"Graphs\">\n";
        print "<tr><td colspan=\"2\"><hr></td></tr>\n";
        print "<tr><td colspan=\"2\" align=\"center\"><h4>$text_sum</h4></td></tr>\n";
        print "<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
        print "</tr>\n";
        print "</table>\n";

        $s_item9 = "packets_lan";
        print "<table align=\"center\" summary=\"Graphs\">\n";
        print "<tr><td colspan=\"2\"><hr></td></tr>\n";
        print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets</h4></td></tr>\n";
        print "<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print "</tr>\n<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item9, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print "</tr>\n";
        print "</table>\n";
        print "</div>\n";
        $tabs++;
      }
    }
  }
  if (-f $file_fs){
    print "<div id =\"tabs-$tabs\">
    <center>
    <h4>Filesystem usage</h4>
    <tbody>
    <tr>
    <table class =\"tabconfig tablesorter\"data-sortby=\"5\">
    <thead>
    <tr><th class = \"sortable\">Filesystem&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Total [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Used [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Available [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Usage [%]&nbsp;&nbsp;</th>
    <th class = \"sortable\">Mounted on&nbsp;&nbsp;</th>
    </tr></thead>\n\n";
    if ( -f "$file_fs" ) {
      open( FH, "< $file_fs" ) || error( "Cannot open $file_fs: $!" . __FILE__ . ":" . __LINE__ );
      my @file_sys_arr = <FH>;
      close(FH);
      foreach my $line (@file_sys_arr)
      {
        chomp ($line);
        (my $filesystem, my $blocks, my $used, my $avaliable, my $usage, my $mounted) = split(" ", $line);
        print "<tr><td>$filesystem</td>
        <td>$blocks</td>
        <td>$used</td>
        <td>$avaliable</td>
        <td>$usage</td>
        <td>$mounted</td></tr>";
      }
    } ## end if ( -f "$file" )
    my $last_update = localtime((stat($file_fs))[9]);
    print "<tfoot><tr><td colspan=\"6\">Last update time: $last_update</td></tr></tfoot>";
    print "</tr>
    </tbody>
    </table>
    </center>
    </div>\n";
    $tab_number++;
  } ## end if ( $item =~ m/file_sys/)
  #print "</div><br>\n";
  return 1;

}

sub print_solaris10
{
  my ($host_url,$server_url,$lpar_url,$item,$type_sam) = @_;
  my $tabs = 1;

  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">MEM</a></li>\n";
  $tabs++;
  print "</ul>\n";

  my $s_item1 = "";
  if ($item =~ /sol11-global|system|total/){
    $s_item1 = "s10_s";
  }
  else{
    $s_item1 = "s10_z";
  }
  $tabs = 1;
  $s_item1 .= "_cpu";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div><br>\n";

  my $s_item2 = "";
  if ($item =~ /sol11-global|system|total/){
    $s_item2 = "s10_s";
  }
  else{
    $s_item2 = "s10_z";
  }
  $s_item2 .= "_mem";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
  print "</tr></table>";
  print "</div><br>\n";

  print "</div><br>\n";
  return 1;
}

sub print_solaris_ldom
{
  my ($host_url,$server_url,$lpar_url,$item,$type_sam) = @_;
  my $tabs = 1;
  my $ldom_uuid = "$lpar_url";
  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  my @solaris_ldoms;
  my @net_path;
  my @solaris_vnet;
  if (-d "$wrkdir/Solaris/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @solaris_ldoms =  grep /_ldom\.mmm$/, readdir(DIR);
    foreach my $solaris_ldom_file (@solaris_ldoms){
      print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU</a></li>\n";
      $tabs++;
      print "  <li class=\"\"><a href=\"#tabs-$tabs\">MEM</a></li>\n";
      $tabs++;
    }
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @net_path = grep !/^\.\.?$|ZONE|_ldom\.mmm$|uuid|solaris11|solaris10|^san|pool/, readdir(DIR);
    my $o1 = 0;
    foreach my $solaris_net_file (@net_path){
      if ($o1 == 0){
        $o1++;
        print "  <li class=\"\"><a href=\"#tabs-$tabs\">NET</a></li>\n";
        $tabs++;
      }
    }
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @solaris_vnet =  grep /^vlan-/, readdir(DIR);
    my $p1 = 0;
    foreach my $solaris_vnet_file (@solaris_vnet){
      if ($p1 == 0){
        $p1++;
        print "  <li class=\"\"><a href=\"#tabs-$tabs\">VNET</a></li>\n";
        $tabs++;
      }
    }
  }
  if (-f"$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/cpu.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU OS</a></li>\n";
    $tabs++;
  }

  if (-d"$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/JOB/"){
    opendir( DIR, "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/JOB/" ) || error( "can't opendir $wrkdir/Solaris--unknown/$ldom_uuid/JOB/: $! :" . __FILE__ . ":" . __LINE__ );
    my @job_path = grep /mmc/, readdir(DIR);
    if (@job_path){
      print "  <li class=\"\"><a href=\"#tabs-$tabs\">JOB</a></li>\n";
      $tabs++;
    }
  }

  if (-f"$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/mem.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Memory OS</a></li>\n";
    $tabs++;
  }
  if (-f"$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/pgs.mmm"){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Paging 1</a></li>\n";
    $tabs++;
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">Paging 2</a></li>\n";
    $tabs++;
  }
  my @solaris_lan_old;
  if (-d "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @solaris_lan_old =  grep /^lan-/, readdir(DIR);
    my $k1 = 0;
    foreach my $solaris_lan_file (@solaris_lan_old){
      if ($k1 == 0){
        $k1++;
        print "  <li class=\"\"><a href=\"#tabs-$tabs\">LAN</a></li>\n";
        $tabs++;
      }
    }
  }
  my @san_path;
  my @san_resp_path1;
  if (-d "$wrkdir/Solaris/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @san_path = grep /^san-c*/, readdir(DIR);
    my $i1 = 0;
    foreach my $san_name(@san_path){
      if ($san_name =~ /san/){
        if ($i1 == 0){
          $i1++;
          print "  <li class=\"\"><a href=\"#tabs-$tabs\">SAN</a></li>\n";
          $tabs++;
          print "  <li class=\"\"><a href=\"#tabs-$tabs\">SAN IOPS</a></li>\n";
          $tabs++;
        }
      }
    }
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @san_resp_path1 = grep /^san_tresp/, readdir(DIR);
    my $j1 = 0;
    foreach my $san_resp_name(@san_resp_path1){
      if ($san_resp_name =~ /san_tresp/){
        if ($j1 == 0){
          $j1++;
          print "  <li class=\"\"><a href=\"#tabs-$tabs\">SAN RESP</a></li>\n";
          $tabs++;
        }
      }
    }
  }
    #print STDERR "1366 JCOM: \$vmware $vmware \$item $item path:$wrkdir/$server/$host/$lpar/FS.csv\n";
  my $file_fs = "$wrkdir/$server/no_hmc/$lpar/FS.csv";
  if (-f $file_fs){
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">FS</a></li>\n";
    $tabs++;
  } ## end if ( $item =~ m/file_sys/)
  my @pool_path;
  if (-d "$wrkdir/Solaris/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @pool_path = grep /pool/, readdir(DIR);
    foreach my $pool_name(@pool_path){
      if ($pool_name =~ /pool/){
        $pool_name =~ s/\.mmm//g;
        print "  <li class=\"\"><a href=\"#tabs-$tabs\">POOL: $pool_name</a></li>\n";
        $tabs++;
      }
    }
  }
  print "</ul>\n";


    $tabs = 1;
  foreach my $solaris_ldom_file (@solaris_ldoms){
    my $s_item1 = "s_ldom_c";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item1,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item1,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item1,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";

    my $s_item2 = "s_ldom_m";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item2,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item2,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item2,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }
  my $a1 = 0;
  foreach my $solaris_net_file (@net_path){
    if ($a1 == 0){
      $a1++;
      my $s_item3 = "s_ldom_n";
      print "<div id=\"tabs-$tabs\">\n";
      $tabs++;
      print "<table border=\"0\">\n";
      print "<tr>";
      print_item ($host_url,$server_url,$lpar_url,$s_item3,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print_item ($host_url,$server_url,$lpar_url,$s_item3,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print "</tr>\n<tr>\n";
      print_item ($host_url,$server_url,$lpar_url,$s_item3,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print_item ($host_url,$server_url,$lpar_url,$s_item3,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print "</tr></table>";

      my $text_sum = "Summary of transferred Bytes a Day";
      $s_item3 = "s_ldom_sum";
      print "<table align=\"center\" summary=\"Graphs\">\n";
      print "<tr><td colspan=\"2\"><hr></td></tr>\n";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>$text_sum</h4></td></tr>\n";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print "</tr>\n";
      print "</table>\n";

      $s_item3 = "s_ldom_pack";
      print "<table align=\"center\" summary=\"Graphs\">\n";
      print "<tr><td colspan=\"2\"><hr></td></tr>\n";
      print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets</h4></td></tr>\n";
      print "<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print_item( $host_url, $server_url, $lpar_url, $s_item3, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      print "</tr></table>";
      print "</div>\n";
    }
  }
  my $q1 = 0;
  foreach my $solaris_vnet_file (@solaris_vnet){
    if ($q1 == 0){
      my $s_item12 = "s_ldom_vnet";
      $host_url ="no_hmc";
      #$server_url ="Solaris--unknown";
      $lpar_url = "$ldom_uuid";
      print "<div id=\"tabs-$tabs\">\n";
      $tabs++;
      $q1++;
      print "<table border=\"0\">\n";
      print "<tr>";
      print_item ($host_url,$server_url,$lpar_url,$s_item12,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print_item ($host_url,$server_url,$lpar_url,$s_item12,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print "</tr>\n<tr>\n";
      print_item ($host_url,$server_url,$lpar_url,$s_item12,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print_item ($host_url,$server_url,$lpar_url,$s_item12,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
      print "</tr></table>";
      print "</div>\n";
    }
  }

  if (-f "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/cpu.mmm"){
    my $s_item7 = "oscpu";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    $lpar_url = "$ldom_uuid";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item7,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }

  if (-d"$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/JOB/"){
    opendir( DIR, "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/JOB/" ) || error( "can't opendir $wrkdir/Solaris--unknown/$ldom_uuid/JOB/: $! :" . __FILE__ . ":" . __LINE__ );
    my @job_path = grep /mmc/, readdir(DIR);
    my $o = 0;
    foreach my $job_name(@job_path){
      if ($o == 0){
        my $s_item_job1 = "jobs";
        my $s_item_job2 = "jobs_mem";
        my $server_url = "Solaris--unknown";
        print "<div id=\"tabs-$tabs\">\n";
        $tabs++;
        $o++;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item_job1,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item_job1,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item_job2,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item_job2,"w",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr></table>";
        print "</div>\n";
      }
    }
  }

  if (-f "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/mem.mmm"){
    my $s_item8 = "mem";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    $lpar_url = "$ldom_uuid";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item8,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }
  if (-f "$wrkdir/Solaris--unknown/no_hmc/$ldom_uuid/pgs.mmm"){
    my $s_item9 = "pg1";
    $host_url ="no_hmc";
    $server_url ="Solaris--unknown";
    $lpar_url = "$ldom_uuid";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item9,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item9,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item9,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item9,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
    my $s_item10 = "pg2";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item ($host_url,$server_url,$lpar_url,$s_item10,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item10,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr>\n<tr>\n";
    print_item ($host_url,$server_url,$lpar_url,$s_item10,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
    print_item ($host_url,$server_url,$lpar_url,$s_item10,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
    print "</tr></table>";
    print "</div>\n";
  }

  my $l = 0;
  foreach my $lan_name(@solaris_lan_old){
    if ($lan_name =~ /lan/){
      if ($l == 0){
        my $s_item11 = "lan";
        $host_url ="no_hmc";
        $server_url ="Solaris--unknown";
        $lpar_url = "$ldom_uuid";
        print "<div id=\"tabs-$tabs\">\n";
        $tabs++;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item11,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item11,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item11,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item11,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr></table>";
        $l++;

        my $text_sum = "Summary of transferred Bytes a Day";
        $s_item11 = "slan";
        print "<table align=\"center\" summary=\"Graphs\">\n";
        print "<tr><td colspan=\"2\"><hr></td></tr>\n";
        print "<tr><td colspan=\"2\" align=\"center\"><h4>$text_sum</h4></td></tr>\n";
        print "<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
        print "</tr>\n";
        print "</table>\n";

        $s_item11 = "packets_lan";
        print "<table align=\"center\" summary=\"Graphs\">\n";
        print "<tr><td colspan=\"2\"><hr></td></tr>\n";
        print "<tr><td colspan=\"2\" align=\"center\"><h4>Packets</h4></td></tr>\n";
        print "<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "d", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "w", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print "</tr>\n<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $s_item11, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
        print "</tr>\n";
        print "</table>\n";
        print "</div>\n";
      }
    }
  }
  my $m = 0;
  foreach my $san_name(@san_path){
    if ($san_name =~ /san-/){
      if ($m == 0){
        $m++;
        my $s_item4 = "s_ldom_san1";
        $san_name =~ s/\.mmm//g;
        print "<div id=\"tabs-$tabs\">\n";
        $server_url ="$ldom_uuid";
        $tabs++;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item4,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item4,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item4,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item4,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr></table>";
        print "</div>\n";
        my $s_item5 = "s_ldom_san2";
        print "<div id=\"tabs-$tabs\">\n";
        $tabs++;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item5,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item5,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item5,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print_item ($host_url,$server_url,$lpar_url,$s_item5,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
        print "</tr></table>";
        print "</div>\n";
      }
    }
  }
  my @san_resp_path2;
  if (-d "$wrkdir/Solaris/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @san_resp_path2 = grep /^san_tresp/, readdir(DIR);
    my $n = 0;
    foreach my $san_resp_name(@san_resp_path2){
      if ($san_resp_name =~ /san_tresp/){
        if ($n == 0){
          my $s_item6 = "s_ldom_san_resp";
          print "<div id=\"tabs-$tabs\">\n";
          $tabs++;
          $n++;
          print "<table border=\"0\">\n";
          print "<tr>";
          print_item ($host_url,$server_url,$lpar_url,$s_item6,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
          print_item ($host_url,$server_url,$lpar_url,$s_item6,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
          print "</tr>\n<tr>\n";
          print_item ($host_url,$server_url,$lpar_url,$s_item6,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
          print_item ($host_url,$server_url,$lpar_url,$s_item6,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
          print "</tr></table>";
          print "</div>\n";
        }
      }
    }
  }

  if (-f $file_fs){
    print "<div id =\"tabs-$tabs\">
    <center>
    <h4>Filesystem usage</h4>
    <tbody>
    <tr>
    <table class =\"tabconfig tablesorter\"data-sortby=\"5\">
    <thead>
    <tr><th class = \"sortable\">Filesystem&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Total [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Used [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Available [GB]&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
    <th class = \"sortable\">Usage [%]&nbsp;&nbsp;</th>
    <th class = \"sortable\">Mounted on&nbsp;&nbsp;</th>
    </tr></thead>\n\n";
    if ( -f "$file_fs" ) {
      open( FH, "< $file_fs" ) || error( "Cannot open $file_fs: $!" . __FILE__ . ":" . __LINE__ );
      my @file_sys_arr = <FH>;
      close(FH);
      foreach my $line (@file_sys_arr)
      {
        chomp ($line);
        (my $filesystem, my $blocks, my $used, my $avaliable, my $usage, my $mounted) = split(" ", $line);
        print "<tr><td>$filesystem</td>
        <td>$blocks</td>
        <td>$used</td>
        <td>$avaliable</td>
        <td>$usage</td>
        <td>$mounted</td></tr>";
      }
    } ## end if ( -f "$file" )
    my $last_update = localtime((stat($file_fs))[9]);
    print "<tfoot><tr><td colspan=\"6\">Last update time: $last_update</td></tr></tfoot>";
    print "</tr>
    </tbody>
    </table>
    </center>
    </div>\n";
    $tab_number++;
    $tabs++;
  } ## end if ( $item =~ m/file_sys/)

        $host_url = $server_url;
        $server_url = "Solaris";
  my @pool_path1;
  if (-d "$wrkdir/Solaris/$ldom_uuid/"){
    opendir( DIR, "$wrkdir/Solaris/$ldom_uuid/" ) || error( "can't opendir $wrkdir/Solaris/$ldom_uuid/: $! :" . __FILE__ . ":" . __LINE__ );
    @pool_path1 = grep /pool/, readdir(DIR);
    foreach my $pool_name(@pool_path1){
      if ($pool_name =~ /pool/){
        my $s_item13 = "solaris_pool";
        print "<div id=\"tabs-$tabs\">\n";
        $pool_name =~ s/\.mmm//g;
        $tabs++;
        #$host_url = $server_url;
        #$server_url = "Solaris";
        $lpar_url = $pool_name;
        print "<table border=\"0\">\n";
        print "<tr>";
        print_item ($host_url,$server_url,$lpar_url,$s_item13,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
        print_item ($host_url,$server_url,$lpar_url,$s_item13,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
        print "</tr>\n<tr>\n";
        print_item ($host_url,$server_url,$lpar_url,$s_item13,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"legend");
        print_item ($host_url,$server_url,$lpar_url,$s_item13,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"legend");
        print "</tr></table>";
        print "</div>\n";
      }
    }
  }

  print "</div><br>\n";
  return 1;
}

sub print_ldom_agg {

  my ($host_url,$server_url,$lpar_url,$item,$type_sam) = @_;
  my $tabs = 1;

  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">CPU</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">MEM</a></li>\n";
  print "</ul>\n";

  my $s_item1 = "s_ldom_agg_c";
  $tabs = 1;
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$s_item1,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print "</tr></table>";
  print "</div><br>\n";

  my $s_item2 = "s_ldom_agg_m";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"d",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"w",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print "</tr>\n<tr>\n";
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"m",$type_sam,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print_item ($host_url,$server_url,$lpar_url,$s_item2,"y",$type_sam_year,$entitle,$detail_yes,"norefr","star",1,"nolegend");
  print "</tr></table>";
  print "</div><br>\n";

  print "</div><br>\n";
  return 1;
}

sub print_hyperv_disk_total {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam ) = @_;
  my $tabs = 1;
  my $my_tab = "nothing";
  $my_tab = "DATA" if ($item =~ "hdt_data");
  $my_tab = "IO" if ($item =~ "hdt_io");
  # print STDERR "2665 $host_url, $server_url, $lpar_url, $item, $type_sam\n";
  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">$my_tab</a></li>\n";
  print "</ul>\n";

  # $item="pool";
  print "<div id=\"tabs-$tabs\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  print "</div><br>\n";
  return 1;

} ## end sub print_hyperv_disk_total

sub print_hyperv_disk_lfd {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam ) = @_;
  my $tabs = 1;
  # print STDERR "2696 $host_url, $server_url, $lpar_url, $item, $type_sam\n";
  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">Capacity</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">Data</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">IO</a></li>\n";
  $tabs++;
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">Latency</a></li>\n";
  print "</ul>\n";

  $tabs = 1;
  my $item_l = "$item"."_cat_$lpar";
  print "<div id=\"tabs-$tabs\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  $item_l = "$item"."_dat_$lpar";
  $tabs++;
  print "<div id=\"tabs-$tabs\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  $item_l = "$item"."_io_$lpar";
  $tabs++;
  print "<div id=\"tabs-$tabs\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  $item_l = "$item"."_lat_$lpar";
  $tabs++;
  print "<div id=\"tabs-$tabs\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item_l, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item_l, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  print "</div><br>\n";
  return 1;

} ## end sub print_hyperv_disk_lfd

sub print_vmw_disk {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam ) = @_;
  my $tabs = 1;

  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  my $tab_head = $item;
  if ($item =~ /^vmnet/) {
    $tab_head = "LAN";
  }
  print "  <li class=\"\"><a href=\"#tabs-$tabs\">$tab_head</a></li>\n";
  $tabs++;
  if ( $hyperv && $item =~ "vmdisk" ) {
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">disk IO</a></li>\n";
    $tabs++;
    # print STDERR "1963 detail-cgi.pl $host_url, $server_url, $lpar_url\n";
    foreach my $local_disk (<$wrkdir/$server/$host/Local_Fixed_Disk_*>) {
      # print STDERR "1965 $local_disk\n";
      (undef, $local_disk) = split ("Local_Fixed_",$local_disk);
      $local_disk =~ s/\.rrm$//;
      print "  <li class=\"\"><a href=\"#tabs-$tabs\">$local_disk</a></li>\n";
      $tabs++;
    }
  }
  if ( $vmware && $item =~ "vmdisk" ) {
    print "  <li class=\"\"><a href=\"#tabs-$tabs\">space</a></li>\n";
  }
  print "</ul>\n";

  $tabs = 1;

  # $item="pool";
  print "<div id=\"tabs-$tabs\">\n";
  $tabs++;
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr></table>";
  print "</div>\n";

  if ( $hyperv && $item =~ "vmdisk" ) {
    my $item = "hdisk_io";
    print "<div id=\"tabs-$tabs\">\n";
    $tabs++;
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>";
    print "</div>\n";

    foreach my $local_disk (<$wrkdir/$server/$host/Local_Fixed_Disk_*>) {
      (undef, $local_disk) = split ("Local_Fixed_",$local_disk);
      $local_disk =~ s/\.rrm$//;
      my $item = "hdisk_$local_disk";
      print "<div id=\"tabs-$tabs\">\n";
      $tabs++;
      print "<table border=\"0\">\n";
      print "<tr>";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      print "</tr></table>";
      print "</div>\n";
    }
  } ## end if ( $hyperv && $item ...)

  if ( $vmware && $item =~ "vmdisk" ) {
    my $file_html_disk = "$wrkdir/$server/$host/disk.html";
    $file_html_disk =~ s/%3A/:/g;    # host can have port vmware:444"
    my $disk_html;
    if ( -f "$file_html_disk" ) {
      open( FH, "< $file_html_disk" );
      $disk_html = do { local $/; <FH> };
      close(FH);
      print "<div id=\"tabs-$tabs\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>\n";
      print "$disk_html";
      print "</td></tr>\n";
      print "<tr><td>\n";
      print "The table is updated every run<br>";
      print "</td></tr>\n";
      print "</table>\n";
      print "</div><br>\n";
    } ## end if ( -f "$file_html_disk")
  } ## end if ( $vmware && $item ...)

  print "</div><br>\n";
  return 1;

} ## end sub print_vmw_disk

sub print_pool {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam ) = @_;

  my $cpu_pool = "CPU POOL";
  my $lpar_vm_agg = "LPARs aggregated";
  if ($vmware || $hyperv) {
    $lpar_vm_agg = "VMs aggregated";
    $cpu_pool = "CPU";
  }
  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">$cpu_pool</a></li>\n";

  if ( $hitachi ) {    # specific tabs for hitachi
    print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">System</a></li>\n";
    print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">$lpar_vm_agg</a></li>\n";
  }
  else {
    if ( !$vmware && !$hyperv ) {    # do not use on our DEMO site or for vmware or for hyperv
      print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">CPU POOL max</a></li>\n";
    }
    if ($hyperv) {
      if (-f "$wrkdir/$server/$host/VM_hosting.vmh") {
        print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">$lpar_vm_agg</a></li>\n";
      }
      print "  <li class=\"$tab_type\"><a href=\"#tabs-4\">Configuration</a></li>\n";
    }
    else {
      print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">$lpar_vm_agg</a></li>\n";
      print "  <li class=\"$tab_type\"><a href=\"#tabs-4\">Configuration</a></li>\n";
    }
  }
  print "</ul>\n";

  # Main server pool
  my $refresh_name = "CPU%20pool";
  my $refresh      = "<div class=\"refresh\"><A HREF=\"/lpar2rrd-cgi/lpar2rrd-realt.sh?source=$refresh_name&hmc=$host_url&mname=$server_url&new_gui=$gui\"></A></div>\n";
  $item = "pool";
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>";
  print "<tr>";
  $item = "trendpool";

  if ( !$vmware && !$hyperv ) {
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  }
  print "</tr></table>";
  print "</div>";

  # Main server pool - MAX
  if ( !$vmware && !$hitachi && !$hyperv ) {    # do not use on our DEMO site
    $item = "pool-max";
    print "<div id=\"tabs-2\">\n";
    print "<div id=\"hiw\"><a href=\"http://www.lpar2rrd.com/max_vrs_average.html\" target=\"_blank\" class=\"nowrap\"><img src=\"css/images/help-browser.gif\" alt=\"Maximum versus Average\" title=\"Maximum versus Average\"></img></a></div>";
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>";
    print "<tr>";
    $item = "trendpool-max";

    if ( !$vmware ) {
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    }
    print "</tr></table>";
    print "</div>";
  } ## end if ( $demo ne "1" && !...)

  if ( $hitachi ) {
    $item = "system";

    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>";
    print "<tr>";
    print "</tr></table>";
    print "</div><br>\n";
  }

  # Main pool - aggregated
  if ($hyperv && !-f "$wrkdir/$server/$host/VM_hosting.vmh") {
    # nothing
  }
  else {
    $item     = "lparagg";
    $lpar_url = "pool-multi";
    print "<div id=\"tabs-3\">\n";
    print "$question_mark";
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>";
    print "</div>";
  }

  if ( !$hitachi ) {
    my $file_html_cpu = "$wrkdir/$server/$host/cpu.html";
    # print STDERR "2627 \$hyperv $hyperv \$file_html_cpu $file_html_cpu\n";
    if ($hyperv) {
      my $file_html_server = "$wrkdir/$server/$host/server.html";
      $file_html_server =~ s/%3A/:/g;    # host can have port vmware:444"
      my $server_html;
      if ( -f "$file_html_server" ) {
        open( FH, "< $file_html_server" );
        $server_html = do { local $/; <FH> };
        close(FH);
        print "<div id=\"tabs-4\">\n";
        print "<table border=\"0\">\n";
        print "<tr><td>\n";
        print "$server_html";
        print "</td></tr>\n";
      }
      $file_html_server = "$wrkdir/$server/$host/NetworkAdapterConfiguration.html";
      # $server_html;
      if ( -f "$file_html_server" ) {
        open( FH, "< $file_html_server" );
        $server_html = do { local $/; <FH> };
        close(FH);
        print "<td>\n";
        print "$server_html";
        print "</td>\n";
      }
    }
    if ( -f "$file_html_cpu" ) {
      open( FH, "< $file_html_cpu" );
      my $cpu_html = do { local $/; <FH> };
      close(FH);
      print "<div id=\"tabs-4\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>\n";
      print "$cpu_html";
      print "</td></tr>\n";
      if ($hyperv) {
        my $file_html_vhd_list = "$wrkdir/$server/$host/vhd_list.html";
        $file_html_vhd_list =~ s/%3A/:/g;    # host can have port vmware:444"
        my $vhd_list_html;
        if ( -f "$file_html_vhd_list" ) {
          open( FH, "< $file_html_vhd_list" );
          $vhd_list_html = do { local $/; <FH> };
          close(FH);
          print "<tr><td>\n";
          print "$vhd_list_html";
          print "</td></tr>\n";
        }
      }
      print "<tr><td>\n";

      if ( !$vmware && !$hyperv) {
        print "The table is updated once a day<br>";
        #print "<A HREF=\"$host_url/$server_url/config.html#CPU_pool\">Detailed configuration</A>";
      }
      elsif ($vmware) {
        print "The table is updated every run<br>";
      }
      elsif ($hyperv) {
       print "Tables are updated every run<br>";
      }
      print "</td></tr>\n";
      print "</table>\n";
      print "</div>";
    } ## end if ( -f "$file_html_cpu")
  } ## end if(! $hitachi)

  print "</div><br>\n";
  return 1;
} ## end sub print_pool

sub print_shpool {
  my ( $host_url, $server_url, $lpar_url, $item, $type_sam ) = @_;

  # `echo "$host_url,$server_url,$lpar_url,$item,$type_sam" >> /tmp/xx333`;

  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"tabhmc\"><a href=\"#tabs-1\">CPU POOL</a></li>\n";

  if ( !$vmware ) {    # do not use on our DEMO site
    print "  <li class=\"tabhmc\"><a href=\"#tabs-2\">CPU POOL max</a></li>\n";
  }

  print "  <li class=\"tabhmc\"><a href=\"#tabs-3\">LPARs aggregated</a></li>\n";
  print "  <li class=\"tabhmc\"><a href=\"#tabs-4\">Configuration</a></li>\n";
  print "</ul>\n";

  # shared pool
  my $pool_id = $lpar_url;
  $pool_id =~ s/SharedPool//;
  my $refresh_name = "CPU%20pool%20$pool_id";
  my $refresh      = "<div class=\"refresh\"><A HREF=\"/lpar2rrd-cgi/lpar2rrd-realt.sh?source=$refresh_name&hmc=$host_url&mname=$server_url&new_gui=$gui\"></A></div>\n";
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, $refresh, "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>";
  print "<tr>";
  $item = "trendshpool";
  # next line should work with $detail_yes, but is bad when adding html_heading
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr></table>";
  print "</div>";

  # shared pool max
  if ( !$vmware ) {    # do not use on our DEMO site
    $item = "shpool-max";
    print "<div id=\"tabs-2\">\n";
    print "<div id=\"hiw\"><a href=\"http://www.lpar2rrd.com/max_vrs_average.html\" target=\"_blank\" class=\"nowrap\"><img src=\"css/images/help-browser.gif\" alt=\"Maximum versus Average\" title=\"Maximum versus Average\"></img></a></div>";
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>";
    print "<tr>";
    $item = "trendshpool-max";
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    print "</tr></table>";
    print "</div>";
  } ## end if ( $demo ne "1" && !...)

  # shared pool - aggregated
  $item = "poolagg";
  print "<div id=\"tabs-3\">\n";

  print "<table border=\"0\">\n";
  print "<tr><td align=\"left\" colspan=\"2\">";
  print "<font size=-1>Note that aggregated graphs here are only informative. <br>\n";
  print "LPAR list in the pool is actual and if it is different than it was in the past then historical graphs might not be accurate. <br>They might not contain proper LPAR list<br><br>\n";
  print "</td></tr>";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>";

  # Configuration
  if ($rest_api == 1){
  my $file_html_cpu = "$wrkdir/$server/$host/cpu_pool_$pool_id.html";
    $file_html_cpu =~ s/%3A/:/g;    # host can have port vmware:444"
    my $cpu_html;

    # print STDERR "\$file_html_cpu $file_html_cpu\n";
    if ( -f "$file_html_cpu" ) {
      open( FH, "< $file_html_cpu" );
      $cpu_html = do { local $/; <FH> };
      close(FH);
      print "<div id=\"tabs-4\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>\n";
      print "$cpu_html";
      print "</td></tr>\n";
      print "<tr><td>\n";
    }
  }
  else{
    my $file_html_cpu = "$wrkdir/$server/$host/cpu.html";
    if ( -f "$file_html_cpu" ) {
      open( FH, "< $file_html_cpu" );
      my @cpu_html = <FH>;
      close(FH);

      my $lpar_pool_alias = "";
      my $pool_id         = $lpar_url;
      $pool_id =~ s/SharedPool//g;
      open( FR, "<$wrkdir/$server/$host/cpu-pools-mapping.txt" ) || error( "Can't open $wrkdir/$server/$host/cpu-pools-mapping.txt : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      foreach my $linep (<FR>) {
        chomp($linep);
        ( my $id, my $pool_name ) = split( /,/, $linep );
        if ( $id == $pool_id ) {
          $lpar_pool_alias = "$pool_name";
          last;
        }
      } ## end foreach my $linep (<FR>)
      close(FR);

      print "<div id=\"tabs-4\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>";
      my @cpu_html_choice = grep /<TABLE|TABLE>|thead>|>$lpar_pool_alias</, @cpu_html;
      print "@cpu_html_choice";
      print "</td></tr>";
      print "<tr><td>";
      print "The table is updated once a day<br>";
      #print "<A HREF=\"$host_url/$server_url/config.html#CPU_pools\">Detailed configuration</A>";
      print "</td></tr>";
      print "</table>";
      print "</div>";
    } ## end if ( -f "$file_html_cpu")
  }

  print "</div><br>\n";
  return 1;
} ## end sub print_shpool

sub print_memory {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print STDERR "2255 detail-cgi.pl memalloc $host_url,$server_url,$lpar_url,$item,$type_sam \$vmware $vmware \$hyperv $hyperv \n" ;
  # test if AMS
  opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server/$host" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files_not_sorted = grep( /.*\.rm.$/, readdir(DIR) );
  closedir(DIR);
  my $ams = @files_not_sorted;

  print "<CENTER>";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">Allocation</a></li>\n";
  if ( !$hyperv && !$hitachi ) {
    print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">Aggregated</a></li>\n";
  }
  if ( $hyperv ) {
    print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">Paging</a></li>\n";
  }
  if ( !$vmware && !$hyperv && !$hitachi) {
    if ( $ams == 0 ) {
      print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">Configuration</a></li>\n";
    }
    else {
      print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">AMS</a></li>\n";
      print "  <li class=\"$tab_type\"><a href=\"#tabs-4\">Configuration</a></li>\n";
    }
  } ## end if ( !$vmware && !$hyperv)
  print "</ul>\n";

  # Memory allocation
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "memalloc";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>";
  print "<tr>";
  $item = "trendmemalloc";

  if ( !$vmware && !$hyperv ) {
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  }
  print "</tr></table>";
  print "</div>\n";

  if ( !$hyperv && !$hitachi ) {

    # Memory aggregation
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "memaggreg";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>";
    print "</div>\n";
  }

  if ( $hyperv ) {
    # paging
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";
    $item = "hyppg1";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr></table>";
    print "</div>\n";
  }

  my $tab_count = 3;
  if ( $ams ne 0 ) {
    $tab_count = 4;

    # Memory AMS
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";
    $item = "memams";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>";
    print "</div>\n";
  } ## end if ( $ams ne 0 )

  # Memory configuration
  my $file_html_mem = "$wrkdir/$server/$host/mem.html";
  my $mem_html;
  if ( -f "$file_html_mem" ) {
    open( FH, "< $file_html_mem" );
    $mem_html = do { local $/; <FH> };
    close(FH);
    print "<div id=\"tabs-$tab_count\">\n";
    print "<table border=\"0\">\n";
    print "<tr><td align=\"center\">\n";
    print "$mem_html";
    print "</td></tr>";
    print "<tr><td>\n";
    print "The table is updated once a day<br>";
    #print "<A HREF=\"$host_url/$server_url/config.html#Memory\">Detailed configuration</A>";
    print "</td></tr>";
    print "</table>\n";
    print "</div><br>\n";
  } ## end if ( -f "$file_html_mem")

  print "</div><br>\n";
  return 1;
} ## end sub print_memory

sub basename {
  my $full = shift;
  my $out  = "";

  # basename without direct function
  my @base = split( /\//, $full );
  foreach my $m (@base) {
    $out = $m;
  }
  return $out;
} ## end sub basename

sub print_hmctotals {
  my ( $host_url, $server, $lpar_url, $item, $entitle ) = @_;

  my $lpar_tab_name = "LPAR";
  if ($vmware) {
    $lpar_tab_name = "CPU VMs";
  }
  my $hmc_pref = "--HMC--";
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">SERVER</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">$lpar_tab_name</a></li>\n";
  if ( !$vmware ) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-3\">COUNT</a></li>\n";
  }

  # find if there are more clusters
  my $file_cluster = "$wrkdir/$server/*/cluster.rrc";

  #$file_cluster =~ s/ /\\ /g;
  my $no_name       = "";
  my @cluster_arr   = <$file_cluster$no_name>;
  my $cluster_count = scalar @cluster_arr;

  # print STDERR "\$cluster_count $cluster_count \@cluster_arr @cluster_arr \$host $host \$server $server \$lpar_url $lpar_url\n";
  if ( $cluster_count > 1 ) {
    print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">CPU CLUSTERS</a></li>\n";
  }

  if ( -f "$wrkdir/$hmc_pref$host_url/cpu.mmx" ) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-4\">CPU</a></li>\n";
  }
  if ( -f "$wrkdir/$hmc_pref$host_url/mem.mmx" ) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-5\">MEMORY</a></li>\n";
  }
  if ( -f "$wrkdir/$hmc_pref$host_url/pgs.mmx" ) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-6\">PAGING</a></li>\n";
  }
  print "</ul>\n";

  print "<CENTER>";

  #hmctotals SERVER
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "multihmc";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";

  if ( !$vmware ) {
    print "Graphs do not take into consideration CPU dedicated partitions which have pre-allocated CPUs.<br>";
  }
  print "</td></tr></table>";

  print "</div>\n";

  #hmctotals LPAR
  print "<div id=\"tabs-2\">\n";
  if ($vmware) {
    print "$question_mark";
  }
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "multihmclpar";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";

  if ( !$vmware ) {
    print "Note that CPU utilization of CPU dedicated partitions is included in the graphs so result can be different from the server graphs in the first tab.<BR>";
  }
  print "</td></tr></table>";

  print "</div>\n";

  # for more vmware clusters
  if ( $cluster_count > 1 ) {
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "multicluster";
    print_item( $host_url, $server_url, "$lpar_url", "$item", "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url,   $item,   "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
    print "</tr>";
    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";
  } ## end if ( $cluster_count > ...)

  #hmctotals COUNT
  if ( !$vmware ) {
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hmccount";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr></table>";

    print "</div>\n";
  } ## end if ( !$vmware )

  if ( -f "$wrkdir/$hmc_pref$host_url/cpu.mmx" ) {
    print "<div id=\"tabs-4\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hmccpu";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr></table>";

    print "</div>\n";
  } ## end if ( -f "$wrkdir/$hmc_pref$host_url/cpu.mmx")
  if ( -f "$wrkdir/$hmc_pref$host_url/mem.mmx" ) {
    print "<div id=\"tabs-5\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hmcmem";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr></table>";

    print "</div>\n";
  } ## end if ( -f "$wrkdir/$hmc_pref$host_url/mem.mmx")
  if ( -f "$wrkdir/$hmc_pref$host_url/pgs.mmx" ) {
    print "<div id=\"tabs-6\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hmcpgs";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr></table>";

    print "</div>\n";
  } ## end if ( -f "$wrkdir/$hmc_pref$host_url/pgs.mmx")
  print "</div><br>\n";

  return 1;
} ## end sub print_hmctotals

sub print_datastore {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print STDERR "2385 detail-cgi.pl datastore $host_url,$server_url,$lpar_url,$item,$entitle,\n";

  my $full_ds_name = "$wrkdir/$server/$host/$lpar";
  if ( ( !-f "$full_ds_name.rrt" || ( -s "$full_ds_name.rrt" == 0 ) ) && ( !-f "$full_ds_name.rrs" || ( -s "$full_ds_name.rrs" == 0 ) ) ) {
    error( "datastore $wrkdir/$server/$host/$lpar has no data " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # soliter ESXi has no data rw or avr
  my $data_exist = 0;
  if ( -f "$wrkdir/$server/$host/$lpar.rrt" ) {
    $data_exist = -s "$wrkdir/$server/$host/$lpar.rrt";
  }
  my $is_iopsr = test_metric_in_rrd( $host, $server, $lpar, "datastore", "Datastore_ReadAvg" );

  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-1\">SPACE</a></li>\n";
  if ($data_exist) {
    print "  <li class=\"\"><a href=\"#tabs-2\">DATA</a></li>\n";
    print "  <li class=\"\"><a href=\"#tabs-3\">IOPS</a></li>\n" if $is_iopsr;
    print "  <li class=\"\"><a href=\"#tabs-4\">IOPS/VM</a></li>\n" if ( -d "$wrkdir/$server/$host/$lpar" );
  }
  if ( -s "$wrkdir/$server/$host/$lpar.rru" ) {
    print "  <li class=\"\"><a href=\"#tabs-5\">Latency</a></li>\n";
  }
  print "  <li class=\"\"><a href=\"#tabs-6\">VM list</a></li>\n";
  print "</ul>\n";

  print "<CENTER>";

  # datastore MEM
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "dsmem";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";
  print "</div>\n";

  if ($data_exist) {

    # datastore Read-Write
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "dsrw";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>";
    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";
    print "</div>\n";

    if ($is_iopsr) {

      # datastore Averaged-R-W
      print "<div id=\"tabs-3\">\n";
      print "<table border=\"0\">\n";
      print "<tr>";

      $item = "dsarw";
      print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
      print "</tr>\n<tr>\n";
      print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
      print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
      print "</tr>";
      print "<tr><td colspan=\"2\" align=\"left\">";
      print "</td></tr></table>";
      print "</div>\n";

      # print "</div><br>\n";

      # datastore IOPS/VM
      if ( -d "$wrkdir/$server/$host/$lpar" ) {
        print "<div id=\"tabs-4\">\n";
        print "<table border=\"0\">\n";
        print "<tr>";

        $item = "ds-vmiops";
        print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
        print "</tr>\n<tr>\n";
        print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
        print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
        print "</tr>";
        print "<tr><td colspan=\"2\" align=\"left\">";
        print "</td></tr></table>";
        print "</div>\n";
      } ## end if ( -d "$wrkdir/$server_url/$host_url/$lpar_url")
    } ## end if ($is_iopsr)
  } ## end if ($data_exist)
  if ( -s "$wrkdir/$server/$host/$lpar.rru" ) {

    # datastore Latency
    print "<div id=\"tabs-5\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "dslat";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>";
    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";
    print "</div>\n";
  } ## end if ( -s "$wrkdir/$server/$host/$lpar.rru")

  print "<div id=\"tabs-6\">\n";
  my $file_html_disk = "$wrkdir/$server/$host/$lpar.html";
  $file_html_disk =~ s/%3A/:/g;    # host can have port vmware:444"
  my $disk_html;

  # print STDERR "detail-cgi.pl 1886 $file_html_disk\n";
  if ( -f "$file_html_disk" ) {
    open( FH, "< $file_html_disk" );
    $disk_html = do { local $/; <FH> };
    close(FH);
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr><td>\n";
    print "$disk_html";
    print "</td></tr>\n";
    print "<tr><td>\n";
    print "The table is updated every run<br>";
    print "</td></tr>\n";
    print "</table>\n";
    print "</div><br>\n";
  } ## end if ( -f "$file_html_disk")
  print "</div>\n";

  print "</div><br>\n";
  return 1;

} ## end sub print_datastore

sub print_resourcepool {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  if ( $lpar_url ne "Resources" ) {
    print "  <li class=\"\"><a href=\"#tabs-2\">CPU</a></li>\n";
  }
  print "  <li class=\"\"><a href=\"#tabs-3\">CPU VMs</a></li>\n";
  if ( $lpar_url ne "Resources" ) {
    print "  <li class=\"\"><a href=\"#tabs-4\">MEMORY</a></li>\n";
  }
  print "</ul>\n";

  print "<CENTER>";

  if ( $lpar_url ne "Resources" ) {

    # resourcepool CPU
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "rpcpu";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>";

    print_item( $host_url, $server_url, $lpar_url, "trendrp", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );

    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";
  } ## end if ( $lpar_url ne "Resources")

  # resourcepool LPAR
  print "<div id=\"tabs-3\">\n";
  print "$question_mark";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "rplpar";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  if ( $lpar_url ne "Resources" ) {

    # resourcepool MEM
    print "<div id=\"tabs-4\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "rpmem";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );

    print "</tr>\n";
    print "<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, "trendrpmem", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    print "</tr>\n";
    print "<tr><td align=\"left\" colspan=\"2\"><br>\n";
    print "</td></tr></table>\n";
    print "</div>\n";
  } ## end if ( $lpar_url ne "Resources")

  print "</div><br>\n";

  return 1;
} ## end sub print_resourcepool

#my $question_mark = "<div id=\"hiw\"><a href=\"http://www.lpar2rrd.com/VMware-GHz-vrs-real_CPU.htm\" target=\"_blank\" class=\"nowrap\"><img src=\"css/images/help-browser.gif\" alt=\"How it works?\" title=\"How it works?\"></img></a></div>";

sub print_cluster {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print STDERR "3581 detail-cgi.pl ($host_url,$server_url,$lpar_url,$item,$entitle)\n";

  if ($hyperv) {
    # print STDERR "3581 cluster hyperv\n";
    print "<div  id=\"tabs\">\n";
    print "<ul>\n";
    print "  <li class=\"\"><a href=\"#tabs-1\">CPU</a></li>\n";
    print "  <li class=\"\"><a href=\"#tabs-2\">CPU VMs</a></li>\n";
    print "  <li class=\"\"><a href=\"#tabs-3\">MEMORY</a></li>\n";
    print "  <li class=\"\"><a href=\"#tabs-4\">NODES</a></li>\n";
    print "  <li class=\"\"><a href=\"#tabs-5\">Config</a></li>\n";
    print "</ul>\n";

    print "<CENTER>";

    #cluster CPU
    print "<div id=\"tabs-1\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hyp_clustsercpu";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>";

    # print_item( $host_url, $server_url, $lpar_url, "trendcluster", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );

    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";

    #clusterCPU VMs
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hyp_clustservms";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>";

    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";

    #cluster MEMORY
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hyp_clustsermem";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>";

    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";

    #cluster SERVER
    print "<div id=\"tabs-4\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "hyp_clustser";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>";

    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";


    print "<div id=\"tabs-5\">\n";
    my $file_html_nodes = "$wrkdir/$server/$host/cluster_list.html";
    my $nodes_html;

    # print STDERR "detail-cgi.pl 3818 $file_html_nodes\n";
    if ( -f "$file_html_nodes" ) {
      open( FH, "< $file_html_nodes" );
      $nodes_html = do { local $/; <FH> };
      close(FH);
      #print "<div id=\"tabs-2\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>\n";
      print "$nodes_html";
      print "</td></tr>\n";
      print "<tr><td>\n";
      my $last_mod_time = ( stat($file_html_nodes) )[9];
      print "The table is updated: ";
      print scalar localtime $last_mod_time;
      print "<br>";
      print "</td></tr>\n";
      print "</table>\n";
    } ## end if ( -f "$file_html_nodes")

    $file_html_nodes = "$wrkdir/$server/$host/node_list.html";
    #$file_html_nodes =~ s/%3A/:/g;    # host can have port vmware:444"

    # print STDERR "detail-cgi.pl 3841 $file_html_nodes\n";
    if ( -f "$file_html_nodes" ) {
      open( FH, "< $file_html_nodes" );
      $nodes_html = do { local $/; <FH> };
      close(FH);
      #print "<div id=\"tabs-2\">\n";
      print "<table border=\"0\">\n";
      print "<tr><td>\n";
      print "$nodes_html";
      print "</td></tr>\n";
      print "<tr><td>\n";
      my $last_mod_time = ( stat($file_html_nodes) )[9];
      print "The table is updated: ";
      print scalar localtime $last_mod_time;
      print "<br>";
      print "</td></tr>\n";
      print "</table>\n";
    } ## end if ( -f "$file_html_nodes")

    print "</div>\n";

    print "</div><br>\n";

    return 1;
  }

  # my $is_power = test_metric_in_rrd( $host, $server, $lpar, $item, "Power_usage_Watt" );
  my $is_power = 0;    # since 4.95-7 do not print power

  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"\"><a href=\"#tabs-2\">CPU</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-3\">CPU VMs</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-4\">MEMORY</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-5\">SERVER</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-6\">CPU READY</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-7\">LAN</a></li>\n";
  print "  <li class=\"\"><a href=\"#tabs-8\">POWER</a></li>\n" if $is_power;
  print "</ul>\n";

  print "<CENTER>";

  #cluster CPU
  print "<div id=\"tabs-2\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustcpu";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>";

  print_item( $host_url, $server_url, $lpar_url, "trendcluster", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );

  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  #hmctotals LPAR
  print "<div id=\"tabs-3\">\n";
  print "$question_mark";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustlpar";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  #cluster MEM
  print "<div id=\"tabs-4\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustmem";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );

  print "</tr>\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, "trendclmem", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr>\n";
  print "<tr><td align=\"left\" colspan=\"2\"><br>\n";
  print "</td></tr></table>\n";
  print "</div>\n";

  #cluster SERVER
  print "<div id=\"tabs-5\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustser";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>";

  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  #cluster RDY
  print "<div id=\"tabs-6\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustlpardy";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  #cluster LAN
  print "<div id=\"tabs-7\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $item = "clustlan";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>";
  print "<tr><td colspan=\"2\" align=\"left\">";
  print "</td></tr></table>";

  print "</div>\n";

  #cluster POWER
  if ($is_power) {
    print "<div id=\"tabs-8\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $item = "clustpow";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "nostar", 1, "legend" );
    print "</tr>";
    print "<tr><td colspan=\"2\" align=\"left\">";
    print "</td></tr></table>";

    print "</div>\n";
    print "<tr>\n";
  } ## end if ($is_power)

  print "</div><br>\n";

  return 1;
} ## end sub print_cluster

# testing: if metric in rrd is during last year or month active = has average value -> returns true, if NaN -> returns false
sub test_metric_in_rrd {
  my ( $server, $host, $lpar, $item, $ds ) = @_;

  my $rrd = "$wrkdir/$server/$host/$lpar";

  if ( $item eq "cluster" ) {
    $rrd = "$wrkdir/$host/$server/cluster.rrc";
  }

  if ( $item eq "datastore" ) {
    $rrd = "$wrkdir/$host/$server/$lpar.rrt";
  }
  if ( !-f $rrd ) {
    error( "Cannot open $rrd: item:$item datastream:$ds " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  if ( -s $rrd == 0 ) {    # no data ?
    return 0;
  }

  my $rrdtool = $ENV{RRDTOOL};

  # start RRD via a pipe
  RRDp::start "$rrdtool";

  $rrd =~ s/:/\\:/g;
  my $answer = "";
  eval {
    RRDp::cmd qq(graph "tmp/name.png"
      "--start" "-1m"
      "--end" "now"
      "DEF:val=$rrd:$ds:AVERAGE"
      "PRINT:val:AVERAGE: %3.3lf"
      );

    $answer = RRDp::read;
  };
  if ($@) {
      error( "Rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
      RRDp::end;
      return 0;
  }
  #my $answer = RRDp::read;

  #  print STDERR "----- testing $rrd - $ds ->res = $answer-$$answer\n";
  if ( $$answer !~ m/NaN/ && $$answer !~ m/nan/ ) {

    # close RRD pipe
    RRDp::end;
    return 1;
  } ## end if ( $$answer !~ m/NaN/...)

  eval {
    RRDp::cmd qq(graph "tmp/name.png"
      "--start" "-1y"
      "--end" "now"
      "DEF:val=$rrd:$ds:AVERAGE"
      "PRINT:val:AVERAGE: %3.3lf"
      );
    $answer = RRDp::read;
  };
  if ($@){
    error( "Rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    RRDp::end;
    return 0;
  }

  # print STDERR "----- testing $rrd - $ds ->res = $answer-$$answer\n";
  if ( $$answer !~ m/NaN/ && $$answer !~ m/nan/ ) {

    # close RRD pipe
    RRDp::end;
    return 1;
  } ## end if ( $$answer !~ m/NaN/...)

  # close RRD pipe
  RRDp::end;
  return 0;
} ## end sub test_metric_in_rrd

# specific to Custom groups, where the platform/host is "Power"
sub print_custom_lpar_details {
  my (@list_rrd) = @_;    #one array parameter can be passed this way
  my $rest_api = is_any_host_rest();
  my $table_head = "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">
<thead><TR> <TH class=\"sortable\" ><B>LPAR name&nbsp;&nbsp;&nbsp;</B></TH> <TH class=\"sortable\" align=\"center\"><B>&nbsp;&nbsp;&nbsp;Mode&nbsp;&nbsp;&nbsp;</B></TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</B></font></TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</B></font></TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</B></font></TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;min VP&nbsp;&nbsp;&nbsp;</TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Virtual&nbsp;&nbsp;&nbsp;</B></font></TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;max VP&nbsp;&nbsp;&nbsp;</TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Sharing mode&nbsp;&nbsp;&nbsp;</B></font></TH> <TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Uncap weight&nbsp;&nbsp;&nbsp;</B></font></TH><TH class=\"sortable\" align=\"center\"><font size=-1><B>&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</B></font></TH><TH class=\"sortable\" align=\"center\"><B>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</B></TH></TR></thead><tbody>";

  my $head_printed = 0;
  my @choice = grep /\.rr[a-z]$/, @list_rrd;
  my $C;
  my $o;
  my $done_conf;
  foreach my $line (@choice) {
    return if index( $line, "\/vmware_VMs\/" ) != -1;    # not for VMWARE
    chomp $line;
    $line =~ s/\.rr[a-z]$//;
    my $lpar = basename($line);
    $line =~ s/(.*)\/.*$/$1/;                            #give me path
    if ($rest_api){
      if (!(defined $done_conf->{$line} || $done_conf->{$line})){
        if (-e "$line/CONFIG.json"){
          $C = read_json("$line/CONFIG.json");
          if (ref($C->{lpar}) eq "HASH"){
            foreach my $key (keys %{$C->{lpar}}){
              $o->{$key} = $C->{lpar}{$key};
            }
          }
        }
      }
      $done_conf->{$line} = "true";
    }
    my $cpu_html = "$line" . "/cpu.html";
    open( FH, " < $cpu_html" ) || error( "Cannot open $cpu_html: $!" . __FILE__ . ":" . __LINE__ ) && next;
    my @cpu = <FH>;
    close FH;
    $lpar =~ s/\&\&1/\//g;
    my @lpar_line = grep /<B>$lpar</, @cpu;
    my $lpl = @lpar_line;

    #`echo "print custom det lpar_line ,$lpar,@lpar_line,$lpl,\n,@choice, " >> /tmp/xpcd`;
    next if $lpl == 0;                                   # nothing to show
    if ( $head_printed == 0 ) {
      print "$table_head";
      $head_printed++;
    }
    if (!$rest_api){
      print @lpar_line;
    }
    else{
      my $mode = "";
      if (defined $o->{$lpar}{SharedProcessorPoolID}){
        $mode = "shared";}
      else{
        if (defined $o->{$lpar}{CurrentSharingMode}){
          $mode = "ded"; }
        }
      my $MinimumProcessingUnits      = "";      $MinimumProcessingUnits = $o->{$lpar}{MinimumProcessingUnits}      if defined($o->{$lpar}{MinimumProcessingUnits});
      my $CurrentProcessingUnits      = "";      $CurrentProcessingUnits = $o->{$lpar}{CurrentProcessingUnits}      if defined($o->{$lpar}{CurrentProcessingUnits});
      my $MaximumProcessingUnits      = "";      $MaximumProcessingUnits = $o->{$lpar}{MaximumProcessingUnits}      if defined($o->{$lpar}{MaximumProcessingUnits});
      my $AllocatedVirtualProcessors  = "";  $AllocatedVirtualProcessors = $o->{$lpar}{AllocatedVirtualProcessors}  if defined($o->{$lpar}{AllocatedVirtualProcessors});
      my $MinimumVirtualProcessors    = "";    $MinimumVirtualProcessors = $o->{$lpar}{MinimumVirtualProcessors}    if defined($o->{$lpar}{MinimumVirtualProcessors});
      my $MaximumVirtualProcessors    = "";    $MaximumVirtualProcessors = $o->{$lpar}{MaximumVirtualProcessors}    if defined($o->{$lpar}{MaximumVirtualProcessors});
      my $CurrentSharingMode          = "";          $CurrentSharingMode = $o->{$lpar}{CurrentSharingMode}          if defined($o->{$lpar}{CurrentSharingMode});
      my $CurrentUncappedWeight       = "";       $CurrentUncappedWeight = $o->{$lpar}{CurrentUncappedWeight}       if defined($o->{$lpar}{CurrentUncappedWeight});
      my $SharedProcessorPoolID       = "";       $SharedProcessorPoolID = $o->{$lpar}{SharedProcessorPoolID}       if defined($o->{$lpar}{SharedProcessorPoolID});
      my $OperatingSystemVersion      = "";      $OperatingSystemVersion = $o->{$lpar}{OperatingSystemVersion}      if defined($o->{$lpar}{OperatingSystemVersion});
      print "<TR>
          <TD><B>$lpar</B></TD>
          <TD align=\"center\">$mode</TD>
          <TD align=\"center\">$MinimumProcessingUnits</TD>
          <TD align=\"center\">$CurrentProcessingUnits</TD>
          <TD align=\"center\">$MaximumProcessingUnits</TD>
          <TD align=\"center\">$MinimumVirtualProcessors</TD>
          <TD align=\"center\">$AllocatedVirtualProcessors</TD>
          <TD align=\"center\">$MaximumVirtualProcessors</TD>
          <TD align=\"center\">$CurrentSharingMode</TD>
          <TD align=\"center\">$CurrentUncappedWeight</TD>
          <TD align=\"center\">$SharedProcessorPoolID</TD>
          <TD align=\"center\" nowrap>$OperatingSystemVersion</TD>
        </TR>\n";
    }
  } ## end foreach my $line (@choice)
  if ( $head_printed != 0 ) {
    print "</tbody></TABLE></CENTER><BR><BR>";
  }
} ## end sub print_custom_lpar_details

# Custom groups for XenServer (XENVM)
sub print_custom_xenserver {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print page contents
  print "<CENTER>";
  print "<div id=\"tabs\">\n";

  # get tabs
  #my @tabs = @{ XenServerMenu::get_tabs( 'vm' ) };
  my @tabs = (
              { cpu_cores => "CPU" },
              { cpu_percent => "CPU %" },
              { memory_used => "MEM used" },
              { memory_free => "MEM free" },
              { storage => "Data" },
              { iops => "IOPS" },
              { latency => "Latency" },
              { net => "Net" }
             );

  if (@tabs) {
    print "<ul>\n";

    my $tab_counter = 1;
    foreach my $tab_header (@tabs) {
      while (my ($tab_type, $tab_label) = each %$tab_header) {
        print "  <li><a href=\"#tabs-$tab_counter\">$tab_label</a></li>\n";
        $tab_counter++;
      }
    }

    print "</ul>\n";
  }

  sub print_tab_contents_custom {
    my ($tab_number, $host_url, $server_url, $lpar_url, $item, $entitle, $detail_yes) = @_;

    my $lpar_url_dec = $lpar_url;
    $lpar_url_dec =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    my $notice_file = "$tmpdir/.custom-group-$lpar_url_dec-n.cmd";
    my @final_notice = "";
    if ( -f $notice_file ) {
      open( FH, "$notice_file" ) || error( "Cannot open $notice_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @final_notice = <FH>;
      close(FH);
    }

    print "<div id=\"tabs-$tab_number\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "<tr><td align=\"center\" colspan=\"2\">@final_notice </td></tr>\n";
    # TODO year trend
    #print "</tr><tr>\n";
    #print_item( $host_url, $server_url, $lpar_url, "custom_cpu_trend", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    print "</tr></table>\n";
    print "</div>\n";
  }

  my @xen_items = ("custom-xenvm-cpu-cores", "custom-xenvm-cpu-percent", "custom-xenvm-memory-used", "custom-xenvm-memory-free", "custom-xenvm-vbd", "custom-xenvm-vbd-iops", "custom-xenvm-vbd-latency", "custom-xenvm-lan");
  for $tab_number (1 .. $#xen_items + 1) {
    print_tab_contents_custom( $tab_number, $host_url, $server_url, $lpar_url, $xen_items[$tab_number - 1], $entitle, $detail_yes );
  }

  # end page contents
  print "</div><br>\n";
  print "</CENTER>\n";
}

# Custom groups for oVirt (OVIRTVM)
sub print_custom_ovirt {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print page contents
  print "<CENTER>";
  print "<div id=\"tabs\">\n";

  # get tabs
  my @tabs = (
              { cpu_core => "CPU cores" },
              { cpu_percent => "CPU %" },
              { memory_used => "MEM used" },
              { memory_free => "MEM free" },
              # { data => "Data" },
              # { latency => "Latency" },
              # { net => "Net" }
             );

  if (@tabs) {
    print "<ul>\n";

    my $tab_counter = 1;
    foreach my $tab_header (@tabs) {
      while (my ($tab_type, $tab_label) = each %$tab_header) {
        print "  <li><a href=\"#tabs-$tab_counter\">$tab_label</a></li>\n";
        $tab_counter++;
      }
    }

    print "</ul>\n";
  }

  sub print_tab_content_custom {
    my ($tab_number, $host_url, $server_url, $lpar_url, $item, $entitle, $detail_yes) = @_;

    my $lpar_url_dec = $lpar_url;
    $lpar_url_dec =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    my $notice_file = "$tmpdir/.custom-group-$lpar_url_dec-n.cmd";
    my @final_notice = "";
    if ( -f $notice_file ) {
      open( FH, "$notice_file" ) || error( "Cannot open $notice_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @final_notice = <FH>;
      close(FH);
    }

    print "<div id=\"tabs-$tab_number\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "<tr><td align=\"center\" colspan=\"2\">@final_notice </td></tr>\n";
    # TODO year trend
    #print "</tr><tr>\n";
    #print_item( $host_url, $server_url, $lpar_url, "custom_cpu_trend", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    print "</tr></table>\n";
    print "</div>\n";
  }

    my @ovirt_items = ( "custom_ovirt_vm_cpu_core", "custom_ovirt_vm_cpu_percent",
                        "custom_ovirt_vm_memory_used", "custom_ovirt_vm_memory_free" );
    # my @ovirt_items = ( "custom-ovirtvm-cpu-percent", "custom-ovirtvm-cpu-core", "custom-ovirtvm-memory-used",
    #                   "custom-ovirtvm-memory-free", "custom-ovirtvm-lan", "custom-ovirtvm-latency",
    #                   "custom-ovirtvm-data" );
  for $tab_number (1 .. $#ovirt_items + 1) {
    print_tab_content_custom( $tab_number, $host_url, $server_url, $lpar_url, $ovirt_items[$tab_number - 1], $entitle, $detail_yes );
  }

  # end page contents
  print "</div><br>\n";
  print "</CENTER>\n";
}

# Custom groups for Solaris Zone (SOLARISZONE)
sub print_custom_solaris {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  # print page contents
  print "<CENTER>";
  print "<div id=\"tabs\">\n";

  # get tabs
  my @tabs = (
              { cpu_used => "CPU cores" },
              { phy_mem_us => "MEM used" },
             );

  if (@tabs) {
    print "<ul>\n";

    my $tab_counter = 1;
    foreach my $tab_header (@tabs) {
      while (my ($tab_type, $tab_label) = each %$tab_header) {
        print "  <li><a href=\"#tabs-$tab_counter\">$tab_label</a></li>\n";
        $tab_counter++;
      }
    }

    print "</ul>\n";
  }

  sub print_tab_content_custom_sol {
    my ($tab_number, $host_url, $server_url, $lpar_url, $item, $entitle, $detail_yes) = @_;

    my $lpar_url_dec = $lpar_url;
    $lpar_url_dec =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
    my $notice_file = "$tmpdir/.custom-group-$lpar_url_dec-n.cmd";
    my @final_notice = "";
    if ( -f $notice_file ) {
      open( FH, "$notice_file" ) || error( "Cannot open $notice_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @final_notice = <FH>;
      close(FH);
    }

    print "<div id=\"tabs-$tab_number\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "<tr><td align=\"center\" colspan=\"2\">@final_notice </td></tr>\n";
    # TODO year trend
    #print "</tr><tr>\n";
    #print_item( $host_url, $server_url, $lpar_url, "custom_cpu_trend", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
    print "</tr></table>\n";
    print "</div>\n";
  }

  my @solaris_items = ( "custom_solaris_cpu", "custom_solaris_mem");
  for $tab_number (1 .. $#solaris_items + 1) {
    print_tab_content_custom_sol( $tab_number, $host_url, $server_url, $lpar_url, $solaris_items[$tab_number - 1], $entitle, $detail_yes );
  }

  # end page contents
  print "</div><br>\n";
  print "</CENTER>\n";
}

# Custom groups for Power and VMware
sub print_custom {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  $lpar_url =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/seg;
  my $list        = "$webdir/custom/$lpar_url/list.txt";
  my $tmp_cmd     = "$tmpdir/custom-group-mem-$lpar_url-d.cmd";
  my $notice_file = "$tmpdir/.custom-group-$lpar_url-n.cmd";

  open( FH, " < $list" ) || error( "Cannot open $list: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @list_rrd = <FH>;
  close FH;
  my @final_notice = "";
  if ( -f $notice_file ) {
    open( FH, "$notice_file" ) || error( "Cannot open $notice_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    @final_notice = <FH>;
    close(FH);
  }
  my @choice;

  print "<div  id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"tabhmc\"><a href=\"#tabs-1\">CPU</a></li>\n";
  @choice = grep /\.rsm$/, @list_rrd;
  if ( ( scalar @choice > 0 ) || ( -f $tmp_cmd ) ) {

    #  if (scalar (grep /\.rsm$/, @list_rrd) >0) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-2\">MEM allocated</a></li>\n";
  }
  @choice = grep /\/mem\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {
    print "  <li class=\"tabagent\"><a href=\"#tabs-3\">MEM</a></li>\n";
  }
  @choice = grep /\/lan-.*\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {
    print "  <li class=\"tabagent\"><a href=\"#tabs-4\">LAN</a></li>\n";
  }
  @choice = grep /\/san-.*\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {
    print "  <li class=\"tabagent\"><a href=\"#tabs-5\">SAN</a></li>\n";
    @choice = grep /\/san-host.*\.mmm$/, @list_rrd;
    if ( scalar @choice > 0 ) {

      # Linux agent reports frames instead of IOPS
      print "  <li class=\"tabagent\"><a href=\"#tabs-6\">Frames</a></li>\n";
    }
    else {
      print "  <li class=\"tabagent\"><a href=\"#tabs-6\">IOPS</a></li>\n";
    }
  } ## end if ( scalar @choice > ...)

  # VMWare custom group
  # CPU is above
  if ( index( $list_rrd[0], "vmware_VMs" ) > -1 ) {
    print "  <li class=\"tabhmc\"><a href=\"#tabs-2\">MEM Active</a></li>\n";

    print "  <li class=\"tabhmc\"><a href=\"#tabs-3\">MEM Granted</a></li>\n";

    print "  <li class=\"tabhmc\"><a href=\"#tabs-4\">DISK</a></li>\n";

    print "  <li class=\"tabhmc\"><a href=\"#tabs-5\">LAN</a></li>\n";
  } ## end if ( index( $list_rrd[...]))
  print "</ul>\n";

  #custom groups CPU
  print "<CENTER>";
  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  $type_sam = "na";
  $item     = "custom";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n";
  print "<tr><td align=\"center\" colspan=\"2\">@final_notice </td></tr>\n";
  print "<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, "custom_cpu_trend", "y", $type_sam_year, $entitle, $detail_no, "norefr", "nostar", 2, "legend" );
  print "</tr></table>\n";
  print_custom_lpar_details(@list_rrd);

  print "</div>\n";

  @choice = grep /\.rsm$/, @list_rrd;
  if ( ( scalar @choice > 0 ) || ( -f $tmp_cmd ) ) {

    #custom groups MEM
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "custommem";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";
  } ## end if ( ( scalar @choice ...))

  @choice = grep /\/mem\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {

    #custom groups OS MEM
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customosmem";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";
  } ## end if ( scalar @choice > ...)

  @choice = grep /\/lan-.*\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {

    #custom groups OS LAN
    print "<div id=\"tabs-4\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customoslan";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";
  } ## end if ( scalar @choice > ...)

  @choice = grep /\/san-.*\.mmm$/, @list_rrd;
  if ( scalar @choice > 0 ) {

    #custom groups OS SAN1
    print "<div id=\"tabs-5\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customossan1";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";

    #custom groups OS SAN2 IOPS
    print "<div id=\"tabs-6\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customossan2";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";
  } ## end if ( scalar @choice > ...)

  # VMWare custom group
  # CPU is above
  if ( index( $list_rrd[0], "vmware_VMs" ) > -1 ) {

    #custom groups VMWare MEM Active
    print "<div id=\"tabs-2\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customvmmemactive";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";

    #custom groups VMWare MEM Consumed
    print "<div id=\"tabs-3\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customvmmemconsumed";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";

    #custom groups VMWare DISK
    print "<div id=\"tabs-4\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customdisk";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";

    #custom groups VMWare LAN
    print "<div id=\"tabs-5\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";

    $type_sam = "na";
    $item     = "customnet";
    print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr></table>@final_notice";
    print_custom_lpar_details(@list_rrd);
    print "</div>\n";
  } ## end if ( index( $list_rrd[...]))

  print "</div><br>\n";

  return 1;
} ## end sub print_custom

sub print_view {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  print "<div  id=\"tabs\">\n";

  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">Daily</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">Weekly</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">Monthly</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-4\">Yearly</a></li>\n";
  print "</ul>\n";
  print "<CENTER>";

  my $inx = 0;
  foreach my $period ( 'd', 'w', 'm', 'y' ) {
    my $pict_count = 0;
    $inx++;
    print "<div id=\"tabs-$inx\">\n";
    print "<table border=\"0\">\n";
    print "<tr>";
    print_item( $host_url, $server_url, "pool",    "pool",     $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print_item( $host_url, $server_url, $lpar_url, "memalloc", $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
    print "</tr>\n<tr>\n";
    print_item( $host_url, $server_url, "pool-multi", "lparagg",   $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print_item( $host_url, $server_url, $lpar_url,    "memaggreg", $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
    print "</tr>\n<tr>\n";

    #  test if AMS
    opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server/$host" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_not_sorted = grep( /.*\.rm.$/, readdir(DIR) );
    closedir(DIR);
    my $ams = @files_not_sorted;

    if ( $ams > 0 ) {
      print_item( $host_url, $server_url, $lpar_url, "memams", $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
      $pict_count++;
    }

    # how many SharedPools
    opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server_url/$host_url" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @share_pools_not_sorted = grep( /SharedPool\d{1,2}\.rr[m,h]/, readdir(DIR) );
    closedir(DIR);
    my @share_pools = sort @share_pools_not_sorted;
    foreach my $sh_pool (@share_pools) {
      $sh_pool =~ s/\.rrm$//;
      $sh_pool =~ s/\.rrh$//;
      print_item( $host_url, $server_url, $sh_pool, "shpool", $period, $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
      $pict_count++;
      print "</tr>\n<tr>\n" if ( !( $pict_count % 2 ) );
    } ## end foreach my $sh_pool (@share_pools)

    # how many lpars
    # 1st try to use cpu.csv, if not then use cpu.html

    my $cpu_html = "$wrkdir/$server/$host/cpu.csv";
    my $lpars_ok = 0;
    my @cpu_array;
    if ( -f $cpu_html && $vmware) {
      if ( open( FC, "< $cpu_html" )) {
        @cpu_array = <FC>;
        close(FC);
        my $item_view = "lpar";
        foreach my $lpar_line (@cpu_array) {
          my ( undef,$v_cpu, $reser_mhz, undef, $shares, $shares_value, $os, $power_state, $tools_status, $vm_uuid, $memorySizeMB ) = split /,/,$lpar_line;
          my $star = "star";    # power lpars in view are dashboard-able
          my $lpar_name = "";
          # print STDERR "5115 \$lpar_line $lpar_line\n";
          if ($vmware) {
            if ( $lpar_line !~ "poweredOn" ) {next};
            $lpar_name = $vm_uuid;
            $star = "nostar";    # vmware VMs in view are NOT dashboard-able
          }
          # here may come later hyperv

          $lpar_name =~ s/\//\&\&1/g;    # replace for "/"
          my $lpar_url = $lpar_name;

          $lpar_url =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
          print_item( $host_url, $server_url, "$lpar_url", $item_view, $period, $type_sam, $entitle, $detail_yes, "norefr", "$star", 1, "legend" );
          $pict_count++;
          print "</tr>\n<tr>\n" if ( !( $pict_count % 2 ) );
          $lpars_ok = 1;
        } ## end foreach my $lpar_line (@cpu_array)
      }
      else {
        error( "Cannot open $cpu_html: $!" . __FILE__ . ":" . __LINE__ );
      }
      # $cmd .= " COMMENT:\"   Memory Count     $mem_count_gb\"";
    }
    if (!$lpars_ok) { # probably not cpu.csv
      $cpu_html = "$wrkdir/$server/$host/cpu.html";

      # print STDERR "3135 detail-cgi.pl ,$server_url,$lpar_url,$item,$type_sam, link $wrkdir/$server/$host/cpu.html\n";
      open( FH, " < $cpu_html" ) || error( "Cannot open $cpu_html: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      my @cpu_html = <FH>;
      close FH;
      my $item_view = "lpar";
      my @lpar_lines = grep {/<B>.+<\/B>/} @cpu_html;
      foreach my $lpar_line (@lpar_lines) {
        ( undef, $lpar_line ) = split( "<B>", $lpar_line );
        ( my $lpar_name, undef ) = split( "</B>", $lpar_line );
        my $star = "star";    # power lpars in view are dashboard-able
        if ($vmware) {
          if ( $lpar_line !~ "poweredOn" ) {next}
          $lpar_name = human_vmware_name( $lpar_name, "neg" );
          $star = "nostar";    # vmware VMs in view are NOT dashboard-able
        }
        if ($hyperv) {
          if ( $lpar_line !~ ">ON<" ) {next}
          $lpar_name = human_vmware_name( $lpar_name, "neg" );
          $star = "nostar";    # hyperv VMs in view are NOT dashboard-able
          $item_view = "hyp-cpu";
        }
        $lpar_name =~ s/\//\&\&1/g;    # replace for "/"
        my $lpar_url = $lpar_name;

        #$lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
        $lpar_url =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

        #$lpar_url =~ s/ /+/g;
        #$lpar_url =~ s/\#/%23/g;
        print_item( $host_url, $server_url, "$lpar_url", $item_view, $period, $type_sam, $entitle, $detail_yes, "norefr", "$star", 1, "legend" );
        $pict_count++;
        print "</tr>\n<tr>\n" if ( !( $pict_count % 2 ) );
      } ## end foreach my $lpar_line (@lpar_lines)
    }
    print "</tr></table>";
    print "</div>\n";
  } ## end foreach my $period ( 'd', 'w'...)
  print "</div><br>\n";

  return 1;
} ## end sub print_view

sub human_vmware_name {
  my $lpar  = shift;
  my $arrow = shift;
  if ( !$vmware && !$hyperv ) { return "$lpar" };
       # only for vmware
       # read file and find human lpar name from uuid or
       # if 'neg' then find uuid from name
       # my $trans_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my $vms_dir = "vmware_VMs";
  $vms_dir = "hyperv_VMs" if $hyperv;
  if ( -f "$wrkdir/$vms_dir/vm_uuid_name.txt" ) {
    open( FR, "< $wrkdir/$vms_dir/vm_uuid_name.txt" );
    foreach my $linep (<FR>) {
      chomp($linep);
      ( my $id, my $name ) = split( /,/, $linep );
      if ( defined($arrow) && "$arrow" eq "neg" ) {
        ( $name, $id ) = split( /,/, $linep );
      }
      if ( "$id" eq "$lpar" ) {
        $lpar = "$name";
        # last; # let it run until end, in case there are more lines for one VM then take the last line
      }
    } ## end foreach my $linep (<FR>)
    close(FR);
  } ## end if ( -f "$wrkdir/vmware_VMs/vm_uuid_name.txt")
  return "$lpar";    #human name - if found, or original
} ## end sub human_vmware_name

sub print_topten_head {
  my ( $server_url, $host_act ) = @_;

  my $lpars  = "LPARs";
  my $server = "server";

  if ( $item eq "topten_vm" ) {
    $lpars  = "VMs";
    $server = "vCenter";
  }
  if ( $server_url eq "" ) {

    #global one
    print "<center><h3>Top10 $lpars per average CPU load </h3></center>";
  }
  else {
    if ( $host_act eq "" ) {
      print "<center><h3>Top10 $lpars per average CPU load for $server : $server_url</h3></center>";
    }
    else {
      if ( -f "$wrkdir/$server_url/$host_act/IVM" ) {
        print "<center><h3>Top10 LPARs per average CPU load for server : $host_act</h3></center>";
      }
      else {
        print "<center><h3>Top10 $lpars per average CPU load for $server : $server_url</h3></center>";
      }
    } ## end else [ if ( $host_act eq "" )]
  } ## end else [ if ( $server_url eq "")]
  return 1;
} ## end sub print_topten_head

sub print_topten_net {
  my ( $period, $server_url ) = @_;

  my $vm_uuid_name_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my @vm_uuid_names     = ();

  my $topten_file = "";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  if ( $item eq "topten_vm" ) {
    print "<TR><TD nowrap ><B>Average<BR>MB/sec</B></TD><TD nowrap><B>VM</B></TD><TD nowrap><B>vCenter</B></TD></TR>";
    $topten_file = "$tmpdir/topten_vm.tmp";
    open( FH, " < $vm_uuid_name_file" ) || error( "Cannot open $vm_uuid_name_file: $!" . __FILE__ . ":" . __LINE__ );
    @vm_uuid_names = <FH>;
    close FH;
  } ## end if ( $item eq "topten_vm")

  my $topten_limit = 0;
  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my @topten = "";
  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;
    if ( "$server_url" ne "" ) {
      #local one
      my @topten_server = grep {/vm_net,$server,/} @topten;
      @topten = @topten_server;
    }
    else{
      my @topten_server = grep {/vm_net/} @topten;
      @topten = @topten_server;
    }
    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
      $vm_uuid      = "" if !defined $vm_uuid;
      $vcenter_uuid = "" if !defined $vcenter_uuid;
      $top_values[4] = 0 if !defined $top_values[4] or $top_values[4] eq "";    # no year value yet, new file
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
    } ## end foreach my $line (@topten)
    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }
  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  my @topten_sorted_load_cpu;
  {
    no warnings;
    @topten_sorted_load_cpu = sort {
      my @b = split( /,/, $b );
      my @a = split( /,/, $a );

      #print "$b[4] --- $a[4]\n";
      $b[1] <=> $a[1]
    } @topten_sorted;
  }
  foreach my $line1 (@topten_sorted_load_cpu) {
    my ( $item_a, $load, $server_t, $lpar_t, $hmc_t, $vm_uuid, $vcenter_uuid, $polo ) = split( ",", $line1 );

    # avoid old files which do not exist in the period
    #print STDERR "$item_a,$load,$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid\n";
    if ( $item eq "topten_vm" ) {
      my $rrd_upd_time = ( stat("$wrkdir/vmware_VMs/$vm_uuid.rrm") )[9];
      if ( $rrd_upd_time < ( time() - $time_sec ) ) {
        next;
      }
    } ## end if ( $item eq "topten_vm")
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

    if ( defined $vcenter_uuid && $vcenter_uuid =~ /vmware_/ ) {
      $top_count++;
      last if $top_count > $topten_limit;

      # in case of UTF8 names in VMWARE take encoded name from vm_uuid_name file
      my @names = grep {/$vm_uuid,/} @vm_uuid_names;
      if ( defined $names[0] && $names[0] ne "" ) {
        chomp $names[0];

        # print STDERR "2793 detail-cgi.pl \$names[0] $names[0]\n";
        $lpar = ( split( ",", $names[0] ) )[1];
      } ## end if ( defined $names[0]...)
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=&lpar=$lpar_urlx&item=vmw-netrw&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?vcenter=$server_t&item=vtop10\">$server_t</A></TD></TR>";
    } ## end if ( defined $vcenter_uuid...)
    else {
      if ( $item_a =~ /load_cpu/ ) {
        $top_count++;
        last if $top_count > $topten_limit;
        $lpar_t =~ s/\.rrm//g;
        print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=vmw-netrw&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=pool&entitle=0&none=none\">$server_t</A></TD></TR>";
      } ## end if ( $item_a =~ /load_cpu/)
    } ## end else [ if ( defined $vcenter_uuid...)]
  } ## end foreach my $line1 (@topten_sorted_load_cpu)

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_net

sub print_topten_iops {
  my ( $period, $server_url ) = @_;

  my $vm_uuid_name_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my @vm_uuid_names     = ();

  my $topten_file = "";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  if ( $item eq "topten_vm" ) {
    print "<TR><TD nowrap ><B>Average<BR>IOPS/sec</B></TD><TD nowrap><B>VM</B></TD><TD nowrap><B>vCenter</B></TD></TR>";
    $topten_file = "$tmpdir/topten_vm.tmp";
    open( FH, " < $vm_uuid_name_file" ) || error( "Cannot open $vm_uuid_name_file: $!" . __FILE__ . ":" . __LINE__ );
    @vm_uuid_names = <FH>;
    close FH;
  } ## end if ( $item eq "topten_vm")

  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my @topten = "";
  my $topten_limit = 0;

  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;
    if ( "$server_url" ne "" ) {
      #local one
      my @topten_server = grep {/vm_iops,$server,/} @topten;
      @topten = @topten_server;
    }
    else{
      my @topten_server = grep {/vm_iops/} @topten;
      @topten = @topten_server;
    }
    #print STDERR ",,@topten,,";
    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t,$top_values[1], $top_values[2], $top_values[3], $top_values[4]) = split( ",", $line );
      $top_values[4] = 0 if !defined $top_values[4] or $top_values[4] eq "";    # no year value yet, new file
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t";
    } ## end foreach my $line (@topten)
    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }

  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;
  my @topten_sorted_load_cpu;
  {
    no warnings;
    @topten_sorted_load_cpu = sort {
      my @b = split( /,/, $b );
      #print "@b!!\n";
      my @a = split( /,/, $a );
      $b[1] <=> $a[1]
    } @topten_sorted;
  }

  foreach my $line1 (@topten_sorted_load_cpu) {
    my ( $item_a, $load, $server_t, $lpar_t) = split( ",", $line1 );

    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

    if ( defined $item_a && $item_a =~ /vm_iops/ ) {
      $top_count++;
      last if $top_count > $topten_limit;

      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=&server=&lpar=$lpar_urlx&item=vmw-iops&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?vcenter=$server_t&item=vtop10\">$server_t</A></TD></TR>";
    } ## end if ( defined $vcenter_uuid...)
  } ## end foreach my $line1 (@topten_sorted_load_cpu)

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_iops

sub print_topten_disk {
  my ( $period, $server_url ) = @_;

  my $vm_uuid_name_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my @vm_uuid_names     = ();

  my $topten_file = "";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  if ( $item eq "topten_vm" ) {
    print "<TR><TD nowrap ><B>Average<BR>MB/sec</B></TD><TD nowrap><B>VM</B></TD><TD nowrap><B>vCenter</B></TD></TR>";
    $topten_file = "$tmpdir/topten_vm.tmp";
    open( FH, " < $vm_uuid_name_file" ) || error( "Cannot open $vm_uuid_name_file: $!" . __FILE__ . ":" . __LINE__ );
    @vm_uuid_names = <FH>;
    close FH;
  } ## end if ( $item eq "topten_vm")
  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my @topten = ();
  my $topten_limit = 0;
  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    my @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;
    if ( "$server_url" ne "" ) {
      #local one
      @topten = grep {/vm_disk,$server,/} @topten;
    }
    else{
      @topten = grep {/vm_disk/} @topten;
    }

    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
      $vm_uuid      = "" if !defined $vm_uuid;
      $vcenter_uuid = "" if !defined $vcenter_uuid;
      $top_values[4] = 0 if !defined $top_values[4] or $top_values[4] eq "";    # no year value yet, new file
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
    } ## end foreach my $line (@topten)
    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }

  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  my @topten_sorted_load_cpu;
  {
    no warnings;
    @topten_sorted_load_cpu = sort {
      my @b = split( /,/, $b );
      my @a = split( /,/, $a );
      #print "$b[1] --- $a[1]\n";
      $b[1] <=> $a[1]
    } @topten_sorted;
  }
  foreach my $line1 (@topten_sorted_load_cpu) {
    my ( $item_a, $load, $server_t, $lpar_t, $hmc_t, $vm_uuid, $vcenter_uuid, $polo ) = split( ",", $line1 );

    # avoid old files which do not exist in the period
    #print STDERR "$item_a,$load,$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid\n";
    if ( $item eq "topten_vm" ) {
      my $rrd_upd_time = ( stat("$wrkdir/vmware_VMs/$vm_uuid.rrm") )[9];
      if ( $rrd_upd_time < ( time() - $time_sec ) ) {
        next;
      }
    } ## end if ( $item eq "topten_vm")
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

    if ( defined $vcenter_uuid && $vcenter_uuid =~ /vmware_/ ) {
      $top_count++;
      last if $top_count > $topten_limit;

      # in case of UTF8 names in VMWARE take encoded name from vm_uuid_name file
      my @names = grep {/$vm_uuid,/} @vm_uuid_names;
      if ( defined $names[0] && $names[0] ne "" ) {
        chomp $names[0];

        # print STDERR "5648 detail-cgi.pl \$names[0] $names[0] \$lpar $lpar\n";
        $lpar = ( split( ",", $names[0] ) )[1];
      } ## end if ( defined $names[0]...)
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=&lpar=$lpar_urlx&item=vmw-diskrw&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?vcenter=$server_t&item=vtop10\">$server_t</A></TD></TR>";
    } ## end if ( defined $vcenter_uuid...)
    else {
      if ( $item_a =~ /load_cpu/ ) {
        $top_count++;
        last if $top_count > $topten_limit;
        $lpar_t =~ s/\.rrm//g;
        print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=vmw-diskrw&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=pool&entitle=0&none=none\">$server_t</A></TD></TR>";
      } ## end if ( $item_a =~ /load_cpu/)
    } ## end else [ if ( defined $vcenter_uuid...)]
  } ## end foreach my $line1 (@topten_sorted_load_cpu)

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_disk

sub print_topten {
  my ( $period, $server_url ) = @_;

  my $vm_uuid_name_file = "$wrkdir/vmware_VMs/vm_uuid_name.txt";
  my @vm_uuid_names     = ();
  # <table><tbody><tr><td><table class =\"tabconfig tablesorter\">
  #  <thead><tr><th class = \"sortable\">SERVER&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>

  my $topten_file = "$tmpdir/topten.tmp";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  if ( $item eq "topten_vm" ) {
    print "<TR><TD nowrap ><B>Average<BR>GHz</B></TD><TD nowrap><B>VM</B></TD><TD nowrap><B>vCenter</B></TD></TR>";
    $topten_file = "$tmpdir/topten_vm.tmp";
    open( FH, " < $vm_uuid_name_file" ) || error( "Cannot open $vm_uuid_name_file: $!" . __FILE__ . ":" . __LINE__ );
    @vm_uuid_names = <FH>;
    close FH;
  } ## end if ( $item eq "topten_vm")
  else {
    print "<TR><TD nowrap ><B>Avg load in cores</B></TD><TD nowrap><B>LPAR</B></TD><TD nowrap><B>Server</B></TD></TR>";
  }
  my @topten = "";
  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my $topten_limit = 0;
  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
  if ( defined $ENV{TOPTEN} ) {
    $topten_limit = $ENV{TOPTEN};
  }
  $topten_limit = 50 if $topten_limit < 1;
  if ( "$server_url" ne "" && "$item" eq "topten_vm" ) {
    #for local top 10
    my @topten_server = grep {/vm_cpu,$server,/} @topten;
    @topten = @topten_server;
  }
  elsif ( "$server_url" eq "" && "$item" eq "topten_vm" ){
    #for global top 10
    my @topten_server = grep {/vm_cpu,/} @topten;
    @topten = @topten_server;
  } ## end if ( "$server_url" ne ...)
  elsif ( "$server_url" ne "" && "$item" eq "topten" ) {
    #for local top 10
    my @topten_server = grep {/load_cpu,$server,/} @topten;
    @topten = @topten_server;
  }
  elsif ( "$server_url" eq "" && "$item" eq "topten" ) {
    #for local top 10
    my @topten_server = grep {/load_cpu,/} @topten;
    @topten = @topten_server;
  }
  else {
    error( "Bad params for topten call period $period server $server_url item $item " . __FILE__ . ":" . __LINE__ ) && return 0;
  }

  foreach my $line (@topten) {
    chomp $line;
    ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
    $vm_uuid      = "" if !defined $vm_uuid;
    $vcenter_uuid = "" if !defined $vcenter_uuid;
    $top_values[4] = 0 if !defined $top_values[4] or $top_values[4] eq "";    # no year value yet, new file
    push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
  } ## end foreach my $line (@topten)
  {
    no warnings;
    @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
  }

  }
  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  my @topten_sorted_load_cpu;
  {
    no warnings;
    @topten_sorted_load_cpu = sort {
      my @b = split( /,/, $b );
      my @a = split( /,/, $a );

      #print "$b[4] --- $a[4]\n";
      $b[1] <=> $a[1]
    } @topten_sorted;
  }
  foreach my $line1 (@topten_sorted_load_cpu) {
    my ( $item_a, $load, $server_t, $lpar_t, $hmc_t, $vm_uuid, $vcenter_uuid, $polo ) = split( ",", $line1 );

    # avoid old files which do not exist in the period
    #print STDERR "$item_a,$load,$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid\n";
    if ( $item eq "topten_vm" ) {
      my $rrd_upd_time = ( stat("$wrkdir/vmware_VMs/$vm_uuid.rrm") )[9];
      if ( $rrd_upd_time < ( time() - $time_sec ) ) {
        next;
      }
    } ## end if ( $item eq "topten_vm")
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

    if ( defined $vcenter_uuid && $vcenter_uuid =~ /vmware_/ ) {
      $top_count++;
      last if $top_count > $topten_limit;

      # in case of UTF8 names in VMWARE take encoded name from vm_uuid_name file
      my @names = grep {/$vm_uuid,/} @vm_uuid_names;
      if ( defined $names[0] && $names[0] ne "" ) {
        chomp $names[0];

        # print STDERR "2793 detail-cgi.pl \$names[0] $names[0]\n";
        $lpar = ( split( ",", $names[0] ) )[1];
      } ## end if ( defined $names[0]...)
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=&lpar=$lpar_urlx&item=vmw-cpu&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?vcenter=$server_t&item=vtop10\">$server_t</A></TD></TR>";
    } ## end if ( defined $vcenter_uuid...)
    else {
      if ( $item_a =~ /load_cpu/ ) {
        $top_count++;
        last if $top_count > $topten_limit;
        $lpar_t =~ s/\.rrm//g;
        print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=lpar&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=pool&entitle=0&none=none\">$server_t</A></TD></TR>";
      } ## end if ( $item_a =~ /load_cpu/)
    } ## end else [ if ( defined $vcenter_uuid...)]
  } ## end foreach my $line1 (@topten_sorted_load_cpu)

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten

sub print_topten_san_iops {
  my ( $period, $server_url ) = @_;

  my $topten_file = "$tmpdir/topten.tmp";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";

  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my $topten_limit = 0;
  my @topten = "";


  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;

    #print STDERR "2741 detail-cgi.pl \$period $period \$server_url $server_url \$server $server \@topten @topten\n";
    if ( "$server_url" ne "" ) {

      #local one
      my @topten_server = grep {/,$server,/} @topten;
      @topten = @topten_server;
    } ## end if ( "$server_url" ne ...)
    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
      $vm_uuid      = "" if !defined $vm_uuid;
      $vcenter_uuid = "" if !defined $vcenter_uuid;
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
    } ## end foreach my $line (@topten)

    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }
  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  print "<br><TD align=\"center\"><TABLE class=\"tabtop10\">";
  print "<TR><TD nowrap ><B>IOPS avrg</B></TD><TD nowrap><B>LPAR</B></TD><TD nowrap><B>SERVER</B></TD></TR>";

  #print "@topten_sorted\n";
  my @topten_sorted_os_san = sort {
    my @b = split( /,/, $b );
    my @a = split( /,/, $a );
    $b[1] <=> $a[1]
  } @topten_sorted;
  foreach my $line1 (@topten_sorted_os_san) {
    ( my $item_a, my $load, my $server_t, my $lpar_t, my $hmc_t, my $vm_uuid, my $vcenter_uuid, undef ) = split( ",", $line1 );
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
    if ( $item_a =~ /^os_san_iops$/ ) {
      $top_count++;
      last if $top_count > $topten_limit;
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=lpar&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=san1&entitle=0&none=none\">$server_t</A></TD></TR>";
    }
  } ## end foreach my $line1 (@topten_sorted_os_san)

  #print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>"; # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_san_iops

sub print_topten_san {
  my ( $period, $server_url ) = @_;
  my $topten_file = "$tmpdir/topten.tmp";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my $topten_limit = 0;
  my @topten = "";
  if (-f $topten_file) {
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;

    #print STDERR "2741 detail-cgi.pl \$period $period \$server_url $server_url \$server $server \@topten @topten\n";
    if ( "$server_url" ne "" ) {

      #local one
      my @topten_server = grep {/,$server,/} @topten;
      @topten = @topten_server;
    } ## end if ( "$server_url" ne ...)
    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
      $vm_uuid      = "" if !defined $vm_uuid;
      $vcenter_uuid = "" if !defined $vcenter_uuid;
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
    } ## end foreach my $line (@topten)

    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }

  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  print "<br><TD align=\"center\"><TABLE class=\"tabtop10\">";
  print "<TR><TD nowrap ><B>MB avrg</B></TD><TD nowrap><B>LPAR</B></TD><TD nowrap><B>SERVER</B></TD></TR>";

  #print "@topten_sorted\n";
  my @topten_sorted_os_san = sort {
    my @b = split( /,/, $b );
    my @a = split( /,/, $a );
    $b[1] <=> $a[1]
  } @topten_sorted;
  foreach my $line1 (@topten_sorted_os_san) {
    ( my $item_a, my $load, my $server_t, my $lpar_t, my $hmc_t, my $vm_uuid, my $vcenter_uuid, undef ) = split( ",", $line1 );
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
    if ( $item_a =~ /^os_san1$/ ) {
      $top_count++;
      last if $top_count > $topten_limit;
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=lpar&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=san1&entitle=0&none=none\">$server_t</A></TD></TR>";
    }

  } ## end foreach my $line1 (@topten_sorted_os_san)

  #print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>"; # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_san

sub print_topten_lan {
  my ( $period, $server_url ) = @_;

  my $topten_file = "$tmpdir/topten.tmp";
  print "<TD align=\"center\"><TABLE class=\"tabtop10\">";
  my @topten = "";
  my @top_values;
  my @topten_not_sorted;
  my @topten_sorted;
  my $topten_limit = 0;
  if (-f $topten_file){
    open( FH, " < $topten_file" ) || error( "Cannot open $topten_file: $!" . __FILE__ . ":" . __LINE__ );
    @topten = <FH>;
    close FH;
    if ( defined $ENV{TOPTEN} ) {
      $topten_limit = $ENV{TOPTEN};
    }
    $topten_limit = 50 if $topten_limit < 1;

    #print STDERR "2741 detail-cgi.pl \$period $period \$server_url $server_url \$server $server \@topten @topten\n";
    if ( "$server_url" ne "" ) {

      #local one
      my @topten_server = grep {/,$server,/} @topten;
      @topten = @topten_server;
    } ## end if ( "$server_url" ne ...)
    foreach my $line (@topten) {
      chomp $line;
      ( my $item_a, my $server_t, my $lpar_t, my $hmc_t, $top_values[1], $top_values[2], $top_values[3], $top_values[4], my $vm_uuid, my $vcenter_uuid ) = split( ",", $line );
      $vm_uuid      = "" if !defined $vm_uuid;
      $vcenter_uuid = "" if !defined $vcenter_uuid;
      push @topten_not_sorted, "$item_a,$top_values[$period],$server_t,$lpar_t,$hmc_t,$vm_uuid,$vcenter_uuid";
    } ## end foreach my $line (@topten)

    {
      no warnings;
      @topten_sorted = sort { $b <=> $a } @topten_not_sorted;
    }
  }
  my $time_sec = 2 * 86400;
  $time_sec = 8 * 86400   if $period == 2;
  $time_sec = 30 * 86400  if $period == 3;
  $time_sec = 365 * 86400 if $period == 4;
  my $top_count = 0;

  print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>";    # to add the data source icon
  print "</TABLE></TD>";
  print "<br><TD align=\"center\"><TABLE class=\"tabtop10\">";
  print "<TR><TD nowrap ><B>MB avrg</B></TD><TD nowrap><B>LPAR</B></TD><TD nowrap><B>SERVER</B></TD></TR>";

  #print "@topten_sorted\n";
  my @topten_sorted_os_san = sort {
    my @b = split( /,/, $b );
    my @a = split( /,/, $a );
    $b[1] <=> $a[1]
  } @topten_sorted;
  foreach my $line1 (@topten_sorted_os_san) {
    ( my $item_a, my $load, my $server_t, my $lpar_t, my $hmc_t, my $vm_uuid, my $vcenter_uuid, undef ) = split( ",", $line1 );
    my $lpar = $lpar_t;
    $lpar =~ s/\.rrm$//;
    $lpar =~ s/\.rrh$//;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;
    my $lpar_urlx = $lpar;

    #$lpar_urlx =~ s/ /+/g;
    $lpar_urlx =~ s/([^a-zA-Z0-9_.!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
    if ( $item_a =~ /^os_lan$/ ) {
      $top_count++;
      last if $top_count > $topten_limit;
      print "<TR><TD  align=\"right\" nowrap>$load</TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_urlx&item=lpar&entitle=0&none=none\">$lpar</A></TD><TD nowrap><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=san1&entitle=0&none=none\">$server_t</A></TD></TR>";
    }
  } ## end foreach my $line1 (@topten_sorted_os_san)

  #print "<ul style=\"display: none\"><li class=\"tabhmc\"></li></ul>"; # to add the data source icon
  print "</TABLE></TD>";
  return 1;
} ## end sub print_topten_lan

sub print_topten_vm {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my $topten_limit = 0;
  if ( defined $ENV{TOPTEN} ) {
    $topten_limit = $ENV{TOPTEN};
  }
  $topten_limit = 50 if $topten_limit < 1;

  #print_topten_head ($server_url,$host_url);
  #print_topten_head ($server,$host_url);

  print "<div  id=\"tabs\">\n";
  print "<CENTER>";

  print "<ul>\n";
  print "  <li class=\"tabhmc\"><a href=\"#tabs-1\">Last Day</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-2\">Last Week</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-3\">Last Month</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-4\">Last Year</a></li>\n";


  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th></th></th><th align=center>IOPS</th></th></th><th align=center>DISK</th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten( "1", "$server_url" );    #day
  print_topten_iops( "1", "$server_url" );
  print_topten_disk( "1", "$server_url" );
  print_topten_net( "1", "$server_url" );
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-2\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th></th></th><th align=center>IOPS</th></th></th><th align=center>DISK</th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten( "2", "$server_url" );    #week
  print_topten_iops( "2", "$server_url" );
  print_topten_disk( "2", "$server_url" );
  print_topten_net( "2", "$server_url" );
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-3\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th></th></th><th align=center>IOPS</th></th></th><th align=center>DISK</th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten( "3", "$server_url" );               #month
  print_topten_iops( "3", "$server_url" );
  print_topten_disk( "3", "$server_url" );
  print_topten_net( "3", "$server_url" );
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-4\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th></th></th><th align=center>IOPS</th></th></th><th align=center>DISK</th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten( "4", "$server_url" );               #year
  print_topten_iops( "4", "$server_url" );
  print_topten_disk( "4", "$server_url" );
  print_topten_net( "4", "$server_url" );
  print "</tr>";
  print "</table>";
  print "</div>\n";


  print "<div><br>Note that this page is refreshed once a day, the first LPAR2RRD run after midnight";
  print "<br>If you want to have different number of lpars here like top 100 etc, then modify parameter TOPTEN=$topten_limit in etc/lpar2rrd.cfg";
  my $file = "$tmpdir/topten_vm.tmp";
  if ( !-f $file ) {
    error( "File $file does not exists(first run after midnight create it)") && return;
  }
  my $last_mod_time = ( stat($file) )[9];
  print "<br>Included are VMs poweredOn only";
  print "<br>Updated: ";
  print scalar localtime $last_mod_time;
  print "<br></div>";

  print "</div><br>\n";

  return 1;
} ## end sub print_topten_vm

sub print_topten_all {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my $topten_limit = 0;
  if ( defined $ENV{TOPTEN} ) {
    $topten_limit = $ENV{TOPTEN};
  }
  $topten_limit = 50 if $topten_limit < 1;

  #print_topten_head ($server_url,$host_url);
  #print_topten_head ($server,$host_url);

  print "<div  id=\"tabs\">\n";
  print "<CENTER>";

  print "<ul>\n";
  print "  <li class=\"tabhmc\"><a href=\"#tabs-1\">Last Day</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-2\">Last Week</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-3\">Last Month</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-4\">Last Year</a></li>\n";

  print "  <li class=\"tabhmc\"><a href=\"#tabs-5\">All Periods</a></li>\n";

  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th><th align=center></th></th><th align=center>SAN IOPS</th><th align=center></th></th><th align=center>SAN</th><th align=center></th></th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
    print_topten ( "1", "$server_url");           #day
    print_topten_san_iops( "1", "$server_url" );  #day
    print_topten_san( "1", "$server_url" );       #day
    print_topten_lan( "1", "$server_url" );       #day
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-2\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr><th align=center>LOAD CPU</th><th align=center></th><th align=center>SAN IOPS</th><th align=center></th></th><th align=center>SAN</th><th align=center></th></th><th align=center>LAN</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten ( "2", "$server_url");             #week
  print_topten_san_iops( "2", "$server_url" );    #week
  print_topten_san( "2", "$server_url" );         #week
  print_topten_lan( "2", "$server_url" );         #week
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-3\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr style=\"vertical-align:top\"><th align=center>LOAD CPU</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten ( "3", "$server_url");             #month
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-4\">\n";
  print "<table align=\"center\" summary=\"Graphs\">\n";
  print "<tr style=\"vertical-align:top\"><th align=center>LOAD CPU</th></tr>";
  print "<tr style=\"vertical-align:top\">";
  print_topten ( "4", "$server_url");             #year
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "<div id=\"tabs-5\">\n";

  print "<CENTER>";
  print "<TABLE class=\"tabtop10\">";
  print "<TR><TD align=\"center\"> <b>Last day</b> </TD><TD  align=\"center\"> <b>Last week</b> </TD></TR>";

  print "<TR style=\"vertical-align:top\">";
  print_topten( "1", "$server_url" );             #day
  print_topten( "2", "$server_url" );             #week
  print "<TR>";
  print "<TR><TD align=\"center\"> <br><br><b>Last month</b> </TD><TD  align=\"center\"> <br><br><b>Last year</b> </TD></TR>";
  print "<TR style=\"vertical-align:top\">";
  print_topten( "3", "$server_url" );             #month
  print_topten( "4", "$server_url" );             #year
  print "</TR></TABLE>";
  print "</center>";
  print "</div>\n";
  print "</center>";

  print "<div><br>Note that this page is refreshed once a day, the first LPAR2RRD run after midnight";
  print "<br>If you want to have different number of lpars here like top 100 etc, then modify parameter TOPTEN=$topten_limit in etc/lpar2rrd.cfg";
  my $file = "$tmpdir/topten.tmp";
  if ( !-f $file ) {
    error( "File $file does not exists(first run after midnight create it)") && return;
  }
  my $last_mod_time = ( stat($file) )[9];
  print "<br>Updated: ";
  print scalar localtime $last_mod_time;
  print "<br></div>";

  print "</div><br>\n";

  return 1;
} ## end sub print_topten_all

sub print_data_check {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my $vmotion_config = "$tmpdir/lpar_start_counter.txt";
  my $tab_check      = "0";

  print "<div  id=\"tabs\">\n";
  print "<CENTER>";

  print "<ul>\n";
  print "  <li ><a href=\"#tabs-1\">Data check</a></li>\n";
  print "  <li ><a href=\"#tabs-2\">OS agent check</a></li>\n";
  print "  <li ><a href=\"#tabs-3\">IBM i</a></li>\n";
  if ( -f $vmotion_config ) {
    print "  <li ><a href=\"#tabs-4\">VMotion</a></li>\n";
    $tab_check = "1";
  }
  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  my $file_html = "$webdir/daily_lpar_check.html";
  if ( -f "$file_html" ) {
    open( FH, "< $file_html" );
    my $dlch_html = do { local $/; <FH> };
    close(FH);
    print "$dlch_html";
  } ## end if ( -f "$file_html" )
  print "</div>\n";

  print "<div id=\"tabs-2\">\n";
  $file_html = "$webdir/daily_agent_check.html";
  if ( -f "$file_html" ) {
    open( FH, "< $file_html" );
    my $dlch_html = do { local $/; <FH> };
    close(FH);
    print "$dlch_html";
  } ## end if ( -f "$file_html" )
  print "</div>\n";

  print "<div id=\"tabs-3\">\n";
  print "<center><h4>Table IBM i</h4>
  <table><tbody><tr><td><table class =\"tabconfig tablesorter\">
  <thead><tr><th class = \"sortable\">SERVER&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
             <th class = \"sortable\">LPAR&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
             <th class = \"sortable\">Agent status&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
             <th class = \"sortable\">Version&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
             <th class = \"sortable\">License expiration&nbsp;&nbsp;</th></tr></thead><tbody>";
  print "</td></tr>\n";

  # read tmp/daily_lpar_check.txt for agent i status
  my @daily_lpar_check      = ();
  my $daily_lpar_check_file = "$tmpdir/daily_lpar_check.txt";
  open( FH, "< $daily_lpar_check_file" ) || error( "Cannot open $daily_lpar_check_file: $!" . __FILE__ . ":" . __LINE__ );
  @daily_lpar_check = <FH>;
  close(FH);
  my $upd_time     = ( stat("$daily_lpar_check_file") )[9];
  my $upd_time_txt = localtime($upd_time);

  my %servers_lpars = ();
  my @files         = glob "$wrkdir/*/*/*--AS400--/license.cfg";
  my $act_time_u = time();

  foreach my $file (@files) {
    my @dirs           = split( "\/", $file );
    my $lpar           = $dirs[-2];
    my $hmc            = $dirs[-3];
    my $server         = $dirs[-4];
    next if -l "$wrkdir/$server"; # skip server sym link
    $lpar =~ s/--AS400--//;

    if ( -f "$wrkdir/$server/$hmc/cpu.cfg") {
      my $cpu_cfg_file = "$wrkdir/$server/$hmc/cpu.cfg";
      open( FH, "< $cpu_cfg_file" ) || error( "Cannot open $cpu_cfg_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
       my @cpu_cfg = <FH>;
      close(FH);
      # looks like lpar_name=as400,lpar_id=6,curr_proc_mode=ded...
      my $test_string = "lpar_name="."$lpar".",";
      my @lpar_line = grep {/^$test_string/} @cpu_cfg;
      next if ! defined $lpar_line[0];
    }

    my $string_to_find = $server . "," . $hmc . "," . $lpar . ",agent";
    $server =~ s/--unknown//;
    next if exists $servers_lpars{ $server . $lpar };

    my $status = "not running";
    if (($act_time_u - (stat($file))[9]) < 86400) {
      $status = "running";
    }
    # read licence
    open( FH, "< $file" ) || error( "Cannot open $file: $!" . __FILE__ . ":" . __LINE__ ) && next;
    my $line = do { local $/; <FH> };
    close(FH);
    chomp $line;

    # read version
    $file =~ s/license\.cfg$/agent.cfg/;
    open( FH, "< $file" ) || error( "Cannot open $file: $!" . __FILE__ . ":" . __LINE__ ) && next;
    my $line_version = do { local $/; <FH> };
    close(FH);
    chomp $line_version;
    # older agent had bug in time - it looked: '43.940 version 1.1.5'
    if ($line_version =~ /version/) {
      (undef, $line_version) = split( "version ",$line_version);
    }

    # my $status = "not updated";
    # my @agent_status = grep /$string_to_find/, @daily_lpar_check;

    # # print STDERR "2987 detail-cgi.pl \$agent_status[0] $agent_status[0]\n";
    # if ( defined $agent_status[0] && $agent_status[0] ne "" ) {
    #   if ( $agent_status[0] =~ "OK" ) {
    #     $status = "running";
    #   }
    #   else {
    #     $status = "not updated";
    #   }
    # } ## end if ( defined $agent_status...)



    print "<tr><td>$server</td><td>$lpar</td><td>$status</td><td>$line_version</td><td>$line</td></tr>\n";
    $servers_lpars{ $server . $lpar } = 1;
  } ## end foreach my $file (@files)
  print "</tbody></table></table>\n";
  print "<center>\"not running\" is when agent data is older 24 hours<br></center>";
  print "The table has been updated at: $upd_time_txt<br></center>";
  print "</div>\n";

  if ( -f $vmotion_config ) {
    print "<div id=\"tabs-4\">\n";
    open( FC, "< $vmotion_config" ) || error( "Cannot read $vmotion_config: $!" . __FILE__ . ":" . __LINE__ );
    my @start = <FC>;
    close(FC);

    print "<table><tbody><tr><td><table class =\"tabconfig tablesorter\">
    <thead><tr><th class= \"sortable\">LPAR</th>
               <th class= \"sortable\">VMotion total</th>
               <th class= \"sortable\">VMotion last day</th>
               <th class= \"sortable\">VMotion last week</th>
               <th class= \"sortable\">VMotion last month</th>
               </tr></thead>";

    # get active VMs from menu
    my $vm_menu_config = "$basedir/tmp/menu_vmware.txt";
    my @vm_menu;
    if ( -f $vm_menu_config ) {
      open( FC, "< $vm_menu_config" ) || error( "Cannot read $vm_menu_config: $!" . __FILE__ . ":" . __LINE__ );
      @vm_menu = <FC>;
      close(FC);
    }
    my @lpar_only = grep {/^L:/} @vm_menu;

    #L:cluster_Cluster01 renamed:192.168.1.124:500f0ccd-ebc7-07c6-4622-0002f450f695:SDMC:/lpar2rrd-cgi/detail.sh?host=pavel.lpar2rrd.com&server=192.168.1.124&lpar=500f0ccd-ebc7-07c6-4622-0002f450f695&item=lpar&entitle=0&gui=1&none=none::Regina_Vcentre:V

    #  print to table
    foreach my $line (@start) {
      next if $line =~ /^\#/;
      ( my $lpar, my $starts, my $last_day, my $last_week, my $last_month ) = split( /;/, $line );
      ( my $vm_line ) = grep( /$lpar:/, @lpar_only );
      next if not defined $vm_line;

      #print STDERR "3631 \$vm_line ,$vm_line,\$lpar ,$lpar,\n";
      ( undef, my $ahref ) = split( /$lpar:/, $vm_line );
      ##print STDERR "3631 \$vm_line ,$vm_line,\n";
      ( my $final_ahref, undef ) = split( /::/, $ahref );

      #print "$final_ahref<br>";
      my $lpar_item = "lpar=";
      ( my $start, undef ) = split( /lpar=/, $final_ahref );
      ( undef, my $end ) = split( /&item=/, $final_ahref );
      my $name_lpar = "$start$lpar_item$lpar&item=$end";
      print "<tr><th><a href=\"$name_lpar\">$lpar</a></th><th>$starts</th><th>$last_day</th><th>$last_week</th><th>$last_month</th></tr>";
    } ## end foreach my $line (@start)
    print "</tbody></td></tr></table></table>";
    my $last_mod_time = localtime( ( stat($vmotion_config) )[9] );
    print "The table summarizes start-stops (vmotions) of VMs. Last week is without last day, last month is without last week. It has been updated: $last_mod_time";
    print "</div>\n";
  } ## end if ( -f $vmotion_config)

  print "</div><br>\n";

  return 1;
} ## end sub print_data_check

sub print_data_servers {
  $rest_api = is_any_host_rest();
  if (!$rest_api){#use old config page
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my @hmc_list  = <$webdir/*\/gui-config-high-sum.html>;
  my $hmc_count = @hmc_list;
  return 0 if !$hmc_count;    # no hmc ?

  my $cvs_lpar   = "lpar-config.csv";
  my $cvs_server = "server-config.csv";

  print "<div  id=\"tabs\">\n";
  print "<CENTER>";

  print "<ul>\n";
  print "  <li ><a href=\"#tabs-1\">Servers</a></li>\n";
  for ( my $i = 0; $i < $hmc_count; $i++ ) {
    my $hmc_path = $hmc_list[$i];
    my @hmcx     = split( "/", $hmc_path );
    my $my_hmc   = @hmcx[ @hmcx - 2 ];
    my $index    = $i + 1;
    print "  <li class=\"hmcsum\"><a href=\"#tabs-$index-a\">$my_hmc</a></li>\n";
    print "  <li class=\"hmcdet\"><a href=\"#tabs-$index-b\">$my_hmc</a></li>\n";
  } ## end for ( my $i = 0; $i < $hmc_count...)
  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  my $file_html = "$webdir/cfg_summary.html";
  my $print_html;
  if ( -f "$file_html" ) {
    open( FH, "< $file_html" );
    $print_html = do { local $/; <FH> };
    close(FH);
    print "$print_html";
  } ## end if ( -f "$file_html" )
  my $host = "";
  print "<a href=\"$cvs_lpar\"><div class=\"csvexport\">CSV LPAR</div></a>";
  print "<a href=\"$cvs_server\"><div class=\"csvexport csvexport_down\">CSV Server</div></a>";

  print "</div>\n";

  for ( my $i = 0; $i < $hmc_count; $i++ ) {

    my $hmc_path = $hmc_list[$i];
    my @hmcx     = split( "/", $hmc_path );
    my $my_hmc   = @hmcx[ @hmcx - 2 ];
    my $index    = $i + 1;

    print "<div id=\"tabs-$index-a\">\n";
    if ( -f "$hmc_path" ) {
      open( FH, "< $hmc_path" );
      my $print_html = do { local $/; <FH> };
      close(FH);
      print "$print_html";
    } ## end if ( -f "$hmc_path" )
    print "<a href=\"$my_hmc/$my_hmc-$cvs_server\"><div class=\"csvexport\">CSV Server</div></a>";
    print "</div>\n";

    print "<div id=\"tabs-$index-b\">\n";
    $hmc_path =~ s/-sum//;
    if ( -f "$hmc_path" ) {
      open( FH, "< $hmc_path" );
      my $print_html = do { local $/; <FH> };
      close(FH);
      print "$print_html";
    } ## end if ( -f "$hmc_path" )
    print "<a href=\"$my_hmc/$my_hmc-$cvs_lpar\"><div class=\"csvexport\">CSV LPAR</div></a>";

    print "</div>\n";
  } ## end for ( my $i = 0; $i < $hmc_count...)

  print "</div><br>\n";

  return 1;
  }
  else{ #use new config page - wip
  my $cli_conf = 0;
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my @hmc_list  = <$webdir/*\/gui-config-high-sum.html>;
  my $hmc_count = @hmc_list;
  if (!$hmc_count){
    @hmc_list = <$webdir/HMC_INFO*.json>;
    $hmc_count = @hmc_list;
  }

  return 0 if !$hmc_count;    # no hmc ?

  my $cvs_lpar   = "lpar-config.csv";
  my $cvs_server = "server-config.csv";
  my $cvs_lpar_rest   = "lpar-config-rest.csv";
  my $cvs_server_rest = "server-config-rest.csv";

  print "<div  id=\"tabs\">\n";
  print "<CENTER>";
  print "<ul>\n";
  my $tab_index = 1;
  print "  <li ><a href=\"#tabs-$tab_index\">Servers</a></li>\n"; $tab_index++;
  if (-e "$webdir/enterprise_pool.html"){
    print "  <li ><a href=\"#tabs-$tab_index\">Enterprise Pool</a></li>\n"; $tab_index++;
  }
  #my @files_interfaces = <$webdir/interfaces*.html>;
  #if (defined $files_interfaces[0]){
  #  print "  <li ><a href=\"#tabs-$tab_index\">Interfaces</a></li>\n"; $tab_index++;
  #}
  my @files_env_conf = <$basedir/tmp/restapi/env_conf_*.json>;
  if (defined  $files_env_conf[0]){
    print "  <li ><a href=\"#tabs-$tab_index\">Interfaces</a></li>\n"; $tab_index++;
  }
  if (-e "$webdir/cfg_summary.html"){
    my $act_time = time();
    my $last_update = ( stat("$webdir/cfg_summary.html") )[9];
#    my $last_update = stat("$webdir/cfg_summary.html");
    $cli_conf = $act_time - $last_update;
    if ($cli_conf <= 86400){
      print "  <li ><a href=\"#tabs-$tab_index\">CLI Config</a></li>\n"; $tab_index++;
    }
  }

  my $file_net_conf = "$basedir/tmp/restapi/net_conf.json";
  if (-e "$file_net_conf"){
    print "  <li ><a href=\"#tabs-$tab_index\">Network Configuration</a></li>\n"; $tab_index++;
  }
  my $file_vscsi_conf = "$basedir/tmp/restapi/vscsi_conf.json";
  if (-e "$file_vscsi_conf"){
    print "  <li ><a href=\"#tabs-$tab_index\">VSCSI</a></li>\n"; $tab_index++;
  }
  my $file_npiv_conf = "$basedir/tmp/restapi/npiv_conf.json";
  if (-e "$file_npiv_conf"){
    print "  <li ><a href=\"#tabs-$tab_index\">NPIV</a></li>\n"; $tab_index++;
  }



  my @details  = <$webdir/*\/*\/detail.html>;
  my @servers;
=begin config tabs
  foreach my $det (@details){
    my ($host, $server) = split ("www/", $det);
    ($server, undef) = split ('/detail.html',$server);
    ($host,$server) = split ('/', $server);
    if ( grep( /^$server$/, @servers )) {
      next;
    }
    push(@servers, $server);
    print "  <li ><a href=\"#tabs-$tab_index\">$server</a></li>\n"; $tab_index++;
  }
=cut
  print "</ul>\n";
  $tab_index = 1;
  print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;
  #my $file_html_rest = "$webdir/config_servers.html";
  my $file_html_rest = "$webdir/config_table_main.html";
  my $print_html;
  if ( -f "$file_html_rest" ) {
    open( FH, "< $file_html_rest" );
    $print_html = do { local $/; <FH> };
    close(FH);
    print "$print_html";
  } ## end if ( -f "$file_html" )
  print "</div>\n";
  my $host = "";

  print "<a href=\"$cvs_lpar\"><div class=\"csvexport\">CSV LPAR</div></a>";
  print "<a href=\"$cvs_server\"><div class=\"csvexport csvexport_down\">CSV Server</div></a>";
  print "<a href=\"$cvs_lpar_rest\"><div class=\"csvexport csvexport_down_3\">CSV LPAR Rest</div></a>";
  print "<a href=\"$cvs_server_rest\"><div class=\"csvexport csvexport_down_4\">CSV Server Rest</div></a>";

  print "</CENTER>\n";

  #Enterprise pool tab
  my $file_html_enterprise = "$webdir/enterprise_pool.html";
  if ( -f "$file_html_enterprise" ) {
    print "<center>";
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;
    open( FH, "< $file_html_enterprise" );
    $print_html = do { local $/; <FH> };
    close(FH);
    print "$print_html";
    print "</div>\n";
    print "</center>";
  } ## end if ( -f "$file_html_enterprise" )

  if (defined $files_env_conf[0]){
  print "<CENTER>\n";
  print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;
  print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1">
      <thead>
      <TR>
        <TH align="center" class="sortable" valign="center">Server</TH>
        <TH align="center" class="sortable" valign="center">Device Name</TH>
        <TH align="center" class="sortable" valign="center">Partition</TH>
        <TH align="center" class="sortable" valign="center">Ports</TH>
        <TH align="center" class="sortable" valign="center">WWPN</TH>
        <TH align="center" class="sortable" valign="center">Description</TH>
      </TR>
      </thead>
      <tbody>';
  my $done;
  foreach my $file_env_conf (@files_env_conf){
  if ( -f "$file_env_conf" ){
    my $env_conf = read_json("$file_env_conf");
    foreach my $server (keys %{$env_conf}){
      foreach my $server_loc (keys %{$env_conf->{$server}}){
        foreach my $c (keys %{$env_conf->{$server}{$server_loc}}){
          my $item = $env_conf->{$server}{$server_loc}{$c};
          if (!$done->{"$server_loc-$c"}){
          print "<TR>
                <TD>$server</TD>
                <TD>$server_loc-$c</TD>
                <TD>$item->{'PartitionName'}</TD>";
                print "<TD>";
                foreach my $Trunk (keys %{$item->{'Trunk'}}){
                  print "$Trunk / $item->{'Trunk'}{$Trunk}{'en_name'}<br>";
                }
                print "</TD>";
                print "<TD>$item->{'WWPN'}</TD>";
                print "<TD>$item->{'Description'}</TD>";
                print "</TR>\n";
          }
          $done->{"$server_loc-$c"} = 1;
        }
      }
    }
=cut
  foreach my $f (@files_interfaces){
    my $file_html_interfaces = $f;
    if ( -f "$file_html_interfaces") {
      open( FH, "< $file_html_interfaces" );
      (undef, $f) = split ("interfaces_", $f);
      ($f,undef) = split ('\.', $f);
      print "<h4>$f</h4>\n";
      $print_html = do { local $/; <FH> };
      close(FH);
      print "$print_html";
    } ## end if ( -f "$file_html_interfaces" )
  }
=cut
  }
  }
    print "
     </tbody>
    </TABLE>";
  print "</div>\n";
  print "</CENTER>\n";
  }

  my $file_html_cli_config = "$webdir/cfg_summary.html";
  if ( -f "$file_html_cli_config" && $cli_conf <= 86400) {
    print "<CENTER>\n";
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;
    open( FH, "< $file_html_cli_config" );
    $print_html = do { local $/; <FH> };
    close(FH);
    print "$print_html";
    print "</div>\n";
    print "</CENTER>\n";
  } ## end if ( -f "$file_html_cli_config" )

  if ( -f "$file_net_conf") {
    my $net_json = read_json($file_net_conf);
    print "<CENTER>\n";
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;

    #Virtual Networks
    print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">Vlan</TH>
     <TH align="center" class="sortable" valign="center">Tagged</TH>
     <TH align="center" class="sortable" valign="center">Switch UUID</TH>
     <TH align="center" class="sortable" valign="center">Vswitch ID</TH>
     <TH align="center" class="sortable" valign="center">Vlan ID</TH>
   </TR>
  </thead>
  <tbody>';
    print "<h3>VirtualNetwork</h3>\n";
    foreach my $s (keys %{$net_json}){
      foreach my $vlan (keys %{$net_json->{$s}{VirtualNetwork}}){
        print "<TR>
              <TD>$s</TD>
              <TD>$vlan</TD>
              <TD>$net_json->{$s}{VirtualNetwork}{$vlan}{TaggedNetwork}</TD>
              <TD>$net_json->{$s}{VirtualNetwork}{$vlan}{AssociatedSwitchUUID}</TD>
              <TD>$net_json->{$s}{VirtualNetwork}{$vlan}{VswitchID}</TD>
              <TD>$net_json->{$s}{VirtualNetwork}{$vlan}{NetworkVLANID}</TD>
              </TR>\n";
      }
    }
    print "
     </tbody>
    </TABLE>";
    #end Virtual Networks

    #SEA
    print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">Ent</TH>
     <TH align="center" class="sortable" valign="center">UniqueDeviceID</TH>
     <TH align="center" class="sortable" valign="center">ThreadModeEnabled</TH>
     <TH align="center" class="sortable" valign="center">LargeSend</TH>
     <TH align="center" class="sortable" valign="center">PortVLANID</TH>
     <TH align="center" class="sortable" valign="center">ConfigurationState</TH>
     <TH align="center" class="sortable" valign="center">JumboFramesEnabled</TH>
     <TH align="center" class="sortable" valign="center">QueueSize</TH>
     <TH align="center" class="sortable" valign="center">QualityOfServiceMode</TH>
     <TH align="center" class="sortable" valign="center">HighAvailabilityMode</TH>
     <TH align="center" class="sortable" valign="center">IPInterface</TH>
     <TH align="center" class="sortable" valign="center">IsPrimary</TH>
   </TR>
  </thead>
  <tbody>';
    print "<h3>SEA</h3>\n";
    foreach my $s (keys %{$net_json}){
    foreach my $ent (keys %{$net_json->{$s}{SEA}}){
      print "<TR>
              <TD>$s</TD>
              <TD>$ent</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{UniqueDeviceID}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{ThreadModeEnabled}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{LargeSend}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{PortVLANID}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{ConfigurationState}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{JumboFramesEnabled}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{QueueSize}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{QualityOfServiceMode}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{HighAvailabilityMode}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{IPInterface}</TD>
              <TD>$net_json->{$s}{SEA}{$ent}{IsPrimary}</TD>
            </TR>\n";
    }
    }
    print "
     </tbody>
    </TABLE>";
    #end SEA

    #Virtual Switch
    print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">Switch</TH>
     <TH align="center" class="sortable" valign="center">SwitchID</TH>
     <TH align="center" class="sortable" valign="center">VlanIDs</TH>
   </TR>
  </thead>
  <tbody>';
    print "<h3>Virtual Switch</h3>\n";
    foreach my $s (keys %{$net_json}){
    foreach my $switch_name (keys %{$net_json->{$s}{VirtualSwitch}}){
      print "<TR>
              <TD>$s</TD>
              <TD>$switch_name</TD>
              <TD>$net_json->{$s}{VirtualSwitch}{$switch_name}{SwitchID}</TD>";
              my $vlan_ids_string = "";
              foreach my $vlan (keys %{$net_json->{$s}{VirtualSwitch}{$switch_name}{SwitchVlans}}){
                $vlan_ids_string .= "$net_json->{$s}{VirtualSwitch}{$switch_name}{SwitchVlans}{$vlan}{NetworkVLANID},";
              }
              $vlan_ids_string =~ s/,$//g;
      print "<TD>$vlan_ids_string</TD>
            </TR>\n";
    }
    }
    print "
     </tbody>
    </TABLE>";
    #end VirtualSwitch

    #Trunk
    print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">Ent</TH>
     <TH align="center" class="sortable" valign="center">MACAddress</TH>
     <TH align="center" class="sortable" valign="center">Location</TH>
     <TH align="center" class="sortable" valign="center">DynamicReconfigurationConnectorName</TH>
     <TH align="center" class="sortable" valign="center">VirtualSwitchID</TH>
     <TH align="center" class="sortable" valign="center">TrunkPriority</TH>
     <TH align="center" class="sortable" valign="center">PortVLANID</TH>
     <TH align="center" class="sortable" valign="center">VirtualSlotNumber</TH>
   </TR>
  </thead>
  <tbody>';
    print "<h3>Trunk</h3>\n";
    foreach my $s (keys %{$net_json}){
    foreach my $ent (keys %{$net_json->{$s}{Trunk}}){
      print "<TR>
              <TD>$s</TD>
              <TD>$ent</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{MACAddress}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{LocationCode}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{DynamicReconfigurationConnectorName}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{VirtualSwitchID}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{TrunkPriority}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{PortVLANID}</TD>
              <TD>$net_json->{$s}{Trunk}{$ent}{VirtualSlotNumber}</TD>
            </TR>\n";
    }
    }
    print "
     </tbody>
    </TABLE>";
    #end VirtualSwitch



    print "</div>\n";
    print "</CENTER>\n";
  } ## end if ( -f "$file_html_cli_config" )

  if ( -f "$file_vscsi_conf" ) {
    print "<center>";
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;

    my $vscsi = read_json($file_vscsi_conf);
      print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1 2">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">VIOS</TH>
     <TH align="center" class="sortable" valign="center">VIOS Adapter</TH>
     <TH align="center" class="sortable" valign="center">Server Slot</TH>
     <TH align="center" class="sortable" valign="center">Client LPAR</TH>
     <TH align="center" class="sortable" valign="center">Client Slot</TH>
     <TH align="center" class="sortable" valign="center">Backing Device</TH>
     <TH align="center" class="sortable" valign="center">Disk Name</TH>
     <TH align="center" class="sortable" valign="center">Disk Partition [GB]</TH>
     <TH align="center" class="sortable" valign="center">Disk Capacity [GB]</TH>
     <TH align="center" class="sortable" valign="center">Disk Label</TH>
   </TR>
  </thead>
  <tbody>';
    foreach my $server_uid (keys %{$vscsi}){
      if ($server_uid eq "") { next; }
      if (ref ($vscsi->{$server_uid}) eq "ARRAY"){
        foreach my $map (@{$vscsi->{$server_uid}}){
          $map->{Storage}{VirtualDisk}{PartitionSize}= sprintf("%.1f", $map->{Storage}{VirtualDisk}{PartitionSize}) if defined $map->{Storage}{VirtualDisk}{PartitionSize};
          $map->{Storage}{VirtualDisk}{DiskCapacity} = sprintf("%.1f", $map->{Storage}{VirtualDisk}{DiskCapacity}) if defined $map->{Storage}{VirtualDisk}{DiskCapacity};
          if (!defined $map->{Storage}{VirtualDisk}{DiskLabel} || $map->{Storage}{VirtualDisk}{DiskLabel} eq "None"){
            $map->{Storage}{VirtualDisk}{DiskLabel} = "";
          }
          print "<tr>";
          print "<TD>$map->{ServerAdapter}{SystemName}</TD>\n";
          print "<TD>$map->{ServerAdapter}{RemoteLogicalPartitionName}</TD>\n";
          print "<TD>$map->{ServerAdapter}{AdapterName}</TD>\n";
          print "<TD>$map->{ServerAdapter}{VirtualSlotNumber}</TD>\n";
          print "<TD>$map->{Partition}</TD>\n";
          print "<TD>$map->{ClientAdapter}{VirtualSlotNumber}</TD>\n";
          print "<TD>$map->{ServerAdapter}{BackingDeviceName}</TD>\n";
          print "<TD>$map->{Storage}{VirtualDisk}{DiskName}</TD>\n";
          print "<TD>$map->{Storage}{VirtualDisk}{PartitionSize}</TD>\n";
          print "<TD>$map->{Storage}{VirtualDisk}{DiskCapacity}</TD>\n";
          print "<TD>$map->{Storage}{VirtualDisk}{DiskLabel}</TD>\n";
          print "</tr>\n";
        }
      }
    }

    print "
     </tbody>
    </TABLE>";
    print "</div>\n";
    print "</center>";
  } ## end if ( -f "$file_html_vscsi" )a


  if ( -f "$file_npiv_conf" ) {
    print "<center>";
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;

    my $npiv = read_json($file_npiv_conf);
      print '<TABLE style="width:90%;" class="tabconfig tablesorter" data-sortby="1 2">
  <thead>
   <TR>
     <TH align="center" class="sortable" valign="center">Server</TH>
     <TH align="center" class="sortable" valign="center">MapPort</TH>
     <TH align="center" class="sortable" valign="center">Local<br>PartitionID</TH>
     <TH align="center" class="sortable" valign="center">Conn<br>PartitionID</TH>
     <TH align="center" class="sortable" valign="center">Conn<br>Virtual<br>Slot<br>Number</TH>
     <TH align="center" class="sortable" valign="center">Server<br>Virtual<br>Slot<br>Number</TH>
     <TH align="center" class="sortable" valign="center">Location<br>Code</TH>

     <TH align="center" class="sortable" valign="center">Local<br>PartitionID</TH>
     <TH align="center" class="sortable" valign="center">Conn<br>Virtual<br>Slot<br>Number</TH>
     <TH align="center" class="sortable" valign="center">Conn<br>PartitionID</TH>
     <TH align="center" class="sortable" valign="center">Client<br>Virtual<br>SlotNumber</TH>
     <TH align="center" class="sortable" valign="center">WWPNs</TH>

     <TH align="center" class="sortable" valign="center">Phys<br>Port<br>Available<br>Ports</TH>
     <TH align="center" class="sortable" valign="center">Phys<br>Port<br>Total<br>Ports</TH>
     <TH align="center" class="sortable" valign="center">Phys<br>Port<br>Port<br>Name</TH>
     <TH align="center" class="sortable" valign="center">Phys<br>Port<br>WWPN</TH>
     <TH align="center" class="sortable" valign="center">Phys<br>Port<br>Location<br>Code</TH>

   </TR>
  </thead>
  <tbody>';
    foreach my $server_uid (keys %{$npiv}){
      my $server_uid_short = substr($server_uid, 0,5);
      if ($server_uid eq "") { next; }
      if (ref ($npiv->{$server_uid}) eq "ARRAY"){
        foreach my $map (@{$npiv->{$server_uid}}){
          print "<tr>";
          print "<TD>server</TD>\n";
          print "<TD>$map->{ServerAdapter}{MapPort}</TD>\n";
          print "<TD>$map->{ServerAdapter}{LocalPartitionID}</TD>\n";
          print "<TD>$map->{ServerAdapter}{ConnectingPartitionID}</TD>\n";
          print "<TD>$map->{ServerAdapter}{ConnectingVirtualSlotNumber}</TD>\n";
          print "<TD>$map->{ServerAdapter}{VirtualSlotNumber}</TD>\n";
          print "<TD>$map->{ServerAdapter}{LocationCode}</TD>\n";

          print "<TD>$map->{ClientAdapter}{LocalPartitionID}</TD>\n";
          print "<TD>$map->{ClientAdapter}{ConnectingVirtualSlotNumber}</TD>\n";
          print "<TD>$map->{ClientAdapter}{ConnectingPartitionID}</TD>\n";
          print "<TD>$map->{ClientAdapter}{VirtualSlotNumber}</TD>\n";
          print "<TD>$map->{ClientAdapter}{WWPNs}</TD>\n";

          print "<TD>$map->{Port}{AvailablePorts}</TD>\n";
          print "<TD>$map->{Port}{TotalPorts}</TD>\n";
          print "<TD>$map->{Port}{PortName}</TD>\n";
          print "<TD>$map->{Port}{WWPN}</TD>\n";
          print "<TD>$map->{Port}{LocationCode}</TD>\n";
          print "</tr>\n";

        }
      }
    }
    print "
     </tbody>
    </TABLE>";
    print "</div>\n";
    print "</center>";
  } ## end if ( -f "$file_npiv_conf" )

  #Server tabs, show each server ant its config from $det
  @servers = ();
=begin config tabs
  foreach my $det (@details){
   my ($host, $server) = split ("www/", $det);
    ($server, undef) = split ('/detail.html',$server);
    ($host,$server) = split ('/', $server);
    if ( grep( /^$server$/, @servers )) {
      next;
    }
    push(@servers, $server);
    print "<div id=\"tabs-$tab_index\">\n"; $tab_index++;
    my $print_html;
    if ( -f "$det" ) {
      open( FH, "< $det" );
      $print_html = do { local $/; <FH> };
      close(FH);
      print "$print_html";
    }
    print "</div>\n";
  }
=cut

#  print "</div>\n";
  print "</div><br>\n";
  return 1;
  }
} ## end sub print_data_servers

sub print_datastore_table_top {
  print "<div  id=\"tabs\">\n";

  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">TOP</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-2\">IOPS Read</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-3\">IOPS Write</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-4\">Data Read</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-5\">Data Write</a></li>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-6\">Used Space</a></li>\n";
  print "</ul>\n";
  print "<CENTER>";

  print "<div id=\"tabs-1\">\n";
  print "
  <div >
    <center>
      <div style='font-size: 0.8rem; margin-top: 1em'>
      <form method='get' action='/lpar2rrd-cgi/detail.sh' id='dstr-top'>
        <label for='from'>From</label>
        <input type='text' id='fromTime' size='16' name='fromTime'>
        <label for='to'>to</label>
        <input type='text' id='toTime' size='16' name='toTime'>
        <input type='submit' style='font-weight: bold;' id='showvolumes' value='Show top datastores'>
    <input type='hidden' name='dstr-top' value='12'>
      </form>
      </div>
    <div id='volresults'></div>
    </center>
  </div>
  ";
  print "</div>\n";

  $host_url   = "nope";
  $server_url = "nope";
  $lpar_url   = "nope";

  print "<div id=\"tabs-2\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  $item = "dstrag_iopsr";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>\n";

  print "<div id=\"tabs-3\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  $item = "dstrag_iopsw";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>\n";

  print "<div id=\"tabs-4\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  $item = "dstrag_datar";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>\n";

  print "<div id=\"tabs-5\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";
  $item = "dstrag_dataw";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>\n";

  print "<div id=\"tabs-6\">\n";

  # datastore USED aggregation
  print "<table border=\"0\">\n";
  print "<tr>";
  $item = "dstrag_used";
  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr></table>";
  print "</div>\n";

  print "</div><br>\n";
  return;
} ## end sub print_datastore_table_top

sub print_datastore_top {
  my ( $sunix, $eunix ) = @_;

  my @all_cluster_name_files = <$wrkdir/*/*/my_cluster_name>;
  # print join("\n",@files)."\n";
  # /home/lpar2rrd/lpar2rrd/data/10.22.11.9/10.22.11.10/my_cluster_name
  # /home/lpar2rrd/lpar2rrd/data/192.168.1.124/pavel.lpar2rrd.com/my_cluster_name

  $sunix =~ s/sunix=//;
  $eunix =~ s/eunix=//;
  print "<br><center><table class=\"tbl2leftotherright tablesorter\" data-sortby=\"4\"><thead><tr> \n
  <th class=\"sortable\" align=\"center\">   VMware alias   </th> \n
  <th class=\"sortable\" align=\"center\">   datastore   </th> \n
  <th class=\"sortable\" align=\"center\">   cluster   </th> \n
  <th class=\"sortable\" align=\"center\">   IO read   <br>   <span>IOps</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   IO write   <br>   <span>IOps</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   read   <br><span>MB/sec</span></th> \n
  <th class=\"sortable\" align=\"center\">   write   <br>   <span>MB/sec</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   latency read<br>   <span>Millisec</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   latency write<br>   <span>Millisec</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   used   <br>   <span>GB</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   free   <br>   <span>GB</span>   </th> \n
  <th class=\"sortable\" align=\"center\">   provisioned   <br>   <span>GB</span>   </th> \n
  </tr></thead><tbody>\n";

  my $rrdtool = $ENV{RRDTOOL};

  # start RRD via a pipe
  RRDp::start "$rrdtool";

  # find all datastores
  my @files = <$wrkdir/vmware_*/datastore_datacenter*/*.rrs>;
  foreach my $file (@files) {

    # avoid old files which do not exist in the period
    my $rrd_upd_time = ( stat("$file") )[9];

    # print STDERR "\$file $file $rrd_upd_time $sunix\n";
    if ( $rrd_upd_time < $sunix ) {

      #if ( $type_sam !~ m/x/ ) {
      next;    # avoid that for historical reports
               # --> not necessary as $req_time = 0
               #}
    } ## end if ( $rrd_upd_time < $sunix)

    # $human_ds_name = find_human_ds_name($server,$host,$lpar); # this is in detail_graph_cgi.pl
    my $file_name      = basename($file);
    my $datastore_uuid = $file_name;
    $datastore_uuid =~ s/\.rrs//;
    my $datastore_path = $file;
    $datastore_path =~ s/$file_name//;

    # print STDERR "\$file $file \$datastore_uuid $datastore_uuid\n";

    my $datastore_name = $datastore_uuid;
    my @names          = <$datastore_path*.$datastore_uuid>;

    # print STDERR "\@names ,@names, $datastore_path*.$datastore_uuid\n";
    if ( @names > 0 && $names[0] ne "" ) {
      $datastore_name = basename( $names[0] );

      # remove extension
      $datastore_name =~ s/\.[^.]+$//;
    } ## end if ( @names > 0 && $names...)

    # print STDERR "\$datastore_name $datastore_name\n";

    my $datacenter_name = "";
    my @dc_name         = <$datastore_path*.dcname>;

    # print STDERR "3440 detail-cgi.pl \@dc_name @dc_name ,$dc_name[0],\n";
    if ( defined $dc_name[0] && $dc_name[0] ne "" ) {
      $datacenter_name = basename( $dc_name[0] );
      $datacenter_name =~ s/\.dcname//;
    }

    # to get cluster name: get datastore VM list table > get ESXi mounted > from ESXI get my cluster name

    my @vm_table = <$datastore_path/$datastore_uuid.html>;
    #print STDERR 7028 \@vm_table @vm_table\n";
    my $cluster_name = "";
    if (defined $vm_table[0] && $vm_table[0] ne "" && open( FH, "< $vm_table[0]" )) {
      my @table = <FH>;
      my $esxi = $table[2];
      close FH;
      if (defined $esxi && $esxi ne "") {
        (undef,$esxi) = split ("server=",$esxi);
        if (defined $esxi && $esxi ne "") { # not actively used datastore
          ($esxi,undef) = split ("lpar=",$esxi);
          $esxi = substr $esxi, 0, -1;
          # print "7037 \$esxi $esxi\n";
          my @cluster_name_file = grep { /$esxi/ } @all_cluster_name_files;
          if (defined $cluster_name_file[0] && $cluster_name_file[0] ne "" && open( FH, "< $cluster_name_file[0]" )) {
            my $clust_name = <FH>;
            close FH;
            # cluster_New Cluster|Hosting
            ($clust_name,undef) = split (/\|/,$clust_name);
            chomp $clust_name;
            $clust_name =~ s/cluster_//;
            if (defined $clust_name && $clust_name ne "") {
              $cluster_name = $clust_name;
            }
          }
        }
      }
    }

    # vmware alias name
    my $vmware_alias = "no VMware alias";
    my $alias_file   = $datastore_path;
    $alias_file =~ s/[^\/]*\/$//;

    # print STDERR "\$alias_file $alias_file\n";
    $alias_file .= "vmware_alias_name";
    if ( -f $alias_file && open( FH, "< $alias_file" ) ) {
      my $name = <FH>;
      close FH;
      chomp $name;
      ( undef, $name ) = split( /\|/, $name );
      $vmware_alias = $name if $name ne "";
    } ## end if ( -f $alias_file &&...)

    #print STDERR "\$vmware_alias $vmware_alias\n";

    my $rrd = $file;
    $rrd =~ s/:/\\:/g;
    RRDp::cmd qq(graph "tmp/name.png"
     "--start" "$sunix"
     "--end" "$eunix"
     "DEF:used=$rrd:Disk_used:AVERAGE"
     "DEF:free=$rrd:freeSpace:AVERAGE"
     "DEF:prov=$rrd:Disk_provisioned:AVERAGE"
     "PRINT:used:AVERAGE: %3.2lf"
     "PRINT:free:AVERAGE: %3.2lf"
     "PRINT:prov:AVERAGE: %3.2lf"
    );
    my $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error("Rrdtool error : $$answer");
      next;
    }
    my $aaa = $$answer;

    # print "$answer\n";
    # print "$$answer\n";
    $aaa =~ s/NaNQ|NaN|nan/0/g;
    ( undef, my $used, my $free, my $prov ) = split( "\n", $aaa );
    chomp $used;
    chomp $free;
    chomp $prov;
    $used = sprintf( '%.0f', $used / 1024 / 1024 );
    $free = sprintf( '%.0f', $free / 1024 / 1024 );
    $prov = sprintf( '%.0f', $prov / 1024 / 1024 );

    # read, write, IOps
    $rrd =~ s/rrs$/rrt/;
    next if !-f $rrd || ( stat $rrd )[7] == 0;    # no exist -> no care
    RRDp::cmd qq(graph "tmp/name.png"
     "--start" "$sunix"
     "--end" "$eunix"
     "DEF:read=$rrd:Datastore_read:AVERAGE"
     "DEF:writ=$rrd:Datastore_write:AVERAGE"
     "DEF:reav=$rrd:Datastore_ReadAvg:AVERAGE"
     "DEF:wrav=$rrd:Datastore_WriteAvg:AVERAGE"
     "PRINT:read:AVERAGE: %3.2lf"
     "PRINT:writ:AVERAGE: %3.2lf"
     "PRINT:reav:AVERAGE: %3.2lf"
     "PRINT:wrav:AVERAGE: %3.2lf"
    );
    $answer = RRDp::read;
    if ( $$answer =~ "ERROR" ) {
      error("Rrdtool error : $$answer");
      next;
    }
    $aaa = $$answer;

    # print "$answer\n";
    # print "$$answer\n";
    $aaa =~ s/NaNQ|NaN|nan/0/g;
    ( undef, my $read, my $writ, my $reav, my $wrav ) = split( "\n", $aaa );
    chomp $read;
    chomp $writ;
    chomp $reav;
    chomp $wrav;
    $read = sprintf( '%.2f', $read / 1024 );
    $writ = sprintf( '%.2f', $writ / 1024 );
    $reav = sprintf( '%.0f', $reav );
    $wrav = sprintf( '%.0f', $wrav );

    # latency
    my $lat_read = "-";
    my $lat_writ = "-";
    $rrd =~ s/rrt$/rru/;
    if ( -f $rrd && ( stat $rrd )[7] > 0 ) {    # no exist -> no care
      RRDp::cmd qq(graph "tmp/name.png"
       "--start" "$sunix"
       "--end" "$eunix"
       "DEF:read=$rrd:Dstore_readLatency:AVERAGE"
       "DEF:write=$rrd:Dstore_writeLatency:AVERAGE"
       "PRINT:read:AVERAGE: %3.2lf"
       "PRINT:write:AVERAGE: %3.2lf"
      );
      $answer = RRDp::read;
      if ( $$answer =~ "ERROR" ) {
        error("Rrdtool error : $$answer");
        next;
      }
      $aaa = $$answer;

      # print "$answer\n";
      # print "$$answer\n";
      $aaa =~ s/NaNQ|NaN|nan/0/g;
      ( undef, $lat_read, $lat_writ ) = split( "\n", $aaa );
      chomp $lat_read;
      chomp $lat_writ;
      $lat_read = sprintf( '%.2f', $lat_read );
      $lat_writ = sprintf( '%.2f', $lat_writ );
    } ## end if ( -f $rrd && ( stat...))

    # my @d_path = split("\/",$file);
    print "<tr><td>$vmware_alias</td><td><A HREF=\"/lpar2rrd-cgi/detail.sh?host=$datacenter_name&server=$vmware_alias&lpar=$datastore_name&item=datastore&entitle=0&none=none\">$datastore_name</td><td>$cluster_name</td><td>$reav</td><td>$wrav</td><td>$read</td><td>$writ</td><td>$lat_read</td><td>$lat_writ</td><td>$used</td><td>$free</td><td>$prov</td></tr>\n";

  } ## end foreach my $file (@files)
  print "</tbody></table>\n";

  # close RRD pipe
  RRDp::end;
} ## end sub print_datastore_top

# it returns first wpar if there is more than 1 wpar on that lpar
# for 1 wpar makes no sense to create aggregated graph

sub find_wpar {
  my $lpar_path = shift;
  my $wpar      = shift;

  if ( $wpar == 1 ) {

    # it is only for global LPARs, not for WPARs itself
    return " ";
  }

  opendir( DIR, $lpar_path ) || return " ";
  my @lpar_dir = readdir(DIR);
  closedir(DIR);

  my $number = 0;
  foreach my $lpar_item (@lpar_dir) {
    chomp($lpar_item);
    if ( -d "$lpar_path/$lpar_item" && -f "$lpar_path/$lpar_item/cpu.mmm" ) {
      $number++;
      if ( $number == 2 ) {
        return $lpar_item;
      }
    } ## end if ( -d "$lpar_path/$lpar_item"...)
  } ## end foreach my $lpar_item (@lpar_dir)
  return " ";
} ## end sub find_wpar

sub timing_debug
{
  my $text     = shift;

  if ( defined $ENV{LPAR2RRD_UI_TIME_DEBUG} ) {
    my $act_time = localtime();
    print STDERR "DEBUG: $0 : $$ : $act_time : $text\n";
  }
  return 1;
}

sub print_hitachi_adapters {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  print "<CENTER>";
  print "<div id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">IO</a></li>\n";
  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "legend" );
  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "</div><br>\n";

  return 1;
} ## end print_hitachi_adapters

sub print_hitachi_adapters_totals {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;

  print "<CENTER>";
  print "<div id=\"tabs\">\n";
  print "<ul>\n";
  print "  <li class=\"$tab_type\"><a href=\"#tabs-1\">Data</a></li>\n";
  print "</ul>\n";

  print "<div id=\"tabs-1\">\n";
  print "<table border=\"0\">\n";
  print "<tr>";

  print_item( $host_url, $server_url, $lpar_url, $item, "d", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "w", $type_sam, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print "</tr>\n<tr>\n";
  print_item( $host_url, $server_url, $lpar_url, $item, "m", $type_sam,      $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );
  print_item( $host_url, $server_url, $lpar_url, $item, "y", $type_sam_year, $entitle, $detail_yes, "norefr", "star", 1, "nolegend" );

  print "</tr>";
  print "</table>";
  print "</div>\n";

  print "</div><br>\n";

  return 1;
}


sub print_this_html_file {
  my ( $host_url, $server_url, $lpar_url, $item, $entitle ) = @_;
  my $file = shift;
  my $html;
  if ( -f "$file" ) {
    open( FH, "< $file" );
    $html = do { local $/; <FH> };
    close(FH);
    print "$html";
  }
}


sub is_host_rest{
  my $host=shift;
  my %hosts = %{ HostCfg::getHostConnections("IBM Power Systems") };
  foreach my $alias (keys %hosts){
    if ($host eq $hosts{$alias}{host}){
      if (defined $hosts{$alias}{auth_api} && $hosts{$alias}{auth_api}){
        return 1;
      }
      else{
        return 0;
      }
      return $hosts{$alias}{auth_api};
    }
 }
}

sub is_any_host_rest {
  if (defined $ENV{DEMO} && $ENV{DEMO} == 1){
    return 1;
  }
  my %hosts = %{ HostCfg::getHostConnections("IBM Power Systems") };
  my $result=0;
  foreach my $alias (keys %hosts){
    if (defined $hosts{$alias}{auth_api} && $hosts{$alias}{auth_api}){
      $result=1;
    }
  }
  return $result;
}

sub read_json{
  my $src=shift;
  if (!defined $src || $src eq ""){
    print "Path to json file(\"$src\") not defined, can't read it in function read_json()\n";
    return -1;
  }
  my $read;
  my $rawcfg;       # helper for file reading
  # read from JSON file
  if ( open( CFG, "$src" ) ) {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
    if ($rawcfg) {
      $read = decode_json($rawcfg);
    }
    close CFG;
    return $read;
  }
  else {
    # error handling
    error (__FILE__.":".__LINE__."Cannot read $src as .json file\n");
  }
}

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( $digit eq '' ) {
    return 0;
  }
  if ( $digit eq 'U' ) {
    return 1;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit
