use warnings;
use strict;
use Date::Parse;
use File::Basename;
use RRDp;
use POSIX qw(strftime);
use File::Glob ':glob';
use CGI::Carp qw(fatalsToBrowser);
use JSON qw(decode_json);
use Data::Dumper;

use XenServerDataWrapper;
use XenServerGraph;
use XenServerMenu;

use OVirtDataWrapper;
use OVirtGraph;

my $alarm       = 0;
my $graph_path  = "";
my $graph_hours = 0;           # $email_graph variable from alrt.pl --> EMAIL_GRAPH from alert.cfg
my $graph_un    = 10;          # if 0 then do not graph UN values
my $color_un    = "#d0d0d0";
my $hmc_iops_divide = 1;

if ( $#ARGV > 0 && $ARGV[0] =~ m/alarm/ ) {

  # it is called from alrt.pl to generate a graph for alarming
  $graph_path  = $ARGV[1];
  $graph_hours = $ARGV[2];
  $alarm       = 1;

  #print "Graph creation : detail-graph-cgi.pl: $ARGV[0] $graph_path \n";
} ## end if ( $#ARGV > 0 && $ARGV...)

use lib "../bin";
use ACL;    # use module ACL.pm
my $useacl = ACL::useACL;

# new system keeping VMs
my $all_vmware_VMs = "vmware_VMs";
my $all_hyperv_VMs = "hyperv_VMs";

#
### global definitions
#
my $pic_col_orig         = "F7FCF8";              # if need to change in $cmd strings
my $DEBUG                = $ENV{DEBUG};
my $errlog               = $ENV{ERRLOG};
my $basedir              = $ENV{INPUTDIR};
my $bindir               = $ENV{BINDIR};
my $rrdtool              = $ENV{RRDTOOL};
my $pic_col              = $ENV{PICTURE_COLOR};
my $step                 = $ENV{SAMPLE_RATE};
my $STEP_HEA             = $ENV{STEP_HEA};
my $hea                  = $ENV{HEA};
my $width                = 400;
my $height               = 150;
my $width_trend          = $width * 2 + 128;      # 93 pixels for 1 X axiss legend and Tobi promo + 50 table space
my $detail_multipication = 3;                     # detail pop up will be 3 times bigger
my $entitle              = 0;                     # default, show entitlement
my $filter_max_lansan    = 100000000000;          # 100GB/sec, filter values above as this is most probably caused by a counter reset (lpar restart)
if ( defined $ENV{FILTER_MAX_LANSAN} ) { $filter_max_lansan = $ENV{FILTER_MAX_LANSAN}; }
my $filter_max_iops = 1000000;                    # 1M IOPS filter values above as this is most probably caused by a counter reset (lpar restart)
if ( defined $ENV{FILTER_MAX_IOPS} ) { $filter_max_iops = $ENV{FILTER_MAX_IOPS}; }
my $filter_max_paging = 500000000;                # 500MB/sec, filter values above as this is most probably caused by a counter reset (lpar restart)
if ( defined $ENV{FILTER_MAX_PAGING} ) { $filter_max_paging = $ENV{FILTER_MAX_PAGING}; }
my $cpu_max_filter = 25;                         # max 2.5k peak in % is allowed (in fact it cannot by higher than 2k% now when 1 logical CPU == 0.05 entitlement)
if ( defined $ENV{CPU_MAX_FILTER} ) { $cpu_max_filter = $ENV{CPU_MAX_FILTER}; }
my $request_method = "GET";
if ( defined $ENV{REQUEST_METHOD} ) { $request_method = $ENV{REQUEST_METHOD}; }

my $tmpdir = "$basedir/tmp";
if ( defined $ENV{TMPDIR_LPAR} ) {
  $tmpdir = $ENV{TMPDIR_LPAR};
}

my $delimiter = "XORUX";                          # this is for rrdtool print lines for clickable legend

my $rrd_legend = "table";
if ( -f "$tmpdir/xorux_test_orig.txt" ) {
  $rrd_legend = "orig";
}
if ( -f "$tmpdir/xorux_test_both.txt" ) {
  $rrd_legend = "both";
}
if ( -f "$tmpdir/xorux_test_table.txt" ) {
  $rrd_legend = "table";
}
if ( !defined $ENV{HTTP_USER_AGENT} ) {
  $rrd_legend = "orig";
}
else {
  if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
  }
  else {
    if ( $ENV{HTTP_USER_AGENT} =~ /MSIE 8.0/ ) {
      $rrd_legend = "orig";
    }
  }
} ## end else [ if ( !defined $ENV{HTTP_USER_AGENT...})]

### graph or graphv
my $graph_cmd = "graph";
if ( -f "$tmpdir/graphv" ) {
  $graph_cmd = "graphv";    # if exists - call this function
}

### right-axis option (rrdtool <1.2015 do not have it)
my $yes_right_axis = 0;
if ( -f "$tmpdir/graph-right-axis" ) {
  $yes_right_axis = 1;
}

my $png_end_heading = "\n";    # this or graphv dimensions
my $html_heading = "";

my $NMON  = "--NMON--";
my $AS400 = "--AS400--";

###  font definitions
my $font_def_normal   = "--font=DEFAULT:7:";
my $font_tit_normal   = "--font=TITLE:9:";
my $font_def_dashbo   = "--font=DEFAULT:6:";
my $font_tit_dashbo   = "--font=TITLE:7:";
my $font_def_popup    = "--font=DEFAULT:10:";
my $font_tit_popup    = "--font=TITLE:13:";
my $font_def_hist     = "--font=DEFAULT:8:";
my $font_tit_hist     = "--font=TITLE:10:";
my $font_def_reporter = "--font=DEFAULT:7:";
my $font_tit_reporter = "--font=TITLE:7:";

my $font_def = $font_def_normal;
my $font_tit = $font_tit_normal;

my $wrkdir = $basedir . "/data";

if ( defined $ENV{NMON_EXTERNAL} ) { $wrkdir .= $ENV{NMON_EXTERNAL} }

if ( -f "$bindir/premium.pl" ) {
  require "$bindir/premium.pl";
}
else {
  require "$bindir/standard.pl";
}

# if this file exists & contains UNIX time, graph CPU_ready from this time, otherwise not limited
my $CPU_ready_time_file = "$wrkdir/" . "$all_vmware_VMs/" . "CPU_ready_time.txt";

#$DEBUG=2;
open( OUT, ">> $errlog" ) if $DEBUG == 2;

# get QUERY_STRING
use Env qw(QUERY_STRING);

timing_debug($QUERY_STRING);

#$QUERY_STRING .= ":.";
#print STDERR "140 detail-graph-cgi.pl \$QUERY_STRING $QUERY_STRING\n"; # if $DEBUG == 2 ;
#print STDERR "$QUERY_STRING\n\n\n";

my @keep_color_mem = "";
my @keep_color_ams = "";

################ get URL parameters (could be GET or POST) and return them as hash refference
#                multiple values with same name create an array
sub getURLparams {
  my ( $buffer, @pairs, $pair, $name, $value, %PAR );

  if ( defined $request_method ) {
    if ( lc $request_method eq "post" ) {
      read( STDIN, $buffer, $ENV{'CONTENT_LENGTH'} );
    }
    else {
      $buffer = $ENV{'QUERY_STRING'};
    }

    # Split information into name/value pairs
    @pairs = split( /&/, $buffer );
    foreach $pair (@pairs) {
      ( $name, $value ) = split( /=/, $pair );
      $value =~ tr/+/ /;
      $value =~ s/%(..)/pack("C", hex($1))/eg;
      if ( exists $PAR{$name} ) {
        if ( ref $PAR{$name} eq "ARRAY" ) {
          push @{ $PAR{$name} }, $value;
        }
        else {
          my $firstVal = $PAR{$name};
          $PAR{$name} = [];
          push @{ $PAR{$name} }, $firstVal;
          push @{ $PAR{$name} }, $value;
        }
      }
      else {
        $PAR{$name} = $value;
      }
    }
    return \%PAR;
  }
  else {
    return 0;
  }
} ## end sub getURLparams

sub urlencode {
  my $s = shift;
  $s =~ s/([^a-zA-Z0-9!~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
  return $s;
} ## end sub urlencode

sub urldecode {
  my $s = shift;
  $s =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
  $s =~ s/\+/ /g;
  return $s;
} ## end sub urldecode

my %params;   # hash containing URL parameters. Use like this: $params{server}

my $qs = getURLparams;
if ( $qs ) {
  %params = %{ $qs };
  # use Data::Dumper;
  # print STDERR "QS: ". $ENV{'QUERY_STRING'} . "\n";
  # print STDERR Data::Dumper->Dump( [ \%params ], [ qw(*params) ] );
}
#print STDERR Dumper \%params;

# assign values from %params for back compatibility
my $host       = $params{host};
my $server     = $params{server};
my $lpar       = $params{lpar};
my $item       = $params{item};
my $time       = $params{time};
my $type_sam   = $params{type_sam};
my $detail     = $params{detail};
my $upper      = $params{upper};
my $amax       = $params{entitle};
my $start_unix = $params{sunix};
my $end_unix   = $params{eunix};
my $height_new = $params{height};
my $width_new  = $params{width};
my $none       = $params{none};

$upper = "" if !defined $upper;

# print STDERR "156 detail-graph-cgi.pl $host:$server:$lpar:$item:$time:$type_sam:$detail:$upper:$amax:$start_unix:$end_unix:$height_new:$width_new\n" ;

my $heading_png = "Content-type: image/png\nCache-Control: max-age=60, must-revalidate\n";
my $first_error = 0;

if ( $alarm == 0 ) {

  # only for CGI-BIN purposes
  if ( $item =~ m/xport/ ) {

    # It should be here to do not influence normal report when XML is not in Perl
    require "$bindir/xml.pl";

    # use XML::Simple; --> it has to be in separete file
    print "Content-type: application/octet-stream\n";
  } ## end if ( $item =~ m/xport/)
  else {
    # normal picture stuff
    #    print "Content-type: image/png\n";
    #    print "Cache-Control: max-age=60, public\n\n"; # workaround for caching on Chrome
    #    print "Cache-Control: max-age=60, must-revalidate\n"; # workaround for caching on Chrome
  } ## end else [ if ( $item =~ m/xport/)]
} ## end if ( $alarm == 0 )

# no URL decode here, as it goes immediately into URL again
# --PH , yes URL decode must be here!!!

# workaround for slash in server name
# for incoming from ext-nmon or from extern query call
my $slash_alias = "∕";    #hexadec 2215 or \342\210\225
$server =~ s/slashslash/$slash_alias/g;

# $host =~ s/\+/ /g; # '+' can be in name since 4.81

# $server =~ s/\+/ /g; # '+' can be in name since 4.81
$server =~ s/\//&&1/g;      #slash workaround
my $acl_item = $lpar;

# $lpar =~ s/\+/ /g; # '+' can be in name since 4.81
$lpar =~ s/\//&&1/g;        #slash workaround
$lpar =~ s/ \[.*\]//g;      # remove alias info
my $wpar = 0;
if ( $lpar =~ m/--WPAR--/ ) {
  $wpar = 1;
}
$lpar =~ s/--WPAR--/\//g;    # WPAR delimiter replace

if ( defined $time && isdigit($time) == 1 ) {

  # since 4.66 there is passed step through hist reports
  $step = $time;
}
if ( defined($type_sam) && !$type_sam eq '' ) {
  $type_sam = 'm' if $type_sam eq 'x';    # came from global hist rep
}

if ( $time !~ m/^d$/ ) {

  # slow down everything except daily graph to let it show as the first one
  select( undef, undef, undef, 0.15 );
}

#my $no_legend = "COMMENT: "; # do not use "COMMENT:\" \"";
my $no_legend = "--interlaced";    # just nope string , it is deprecated anyway

#my $full_size="--full-size-mode";
#--no-gridfit
if ( defined $detail && isdigit($detail) == 0 ) {
  $detail = 0;
}

if ( $detail == 2 ) {

  # dashboard size
  if ( defined $ENV{DASHB_RRDHEIGHT} ) {
    $height = $ENV{DASHB_RRDHEIGHT};
  }
  else {
    $height = 50;
  }
  if ( defined $ENV{DASHB_RRDWIDTH} ) {
    $width = $ENV{DASHB_RRDWIDTH};
  }
  else {
    $width = 120;
  }
  $font_def  = $font_def_dashbo;
  $font_tit  = $font_tit_dashbo;
  $no_legend = "--no-legend";
} ## end if ( $detail == 2 )

if ( $detail == 1 ) {
  $font_def = $font_def_popup;
  $font_tit = $font_tit_popup;
}

my $super_detail = 0;    # instead of detail 3
if ( $detail == 3 ) {
  $super_detail = 3;

  # customer size
  if ( defined $ENV{CUSTOMER_RRDHEIGHT} ) {
    $height = $ENV{CUSTOMER_RRDHEIGHT};
  }
  else {
    $height = 150;
  }
  if ( defined $ENV{CUSTOMER_RRDWIDTH} ) {
    $width = $ENV{CUSTOMER_RRDWIDTH};
  }
  else {
    $width = 550;
  }

  #$font_def =  $font_def_dashbo;
  #$font_tit =  $font_tit_dashbo;
  $no_legend = "--no-legend";
  $detail    = 2;               # same as dashboard
} ## end if ( $detail == 3 )

if ( $detail == 0 ) {
  if ( defined $ENV{CUSTOMER_RRDHEIGHT} ) {
    $height = $ENV{CUSTOMER_RRDHEIGHT};
  }
  if ( defined $ENV{CUSTOMER_RRDWIDTH} ) {
    $width = $ENV{CUSTOMER_RRDWIDTH};
  }
}


#my $disable_rrdtool_tag = "COMMENT: ";
my $disable_rrdtool_tag = "--interlaced";    #just nope string, it is deprecated anyway
my $rrd_ver             = $RRDp::VERSION;
if ( isdigit($rrd_ver) && $rrd_ver > 1.34 ) {

  # graphv is supported since rrdtool-1.3rc5
  $disable_rrdtool_tag = "--disable-rrdtool-tag";
  $width_trend         = $width_trend - 15;         # trends must be wider due to 2 x promo
} ## end if ( isdigit($rrd_ver)...)

# entitle, show (0) or do not show (1) LPAR entitlement (DTE Energy extension)
if ( $time =~ m/entitle=/ ) {
  $entitle = $time;                                 # entitle is on $time possition for graph details where upper is not used
}
if ( $upper =~ m/entitle=/ ) {
  $entitle = $upper;                                # just to be sure
}
if ( defined($amax) && !$amax eq '' && $amax =~ m/entitle=/ ) {
  $entitle = $amax;                                 # just to be sure
}
# $entitle =~ s/entitle=//;
$entitle = $amax;

if (!defined $entitle || $entitle eq ""){
  $entitle = 0;
}

my $name_out = "lpar2rrd-$host-$server-$lpar-$item-$time-$type_sam-$$.png";
$name_out =~ s/;//g;                                # ";" cannot be in the path
$name_out =~ s/\///g;                               # "/" cannot be in the path
$name_out =~ s/^/\/var\/tmp\//;
if ( $host eq '' || $server eq '' || $lpar eq '' ) {
  error( "One of passed args: hmc/server/lpar is null ($QUERY_STRING) " . __FILE__ . ":" . __LINE__ );
  exit(1);
}

###   if VMWARE or HYPER-V
my $vmware = 0;
my %unix_commands = (); # used for jobs for transport sometimes very long cmds to html legend table

# print STDERR "397 detail-graph-cgi.pl test vmware or hyperv for $wrkdir / $server / $host/ \$item $item   (vmware.txt or hyperv.txt)\n";
$vmware = 1 if ( -f "$wrkdir/$server/$host/vmware.txt" );
$vmware = 1 if $server eq "vmware_VMs";
if ( !$vmware ) {
  my @files = glob "$wrkdir/$server/*/vmware.txt";
  $vmware = 1 if ( defined $files[0] && ( $files[0] ne "" ) && index( $files[0], "vmware.txt" ) != -1 );

  # print STDERR "295 files tested @files\n";
} ## end if ( !$vmware )
my $hyperv = 0;
if ( !$vmware ) {    # then try if hyperv
  # $hyperv = 1 if ( -f "$wrkdir/$server/$host/hyperv.txt" );
  $hyperv = 1 if $server =~ /^windows/;
  $server =~ s/\&\&1/\//;
}

# workaround to avoid other tech's (=unwanted) branches, although Power, Solaris and perhaps others still remain
my $hitachi   = 0;
my $xenserver = 0;
my $ovirt     = 0;

if ( $server =~ /^Hitachi$/ ) {
  $hitachi = 1;
  $vmware = $hyperv = 0;
}
elsif ( $host =~ /^XenServer$/ ) {
  $xenserver = 1;
  $vmware = $hyperv = 0;
}
elsif ( $host =~ /^oVirt$/ ) {
  $ovirt = 1;
  $vmware = $hyperv = 0;
}

# item as4job_xxx is called with none=<rrd filename of job to graph in detail>
# not used with historical reports
my $as4job;
if ( $item =~ /^as4job/ ) {
  $as4job     = $none;
  $start_unix = "";
  $as4job =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
} ## end if ( $item =~ /^as4job/)

# item powlin_job is called with none=<job pid to graph in detail>
# not used with historical reports
my $powlin_job;
if ( $item =~ /^powlin_job/ ) {
  $powlin_job = $none;
  $start_unix = "";
  $powlin_job =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
} ## end if ( $item =~ /^powlin_job/)

# item powlin_mem is called with none=<job pid to graph in detail>
# not used with historical reports
my $powlin_mem;
if ( $item =~ /^powlin_mem/ ) {
  $powlin_mem = $none;
  $start_unix = "";
  $powlin_mem =~ s/\%([A-Fa-f0-9]{2})/pack('C', hex($1))/seg;
} ## end if ( $item =~ /^powlin_mem/)

# parameters called only from historical reports, normally they are NULL
if ( !$start_unix eq '' ) {
  $start_unix =~ s/sunix=//;
}
if ( !$end_unix eq '' ) {
  $end_unix =~ s/eunix=//;
}
if ( !$height_new eq '' && ($detail == 0 || $detail == 9 || $detail == 7 )) {
  $height_new =~ s/height=//;
  if ( isdigit($height_new) > 0 ) {
    $height = $height_new;    #replace HEIGHT by passed one
  }
} ## end if ( !$height_new eq ''...)
if ( !$width_new eq '' && ($detail == 0 || $detail == 9 || $detail == 7 ) ) {
  $width_new =~ s/width=//;
  if ( isdigit($width_new) > 0 ) {
    $width = $width_new;      #replace WIDTH by passed one
  }
} ## end if ( !$width_new eq ''...)

if ( $alarm == 1 ) {
  $name_out = $graph_path;
}

if ( $detail == 1 ) {
  $width       = $width * $detail_multipication;
  $width_trend = $width_trend * $detail_multipication;
  $height      = $height * $detail_multipication;
}
if ( $detail < 2 ) {
  if ( defined $height_new && $height_new ne '' && defined $width_new && $width_new ne '' ) {
    $height_new =~ s/height=//;
    $width_new =~ s/width=//;
    if ( isdigit($height_new) > 0 && isdigit($width_new) > 0 ) {
      $height = $height_new;    #replace HEIGHT by passed one
      $width  = $width_new;     #replace WIDTH by passed one
                                # according width change fonts
      if ( $width < 800 ) {     # same as hist rep
        $font_def_popup = $font_def_hist;
        $font_tit_popup = $font_tit_hist;
        $font_def       = $font_def_hist;
        $font_tit       = $font_tit_hist;
      } ## end if ( $width < 800 )
    } ## end if ( isdigit($height_new...))
  } ## end if ( defined $height_new...)
} ## end if ( $detail < 2 )

# detail for graphs created by Reporter
if ( $detail == 10 ) {
  $font_def = $font_def_reporter;
  $font_tit = $font_tit_reporter;
  if ( defined $height_new && isdigit($height_new) && defined $width_new && isdigit($width_new) ) {
    $height_new  =~ s/height=//;
    $width_new   =~ s/width=//;
    $height      = $height_new;
    $width       = $width_new;
    $width_trend = $width;
  }
}

### test ACL ###

if ( $useacl && !$vmware && !$ovirt && !$xenserver && !defined $ENV{NMON_EXTERNAL} && $alarm != 1 ) {    #NMON grapher if always free, the same with allerting

  my $acl_source = $server;
  my $acl_platform;
  my $acl_subsys;
  if ( $item =~ /^custom/ ) {
    $acl_source = $lpar;
    $acl_subsys = "CUSTOM";
  }
  elsif ( $vmware ) {
    $acl_subsys   = "VM";
    $acl_platform = "VMWARE";
  }
  elsif ($ovirt) {
    $acl_platform = "OVIRT";
    $acl_subsys   = "OVIRTVM";
  }
  elsif ($xenserver) {
    $acl_platform = "XENSERVER";
    $acl_subsys   = "XENVM";
  }
  elsif ( $server =~ /--unknown$/ || $host eq "no_hmc" ) {
    $acl_source   =~ s/--unknown//;
    $acl_platform = "UNMANAGED";
    $acl_subsys   = "SERVER";
  }
  else {
    $acl_platform = "POWER";
    $acl_subsys   = "LPAR";
    if ( $item  =~ /^pool|^trendpool|^shpool|^trendshpool|^poolagg|^lparagg/ ) {
      $acl_item    = $lpar;
      $acl_subsys  = "POOL";
    }
  }
  if ( $acl_item =~ /--NMON--$/ ) {
    $acl_item =~ s/--NMON--//;
  }


  if ( !ACL::isAdmin ) {
    if ( !ACL::canShow( $acl_platform, "$acl_subsys", "$acl_source", "$acl_item" ) ) {    #  ACL::canShow vrací 0 jestliže nevyhovuje ani jedna z ACL definicí
                                                                              #print STDERR "$cust_group, $server, $lpar, $item, test ACL \n";
      error( "ACL: not allowed for user $ENV{REMOTE_USER} $acl_platform, $acl_subsys, $acl_source, $acl_item, $item " . __FILE__ . ":" . __LINE__ );
      exit(1);
    }
  } ## end if ( !ACL::isAdmin )
} ## end if ( $useacl && !$vmware...)

# start RRD via a pipe
RRDp::start "$rrdtool";

timing_debug("rrdtool start");

# keep here green - yellow - red - blue ...
my @color = ( "#FF0000", "#0000FF", "#C0C0C0", "#FFFF00", "#00FFFF", "#FFA500", "#00FF00", "#808080", "#FF00FF", "#800080", "#FDD017", "#0000A0", "#3BB9FF", "#008000", "#800000", "#ADD8E6", "#F778A1", "#800517", "#736F6E", "#F52887", "#C11B17", "#5CB3FF", "#A52A2A", "#FF8040", "#2B60DE", "#736AFF", "#1589FF", "#98AFC7", "#8D38C9", "#307D7E", "#F6358A", "#151B54", "#6D7B8D", "#FDEEF4", "#FF0080", "#F88017", "#2554C7", "#FFF8C6", "#D4A017", "#306EFF", "#151B8D", "#9E7BFF", "#EAC117", "#E0FFFF", "#15317E", "#6C2DC7", "#FBB917", "#FCDFFF", "#15317E", "#254117", "#FAAFBE", "#357EC7", "#4AA02C", "#38ACEC" );
my $color_max = 53;    # 0 - 53 is 54 colors

my @table_out = "";
my $ret = create_graph( $host, $server, $lpar, $item, $time, $name_out, $type_sam, $upper, $alarm, $graph_hours, $amax, $start_unix, $end_unix, $entitle );

# close RRD pipe
RRDp::end;

if ( $alarm == 0 && $ret == 1 && $item !~ m/xport/ ) {
  timing_debug("print_png");
  print_png($name_out);
}

close(OUT) if $DEBUG == 2;
timing_debug("exit");

exit(0);

sub create_graph {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $item        = shift;
  my $time        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $upper       = shift;
  my $alarm       = shift;
  my $graph_hours = shift;
  my $amax        = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $entitle     = shift;
  my $text        = "";
  my $done        = 0;
  my $sriov_log_port_list;
  if (-e "$basedir/data/$server/$host/sriov_log_port_list.json"){
    $sriov_log_port_list = read_json("$basedir/data/$server/$host/sriov_log_port_list.json");
  }
  if ( $item =~ m/power_lan/ || $item =~ m/power_san/ || $item =~ m/power_sas/ || $item =~ m/power_hea/ ) { graph_power_adapters2 ($host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item ); $done = 1; }
  if ( $item =~ m/power_sri/ ) {
    graph_these_rrd_files( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "rasrm", $sriov_log_port_list); $done=1;
  }
  if ( $lpar =~ m/lan-totals/ && $item =~ m/power_lan/ ){ graph_power_adapters_agg2( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "ralm"); $done = 1; }
  if ( $lpar =~ m/hea-totals/ && $item =~ m/power_hea/ ){ graph_power_adapters_agg2( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "rahm"); $done = 1; }
  if ( $lpar =~ m/sri-totals/ && $item =~ m/power_sri/ ){ graph_power_adapters_agg2( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "rasrm"); $done = 1; }
  if ( $lpar =~ m/san-totals/ && $item =~ m/power_san/ ){ graph_power_adapters_agg2( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "rasm"); $done = 1; }
  if ( $lpar =~ m/sas-totals/ && $item =~ m/power_sas/ ){ graph_power_adapters_agg2( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $item, "rapm"); $done = 1; }
  if ( $item =~ m/^lpar$/ ) { graph_lpar( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $super_detail );$done=1; }
  if ( $item =~ m/^pool$/ )     { graph_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^pool-max$/ ) { graph_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustcpu$/ ) { graph_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^rpcpu$/ )    { graph_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/shpool/ ) { graph_shpool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/custom_cpu_trend/ ) { graph_custom_trend( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group" ); $done = 1; }
  if ( $item =~ m/custom$/ )       { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group" ); $done = 1; }
  if ( $item =~ m/customxport$/ )  { graph_custom_xport( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group" ); $done = 1; }
  if ( $item =~ m/custommem$/ )    { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-mem" );     $done = 1; }
  if ( $item =~ m/customosmem$/ )  { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-mem-os" );  $done = 1; }
  if ( $item =~ m/customoslan$/ )  { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-lan-os" );  $done = 1; }
  if ( $item =~ m/customossan1$/ ) { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-san1-os" ); $done = 1; }
  if ( $item =~ m/customossan2$/ ) { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-san2-os" ); $done = 1; }
  if ( $item =~ m/customdisk$/ )   { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-disk" );    $done = 1; }
  if ( $item =~ m/customnet$/ )    { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-net" );     $done = 1; }
  if ( $item =~ m/customvmmemactive$/ ) { graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-vmmem" );   $done = 1; }
  if ( $item =~ m/customvmmemconsumed$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-vmmem" ); $done = 1; }
  # Custom groups for XenServer
  # hint: the $item inside the graph is the beginning of the `filename.cmd` (part before $lpar$)
  if ( $item =~ m/custom-xenvm-cpu-percent$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-cpu-percent" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-cpu-cores$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-cpu-cores" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-memory-used$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-memory-used" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-memory-free$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-memory-free" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-vbd$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-vbd" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-vbd-iops$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-vbd-iops" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-vbd-latency$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-vbd-latency" ); $done = 1; }
  if ( $item =~ m/custom-xenvm-lan$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-xenvm-lan" ); $done = 1; }
  # end Custom groups for XenServer

  # Custom groups for oVirt
  if ( $item =~ m/custom_ovirt_vm_cpu_percent$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-ovirtvm-cpu-percent" ); $done = 1; }
  if ( $item =~ m/custom_ovirt_vm_cpu_core$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-ovirtvm-cpu-core" ); $done = 1; }
  if ( $item =~ m/custom_ovirt_vm_memory_used$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-ovirtvm-memory-used" ); $done = 1; }
  if ( $item =~ m/custom_ovirt_vm_memory_free$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-ovirtvm-memory-free" ); $done = 1; }
  # if ( $item =~ m/custom-ovirtvm-lan$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-ovirtvm-lan" ); $done = 1; }
  # end Custom groups for oVirt
  # Custom groups for Solaris
  if ( $item =~ m/custom_solaris_cpu$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-solaris-zone-cpu_used" ); $done = 1; }
  if ( $item =~ m/custom_solaris_mem$/){ graph_custom( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix, "custom-group-solaris-zone-phy_mem_us" ); $done = 1; }
  # end Custom groups for oVirt
  if ( $item =~ m/^lparagg$/ )      { graph_lparagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^multicluster$/ ) { graph_lparagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^trend$/ ) { graph_trend( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail ); $done = 1; }
  if ( $item =~ m/^trendpool$/ )     { graph_trend_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail ); $done = 1; }
  if ( $item =~ m/^trendpool-max$/ ) { graph_trend_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail ); $done = 1; }
  if ( $item =~ m/^trendshpool$/ )     { graph_trend_shpool( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail ); $done = 1; }
  if ( $item =~ m/^trendshpool-max$/ ) { graph_trend_shpool( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail ); $done = 1; }
  if ( $item =~ m/^poolagg$/ ) { graph_poolagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustpow$/ ) { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustmem$/ ) { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^rpmem$/ )    { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsmem$/ )    { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^memalloc$/ ) { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^trendmemalloc$/ ) { graph_trendmemalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^memaggreg$/ ) { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^memams$/ ) { graph_amsaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^wpar_mem$/ )  { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^wpar_cpu$/ )  { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ams/ ) { graph_ams( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/lpm/ )  { graph_lpm( $host, $server, $lpar, $time, $name_out, $type_sam, $detail ); $done = 1; }
  if ( $item =~ m/^hea/ ) { graph_hea( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^oscpu/ )  { graph_oscpu( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hmccpu/ ) { graph_oscpu( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^lan/ )      { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^san1/ )     { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^san2/ )     { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^san_resp/ ) { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^sea/ )      { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^packets/ )  { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^pagingagg/ ) { graph_osmem_agg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $amax, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^jobs$/ )      { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $amax, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^jobs_mem/ )   { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $amax, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^powlin_job/ ) { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $amax, $start_unix, $end_unix, $powlin_job ); $done = 1; }
  if ( $item =~ m/^powlin_mem/ ) { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $amax, $start_unix, $end_unix, $powlin_mem ); $done = 1; }
  if ( $item =~ m/^wlm-cpu/ ) { graph_lparagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^wlm-mem/ ) { graph_lparagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^wlm-dkio/ ) { graph_lparagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }

  #  if ( $item =~ m/^tps/ ) {         graph_tps_lparagg ($host,$server,$lpar,$time,$name_out,$type_sam,$detail,$item); $done=1;}
  if ( $item =~ m/^pg2/ )    { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hmcpgs/ ) { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^slan/ )  { graph_ent_sum( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ssan1/ ) { graph_ent_sum( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ssan2/ ) { graph_ent_sum( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ssea/ )  { graph_ent_sum( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ame/ ) { graph_ame_mem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^codused$/ ) { graph_cod_used( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^codunreport$/ ) { graph_cod_unreport( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^mem$/ )    { graph_osmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hmcmem$/ ) { graph_osmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hmccount$/ ) { graph_hmccount( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^pg1$/ )    { graph_paging( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyppg1$/ ) { graph_paging( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^multihmclpar$/ ) { graph_multihmclpar( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustlpar$/ )    { graph_multihmclpar( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustlpardy$/ )  { graph_multihmclpar( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustlan$/ )     { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^rplpar$/ )       { graph_multihmclpar( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^multihmc$/ ) { graph_multihmc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^clustser$/ ) { graph_multihmc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-mem$/ ) { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-disk/ ) { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-iops/ )  { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ds-vmiops/ ) { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-net/ )   { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-swap$/ ) { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-comp$/ ) { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-vmotion$/ ) { graph_vm_vmotion( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmdisk/ )    { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hdisk_io/ )  { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmnet/ )     { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsrw/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsarw/ )     { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dslat/ )     { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-proc/ )  { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^vmw-ready/ ) { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^trendcluster$/ ) { graph_trendcluster( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^trendrp$/ )      { graph_trendcluster( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^trendvm/ )       { graph_trendcluster( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }    # trendvmem too
  if ( $item =~ m/^trendrpmem/ )    { graph_trendcluster( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^trendclmem/ )    { graph_trendcluster( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours ); $done = 1; }
  if ( $item =~ m/^dstrag_/ ) { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }

  # AS400 section
  if ( $item =~ m/^S0200ASPJOB/ ) { graph_AS400_jobs( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ASP/ )         { graph_AS400_jobs( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^ADDR/ )        { graph_AS400_jobs( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^size$/ )      { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^res$/ )       { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^threads$/ )   { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^faults$/ )    { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^pages$/ )     { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^job_cpu$/ )   { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^waj$/ )       { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^disk_io$/ )   { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^disks$/ )     { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^disk_busy$/ ) { graph_memaggreg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^cap_/ )       { graph_aspmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^data_as$/ )   { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^iops_as$/ )   { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^as4job/ )     { graph_as4job( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix, $as4job ); $done = 1; }
  if ( $item =~ m/^asp_cap$/ )   { graph_aspcap( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^data_ifcb$/ )   { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^paket_ifcb$/ )  { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dpaket_ifcb$/ ) { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsk_svc_as$/ )  { graph_aspmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsk_wait_as$/ ) { graph_aspmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^dsk_latency$/ ) { graph_aspmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }

  # HYPERV section
  if ( $item =~ m/^hyp-disk/ )     { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyp-net/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyp-mem$/ )     { graph_osmem( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyp-cpu/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyp-vmotion$/ ) { graph_vm_vmotion( $host, $server, $lpar, $time, $name_out, $type_sam, $upper, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^lfd_cat_/ )     { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hyp_clustser/ ) { frame_multi( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hdt_data/ )     { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^hdt_io/ )       { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^lfd_dat/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^lfd_io/ )       { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^csv_cat_/ )     { graph_memalloc( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^csv_dat/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^csv_io/ )       { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^lfd_lat/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^csv_lat/ )      { graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }

  # SOLARIS 11 section
  if ($item =~ m/^solaris_zone_cpu|^solaris_zone_os_cpu|^solaris_zone_mem|^solaris_zone_net/){ graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_c|^s_ldom_m/){ graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_n/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_vnet/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_san1/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_san2/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_san_resp/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_sum/ )  { graph_ent_sum( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_pack/ )  { graph_ent( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^san_no_ldom1/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^san_no_ldom2/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^san_no_ldom3/){ graph_ent ( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $item, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_agg_c/){ graph_ldomagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s_ldom_agg_m/){ graph_ldomagg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^solaris_pool$/){ graph_pool( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix ); $done = 1; }
  # SOLARIS 10 section
  if ($item =~ m/^s10_g_cpu|^s10_g_mem/){ graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ($item =~ m/^s10_z_cpu|^s10_z_mem/){ graph_vmw_disk( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }

  # Hitachi section
  if ( $item =~ m/^system/ ) { graph_hitachi_percentages( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^cpu-percentages/ ) { graph_hitachi_percentages( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^(hitachi-lan|hitachi-san)$/ ) { graph_hitachi_adapters( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^(hitachi-lan-totals|hitachi-san-totals)$/ ) { graph_hitachi_adapters_agg( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }

  # XenServer section
  if ( $item =~ m/^xen-(host|disk|lan|vm)/ && $item !~ m/^xen-(.*)-aggr$/ ) { graph_xenserver( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  if ( $item =~ m/^xen-(host|vm|pool|disk|lan)-(.*)-aggr$/ ) { graph_xenserver_totals( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }

  #oVirt section
  if ( $item =~ /^ovirt_(storage_domain|storage_domains_total|host_nic|vm|cluster|host)_aggr.*$/ ) { graph_ovirt_totals( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }
  elsif ( $item =~ /^ovirt_(host|host_nic|vm|disk|storage_domain)_.*$/ ) { graph_ovirt( $host, $server, $lpar, $time, $name_out, $type_sam, $detail, $start_unix, $end_unix ); $done = 1; }

  if ( $done == 0 ) {
    error( "Graph error : Not found \"item\": $item : $QUERY_STRING " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  return 1;
} ## end sub create_graph

# read tmp/menu.txt
sub read_menu {
  my $menu_ref = shift;
  open( FF, "<$tmpdir/menu.txt" ) || error( "can't open $tmpdir!menu.txt: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  @$menu_ref = (<FF>);
  close(FF);
  return;
} ## end sub read_menu

# Print the png out
sub print_png {
  my $name_out = shift;

  if ( !-f "$name_out" ) {
    error( "Graph rrdtool error : print_png : file does not exist : $name_out " . __FILE__ . ":" . __LINE__ );
  }
  ## following was only for IE8, we do not support on our demo since 5.08
  #if ( defined $ENV{DEMO} && ( $ENV{DEMO} == 1 ) ) {
  #  my $name_out_8 = "$name_out" . ".png";
  #  `convert "$name_out" -colors 256 PNG8:"$name_out_8";`;
  #  ` mv "$name_out_8" "$name_out"`;
  #  # print STDERR "convert in use for $name_out\n";
  #} ## end if ( defined $ENV{DEMO...})

  if ($graph_cmd eq "graph") {
    if ($detail == 9) {
      $detail = 0;
    }
  }

  if ($detail != 1) {
  #  if ($detail == 9) {
    $html_heading =~ s/\d\d:\d\d:\d\d.*\d\d\.\d\d\d\d//;
  }
   #print STDERR "650 \$detail $detail \$rrd_legend ,$rrd_legend, \@table_out ,@table_out, \$item ,$item,\n";

  #  if ( @table_out > 1 && ($detail < 1 || $detail == 9) && ( $rrd_legend ne "orig" ) ) {
  if ( @table_out > 1 && ( $detail < 1 || $detail == 9) && ( $rrd_legend ne "orig" ) ) {
    $heading_png =~ s/image\/png/application\/json/;
    print "$heading_png";
    if ($html_heading ne "") {
      chomp $png_end_heading;
      chomp $png_end_heading;
      if ($png_end_heading !~ "X-RRD") {
        $png_end_heading = "X-RRDGraph-Properties::::::"."$png_end_heading"; # fake
        # $png_end_heading = "X-RRDGraph-Properties: 65:15:400:150:1474287623:1505823623:"."$png_end_heading";
      }
      $png_end_heading = "$png_end_heading:".encode_base64( $html_heading, "" )."\n\n";
    }
    print "$png_end_heading";    # for graphv possibility
    # print STDERR "651 \$heading_png ,$heading_png, \$png_end_heading ,$png_end_heading,\n";
    my $picture = do {
      local $/ = undef;
      open my $fh, "<", $name_out || error( "Cannot open  $name_out: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      <$fh>;
    };
    my $encoded_pict = encode_base64( $picture, "" );
    $encoded_pict = "data:image/png;base64,$encoded_pict";
    my $table_one_string = "@table_out";
    my $encoded_table_out = encode_base64( $table_one_string, "" );

    my $lll = length $encoded_pict;
    my $llx = length $encoded_table_out;

    #print STDERR "length encoded pict is $lll\n";
    #print STDERR "length encoded table is $llx\n";

    print "{\"img\":\"$encoded_pict\", \"table\": \"$encoded_table_out\"}";

  } ## end if ( @table_out > 1 &&...)
  else {
    print "$heading_png";
    # print STDERR "685 \$heading_png ,$heading_png, \$html_heading ,$html_heading,\n";
    if ($html_heading ne "") {
      chomp $png_end_heading;
      chomp $png_end_heading;
      $png_end_heading = "$png_end_heading:".encode_base64( $html_heading, "" )."\n\n";
      #$png_end_heading = "$png_end_heading:$html_heading\n\n";
    }
    if ($png_end_heading !~ "X-RRD") {
      $png_end_heading = "X-RRDGraph-Properties::::::"."$png_end_heading"; # fake
      # $png_end_heading = "X-RRDGraph-Properties: 65:15:400:150:1474287623:1505823623:"."$png_end_heading";
    }
    if ($detail == 1 || $html_heading eq "") {
      print "\n";
    }
    else {
      print "$png_end_heading";    # for graphv possibility
      # print STDERR "696 \$item $item no table \$png_end_heading ,$png_end_heading, \$html_heading ,$html_heading,\n";
    }

    open( PNG, "< $name_out" ) || error( "Cannot open  $name_out: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    binmode(PNG);
    while ( read( PNG, $b, 4096 ) ) {
      print "$b";
    }
    close(PNG);
  } ## end else [ if ( @table_out > 1 &&...)]

  unlink("$name_out");
  return 1;
} ## end sub print_png

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  my $basedir  = $ENV{INPUTDIR};
  chomp($text);

  #print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $host:$server:$lpar:$item:$time:$type_sam:$detail:$upper : $text \n";

  if ( $alarm == 1 ) {
    print "$act_time: $text : $!\n";
    return 1;
  }
  my $base64 = encode_base64($text, '');
  print "X-XoruX-Error: $base64\n";

  print "\n";    # print missing enter to finish header HTTP
                 # print error picture to check error-cgi.log or web error log
  if ( $text =~ /ACL: not allowed/ && -f "$basedir/html/acl_error.png" ) {
    open( PNG, "< $basedir/html/acl_error.png" );

    # || error("Cannot open  $basedir/html/error.png: $!".__FILE__.":".__LINE__) && return 0;
  }
  else {
    open( PNG, "< $basedir/html/error.png" )

      #|| error("Cannot open  $basedir/html/error.png: $!".__FILE__.":".__LINE__) && return 0;
  }
  binmode(PNG);
  while ( read( PNG, $b, 4096 ) ) {
    print "$b";
  }
  close(PNG);

  # print STDERR "err png printed \n";
  return 1;
} ## end sub error

sub ret_graph_param {

  # for graphv returns
  #  "X-RRDGraph-Properties: $graph_left:$graph_top:$graph_width:$graph_height:$graph_start:$graph_end\n\n"
  # for graph returns
  #  "\n"
  # if exist print lines containing $delimiter -> prepares @table_out for clickable legend
  # if defined second parameter > Updated time for LAN SAN1 SAN2 SEA & others for clickable legend

  my $answ = shift;

  # print STDERR 938,substr($answ,0,19),"\$answ\n";
  my $updated = shift;
  my $iops    = shift;
  my $rest_api_type = shift;
  if ( !defined($rest_api_type)) {
    $rest_api_type = "";
  }
  if ( !defined($iops) || $iops eq '' ) {

    # Linux standalones reports Frames
    $iops = "IOPS";
  }

  if ( !defined $updated ) {    # line Updated
    $updated = "";
  }

  my $graphv_line = "";         # end of HTML heading
  my $test_item   = "graph_";
  my @rrd_result;
  if ( $answ =~ /0x[0-9a-fA-F]{5,}/ ) {
    @rrd_result = split( '\n', $$answ );
  }
  else {
    @rrd_result = split( '\n', $answ );
  }

  # print STDERR "571 \@rrd_result @rrd_result\n";

  for (@rrd_result) {s/\"//g};               # filtr
  for (@rrd_result) {s/print\[.*\] =//g};    #  1st part of print line in newer rrd versions

  my @rrd_print = grep {/$delimiter/} @rrd_result;    #print lines for legend
  # print STDERR "578 rrdprint @rrd_print\n";
  my @matches = grep {/^$test_item/} @rrd_result;     # for zoom
  my $param_num = @matches;

  if ( $param_num > 0 && $param_num != 6 ) {
    print STDERR "cmd graphv did not give 6 graph_ params, gave $param_num " . __FILE__ . ":" . __LINE__ . "\n";

    # return $graphv_line;   # let it go on, 5 params is maybe enough so lets try
  }
  if ( $param_num > 0 ) {
    $graphv_line = "X-RRDGraph-Properties: ";
    for ( my $i = 0; $i < $param_num; $i++ ) {
      ( undef, my $dimension ) = split( "=", $matches[$i] );
      $dimension *= 1;
      $graphv_line .= $dimension;
      if ( $i != 5 ) {
        $graphv_line .= ":";
      }
    } ## end for ( my $i = 0; $i < $param_num...)
    $graphv_line .= "\n\n";
  } ## end if ( $param_num > 0 )
  else {
    $graphv_line .= "\n";
  }

  # print STDERR "994 \$item $item \$html_heading $html_heading \$test_item $test_item \@rrd_print @rrd_print\n";
  # print STDERR "\$graphv_line $graphv_line\n";

  #
  #now prepare HTML table for clickable legend for pool, shared pool, hmctotals ... etc
  #

  if ( @rrd_print < 1 ) {
    return "$graphv_line";
  }
  my $item_ret = "";

  # print STDERR "766 detail-graph-cgi.pl \@rrd_print @rrd_print\n";
  # print STDERR "\$rrd_print[0] $rrd_print[0]\n";
  ( undef, $item_ret, undef ) = split( "$delimiter", $rrd_print[0] );
  # print STDERR "\$item_ret $item_ret\n";
  if ( defined $item_ret && $item_ret =~ /poolagg/ || $item_ret =~ /lparagg/ || $item_ret =~ /multihmcframe/ || $item_ret =~ /multicluster/ || $item_ret =~ /s_ldom_agg_c/ || $item_ret =~ /s_ldom_agg_m/ | $item_ret =~ /custom_solaris_cpu/ || $item_ret =~ /custom_solaris_mem/) {
    my $units = "cores";
    $units = "GHz  " if $vmware && $item_ret !~ /multihmcframe/;
    my $headline_in_html_legend = "Utilization in CPU $units";
    if ($item_ret =~ /s_ldom_agg_m|custom_solaris_mem/){
      $headline_in_html_legend = "Used memory in GB";
    }
    # prepare HTML table for pool, shared pool, hmctotals - server
    my $line1 = "";
    my $line2 = "";
    if ($item_ret =~ /custom_solaris_cpu|custom_solaris_mem/) {
      #$line1 = "<table class=\"tablesorter tablegend\" data-sortby='4'>
      $line1 = "<p class='custom0'>$headline_in_html_legend</p>
      <table class=\"tablesorter tablegend\" data-sortby='4'>
      <thead>
      <tr>
      <th>&nbsp;</th>
      <th class=\"sortable header toleft\">Zone</th>
      <th class=\"sortable header toleft\">LDOM/CDOM/Global zone</th>
      <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
      <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
      </thead>
      <tbody>";
      $line2 = "    <tr>
      <td class=\"legsq\">xorux_lpar_color</td>
      <td class=\"clickabletd\">xorux_lpar_name</td>
      <td class=\"clickabletd\">xorux_server_name</td>
      <td>xorux_lpar_avg</td>
      <td>xorux_lpar_max</td>
      </tr>";
    }else{
      $line1 = "<table class=\"tablesorter tablegend\" data-sortby='4'>
      <thead>
      <tr>
      <th>&nbsp;</th>
      <th class=\"sortable header toleft\">$headline_in_html_legend:</th>
      <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
      <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
      </thead>
      <tbody>";
      $line2 = "    <tr>
      <td class=\"legsq\">xorux_lpar_color</td>
      <td class=\"clickabletd\">xorux_lpar_name</td>
      <td>xorux_lpar_avg</td>
      <td>xorux_lpar_max</td>
      </tr>";
    }
    my $line2_tcpu = "<tfoot> <tr>
    <td class=\"legsq\">#888888</td>
    <td class=\"toleft\">Total available in pools</td>
        <td></td>
        <td>xorux_lpar_max</td>
      </tr></tfoot>";
    my $line2_tcpu_act = "";

    # MAX values for HMC totals -> server
    my $line2_cpu_max = "<tfoot> <tr>
        <td class=\"legsq\">#0088FF</td>
        <td class=\"toleft\">Total max</td>
        <td></td>
        <td>xorux_lpar_max</td>
      </tr></tfoot>";
    my $line2_cpu_max_act = "";

    my $line3 = "  </tbody>  </table>";
    push @table_out, $line1;
    while ( my $print_line = shift(@rrd_print) ) {
      if ( index( $print_line, "MAXTCPU" ) ne -1 ) {    #CPUs total available in pools
        $line2_tcpu_act = $line2_tcpu;
        ( my $maxtcpu, undef ) = split( "MAX", $print_line );
        $line2_tcpu_act =~ s/xorux_lpar_max/$maxtcpu/;
        next;
      } ## end if ( index( $print_line...))
      if ( index( $print_line, "MAXCPUMAX" ) ne -1 ) {    #MAX values from pool.xrm
        $line2_cpu_max_act = $line2_cpu_max;
        ( my $maxtcpu, undef ) = split( "MAX", $print_line );
        $line2_cpu_max_act =~ s/xorux_lpar_max/$maxtcpu/;
        next;
      } ## end if ( index( $print_line...))
      my $line2_act = $line2;
      #print STDERR "line1160-$print_line\n";
      ( my $average, undef, my $lpar_space, my $color, my $rrd_file_all ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $max, undef ) = split( "$delimiter", $print_line );
      if ( !defined $rrd_file_all ) {                     #if not in 1st line > try second
        ( undef, $rrd_file_all ) = split( "$delimiter", $print_line );
      }
      my @pathx           = split( "/", $rrd_file_all );
      my $hmc_t           = @pathx[ @pathx - 2 ];
      my $server_t = "";
      if ($item_ret =~ /custom_solaris_cpu|custom_solaris_mem/) {
       $hmc_t = "Solaris";
       $lpar_space =~ s/\s+//g;
       $server_t = @pathx[ @pathx - 3 ];
      }else{
        $server_t        = @pathx[ @pathx - 3 ];
      }
      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;
      my $lpar_url = $lpar_space_show;

      # $lpar_url =~ s/ /\+/g;
      # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
      # $lpar_url =~ s/\#/%23/g;
      $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
      if ($hmc_t eq "ZONE"){ $lpar_space_show = $server_t } ##### LDOM name
      my $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";
      if ($item_ret =~ /custom_solaris_cpu|custom_solaris_mem/) {
        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=solaris_zone_cpu&entitle=0&none=none\">$lpar_space_show</a>";
      }
      if ( $item_ret =~ /multihmcframe/ ) {
        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=pool&entitle=0&none=none\">$lpar_space_show</a>";
        if ($hyperv) {
          $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=windows/$server_t&lpar=pool&item=pool&entitle=0&none=none\">$lpar_space_show</a>";
        }
      }
      if ( $item_ret =~ /multicluster/ ) {
        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=cluster&item=cluster&square_item=clustcpu\">$lpar_space_show</a>";

        # print STDERR "692 $lpar_space_show\n";
      }

      if ( $hitachi ) {
        $query_lpar ="<a href=\"/lpar2rrd-cgi/detail.sh?host=$server_t&server=$hmc_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";
      }

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      if ($item_ret =~ /custom_solaris_cpu|custom_solaris_mem/) {
        $line2_act =~ s/xorux_server_name/$server_t/;
        $line2_act =~ s/total/-/g;
      }
      $line2_act =~ s/xorux_lpar_color/$color/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $line2_tcpu_act ne "" ) {
      push @table_out, $line2_tcpu_act;
    }
    if ( $line2_cpu_max_act ne "" ) {
      push @table_out, $line2_cpu_max_act;
    }
    push @table_out, $line3;

    # print STDERR "1067 @table_out\n";
    # prepare file with table
    # my $sss = "/tmp/table_out.html";
    # open (FF, ">$sss");
    # print FF "@table_out\n";
    # close (FF);
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /vmw-vmotion/ || $item_ret =~ /hyp-vmotion/ ) ) {
    my $line1 = "<table class=\"tablesorter tablegend\" data-sortby='5'>
    <thead>
      <tr>
        <th>&nbsp;</th>
        <th class=\"toleft\">VM</th>
        <th class=\"sortable header toleft\">server</th>
        <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2 = "    <tr>
	    <td class=\"legsq\">xorux_lpar_color</td>
  	  <td class=\"toleft\">xorux_lpar_name</td>
      <td class=\"clickabletd\">xorux_server_name</td>
	    <td>xorux_lpar_avg</td>
  	  <td>xorux_lpar_max</td>
      </tr>";
    my $line4_updated = "";
    if ( $updated =~ "Updated" ) {
      $line4_updated = " <tr>
        <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
      </tr>";
    }
    push @table_out, $line1;

    my ( $average, $server_space, $lpar_space, $color, $host, $server );
    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2;
      ( $average, undef, $server_space, $lpar_space ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $max, $color, $host, $server ) = split( " $delimiter ", $print_line );

      my $lpar_space_show   = $lpar_space;
      my $server_space_show = $server_space;
      $lpar_space_show =~ s/&&1/\//g;
      $server_space_show =~ s/&&1/\//g;
      my $lpar_url     = $lpar_space_show;
      my $query_lpar   = $lpar_space_show;
      my $server_url   = $server_space_show;
      my $query_server = $server_space_show;

      $server_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

      $query_server = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$host&server=$server&lpar=pool&item=pool&entitle=0&none=none\">$server_space_show</a>";

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_server_name/$query_server/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color/$color/;

      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    my $tbody_end = "</tbody>";
    push @table_out, $tbody_end;

    my $total_lines_to_print = "<tfoot>" . $line4_updated . "</tfoot></table>";
    push @table_out, $total_lines_to_print;

    # print STDERR "@table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /memaggreg_waj/ || $item_ret =~ "memaggreg_cpu" || $item_ret =~ /memaggreg_disk/ ) ) {    # other memaggreg_... see following if
                                                                                                                                     # prepare table for memqaggreg

    my $line1_memaggreg = "<table class=\"tablesorter tablegend\" data-sortby='6 7'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">Jobname Jobno</th>
  <th class=\"sortable header toleft\">User</th>
  <th class=\"sortable header toleft\">Type&nbsp;&nbsp;</th>
	<th class=\"sortable header\">AvrgRun&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Avrg&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Max&nbsp;&nbsp;</th>
  <th class=\"sortable header toleft\">Function</th>
  <th class=\"sortable header toleft\">Status&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_memaggreg = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"toleft\">Subsystem</td>
  <td class=\"toleft\">User</td>
  <td class=\"toleft\">Type</td>
	<td>xorux_lpar_tavg</td>
  <td>xorux_lpar_avg</td>
  <td>xorux_lpar_max</td>
  <td class=\"toleft\">Function</td>
  <td class=\"toleft\">Status</td>
      </tr>";

    my $line3_memaggreg = "  </tbody>  </table>";

    my $line3_memaggreg_day = " </tbody>
    <tfoot>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memaggreg;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_memaggreg;
      ( my $average, undef, my $lpar_space ) = split( " $delimiter ", $print_line );
      ( $lpar_space, my $user, my $upid ) = split( " ", $lpar_space );
      $print_line = shift(@rrd_print);
      ( my $max, my $color, my $subtype, my $function, my $status, my $filename ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $avrgt, undef ) = split( " $delimiter ", $print_line );
      $avrgt = "0.00" if $avrgt =~ /NaNQ/;

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_tavg/$avrgt/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $filename =~ s/\.mmc$//;
      $line2_act =~ s/>xorux_lpar_color/ data-job=\"$filename\">xorux_lpar_color/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      $line2_act =~ s/Subsystem/$lpar_space $upid/;
      $line2_act =~ s/Type/$subtype/;
      $line2_act =~ s/User/$user/;
      $line2_act =~ s/Function/$function/;
      $line2_act =~ s/Status/$status/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memaggreg_day =~ s/COMMENT://;
      $line3_memaggreg_day =~ s/\\//g;
      push @table_out, $line3_memaggreg_day;
    }
    else {
      push @table_out, $line3_memaggreg;
    }

    # push  @table_out, $line3_memaggreg;
    # print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ "memaggreg_jobcpu" ) ) {    # CPUTOT jobs, other memaggreg_... see following if
                                                                       # prepare table for memaggreg
    my $line1_memaggreg = "<table class=\"tablesorter tablegend\" data-sortby='5 6'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">Job PID&nbsp;&nbsp;</th>
  <th class=\"sortable header toleft\">User&nbsp;&nbsp;</th>
	<th class=\"sortable header\">AvrgRun&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Avrg&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;</th>
  <th class=\"sortable header toleft\">Command</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_memaggreg = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"toleft\">Subsystem</td>
  <td class=\"toleft\">User</td>
	<td>xorux_lpar_tavg</td>
  <td>xorux_lpar_avg</td>
  <td>xorux_lpar_max</td>
  <td class=\"toleft\">Function</td>
      </tr>";

    my $line3_memaggreg = "  </tbody>  </table>";

    my $line3_memaggreg_day = " </tbody>
    <tfoot>
      <td colspan=\"6\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memaggreg;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_memaggreg;
      ( my $average, undef, my $lpar_space ) = split( " $delimiter ", $print_line );

      # ($lpar_space, my $user) = split(" ",$lpar_space);
      $print_line = shift(@rrd_print);
      # ( my $max, my $color, my $user, my $function, my $filename ) = split( " $delimiter ", $print_line );
      ( my $max, my $color, my $user, my $filename ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $avrgt, undef ) = split( " $delimiter", $print_line );
      $avrgt = "0.00" if $avrgt =~ /[Nn]a[Nn]/;

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_tavg/$avrgt/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $filename =~ s/\.mmc$//;
      $line2_act =~ s/>xorux_lpar_color/ data-job=\"$filename\">xorux_lpar_color/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      $line2_act =~ s/Subsystem/$lpar_space/;

      #$line2_act =~ s/Type/$subtype/;
      $line2_act =~ s/User/$user/;
      $filename =~ s/\.cfg$//;
      my $function = $unix_commands{$filename};
      # print STDERR "1138 \$filename $filename\n";
      $function = "not_defined" if !defined $function;
      $line2_act =~ s/Function/$function/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memaggreg_day =~ s/COMMENT://;
      $line3_memaggreg_day =~ s/\\//g;
      push @table_out, $line3_memaggreg_day;
    }
    else {
      push @table_out, $line3_memaggreg;
    }

    # push  @table_out, $line3_memaggreg;
    # print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /memaggreg_rq_disks/ || $item_ret =~ /memaggreg_busy_disks/ ) ) {    # other memaggreg_... see following if
                                                                                                                # prepare table for memaggreg
    my $line1_memaggreg = "<table class=\"tablesorter tablegend\" data-sortby='5 6'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">Disk serial</th>
  <th class=\"sortable header toleft\">ASP</th>
	<th class=\"sortable header\">AvrgRun&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Avrg&nbsp;&nbsp;</th>
  <th class=\"sortable header\">Max&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_memaggreg = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"toleft\">Subsystem</td>
  <td class=\"toleft\">User</td>
	<td>xorux_lpar_tavg</td>
  <td>xorux_lpar_avg</td>
  <td>xorux_lpar_max</td>
      </tr>";

    my $line3_memaggreg = "  </tbody>  </table>";

    my $line3_memaggreg_day = " </tbody>
    <tfoot>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memaggreg;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_memaggreg;
      ( my $average, undef, my $lpar_space ) = split( " $delimiter ", $print_line );
      ( $lpar_space, my $user, my $upid ) = split( " ", $lpar_space );
      $print_line = shift(@rrd_print);
      ( my $max, my $color, my $subtype, my $function, my $status, my $filename ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);
      ( my $avrgt, undef ) = split( " $delimiter ", $print_line );
      $avrgt = "0.00" if $avrgt =~ /NaNQ/;

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_tavg/$avrgt/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $filename =~ s/\.mmc$//;
      $line2_act =~ s/>xorux_lpar_color/ data-job=\"$filename\">xorux_lpar_color/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      $line2_act =~ s/Subsystem/$lpar_space/;
      $line2_act =~ s/User/$user/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memaggreg_day =~ s/COMMENT://;
      $line3_memaggreg_day =~ s/\\//g;
      push @table_out, $line3_memaggreg_day;
    }
    else {
      push @table_out, $line3_memaggreg;
    }

    # push  @table_out, $line3_memaggreg;
    # print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && $item_ret =~ /dstrag_/ ) {    # other memaggreg_... see following if
                                                          # prepare table for memaggreg
    my $is_max1 = "";
    my $is_max2 = "";

    #my $is_type1 = "";
    #my $is_type2 = "";
    my $allocated_0 = "VMware alias";
    my $allocated   = "";

    $allocated = 'Datastore name &nbsp;&nbsp;&nbsp;&nbsp; [GB]'     if $item_ret =~ /dstrag_used/;
    $allocated = 'Datastore name'                                   if $item_ret =~ /dstrag_iops/;
    $allocated = 'Datastore name &nbsp;&nbsp;&nbsp;&nbsp; [MB/sec]' if $item_ret =~ /dstrag_data/;

    $is_max1 = "<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>";
    $is_max2 = "<td>xorux_lpar_max</td>";

    my $line1_memaggreg = "<table class=\"tablesorter tablegend\" data-sortby='5'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$allocated_0</th>
  <th class=\"sortable header toleft\">$allocated</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
  $is_max1
      </tr>
    </thead>
    <tbody>";
    my $line2_memaggreg = "    <tr>
	<td class=\"legsq noclick\">xorux_lpar_color</td>
	<td class=\"toleft\">xorux_alias_name</td>
  <td class=\"clickabletd\">xorux_datastore_name</td>
	<td>xorux_lpar_avg</td>
  $is_max2
      </tr>";

    my $line3_memaggreg = "  </tbody>  </table>";

    my $line3_memaggreg_day = " </tbody>
    <tfoot>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memaggreg;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_memaggreg;

      # print STDERR "1056 \$print_line $print_line\n";
      ( my $average, undef, my $alias_name, my $datastore_name, my $color ) = split( " $delimiter ", $print_line );
      $print_line = shift(@rrd_print);

      # print STDERR "1059 \$print_line $print_line\n";
      ( my $max, my $datacenter_name ) = split( " $delimiter ", $print_line );
      $line2_act =~ s/xorux_lpar_max/$max/;

      my $lpar_space_show = $datastore_name;
      $lpar_space_show =~ s/&&1/\//g;

      #my $lpar_url = $lpar_space_show;
      my $query_lpar = $lpar_space_show;

      #if ( defined $rrd_file_all && $rrd_file_all ne "") {
      if ( defined $datacenter_name && $datacenter_name ne "" ) {

        # my @pathx = split ("/",$rrd_file_all);
        # my $hmc_t = @pathx[@pathx - 2];
        # my $server_t = @pathx[@pathx - 3];
        # my $lpar_url = @pathx[@pathx - 1];
        # $lpar_url =~ s/\.rrs$//;
        # $lpar_url =~ s/\.rrt$//;
        #  $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=datastore&entitle=0&none=none\">$lpar_space_show</a>";
        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$datacenter_name&server=$alias_name&lpar=$datastore_name&item=datastore&entitle=0&none=none\">$lpar_space_show</a>";
      } ## end if ( defined $datacenter_name...)
      else {
        $line2_act =~ s/clickabletd/toleft/;    # Firmware , Available are not
      }
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_datastore_name/$query_lpar/;
      $line2_act =~ s/xorux_alias_name/$alias_name/;
      $line2_act =~ s/xorux_lpar_color/$color/;

      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memaggreg_day =~ s/COMMENT://;
      $line3_memaggreg_day =~ s/\\//g;
      push @table_out, $line3_memaggreg_day;
    }
    else {
      push @table_out, $line3_memaggreg;
    }

    #push  @table_out, $line3_memaggreg;
    # print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /memaggreg/ || $item_ret =~ /wpar_mem/ || $item_ret =~ /wpar_cpu/ ) ) {    # also: memaggreg_threads, memaggreg_res, memaggreg_size
                                                                                                                      # prepare table for memaggreg
    my $is_max1   = "";
    my $is_max2   = "";
    my $is_type1  = "";
    my $is_type2  = "";
    my $allocated = 'Current allocated memory in GB&nbsp;&nbsp;';
    $allocated = 'Current granted memory in GB&nbsp;&nbsp;' if $vmware;
    $allocated = 'Current CPU load in cores&nbsp;&nbsp;'    if $item_ret =~ /wpar_cpu/;
    $allocated = 'Current used memory in GB&nbsp;&nbsp;'    if $item_ret =~ /wpar_mem/;

    if ( $item_ret =~ /wpar_/ ) {
      $is_max1 = "<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $is_max2 = "<td>xorux_lpar_max</td>";
    }
    if ( $item_ret =~ /memaggreg_threads/ || $item_ret =~ /memaggreg_res/ ) {
      $allocated = 'Current number of threads&nbsp;&nbsp;&nbsp;' if $item_ret =~ /memaggreg_threads/;
      $is_max1   = "<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $is_max2   = "<td>xorux_lpar_max</td>";
    }
    if ( $item_ret =~ /memaggreg_size/ ) {
      $is_max1  = "<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $is_max2  = "<td>xorux_lpar_max</td>";
      $is_type1 = "<th class=\"sortable header\">type&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $is_type2 = "<td>xorux_lpar_type</td>";
    } ## end if ( $item_ret =~ /memaggreg_size/)

    my $line1_memaggreg = "<table class=\"tablesorter tablegend\" data-sortby='4 3'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$allocated</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
  $is_max1
  $is_type1
      </tr>
    </thead>
    <tbody>";
    my $line2_memaggreg = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_lpar_avg</td>
  $is_max2
  $is_type2
      </tr>";

    my $line3_memaggreg = "  </tbody>  </table>";

    my $line3_memaggreg_day = " </tbody>
    <tfoot>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memaggreg;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_memaggreg;
      ( my $average, undef, my $lpar_space, my $color, my $rrd_file_all ) = split( " $delimiter ", $print_line );
      if ( !defined $color ) {
        $print_line = shift(@rrd_print);
        ( undef, $color, $rrd_file_all ) = split( " $delimiter ", $print_line );
      }

      if ( $item_ret =~ /memaggreg_threads/ || $item_ret =~ /memaggreg_res/ || $item_ret =~ /wpar_/ ) {
        $print_line = shift(@rrd_print);
        ( my $max ) = split( " $delimiter", $print_line );
        $line2_act =~ s/xorux_lpar_max/$max/;
      }
      if ( $item_ret =~ /memaggreg_size/ ) {
        $print_line = shift(@rrd_print);
        ( my $max ) = split( " $delimiter", $print_line );
        $line2_act =~ s/xorux_lpar_max/$max/;
        $print_line = shift(@rrd_print);
        ( my $type ) = split( " $delimiter", $print_line );
        my $type_txt = "*other";
        if ( $type !~ /NaNQ/ ) {
          $type_txt = "*FIXED" if $type == 0;    # see lpar2rrd-daemon.pl - AS400 section
          $type_txt = "*SAME"  if $type == 1;
          $type_txt = "*CALC"  if $type == 2;
        }
        else {
          $type_txt = "";
        }
        $line2_act =~ s/xorux_lpar_type/$type_txt/;
      } ## end if ( $item_ret =~ /memaggreg_size/)

      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;
      my $lpar_url   = $lpar_space_show;
      my $query_lpar = $lpar_space_show;
      if ( defined $rrd_file_all && $rrd_file_all ne "" ) {
        my @pathx    = split( "/", $rrd_file_all );
        my $hmc_t    = @pathx[ @pathx - 2 ];
        my $server_t = @pathx[ @pathx - 3 ];

        # $lpar_url =~ s/ /\+/g;
        # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
        # $lpar_url =~ s/\#/%23/g;
        $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;
        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";
      } ## end if ( defined $rrd_file_all...)
      else {
        $line2_act =~ s/clickabletd/toleft/;    # Firmware , Available are not
      }
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      if ( $lpar_space =~ "Available memory" ) {
        $line2_act = "<tfoot>" . $line2_act . "</tfoot>";
      }
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memaggreg_day =~ s/COMMENT://;
      $line3_memaggreg_day =~ s/\\//g;
      push @table_out, $line3_memaggreg_day;
    }
    else {
      push @table_out, $line3_memaggreg;
    }

    #push  @table_out, $line3_memaggreg;
    # print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && $item_ret =~ /pagingagg/ ) {

    # prepare table for paging aggregated
    my $line1_osmem_agg = "<table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
	<th class=\"sortable header toleft\">[MB/s]</th>
	<th>READ</th>
	<th class=\"sortable header\">AVG&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">MAX&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE</th>
	<th class=\"sortable header\">AVG&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">MAX&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line2_osmem_agg = "   <tr>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_lpar_color_read</td>
	<td>xorux_lpar_avg_read</td>
	<td>xorux_lpar_max_read</td>
	<td class=\"legsq\">xorux_lpar_color_write</td>
	<td>xorux_lpar_avg_write</td>
	<td>xorux_lpar_max_write</td>
      </tr>\n";
    my $line3_osmem_agg = " </tbody> </table>\n";

    push @table_out, $line1_osmem_agg;

    # print STDERR "pagingagg \@rrd_print @rrd_print\n";

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_osmem_agg;
      ( my $average_read, undef, my $lpar_space, my $color_read, my $rrd_file_all ) = split( " $delimiter ", $print_line );

      $print_line = shift(@rrd_print);
      ( my $max_read, undef ) = split( "$delimiter", $print_line );

      $print_line = shift(@rrd_print);
      ( my $average_write, my $color_write ) = split( "$delimiter", $print_line );

      $print_line = shift(@rrd_print);
      ( my $max_write, undef ) = split( "$delimiter", $print_line );

      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;
      my $lpar_url = $lpar_space_show;
      my @pathx    = split( "/", $rrd_file_all );
      my $hmc_t    = @pathx[ @pathx - 3 ];
      my $server_t = @pathx[ @pathx - 4 ];

      # $lpar_url =~ s/ /\+/g;
      # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
      # $lpar_url =~ s/\#/%23/g;
      $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

      my $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";

      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/xorux_lpar_color_read/$color_read/;
      $line2_act =~ s/xorux_lpar_avg_read/$average_read/;
      $line2_act =~ s/xorux_lpar_max_read/$max_read/;
      $line2_act =~ s/xorux_lpar_color_write/$color_write/;
      $line2_act =~ s/xorux_lpar_avg_write/$average_write/;
      $line2_act =~ s/xorux_lpar_max_write/$max_write/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    push @table_out, $line3_osmem_agg;
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && $item_ret =~ / lan/ || $item_ret =~ / san1/ || $item_ret =~ / san2 $/ || $item_ret =~ /sea/ || $item_ret =~ /san_resp/ || $item_ret =~ /faults/ || $item_ret =~ /pages/ || $item_ret =~ /data_as/ || $item_ret =~ /iops_as/ || $item_ret =~ /data_ifcb/ || $item_ret =~ /paket_ifcb/ || $item_ret =~ /packets_lan/ || $item_ret =~ /packets_sea/ || $item_ret =~ /vmw-iops/ || $item_ret =~ /ds-vmiops/ || $item_ret =~ /hitachi-lan-totals/ || $item_ret =~ /hitachi-san-totals/ || $item_ret =~ /power_san/ || $item_ret =~ /power_lan/ || $item_ret =~ /power_sas/ || $item_ret =~ /power_hea/ || $item_ret =~ /power_sri/ || $item_ret =~ /s_ldom_n/ || $item_ret =~ /s_ldom_san1/ || $item_ret =~ /s_ldom_san2/ || $item_ret =~ /s_ldom_san_resp/ || $item_ret =~ /s_ldom_pack/ || $item_ret =~ /san_no_ldom1/ || $item_ret =~ /s_ldom_vnet/ || $item_ret =~ /clustlan/) {
    my $Int = "Int";
    $Int = "   " if ( $item_ret =~ /data_as/ || $item_ret =~ /iops_as/ || $item_ret =~ /data_ifcb/ || $item_ret =~ /paket_ifcb/ );
    $Int = "Datastore" if $item_ret =~ /vmw-iops/;
    $Int = "VM " if $item_ret =~ /ds-vmiops/;
    $Int = "server" if $item_ret =~ /clustlan/;

    # prepare table for LAN SAN1 SAN2 SEA SAN_RESP FAULT(AS400) PAGES(AS400)


    my $line1_lan = "<p class='custom0'>[MB/sec]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
	<th>READ\</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE\</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_data = "<p class='custom0'>[MB/sec]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th class=\"toleft\">Location\</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg</th>
        <th class=\"sortable header toleft\">Max</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg</th>
        <th class=\"sortable header toleft\">Max</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_hea_data = "<p class='custom0'>[MB/sec]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_hea_io = "<p class='custom0'>[Packets/sec]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_lan_io = "<p class='custom0'>[Packets/sec]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th class=\"toleft\">Location\</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_san_io = "<p class='custom0'>[IOPS]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th class=\"toleft\">Location\</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_sas_io = "<p class='custom0'>[IOPS]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int</th>
        <th class=\"toleft\">Location\</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_sri_io = "<p class='custom0'>[Packets/sec]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int (Lpar)</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line1_power_sri_data = "<p class='custom0'>[MB/s]</p>
    <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Int (Lpar)</th>
        <th>READ\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th>WRITE\</th>
        <th class=\"sortable header toleft\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header toleft\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";


    my $line1_san1 = "<p class='custom0'>[MB/sec]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
	<th class=\"sortable header toleft\">$Int</th>
	<th>READ</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line1_san2 = "<p class='custom0'>[$iops]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
      <tr>
	<th class=\"sortable header toleft\">$Int</th>
	<th>READ</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line1_san_resp = "<p class='custom0'>[ms]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
      <tr>
	<th class=\"sortable header toleft\">Int</th>
	<th>READ</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line2_san_resp = "<p class='custom0'>[ms]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
      <tr>
	<th class=\"sortable header toleft\">Int</th>
	<th>READ/WRITE</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line1_sea = "<p class='custom0'>[MB/sec]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
	<th class=\"sortable header toleft\">Phys (SEA)</th>
	<th>READ\</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>WRITE\</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";
    my $line1_faults = "
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">Name</th>
	<th>DB</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>non\/DB</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n";

    my $line2_lan = "   <tr>
	<td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_lpar_color_read</td>
	<td>xorux_lpar_avg_read</td>
	<td>xorux_lpar_max_read&nbsp;&nbsp;&nbsp;&nbsp;</td>
	<td class=\"legsq\">xorux_lpar_color_write</td>
	<td>xorux_lpar_avg_write</td>
	<td>xorux_lpar_max_write</td>
      </tr>\n";

    my $line2_san_resp1 = "   <tr>
	<td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_lpar_color_read</td>
	<td>xorux_lpar_avg_read</td>
	<td>xorux_lpar_max_read&nbsp;&nbsp;&nbsp;&nbsp;</td>
      </tr>\n";

    my $line2_power_lan = "
      <tr>
        <td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
        <td class=\"toleft\">xorux_physical_location</td>
        <td class=\"legsq\">xorux_lpar_color_read</td>
        <td class=\"toleft\">xorux_lpar_avg_read</td>
        <td class=\"toleft\">xorux_lpar_max_read&nbsp;&nbsp;&nbsp;&nbsp;</td>
        <td class=\"legsq\">xorux_lpar_color_write</td>
        <td class=\"toleft\">xorux_lpar_avg_write</td>
        <td class=\"toleft\">xorux_lpar_max_write</td>
      </tr>\n";
    my $line2_power_sri = "
      <tr>
        <td class=\"toleft\">xorux_physical_location</td>
        <td class=\"legsq\">xorux_lpar_color_read</td>
        <td class=\"toleft\">xorux_lpar_avg_read</td>
        <td class=\"toleft\">xorux_lpar_max_read&nbsp;&nbsp;&nbsp;&nbsp;</td>
        <td class=\"legsq\">xorux_lpar_color_write</td>
        <td class=\"toleft\">xorux_lpar_avg_write</td>
        <td class=\"toleft\">xorux_lpar_max_write</td>
      </tr>\n";

    my $line2_power_hea = "
      <tr>
        <td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
        <td class=\"legsq\">xorux_lpar_color_read</td>
        <td class=\"toleft\">xorux_lpar_avg_read</td>
        <td class=\"toleft\">xorux_lpar_max_read&nbsp;&nbsp;&nbsp;&nbsp;</td>
        <td class=\"legsq\">xorux_lpar_color_write</td>
        <td class=\"toleft\">xorux_lpar_avg_write</td>
        <td class=\"toleft\">xorux_lpar_max_write</td>
      </tr>\n";

    my $line3_lan = " </tbody> </table>\n";

    my $line3_lan_day = " </tbody>
    <tfoot><tr>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tr> </tfoot> </table>\n";
    if ($item_ret =~ /san_resp/){ $item_ret =~ s/^\s+|\s+$//g; }
    if (    $item_ret =~ /power_lan/ && $rest_api_type eq "data"){  push @table_out, $line1_power_data;}
    elsif ( $item_ret =~ /power_lan/ && $rest_api_type eq "packets"){  push @table_out, $line1_power_lan_io;}
    elsif ( $item_ret =~ /power_san/ && $rest_api_type eq "data"){  push @table_out, $line1_power_data;}
    elsif ( $item_ret =~ /power_san/ && $rest_api_type eq "packets" ){  push @table_out, $line1_power_san_io;}
    elsif ( $item_ret =~ /power_hea/ && $rest_api_type eq "data"){  push @table_out, $line1_power_hea_data;}
    elsif ( $item_ret =~ /power_hea/ && $rest_api_type eq "packets" ){  push @table_out, $line1_power_hea_io;}
    elsif ( $item_ret =~ /power_sas/ && $rest_api_type eq "data" ){  push @table_out, $line1_power_data;}
    elsif ( $item_ret =~ /power_sas/ && $rest_api_type eq "packets"){  push @table_out, $line1_power_sas_io;}
    elsif ( $item_ret =~ /power_sri/ && $rest_api_type eq "data"){  push @table_out, $line1_power_sri_data;}
    elsif ( $item_ret =~ /power_sri/ && $rest_api_type eq "packets"){  push @table_out, $line1_power_sri_io;}
    elsif ( $item_ret =~ /lan/ && $item_ret !~ /packets/ && $item_ret !~ /clustlan/)  { push @table_out, $line1_lan; }
    elsif ( $item_ret =~ /clustlan/)  { push @table_out, $line1_san1;}
    elsif ( $item_ret =~ /s_ldom_n|s_ldom_vnet/ && $item_ret !~ /packets/ )  { push @table_out, $line1_lan; }
    elsif ( $item_ret =~ /hitachi-san-totals/ )  { push @table_out, $line1_lan; }
    elsif ( $item_ret =~ /sea/ && $item_ret !~ /packets/)  { push @table_out, $line1_sea; }
    elsif ( $item_ret =~ /san1/ )      { push @table_out, $line1_san1; }
    elsif ( $item_ret =~ /san2 $/ )    { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /s_ldom_san1/ ) { push @table_out, $line1_san1; }
    elsif ( $item_ret =~ /s_ldom_san2/ ) { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /san_no_ldom1/ ) { push @table_out, $line1_san1; }
    elsif ( $item_ret =~ /san_no_ldom2/ ) { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /^san_resp/ )  { push @table_out, $line1_san_resp; }
    elsif ( $item_ret =~ /^s_ldom_san_resp/ )  { push @table_out, $line2_san_resp; }
    elsif ( $item_ret =~ /faults/ )    { push @table_out, $line1_faults; }
    elsif ( $item_ret =~ /pages/ )     { push @table_out, $line1_faults; }
    elsif ( $item_ret =~ /data_as/ )   { push @table_out, $line1_san1; }
    elsif ( $item_ret =~ /iops_as/ )   { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /data_ifcb/ ) { push @table_out, $line1_san1; }
    elsif ( $item_ret =~ /vmw-iops/ )  { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /ds-vmiops/ ) { push @table_out, $line1_san2; }
    elsif ( $item_ret =~ /paket_ifcb/ || $item_ret =~ /packets_lan/ || $item_ret =~ /packets_sea/ || $item_ret =~ /s_ldom_pack/ ) {
      $line1_san1 =~ s/MB/Packets/;
      push @table_out, $line1_san1;
    }
#    print STDERR Dumper(@rrd_print) . "\n";
    my $sriov_aliases;
    if (-e "$basedir/data/$server/$host/sriov_aliases.json"){
      $sriov_aliases = read_json("$basedir/data/$server/$host/sriov_aliases.json");
    }
    my @rrd_print1 = @rrd_print;
    if ($item_ret =~ /^s_ldom_san_resp/){
      while ( my $print_line1 = shift(@rrd_print1)){
        my $line2_act = $line2_san_resp1;
        ( my $average_read, undef, my $lpar_space, my $color_read, my $rrd_file_all ) = split( " $delimiter ", $print_line1 );
        if ($lpar_space =~ m/\.$/){
          $lpar_space = substr($lpar_space, 0, -1);
        }
        $print_line1 = shift(@rrd_print1);
        ( my $max_read, my $bubble_info ) = split( "$delimiter", $print_line1 );
        my $lpar_space_show = $lpar_space;
        $lpar_space_show =~ s/&&1/\//g;
        $line2_act =~ s/xorux_lpar_name/$lpar_space_show/;
        $line2_act =~ s/xorux_lpar_color_read/$color_read/;
        $line2_act =~ s/xorux_lpar_avg_read/$average_read/;
        $line2_act =~ s/xorux_lpar_max_read/$max_read/;
        $line2_act =~ s/bubble_info/$bubble_info/;
        push @table_out, $line2_act;
    }
}
if ($item_ret !~ "s_ldom_san_resp"){
  while ( my $print_line = shift(@rrd_print) ) {
    my $line2_act;
    $line2_act = $line2_lan;
    if ($item_ret =~ /power_lan/ || $item_ret =~ /power_san/ || $item_ret =~ /power_sas/) {
      $line2_act = $line2_power_lan;
    }
    if ($item_ret =~ /power_hea/){
      $line2_act = $line2_power_hea;
    }
    if ($item_ret =~ /power_sri/){
      $line2_act = $line2_power_sri;
    }
    #print STDERR"$print_line\n!!!";
    my $average_read; my $physical_location; my $lpar_space; my $color_read; my $rrd_file_all;
    if ($item_ret =~ /power_/){
     ( $average_read, undef, $physical_location, $lpar_space, $color_read, $rrd_file_all ) = split( " $delimiter ", $print_line );
    }
    else{
      ( $average_read, undef, $lpar_space, $color_read, $rrd_file_all ) = split( " $delimiter ", $print_line );
    }
    if ($lpar_space =~ m/\.$/){
      $lpar_space = substr($lpar_space, 0, -1);
    }
    if ($item_ret =~ /power_sri/){
      if (defined $sriov_aliases->{$lpar_space} && $sriov_aliases->{$lpar_space} ne ""){
        $lpar_space = "$lpar_space ($sriov_aliases->{$lpar_space})";
      }
      else{
        $lpar_space = "$lpar_space";
      }
    }
    $print_line = shift(@rrd_print);
    ( my $max_read, my $bubble_info ) = split( "$delimiter", $print_line );
    $bubble_info = "" if $bubble_info eq " ";
    $bubble_info = "" if $item_ret =~ "hitachi"; # empty string -> lpar_name will be showed
    # print STDERR "1168 \$bubble_info $bubble_info\n";

    $print_line = shift(@rrd_print);
    ( my $average_write, my $color_write ) = split( "$delimiter", $print_line );

    $print_line = shift(@rrd_print);
    ( my $max_write, undef ) = split( "$delimiter", $print_line );

    my $lpar_space_show = $lpar_space;
    $lpar_space_show =~ s/&&1/\//g;
    if ( $item_ret =~ /power_/ ) {
      my $test;
      if ($item_ret =~ /power_sri/){
        ($test, undef) = split (" \\(", $lpar_space);
      }
      else{
        $test = $lpar_space;
      }
      $test =~ s/-S[0-9]*$//g;
      $item_ret =~ s/^\s+|\s+$//g;
      my $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$host&server=$server&lpar=$test&item=$item_ret&entitle=0&none=none\">$lpar_space</a>";
#                                  lpar2rrd-cgi/detail.sh?host=vhmc &server=P02DR__9117-MMC-SN44K8102&lpar=44K8102-V1-C5-T1.ralm&item=power_lan&entitl
      if ($item_ret !~ /power_sri/){
        $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      }
      else{
        $line2_act =~ s/xorux_physical_location/$query_lpar/;
      }
    }
    if ( $item_ret =~ /ds-vmiops/ ) {
      my $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=&server=&lpar=$lpar_space&item=vmw-iops&entitle=0&none=none\">$lpar_space_show</a>";
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
    }
    else {
      $line2_act =~ s/xorux_lpar_name/$lpar_space_show/;
    }
    $line2_act =~ s/xorux_lpar_color_read/$color_read/;
    $line2_act =~ s/xorux_physical_location/$physical_location/;
    $line2_act =~ s/xorux_lpar_avg_read/$average_read/;
    $line2_act =~ s/xorux_lpar_max_read/$max_read/;
    $line2_act =~ s/xorux_lpar_color_write/$color_write/;
    $line2_act =~ s/xorux_lpar_avg_write/$average_write/;
    $line2_act =~ s/xorux_lpar_max_write/$max_write/;
    $line2_act =~ s/bubble_info/$bubble_info/;

    # print STDERR "1182 \$line2_act $line2_act \$bubble_info ,$bubble_info,\n";
    push @table_out, $line2_act;
  } ## end while ( my $print_line = ...)
}
    if ( $updated =~ /Updated/ ) {
      $line3_lan_day =~ s/COMMENT://;
      $line3_lan_day =~ s/\\//g;
      push @table_out, $line3_lan_day;
    }
    else {
      push @table_out, $line3_lan;
    }

    # print STDERR "1024 @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  # XenServer
  if ( defined $item_ret && ( $item_ret =~ /(xen-(.*)-aggr|custom-xenvm-(.*))/ ) ) {
    my @header;
    my $units;
    my $column_count = 7;
    my $link_type = '';

    if ( $item_ret =~ /(vbd|lan)/ ) {
      $column_count = 8;
      if ( $item_ret =~ /xenvm/ ) {
        @header    = ( 'Pool', 'VM', 'READ', 'WRITE' );
        $link_type = 'vm';
      } else { # xen-(lan|disk)
        if ( $item_ret =~ /lan/ ) {
          @header  = ( 'Host', 'Int', 'READ', 'WRITE' );
        } else { # xen-disk
          @header  = ( 'Host', 'Storage', 'READ', 'WRITE' );
        }
      }
      if ( $item_ret =~ /lan/ ) { $units = '[MB/sec]'; }
      elsif ( $item_ret =~ /latency/ ) { $units = '[millisec]'; }
      elsif ( $item_ret =~ /iops/ ) { $units = '[IOPS]'; }
      else { $units = '[MiB/sec]'; }
    } elsif ( $item_ret =~ /xen-host-(cpu-percent|cpu-cores|memory-used|memory-free)-aggr/ ) {
      $column_count = 4;
      @header    = ( 'Host', '', '', '' );
      $link_type = 'host';

      if ( $item_ret =~ /cpu-cores/ ) { $units = '[cores]'; }
      elsif ( $item_ret =~ /cpu-percent/ ) { $units = '[%]'; }
      else { $units = '[GiB]'; }
    } elsif ( $item_ret =~ /(xen-vm|xen-host-vm|custom-xenvm)-(cpu-percent|cpu-cores|memory-used|memory-free)/ ) {
      $column_count = 5;
      if ( $item_ret =~ /custom-xenvm/ ) {
        @header     = ( 'Pool', 'VM', '', '' );
      } else {
        @header     = ( 'Host', 'VM', '', '' );
      }
      $link_type    = 'vm';

      if ( $item_ret =~ /cpu-cores/ ) { $units = '[cores]'; }
      elsif ( $item_ret =~ /cpu-percent/ ) { $units = '[%]'; }
      else { $units = '[GiB]'; }
    }

    # the rest of this block is literally the same as in the oVirt branch, except the 8col table and different ad-hoc hypertext

    my $table_header_4col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$header[0]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>
END

    my $table_header_5col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$header[0]</th>
	<th class=\"sortable header toleft\">$header[1]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>
END

    my $table_header_7col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">$header[0]</th>
	<th>$header[1]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>$header[2]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n
END

    my $table_header_8col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 8'>
    <thead>
      <tr>
	<th class=\"sortable header toleft\">$header[0]</th>
	<th class=\"sortable header toleft\">$header[1]</th>
	<th>$header[2]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>$header[3]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>
END

    my $table_row_4col = <<END;
      <tr>
	<td class=\"legsq\">xorux_val1_color</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max</td>
      </tr>
END

    my $table_row_5col = <<END;
      <tr>
	<td class=\"legsq\">xorux_val1_color</td>
	<td class=\"clickabletd\">xorux_server_name</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max</td>
      </tr>
END

    my $table_row_7col = <<END;
      <tr>
	<td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_val1_color</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max&nbsp;</td>
	<td class=\"legsq\">xorux_val2_color</td>
	<td>xorux_val2_avg</td>
	<td>xorux_val2_max</td>
      </tr>\n
END

    my $table_row_8col = <<END;
      <tr>
	<td class=\"clickabletd\">xorux_server_name</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_val1_color</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max&nbsp;</td>
	<td class=\"legsq\">xorux_val2_color</td>
	<td>xorux_val2_avg</td>
	<td>xorux_val2_max</td>
      </tr>\n
END

    my $table_end     = " </tbody> </table>\n";
    my $table_end_day = <<END;
    </tbody>
    <tfoot>
      <tr>
        <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
      </tr>
    </tfoot>
  </table>\n
END

    if ( $column_count == 4 ) {
      push @table_out, $table_header_4col;
    }
    elsif ( $column_count == 5 ) {
      push @table_out, $table_header_5col;
    }
    elsif ( $column_count == 7 ) {
      push @table_out, $table_header_7col;
    }
    elsif ( $column_count == 8 ) {
      push @table_out, $table_header_8col;
    }
##    push @table_out, ( $column_count == 4 ? $table_header_4col : $table_header_7col );

    while ( my $print_line = shift(@rrd_print) ) {
      my $table_row;# = ( $column_count == 4 ? $table_row_4col : $table_row_7col );

      if ( $column_count == 4 ) {
        $table_row = $table_row_4col;
      }
      elsif ( $column_count == 5 ) {
        $table_row = $table_row_5col;
      }
      elsif ( $column_count == 7 ) {
        $table_row = $table_row_7col;
      }
      elsif ( $column_count == 8 ) {
        $table_row = $table_row_8col;
      }

      ( my $val1_avg, my $item, my $name_space, my $val1_color, undef ) = split( " $delimiter ", $print_line );
      ( my $val1_max, my $bubble_info, my $cluster, my $uuid ) = split( "$delimiter", shift(@rrd_print) );
      $bubble_info = "" if $bubble_info eq " ";

      # note: parse the $item to get "item" in data-url for detail-graph preview; this will replace "xorux_val1_color"

      my $name_space_show = $name_space;
      $name_space_show =~ s/&&1/\//g;

      if ( $link_type ne '' ) {
        $uuid =~ s/^\s+|\s+$//g;
        my $link_url = XenServerMenu::get_url( { type => ${link_type}, ${link_type} => $uuid } );
        my $link = "<a href=\"${link_url}\">$name_space_show</a>";
        $table_row =~ s/xorux_lpar_name/$link/;
      } else {
        $table_row =~ s/xorux_lpar_name/$name_space_show/;
      }
      $table_row =~ s/xorux_val1_color/$val1_color/;
      $table_row =~ s/xorux_val1_avg/$val1_avg/;
      $table_row =~ s/xorux_val1_max/$val1_max/;

      if ( $column_count == 5 ) {
        $table_row =~ s/xorux_server_name/$cluster/;
      } elsif ( $column_count == 7 || $column_count == 8 ) {
        ( my $val2_avg, my $val2_color ) = split( "$delimiter", shift( @rrd_print ) );
        ( my $val2_max, undef )          = split( "$delimiter", shift( @rrd_print ) );

        if ( $column_count == 8 ) {
          $table_row =~ s/xorux_server_name/$cluster/;
        }
        $table_row =~ s/xorux_val2_color/$val2_color/;
        $table_row =~ s/xorux_val2_avg/$val2_avg/;
        $table_row =~ s/xorux_val2_max/$val2_max/;
        $table_row =~ s/bubble_info/$bubble_info/;
      }

      push @table_out, $table_row;
    } ## end while ( my $print_line = ...)

    if ( $updated =~ /Updated/ ) {
      $table_end_day =~ s/(COMMENT:|\\)//g;
      push @table_out, $table_end_day;
    }
    else {
      push @table_out, $table_end;
    }

    return $graphv_line;
  } ## end XenServer

  # oVirt
  if ( defined $item_ret && ( $item_ret =~ /ovirt_.*_aggr_|custom_ovirt_vm_.*/ ) ) {
    my @header;
    my $units;
    my $column_count = 7;
    my $link_type;
#custom-ovirtvm-cpu-percent
    if ( $item_ret =~ /(domain|domains_total|vm)_aggr_data/ ) {
      $link_type    = 'disk';
      @header       = ( $1 eq 'domains_total' ? 'Storage domain & Disk' : 'Disk', 'READ', 'WRITE' );
      $units        = '[MB/sec]';
    }
    elsif ( $item_ret =~ /(domain|domains_total|vm)_aggr_latency/ ) {
      $link_type    = 'disk';
      @header       = ( $1 eq 'domains_total' ? 'Storage domain & Disk' : 'Disk', 'READ', 'WRITE' );
      $units        = '[millisec]';
    }
    elsif ( $item_ret =~ /(host_nic|vm)_aggr_net/ ) {
      $link_type    = $1;
      @header       = ( 'Int', 'READ', 'WRITE' );
      $units        = '[MB/sec]';
    }
    elsif ( $item_ret =~ /(host|cluster)_aggr_(vm|host)_(cpu_core|cpu_percent|mem_used_percent|mem_used|mem_free)/ ) {
      $column_count = 4;
      $link_type    = $2;
      @header       = ( ( $2 eq 'vm' ? uc $2 : ucfirst $2 ), '', '' );

      if ( $3 eq 'cpu_core' ) { $units = '[cores]'; }
      elsif ( $3 eq 'cpu_percent' || $3 eq 'mem_used_percent' ) { $units = '[%]'; }
      else { $units = '[GB]'; }
    }
    elsif ( $item_ret =~ /custom_ovirt_vm_(cpu_core|cpu_percent|mem_used|mem_free)/ ) {
      $column_count = 5;
      @header       = ( 'Cluster', 'VM', '' );

      if ( $1 eq 'cpu_cores' ) { $units = '[cores]'; }
      elsif ( $1 eq 'cpu_percent' ) { $units = '[%]'; }
      else { $units = '[GB]'; }
    }

    my $table_header_4col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$header[0]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>
END

    my $table_header_5col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='5'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$header[0]</th>
	<th class=\"sortable header toleft\">$header[1]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>
END

    my $table_header_7col = <<END;
  <p class='custom0'>$units</p>
  <table class=\"tablesorter tablegend\" data-sortby='4 7'>
    <thead>
      <tr>
        <th class=\"sortable header toleft\">$header[0]</th>
	<th>$header[1]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th>$header[2]</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>\n
END

    my $table_row_4col = <<END;
      <tr>
	<td class=\"legsq\">xorux_val1_color</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max</td>
      </tr>
END

    my $table_row_5col = <<END;
      <tr>
	<td class=\"legsq\">xorux_val1_color</td>
	<td class=\"clickabletd\">xorux_server_name</td>
	<td class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max</td>
      </tr>
END

    my $table_row_7col = <<END;
      <tr>
	<td title=\"bubble_info\" class=\"toleft\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_val1_color</td>
	<td>xorux_val1_avg</td>
	<td>xorux_val1_max&nbsp;</td>
	<td class=\"legsq\">xorux_val2_color</td>
	<td>xorux_val2_avg</td>
	<td>xorux_val2_max</td>
      </tr>\n
END

    my $table_end     = " </tbody> </table>\n";
    my $table_end_day = <<END;
    </tbody>
    <tfoot>
      <tr>
        <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
      </tr>
    </tfoot>
  </table>\n
END

    if ( $column_count == 4 ) {
      push @table_out, $table_header_4col;
    }
    elsif ( $column_count == 5 ) {
      push @table_out, $table_header_5col;
    }
    elsif ( $column_count == 7 ) {
      push @table_out, $table_header_7col;
    }
##    push @table_out, ( $column_count == 4 ? $table_header_4col : $table_header_7col );

    while ( my $print_line = shift(@rrd_print) ) {
      my $table_row;# = ( $column_count == 4 ? $table_row_4col : $table_row_7col );

      if ( $column_count == 4 ) {
        $table_row = $table_row_4col;
      }
      elsif ( $column_count == 5 ) {
        $table_row = $table_row_5col;
      }
      elsif ( $column_count == 7 ) {
        $table_row = $table_row_7col;
      }

      ( my $val1_avg, my $item, my $name_space, my $val1_color, undef ) = split( " $delimiter ", $print_line );
      ( my $val1_max, my $bubble_info, my $cluster, my $uuid, my $uuid_owner ) = split( " $delimiter ", shift(@rrd_print) );
      $bubble_info = "" if $bubble_info eq " ";
      $item =~ s/^custom_|cluster_aggr_|host_aggr_|_used(_percent)?$|_free$//g;

      my $name_space_show = $name_space;
      $name_space_show =~ s/&&1/\//g;

#      $table_row =~ s/xorux_lpar_name/$name_space_show/;
#      $table_row =~ s/xorux_val1_color/$val1_color/ unless $column_count == 5;
      $table_row =~ s/xorux_val1_avg/$val1_avg/;
      $table_row =~ s/xorux_val1_max/$val1_max/;

      if ( $column_count == 4 ) {
        my $data_url = " data-url=\"/lpar2rrd-cgi/detail-graph.sh?host=ovirt&server=nope&lpar=$uuid&item=$item&time=$time&type_sam=$time&detail=1\"";
        my $link = "<a href=\"/lpar2rrd-cgi/detail.sh?platform=oVirt&type=$link_type\&$link_type=$uuid\">$name_space_show</a>";

        $table_row =~ s/xorux_lpar_name/$link/;
        $table_row =~ s/>xorux_val1_color/$data_url>$val1_color/
      }
      elsif ( $column_count == 5 ) {
        my $data_url = " data-url=\"/lpar2rrd-cgi/detail-graph.sh?host=ovirt&server=nope&lpar=$uuid&item=$item&time=$time&type_sam=$time&detail=1\"";
        my $link = "<a href=\"/lpar2rrd-cgi/detail.sh?platform=oVirt&type=vm&vm=$uuid\">$name_space_show</a>";
        my $link_owner = "<a href=\"/lpar2rrd-cgi/detail.sh?platform=oVirt&type=cluster_aggr&cluster=$uuid_owner\">$cluster</a>";

        $table_row =~ s/xorux_lpar_name/$link/;
        $table_row =~ s/>xorux_val1_color/$data_url>$val1_color/;
        $table_row =~ s/xorux_server_name/$link_owner/;
      }
      elsif ( $column_count == 7 ) {
        if ( $link_type eq 'host_nic' ) {
          my $link = "<a href=\"/lpar2rrd-cgi/detail.sh?platform=oVirt&type=host_nic&host=$uuid_owner&nic=$uuid\">$name_space_show</a>";
          $table_row =~ s/xorux_lpar_name/$link/;
        }
        elsif ( $link_type eq 'disk' ) {
          my $link = "<a href=\"/lpar2rrd-cgi/detail.sh?platform=oVirt&type=disk&disk=$uuid\">$name_space_show</a>";
          $table_row =~ s/xorux_lpar_name/$link/;
        }
        else {
          $table_row =~ s/xorux_lpar_name/$name_space_show/;
        }

        ( my $val2_avg, my $val2_color ) = split( "$delimiter", shift( @rrd_print ) );
        ( my $val2_max, undef )          = split( "$delimiter", shift( @rrd_print ) );

        $table_row =~ s/xorux_val1_color/$val1_color/;
        $table_row =~ s/xorux_val2_color/$val2_color/;
        $table_row =~ s/xorux_val2_avg/$val2_avg/;
        $table_row =~ s/xorux_val2_max/$val2_max/;
        $table_row =~ s/bubble_info/$bubble_info/;
      }

      push @table_out, $table_row;
    } ## end while ( my $print_line = ...)

    if ( $updated =~ /Updated/ ) {
      $table_end_day =~ s/(COMMENT:|\\)//g;
      push @table_out, $table_end_day;
    }
    else {
      push @table_out, $table_end;
    }

    return $graphv_line;
  } ## end if ( defined $item_ret && ( $item_ret =~ /^ovirt_.*_aggr_/ ) ) {

  if ( defined $item_ret && ( $item_ret =~ /wlm-cpu/ || $item_ret =~ /wlm-mem/ || $item_ret =~ /wlm-dkio/ ) )
    {
    my $line1_wlm = "<table class=\"tablesorter tablegend\" data-sortby=\"4\">
    <thead>
      <tr>

	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">Class&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_wlm = "<tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"toleft\">Class</td>
        <td>xorux_lpar_avg</td>
        <td>xorux_lpar_max</td>
      </tr>";

    my $line3_wlm = "  </tbody>  </table>";

    my $line3_wlm_day = " </tbody>
      <tfoot>
      <td colspan=\"4\" class=\"tdupdated toleft\">$updated</td>
      </tfoot> </table>\n";

    push @table_out, $line1_wlm;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_wlm;
      ( my $average, undef, my $lpar_space, my $color ) = split( " $delimiter ", $print_line );
      # ($lpar_space, my $user) = split(" ",$lpar_space);
      $print_line = shift(@rrd_print);
      # ( my $max, my $color, my $user, my $function, my $filename ) = split( " $delimiter ", $print_line );
      ( my $max, my $filename ) = split( " $delimiter ", $print_line );

      # $lpar_space =~ s/wlm-//g;
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/Class/$lpar_space/;
      $filename =~ s/\.mmm$//;
      $line2_act =~ s/xorux_lpar_color/$color/;

      #$line2_act =~ s/Type/$subtype/;
      # print STDERR "1138 \$filename $filename\n";
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_wlm_day =~ s/COMMENT://;
      $line3_wlm_day =~ s/\\//g;
      push @table_out, $line3_wlm_day;
    }
    else {
      push @table_out, $line3_wlm;
    }

    # push  @table_out, $line3_wlm;
     #print STDERR "\@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /memams-day/ || $item_ret =~ /cap_used-day/ || $item_ret =~ /cap_free-day/ || $item_ret =~ /cap_proc-day/ || $item_ret =~ /dsk_svc_as-day/ || $item_ret =~ /dsk_wait_as-day/ || $item_ret =~ /dsk_latency-day/) ) {

    # prepare table for memams day

    my $unit = "GB";
    $unit = "percent" if $item_ret =~ /cap_proc-day/;
    my $alloc_free = "Current allocated in $unit";
    $alloc_free = "Current free in $unit" if $item_ret =~ /cap_free-day/;
    $alloc_free = "Disk service time in [ms]" if $item_ret =~ /dsk_svc_as-day/;
    $alloc_free = "Disk wait time in [ms]" if $item_ret =~ /dsk_wait_as-day/;
    $alloc_free = "Disk latency time in [ms]" if $item_ret =~ /dsk_latency-day/;

    my $line1_memams_day = "<table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th>&nbsp;</th>
	<th class=\"sortable header toleft\">$alloc_free:</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Last&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_memams_day = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td title=\"bubble_info\" class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_lpar_avg</td>
	<td>xorux_lpar_max</td>
	<td>xorux_lpar_last</td>
      </tr>";
    my $line3_memams = "  </tbody> </table>";

    my $line3_memams_day = " </tbody>
    <tfoot>
      <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
    </tfoot> </table>\n";

    push @table_out, $line1_memams_day;

    my $bubble_info = "";

    while ( my $print_line = shift(@rrd_print) ) {
      my $data_url  = "";
      my $line2_act = $line2_memams_day;
      ( my $average, undef, my $lpar_space, my $color, my $rrd_file_all ) = split( " $delimiter ", $print_line );

      $print_line = shift(@rrd_print);
      my $max             = "";
      my $lpar_space_orig = "";
      if ( defined $print_line ) {
        ( $max, $bubble_info, $lpar_space_orig ) = split( " $delimiter ", $print_line );
      }
      $print_line = shift(@rrd_print);
      my $last = "";
      if ( defined $print_line ) {
        ( $last, undef ) = split( "$delimiter", $print_line );
      }
      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;
      my $lpar_url   = $lpar_space_show;
      my $query_lpar = $lpar_space_show;
      if ( defined $rrd_file_all && $rrd_file_all ne "" ) {
        my @pathx    = split( "/", $rrd_file_all );
        my $hmc_t    = @pathx[ @pathx - 2 ];
        my $server_t = @pathx[ @pathx - 3 ];
        if ( $item_ret =~ /cap_/ ) {
          $hmc_t    = @pathx[ @pathx - 4 ];
          $server_t = @pathx[ @pathx - 5 ];
        }

        # $lpar_url =~ s/ /\+/g;
        # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
        # $lpar_url =~ s/\#/%23/g;
        $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";

        if ( $item_ret =~ /cap_/ ) {

          #          my $as_lpar = @pathx[@pathx - 3]."/ASP/".$lpar_url;
          my $as_lpar = @pathx[ @pathx - 3 ] . "/ASP/" . $lpar_space_orig;

          # print STDERR "1333 detail-graph-cgi.pl \$rrd_file_all $rrd_file_all \$hmc_t $hmc_t \$server_t $server_t \$lpar_url $lpar_url \$as_lpar $as_lpar\n";
          $data_url = " data-url=\"/lpar2rrd-cgi/detail-graph.sh?host=$hmc_t&server=$server_t&lpar=$as_lpar&item=asp_cap&time=$time&type_sam=$time&detail=1\"";
        } ## end if ( $item_ret =~ /cap_/)
      } ## end if ( defined $rrd_file_all)
      else {
        $line2_act =~ s/clickabletd/toleft/;    # Configured max not
      }

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_last/$last/;
      if ( $item_ret =~ /cap_/ ) {
        $query_lpar = $lpar_url;
      }
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/>xorux_lpar_color/$data_url>$color/;
      $bubble_info = "" if !defined $bubble_info;
      $line2_act =~ s/bubble_info/$bubble_info/;
      if ( $lpar_space =~ "Configured max" ) {
        $line2_act = "<tfoot>" . $line2_act . "</tfoot>";
      }

      # print STDERR "1345 detail-graph-cgi.pl \$line2_act $line2_act \$bubble_info $bubble_info\n";
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_memams_day =~ s/COMMENT://;
      $line3_memams_day =~ s/\\//g;
      push @table_out, $line3_memams_day;
    }
    else {
      push @table_out, $line3_memams;
    }
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && ( $item_ret =~ /memams-other/ || $item_ret =~ /cap_used-other/ || $item_ret =~ /cap_free-other/ || $item_ret =~ /cap_proc-other/  || $item_ret =~ /dsk_svc_as-other/ || $item_ret =~ /dsk_wait_as-other/ || $item_ret =~ /dsk_latency-other/) ) {

    # prepare table for amsaggreg others

    my $unit = "GB";
    $unit = "percent" if $item_ret =~ /cap_proc-other/;

    my $alloc_free = "Current allocated in $unit";
    $alloc_free = "Current free in $unit" if $item_ret =~ /cap_free-other/;
    $alloc_free = "Disk service time in [ms]" if $item_ret =~ /dsk_svc_as-other/;
    $alloc_free = "Disk wait time in [ms]" if $item_ret =~ /dsk_wait_as-other/;
    $alloc_free = "Disk latency time in [ms]" if $item_ret =~ /dsk_latency-other/;

    my $line1_memams_other = "<table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th>&nbsp;</th>
  <th class=\"sortable header toleft\">$alloc_free:</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_memams_other = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
  <td title=\"bubble_info\" class=\"clickabletd\">xorux_lpar_name</td>
	<td>xorux_lpar_avg</td>
	<td>xorux_lpar_max</td>
      </tr>";
    my $line3_memams_other = "  </tbody> </table>";

    push @table_out, $line1_memams_other;
    my $bubble_info = "";

    while ( my $print_line = shift(@rrd_print) ) {
      my $data_url        = "";
      my $lpar_space_orig = "";
      my $line2_act       = $line2_memams_other;
      ( my $average, undef, my $lpar_space, my $color, my $rrd_file_all ) = split( " $delimiter ", $print_line );

      $print_line = shift(@rrd_print);
      my $max = "";
      if ( defined $print_line ) {
        ( $max, $bubble_info, $lpar_space_orig ) = split( " $delimiter ", $print_line );
      }
      my $lpar_space_show = $lpar_space;
      $lpar_space_show =~ s/&&1/\//g;
      my $lpar_url   = $lpar_space_show;
      my $query_lpar = $lpar_space_show;
      if ( defined $rrd_file_all && $rrd_file_all ne "" ) {
        my @pathx    = split( "/", $rrd_file_all );
        my $hmc_t    = @pathx[ @pathx - 2 ];
        my $server_t = @pathx[ @pathx - 3 ];
        if ( $item_ret =~ /cap_/ ) {
          $hmc_t    = @pathx[ @pathx - 4 ];
          $server_t = @pathx[ @pathx - 5 ];
        }

        # $lpar_url =~ s/ /\+/g;
        # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
        # $lpar_url =~ s/\#/%23/g;
        $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

        $query_lpar = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";

        if ( $item_ret =~ /cap_/ ) {

          #my $as_lpar = @pathx[@pathx - 3]."/ASP/".$lpar_url;
          my $as_lpar = @pathx[ @pathx - 3 ] . "/ASP/" . $lpar_space_orig;

          # print STDERR "1333 detail-graph-cgi.pl \$rrd_file_all $rrd_file_all \$hmc_t $hmc_t \$server_t $server_t \$lpar_url $lpar_url \$as_lpar $as_lpar\n";
          $data_url = " data-url=\"/lpar2rrd-cgi/detail-graph.sh?host=$hmc_t&server=$server_t&lpar=$as_lpar&item=asp_cap&time=$time&type_sam=$time&detail=1\"";
        } ## end if ( $item_ret =~ /cap_/)
      } ## end if ( defined $rrd_file_all)
      else {
        $line2_act =~ s/clickabletd/toleft/;    # Configured max not
      }

      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      if ( $item_ret =~ /cap_/ ) {
        $query_lpar = $lpar_url;
      }
      $line2_act =~ s/xorux_lpar_name/$query_lpar/;
      $line2_act =~ s/>xorux_lpar_color/$data_url>$color/;
      $bubble_info = "" if !defined $bubble_info;
      $line2_act =~ s/bubble_info/$bubble_info/;
      if ( $lpar_space =~ "Configured max" ) {
        $line2_act = "<tfoot>" . $line2_act . "</tfoot>";
      }
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    push @table_out, $line3_memams_other;
    return $graphv_line;
  } ## end if ( defined $item_ret...)

  if ( defined $item_ret && $item_ret =~ /san2-nmon/ ) {

    # prepare table for SAN2NMON

    my $line1_san2nmon = "<p class='custom0'>[$iops]</p>
  <table class=\"tablesorter tablegend\" data-sortby='4'>
    <thead>
      <tr>
	<th class=\"sortable header toleft\">Int</th>
	<th>TOTAL</th>
	<th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
	<th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2_san2nmon = "    <tr>
	<td class=\"toleft\">xorux_lpar_name</td>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td>xorux_lpar_avg</td>
	<td>xorux_lpar_max</td>
      </tr>";

    my $line3_san2nmon_day = "  </tbody>
     <tfoot> <tr>
	<td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
      </tr></tfoot>  </table>";
    my $line3_san2nmon = "  </tbody></table>";

    push @table_out, $line1_san2nmon;

    while ( my $print_line = shift(@rrd_print) ) {
      my $line2_act = $line2_san2nmon;
      ( my $average, undef, my $lpar_space, my $color ) = split( "$delimiter", $print_line );

      $print_line = shift(@rrd_print);
      ( my $max, undef ) = split( "$delimiter", $print_line );
      $line2_act =~ s/xorux_lpar_avg/$average/;
      $line2_act =~ s/xorux_lpar_max/$max/;
      $line2_act =~ s/xorux_lpar_name/$lpar_space/;
      $line2_act =~ s/xorux_lpar_color/$color/;
      push @table_out, $line2_act;
    } ## end while ( my $print_line = ...)
    if ( $updated =~ /Updated/ ) {
      $line3_san2nmon_day =~ s/COMMENT://;
      $line3_san2nmon_day =~ s/\\//g;
      push @table_out, $line3_san2nmon_day;
    }
    else {
      push @table_out, $line3_san2nmon;
    }
    return $graphv_line;
  } ## end if ( defined $item_ret...)

########        custom groups ######   &   hmc totals lpar

  if (
    defined $item_ret
    && ( $item_ret =~ /multiview-lpar/
      || $item_ret =~ /multiview-shpool/
      || $item_ret =~ /multiview-mem/
      || $item_ret =~ /multiview-osmem/
      || $item_ret =~ /multiview-ent/
      || $item_ret =~ /multiview-disk/
      || $item_ret =~ /multiview-net/
      || $item_ret =~ /multihmclpar/
      || $item_ret =~ /hyp_clustservms/)
    )
  {
    # prepare HTML table for custom group
    my $lpar_pool = "Lpar";
    $lpar_pool = "VM" if ( $item_ret =~ /multiview-disk/ || $item_ret =~ /multihmclpar_vm/ || $item_ret =~ /multiview-net/ || $item_ret =~ /multihmclpar_mem/ || $item_ret =~ /hyp_clustservms/ );
    my $max_pool       = "";
    my $max_pool_value = "";
    my $hmc_rank       = 2;        # for lpar, shpool, mem
    my $server_rank    = 3;        # for lpar, shpool, mem
    my $item_leg       = "lpar";

    my $line0 = "<p class='custom0'>Utilization in CPU cores</p>";
    ($vmware)                           && ( $line0 = "<p class='custom0'>Utilization in CPU GHz  </p>" );
    ( $item_ret =~ /multiview-disk/ )   && ( $line0 = "<p class='custom0'>Disk MB/sec  </p>" );
    ( $item_ret =~ /multiview-net/ )    && ( $line0 = "<p class='custom0'>Network MB/sec  </p>" );
    ( $item_ret =~ /multihmclpar_vm/ )  && ( $line0 = "<p class='custom0'>Utilization in CPU GHz  </p>" );
    ( $item_ret =~ /multihmclpar_mem/ ) && ( $line0 = "<p class='custom0'>Active Memory in GB  </p>" );
    ( $item_ret =~ /multihmclpar_memgrant/ ) && ( $line0 = "<p class='custom0'>Granted Memory in GB  </p>" );
    ( $item_ret =~ /multiview-mem/ )    && ( $line0 = "<p class='custom0'>Allocation in GB</p>" );
    ( $item_ret =~ /multihmclpardy/ )   && do {
      $line0    = "<p class='custom0'>CPU-ready-%/vCPU</p>";
      $item_leg = "vmw-ready";
    };
    ( $item_ret =~ /multiview-osmem/ ) && do {
      $line0       = "<p class='custom0'>Used memory in GBytes</p>";
      $hmc_rank    = 3;
      $server_rank = 4;
    };
    ( $item_ret =~ /multiview-ent-[lan|san]/ ) && do {
      $line0          = "<p class='custom0'>[MB/sec]</p>";
      $max_pool       = "<th class=\"sortable header toleft\">int&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $max_pool_value = "<td class=\"toleft\">xorux_int_adapter</td>";
      $hmc_rank       = 3;
      $server_rank    = 4;
    };
    ( $item_ret =~ /multiview-ent-san2/ ) && do {
      $line0       = "<p class='custom0'>[$iops]</p>";
      $hmc_rank    = 3;
      $server_rank = 4;
    };
    ( $item_ret =~ /multiview-shpool/ ) && do {
      $lpar_pool      = "Pool";
      $max_pool       = "<th class=\"sortable header\">Max pool&nbsp;&nbsp;&nbsp;&nbsp;</th>";
      $max_pool_value = "<td>xorux_max_pool</td>";
    };
    my $line1 = "<table class=\"tablesorter tablegend\" data-sortby='6 5'>
    <thead>
      <tr>
        <th>&nbsp;</th>
        <th class=\"sortable header toleft\">Server</th>
        <th class=\"sortable header toleft\">$lpar_pool</th>
        $max_pool
        <th class=\"sortable header\">Avrg&nbsp;&nbsp;&nbsp;&nbsp;</th>
        <th class=\"sortable header\">Max&nbsp;&nbsp;&nbsp;&nbsp;</th>
      </tr>
    </thead>
    <tbody>";
    my $line2 = "    <tr>
	<td class=\"legsq\">xorux_lpar_color</td>
	<td class=\"clickabletd\">xorux_server_name</td>
        <td class=\"clickabletd\">xorux_lpar_name</td>
        $max_pool_value
	<td>xorux_lpar_avg</td>
	<td>xorux_lpar_max</td>
      </tr>";
    my $line4_updated = "";
    if ( $updated =~ "Updated" ) {
      $line4_updated = " <tr>
        <td colspan=\"7\" class=\"tdupdated toleft\">$updated</td>
      </tr>";
    }
    my $line4_note = "</tbody> <tr> <td colspan=\"7\">(Note that for CPU dedicated LPARs is always shown their whole entitlement)</td></tr></table>\n";

    push @table_out, $line0;
    push @table_out, $line1;

    my $total_lines          = "";
    my $total_lines_to_print = "";
    my ( $average, $server_space, $lpar_space, $int, $color, $rrd_file_all );
    while ( my $print_line = shift(@rrd_print) ) {
      if ( $print_line !~ "$delimiter Total" ) {
        my $line2_act = $line2;
        if ( $item_ret !~ /multiview-ent/ ) {
          ( $average, undef, $server_space, $lpar_space ) = split( " $delimiter ", $print_line );
        }
        else {    # also multiview-ent[-lan|-san|-san2]
          ( $average, undef, $server_space, $lpar_space, $int ) = split( " $delimiter ", $print_line );
          $line2_act =~ s/xorux_int_adapter/$int/;
        }
        if ( $item_ret =~ /multiview-shpool/ ) {
          $print_line = shift(@rrd_print);
          ( my $mx_pool, undef ) = split( "$delimiter", $print_line );
          $line2_act =~ s/xorux_max_pool/$mx_pool/;
        }
        $print_line = shift(@rrd_print);
        ( my $average, $color, $rrd_file_all ) = split( "$delimiter", $print_line );
        $print_line = shift(@rrd_print);
        ( my $max, undef ) = split( "$delimiter", $print_line );

        my $lpar_space_show   = $lpar_space;
        my $server_space_show = $server_space;
        $lpar_space_show =~ s/&&1/\//g;
        $server_space_show =~ s/&&1/\//g;
        my $lpar_url     = $lpar_space_show;
        my $query_lpar   = $lpar_space_show;
        my $server_url   = $server_space_show;
        my $query_server = $server_space_show;
        if ( defined $rrd_file_all ) {

          #print STDERR "1862 clickable \$rrd_file_all $rrd_file_all \$lpar_space_show ,$lpar_space_show,\n";
          my @pathx     = split( "/", $rrd_file_all );
          my $hmc_t     = @pathx[ @pathx - $hmc_rank ];
          my $server_t  = @pathx[ @pathx - $server_rank ];
          my $file_name = @pathx[ @pathx - 1 ];
          if ( $item_ret =~ /multihmclpardy/ ) {    # for VM prepare UUID
                                                    # $lpar_url = $file_name;
            $lpar_url = $lpar_space;

            # $lpar_url =~ s/\.rrm$//;
          } ## end if ( $item_ret =~ /multihmclpardy/)
          # print STDERR "2351 \$hmc_t $hmc_t \$server_t $server_t \$file_name $file_name\n";
          if ($item_ret =~ /hyp_clustservms/) {
            $server_t = "windows/$server_t";
            $lpar_url = $file_name;
            $lpar_url =~ s/\.rrm$//;
          }
          # $lpar_url =~ s/ /\+/g;
          # $lpar_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
          # $lpar_url =~ s/\#/%23/g;
          $lpar_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

          # $server_url =~ s/ /\+/g;
          # $server_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg; # PH: keep it is it is exactly!!!
          # $server_url =~ s/\#/%23/g;
          $server_url =~ s/([^a-zA-Z0-9_.!:~*()'\''-])/sprintf("%%%02X", ord($1))/ge;

          if ( $file_name =~ /SharedPool/ ) {
            $file_name =~ s/\.rr.$//;
            $lpar_url = $file_name;
          }    # &none=none&square_item=$item_leg
          # remove lpar alias   aix1 [ala/ix1]
          my $lpar_url_without_alias = $lpar_url;
          $lpar_url_without_alias =~ s/%5B.*%5D//;
          $query_lpar   = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=$lpar_url_without_alias&item=lpar&entitle=0&none=none\">$lpar_space_show</a>";
          $query_server = "<a href=\"/lpar2rrd-cgi/detail.sh?host=$hmc_t&server=$server_t&lpar=pool&item=pool&entitle=0&none=none\">$server_space_show</a>";

          ( $lpar_space_show =~ /^CPU pool$/ ) && ( $query_lpar =~ s/lpar=CPU%20pool\&item=lpar/lpar=pool&item=pool/ );    # replace CPU pool by "pool"
          ( $lpar_url =~ /SharedPool/ ) && ( $query_lpar =~ s/item=lpar/item=pool/ );
          #      ( $server_t =~ /windows/ ) && ( $query_lpar =~ s/item=lpar/item=hyp-cpu/ ); #neco jineho
        } ## end if ( defined $rrd_file_all)
        $line2_act =~ s/xorux_lpar_avg/$average/;
        $line2_act =~ s/xorux_lpar_max/$max/;
        $line2_act =~ s/xorux_server_name/$query_server/;
        $line2_act =~ s/xorux_lpar_name/$query_lpar/;
        $line2_act =~ s/xorux_lpar_color/$color/;

        #        ( $item_ret =~ /multiview-mem/)   && ( $line2_act =~ s/legsq//);

        push @table_out, $line2_act;
      } ## end if ( $print_line !~ "$delimiter Total")
      else {
        my $total_text = "Total";
        ( my $average, undef ) = split( "$delimiter", $print_line );
        my $line2_act = $line2;
        $line2_act =~ s/clickabletd/toleft/g;    # Total not

        ( $print_line =~ /Total max/ ) && do {
          $line2_act =~ s/xorux_lpar_avg//;
          $line2_act =~ s/xorux_lpar_max//;
          $line2_act =~ s/xorux_lpar_name//;
          $line2_act =~ s/xorux_max_pool/$average/;
          $line2_act =~ s/xorux_server_name/Total max pool units/;
          $line2_act =~ s/xorux_lpar_color/#000000/;    #$color
          $total_lines .= $line2_act;
          $line2_act = "";
        };
        ( $print_line =~ /Total entitled processor cores/ ) && do {
          $line2_act =~ s/xorux_lpar_avg/$average/;
          $line2_act =~ s/xorux_lpar_max//;
          $line2_act =~ s/xorux_lpar_name//;
          $line2_act =~ s/xorux_max_pool//;
          $line2_act =~ s/xorux_server_name/Total entitled processor cores/;
          $line2_act =~ s/xorux_lpar_color/#000000/;    #$color
          $total_lines .= $line2_act;
          $line2_act = "";
        };
        ( $print_line =~ /Total ava/ ) && do {
          $line2_act =~ s/xorux_lpar_avg/$average/;
          $line2_act =~ s/xorux_lpar_max//;
          $line2_act =~ s/xorux_lpar_name//;
          $line2_act =~ s/xorux_max_pool//;
          $line2_act =~ s/xorux_server_name/Total available/;
          $line2_act =~ s/xorux_lpar_color/#000000/;    #$color
          $total_lines .= $line2_act;
          $line2_act = "";
        };
        ( $print_line =~ /Total ave/ ) && do {
          $line2_act =~ s/xorux_lpar_avg/$average/;
          $line2_act =~ s/xorux_lpar_max//;
          $line2_act =~ s/xorux_lpar_name//;
          $line2_act =~ s/xorux_max_pool//;
          $line2_act =~ s/xorux_server_name/Total average/;
          $line2_act =~ s/xorux_lpar_color//;           #$color is no color
          $total_lines .= $line2_act;
          $line2_act = "";
        };
        $total_lines_to_print .= $total_lines;
        $total_lines = "";
        push @table_out, $line2_act if ( $line2_act ne "" );
      } ## end else [ if ( $print_line !~ "$delimiter Total")]
    } ## end while ( my $print_line = ...)
    my $tbody_end = "</tbody>";
    push @table_out, $tbody_end;

    $total_lines_to_print = "<tfoot>" . $total_lines_to_print . $line4_updated . "</tfoot></table>";
    push @table_out, $total_lines_to_print;

    # print STDERR "1951 \@table_out @table_out\n";
    return $graphv_line;
  } ## end if ( defined $item_ret...)
  if ( $item_ret =~ m/power_.an/){
    return;
  }
  error( "sub ret_graph_param - not recognised item : $item_ret " . __FILE__ . ":" . __LINE__ );
  #print STDERR "1700 detail-graph-cgi.pl ??? \@table_out @table_out\n";
} ## end sub ret_graph_param

sub graph_lpar {
  my $host         = shift;
  my $server       = shift;
  my $lpar         = shift;
  my $type         = shift;
  my $name_out     = shift;
  my $type_sam     = shift;
  my $upper        = shift;
  my $detail       = shift;
  my $graph_hours  = shift;
  my $entitle      = shift;             # 1 do not show entitle area
  my $start_unix   = shift;
  my $end_unix     = shift;
  my $super_detail = shift;
  my $t            = "COMMENT:\" \"";
  my $t2           = "COMMENT:\\n";
  my $step_new     = $step;
  my $last         = "COMMENT:\" \"";
  my $rigid        = "\n --rigid";

  # print STDERR "2475 detail-graph-cgi.pl sub graph_lpar ,$host,$server,$lpar,$type,$item,$type_sam,$detail,$graph_hours \$hyperv $hyperv \$vmware $vmware\n";

  my $type_orig = $type;
  # print STDERR "2189 \$detail $detail \$type $type\n";

  #  my $in_cores = 1; # graph for vmware is in cores
  my $in_cores     = 0;    # graph for vmware is in MHz
                           #my $in_cores = 2;  # graph for vmware has right axis
  my $one_core_hz  = 1;
  my $one_core_mhz = 1;
  my $one_core_ghz = 1;

  # if (time % 2 == 0) {$in_cores = 1};  # just for fun

  #  my $vmware = 0;  # is global variable

  my $keep_virtual = 0;
  if ( $ENV{KEEP_VIRTUAL} ) {
    $keep_virtual = $ENV{KEEP_VIRTUAL};    # keep number of virt processors in RRD --> etc/.magic
  }
  else {
    $keep_virtual = 0;
  }

  #print "001 $host:$server:$lpar:$name_out:$type_sam\n";
  # print STDERR "v graph_lpar\$upper $upper\n";

  if ( !defined $upper || $upper eq '' || $upper =~ "entitle" || $upper =~ m/none/ || $upper == 0 ) {    # entitle  badly sent param
    $upper = 0.1;
  }

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $units_exponent = "--units-exponent=1.00";
  $units_exponent = "" if $vmware;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lpar: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $super_detail == 3 ) {

        # GFR request
        $xgrid = " --x-grid=MINUTE:60:HOUR:1:HOUR:2:0:%H ";
      }
      elsif ( $detail != 10 ) {
        $xgrid = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $header_prefix = "LPAR:";
  my $rrd           = "$wrkdir/$server/$host/$lpar.rr$type_sam";

  if ($vmware) {
    if ( $detail == 1 ) {
      $lpar = human_vmware_name( $lpar, "neg" );
    }
    $rrd           = "$wrkdir/$all_vmware_VMs/$lpar.rrm";
    $header_prefix = "VM:";
  } ## end if ($vmware)
  if ($hyperv) {
    if ( $detail == 1 ) {
      $lpar = human_vmware_name( $lpar, "neg", $server );
    }
    $rrd           = "$wrkdir/$server/$all_hyperv_VMs/$lpar.rrm";
    $header_prefix = "VM:";
  } ## end if ($hyperv)
  if ($hitachi) {
    $rrd           = "$wrkdir/$server/$host/$lpar.hlm";
  } ## end if ($hitachi)

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $lpar_slash = human_vmware_name($lpar,"",$server);
  if ($vmware) {
    $lpar_slash =~ s/\(.*\)//;
  }

  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt      = " ";
    $header_prefix = "";
  }

  my $header = "$header_prefix $lpar_slash_nmon :$last_txt$text";
  my $header_only = $header;

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
      if ( $detail == 10 ) {
        $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  my $vcpu = 0;

  if ( ( $type =~ "d" && $graph_hours == 0 ) || $vmware ) { # || $hyperv ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    if ( $type =~ "d" ) {
      $l =~ s/:/\\:/g;
      $t = "COMMENT:Updated\\:\" $l \"";
    }

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );

    # print STDERR "\@row_arr @row_arr\n";
    my $m = "";
    my $i = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );
        if ($vmware) {

          # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $utiltot_mhz = 1;
            my $utiltot_ghz = 1;
            $utiltot_mhz = sprintf( "%e", $m_arr[2] );
            $utiltot_ghz = $utiltot_mhz / 1000;
            $one_core_hz = sprintf( "%e", $m_arr[3] );
            if ($in_cores) {
              my $util = sprintf( "%5.2f", $utiltot_mhz / $one_core_hz * 1000000 );
              if ( $type !~ "d" ) {next}
              $last = "COMMENT:\"Last CPU usage in cores      $util\"";
              if ( $in_cores == 2 ) {
                my $utlt_mhz = sprintf( "%5.0f", $utiltot_mhz );
                my $utlt_ghz = sprintf( "%5.2f", $utiltot_ghz );
                $last = "COMMENT:\"Last CPU usage                $util    $utlt_ghz\"";
              }
            } ## end if ($in_cores)
            else {
              $utiltot_mhz = $utiltot_mhz / 1000;
              my $util = sprintf( "%5.2f", $utiltot_mhz );
              $last = "COMMENT:\"Last utilization            $util\"";
            }

            # get vCPU
            # print STDERR "\$m_arr[18] $m_arr[18]\n";
            $vcpu = $m_arr[18];
            $vcpu = sprintf( "%e", $vcpu );
            $vcpu = $vcpu * 1;
            if ( !defined $vcpu ) {
              $vcpu = 'not defined';
            }
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end if ($vmware)
	elsif ($hitachi) {
	  if ( $m_arr[1] =~ /\d/ ) {
	    my $util = sprintf( "%5.2f", sprintf( "%e", $m_arr[1] ));
	    $last = "COMMENT:\"Last utilization in CPU cores  $util\"";
	  }
	} ## end if ($hitachi)
        else {
          # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $entitled_cycles = 1;
            my $capped_cycles   = 1;
            my $uncapped_cycles = 1;
            my $curr_proc       = 1;

            if ( $keep_virtual == 1 ) {

              # when KEEP_VIRTUAL then virtual_procs is the second field
              $entitled_cycles = sprintf( "%e", $m_arr[3] );
              $capped_cycles   = sprintf( "%e", $m_arr[4] );
              $uncapped_cycles = sprintf( "%e", $m_arr[5] );
              $curr_proc       = sprintf( "%e", $m_arr[1] );
            } ## end if ( $keep_virtual == ...)
            else {
              $entitled_cycles = sprintf( "%e", $m_arr[2] );
              $capped_cycles   = sprintf( "%e", $m_arr[3] );
              $uncapped_cycles = sprintf( "%e", $m_arr[4] );
              $curr_proc       = sprintf( "%e", $m_arr[1] );
            } ## end else [ if ( $keep_virtual == ...)]
            if ( $entitled_cycles != 0 ) {
              my $util = sprintf( "%5.2f", ( $capped_cycles + $uncapped_cycles ) / $entitled_cycles ) * $curr_proc;

              #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");

              $last = "COMMENT:\"Last utilization in CPU cores  $util\"";
            } ## end if ( $entitled_cycles ...)
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end else [ if ($vmware) ]
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( ( $type =~ "d" &&...))

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  #  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  $vertical_label = "--vertical-label=\"CPU usage in Hz\"" if $vmware && !$in_cores;
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # print STDERR "creating graph : $host:$server:$lpar_slash:$type_sam:$type:$name_out:$rrd:$entitle:$item \$header $header\n" if $DEBUG ;

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  if ( $in_cores == 2 && $vmware ) {

    #  $cmd .= " --right-axis-label \"CPU usage in MHz\"";
    $one_core_mhz = int( $one_core_hz / 1000000 );
    $one_core_ghz = $one_core_mhz / 1000;
    if ($yes_right_axis) {
      $cmd .= " --right-axis $one_core_ghz:0";
      $cmd .= " --right-axis-format %1.2lf";
    }
  } ## end if ( $in_cores == 2 &&...)

  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper";
  $cmd .= " $vertical_label";
  $cmd .= " $units_exponent";           #--units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  #  for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;                 #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "", \@rrd_files, $cgi );

  # print STDERR "2505 lpm_find_files is returning @rrd_files\n";

  my $i = -1;                           # iterators for LPM cycle
  my $j;

  my $cores_mhz = "cores";              # for vmware graph legend
  if ( $in_cores == 2 ) {
    $cores_mhz = "     ";
  }
  my $gprint_form = "8.2lf";

  if ($vmware) {

    foreach $rrd (@rrd_files) {         # LPM cycle
      chomp($rrd);
      $i++;
      $j = $i - 1;
      $rrd =~ s/:/\\:/g;

      $cmd .= " DEF:ent${i}=\"$rrd\":CPU_Alloc:AVERAGE";
      $cmd .= " DEF:utl${i}=\"$rrd\":CPU_usage:AVERAGE";
      $cmd .= " DEF:hz${i}=\"$rrd\":host_hz:AVERAGE";
      $cmd .= " DEF:vcpu${i}=\"$rrd\":vCPU:AVERAGE";
      if ( $i == 0 ) {
        $cmd .= " CDEF:entr${i}=ent${i}";
        $cmd .= " CDEF:utlr${i}=utl${i}";
        $cmd .= " CDEF:hzr${i}=hz${i}";
        $cmd .= " CDEF:vcpur${i}=vcpu${i}";
        next;
      } ## end if ( $i == 0 )
      $cmd .= " CDEF:entr${i}=entr${j},UN,ent${i},entr${j},IF";
      $cmd .= " CDEF:utlr${i}=utlr${j},UN,utl${i},utlr${j},IF";
      $cmd .= " CDEF:hzr${i}=hzr${j},UN,hz${i},hzr${j},IF";
      $cmd .= " CDEF:hzr${i}=vcpur${j},UN,vcpu${i},vcpur${j},IF";
    } ## end foreach $rrd (@rrd_files)
    $cmd .= " CDEF:cpu_entitl_mhz=entr${i}";
    $cmd .= " CDEF:utiltot_mhz=utlr${i}";
    $cmd .= " CDEF:one_core_hz=hzr${i}";
    $cmd .= " CDEF:vcpu_num=vcpur${i}";

    if ($in_cores) {
      $cmd .= " CDEF:utiltot=utiltot_mhz,one_core_hz,/,1000000,*";
      $cmd .= " CDEF:cpu_entitl=cpu_entitl_mhz,one_core_hz,/,1000000,*";
    }
    else {    # in GHz
      $cmd .= " CDEF:utiltotgp=utiltot_mhz,1000,/";
      $cmd .= " CDEF:cpu_entitlgp=cpu_entitl_mhz,1000,/";
      $cmd .= " CDEF:utiltot=utiltot_mhz,1000,*,1000,*";
      $cmd .= " CDEF:cpu_entitl=cpu_entitl_mhz,1000,*,1000,*";
      $cores_mhz   = "     ";
      $gprint_form = "5.1lf";
    } ## end else [ if ($in_cores) ]
  } ## end if ($vmware)
  elsif ($hitachi) {
    $cmd .= " DEF:cores=\"$rrd\":cores:AVERAGE";
  } ## end if ($hitachi)
  else {
    foreach my $rrd_tot (@rrd_files) {    # LPM cycle
      chomp($rrd_tot);

      # LPM identifies all servers, however not all HMC under servers
      # Add here data from all HMcs
      my $lpar_name  = basename($rrd_tot);
      my $server_dir = dirname($rrd_tot) . "/..";
      my $file_pth   = "$server_dir/*/$lpar_name";

      $file_pth =~ s/ /\\ /g;
      my $no_name = "";

      my @files = (<$file_pth$no_name>);    # unsorted, workaround for space in names
      foreach $rrd (@files) {               # LPM alias cycle
        chomp($rrd);

        $i++;
        $j = $i - 1;
        $rrd =~ s/:/\\:/g;

        $cmd .= " DEF:cur${i}=\"$rrd\":curr_proc_units:AVERAGE";
        $cmd .= " DEF:ent${i}=\"$rrd\":entitled_cycles:AVERAGE";
        $cmd .= " DEF:cap${i}=\"$rrd\":capped_cycles:AVERAGE";
        $cmd .= " DEF:uncap${i}=\"$rrd\":uncapped_cycles:AVERAGE";
        if ( $i == 0 ) {
          $cmd .= " CDEF:curr${i}=cur${i}";
          $cmd .= " CDEF:entr${i}=ent${i}";
          $cmd .= " CDEF:capr${i}=cap${i}";
          $cmd .= " CDEF:uncapr${i}=uncap${i}";
          next;
        } ## end if ( $i == 0 )
        $cmd .= " CDEF:curr${i}=curr${j},UN,cur${i},curr${j},IF";
        $cmd .= " CDEF:entr${i}=entr${j},UN,ent${i},entr${j},IF";
        $cmd .= " CDEF:capr${i}=capr${j},UN,cap${i},capr${j},IF";
        $cmd .= " CDEF:uncapr${i}=uncapr${j},UN,uncap${i},uncapr${j},IF";
      } ## end foreach $rrd (@files)
    } ## end foreach my $rrd_tot (@rrd_files)
    $cmd .= " CDEF:cur=curr${i}";
    $cmd .= " CDEF:ent=entr${i}";
    $cmd .= " CDEF:cap_peak=capr${i}";
    $cmd .= " CDEF:uncap=uncapr${i}";

    # filtering peaks caused by LPM or changing entitled, if cap CPU util is > entitled --> UNKN
    # usualy cap counter is affected only
    # sometimes might happen that even in normal load capped util is little higher than entitled! Therefore using 1.2
    $cmd .= " CDEF:cap=cap_peak,ent,/,1.2,GT,UNKN,cap_peak,IF";

    $cmd .= " CDEF:tot=cap,uncap,+";
    $cmd .= " CDEF:util=tot,ent,/,$cpu_max_filter,GT,UNKN,tot,ent,/,IF";
    $cmd .= " CDEF:utilperct=util,100,*";
    $cmd .= " CDEF:utiltot=util,cur,*";

  } ## end else [ if ($vmware) ]

  #  if ($in_cores == 2 && $vmware) {
  #     $cmd .= " COMMENT:\"   Average                    cores    GHz (right axis)\\n\"";
  #  }
  if ($vmware) {
    $cmd .= " COMMENT:\"    GHz                     Avrg     Max\\n\"";
  }
  else {
    $cmd .= " COMMENT:\"   Average   \\n\"";
  }
  if ( $entitle != 1 && !$hitachi) {

    # default one, show entitlement area
    # else do not show entitled area --> customers request

    if ( !$vmware ) {
      $cmd .= " AREA:cur#00FF00:\" Entitled processor cores\"";
      $cmd .= " GPRINT:cur:AVERAGE:\" %5.2lf\"";
      $cmd .= " $t2";
    }
    else {
      $cmd .= " AREA:cpu_entitl#00FF00:\" Reserved CPU $cores_mhz   \"";
      $cmd .= " GPRINT:cpu_entitlgp:AVERAGE:\" %$gprint_form\"";
      $cmd .= " GPRINT:cpu_entitlgp:MAX:\" %$gprint_form\"";
      if ( $in_cores == 2 ) {
        $cmd .= " CDEF:cpu_entitl_ghz=cpu_entitl_mhz,1000,/";
        $cmd .= " GPRINT:cpu_entitl_ghz:AVERAGE:\" %5.2lf\"";
      }
      $cmd .= " $t2";
    } ## end else [ if ( !$vmware ) ]
  } ## end if ( $entitle != 1 )

  if ($vmware) {
    $cmd .= " LINE1:utiltot#FF0000:\" CPU usage    $cores_mhz   \"";
    $cmd .= " GPRINT:utiltotgp:AVERAGE:\" %$gprint_form\"";
    $cmd .= " GPRINT:utiltotgp:MAX:\" %$gprint_form\"";
    if ( $in_cores == 2 ) {
      $cmd .= " CDEF:utiltot_ghz=utiltot_mhz,1000,/";
      $cmd .= " GPRINT:utiltot_ghz:AVERAGE:\" %5.2lf\"";
    }
  } ## end if ($vmware)
  elsif ($hitachi) {
    $cmd .= " LINE1:cores#FF0000:\" Utilization in CPU cores\"";
    $cmd .= " GPRINT:cores:AVERAGE:\" %5.2lf\"";
  }
  else {
    $cmd .= " LINE1:utiltot#FF0000:\" Utilization in CPU cores\"";
    $cmd .= " GPRINT:utiltot:AVERAGE:\" %5.2lf\"";
  }
  $cmd .= " $t2";

  if ( $entitle != 1 && !$hitachi) {

    # default one, show entitlement area
    # else do not show entitled area --> customers request

    if ( !$vmware ) {
      $cmd .= " COMMENT:\"Entitled CPU utilization  \"";
      $cmd .= " GPRINT:utilperct:AVERAGE:\" %5.1lf %%\"";
      $cmd .= " $t2";
    }
    else {
      #      $cmd .= " COMMENT:\"   CPU usage of Reserved   \"";
      #      $cmd .= " CDEF:cpu_entitl_proc=utiltot,cpu_entitl,/,100,*";
      $cmd .= " COMMENT:\"    vCPU in units      \"";
      $cmd .= " GPRINT:vcpu_num:AVERAGE:\" %4.0lf\"";
      $cmd .= " GPRINT:vcpu_num:MAX:\" %5.0lf\"";
      $cmd .= " $t2";
    } ## end else [ if ( !$vmware ) ]

  } ## end if ( $entitle != 1 )

  $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #  "VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_lpar : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_lpar

sub graph_vm_vmotion {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $upper       = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;             # 1 do not show entitle area
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \"";
  my $tl;                              # for clickable legend
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT:\" \"";
  my $rigid      = "\n --rigid";
  my $act_time_u = time();
  my $req_time   = 0;

  # print STDERR "2610 sub graph_lpar_vm ,$host,$server,$lpar,$type,$item,$type_sam,$graph_hours,$vmware,$hyperv\n";

  #  my $in_cores = 1; # graph for vmware is in cores
  my $in_cores     = 0;    # graph for vmware is in MHz
                           #my $in_cores = 2;  # graph for vmware has right axis
  my $one_core_hz  = 1;
  my $one_core_mhz = 1;
  my $one_core_ghz = 1;

  if ( $upper eq '' || $upper =~ /entitle/ || $upper == 0 ) {
    $upper = 0.1;
  }
  $upper = 100 if $vmware;    # for usage in %

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lpar: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  my $all_vms = $all_vmware_VMs;
  $all_vms = $all_hyperv_VMs if $hyperv;

  my $rrd = "";
  $rrd = "$wrkdir/$all_vms/$lpar.rrm";
  $rrd = "$wrkdir/$server/$all_vms/$lpar.rrm" if $hyperv;
  # print STDERR "3047  detail-graph-cgi.pl \$host $host \$server $server \$lpar $lpar\n";

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  if ( $detail == 1 ) {
    $lpar = human_vmware_name( $lpar, "neg",$server );
  }

  my $lpar_slash = human_vmware_name($lpar,"",$server);

  # if want to remove ( ... )
  $lpar_slash =~ s/\(.*\)//;

  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  my $lpar_slash_nmon_25 = $lpar_slash_nmon;

  # max 25 chars
  $lpar_slash_nmon_25 = substr( $lpar_slash_nmon_25, 0, 24 );
  #
  my $last_txt = "$lpar_slash_nmon:last ";
  my $head     = "VM";
  my $header   = "$head:Motion:$last_txt$text";

  #  my $last_txt = " last ";
  my $header_prefix = "VM:";
  if ( $detail == 2 ) {
    $last_txt      = "";
    $header_prefix = "Motion";
    $header = "$header_prefix:$lpar_slash_nmon:$last_txt$text";
  }
  if (!-d "$wrkdir/$server/$host") {
    # find host (= vcenter_name) cus $host contains cluster name from DSHB
    my @ppath = `ls $wrkdir/$server`;
    $host = "";
    foreach my $item (@ppath) {
      chomp $item;
      # print STDERR "2836 \$item ,$item, $wrkdir/$item\n";
      next if $item eq "";
      if (-d "$wrkdir/$server/$item") {
        $host = $item;
        last
      }
    }
    if ($host eq "") {
      error( "no host: Graph vmotion error server $server hosts:@ppath: " . __FILE__ . ":" . __LINE__ );
      return;
    }
    # print STDERR "2848 \$host,$host,\$server,$server,\$lpar,$lpar,\n";
  }

  #  my $header = "$header_prefix $lpar_slash_nmon :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      $start_time = $start_unix;
      $req_time   = $start_time;    # set here too
      $end_time   = $end_unix;
      $xgrid      = "";             # just a trick

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  # if (($type =~ "d" && $graph_hours == 0 ) || $vmware) { # always for vmware
  my $rrd_last = rrd_file_actual("$rrd");
  RRDp::cmd qq(last "$rrd_last");
  my $last_tt = RRDp::read;
  chomp($$last_tt);
  my $l = localtime($$last_tt);

  # following must be for RRD 1.2+
  if ( $type =~ "d" ) {
    $tl = "Updated:  $l";
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";
  }

  # get LAST value from RRD
  RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
  my $row = RRDp::read;
  chomp($$row);
  my @row_arr = split( /\n/, $$row );
  my $m       = "";
  my $i       = 0;

  foreach $m (@row_arr) {
    chomp($m);
    $i++;
    if ( $i == 3 ) {
      my @m_arr = split( / /, $m );

      #        if ($vmware) {
      # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
      if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ ) {

        #print "m : $m\n" if $DEBUG ;
        #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
        my $utiltot_mhz = 1;
        my $utiltot_ghz = 1;
        $utiltot_mhz = sprintf( "%e", $m_arr[2] );
        $utiltot_ghz = $utiltot_mhz / 1000;
        $one_core_hz = sprintf( "%e", $m_arr[3] );
        if ($in_cores) {
          my $util = sprintf( "%5.2f", $utiltot_mhz / $one_core_hz * 1000000 );
          if ( $type !~ "d" ) {next}
          $last = "COMMENT:\"Last CPU usage in cores      $util\"";
          if ( $in_cores == 2 ) {
            my $utlt_mhz = sprintf( "%5.0f", $utiltot_mhz );
            my $utlt_ghz = sprintf( "%5.2f", $utiltot_ghz );
            $last = "COMMENT:\"Last CPU usage                $util    $utlt_ghz\"";
          }
        } ## end if ($in_cores)
        else {
          my $util = sprintf( "%5.0f", $utiltot_mhz );
          $last = "COMMENT:\"Last utilization in CPU MHz $util\"";
        }
      } ## end if ( $m_arr[1] =~ /\d/...)

      #        }
    } ## end if ( $i == 3 )
  } ## end foreach $m (@row_arr)

  #   }
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=\"CPU cores\"";

  # $vertical_label="--vertical-label=\"CPU usage in Hz\"" if $vmware && !$in_cores;
  $vertical_label = "--vertical-label=\"CPU usage in %\"" if $vmware && !$in_cores;
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # print STDERR "3204 creating graph : $host:$server:$lpar_slash:$lpar:$type_sam:$type:$name_out:$rrd:$entitle:$item\n" ;

  if ( $host eq "nope" ) {               # from historical report, just for now, you must think again situation in hist reports !
    my @menu;
    read_menu( \@menu );
    # print STDERR "2788 menu.txt @menu\n";
    my @matches = grep { /^L/ && /$lpar/ } @menu;
    # print STDERR "2790 @matches\n";
    ( undef, $host ) = split( "host=", $matches[0] );
    ( $host, undef ) = split( "&", $host );

    # print STDERR "2793 detail-graph-cgi.pl \$host $host\n";
  } ## end if ( $host eq "nope" )
  $host =~ s/&/\\&/g;                    # otherwise "grep" complaints
  my @path_lines = ();
  if ($vmware) {
    @path_lines = `grep "$lpar" "$wrkdir"/*/$host/VM_hosting.vmh`;    # attention: $host is vcenter
  }                                                                    # print STDERR "2796 detail-graph-cgi.pl \@path_lines @path_lines\n";
  elsif ($hyperv) {
    @path_lines = `grep "$lpar" "$wrkdir/$server"/*/VM_hosting.vmh`;
  }
  else {
    error( "$server $host $lpar : Graph error : vmotion but neither vmware not hyperv : exiting" . __FILE__ . ":" . __LINE__ );
    exit;
  }
  # print STDERR "3229 detail-graph-cgi.pl \@path_lines @path_lines\n";
  my %lines = (); # keeping info: {server_name}[$inx]='start=1234567890:end=1234567890'
                  # typical data - always dual lines because esxi name | uuid, and possible end .. start .. end ..
                  # /home/lpar2rrd/dev/data/10.100.1.24/10.100.1.30/VM_hosting.vmh:502ab669-4c97-778a-7f5f-d9552e6733f3:start=1441406400
                  # /home/lpar2rrd/dev/data/0aef332e-09cf-11e3-8a09-5cf3fcdd4404/10.100.1.30/VM_hosting.vmh:502ab669-4c97-778a-7f5f-d9552e6733f3:start=1441406400
                  # /home/lpar2rrd/dev/data/0aef332e-09cf-11e3-8a09-5cf3fcdd4404/10.100.1.30:2430/VM_hosting.vmh:502ab669-4c97-778a-7f5f-d9552e6733f3:start=1441406400 # could be port
  # in case hyperv
  #                 /home/lpar2rrd/lpar2rrd/data/windows/domain_ad.xorux.com/HVNODE02/VM_hosting.vmh:8127B1FB-FFD6-4A9F-82BC-36CF9F18F03D:start=1532072416

  foreach my $line (@path_lines) {
    chomp($line);

    # print STDERR "3244, $line\n";
    ( my $server_path, my $time_part ) = split( "hosting.vmh:", $line );
    ( undef, $time_part ) = split( ":", $time_part, 2 );

    my $server_tmp = dirname($server_path);
    $server_tmp = dirname($server_tmp) if not $hyperv;
    my $server     = basename($server_tmp);
    next if $server =~ m{.{8}-.{4}-.{4}-.{4}-.{12}};     # skip link uuid server line

    # print STDERR "3253, $line\n";
    if ( index( $time_part, "start=" ) == -1 ) {
      next;                   #some tresh
    }
    my @st_end = split( "start=", $time_part );

    # print STDERR "2711 \@st_end @st_end\n";
    my $inx = 0;
    foreach my $st_line (@st_end) {

      # print STDERR "3220 \$st_line $st_line\n";
      $st_line =~ s/:$//;
      if ( $st_line eq "" ) {
        next;                 # here the 1st atom (empty)
      }

      if ( index( $st_line, "end=" ) == -1 ) {                         # only one start without end
        # there can be more lines only with start eg. some non-standard operation with ESXi when manually transferred to other cluster or vCenter
        # in any case: 'end' can not be newer than ESXi pool
        my $clean_server_path = $server_path;
        $clean_server_path =~ s/VM_//;
        my $last_mod_server_pool_time = (stat ("$clean_server_path/pool.rrm"))[9];
        # print STDERR "3275 $clean_server_path/pool.rrm\n";
        my $diff_end_time = 0;
        $diff_end_time = $last_mod_server_pool_time if ($last_mod_server_pool_time < $end_time && $last_mod_server_pool_time > $st_line);
        if ($diff_end_time > 0) {
          $lines{$server}[$inx] = "start=$st_line:end=$diff_end_time";
        }
        else {
          $lines{$server}[$inx] = "start=$st_line:end=$end_time";
        }
        $inx++;
        next;
      }
      $lines{$server}[$inx] = "start=$st_line";
      $inx++;
    } ## end foreach my $st_line (@st_end)
  } ## end foreach my $line (@path_lines)

  # print STDERR Dumper(\%lines);

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  if ( $in_cores == 2 && $vmware ) {

    #  $cmd .= " --right-axis-label \"CPU usage in MHz\"";
    $one_core_mhz = int( $one_core_hz / 1000000 );
    $one_core_ghz = $one_core_mhz / 1000;
    if ($yes_right_axis) {
      $cmd .= " --right-axis $one_core_ghz:0";
      $cmd .= " --right-axis-format %1.2lf";
    }
  } ## end if ( $in_cores == 2 &&...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper";
  $cmd .= " $vertical_label";

  #  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  my $color_index = -1;
  my $s           = 101; #in case too many v-motions
  if ($vmware) {

    #    $cmd .= " DEF:utiltot_mhz=\"$wrkdir/$all_vms/$lpar.rrm\":CPU_usage:AVERAGE"; # only once

    #  @ds    = ("CPU_usage_Proc","vCPU","host_hz","CPU_usage")   if $item eq "vmw-proc";
    my $kbmb = 100;

    $cmd .= " DEF:ids0=\"$wrkdir/$all_vms/$lpar.rrm\":CPU_usage_Proc:AVERAGE";
    $cmd .= " DEF:ids1=\"$wrkdir/$all_vms/$lpar.rrm\":vCPU:AVERAGE";
    $cmd .= " DEF:ids2=\"$wrkdir/$all_vms/$lpar.rrm\":host_hz:AVERAGE";
    $cmd .= " DEF:ids3=\"$wrkdir/$all_vms/$lpar.rrm\":CPU_usage:AVERAGE";

    $cmd .= " CDEF:CPU_usage_Proc=ids0,$kbmb,/";    # orig
    $cmd .= " CDEF:pageout_b_nf=ids1,$kbmb,/";

    $cmd .= " CDEF:vCPU=ids1,1,/";                                                               # number
    $cmd .= " CDEF:host_MHz=ids2,1000,/,1000,/";                                                 # to be in MHz
    $cmd .= " CDEF:CPU_usage=ids3,1,/";                                                          # MHz
    $cmd .= " CDEF:CPU_usage_res=CPU_usage,host_MHz,/,vCPU,/,100,*";                             # usage proc counted
                                                                                                 #    $cmd .= " CDEF:pagein_b=CPU_usage_res,1,/"; # counted
                                                                                                 #    $cmd .= " CDEF:pagein_b=CPU_usage_Proc,1,/"; # orig from counter metric
    $cmd .= " CDEF:pagein_b_raw=CPU_usage_Proc,UN,CPU_usage_res,CPU_usage_Proc,IF";
    $cmd .= " CDEF:pagein_b=pagein_b_raw,UN,UNKN,pagein_b_raw,100,GT,100,pagein_b_raw,IF,IF";    # cut more than 100%, VMware does the same
    $cmd .= " CDEF:utiltot_mhz=pagein_b,1,*";
  } ## end if ($vmware)
  if ($hyperv) {
    $cmd .= " DEF:utiltot_perc=\"$wrkdir/$server/$all_vms/$lpar.rrm\":PercentTotalRunTime:AVERAGE";
    $cmd .= " DEF:utiltot_time=\"$wrkdir/$server/$all_vms/$lpar.rrm\":Timestamp_PerfTime:AVERAGE";
    $cmd .= " CDEF:utiltot_mhz=utiltot_perc,utiltot_time,/,1,/";
  }

  #  $cmd .= " COMMENT:\\\"  VM                        Server            Util[GHz] AVG      MAX \\l\\\"" if $detail == 1;
  $cmd .= " COMMENT:\\\"  VM                        Server            Util[%]   AVG      MAX \\l\\\"" if ($detail == 1 || $detail == 7) && $vmware;
  $cmd .= " COMMENT:\\\"  VM                        Server                      AVG      MAX \\l\\\"" if $detail == 1 && !$vmware;
  while ( my ( $key, $value ) = each %lines ) { # for every server
    # print STDERR "2805 detail-graph-cgi.pl key: $key, value: $lines{$key}[0]\n"; # $lines{$key}[1]\n";

    $color_index++;
    my $cmd_tmp = "";
    my ( $t_start, $t_end );
    my $inx = 0;
    my $i   = 1;
    my $k   = 0;

    while ( defined $lines{$key}[$inx] ) {                                                       # for all times
      ( undef, $t_start, $t_end ) = split( "=", $lines{$key}[$inx] );
      $inx++;
      $t_end   = substr( $t_end,   0, 10 );
      $t_start = substr( $t_start, 0, 10 );
      $t_start = $t_start * 1;
      $t_end   = $t_end * 1;

      # print STDERR "\$start_time $start_time \$t_start $t_start \$end_time $end_time \$t_end $t_end\n";
      if ( $t_start < $start_time ) { $t_start = $start_time }
      if ( $t_end > $end_time )     { $t_end   = $end_time }
      if ( $t_start >= $t_end )     {next}

      # print STDERR "graphing $t_start $t_end\n";
      $cmd_tmp .= " CDEF:utiltot_mhzt${i}${s}=TIME,$t_start,GE,TIME,$t_end,LE,+,2,EQ,utiltot_mhz,UNKN,IF";    #temp
      if ( $i == 1 ) {
        $cmd_tmp .= " CDEF:utiltot_mh${i}${s}=utiltot_mhzt${i}${s},1,*";
      }
      else {
        $cmd_tmp .= " CDEF:utiltot_mh${i}${s}=utiltot_mh${k}${s},UN,utiltot_mhzt${i}${s},utiltot_mh${k}${s},IF";
      }
      $i++;
      $k++;
    } ## end while ( defined $lines{$key...})
    if ( $t_end <= $start_time ) { # out of start time of picture
      next;
    }
    if ( $cmd_tmp eq "" ) { # nothing to graph
      next;
    }
    $cmd .= "$cmd_tmp";
    $i--;

    #    $cmd .= " CDEF:utiltot_ghz${i}${s}=utiltot_mh${i}${s},1000,/"; # GHz
    #    $cmd .= " CDEF:utiltot_hz${i}${s}=utiltot_mh${i}${s},1000000,*"; # Hz

    $cmd .= " CDEF:utiltot_ghz${i}${s}=utiltot_mh${i}${s},1,*";    # GHz
    $cmd .= " CDEF:utiltot_hz${i}${s}=utiltot_mh${i}${s},1,*";     # Hz

    my $lpar_space = $lpar_slash_nmon;

    # to keep same count of characters
    $lpar_space =~ s/\\:/:/g;
    $lpar_space = sprintf( "%-25s", $lpar_space );

    # max 25 chars
    $lpar_space = substr( $lpar_space, 0, 24 );
    my $lpar_space_colon = $lpar_space;
    $lpar_space_colon =~ s/:/\\:/g;
    #
    my $server_space = $key;

    # to keep same count of characters
    $server_space =~ s/\\:/:/g;
    $server_space = sprintf( "%-25s", $server_space );
    $server_space = substr( $server_space, 0, 24 );
    my $server_space_colon = $server_space;
    $server_space_colon =~ s/:/\\:/g;
    my $lpar_server_legend = "$lpar_space  $server_space";
    $lpar_server_legend =~ s/:/\\:/g;
    $lpar_server_legend =~ s/%/%%/g;    # anti '%']
    my $host_colon = $host;
    $host_colon =~ s/:/\\:/g;

    if ( $detail == 1 || $detail == 7) {
      $cmd .= " LINE2:utiltot_hz${i}${s}$color[$color_index]:\"$lpar_server_legend\"";
    }
    else {
      $cmd .= " LINE2:utiltot_hz${i}${s}$color[$color_index]:";
    }
    if ($vmware) {
      $cmd .= " GPRINT:utiltot_ghz${i}${s}:AVERAGE:\"%5.2lf \"" if $detail == 1 || $detail == 7;
      $cmd .= " GPRINT:utiltot_ghz${i}${s}:MAX:\" %5.2lf \""    if $detail == 1 || $detail == 7;

      $cmd .= " PRINT:utiltot_ghz${i}${s}:AVERAGE:\"%3.2lf $delimiter $item $delimiter $server_space_colon $delimiter $lpar_space_colon\"";
      $cmd .= " PRINT:utiltot_ghz${i}${s}:MAX:\\\"%3.2lf $delimiter $color[$color_index] $delimiter $host_colon $delimiter $key\\\"";
    } ## end if ($vmware)
    if ($hyperv) {
      $cmd .= " GPRINT:utiltot_hz${i}${s}:AVERAGE:\"%5.2lf \"" if $detail == 1 || $detail == 7;
      $cmd .= " GPRINT:utiltot_hz${i}${s}:MAX:\" %5.2lf \""    if $detail == 1 || $detail == 7;

      $cmd .= " PRINT:utiltot_hz${i}${s}:AVERAGE:\"%3.2lf $delimiter $item $delimiter $server_space_colon $delimiter $lpar_space_colon\"";
      $cmd .= " PRINT:utiltot_hz${i}${s}:MAX:\\\"%3.2lf $delimiter $color[$color_index] $delimiter $host_colon $delimiter $key\\\"";
    } ## end if ($hyperv)

    $cmd .= " $t2";
    $s++;
  } ## end while ( my ( $key, $value...))

  # $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t" if $detail == 1 || $detail == 7;
  $cmd .= " HRULE:0#000000";

  #  "VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  # print STDERR "2766 detail-graph-cgi.pl \$cmd $cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_lpar : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param( $ret, $tl );
  return 0;
} ## end sub graph_vm_vmotion

sub graph_trend {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $upper    = shift;
  my $detail   = shift;
  my $t        = "COMMENT: ";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $xgrid    = "--x-grid=MONTH:1:MONTH:1:MONTH:1:0:%b";

  if ( $upper eq '' || $upper == 0 ) {
    $upper = 0.1;
  }

  if ( -f "$wrkdir/$server/$host/vmware.txt" ) {return}

  my $rrd     = "$wrkdir/$server/$host/$lpar.rr$type_sam";
  my $rrd_lpm = "$wrkdir/$server/*/$lpar.rr$type_sam";

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "LPAR: $lpar_slash_nmon :$last_txt year";

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  # Trend for yearly chart 1y back  + 1y forward
  #print "creating graph : $host:$server:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
  #error ( "creating graph : $host:$server:$lpar:$type_sam:$type:trend");

  $rrd =~ s/:/\\:/g;
  $rrd_lpm =~ s/:/\\:/g;
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  if ( $detail == 10 ) {
    $header = "$server : LPAR: $lpar_slash_nmon :$last_txt year";
  }

  my $cmd .= "$graph_cmd \"$name_out\"";
  $cmd    .= " --title \"$header - trend\"";
  $cmd    .= " --start now-1$type";
  $cmd    .= " --end now+1$type";
  $cmd    .= " --imgformat PNG";
  $cmd    .= " --slope-mode";
  $cmd    .= " $no_minor";
  $cmd    .= " --width=$width_trend";
  $cmd    .= " --height=$height";
  $cmd    .= " --step=$step";
  $cmd    .= " --lower-limit=0.00";
  $cmd    .= " --color=BACK#$pic_col";
  $cmd    .= " --color=SHADEA#$pic_col";
  $cmd    .= " --color=SHADEB#$pic_col";
  $cmd    .= " --color=CANVAS#$pic_col";
  $cmd    .= " --alt-autoscale-max";
  $cmd    .= " --upper-limit=0.1";
  $cmd    .= " $vertical_label";
  $cmd    .= " --units-exponent=1.00";
  $cmd    .= " --alt-y-grid";
  $cmd    .= " $no_legend";
  $cmd    .= " $disable_rrdtool_tag";
  $cmd    .= " $font_def";
  $cmd    .= " $font_tit";
  $cmd    .= " $xgrid";

  # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
  my ( $result_cmd, $CPU_cur_d, $CPU_ent_d, $CPU_cap_d, $CPU_uncap_d, $CPU_cur_m, $CPU_ent_m, $CPU_cap_m, $CPU_uncap_m, $CPU_cur_y, $CPU_ent_y, $CPU_cap_y, $CPU_uncap_y, $CPU_cur, $CPU_ent, $CPU_cap, $CPU_uncap ) = LPM_easy_trend( "$rrd_lpm", "curr_proc_units", "entitled_cycles", "capped_cycles", "uncapped_cycles" );

  $cmd .= $result_cmd;
  $cmd .= " CDEF:curd=$CPU_cur_d";
  $cmd .= " CDEF:curm=$CPU_cur_m";
  $cmd .= " CDEF:cury=$CPU_cur_y";
  $cmd .= " CDEF:cur=$CPU_cur";

  $cmd .= " CDEF:entd=$CPU_ent_d";
  $cmd .= " CDEF:entm=$CPU_ent_m";
  $cmd .= " CDEF:enty=$CPU_ent_y";
  $cmd .= " CDEF:ent=$CPU_ent";

  $cmd .= " CDEF:capd_peak=$CPU_cap_d";
  $cmd .= " CDEF:capm_peak=$CPU_cap_m";
  $cmd .= " CDEF:capy_peak=$CPU_cap_y";
  $cmd .= " CDEF:cap_peak=$CPU_cap";

  # filtering peaks caused by LPM or changing entitled, if cap CPU util is > entitled --> UNKN
  # usualy cap counter is affected only
  # sometimes might happen that even in normal load capped util is little higher than entitled! Therefore using 1.2
  $cmd .= " CDEF:capd=capd_peak,ent,/,1.2,GT,UNKN,capd_peak,IF";
  $cmd .= " CDEF:capm=capm_peak,ent,/,1.2,GT,UNKN,capm_peak,IF";
  $cmd .= " CDEF:capy=capy_peak,ent,/,1.2,GT,UNKN,capy_peak,IF";
  $cmd .= " CDEF:cap=cap_peak,ent,/,1.2,GT,UNKN,cap_peak,IF";


  $cmd .= " CDEF:uncapd=$CPU_uncap_d";
  $cmd .= " CDEF:uncapm=$CPU_uncap_m";
  $cmd .= " CDEF:uncapy=$CPU_uncap_y";
  $cmd .= " CDEF:uncap=$CPU_uncap";

  $cmd .= " CDEF:totd=capd,uncapd,+";
  $cmd .= " CDEF:totm=capm,uncapm,+";
  $cmd .= " CDEF:toty=capy,uncapy,+";
  $cmd .= " CDEF:tot=cap,uncap,+";
  $cmd .= " CDEF:utild=totd,entd,/,$cpu_max_filter,GT,UNKN,totd,entd,/,IF";
  $cmd .= " CDEF:utilm=totm,entm,/,$cpu_max_filter,GT,UNKN,totm,entm,/,IF";
  $cmd .= " CDEF:utily=toty,enty,/,$cpu_max_filter,GT,UNKN,toty,enty,/,IF";
  $cmd .= " CDEF:util=tot,ent,/,$cpu_max_filter,GT,UNKN,tot,ent,/,IF";
  $cmd .= " CDEF:utilperct=util,100,*";
  $cmd .= " CDEF:utiltotd=utild,curd,*";
  $cmd .= " CDEF:utiltotm=utilm,curm,*";
  $cmd .= " CDEF:utiltoty=utily,cury,*";
  $cmd .= " CDEF:utiltot=util,cur,*";
  $cmd .= " VDEF:Dd=utiltotd,LSLSLOPE";
  $cmd .= " VDEF:Hd=utiltotd,LSLINT";
  $cmd .= " CDEF:utiltottrendd=utiltotd,POP,Dd,COUNT,*,Hd,+";
  $cmd .= " VDEF:Dm=utiltotm,LSLSLOPE";
  $cmd .= " VDEF:Hm=utiltotm,LSLINT";
  $cmd .= " CDEF:utiltottrendm=utiltotm,POP,Dm,COUNT,*,Hm,+";
  $cmd .= " VDEF:Dy=utiltoty,LSLSLOPE";
  $cmd .= " VDEF:Hy=utiltoty,LSLINT";
  $cmd .= " CDEF:utiltottrendy=utiltoty,POP,Dy,COUNT,*,Hy,+";
  $cmd .= " COMMENT:\"   Average   \\n\"";
  $cmd .= " AREA:cur#00FF00:\" Entitled processor cores \"";
  $cmd .= " GPRINT:cur:AVERAGE:\" %4.2lf\"";
  $cmd .= " $t2";
  $cmd .= " LINE1:utiltot#FF0000:\" Utilization in CPU cores\"";
  $cmd .= " GPRINT:utiltot:AVERAGE:\" %5.2lf\"";
  $cmd .= " $t2";
  $cmd .= " COMMENT:\"Entitled CPU utilization   \"";
  $cmd .= " GPRINT:utilperct:AVERAGE:\" %4.2lf\"";
  $cmd .= " COMMENT:\"\%\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendd#0088FF:\" Utilization in CPU cores - last 1 month trend\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendm#80FFFF:\" Utilization in CPU cores - last 3 months trend\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendy#FF8080:\" Utilization in CPU cores - last year trend\"";
  $cmd .= " $t2";
  $cmd .= " HRULE:0#000000";

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;

} ## end sub graph_trend

sub graph_ams {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $upper       = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;             # 1 do not show entitle area
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \"";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT:\" \"";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ams: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  my $rrd = "$wrkdir/$server/$host/$lpar.rm$type_sam";

  # print "001 AMS $rrd\n";

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;          # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;       # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "Memory: $lpar_slash_nmon :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "Memory: $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ "d" ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          #my $curr_mem = sprintf("%e",$m_arr[1]);
          #my $phys_run_mem    = sprintf("%e",$m_arr[2]);
          if ( $m_arr[2] > 0 ) {
            my $last_mem   = sprintf( "%6.2d", $m_arr[2] ) / 1024;
            my $last_mem_g = sprintf( "%.2f",  $m_arr[2] / 1024 );
            $last = "COMMENT:\"Last memory consuption in GB         $last_mem_g\"";
          }
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $vertical_label = "--vertical-label=\"Memory in GB\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # `echo "809: detail-graph-cgi AMS-lpar : $host:$server:$lpar_slash:$type_sam:$type:$start_unix:$end_unix" >> /tmp/xAMS-lpar`;
  #print "graphing AMS   : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;

  my $cmd = "";
  if ($detail == 9) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  # for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;    #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "ams", \@rrd_files, $cgi );

  #print STDERR "lpm_find_files is returning @rrd_files\n";

  my $i = -1;
  my $j;

  foreach $rrd (@rrd_files) {    # LPM cycle
    chomp($rrd);
    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    $cmd .= " DEF:cur${i}=\"$rrd\":phys_run_mem:AVERAGE";
    $cmd .= " DEF:ent${i}=\"$rrd\":curr_mem:AVERAGE";
    $cmd .= " DEF:io${i}=\"$rrd\":io_ent:AVERAGE";

    if ( $i == 0 ) {
      $cmd .= " CDEF:curr${i}=cur${i}";
      $cmd .= " CDEF:entr${i}=ent${i}";
      $cmd .= " CDEF:ior${i}=io${i}";
      next;
    } ## end if ( $i == 0 )
    $cmd .= " CDEF:curr${i}=curr${j},UN,cur${i},curr${j},IF";
    $cmd .= " CDEF:entr${i}=entr${j},UN,ent${i},entr${j},IF";
    $cmd .= " CDEF:ior${i}=ior${j},UN,io${i},ior${j},IF";
  } ## end foreach $rrd (@rrd_files)
  $cmd .= " CDEF:cur=curr${i}";
  $cmd .= " CDEF:ent=entr${i}";
  $cmd .= " CDEF:io=ior${i}";

  $cmd .= " CDEF:entg=ent,1024,/";
  $cmd .= " CDEF:curg=cur,1024,/";
  $cmd .= " CDEF:iog=io,1024,/";
  $cmd .= " COMMENT:\"   Average   \\n\"";
  $cmd .= " AREA:entg#00FF00:\" Current allocated memory    \"";
  $cmd .= " GPRINT:entg:AVERAGE:\" %6.2lf\"";
  $cmd .= " $t2";
  $cmd .= " LINE1:iog#0080FF:\" Current allocated IO memory \"";
  $cmd .= " GPRINT:iog:AVERAGE:\" %6.2lf\"";
  $cmd .= " $t2";
  $cmd .= " LINE1:curg#FF0000:\" Actuall physical consumption\"";
  $cmd .= " GPRINT:curg:AVERAGE:\" %6.2lf\"";
  $cmd .= " $t2";
  $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #  "VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_ams : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_ams

sub graph_ent_sum {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $item       = shift;                 # "slan" or "ssan1" or "ssan2" or "ssea"
  my $start_unix = shift;
  my $end_unix   = shift;

  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT: ";
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ent_sum : $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }
  my $dir = "";
  if ($item eq "s_ldom_sum"){
    $dir = "$wrkdir/Solaris/$lpar/";
  }else{
    $dir = "$wrkdir/$server/$host/$lpar/";
  }

  my $iops = "IOPS";
  opendir( my $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ );
  my @files_iops = grep { /^san-host/ && /\.mmm$/ } readdir($dh) if $item ne "s_ldom_sum";
#  my @files_sum = grep !/^\.\.?$|ZONE|_ldom\.mmm$|ldom_name|solaris11|solaris10|^san/, readdir($dh) if $item eq "s_ldom_sum";
  closedir $dh;
  if ( @files_iops > 0 ) {

    # Linux standalones provide only SAN frames
    $iops = "Frames";
  }
#print STDERR"@files_sum\n";
  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  my $san2nmon = ( $item =~ /^ssan2/ && index( $lpar, "--NMON--" ) ne -1 );
  my $minus_one = -1;

  my $fpn      = "lan";              # according to $item
  my $gpr_ave  = "5.1lf";
  my $gpr_max  = "6.1lf";
  my $divider  = 1073741824;
  my $last_txt = " last ";
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  my $header_prefix = "Eth:";
  if ( $detail == 1 ) {
    $header_prefix = "SUM Eth:";
  }
  if ( $detail == 2 ) {
    $last_txt      = "";
    $lpar_slash_nmon_head = "" if $server eq "Linux";
    $header_prefix = "ETH SUM";
    if ( $wpar == 1 ) {
      $header_prefix = "ETH SUM";
    }
  } ## end if ( $detail == 2 )
  my $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

  my $twrite   = "WRITE    ";
  my $tread    = "READ     ";
  my $ds_name1 = "recv_bytes";
  my $ds_name2 = "trans_bytes";
  if ( $item eq "slan" ) {
    $ds_name2 = "recv_bytes";
    $ds_name1 = "trans_bytes";
    $twrite   = "WRITE";
    $tread    = "READ  ";
  } ## end if ( $item eq "slan" )
  if ($item eq "s_ldom_sum"){
    $ds_name2 = "rbytes";
    $ds_name1 = "obytes";
    $twrite   = "WRITE";
    $tread    = "READ  ";
  }
  my $headline1      = "COMMENT:[ GB ]           Max";
  my $headline2      = "COMMENT:Interface     READ (Max)          WRITE (Max)";
  my $vertical_label = "--vertical-label=Read - Bytes - Write";

  my $filter = $filter_max_lansan;

  if ( $item eq "ssan1" ) {
    # $fpn = "san-fcs" ;
    $fpn    = "san-";
    $header = "FCS: $lpar_slash_nmon :$last_txt$text";
    if ( $detail == 1 ) {
      $header = "SUM FCS: $lpar_slash_nmon :$last_txt$text";
    }
  } ## end if ( $item eq "ssan1" )

  if ( $item eq "ssea" ) {
    # $fpn = "sea-ent" ;
    $fpn = "sea-";
    my $header_prefix = "SEA:";
    if ( $detail == 1 ) {
      $header_prefix = "SUM SEA:";
    }
    my $lpar_slash_nmon_head = $lpar_slash_nmon;
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "SEA";
      if ( $wpar == 1 ) {
        $header_prefix = "SEA:";
      }
    } ## end if ( $detail == 2 )
    $header   = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    $ds_name2 = "recv_bytes";
    $ds_name1 = "trans_bytes";
    $twrite   = "WRITE";
    $tread    = "READ  ";
  } ## end if ( $item eq "ssea" )

  if ( $item eq "ssan2" ) {
    $gpr_ave = "8.0lf";
    $gpr_max = "8.0lf";
    $divider = 1;

    # $fpn = "san-fcs" ;
    $fpn = "san-";
    my $header_prefix = "$iops:";
    if ( $detail == 1 ) {
      $header_prefix = "SUM $iops:";
    }
    my $lpar_slash_nmon_head = $lpar_slash_nmon;
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "$iops:";
      if ( $wpar == 1 ) {
        $header_prefix = "$iops:";
      }
    } ## end if ( $detail == 2 )
    $header   = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    if ( $iops =~ m/Frames/ ) {
      # Linux san-host* have swapped in/out frames, fixed in 5.07-4
      $ds_name1       = "iops_out";
      $ds_name2       = "iops_in";
    }
    else {
      $ds_name1       = "iops_in";
      $ds_name2       = "iops_out";
    }

    $headline1      = "COMMENT:[ $iops/day ]        Max";
    $headline2      = "COMMENT:Interface     READ (Max)          WRITE (Max)";
    $vertical_label = "--vertical-label=$iops requests";
    if ($san2nmon) {
      $minus_one = 1;
      $tread     = "TOTAL    ";
    }
  } ## end if ( $item eq "ssan2" )

  if ( $item =~ m/san2/ ) {
    $filter = $filter_max_iops;
  }

  if ( $type =~ m/y/ ) {

    # lower limits for yearly graphs as they are averaged ....
    $filter = $filter / 10;
  }
  if ( $type =~ m/m/ ) {

    # lower limits for monthly graphs as they are averaged ....
    $filter = $filter / 2;
  }

  my @files;
  opendir( $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files_unsorted = grep { /^$fpn/ && /\.mmm$/ } readdir($dh);
  rewinddir($dh);
  my @files_sum = grep !/^\.\.?$|ZONE|_ldom\.mmm$|uuid|solaris11|solaris10|^san/, readdir($dh) if $item eq "s_ldom_sum";
  if ($item ne "s_ldom_sum"){
    @files = sort @files_unsorted;
  }
  else{
    @files = sort @files_sum;
  }
  closedir $dh;
  my @rev = reverse(@files);    #for case study purpose
  # print STDERR "0001 ENT SUM \$fpn $fpn \@files @files\n";

  my $col_indx   = 0;
  my $line_indx  = 0;             # place enter every 3rd line
  my $file       = "";
  my $cmd        = "";
  my $j          = 0;
  my $act_time   = localtime();
  my $act_time_u = time();
  my $req_time   = 0;
  my ( @recb, @trab, @rname, @rpath, @avgx );
  my $days = 32;

  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
    $days     = 365;
  }

  #if ( $type =~ "y" ) {
  #   $step_new=86400;
  #}

  #print "graphing ENT_SUM   : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  # print STDERR "3744 \$detail $detail \$start_time $start_time \$end_time $end_time \$header $header\n";

  if ($detail == 9) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }
#  $cmd .= 'RRDp::cmd qq(graph "$name_out"
$cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_time"
        "--end" "$end_time"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "--step=86400"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "$vertical_label"
        "--alt-y-grid"
        "$xgrid"
        "$no_legend"
        "$disable_rrdtool_tag"
        "--base=1024"
        "$font_def"
        "$font_tit"
        "$headline1"
        "$t2"
     ';

  # sum the databases
  # take care: if all dtbs are old means nothing to graph
  #            either only one dtb or more dtbs

  my $i             = 0;
  my $summ_r        = "";
  my $summ_w        = "";
  my $count_avg_day = "";

  # variables for LPM use
  my $k = 0;    # indexes for LPM cycle
  my $l = 0;
  my ( @rb, @rbr, @tb, @tbr );
  my @rrd_files_names;
  my $ifn = -1;    # index for this array

  foreach $file (@files) {

    #   foreach $file (@rev)
    chomp($file);
    my $rrd = "";
    if ($item ne "s_ldom_sum"){
      $rrd = "$wrkdir/$server/$host/$lpar/$file";
    }
    else{
      $rrd = "$wrkdir/Solaris/$lpar/$file";
    }
    # avoid old files which do not exist in the period
    my $rrd_upd_time = ( stat("$rrd") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }
    $count_avg_day = 1;    #since we do not print AVERAGE any more

    #    if ($i == 0) {   # prepare better AVG
    #      $count_avg_day = rrd_running_days($rrd,$days);
    #      if ( ! ($count_avg_day eq ($count_avg_day+0))) {
    #         error ("error in rrd_running : $count_avg_day ");
    #         $count_avg_day = 1;
    #      }
    #      else {
    #         if ( "$type" =~ "m" ) {
    #            $count_avg_day = 32 / $count_avg_day ;
    #         }
    #         else {
    #            $count_avg_day = 365 / $count_avg_day ;
    #         }
    #      }
    #    }
    my $rrd_name = $file;
    $rrd_name =~ s/.mmm//;
    $rrd_name =~ s/lan-//;
    $rrd_name =~ s/san-//;
    $rrd_name =~ s/sea-//;
    $rrd_name = sprintf( "%-8s", $rrd_name );    #add spaces, 8 chars tot - format graph legend

    # print STDERR "graph_ent_sum $wrkdir/$server/$host/$lpar/$file
    #  $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    # print STDERR "001 ENT SUM $rrd\n";

    my $rrd_orig_colon = $rrd;
    $rrd =~ s/:/\\:/g;    #anti ':'

    $rname[$i] = "$rrd_name";    # for graphing
    $rpath[$i] = "$rrd";
    $recb[$i]  = "rcb${i}";
    $trab[$i]  = "trb${i}";
    $avgx[$i]  = "avg${i}";

    # start of LPM fun

    # for LPM
    $l = $k;
    my @rrd_files;
    my $lpm_suff = "rrl";
    my $cgi      = 0;
    $rrd_files[0] = $rrd_orig_colon;    #actual is the first

    lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, $file, \@rrd_files, $cgi );

    # print STDERR "graph_ent_sum lpm_find_files is returning @rrd_files\n";

    foreach $rrd (@rrd_files) {    # LPM cycle
      chomp($rrd);
      $rrd =~ s/:/\\:/g;
      $ifn++;
      $rrd_files_names[$ifn] = "$rrd";
      $rb[$k]                = "data1_stream${k}";
      $rbr[$k]               = "data1_stream_lpm${k}";
      $tb[$k]                = "data2_stream${k}";
      $tbr[$k]               = "data2_stream_lpm${k}";

      $cmd .= '"DEF:$rb[$k]=$rrd_files_names[$ifn]:$ds_name1:AVERAGE"
             "DEF:$tb[$k]=$rrd_files_names[$ifn++]:$ds_name2:AVERAGE"
            ';
      if ( $k == $l ) {
        $cmd .= '"CDEF:$rbr[$k]=$rb[$k]"
             "CDEF:$tbr[$k]=$tb[$k++]"
             ';
        $k++;
        next;
      } ## end if ( $k == $l )
      $cmd .= '"CDEF:$rbr[$k]=$rbr[$l],UN,$rb[$k],$rbr[$l],IF"
             "CDEF:$tbr[$k]=$tbr[$l],UN,$tb[$k++],$tbr[$l++],IF"
            ';
      $l++;
      $k++;
    } ## end foreach $rrd (@rrd_files)
    $cmd .= '"CDEF:$recb[$i]_nf=$rbr[$l]"
           "CDEF:$trab[$i]_nf=$tbr[$l++]"
          ';

    # end of LPM fun

    # print STDERR "3923: detail-graph-cgi - $rrd_name:$rrd:rcb${i}:$recb[$i]:$font_def:$font_tit:@files\n";

    #$cmd .= '"DEF:$recb[$i]_nf=$rpath[$i]:$ds_name1:AVERAGE"
    #         "DEF:$trab[$i]_nf=$rpath[$i]:$ds_name2:AVERAGE"

    $cmd .= '"CDEF:$recb[$i]=$recb[$i]_nf,$filter,GT,0,$recb[$i]_nf,IF"
          "CDEF:$recb[$i]_n=$recb[$i],UN,0,$recb[$i],IF"
          "CDEF:$trab[$i]=$trab[$i]_nf,$filter,GT,0,$trab[$i]_nf,IF"
          "CDEF:$trab[$i]_n=$trab[$i],UN,0,$trab[$i++],IF"

          ';
    if ( $i == 0 ) {
      $summ_r = $recb[$i] . "_n";
      $summ_w = $trab[$i] . "_n";
    }
    else {
      $summ_r .= "," . $recb[$i] . "_n,+";
      $summ_w .= "," . $trab[$i] . "_n,+";
    }
    $i++;
  }    # end of foreach

  if ( $i == 0 ) {
    error( "$host:$server:$lpar_slash Graph rrdtool error : no active dbs ETH,FCS,SEA  " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  #  "GPRINT:trans_smb_n:AVERAGE: %$gpr_ave"
  #   "GPRINT:recv_smb_n:AVERAGE: %$gpr_ave"

  $cmd .= '"CDEF:recv=$summ_r"
           "CDEF:trans=$summ_w"
           "CDEF:recv_s=recv,86400,*"
           "CDEF:recv_smb=recv_s,$divider,/"
           "CDEF:recv_smb_n=recv_smb,$count_avg_day,*"
           "CDEF:trans_s=trans,86400,*"
           "CDEF:trans_smb=trans_s,$divider,/"
           "CDEF:trans_smb_n=trans_smb,$count_avg_day,*"
           "CDEF:recv_neg=recv_s,$minus_one,*"
  ';
  if ( !$san2nmon ) {
    $cmd .= '"AREA:trans_s#FFFF00:$twrite "
       "GPRINT:trans_smb:MAX: %$gpr_max"
       "$t2"
  ';
  } ## end if ( !$san2nmon )
  $cmd .= '"AREA:recv_neg#00FF00:$tread "
             "GPRINT:recv_smb:MAX: %$gpr_max"
             "$t2"
             "$t"
             "HRULE:0#000000"
    );';

  #       "VRULE:0#000000"    # --> it is causing sigsegv on linuxeS

  #"CDEF:recv=recv_nf,$filter,GT,UNKN,recv_nf,IF"
  #"CDEF:trans=trans_nf,$filter,GT,UNKN,trans_nf,IF"
  #if you want to see what is generated
  # my $sss = "/var/tmp/sss.out";
  # open (FF, ">$sss");
  #      print FF "$filter: $cmd\n";
  # close (FF);

  # if you want to see interpolated $cmd
  # care what file name is in sub alfa
  #   $i = 0;
  #  my $t_cmd = $cmd;
  #  $t_cmd =~ s/RRDp::cmd qq\(graph /alfa\(/;
  #  $t_cmd =~ s/\s\"/\n\.\"/g;
  #  eval($t_cmd);

  $i   = 0;
  $k   = 0;
  $l   = 0;
  $ifn = 0;
  eval $cmd;
  if ($@) {
    error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
  }
  else {
    $ret = RRDp::read;
    if ( $$ret =~ "ERROR" ) {
      error( "$host:$server Graph rrdtool error : $$ret " . __FILE__ . ":" . __LINE__ );
    }
    $png_end_heading = ret_graph_param($ret);
  } ## end else [ if ($@) ]
  return 0;
} ## end sub graph_ent_sum

sub graph_ent {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $item        = shift;                 #lan or pg2 or san1 or san2 or sea, !?!?!?!?! if contains "a" then AREA STACK
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $step_new    = $step;
  my $t           = "";
  my $t2          = "COMMENT:\\n";
  my $last        = "COMMENT: ";
  my $upper_limit = 0.1;
  my $start_time  = "now-1$type";
  my $end_time    = "now-1$type+1$type";
  my $vertical_label;

  # print STDERR "3814 \$host $host \$server $server \$lpar $lpar \$item $item\n";
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  my $human_ds_name = "";
  my $human_cluster_name = "";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains
      }
      if ( $item eq 'ds-vmiops' ) {
        $human_ds_name = find_human_ds_name( $server, $host, $lpar );
        # print STDERR "4362 \$human_ds_name $human_ds_name\n";
     }
    }
  } ## end if ( !$start_unix eq ''...)
  if ( $item eq 'ds-vmiops' && $detail == 1) {
    $human_ds_name = find_human_ds_name( $server, $host, $lpar );
  }

  my $Int = "Int";
  $Int = "   " if ( $item eq "data_as" || $item eq "iops_as" || $item eq "data_ifcb" || $item eq "paket_ifcb" || $item eq "dpaket_ifcb" || $item eq "packets_lan" || $item eq "packets_sea" || $item eq "s_ldom_pack" );

  my $headline2 = "COMMENT:$Int      READ     Avg      Max      WRITE     Avg      Max";
  $headline2 = "COMMENT:Int     READ   Avg      Max    WRITE   Avg      Max"           if ( $item =~ "^lan" );
  $headline2 = "COMMENT:Datastore                  READ   Avg     Max    WRITE    Avg    Max" if ( $item eq "vmw-iops" );
  $headline2 = "COMMENT:VM                         READ   Avg     Max    WRITE    Avg    Max" if ( $item eq "ds-vmiops" );
  $headline2 = "COMMENT:Int     READ/IN   Avg      Max    WRITE/OUT   Avg      Max"           if ( $item eq "s_ldom_n");
  $headline2 = "COMMENT:Int     READ/IN   Avg      Max    WRITE/OUT   Avg      Max"           if ( $item eq "s_ldom_vnet");
  $headline2 = "COMMENT:Int     READ/WRITE   Avg      Max"           if ( $item eq "s_ldom_san_resp");
  $headline2 = "COMMENT:server  READ   Avg      Max    WRITE   Avg      Max"           if ( $item eq "clustlan" );

  #if ( $detail == 1 ) {
  #  if ($item =~ "^lan") {
  #     $headline2 = "COMMENT:Int     READ   Avg      Max    WRITE   Avg      Max        Int      READ   Avg      Max     WRITE   Avg      Max";
  #  }
  #  else {
  #     $headline2 = "COMMENT:$Int      READ     Avg      Max      WRITE     Avg      Max        $Int       READ     Avg      Max       WRITE     Avg      Max";
  #  }
  #}

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ent: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                # some workaround, it should not go here
  }
  my @files;

  my $dir = "$wrkdir/$server/$host/$lpar/";
  if ( $item eq "data_as" || $item eq "iops_as" ) {
    $dir = "$wrkdir/$server/$host/$lpar--AS400--/ASP/";
  }
  elsif ( $item eq "clustlan" ) {
    # print STDERR "4910 clustlan \$host $host \$type $type \$server $server \$lpar $lpar \n";
    my $tmp_file     = "$basedir/tmp/multi-hmc-$server-$host-$type.cmd";
    if ( $type eq "a" ) { $tmp_file = "$basedir/tmp/multi-hmc-$server-$host-d.cmd"; } # Reporter TimeRange mode
    ( my $rrd, $human_cluster_name ) = find_human_cluster_name();
    # print STDERR "4914 clustlan $rrd, $human_cluster_name, $tmp_file\n";
    open( FH, "< $tmp_file" ) || error( "clustlan: Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
     my $line = <FH>;
    close(FH);
    @files = split("DEF:cpu_entitl_mhz", $line);
    shift @files; #remove 1st line
    foreach (@files) {
      # DEF:cpu_entitl_mhz0="/home/lpar2rrd/lpar2rrd/data/10.22.11.14/10.22.11.10/pool.rrm":CPU_Alloc:AVERAGE
      # 0="/home/lpar2rrd/lpar2rrd/data/192.168.1.8/pavel.lpar2rrd.com\:443/pool.rrm":CPU_Alloc:AVERAGE
      chomp $_;
      $_ =~ s/:CPU_Alloc.*//;
      $_ =~ s/^.*(=)//;
      $_ =~ s/\"//g;
      # print STDERR "4926 $_\n";
    }
  }
  elsif ( $item eq "data_ifcb" || $item eq "paket_ifcb" || $item eq "dpaket_ifcb" ) {
    $dir = "$wrkdir/$server/$host/$lpar--AS400--/IFC/";
  }
  elsif ( $item eq "hmcpgs" ) {
    $dir  = "$wrkdir/--HMC--$host/";
    $lpar = "$host";
  }
  elsif ( $item eq "vmw-iops" ) {
    # print STDERR "4079 $wrkdir/$server/$host/my_vcenter_name\n";
    if ($host eq "nope" || $host =~ /cluster_/) { # from server hist report - necessary to find vCenter, what about get it directly from histrep.pl ?
      my @path_temp = <$wrkdir/$server/*>;
      # print STDERR "4081 \@path_temp @path_temp\n";
      return if ! defined $path_temp[0]; # some trash
      $host = (split(/\//,$path_temp[0]))[-1];
    }
    # print STDERR "4084 $wrkdir/$server/$host/my_vcenter_name \$detail $detail\n";
    # if there is data for IOPS under any datastore in vCenter/datacenter/datastore/*.rrv
    my $vm_iops = 0;
    if ( open( FH, "< $wrkdir/$server/$host/my_vcenter_name" ) ) {
      my $vcenter_name = <FH>;
      close(FH);
      chomp $vcenter_name;    # e.g. '10.22.11.10|Regina-new'
                              # now look for the appropriate vCenter
      ( undef, $vcenter_name ) = split( /\|/, $vcenter_name );

      # print STDERR "3872 \$vcenter_name $vcenter_name\n";
      my $vCenter = "";
      foreach my $center (<$wrkdir/vmware_*>) {
        next if $center =~ "vmware_VMs";
        next if !open( FH, "< $center/vmware_alias_name" );
        my $vmware_alias_name = <FH>;
        close(FH);
        next if $vmware_alias_name !~ /$vcenter_name$/;
        $vCenter = $center;
        last;
      } ## end foreach my $center (<$wrkdir/vmware_*>)

      # print STDERR "4185 found \$vCenter $vCenter $vCenter/*/*/$lpar.rrv\n";
      if ( $vCenter ne "" ) {
        @files = <$vCenter/*/*/$lpar.rrv>;

        # print STDERR "4189 \@files @files\n";
        $lpar = human_vmware_name($lpar);
      } ## end if ( $vCenter ne "" )
    } ## end if ( open( FH, "< $wrkdir/$server/$host/my_vcenter_name"...))
  } ## end elsif ( $item eq "vmw-iops")
  elsif ($item =~ /^s_ldom_n/){
    if ($server =~ /Solaris--unknown|Solaris\d+--unknown/){$server=$lpar;}
    opendir( DIR, "$wrkdir/Solaris/$server/" ) || error( "can't opendir $wrkdir/Solaris/$server/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @ldom_path = grep !/^\.\.?$|ZONE|_ldom\.mmm$|uuid|solaris11|solaris10|^san|^vlan/, readdir(DIR);
    foreach my $ldom_file (@ldom_path){
      chomp $ldom_file;
      push (@files,"$ldom_file\n");
    }
  }
  elsif ($item =~ /^s_ldom_vnet/){
    if ($server =~ /Solaris--unknown|Solaris\d+--unknown/){$server=$lpar;}
    opendir( DIR, "$wrkdir/Solaris/$server/" ) || error( "can't opendir $wrkdir/Solaris/$server/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @ldom_path = grep /^vlan-/, readdir(DIR);
    foreach my $ldom_file (@ldom_path){
      chomp $ldom_file;
      push (@files,"$ldom_file\n");
    }
  }
  elsif ($item =~ /^s_ldom_san1|^s_ldom_san2/){
    if ($server !~ /Solaris--unknown|Solaris\d+--unknown/){
      opendir( DIR, "$wrkdir/Solaris/$server/" ) || error( "can't opendir $wrkdir/Solaris/$server/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
      my @san_path = grep /^san-c/, readdir(DIR);
      foreach my $san_file (@san_path){
        chomp $san_file;
        push (@files,"$san_file\n");
      }
    }
   else{
      opendir( DIR, "$wrkdir/Solaris/$lpar/" ) || error( "can't opendir $wrkdir/Solaris/$lpar/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
      my @san_path = grep /^san-c/, readdir(DIR);
      foreach my $san_file (@san_path){
        chomp $san_file;
        push (@files,"$san_file\n");
      }
   }
  }
  elsif ($item =~ /^san_no_ldom1|^san_no_ldom2/){
    opendir( DIR, "$wrkdir/Solaris/$lpar/" ) || error( "can't opendir $wrkdir/Solaris/$lpar/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @san_path = grep /^san-c/, readdir(DIR);
    foreach my $san_file (@san_path){
      chomp $san_file;
      push (@files,"$san_file\n");
    }
  }
  elsif ($item =~ /s_ldom_san_resp/){
    if ($server !~ /Solaris--unknown|Solaris\d+--unknown/){
      opendir( DIR, "$wrkdir/Solaris/$server/" ) || error( "can't opendir $wrkdir/Solaris/$server/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
      my @san_resp_path = grep /^san_tresp/, readdir(DIR);
      foreach my $san_resp_file (@san_resp_path){
        chomp $san_resp_file;
        push (@files,"$san_resp_file\n");
      }
    }
    else{
      opendir( DIR, "$wrkdir/Solaris/$lpar/" ) || error( "can't opendir $wrkdir/Solaris/$lpar/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
      my @san_resp_path = grep /^san_tresp/, readdir(DIR);
      foreach my $san_resp_file (@san_resp_path){
        chomp $san_resp_file;
        push (@files,"$san_resp_file\n");
      }
    }
  }
  elsif ($item =~ /s_ldom_pack/){
    opendir( DIR, "$wrkdir/Solaris/$lpar/" ) || error( "can't opendir $wrkdir/Solaris/$server/: $! :" . __FILE__ . ":" . __LINE__ ) && next;
    my @files_sum = grep !/^\.\.?$|ZONE|_ldom\.mmm$|uuid|solaris11|solaris10|^san/, readdir(DIR);
    foreach my $ldom_pack (@files_sum){
      chomp $ldom_pack;
      push (@files,"$ldom_pack\n");
    }
  }

  # print STDERR "4195 detail-graph-cgi.pl \$item $item $dir\n";
  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $ds_name1 = "recv_bytes";
  my $ds_name2 = "trans_bytes";

  my $fpn = "lan-";                  # according to $item

  my $last_txt      = " last ";
  my $header_prefix = "Eth:";
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  if ( $detail == 2 ) {
    $last_txt      = "";
    $lpar_slash_nmon_head = "" if $server eq "Linux";
    $header_prefix = "ETH:";
    $header_prefix = "ETH PCK" if $item =~ /packets_lan/;
    if ( $wpar == 1 ) {
      $header_prefix = "ETH";
    }
  } ## end if ( $detail == 2 )

  $header_prefix = "DS:IOPS:" if $item eq "ds-vmiops";
  $header_prefix = "VM:IOPS:" if $item eq "vmw-iops";
  $header_prefix = "IOPS:" if $item eq "vmw-iops" && $detail == 2;
  $lpar_slash_nmon_head = "" if $item eq "ds-vmiops" or $item eq "clustlan" ;
  $header_prefix = "Server:LAN" if $item eq "clustlan";

  my $header = "$header_prefix$lpar_slash_nmon_head:$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      $header = "$header_prefix$lpar_slash_nmon_head : $start_human - $end_human" if $item eq "ds-vmiops";
      if ( $detail == 10 ) {
        $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $headline1 = "COMMENT:Average [MB/sec]";
  $vertical_label = "--vertical-label=Read - Bytes/sec - Write";

  my $g_format = "6.2lf";
  my $divider  = 1000000;

  if ( $item =~ "^lan" ) {
    $ds_name2 = "recv_bytes";
    $ds_name1 = "trans_bytes";
  }

  if ( $item eq "vmw-iops" || $item eq "ds-vmiops" ) {
    $ds_name1 = "vm_iops_read";
    $ds_name2 = "vm_iops_write";
  }

  if ( $item =~ "packets_" ) {
    $ds_name2       = "recv_packets";
    $ds_name1       = "trans_packets";
    $headline1      = "COMMENT:Average [Packets/sec]";
    $vertical_label = "--vertical-label=Read - Packets/sec - Write";
    $divider        = 1;
    $g_format       = "6.0lf";
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux" || $server eq "AS400";
      # $header_prefix = "PACKETS";
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";
  } ## end if ( $item =~ "packets_")

  if ( $item =~ "clustlan" ) {
    $ds_name1       = "Network_received";
    $ds_name2       = "Network_transmitted";
    $headline1      = "COMMENT:Average [Bytes/sec]";
    $vertical_label = "--vertical-label=Read - Bytes/sec - Write";
    $divider        = 1000;
    $g_format       = "6.2lf";
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux" || $server eq "AS400";
      # $header_prefix = "PACKETS";
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";
    $no_legend = "--interlaced";
  } ## end if ( $item =~ "clustlan")
  if ( $item =~ m/^san|^s_ldom_san1|^s_ldom_san2|^san_no_ldom1|^san_no_ldom2/ ) {
    $fpn = "san-";
    my $header_prefix = "SAN:";
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "SAN";
      if ( $wpar == 1 ) {
        $header_prefix = "SAN";
      }
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
  } ## end if ( $item =~ m/^san/ )
  if ( $item =~ m/^data_as/ || $item =~ m/^iops_as/ || $item =~ m/^data_ifcb/ || $item =~ m/^paket_ifcb/ || $item eq "dpaket_ifcb" ) {
    $fpn = "";
    my $h_pref = "";
    $h_pref = "ASP DATA:" if $item eq "data_as";
    $h_pref = "ASP IOPS:" if $item eq "iops_as";
    $h_pref = "IFCB LAN:" if $item eq "data_ifcb";
    $h_pref = "IFCB packets:" if $item eq "paket_ifcb";
    $h_pref = "IFCB packets disc:" if $item eq "dpaket_ifcb";
    my $header_prefix = $h_pref;
    if ( $detail == 2 ) {
      $last_txt      = "";
      $lpar_slash_nmon_head = "" if $server eq "Linux" || $server eq "as400";
      $h_pref = "Packets:" if $item eq "paket_ifcb";
      $h_pref = "Packets disc:" if $item eq "dpaket_ifcb";
      $header_prefix = $h_pref;
      if ( $wpar == 1 ) {
        $header_prefix = $h_pref;
      }
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head:$last_txt$text";

    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
    # 5.07-6 fix, read/write were wrong, replaced each other
    if ( $item =~ m/^data_ifcb/ ) {
      $ds_name1 = "par3";
      $ds_name2 = "par4";
    }
    # 5.05 fix, read/write were wrong, replaced each other
    if ( $item eq "data_as" ) {
      $ds_name1 = "par3";
      $ds_name2 = "par4";
    }
    if ( $item =~ "paket_ifcb" ) {
      # 5.07-6 fix, read/write were wrong, replaced each other
      $ds_name1       = "par7";
      $ds_name2       = "par8";
      $divider        = 1;
      $vertical_label = "--vertical-label=Read - Packets/sec - Write";
      $g_format       = "6.0lf";
      $headline1      = "COMMENT:Average [Packets/sec]";
      if ( $item eq "dpaket_ifcb" ) {
        $ds_name1 = "par5";
        $ds_name2 = "par6";
      }
    } ## end if ( $item =~ "paket_ifcb")
    if ( $item eq "iops_as" ) {
      $ds_name1 = "par5";
      $ds_name2 = "par6";
      $g_format = "6.0lf";
      $divider  = 1;
    } ## end if ( $item eq "iops_as")
  } ## end if ( $item =~ m/^data_as/...)
  if ( $item =~ m/^sea/ || $item =~ m/^packets_sea/) {
    $fpn = "sea-";
    my $header_prefix = "SEA:";
    if ( $detail == 2 ) {
      $last_txt      = "";
      my $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "SEA";
      $header_prefix = "SEA PCK" if $item =~ m/^packets_sea/;
      if ( $wpar == 1 ) {
        $header_prefix = "SEA";
      }
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
    $ds_name2       = "recv_bytes";
    $ds_name1       = "trans_bytes";
    $ds_name2       = "recv_packets" if $item =~ m/packets_sea/;
    $ds_name1       = "trans_packets" if $item =~ m/packets_sea/;
    $vertical_label = "--vertical-label=Read - Bytes/sec - Write" if $item =~ m/^sea/ ;
    $headline1      = "COMMENT:Average [MB/sec]";
    $headline1      = "COMMENT:Average [Packets/sec]" if $item =~ m/packets_sea/;
    $headline2      = "COMMENT:Phys  (SEA)  READ  Avg      Max    WRITE   Avg      Max";
    if ( $detail == 1 ) {
      $headline2 .= "        Phys  (SEA)  READ   Avg      Max     WRITE   Avg      Max";
    }
  } ## end if ( $item =~ m/^sea/ )
  if ( $item eq "pg2" || $item eq "hmcpgs" ) {
    $fpn = "pgs";
    my $header_prefix = "PAGING SPACE:";
    if ( $detail == 2 ) {
      $last_txt      = "";
      my $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "PG2";
      if ( $wpar == 1 ) {
        $header_prefix = "PG";
      }
    } ## end if ( $detail == 2 )
    $header      = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    $upper_limit = 100;
    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
    $ds_name1  = "paging_space";
    $ds_name2  = "percent";
    $headline1 = "COMMENT:Usage [%]  AVRG   MAX";

    #  $headline1 = "COMMENT:Average usage in %";
    $headline2      = "COMMENT:             ";
    $vertical_label = "--vertical-label=Paging utilization in %";
  } ## end if ( $item eq "pg2" ||...)
  if ( $item =~ m/^san2/ ) {
    $fpn = "san-";
  }
  if ( $item =~ m/^san_resp/ ) {
    $fpn = "san_resp-";
  }
  if ( $item =~ m/^iops_as/ ) {
    $vertical_label = "--vertical-label=Read - IOPS - Write";
    $headline1      = "COMMENT:Average [IO per sec]";
  }
  if ( $item eq "vmw-iops" || $item eq "ds-vmiops" ) {
    $vertical_label = "--vertical-label=Read - IOPS - Write";
    $headline1      = "COMMENT: [IO per sec]";
    $g_format       = "4.0lf";
    $divider        = 1;
  } ## end if ( $item eq "vmw-iops"...)
  if($item eq "s_ldom_n" or $item eq "s_ldom_vnet") {
    $ds_name1       = "rbytes";
    $ds_name2       = "obytes";
    $g_format       = "4.2lf";
    $divider        = 1000;
  }
  if($item eq "s_ldom_san1" || $item eq "san_no_ldom1") {
    $ds_name1       = "recv_bytes";
    $ds_name2       = "trans_bytes";
    $g_format       = "4.2lf";
    #$divider        = 1000;
  }
  if($item eq "s_ldom_san2") {
    $ds_name1       = "iops_in";
    $ds_name2       = "iops_out";
    $g_format       = "4.0lf";
    $divider        = 1;
    $vertical_label = "--vertical-label=Read - IOPS - Write";
  }
  if($item eq "s_ldom_pack") {
    $ds_name1       = "ipackets";
    $ds_name2       = "opackets";
    $g_format       = "4.0lf";
    $divider        = 1;
    $headline1      = "COMMENT:Average [Packets/sec]";
    $vertical_label = "--vertical-label=Read - Packets/sec - Write";
  }


  my $dh;
  if ( $item eq "data_as" || $item eq "iops_as" || $item =~ m/^data_ifcb/ || $item =~ m/^paket_ifcb/ || $item =~ m/^dpaket_ifcb/ ) {
    my $file_test = "4\.mmc";
    $file_test = "mmc" if $item =~ m/^data_ifcb/ || $item =~ m/^paket_ifcb/ || $item =~ m/^dpaket_ifcb/;
    opendir( $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_unsorted = grep { (/$file_test$/) } readdir($dh);
    @files = sort @files_unsorted;
    closedir $dh;
    ( @files > 0 ) || error( "no files $dh to graph in $dir: " . __FILE__ . ":" . __LINE__ ) && return 0;

    # print STDERR "3409 detail-graph-cgi.pl \@files @files\n";
  } ## end if ( $item eq "data_as"...)
  elsif ( $item eq "ds-vmiops" ) {
    opendir( $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    @files = grep { (/\.rrv$/) } readdir($dh);
    closedir $dh;
    ( @files > 0 ) || error( "no files $dh to graph in $dir: " . __FILE__ . ":" . __LINE__ ) && return 0;
  } ## end elsif ( $item eq "ds-vmiops")
  elsif ( $item ne "vmw-iops" && $item ne "s_ldom_n" && $item ne "s_ldom_san1" && $item ne "s_ldom_san2" && $item ne "s_ldom_san_resp" && $item ne "s_ldom_pack" && $item ne "san_no_ldom1" && $item ne "s_ldom_vnet" && $item ne "clustlan" ) {
    opendir( $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_unsorted = grep { /^$fpn/ && ( /\.mmm$/ || /\.mmx$/ ) } readdir($dh);
    @files = sort @files_unsorted;
    closedir $dh;
    ( @files > 0 ) || error( "no files $dh to graph in $dir: " . __FILE__ . ":" . __LINE__ ) && return 0;
  } ## end elsif ( $item ne "vmw-iops")

  # print STDERR "4099 detail-graph-cgi.pl \$item $item \@files @files\n";

  my $iops       = "IOPS";
  my @files_iops = ();
  if ( $item ne "vmw-iops" && $item ne "ds-vmiops" && $item ne "s_ldom_n" && $item ne "s_ldom_san1" && $item ne "s_ldom_san2" && $item ne "s_ldom_san_resp" && $item ne "s_ldom_pack" && $item ne "san_no_ldom1" && $item ne "s_ldom_vnet" && $item ne "clustlan" ) {
    opendir( $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    @files_iops = grep { /^san-host/ && /\.mmm$/ } readdir($dh);
    closedir $dh;
    if ( @files_iops > 0 ) {

      # Linux standalones provide only SAN frames
      $iops = "Frames";
    }
  } ## end if ( $item ne "vmw-iops"...)

  # find date of latest file
  # prepare string in format: Updated: Fri Jul 17 10:40:01 2015

  my $last_update = 0;
  foreach my $file (@files) {
    my $rrd_ldom = "";
    if ($item eq "s_ldom_n" or $item eq "s_ldom_san1" or $item eq "s_ldom_san2" or $item eq "s_ldom_san_resp" or $item eq "s_ldom_pack" or $item eq "s_ldom_vnet"){
      if ($server !~ /Solaris--unknown|Solaris\d+--unknown/){
        $rrd_ldom = "$wrkdir/Solaris/$server/$file";
        chomp $rrd_ldom;
      }
      else{
        if ($server =~ /Solaris--unknown|Solaris\d+--unknown/){$server=$lpar;}
        $rrd_ldom = "$wrkdir/Solaris/$lpar/$file";
        chomp $rrd_ldom;
      }
    }
    if ( $item eq "san_no_ldom1" ){
      $rrd_ldom = "$wrkdir/Solaris/$lpar/$file";
      chomp $rrd_ldom;
    }
    my $update = 0;
    if ( $item eq "vmw-iops" or $item eq "clustlan") {
      $file =~ s/\\//g;
      $update = ( stat("$file") )[9];
    }
    elsif ( $item eq "s_ldom_n" or $item eq "s_ldom_san1" or $item eq "s_ldom_san2" or $item eq "s_ldom_san_resp" or $item eq "s_ldom_pack" or $item eq "san_no_ldom1" or $item eq "s_ldom_vnet") {
      $update = ( stat("$rrd_ldom") )[9];
    }
    else {
      $update = ( stat("$dir/$file") )[9];
    }
    # $update = ( stat("$file") )[9] if ( $item eq "vmw-iops" );
    # $update = ( stat("$rrd_ldom") )[9] if ( $item eq "s_ldom_n" or $item eq "s_ldom_san1" or $item eq "s_ldom_san2" or $item eq "s_ldom_san_resp" or $item eq "s_ldom_pack" or $item eq "s_ldom_vnet");
    # $update = ( stat("$rrd_ldom") )[9] if ( $item eq "san_no_ldom1" );
    if ( $update > $last_update ) {
      $last_update = $update;
    }
  } ## end foreach my $file (@files)

  # print STDERR "Updated: " . localtime($last_update) . "\n";
  my $ltime = localtime($last_update);
  $ltime =~ s/:/\\\:/g;
  $t = "COMMENT:Updated\\\: $ltime ";

  # prepare for later testing
  my $san2nmon  = 0;    #means xin/xout separately
  my $minus_one = -1;

  if ( $item =~ /^san2/ && index( $lpar, "--NMON--" ) ne -1 ) {
    my $name_cfg = "";
    my $file_cfg = "$dir$files[0]";
    $file_cfg =~ s/mmm$/cfg/;
    if ( -f "$file_cfg" ) {
      open( FF, "<$file_cfg" ) || error( "can't open $file_cfg: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
      $name_cfg = (<FF>);
      close(FF);
      if ( $name_cfg !~ /full/ ) { $san2nmon = 1; }

      # signal for detail-graph-cgi.pl to graph xin/xout NOT separately
    } ## end if ( -f "$file_cfg" )
  } ## end if ( $item =~ /^san2/ ...)

  # print STDERR "san2nmon je $san2nmon\n";

  my $filter = $filter_max_lansan;
  if ( $item =~ m/^san2/ ) {
    $g_format = "6.0lf";
    $divider  = 1;
    $fpn      = "san-";
    my $header_prefix = "$iops:";
    if ( $detail == 2 ) {
      $last_txt      = " ";
      my $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "$iops";
      if ( $wpar == 1 ) {
        $header_prefix = "$iops";
      }
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
    if ( $iops =~ m/Frames/ ) {
      # Linux san-host* have swapped in/out frames, fixed in 5.07-4
      $ds_name1       = "iops_out";
      $ds_name2       = "iops_in";
    }
    else {
      $ds_name1       = "iops_in";
      $ds_name2       = "iops_out";
    }
    $headline1      = "COMMENT:Average [IO per sec]";
    $vertical_label = "--vertical-label=Read - $iops - Write";
    $filter         = $filter_max_iops;
    if ($san2nmon) {
      $minus_one      = 1;
      $vertical_label = "--vertical-label=$iops Total";
      ( $headline2, undef ) = split( "WRITE", $headline2 );
      $headline2 =~ s/READ/TOTAL/;
      if ( $detail == 1 ) {
        ( undef, my $piece ) = split( ":", $headline2 );
        $headline2 .= $piece;
      }
    } ## end if ($san2nmon)
  } ## end if ( $item =~ m/^san2/)

  if ( $item =~ m/^san_resp/ || $item =~ m/^s_ldom_san_resp/ ) {
    $g_format = "6.2lf";
    $divider  = 1;
    $fpn      = "san_resp-";
    if( $item =~ m/^s_ldom_san_resp/) {
      $fpn = "san_tresp-";
    }
    my $header_prefix = "Response time:";
    if ( $detail == 2 ) {
      $last_txt      = " ";
      my $lpar_slash_nmon_head = "" if $server eq "Linux";
      $header_prefix = "RESP:";
      if ( $wpar == 1 ) {
        $header_prefix = "RESP:";
      }
    } ## end if ( $detail == 2 )
    $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

    if ( !$start_unix eq '' && !$end_unix eq '' ) {
      if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

        # historical reports
        my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
        my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
        $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
        if ( $detail == 10 ) {
          $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
        }
      } ## end if ( isdigit($start_unix...))
    } ## end if ( !$start_unix eq ''...)
    if ($item =~ m/^san_resp/){
      $ds_name1 = "resp_t_r";
      $ds_name2 = "resp_t_w";
    }
    else{
      $ds_name1 = "resp_all";
    }
    $headline1      = "COMMENT:Average [ms]";
    $vertical_label = "--vertical-label=Read - ms - Write";
    $headline2 = "COMMENT:Int     READ/WRITE   Avg      Max"           if ( $item eq "s_ldom_san_resp");
  } ## end if ( $item =~ m/^san_resp/)

  if ($detail == 9) {
    $html_heading = $header;
    $header = "";
  }
  elsif ( $detail != 10 && $detail != 9 && $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
  }

  if ( $type =~ m/y/ ) {

    # lower limits for yearly graphs as they are averaged ....
    $filter = $filter / 10;
  }
  if ( $type =~ m/m/ ) {

    # lower limits for monthly graphs as they are averaged ....
    $filter = $filter / 2;
  }
  my $color_indx = -1;            # prepare color index
  my $line_indx  = 0;             # place enter every 3rd line
  my $file       = "";
  my $i          = 0;
  my $cmd        = "";
  my $cmdx       = "";
  my $j          = 0;
  my $act_time   = localtime();
  my $act_time_u = time();
  my $req_time   = 0;
  my ( @recb, @trab, @rname, @rpath, @avgx, @avgy, @rpathlegend, @rnamelegend, @bubble_info );

  # @bubble_info = ""; # legend bubble for lpar (ASP) name
  my $lnx = 0;

  # variables for LPM use
  my $k = 0;    # indexes for LPM cycle
  my $l = 0;
  my ( @rb, @rbr, @tb, @tbr );
  my @rrd_files_names;
  my $ifn = -1;    # index for this array

  my $gt_ind = 0;  # must be own index for gtype
  my @gtype;
  $gtype[0] = "LINE1";
  $gtype[1] = "LINE1";

  #  $gtype[1] = "STACK";
  # if ($item =~ /^iops/ ) { $item .="a" };
  # if ($item =~ /^fic/ ) { $item .="a" };
  # if ($item =~ /^en/ ) { $item .="a" };
  if ( $item =~ m/a/ ) { $gtype[0] = "AREA" }
  if ( $item =~ m/a/ ) { $gtype[1] = "STACK" }
  if ( $item =~ m/^san_resp|^s_ldom_san_resp/ ) {
    $gtype[0] = "LINE1";
    $gtype[1] = "LINE1";
  }
  if ( $item eq "ds-vmiops" ) {
    $gtype[0] = "AREA";
    $gtype[1] = "STACK";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  foreach $file (@files) {
    chomp($file);
    $bubble_info[$i] = "";    # legend bubble for lpar (ASP) name
    my $fake_name = $file;
    my $rrd = "";
    if ($item eq "s_ldom_n" or $item eq "s_ldom_vnet"){
      $gtype[0] = "AREA";
      $gtype[1] = "STACK";
      $rrd = "$wrkdir/Solaris/$server/$file";
    }
    elsif ($item eq "s_ldom_san1" or $item eq "s_ldom_san2" or $item eq "s_ldom_san_resp" or $item eq "s_ldom_pack"){
      if ($server !~ /Solaris--unknown|Solaris\d+--unknown/){
        $rrd = "$wrkdir/Solaris/$server/$file";
      }
      else{
        $rrd = "$wrkdir/Solaris/$lpar/$file";
      }
     }
    elsif ($item eq "san_no_ldom1"){
      $rrd = "$wrkdir/Solaris/$lpar/$file";
    }
    elsif ($item eq "clustlan"){
            $rrd = $file;
            $fake_name = (split ("/",$file))[-3]; # get server
    }
    else{
      $rrd = "$wrkdir/$server/$host/$lpar/$file";
    }
    if ( $item eq "data_as" || $item eq "iops_as" ) {
      $rrd = "$wrkdir/$server/$host/$lpar--AS400--/ASP/$file";
      ( $fake_name, $bubble_info[$i] ) = asp_fake_name($rrd);
      $bubble_info[$i] =~ s/:/\\:/g;
    }
    elsif ( $item eq "data_ifcb" || $item =~ m/^paket_ifcb/ || $item =~ m/^dpaket_ifcb/ ) {
      $rrd = "$wrkdir/$server/$host/$lpar--AS400--/IFC/$file";
      my $file_cfg = $rrd;
      $file_cfg =~ s/mmc/cfg/;
      if ( -f $file_cfg && open( FF, "<$file_cfg" ) ) {
        $bubble_info[$i] = (<FF>);
        close(FF);
        chomp( $bubble_info[$i] );
        $bubble_info[$i] =~ s/:/\\:/g;
      } ## end if ( -f $file_cfg && open...)
      else {
        print STDERR "can't open $file_cfg: $! :" . __FILE__ . ":" . __LINE__ . "\n";
      }
    } ## end elsif ( $item eq "data_ifcb"...)
    elsif ( $item eq "hmcpgs" ) {
      $rrd  = "$wrkdir/--HMC--$host/$file";
      $item = "pg2";
    }
    elsif ( $item eq "ds-vmiops" ) {
      $fake_name = $file;
      $fake_name =~ s/\.rrv//;
      $fake_name = human_vmware_name($fake_name);
      # do not show UUID names, these are surely old & not interesting
      if ($fake_name =~ m{.{8}-.{4}-.{4}-.{4}-.{12}}) {
        next;
      }
      # print STDERR "5369 \$file $file $wrkdir/$server/$host/$lpar/$file\n";
      # avoid old files which do not exist in the period
      my $rrd_upd_time = (stat("$wrkdir/$server/$host/$lpar/$file"))[9];
      if ( $rrd_upd_time < $req_time ) {
        if ( $type_sam !~ m/x/ ) {
          next; # aviod that for historical reports
          # --> not necessary as $req_time = 0
        }
      }
    }
    elsif ( $item eq "vmw-iops" ) {

      # print STDERR "4301 \$file $file\n";
      $rrd = "$file";

      # prepare datastore name
      my $datastore_uuid = ( split( /\//, $file ) )[-2];
      my $datastore_name = $datastore_uuid;
      my $datastore_path = $file;
      $datastore_path =~ s/$datastore_uuid.*//;
      my @names = <$datastore_path*.$datastore_uuid>;
      if ( @names > 0 && $names[0] ne "" ) {
        $datastore_name = basename( $names[0] );

        # remove extension
        $datastore_name =~ s/\.[^.]+$//;

        # $datastore_name =~ s/:/\\:/g;
        $fake_name = $datastore_name;
      } ## end if ( @names > 0 && $names...)

      # print STDERR "4329 detail-graph-cgi.pl \$datastore_name $datastore_name\n";
    } ## end elsif ( $item eq "vmw-iops")

    # PH --> show all files here
    # avoid old files which do not exist in the period
    #my $rrd_upd_time = (stat("$rrd"))[9];
    # `echo "1200: detail-graph-cgi - $rrd:$rrd_upd_time:$req_time" >> /tmp/e1099`;
    #if ( $rrd_upd_time < $req_time ) {
    #if ( $type_sam !~ m/x/ ) {
    #    next; # aviod that for historical reports
    # --> not necessary as $req_time = 0
    #}
    #}

    my $rrd_name = $fake_name;    # prepare name for gprint
    $rrd_name =~ s/.mmm//;
    $rrd_name =~ s/lan-//;
    $rrd_name =~ s/san-//;
    $rrd_name =~ s/san_resp-//;
    $rrd_name =~ s/san_tresp-//;
    $rrd_name =~ s/.mmx//;
    $rrd_name =~ s/Parm4.mmc//;
    $rrd_name =~ s/.mmc//;

    if ( $rrd_name =~ m/sea-/ && $item ne "vmw-iops" && $item ne "ds-vmiops" && $item ne "s_ldom_n" && $item ne "s_ldom_vnet" ) {
      my $name_cfg = "";
      my $file_cfg = "$wrkdir/$server/$host/$lpar/$rrd_name.cfg";
      if ( -f "$file_cfg" ) {
        open( FF, "<$file_cfg" ) || error( "can't open $file_cfg: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
        $name_cfg = (<FF>);
        close(FF);
        chomp($name_cfg);
        if ( length($name_cfg) == 4 ) { $name_cfg .= " "; }
      } ## end if ( -f "$file_cfg" )
      if ( length($rrd_name) == 8 ) { $rrd_name .= " " }
      $rrd_name .= "(" . $name_cfg . ")";
      $rrd_name =~ s/sea-//;
    } ## end if ( $rrd_name =~ m/sea-/...)
    my $rrd_name_legend = $rrd_name;
    $rrd_name_legend =~ s/:/\\:/g;
    $rrd_name_legend =~ s/%/%%/g;

    # add spaces to rrd name to have xxx chars total (for formating graph legend)
    my $xxx = 8;
    $xxx = 25 if $item eq "vmw-iops" || $item eq "ds-vmiops";
    for ( my $k = length($rrd_name); $k < $xxx; $k++ ) {
      $rrd_name .= " ";
    }

    #print "$wrkdir/$server/$host/$lpar/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    if ( $i eq 0 ) {    # first cycle only
      if ( ( $type =~ "d" ) && ( $t eq "" ) ) {
        RRDp::cmd qq(last "$rrd");
        my $last_tt = RRDp::read;
        chomp($$last_tt);
        my $l = localtime($$last_tt);

        # following must be for RRD 1.2+
        $l =~ s/:/\\\:/g;
        $t = "COMMENT:Updated\\\: $l ";
      } ## end if ( ( $type =~ "d" ) ...)

      #if ( $type =~ "y" ) {
      #   $step_new=86400;
      #}

      # find rrd database file step

      if ( index( $lpar, "--NMON--" ) ne -1 ) {    #you can change step only for NMON
        RRDp::cmd qq("info" "$rrd");
        my $answer = RRDp::read;
        if ( $$answer =~ "ERROR" ) {
          error( "Rrdtool error : $$answer " . __FILE__ . ":" . __LINE__ );
        }
        else {
          my ($step_from_rrd) = $$answer =~ m/step = (\d+)/;
          if ( $step_from_rrd > 0 ) {
            $step_new = $step_from_rrd;
          }
        } ## end else [ if ( $$answer =~ "ERROR")]
      } ## end if ( index( $lpar, "--NMON--"...))

      #$step_new = 600;

      # print STDERR "5499 graphing ENT   : $host:$server:$lpar_slash:$type_sam:$type \$header $header\n" if $DEBUG ;

      if ( $detail == 2 ) {
        $vertical_label = "--interlaced";    # nope string
      }
      # print STDERR "5504 \$detail $detail \$rrd_legend $rrd_legend \$item $item \$no_legend $no_legend\n";
      if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" && ( $item =~ m/^san/ || $item =~ m/^lan/ || $item =~ m/^sea/ || $item =~ m/^data_as/ || $item =~ m/^iops_as/ || $item =~ m/^data_ifcb/ || $item =~ m/^paket_ifcb/ || $item =~ m/^dpaket_ifcb/ || $item =~ m/packets_lan/ || $item =~ m/packets_sea/ || $item eq "vmw-iops" || $item eq "ds-vmiops" || $item eq "s_ldom_n" || $item eq "s_ldom_san1" || $item eq "s_ldom_san2" || $item eq "s_ldom_san_resp" || $item eq "s_ldom_pack" || $item eq "san_no_ldom1" || $item eq "s_ldom_vnet" || $item eq "clustlan")) {
        $no_legend = "--no-legend";
      }
      $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_time"
        "--end" "$end_time"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "--step=$step_new"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--upper-limit" "$upper_limit"
        "--lower-limit" "0"
        "$vertical_label"
        "--alt-y-grid"
        "$xgrid"
        "$no_legend"
        "$disable_rrdtool_tag"
        "$font_def"
        "$font_tit"
        "$headline1"
        "$t2"
       ';
      if ( $item ne "pg2" ) {
        $cmd .= '"$headline2"
              "$t2"
       ';
      }
    } ## end if ( $i eq 0 )

    # every cycle
    #   because of anti ':' moved here
    my $rrd_orig_colon = $rrd;
    $rrd =~ s/:/\\:/g;    #anti ':'
                          # $rrd_name = s/:/\\:/g;
    $rname[$i]       = "$rrd_name";    # for graphing
    $rpath[$i]       = "$rrd";
    $rpathlegend[$i] = "$rrd";
    $rpathlegend[$i] =~ s/&&1/\//g;
    $rpathlegend[$i] =~ s/%/%%/g;
    $rnamelegend[$i] = $rrd_name_legend;

    if ( $item eq "vmw-iops" || $item eq "ds-vmiops" ) {
      $rpathlegend[$i] = "rrd";
      $rnamelegend[$i] = "$rrd_name_legend";
    }

    $recb[$i] = "rcb${i}";
    $trab[$i] = "trb${i}";
    $avgx[$i] = "avg${i}";
    $avgy[$i] = "avgy${i}";

    # start of LPM fun

    # for LPM
    $l = $k;
    my @rrd_files;
    my $lpm_suff = "rrl";
    my $cgi      = 0;
    $rrd_files[0] = $rrd_orig_colon;    #actual is the first

    lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, $file, \@rrd_files, $cgi );

    # print STDERR "5572 detail-graph-cgi.pl lpm_find_files is returning @rrd_files\n";

    foreach $rrd (@rrd_files) {    # LPM cycle
      chomp($rrd);
      $rrd =~ s/:/\\:/g;
      $ifn++;
      $rrd_files_names[$ifn] = "$rrd";
      $rb[$k]                = "data1_stream${k}";
      $rbr[$k]               = "data1_stream_lpm${k}";
      $tb[$k]                = "data2_stream${k}";
      $tbr[$k]               = "data2_stream_lpm${k}";

      if ( $item eq "data_as" ) {
        # fix 5.07-6, ASP data must be multiplayed by 512 to get real numbers
        $cmd .= '"DEF:$rb[$k]-tmp=$rrd_files_names[$ifn]:$ds_name1:AVERAGE"
                 "DEF:$tb[$k]-tmp=$rrd_files_names[$ifn++]:$ds_name2:AVERAGE"
                 "CDEF:$rb[$k]=$rb[$k]-tmp,512,*"
                 "CDEF:$tb[$k]=$tb[$k]-tmp,512,*"
                ';
      }
      elsif ($item eq "s_ldom_san_resp"){
        $cmd .= '"DEF:$rb[$k]=$rrd_files_names[$ifn++]:$ds_name1:AVERAGE"
        ';
      }
      else {
        $cmd .= '"DEF:$rb[$k]=$rrd_files_names[$ifn]:$ds_name1:AVERAGE"
        "DEF:$tb[$k]=$rrd_files_names[$ifn++]:$ds_name2:AVERAGE"
        ';
      }
      if ($item ne "s_ldom_san_resp"){
        if ( $k == $l ) {
          $cmd .= '"CDEF:$rbr[$k]=$rb[$k]"
          "CDEF:$tbr[$k]=$tb[$k++]"
          ';
          $k++;
          next;
        } ## end if ( $k == $l )
        $cmd .= '"CDEF:$rbr[$k]=$rbr[$l],UN,$rb[$k],$rbr[$l],IF"
        "CDEF:$tbr[$k]=$tbr[$l],UN,$tb[$k++],$tbr[$l++],IF"
        ';
        $l++;
        $k++;
      }
      else{
        if ( $k == $l ) {
          $cmd .= '"CDEF:$rbr[$k]=$rb[$k++]"
          ';
          $k++;
          next;
        } ## end if ( $k == $l )
        $cmd .= '"CDEF:$rbr[$k]=$rbr[$l],UN,$rb[$k++],$rbr[$l++],IF"
        ';
        $l++;
        $k++;
      }
    } ## end foreach $rrd (@rrd_files)
    if ($item ne "s_ldom_san_resp"){
      $cmd .= '"CDEF:$recb[$i]_nf=$rbr[$l]"
               "CDEF:$trab[$i]_nf=$tbr[$l++]"
              ';

      # end of LPM fun

      # $cmd .= '"DEF:$recb[$i]_nf=$rpath[$i]:$ds_name1:AVERAGE" #until 4.69 without LPM
      #          "DEF:$trab[$i]_nf=$rpath[$i]:$ds_name2:AVERAGE"

      $cmd .= ' "CDEF:$recb[$i]=$recb[$i]_nf,$filter,GT,UNKN,$recb[$i]_nf,IF"
            "CDEF:$trab[$i]=$trab[$i]_nf,$filter,GT,UNKN,$trab[$i]_nf,IF"
            "CDEF:$recb[$i]-neg=$recb[$i],$minus_one,*"
            "CDEF:$recb[$i]-mil=$recb[$i],$divider,/"
            "CDEF:$trab[$i]-mil=$trab[$i],$divider,/"
           ';

      #  my $lpar_space_proc = $rrd_name;
      #  $lpar_space_proc =~ s/%/%%/g;  # anti '%'
    }
    else{
    $cmd .= '"CDEF:$recb[$i]_nf=$rbr[$l++]"
            ';
    $cmd .= ' "CDEF:$recb[$i]=$recb[$i]_nf,$filter,GT,UNKN,$recb[$i]_nf,IF"
          "CDEF:$recb[$i]-mil=$recb[$i],$divider,/"
         ';
    }
    if ( $item eq "pg2" ) {

      #    $color_indx = $color_indx + 2;
      $cmd .= '"CDEF:$avgx[$i]=$recb[$i],100000,/,$trab[$i],*"
              "CDEF:$avgy[$i]=$recb[$i],1000,/"
              "LINE1:$trab[$i]$color[++$color_indx % ($color_max +1)]: "
              "GPRINT:$trab[$i]:AVERAGE:       %2.1lf"
              "GPRINT:$trab[$i]:MAX:%2.1lf"
              "COMMENT:( from"
              "GPRINT:$avgy[$i++]:AVERAGE:%4.1lf GB )"
              "$t2"
             ';    #do you understand difference $i++ above and below ??
      $i++;
    } ## end if ( $item eq "pg2" )
    else {         # LAN SAN1 SAN2 SEA
      if ($item ne "s_ldom_san_resp"){
        $cmd .= '"COMMENT:$rname[$i]"
        "$gtype[$gt_ind++>0]:$recb[$i]-neg$color[++$color_indx % ($color_max +1)]: "
        "GPRINT:$recb[$i]-mil:AVERAGE:%$g_format"
        ';
        if ($san2nmon) {
          $cmd .= '"PRINT:$recb[$i]-mil:AVERAGE:%$g_format $delimiter $item-nmon $delimiter $rnamelegend[$i] $delimiter $color[$color_indx % ($color_max +1)] $delimiter $rpathlegend[$i]"
          "PRINT:$recb[$i]-mil:MAX:%$g_format $delimiter"
          "GPRINT:$recb[$i++]-mil:MAX: %$g_format    "
          ';
        } ## end if ($san2nmon)
        else {
          $cmd .= '"GPRINT:$recb[$i]-mil:MAX: %$g_format    "
          "PRINT:$recb[$i]-mil:AVERAGE:%$g_format $delimiter $item $delimiter $rnamelegend[$i] $delimiter $color[$color_indx % ($color_max +1)] $delimiter $rpathlegend[$i]"
          "PRINT:$recb[$i]-mil:MAX:%$g_format $delimiter $bubble_info[$i]"
          "$gtype[$gt_ind>0]:$lnx$color[++$color_indx % ($color_max +1)]: "
          "GPRINT:$trab[$i]-mil:AVERAGE: %$g_format"
          "PRINT:$trab[$i]-mil:AVERAGE:%$g_format $delimiter $color[$color_indx % ($color_max +1)]"
          "PRINT:$trab[$i]-mil:MAX:%$g_format $delimiter"
          "GPRINT:$trab[$i++]-mil:MAX: %$g_format    "
          ';
        }
      } ## end else [ if ($san2nmon) ]

      #              "PRINT:$recb[$i]-mil:AVERAGE:%$g_format $delimiter $item $delimiter $rnamelegend[$i] $delimiter $color[$color_indx % ($color_max +1)] $delimiter $rpathlegend[$i]"
      if ( $j == 0 ) {
        $cmdx .= '"LINE1:$lnx:"
                 ';
      }
      if ($item ne "s_ldom_san_resp"){
        $cmdx .= '"$gtype[$j>0]:$trab[$j++]$color[(2*$j - 1) % ($color_max +1)]:"
        ';
      }
      else {
        $cmd .= '"COMMENT:$rname[$i]"
        "$gtype[$gt_ind++>0]:$recb[$i]$color[++$color_indx % ($color_max +1)]: "
        "GPRINT:$recb[$i]-mil:AVERAGE:%$g_format"
        "GPRINT:$recb[$i]-mil:MAX: %$g_format    "
        "PRINT:$recb[$i]-mil:AVERAGE:%$g_format $delimiter $item $delimiter $rnamelegend[$i] $delimiter $color[$color_indx % ($color_max +1)] $delimiter $rpathlegend[$i] $delimiter"
        "PRINT:$recb[$i]-mil:MAX:%$g_format $delimiter $bubble_info[$i++]"
        ';
      }

      #     "GPRINT:$trab[$i]-mil:AVERAGE: %$g_format"
      #     "GPRINT:$trab[$i++]-mil:MAX: %$g_format    "

      if ( $detail == 1 ) {

        #if (1 == $i % 2) {
        $cmd .= '  "$t2"
                ';    # There is remainder
                        #}
      }    #do you understand difference $i++ above and below ??
      else {
        $cmd .= '  "$t2"
                ';
      }
      $i++;
      $j++;
    } ## end else [ if ( $item eq "pg2" ) ]
  }  # end of foreach
  if ( $cmd eq "" ) {
    #error( "Graph rrdtool error no valid rrd file : $item : " . __FILE__ . ":" . __LINE__ );
    $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_time"
        "--end" "$end_time"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "--step=$step_new"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--upper-limit" "$upper_limit"
        "--lower-limit" "0"
            ';
    #return 0;
  }
  if ( !$san2nmon ) {
    $cmd .= $cmdx;
  }

  if ( $detail == 1 ) {
    $cmd .= '"$t2"
      "$t"
      "HRULE:0#000000"
   );';
  } ## end if ( $detail == 1 )
  else {
    if ( $type =~ "d" ) {
      $cmd .= '"$t"
      "HRULE:0#000000"
     );';
    }
    else {
      $cmd .= '"HRULE:0#000000"
     );';
      $t = "";
    }
  } ## end else [ if ( $detail == 1 ) ]

  # "VRULE:0#000000" --> causing sigsegv on Linux

  # if you want to see non interpolated $cmd
  # print STDERR "5761 \$cmd $cmd\n";
  #  # if ( $item =~  /^san1/ ) {
  #     my $sss = "/var/tmp/sss.out";
  #     open (FF, ">>$sss");
  #     print FF "$cmd\n";
  #     close (FF);
  # # }

  # $i      = 0;
  # $j      = 0;
  # $k      = 0;
  # $l      = 0;
  # $ifn    = 0;
  # $gt_ind = 0;

  # if you want to see interpolated $cmd
  # care what file name is in sub alfa
  #  my $t_cmd = $cmd;
  #  $t_cmd =~ s/RRDp::cmd qq\(\$graph_cmd /alfa\(/;
  #  $t_cmd =~ s/\s\"/\n\.\"/g;
  #  eval($t_cmd);

  $i      = 0;
  $j      = 0;
  $k      = 0;
  $l      = 0;
  $ifn    = 0;
  $gt_ind = 0;
  eval $cmd;

  if ($@) {
    error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
  }
  else {
    my $answ = RRDp::read;
    if ( $$answ =~ "ERROR" ) {
      error( "$host:$server Graph rrdtool error : $$answ : " . __FILE__ . ":" . __LINE__ );
    }
    $png_end_heading = ret_graph_param( $answ, $t, $iops );
  } ## end else [ if ($@) ]
  return 0;
} ## end sub graph_ent

sub rrd_running_days {
  my $rrd  = shift;
  my $days = shift;    #inspected time interval in days from now into the past, max 365 days

  # returns nr of days rounded up, so it is ( 1 up to $days - max is rrdtool first)
  # returns nr (1 .. 365) or text if any error
  # example of call
  #  my $nrd = rrd_running_days ($file,4);
  #
  # example of result :
  #                     recv_bytes         trans_bytes        recv_packets       trans_packets
  #
  # 1392508800: NaNQ NaNQ NaNQ NaNQ
  # 1392595200: NaNQ NaNQ NaNQ NaNQ
  # 1392681600: NaNQ NaNQ NaNQ NaNQ
  # 1392768000: 0.0000000000e+00 8.8804282407e+01 NaNQ NaNQ
  # 1392854400: NaNQ NaNQ NaNQ NaNQ
  #
  # in this case returns 2

  return "error file not existing" if !-f "$rrd";
  return "bad nr of days $days" if ( $days < 1 || $days > 365 );

  my $ctime  = time;
  my $rrdres = 86400;
  my $daysd  = $days . "d";
  my @answer = `rrdtool fetch $rrd AVERAGE -r $rrdres -e @{[int($ctime/$rrdres)*$rrdres]} -s e-$daysd`;

  #  if ($answer[0] =~ m/rror/) { return $answer[0]; }
  for ( my $i = 2; $i < ( $days + 2 ); $i++ ) {

    # print "$i,$answer[$i]";
    ( undef, my $nan, undef ) = split( / /, $answer[$i] );
    next if ( $nan =~ m/NaN/ || $nan =~ m/nan/ );
    return ( $days - $i + 3 );
  } ## end for ( my $i = 2; $i < (...))
  return (1);
} ## end sub rrd_running_days

sub graph_ame_mem {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT: ";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  # due to historical reports where is specific time passed
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtoll complains
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ent: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                       # some workaround, it should not go here
  }

  my $dir        = "$wrkdir/$server/$host/$lpar/";
  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "AME: $lpar_slash_nmon :$last_txt$text";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "AME: $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  # my $ds_name1 = "ame_mem";
  # my $ds_name2 = "ame_ratio";
  # my $ds_name3 = "ame_deficit";
  my $headline1      = "COMMENT:[GB]                    Avrg      Max";
  my $headline2      = "COMMENT:       ";
  my $vertical_label = "--vertical-label=Bytes";
  my $cmd            = "";
  my $act_time       = localtime();
  my $act_time_u     = time();
  my $req_time       = 0;
  my ( @recb, @trab, @rname, @rpath );

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  my $rrd = "$wrkdir/$server/$host/$lpar/ame.mmm";

  # avoid old files which do not exist in the period
  #my $rrd_upd_time = (stat("$rrd"))[9];
  #if ( $rrd_upd_time < $req_time ) {
  #  next;
  #}

  my $rrd_name = "ame.mmm";
  $rrd_name =~ s/.mmm//;

  # add spaces to rrd name to have 8 chars total (for formating graph legend)
  for ( my $k = length($rrd_name); $k < 8; $k++ ) {
    $rrd_name .= " ";
  }

  # print "001  AME_MEM  $rrd\n";

  # `echo "809: detail-graph-cgi - $rrd_name:$rrd" >> /tmp/e099`;

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\\:/g;
    $t = "COMMENT:Updated\\\: $l ";
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #   $step_new=86400;
  #}

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  #print "graphing AME_MEM  : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--end" "$end_time"
        "--start" "$start_time"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "--step=$step_new"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "$vertical_label"
        "--alt-y-grid"
        "$xgrid"
        "$no_legend"
        "$disable_rrdtool_tag"
        "$font_def"
        "$font_tit"
      "$headline1"
      "$t2"
';

  # for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;    #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "ame.mmm", \@rrd_files, $cgi );

  # print STDERR "3824 $type lpm_find_files is returning @rrd_files\n";

  my $i = -1;
  my $j;

  foreach $rrd (@rrd_files) {    # LPM cycle
    chomp($rrd);
    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    $cmd .= " DEF:mem${i}=\"$rrd\":ame_mem:AVERAGE";
    $cmd .= " DEF:ratio_val${i}=\"$rrd\":ame_ratio:AVERAGE";
    $cmd .= " DEF:deficit${i}=\"$rrd\":ame_deficit:AVERAGE";

    if ( $i == 0 ) {
      $cmd .= " CDEF:memr${i}=mem${i}";
      $cmd .= " CDEF:ratio_valr${i}=ratio_val${i}";
      $cmd .= " CDEF:deficitr${i}=deficit${i}";
      next;
    } ## end if ( $i == 0 )
    $cmd .= " CDEF:memr${i}=memr${j},UN,mem${i},memr${j},IF";
    $cmd .= " CDEF:ratio_valr${i}=ratio_valr${j},UN,ratio_val${i},ratio_valr${j},IF";
    $cmd .= " CDEF:deficitr${i}=deficitr${j},UN,deficit${i},deficitr${j},IF";
  } ## end foreach $rrd (@rrd_files)
  $cmd .= " CDEF:mem=memr${i}";
  $cmd .= " CDEF:ratio_val=ratio_valr${i}";
  $cmd .= " CDEF:deficit=deficitr${i}";

  # $cmd .= '"DEF:mem=$rrd:$ds_name1:AVERAGE"
  #          "DEF:ratio_val=$rrd:$ds_name2:AVERAGE"
  #          "DEF:deficit=$rrd:$ds_name3:AVERAGE"

  $cmd .= ' "CDEF:ratio=ratio_val,100,/"
          "CDEF:memg=mem,1024,/"
          "CDEF:memb=mem,1048576,*"
          "CDEF:deficitg=deficit,1024,/"
          "CDEF:deficitb=deficit,1048576,*"
         ';
  $cmd .= '"LINE1:memb#00FF00:Compressed memory"
          "GPRINT:memg:AVERAGE: %6.2lf"
          "GPRINT:memg:MAX: %6.2lf"
          "$t2"
          "LINE1:deficitb#FF0000:Deficit memory   "
          "GPRINT:deficitg:AVERAGE: %6.2lf"
          "GPRINT:deficitg:MAX: %6.2lf"
          "$t2"
          "GPRINT:ratio:AVERAGE:  Compression Ratio   %6.1lf"
          "GPRINT:ratio:MAX: %6.1lf"
          "$t2"
         ';
  $cmd .= '"$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
   );';

  # print STDERR "3882 $cmd\n/";
  #"LINE1:ratio#FF0000: "
  # in /var/tmp you can see what is generated
  #  my $ss1 = "/var/tmp/sss.out";
  #  open (FF, ">$ss1");
  #       print FF "$cmd\n";
  #  close (FF);

  # if you want to see interpolated $cmd
  # care what file name is in sub alfa
  #  my $t_cmd = $cmd;
  #  $t_cmd =~ s/RRDp::cmd qq\(graph /alfa\(/;
  #  $t_cmd =~ s/\s\"/\n\.\"/g;
  #  eval($t_cmd);

  eval $cmd;
  if ($@) {
    error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
  }
  else {
    my $answ = RRDp::read;
    if ( $$answ =~ "ERROR" ) {
      error( "$host:$server Graph rrdtool error : $$answ : " . __FILE__ . ":" . __LINE__ );
    }

    # print STDERR "3907 $$answ\n";
    $png_end_heading = ret_graph_param($answ);
  } ## end else [ if ($@) ]
  return 0;
} ## end sub graph_ame_mem

sub graph_lpm {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $detail   = shift;
  $type_sam = "m";    # since 4.60 nothing else, no .rrd anymore
  my $step     = $ENV{SAMPLE_RATE};
  my $step_new = $step;
  my $act_time = time();
  my $lpm_suff = "rrl";

  my $xgrid = xgrid_set( $type, $detail );
  $xgrid =~ s/^--x-grid=//;    # exclude this
  my $text = text_set($type);

  #if ( $type =~ m/y/ ) {
  #  # detection of daily data source : rrm vrs rrd
  #  if ( ! -f "$wrkdir/$server/$host/IVM" && ! -f "$wrkdir/$server/$host/SDMC" ) {
  #    $type_sam = "d";
  #    $lpm_suff = "rrk";
  #  }
  #}
  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lpm: $type : $host:$server:$lpar:$type:$name_out:$type_sam " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  # LPM has been already detected in detail-cgi.pl

  my $rrd = "$wrkdir/$server/$host/$lpar.rm$type_sam";

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace

  lpm( "", "", "", "", "", "", "", "", $wrkdir, $name_out, $server, $host, $lpar, $lpar_slash, $text, $type_sam, $type, $step_new, $xgrid, $act_time, $pic_col, $DEBUG, $rrdtool, $lpm_suff, \@color );

  return 1;
} ## end sub graph_lpm

sub find_human_cluster_name {

  # when cluster dir name is moref : from 4.82
  # find approprite dirname for given cluster name
  my $server_vmw_name;
  my $rrd     = "$wrkdir/$server/$host/cluster.rrc";
  my @files   = glob "$wrkdir/$server/*/cluster_name_*";
  my $rrd_res = "";
  foreach my $file (@files) {
    next if $file eq "";

    # print STDERR "4261 detail-graph-cgi.pl $file $host\n";
    next if ( index( $file, "/$host/" ) == -1 );

    # print STDERR "4263 detail-graph-cgi.pl $file $host\n";
    # example "/home/lpar2rrd/lpar2rrd/data/vmware_7f812c15-81a6-4dfe-85e3-6c9e973985f7/cluster_domain-c7/cluster_name_cluster_Cluster01 renamed"
    ( undef, $server_vmw_name ) = split( "cluster_name_cluster_", $file );
    $file =~ s/cluster_name_.*//;
    $file =~ s/.*cluster_//;

    # print STDERR "4078 $file\n";
    $rrd_res = "$wrkdir/$server/cluster_$file/cluster.rrc";
    last;
  } ## end foreach my $file (@files)
  if ( !-f "$rrd" ) {
    $rrd = $rrd_res;
  }
  return ( $rrd, $server_vmw_name );
} ## end sub find_human_cluster_name

sub find_human_rpool_name {
  my $rpool_name = shift;
  my @files      = glob "$wrkdir/$server/$host/*.$rpool_name";
  foreach my $file (@files) {
    $rpool_name = ( split( /\//, $file ) )[-1];
    $rpool_name =~ s/\..*$//;
    last;
  }
  return $rpool_name;
} ## end sub find_human_rpool_name

sub graph_pool {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  # print STDERR "5585 \$host $host \$server $server \$lpar $lpar\n";
  my $t        = "COMMENT:\" \"";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $last     = "COMMENT:\" \"";
  #  my $in_cores = 1; # graph for vmware is in cores
  #  my $in_cores = 0; # graph for vmware is in MHz
  my $in_cores     = 2;         # graph for vmware has right axis
  my $one_core_hz  = 1000000;
  my $one_core_mhz = 1;

  #  my $vmware ;  # is global variable

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_pool: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $rrd;
  if ( $hitachi ) {
    $rrd = "$wrkdir/$server/$host/SYS-CPU.hrm";
  }
  elsif ( $hyperv) {
    $rrd = "$wrkdir/$server/$host/$lpar.rr$type_sam";
    # print STDERR "5810 $wrkdir/$server/$host/$lpar.rr$type_sam\n";
  }
  elsif( $server eq "Solaris"){
    $rrd = "$wrkdir/$server/$host/$lpar.mmm";
  }
  else {
    $rrd = "$wrkdir/$server/$host/$lpar.rr$type_sam";
  }
  my $server_vmw_name = "";

  if ( $item eq "clustcpu" ) {
    ( $rrd, $server_vmw_name ) = find_human_cluster_name();
    $in_cores = 0;    # graph for vmware is in MHz
  }

  if ( $item eq "rpcpu" ) {
    ( undef, $server_vmw_name ) = find_human_cluster_name();
    my $rpool_name = find_human_rpool_name($lpar);
    $server_vmw_name .= ": $rpool_name";

    $rrd      = "$wrkdir/$server/$host/$lpar.rrc";
    $in_cores = 0;                                   # graph for vmware is in MHz
  } ## end if ( $item eq "rpcpu" )

  # print STDERR "4305 graph pool \$rrd $rrd ,$server,$lpar,$item,$type,$name_out,$type_sam,$detail,$graph_hours,$entitle,$start_unix,$end_unix,\n";

  if ( !-f "$rrd" || ( stat $rrd )[7] == 0 ) {
    error( "$rrd does not exists or 0B size for item $item " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  my $proc_limit_ghz = 0;
  my $total          = 0;
  my $res            = 0;
  my $unres          = 0;
  if ($vmware) {    # read cfg file
    if ( ( $item ne "clustcpu" ) && ( $item ne "rpcpu" ) ) {
      my $linepx = "";
      if ( -f "$wrkdir/$server/$host/host.cfg" ) {
        open( FR, "< $wrkdir/$server/$host/host.cfg" );
        foreach my $linep (<FR>) {
          if ( $linep =~ /CPU/ ) {
            $linepx = $linep;
            last;
          }
        } ## end foreach my $linep (<FR>)
      } ## end if ( -f "$wrkdir/$server/$host/host.cfg")
      close(FR);
      if ( $linepx =~ /CPU/ ) {
        ( undef, my $rest_line ) = split( " ", $linepx, 2 );

        # print STDERR "\$rest_line $rest_line\n";
        ( undef, $res, $unres, undef, undef, $total ) = split( " ", $rest_line );

        # print STDERR "\$res $res \$unres $unres \$total $total\n";
      } ## end if ( $linepx =~ /CPU/ )
    } ## end if ( ( $item ne "clustcpu"...))
  } ## end if ($vmware)

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "CPU pool :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "CPU pool: $server :$last_txt$text";
    if ( $item eq "clustcpu" || $item eq "rpcpu" ) {

      # $header = "CPU pool: $host :$last_txt$text";
      $header = "CPU pool: $last_txt$text";
    }
    elsif ( $hitachi ) {

      # $header = "CPU pool: $host :$last_txt$text";
      $header = "CPU pool: $host :$last_txt$text";
    }
  } ## end if ( $detail == 1 )

  if ( $detail == 2 ) {
    # $header = "$server :$last_txt$text";
    $header = "pool :$last_txt$text";
    if ($item =~ /-max/) {
      $header = "pool :max:$last_txt$text";
    }
    if ( $item eq "clustcpu" || $item eq "rpcpu" ) {

      # $header = "CPU pool: $host :$last_txt$text";
      $header = "CPU pool: $last_txt$text";
    }
  } ## end if ( $detail == 2 )

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "CPU pool: $server : $start_human - $end_human";
      if ( $item eq "clustcpu" || $item eq "rpcpu" ) {
        $header = "CPU pool: $server_vmw_name : $start_human - $end_human";
      }
      elsif ( $hitachi ) {
        $header = "CPU pool: $host : $start_human - $end_human";
      }
      if ( $detail != 10 && $detail != 9 && $width < 700) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  #  if (($type =~ "d" && $graph_hours == 0 ) || ($type =~ "d" && $vmware)) {
  if ( ( $type =~ "d" && $graph_hours == 0 ) || ($vmware) ) {
    my $rrd_last = $rrd;
    $rrd_last = rrd_file_actual("$rrd") if ! $hyperv; #

    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    my $l       = localtime($$last_tt);
    # print STDERR "5728 \$rrd_last $rrd_last \$last_tt $last_tt \$l $l\n";

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    if ( $type =~ "d" ) {
      $t = "COMMENT:Updated\\:\" $l \"";
    }

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );
        if ($vmware) {

          # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[13] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $utiltot_mhz = 1;

            # my $one_core_hz    = 1;

            if ( ( $item eq "clustcpu" ) || ( $item eq "rpcpu" ) ) {
              $utiltot_mhz = sprintf( "%e", $m_arr[1] );
            }
            else {
              $utiltot_mhz = sprintf( "%e", $m_arr[2] );
            }
            $one_core_hz = sprintf( "%e", $m_arr[3] );

            # prepare right axis %
            if ( $item eq "rpcpu" ) {
              if ( $m_arr[13] != 0 ) {
                $proc_limit_ghz = sprintf( "%e", 100 / ( $m_arr[13] / 1000 ) );
              }
            }

            if ($in_cores) {
              my $util = sprintf( "%6.1f", $utiltot_mhz / $one_core_hz * 1000000 );
              if ( $type !~ "d" ) {next}
              $last = "COMMENT:\"Last CPU usage in cores     $util\"";
              if ( $item eq "pool-max" ) {
                $last = "COMMENT:\"Last CPU usage in cores       $util\"";
              }
              if ( $in_cores == 2 ) {
                my $utlt_mhz = sprintf( "%7.0f", $utiltot_mhz );
                my $utlt_ghz = sprintf( "%6.1f", $utiltot_mhz / 1000 );
                $last = "COMMENT:\"Last CPU usage             $util  $utlt_ghz\"";
                if ( $item eq "pool-max" ) {
                  $last = "COMMENT:\"Last CPU usage               $util   $utlt_ghz\"";
                }
              } ## end if ( $in_cores == 2 )
            } ## end if ($in_cores)
            else {
              my $util = sprintf( "%6.2f", $utiltot_mhz / 1000 );
              $last = "COMMENT:\"Last utilization in CPU GHz   $util\"";
              if ( $item eq "pool-max" ) {
                $last = "COMMENT:\"Last utilization in CPU GHz     $util\"";
              }
            } ## end else [ if ($in_cores) ]
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end if ($vmware)
        elsif ($hyperv) {
          # print STDERR "4751 detail-graph-cgi.pl \@m_arr @m_arr\n";
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ ) {
            my $util = sprintf( "%6.2f", $m_arr[1] / $m_arr[2] * $m_arr[3] / 100000 / 100 );
            $last = "COMMENT:\"Last CPU usage in cores  $util\"";
          }
        } ## end elsif ($hyperv)
        elsif ($hitachi) {
          if ( $m_arr[4] =~ /\d/ ) {
            my $usedp    = sprintf( "%e", $m_arr[4] );
            my $util = sprintf( "%.2f", $usedp );
            $last = "COMMENT:\"Last CPU usage in cores  $util\"";
          }
        } ## end elsif ($hitachi)
        else {
          # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $total_pool_cycles = sprintf( "%e", $m_arr[1] );
            my $utilized_pool_cyc = sprintf( "%e", $m_arr[2] );
            my $conf_proc_units   = sprintf( "%e", $m_arr[3] );
            my $bor_proc_units    = sprintf( "%e", $m_arr[4] );
            if ( $total_pool_cycles != 0 ) {
              my $util = sprintf( "%.2f", ( $conf_proc_units + $bor_proc_units ) * ( $utilized_pool_cyc / $total_pool_cycles ) );

              #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");

              $last = "COMMENT:\"Last utilization in CPU cores  $util\"";
              if ( $item eq "pool-max" ) {
                $last = "COMMENT:\"Last utilization in CPU cores    $util\"";
              }
            } ## end if ( $total_pool_cycles...)
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end else [ if ($vmware) ]
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( ( $type =~ "d" &&...))

  #  if ( $type =~ "y" ) {
  #    $step_new=86400;
  #  }

  #my $rrd_y = "";
  #if ( $type_sam =~ "y" ) {
  #  # u rocnich grafu nejdrive pouzit rrm data a pokud nejsou tak teprve rrd
  #  if ( $rrd !~ m/\.rrd$/ ) {
  #    $rrd_y = $rrd;
  #    $rrd_y =~ s/\.rrm$/\.rrd/;
  #    if ( -f $rrd_y ) {
  #     $rrd = $rrd_y;
  #    }
  #}

  #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  #  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label;

  if ($vmware && !$in_cores) { $vertical_label = "--vertical-label=\"CPU usage in GHz\""; }
  elsif ($hitachi) { $vertical_label = "--vertical-label=\"CPU usage %\""; }
  elsif ($item eq "solaris_pool"){  $vertical_label = "--vertical-label=\"Usage\""; }
  else { $vertical_label = "--vertical-label=\"CPU cores\""; }

  #$vertical_label="--vertical-label=\"CPU usage in %\"" if -f "$wrkdir/$server/$host/hyperv.txt";

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  $rrd =~ s/:/\\:/g;
  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  if ( $in_cores == 2 && $vmware ) {

    #  $cmd .= " --right-axis-label \"CPU usage in MHz\"";
    $one_core_mhz = int( $one_core_hz / 1000000 );
    my $one_core_ghz = $one_core_hz / 1000000000;
    if ($yes_right_axis) {
      $cmd .= " --right-axis $one_core_ghz:0";
      $cmd .= " --right-axis-format %1.1lf";
    }
  } ## end if ( $in_cores == 2 &&...)
  if ( $item eq "rpcpu" && $proc_limit_ghz != 0 ) {    # right axis limit
    $cmd .= " --right-axis $proc_limit_ghz:0";
    $cmd .= " --right-axis-format %3.0lf";
  }

  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $xgrid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";

  my $file_pth = "$wrkdir/$server/*/$lpar.rr$type_sam";

  if ($vmware) {

    if ( $item eq "clustcpu" ) {

      # for Total MHz create sum of all ESXi CPU pool Total MHz (CPU_Alloc) in this cluster
      my $hosts_in_cluster = $rrd;
      $hosts_in_cluster =~ s/cluster.rrc/hosts_in_cluster/;
      open( FF, "<$hosts_in_cluster" ) || error( "can't open $hosts_in_cluster: $! :" . __FILE__ . ":" . __LINE__ ) && return;
      my @hosts = (<FF>);
      close(FF);

      # print STDERR "5260 \@hosts @hosts\n"; # e.g. 10.22.11.8XORUX10.22.11.10
      my $i       = 0;
      my $rpn_sum = "";
      foreach my $one_host (@hosts) {
        chomp $one_host;
        ( my $host, my $cluster ) = split( "XORUX", $one_host );

        # print STDERR "5263 testing $wrkdir/$host/$cluster/pool.rrm\n";
        if ( -f "$wrkdir/$host/$cluster/pool.rrm" ) {
          $cluster =~ s/:/\\:/;    # if there is port
          $cmd .= " DEF:hosts_cpu${i}=\"$wrkdir/$host/$cluster/pool.rrm\":CPU_Alloc:AVERAGE";
          if ( $i == 0 ) {
            $rpn_sum = "hosts_cpu0";
          }
          else {
            $rpn_sum .= ",hosts_cpu${i},+";
          }
          $i++;
        } ## end if ( -f "$wrkdir/$host/$cluster/pool.rrm")
      } ## end foreach my $one_host (@hosts)
      $i > 0 || error( "Cluster has no ESXi servers in $hosts_in_cluster: $! :" . __FILE__ . ":" . __LINE__ ) && return;

      # print STDERR "5279 \$rpn_sum $rpn_sum\n";
      $cmd .= " CDEF:hosts_cpu=$rpn_sum,1000,/";
      $cmd .= " DEF:cpu_MHz=\"$rrd\":CPU_total_MHz:AVERAGE";
      $cmd .= " CDEF:cpu=cpu_MHz,1000,/";
      $cmd .= " DEF:cpuutiltot_MHz=\"$rrd\":CPU_usage_MHz:AVERAGE";
      $cmd .= " CDEF:cpuutiltot=cpuutiltot_MHz,1000,/";
      $cmd .= " COMMENT:\"   GHz                          Avg      Max\\n\"";
    } ## end if ( $item eq "clustcpu")
    elsif ( $item eq "rpcpu" ) {
      $cmd .= " DEF:cpuutiltot_MHz=\"$rrd\":CPU_usage_MHz:AVERAGE";
      $cmd .= " CDEF:cpuutiltot=cpuutiltot_MHz,1000,/";
      $cmd .= " DEF:cpu_limit_MHz=\"$rrd\":CPU_limit:AVERAGE";
      $cmd .= " CDEF:cpu_limit=cpu_limit_MHz,1000,/";
      $cmd .= " DEF:cpu_reservation_MHz=\"$rrd\":CPU_reservation:AVERAGE";
      $cmd .= " CDEF:cpu_reservation=cpu_reservation_MHz,1000,/";
      if ( $proc_limit_ghz != 0 ) {
        $cmd .= " COMMENT:\"   GHz                           Avg      Max  (right axis = % of Limit)\\n\"";
      }
      else {
        $cmd .= " COMMENT:\"   GHz                           Avg      Max\\n\"";
      }
    } ## end elsif ( $item eq "rpcpu" )

    else {
      my ( $result_cmd, $CPU_Alloc, $CPU_usage, $host_hz ) = LPM_easy( $file_pth, "0", "0", "CPU_Alloc", "CPU_usage", "host_hz" );

      $cmd .= $result_cmd;

      $cmd .= " CDEF:cpu_entitl_mhz=$CPU_Alloc";
      $cmd .= " CDEF:utiltot_mhz=$CPU_usage";
      $cmd .= " CDEF:one_core_hz=$host_hz";
      $cmd .= " COMMENT:\"   Average                   cores      Ghz (right axis)\\n\"";
    } ## end else [ if ( $item eq "clustcpu")]
    if ($in_cores) {
      $cmd .= " CDEF:cpuutiltot=utiltot_mhz,one_core_hz,/,1000000,*";
      $cmd .= " CDEF:cpu=cpu_entitl_mhz,one_core_hz,/,1000000,*";
      $cmd .= " CDEF:cpu_entitl_ghz=cpu_entitl_mhz,1000,/";
      $cmd .= " CDEF:utiltot_ghz=utiltot_mhz,1000,/";

      #         $cmd .= " CDEF:reserved=cpu,POP,$res,one_core_hz,/,1000000,*";
      #         $cmd .= " AREA:reserved#00FF00:\" Reserved for VM          \"";
      #         $cmd .= " GPRINT:reserved:AVERAGE:\" %2.1lf\"";
      #         $cmd .= " CDEF:reserve=cpu,POP,$res,1,*";
      #         $cmd .= " GPRINT:reserve:AVERAGE:\" %6.0lf\"";
      #         $cmd .= " $t2";
      #         $cmd .= " CDEF:unreserved=cpu,POP,$unres,one_core_hz,/,1000000,*";
      #         $cmd .= " STACK:unreserved#FFFF00:\" Unreserved               \"";
      #         $cmd .= " GPRINT:unreserved:AVERAGE:\" %2.1lf\"";
      #         $cmd .= " CDEF:unreserve=cpu,POP,$unres,1,*";
      #         $cmd .= " GPRINT:unreserve:AVERAGE:\" %6.0lf\"";
      #         $cmd .= " $t2";
      my $system = $total - $res - $unres;

      #         $cmd .= " CDEF:system=cpu,POP,$system,one_core_hz,/,1000000,*";
      #         $cmd .= " STACK:system#0080FF:\" System                   \"";
      #         $cmd .= " GPRINT:system:AVERAGE:\" %2.1lf\"";
      #         $cmd .= " CDEF:syste=cpu,POP,$system,1,*";
      #         $cmd .= " GPRINT:syste:AVERAGE:\" %6.0lf\"";
      #         $cmd .= " $t2";
      my $form_dec = "%6.1lf";
      $cmd .= " AREA:cpu#00FF00:\" Total                 \"";
      $cmd .= " GPRINT:cpu:AVERAGE:\"$form_dec\"";
      $cmd .= " GPRINT:cpu_entitl_ghz:AVERAGE:\"$form_dec\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization           \"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"$form_dec\"";
      $cmd .= " GPRINT:utiltot_ghz:AVERAGE:\"$form_dec\"";
      $cmd .= " $t2";
    } ## end if ($in_cores)
    else {
      if ( $item eq "rpcpu" ) {
        $cmd .= " AREA:cpu_reservation#00FF00:\" Reservation             \"";
        $cmd .= " GPRINT:cpu_reservation:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpu_reservation:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
        $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization             \"";
        $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpuutiltot:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
        $cmd .= " LINE1:cpu_limit#000000:\" Limit                   \"";
        $cmd .= " GPRINT:cpu_limit:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpu_limit:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
      } ## end if ( $item eq "rpcpu" )
      elsif ( $item eq "clustcpu" ) {
        $cmd .= " LINE1:hosts_cpu#000000:\" Total                   \"";
        $cmd .= " GPRINT:hosts_cpu:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:hosts_cpu:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
        $cmd .= " AREA:cpu#00FF00:\" Total effective         \"";
        $cmd .= " GPRINT:cpu:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpu:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
        $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization             \"";
        $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpuutiltot:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
      } ## end elsif ( $item eq "clustcpu")
      else {
        $cmd .= " AREA:cpu#00FF00:\" Total                   \"";
        $cmd .= " GPRINT:cpu:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpu:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
        $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization             \"";
        $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %6.1lf\"";
        $cmd .= " GPRINT:cpuutiltot:MAX:\" %6.1lf\"";
        $cmd .= " $t2";
      } ## end else [ if ( $item eq "rpcpu" )]
    } ## end else [ if ($in_cores) ]
  } ## end if ($vmware)
  elsif ($hitachi) {
    $cmd .= " --upper-limit=100";
    $cmd .= " DEF:cpu_usedp=\"$rrd\":cpu_usedp:AVERAGE";
    $cmd .= " COMMENT:\"[%]                      Avrg\\n\"";
    $cmd .= " LINE1:cpu_usedp#FF0000:\" Usage           \"";
    $cmd .= " GPRINT:cpu_usedp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
  } ## end elsif ($hitachi)
  elsif ($item eq "solaris_pool"){
    $cmd .= " DEF:size_c=\"$rrd\":size_in_cores:AVERAGE";
    $cmd .= " DEF:used_s=\"$rrd\":used:AVERAGE";
    $cmd .= " COMMENT:\"                       Avrg\\n\"";
    $cmd .= " COMMENT:\"Average   \\n\"";
    $cmd .= " AREA:size_c#00FF00:\" Size CPU      \"";
    $cmd .= " GPRINT:size_c:AVERAGE:\"    %3.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE1:used_s#FF0000:\" Used CPU   \"";
    $cmd .= " GPRINT:used_s:AVERAGE:\"       %3.2lf\"";
    $cmd .= " $t2";
  }
  else {
    #       my $file_pth = "$wrkdir/$server/*/$lpar.rr$type_sam";

    my ( $result_cmd, $total_pool_cycles, $utilized_pool_cyc, $conf_proc_units, $bor_proc_units ) = LPM_easy( $file_pth, "0", "0", "total_pool_cycles", "utilized_pool_cyc", "conf_proc_units", "bor_proc_units" );

    if ( $item eq "pool-max" ) {
      my $file_pth_max = "$wrkdir/$server/*/$lpar.xr$type_sam";

      my ( $result_cmd_max, $total_pool_cycles_max, $utilized_pool_cyc_max, $conf_proc_units_max, $bor_proc_units_max ) = LPM_easy( $file_pth_max, "0", "0", "total_pool_cycles", "utilized_pool_cyc", "conf_proc_units", "bor_proc_units" );

      # prepare DEF|CDEF for max
      $total_pool_cycles_max =~ s/var/varmax/g;
      $utilized_pool_cyc_max =~ s/var/varmax/g;
      $conf_proc_units_max =~ s/var/varmax/g;
      $bor_proc_units_max =~ s/var/varmax/g;

      my $result_to_parse = $result_cmd_max;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      my $result_cmd_max_new;

      foreach my $cmd_line (@result_cmd_lines) {
        chomp $cmd_line;

        #print "$cmd_line\n";
        $cmd_line =~ s/^\s+//g;
        $cmd_line =~ s/\s+$//g;

        my $line_max = $cmd_line;

        if ( $cmd_line =~ "^DEF:" ) {
          $line_max =~ s/^DEF:var/DEF:varmax/;
          $line_max =~ s/:AVERAGE$/:MAX/;
        }
        if ( $cmd_line =~ "^CDEF:" ) {
          $line_max =~ s/var/varmax/g;
        }

        $result_cmd_max_new .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)

      $cmd .= $result_cmd_max_new;
      $cmd .= " CDEF:totcyc_max=$total_pool_cycles_max";
      $cmd .= " CDEF:uticyc_max=$utilized_pool_cyc_max";
      $cmd .= " CDEF:cpu_max=$conf_proc_units_max";
      $cmd .= " CDEF:cpubor_max=$bor_proc_units_max";

      $cmd .= " CDEF:totcpu_max=cpu_max,cpubor_max,+";
      $cmd .= " CDEF:fail_max=uticyc_max,totcyc_max,GT,1,0,IF";
      $cmd .= " CDEF:cpuutil_max=uticyc_max,totcyc_max,GT,UNKN,uticyc_max,totcyc_max,/,IF";
      $cmd .= " CDEF:cpuutiltot_max=cpuutil_max,totcpu_max,*";
      $cmd .= " CDEF:utilisa_max=cpuutil_max,100,*";

      $cmd .= $result_cmd;
      $cmd .= " CDEF:totcyc=$total_pool_cycles";
      $cmd .= " CDEF:uticyc=$utilized_pool_cyc";
      $cmd .= " CDEF:cpu=$conf_proc_units";
      $cmd .= " CDEF:cpubor=$bor_proc_units";

      $cmd .= " CDEF:totcpu=cpu,cpubor,+";
      $cmd .= " CDEF:fail=uticyc,totcyc,GT,1,0,IF";
      $cmd .= " CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF";
      $cmd .= " CDEF:cpuutiltot=cpuutil,totcpu,*";
      $cmd .= " CDEF:utilisa=cpuutil,100,*";

      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:cpu#00FF00:\" Configured CPU cores      \"";
      $cmd .= " GPRINT:cpu:AVERAGE:\" %3.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:cpubor#FFFF00:\" Not assigned CPU cores    \"";
      $cmd .= " GPRINT:cpubor:AVERAGE:\" %3.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in CPU cores  \"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %3.2lf\"";
      $cmd .= " COMMENT:(";
      $cmd .= " GPRINT:utilisa:AVERAGE:\" %2.1lf\"";
      $cmd .= " COMMENT:\%)";
      $cmd .= " $t2";
      if ( $graph_cmd =~ m/graphv$/ ) {
        $cmd .= " LINE1:cpuutiltot_max#0088FF:\" Maximum peaks in CPU cores\":dashes=1,2";
      }
      else {
        # RRDTool 1.2.x does not support dashed lines
        $cmd .= " LINE1:cpuutiltot_max#0088FF:\" Maximum peaks in CPU cores\"";
      }
      $cmd .= " GPRINT:cpuutiltot_max:MAX:\" %3.2lf\"";
      $cmd .= " $t2";

    } ## end if ( $item eq "pool-max")
    elsif ($hyperv) {

      # print STDERR "6181 detail-graph-cgi.pl pool hyperv \$rrd $rrd\n";
      $cmd .= " DEF:cpu_perc=\"$rrd\":PercentTotalRunTime:AVERAGE";
      $cmd .= " DEF:cpu_time=\"$rrd\":Timestamp_PerfTime:AVERAGE";
      $cmd .= " DEF:cpu_freq=\"$rrd\":Frequency_PerfTime:AVERAGE";

      #$cmd .= " CDEF:prev_perc=PREV(cpu_perc)";
      #$cmd .= " CDEF:prev_time=PREV(cpu_time)";
      $cmd .= " CDEF:cpuutiltot=cpu_perc,cpu_time,/,cpu_freq,*,100000,/,100,/";    # to be in cores

      #         $cmd .= " COMMENT:\"   Average   \\n\"";
      #         $cmd .= " AREA:cpu#00FF00:\" Configured CPU cores    \"";
      #         $cmd .= " GPRINT:cpu:AVERAGE:\" %3.1lf\"";
      #         $cmd .= " $t2";
      #         $cmd .= " STACK:cpubor#FFFF00:\" Not assigned CPU cores  \"";
      #         $cmd .= " GPRINT:cpubor:AVERAGE:\" %3.1lf\"";
      #         $cmd .= " $t2";

      # find out max cores of the server from first line of cpu.html im comment
      my $max_cpu_cores = 0;
      if ( open( FR, "< $wrkdir/$server/$host/cpu.html" ) ) {
        my $firstLine = <FR>;    # example <BR><CENTER><TABLE class="tabconfig tablesorter"><!cores:8>
        close FR;
        ( undef, my $m_cpu_cores ) = split( "cores:", $firstLine );
        if ( defined $m_cpu_cores ) {
          $m_cpu_cores =~ s/>//;
          $max_cpu_cores = $m_cpu_cores;
        }
      } ## end if ( open( FR, "< $wrkdir/$server/$host/cpu.html"...))
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %3.2lf\"";
      $cmd .= " $t2";
      $cmd .= " COMMENT:\"Max cpu cores           $max_cpu_cores\"";
      $cmd .= " $t2";

    } ## end elsif ($hyperv)
    else {
      $cmd .= $result_cmd;
      $cmd .= " CDEF:totcyc=$total_pool_cycles";
      $cmd .= " CDEF:uticyc=$utilized_pool_cyc";
      $cmd .= " CDEF:cpu=$conf_proc_units";
      $cmd .= " CDEF:cpubor=$bor_proc_units";

      $cmd .= " CDEF:totcpu=cpu,cpubor,+";
      $cmd .= " CDEF:fail=uticyc,totcyc,GT,1,0,IF";
      $cmd .= " CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF";
      $cmd .= " CDEF:cpuutiltot=cpuutil,totcpu,*";
      $cmd .= " CDEF:utilisa=cpuutil,100,*";

      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:cpu#00FF00:\" Configured CPU cores    \"";
      $cmd .= " GPRINT:cpu:AVERAGE:\" %3.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:cpubor#FFFF00:\" Not assigned CPU cores  \"";
      $cmd .= " GPRINT:cpubor:AVERAGE:\" %3.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\" %3.2lf\"";
      $cmd .= " COMMENT:(";
      $cmd .= " GPRINT:utilisa:AVERAGE:\" %2.1lf\"";
      $cmd .= " COMMENT:\%)";
      $cmd .= " $t2";
    } ## end else [ if ( $item eq "pool-max")]
  } ## end else [ if ($vmware) ]
  $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";

  $cmd =~ s/\\"/"/g;

  # print STDERR "001 $cmd\n";

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_pool : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_pool

# Note!!
# this function is defined in detail-graph-cgi.pl as LPM_easy and in premium.pl as LPM_easy_premium
# when you make any change then replicate it in both files!!
#

sub LPM_easy {

  # this is for case e.g. HMC change, similar to LPM
  # join data streams from rrd files in one data_stream
  # call:
  #  my ($result_cmd,$result_stream_1,...,$result_stream_x) = LPM_easy($path_to_find_files,$var_indx,$req_time,$data_stream_1,...,$data_stream_x);
  # no limit for x
  # $result_cmd is CMD string for RRD

  my $file_pth = shift @_;    # path to find files
  my $var_indx = shift @_;    # to make variables unique in aggregated graphs
  my $req_time = shift @_;    # only files with newer timestamp are taken into consideration
  $file_pth =~ s/ /\\ /g;
  my $no_name = "";

  my @files = (<$file_pth$no_name>);    # unsorted, workaround for space in names
                                        # print STDERR "found pool files: @files\n";

  my @ds = @_;

  #print STDERR "000 in sub LPM_easy \@ds @ds\n";

  # prepare help variables
  my $prep_names = "";
  for ( my $x = 0; $x < @ds; $x++ ) { $prep_names .= "var" . $var_indx . $x . "," }
  my @ids = split( ",", "$prep_names" );
  $prep_names = "";
  for ( my $x = 0; $x < @ds; $x++ ) { $prep_names .= "var_r" . $var_indx . $x . "," }
  my @rids = split( ",", "$prep_names" );

  my $i = -1;
  my $j;
  my $rrd = "";
  my $cmd = "";

  foreach my $rrd (@files) {    # LPM alias cycle
    chomp($rrd);

    if ( $req_time > 0 ) {
      my $rrd_upd_time = ( stat("$rrd") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
    } ## end if ( $req_time > 0 )

    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

    if ( $i == 0 ) {
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
      next;
    }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
  } ## end foreach my $rrd (@files)

  if ( $i == -1 ) {

    # no fresh file has been found, do it once more qithout restriction to show at least the empty graph
    foreach my $rrd (@files) {    # LPM alias cycle
      chomp($rrd);

      $i++;
      $j = $i - 1;
      $rrd =~ s/:/\\:/g;

      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

      if ( $i == 0 ) {
        for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
        next;
      }
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
    } ## end foreach my $rrd (@files)
  } ## end if ( $i == -1 )

  my $ret_string = "";
  for ( my $k = 0; $k < @ds; $k++ ) { $ret_string .= "$rids[$k]${i},"; }

   #print STDERR "\$cmd $cmd \$ret_string $ret_string\n";

  #print STDERR "001 $cmd,split(",",$ret_string)\n";
  return ( $cmd, split( ",", $ret_string ) );
} ## end sub LPM_easy

sub graph_shpool {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ams: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $rrd = "$wrkdir/$server/$host/$lpar.rr$type_sam";

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/SharedPool//g;    # to show slash and not &&1 which is general replacemnt for it

  my $lpar_pool_alias = $lpar;
  $lpar_pool_alias =~ s/SharedPool/CPU pool /g;

  # add CPU pool alias into png header (if exists and it is not a default CPU pool)
  my $pool_id = $lpar;
  $pool_id =~ s/SharedPool//g;

  if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt" ) {
    open( FR, "< $wrkdir/$server/$host/cpu-pools-mapping.txt" );
    foreach my $linep (<FR>) {
      chomp($linep);
      ( my $id, my $pool_name ) = split( /,/, $linep );
      if ( $id == $pool_id ) {
        $lpar_pool_alias = "$pool_name";
        last;
      }
    } ## end foreach my $linep (<FR>)
    close(FR);
  } ## end if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt")

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "CPU pool: $lpar_pool_alias :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "CPU pool: $server $lpar_pool_alias :$last_txt$text";
  }

  if ( $detail == 2 ) {
    # $header = "$server $lpar_pool_alias :$last_txt$text";
    $header = "$lpar_pool_alias :$last_txt$text";
    if ($item =~ /-max/) {
      $header = "$lpar_pool_alias :max:$last_txt$text";
    }
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "CPU pool: $server $lpar_pool_alias : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  #print "Graph pool :$lpar\n";
  if ( $type =~ "d" && $graph_hours == 0 ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    my $l       = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\: $l ";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;
    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
          my $total_pool_cycles = sprintf( "%e", $m_arr[1] );
          my $utilized_pool_cyc = sprintf( "%e", $m_arr[2] );
          my $max_pool_units    = sprintf( "%e", $m_arr[3] );
          my $res_pool_units    = sprintf( "%e", $m_arr[4] );
          if ( $total_pool_cycles != 0 ) {
            my $util = sprintf( "%.2f", ( $utilized_pool_cyc / $total_pool_cycles ) * $max_pool_units );

            #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");

            $last = "COMMENT:Last utilization in CPU cores  $util";
            if ( $item eq "shpool-max" ) {
              $last = "COMMENT:Last utilization in CPU cores    $util";
            }
          } ## end if ( $total_pool_cycles...)
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" && $graph_hours...)

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  #  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=CPU cores";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  my $cmd = "";
  my ( $result_cmd,     $max_pool_units,     $res_pool_units,     $total_pool_cycles,     $utilized_pool_cyc );
  my ( $result_cmd_max, $max_pool_units_max, $res_pool_units_max, $total_pool_cycles_max, $utilized_pool_cyc_max );
  if ( $lpar =~ "SharedPool[1-9]" || $lpar =~ "SharedPool[1-9][0-9]" ) {

    #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;

    $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "--step=$step_new"
      "--lower-limit=0.00"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--alt-autoscale-max"
      "--upper-limit=0.1"
      "$vertical_label"
      "--units-exponent=1.00"
      "--alt-y-grid"
      "$xgrid"
      "$no_legend"
      "$disable_rrdtool_tag"
      "$font_def"
      "$font_tit"
    ';

    my $file_pth = "$wrkdir/$server/*/$lpar.rr$type_sam";
    ( $result_cmd, $max_pool_units, $res_pool_units, $total_pool_cycles, $utilized_pool_cyc ) = LPM_easy( $file_pth, "0", "0", "max_pool_units", "res_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

    if ( $item eq "shpool-max" ) {
      my $file_pth_max = "$wrkdir/$server/*/$lpar.xr$type_sam";

      ( $result_cmd_max, $max_pool_units_max, $res_pool_units_max, $total_pool_cycles_max, $utilized_pool_cyc_max ) = LPM_easy( $file_pth_max, "0", "0", "max_pool_units", "res_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

      # prepare DEF|CDEF for max
      $max_pool_units_max =~ s/var/varmax/g;
      $res_pool_units_max =~ s/var/varmax/g;
      $total_pool_cycles_max =~ s/var/varmax/g;
      $utilized_pool_cyc_max =~ s/var/varmax/g;

      my $result_to_parse = $result_cmd_max;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      my $result_cmd_max_new;

      foreach my $cmd_line (@result_cmd_lines) {
        chomp $cmd_line;

        #print "$cmd_line\n";
        $cmd_line =~ s/^\s+//g;
        $cmd_line =~ s/\s+$//g;

        my $line_max = $cmd_line;

        if ( $cmd_line =~ "^DEF:" ) {
          $line_max =~ s/^DEF:var/DEF:varmax/;
          $line_max =~ s/:AVERAGE$/:MAX/;
        }
        if ( $cmd_line =~ "^CDEF:" ) {
          $line_max =~ s/var/varmax/g;
        }

        $result_cmd_max_new .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)

      $cmd .= $result_cmd;
      $cmd .= $result_cmd_max_new;
      $cmd .= '
        "CDEF:max_max=$max_pool_units_max"
        "CDEF:res_max=$res_pool_units_max"
        "CDEF:totcyc_max=$total_pool_cycles_max"
        "CDEF:uticyc_max=$utilized_pool_cyc_max"
        "CDEF:max1_max=max_max,res_max,-"
        "CDEF:cpuutil_max=uticyc_max,totcyc_max,GT,UNKN,uticyc_max,totcyc_max,/,IF"
        "CDEF:cpuutiltot_max=cpuutil_max,max_max,*"
        "CDEF:utilisa_max=cpuutil_max,100,*"
        "CDEF:max=$max_pool_units"
        "CDEF:res=$res_pool_units"
        "CDEF:totcyc=$total_pool_cycles"
        "CDEF:uticyc=$utilized_pool_cyc"
        "CDEF:max1=max,res,-"
        "CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "$t2"
        "AREA:res#00FF00: Reserved CPU cores        "
        "GPRINT:res:AVERAGE: %2.1lf"
        "$t2"
        "STACK:max1#FFFF00: Max CPU cores             "
        "GPRINT:max:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores  "
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa_max:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"';
      if ( $graph_cmd =~ m/graphv$/ ) {
        $cmd .= ' "LINE1:cpuutiltot_max#0088FF: Maximum peaks in CPU cores:dashes=1,2"';
      }
      else {
        # RRDTool 1.2.x does not support dashed lines
        $cmd .= ' "LINE1:cpuutiltot_max#0088FF: Maximum peaks in CPU cores"';
      }
      $cmd .= '
        "GPRINT:cpuutiltot_max:MAX: %3.2lf"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
      );';    #      "GPRINT:max:AVERAGE: %2.1lf"  # must be max , not max1, fixed in 4.54

    } ## end if ( $item eq "shpool-max")
    else {
      $cmd .= $result_cmd;
      $cmd .= '
        "CDEF:max=$max_pool_units"
        "CDEF:res=$res_pool_units"
        "CDEF:totcyc=$total_pool_cycles"
        "CDEF:uticyc=$utilized_pool_cyc"
        "CDEF:max1=max,res,-"
        "CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "$t2"
        "AREA:res#00FF00: Reserved CPU cores      "
        "GPRINT:res:AVERAGE: %2.1lf"
        "$t2"
        "STACK:max1#FFFF00: Max CPU cores           "
        "GPRINT:max:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
      );';    #      "GPRINT:max:AVERAGE: %2.1lf"  # must be max , not max1, fixed in 4.54
    } ## end else [ if ( $item eq "shpool-max")]
  } ## end if ( $lpar =~ "SharedPool[1-9]"...)
  else {
    #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
    $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "--step=$step_new"
      "--lower-limit=0.00"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--alt-autoscale-max"
      "--upper-limit=0.1"
      "$vertical_label"
      "--units-exponent=1.00"
      "--alt-y-grid"
      "$xgrid"
      "$no_legend"
      "$disable_rrdtool_tag"
      "$font_def"
      "$font_tit"
    ';

    my $file_pth = "$wrkdir/$server/*/$lpar.rr$type_sam";

    ( $result_cmd, $max_pool_units, $total_pool_cycles, $utilized_pool_cyc ) = LPM_easy( $file_pth, "0", "0", "max_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

    if ( $item eq "shpool-max" ) {
      my $file_pth_max = "$wrkdir/$server/*/$lpar.xr$type_sam";

      ( $result_cmd_max, $max_pool_units_max, $total_pool_cycles_max, $utilized_pool_cyc_max ) = LPM_easy( $file_pth_max, "0", "0", "max_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

      # prepare DEF|CDEF for max
      $max_pool_units_max =~ s/var/varmax/g;
      $total_pool_cycles_max =~ s/var/varmax/g;
      $utilized_pool_cyc_max =~ s/var/varmax/g;

      my $result_to_parse = $result_cmd_max;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      my $result_cmd_max_new;

      foreach my $cmd_line (@result_cmd_lines) {
        chomp $cmd_line;

        #print "$cmd_line\n";
        $cmd_line =~ s/^\s+//g;
        $cmd_line =~ s/\s+$//g;

        my $line_max = $cmd_line;

        if ( $cmd_line =~ "^DEF:" ) {
          $line_max =~ s/^DEF:var/DEF:varmax/;
          $line_max =~ s/:AVERAGE$/:MAX/;
        }
        if ( $cmd_line =~ "^CDEF:" ) {
          $line_max =~ s/var/varmax/g;
        }

        $result_cmd_max_new .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)

      $cmd .= $result_cmd;
      $cmd .= $result_cmd_max_new;
      $cmd .= '
        "CDEF:max_max=$max_pool_units_max"
        "CDEF:totcyc_max=$total_pool_cycles_max"
        "CDEF:uticyc_max=$utilized_pool_cyc_max"
        "CDEF:cpuutil_max=uticyc_max,totcyc_max,GT,UNKN,uticyc_max,totcyc_max,/,IF"
        "CDEF:cpuutiltot_max=cpuutil_max,max_max,*"
        "CDEF:utilisa_max=cpuutil_max,100,*"
        "CDEF:max=$max_pool_units"
        "CDEF:totcyc=$total_pool_cycles"
        "CDEF:uticyc=$utilized_pool_cyc"
        "CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "$t2"
        "AREA:max#00FF00: Max CPU cores             "
        "GPRINT:max:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores  "
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"';
      if ( $graph_cmd =~ m/graphv$/ ) {
        $cmd .= ' "LINE1:cpuutiltot_max#0088FF: Maximum peaks in CPU cores:dashes=1,2"';
      }
      else {
        # RRDTool 1.2.x does not support dashed lines
        $cmd .= ' "LINE1:cpuutiltot_max#0088FF: Maximum peaks in CPU cores"';
      }
      $cmd .= '
        "GPRINT:cpuutiltot_max:MAX: %3.2lf"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
      );';
    } ## end if ( $item eq "shpool-max")
    else {
      $cmd .= $result_cmd;
      $cmd .= '
        "CDEF:max=$max_pool_units"
        "CDEF:totcyc=$total_pool_cycles"
        "CDEF:uticyc=$utilized_pool_cyc"
        "CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF"
        "CDEF:cpuutiltot=cpuutil,max,*"
        "CDEF:utilisa=cpuutil,100,*"
        "COMMENT:   Average   \\n"
        "$t2"
        "AREA:max#00FF00: Max CPU cores           "
        "GPRINT:max:AVERAGE: %2.1lf"
        "$t2"
        "LINE1:cpuutiltot#FF0000: Utilization in CPU cores"
        "GPRINT:cpuutiltot:AVERAGE: %2.2lf"
        "COMMENT:("
        "GPRINT:utilisa:AVERAGE: %2.1lf"
        "COMMENT:\%)"
        "$t2"
        "$last"
        "$t2"
        "$t"
        "HRULE:0#000000"
      );';
    } ## end else [ if ( $item eq "shpool-max")]
  } ## end else [ if ( $lpar =~ "SharedPool[1-9]"...)]
  $cmd =~ s/\\"/"/g;
  eval($cmd);
  if ($@) {
    error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
  }
  else {
    my $answ = RRDp::read;
    if ( $$answ =~ "ERROR" ) {
      error( "$host:$server Graph rrdtool error : $$answ " . __FILE__ . ":" . __LINE__ );
    }
    $png_end_heading = ret_graph_param($answ);
  } ## end else [ if ($@) ]
  return 0;
} ## end sub graph_shpool

sub graph_osmem_agg {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $item       = shift;
  my $amax       = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $ds_name1   = "page_in";
  my $ds_name2   = "page_out";
  my $headline1  = "COMMENT:             [MB/s]        READ   Avg    Max   WRITE  Avg    Max";
  if ( $detail == 1 ) {
    $headline1 = "COMMENT:             [MB/s]        READ   Avg    Max   WRITE  Avg    Max                  [MB/s]        READ   Avg    Max   WRITE  Avg    Max";
  }

  my $vertical_label = "--vertical-label=READ  Paging MB/s  WRITE";
  my $t              = "COMMENT: ";
  my $t2             = "COMMENT:\\n";
  my $step_new       = $step;
  my $last           = "COMMENT: ";
  my $act_time       = localtime();
  my $act_time_u     = time();
  my $req_time       = 0;
  my $line_items     = 0;                                             # how many items in the legend per a line (default 2, when detail then 3)

  if ( $detail == 1 ) {
    $line_items = 2;
  }

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_osmem_agg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "Paging activity aggregated :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "Paging activity aggregated: $server :$last_txt$text";
  }

  if ( $detail == 2 ) {
    # $header = "Paging: $server :$last_txt$text";
    $header = "Paging agg:$last_txt$text";
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "Paging activity aggregated: $server : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my $i   = 0;
  my $cmd = "";

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }    #"--units-exponent=1.00"

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  if ( $detail < 1 && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_time"
        "--end" "$end_time"
        "--imgformat" "PNG"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "--step=$step"
        "--lower-limit=0.00"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "$vertical_label"
        "--units-exponent=1.00"
        "--alt-y-grid"
        "$xgrid"
        "$no_legend"
        "$disable_rrdtool_tag"
        "--base=1024"
        "$font_def"
        "$font_tit"
        "$headline1"
        "$t2"
';
  my $color_indx = -1;
  my $line_indx  = 0;    # place enter every 3rd line

  my $server_space = $server;
  if ( $server =~ m/ / ) {    # workaround for server name with a space inside, nothing else works, grrr
    $server_space = "\"" . $server . "\"";
  }
  my $wrkdir_space = $wrkdir;
  if ( $wrkdir =~ m/ / ) {    # workaround for server name with a space inside, nothing else works, grrr
    $wrkdir_space = "\"" . $wrkdir . "\"";
  }
  my $host_space = $host;
  if ( $host =~ m/ / ) {      # workaround for server name with a space inside, nothing else works, grrr
    $host_space = "\"" . $host . "\"";
  }
  # print STDERR "7127 $wrkdir_space/$server_space/$host_space\n";
  $server_space = urldecode($server_space);
  $host_space = urldecode($host_space);
  # print STDERR "7137 $wrkdir_space/$server_space/$host_space\n";

  my @files_unsorted = (<$wrkdir_space/$server_space/$host_space/*/pgs\.mmm>);

  #  my @files_unsorted = grep(/pgs\.mmm$/,readdir(DIR));
  my @files = sort { lc $a cmp lc $b } @files_unsorted;

  if ( $type_sam =~ "d" ) {
    $req_time = $act_time_u - 31536000;
  }

  my $filter = $filter_max_paging;

  # values in rrdtool are in pages == 4096B
  if ( $type =~ m/y/ ) {

    # lower limits for yearly graphs as they are averaged ....
    $filter = $filter / 10;
  }
  if ( $type =~ m/m/ ) {

    # lower limits for monthly graphs as they are averaged ....
    $filter = $filter / 2;
  }

  my $alimit = 10;

  # print STDERR  "7154 amax:$amax, alimit:$alimit, detail:$detail\n";
  if ( $amax !~ m/none=/ ) {
    #( undef, $alimit ) = split( /=/, $amax );
    $alimit = $amax;
    chomp($alimit);
    if ( !isdigit($alimit) ) { $alimit = 10; }
    if ( $alimit <= 0 )      { $alimit = 0; }
    if ( $detail == 2 )      { $alimit = 10; }    # for dashboard only default
  } ## end if ( $amax !~ m/none=/)
  $alimit = $alimit * 1024;                       # to bytes ...

  my $last_dir = "";
  my ( @recb, @trab, @rname, @rpath, @avgx, @avgy, @lpnames );
  my $lpar_space                     = "";
  my @lpar_space_proc                = "";
  my @wrkdir_server_host_file_legend = "";
  my $i_lpar                         = 0;

  foreach my $file (@files) {
    chomp($file);

    if ( $file =~ m/--NMON--/ ) {
      next;    # skip nmon files
    }

    #`echo "2913: detail-graph-cgi - $file" >> /tmp/e099`;

    # avoid old files - lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$file") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    # get MAX value from RRD
    my $answer;

    #      $file =~ s/:/\\:/g;

    my $file_space = $file;
    $file_space =~ s/:/\\:/g;

    # print STDERR "testing pagingagg files \$file $file\n";

    RRDp::cmd qq(graph "$name_out"
      "--start" "now-1$type"
      "--end" "now-1$type+1$type"
      "--step=$step"
      "DEF:item_nf=$file_space:$ds_name1:AVERAGE"
      "DEF:item2_nf=$file_space:$ds_name2:AVERAGE"
      "CDEF:item_nf_b=item_nf,4096,*"
      "CDEF:item2_nf_b=item2_nf,4096,*"
      "CDEF:item=item_nf_b,$filter,GT,UNKN,item_nf_b,IF"
      "CDEF:item2=item2_nf_b,$filter,GT,UNKN,item2_nf_b,IF"
      "PRINT:item:MAX: %10.0lf"
      "PRINT:item2:MAX: %10.0lf"
  );

    $answer = RRDp::read;
    if ( $$answer =~ m/NaN/ || $$answer =~ m/nan/ ) {

      # error ( "Graph rrdtool error instead of MAX returns : $$answer in osmem_agg detail-graph-cgi");
      next;    #return 0;
    }
    chomp($$answer);
    my $addr   = "";
    my $maxin  = "";
    my $maxout = "";
    ( $addr, $maxin, $maxout ) = split( / +/, $$answer );
    chomp($maxout);
    chomp($maxin);

    # `echo  "01 $file - amax:$amax, alimit:$alimit, maxin:$maxin, maxout:$maxout" >> /tmp/e099`;
    # when it is called from DB then it contains none=1402990627
    if ( !isdigit($maxin) || !isdigit($maxout) ) {
      next;    # no data
    }
    if ( $maxin < $alimit && $maxout < $alimit ) {
      next;
    }

    # print STDERR "7235 $file - amax:$amax, alimit:$alimit, maxin:$maxin, maxout:$maxout\n";
    #lpar name is last but one
    my @n_parts = split( /\//, $file );
    my $len = @n_parts;
    $lpar = $n_parts[ $len - 2 ];

    $lpar_space = $lpar;
    $lpar_space_proc[$i_lpar] = $lpar;
    $lpar_space =~ s/\&\&1/\//g;

    # add spaces to lpar name to have 25 chars total (for formatting graph legend)
    # to keep same count of characters
    $lpar_space =~ s/\\:/:/g;
    $lpar_space = sprintf( "%-25s", $lpar_space );
    $lpar_space =~ s/:/\\:/g;
    $lpar_space_proc[$i_lpar] =~ s/:/\\:/g;
    $lpar_space_proc[$i_lpar] =~ s/%/%%/g;    # anti '%'

    $wrkdir_server_host_file_legend[$i_lpar] = "$file";
    $wrkdir_server_host_file_legend[$i_lpar] =~ s/%/%%/g;
    $wrkdir_server_host_file_legend[$i_lpar] =~ s/:/\\:/g;

    $rpath[$i] = "$file";
    $rpath[$i] =~ s/:/\\:/g;
    $lpnames[$i] = "$lpar_space";
    $recb[$i]    = "rcb${i}";
    $trab[$i]    = "trb${i}";
    $avgx[$i]    = "avg${i}";
    $avgy[$i]    = "avgy${i}";

    # it is stored in pages 1 page == 4096 bytes
    $cmd .= '"DEF:$recb[$i]=$rpath[$i]:$ds_name1:AVERAGE"
           "DEF:$trab[$i]=$rpath[$i]:$ds_name2:AVERAGE"
           "CDEF:$recb[$i]_b=$recb[$i],4096,*"
           "CDEF:$trab[$i]_b=$trab[$i],4096,*"
           "CDEF:$recb[$i]_b_nf=$recb[$i]_b,$filter,GT,0,$recb[$i]_b,IF"
           "CDEF:$trab[$i]_b_nf=$trab[$i]_b,$filter,GT,0,$trab[$i]_b,IF"
           "CDEF:$recb[$i]_mb=$recb[$i]_b_nf,1048576,/"
           "CDEF:$trab[$i]_mb=$trab[$i]_b_nf,1048576,/"
           "CDEF:$recb[$i]_mbn=$recb[$i]_mb,UN,0,$recb[$i]_mb,IF"
           "CDEF:$recb[$i]_mbnm=0,$recb[$i]_mbn,-"
           "CDEF:$trab[$i]_mbn=$trab[$i]_mb,UN,0,$trab[$i]_mb,IF"
          ';
    $cmd .= '"COMMENT:$lpnames[$i]"
                   ';

    $cmd .= '"LINE1:$recb[$i]_mbnm$color[++$color_indx % ($color_max +1)]: "
                "GPRINT:$recb[$i]_mb:AVERAGE:%5.1lf"
            ';
    $cmd .= '"PRINT:$recb[$i]_mb:AVERAGE:%5.1lf $delimiter $item $delimiter $lpar_space_proc[$i_lpar] $delimiter $color[$color_indx % ($color_max +1)] $delimiter $wrkdir_server_host_file_legend[$i_lpar++]"
            ';
    $cmd .= '"GPRINT:$recb[$i]_mb:MAX:%5.1lf"
                "PRINT:$recb[$i]_mb:MAX:%5.1lf $delimiter"
                "LINE1:$trab[$i]_mbn$color[++$color_indx % ($color_max +1)]: "
                "GPRINT:$trab[$i]_mb:AVERAGE:%5.1lf"
                "PRINT:$trab[$i]_mb:AVERAGE:%5.1lf $delimiter $color[$color_indx % ($color_max +1)]"
                "PRINT:$trab[$i]_mb:MAX:%5.1lf $delimiter"
                "GPRINT:$trab[$i++]_mb:MAX:%5.1lf"
            ';

    if ( $detail == 1 ) {
      if ( 1 == $i % 2 ) {
        $cmd .= '  "$t2"
                  ';    # There is remainder
      }
    }    #do you understand difference $i++ above and below ??
    else {
      $cmd .= '  "$t2"
                ';
    }

    # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
    $i++;
    $i_lpar++;
    if ( $line_indx == $line_items ) {

      # put carriage return after each second lpar in the legend
      #      $cmd .= " COMMENT:\\\"\\l\\\"";
      #       $cmd .= '" COMMENT: \\n"
      #        ';
      $line_indx = 0;
    } ## end if ( $line_indx == $line_items)
    else {
      $line_indx++;
    }
  } ## end foreach my $file (@files)
  $cmd .= '  "$t2" ';
  $cmd .= '"HRULE:0#000000"
 );';

  #          "VRULE:0#000000"

  #  $cmd =~ s/\\"/"/g;

  #if you want to see what is generated
  # my $sss = "/var/tmp/sss.out";
  # open (FF, ">$sss");
  # print FF "$cmd\n";
  # close (FF);

  # if you want to see interpolated $cmd
  # care what file name is in sub alfa
  #  $i_lpar = 0;
  #  $i = 0;
  #  $color_indx = -1;
  #  my $t_cmd = $cmd;
  #  $t_cmd =~ s/RRDp::cmd qq\(\$graph_cmd /alfa\(/;
  #  $t_cmd =~ s/\s\"/\n\.\"/g;
  #  eval($t_cmd);

  $cmd =~ s/\\"/"/g;
  $i_lpar     = 0;
  $i          = 0;
  $color_indx = -1;
  eval($cmd);
  if ($@) {
    error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
  }
  else {
    my $answ = RRDp::read;
    if ( $$answ =~ "ERROR" ) {
      error( "$host:$server Graph rrdtool error : $$answ " . __FILE__ . ":" . __LINE__ );
    }
    $png_end_heading = ret_graph_param($answ);
  } ## end else [ if ($@) ]
  return 0;

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
} ## end sub graph_osmem_agg

sub graph_lparagg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \" ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $line_items  = 0;                  # how many items in the legend per a line (default 2, when detail then 3)

  if ( $detail == 1 ) {
    $line_items = 2;
  }
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lparagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $head = "LPARs";
  $head = "VMs"      if $vmware;
  $head = "CLUSTERs" if ( $item eq "multicluster" );
  $head = "WLM_CPU"  if ( $item eq "wlm-cpu");
  $head = "WLM_MEM"  if ( $item eq "wlm-mem");
  $head = "WLM_DKIO" if ( $item eq "wlm-dkio");
  my $header = "$head aggregated :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "$head aggregated: $server :$last_txt$text";
    $header = "$head aggregated: $lpar :$last_txt$text" if ( $item =~ "wlm");
    $header = "$head aggregated: $last_txt$text" if ( $item eq "multicluster" );
    $header = "$head aggregated: $host :$last_txt$text" if ( $hitachi );
  }

  if ( $detail == 2 ) {
    # $header = "$server :$last_txt$text";
    $header = "LPARs aggregated :$last_txt$text";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  # my $start_time = "now-1$type";
  # my $end_time = "now-1$type+1$type";

  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  # print STDERR "$start_time $end_time \n";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $req_time   = $start_time;    # set here too
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "";             # just a trick
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$head aggregated: $server : $start_human - $end_human";
      $header = "$head aggregated: $lpar : $start_human - $end_human" if ( $item =~ "wlm");
      if ( $item eq "multicluster" ) {
        $header = "$head aggregated: $start_human - $end_human";
      }
      elsif ( $hitachi ) {
        $header = "$head aggregated: $host : $start_human - $end_human";
      }
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my @cluster_arr;
  my @files = "";
  my %wlm_colors;

  if ( $item eq "multicluster" ) {

    # print STDERR "multicluster ------- $wrkdir/$server/$host\n";
    # find if there are more clusters
    my $file_cluster = "$wrkdir/$server/*/cluster.rrc";

    # $file_cluster =~ s/ /\\ /g;
    my $no_name = "";
    @cluster_arr = <$file_cluster$no_name>;
    @files       = @cluster_arr;
    my $cluster_count = scalar @cluster_arr;

    # print STDERR "multicluster ....... \$cluster_count $cluster_count \@cluster_arr @cluster_arr\n";
  } ## end if ( $item eq "multicluster")
  elsif ( $vmware && ( $item ne "multicluster" ) ) {    # server ESXi

    my $hosting_file = "$wrkdir/$server/$host/VM_hosting.vmh";
    my @vm_hosting   = ();
    if ( -f "$hosting_file" ) {
      open( FHC, "< $hosting_file" ) || error( "cannot open file : $hosting_file " . __FILE__ . ":" . __LINE__ ) && return 0;
      @vm_hosting = <FHC>;
      close(FHC);
    }

    #prepare VM file names
    @files = @vm_hosting;

    #  print STDERR "6432 $wrkdir/$server/$host/VM_hosting.vmh \@files @files\n";
  } ## end elsif ( $vmware && ( $item...))
  elsif ( $hyperv && ( $item ne "multicluster" ) ) {    # server hyperv

    my $hosting_file = "$wrkdir/$server/$host/VM_hosting.vmh";
    my @vm_hosting   = ();
    if ( -f "$hosting_file" ) {
      open( FHC, "< $hosting_file" ) || error( "cannot open file : $hosting_file " . __FILE__ . ":" . __LINE__ ) && return 0;
      @vm_hosting = <FHC>;
      close(FHC);
    }

    #prepare VM file names
    @files = @vm_hosting;
    # print STDERR "6773 $wrkdir/$server/$host/VM_hosting.vmh \@files @files\n";
  } ## end elsif ( $hyperv && ( $item...))
  elsif ( $hitachi ) {
    opendir( DIR, "$wrkdir/$server/$host" ) || error( "can't opendir $wrkdir/$server/$host: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_unsorted = grep( /\.hlm$/, readdir(DIR) );
    @files = sort { lc $a cmp lc $b } @files_unsorted;
    closedir(DIR);
  }
  elsif ( $item =~ "wlm-") {
    opendir( DIR, "$wrkdir/$server/$host/$lpar/" ) || error( " directory does not exists : $wrkdir/$server/$host/$lpar/ " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_unsorted = grep( /^wlm-.*\.mmm$/, readdir(DIR) );
    my @files_temp = sort { lc $a cmp lc $b } @files_unsorted;
    closedir(DIR);
    my $file_count = @files;
    $file_count--; #last one is "\n"
    foreach my $wlm_file ( @files_temp ) {
      if ( $wlm_file =~ "TOTAL") {
        next;
      }
      push @files, $wlm_file;
    }
    my $temp = @files;
    my $color_file = "$wrkdir/$server/$host/$lpar/wlm.col";
    if ( -f $color_file ) {
      open ( WLM, "< $color_file") || error( "cannot open file : $color_file $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      my  @wlm_col = <WLM>;
      my $wlm_file_count = @wlm_col;
      for ( my $k = 0; $k < $wlm_file_count; $k++) {
        chomp($wlm_col[$k]);
        my $file_check = 0;
        $wlm_col[$k] =~ s/:[0-9]*//g;
        foreach my $file_tmp ( @files ) { #get rid of TOTAL and classes with subclasses (it will save few colors)
          if ( $file_tmp eq $wlm_col[$k] ) { $file_check = 1; last}
          else { $file_check = 0; }
        }
        if ( $file_check == 0) { next; }
        $wlm_colors{$wlm_col[$k]}=$k%53;
      }
      close(WLM);
    }
    #print STDERR "AFTER 7405 detail-graph-cgi.pl \@files @files\n";
  } ## end elsif ( $item =~ "wlm-")
  else {
    opendir( DIR, "$wrkdir/$server/$host" ) || error( "can't opendir $wrkdir/$server/$host: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
    if ( $type_sam =~ "m" ) {
      my @files_unsorted = grep( /\.rrm$/, readdir(DIR) );
      for (@files_unsorted) {s/\.rrm$/    /}
      @files = sort { lc $a cmp lc $b } @files_unsorted;
      for (@files) {s/    $/\.rrm/}
    } ## end if ( $type_sam =~ "m" )
    if ( $type_sam =~ "h" ) {
      my @files_unsorted = grep( /\.rrh$/, readdir(DIR) );
      @files = sort { lc $a cmp lc $b } @files_unsorted;
    }
    if ( $type_sam =~ "d" ) {
      my @files_unsorted = grep( /\.rrd$/, readdir(DIR) );
      @files = sort { lc $a cmp lc $b } @files_unsorted;
      $req_time = $act_time_u - 31536000;
    }
    closedir(DIR);
  } ## end else [ if ( $item eq "multicluster")]

  my $file = "";
  my $i    = 0;

  #  my $lpar = "";
  my $aix_ser = $lpar;
  my $cmd = "";
  my $j   = 0;

  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=\"CPU cores\"";

  if ($vmware) {
    $vertical_label = "--vertical-label=\"CPU GHz\"";
  }
  elsif ($item =~ "wlm-dkio" || $item =~ "wlm-cpu") {
      $vertical_label = "--vertical-label=\"%\"";
  }
  elsif ($item =~ "wlm-mem" ) {
      $vertical_label = "--vertical-label=\"GB\"";
  }

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  my $units = "cores";
  $units = "GHz  " if $vmware;

  if ( $detail == 1 ) {
    if ($item =~ "wlm-") {
      $cmd .= " COMMENT:\"Class                      average  maximum   Class                      average  maximum   Class                      average  maximum\\l\"";
    }
    else {
      $cmd .= " COMMENT:\"Utilization in CPU $units\\:    average  maximum                               average  maximum                               average  maximum\\l\"";
    }
  }
  else {
    $cmd .= " COMMENT:\"Utilization in CPU $units\\:    average  maximum\\l\"";
  }

  # open a file with stored colours
  my @color_save = ();
  if ( -f "$wrkdir/$server/$host/lpars.col" ) {
    open( FHC, "< $wrkdir/$server/$host/lpars.col" ) || error( "cannot open file : $wrkdir/$server/$host/lpars.col " . __FILE__ . ":" . __LINE__ ) && return 0;
    @color_save = <FHC>;
    close(FHC);
  }

  # print STDERR "8147 \@color_save @color_save\n";
  # print STDERR "rq_time: $req_time \n";
  my $lpar_color_index = -1;
  my $color_WLM_counter= 0;
  my $gtype            = "AREA";
  my $color_indx       = 0;
  my $line_indx        = 0;        # place enter every 3rd line
  my $files_index      = -1;       # for vmware it is color index
  foreach $file (@files) {
    chomp($file);
    my $file_orig = $file;         # keep for vmware
                                   #    $files_index = $files_index++ % $color_max; # not possible !!!
    $files_index++;
    $files_index = $files_index % $color_max;

    if ($vmware) {
      $lpar_color_index++;
      if ( $item eq "multicluster" ) {
        my @arr_path = split( "\/", $file );
        $file   = $arr_path[-1];
        $host   = $arr_path[-2];
        $server = $arr_path[-3];

        # avoid old lpars which do not exist in the period
        my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
        next if ( $rrd_upd_time < $req_time );
      } ## end if ( $item eq "multicluster")
      else {
        $file =~ s/:.*/\.rrm/;
        next if !-f "$wrkdir/vmware_VMs/$file";

        # avoid old lpars which do not exist in the period
        my $rrd_upd_time = ( stat("$wrkdir/vmware_VMs/$file") )[9];
        next if ( $rrd_upd_time < $req_time );
      } ## end else [ if ( $item eq "multicluster")]
    } ## end if ($vmware)
    elsif ($hyperv) {
      $lpar_color_index++;
      if ( $item eq "multicluster" ) {
        my @arr_path = split( "\/", $file );
        $file   = $arr_path[-1];
        $host   = $arr_path[-2];
        $server = $arr_path[-3];

        # avoid old lpars which do not exist in the period
        my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
        next if ( $rrd_upd_time < $req_time );
      } ## end if ( $item eq "multicluster")
      else {
        $file =~ s/:.*/\.rrm/;
        # print STDERR "7722 $wrkdir / $host / $server / $file\n";
        next if !-f "$wrkdir/$server/hyperv_VMs/$file";

        # avoid old lpars which do not exist in the period
        my $rrd_upd_time = ( stat("$wrkdir/$server/hyperv_VMs/$file") )[9];
        next if ( $rrd_upd_time < $req_time );
      } ## end else [ if ( $item eq "multicluster")]
    } ## end elsif ($hyperv)
    elsif ( $item =~ "wlm-cpu") {
      next if !-f "$wrkdir/$server/$host/$aix_ser/$file";
      my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$aix_ser/$file") )[9];
      next if ( $rrd_upd_time < $req_time );
      } ## end elsif ( $item =~ "wlm-")
    elsif ( $item =~ "wlm-mem") {
      next if !-f "$wrkdir/$server/$host/$aix_ser/$file";
      my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$aix_ser/$file") )[9];
      next if ( $rrd_upd_time < $req_time );
      } ## end elsif ( $item =~ "wlm-")
    elsif ( $item =~ "wlm-dkio") {
      next if !-f "$wrkdir/$server/$host/$aix_ser/$file";
      my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$aix_ser/$file") )[9];
      next if ( $rrd_upd_time < $req_time );
      } ## end elsif ( $item =~ "wlm-")
    else {
      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
    } ## end else [ if ($vmware) ]

    $lpar = $file;
    $lpar =~ s/.rrh//;
    $lpar =~ s/.rrm//;
    $lpar =~ s/.rrd//;
    $lpar =~ s/.mmm//;
    $lpar =~ s/.hlm//; # hitachi suffix
    $lpar =~ s/wlm-// if ( $item =~ "wlm-");

    my $lpar_space_proc = human_vmware_name($lpar,"",$server);
    if ($vmware) {

      #     $lpar_space_proc =~ s/\(.*\)//; # maybe get away uuid in brackets ?
      if ( $item eq "multicluster" ) {    # for legend
                                          # print STDERR "5491 $file_orig,$file,$host,$server,\n";
        my $dir = $file_orig;
        $dir =~ s/$file$//;
        opendir( my $dh, $dir ) || error( "can't opendir $dir: $! :" . __FILE__ . ":" . __LINE__ ) && next;
        my @namefile = grep {/^cluster_name_/} readdir($dh);
        if ( defined $namefile[0] && $namefile[0] ne "" ) {
          $lpar_space_proc = $namefile[0];
          $lpar_space_proc =~ s/cluster_name_//;
        }
        else {
          $lpar_space_proc = $host;
        }
        closedir($dh);
      } ## end if ( $item eq "multicluster")
    } ## end if ($vmware)
    my $lpar_space = $lpar_space_proc;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for ( my $k = length($lpar_space); $k < 25; $k++ ) {
      $lpar_space .= " ";
    }

    # max 25 chars
    $lpar_space = substr( $lpar_space, 0, 24 );

    # Exclude pools and  memory
    if ( $lpar =~ m/^pool$/ || $lpar =~ m/^mem$/ || $lpar =~ /^SharedPool[0-9]$/ || $lpar =~ m/^SharedPool[1-9][0-9]$/ || $lpar =~ m/^mem-pool$/ || $lpar =~ m/^cod$/ ) {
      next;
    }

    # Found out stored color index to keep same color for lpars across all graphs
    if ( !$vmware && !$hyperv && !$hitachi) {
      unless ( $item =~ "wlm-") {
        $lpar_color_index = -1;
        for (@color_save) {
          $lpar_color_index++;

          #  last if "$_" =~ /$lpar_space_proc/;
          last if ( index( "$_", $lpar_space_proc ) != -1 );
        } ## end for (@color_save)
        if ( $lpar_color_index != -1 ) {
          ( $lpar_color_index, undef ) = split( " : ", $color_save[$lpar_color_index] );
        }
      } ## end unless ( $item =~ "wlm-")
    } ## end if ( !$vmware & !$hyperv)

    #print "$wrkdir/$server/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%

    my $wrkdir_server_host_file = "$wrkdir/$server/$host/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $wrkdir_server_host_file;
    if ( $item eq "multicluster" ) {
      $wrkdir_server_host_file_legend = "$wrkdir/$server/$lpar_space_proc/$file";
      $wrkdir_server_host_file_legend =~ s/:/\\:/g;
    }
    $wrkdir_server_host_file_legend =~ s/%/%%/g;

    my $wrkdir_server_host_file_lpm = "$wrkdir/$server/*/$file";
    $wrkdir_server_host_file_lpm =~ s/:/\\:/g;

    if ( $vmware || $hyperv ) {
      if ( $item eq "multicluster" ) {
        $cmd .= " DEF:utiltot_mhz${i}=\"$wrkdir_server_host_file\":CPU_usage_MHz:AVERAGE";
        $cmd .= " CDEF:utiltot${i}=utiltot_mhz${i},1000,/";
      }
      else {
        $wrkdir_server_host_file = "$wrkdir/vmware_VMs/$file" if $vmware;
        $wrkdir_server_host_file = "$wrkdir/$server/hyperv_VMs/$file" if $hyperv;

        # analyse $file_orig
        # 502a854c-d67c-5015-8f48-f161d122eae0:start=1440417087:end=1440418087:start=144419087:end=1440420087:start=1440421087..."
        # for graphing -> there are $start_time and $end_time, both unix
        my @st_end = split( "start", $file_orig );
        if ( scalar @st_end == 2 && $st_end[1] !~ 'end' ) {    # one start, no end -> no problem
          $cmd .= " DEF:utiltot_mhzt${i}=\"$wrkdir_server_host_file\":CPU_usage:AVERAGE" if $vmware;
          if ($hyperv) {
            $cmd .= " DEF:utiltot_mhzt_perc${i}=\"$wrkdir_server_host_file\":PercentTotalRunTime:AVERAGE";
            $cmd .= " DEF:utiltot_mhzt_time${i}=\"$wrkdir_server_host_file\":Timestamp_PerfTime:AVERAGE";
            $cmd .= " DEF:utiltot_mhzt_freq${i}=\"$wrkdir_server_host_file\":Frequency_PerfTime:AVERAGE";
            $cmd .= " DEF:utiltot_mhzt_cpu${i}=\"$wrkdir_server_host_file\":vCPU:AVERAGE";
            $cmd .= " CDEF:utiltot_mhzt${i}=utiltot_mhzt_perc${i},utiltot_mhzt_time${i},/,utiltot_mhzt_freq${i},*,100000,/,100,/,utiltot_mhzt_cpu${i},*";
          } ## end if ($hyperv)
          my $t_start = substr( $st_end[1], 1, 11 );
          $t_start = $t_start * 1;
          $cmd .= " CDEF:utiltot_mhz${i}=TIME,$t_start,LT,UNKN,utiltot_mhzt${i},IF";
        } ## end if ( scalar @st_end ==...)
        else {                                                 # more than only one start -> needs analyse
          # print STDERR "7343 \$i $i more starts $file_orig\n @st_end\n";
          my $cmd_tmp = "";
          $cmd_tmp = " DEF:utiltot_m${i}=\"$wrkdir_server_host_file\":CPU_usage:AVERAGE" if $vmware;    #:start=$t_start:end=$t_end";
          if ($hyperv) {
            $cmd_tmp .= " DEF:utiltot_mhzt_perc${i}=\"$wrkdir_server_host_file\":PercentTotalRunTime:AVERAGE";    #:start=$t_start:end=$t_end";
            $cmd_tmp .= " DEF:utiltot_mhzt_time${i}=\"$wrkdir_server_host_file\":Timestamp_PerfTime:AVERAGE";
            $cmd_tmp .= " DEF:utiltot_mhzt_freq${i}=\"$wrkdir_server_host_file\":Frequency_PerfTime:AVERAGE";
            $cmd .= " DEF:utiltot_mhzt_cpu${i}=\"$wrkdir_server_host_file\":vCPU:AVERAGE";
            $cmd_tmp .= " CDEF:utiltot_m${i}=utiltot_mhzt_perc${i},utiltot_mhzt_time${i},/,utiltot_mhzt_freq${i},*,100000,/,100,/,utiltot_mhzt_cpu${i},*";
          }
          my $ik = 1000;
          # Multi graph rrdtool error : ERROR: trying to reuse vname utiltot_mh130 at /home/lpar2rrd/lpar2rrd/bin/detail-graph-cgi.pl line 7772
          # there was a problem when v-motion was more than hundred in the picture time - day or week, that is why starts not 0 but 1000
          $cmd_tmp .= " CDEF:utiltot_mh${i}${ik}=utiltot_m${i},UN,UNKN,UNKN,IF";
          my $in = 1001;

          # print STDERR "----- @st_end\n";
          shift @st_end; # remove VM name

          my ( $t_start, $t_end );
          foreach my $times (@st_end) {
            $times =~ s/:$//;
            ( undef, $t_start, $t_end ) = split( "=", $times );
            if ( !defined $t_end ) {
              $t_end = $end_time;
            }
            else {
              $t_end = substr( $t_end, 0, 10 );
            }
            $t_start = substr( $t_start, 0, 10 );
            $t_start = $t_start * 1;
            $t_end   = $t_end * 1;

            #print STDERR "\$start_time $start_time \$t_start $t_start \$end_time $end_time \$t_end $t_end\n";
            if ( $t_start < $start_time ) { $t_start = $start_time }
            if ( $t_end > $end_time )     { $t_end   = $end_time }
            if ( $t_start >= $t_end )     {next}

            #print STDERR "graphing $t_start $t_end\n";

            $cmd_tmp .= " CDEF:utiltot_mha${i}${in}=TIME,$t_start,GE,TIME,$t_end,LE,+,2,EQ,utiltot_m${i},UNKN,IF";
            $cmd_tmp .= " CDEF:utiltot_mh${i}${in}=utiltot_mh${i}${ik},UN,utiltot_mha${i}${in},utiltot_mh${i}${ik},IF";
            $in++;
            $ik++;
          } ## end foreach my $times (@st_end)

          if ( $t_end <= $start_time ) {    # out of start time of picture
            $i++;
            next;
          }
          $cmd .= "$cmd_tmp";
          $in--;
          $cmd .= " CDEF:utiltot_mhz${i}=utiltot_mh${i}${in},1,*";
        } ## end else [ if ( scalar @st_end ==...)]
        $cmd .= " CDEF:utiltot${i}=utiltot_mhz${i},1000,/" if $vmware;
        $cmd .= " CDEF:utiltot${i}=utiltot_mhz${i},1,*"    if $hyperv;
        $lpar_color_index = $files_index;
      } ## end else [ if ( $item eq "multicluster")]
    } ## end if ( $vmware || $hyperv)
    elsif ( $hitachi ) {
      $cmd .= " DEF:cores${i}=\"$wrkdir_server_host_file\":cores:AVERAGE";
      $cmd .= " CDEF:utiltot${i}=cores${i}";
    } ## end elsif ( $hitachi )
    elsif ( $item =~ "wlm-cpu") {
      $wrkdir_server_host_file_legend = "$wrkdir/$server/$host/$aix_ser/$file";
      $cmd .= " DEF:cpu_wlm${i}=\"$wrkdir_server_host_file_legend\":wlm_cpu:AVERAGE";
      $cmd .= " CDEF:utiltot${i}=cpu_wlm${i}";

      my $color_added = 0;
      my $tmp_val = 0;
      foreach my $hash_test ( keys %wlm_colors) {
        chomp($hash_test);
        my $tmp_a = $wlm_colors{$hash_test};
        if ( $tmp_a >= $tmp_val ) { $tmp_val = $tmp_a };
        $lpar_color_index = $tmp_val;
        if ($hash_test eq $file ) {
          $lpar_color_index = $wlm_colors{$hash_test};
          $color_added=1;
          $color_WLM_counter++;
          last;
        }
      }
      if ($color_added == "0"){
        $color_WLM_counter++;
        $lpar_color_index = $color_WLM_counter%53;
      }
    } ## end if ($item =~ "wlm-cpu")
    elsif ( $item =~ "wlm-mem") {

      $wrkdir_server_host_file_legend = "$wrkdir/$server/$host/$aix_ser/$file";
      $cmd .= " DEF:mem_wlm${i}=\"$wrkdir_server_host_file_legend\":wlm_mem:AVERAGE";
      $cmd .= " CDEF:mem_wlm_g${i}=mem_wlm${i},1024,/";
      $cmd .= " CDEF:utiltot${i}=mem_wlm_g${i}";

      my $color_added = 0;
      my $tmp_val = 0;
      foreach my $hash_test ( keys %wlm_colors) {
        chomp($hash_test);
        my $tmp_a = $wlm_colors{$hash_test};
        if ( $tmp_a >= $tmp_val ) { $tmp_val = $tmp_a };

        $lpar_color_index = $tmp_val;
        if ($hash_test eq $file ) {
          $lpar_color_index = $wlm_colors{$hash_test};
          $color_added=1;
          $color_WLM_counter++;
          last;
        }
      }
      if ($color_added == "0"){
        $color_WLM_counter++;
        $lpar_color_index = $color_WLM_counter%53;
      }
    } ## end elsif ( $item =~ "wlm-mem")
    elsif ( $item =~ "wlm-dkio") {

      $wrkdir_server_host_file_legend = "$wrkdir/$server/$host/$aix_ser/$file";
      $cmd .= " DEF:dkio_wlm${i}=\"$wrkdir_server_host_file_legend\":wlm_dkio:AVERAGE";
      $cmd .= " CDEF:utiltot${i}=dkio_wlm${i}";

      my $color_added = 0;
      my $tmp_val = 0;
      foreach my $hash_test ( keys %wlm_colors) {
        chomp($hash_test);
        my $tmp_a = $wlm_colors{$hash_test};
        if ( $tmp_a >= $tmp_val ) { $tmp_val = $tmp_a };

        $lpar_color_index = $tmp_val;
        if ($hash_test eq $file ) {
          $lpar_color_index = $wlm_colors{$hash_test};
          $color_added=1;
          $color_WLM_counter++;
          last;
        }
      }
      if ($color_added == "0"){
        $color_WLM_counter++;
        $lpar_color_index = $color_WLM_counter%53;
      }
    } ## end elseif ( $item =~ "wlm-dkio")
    else {

      # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
      my ( $result_cmd, $CPU_cap, $CPU_uncap, $CPU_ent, $CPU_cur ) = LPM_easy( "$wrkdir_server_host_file_lpm", $i, $req_time, "capped_cycles", "uncapped_cycles", "entitled_cycles", "curr_proc_units" );

      $cmd .= $result_cmd;
      $cmd .= " CDEF:cap_peak${i}=$CPU_cap";
      $cmd .= " CDEF:uncap${i}=$CPU_uncap";
      $cmd .= " CDEF:ent${i}=$CPU_ent";
      $cmd .= " CDEF:cur${i}=$CPU_cur";

      # bulid RRDTool cmd
      #$cmd .= " DEF:cap${i}=\"$wrkdir_server_host_file\":capped_cycles:AVERAGE";
      #$cmd .= " DEF:uncap${i}=\"$wrkdir_server_host_file\":uncapped_cycles:AVERAGE";
      #$cmd .= " DEF:ent${i}=\"$wrkdir_server_host_file\":entitled_cycles:AVERAGE";
      #$cmd .= " DEF:cur${i}=\"$wrkdir_server_host_file\":curr_proc_units:AVERAGE";

      # filtering peaks caused by LPM or changing entitled, if cap CPU util is > entitled --> UNKN
      # usualy cap counter is affected only
      # sometimes might happen that even in normal load capped util is little higher than entitled! Therefore using 1.2
      $cmd .= " CDEF:cap${i}=cap_peak${i},ent${i},/,1.2,GT,UNKN,cap_peak${i},IF";

      $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
      $cmd .= " CDEF:util${i}=tot${i},ent${i},/,$cpu_max_filter,GT,UNKN,tot${i},ent${i},/,IF";

      $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
      $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
    } ## end else [ if ( $vmware || $hyperv)]
    my $index_to_display = $color_indx;
    if ( $lpar_color_index > -1 ) {
      $index_to_display = $lpar_color_index;
    }

    $cmd .= " $gtype:utiltot${i}$color[$index_to_display]:\"$lpar_space\"";
    #print STDERR " \$gtype:utiltot${i}\$color[$index_to_display]:\"\$lpar_space\" $gtype:utiltot${i}$color[$index_to_display]:\"$lpar_space\"\n";
    #my $color_count = @color;

    $cmd .= " GPRINT:utiltot${i}:AVERAGE:\"%5.2lf \"";
    $cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \"";

    $cmd .= " PRINT:utiltot${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$index_to_display]\"";
    $cmd .= " PRINT:utiltot${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

    #$cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \\l\"";
    # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)

    $color_indx = ( $color_indx + 1 ) % ( $color_max + 1 );

    #    $col_indx++;
    $gtype = "STACK";
    $i++;
    if ( $line_indx == $line_items ) {

      # put carriage return after each second lpar in the legend
      $cmd .= " COMMENT:\"\\l\"";
      $line_indx = 0;
    } ## end if ( $line_indx == $line_items)
    else {
      $line_indx++;
    }
  } ## end foreach $file (@files)

  $cmd .= " COMMENT:\"\\l\""; # last legend line in detail not across whole line

  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # print STDERR "7464 detail-graph-cgi.pl lparagg \$cmd $cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_lparagg

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  if ( !defined $digit || $digit eq '' ) {
    return 0;
  }
  if ( $digit eq 'U' ) {
    return 1;
  }

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;
  $digit_work =~ s/^-//;
  $digit_work =~ s/e//;
  $digit_work =~ s/\+//;
  $digit_work =~ s/\-//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

sub graph_cod_used {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $step        = 3600;            # --PH dodelat --> no problem it could be here 60 fixed
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $time_range  = "hour";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $type =~ m/m/ ) {
    $step       = 86400;             # daily step
    $time_range = "day";
  }
  if ( $type =~ m/y/ ) {
    $step       = 86400;             # daily step
    $time_range = "day";
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_ams: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $rrd = "$wrkdir/$server/$host/$lpar.rr$type_sam";

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "Capacity on Demand - used:$last_txt$text";

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  #my $rrd_y = "";
  #if ( $type_sam =~ "y" ) {
  #  # u rocnich grafu nejdrive pouzit rrm data a pokud nejsou tak teprve rrd
  #  if ( $rrd !~ m/\.rrd$/ ) {
  #    $rrd_y = $rrd;
  #    $rrd_y =~ s/\.rrm$/\.rrd/;
  #    if ( -f $rrd_y ) {
  #     $rrd = $rrd_y;
  #    }
  #}

  #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=CoD in minutes per $time_range";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  $rrd =~ s/:/\\:/g;
  RRDp::cmd qq(graph "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "--step=$step"
      "--lower-limit=0.00"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--alt-autoscale-max"
      "--upper-limit=0.1"
      "$vertical_label"
      "--units-exponent=1.00"
      "--alt-y-grid"
      "$xgrid"
      "$no_legend"
      "$disable_rrdtool_tag"
      "$font_def"
      "$font_tit"
      "DEF:used=$rrd:used_proc_min:AVERAGE"
      "CDEF:used_null=used,UN,0,used,IF"
      "CDEF:used_s=used_null,$step,*"
      "AREA:used_s#00FF00: Used mins per $time_range  "
      "GPRINT:used_s:AVERAGE: %4.2lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
  );
  my $answer = RRDp::read;
  if ( $$answer =~ "ERROR" ) {
    error( "$host:$server Graph rrdtool error : $$answer " . __FILE__ . ":" . __LINE__ );
  }

  return 0;
} ## end sub graph_cod_used

sub graph_cod_unreport {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $head        = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST: %4.0lf";
    $head = "COMMENT:                            Average   Last\\n";
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_cod: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $rrd = "$wrkdir/$server/$host/$lpar.rr$type_sam";

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "Capacity on Demand - unreported:$last_txt$text";

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  #my $rrd_y = "";
  #if ( $type_sam =~ "y" ) {
  #  # u rocnich grafu nejdrive pouzit rrm data a pokud nejsou tak teprve rrd
  #  if ( $rrd !~ m/\.rrd$/ ) {
  #    $rrd_y = $rrd;
  #    $rrd_y =~ s/\.rrm$/\.rrd/;
  #    if ( -f $rrd_y ) {
  #     $rrd = $rrd_y;
  #    }
  #}

  #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=CoD in minutes";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  $rrd =~ s/:/\\:/g;

  #   "--lower-limit=0.00" --> it might be negative
  RRDp::cmd qq(graph "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "now-1$type+1$type"
      "--imgformat" "PNG"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "--step=$step"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "--alt-autoscale-max"
      "--upper-limit=0.1"
      "$vertical_label"
      "--units-exponent=1.00"
      "--alt-y-grid"
      "$xgrid"
      "$no_legend"
      "$disable_rrdtool_tag"
      "$font_def"
      "$font_tit"
      "$head"
      "DEF:unrep=$rrd:unreported_proc_min:AVERAGE"
      "LINE1:unrep#FF0000: Unreported minutes      "
      "GPRINT:unrep:AVERAGE: %4.0lf"
      "$t"
      "$t2"
      "HRULE:0#000000"
      "VRULE:0#000000"
  );
  my $answer = RRDp::read;
  if ( $$answer =~ "ERROR" ) {
    error( "$host:$server Graph rrdtool error : $$answer " . __FILE__ . ":" . __LINE__ );
  }

  return 0;
} ## end sub graph_cod_unreport

sub graph_osmem {
  my ( $host, $server, $lpar, $type, $name_out, $type_sam, $detail, $start_unix, $end_unix ) = @_;
  my $t          = "COMMENT:\" \"";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT:\" \"";
  my $head       = "COMMENT:\"                            Average   \\n\"";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST: %4.0lf";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";           # just a trick to place there something otherwise rrdtoll complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_osmem: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                           # some workaround, it should not go here
  }

  my $rrd = "$wrkdir/$server/$host/$lpar/mem.mmm";
  # print STDERR "9555 \$rrd $rrd\n";

  if ( $item eq "hmcmem" ) {
    $rrd  = "$wrkdir/--HMC--$host/mem.mmx";
    $lpar = "$host";
  }
  elsif ($hyperv) {
    # print STDERR "8314 $wrkdir / $host / $server / $lpar\n";
    if ( $item =~ 'hyp-mem' ) {    # VM disk
      $rrd = "$wrkdir/$server/$all_hyperv_VMs/$lpar.rrm";
    }
  }

  # print "001 MEM $rrd\n";

  my $lpar_slash = $lpar;
  if ($hyperv) {
    $lpar_slash = human_vmware_name($lpar,"",$server);
  }
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  my $last_txt      = " last ";
  my $header_prefix = "OS Memory:";

  if ( $detail == 2 ) {
    $last_txt      = "";
    $lpar_slash_nmon_head = "" if $server eq "Linux";
    $header_prefix = "MEM";
    if ( $wpar == 1 ) {
      $header_prefix = "MEM:";
    }
  } ## end if ( $detail == 2 )

  my $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700) {
        $header = "$start_human - $end_human";
      }
      if ( $detail == 10 ) {
        $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  my $availmem = "";
  if ( $type =~ "d" || $hyperv ) {

    # my $rrd_last = rrd_file_actual ("$rrd"); # no it is hard-linked through all HMCs
    my $rrd_last = $rrd;
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $type =~ "d" && $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          $last = "";

          #if ( $m_arr[2] > 0 ) {
          #  my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
          #  my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
          #  $last ="COMMENT:Last memory consuption in GB         $last_mem_g";
          #}
        } ## end if ( $m_arr[1] =~ /\d/...)
        $availmem =  $m_arr[6]; # AvailableMemory for hyperv VM
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  #print STDERR "graphing OSMEM   : $host:$server:$lpar_slash:$type_sam:$type\n";
  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $vertical_label = "--vertical-label=GB";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  my $cmd = "";
  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $xgrid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";

  my $form_dec = "%6.1lf";

  #print "graphing OSMEM   : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  if ( $wpar == 0 ) {
    if ( $item eq "hmcmem" ) {
      $rrd =~ s/:/\\:/g;

      $cmd .= " DEF:size=$rrd:size:AVERAGE";
      $cmd .= " DEF:used=$rrd:nuse:AVERAGE";
      $cmd .= " DEF:free=$rrd:free:AVERAGE";
      $cmd .= " DEF:in_use_work=$rrd:in_use_work:AVERAGE";
      $cmd .= " DEF:in_use_clnt=$rrd:in_use_clnt:AVERAGE";
      $cmd .= " DEF:pin=$rrd:pin:AVERAGE";
      $cmd .= " CDEF:free_g=free,1048576,/";
      $cmd .= " CDEF:usedg=used,1048576,/";
      $cmd .= " CDEF:in_use_clnt_g=in_use_clnt,1048576,/";
      $cmd .= " CDEF:used_realg=usedg,in_use_clnt_g,-";
      $cmd .= " CDEF:pin_g=pin,1048576,/";
      $cmd .= " COMMENT:\"   [GB]             AVRG      MAX  \\n\"";
      $cmd .= " AREA:usedg#FF4040:\" Used memory \"";
      $cmd .= " GPRINT:usedg:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:usedg:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      $cmd .= " STACK:pin_g#0080FF:\" FS cache    \"";
      $cmd .= " GPRINT:pin_g:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:pin_g:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      $cmd .= " STACK:free_g#00FF00:\" Free        \"";
      $cmd .= " GPRINT:free_g:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:free_g:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      $cmd .= " $t";
      $cmd .= " HRULE:0#000000";

      #     $cmd .= " VRULE:0#000000"
      # print STDERR "\$cmd $cmd\n";
    } ## end if ( $item eq "hmcmem")
    elsif ( $item eq 'hyp-mem' ) {
      $rrd =~ s/:/\\:/g;
      $cmd .= " DEF:size=$rrd:TotalPhysMemory:AVERAGE";
      $cmd .= " CDEF:sizeg=size,1024,/";
      $cmd .= " COMMENT:\"   [GB]             AVRG      MAX  \\n\"";
      if ($availmem < 100) {
        # print STDERR "7987 found lower than 100 %\n";
        $cmd .= " DEF:avmemproc=$rrd:MemoryAvailable:AVERAGE";
        $cmd .= " CDEF:sizeused=100,avmemproc,-,sizeg,*,100,/";
        $cmd .= " CDEF:sizefree=sizeg,sizeused,-";
        $cmd .= " AREA:sizeused#FF4040:\" Used memory \"";
        $cmd .= " GPRINT:sizeused:AVERAGE:\" $form_dec\"";
        $cmd .= " GPRINT:sizeused:MAX:\" $form_dec\"";
        $cmd .= " $t2";
        $cmd .= " STACK:sizefree#00FF00:\" Free memory \"";
        $cmd .= " GPRINT:sizefree:AVERAGE:\" $form_dec\"";
        $cmd .= " GPRINT:sizefree:MAX:\" $form_dec\"";
      }
      else {
        $cmd .= " AREA:sizeg#FF4040:\" Used memory \"";
        $cmd .= " GPRINT:sizeg:AVERAGE:\" $form_dec\"";
        $cmd .= " GPRINT:sizeg:MAX:\" $form_dec\"";
      }
      $cmd .= " $t2";
      $cmd .= " $t";
      $cmd .= " HRULE:0#000000";

    }
    else {    # OS MEM

      # for LPM
      my @rrd_files;
      my $lpm_suff = "rrl";
      my $cgi      = 0;
      $rrd_files[0] = $rrd;    #actual is the first

      lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "mem.mmm", \@rrd_files, $cgi );

      #print STDERR "lpm_find_files is returning @rrd_files\n";

      my $i = -1;
      my $j;

      foreach $rrd (@rrd_files) {    # LPM cycle
        chomp($rrd);
        $i++;
        $j = $i - 1;
        $rrd =~ s/:/\\:/g;

        $cmd .= " DEF:size${i}=\"$rrd\":size:AVERAGE";
        $cmd .= " DEF:used${i}=\"$rrd\":nuse:AVERAGE";
        $cmd .= " DEF:free${i}=\"$rrd\":free:AVERAGE";
        $cmd .= " DEF:in_use_work${i}=\"$rrd\":in_use_work:AVERAGE";
        $cmd .= " DEF:in_use_clnt${i}=\"$rrd\":in_use_clnt:AVERAGE";
        $cmd .= " DEF:pin${i}=\"$rrd\":pin:AVERAGE";

        if ( $i == 0 ) {
          $cmd .= " CDEF:sizer${i}=size${i}";
          $cmd .= " CDEF:usedr${i}=used${i}";
          $cmd .= " CDEF:freer${i}=free${i}";
          $cmd .= " CDEF:in_use_workr${i}=in_use_work${i}";
          $cmd .= " CDEF:in_use_clntr${i}=in_use_clnt${i}";
          $cmd .= " CDEF:pinr${i}=pin${i}";
          next;
        } ## end if ( $i == 0 )
        $cmd .= " CDEF:sizer${i}=sizer${j},UN,size${i},sizer${j},IF";
        $cmd .= " CDEF:usedr${i}=usedr${j},UN,used${i},usedr${j},IF";
        $cmd .= " CDEF:freer${i}=freer${j},UN,free${i},freer${j},IF";
        $cmd .= " CDEF:in_use_workr${i}=in_use_workr${j},UN,in_use_work${i},in_use_workr${j},IF";
        $cmd .= " CDEF:in_use_clntr${i}=in_use_clntr${j},UN,in_use_clnt${i},in_use_clntr${j},IF";
        $cmd .= " CDEF:pinr${i}=pinr${j},UN,pin${i},pinr${j},IF";
      } ## end foreach $rrd (@rrd_files)
      $cmd .= " CDEF:size=sizer${i}";
      $cmd .= " CDEF:used=usedr${i}";
      $cmd .= " CDEF:free=freer${i}";
      $cmd .= " CDEF:in_use_work=in_use_workr${i}";
      $cmd .= " CDEF:in_use_clnt=in_use_clntr${i}";
      $cmd .= " CDEF:pin=pinr${i}";

      $cmd .= " CDEF:free_g=free,1048576,/";
      $cmd .= " CDEF:usedg=used,1048576,/";
      $cmd .= " CDEF:in_use_clnt_g=in_use_clnt,1048576,/";
      $cmd .= " CDEF:used_realg=usedg,in_use_clnt_g,-";
      $cmd .= " CDEF:pin_g=pin,1048576,/";
      $cmd .= " COMMENT:\"   [GB]             AVRG      MAX  \\n\"";
      $cmd .= " AREA:used_realg#FF4040:\" Used memory \"";
      $cmd .= " GPRINT:used_realg:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:used_realg:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      $cmd .= " STACK:in_use_clnt_g#0080FF:\" FS cache    \"";
      $cmd .= " GPRINT:in_use_clnt_g:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:in_use_clnt_g:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      $cmd .= " STACK:free_g#00FF00:\" Free        \"";
      $cmd .= " GPRINT:free_g:AVERAGE:\" $form_dec\"";
      $cmd .= " GPRINT:free_g:MAX:\" $form_dec\"";
      $cmd .= " $t2";
      if ( !( $rrd =~ /Solaris/ || $rrd =~ /Linux/ ) ) {
        $cmd .= " LINE1:pin_g#000000:\" Pinned      \"";
        $cmd .= " GPRINT:pin_g:AVERAGE:\" $form_dec\"";
        $cmd .= " GPRINT:pin_g:MAX:\" $form_dec\"";
        $cmd .= " $t2";
      } ## end if ( !( $rrd =~ /Solaris/...))
      $cmd .= " $t";
      $cmd .= " HRULE:0#000000";

      #      "VRULE:0#000000"

      # below should be ideal but it does not work .....
      #"--legend-direction=bottomup"
      #
      # initially : in_use_work_g + in_use_clnt_g (cache) + free_g == size
      # problem on AIX 5.3 when large pages (16MB) are used, then in_use_work_g does not cover them
      # Solution : (used - in_use_clnt_g)  + in_use_clnt_g (cache) + free_g == size
      #
      #   "$last"
    } ## end else [ if ( $item eq "hmcmem")]
  } ## end if ( $wpar == 0 )
  else {
    # WPARs
    $rrd =~ s/:/\\:/g;
    $cmd .= " DEF:size=\"$rrd\":size:AVERAGE";
    $cmd .= " DEF:used=\"$rrd\":nuse:AVERAGE";
    $cmd .= " DEF:free=\"$rrd\":free:AVERAGE";
    $cmd .= " DEF:in_use_work=\"$rrd\":in_use_work:AVERAGE";
    $cmd .= " DEF:in_use_clnt=\"$rrd\":in_use_clnt:AVERAGE";
    $cmd .= " DEF:pin=\"$rrd\":pin:AVERAGE";
    $cmd .= " CDEF:free_g=free,1048576,/";
    $cmd .= " CDEF:usedg=used,1048576,/";
    $cmd .= " CDEF:in_use_clnt_g=in_use_clnt,1048576,/";
    $cmd .= " CDEF:used_realg=usedg,in_use_clnt_g,-";
    $cmd .= " CDEF:pin_g=pin,1048576,/";
    $cmd .= " COMMENT:\"   [GB]             AVRG      MAX  \\n\"";
    $cmd .= " AREA:used_realg#FF4040:\" Used memory \"";
    $cmd .= " GPRINT:used_realg:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:used_realg:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:in_use_clnt_g#0080FF:\" FS cache    \"";
    $cmd .= " GPRINT:in_use_clnt_g:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:in_use_clnt_g:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " LINE1:pin_g#000000:\" Pinned      \"";
    $cmd .= " GPRINT:pin_g:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:pin_g:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";

    #   "VRULE:0#000000"
  } ## end else [ if ( $wpar == 0 ) ]

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_osmem : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_osmem

sub graph_oscpu {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $item       = shift;                                                     # "slan" or "ssan1" or "ssan2" or "ssea"
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT:\" \"";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT:\" \"";
  my $head       = "COMMENT:\"                            Average   \\n\"";

  #$server =~ s/--unknown//;
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST:\" %4.0lf\"";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";           # just a trick to place there something otherwise rrdtool complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_cpuos: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                           # some workaround, it should not go here
  }
  my $rrd = "";
  if ($server =~ /Solaris$/){
    $rrd = "$wrkdir/Solaris--unknown/no_hmc/$lpar/cpu.mmm";
  }else{
    $rrd = "$wrkdir/$server/$host/$lpar/cpu.mmm";
  }

  if ( $item eq "hmccpu" ) {
    $rrd  = "$wrkdir/--HMC--$host/cpu.mmx";
    $lpar = "$host";
  }

  # print "001 CPUOS $rrd\n";
  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;          # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;       # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  my $last_txt      = " last ";
  my $header_prefix = "OS CPU:";
  if ( $detail == 2 ) {
    $lpar_slash_nmon_head = "" if $server eq "Linux";
    $last_txt      = "";
    $header_prefix = "OSCPU";
    if ( $wpar == 1 ) {
      $header_prefix = "";
    }
  } ## end if ( $detail == 2 )
  my $header = "$header_prefix $lpar_slash_nmon_head:$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700) {
        $header = "$start_human - $end_human";
      }
      if ( $detail == 10 ) {
        $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  if ( $type =~ "d" ) {

    # my $rrd_last = rrd_file_actual ("$rrd"); # no it is hard-linked through all HMCs
    my $rrd_last = $rrd;
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          $last = "";

          #if ( $m_arr[2] > 0 ) {
          #  my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
          #  my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
          #  $last ="COMMENT:Last memory consuption in GB         $last_mem_g";
          #}
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $vertical_label = "--vertical-label=%";
  $vertical_label = "--vertical-label=\"CPU cores\"" if $rrd =~ /wpar/;

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # common part
  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $xgrid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";

  my $form_dec = "%6.1lf";

  if ( $item eq "hmccpu" ) {

    #print "graphing CPUOS for hmc   : $host:$type_sam:$type\n" if $DEBUG ;
    $rrd =~ s/:/\\:/g;

    $cmd .= " --upper-limit=100";
    $cmd .= " --rigid";

    $cmd .= " DEF:cpui=$rrd:cpu_id:AVERAGE";
    $cmd .= " DEF:cpus=$rrd:cpu_sy:AVERAGE";
    $cmd .= " DEF:cpuu=$rrd:cpu_us:AVERAGE";
    $cmd .= " DEF:cpuw=$rrd:cpu_wa:AVERAGE";
    $cmd .= " CDEF:summ=cpus,cpuu,cpuw,cpui,+,+,+";
    $cmd .= " CDEF:cpusy=cpus,summ,/,100,*";
    $cmd .= " CDEF:cpuus=cpuu,summ,/,100,*";
    $cmd .= " CDEF:cpuwa=cpuw,summ,/,100,*";
    $cmd .= " CDEF:stog=100,cpusy,-,cpuus,-,cpuwa,-";
    $cmd .= " COMMENT:\"   [%]              AVRG      MAX  \\n\"";
    $cmd .= " AREA:cpusy#0080FF:\" Sys         \"";
    $cmd .= " GPRINT:cpusy:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpusy:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:cpuus#FFFF00:\" User        \"";
    $cmd .= " GPRINT:cpuus:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpuus:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:cpuwa#FF4040:\" IO wait     \"";
    $cmd .= " GPRINT:cpuwa:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpuwa:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:stog#00FF00:\" Idle        \"";
    $cmd .= " GPRINT:stog:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:stog:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";
    $cmd .= " VRULE:0#000000";

    $cmd =~ s/\\"/"/g;

    eval {
      RRDp::cmd qq($cmd);
      $ret = RRDp::read;
    };
    if ($@) {
      error( "$host : Graph rrdtool error : graph_oscpu : $@ " . __FILE__ . ":" . __LINE__ );
      return 1;
    }
    $png_end_heading = ret_graph_param($ret);
    return 0;
  } ## end if ( $item eq "hmccpu")

  # for LPM

  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;    #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "cpu.mmm", \@rrd_files, $cgi );

  #print STDERR "lpm_find_files is returning @rrd_files\n";

  my $i = -1;
  my $j;

  if ( $wpar == 0 ) {

    #print "graphing CPUOS   : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;

    $cmd .= " --upper-limit=100";
    $cmd .= " --rigid";

    foreach $rrd (@rrd_files) {    # LPM cycle
      chomp($rrd);
      $i++;
      $j = $i - 1;
      $rrd =~ s/:/\\:/g;

      $cmd .= " DEF:entitled${i}=\"$rrd\":entitled:AVERAGE";
      $cmd .= " DEF:cpusy${i}=\"$rrd\":cpu_sy:AVERAGE";
      $cmd .= " DEF:cpuus${i}=\"$rrd\":cpu_us:AVERAGE";
      $cmd .= " DEF:cpuwa${i}=\"$rrd\":cpu_wa:AVERAGE";

      if ( $i == 0 ) {
        $cmd .= " CDEF:entitledr${i}=entitled${i}";
        $cmd .= " CDEF:cpusyr${i}=cpusy${i}";
        $cmd .= " CDEF:cpuusr${i}=cpuus${i}";
        $cmd .= " CDEF:cpuwar${i}=cpuwa${i}";
        next;
      } ## end if ( $i == 0 )
      $cmd .= " CDEF:entitledr${i}=entitledr${j},UN,entitled${i},entitledr${j},IF";
      $cmd .= " CDEF:cpusyr${i}=cpusyr${j},UN,cpusy${i},cpusyr${j},IF";
      $cmd .= " CDEF:cpuusr${i}=cpuusr${j},UN,cpuus${i},cpuusr${j},IF";
      $cmd .= " CDEF:cpuwar${i}=cpuwar${j},UN,cpuwa${i},cpuwar${j},IF";
    } ## end foreach $rrd (@rrd_files)
    $cmd .= " CDEF:entitled=entitledr${i}";
    $cmd .= " CDEF:cpusy=cpusyr${i}";
    $cmd .= " CDEF:cpuus=cpuusr${i}";
    $cmd .= " CDEF:cpuwa=cpuwar${i}";

    $cmd .= " CDEF:stog=100,cpusy,-,cpuus,-,cpuwa,-";
    $cmd .= " COMMENT:\"   [%]              AVRG      MAX  \\n\"";
    $cmd .= " AREA:cpusy#0080FF:\" Sys         \"";
    $cmd .= " GPRINT:cpusy:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpusy:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:cpuus#FFFF00:\" User        \"";
    $cmd .= " GPRINT:cpuus:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpuus:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:cpuwa#FF4040:\" IO wait     \"";
    $cmd .= " GPRINT:cpuwa:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpuwa:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:stog#00FF00:\" Idle        \"";
    $cmd .= " GPRINT:stog:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:stog:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";

    #  "VRULE:0#000000"
  } ## end if ( $wpar == 0 )
  else {    # WPARs
    $rrd =~ s/:/\\:/g;

    $cmd .= " DEF:pc=\"$rrd\":entitled:AVERAGE";
    $cmd .= " DEF:cpusy=\"$rrd\":cpu_sy:AVERAGE";
    $cmd .= " DEF:cpuus=\"$rrd\":cpu_us:AVERAGE";
    $cmd .= " CDEF:proc=pc,100,/";
    $cmd .= " CDEF:cpusyc=cpusy,proc,*";
    $cmd .= " CDEF:cpuusc=cpuus,proc,*";
    $cmd .= " COMMENT:\"CPU cores           AVRG      MAX  \\n\"";
    $cmd .= " AREA:cpusyc#0080FF:\" Sys         \"";
    $cmd .= " GPRINT:cpusyc:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpusyc:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " STACK:cpuusc#FFFF00:\" User        \"";
    $cmd .= " GPRINT:cpuusc:AVERAGE:\" $form_dec\"";
    $cmd .= " GPRINT:cpuusc:MAX:\" $form_dec\"";
    $cmd .= " $t2";
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";

    # $cmd .= " VRULE:0#000000";
  } ## end else [ if ( $wpar == 0 ) ]

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_oscpu : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_oscpu

sub graph_paging {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT:\" \"";
  my $t2         = "COMMENT:\"\\n\"";
  my $step_new   = $step;
  my $last       = "COMMENT: ";
  my $head       = "COMMENT:                            Average   Max\\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  # due to historical reports where is specific time passed
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST: %4.0lf";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";           # just a trick to place there something otherwise rrdtool complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_paging: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                           # some workaround, it should not go here
  }

  my $rrd = "$wrkdir/$server/$host/$lpar/pgs.mmm";

  if ($hyperv) {
    $rrd = "$wrkdir/$server/$host/pool.rrm";
  }

  # print "001 pgs $rrd\n";

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;          # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;       # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  my $last_txt      = " last ";
  my $header_prefix = "paging 1:";

  if ( $detail == 2 ) {
    $last_txt      = "";
    $header_prefix = "PG1";
    $lpar_slash_nmon_head = "" if $server eq "Linux";
    if ( $wpar == 1 ) {
      $header_prefix = "PG1";
    }
  } ## end if ( $detail == 2 )

  my $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700) {
        $header = "$start_human - $end_human";
      }
      if ( $detail == 10 ) {
        $header = "$server : $header_prefix $lpar_slash_nmon : $start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  if ( $type =~ "d" ) {

    # my $rrd_last = rrd_file_actual ("$rrd"); # no it is hard-linked through all HMCs
    my $rrd_last = $rrd;
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          $last = "";

          #if ( $m_arr[2] > 0 ) {
          #  my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
          #  my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
          #  $last ="COMMENT:Last memory consumption in GB         $last_mem_g";
          #}
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( !-f "$rrd" ) {
    error( "$rrd does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $filter = $filter_max_paging;
  if ($hyperv) {
    $filter *= 1000; # seems to be better
  }

  # values in rrdtool are in pages == 4096B
  if ( $type =~ m/y/ ) {

    # lower limits for yearly graphs as they are averaged ....
    $filter = $filter / 10;
  }
  if ( $type =~ m/m/ ) {

    # lower limits for monthly graphs as they are averaged ....
    $filter = $filter / 2;
  }

  my $vertical_label = "--vertical-label=MB/sec";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;                  #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "pgs.mmm", \@rrd_files, $cgi );

  # print STDERR "8627 lpm_find_files is returning @rrd_files\n";

  # print STDERR "graphing PAGING   : $rrd: $host:$server:$lpar_slash:$type_sam:$type:$start_time:$end_time,\$detail $detail,\$header $header \$no_legend $no_legend\n";

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  $cmd .= " --base=1024";

  my $i = -1;
  my $j;

  foreach $rrd (@rrd_files) {    # LPM cycle
    chomp($rrd);
    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    if ($hyperv) {
      $cmd .= " DEF:pagein${i}=\"$rrd\":PagesInputPersec:AVERAGE";
      $cmd .= " DEF:pageout${i}=\"$rrd\":PagesOutputPersec:AVERAGE";
    }
    else {
      $cmd .= " DEF:pagein${i}=\"$rrd\":page_in:AVERAGE";
      $cmd .= " DEF:pageout${i}=\"$rrd\":page_out:AVERAGE";
    }
    if ( $i == 0 ) {
      $cmd .= " CDEF:pageinr${i}=pagein${i}";
      $cmd .= " CDEF:pageoutr${i}=pageout${i}";
      next;
    }
    $cmd .= " CDEF:pageinr${i}=pageinr${j},UN,pagein${i},pageinr${j},IF";
    $cmd .= " CDEF:pageoutr${i}=pageoutr${j},UN,pageout${i},pageoutr${j},IF";
  } ## end foreach $rrd (@rrd_files)
  $cmd .= " CDEF:pagein=pageinr${i}";
  $cmd .= " CDEF:pageout=pageoutr${i}";

  $cmd .= " CDEF:pagein_b_nf=pagein,4096,*"; # 4 kiB pages
  $cmd .= " CDEF:pageout_b_nf=pageout,4096,*";
  $cmd .= " CDEF:pagein_b=pagein_b_nf,$filter,GT,UNKN,pagein_b_nf,IF";
  $cmd .= " CDEF:pageout_b=pageout_b_nf,$filter,GT,UNKN,pageout_b_nf,IF";
  if ($hyperv) {
    $cmd .= " CDEF:pagein_mb=pagein_b,1048576,/,300,/"; # 5 mins perf data
    $cmd .= " CDEF:pagein_mb_neg=pagein_mb,-1,*";
    $cmd .= " CDEF:pageout_mb=pageout_b,1048576,/,300,/";
  }
  else {
    $cmd .= " CDEF:pagein_mb=pagein_b,1048576,/";
    $cmd .= " CDEF:pagein_mb_neg=pagein_mb,-1,*";
    $cmd .= " CDEF:pageout_mb=pageout_b,1048576,/";
  }
  $cmd .= " COMMENT:\"[MB/sec]     Avrg      Max\\n\"";
  $cmd .= " LINE1:pageout_mb#0080FF:\" Page out    \"";
  $cmd .= " GPRINT:pageout_mb:AVERAGE:\" %6.3lf\"";
  $cmd .= " GPRINT:pageout_mb:MAX:%6.3lf";
  $cmd .= " $t2";
  $cmd .= " LINE1:pagein_mb_neg#FF4040:\" Page in     \"";
  $cmd .= " GPRINT:pagein_mb:AVERAGE:\" %6.3lf\"";
  $cmd .= " GPRINT:pagein_mb:MAX:%6.3lf";

  #  if ( $graph_un ) {
  #    $cmd .= " CDEF:unavailable=pagein_mb,UN,INF,0,IF";
  #    $cmd .= " AREA:unavailable$color_un";
  #    $cmd .= " CDEF:unavailable_out=pageout_mb,UN,INF,0,IF,-1,*";
  #    $cmd .= " AREA:unavailable_out$color_un";
  #  }
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #      " VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_paging : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_paging

sub graph_multihmc    # CPU
{
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time_human = "";
  my $end_time_human   = "";
  my $server_vmw_name  = "";

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time       = $start_unix;
      $end_time         = $end_unix;
      $xgrid            = "";
      $start_time_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      $end_time_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  # print STDERR "10675 \$item $item \$host $host \$type $type \$server $server \$lpar $lpar \n";
  my $tmp_file     = "$basedir/tmp/multi-hmc-$host-$type.cmd";
  if ( $type eq "a" ) { $tmp_file = "$basedir/tmp/multi-hmc-$host-d.cmd"; } # Reporter TimeRange mode
  my $tmp_file_web = "/var/tmp/multi-hmc-$host-$type-web.cmd";
  if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-$host-d-web.cmd"; } # Reporter TimeRange mode

  if ( $item eq "clustser" ) {
    $tmp_file     = "$basedir/tmp/multi-hmc-$server-$host-$type.cmd";
    if ( $type eq "a" ) { $tmp_file = "$basedir/tmp/multi-hmc-$server-$host-d.cmd"; } # Reporter TimeRange mode
    $tmp_file_web = "/var/tmp/multi-hmc-$server-$host-$type-web.cmd";
    if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-$server-$host-d-web.cmd"; } # Reporter TimeRange mode
    ( my $rrd, $server_vmw_name ) = find_human_cluster_name();
  }
  if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  # replace graph size and total value
  open( FH, "< $tmp_file" ) || error( "multihmc detail: Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    $line =~ s/graph .* --title/$graph_cmd \"$name_out\" --title/;
    $line =~ s/ --start .* --end .* --imgformat PNG/ --start $start_time --end $end_time --imgformat PNG /;
    $line =~ s/ --width=400/ --width=$width  $font_def /;
    $line =~ s/ --height=150/ --height=$height $font_tit $disable_rrdtool_tag $no_legend /;

    # change canvas color
    $line =~ s/#$pic_col_orig/#$pic_col/g;

   if ( $detail == 9 ) {
     # --title "Servers aggregated : last day" --start
     my $s_start = index($line, "--title \"") +9;
     my $s_stop = index($line, " --start") -1;
     $html_heading = substr($line, $s_start, $s_stop - $s_start);
     $line =~ s/$html_heading//;
     if ( $start_time_human ne "" ) {
       $html_heading =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
       $html_heading =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
       $html_heading =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
       $html_heading =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
     } ## end elsif ( $start_time_human...)
   }
   else {

    if ( $start_time_human ne "" && $width < 700 ) {    # means hist reports or zoom
                                                        #$line =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
                                                        #$line =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
                                                        #$line =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
                                                        #$line =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
                                                        # new zoom is a small picture
      $line =~ s/--title.*: last day/--title \" $start_time_human - $end_time_human/;
      $line =~ s/--title.*: last month/--title \" $start_time_human - $end_time_human/;
      $line =~ s/--title.*: last week/--title \" $start_time_human - $end_time_human/;
      $line =~ s/--title.*: last year/--title \" $start_time_human - $end_time_human/;
    } ## end if ( $start_time_human...)
    elsif ( $start_time_human ne "" ) {
      $line =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
      $line =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
      $line =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
      $line =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
    } ## end elsif ( $start_time_human...)
   }
    $line =~ s/ --x-grid=.* COMMENT:\"Utilization/ $xgrid COMMENT:\"Utilization/;
    if ( $detail == 2 ) {
      $line =~ s/ --vertical-label=.* --units/ --units/;
      $line =~ s/last //;
    }
    $cmd = $line;
  } ## end foreach my $line (@lines)

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "multihmc detail: Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($$ret);
  return 0;
} ## end sub graph_multihmc

sub graph_multihmclpar {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time_human = "";
  my $end_time_human   = "";
  my $server_vmw_name  = "";

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time       = $start_unix;
      $end_time         = $end_unix;
      if ( $detail != 10 ) {
        $xgrid = "";
      }
      $start_time_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      $end_time_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  # print STDERR "8096 detail-graph-cgi.pl \$host $host \$server $server \$lpar $lpar \$detail $detail \$type $type\n";
  my $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$host-$type.cmd";
  if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$host-d.cmd"; } # Reporter TimeRange mode
  my $tmp_file_web = "/var/tmp/multi-hmc-lpar-$host-$type-web.cmd";
  if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-lpar-$host-d-web.cmd"; } # Reporter TimeRange mode

  if ( $item eq "clustlpar" ) {
    $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$server-$host-$type.cmd";
    if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$server-$host-d.cmd"; } # Reporter TimeRange mode
    $tmp_file_web = "/var/tmp/multi-hmc-lpar-$server-$host-$type-web.cmd";
    if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-lpar-$server-$host-d-web.cmd"; } # Reporter TimeRange mode
    ( undef, $server_vmw_name ) = find_human_cluster_name();
  }

  if ( $item eq "clustlpardy" ) {
    $tmp_file     = "$basedir/tmp/multi-hmc-lpar-rdy-$server-$host-$type.cmd";
    if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/multi-hmc-lpar-rdy-$server-$host-d.cmd"; } # Reporter TimeRange mode
    $tmp_file_web = "/var/tmp/multi-hmc-lpar-rdy-$server-$host-$type-web.cmd";
    if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-lpar-rdy-$server-$host-d-web.cmd"; } # Reporter TimeRange mode
    ( undef, $server_vmw_name ) = find_human_cluster_name();
  }

  if ( $item eq "rplpar" ) {
    $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$server-$lpar-$type.cmd";
    if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/multi-hmc-lpar-$server-$lpar-d.cmd"; } # Reporter TimeRange mode
    $tmp_file_web = "/var/tmp/multi-hmc-lpar-$server-$lpar-$type-web.cmd";
    if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/multi-hmc-lpar-$server-$lpar-d-web.cmd"; } # Reporter TimeRange mode
    ( undef, $server_vmw_name ) = find_human_cluster_name();
    $server_vmw_name .= ": " . find_human_rpool_name($lpar);
  } ## end if ( $item eq "rplpar")

  if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  # replace graph size
  # print STDERR "8870 detail-graph-cgi.pl \$tmp_file $tmp_file\n";
  open( FH, "< $tmp_file" ) || error( "multihmclpar detail: Can't open $tmp_file : $! " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    $line =~ s/graph .* --title/$graph_cmd \"$name_out\" --title/;
    $line =~ s/ --start .* --end .* --imgformat PNG/ --start $start_time --end $end_time --imgformat PNG /;
    $line =~ s/ --width=400/ --width=$width  $font_def /;
    $line =~ s/ --height=150/ --height=$height $font_tit $disable_rrdtool_tag $no_legend /;

    # change canvas color
    $line =~ s/#$pic_col_orig/#$pic_col/g;
    if ( $detail == 10 ) {
      $line =~ s/ --width=[0-9]+ / --width=$width  $font_def_reporter /;
      $line =~ s/ --height=[0-9]+ / --height=$height $font_tit_reporter /;
      $line =~ s/ --color=BACK#[A-Za-z0-9]+ / --color=BACK#FFF /;
      $line =~ s/ --color=SHADEA#[A-Za-z0-9]+ / --color=SHADEA#FFF /;
      $line =~ s/ --color=SHADEB#[A-Za-z0-9]+ / --color=SHADEB#FFF /;
      $line =~ s/ --color=CANVAS#[A-Za-z0-9]+ / --color=CANVAS#FFF /;
    }

    #  print STDERR "8456 \$item $item \$xgrid ,$xgrid, \$start_time_human ,$start_time_human,\n";
    if ( $detail == 9 ) {
      # --title "Servers aggregated : last day" --start
      my $s_start = index($line, "--title \"") +9;
      my $s_stop = index($line, " --start") -1;
      $html_heading = substr($line, $s_start, $s_stop - $s_start);
      $line =~ s/$html_heading//;
      if ( $start_time_human ne "" ) {
        $html_heading =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
        $html_heading =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
        $html_heading =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
        $html_heading =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
      } ## end elsif ( $start_time_human...)
      #print STDERR "8891 \$html_heading $html_heading\n \$line $line";
    }
    else {
      if ( $start_time_human ne "" && $width < 700 && $detail != 10 ) {    # means hist reports or zoom
        #$line =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
        #$line =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
        #$line =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
        #$line =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
        # new zoom is a small picture
        $line =~ s/--title.*: last day/--title \" $start_time_human - $end_time_human/;
        $line =~ s/--title.*: last month/--title \" $start_time_human - $end_time_human/;
        $line =~ s/--title.*: last week/--title \" $start_time_human - $end_time_human/;
        $line =~ s/--title.*: last year/--title \" $start_time_human - $end_time_human/;
      } ## end if ( $start_time_human...)
      elsif ( $start_time_human ne "" ) {
        $line =~ s/: last day/ $server_vmw_name: $start_time_human - $end_time_human/;
        $line =~ s/: last month/ $server_vmw_name: $start_time_human - $end_time_human/;
        $line =~ s/: last week/ $server_vmw_name: $start_time_human - $end_time_human/;
        $line =~ s/: last year/ $server_vmw_name: $start_time_human - $end_time_human/;
      } ## end elsif ( $start_time_human...)
    }
    $line =~ s/ --x-grid=.* COMMENT:\"Utilization/ $xgrid COMMENT:\"Utilization/;
    $line =~ s/ --x-grid=.* COMMENT:\"VM READY/ $xgrid COMMENT:\"VM READY/;
    if ( $detail == 2 ) {
      $line =~ s/ --vertical-label=.* --units/ --units/;
      $line =~ s/last //;
    }
    $cmd = $line;
  } ## end foreach my $line (@lines)

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "multihmclpar detail: Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($$ret);
  return 0;
} ## end sub graph_multihmclpar

sub graph_custom_trend {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $name_source = shift;

  #print STDERR "graph custom params $host,$server,$lpar,$type,$name_out,$type_sam,$detail,$start_unix,$end_unix,$name_source,\n";
  my $hist_rep = 0;

  my $start_time_human = "";
  my $end_time_human   = "";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time = "now-1$type";
  my $end_time   = "now+1$type";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $hist_rep         = 1;
      $start_time       = $start_unix;
      $end_time         = $end_unix;
      $xgrid            = "";
      $start_time_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      $end_time_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $tmp_file     = "$basedir/tmp/$name_source-$lpar-cpu_trend-$type.cmd";
  if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/$name_source-$lpar-cpu_trend-d.cmd"; } # Reporter TimeRange mode
  my $tmp_file_web = "/var/tmp/$name_source-$lpar-$type-web.cmd";
  if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/$name_source-$lpar-d-web.cmd"; } # Reporter TimeRange mode

  # replace graph size and other params
  open( FH, "< $tmp_file" ) || error( "custom detail: Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";

  my $no_legend_too = "";

  #if ( $rrd_legend eq "table" ) {
  #  $no_legend_too = "--no-legend ";
  #}

  foreach my $line (@lines) {
    $line =~ s/graph .* --title/$graph_cmd \"$name_out\" --title/;
    $line =~ s/ --start .* --end .* --imgformat PNG/ --start $start_time --end $end_time --imgformat PNG /;

    # change canvas color
    $line =~ s/#$pic_col_orig/#$pic_col/g;
    if ( $detail == 0 ) {
      $line =~ s/ --width=400/ --width=$width  $font_def $no_legend_too/;
      $line =~ s/ --height=150/ --height=$height $font_tit /;
    }
    if ( $detail == 10 ) {
      $line =~ s/ --width=[0-9]+ / --width=$width  $font_def_reporter /;
      $line =~ s/ --height=[0-9]+ / --height=$height $font_tit_reporter /;
      $line =~ s/ --color=BACK#[A-Za-z0-9]+ / --color=BACK#FFF /;
      $line =~ s/ --color=SHADEA#[A-Za-z0-9]+ / --color=SHADEA#FFF /;
      $line =~ s/ --color=SHADEB#[A-Za-z0-9]+ / --color=SHADEB#FFF /;
      $line =~ s/ --color=CANVAS#[A-Za-z0-9]+ / --color=CANVAS#FFF /;
    }
    $cmd = $line;
  } ## end foreach my $line (@lines)

  # prepare Updated
  my $updated = substr( $cmd, index( $cmd, "Updated" ), index( $cmd, "HRULE" ) - index( $cmd, "Updated" ) - 2 );
  $updated =~ s/\\//g;
  $updated =~ s/COMMENT://g;
  $updated =~ s/\"//g;
  $updated =~ s/ l//g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "custom detail: Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param( $$ret, $updated );
  return 0;
} ## end sub graph_custom_trend

sub graph_custom {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $name_source = shift;

  # print STDERR "9035 graph custom params $host,$server,$lpar,$type,$name_out,$type_sam,$detail,$start_unix,$end_unix,$name_source,\n";
  my $hist_rep = 0;

  my $start_time_human = "";
  my $end_time_human   = "";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $hist_rep         = 1;                                                      # hist reports or zoom
      $type             = 'm';
      $start_time       = $start_unix;
      $end_time         = $end_unix;
      if ( $detail != 10 ) {
        $xgrid            = "";
      }
      $start_time_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      $end_time_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $tmp_file     = "$basedir/tmp/$name_source-$lpar-$type.cmd";
  if ( $type eq "a" ) { $tmp_file     = "$basedir/tmp/$name_source-$lpar-d.cmd"; } # Reporter TimeRange mode
  my $tmp_file_web = "/var/tmp/$name_source-$lpar-$type-web.cmd";
  if ( $type eq "a" ) { $tmp_file_web = "/var/tmp/$name_source-$lpar-d-web.cmd"; } # Reporter TimeRange mode

  # replace graph size and other params
  open( FH, "< $tmp_file" ) || error( "custom detail: Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";

  my $no_legend_too = "";
  if ( $rrd_legend eq "table" ) {
    $no_legend_too = "--no-legend ";
  }
  foreach my $line (@lines) {
    $line =~ s/graph .* --title/$graph_cmd \"$name_out\" --title/;
    $line =~ s/ --start .* --end .* --imgformat PNG/ --start $start_time --end $end_time --imgformat PNG /;

    # change canvas color
    $line =~ s/#$pic_col_orig/#$pic_col/g;
    # print STDERR "8630 detail-graph-cgi.pl \$hist_rep $hist_rep \$detail $detail \$width $width\n";
    if ( $hist_rep == 1 ) {    # hist reports or zoom

      if ( $detail == 1 ) {
        $line =~ s/ --width=400/ --width=$width  $font_def_popup  /;
        $line =~ s/ --height=150/ --height=$height $font_tit_popup /;
      }
      if ( $detail == 0 || $detail == 9 ) {
        $line =~ s/ --width=400/ --width=$width  $font_def_hist $no_legend_too/;
        $line =~ s/ --height=150/ --height=$height $font_tit_hist /;
      }
      if ( $detail != 10 ) {
        $line =~ s/ --width=400/ --width=$width $font_def_hist/;
        $line =~ s/ --height=150/ --height=$height $font_tit_hist/;
        $line =~ s/ --x-grid=.*:%[a-z,A-Z] / /;
      }
      if ( $detail == 10 ) {
        $line =~ s/ --width=400/ --width=$width  $font_def_reporter  /;
        $line =~ s/ --height=150/ --height=$height $font_tit_reporter /;
        $line =~ s/ --color=BACK#[A-Za-z0-9]+ / --color=BACK#FFF /;
        $line =~ s/ --color=SHADEA#[A-Za-z0-9]+ / --color=SHADEA#FFF /;
        $line =~ s/ --color=SHADEB#[A-Za-z0-9]+ / --color=SHADEB#FFF /;
        $line =~ s/ --color=CANVAS#[A-Za-z0-9]+ / --color=CANVAS#FFF /;
        $line =~ s/ --x-grid=.*:%[a-z,A-Z] / $xgrid /;
      }
      if ( $width < 700 && $detail != 10 ) {    # cus zoom can be only a small picture
        if ( $detail == 9 ) {
          # --title "Servers aggregated : last day" --start
          my $s_start = index($line, "--title \"") +9;
          my $s_stop = index($line, " --start") -1;
          $html_heading = substr($line, $s_start, $s_stop - $s_start);
          $line =~ s/$html_heading//;
        }
        else {
          $line =~ s/--title.*: last day/--title \" $start_time_human - $end_time_human/;
          $line =~ s/--title.*: last month/--title \" $start_time_human - $end_time_human/;
          $line =~ s/--title.*: last week/--title \" $start_time_human - $end_time_human/;
          $line =~ s/--title.*: last year/--title \" $start_time_human - $end_time_human/;
        }
      } ## end if ( $width < 700 )
      else {
        if ( $detail == 9 ) {
          # --title "Servers aggregated : last day" --start
          my $s_start = index($line, "--title \"") +9;
          my $s_stop = index($line, " --start") -1;
          $html_heading = substr($line, $s_start, $s_stop - $s_start);
          $line =~ s/$html_heading//;

          $html_heading =~ s/: last day/ : $start_time_human - $end_time_human/;
          $html_heading =~ s/: last month/ : $start_time_human - $end_time_human/;
          $html_heading =~ s/: last week/ : $start_time_human - $end_time_human/;
          $html_heading =~ s/: last year/ : $start_time_human - $end_time_human/;
        }
        else {
          $line =~ s/: last day/ : $start_time_human - $end_time_human/;
          $line =~ s/: last month/ : $start_time_human - $end_time_human/;
          $line =~ s/: last week/ : $start_time_human - $end_time_human/;
          $line =~ s/: last year/ : $start_time_human - $end_time_human/;
        }
      } ## end else [ if ( $width < 700 ) ]
    } ## end if ( $hist_rep == 1 )
    else {
      if ( $detail == 9 ) {
        # --title "Servers aggregated : last day" --start
        my $s_start = index($line, "--title \"") +9;
        my $s_stop = index($line, " --start") -1;
        $html_heading = substr($line, $s_start, $s_stop - $s_start);
        $line =~ s/$html_heading//;
      }
      if ( $detail == 1 ) {
        $line =~ s/ --width=400/ --width=$width  $font_def  /;
        $line =~ s/ --height=150/ --height=$height $font_tit /;
        $line =~ s/ --x-grid=.*:%[a-z,A-Z] / $xgrid /;
      }
      if ( $detail == 2 ) {
        $line =~ s/ --width=400/ --width=$width $font_def  /;
        $line =~ s/ --height=150/ --height=$height $font_tit $no_legend $disable_rrdtool_tag /;
        $line =~ s/: last /: /;
        $line =~ s/ Custom group[^:]*://;
        $line =~ s/ 4 weeks/ month/;
        $line =~ s/ --x-grid=.*:%[a-z,A-Z] / $xgrid /;
        $line =~ s/ --vertical-label=\"CPU cores\"/ /;
        $line =~ s/ --vertical-label=\"GB\"/ /;
        $line =~ s/ --vertical-label=\".* - Write\"/ /;
        $line =~ s/ --vertical-label=\".+?\"/ /;
      } ## end if ( $detail == 2 )
      if ( $detail == 0 || $detail == 9 ) {
        $line =~ s/ --width=400/ --width=$width  $font_def $no_legend_too/;
        $line =~ s/ --height=150/ --height=$height $font_tit /;
      }
    } ## end else [ if ( $hist_rep == 1 ) ]
    $cmd = $line;
  } ## end foreach my $line (@lines)

  # prepare Updated
  my $updated = substr( $cmd, index( $cmd, "Updated" ), index( $cmd, "HRULE" ) - index( $cmd, "Updated" ) - 2 );
  $updated =~ s/\\//g;
  $updated =~ s/COMMENT://g;
  $updated =~ s/\"//g;
  $updated =~ s/ l//g;
  # for customvmmemconsumed of VMS change some strings
  if ( $item =~ m/customvmmemconsumed$/ ) {
    $cmd =~ s/_active/_granted/g;
    $cmd =~ s/Active /Granted /g;
    $cmd =~ s/multihmclpar_mem/multihmclpar_memgrant/g;
  }

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "custom detail: Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param( $$ret, $updated );
  return 0;
} ## end sub graph_custom

# send to file string generated for RRD
sub alfa {
  my $cmd = shift;

  my $sss = "/var/tmp/ssss.out";
  open( FF, ">$sss" );
  print FF "$cmd\n";
  close(FF);
} ## end sub alfa

sub xgrid_set {
  my $type   = shift;
  my $detail = shift;
  my $xgrid  = "--x-grid=";

  if ( $type =~ m/d/ ) {
    if ( $detail == 0 || $detail == 9 ) {
      $xgrid .= "MINUTE:60:HOUR:2:HOUR:4:0:%H";
    }
    if ( $detail == 1 || $detail == 10 ) {
      $xgrid .= "MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    if ( $detail == 2 ) {

      # dashboard
      #$xgrid .= "MINUTE:60:HOUR:1:HOUR:1:0:%H";
      if ( $super_detail == 3 ) {
        $xgrid .= "HOUR:1:DAY:1:HOUR:2:0:%H";
      }
      else {
        $xgrid .= "HOUR:6:DAY:1:HOUR:6:0:%H";
      }
    } ## end if ( $detail == 2 )
  } ## end if ( $type =~ m/d/ )
  if ( $type =~ m/w/ ) {
    if ( $detail == 0 || $detail == 9 || $detail == 10 ) {
      $xgrid .= "HOUR:8:DAY:1:DAY:1:0:%a";
    }
    if ( $detail == 1 ) {
      $xgrid .= "HOUR:12:HOUR:6:HOUR:12:0:%a-%H";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "DAY:1:DAY:7:DAY:1:0:%d";
    }
  } ## end if ( $type =~ m/w/ )
  if ( $type =~ m/m/ ) {
    if ( $detail == 0 || $detail == 9 ) {
      $xgrid .= "DAY:1:DAY:2:DAY:2:0:%d";
    }
    if ( $detail == 1 || $detail == 10 ) {
      $xgrid .= "HOUR:12:DAY:1:DAY:1:0:%d";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "DAY:7:MONTH:1:DAY:7:0:%d";
    }
  } ## end if ( $type =~ m/m/ )
  if ( $type =~ m/y/ ) {
    if ( $detail == 0 || $detail == 9 ) {
      $xgrid .= "MONTH:1:MONTH:1:MONTH:1:0:%b";
    }
    if ( $detail == 1 || $detail == 10 ) {
      $xgrid .= "MONTH:1:MONTH:1:MONTH:1:0:%b";
    }
    if ( $detail == 2 ) {

      # dashboard
      $xgrid .= "MONTH:4:MONTH:12:MONTH:4:0:%b";
    }
  } ## end if ( $type =~ m/y/ )
  if ( $type eq "a" ) { # let them auto xgrid by rrdtool
    $xgrid = "--alt-y-grid";
  }

  return $xgrid;
} ## end sub xgrid_set

sub text_set {
  my $type = shift;
  my $text = "day";

  $text = "day";
  if ( $type =~ m/w/ ) {
    $text = "week";
  }
  if ( $type =~ m/m/ ) {
    $text = "month";
  }
  if ( $type =~ m/y/ ) {
    $text = "year";
  }
  return $text;
} ## end sub text_set

sub graph_custom_xport {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $name_source = shift;
  my $hist_rep    = 0;

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $hist_rep = 1;
    }
  }

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $tmp_file     = "$basedir/tmp/$name_source-$lpar-$type-xpo.cmd";
  my $tmp_file_web = "/var/tmp/$name_source-$lpar-$type-web.cmd";

  # replace graph size and total value
  open( FH, "< $tmp_file" ) || error( "custom detail: XPORT: Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @lines = <FH>;
  close(FH);
  my $cmd = "";
  foreach my $line (@lines) {
    if ( $hist_rep == 1 ) {
      $line =~ s/ --start .* --end .* --step / --start $start_unix --end $end_unix --step /;
    }
    $cmd = $line;
  } ## end foreach my $line (@lines)

  my @ret_arr = "";
  my $rest    = "";
  eval {
    RRDp::cmd qq($cmd);

    #    @ret_arr = RRDp::read;
    $rest = RRDp::read;
  };
  if ($@) {
    error( "custom detail: Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # converting from XML to CSV
  my $out     = 0;
  my $out_txt = "";
  @ret_arr = $$rest;
  foreach my $line (@ret_arr) {
    if ( $out == 0 ) {
      if ( $line =~ m/xml version=/ ) {
        $out_txt .= $line;
        $out = 1;
      }
    } ## end if ( $out == 0 )
    else {
      if ( $line !~ m/^OK u:/ ) {
        $out_txt .= $line;
      }
    }
  } ## end foreach my $line (@ret_arr)
  print "Content-Disposition: attachment;filename=\"CustomGroup\_$lpar.csv\"\n\n";
  xport_print( $out_txt, 1 );

  return 0;
} ## end sub graph_custom_xport

sub xport_print {
  my $xml_org = shift;
  my $multi   = shift;
  my $xml     = "";

  if ( $multi == 1 ) {

    #print OUT "--xport-- $xml_org\n";
    $xml = XMLin($xml_org);
  }
  else {
    #print OUT "--xport++ $$xml_org\n";
    $xml = XMLin($$xml_org);
  }
  if ( ref $xml->{meta}{legend}{entry} eq "ARRAY" ) {
    foreach my $item ( @{ $xml->{meta}{legend}{entry} } ) {    # in case semicolon in lpar name
      $item = "\"" . $item . "\"";
    }
  }
  else {
    $xml->{meta}{legend}{entry} = "\"" . $xml->{meta}{legend}{entry} . "\"";
  }

  print join( ";", 'Timestamp DD.MM.YYYY HH:MM', @{ $xml->{meta}{legend}{entry} } ), "\n";
  foreach my $row ( @{ $xml->{data}{row} } ) {
    my $time = strftime "%d.%m.%y %H:%M:%S", localtime( $row->{t} );
    my $line = join( ";", $time, @{ $row->{v} } );
    $line =~ s/NaNQ/0/g;
    $line =~ s/NaN/0/g;
    $line =~ s/nan/0/g;
    print "$line\n";
  } ## end foreach my $row ( @{ $xml->...})
  return 0;
} ## end sub xport_print

sub graph_hea {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $file       = shift;    #as $item
  my $start_unix = shift;
  my $end_unix   = shift;

  # `echo "graph hea $host,$server,$lpar,$type,$detail" >> /tmp/dgc-123`;
  my $t  = "COMMENT: ";
  my $t2 = "COMMENT:\\n";

  chomp($file);
  ( undef, my $adapter_id, my $port ) = split( /-/, $file );
  $port =~ s/\.db//;

  #print "-- $file $port\n";
  my $rrd = "$wrkdir/$server/$host/$file";

  my $first        = 0;
  my $act_time     = time();
  my $port_group   = 0;
  my $phys_port_id = 0;

  if ( $rrd =~ "HSCL" || $rrd =~ "VIOSE0" ) { error( "hea: contain HSCL or VIOSE0 " . __FILE__ . ":" . __LINE__ ) && return 0; }

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $header = "HEA $port : last $text";
  if ( $detail == 2 ) {
    $header = "HEA $port : $text";
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "HEA $port : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  #`echo "$xgrid,$type,$detail" >> /tmp/e125`;

  # It must be here
  #  draw_graph_hea ("day","d","MINUTE:60:HOUR:2:HOUR:4:0:%H",$managedname,$rrd,$adapter_id,$port);
  # draw_graph_hea ("week","w","HOUR:8:DAY:1:DAY:1:86400:%a",$managedname,$rrd,$adapter_id,$port);
  # draw_graph_hea ("month","m","DAY:1:DAY:2:DAY:2:0:%d",$managedname,$rrd,$adapter_id,$port);
  # draw_graph_hea ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$adapter_id,$port);
  # draw_graph_hea_summ ("month","m","DAY:1:DAY:2:DAY:2:0:%d",$managedname,$rrd,$adapter_id,$port);
  # draw_graph_hea_summ ("year","y","MONTH:1:MONTH:1:MONTH:1:0:%b",$managedname,$rrd,$adapter_id,$port);

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt = RRDp::read;
    my $l       = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\: $l ";
  } ## end if ( $type =~ "d" )

  my $vertical_label = "--vertical-label=Bytes/sec";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  #  print "creating graph : $host:$server:$type:$adapter_id:$port\n" if $DEBUG ;

  # filter everything above 11Gbites
  my $filter = 1100000000;

  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  $rrd =~ s/:/\\:/g;
  RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "--step=$STEP_HEA"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--alt-autoscale-max"
      "$xgrid"
      "--base=1024"
      "--alt-y-grid"
      "DEF:recv_np=$rrd:recv_octets:AVERAGE"
      "DEF:trans_np=$rrd:trans_octets:AVERAGE"
      "CDEF:recv=recv_np,$filter,GT,UNKN,recv_np,IF"
      "CDEF:trans=trans_np,$filter,GT,UNKN,trans_np,IF"
      "CDEF:recvkb=recv,1024,/"
      "CDEF:transkb=trans,1024,/"
      "CDEF:trans-neg=0,trans,-"
      "COMMENT:   Average  in kB\\n"
      "AREA:recv#00FF00: Received   "
      "GPRINT:recvkb:AVERAGE: %8.1lf"
      "$t2"
      "AREA:trans-neg#FFFF00: Transmited "
      "GPRINT:transkb:AVERAGE: %8.1lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
  my $answ = RRDp::read;
  if ( $$answ =~ "ERROR" ) {
    error( "$host:$server Graph rrdtool error : $$answ " . __FILE__ . ":" . __LINE__ );
  }
  $png_end_heading = ret_graph_param($answ);
  return 0;
} ## end sub graph_hea

sub graph_trend_pool {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $upper    = shift;
  my $detail   = shift;
  my $t        = "COMMENT: ";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $last     = "COMMENT: ";
  my $xgrid    = "--x-grid=MONTH:1:MONTH:1:MONTH:1:0:%b";

  if ( $upper eq '' || $upper == 0 ) {
    $upper = 0.1;
  }

  my $rrd     = "$wrkdir/$server/$host/$lpar.rr$type_sam";
  my $rrd_lpm = "$wrkdir/$server/*/$lpar.rr$type_sam";

  if ( $item eq "trendpool-max" ) {
    $rrd     = "$wrkdir/$server/$host/$lpar.xr$type_sam";
    $rrd_lpm = "$wrkdir/$server/*/$lpar.xr$type_sam";
  }

  if ($hitachi) {
    $rrd  = "$wrkdir/$server/$host/SYS-CPU.hrm";
  }

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "CPU pool :$last_txt year";

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  elsif ( $hitachi ) {
    $vertical_label = "--vertical-label=\"CPU usage %\"";
  }

  # Trend for yearly chart 1y back  + 1y forward
  #print "creating graph : $host:$server:$lpar:$type_sam:$type:trend\n" if $DEBUG ;

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $rrd =~ s/:/\\:/g;
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  if ( $detail == 10 ) {
    $header = "$server : CPU pool :$last_txt year";
  }

  my $cmd .= "$graph_cmd \"$name_out\"";
  $cmd    .= " --title \"$header - trend\"";
  $cmd    .= " --start now-1$type";
  $cmd    .= " --end now+1$type";
  $cmd    .= " --imgformat PNG";
  $cmd    .= " --slope-mode";
  $cmd    .= " $no_minor";
  $cmd    .= " --width=$width_trend";
  $cmd    .= " --height=$height";
  $cmd    .= " --step=$step";
  $cmd    .= " --lower-limit=0.00";
  $cmd    .= " --color=BACK#$pic_col";
  $cmd    .= " --color=SHADEA#$pic_col";
  $cmd    .= " --color=SHADEB#$pic_col";
  $cmd    .= " --color=CANVAS#$pic_col";
  $cmd    .= " --alt-autoscale-max";
  $cmd    .= " --upper-limit=0.1";
  $cmd    .= " $vertical_label";
  $cmd    .= " --units-exponent=1.00";
  $cmd    .= " --alt-y-grid";
  $cmd    .= " $no_legend";
  $cmd    .= " $disable_rrdtool_tag";
  $cmd    .= " $font_def";
  $cmd    .= " $font_tit";
  $cmd    .= " $xgrid";

  if ( $hitachi ) {
    $cmd .= " --upper-limit=100";
    $cmd .= " DEF:usedp=\"$rrd\":cpu_usedp:AVERAGE";
    $cmd .= " DEF:usedp_1m=\"$rrd\":cpu_usedp:AVERAGE:start=-1m";
    $cmd .= " DEF:usedp_3m=\"$rrd\":cpu_usedp:AVERAGE:start=-3m";
    $cmd .= " DEF:usedp_1y=\"$rrd\":cpu_usedp:AVERAGE:start=-1y";
    $cmd .= " VDEF:Dd=usedp_1m,LSLSLOPE";
    $cmd .= " VDEF:Hd=usedp_1m,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-1m=usedp_1m,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=usedp_3m,LSLSLOPE";
    $cmd .= " VDEF:Hm=usedp_3m,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-3m=usedp_3m,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=usedp_1y,LSLSLOPE";
    $cmd .= " VDEF:Hy=usedp_1y,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-1y=usedp_1y,POP,Dy,COUNT,*,Hy,+";

    $cmd .= " COMMENT:\"   Average          \\n\"";
    $cmd .= " LINE1:usedp#FF0000:\" CPU usage\"";
    $cmd .= " GPRINT:usedp:AVERAGE:\"    %2.2lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-1m#0080FF:\" CPU usage    - last 1 month trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-3m#80FFFF:\" CPU usage    - last 3 months trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-1y#FF8080:\" CPU usage    - last year trend\"";
    $cmd .= " $last";
    $cmd .= " $t2";
  } ## end if ( $hitachi )
  else {
    $rrd_lpm =~ s/:/\\:/g;

    # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
    my ( $result_cmd, $CPU_tot_d, $CPU_util_d, $CPU_conf_d, $CPU_bor_d, $CPU_tot_m, $CPU_util_m, $CPU_conf_m, $CPU_bor_m, $CPU_tot_y, $CPU_util_y, $CPU_conf_y, $CPU_bor_y, $CPU_tot, $CPU_util, $CPU_conf, $CPU_bor ) = LPM_easy_trend( "$rrd_lpm", "total_pool_cycles", "utilized_pool_cyc", "conf_proc_units", "bor_proc_units" );

    if ( $item eq "trendpool-max" ) {
      my $result_to_parse = $result_cmd;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      $result_cmd = "";

      foreach my $cmd_line (@result_cmd_lines) {
	chomp $cmd_line;

	#print STDERR "$cmd_line\n";
	$cmd_line =~ s/^\s+//g;
	$cmd_line =~ s/\s+$//g;

	my $line_max = $cmd_line;

	if ( $cmd_line =~ "^DEF:" ) {
	  $line_max =~ s/:AVERAGE$/:MAX/;
	  $line_max =~ s/:AVERAGE:start=/:MAX:start=/;
	}
	$result_cmd .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)
    } ## end if ( $item eq "trendpool-max")

    $cmd .= $result_cmd;
    $cmd .= " CDEF:totcycd=$CPU_tot_d";
    $cmd .= " CDEF:totcycm=$CPU_tot_m";
    $cmd .= " CDEF:totcycy=$CPU_tot_y";
    $cmd .= " CDEF:totcyc=$CPU_tot";

    $cmd .= " CDEF:uticycd=$CPU_util_d";
    $cmd .= " CDEF:uticycm=$CPU_util_m";
    $cmd .= " CDEF:uticycy=$CPU_util_y";
    $cmd .= " CDEF:uticyc=$CPU_util";

    $cmd .= " CDEF:cpud=$CPU_conf_d";
    $cmd .= " CDEF:cpum=$CPU_conf_m";
    $cmd .= " CDEF:cpuy=$CPU_conf_y";
    $cmd .= " CDEF:cpu=$CPU_conf";

    $cmd .= " CDEF:cpubord=$CPU_bor_d";
    $cmd .= " CDEF:cpuborm=$CPU_bor_m";
    $cmd .= " CDEF:cpubory=$CPU_bor_y";
    $cmd .= " CDEF:cpubor=$CPU_bor";

    $cmd .= " CDEF:totcpud=cpud,cpubord,+";
    $cmd .= " CDEF:totcpum=cpum,cpuborm,+";
    $cmd .= " CDEF:totcpuy=cpuy,cpubory,+";
    $cmd .= " CDEF:totcpu=cpu,cpubor,+";
    $cmd .= " CDEF:cpuutild=uticycd,totcycd,GT,UNKN,uticycd,totcycd,/,IF";
    $cmd .= " CDEF:cpuutilm=uticycm,totcycm,GT,UNKN,uticycm,totcycm,/,IF";
    $cmd .= " CDEF:cpuutily=uticycy,totcycy,GT,UNKN,uticycy,totcycy,/,IF";
    $cmd .= " CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF";
    $cmd .= " CDEF:cpuutiltotd=cpuutild,totcpud,*";
    $cmd .= " CDEF:cpuutiltotm=cpuutilm,totcpum,*";
    $cmd .= " CDEF:cpuutiltoty=cpuutily,totcpuy,*";
    $cmd .= " CDEF:cpuutiltot=cpuutil,totcpu,*";
    $cmd .= " CDEF:utilisa=cpuutil,100,*";
    $cmd .= " VDEF:Dd=cpuutiltotd,LSLSLOPE";
    $cmd .= " VDEF:Hd=cpuutiltotd,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=cpuutiltotm,LSLSLOPE";
    $cmd .= " VDEF:Hm=cpuutiltotm,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=cpuutiltoty,LSLSLOPE";
    $cmd .= " VDEF:Hy=cpuutiltoty,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+";

    if ( $item eq "trendpool-max" ) {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:cpu#00FF00:\" Configured CPU cores    \"";
      $cmd .= " GPRINT:cpu:AVERAGE:\"      %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:cpubor#FFFF00:\" Not assigned CPU cores  \"";
      $cmd .= " GPRINT:cpubor:AVERAGE:\"      %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Avrg of max peaks in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"%2.2lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Maximum peaks in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Maximum peaks in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Maximum peaks in CPU cores - last year trend\"";
      $cmd .= " $last";
      $cmd .= " $t2";
    } ## end if ( $item eq "trendpool-max")
    else {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:cpu#00FF00:\" Configured CPU cores    \"";
      $cmd .= " GPRINT:cpu:AVERAGE:\"  %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:cpubor#FFFF00:\" Not assigned CPU cores  \"";
      $cmd .= " GPRINT:cpubor:AVERAGE:\"  %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"  %2.2lf\"";
      $cmd .= " COMMENT:\"(\"";
      $cmd .= " GPRINT:utilisa:AVERAGE:\" %2.1lf\"";
      $cmd .= " COMMENT:\"\%)\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Utilization in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Utilization in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Utilization in CPU cores - last year trend\"";
      $cmd .= " $last";
      $cmd .= " $t2";
    } ## end else [ if ( $item eq "trendpool-max")]
  } ## end [ if ( $hitachi )]
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_trend_pool

sub graph_trend_shpool {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $upper    = shift;
  my $detail   = shift;
  my $t        = "COMMENT: ";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $last     = "COMMENT: ";
  my $xgrid    = "--x-grid=MONTH:1:MONTH:1:MONTH:1:0:%b";

  if ( $upper eq '' || $upper == 0 ) {
    $upper = 0.1;
  }

  my $rrd     = "$wrkdir/$server/$host/$lpar.rr$type_sam";
  my $rrd_lpm = "$wrkdir/$server/*/$lpar.rr$type_sam";

  if ( $item eq "trendshpool-max" ) {
    $rrd     = "$wrkdir/$server/$host/$lpar.xr$type_sam";
    $rrd_lpm = "$wrkdir/$server/*/$lpar.xr$type_sam";
  }

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacemnt for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  # add CPU pool alias into png header (if exists and it is not a default CPU pool)
  my $pool_id = $lpar;
  $pool_id =~ s/SharedPool//g;

  my $lpar_pool_alias = $lpar_slash_nmon;
  if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt" ) {
    open( FR, "< $wrkdir/$server/$host/cpu-pools-mapping.txt" );
    foreach my $linep (<FR>) {
      chomp($linep);
      ( my $id, my $pool_name ) = split( /,/, $linep );
      if ( $id == $pool_id ) {
        $lpar_pool_alias = "$pool_name";
        last;
      }
    } ## end foreach my $linep (<FR>)
    close(FR);
  } ## end if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt")

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "POOL: $lpar_pool_alias :$last_txt year";

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # Trend for yearly chart 1y back  + 1y forward
  #print "creating graph : $host:$server:$lpar:$type_sam:$type:trend\n" if $DEBUG ;

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $rrd =~ s/:/\\:/g;
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  if ( $detail == 10 ) {
    $header = "$server : POOL: $lpar_pool_alias :$last_txt year";
  }

  my $cmd .= "$graph_cmd \"$name_out\"";
  $cmd    .= " --title \"$header - trend\"";
  $cmd    .= " --start now-1$type";
  $cmd    .= " --end now+1$type";
  $cmd    .= " --imgformat PNG";
  $cmd    .= " --slope-mode";
  $cmd    .= " $no_minor";
  $cmd    .= " --width=$width_trend";
  $cmd    .= " --height=$height";
  $cmd    .= " --step=$step";
  $cmd    .= " --lower-limit=0.00";
  $cmd    .= " --color=BACK#$pic_col";
  $cmd    .= " --color=SHADEA#$pic_col";
  $cmd    .= " --color=SHADEB#$pic_col";
  $cmd    .= " --color=CANVAS#$pic_col";
  $cmd    .= " --alt-autoscale-max";
  $cmd    .= " --upper-limit=0.1";
  $cmd    .= " $vertical_label";
  $cmd    .= " --units-exponent=1.00";
  $cmd    .= " --alt-y-grid";
  $cmd    .= " $no_legend";
  $cmd    .= " $disable_rrdtool_tag";
  $cmd    .= " $font_def";
  $cmd    .= " $font_tit";
  $cmd    .= " $xgrid";

  $rrd_lpm =~ s/:/\\:/g;

  if ( $lpar !~ m/^SharedPool0$/ ) {

    # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
    my ( $result_cmd, $CPU_max_d, $CPU_res_d, $CPU_totcyc_d, $CPU_uticyc_d, $CPU_max_m, $CPU_res_m, $CPU_totcyc_m, $CPU_uticyc_m, $CPU_max_y, $CPU_res_y, $CPU_totcyc_y, $CPU_uticyc_y, $CPU_max, $CPU_res, $CPU_totcyc, $CPU_uticyc ) = LPM_easy_trend( "$rrd_lpm", "max_pool_units", "res_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

    if ( $item eq "trendshpool-max" ) {
      my $result_to_parse = $result_cmd;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      $result_cmd = "";

      foreach my $cmd_line (@result_cmd_lines) {
        chomp $cmd_line;

        #print STDERR "$cmd_line\n";
        $cmd_line =~ s/^\s+//g;
        $cmd_line =~ s/\s+$//g;

        my $line_max = $cmd_line;

        if ( $cmd_line =~ "^DEF:" ) {
          $line_max =~ s/:AVERAGE$/:MAX/;
          $line_max =~ s/:AVERAGE:start=/:MAX:start=/;
        }
        $result_cmd .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)
    } ## end if ( $item eq "trendshpool-max")

    $cmd .= $result_cmd;
    $cmd .= " CDEF:maxd=$CPU_max_d";
    $cmd .= " CDEF:maxm=$CPU_max_m";
    $cmd .= " CDEF:maxy=$CPU_max_y";
    $cmd .= " CDEF:max=$CPU_max";

    $cmd .= " CDEF:resd=$CPU_res_d";
    $cmd .= " CDEF:resm=$CPU_res_m";
    $cmd .= " CDEF:resy=$CPU_res_y";
    $cmd .= " CDEF:res=$CPU_res";

    $cmd .= " CDEF:totcycd=$CPU_totcyc_d";
    $cmd .= " CDEF:totcycm=$CPU_totcyc_m";
    $cmd .= " CDEF:totcycy=$CPU_totcyc_y";
    $cmd .= " CDEF:totcyc=$CPU_totcyc";

    $cmd .= " CDEF:uticycd=$CPU_uticyc_d";
    $cmd .= " CDEF:uticycm=$CPU_uticyc_m";
    $cmd .= " CDEF:uticycy=$CPU_uticyc_y";
    $cmd .= " CDEF:uticyc=$CPU_uticyc";

    $cmd .= " CDEF:max1=max,res,-";
    $cmd .= " CDEF:cpuutild=uticycd,totcycd,/";
    $cmd .= " CDEF:cpuutilm=uticycm,totcycm,/";
    $cmd .= " CDEF:cpuutily=uticycy,totcycy,/";
    $cmd .= " CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF";
    $cmd .= " CDEF:cpuutiltotd=cpuutild,maxd,*";
    $cmd .= " CDEF:cpuutiltotm=cpuutilm,maxm,*";
    $cmd .= " CDEF:cpuutiltoty=cpuutily,maxy,*";
    $cmd .= " CDEF:cpuutiltot=cpuutil,max,*";
    $cmd .= " CDEF:utilisa=cpuutil,100,*";
    $cmd .= " VDEF:Dd=cpuutiltotd,LSLSLOPE";
    $cmd .= " VDEF:Hd=cpuutiltotd,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=cpuutiltotm,LSLSLOPE";
    $cmd .= " VDEF:Hm=cpuutiltotm,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=cpuutiltoty,LSLSLOPE";
    $cmd .= " VDEF:Hy=cpuutiltoty,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+";

    if ( $item eq "trendshpool-max" ) {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:res#00FF00:\" Reserved CPU cores      \"";
      $cmd .= " GPRINT:res:AVERAGE:\"      %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:max1#FFFF00:\" Max CPU cores           \"";
      $cmd .= " GPRINT:max1:AVERAGE:\"      %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Avrg of max peaks in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"%2.2lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Maximum peaks in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Maximum peaks in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Maximum peaks in CPU cores - last year trend\"";
      $cmd .= " $last";
      $cmd .= " $t2";
    } ## end if ( $item eq "trendshpool-max")
    else {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:res#00FF00:\" Reserved CPU cores      \"";
      $cmd .= " GPRINT:res:AVERAGE:\"  %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " STACK:max1#FFFF00:\" Max CPU cores           \"";
      $cmd .= " GPRINT:max1:AVERAGE:\"  %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"  %2.2lf\"";
      $cmd .= " COMMENT:\"(\"";
      $cmd .= " GPRINT:utilisa:AVERAGE:\" %2.1lf\"";
      $cmd .= " COMMENT:\"\%)\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Utilization in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Utilization in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Utilization in CPU cores - last year trend\"";
      $cmd .= " $last";
      $cmd .= " $t2";
    } ## end else [ if ( $item eq "trendshpool-max")]
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";
  } ## end if ( $lpar !~ m/^SharedPool0$/)
  else {
    # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
    my ( $result_cmd, $CPU_max_d, $CPU_totcyc_d, $CPU_uticyc_d, $CPU_max_m, $CPU_totcyc_m, $CPU_uticyc_m, $CPU_max_y, $CPU_totcyc_y, $CPU_uticyc_y, $CPU_max, $CPU_totcyc, $CPU_uticyc ) = LPM_easy_trend( "$rrd_lpm", "max_pool_units", "total_pool_cycles", "utilized_pool_cyc" );

    if ( $item eq "trendshpool-max" ) {
      my $result_to_parse = $result_cmd;
      $result_to_parse =~ s/^\s+//g;
      $result_to_parse =~ s/\s+$//g;
      $result_to_parse =~ s/ ([C]*DEF:)/\n$1/g;

      my @result_cmd_lines = split( "\n", $result_to_parse );
      $result_cmd = "";

      foreach my $cmd_line (@result_cmd_lines) {
        chomp $cmd_line;

        #print STDERR "$cmd_line\n";
        $cmd_line =~ s/^\s+//g;
        $cmd_line =~ s/\s+$//g;

        my $line_max = $cmd_line;

        if ( $cmd_line =~ "^DEF:" ) {
          $line_max =~ s/:AVERAGE$/:MAX/;
          $line_max =~ s/:AVERAGE:start=/:MAX:start=/;
        }
        $result_cmd .= " $line_max";
      } ## end foreach my $cmd_line (@result_cmd_lines)
    } ## end if ( $item eq "trendshpool-max")

    $cmd .= $result_cmd;
    $cmd .= " CDEF:maxd=$CPU_max_d";
    $cmd .= " CDEF:maxm=$CPU_max_m";
    $cmd .= " CDEF:maxy=$CPU_max_y";
    $cmd .= " CDEF:max=$CPU_max";

    $cmd .= " CDEF:totcycd=$CPU_totcyc_d";
    $cmd .= " CDEF:totcycm=$CPU_totcyc_m";
    $cmd .= " CDEF:totcycy=$CPU_totcyc_y";
    $cmd .= " CDEF:totcyc=$CPU_totcyc";

    $cmd .= " CDEF:uticycd=$CPU_uticyc_d";
    $cmd .= " CDEF:uticycm=$CPU_uticyc_m";
    $cmd .= " CDEF:uticycy=$CPU_uticyc_y";
    $cmd .= " CDEF:uticyc=$CPU_uticyc";

    $cmd .= " CDEF:cpuutild=uticycd,totcycd,/";
    $cmd .= " CDEF:cpuutilm=uticycm,totcycm,/";
    $cmd .= " CDEF:cpuutily=uticycy,totcycy,/";
    $cmd .= " CDEF:cpuutil=uticyc,totcyc,GT,UNKN,uticyc,totcyc,/,IF";
    $cmd .= " CDEF:cpuutiltotd=cpuutild,maxd,*";
    $cmd .= " CDEF:cpuutiltotm=cpuutilm,maxm,*";
    $cmd .= " CDEF:cpuutiltoty=cpuutily,maxy,*";
    $cmd .= " CDEF:cpuutiltot=cpuutil,max,*";
    $cmd .= " CDEF:utilisa=cpuutil,100,*";
    $cmd .= " VDEF:Dd=cpuutiltotd,LSLSLOPE";
    $cmd .= " VDEF:Hd=cpuutiltotd,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendd=cpuutiltotd,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=cpuutiltotm,LSLSLOPE";
    $cmd .= " VDEF:Hm=cpuutiltotm,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendm=cpuutiltotm,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=cpuutiltoty,LSLSLOPE";
    $cmd .= " VDEF:Hy=cpuutiltoty,LSLINT";
    $cmd .= " CDEF:cpuutiltottrendy=cpuutiltoty,POP,Dy,COUNT,*,Hy,+";

    if ( $item eq "trendshpool-max" ) {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:max#00FF00:\" Max CPU cores           \"";
      $cmd .= " GPRINT:max:AVERAGE:\"      %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Avrg of max peaks in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"%2.2lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Maximum peaks in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Maximum peaks in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Maximum peaks in CPU cores - last year trend\"";
      $cmd .= " $t2";
    } ## end if ( $item eq "trendshpool-max")
    else {
      $cmd .= " COMMENT:\"   Average   \\n\"";
      $cmd .= " AREA:max#00FF00:\" Max CPU cores           \"";
      $cmd .= " GPRINT:max:AVERAGE:\"  %2.1lf\"";
      $cmd .= " $t2";
      $cmd .= " LINE1:cpuutiltot#FF0000:\" Utilization in CPU cores\"";
      $cmd .= " GPRINT:cpuutiltot:AVERAGE:\"  %2.2lf\"";
      $cmd .= " COMMENT:\"(\"";
      $cmd .= " GPRINT:utilisa:AVERAGE:\" %2.1lf\"";
      $cmd .= " COMMENT:\"\%)\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendd#0080FF:\" Utilization in CPU cores - last 1 month trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendm#80FFFF:\" Utilization in CPU cores - last 3 months trend\"";
      $cmd .= " $t2";
      $cmd .= " LINE2:cpuutiltottrendy#FF8080:\" Utilization in CPU cores - last year trend\"";
      $cmd .= " $t2";
    } ## end else [ if ( $item eq "trendshpool-max")]
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";
  } ## end else [ if ( $lpar !~ m/^SharedPool0$/)]

  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_trend_shpool

sub graph_poolagg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \" ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $line_items  = 0;                  # how many items in the legend per a line (default 2, when detail then 3)

  if ( $detail == 1 ) {
    $line_items = 2;
  }

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_poolagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $header = "LPARs in POOL aggregated :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "LPARs in POOL aggregated: $server :$last_txt$text";
  }

  # find shpool alias
  my $lpar_pool_alias = $lpar;
  $lpar_pool_alias =~ s/SharedPool/CPU pool /g;

  # add CPU pool alias into png header (if exists and it is not a default CPU pool)
  my $pool_id = $lpar;
  $pool_id =~ s/SharedPool//g;

  if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt" ) {
    open( FR, "< $wrkdir/$server/$host/cpu-pools-mapping.txt" );
    foreach my $linep (<FR>) {
      chomp($linep);
      ( my $id, my $pool_name ) = split( /,/, $linep );
      if ( $id == $pool_id ) {
        $lpar_pool_alias = "$pool_name";
        last;
      }
    } ## end foreach my $linep (<FR>)
    close(FR);
  } ## end if ( -f "$wrkdir/$server/$host/cpu-pools-mapping.txt")

  if ( $detail == 2 ) {
    # $header = "$server agg:$last_txt$text";
    $header = "$lpar_pool_alias agg:$last_txt$text";
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "";            # just a trick
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "LPARs in POOL aggregated: $server : $start_human - $end_human";
      if ( $detail == 10 ) {
        $header = "LPARs aggregated: $server $lpar_pool_alias : $start_human - $end_human";
      }
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my $i   = 0;
  my $cmd = "";
  my $j   = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  #  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  # my $cmd_out = "graph \"no_picture\"";

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  if ( $detail == 1 ) {
    $cmd .= " COMMENT:\"Utilization in CPU cores\\:    average  maximum                               average  maximum                               average  maximum\\l\"";
  }
  else {
    $cmd .= " COMMENT:\"Utilization in CPU cores\\:    average  maximum\\l\"";
  }

  # select pool and its lpars
  $pool_id = $lpar;
  $pool_id =~ s/SharedPool//g;
  if ( isdigit($pool_id) == 0 ) {
    error( "CPU pools id is not a digit: $pool_id ($lpar) " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # read CPU pool cfg from : data/p710/sdmc/cpu.cfg
  # lpar_name=demo,lpar_id=7,curr_shared_proc_pool_id=1,......
  open( POOLIN, "< $wrkdir/$server/$host/cpu.cfg" ) || error( "Cannot open for reading $wrkdir/$server/$host/cpu.cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;

  #`echo " 01 $wrkdir/$server/$host/cpu.cfg" >> /tmp/x333`;
  my $file_index = 0;
  my @files      = "";
  while ( my $line = <POOLIN> ) {

    # find our lpar list
    chomp($line);
    ( my $lpar_name, my $lpar_id, my $pool_id_act ) = split( /,/, $line );
    $lpar_name =~ s/^lpar_name=//;
    $lpar_name =~ s/\//&&1/g;
    $pool_id_act =~ s/^curr_shared_proc_pool_id=//;
    if ( isdigit($pool_id_act) == 1 && $pool_id == $pool_id_act && -f "$wrkdir/$server/$host/$lpar_name.rr$type_sam" ) {

      #`echo " 03 - $lpar_name.rr$type_sam : $pool_id == $pool_id_act " >> /tmp/x333`;
      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$lpar_name.rr$type_sam") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
      $files[$file_index] = "$lpar_name.rr$type_sam";
      $file_index++;

      #`echo " 04 - $lpar_name.rr$type_sam : $pool_id == $pool_id_act " >> /tmp/x333`;
    } ## end if ( isdigit($pool_id_act...))
  } ## end while ( my $line = <POOLIN>)
  close POOLIN;

  # there can be 0 found lpars, no problem at all ... not all pools have necessary lpars

  my $gtype      = "AREA";
  my $color_indx = 0;
  my $line_indx  = 0;        # place enter every 3rd line
  foreach my $file (@files) {
    chomp($file);
    if ( $file eq '' ) {
      next;                  # must be here for some reason when there is not any lpar found
    }

    $lpar = $file;
    $lpar =~ s/.rrh//;
    $lpar =~ s/.rrm//;
    $lpar =~ s/.rrd//;
    my $lpar_space_proc = $lpar;
    my $lpar_space      = $lpar;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for ( my $k = length($lpar_space); $k < 25; $k++ ) {
      $lpar_space .= " ";
    }

    # Exclude pools and memory
    if ( $lpar =~ m/^pool$/ || $lpar =~ m/^mem$/ || $lpar =~ /^SharedPool[0-9]$/ || $lpar =~ m/^SharedPool[1-9][0-9]$/ || $lpar =~ m/^mem-pool$/ || $lpar =~ m/^cod$/ ) {
      next;
    }

    #print "$found $lpar $color[$found]\n";
    #print "$wrkdir/$server/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3);

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%
    my $wrkdir_server_host_file = "$wrkdir/$server/$host/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $wrkdir_server_host_file;
    $wrkdir_server_host_file_legend =~ s/%/%%/g;

    # bulid RRDTool cmd
    $cmd .= " DEF:cap_peak${i}=\"$wrkdir_server_host_file\":capped_cycles:AVERAGE";
    $cmd .= " DEF:uncap${i}=\"$wrkdir_server_host_file\":uncapped_cycles:AVERAGE";
    $cmd .= " DEF:ent${i}=\"$wrkdir_server_host_file\":entitled_cycles:AVERAGE";
    $cmd .= " DEF:cur${i}=\"$wrkdir_server_host_file\":curr_proc_units:AVERAGE";

    # filtering peaks caused by LPM or changing entitled, if cap CPU util is > entitled --> UNKN
    # usualy cap counter is affected only
    # sometimes might happen that even in normal load capped util is little higher than entitled! Therefore using 1.2
    $cmd .= " CDEF:cap${i}=cap_peak${i},ent${i},/,1.2,GT,UNKN,cap_peak${i},IF";


    $cmd .= " CDEF:tot${i}=cap${i},uncap${i},+";
    $cmd .= " CDEF:util${i}=tot${i},ent${i},/";
    $cmd .= " CDEF:utiltotu${i}=util${i},cur${i},*";
    $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";

    $cmd .= " $gtype:utiltot${i}$color[$color_indx]:\"$lpar_space\"";

    $cmd .= " PRINT:utiltot${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_indx] $delimiter $wrkdir_server_host_file_legend\"";
    $cmd .= " PRINT:utiltot${i}:MAX:\" %5.2lf $delimiter \"";

    $color_indx = ( $color_indx + 1 ) % ( $color_max + 1 );

    $cmd .= " GPRINT:utiltot${i}:AVERAGE:\"%5.2lf \"";
    $cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \"";

    #$cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \\l\"";
    # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
    $gtype = "STACK";
    $i++;
    if ( $line_indx == $line_items ) {

      # put carriage return after each second lpar in the legend
      $cmd .= " COMMENT:\"\\l\"";
      $line_indx = 0;
    } ## end if ( $line_indx == $line_items)
    else {
      $line_indx++;
    }
  } ## end foreach my $file (@files)

  $cmd .= " COMMENT:\"\\l\"";

  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph pool agg rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_poolagg

sub graph_hmccount {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT: ";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT: ";
  my $head       = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT: ";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  if ( ( $type !~ m/m/ ) && ( $type !~ m/y/ ) ) {
    error( "Graph rrdtool error : graph_hmccount: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_hmccount: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }

  my $rrx_name = "$wrkdir/$host\.rrx";

  # test if exists database for lpars and servers for hmc
  if ( !-f $rrx_name ) {
    return 0;    # some workaround, it should not go here
  }

  $t = "COMMENT:\\s";
  my $start_rrd = -( 86400 * 365 );
  my $season    = "year";

  if ( $type =~ m/m/ ) {
    RRDp::cmd qq(last "$rrx_name");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t         = "COMMENT:Updated once a day\\: $l ";
    $start_rrd = -( 86400 * 31 );
    $season    = "month";
  } ## end if ( $type =~ m/m/ )
  my $vertical_label = "--vertical-label=\"Quantity\"";
  my $epoch          = "last $season";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
    $epoch          = $season;
  }

  $rrx_name =~ s/:/\\:/g;
  $html_heading = "fake heading";
  my $header = "$host totals : $epoch";
  if ($detail == 9) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }
#$cmd .= 'RRDp::cmd qq($graph_cmd "$name_out"

  # `echo "graph hmccount $host,$server,$lpar,$type,$detail,$width,$height,$font_def,$font_tit," >> /tmp/hmccount`;
  RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_rrd"
        "--end" "now"
        "--imgformat" "PNG"
        "$disable_rrdtool_tag"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "$font_def"
        "$font_tit"
        "$no_legend"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "--alt-autoscale-max"
        "$vertical_label"
        "--units-exponent=1.00"
        "--alt-y-grid"
    "DEF:roundtrip=$rrx_name:servers:MAX"
    "DEF:roundtrip1=$rrx_name:lpars:MAX"
    "DEF:rts=$rrx_name:servers:LAST"
    "DEF:rtl=$rrx_name:lpars:LAST"
    "DEF:rtsa=$rrx_name:servers:AVERAGE"
    "DEF:rtla=$rrx_name:lpars:AVERAGE"
    "COMMENT:          "
    "COMMENT:average  "
    "COMMENT:maximum  "
    "COMMENT:last  \\s"
    "COMMENT:\\s"
    "LINE1:roundtrip#FF0000:servers"
    "GPRINT:rtsa:AVERAGE:%8.0lf"
    "GPRINT:roundtrip:MAX:%9.0lf"
    "GPRINT:rts:LAST:%6.0lf\\s"
    "COMMENT:\\s"
    "LINE1:roundtrip1#0000FF:lpars  "
    "GPRINT:rtla:AVERAGE:%8.0lf"
    "GPRINT:roundtrip1:MAX:%9.0lf"
    "GPRINT:rtl:LAST:%6.0lf\\s"
    "COMMENT:\\s"
    "COMMENT:\\s"
    "$t"
  );
  my $answer = RRDp::read;
  if ( $$answer =~ "ERROR" ) {
    error( "$host:$server Graph rrdtool hmccount error : $$answer " . __FILE__ . ":" . __LINE__ );
  }

  return 0;
} ## end sub graph_hmccount

sub graph_memalloc {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;

  my $t = "COMMENT: ";
  if ( $item eq "clustmem" ) {
    $t = "COMMENT:\" \"";
  }
  my $t2               = "COMMENT:\\n";
  my $step_new         = $step;
  my $last             = "COMMENT: ";
  my $head             = "COMMENT:                            Average   \\n";
  my $dsmem_ident      = "";                                                    # ds name for hist rep
  my $rpool_name_ident = "";                                                    # rp name for hist rep

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph Memory Alloc: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                                                                   # some workaround, it should not go here
  }

  my $rrd = "$wrkdir/$server/$host/mem.rr$type_sam";
  my $file_pth = "$wrkdir/$server/*/mem.rr$type_sam";
  my ( $result_cmd, $curr_avail_mem, $sys_firmware_mem, $conf_sys_mem ) = LPM_easy( $file_pth, "0", "0", "curr_avail_mem", "sys_firmware_mem", "conf_sys_mem");

  # test if exists database for lpars and servers for hmc

  my $server_vmw_name;
  if ($vmware) {
    $rrd = "$wrkdir/$server/$host/pool.rrm";

    if ( ( $item eq "clustmem" ) || ( $item eq "clustpow" ) ) {
      ( $rrd, $server_vmw_name ) = find_human_cluster_name();
      if ( !-f "$rrd" ) {
        error( "graph_:memalloc no such cluster $wrkdir/$server/$host/cluster.rrc " . __FILE__ . ":" . __LINE__ );
        return;
      }
      if ( $server_vmw_name ne "" ) {
        $text = "$text : $server_vmw_name";
      }
    } ## end if ( ( $item eq "clustmem"...))
    elsif ( $item eq "rpmem" ) {

      # $rrd= "$wrkdir/$server/$host/Resources.rrc";
      $rrd = "$wrkdir/$server/$host/$lpar.rrc";
      ( undef, $server_vmw_name ) = find_human_cluster_name();
      $rpool_name_ident = $server_vmw_name . ": " . find_human_rpool_name($lpar);
    } ## end elsif ( $item eq "rpmem" )
    elsif ( $item eq "dsmem" ) {
      $rrd = "$wrkdir/$server/$host/$lpar.rrs";
      $dsmem_ident = find_human_ds_name( $server, $host, $lpar );
    }

    if ( !-f $rrd ) {
      error( "$rrd does not exists (vmware) " . __FILE__ . ":" . __LINE__ );
      return 0;    # some workaround, it should not go here
    }
  } ## end if ($vmware)
  elsif ($hyperv) {
    if ($item =~ /lfd_cat_/) {
      my $lfd_disk = $item;
      $lfd_disk =~ s/lfd_cat_//;
      $lfd_disk =~ s/\%3A/:/;           # can come when zoom
      $rrd = "$wrkdir/$server/$host/Local_Fixed_Disk_$lfd_disk.rrm";
      # print STDERR "10587 \$rrd $rrd\n";
    }
    elsif ($item =~ /csv_cat_/) {
      my $lfd_disk = $item;
      print STDERR "12651 \$item $item\n";
      #$lfd_disk =~ s/csv_cat_CSV//;
      $lfd_disk =~ s/csv_cat_//;
      #$rrd = "$wrkdir/$server/$host/Cluster_Storage_Volume$lfd_disk.rrm";
      $rrd = "$wrkdir/$server/$host/Cluster_Storage_$lfd_disk.rrm";
    }
    else {
      $rrd = "$wrkdir/$server/$host/pool.rrm";
    }
    if ( !-f $rrd ) {
      error( "$rrd does not exists (hyperv) " . __FILE__ . ":" . __LINE__ );
      return 0;    # some workaround, it should not go here
    }
  } ## end elsif ($hyperv)
  elsif ($hitachi) {
    $rrd = "$wrkdir/$server/$host/SYS-MEM.hrm";
    if ( !-f $rrd ) {
      error( "$rrd does not exists (hitachi) " . __FILE__ . ":" . __LINE__ );
      return 0;    # some workaround, it should not go here
    }
  }
  else {           # POWER server
    if ( !-f $rrd ) {
      error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
      return 0;    # some workaround, it should not go here
    }

  } ## end else [ if ($vmware) ]

  # do not update charts if there is not new data in RRD DB
  #    if ( -f "$rrd" ) {
  #      $rrd_time = (stat("$rrd"))[9];
  #      if ( ($act_time - $rrd_time) > $DELETED_LPARS_TIME ) { #10 days
  #         #error ("----$rrd");
  #        return 0;
  #      }
  #   }

  my $header         = "Memory usage: last $text";
  my $vertical_label = "--vertical-label=Memory in GBytes";
  if ( $item eq "clustmem" || $item eq "rpmem" || $item =~ "lfd_cat" || $item =~ "csv_cat" ) {
    $vertical_label = "--vertical-label=Memory in Bytes";
  }
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";          # nope string
    $header         = "Memory usage: $text";
  }
  if ( $item eq "clustpow" ) {
    $header         = "Power usage: last $text";
    $vertical_label = "--vertical-label=Power in Watts";
    if ( $detail == 2 ) {
      $vertical_label = "--interlaced";         # nope string
      $header         = "Power usage: $text";
    }
  } ## end if ( $item eq "clustpow")
  if ( $item eq "dsmem" || $item =~ "hdisk_Disk") {
    $header         = "Space usage: last $text";
    $vertical_label = "--vertical-label=Space in Bytes";
    if ( $detail == 2 ) {
      $vertical_label = "--interlaced";         # nope string
      $header         = "Space usage: $text";
    }
  } ## end if ( $item eq "dsmem" )
  if ( $hitachi ) {
    $vertical_label = "--vertical-label=Memory usage %";
    if ( $detail == 2 ) {
      $vertical_label = "--interlaced";         # nope string
      $header         = "Space usage: $text";
    }
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "COMMENT: ";                # just a trick to place there something otherwise rrdtool complains
        if ( $item eq "clustmem" ) {
          $xgrid = "COMMENT:\" \"";
        }
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);

      # $header = "Memory usage $text: $start_human - $end_human";
      $header = "Memory usage: $start_human - $end_human";
      $header = "Space usage:$dsmem_ident $start_human - $end_human" if ( $item eq "dsmem" || $item =~ "hdisk_Disk" );
      $header = "Space usage:$rpool_name_ident $start_human - $end_human" if ( $item eq "rpmem" );
      $header = "Power usage $text: $start_human - $end_human" if ( $item eq "clustpow" );
      if ( $detail == 10 ) {
        $header = "$server : Memory usage: $start_human - $end_human";
        $header = "Space usage:$dsmem_ident $start_human - $end_human" if ( $item eq "dsmem" || $item =~ "hdisk_Disk" );
        $header = "Space usage:$rpool_name_ident $start_human - $end_human" if ( $item eq "rpmem" );
        $header = "Power usage $text: $start_human - $end_human" if ( $item eq "clustpow" );
      }
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  # print "creating graph : $host:$server:$lpar:$type_sam:$type\n" if $DEBUG ;
  #print "Graph mem  : $lpar\n";
  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt = RRDp::read;
    my $l       = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\: $l ";
    if ( $item eq "clustmem" ) {
      $t = "COMMENT:Updated\\:\" $l \"";
    }
  } ## end if ( $type =~ "d" )

  #if ( $type =~ "y" ) {
  #    $step_new=86400;
  #}

  $rrd =~ s/:/\\:/g;

  #      "DEF:free=$rrd:curr_avail_mem:AVERAGE:start=1440030000:end=1440100000:start=1440110000:end=1440120000"
  #      "DEF:fw=$rrd:sys_firmware_mem:AVERAGE:start=1440030000:end=1440100000:start=1440110000:end=1440120000"
  #      "DEF:tot=$rrd:conf_sys_mem:AVERAGE:start=1440030000:end=1440100000:start=1440110000:end=1440120000"

  if ($hyperv) {
    if ($item =~ /lfd_cat_/ || $item =~ /csv_cat_/) {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "$xgrid"
      "DEF:freeb=$rrd:FreeSpace:AVERAGE"
      "DEF:totb=$rrd:Size:AVERAGE"
      "CDEF:grantb=totb,freeb,-"
      "CDEF:tott=totb,1024,/,1024,/,1024,/,1024,/"
      "CDEF:grantt=grantb,1024,/,1024,/,1024,/,1024,/"
      "COMMENT:   TB                 Avg       Max\\n"
      "AREA:totb#00FF00: Total          "
      "GPRINT:tott:AVERAGE: %6.2lf"
      "GPRINT:tott:MAX: %6.2lf"
      "$t2"
      "AREA:grantb#0080FF: Used           "
      "GPRINT:grantt:AVERAGE: %6.2lf"
      "GPRINT:grantt:MAX: %6.2lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    }
    else {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "$xgrid"
      "DEF:free=$rrd:AvailableMBytes:AVERAGE"
      "DEF:tot=$rrd:TotalPhysMemory:AVERAGE"
      "DEF:cachebytes=$rrd:CacheBytes:AVERAGE"
      "CDEF:freeg=free,1024,/"
      "CDEF:totg=tot,1024,/,1024,/,1024,/"
      "CDEF:cacheg=cachebytes,1024,/,1024,/,1024,/"
      "CDEF:used_comp=totg,freeg,-"
      "CDEF:used=used_comp,0,LE,0,used_comp,IF"
      "CDEF:freegc=freeg,cacheg,-"
      "COMMENT:   [GiB]              Avrg     Max\\n"
      "AREA:used#FF4040: Used memory  "
      "GPRINT:used:AVERAGE:    %6.2lf"
      "GPRINT:used:MAX: %6.2lf"
      "$t2"
      "STACK:cacheg#0080FF: Cache        "
      "GPRINT:cacheg:AVERAGE:    %6.2lf"
      "GPRINT:cacheg:MAX: %6.2lf"
      "$t2"
      "STACK:freegc#00FF00: Free memory    "
      "GPRINT:freegc:AVERAGE:  %6.2lf"
      "GPRINT:freegc:MAX: %6.2lf"
      "$t2"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
    }
  } ## end if ($hyperv)
  elsif ($hitachi) {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--upper-limit=100"
      "--base=1024"
      "$xgrid"
      "DEF:usedp=$rrd:mem_usedp:AVERAGE"
      "COMMENT:[%]   \\n"
      "LINE1:usedp#FF0000: Usage"
      "GPRINT:usedp:AVERAGE: %4.2lf"
      "$t2"
      "$t"
    );
  } ## end ($hitachi)
  elsif ( !$vmware ) {
    my $cmd;
    $cmd .= "$graph_cmd \"$name_out\"
      \"--title\" \"$header\"
      \"--start\" \"$start_time\"
      \"--end\" \"$end_time\"
      \"--imgformat\" \"PNG\"
      \"$disable_rrdtool_tag\"
      \"--slope-mode\"
      \"--width=$width\"
      \"--height=$height\"
      \"$font_def\"
      \"$font_tit\"
      \"$no_legend\"
      \"--step=$step_new\"
      \"--lower-limit=0\"
      \"--color=BACK#$pic_col\"
      \"--color=SHADEA#$pic_col\"
      \"--color=SHADEB#$pic_col\"
      \"--color=CANVAS#$pic_col\"
      \"$vertical_label\"
      \"--upper-limit=1\"
      \"--alt-autoscale-max\"
      \"--base=1024\"
      \"$xgrid\"";
      $cmd .= $result_cmd;
      $cmd .= "
      \"CDEF:free=$curr_avail_mem\"
      \"CDEF:fw=$sys_firmware_mem\"
      \"CDEF:tot=$conf_sys_mem\"
      \"CDEF:freeg=free,1024,/\"
      \"CDEF:fwg=fw,1024,/\"
      \"CDEF:totg=tot,1024,/\"
      \"CDEF:used=totg,freeg,-\"
      \"CDEF:used1=used,fwg,-\"
      \"COMMENT:   Average   \\n\"
      \"AREA:fwg#0080FF: Firmware memory\"
      \"GPRINT:fwg:AVERAGE: %4.2lf GB\"
      \"$t2\"
      \"STACK:used1#FF4040: Used memory    \"
      \"GPRINT:used1:AVERAGE: %4.2lf GB\"
      \"$t2\"
      \"STACK:freeg#00FF00: Free memory    \"
      \"GPRINT:freeg:AVERAGE: %4.2lf GB\"
      \"$t2\"
      \"$t\"
      \"HRULE:0#000000\"
      \"VRULE:0#000000\"
    ";
    RRDp::cmd qq($cmd);
  } ## end elsif ( !$vmware )
  elsif ( $item eq "rpmem" ) {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "$xgrid"
      "DEF:granted=$rrd:Memory_granted_KB:AVERAGE"
      "DEF:active=$rrd:Memory_active_KB:AVERAGE"
      "DEF:consumed=$rrd:Memory_consumed_KB:AVERAGE"
      "DEF:balloon=$rrd:Memory_baloon_KB:AVERAGE"
      "DEF:swap=$rrd:Memory_swap_KB:AVERAGE"
      "DEF:reservation=$rrd:Memory_reservation:AVERAGE"
      "DEF:limit=$rrd:Memory_limit:AVERAGE"
      "CDEF:grantb=granted,1024,*"
      "CDEF:activeb=active,1024,*"
      "CDEF:consumb=consumed,1024,*"
      "CDEF:balloonb=balloon,1024,*"
      "CDEF:swapb=swap,1024,*"
      "CDEF:reservationb=reservation,1024,*,1024,*"
      "CDEF:limitb=limit,1024,*,1024,*"
      "CDEF:grantg=granted,1024,/,1024,/"
      "CDEF:activeg=active,1024,/,1024,/"
      "CDEF:consumg=consumed,1024,/,1024,/"
      "CDEF:balloong=balloon,1024,/,1024,/"
      "CDEF:swapg=swap,1024,/,1024,/"
      "CDEF:reservationg=reservation,1024,/"
      "CDEF:limitg=limit,1024,/"
      "COMMENT:   GB                 Avg       Max\\n"
      "AREA:reservationb#00FF00: Reservation    "
      "GPRINT:reservationg:AVERAGE: %6.1lf"
      "GPRINT:reservationg:MAX: %6.1lf"
      "$t2"
      "LINE:grantb#0080FF: Granted        "
      "GPRINT:grantg:AVERAGE: %6.1lf"
      "GPRINT:grantg:MAX: %6.1lf"
      "$t2"
      "LINE:consumb#00FFFF: Consumed       "
      "GPRINT:consumg:AVERAGE: %6.1lf"
      "GPRINT:consumg:MAX: %6.1lf"
      "$t2"
      "LINE:activeb#FF0000: Active         "
      "GPRINT:activeg:AVERAGE: %6.1lf"
      "GPRINT:activeg:MAX: %6.1lf"
      "$t2"
      "LINE:balloonb#FFFF00: Balloon        "
      "GPRINT:balloong:AVERAGE: %6.1lf"
      "GPRINT:balloong:MAX: %6.1lf"
      "$t2"
      "LINE:swapb#FFA000: Swap out       "
      "GPRINT:swapg:AVERAGE: %6.1lf"
      "GPRINT:swapg:MAX: %6.1lf"
      "$t2"
      "LINE:limitb#000000: Limit          "
      "GPRINT:limitg:AVERAGE: %6.1lf"
      "GPRINT:limitg:MAX: %6.1lf (0 means Unlimited)"
      "$t2"
      "$t"
      "HRULE:0#000000"
	  "VRULE:0#000000"
	);
  } ## end elsif ( $item eq "rpmem" )
  elsif ( $item eq "dsmem" ) {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
	  "--start" "$start_time"
	  "--end" "$end_time"
	  "--imgformat" "PNG"
	  "$disable_rrdtool_tag"
	  "--slope-mode"
	  "--width=$width"
	  "--height=$height"
	  "$font_def"
	  "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "$xgrid"
      "DEF:total=$rrd:Disk_capacity:AVERAGE"
      "DEF:granted=$rrd:Disk_used:AVERAGE"
      "DEF:active=$rrd:Disk_provisioned:AVERAGE"
      "CDEF:grantb=granted,1024,*"
      "CDEF:activeb=active,1024,*"
      "CDEF:totb=total,1024,*"
      "CDEF:grantt=granted,1024,/,1024,/,1024,/"
      "CDEF:activet=active,1024,/,1024,/,1024,/"
      "CDEF:tott=total,1024,/,1024,/,1024,/"
      "COMMENT:   TB                 Avg       Max\\n"
      "AREA:totb#00FF00: Total          "
      "GPRINT:tott:AVERAGE: %6.2lf"
      "GPRINT:tott:MAX: %6.2lf"
      "$t2"
      "AREA:grantb#0080FF: Used           "
      "GPRINT:grantt:AVERAGE: %6.2lf"
      "GPRINT:grantt:MAX: %6.2lf"
      "$t2"
      "LINE:activeb#FF0000: Provisioned    "
      "GPRINT:activet:AVERAGE: %6.2lf"
      "GPRINT:activet:MAX: %6.2lf"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );
  } ## end elsif ( $item eq "dsmem" )
  elsif ( $item eq "clustmem" ) {

    # for Total MEM create sum of all ESXi CPU pool GB (Memory_Host_Size) in this cluster
    my $cmd_x            = "";
    my $hosts_in_cluster = $rrd;
    $hosts_in_cluster =~ s/cluster.rrc/hosts_in_cluster/;
    open( FF, "<$hosts_in_cluster" ) || error( "can't open $hosts_in_cluster: $! :" . __FILE__ . ":" . __LINE__ ) && return;
    my @hosts = (<FF>);
    close(FF);

    # print STDERR "10206 \@hosts @hosts\n"; # e.g. 10.22.11.8XORUX10.22.11.10
    my $i       = 0;
    my $rpn_sum = "";
    foreach my $one_host (@hosts) {
      chomp $one_host;
      ( my $host, my $cluster ) = split( "XORUX", $one_host );

      # print STDERR "10212 testing $wrkdir/$host/$cluster/pool.rrm\n";
      if ( -f "$wrkdir/$host/$cluster/pool.rrm" ) {
        $cluster =~ s/:/\\:/;    # if there is port
        $cmd_x .= " DEF:hosts_mem${i}=\"$wrkdir/$host/$cluster/pool.rrm\":Memory_Host_Size:AVERAGE";
        if ( $i == 0 ) {
          $rpn_sum = "hosts_mem0";
        }
        else {
          $rpn_sum .= ",hosts_mem${i},+";
        }
        $i++;
      } ## end if ( -f "$wrkdir/$host/$cluster/pool.rrm")
    } ## end foreach my $one_host (@hosts)
    $i > 0 || error( "Cluster has no ESXi servers in $hosts_in_cluster: $! :" . __FILE__ . ":" . __LINE__ ) && return;

    # print STDERR "10226 \$rpn_sum $rpn_sum\n";
    $cmd_x .= " CDEF:hosts_mem=$rpn_sum,1,/";
    $cmd_x .= " CDEF:hosts_memg=$rpn_sum,1024,/,1024,/,1024,/";
    my $cmd = "$graph_cmd \"$name_out\"";
    $cmd .= " --title \"$header\"";
    $cmd .= " --start $start_time";
    $cmd .= " --end $end_time";
    $cmd .= " --imgformat PNG";
    $cmd .= " $disable_rrdtool_tag";
    $cmd .= " --slope-mode";
    $cmd .= " --width=$width";
    $cmd .= " --height=$height";
    $cmd .= " $font_def";
    $cmd .= " $font_tit";
    $cmd .= " $no_legend";
    $cmd .= " --step=$step_new";
    $cmd .= " --lower-limit=0";
    $cmd .= " --color=BACK#$pic_col";
    $cmd .= " --color=SHADEA#$pic_col";
    $cmd .= " --color=SHADEB#$pic_col";
    $cmd .= " --color=CANVAS#$pic_col";
    $cmd .= " \"$vertical_label\"";
    $cmd .= " --upper-limit=1";
    $cmd .= " --alt-autoscale-max";
    $cmd .= " --base=1024";
    $cmd .= " $xgrid";
    $cmd .= " $cmd_x";
    $cmd .= " DEF:total=$rrd:Memory_total_MB:AVERAGE";
    $cmd .= " DEF:granted=$rrd:Memory_granted_KB:AVERAGE";
    $cmd .= " DEF:active=$rrd:Memory_active_KB:AVERAGE";
    $cmd .= " DEF:consumed=$rrd:Memory_consumed_KB:AVERAGE";
    $cmd .= " DEF:balloon=$rrd:Memory_baloon_KB:AVERAGE";
    $cmd .= " DEF:swap=$rrd:Memory_swap_KB:AVERAGE";
    $cmd .= " CDEF:grantg=granted,1024,/,1024,/";
    $cmd .= " CDEF:activeg=active,1024,/,1024,/";
    $cmd .= " CDEF:totg=total,1024,/";
    $cmd .= " CDEF:consumg=consumed,1024,/,1024,/";
    $cmd .= " CDEF:balloong=balloon,1024,/,1024,/";
    $cmd .= " CDEF:swapg=swap,1024,/,1024,/";
    $cmd .= " CDEF:grantb=granted,1024,*";
    $cmd .= " CDEF:activeb=active,1024,*";
    $cmd .= " CDEF:totb=total,1024,*,1024,*";
    $cmd .= " CDEF:consumb=consumed,1024,*";
    $cmd .= " CDEF:balloonb=balloon,1024,*";
    $cmd .= " CDEF:swapb=swap,1024,*";
    $cmd .= " COMMENT:\"   GB                 Avg       Max\\n\"";
    $cmd .= " LINE:hosts_mem#000000:\" Total          \"";
    $cmd .= " GPRINT:hosts_memg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:hosts_memg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " AREA:totb#00FF00:\" Total effective\"";
    $cmd .= " GPRINT:totg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:totg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE:grantb#0080FF:\" Granted        \"";
    $cmd .= " GPRINT:grantg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:grantg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE:consumb#00FFFF:\" Consumed       \"";
    $cmd .= " GPRINT:consumg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:consumg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE:activeb#FF0000:\" Active         \"";
    $cmd .= " GPRINT:activeg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:activeg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE:balloonb#FFFF00:\" Balloon        \"";
    $cmd .= " GPRINT:balloong:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:balloong:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE:swapb#FFA000:\" Swap out       \"";
    $cmd .= " GPRINT:swapg:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:swapg:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " $t";
    $cmd .= " HRULE:0#000000";
    $cmd .= " VRULE:0#000000";

    #    ); #00FFFF AEA79F 77216F
    # print STDERR "10305 $cmd\n";
    RRDp::cmd qq($cmd);

  } ## end elsif ( $item eq "clustmem")
  elsif ( $item eq "clustpow" ) {
    RRDp::cmd qq($graph_cmd "$name_out"
      "--title" "$header"
      "--start" "$start_time"
      "--end" "$end_time"
      "--imgformat" "PNG"
      "$disable_rrdtool_tag"
      "--slope-mode"
      "--width=$width"
      "--height=$height"
      "$font_def"
      "$font_tit"
      "$no_legend"
      "--step=$step_new"
      "--lower-limit=0"
      "--color=BACK#$pic_col"
      "--color=SHADEA#$pic_col"
      "--color=SHADEB#$pic_col"
      "--color=CANVAS#$pic_col"
      "$vertical_label"
      "--upper-limit=1"
      "--alt-autoscale-max"
      "--base=1024"
      "$xgrid"
      "DEF:totg=$rrd:Power_cup_Watt:AVERAGE"
      "DEF:activeg=$rrd:Power_usage_Watt:AVERAGE"
      "CDEF:totgk=totg,1000,/"
      "CDEF:activegk=activeg,1000,/"
      "COMMENT:   Average   \\n"
      "AREA:totg#00FF00: Power Cup      "
      "GPRINT:totgk:AVERAGE: %4.1lf kWatt"
      "$t2"
      "LINE:activeg#FF0000: Power Actual   "
      "GPRINT:activegk:AVERAGE: %4.1lf kWatt"
      "$t2"
      "$t"
      "HRULE:0#000000"
      "VRULE:0#000000"
    );    #  "CDEF:activeg=active,1024,/,1024,/"
          #  "CDEF:totg=total,1024,/,1024,/,1024,/"
  } ## end elsif ( $item eq "clustpow")

  else {    #server mem vmware
    my $form_dec = "%6.1lf";
    RRDp::cmd qq($graph_cmd "$name_out"
        "--title" "$header"
        "--start" "$start_time"
        "--end" "$end_time"
        "--imgformat" "PNG"
        "$disable_rrdtool_tag"
        "--slope-mode"
        "--width=$width"
        "--height=$height"
        "$font_def"
        "$font_tit"
        "$no_legend"
        "--step=$step_new"
        "--lower-limit=0"
        "--color=BACK#$pic_col"
        "--color=SHADEA#$pic_col"
        "--color=SHADEB#$pic_col"
        "--color=CANVAS#$pic_col"
        "$vertical_label"
        "--upper-limit=1"
        "--alt-autoscale-max"
        "--base=1024"
        "$xgrid"
        "DEF:total=$rrd:Memory_Host_Size:AVERAGE"
        "DEF:baloon=$rrd:Memory_baloon:AVERAGE"
        "DEF:granted=$rrd:Memory_granted:AVERAGE"
        "DEF:active=$rrd:Memory_active:AVERAGE"
        "CDEF:grantg=granted,1024,/,1024,/"
        "CDEF:activeg=active,1024,/,1024,/"
        "CDEF:baloong=baloon,1024,/,1024,/"
        "CDEF:totg=total,1024,/,1024,/,1024,/"
        "COMMENT:    GB                 Avrg     Max\\n"
        "AREA:totg#00FF00: Total          "
        "GPRINT:totg:AVERAGE: $form_dec"
        "GPRINT:totg:MAX: $form_dec"
        "$t2"
        "AREA:grantg#0080FF: Granted        "
        "GPRINT:grantg:AVERAGE: $form_dec"
        "GPRINT:grantg:MAX: $form_dec"
        "$t2"
        "LINE:activeg#FF0000: Active         "
        "GPRINT:activeg:AVERAGE: $form_dec"
        "GPRINT:activeg:MAX: $form_dec"
        "$t2"
        "LINE:baloong#FFFF00: Baloon         "
        "GPRINT:baloong:AVERAGE: $form_dec"
        "GPRINT:baloong:MAX: $form_dec"
        "$t2"
        "$t"
        "HRULE:0#000000"
        "VRULE:0#000000"
      );
  } ## end else [ if ($hyperv) ]
  my $answ = RRDp::read;
  if ( $$answ =~ "ERROR" ) {
    error( "$host:$server Graph rrdtool error Memory Alloc : $$answ " . __FILE__ . ":" . __LINE__ );
  }
  $png_end_heading = ret_graph_param($answ);
  return 0;
} ## end sub graph_memalloc

sub graph_trendmemalloc {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $detail   = shift;
  my $t        = "COMMENT: ";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $last     = "COMMENT: ";
  my $head     = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph Memory Alloc: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }

  my $rrd     = "$wrkdir/$server/$host/mem.rr$type_sam";
  my $rrd_lpm = "$wrkdir/$server/*/mem.rr$type_sam";

  if ( $hitachi ) {
    $rrd = "$wrkdir/$server/$host/SYS-MEM.hrm";
  }

  # test if exists database for lpars and servers for hmc
  if ( !-f $rrd ) {
    return 0;    # some workaround, it should not go here
  }
  my $header = "Memory usage: last $text";

  #print "creating graph : $host:$server:$lpar:$type_sam:$type\n" if $DEBUG ;

  $step_new = 86400;

  #  if ( $type =~ "y" ) {
  # Trend for yearly chart 1y back  + 1y forward
  #    print "creating graph : $host:$server:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
  #  }

  my $vertical_label = "--vertical-label=\"Memory in GBytes\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  elsif ( $hitachi ) {
    $vertical_label = "--vertical-label=\"Memory usage %\"";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  if ( $detail == 10 ) {
    $header = "$server : Memory usage: last $text";
  }

  $rrd =~ s/:/\\:/g;
  my $cmd = "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header - trend\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width_trend";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  if ( $hitachi ) {
    $cmd .= " --upper-limit=100";
    $cmd .= " DEF:usedp=\"$rrd\":mem_usedp:AVERAGE";
    $cmd .= " DEF:usedp_1m=\"$rrd\":mem_usedp:AVERAGE:start=-1m";
    $cmd .= " DEF:usedp_3m=\"$rrd\":mem_usedp:AVERAGE:start=-3m";
    $cmd .= " DEF:usedp_1y=\"$rrd\":mem_usedp:AVERAGE:start=-1y";
    $cmd .= " VDEF:Dd=usedp_1m,LSLSLOPE";
    $cmd .= " VDEF:Hd=usedp_1m,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-1m=usedp_1m,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=usedp_3m,LSLSLOPE";
    $cmd .= " VDEF:Hm=usedp_3m,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-3m=usedp_3m,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=usedp_1y,LSLSLOPE";
    $cmd .= " VDEF:Hy=usedp_1y,LSLINT";
    $cmd .= " CDEF:cpuusagetrend-1y=usedp_1y,POP,Dy,COUNT,*,Hy,+";

    $cmd .= " COMMENT:\"   Average     \\n\"";
    $cmd .= " LINE1:usedp#FF0000:\" Memory usage\"";
    $cmd .= " GPRINT:usedp:AVERAGE:\"    %2.2lf\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-1m#0080FF:\" Memory usage    - last 1 month trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-3m#80FFFF:\" Memory usage    - last 3 months trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:cpuusagetrend-1y#FF8080:\" Memory usage    - last year trend\"";
    $cmd .= " $last";
    $cmd .= " $t2";
  } ## end if ( $hitachi )
  else {
    $rrd_lpm =~ s/:/\\:/g;

    # to use data from all HMCs (not real LPM, LPM is between servers what this does not do
    my ( $result_cmd, $CPU_free_d, $CPU_fw_d, $CPU_tot_d, $CPU_free_m, $CPU_fw_m, $CPU_tot_m, $CPU_free_y, $CPU_fw_y, $CPU_tot_y, $CPU_free, $CPU_fw, $CPU_tot ) = LPM_easy_trend( "$rrd_lpm", "curr_avail_mem", "sys_firmware_mem", "conf_sys_mem" );

    $cmd .= $result_cmd;
    $cmd .= " CDEF:freed=$CPU_free_d";
    $cmd .= " CDEF:freem=$CPU_free_m";
    $cmd .= " CDEF:freey=$CPU_free_y";
    $cmd .= " CDEF:free=$CPU_free";

    $cmd .= " CDEF:fwd=$CPU_fw_d";
    $cmd .= " CDEF:fwm=$CPU_fw_m";
    $cmd .= " CDEF:fwy=$CPU_fw_y";
    $cmd .= " CDEF:fw=$CPU_fw";

    $cmd .= " CDEF:totd=$CPU_tot_d";
    $cmd .= " CDEF:totm=$CPU_tot_m";
    $cmd .= " CDEF:toty=$CPU_tot_y";
    $cmd .= " CDEF:tot=$CPU_tot";

    $cmd .= " CDEF:freegd=freed,1024,/";
    $cmd .= " CDEF:freegm=freem,1024,/";
    $cmd .= " CDEF:freegy=freey,1024,/";
    $cmd .= " CDEF:freeg=free,1024,/";
    $cmd .= " CDEF:fwgd=fwd,1024,/";
    $cmd .= " CDEF:fwgm=fwm,1024,/";
    $cmd .= " CDEF:fwgy=fwy,1024,/";
    $cmd .= " CDEF:fwg=fw,1024,/";
    $cmd .= " CDEF:totgd=totd,1024,/";
    $cmd .= " CDEF:totgm=totm,1024,/";
    $cmd .= " CDEF:totgy=toty,1024,/";
    $cmd .= " CDEF:totg=tot,1024,/";
    $cmd .= " CDEF:usedd=totgd,freegd,-";
    $cmd .= " CDEF:usedm=totgm,freegm,-";
    $cmd .= " CDEF:usedy=totgy,freegy,-";
    $cmd .= " CDEF:used=totg,freeg,-";
    $cmd .= " CDEF:used1d=usedd,fwgd,-";
    $cmd .= " CDEF:used1m=usedm,fwgm,-";
    $cmd .= " CDEF:used1y=usedy,fwgy,-";
    $cmd .= " CDEF:used1=used,fwg,-";
    $cmd .= " CDEF:usedtotd=used1d,fwgd,+";
    $cmd .= " CDEF:usedtotm=used1m,fwgm,+";
    $cmd .= " CDEF:usedtoty=used1y,fwgy,+";
    $cmd .= " CDEF:usedtot=used1,fwg,+";
    $cmd .= " VDEF:Dd=usedtotd,LSLSLOPE";
    $cmd .= " VDEF:Hd=usedtotd,LSLINT";
    $cmd .= " CDEF:usedtottrendd=usedtotd,POP,Dd,COUNT,*,Hd,+";
    $cmd .= " VDEF:Dm=usedtotm,LSLSLOPE";
    $cmd .= " VDEF:Hm=usedtotm,LSLINT";
    $cmd .= " CDEF:usedtottrendm=usedtotm,POP,Dm,COUNT,*,Hm,+";
    $cmd .= " VDEF:Dy=usedtoty,LSLSLOPE";
    $cmd .= " VDEF:Hy=usedtoty,LSLINT";
    $cmd .= " CDEF:usedtottrendy=usedtoty,POP,Dy,COUNT,*,Hy,+";
    $cmd .= " COMMENT:\"   Average   \\n\"";
    $cmd .= " AREA:fwg#0080FF:\" Firmware memory\"";
    $cmd .= " GPRINT:fwg:AVERAGE:\" %4.2lf GB\"";
    $cmd .= " $t2";
    $cmd .= " STACK:used1#FF8080:\" Used memory    \"";
    $cmd .= " GPRINT:used1:AVERAGE:\" %4.2lf GB\"";
    $cmd .= " $t2";
    $cmd .= " STACK:freeg#00FF00:\" Free memory    \"";
    $cmd .= " GPRINT:freeg:AVERAGE:\" %4.2lf GB\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:usedtottrendd#0000FF:\" Used memory total - last 1 month trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:usedtottrendm#80FFFF:\" Used memory total - last 3 months trend\"";
    $cmd .= " $t2";
    $cmd .= " LINE2:usedtottrendy#FF8080:\" Used memory total - last year trend\"";
    $cmd .= " $t";
  } ## end else [ if ( $hitachi )]

  $cmd .= " HRULE:0#000000";

  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;

} ## end sub graph_trendmemalloc

sub legend_item_format {
  my $atom    = shift;
  my $leg_len = shift;

  # add spaces to lpar name to have $leg_len chars total (for formating graph legend)
  for ( my $k = length($atom); $k < $leg_len; $k++ ) {
    $atom .= " ";
  }

  # max $leg_len chars
  $atom = substr( $atom, 0, $leg_len - 1 );
  return $atom;
} ## end sub legend_item_format

sub graph_memaggreg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $powlin_job  = shift;    # pid number

  #$server =~ s/--unknown$//;
  my $lpar_orig = $lpar;                                                 # for wpar use
  my $t         = "COMMENT: ";
  my $t2        = "COMMENT:\\n";
  my $step_new  = $step;
  my $last      = "COMMENT: ";
  my $head      = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph Memory Aggreg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }
  my $req_time   = 0;
  my $act_time   = localtime();
  my $act_time_u = time();

  my $Datastore_DS_name = "";

  my $header         = "Memory aggregated : last $text";
  my $vertical_label = "--vertical-label=Memory in GBytes";
  if ( $item eq "wpar_cpu" ) {
    $header         = "CPU aggregated : last $text";
    $vertical_label = "--vertical-label=cores";
  }
  if ( $item eq "threads" ) {
    $header         = "Threads aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=Threads";
  }
  if ( $item eq "faults" ) {
    $header         = "Faults aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=Faults";
  }
  if ( $item eq "pages" ) {
    $header         = "Pages aggregated : $lpar :  last $text";
    $vertical_label = "--vertical-label=Pages";
  }
  if ( $item eq "job_cpu" ) {
    $header         = "CPU aggregated % : $lpar : last $text";
    $vertical_label = "--vertical-label=CPU %";
  }
  if ( $item eq "powlin_job" ) {
    my $pid = $powlin_job;
    $pid =~ s/\.cfg$//;
    # $header         = "server $server: JOB $pid: last $text";
    $header         = "$lpar : CPU JOB $pid: last $text";
    $vertical_label = "--vertical-label=CPU cores";
  } ## end if ( $item eq "powlin_job")
  if ( $item eq "powlin_mem" ) {
    my $pid = $powlin_job;
    $pid =~ s/\.cfg$//;
    # $header         = "server $server: JOB $pid: last $text";
    $header         = "$lpar : MEM JOB $pid: last $text";
    $vertical_label = "--vertical-label=Memory MB";
  } ## end if ( $item eq "powlin_mem")
  if ( $item eq "waj" ) {
    $header         = "CPU aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=CPU cores";
  }
  if ( $item eq "jobs" ) {
    $header         = "JOB CPU aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=CPU cores";
  }
  if ( $item eq "jobs_mem" ) {
    $header         = "JOB Memory aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=Memory GB";
  }
  if ( $item eq "disk_io" ) {
    $header         = "IO aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=Disk IO";
  }
  if ( $item eq "disks" ) {
    $header         = "DISK IO aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=Disk IO/sec";
  }
  if ( $item eq "disk_busy" ) {
    $header         = "DISK Busy : $lpar : last $text";
    $vertical_label = "--vertical-label=Disk Busy %";
  }
  if ( $item eq "size" ) {
    $header         = "Memory aggregated : $lpar : last $text";
    $vertical_label = "--vertical-label=GBytes";
  }
  if ( $item eq "dstrag_used" ) {
    $header         = "Datastores USED : last $text";
    $vertical_label = "--vertical-label=USED Space [TB]";
  }
  if ( $item eq "dstrag_iopsr" ) {
    $header            = "Datastores IOPS read : last $text";
    $vertical_label    = "--vertical-label=IOPS read";
    $Datastore_DS_name = "Datastore_ReadAvg";
  }
  if ( $item eq "dstrag_iopsw" ) {
    $header            = "Datastores IOPS write : last $text";
    $vertical_label    = "--vertical-label=IOPS write";
    $Datastore_DS_name = "Datastore_WriteAvg";
  }
  if ( $item eq "dstrag_datar" ) {
    $header            = "Datastores DATA read : last $text";
    $vertical_label    = "--vertical-label=READ MB/sec";
    $Datastore_DS_name = "Datastore_read";
  }
  if ( $item eq "dstrag_dataw" ) {
    $header            = "Datastores DATA write : last $text";
    $vertical_label    = "--vertical-label=WRITE MB/sec";
    $Datastore_DS_name = "Datastore_write";
  }
  if ( $detail == 2 ) {
    my $lpart = ": $lpar ";
    $lpart    = "" if $server eq "as400";
    $vertical_label = "--interlaced";     # nope string
    $header         = "Memory aggregated:$text";
    $header         = "Threads agg$lpart:$text" if ( $item eq "threads" );
    $header         = "Faults agg$lpart:$text" if ( $item eq "faults" );
    $header         = "Pages agg$lpart:$text" if ( $item eq "pages" );
    $header         = "CPU agg %$lpart:$text" if ( $item eq "job_cpu" );
    $header         = "JOB$lpart:$text" if ( $item eq "jobs" );
    $header         = "JOB CPU:$text" if ( $item eq "jobs" && $server eq "Linux" );
    $header         = "CPU agg$lpart:$text" if ( $item eq "waj" );
    $header         = "JOB MEM:$text" if ( $item eq "jobs_mem" && $server eq "Linux" );
    $header         = "MEM agg$lpart:$text" if ( $item eq "size" );
    $header         = "IO agg$lpart:$text" if ( $item eq "disk_io" );
    $header         = "DISK IO agg$lpart:$text" if ( $item eq "disks" );
    $header         = "DISK Busy$lpart:$text" if ( $item eq "disk_busy" );
    $header         = "Datastores IOPS read :$text" if ( $item eq "dstrag_iopsr" );
    $header         = "Datastores IOPS write :$text" if ( $item eq "dstrag_iopsw" );
    $header         = "Datastores DATA read :$text" if ( $item eq "dstrag_datar" );
    $header         = "Datastores DATA write :$text" if ( $item eq "dstrag_dataw" );
    $header         = "Datastores USED :$text" if ( $item eq "dstrag_used" );
    $header         = "CPU job: $text" if ( $item eq "powlin_job" );
    $header         = "Memory job: $text" if ( $item eq "powlin_mem" );
  } ## end if ( $detail == 2 )
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "";            # just a trick
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "Memory aggregated: $start_human - $end_human";
      $header = "Threads aggregated: $start_human - $end_human" if ( $item eq "threads" );
      $header = "Faults aggregated: $start_human - $end_human" if ( $item eq "faults" );
      $header = "Pages aggregated: $start_human - $end_human" if ( $item eq "pages" );
      $header = "CPU aggregated %: $start_human - $end_human" if ( $item eq "job_cpu" );
      $header = "CPU aggregated: $start_human - $end_human" if ( $item eq "waj" || $item eq "jobs" );
      $header = "MEM aggregated: $start_human - $end_human" if ( $item eq "jobs_mem" );
      $header = "IO aggregated: $start_human - $end_human" if ( $item eq "disk_io" );
      $header = "DISK IO aggregated: $start_human - $end_human" if ( $item eq "disks" );
      $header = "DISK Busy : $start_human - $end_human" if ( $item eq "disk_busy" );
      $header = "Datastores USED: $start_human - $end_human" if ( $item eq "dstrag_used" );
      $header = "Datastores IOPS read: $start_human - $end_human" if ( $item eq "dstrag_iopsr" );
      $header = "Datastores IOPS write: $start_human - $end_human" if ( $item eq "dstrag_iopsw" );
      $header = "Datastores DATA read: $start_human - $end_human" if ( $item eq "dstrag_datar" );
      $header = "Datastores DATA write: $start_human - $end_human" if ( $item eq "dstrag_dataw" );
      if ( $detail == 10 ) {
        $header = "$server : Memory aggregated: $start_human - $end_human";
      }
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  my @files        = ();
  my $cmd_def_jobs = "";    # holds RRDTOOL DEF directives for cputopx.mmc
  my $count_files_cputot;   # holds number of these files

  if ($vmware || $hyperv) {

    # prepare list of active registered VMs on server
    # this is not OK, it should be same as lparagg for CPU pool
    # now it takes only currently active VMS, not taking VMs active before !!
    my $registered_VM_file = "$wrkdir/$server/$host/VM_hosting.vmh";

    # print STDERR "------- \$registered_VM_file $registered_VM_file\n";
    open( my $fh, "$registered_VM_file" ) or error( "can't open $registered_VM_file, server has not had any VMs: $!" . __FILE__ . ":" . __LINE__ ) && return;
    while ( my $row = <$fh> ) {
      ( my $vm, undef ) = split( ":", $row );
      $vm .= ".rrm";
      my $pos = rindex( $row, ':' );
      next if $pos < 0;    # some trash
                           # push @files, "$vm"; # these are all active and non active
      if ( substr( $row, $pos + 1, 5 ) eq "start" ) {    # keep item
        push @files, "$vm";
      }
    } ## end while ( my $row = <$fh> )
    close $fh;

    # print STDERR "-------  \@files @files\n";
  } ## end if ($vmware)
  elsif ( $item eq "dstrag_used" ) {
    @files = <$wrkdir/vmware_*/datastore_datacenter*/*.rrs>;
  }
  elsif ( $item =~ "dstrag_" ) {
    @files = <$wrkdir/vmware_*/datastore_datacenter*/*.rrt>;
  }
  elsif ( $item eq "wpar_mem" || $item eq "wpar_cpu" ) {
    opendir( DIR, "$wrkdir/$server/$host/$lpar_orig" ) or error( "directory does not exists : $wrkdir/$server/$host/$lpar_orig " . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_dir = readdir(DIR);
    closedir(DIR);
    my $file_name = "mem";
    $file_name = "cpu" if $item eq "wpar_cpu";
    foreach my $file (@files_dir) {
      next if $file =~ /^\.$/;
      if ( !-d "$wrkdir/$server/$host/$lpar_orig/$file" || !-f "$wrkdir/$server/$host/$lpar_orig/$file/cpu.mmm" ) {
        next;
      }

      #print STDERR "10428 detail-graph-cgi.pl found wpar dir $file\n";
      push @files, "$file/$file_name.mmm";
    } ## end foreach my $file (@files_dir)
  } ## end elsif ( $item eq "wpar_mem"...)
  elsif ( $item eq "jobs" || $item eq "powlin_job" || $item eq "jobs_mem" || $item eq "powlin_mem" ) {
    opendir( DIR, "$wrkdir/$server/$host/$lpar_orig/JOB" ) or error( "directory does not exists : $wrkdir/$server/$host/$lpar_orig/JOB " . __FILE__ . ":" . __LINE__ ) && return 0;
    @files = grep( /\.cfg$/, readdir(DIR) );
    rewinddir(DIR);
    my @files_cpu = grep( /\.mmc$/, readdir(DIR) );
    closedir(DIR);
    $count_files_cputot = scalar @files_cpu;    # for later use
    if ( $item eq "powlin_job" || $item eq "powlin_mem" ) {
      @files = ();
      $files[0] = $powlin_job;
    }

    # print STDERR "10639 detail-graph-cgi.pl $files[0]\n";
    # prepare RRDTOOL DEF directives
    # $cmd_def_jobs
    my $i         = 0;
    my $path_file = "$wrkdir/$server/$host/$lpar_orig";
    $path_file =~ s/:/\\:/g;
    foreach my $file (@files_cpu) {

      # print STDERR "10648 detail-graph-cgi.pl $file\n";
      $cmd_def_jobs .= " DEF:pid${i}=\"$path_file/JOB/$file\":pid:AVERAGE";
      $cmd_def_jobs .= " DEF:td${i}=\"$path_file/JOB/$file\":time_diff:AVERAGE";
      $cmd_def_jobs .= " DEF:rss${i}=\"$path_file/JOB/$file\":rss:AVERAGE";
      $i++;
    } ## end foreach my $file (@files_cpu)

    # print STDERR "$cmd_def_jobs\n";
  } ## end elsif ( $item eq "jobs" ||...)
  else {
    if ( $item eq "size" || $item eq "res" || $item eq "threads" || $item eq "faults" || $item eq "pages" || $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" || $item eq "disks" || $item eq "disk_busy" ) {    # AS400
      my $Parmx = "Parm1";
      $Parmx = "Parm3" if $item eq "threads";
      $Parmx = "Parm2" if $item eq "faults";
      $Parmx = "Parm2" if $item eq "pages";

      if ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" ) {

        # print STDERR "8885 detail-graph-cgi.pl size $wrkdir/$server/$host/$lpar\n";
        opendir( DIR, "$wrkdir/$server/$host/$lpar$AS400/JOB" ) || error( " directory does not exists : $wrkdir/$server/$host/$lpar$AS400/JOB " . __FILE__ . ":" . __LINE__ ) && return 0;

        #opendir(DIR, "$wrkdir/$server/$host/$lpar/JOB") || error(" directory does not exists : $wrkdir/$server/$host/$lpar/job ".__FILE__.":".__LINE__) && return 0;

        my @files_unsorted = grep( /.*\.mmc$/, readdir(DIR) );
        @files = sort { lc $a cmp lc $b } @files_unsorted;

        # print STDERR "9028 detail-graph-cgi.pl \@files @files\n";
      } ## end if ( $item eq "job_cpu"...)
      elsif ( $item eq "disks" || $item eq "disk_busy" ) {
        opendir( DIR, "$wrkdir/$server/$host/$lpar$AS400/DSK" ) || error( " directory does not exists : $wrkdir/$server/$host/$lpar$AS400/DSK " . __FILE__ . ":" . __LINE__ ) && return 0;

        my @files_unsorted = grep( /.*\.mmc$/, readdir(DIR) );
        @files = sort { lc $a cmp lc $b } @files_unsorted;

        # print STDERR "9885 detail-graph-cgi.pl \@files @files\n";
      } ## end elsif ( $item eq "disks" ...)
      else {
        # print STDERR "8885 detail-graph-cgi.pl size $wrkdir/$server/$host/$lpar\n";
        opendir( DIR, "$wrkdir/$server/$host/$lpar$AS400" ) || error( " directory does not exists : $wrkdir/$server/$host/$lpar$AS400 " . __FILE__ . ":" . __LINE__ ) && return 0;
        my @files_unsorted = grep( /S0400[0-9][0-9].*$Parmx\.mmm$/, readdir(DIR) );    # take only S0400[0-9][0-9], leave out older files with direct names
                                                                                       #my @files_unsorted = grep(/S0400.*$Parmx\.mmm$/,readdir(DIR));
        @files = sort { lc $a cmp lc $b } @files_unsorted;

        # print STDERR "8889 detail-graph-cgi.pl \@files @files\n";
      } ## end else [ if ( $item eq "job_cpu"...)]
    } ## end if ( $item eq "size" ||...)
    else {
      opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server/$host " . __FILE__ . ":" . __LINE__ ) && return 0;
      if ( $type_sam =~ "m" ) {
        my @files_unsorted = grep( /\.r[m|s]m$/, readdir(DIR) );

        # @files = sort { lc $a cmp lc $b } @files_unsorted;
        @files = @files_unsorted;
      } ## end if ( $type_sam =~ "m" )
      rewinddir(DIR);
      if ( $type_sam =~ "h" ) {
        my @files_unsorted = grep( /\.r[m|s]h$/, readdir(DIR) );

        #@files = sort { lc $a cmp lc $b } @files_unsorted;
        @files = @files_unsorted;
      } ## end if ( $type_sam =~ "h" )
      rewinddir(DIR);
      if ( $type_sam =~ "d" ) {
        my @files_unsorted = grep( /\.r[m|s]d$/, readdir(DIR) );

        #@files = sort { lc $a cmp lc $b } @files_unsorted;
        @files    = @files_unsorted;
        $req_time = $act_time_u - 31536000;
      } ## end if ( $type_sam =~ "d" )
    } ## end else [ if ( $item eq "size" ||...)]
    closedir(DIR);
  } ## end else [ if ($vmware) ]

  my $file               = "";
  my $i                  = 0;
  my $cmd                = "";
  my $j                  = 0;
  my $cfg_text           = "";
  my $subsystem_name     = "";
  my $unix_cmd           = "";
  my $type_subtype       = "";
  my $status_aj          = "";
  my $function_name_type = "";
  my $proctime_total     = "";
  my $asp_num            = "";

  $status_aj = "";

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  if ( scalar @files > 1000 ) {
    $no_legend = "--no-legend";
  }

  # $step_new = 300;
  # print STDERR "10138 detail-graph-cgi.pl \$step_new $step_new \$item $item\n";

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";

  #  $cmd .= " --upper-limit=0.1";
  $cmd .= " \\\"$vertical_label\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";
  if ($vmware) {
    $cmd .= " COMMENT:\\\"Current granted memory in GB\\:      average\\l\\\"";
  }
  else {
    if ( $item eq "wpar_cpu" ) {
      $cmd .= " COMMENT:\\\"Current CPU load in cores\\:         average     max\\l\\\"";
    }
    elsif ( $item eq "threads" ) {
      $cmd .= " COMMENT:\\\"Current number of threads\\:         average          max\\l\\\"";
    }
    elsif ( $item eq "res" ) {
      $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average          max\\l\\\"";
    }
    elsif ( $item eq "size" ) {
      $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average          max          type\\l\\\"";
    }
    elsif ( $item eq "faults" ) {
      $cmd .= " COMMENT:\\\"Faults \\:                   DB      AVG    MAX   non DB  AVG    MAX\\l\\\"";
    }
    elsif ( $item eq "pages" ) {
      $cmd .= " COMMENT:\\\"Pages \\:                    DB      AVG    MAX   non DB  AVG    MAX\\l\\\"";
    }
    elsif ( $item eq "job_cpu" ) {
      $cmd .= " COMMENT:\\\"Jobname      User       Jobno     Type   AvgRun     Avg      MAX   Function      Status\\l\\\"";
    }
    elsif ( $item eq "jobs" || $item eq "powlin_job" || $item eq "jobs_mem" || $item eq "powlin_mem" ) {
      $cmd .= " COMMENT:\\\"Job PID          User               AvgRun     Avg       MAX    Command\\l\\\"";
    }
    elsif ( $item eq "waj" ) {
      $cmd .= " COMMENT:\\\"Jobname      User       Jobno     Type   AvgRun     Avg      MAX   Function      Status\\l\\\"";
    }
    elsif ( $item eq "disk_io" ) {
      $cmd .= " COMMENT:\\\"Jobname      User       Jobno     Type   AvgRun      Avg       MAX   Function      Status\\l\\\"";
    }
    elsif ( $item eq "disks" || $item eq "disk_busy" ) {
      $cmd .= " COMMENT:\\\"  Disk-serial                ASP       AvgRun       Avg       MAX\\l\\\"";
    }
    elsif ( $item eq "dstrag_used" ) {
      $cmd .= " COMMENT:\\\"  VMware alias    Datastore                             [GB]  AVG       MAX\\l\\\"";
    }
    elsif ( $item eq "dstrag_iopsr" ) {
      $cmd .= " COMMENT:\\\"  VMware alias    Datastore                             READ  AVG       MAX\\l\\\"";
    }
    elsif ( $item eq "dstrag_iopsw" ) {
      $cmd .= " COMMENT:\\\"  VMware alias    Datastore                            WRITE  AVG       MAX\\l\\\"";
    }
    elsif ( $item eq "dstrag_datar" ) {
      $cmd .= " COMMENT:\\\"  VMware alias    Datastore                           MB/sec  AVG       MAX\\l\\\"";
    }
    elsif ( $item eq "dstrag_dataw" ) {
      $cmd .= " COMMENT:\\\"  VMware alias    Datastore                           MB/sec  AVG       MAX\\l\\\"";
    }
    else {
      $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average\\l\\\"";
    }
  } ## end else [ if ($vmware) ]
  my $gtype = "AREA";

  my $wrkdir_managedname_host_mem;
  if ($vmware || $hyperv) {
    $wrkdir_managedname_host_mem = "$wrkdir/$server/$host/pool.rrm";
  }
  else {
    $wrkdir_managedname_host_mem = "$wrkdir/$server/$host/mem.rr$type_sam";
  }
  $wrkdir_managedname_host_mem =~ s/:/\\:/g;

  if ($vmware) {
    $cmd .= " DEF:max=\\\"$wrkdir_managedname_host_mem\\\":Memory_Host_Size:AVERAGE";
    $cmd .= " CDEF:maxg=max,1024,/,1024,/,1024,/";
    $gtype = "AREA";
  }
  elsif ($hyperv) {
    $cmd .= " DEF:max=\\\"$wrkdir_managedname_host_mem\\\":TotalPhysMemory:AVERAGE";
    $cmd .= " CDEF:maxg=max,1024,/,1024,/,1024,/";
    $gtype = "AREA";
  }
  else {
    if ( $item ne "size" && $item ne "res" && $item ne "threads" && $item ne "faults" && $item ne "pages" && $item ne "job_cpu" && $item ne "waj" && $item ne "disk_io" && $item ne "disks" && $item ne "disk_busy" && $item !~ "dstrag_" && $item !~ "wpar_" && $item ne "jobs" && $item ne "powlin_job" && $item ne "jobs_mem" && $item ne "powlin_mem" ) {    # not for AS400 & not for Datastores TOP & not for wpar_
      $cmd .= " DEF:max=\\\"$wrkdir_managedname_host_mem\\\":conf_sys_mem:AVERAGE";
      $cmd .= " CDEF:maxg=max,1024,/";

      $cmd .= " DEF:fw=\\\"$wrkdir_managedname_host_mem\\\":sys_firmware_mem:AVERAGE";
      $cmd .= " CDEF:fwg=fw,1024,/";

      $cmd .= " $gtype:fwg#0080FF:\\\"Firmware                 \\\"";
      $cmd .= " GPRINT:fwg:AVERAGE:\\\"     %6.2lf \\l\\\"";
      $cmd .= " PRINT:fwg:AVERAGE:\"%6.2lf $delimiter $item $delimiter Firmware $delimiter #0080FF\"";
      $cmd .= " COMMENT:\\n";
      $gtype = "STACK";
    } ## end if ( $item ne "size" &&...)
    elsif ( $item eq "jobs" || $item eq "powlin_job" || $item eq "jobs_mem" || $item eq "powlin_mem" ) {
      $cmd .= $cmd_def_jobs;

      # print STDERR "10870 \$cmd $cmd\n";
    }
  } ## end else [ if ($vmware) ]
  my $found_at_least_one = 0;
  my $found              = -1;
  $found = 0 if $item =~ /wpar_/;    # first color is left out for total line
  my $lpar_as400 = $lpar;

  my $form_dec    = "%6.2lf";
  my $last_update = 0;
  my $l           = 0;               # used for jobs

  my $vms_dir = "vmware_VMs";
  $vms_dir = "hyperv_VMs" if $hyperv;

  foreach $file (@files) {
    $found++;
    chomp($file);
    $found_at_least_one = 1;
    if ( $item eq "size" || $item eq "threads" ) {
      $found = substr( $file, 4, 3 ) - 1;    # S040004Parm1.mmm see 004 as color number
    }

    #if ($item eq "faults" || $item eq "pages") {
    #  $found = (substr ($file,4,3) - 1) * 2;
    #}
    # print STDERR "10611 detail-graph-cgi.pl \$file $file \$found $found\n";

    my $rrd_upd_time;

    # avoid old lpars which do not exist in the period
    if ($vmware || $hyperv) {

      # for vmware then in future better test will be created, now all active registered VMs have been taken, some can be poweredoff or suspended
      $rrd_upd_time = ( stat("$wrkdir/$vms_dir/$file") )[9];

      # $found++;
      if ( !defined $rrd_upd_time || $rrd_upd_time < $req_time ) {
        next;
      }
    } ## end if ($vmware)
    else {
      if ( $item eq "size" || $item eq "res" || $item eq "threads" || $item eq "faults" || $item eq "pages" || $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" || $item eq "disks" || $item eq "disk_busy" ) {    # AS400
        # print STDERR "9030 detail-graph-cgi.pl $wrkdir/$server/$host/$lpar$AS400/$file\n";
        if ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" || $item eq "disks" || $item eq "disk_busy" ) {
          my $dir_name = "JOB";
          $dir_name = "DSK" if $item eq "disks" || $item eq "disk_busy";
          $rrd_upd_time = ( stat("$wrkdir/$server/$host/$lpar_as400$AS400/$dir_name/$file") )[9];

          #$rrd_upd_time = (stat("$wrkdir/$server/$host/$lpar_as400/$dir_name/$file"))[9];
          next if ( $rrd_upd_time < $req_time );

          # test if -nan
          # "DEF:cur=$wrkdir/$server/$host/$lpar_as400/$dir_name/$file:par3:AVERAGE"
          my $cmdix = 'RRDp::cmd qq(graph "$name_out"
            "--title" "$header"
            "--start" "$start_time"
            "--end" "$end_time"
            "DEF:cur=$wrkdir/$server/$host/$lpar_as400$AS400/$dir_name/$file:par3:AVERAGE"
            "CDEF:curg=cur,1,/"
            "PRINT:curg:AVERAGE: ""$form_dec"
          )';
          eval $cmdix;
          if ($@) {
            error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
          }
          else {
            my $answ = RRDp::read;
            if ( $$answ =~ "ERROR" ) {
              error( "$host:$server Graph rrdtool error : $$answ " . __FILE__ . ":" . __LINE__ );
              next;
            }
            chomp($$answ);

            # print STDERR "10101 detail-graph-cgi.pl \$answ ,$answ, ,$$answ,\n";
            next if $$answ =~ /[Nn]a[Nn]/;

            # prepare text values
            my $file_cfg = "$wrkdir/$server/$host/$lpar_as400$AS400/$dir_name/$file";

            #my $file_cfg = "$wrkdir/$server/$host/$lpar_as400/$dir_name/$file";
            $file_cfg =~ s/mmc$/cfg/;
            $cfg_text = "";
            if ( -f "$file_cfg" ) {
              open( FILE, "$file_cfg" ) or error( "cant open $file_cfg" . __FILE__ . ":" . __LINE__ ) && next;
              $cfg_text = <FILE>;
              close(FILE);
              if ( $item eq "disks" || $item eq "disk_busy" ) {
                $asp_num = $cfg_text;
                chomp $asp_num;
                $asp_num = "0" . $asp_num if $asp_num < 100;
                $asp_num = "0" . $asp_num if $asp_num < 10;

                # fake pool name
                if ( $asp_num < 33 && $asp_num != 1 ) {
                  $asp_num = "ASP" . $asp_num;
                }
                elsif ( $asp_num == 1 ) {
                  $asp_num = "*SYSTEM";
                }
                elsif ( $asp_num > 32 ) {

                  # take ASP name from ASP config file
                  my $asp_cfg = "$wrkdir/$server/$host/$lpar_as400$AS400/ASP/ASP" . $asp_num . "Parm.cfg";

                  # print STDERR "10313 \$asp_cfg $asp_cfg\n";
                  if ( -f "$asp_cfg" ) {
                    open( FILE, "$asp_cfg" ) or error( "cant open $file_cfg" . __FILE__ . ":" . __LINE__ ) && next;
                    my $cfg_text = <FILE>;
                    close(FILE);
                    ( my $subsystem_name, undef ) = split( /,/, $cfg_text );

                    # print STDERR "\$cfg_text $cfg_text \$subsystem_name $subsystem_name\n";
                    $subsystem_name =~ s/Res name: //;
                    $asp_num = $subsystem_name;
                  } ## end if ( -f "$asp_cfg" )
                } ## end elsif ( $asp_num > 32 )
              } ## end if ( $item eq "disks" ...)
              else {
                #  my $job_config_pattern = "subsystem_name|user_name|type_subtype|status_aj|function_name_type|proctime_total";
                ( $subsystem_name, undef, $type_subtype, $status_aj, $function_name_type, $proctime_total ) = split( /\|/, $cfg_text );
                $proctime_total =~ s/\./\\:/g;
                $subsystem_name = sprintf( "%-15s", $subsystem_name );
                $subsystem_name = substr( $subsystem_name, 0, 15 );
                $type_subtype = sprintf( "%-3s", $type_subtype );
                $type_subtype = substr( $type_subtype, 0, 3 );
                $status_aj = sprintf( "%-4s", $status_aj );
                $status_aj = substr( $status_aj, 0, 4 );
                $function_name_type = sprintf( "%-14s", $function_name_type );
                $function_name_type = substr( $function_name_type, 0, 14 );

                # print STDERR "9241 $subsystem_name, $type_subtype, $status_aj, $function_name_type, $proctime_total\n";
              } ## end else [ if ( $item eq "disks" ...)]
            } ## end if ( -f "$file_cfg" )
          } ## end else [ if ($@) ]
        } ## end if ( $item eq "job_cpu"...)
        else {
          $rrd_upd_time = ( stat("$wrkdir/$server/$host/$lpar_as400$AS400/$file") )[9];

          #$rrd_upd_time = (stat("$wrkdir/$server/$host/$lpar_as400/$file"))[9];
          next if ( $rrd_upd_time < $req_time );
        } ## end else [ if ( $item eq "job_cpu"...)]
      } ## end if ( $item eq "size" ||...)
      elsif ( $item =~ "dstrag_" ) {

        # print STDERR "10439 detail-graph-cgi.pl \$file $file\n";
        next if -s $file < 1;
        $rrd_upd_time = ( stat("$file") )[9];
        if ( $rrd_upd_time < $req_time ) {
          next;
        }
      } ## end elsif ( $item =~ "dstrag_")
      elsif ( $item =~ "wpar_" ) {

        # print STDERR "10736 detail-graph-cgi.pl \$file $wrkdir/$server/$host/$lpar_orig/$file\n";
        $rrd_upd_time = ( stat("$wrkdir/$server/$host/$lpar_orig/$file") )[9];
        if ( $rrd_upd_time < $req_time ) {
          next;
        }
      } ## end elsif ( $item =~ "wpar_" )
      elsif ( $item eq "jobs" || $item eq "powlin_job" || $item eq "jobs_mem" || $item eq "powlin_mem" ) {
        next if !-f "$wrkdir/$server/$host/$lpar_orig/JOB/$file";
        $rrd_upd_time = ( stat("$wrkdir/$server/$host/$lpar_orig/JOB/$file") )[9];
        next if $rrd_upd_time < $req_time;
        if ( $start_time !~ /now/ ) {    # only for zoom
          next if $rrd_upd_time < $start_time;
        }

        # prepare data from cfg for legend
        open( FILE, "$wrkdir/$server/$host/$lpar_orig/JOB/$file" ) or error( "cant open $wrkdir/$server/$host/$lpar_orig/JOB/$file" . __FILE__ . ":" . __LINE__ ) && next;
        $cfg_text = <FILE>;
        close(FILE);

        # my $job_cfg = "zabbix:/usr/sbin/zabbix_agentd listener #1 [waiting for connection]:21";
        ( $type_subtype, $subsystem_name, $proctime_total ) = split( /:/, $cfg_text );
        # print STDERR "11972 \$type_subtype  $type_subtype \$subsystem_name $subsystem_name \$proctime_total $proctime_total\n";
        $unix_cmd = $subsystem_name;
        if ( $detail == 1 ) {            # detail can have longer legend
          $subsystem_name = substr $subsystem_name, 0, 90;
        }
        else {
          $subsystem_name = substr $subsystem_name, 0, 50;
        }
      } ## end elsif ( $item eq "jobs" ||...)
      else {
        $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
        if ( $rrd_upd_time < $req_time ) {
          next;
        }
      } ## end else [ if ( $item eq "size" ||...)]
    } ## end else [ if ($vmware) ]
    if ( $rrd_upd_time > $last_update ) {
      $last_update = $rrd_upd_time;
    }

    $lpar = $file;
    $lpar =~ s/.rmh$//;
    $lpar =~ s/.rmm$//;
    $lpar =~ s/.rmd$//;
    $lpar =~ s/.rsm$//;
    $lpar =~ s/.rsh$//;
    $lpar =~ s/.rsd$//;
    $lpar =~ s/.rrm$//;    # for vmware
    $lpar =~ s/.mmm$//;    # for AS400 & wpar aggreg
    $lpar =~ s/.mmc$//;    # for AS400
    $lpar =~ s/.rrs$//;    # for datastores
    $lpar =~ s/.rrt$//;    # for datastores

    my $lpar_space_proc = human_vmware_name($lpar);

    my $leg_len = 25;

    my $vmware_alias    = "";
    my $datastore_name  = "";
    my $datacenter_name = "";

    if ( $item eq "size" || $item eq "res" || $item eq "threads" || $item eq "faults" || $item eq "pages" ) {    # AS400
      $lpar_space_proc =~ s/S0400//;
      $lpar_space_proc =~ s/Parm..mmm//;
      $lpar_space_proc =~ s/Parm.//;

      # cfg info about pools in file e.g. data/server/hmc/lpar--AS400--/S040005Parm<3>.mmm
      # print STDERR "9562 detail-graph-cgi.pl $wrkdir/$server/$host/$lpar_as400/$file\n";
      my $file_cfg = "$wrkdir/$server/$host/$lpar_as400$AS400/$file";
      $file_cfg =~ s/mmm$/cfg/;
      $file_cfg =~ s/Parm./Parm/;
      my $cfg_text = "";
      if ( -f "$file_cfg" ) {
        open( FILE, "$file_cfg" ) or error( "cant open $file_cfg" . __FILE__ . ":" . __LINE__ ) && next;
        $cfg_text = <FILE>;
        close(FILE);
        $lpar_space_proc .= " " . $cfg_text;
      } ## end if ( -f "$file_cfg" )
    } ## end if ( $item eq "size" ||...)
    elsif ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" || $item =~ /wpar_/ ) {
      $lpar_space_proc =~ s/\.mmm$//;
    }
    elsif ( $item =~ "dstrag_" ) {

      # $lpar_space_proc = $file = '/home/lpar2rrd/lpar2rrd/data/vmware_7f812c15-81a6-4dfe-85e3-6c9e973985f7/datastore_datacenter-2/5e336b5d-d04e4379'
      # find datastore name and vCenter alias and datacenter name
      # print STDERR "10492 detail-graph-cgi.pl \$file $file \$lpar $lpar \$lpar_space_proc $lpar_space_proc\n";
      my $file_name      = basename($file);
      my $datastore_uuid = $file_name;
      $datastore_uuid =~ s/\.rrs//;
      $datastore_uuid =~ s/\.rrt//;
      my $datastore_path = $file;
      $datastore_path =~ s/$file_name//;

      # print STDERR "10500 detail-graph-cgi.pl \$file $file \$datastore_uuid $datastore_uuid\n";

      my @dc_name = <$datastore_path*.dcname>;

      #print STDERR "10642 detail-graph-cgi.pl \@dc_name @dc_name ,$dc_name[0],\n";
      if ( defined $dc_name[0] && $dc_name[0] ne "" ) {
        $datacenter_name = basename( $dc_name[0] );
        $datacenter_name =~ s/\.dcname//;
      }

      $datastore_name = $datastore_uuid;
      my @names = <$datastore_path*.$datastore_uuid>;
      if ( @names > 0 && $names[0] ne "" ) {
        $datastore_name = basename( $names[0] );

        # remove extension
        $datastore_name =~ s/\.[^.]+$//;
        $datastore_name =~ s/:/\\:/g;
      } ## end if ( @names > 0 && $names...)

      # print STDERR "10509 detail-graph-cgi.pl \$datastore_name $datastore_name\n";

      # vmware alias name
      $vmware_alias = "no VMware alias";
      my $alias_file = $datastore_path;
      $alias_file =~ s/[^\/]*\/$//;

      # print STDERR "10515 detail-graph-cgi.pl \$alias_file $alias_file\n";
      $alias_file .= "vmware_alias_name";
      if ( -f $alias_file && open( FH, "< $alias_file" ) ) {
        my $name = <FH>;
        close FH;
        chomp $name;
        ( undef, $name ) = split( /\|/, $name );
        $vmware_alias = $name if $name ne "";
      } ## end if ( -f $alias_file &&...)

      #print STDERR "10524 detail-graph-cgi.pl \$vmware_alias $vmware_alias\n";
      $lpar_space_proc = "$vmware_alias $datastore_name";
      $leg_len         = 50;
    } ## end elsif ( $item =~ "dstrag_")
    my $lpar_space = $lpar_space_proc;

    $lpar_space =~ s/\&\&1/\//g;

    $leg_len = 29 if ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" );

    $lpar_space = legend_item_format( $lpar_space, $leg_len );

    # Exclude pools and memory
    if ( ( ("$lpar" eq "pool") && (length($lpar) == 4) ) || ( ( "$lpar" eq "mem" ) && ( length($lpar) == 3 ) ) || ( $lpar =~ "SharedPool[0-9]" ) || ( $lpar =~ "SharedPool[1-9][0-9]" ) ) {
      next;
    }

    if ( $item eq "size" || $item eq "res" || $item eq "threads" || $item eq "faults" || $item eq "pages" ) {
      print STDERR "$wrkdir/$server/$host/$lpar_as400$AS400/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );
    }
    elsif ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" ) {
      print STDERR "$wrkdir/$server/$host/$lpar_as400$AS400/JOB/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );
    }
    elsif ( $item eq "disks" || $item eq "disk_busy" ) {
      print STDERR "10245 $wrkdir/$server/$host/$lpar_as400$AS400/DSK/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );
    }
    else {
      print STDERR "$wrkdir/$server/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );
    }

    # to keep same count of characters
    $lpar_space =~ s/\\:/:/g;
    $lpar_space =~ s/\n//g;
    $lpar_space = sprintf( "%-$leg_len" . "s", $lpar_space );
    $lpar_space =~ s/:/\\:/g;
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%

    my $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$file";
    if ( $item =~ /^wpar_/ ) {
      $wrkdir_managedname_host_mem = "$wrkdir/$server/$host/$lpar_orig/$file";

      # print STDERR "10879 $wrkdir/$server/$host/$lpar_orig/$file\n";
    }

    $wrkdir_managedname_host_file =~ s/:/\\:/g;
    my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
    $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

    if ( $item eq "size" || $item eq "res" || $item eq "threads" || $item eq "faults" || $item eq "pages" ) {
      $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400$AS400/$file";

      #$wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;
      $wrkdir_managedname_host_file_legend = "";    # to be not clickable
    } ## end if ( $item eq "size" ||...)
    if ( $item eq "job_cpu" || $item eq "waj" || $item eq "disk_io" ) {
      $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400$AS400/JOB/$file";

      #$wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400/JOB/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;
      $wrkdir_managedname_host_file_legend = "";    # to be not clickable
    } ## end if ( $item eq "job_cpu"...)
    if ( $item eq "disks" || $item eq "disk_busy" ) {
      $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400$AS400/DSK/$file";

      #$wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_as400/DSK/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;
      $wrkdir_managedname_host_file_legend = "";    # to be not clickable
    } ## end if ( $item eq "disks" ...)
    if ( $item =~ "dstrag_" ) {
      $wrkdir_managedname_host_file = "$file";

      # print STDERR "10741 detail-graph-cgi.pl \$file $file\n";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;

      # $wrkdir_managedname_host_file_legend = ""; # to be not clickable
    } ## end if ( $item =~ "dstrag_")
    if ( $item =~ "wpar_" ) {
      $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$lpar_orig/$file";

      # print STDERR "10741 detail-graph-cgi.pl \$file $file\n";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;

      # $wrkdir_managedname_host_file_legend = ""; # to be not clickable
    } ## end if ( $item =~ "wpar_" )

    if ($vmware || $hyperv) {
      # new system
      $wrkdir_managedname_host_file = "$wrkdir/$vms_dir/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;
    } ## end if ($vmware)

    # bulid RRDTool cmd
    my $mem_item = "Memory_granted"; # for vmware
    $mem_item = "TotalPhysMemory" if $hyperv;
    my $gb_mem= "1024,/,1024,/";
    $gb_mem= "1024,/" if $hyperv;

    if ( $vmware || $hyperv) {
      if ($item eq "memaggreg") { $form_dec    = "%6.1lf";}
      $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":$mem_item:AVERAGE";
      $cmd .= " CDEF:curg${i}=cur${i},$gb_mem";
      $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
      $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\l\\\"";
      $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc\"";
      $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";

      # print STDERR "10771 detail-graph-cgi.pl \$lpar_space_proc $lpar_space_proc \$wrkdir_managedname_host_file_legend $wrkdir_managedname_host_file_legend\n";
    } ## end else [ if ( !$vmware ) ]
    elsif ( $item eq "size" ) {
        $form_dec = "%6.1lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par5:AVERAGE";
        $cmd .= " DEF:type${i}=\\\"$wrkdir_managedname_host_file\\\":par3:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1024,/,1024,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:type${i}:LAST:\\\"   %6.0lf  (0 - *FIXED, 1 - *SAME, 2 - *CALC)  \\l\\\"";
        my $item_spec = "memaggreg_size";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
        $cmd .= " PRINT:type${i}:LAST:\"$form_dec $delimiter\"";
      } ## end if ( $item eq "size" )
      elsif ( $item eq "res" ) {
        $form_dec = "%6.1lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1024,/,1024,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"     $form_dec \\l\\\"";
        my $item_spec = "memaggreg_res";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item eq "res" )
      elsif ( $item eq "threads" ) {
        $form_dec = "%6.0lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"     $form_dec \\l\\\"";
        my $item_spec = "memaggreg_threads";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item eq "threads")
      elsif ( $item eq "faults" ) {
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par4:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1,/";
        $cmd .= " DEF:cus${i}=\\\"$wrkdir_managedname_host_file\\\":par7:AVERAGE";
        $cmd .= " COMMENT:\\\"$lpar_space\\\"";
        my $color_r  = $found;
        my $form_dec = "%5.0lf";
        $cmd .= " $gtype:cur${i}$color[$color_r]:\\\" \\\"";
        $cmd .= " GPRINT:cur${i}:AVERAGE:\\\"%5.0lf\\\"";
        $cmd .= " GPRINT:cur${i}:MAX:\\\"%5.0lf\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_r]\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
        $color_r = ( $found + 27 ) % ( $color_max + 1 );
        $cmd .= " STACK:cus${i}$color[$color_r]:\\\" \\\"";
        $cmd .= " GPRINT:cus${i}:AVERAGE:\\\"%5.0lf\\\"";
        $cmd .= " GPRINT:cus${i}:MAX:\\\"%5.0lf\\l\\\"";
        $cmd .= " PRINT:cus${i}:AVERAGE:\"$form_dec $delimiter $color[$color_r]\"";
        $cmd .= " PRINT:cus${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item eq "faults")
      elsif ( $item eq "pages" ) {
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par5:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1,/";
        $cmd .= " DEF:cus${i}=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
        $cmd .= " COMMENT:\\\"$lpar_space\\\"";
        my $color_r  = $found;
        my $form_dec = "%5.0lf";
        $cmd .= " $gtype:cur${i}$color[$color_r]:\\\" \\\"";
        $cmd .= " GPRINT:cur${i}:AVERAGE:\\\"%5.0lf\\\"";
        $cmd .= " GPRINT:cur${i}:MAX:\\\"%5.0lf\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_r]\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
        $color_r = ( $found + 27 ) % ( $color_max + 1 );

        #$color_r = ++$found % ($color_max +1);
        $cmd .= " STACK:cus${i}$color[$color_r]:\\\" \\\"";
        $cmd .= " GPRINT:cus${i}:AVERAGE:\\\"%5.0lf\\\"";
        $cmd .= " GPRINT:cus${i}:MAX:\\\"%5.0lf\\l\\\"";
        $cmd .= " PRINT:cus${i}:AVERAGE:\"$form_dec $delimiter $color[$color_r]\"";
        $cmd .= " PRINT:cus${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item eq "pages" )
      elsif ( $item eq "job_cpu" ) {    #AS400
        $form_dec = "%6.1lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par3:AVERAGE";
        $cmd .= " CDEF:curgt${i}=cur${i},1,/";
        $cmd .= " CDEF:curg${i}=cur${i},UN,0,cur${i},IF,1,/";                        #avg across entire season - d,w
        my $lpar_space_short = $lpar_space;
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space_short $type_subtype\\\"";
        $cmd .= " GPRINT:curgt${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\" $form_dec $function_name_type $status_aj \\l\\\"";
        my $item_spec = "memaggreg_cpu";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_short\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $type_subtype $delimiter $function_name_type $delimiter $status_aj $delimiter $file\"";
        $cmd .= " PRINT:curgt${i}:AVERAGE:\"$form_dec $delimiter $delimiter\"";
      } ## end elsif ( $item eq "job_cpu")
      elsif ( $item eq "jobs" || $item eq "powlin_job" || $item eq "jobs_mem" || $item eq "powlin_mem" ) {    # Power & Linux
        $form_dec = "%6.2lf";
        my $pid = $file;
        $pid =~ s/\.cfg$//;
        my $cmd_tmp = "";

        for ( my $k = 0; $k < $count_files_cputot; $k++ ) {
          if ( $k == 0 ) {
            if ( $item eq "jobs_mem" || $item eq "powlin_mem" ) {
              $cmd_tmp .= " CDEF:curgx${k}v${i}=pid0,$pid,EQ,rss0,UNKN,IF";
            }
            else {
              $cmd_tmp .= " CDEF:curgx${k}v${i}=pid0,$pid,EQ,td0,UNKN,IF";
            }
          } ## end if ( $k == 0 )
          else {
            $l = $k - 1;
            if ( $item eq "jobs_mem" || $item eq "powlin_mem" ) {
              $cmd_tmp .= " CDEF:curgx${k}v${i}=pid${k},$pid,EQ,rss${k},curgx${l}v${i},IF";
            }
            else {
              $cmd_tmp .= " CDEF:curgx${k}v${i}=pid${k},$pid,EQ,td${k},curgx${l}v${i},IF";
            }
          } ## end else [ if ( $k == 0 ) ]
        } ## end for ( my $k = 0; $k < $count_files_cputot...)
        $l++;
        $l = 0 if $l == 1;    # special case - only one file
        if ( $item eq "jobs_mem" || $item eq "powlin_mem" ) {
          $cmd_tmp .= " CDEF:curgt${l}${i}=curgx${l}v${i},1024,/,1024,/";    # to GiB
        }
        else {
          $cmd_tmp .= " CDEF:curgt${l}${i}=curgx${l}v${i},1800,/";
        }

        # filter files having -nan AVERAGE-RUN
        my $cmdix = "$graph_cmd \\\"$name_out\\\"";
        $cmdix .= " --title \\\"$header\\\"";
        $cmdix .= " --start $start_time";
        $cmdix .= " --end $end_time";
        $cmdix .= $cmd_def_jobs;
        $cmdix .= $cmd_tmp;
        $cmdix .= " PRINT:curgt${l}${i}:AVERAGE:\\\"$form_dec\\\"";

        # print STDERR "11645 detail-graph-cgi.pl \$cmdix $cmdix\n";

        eval { RRDp::cmd qq($cmdix) };
        if ($@) {
          error( "Graph error in eval cmd : !$@! " . __FILE__ . ":" . __LINE__ );
        }
        else {
          my $answ = RRDp::read;
          if ( $$answ =~ "ERROR" ) {
            error( "$host:$server Graph rrdtool error : $$answ " . __FILE__ . ":" . __LINE__ );
            next;
          }
          chomp($$answ);

          # print STDERR "11498 detail-graph-cgi.pl \$answ ,$answ, ,$$answ,\n";
          next if $$answ =~ /[Nn]a[Nn]/;
        } ## end else [ if ($@) ]

        $cmd .= $cmd_tmp;

        # trick to simulate rrd file heartbeat of two steps
        $cmd .= " CDEF:curg_temp${l}${i}=curgt${l}${i},UN,0,curgt${l}${i},IF";    #avg across entire season - d,w";

        # if (prev3 >0 && prev2=0 && prev1=0 && actual_value >0) then prev3 else prev2; create act. value on place prev2 that is why the shift of two steps
        $cmd .= " CDEF:prev1a${l}${i}=PREV(curg_temp${l}${i})";
        $cmd .= " CDEF:prev2a${l}${i}=PREV(prev1a${l}${i})";
        $cmd .= " CDEF:prev3a${l}${i}=PREV(prev2a${l}${i})";
        $cmd .= " CDEF:curg_a${l}${i}=prev3a${l}${i},0,GT,prev2a${l}${i},0,EQ,prev1a${l}${i},0,EQ,curg_temp${l}${i},0,GT,+,+,+,4,EQ,prev3a${l}${i},prev2a${l}${i},IF";
        $cmd .= " SHIFT:curg_a${l}${i}:-3600";

        # if (prev2 > 0 && prev1 = 0 && actual_value > 0) then prev2 else prev1; create act. value on place prev1 that is why the shift of one step
        $cmd .= " CDEF:prev1b${l}${i}=PREV(curg_a${l}${i})";
        $cmd .= " CDEF:prev2b${l}${i}=PREV(prev1b${l}${i})";
        $cmd .= " CDEF:curg${l}${i}=prev2b${l}${i},0,GT,prev1b${l}${i},0,EQ,curg_a${l}${i},0,GT,+,+,3,EQ,prev2b${l}${i},prev1b${l}${i},IF"; # comment for orig graph
        $cmd .= " SHIFT:curg${l}${i}:-1800"; # comment for orig graph

        # prepare latest three values from original  - only for zoom times, not for basic day and week graphs
        # no this gives no good graphs
        #if ( isdigit($start_time) > 0 && isdigit($end_time) > 0 ) {
        #  my $step_count = ($end_time - $start_time -300 ) / 1800 - 1;
        #  $cmd .= " CDEF:curg_three${l}${i}=COUNT,$step_count,GT,curg_temp${l}${i},0,IF";
        #}

        # if you want to see original graph without heartbeat
        # $cmd .= " CDEF:curg${l}${i}=curgt${l}${i},UN,0,curgt${l}${i},IF";    #avg across entire season - d,w";

        my $lpar_space_short = $file;
        $lpar_space_short =~ s/\.cfg$//;
        $lpar_space_short = sprintf( "%-14s", $lpar_space_short );
        $type_subtype     = sprintf( "%-14s", $type_subtype );

        if ( $item eq "powlin_job" || $item eq "powlin_mem" ) {              # Power & Linux
          $gtype = "LINE";
        }
        #$cmd .= " $gtype:curg_three${l}${i}$color[$found % ($color_max +1)]";
        $type_subtype =~ s/%/%%/;
        $type_subtype =~ s/"//g;
        $cmd .= " $gtype:curg${l}${i}$color[$found % ($color_max +1)]:\\\"$lpar_space_short $type_subtype\\\"";
        #if ( isdigit($start_time) > 0 && isdigit($end_time) > 0 ) {
        #  $cmd .= " $gtype:curg_three${l}${i}$color[$found % ($color_max +1)]";
        #}
        $cmd .= " GPRINT:curgt${l}${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${l}${i}:AVERAGE:\\\" $form_dec \\\"";
        $subsystem_name =~ s/%/%%/;
        $subsystem_name =~ s/"//g;
        $cmd .= " GPRINT:curg${l}${i}:MAX:\\\" $form_dec $subsystem_name\\l\\\"";
        my $item_spec = "memaggreg_jobcpu";
        $cmd .= " PRINT:curg${l}${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_short\"";
        # $cmd .= " PRINT:curg${l}${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $type_subtype $delimiter $subsystem_name $delimiter $file\"";
        $cmd .= " PRINT:curg${l}${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $type_subtype $delimiter $file\"";
        $file =~ s/\.cfg$//;
        $unix_commands{$file} = $unix_cmd;
        $cmd .= " PRINT:curgt${l}${i}:AVERAGE:\"$form_dec $delimiter\"";
        $i++;

        # print STDERR "11559 \$cmd $cmd\n";
      } ## end elsif ( $item eq "jobs" ||...)
      elsif ( $item eq "disk_io" ) {
        $form_dec = "%7.0lf";
        $form_dec = "%7.0lf" if $detail == 1;
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par7:AVERAGE";
        $cmd .= " CDEF:curgt${i}=cur${i},1,/";
        $cmd .= " CDEF:curg${i}=cur${i},UN,0,cur${i},IF,1,/";                        #avg across entire season - d,w
        my $lpar_space_short = $lpar_space;
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space_short $type_subtype\\\"";
        $cmd .= " GPRINT:curgt${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\" $form_dec $function_name_type $status_aj \\l\\\"";
        my $item_spec = "memaggreg_disk";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_short\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $type_subtype $delimiter $function_name_type $delimiter $status_aj $delimiter $file\"";
        $cmd .= " PRINT:curgt${i}:AVERAGE:\"$form_dec $delimiter $delimiter\"";
      } ## end elsif ( $item eq "disk_io")
      elsif ( $item eq "waj" ) {
        $form_dec = "%5.2lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par8:AVERAGE";
        $cmd .= " CDEF:curgt${i}=cur${i},1000,/";
        $cmd .= " CDEF:curg${i}=cur${i},UN,0,cur${i},IF,1000,/";                     #avg across entire season - d,w
        my $lpar_space_short = $lpar_space;

        # print STDERR "\$lpar_space $lpar_space\n";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space_short $type_subtype\\\"";
        $cmd .= " GPRINT:curgt${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\" $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\" $form_dec $function_name_type $status_aj \\l\\\"";
        my $item_spec = "memaggreg_waj";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_short\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $type_subtype $delimiter $function_name_type $delimiter $status_aj $delimiter $file\"";
        $cmd .= " PRINT:curgt${i}:AVERAGE:\"$form_dec $delimiter $delimiter\"";
      } ## end elsif ( $item eq "waj" )
      elsif ( $item eq "disks" || $item eq "disk_busy" ) {
        $form_dec = "%5.1lf";
        $form_dec = "%6.1lf" if $detail == 1;
        my $par_x = "par4";    # for disk RQ
        $par_x = "par3" if $item eq "disk_busy";    # for disk busy
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":$par_x:AVERAGE";
        $cmd .= " CDEF:curgt${i}=cur${i},1,/";
        $cmd .= " CDEF:curg${i}=cur${i},UN,0,cur${i},IF,1,/";                          #avg across entire season - d,w
        my $lpar_space_short = $lpar_space;

        if ( $item eq "disk_busy" ) {
          $gtype = "LINE";
        }
        if ( $item eq "disks" ) {
          $form_dec = "%6.0lf";
          $form_dec = "%6.0lf" if $detail == 1;
        }
        $asp_num = legend_item_format( $asp_num, 10 );

        # print STDERR "\$form_dec $form_dec \$asp_num ,$asp_num,\n";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space_short $asp_num\\\"";
        $cmd .= " GPRINT:curgt${i}:AVERAGE:\\\"$form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"$form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        my $item_spec = "memaggreg_rq_disks";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item_spec $delimiter $lpar_space_short $asp_num\"";

        #$cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $file\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter N $delimiter N $delimiter N $delimiter $file\"";
        $cmd .= " PRINT:curgt${i}:AVERAGE:\"$form_dec $delimiter $delimiter\"";
      } ## end elsif ( $item eq "disks" ...)
      elsif ( $item eq "dstrag_used" ) {

        # print STDERR "10888 detail-graph-cgi.pl \$wrkdir_managedname_host_file $wrkdir_managedname_host_file\n";
        $form_dec = "%6.0lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":Disk_used:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1024,/,1024,/";
        $cmd .= " CDEF:curgt${i}=cur${i},1024,/,1024,/,1024,/";
        $cmd .= " $gtype:curgt${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $vmware_alias $delimiter $datastore_name $delimiter $color[$found % ($color_max +1)]\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $datacenter_name\"";
      } ## end elsif ( $item eq "dstrag_used")
      elsif ( $item =~ "dstrag_iops" ) {    # iopsr & iopsw
                                            # print STDERR "10899 detail-graph-cgi.pl \$wrkdir_managedname_host_file $wrkdir_managedname_host_file\n";
        $form_dec = "%6.0lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":$Datastore_DS_name:AVERAGE";  # Datastore_ReadAvg Datastore_WriteAvg
        $cmd .= " CDEF:curg${i}=cur${i},1,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $vmware_alias $delimiter $datastore_name $delimiter $color[$found % ($color_max +1)]\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $datacenter_name\"";
      } ## end elsif ( $item =~ "dstrag_iops")
      elsif ( $item =~ "dstrag_data" ) { # datar & dataw
                                         # print STDERR "10930 detail-graph-cgi.pl \$wrkdir_managedname_host_file $wrkdir_managedname_host_file\n";
        $form_dec = "%6.2lf";
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":$Datastore_DS_name:AVERAGE";                                                                            # Datastore_ReadAvg Datastore_WriteAvg
        $cmd .= " CDEF:curg${i}=cur${i},1000,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $vmware_alias $delimiter $datastore_name $delimiter $color[$found % ($color_max +1)]\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $datacenter_name\"";
      } ## end elsif ( $item =~ "dstrag_data")
      elsif ( $item =~ "wpar_mem" ) {

        # print STDERR "11112 detail-graph-cgi.pl \$wrkdir_managedname_host_file $wrkdir_managedname_host_file\n";
        $form_dec = "%6.2lf";
        my $rrd = $wrkdir_managedname_host_file;
        $cmd .= " DEF:used${i}=\"$rrd\":nuse:AVERAGE";
        $cmd .= " DEF:in_use_clnt${i}=\"$rrd\":in_use_clnt:AVERAGE";
        $cmd .= " CDEF:usedg${i}=used${i},1048576,/";
        $cmd .= " CDEF:in_use_clnt_g${i}=in_use_clnt${i},1048576,/";
        $cmd .= " CDEF:curg${i}=usedg${i},in_use_clnt_g${i},-";
        $lpar_space =~ s/\/mem/    /;
        $lpar_space_proc =~ s/\/mem/    /;
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item =~ "wpar_mem")
      elsif ( $item =~ "wpar_cpu" ) {

        # print STDERR "11160 detail-graph-cgi.pl \$wrkdir_managedname_host_file $wrkdir_managedname_host_file\n";
        $form_dec = "%6.2lf";
        my $rrd = $wrkdir_managedname_host_file;

        $cmd .= " DEF:pc${i}=\"$rrd\":entitled:AVERAGE";
        $cmd .= " DEF:cpusy${i}=\"$rrd\":cpu_sy:AVERAGE";
        $cmd .= " DEF:cpuus${i}=\"$rrd\":cpu_us:AVERAGE";
        $cmd .= " CDEF:proc${i}=pc${i},100,/";
        $cmd .= " CDEF:cpusyc${i}=cpusy${i},proc${i},*";
        $cmd .= " CDEF:cpuusc${i}=cpuus${i},proc${i},*";
        $cmd .= " CDEF:curg${i}=cpuusc${i},cpusyc${i},+";
        $lpar_space =~ s/\/cpu/    /;
        $lpar_space_proc =~ s/\/cpu/    /;
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter\"";
      } ## end elsif ( $item =~ "wpar_cpu")
      else {
        $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":curr_mem:AVERAGE";
        $cmd .= " CDEF:curg${i}=cur${i},1024,/";
        $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";
        $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     $form_dec \\\"";
        $cmd .= " GPRINT:curg${i}:MAX:\\\"$form_dec \\l\\\"";
        $cmd .= " PRINT:curg${i}:AVERAGE:\"$form_dec $delimiter $item $delimiter $lpar_space_proc\"";
        $cmd .= " PRINT:curg${i}:MAX:\"$form_dec $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\"";
    } ## end if ( $vmware )

    # put carriage return after each second lpar in the legend
    if ( $j == 1 ) {
      $j = 0;
    }
    else {
      # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++;
    }
    $i++;
    $gtype = "STACK";
  } ## end foreach $file (@files)

  if ( $found_at_least_one == 0 ) {

    #    return 0; # no source has been found
  }

  $cmd .= " COMMENT:\\n";
  if ( $item ne "size" && $item ne "res" && $item ne "threads" && $item ne "faults" && $item ne "pages" && $item ne "job_cpu" && $item ne "waj" && $item ne "disk_io" && $item ne "disks" && $item ne "disk_busy" && $item !~ "dstrag_" && $item !~ "wpar_" && $item ne "jobs" && $item ne "powlin_job" && $item ne "jobs_mem" && $item ne "powlin_mem" ) {
    $cmd .= " LINE2:maxg#000000:\\\"Available memory         \\\"";
    $cmd .= " GPRINT:maxg:AVERAGE:\\\"     %6.2lf \\l\\\"";
    $cmd .= " PRINT:maxg:AVERAGE:\"%6.2lf $delimiter $item $delimiter Available memory $delimiter #000000\"";
  }

  # for graph wpar_cpu utilization of the whole LPAR
  if ( $item eq "wpar_cpu" ) {

    # hmc data for lpar utilization
    # print STDERR "11257 detail-graph-cgi.pl LPAR util line for $wrkdir/$server/$host/$lpar_orig.rrm\n";
    my $rrd_lpar = "$wrkdir/$server/$host/$lpar_orig.rrm";
    if ( -f $rrd_lpar ) {
      $cmd .= " DEF:cur=\"$rrd_lpar\":curr_proc_units:AVERAGE";
      $cmd .= " DEF:ent=\"$rrd_lpar\":entitled_cycles:AVERAGE";
      $cmd .= " DEF:cap_peak=\"$rrd_lpar\":capped_cycles:AVERAGE";
      $cmd .= " DEF:uncap=\"$rrd_lpar\":uncapped_cycles:AVERAGE";

      # filtering peaks caused by LPM or changing entitled, if cap CPU util is > entitled --> UNKN
      # usualy cap counter is affected only
      # sometimes might happen that even in normal load capped util is little higher than entitled! Therefore using 1.2
      $cmd .= " CDEF:cap=cap_peak,ent,/,1.2,GT,UNKN,cap_peak,IF";

      $cmd .= " CDEF:tot=cap,uncap,+";
      $cmd .= " CDEF:util=tot,ent,/,$cpu_max_filter,GT,UNKN,tot,ent,/,IF";
      $cmd .= " CDEF:utilperct=util,100,*";
      $cmd .= " CDEF:utiltot=util,cur,*";

      $cmd .= " LINE1:utiltot#FF0000:\"Total utilization\"";
      $cmd .= " GPRINT:utiltot:AVERAGE:\"        %6.2lf\"";
      $cmd .= " GPRINT:utiltot:MAX:\" %6.2lf \\l\\\"";

      $cmd .= " PRINT:utiltot:AVERAGE:\"%6.2lf $delimiter $item $delimiter Total utilization\"";
      $cmd .= " PRINT:utiltot:AVERAGE:\"%6.2lf $delimiter #FF0000\"";
      $cmd .= " PRINT:utiltot:MAX:\"%6.2lf $delimiter\"";
    } ## end if ( -f $rrd_lpar )
  } ## end if ( $item eq "wpar_cpu")

  # for graph wpar_mem utilization of the whole LPAR
  if ( $item eq "wpar_mem" ) {

    # OS or NMON agent data for lpar mem utilization
    # either $wrkdir/$server/$host/$lpar/mem.mmm or $wrkdir/$server/$host/$lpar--NMON--/mem.mmm or nothing";
    my $rrd_lpar = "$wrkdir/$server/$host/$lpar_orig/mem.mmm";
    if ( !-f $rrd_lpar ) {
      $rrd_lpar = "$wrkdir/$server/$host/$lpar_orig--NMON--/mem.mmm";
    }
    if ( -f $rrd_lpar ) {
      $cmd .= " DEF:size=\"$rrd_lpar\":size:AVERAGE";
      $cmd .= " DEF:used=\"$rrd_lpar\":nuse:AVERAGE";
      $cmd .= " DEF:in_use_clnt=\"$rrd_lpar\":in_use_clnt:AVERAGE";

      $cmd .= " CDEF:sizeg=size,1048576,/";
      $cmd .= " CDEF:usedg=used,1048576,/";
      $cmd .= " CDEF:in_use_clnt_g=in_use_clnt,1048576,/";
      $cmd .= " CDEF:used_realg=usedg,in_use_clnt_g,-";

      #      $cmd .= " COMMENT:\"   [GB]             AVRG      MAX  \\n\"";
      $cmd .= " LINE1:used_realg#FF0000:\"Used LPAR \"";
      $cmd .= " GPRINT:used_realg:AVERAGE:\"                    %6.2lf\"";
      $cmd .= " GPRINT:used_realg:MAX:\" %6.2lf \\l\\\"";

      $cmd .= " LINE1:sizeg#000000:\"Total LPAR\"";
      $cmd .= " GPRINT:sizeg:AVERAGE:\"                    %6.2lf\"";
      $cmd .= " GPRINT:sizeg:MAX:\" %6.2lf \\l\\\"";

      $cmd .= " PRINT:used_realg:AVERAGE:\"%6.2lf $delimiter $item $delimiter Used LPAR\"";
      $cmd .= " PRINT:used_realg:AVERAGE:\"%6.2lf $delimiter #FF0000\"";
      $cmd .= " PRINT:used_realg:MAX:\"%6.2lf $delimiter\"";

      $cmd .= " PRINT:sizeg:AVERAGE:\"%6.2lf $delimiter $item $delimiter Total LPAR\"";
      $cmd .= " PRINT:sizeg:AVERAGE:\"%6.2lf $delimiter #000000\"";
      $cmd .= " PRINT:sizeg:MAX:\"%6.2lf $delimiter\"";
    } ## end if ( -f $rrd_lpar )
  } ## end if ( $item eq "wpar_mem")

  #if ( "$type" =~ "d" ) {
  my $ltime = localtime($last_update);
  $ltime =~ s/:/\\\:/g;
  $t = "COMMENT:\\\"Updated\\\: $ltime\\\"";
  $cmd .= " $t";
  $t = "COMMENT:Updated\\\: $ltime";

  #}

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # print STDERR "12631 detail-graph-cgi.pl $cmd\n";

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph mem aggregated rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    if ($@ =~ "Legend too long") {
      print STDERR "Here is the cmd with that long legend:$cmd\n\n";
    }
    return 1;
  }
  $t = "" if "$type" !~ "d";
  $png_end_heading = ret_graph_param( $ret, $t );
  return 0;
} ## end sub graph_memaggreg

sub graph_amsaggreg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $head        = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph Memory Aggreg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;    # some workaround, it should not go here
  }
  my $req_time   = 0;
  my $act_time   = localtime();
  my $act_time_u = time();

  my $header         = "Active Memory Sharing : last $text";
  my $vertical_label = "--vertical-label=Memory in GBytes";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";                   # nope string
    $header         = "Active Memory Sharing: $text";
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "";                                 # just a trick

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "Active Memory Sharing: $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my @files = "";
  opendir( DIR, "$wrkdir/$server/$host" ) || error( " directory does not exists : $wrkdir/$server/$host" . __FILE__ . ":" . __LINE__ ) && return 0;
  if ( $type_sam =~ "m" ) {
    my @files_unsorted = grep( /\.rmm$/, readdir(DIR) );
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "h" ) {
    my @files_unsorted = grep( /\.rmh$/, readdir(DIR) );
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( $type_sam =~ "d" ) {
    my @files_unsorted = grep( /\.rmd$/, readdir(DIR) );
    @files = sort { lc $a cmp lc $b } @files_unsorted;
    $req_time = $act_time_u - 31536000;
  }
  closedir(DIR);
  my $file = "";
  my $i    = 0;
  my $cmd  = "";
  my $j    = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  if ( ($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " \\\"$vertical_label\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";
  if ( "$type" =~ "d" ) {
    $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average    maximum     last\\l\\\"";
  }
  else {
    $cmd .= " COMMENT:\\\"Current allocated memory in GB\\:    average    maximum\\l\\\"";
  }
  my $gtype = "AREA";

  my $wrkdir_managedname_host_mem_pool = "$wrkdir/$server/$host/mem-pool.rr$type_sam";
  $wrkdir_managedname_host_mem_pool =~ s/:/\\:/g;

  if ( -f "$wrkdir/$server/$host/mem-pool.rr$type_sam" ) {
    $cmd .= " DEF:cur_pool=\\\"$wrkdir_managedname_host_mem_pool\\\":curr_pool_mem:AVERAGE";
    $cmd .= " CDEF:cur_poolg=cur_pool,1024,/";
  }

  my $found_at_least_one = 0;
  my $last_update        = 0;
  my $found              = -1;

  foreach $file (@files) {
    chomp($file);
    $found++;
    $found_at_least_one = 1;

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }
    if ( $rrd_upd_time > $last_update ) {
      $last_update = $rrd_upd_time;
    }

    $lpar = $file;
    $lpar =~ s/.rmh//;
    $lpar =~ s/.rmm//;
    $lpar =~ s/.rmd//;
    my $lpar_space_proc = $lpar;
    my $lpar_space      = $lpar;
    $lpar_space =~ s/\&\&1/\//g;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for ( my $k = length($lpar_space); $k < 25; $k++ ) {
      $lpar_space .= " ";
    }

    # Exclude pools and memory
    if ( ( ( "$lpar" eq "pool" ) && ( length($lpar) == 4 ) ) || ( ( "$lpar" eq "mem" ) && ( length($lpar) == 3 ) ) || ( $lpar =~ "SharedPool[0-9]" ) || ( $lpar =~ "SharedPool[1-9][0-9]" ) ) {
      next;
    }

    print "$wrkdir/$server/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );

    # to keep same count of characters
    $lpar_space =~ s/\\:/:/g;
    $lpar_space = sprintf( "%-25s", $lpar_space );
    $lpar_space =~ s/:/\\:/g;
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%'

    my $wrkdir_managedname_host_file = "$wrkdir/$server/$host/$file";
    $wrkdir_managedname_host_file =~ s/:/\\:/g;
    my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
    $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

    # bulid RRDTool cmd
    $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":phys_run_mem:AVERAGE";
    $cmd .= " CDEF:curg${i}=cur${i},1024,/";
    $cmd .= " $gtype:curg${i}$color[$found % ($color_max +1)]:\\\"$lpar_space\\\"";

    $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.2lf \\\"";
    my $item_day_other = "$item-other";
    if ( "$type" =~ "d" ) {
      $item_day_other = "$item-day";
    }

    $cmd .= " PRINT:curg${i}:AVERAGE:\\\"%6.2lf $delimiter $item_day_other $delimiter $lpar_space_proc $delimiter $color[$found % ($color_max +1)] $delimiter $wrkdir_managedname_host_file_legend\\\"";
    if ( "$type" =~ "d" ) {
      $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\\"";
      $cmd .= " PRINT:curg${i}:MAX:\\\"%6.2lf $delimiter \\\"";
      $cmd .= " GPRINT:curg${i}:LAST:\\\" %6.2lf \\l\\\"";
      $cmd .= " PRINT:curg${i}:LAST:\\\"%6.2lf $delimiter \\\"";
    } ## end if ( "$type" =~ "d" )
    else {
      $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.2lf \\l\\\"";
      $cmd .= " PRINT:curg${i}:MAX:\\\"%6.2lf $delimiter \\\"";
    }

    # put carriage return after each second lpar in the legend
    if ( $j == 1 ) {
      $j = 0;
    }
    else {
      # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++;
    }
    $i++;
    $gtype = "STACK";
  } ## end foreach $file (@files)

  if ( $found_at_least_one == 0 ) {
    return 0;    # no source has been found
  }

  if ( -f "$wrkdir/$server/$host/mem-pool.rr$type_sam" ) {
    $cmd .= " COMMENT:\\n";
    $cmd .= " LINE2:cur_poolg#000000:\\\"Configured max mem pool  \\\"";
    $cmd .= " GPRINT:cur_poolg:AVERAGE:\\\"     %6.2lf \\l\\\"";
    $cmd .= " PRINT:cur_poolg:AVERAGE:\"%6.2lf $delimiter $item-end $delimiter Configured max mem pool $delimiter #000000\"";

  } ## end if ( -f "$wrkdir/$server/$host/mem-pool.rr$type_sam")

  #if ( "$type" =~ "d" ) {
  my $ltime = localtime($last_update);
  $ltime =~ s/:/\\\:/g;
  $t = "COMMENT:\\\"Updated\\\: $ltime\\\"";
  $cmd .= " $t";

  #}

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph AMS rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  #$t = "" if "$type" !~ "d";
  $png_end_heading = ret_graph_param( $ret, $t );
  return 0;
} ## end sub graph_amsaggreg

sub graph_ldomagg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \" ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $line_items  = 0;                  # how many items in the legend per a line (default 2, when detail then 3)

  my $last_day_back = $act_time_u - 86400;

  if ( $detail == 1 ) {
    $line_items = 2;
  }
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lparagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }
  my $head = "LDOMs cpu";
  if ($item eq "s_ldom_agg_c"){
    $head = "LDOMs cpu";
  }
  elsif($item eq "s_ldom_agg_m"){
    $head = "LDOMs mem";
  }
  my $header = "$head aggregated :$last_txt$text";

  if ( $detail == 1 ) {
    $header = "$head aggregated: $server :$last_txt$text";
    $header = "$head aggregated: $lpar :$last_txt$text" if ( $item =~ "wlm");
    $header = "$head aggregated: $last_txt$text" if ( $item eq "multicluster" );
    $header = "$head aggregated: $host :$last_txt$text" if ( $hitachi );
  }

  if ( $detail == 2 ) {
    # $header = "$server :$last_txt$text";
    $header = "LDOMs aggregated :$last_txt$text";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  # my $start_time = "now-1$type";
  # my $end_time = "now-1$type+1$type";

  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  # print STDERR "$start_time $end_time \n";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $req_time   = $start_time;    # set here too
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid      = "";             # just a trick
      }

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$head aggregated: $server : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  my @cluster_arr;
  my @files = "";
#  print STDERR"$wrkdir/$server/$host/$lpar/\n";
  opendir( DIR, "$wrkdir/Solaris/" ) || error( " directory does not exists : $wrkdir/Solaris/ " . __FILE__ . ":" . __LINE__ ) && return 0;
  my @ldom_names = grep !/^\.\.?$/, readdir(DIR);
  closedir(DIR);
  foreach my $ldom_name(@ldom_names){
    chomp $ldom_name;
    if (-d "$wrkdir/Solaris/$ldom_name/ZONE/"){
      if (-f "$wrkdir/Solaris/$ldom_name/ZONE/total.mmm"){
      push @files,"$wrkdir/Solaris/$ldom_name/ZONE/total.mmm\n";
      }
    }
  }

#  my $file_count = @files;


  my $file = "";
  my $i    = 0;

  #  my $lpar = "";
  my $aix_ser = $lpar;
  my $cmd = "";
  my $j   = 0;

  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "";
  if ($item eq "s_ldom_agg_c"){
    $vertical_label = "--vertical-label=\"CPU cores\"";
  }
  elsif($item eq "s_ldom_agg_m"){
    $vertical_label = "--vertical-label=\"GB\"";
  }


  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  my $units = "cores";

  if ($item eq "s_ldom_agg_c"){
    if ( $detail == 1 ) {
      $cmd .= " COMMENT:\"Utilization in CPU $units\\:    average  maximum                             average  maximum                               average  maximum\\l\"";
    }
    else {
      $cmd .= " COMMENT:\"Utilization in CPU $units\\:    average  maximum\\l\"";
    }
  }
  elsif ($item eq "s_ldom_agg_m"){
    if ( $detail == 1 ) {
      $cmd .= " COMMENT:\"Used memory in GB\\:       average  maximum                           average  maximum                               average  maximum\\l\"";
    }
    else {
      $cmd .= " COMMENT:\"Used Memory in GB\\:    average  maximum\\l\"";
    }
  }

  my $lpar_color_index = -1;
  my $color_WLM_counter= 0;
  my $gtype            = "AREA";
  my $color_indx       = 0;
  my $line_indx        = 0;        # place enter every 3rd line
  my $files_index      = -1;       # for vmware it is color index
  foreach $file (@files) {
    chomp($file);
    if (defined $file && $file eq ""){next}
    my $rrd_file_update = ( stat("$file") )[9];
    #if ($rrd_file_update < $last_day_back) {next}

    if ( "$type" =~ "d" ) {
      $req_time = $act_time_u - 86400;
      if ($rrd_file_update < $req_time) {next}
    }
    if ( "$type" =~ "w" ) {
      $req_time = $act_time_u - 604800;
      if ($rrd_file_update < $req_time) {next}
    }
    if ( "$type" =~ "m" ) {
      $req_time = $act_time_u - 2764800;
      if ($rrd_file_update < $req_time) {next}
    }
    if ("$type" =~ "y" ) {
      $req_time = $act_time_u - 31536000;
      if ($rrd_file_update < $req_time) {next}
    }
    my $file_orig = $file;         # keep for vmware
    #    $files_index = $files_index++ % $color_max; # not possible !!!
    $files_index++;
    $files_index = $files_index % $color_max;

    $lpar_color_index++;
    my @arr_path = split( "\/", $file );
    $file   = $arr_path[-1];
    $host   = $arr_path[-2];
    $server = $arr_path[-3];
    # avoid old lpars which do not exist in the period
    #my $rrd_upd_time = ( stat("$wrkdir/$server/$host/$file") )[9];
    #next if ( $rrd_upd_time < $req_time );


    $lpar = $file;
    $lpar =~ s/.rrh//;
    $lpar =~ s/.rrm//;
    $lpar =~ s/.rrd//;
    $lpar =~ s/.mmm//;
    $lpar =~ s/.hlm//; # hitachi suffix
    $lpar =~ s/wlm-// if ( $item =~ "wlm-");

    my $lpar_space_proc = human_vmware_name($lpar,"",$server);
    my $lpar_space = $lpar_space_proc;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    for ( my $k = length($lpar_space); $k < 25; $k++ ) {
      $lpar_space .= " ";
    }

    # max 25 chars
    $lpar_space = substr( $lpar_space, 0, 24 );

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%

    my $wrkdir_server_host_file = "$wrkdir/Solaris/$server/$host/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $wrkdir_server_host_file;
    $wrkdir_server_host_file_legend =~ s/%/%%/g;
    my $server_space = $server;
    $server_space =~ s/\&\&1/\//g;
    for ( my $k = length($server_space); $k < 25; $k++ ) {
      $server_space .= " ";
    }

    # max 25 chars
    $server_space = substr( $server_space, 0, 24 );

    $server_space =~ s/:/\\:/g;    # anti ':'
    my $ds_name1 = "";
    my $lf_format = "";
    ###### CMD PART
    if ($item eq "s_ldom_agg_c"){
      $ds_name1 = "cpu_used";
      $lf_format = "%5.2lf"
    }
    elsif($item eq "s_ldom_agg_m"){
      $ds_name1 = "phy_mem_us";
      $lf_format = "%3.1lf"
    }
    $cmd .= " DEF:util_tot${i}=\"$wrkdir_server_host_file\":$ds_name1:AVERAGE";
    if($item eq "s_ldom_agg_m"){
      $cmd .= " CDEF:utiltot${i}=util_tot${i},1000,/,1000,/";
    }
    elsif($item eq "s_ldom_agg_c"){
      $cmd .= " CDEF:utiltot${i}=util_tot${i},1,/";
    }
    my $index_to_display = $color_indx;
    if ( $lpar_color_index > -1 ) {
      $index_to_display = $lpar_color_index;
    }
    $cmd .= " $gtype:utiltot${i}$color[$index_to_display]:\"$server_space\"";
    $cmd .= " GPRINT:utiltot${i}:AVERAGE:\"$lf_format \"";
    $cmd .= " GPRINT:utiltot${i}:MAX:\" $lf_format \"";

    $cmd .= " PRINT:utiltot${i}:AVERAGE:\"$lf_format $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$index_to_display]\"";
    $cmd .= " PRINT:utiltot${i}:MAX:\" $lf_format $delimiter $wrkdir_server_host_file_legend\"";

    #$cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \\l\"";
    # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)

    $color_indx = ( $color_indx + 1 ) % ( $color_max + 1 );

    #    $col_indx++;
    $gtype = "STACK";
    $i++;
    if ( $line_indx == $line_items ) {

      # put carriage return after each second lpar in the legend
      $cmd .= " COMMENT:\"\\l\"";
      $line_indx = 0;
    } ## end if ( $line_indx == $line_items)
    else {
      $line_indx++;
    }

  }
  $cmd .= " COMMENT:\"\\l\""; # last legend line in detail not across whole line

  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  # print STDERR "7464 detail-graph-cgi.pl lparagg \$cmd $cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;

}

sub encode_base64 {
  if ( $] >= 5.006 ) {
    require bytes;
    if ( bytes::length( $_[0] ) > length( $_[0] )
      || ( $] >= 5.008 && $_[0] =~ /[^\0-\xFF]/ ) )
    {
      require Carp;
      Carp::croak("The Base64 encoding is only defined for bytes");
    } ## end if ( bytes::length( $_...))
  } ## end if ( $] >= 5.006 ) ([)

  use integer;

  my $eol = $_[1];
  $eol = "\n" unless defined $eol;

  my $res = pack( "u", $_[0] );

  # Remove first character of each line, remove newlines
  $res =~ s/^.//mg;
  $res =~ s/\n//g;

  $res =~ tr|` -_|AA-Za-z0-9+/|;    # `# help emacs
                                    # fix padding at the end
  my $padding = ( 3 - length( $_[0] ) % 3 ) % 3;
  $res =~ s/.{$padding}$/'=' x $padding/e if $padding;

  # break encoded string into lines of no more than 76 characters each
  if ( length $eol ) {
    $res =~ s/(.{1,76})/$1$eol/g;
  }
  return $res;
} ## end sub encode_base64

sub human_vmware_name {
  my $lpar  = shift;
  my $arrow = shift;
  my $local_server = shift;

  $arrow = "" if !defined $arrow;

  return "$lpar" if ( !$vmware && !$hyperv );    # only for vmware or hyperv
                                                 # read file and find human lpar name from uuid or
                                                 # if 'neg' then find uuid from name
                                                 # print STDERR "11609 \$lpar $lpar \$arrow $arrow\n";
  my $vms_dir = "vmware_VMs";
  if ($hyperv) {
    $vms_dir = "$local_server/$all_hyperv_VMs";
    if ($local_server !~ /^windows/) { ; #$wrkdir/windows/$host/$all_hyperv_VMs/$lpar.rrm";
      $vms_dir = "windows/$vms_dir";
    }
  }
  # my $trans_file = "$wrkdir/$server/$host/lpar_trans.txt"; # old solution on servers
  my $trans_file = "$wrkdir/$vms_dir/vm_uuid_name.txt";
  # print STDERR "13770 $wrkdir/$vms_dir/vm_uuid_name.txt \n";
  if ( -f "$trans_file" ) {
    my $name      = "";
    my $file_time = 0;

    # there can be more UUID for same Vm name when param is 'neg', choose latest one
    open( FR, "< $trans_file" );
    foreach my $linep (<FR>) {
      chomp($linep);
      # print STDERR "12956 $linep\n";
      ( my $id, my $name_tmp, undef ) = split( /,/, $linep );
      if ( "$arrow" eq "neg" ) {
        ( $name_tmp, $id, undef ) = split( /,/, $linep );
        # print STDERR "12960 \$id $id \$lpar $lpar\n";
        if ( "$id" eq "$lpar" ) {
          next if !-f "$wrkdir/$vms_dir/$name_tmp.rrm";
          my $act_file_time = ( stat("$wrkdir/$vms_dir/$name_tmp.rrm") )[9];
          if ( $act_file_time > $file_time ) {
            $file_time = $act_file_time;
            $name      = $name_tmp;

            # print STDERR "11630 \$name $name \$file_time $file_time\n";
          } ## end if ( $act_file_time > ...)
        } ## end if ( "$id" eq "$lpar" )
      } ## end if ( "$arrow" eq "neg")
      else {
        if ( "$id" eq "$lpar" ) {
          $name = $name_tmp;
          last;
        }
      } ## end else [ if ( "$arrow" eq "neg")]
    } ## end foreach my $linep (<FR>)
    close(FR);
    $lpar = "$name" if $name ne "";
  } ## end if ( -f "$trans_file" )
  return "$lpar";    #human name - if found, or original
} ## end sub human_vmware_name

sub find_human_ds_name {
  my ( $server, $host, $lpar ) = @_;
  my @files     = glob "$wrkdir/$server/$host/*.$lpar";
  my $lpar_name = $lpar;
  if ( defined $files[0] && $files[0] ne "" ) {
    $lpar_name = ( split( "\/", $files[0] ) )[-1];
    $lpar_name =~ s/\.$lpar$//;
  }
  return $lpar_name;
} ## end sub find_human_ds_name

sub graph_vmw_disk {
  my ( $host, $server, $lpar, $type, $name_out, $type_sam, $detail, $start_unix, $end_unix ) = @_;
  my $t        = "COMMENT:\" \"";
  my $t2       = "COMMENT:\"\\n\"";
  my $step_new = $step;
  my $last     = "COMMENT: ";
  my $head     = "COMMENT:                            Average   Max\\n";

  my $xgrid         = xgrid_set( $type, $detail );
  my $text          = text_set($type);
  my $human_ds_name = "";

  my $filter = 0;

  # due to historical reports where is specific time passed
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      if ( $detail != 10 ) {
        $xgrid = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
      }
                                        # finding human datastore name
      if ( $item eq 'dsrw' || $item eq 'dsarw' || $item eq 'dslat' ) {
        $human_ds_name = find_human_ds_name( $server, $host, $lpar );
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST: %4.0lf";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
  }

  # print STDERR "12966 detail-graph-cgi.pl $host:$server:$lpar \$detail $detail \$start_unix $start_unix \$end_unix $end_unix \$item $item\n";
  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_vmw_disk: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                    # some workaround, it should not go here
  }

  my $rrd;
  # print STDERR "13864 \$hyperv $hyperv $wrkdir / $server / $host / $lpar\n";
  if ( $item =~ 'vmdisk' || $item =~ 'vmnet' ) {    # ESXi server tabs
    $lpar = "pool";
    $rrd  = "$wrkdir/$server/$host/$lpar.rrm";
  }
  elsif ( $item =~ 'dsrw' || $item =~ 'dsarw' ) {    # datastore Read Write, Averaged
    $rrd = "$wrkdir/$server/$host/$lpar.rrt";
  }
  elsif ( $item =~ 'dslat' ) {                       # datastore Latency
    $rrd = "$wrkdir/$server/$host/$lpar.rru";
  }
  elsif ($hyperv) {
    if ( $item =~ 'hyp-disk' || $item =~ 'hyp-mem' || $item =~ 'hyp-cpu' || $item =~ 'hyp-net' ) {    # VM disk
      $rrd = "$wrkdir/$server/$all_hyperv_VMs/$lpar.rrm";
    }
    elsif ($item =~ 'lfd_dat' || $item =~ 'lfd_io' || $item =~ 'lfd_lat') {
      $rrd = "$wrkdir/$server/$host/Local_Fixed_Disk_$lpar.rrm";
    }
    elsif ($item =~ 'csv_dat' || $item =~ 'csv_io' || $item =~ 'csv_lat') {
      $lpar =~ s/CSV//;
      #$rrd = "$wrkdir/$server/$host/Cluster_Storage_Volume$lpar.rrm";
      $rrd = "$wrkdir/$server/$host/Cluster_Storage_$lpar.rrm";
    }
    else { #server
      $lpar = "pool";
      $rrd  = "$wrkdir/$server/$host/$lpar.rrm";
    }
    # print STDERR "13883 \$rrd $rrd\n";
  } ## end elsif ($hyperv)
  elsif ($item =~ /^solaris_zone_cpu|^solaris_zone_os_cpu|^solaris_zone_mem|^solaris_zone_net/){
    $rrd = "$wrkdir/Solaris/$server/ZONE/$lpar.mmm";
  }
  elsif ($item =~ /^s_ldom_c|^s_ldom_m/){
    my $char_l = "_ldom";
    if ($server =~ /Solaris--unknown|Solaris\d+--unknown/){$server=$lpar;}
    $rrd = "$wrkdir/Solaris/$server/$lpar$char_l.mmm";
  }
  elsif ($item =~ /^s10_g_cpu|^s10_g_mem/){
    $rrd = "$wrkdir/Solaris/$server/ZONE/$lpar.mmm";
  }
  elsif ($item =~ /^s10_z_cpu|^s10_z_mem/){
    $rrd = "$wrkdir/Solaris/$server/ZONE/$lpar.mmm";
  }
  else {                                                                        # VM tabs
    $rrd = "$wrkdir/$all_vmware_VMs/$lpar.rrm";

    # print STDERR "14476 detail-graph-cgi.pl vmw_disk $rrd \$server $server \$host $host\n";
  }

  if ( !-f "$rrd" ) {
    error( "for $item file $rrd does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $lpar_slash = human_vmware_name($lpar,"",$server);

  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  $lpar_slash_nmon =~ s/\(.*\)//;

  my $header_prefix  = "VM :";
  my $head_pref      = "";
  my $legend_head    = " COMMENT:\"    GB               Avrg     Max\\n\"";
  my $vertical_label = "--vertical-label=\"Memory in GBytes\"";
  my $leg_write      = "Mem baloon  ";
  my $leg_read       = "Mem active  ";
  my $leg_usage      = "Mem granted ";
  my $gprint_frm     = "%6.2lf";
  my $upper_limit    = 0.1;

  if ( $item =~ "hyp-net" ) {
    $header_prefix  = "VM :";
    $head_pref      = "";
    $legend_head    = " COMMENT:\"[MB/sec]               Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=\"Read MB/s Write\"";
    $leg_write      = "";
    $leg_read       = "";
    $leg_usage      = "";
  } ## end if ( $item =~ "hyp-net")
  if ( $item =~ "hyp-mem" ) {
    $header_prefix  = "VM :";
    $head_pref      = "";
    $legend_head    = " COMMENT:\"    GB               Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=\"Memory in GBytes\"";
    $leg_write      = "";
    $leg_read       = "";
    $leg_read       = "Usage" if $item eq "vmw-disk" || $item eq "vmdisk";
    $leg_usage      = "";
  } ## end if ( $item =~ "hyp-mem")
  if ( $item =~ "hyp-cpu" ) {
    $head_pref      = "";
    $legend_head    = " COMMENT:\"[cores]          Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=CPU-usage";
    $leg_write      = "Usage     ";
    $leg_read       = "Usage     ";
    $leg_usage      = "";
    $upper_limit    = 0.1;
  } ## end if ( $item =~ "hyp-cpu")
  if ( $item =~ "vmw-disk" || $item =~ "vmdisk" || $item =~ "hyp-disk" ) {
    #$header_prefix  = "Disk:";
    $head_pref      = "Disk";
    $legend_head    = " COMMENT:\"[MB/sec]        Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=Read-MB/sec-Write";
    $vertical_label = "--vertical-label=Usage-MB/sec" if $item eq "vmw-disk" || $item eq "vmdisk";
    $leg_write      = "Write";
    $leg_read       = "Read ";
    $leg_read       = "Usage" if $item eq "vmw-disk" || $item eq "vmdisk";
    $leg_usage      = "";
    $filter         = $filter_max_lansan / 1000;
  } ## end if ( $item =~ "vmw-disk"...)
  if ( $item =~ "vmw-net" || $item =~ "vmnet" ) {
    #$header_prefix  = "LAN:";
    $head_pref      = "LAN";
    $legend_head    = " COMMENT:\"[MB/sec]          Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=Read-MB/sec-Write";
    $vertical_label = "--vertical-label=Usage-MB/sec" if $item eq "vmw-net" || $item eq "vmnet";
    $leg_write      = "Write    ";
    $leg_read       = "Read     ";
    $leg_read       = "Usage    " if $item eq "vmw-net" || $item eq "vmnet";

    # $leg_usage = "Net usage";
    $filter = $filter_max_lansan / 1000;
  } ## end if ( $item =~ "vmw-net"...)
  if ( $item eq "vmw-swap" ) {
    $head_pref      = "Swap";
    $legend_head    = " COMMENT:\"[MB/sec]          Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=Read-MB/sec-Write";
    $leg_write      = "In       ";
    $leg_read       = "Out      ";
  } ## end if ( $item eq "vmw-swap")
  if ( $item eq "vmw-comp" ) {
    $head_pref      = "Comp";
    $legend_head    = " COMMENT:\"[KB/sec]          Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=Read-KB/sec-Write";
    $leg_write      = "Compres  ";
    $leg_read       = "Decompres";
  } ## end if ( $item eq "vmw-comp")
  if ( $item eq "dsrw" ) {
    $header_prefix   = "DS :";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[MB/sec]          Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=Read-MB/sec-Write";
    $leg_write       = "Write    ";
    $leg_read        = "Read     ";
    $lpar_slash_nmon = "DATA $human_ds_name";
  } ## end if ( $item eq "dsrw" )
  if ( $item eq "dsarw" ) {
    $header_prefix   = "DS :";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[IOPS/sec]          Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=Read-IOPS-Write";
    $leg_write       = "Write    ";
    $leg_read        = "Read     ";
    $gprint_frm      = "%7.0lf";
    $lpar_slash_nmon = "IOPS $human_ds_name";
  } ## end if ( $item eq "dsarw" )
  if ( $item eq "dslat" || $item =~ "lfd_lat" || $item =~ "csv_lat") {
    $header_prefix   = "DS :";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[Latency millisec]    Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=Read-Latency-Write";
    $leg_write       = "Write      ";
    $leg_read        = "Read       ";
    $gprint_frm      = "%7.2lf";
    $lpar_slash_nmon = "Total Latency $human_ds_name";
    $filter          = 10000;
  } ## end if ( $item eq "dslat" )
  if ( $item =~ "vmw-proc" ) {
    $head_pref      = "%";
    $legend_head    = " COMMENT:\"[%]                 Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=CPU-usage-%";
    $leg_write      = "Usage      ";
    $leg_read       = "Usage      ";
    $leg_usage      = "";
    $upper_limit    = 100;
    $gprint_frm      = "%5.1lf";
  } ## end if ( $item =~ "vmw-proc")
  if ( $item =~ "vmw-ready" ) {
    $head_pref   = "Ready";
    $legend_head = " COMMENT:\"[%]                    Avrg      Max\\n\"";

    #$legend_head = " COMMENT:\"Millisec          Avrg      Max          Avrg[%]  Max[%] (Right axis)\\n\"";
    $vertical_label = "--vertical-label=CPU-ready-%";
    $leg_write      = "CPU READY/vCPU";
    $leg_read       = "CPU READY/vCPU";
    $leg_usage      = "";
    $upper_limit    = "100";
  } ## end if ( $item =~ "vmw-ready")
  if ( $item =~ "vmw-mem" ) {
    $head_pref   = "MEM";
  }
  if ( $item eq "hdt_data" || ($item =~ 'lfd_dat' ) || ($item =~ 'csv_dat' ) ) {
    $header_prefix   = "DS :";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[MB/sec]            Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=Read-MB/s-Write";
    $leg_write       = "Write    ";
    $leg_read        = "Read     ";
    $gprint_frm      = "%7.0lf";
    $lpar_slash_nmon = "R/W $human_ds_name";
    $filter          = 5000000000000;
  } ## end if ( $item eq "hdt_data")

  if ( $item eq "hdt_io" || $item =~ 'lfd_io' || ($item =~ 'csv_io' )) {
    $header_prefix   = "DS :";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[IOPS/sec]          Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=Read-IOPS-Write";
    $leg_write       = "Write    ";
    $leg_read        = "Read     ";
    $gprint_frm      = "%7.0lf";
    $lpar_slash_nmon = "IOPS $human_ds_name";
    $filter          = 50000;
  } ## end if ( $item eq "hdt_io")

  if ( $item eq "solaris_zone_cpu") {
    $header_prefix   = "ZONE CPU ";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"                              Avrg   Max\\n\"";
    $vertical_label  = "--vertical-label=\"CPU cores\"";
    $leg_read        = "Usage in CPU cores    ";
    $gprint_frm      = "%3.2lf";
    #$lpar_slash_nmon = "";
    $filter          = 50000;
  }
  if ( $item eq "solaris_zone_mem") {
    $header_prefix   = "ZONE Memory ";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[GB]              Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=\"GB\"";
    $leg_write       = "Capped memory    ";
    $leg_read        = "Used memory     ";
    $gprint_frm      = "%3.1lf";
    ##$lpar_slash_nmon = "";
    $filter          = 100;
  }
  if ( $item eq "solaris_zone_os_cpu") {
    $header_prefix   = "ZONE OS CPU ";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[%]                 Avrg   Max\\n\"";
    $vertical_label  = "--vertical-label=\"CPU-usage-%\"";
    $leg_write       = "Usage    ";
    $leg_read        = "Usage     ";
    $gprint_frm      = "%3.2lf";
    #$lpar_slash_nmon = "";
    $upper_limit          = 100;
  }
  if ( $item eq "solaris_zone_net") {
    $header_prefix   = "ZONE NET";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[%]                 Avrg   Max\\n\"";
    $vertical_label  = "--vertical-label=\"NET-usage-%\"";
    $leg_write       = "Usage    ";
    $leg_read        = "Usage     ";
    $gprint_frm      = "%3.2lf";
    #$lpar_slash_nmon = "";
    $upper_limit          = 100;
  }
  if ($item eq "s_ldom_c"){
    $header_prefix   = "LDOM CPU ";
    $head_pref       = "";
    $legend_head    = " COMMENT:\"[%]                 Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=CPU-usage-%";
    $leg_write      = "Usage      ";
    $leg_read       = "Usage      ";
    $leg_usage      = "";
    $upper_limit    = 100;
  }
  if ( $item eq "s_ldom_m") {
    $header_prefix   = "LDOM Memory ";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[GB]              Avrg      Max\\n\"";
    $vertical_label  = "--vertical-label=\"GB\"";
    $leg_write       = "Capped memory    ";
    $leg_read        = "Allocated memory     ";
    $gprint_frm      = "%3.1lf";
    $filter          = 100;
  }
    if ( $item eq "s10_g_cpu" || $item eq "s10_z_cpu") {
    $header_prefix   = "ZONE CPU ";
    $head_pref       = "";
    $legend_head    = " COMMENT:\"[%]                 Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=CPU-usage-%";
    $leg_write      = "Usage      ";
    $leg_read       = "Usage      ";
    $leg_usage      = "";
    $upper_limit    = 100;
  }
  if ($item eq "s10_g_mem" || $item eq "s10_z_mem") {
    $header_prefix   = "ZONE Memory ";
    $head_pref       = "";
    $legend_head     = " COMMENT:\"[%]                 Avrg   Max\\n\"";
    $vertical_label  = "--vertical-label=MEM-usage-%";
    $leg_write       = "Usage    ";
    $leg_read        = "Usage     ";
    $gprint_frm      = "%3.2lf";
    #$lpar_slash_nmon = "";
    $upper_limit          = 100;
  }


  # print STDERR "11193 detail-graph-cgi.pl ".time."\$item $item\n";
  my $last_txt = " last ";

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
    $last_txt      = "";
    $header_prefix = $head_pref;
    if ( $wpar == 1 ) {
      $header_prefix = $head_pref;
    }
  } else { ## end if ( $detail == 2 )
    $header_prefix .= $head_pref;
  }

  my $header = "$header_prefix:$lpar_slash_nmon:$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
      if ( $detail != 10 && $detail != 9 && $width < 700 ) {
        $header = "$start_human - $end_human";
      }
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  my $my_max = 15;

  if ( $type =~ "d" ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          $last = "";

          #if ( $m_arr[2] > 0 ) {
          #  my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
          #  my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
          #  $last ="COMMENT:Last memory consumption in GB         $last_mem_g";
          #}
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  # for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;    #actual is the first
  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "", \@rrd_files, $cgi );

  # print STDERR "lpm_find_files is returning @rrd_files\n";

  # print "graphing PAGING   : $host:$server:$lpar_slash:$type_sam:$type\n" if $DEBUG ;
  # print STDERR "graphing PAGING   : $rrd: $host:$server:$lpar_slash:$type_sam:$type:$start_time:$end_time\n";

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper_limit";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  $cmd .= " --base=1024";

  my $i = -1;
  my $j;

  ## define data stream names
  my @ds = ( "Memory_granted", "Memory_active", "Memory_baloon" );
  @ds = ( "Disk_read",        "Disk_write" )          if ( $item eq "vmw-diskrw" || $item eq "vmdiskrw" );
  @ds = ( "Disk_usage",       "Disk_usage" )          if ( $item eq "vmw-disk"   || $item eq "vmdisk" );     # fake for easy testing later
  @ds = ( "Network_received", "Network_transmitted" ) if ( $item eq "vmw-netrw"  || $item eq "vmnetrw" );
  @ds = ( "Network_usage",    "Network_usage" )       if ( $item eq "vmw-net"    || $item eq "vmnet" );      # fake for easy testing later
  @ds = ( "Memory_swapin",    "Memory_swapout" )      if $item eq "vmw-swap";
  @ds = ( "Memory_compres",   "Memory_decompres" )    if $item eq "vmw-comp";
  @ds = ( "Datastore_read",   "Datastore_write" )     if $item eq "dsrw";
  @ds = ( "Datastore_ReadAvg",  "Datastore_WriteAvg" )  if $item eq "dsarw";
  @ds = ( "Dstore_readLatency", "Dstore_writeLatency" ) if $item eq "dslat";
  @ds = ( "CPU_usage_Proc", "vCPU", "host_hz", "CPU_usage" ) if $item eq "vmw-proc";
  @ds = ( "CPU_ready_ms", "vCPU" ) if $item eq "vmw-ready";

  @ds = ( "DiskReadBytesPersec", "DiskWriteBytesPerse", "Timestamp_PerfTime", "Frequency_PerfTime" ) if ( $item eq "hdt_data" || ($item =~ 'lfd_dat' ) || ($item =~ 'csv_dat' ));
  @ds = ( "BytesReceivedPersec", "BytesSentPersec" )     if ( $item eq "vmnetrw"  && $hyperv );
  @ds = ( "DiskReadsPersec",     "DiskWritesPersec", "Timestamp_PerfTime", "Frequency_PerfTime" ) if ( $item eq "hdt_io" || $item =~ 'lfd_io' || ($item =~ 'csv_io' ));
  @ds = ( "AvgDisksecPerRead", "AvgDisksecPerReadB", "AvgDisksecPerWrite", "AvgDisksecPerWriteB", "Frequency_PerfTime" ) if ( $item =~ "lfd_lat" || ($item =~ 'csv_lat' ));

  @ds = ( "BytesReceivedPersec", "BytesSentPersec" ) if ( $item eq "hyp-net" && $hyperv );
  @ds = ( "DiskReadBytesPersec", "DiskWriteBytesPerse" ) if ( $item eq "hyp-disk" && $hyperv );
  @ds = ( "TotalPhysMemory",     "TotalPhysMemory" )     if ( $item eq "hyp-mem"  && $hyperv );
  @ds = ( "PercentTotalRunTime", "Timestamp_PerfTime", "Frequency_PerfTime", "vCPU" ) if $item eq "hyp-cpu";

  #### SOLARIS SECTION
  @ds = ( "cpu_used", "cpu_used" )      if $item eq "solaris_zone_cpu";
  @ds = ( "cpu_used_perc", "cpu_used_perc" )      if $item eq "solaris_zone_os_cpu";
  @ds = ( "phy_mem_us", "cap_used_in_perc", "allocated_memory", "phy_mem_us_in_perc"  )      if $item eq "solaris_zone_mem";
  @ds = ( "phy_net_in_perc", "phy_net_in_perc" )      if $item eq "solaris_zone_net";
  @ds = ( "cpu_util", "cpu_util" )      if $item eq "s_ldom_c";
  @ds = ( "mem_allocated", "mem_allocated" )      if $item eq "s_ldom_m";
  @ds = ( "cpu_perc", "cpu_perc" )      if $item eq "s10_g_mem" || $item eq "s10_z_mem";
  @ds = ( "mem_perc", "mem_perc" )      if $item eq "s10_g_cpu" || $item eq "s10_z_cpu";


  ## prepare variables for up to 5 stream results and help variables
  my @ids  = ( "first",   "second",   "third",   "fourth",   "fifth" );
  my @rids = ( "first_r", "second_r", "third_r", "fourth_r", "fifth_r" );    # up to 5 streams help var

  foreach $rrd (@rrd_files) {                                                # LPM cycle
    chomp($rrd);
    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

    if ( $i == 0 ) {
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
      next;
    }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
  } ## end foreach $rrd (@rrd_files)
  for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$ids[$k]=$rids[$k]${i}"; }

  my $kbmb = 1024;
  if ( $item =~ "vmw-net" || $item =~ "vmnet" ) {
    $kbmb = 1000;
  }
  if ( $item eq "vmw-swap" ) {
    $kbmb = 1000;
  }
  if ( $item eq "dsarw" || $item eq "dslat" ) {
    $kbmb = 1;
  }
  if ( $item eq "hdt_io" || ($item =~ 'lfd_io' ) || ($item =~ 'csv_io' )) {
    $kbmb = 1;
  }
  if ( $item eq "lfd_lat" || ($item =~ 'csv_lat' )) {
    $kbmb = 1000;
  }
  if ($item eq "hdt_data" || ($item =~ 'lfd_dat' ) || ($item =~ 'csv_dat' )) {
    $kbmb = 1000000;
  }
  if ( $item eq "vmw-proc" ) {
    $kbmb = 100;
  }
  if ( $item eq "vmw-ready" ) {
    $kbmb = 1;
  }
  if ( $hyperv && ( $item =~ "vmnet" || $item =~ "vmdisk" || $item =~ "hyp-disk" || $item =~ "hyp-net") ) {
    $kbmb *= 1000 * 1000;
  }

  $cmd .= $legend_head;

  if ( $item eq "vmw-mem" ) {
    $gprint_frm = "%6.1lf";
    $cmd .= " CDEF:usage=$ids[0],$kbmb,/,$kbmb,/";

    #    $cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/,$kbmb,/";
    #    $cmd .= " CDEF:pagein_b=$ids[2],$kbmb,/";  # baloon is MB ?
    #    $cmd .= " CDEF:pagein_b_nf=$ids[2],$kbmb,/";  # baloon is MB ?
    $cmd .= " CDEF:pageout_b_nf=$ids[2],$kbmb,/,$kbmb,/";    # baloon is MB ?
    $cmd .= " CDEF:pagein_b=$ids[1],$kbmb,/,$kbmb,/";        # active
    $cmd .= " CDEF:pagein_b_nf=$ids[1],$kbmb,/,$kbmb,/";     # active
    $cmd .= " AREA:usage#00FF00:\" $leg_usage  \"";
    $cmd .= " GPRINT:usage:AVERAGE:\" $gprint_frm\"";
    $cmd .= " GPRINT:usage:MAX:$gprint_frm";
    $cmd .= " $t2";
  } ## end if ( $item eq "vmw-mem")
  elsif ( $item eq "vmw-proc" ) {
    $cmd .= " CDEF:CPU_usage_Proc=$ids[0],$kbmb,/";          # orig
    $cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/";

    $cmd .= " CDEF:vCPU=$ids[1],1,/";                                                            # number
    $cmd .= " CDEF:host_MHz=$ids[2],1000,/,1000,/";                                              # to be in MHz
    $cmd .= " CDEF:CPU_usage=$ids[3],1,/";                                                       # MHz
    $cmd .= " CDEF:CPU_usage_res=CPU_usage,host_MHz,/,vCPU,/,100,*";                             # usage proc counted
                                                                                                 #    $cmd .= " CDEF:pagein_b=CPU_usage_res,1,/"; # counted
                                                                                                 #    $cmd .= " CDEF:pagein_b=CPU_usage_Proc,1,/"; # orig from counter metric
    $cmd .= " CDEF:pagein_b_raw=CPU_usage_Proc,UN,CPU_usage_res,CPU_usage_Proc,IF";
    $cmd .= " CDEF:pagein_b=pagein_b_raw,UN,UNKN,pagein_b_raw,100,GT,100,pagein_b_raw,IF,IF";    # cut more than 100%, VMware does the same
  } ## end elsif ( $item eq "vmw-proc")
  elsif ( $item eq "vmw-ready" ) {
    $cmd .= " CDEF:pagein_bn=$ids[0],200,/,$ids[1],/";

    # care for start time for CPU_ready
    my $stime = 0;
    if ( -f $CPU_ready_time_file ) {
      if ( open( FF, "<$CPU_ready_time_file" ) ) {
        $stime = (<FF>);
        close(FF);

        # print STDERR "12062 \$stime $stime\n";
        chomp $stime;
        $stime *= 1;
      } ## end if ( open( FF, "<$CPU_ready_time_file"...))
    } ## end if ( -f $CPU_ready_time_file)
    $cmd .= " CDEF:pagein_b=TIME,$stime,LT,0,pagein_bn,IF";
    $cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/";
  } ## end elsif ( $item eq "vmw-ready")
  elsif ( $item eq "hyp-cpu" ) {
    # $cmd .= " CDEF:CPU_usage_Proc=$ids[0],$ids[1],/,$ids[2],*,100000,/,100,/";    # % to be in cores it was 25 but wrong or should it be 100 ?
    $cmd .= " CDEF:CPU_usage_Proc=$ids[0],$ids[1],/,$ids[2],*,100000,/,100,/,$ids[3],*";    # % to be in cores
                                                                                 #$cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/";

    $cmd .= " CDEF:vcpu=$ids[3],1,/";       # number
    #$cmd .= " CDEF:host_MHz=$ids[2],1000,/,1000,/";    # to be in MHz
    #$cmd .= " CDEF:CPU_usage=$ids[3],1,/";  # MHz
    #$cmd .= " CDEF:CPU_usage_res=CPU_usage,host_MHz,/,vCPU,/,100,*"; # usage proc counted
    #    $cmd .= " CDEF:pagein_b=CPU_usage_res,1,/"; # counted
    #    $cmd .= " CDEF:pagein_b=CPU_usage_Proc,1,/"; # orig from counter metric
    #$cmd .= " CDEF:pagein_b_raw=CPU_usage_Proc,UN,CPU_usage_res,CPU_usage_Proc,IF";
    # $cmd .= " CDEF:pagein_b=CPU_usage_Proc,UN,0,CPU_usage_Proc,IF";
    $cmd .= " CDEF:pagein_b=CPU_usage_Proc";

    # $cmd .= " CDEF:pagein_b=pagein_b_raw,UN,UNKN,pagein_b_raw,100,GT,100,pagein_b_raw,IF,IF";  # cut more than 100%, VMware does the same
  } ## end elsif ( $item eq "hyp-cpu")
  elsif ( $item eq "solaris_zone_cpu" || $item eq "solaris_zone_os_cpu" || $item eq "solaris_zone_net" || $item eq "s10_g_cpu" || $item eq "s10_z_cpu" || $item eq "s10_g_mem" || $item eq "s10_z_mem" || $item eq "s_ldom_c"){
    $cmd .= " CDEF:pagein_b=$ids[0],1,/";
  }
  elsif ( $item eq "solaris_zone_mem"){
    $cmd .= " CDEF:phy_mem_us=$ids[0],1,/";
    $cmd .= " CDEF:cap_used_in_perc=$ids[1],1,/";
    $cmd .= " CDEF:allocated_memory=$ids[2],1,/";
    $cmd .= " CDEF:phy_mem_us_in_perc=$ids[3],1,/";
    $cmd .= " CDEF:phy_mem_used_res=phy_mem_us,1000,/,1000,/";
    $cmd .= " CDEF:res1=allocated_memory,cap_used_in_perc,/";
    $cmd .= " CDEF:res2=res1,phy_mem_us_in_perc,*";
    $cmd .= " CDEF:res3=res2,1000,/,1000,/";

  }
  elsif ($item eq "s_ldom_m"){
    $cmd .= " CDEF:mem_allocated=$ids[0],1,/";
    $cmd .= " CDEF:mem_allocated_a=mem_allocated,100000,/,10000,/";
  }
  else {
    if ( $filter > 0 ) {
      if (($item eq "hdt_data") || ($item eq "hdt_io") || ($item =~ 'lfd_dat' || $item =~ 'lfd_io' ) || ($item =~ 'csv_dat' || $item =~ 'csv_io' )) {
        $cmd .= " CDEF:pagein_b_tmp=$ids[0],$ids[2],/,$ids[3],*,$kbmb,/";
        $cmd .= " CDEF:pagein_b=pagein_b_tmp,$filter,GT,0,pagein_b_tmp,IF";
        # print STDERR "14466 \$item $item \$kbmb $kbmb @rrd_files\n";
        $cmd .= " CDEF:pagein_b_nf=pagein_b,-1,*";
        $cmd .= " CDEF:pageout_b_nf_tmp=$ids[1],$ids[2],/,$ids[3],*,$kbmb,/";
        $cmd .= " CDEF:pageout_b_nf=pageout_b_nf_tmp,$filter,GT,0,pageout_b_nf_tmp,IF";
      }
      elsif ($item =~ "lfd_lat" || ($item =~ 'csv_lat' )) {
        $cmd .= " CDEF:pagein_b_tmp=$ids[1],0,EQ,0,$ids[0],$ids[1],/,$ids[4],/,$kbmb,*,IF";
        $cmd .= " CDEF:pagein_b=pagein_b_tmp,$filter,GT,0,pagein_b_tmp,IF";
        $cmd .= " CDEF:pagein_b_nf=pagein_b,-1,*";
        $cmd .= " CDEF:pageout_b_nf_tmp=$ids[3],0,EQ,0,$ids[2],$ids[3],/,$ids[4],/,$kbmb,*,IF";
        $cmd .= " CDEF:pageout_b_nf=pageout_b_nf_tmp,$filter,GT,0,pageout_b_nf_tmp,IF";
      }
      else {
        $cmd .= " CDEF:pagein_b=$ids[0],$kbmb,/,$filter,GT,0,$ids[0],$kbmb,/,IF";
        $cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/,$filter,GT,0,$ids[1],$kbmb,/,IF";
        # $cmd .= " CDEF:pagein_b_nf=$ids[0],-$kbmb,/";
        $cmd .= " CDEF:pagein_b_nf=pagein_b,-1,*";
      }
    } ## end if ( $filter > 0 )
    else {
      $cmd .= " CDEF:pagein_b=$ids[0],$kbmb,/";
      $cmd .= " CDEF:pageout_b_nf=$ids[1],$kbmb,/";
      $cmd .= " CDEF:pagein_b_nf=$ids[0],-$kbmb,/";
    }
  } ## end else [ if ( $item eq "vmw-mem")]

  if ( !( $item eq "vmw-disk" || $item eq "vmdisk" || $item eq "vmw-net" || $item eq "vmnet" || $item eq "vmw-proc" || $item eq "vmw-ready" || $item eq "hyp-net" || $item eq "hyp-mem" || $item eq "hyp-cpu" || $item eq "solaris_zone_cpu" || $item eq "solaris_zone_os_cpu" || $item eq "solaris_zone_mem"  || $item eq "solaris_zone_net" || $item eq "s10_g_cpu" || $item eq "s10_z_cpu" || $item eq "s10_g_mem" || $item eq "s10_z_mem" || $item eq "s_ldom_c" || $item eq "s_ldom_m" )  ) {
    $cmd .= " LINE1:pageout_b_nf#0080FF:\" $leg_write  \"";
    $cmd .= " GPRINT:pageout_b_nf:AVERAGE:\" $gprint_frm\"";
    $cmd .= " GPRINT:pageout_b_nf:MAX:$gprint_frm";
  }

  $cmd .= " $t2";
  if ( $item eq "vmw-disk" || $item eq "vmdisk" || $item eq "vmw-net" || $item eq "vmnet" || $item eq "vmw-proc" || $item eq "vmw-ready" || $item eq "hyp-net" || $item eq "hyp-mem" || $item eq "hyp-cpu" || $item eq "solaris_zone_cpu" || $item eq "solaris_zone_os_cpu" || $item eq "solaris_zone_net" || $item eq "s10_g_cpu" || $item eq "s10_z_cpu" || $item eq "s10_g_mem" || $item eq "s10_z_mem" || $item eq "s_ldom_c") {
    $cmd .= " LINE1:pagein_b#FF4040:\" $leg_read  \"";
  }
  elsif ( $item eq "solaris_zone_mem" || $item eq "s_ldom_m"){
  #  $cmd .= " CDEF:pagein_b_res1=pagein_b,1000,/,1000,/";
  #  $cmd .= " LINE:pagein_b_res1#FF4040:\" $leg_read  \"";
  }
  else {
    $cmd .= " LINE1:pagein_b_nf#FF4040:\" $leg_read  \"";
  }
  if ( $item eq "solaris_zone_mem"){
    $cmd .= " LINE:res3#000000:\" $leg_write  \"";
    $cmd .= " GPRINT:res3:AVERAGE:\" $gprint_frm\"";
    $cmd .= " GPRINT:res3:MAX:$gprint_frm";
    my $t3         = "COMMENT:\\n";
    $cmd .= " $t3";
    $cmd .= " LINE:phy_mem_used_res#FF4040:\" $leg_read  \"";
    $cmd .= " GPRINT:phy_mem_used_res:AVERAGE:\"  $gprint_frm\"";
    $cmd .= " GPRINT:phy_mem_used_res:MAX:$gprint_frm";

  }
  elsif ($item eq "s_ldom_m" ){
    $cmd .= " LINE:mem_allocated_a#FF4040:\" $leg_read  \"";
    $cmd .= " GPRINT:mem_allocated_a:AVERAGE:\"  $gprint_frm\"";
    $cmd .= " GPRINT:mem_allocated_a:MAX:$gprint_frm";
  }
  else{
    $cmd .= " GPRINT:pagein_b:AVERAGE:\" $gprint_frm\"";
    $cmd .= " GPRINT:pagein_b:MAX:$gprint_frm";
  }
  if ( $item eq "vmw-ready" ) {

    # necessary to get MAX value to be able to prepare %RDY
    #my $r_end   = "now";
    #my $r_start = "-1$type";
    #if ( ! $start_unix eq '' && ! $end_unix eq '' ) {
    #  if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
    #    $r_start = $start_unix;
    #    $r_end   = $end_unix;
    #  }
    #}
    #RRDp::cmd qq("graph" "example.png"
    #  "--end" "$r_end" "--start" "$r_start"
    #  "DEF:usage=$rrd:CPU_ready_ms:AVERAGE"
    #  "DEF:num_cpu=$rrd:vCPU:AVERAGE"
    #  "PRINT:usage:MAX:%6.2lf"
    #  "PRINT:num_cpu:MAX:%6.0lf"
    #);
    #my $row = RRDp::read;
    #chomp($$row);
    ## print STDERR "$$row\n";
    #(undef, my $max_num, my $v_cpu) = split("\n",$$row);
    #$max_num *=1;
    #my $max_num_ax =100/ $max_num ;
    #$v_cpu *=1;
    ## print STDERR "\$max_num $max_num \$v_cpu $v_cpu\n";

    #$cmd .= " $t2";
    #$cmd .= " CDEF:pagein_b_rdy_leg=pagein_b,200,/,$v_cpu,/";
    ##$cmd .= " CDEF:pagein_b_rdy=pagein_b_rdy_leg,$max_num,*,100,/";
    #$cmd .= " LINE1:pagein_b_rdy#0000FF:\" RDY divided by vCPU              \"";
    #$cmd .= " GPRINT:pagein_b_rdy_leg:AVERAGE:\" $gprint_frm\"";
    #$cmd .= " GPRINT:pagein_b_rdy_leg:MAX:$gprint_frm";

    #$cmd .= " --right-axis $max_num_ax:0";
  } ## end if ( $item eq "vmw-ready")

  if ( $item eq "vmw-proc" || $item eq "vmw-ready" || $item eq "hyp-cpu") {
    $cmd .= " $t2";
    if ( $item eq "vmw-proc" ) {
      $cmd .= " CDEF:vcpu=pageout_b_nf,$kbmb,*";
      $cmd .= " COMMENT:\"   vCPU[units]\"";
    }
    elsif ( $item eq "vmw-ready" ) {
      $cmd .= " CDEF:vcpu=pageout_b_nf,$kbmb,*";
      $cmd .= " COMMENT:\"      vCPU[units]\"";
    }
    elsif ( $item eq "hyp-cpu" ) {
      #$cmd .= " CDEF:vcpu=pageout_b_nf,$kbmb,*";
      $cmd .= " COMMENT:\"   vCPU[units]\"";
    }

    $cmd .= " GPRINT:vcpu:AVERAGE:\"%6.0lf\"";
    $cmd .= " GPRINT:vcpu:MAX:\"%6.0lf\"";
  } ## end if ( $item eq "vmw-proc"...)
  if ( $item eq "vmw-mem") {
    $cmd .= " $t2";
    # find MemoryCount GB
    my $mem_count_gb = "undefined";
    my $cpu_html = "$wrkdir/$server/$host/cpu.csv";
    my @cpu_array;
    if (( -f $cpu_html ) && open( FC, "< $cpu_html" )) {
      @cpu_array = <FC>;
      close(FC);
      chomp $lpar_slash;
      #print STDERR "15140 CPU-HTML: $cpu_html\n";
      #print STDERR "15141 ,$lpar_slash,\n";
      $lpar_slash =~ s/ \[.*\]//g;
      my ($grep_vm) = grep /^$lpar_slash,/, @cpu_array;
      if ( defined $grep_vm && $grep_vm ne '' ) {
        chomp $grep_vm;
        my ( undef,$v_cpu, $reser_mhz, undef,$shares, $shares_value, $os, $power_state, $tools_status, undef, $memorySizeMB ) = split /,/,$grep_vm;
        $mem_count_gb = sprintf( "$gprint_frm", $memorySizeMB/1024);
      }
    }
    $cmd .= " COMMENT:\"   Memory Count     $mem_count_gb\"";
  }
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #      " VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  # print STDERR "12945 graph_vmw_disk \$cmd $cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_vmw_disk: $item : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_vmw_disk

sub LPM_easy_trend {

  # this is for case e.g. HMC change, similar to LPM
  # join data streams from rrd files in one data_stream
  # call:
  #  my ($result_cmd,$result_stream_1,...,$result_stream_x) = LPM_easy($path_to_find_files,$data_stream_1,...,$data_stream_x);
  # no limit for x
  # $result_cmd is CMD string for RRD

  my $file_pth = shift @_;    # path to find files
  $file_pth =~ s/ /\\ /g;
  my $no_name = "";

  my @files = (<$file_pth$no_name>);    # unsorted, workaround for space in names
                                        # print STDERR "found pool files: @files\n";

  my @ds = @_;

  #print STDERR "000 in sub LPM_easy \@ds @ds\n";

  # prepare help variables
  my $prep_names = "";
  for ( my $x = 0; $x < @ds; $x++ ) { $prep_names .= "var" . $x . "," }
  my @ids = split( ",", "$prep_names" );
  $prep_names = "";
  for ( my $x = 0; $x < @ds; $x++ ) { $prep_names .= "var_r" . $x . "," }
  my @rids = split( ",", "$prep_names" );

  my $i = -1;
  my $j;
  my $rrd = "";
  my $cmd = "";

  foreach my $rrd (@files) {    # LPM alias cycle
    chomp($rrd);

    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}_1m=\"$rrd\":$ds[$k]:AVERAGE:start=-1m"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}_3m=\"$rrd\":$ds[$k]:AVERAGE:start=-3m"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}_1y=\"$rrd\":$ds[$k]:AVERAGE:start=-1y"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

    if ( $i == 0 ) {
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_1m=$ids[$k]${i}_1m"; }
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_3m=$ids[$k]${i}_3m"; }
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_1y=$ids[$k]${i}_1y"; }
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
      next;
    } ## end if ( $i == 0 )
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_1m=$rids[$k]${j}_1m,UN,$ids[$k]${i}_1m,$rids[$k]${j}_1m,IF"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_3m=$rids[$k]${j}_3m,UN,$ids[$k]${i}_3m,$rids[$k]${j}_3m,IF"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}_1y=$rids[$k]${j}_1y,UN,$ids[$k]${i}_1y,$rids[$k]${j}_1y,IF"; }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
  } ## end foreach my $rrd (@files)

  my $ret_string = "";
  for ( my $k = 0; $k < @ds; $k++ ) { $ret_string .= "$rids[$k]${i}_1m,"; }
  for ( my $k = 0; $k < @ds; $k++ ) { $ret_string .= "$rids[$k]${i}_3m,"; }
  for ( my $k = 0; $k < @ds; $k++ ) { $ret_string .= "$rids[$k]${i}_1y,"; }
  for ( my $k = 0; $k < @ds; $k++ ) { $ret_string .= "$rids[$k]${i},"; }

  #print STDERR "001 $cmd,split(",",$ret_string)\n";
  return ( $cmd, split( ",", $ret_string ) );
} ## end sub LPM_easy_trend

# it identifies all files under certain server and returns the one with the most recent timestamp
# it is just to find out most recent rrd file for the last_time purposes

sub rrd_file_actual {
  my ($rrd_file) = @_;

  # print STDERR "12163 detail-graph-cgi.pl sub rrd_file_actual \$rrd_file $rrd_file\n";
  # LPM identifies all servers, however not all HMC under servers
  # Add here data from all HMcs
  # take care if NMON or not
  my $lpar_name      = basename($rrd_file);
  my $server_dir_tmp = dirname($rrd_file);
  my $server_dir     = dirname($server_dir_tmp);
  my $file_pth       = "$server_dir/*/$lpar_name";

  $file_pth =~ s/ /\\ /g;
  my $no_name          = "";
  my $rrd_upd_time_max = 0;

  my @files = (<$file_pth$no_name>);    # unsorted, workaround for space in names
  foreach my $rrd_act (@files) {
    chomp($rrd_act);
    my $rrd_upd_time = ( stat("$rrd_act") )[9];
    if ( $rrd_upd_time < $rrd_upd_time_max ) {
      next;
    }
    if ( ( $rrd_file =~ "--NMON--" && $rrd_act =~ "--NMON--" ) || ( $rrd_file !~ "--NMON--" && $rrd_act !~ "--NMON--" ) ) {
      $rrd_upd_time_max = $rrd_upd_time;
      $rrd_file         = $rrd_act;
    }
  } ## end foreach my $rrd_act (@files)

  # print STDERR "12188 detail-graph-cgi.pl sub rrd_file_actual \$rrd_file $rrd_file\n";

  return $rrd_file;
} ## end sub rrd_file_actual

sub graph_trendcluster {
  my $host     = shift;
  my $server   = shift;
  my $lpar     = shift;
  my $type     = shift;
  my $name_out = shift;
  my $type_sam = shift;
  my $upper    = shift;
  my $detail   = shift;
  my $t        = "COMMENT: ";
  my $t2       = "COMMENT:\\n";
  my $step_new = $step;
  my $xgrid    = "--x-grid=MONTH:1:MONTH:1:MONTH:1:0:%b";

  if ( $upper eq '' || $upper == 0 ) {
    $upper = 0.1;
  }

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt = " ";
  }

  my $vertical_label = "--vertical-label=\"CPU GHz\"";
  my $rrd            = "";
  my $header         = "";

  if ( $item eq "trendcluster" ) {

    #$host =~ s/^cluster_//;
    #$header = "Cluster: $host :$last_txt year";
    $header = "CPU pool :$last_txt year";
    $rrd    = "$wrkdir/$server/$host/cluster.rrc";
  } ## end if ( $item eq "trendcluster")
  elsif ( $item eq "trendrp" ) {

    #$header = "Resourcepool: $lpar :$last_txt year";
    $header = "CPU pool :$last_txt year";
    $rrd    = "$wrkdir/$server/$host/$lpar.rrc";
  } ## end elsif ( $item eq "trendrp")
  elsif ( $item eq "trendvm" ) {
    $header = "VM : $last_txt year";
    $rrd    = "$wrkdir/vmware_VMs/$lpar.rrm";
  }
  elsif ( $item eq "trendvmem" ) {
    $header         = "VM : $last_txt year";
    $rrd            = "$wrkdir/vmware_VMs/$lpar.rrm";
    $vertical_label = "--vertical-label=\"Memory in GBytes\"";
  }
  elsif ( $item eq "trendrpmem" ) {
    $header         = "Resourcepool : $last_txt year";
    $rrd            = "$wrkdir/$server/$host/$lpar.rrc";
    $vertical_label = "--vertical-label=\"Memory in GBytes\"";
  }
  elsif ( $item eq "trendclmem" ) {
    $header         = "Cluster : $last_txt year";
    $rrd            = "$wrkdir/$server/$host/cluster.rrc";
    $vertical_label = "--vertical-label=\"Memory in GBytes\"";
  }
  else {
    error("creating graph : $host:$server:$lpar:$type_sam:$type:$item:trend");
    return;
  }

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # Trend for yearly chart 1y back  + 1y forward
  #print "creating graph : $host:$server:$lpar:$type_sam:$type:trend\n" if $DEBUG ;
  #error ( "creating graph : $host:$server:$lpar:$type_sam:$type:trend");

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  $rrd =~ s/:/\\:/g;
  $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );

  my $cmd .= "$graph_cmd \"$name_out\"";
  $cmd    .= " --title \"$header - trend\"";
  $cmd    .= " --start now-1$type";
  $cmd    .= " --end now+1$type";
  $cmd    .= " --imgformat PNG";
  $cmd    .= " --slope-mode";
  $cmd    .= " $no_minor";
  $cmd    .= " --width=$width_trend";
  $cmd    .= " --height=$height";
  $cmd    .= " --step=$step";
  $cmd    .= " --lower-limit=0.00";
  $cmd    .= " --color=BACK#$pic_col";
  $cmd    .= " --color=SHADEA#$pic_col";
  $cmd    .= " --color=SHADEB#$pic_col";
  $cmd    .= " --color=CANVAS#$pic_col";
  $cmd    .= " --alt-autoscale-max";
  $cmd    .= " --upper-limit=0.1";
  $cmd    .= " $vertical_label";
  $cmd    .= " --units-exponent=1.00";
  $cmd    .= " --alt-y-grid";
  $cmd    .= " $no_legend";
  $cmd    .= " $disable_rrdtool_tag";
  $cmd    .= " $font_def";
  $cmd    .= " $font_tit";
  $cmd    .= " $xgrid";

  $cmd .= " DEF:cpu_MHz=\"$rrd\":CPU_total_MHz:AVERAGE"   if $item eq "trendcluster";
  $cmd .= " DEF:cpu_MHz=\"$rrd\":CPU_reservation:AVERAGE" if $item eq "trendrp";
  $cmd .= " CDEF:cur=cpu_MHz,1000,/"                      if ( $item eq "trendrp" || $item eq "trendcluster" );

  $cmd .= " DEF:cpu_limit=\"$rrd\":CPU_limit:AVERAGE" if $item eq "trendrp";
  $cmd .= " CDEF:cpu_limg=cpu_limit,1000,/"           if $item eq "trendrp";

  my $usage = "CPU_usage_MHz";
  $usage = "CPU_usage"        if $item eq "trendvm";
  $usage = "Memory_active"    if $item eq "trendvmem";
  $usage = "Memory_active_KB" if $item eq "trendrpmem";
  $usage = "Memory_active_KB" if $item eq "trendclmem";
  $cmd .= " DEF:mem_gra=\"$rrd\":Memory_granted:AVERAGE" if $item eq "trendvmem";
  $cmd .= " CDEF:mem_g=mem_gra,1000000,/" if $item eq "trendvmem";

  my $kbmb = 1000;
  $kbmb = 1000000 if $item eq "trendvmem";
  $kbmb = 1000000 if $item eq "trendrpmem";
  $kbmb = 1000000 if $item eq "trendclmem";

  $cmd .= " DEF:cpuutiltot_MHz_1m=\"$rrd\":$usage:AVERAGE:start=-1m";
  $cmd .= " CDEF:utiltotd=cpuutiltot_MHz_1m,$kbmb,/";
  $cmd .= " DEF:cpuutiltot_MHz_3m=\"$rrd\":$usage:AVERAGE:start=-3m";
  $cmd .= " CDEF:utiltotm=cpuutiltot_MHz_3m,$kbmb,/";
  $cmd .= " DEF:cpuutiltot_MHz_y=\"$rrd\":$usage:AVERAGE:start=-1y";
  $cmd .= " CDEF:utiltoty=cpuutiltot_MHz_y,$kbmb,/";
  $cmd .= " DEF:cpuutiltot_MHz=\"$rrd\":$usage:AVERAGE";
  $cmd .= " CDEF:utiltot=cpuutiltot_MHz,$kbmb,/";

  $cmd .= " VDEF:Dd=utiltotd,LSLSLOPE";
  $cmd .= " VDEF:Hd=utiltotd,LSLINT";
  $cmd .= " CDEF:utiltottrendd=utiltotd,POP,Dd,COUNT,*,Hd,+";
  $cmd .= " VDEF:Dm=utiltotm,LSLSLOPE";
  $cmd .= " VDEF:Hm=utiltotm,LSLINT";
  $cmd .= " CDEF:utiltottrendm=utiltotm,POP,Dm,COUNT,*,Hm,+";
  $cmd .= " VDEF:Dy=utiltoty,LSLSLOPE";
  $cmd .= " VDEF:Hy=utiltoty,LSLINT";
  $cmd .= " CDEF:utiltottrendy=utiltoty,POP,Dy,COUNT,*,Hy,+";
  $cmd .= " COMMENT:\"   Average   \\n\"";

  $cmd .= " AREA:cur#00FF00:\" Total/reservation GHz  \"" if ( $item eq "trendrp" || $item eq "trendcluster" );
  $cmd .= " GPRINT:cur:AVERAGE:\" %6.2lf\""               if ( $item eq "trendrp" || $item eq "trendcluster" );
  $cmd .= " $t2"                                          if ( $item eq "trendrp" || $item eq "trendcluster" );

  #    $cmd .= " AREA:mem_g#00FF00:\" Memory granted GB     \"" if $item eq "trendvmem";
  #    $cmd .= " GPRINT:mem_g:AVERAGE:\" %6.2lf\""              if $item eq "trendvmem";
  #    $cmd .= " $t2"                                           if $item eq "trendvmem";

  $cmd .= " LINE1:cpu_limg#000000:\" Limit in GHz           \"" if $item eq "trendrp";
  $cmd .= " GPRINT:cpu_limg:AVERAGE:\" %6.2lf\""                if $item eq "trendrp";
  $cmd .= " $t2"                                                if $item eq "trendrp";

  my $unit = "GHz";
  $unit = "GB" if $item eq "trendvmem";
  $unit = "GB" if $item eq "trendrpmem";
  $unit = "GB" if $item eq "trendclmem";

  $cmd .= " LINE1:utiltot#FF0000:\" Utilization in $unit     \"";
  $cmd .= " GPRINT:utiltot:AVERAGE:\" %6.2lf\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendd#0088FF:\" Utilization in $unit       - last 1 month trend\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendm#80FFFF:\" Utilization in $unit       - last 3 months trend\"";
  $cmd .= " $t2";
  $cmd .= " LINE2:utiltottrendy#FF8080:\" Utilization in $unit       - last year trend\"";
  $cmd .= " $t2";
  $cmd .= " HRULE:0#000000";

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_trendcluster

sub graph_AS400_jobs {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT:\" \"";
  my $t2         = "COMMENT:\"\\n\"";
  my $step_new   = $step;
  my $last       = "COMMENT: ";
  my $head       = "COMMENT:                            Average   Max\\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  # due to historical reports where is specific time passed
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT:\" \\g\"";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST: %4.0lf";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }

  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";              # just a trick to place there something otherwise rrdtool complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_AS400_jobs: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $lpar eq '' ) {
    return 0;                              # some workaround, it should not go here
  }

  my $rrd = "$wrkdir/$server/$host/$lpar--AS400--/S0200ASPJOB.mmm";

  #my $rrd = "$wrkdir/$server/$host/$lpar/S0200ASPJOB.mmm";
  if ( $item eq "ADDR" ) {
    $rrd = "$wrkdir/$server/$host/$lpar--AS400--/S0200ADDR.mmm";

    #$rrd = "$wrkdir/$server/$host/$lpar/S0200ADDR.mmm";
  }

  # print STDERR "10753 detail-graph-cgi.pl \$rrd $rrd\n";

  if ( !-f "$rrd" ) {
    error( "$rrd does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $lpar_slash = $lpar;    # = human_vmware_name($lpar);

  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  $lpar_slash_nmon =~ s/\(.*\)//;

  my $header_prefix  = "LPAR :";
  my $head_pref      = "";
  my $legend_head    = " COMMENT:\"                     Avrg     Max\\n\"";
  my $vertical_label = "--vertical-label=\"JOBS & THREADS\"";
  my $leg_write      = "JOBS Total  ";
  my $leg_read       = "JOBS active ";
  my $leg_usage      = "Threads     ";
  my $gprint_frm     = "%6.0lf";
  my $upper_limit    = 0.1;

  if ( $item eq "ASP" ) {
    $head_pref      = "";
    $legend_head    = " COMMENT:\"[GB]            Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=GB";
    $leg_write      = "Total";
    $leg_read       = "Used ";
    $leg_usage      = "";
  } ## end if ( $item eq "ASP" )

  if ( $item eq "ADDR" ) {
    $head_pref      = "";
    $legend_head    = " COMMENT:\"[PERCENT]            Avrg      Max\\n\"";
    $vertical_label = "--vertical-label=%";
    $leg_write      = "Total";
    $leg_read       = "Used ";
    $leg_usage      = "";
  } ## end if ( $item eq "ADDR" )

  my $last_txt = " last ";
  my $lpar_slash_nmon_head = $lpar_slash_nmon;

  #  my $header_prefix = "VM :";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
    $last_txt  = "";
    if ($item eq "S0200ASPJOB") {
      $head_pref = "JOBS";
      $lpar_slash_nmon_head = "" if $server eq "as400";
    }
    $header_prefix = $head_pref;
    if ( $wpar == 1 ) {
      $header_prefix = $head_pref;
    }
  } ## end if ( $detail == 2 )

  my $header = "$header_prefix $lpar_slash_nmon_head :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  if ( $type =~ "d" ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    $l =~ s/:/\\:/g;
    $t = "COMMENT:Updated\\:\" $l \"";

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );
    my $m       = "";
    my $i       = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );

        # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
        if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ ) {

          #print "m : $m\n" if $DEBUG ;
          #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;

          $last = "";

          #if ( $m_arr[2] > 0 ) {
          #  my $last_mem = sprintf( "%6.2d",$m_arr[2]) / 1024;
          #  my $last_mem_g = sprintf( "%.2f",$m_arr[2] / 1024);
          #  $last ="COMMENT:Last memory consumption in GB         $last_mem_g";
          #}
        } ## end if ( $m_arr[1] =~ /\d/...)
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( $type =~ "d" )

  # for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;    #actual is the first
                           #lpm_find_files ($wrkdir,$server,$host,$lpar,"\$lpar_slash",$lpm_suff,$type_sam,$type,"",\@rrd_files,$cgi);
                           # print STDERR "10543 detail-graph-cgi.pl lpm_find_files is returning array @rrd_files\n";

  # print STDERR "graphing AS400_jobs   : $rrd: $host:$server:$lpar_slash:$type_sam:$type:$start_time:$end_time\n";

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper_limit";
  $cmd .= " $vertical_label";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  $cmd .= " --base=1024";

  my $i = -1;
  my $j;

  ## define data stream names
  my @ds = ( "par3", "par4", "par5", "par6", "par7", "par8" );

  ## prepare variables for up to 6 stream results and help variables
  my @ids  = ( "first",   "second",   "third",   "fourth",   "fifth",   "sixth" );
  my @rids = ( "first_r", "second_r", "third_r", "fourth_r", "fifth_r", "sixth_r" );    # up to 6 streams help var

  foreach $rrd (@rrd_files) {                                                           # LPM cycle
    chomp($rrd);
    $i++;
    $j = $i - 1;
    $rrd =~ s/:/\\:/g;

    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

    if ( $i == 0 ) {
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
      next;
    }
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
  } ## end foreach $rrd (@rrd_files)
  for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$ids[$k]=$rids[$k]${i}"; }

  my $kbmb = 1024;
  if ( $item =~ "vmw-net" || $item =~ "vmnet" ) {
    $kbmb = 1000;
  }
  if ( $item eq "vmw-swap" ) {
    $kbmb = 1000;
  }
  if ( $item eq "dsarw" ) {
    $kbmb = 1;
  }
  if ( $item eq "vmw-proc" ) {
    $kbmb = 100;
  }
  if ( $item eq "vmw-ready" ) {
    $kbmb = 1;
  }

  $cmd .= $legend_head;

  if ( $item eq "S0200ASPJOB" ) {
    $cmd .= " CDEF:jobs_total=$ids[0]";
    $cmd .= " CDEF:jobs_activ=$ids[4]";

    # value threads is in S0200PROCS:6
    # so it is necessary the same precedure - LPM
    # for LPM
    @rrd_files = ();
    $lpm_suff  = "rrl";
    $cgi       = 0;
    $rrd       = "$wrkdir/$server/$host/$lpar--AS400--/S0200PROCS.mmm";

    #$rrd = "$wrkdir/$server/$host/$lpar/S0200PROCS.mmm";
    $rrd_files[0] = $rrd;    #actual is the first
                             #lpm_find_files ($wrkdir,$server,$host,$lpar,"\$lpar_slash",$lpm_suff,$type_sam,$type,"",\@rrd_files,$cgi);
                             # print STDERR "11723 detail-graph-cgi.pl lpm_find_files is returning array @rrd_files\n";

    # print STDERR "11725 graphing AS400_jobs   : $rrd: $host:$server:$lpar_slash:$type_sam:$type:$start_time:$end_time\n";
    $i = -1;

    #$j ;

    ## define data stream names
    @ds   = ("par8");
    @ids  = ("seventh");
    @rids = ("seventh_r");    # up to 1 streams help var

    foreach $rrd (@rrd_files) {    # LPM cycle
      chomp($rrd);
      $i++;
      $j = $i - 1;
      $rrd =~ s/:/\\:/g;

      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " DEF:$ids[$k]${i}=\"$rrd\":$ds[$k]:AVERAGE"; }

      if ( $i == 0 ) {
        for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$ids[$k]${i}"; }
        next;
      }
      for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$rids[$k]${i}=$rids[$k]${j},UN,$ids[$k]${i},$rids[$k]${j},IF"; }
    } ## end foreach $rrd (@rrd_files)
    for ( my $k = 0; $k < @ds; $k++ ) { $cmd .= " CDEF:$ids[$k]=$rids[$k]${i}"; }

    $cmd .= " CDEF:threads=$ids[0]";
    $cmd .= " LINE1:threads#00FF00:\" THREADS     \"";
    $cmd .= " GPRINT:threads:AVERAGE:\" %6.0lf\"";
    $cmd .= " GPRINT:threads:MAX:%6.0lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:jobs_total#0080FF:\" JOBS total  \"";
    $cmd .= " GPRINT:jobs_total:AVERAGE:\" %6.0lf\"";
    $cmd .= " GPRINT:jobs_total:MAX:%6.0lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:jobs_activ#FF4040:\" JOBS active \"";
    $cmd .= " GPRINT:jobs_activ:AVERAGE:\" %6.0lf\"";
    $cmd .= " GPRINT:jobs_activ:MAX:%6.0lf";
    $cmd .= " $t2";
  } ## end if ( $item eq "S0200ASPJOB")
  elsif ( $item eq "ASP" ) {
    $cmd .= " CDEF:total_MB=$ids[1]";
    $cmd .= " CDEF:total=total_MB,1024,/";
    $cmd .= " CDEF:used_proc=$ids[2]";
    $cmd .= " CDEF:used=total,100,/,used_proc,*";
    $cmd .= " AREA:total#00FF00:\" TOTAL   \"";
    $cmd .= " GPRINT:total:AVERAGE:\" %6.0lf\"";
    $cmd .= " GPRINT:total:MAX:%6.0lf";
    $cmd .= " $t2";
    $cmd .= " AREA:used#FF4040:\" USED    \"";
    $cmd .= " GPRINT:used:AVERAGE:\" %6.0lf\"";
    $cmd .= " GPRINT:used:MAX:%6.0lf";
    $cmd .= " $t2";
  } ## end elsif ( $item eq "ASP" )
  elsif ( $item eq "ADDR" ) {
    $cmd .= " CDEF:perm=$ids[0]";
    $cmd .= " CDEF:temp=$ids[1]";
    $cmd .= " CDEF:temp_256MB=$ids[2]";
    $cmd .= " CDEF:temp_4GB=$ids[3]";
    $cmd .= " CDEF:perm_256MB=$ids[4]";
    $cmd .= " CDEF:perm_4GB=$ids[5]";

    #    $cmd .= " CDEF:total=total_MB,1024,/";
    #    $cmd .= " CDEF:used_proc=$ids[2]";
    #    $cmd .= " CDEF:used=total,100,/,used_proc,*";
    $cmd .= " LINE1:perm#FF0000:\" PERM         \"";         #"#FF0000", "#0000FF", "#C0C0C0", "#FFFF00", "#00FFFF", "#FFA500", "#00FF00", "#808080", "#FF00FF", "#800080"
    $cmd .= " GPRINT:perm:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:perm:MAX:%6.3lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:temp#0000FF:\" TEMP         \"";
    $cmd .= " GPRINT:temp:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:temp:MAX:%6.3lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:temp_256MB#C0C0C0:\" temp 256MB   \"";
    $cmd .= " GPRINT:temp_256MB:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:temp_256MB:MAX:%6.3lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:temp_4GB#FFFF00:\" temp 4GB     \"";
    $cmd .= " GPRINT:temp_4GB:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:temp_4GB:MAX:%6.3lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:perm_256MB#00FFFF:\" perm 256MB   \"";
    $cmd .= " GPRINT:perm_256MB:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:perm_256MB:MAX:%6.3lf";
    $cmd .= " $t2";
    $cmd .= " LINE1:perm_4GB#FFA500:\" perm 4GB     \"";
    $cmd .= " GPRINT:perm_4GB:AVERAGE:\" %6.3lf\"";
    $cmd .= " GPRINT:perm_4GB:MAX:%6.3lf";
    $cmd .= " $t2";

  } ## end elsif ( $item eq "ADDR" )
  else {
    error( "not known item $item " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #      " VRULE:0#000000";

  $cmd =~ s/\\"/"/g;

  # print STDERR "$cmd\n";
  my $ret = "";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : AS400_jobs: $item : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_AS400_jobs

sub graph_as4job {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $upper       = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;             # 1 do not show entitle area
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $as4job      = shift;             # rrd filename of job
  my $t           = "COMMENT:\" \"";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT:\" \"";
  my $rigid       = "\n --rigid";

  # print STDERR "sub graph_as4job ,$host,$server,$lpar,$type,$item,$type_sam,$graph_hours,$as4job\n";
  # $item as4job_dbusy, as4job_diskio_sec

  # $lpar .= "--AS400--";

  my $in_cores     = 0;    # graph for vmware is in MHz
  my $one_core_hz  = 1;
  my $one_core_mhz = 1;
  my $one_core_ghz = 1;

  my $keep_virtual = 0;
  if ( $ENV{KEEP_VIRTUAL} ) {
    $keep_virtual = $ENV{KEEP_VIRTUAL};    # keep number of virt processors in RRD --> etc/.magic
  }
  else {
    $keep_virtual = 0;
  }

  # print STDERR "in graph_as4job \$upper $upper\n";

  if ( $upper eq '' || $upper =~ "entitle" || $upper == 0 ) {    # entitle is badly sent param
    $upper = 0.1;
  }

  my $units_exponent = "--units-exponent=1.00";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_as4job: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  my $header_prefix = "JOB:";
  $header_prefix = "DISK:" if $item =~ /dbusy/ || $item =~ /diskio_sec/;

  my $rrd = "$wrkdir/$server/$host/$lpar--AS400--/JOB/$as4job.mmc";
  $rrd = "$wrkdir/$server/$host/$lpar--AS400--/DSK/$as4job.mmc" if $item =~ /dbusy/ || $item =~ /diskio_sec/;

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $lpar_slash = $lpar;
  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;
  $lpar_slash_nmon =~ s/--AS400--//;
  $lpar_slash_nmon .= " : $as4job";

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt      = " ";
    $header_prefix = "";
  }

  my $header = "$header_prefix $lpar_slash_nmon :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  #  my $vcpu = 0;

  if ( ( $type =~ "d" && $graph_hours == 0 ) || $vmware ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    if ( $type =~ "d" ) {
      $l =~ s/:/\\:/g;
      $t = "COMMENT:Updated\\:\" $l \"";
    }

    # get LAST value from RRD
    #RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    #my $row = RRDp::read;
    #chomp($$row);
    #my @row_arr = split(/\n/,$$row);
    # #print STDERR "12092 detail-graph-cgi.pl \@row_arr @row_arr\n";
    #my $m = "";
    #my $i = 0;

    #foreach $m (@row_arr) {
    #  chomp($m);
    #   $i++;
    #   if ( $i == 3 ) {
    #     my @m_arr = split(/ /,$m);
    #     # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
    #     if ($m_arr[1] =~ /\d/ && $m_arr[5] =~ /\d/ && $m_arr[6] =~ /\d/ ) {
    #       # print STDERR "\n$m_arr[1] $m_arr[5] $m_arr[6]";# if $DEBUG ;
    #       my $util = "";
    #       $util = sprintf ("%5.2f",$m_arr[1]) if $item eq "as4job_proc";
    #       $util = sprintf ("%5.2f",$m_arr[6]/1000) if $item eq "as4job_core";
    #       $util = sprintf ("%5.2f",$m_arr[5]) if $item eq "as4job_disk";
    #       $last ="COMMENT:\"Last utilization      $util\"";
    #     }
    #   }
    #}
  } ## end if ( ( $type =~ "d" &&...))

  #  my $start_time = "now-1$type";
  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $v_label = "CPU %";
  $v_label = "CPU cores"   if $item eq "as4job_core";
  $v_label = "Disk IO"     if $item eq "as4job_disk";
  $v_label = "Disk Busy %" if $item eq "as4job_dbusy";
  $v_label = "Disk IO/sec" if $item eq "as4job_diskio_sec";

  my $vertical_label = "--vertical-label=\"$v_label\"";
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  # print STDERR " detail-graph-cgi.pl 11276 creating graph : $host:$server:$lpar_slash:$type_sam:$type:$name_out:$rrd:$entitle:$item\n"; # if $DEBUG ;

  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;

  my $cmd = "";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";

  #  $cmd .= " --upper-limit=$upper";
  $cmd .= " $vertical_label";
  $cmd .= " $units_exponent";        #--units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  #  for LPM
  my @rrd_files;
  my $lpm_suff = "rrl";
  my $cgi      = 0;
  $rrd_files[0] = $rrd;              #actual is the first

  lpm_find_files( $wrkdir, $server, $host, $lpar, "\$lpar_slash", $lpm_suff, $type_sam, $type, "", \@rrd_files, $cgi );

  #print STDERR "lpm_find_files is returning @rrd_files\n";

  my $i = -1;                        # iterators for LPM cycle
  my $j;

  my $cores_mhz = "cores";           # for vmware graph legend
  if ( $in_cores == 2 ) {
    $cores_mhz = "     ";
  }
  my $gprint_form = "8.2lf";

  foreach my $rrd_tot (@rrd_files) {    # LPM cycle
    chomp($rrd_tot);

    # LPM identifies all servers, however not all HMC under servers
    # Add here data from all HMcs
    my $lpar_name  = basename($rrd_tot);
    my $server_dir = dirname($rrd_tot) . "/..";
    my $file_pth   = "$server_dir/*/$lpar_name";

    $file_pth =~ s/ /\\ /g;
    my $no_name = "";

    my @files = (<$file_pth$no_name>);    # unsorted, workaround for space in names
    foreach $rrd (@files) {               # LPM alias cycle
      chomp($rrd);

      $i++;
      $j = $i - 1;
      $rrd =~ s/:/\\:/g;

      $cmd .= " DEF:cur${i}=\"$rrd\":par3:AVERAGE" if $item eq "as4job_proc" || $item eq "as4job_dbusy";
      $cmd .= " DEF:cur${i}=\"$rrd\":par8:AVERAGE" if $item eq "as4job_core";
      $cmd .= " DEF:cur${i}=\"$rrd\":par7:AVERAGE" if $item eq "as4job_disk";
      $cmd .= " DEF:cur${i}=\"$rrd\":par4:AVERAGE" if $item eq "as4job_diskio_sec";

      if ( $i == 0 ) {
        $cmd .= " CDEF:curr${i}=cur${i}";
        next;
      }
      $cmd .= " CDEF:curr${i}=curr${j},UN,cur${i},curr${j},IF";
    } ## end foreach $rrd (@files)
  } ## end foreach my $rrd_tot (@rrd_files)
  if ( $item eq "as4job_core" ) {
    $cmd .= " CDEF:cur=curr${i},1000,/";
  }
  else {
    $cmd .= " CDEF:cur=curr${i}";
  }

  #        $cmd .= " CDEF:curgt${i}=cur${i},1,/";
  $cmd .= " CDEF:curg=cur,UN,0,cur,IF,1,/";    #avg across entire season - d,w

  $cmd .= " COMMENT:\"                 AvgRun    Avg      Max\\n\"";

  if ( $entitle != 1 ) {

    # default one, show entitlement area
    # else do not show entitled area --> customers request

    $cmd .= " LINE:cur#FF0000:\" Utilization\"";
    $cmd .= " GPRINT:cur:AVERAGE:\" %5.2lf\"";
    $cmd .= " GPRINT:curg:AVERAGE:\" %5.2lf\"";
    $cmd .= " GPRINT:cur:MAX:\" %5.2lf\"";
    $cmd .= " $t2";
  } ## end if ( $entitle != 1 )

  $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #  "VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  # print STDERR "$cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_as4job : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_as4job

sub graph_aspmem {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT: ";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $head        = "COMMENT:                            Average   \\n";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph ASP MEM: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $lpar_orig   = $lpar;
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $upper_limit = "0.1";

  my $vertical_label = "--vertical-label=GBytes";
  my $header         = "ASP : last $text";
  if ( $item =~ m/cap_free/ ) {
    $header = "ASP Free: $lpar : last $text";
  }
  elsif ( $item =~ m/cap_used/ ) {
    $header = "ASP Used: $lpar : last $text";
  }
  elsif ( $item =~ m/cap_proc/ ) {
    $header         = "ASP percent : last $text";
    $vertical_label = "--vertical-label=percent";
    $upper_limit    = "100";
  }
  elsif ( $item =~ m/dsk_svc_as/ ) {
    $header = "DSK SVC: $lpar : last $text";
    $vertical_label = "--vertical-label=millisec";
  }
  elsif ( $item =~ m/dsk_wait_as/ ) {
    $header = "DSK WAIT: $lpar : last $text";
    $vertical_label = "--vertical-label=millisec";
  }
  elsif ( $item =~ m/dsk_latency/ ) {
    $header = "DSK LATENCY: $lpar : last $text";
    $vertical_label = "--vertical-label=millisec";
  }

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";         # nope string
    $header         = "ASP Memory: $text";
    my $lpart = ": $lpar ";
    $lpart    = "" if $server eq "as400";
    if ( $item =~ m/cap_free/ ) {
      $header = "ASP Free$lpart:$text";
    }
    elsif ( $item =~ m/cap_used/ ) {
      $header = "ASP Used$lpart:$text";
    }
    elsif ( $item =~ m/cap_proc/ ) {
      $header = "ASP %$lpart:$text";
    }
    elsif ( $item =~ m/dsk_svc_as/ ) {
      $header = "DSK SVC$lpart:$text";
    }
    elsif ( $item =~ m/dsk_wait_as/ ) {
      $header = "DSK WAIT$lpart:$text";
    }
    elsif ( $item =~ m/dsk_latency/ ) {
      $header = "DSK LAT$lpart:$text";
    }
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "";                       # just a trick

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "ASP : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)

  my $aspdir = "$wrkdir/$server/$host/$lpar--AS400--/ASP";
  if ( $item =~ m/dsk_svc_as/ || $item =~ m/dsk_wait_as/ || $item =~ m/dsk_latency/) {
     $aspdir = "$wrkdir/$server/$host/$lpar--AS400--/LTC";
  }
  my @files  = "";
  if ( $type_sam =~ "m" || $type_sam =~ "x" ) {
    opendir( DIR, "$aspdir" ) || error( " directory does not exists : $aspdir" . __FILE__ . ":" . __LINE__ ) && return 0;
    my @files_unsorted = ();
    if ( $item =~ m/dsk_svc_as/ || $item =~ m/dsk_wait_as/ || $item =~ m/dsk_latency/) {
      @files_unsorted = grep( /LTC.*Parm1\.mmc$/, readdir(DIR) );
    }
    else {
      @files_unsorted = grep( /Parm3\.mmc$/, readdir(DIR) );
    }
    closedir(DIR);
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  } ## end if ( $type_sam =~ "m" ...)

  # print STDERR "16351 detail-graph-cgi.pl $wrkdir/$server/$host/$lpar ,@files,\n";
  my $file = "";
  my $i    = 0;
  my $cmd  = "";
  my $j    = 0;

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }

  if ( ($detail < 1 || $detail ==9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper_limit";
  $cmd .= " \\\"$vertical_label\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";

  my $alloc_free = "allocated memory in GB\\:";
  $alloc_free = "free memory in GB\\:     "  if $item eq "cap_free";
  $alloc_free = "used memory in percent\\: " if $item eq "cap_proc";

  if ( $item =~ m/dsk_svc_as/ ) {
    $cmd .= " COMMENT:\\\"Disk service time in millisec       average    maximum\\l\\\"" if "$type" !~ "d";
    $cmd .= " COMMENT:\\\"Disk service time in millisec       average    maximum     last\\l\\\"" if "$type" =~ "d";
  }
  elsif ( $item =~ m/dsk_wait_as/) {
    $cmd .= " COMMENT:\\\"Disk wait time in millisec          average    maximum\\l\\\"" if "$type" !~ "d";;
    $cmd .= " COMMENT:\\\"Disk wait time in millisec          average    maximum     last\\l\\\"" if  ("$type" =~ "d" );
  }
  elsif ( $item =~ m/dsk_latency/) {
    $cmd .= " COMMENT:\\\"Disk latency time in millisec       average    maximum\\l\\\"" if "$type" !~ "d";;
    $cmd .= " COMMENT:\\\"Disk latency time in millisec       average    maximum     last\\l\\\"" if  ("$type" =~ "d" );
  }
  else {
    if ( "$type" =~ "d" ) {
      $cmd .= " COMMENT:\\\"Current $alloc_free    average    maximum     last\\l\\\"";
    }
    else {
      $cmd .= " COMMENT:\\\"Current $alloc_free    average    maximum\\l\\\"";
    }
  }
  my $gtype = "AREA";

  my $found_at_least_one = 0;
  my @bubble_info = ();
  my $last_update = 0;
  my $found = -1; # colors

  foreach $file (@files) {
    chomp($file);
    # prepare color according ASP number
    #my $found = "";
    #if ( $item =~ m/dsk_svc_as/ || $item =~ m/dsk_wait_as/) {
    #  $found = $file;
    #  $found =~ s/LTC//;
    #  $found =~ s/Parm1\.mmc//;
    #  $found = ($found + 1 ) % ( $color_max + 1 );
    #}
    #else {
    #  $found = ( ( substr $file, 3, 3 ) + 1 ) % ( $color_max + 1 );    # ASP033 see 033 as color, +1 cus color 33 is not nice
    #}
    $found++;
    $found = $found % ( $color_max + 1 );

    $found_at_least_one = 1;

    # avoid old lpars which do not exist in the period
    my $rrd_upd_time = ( stat("$aspdir/$file") )[9];
    if ( $rrd_upd_time < $req_time ) {
      next;
    }

    # find date of latest file
    # prepare string in format: Updated: Fri Jul 17 10:40:01 2015

    if ( $rrd_upd_time > $last_update ) {
      $last_update = $rrd_upd_time;
    }

    my $ltime = localtime($last_update);
    $ltime =~ s/:/\\\:/g;
    $t = "COMMENT:Updated\\\: $ltime ";

    $lpar = $file;
    $lpar =~ s/Parm3.rrc//;
    my $lpar_space_proc = $lpar;
    my $lpar_space      = $lpar;
    $lpar_space =~ s/\&\&1/\//g;
    my $lpar_space_orig = $lpar_space;    # for cap_
    $lpar_space_orig =~ s/Parm3.mmc//;
    $lpar_space_orig =~ s/:/\\:/g;

    # add spaces to lpar name to have 25 chars total (for formating graph legend)
    for ( my $k = length($lpar_space); $k < 25; $k++ ) {
      $lpar_space .= " ";
    }

    my $fake_name = "";
    if ( $item eq "dsk_svc_as" || $item eq "dsk_wait_as" || $item eq "dsk_latency") {
      $gtype = "LINE";
      my $rrd = "$aspdir/$file";
      #( $fake_name, $bubble_info[$i] ) = asp_fake_name($rrd);
      #$bubble_info[$i] =~ s/:/\\:/g;
      $bubble_info[$i] = "";
      $fake_name = $file;
      $fake_name =~ s/LTC//;
      $fake_name =~ s/Parm1.mmc//;
      # print STDERR "16442 \$fake_name $fake_name\n";
      if ($fake_name == 1) {
        $fake_name = "*SYSTEM";
      }
      elsif ($fake_name < 10) {
        $fake_name = "ASP00$fake_name";
      }
      elsif ($fake_name < 33) {
        $fake_name = "ASP0$fake_name";
      }
      else {
        $fake_name = $fake_name - 32;
        $fake_name = "IASP0$fake_name";
      }
    }
    else {
      ( $fake_name, $bubble_info[$i] ) = asp_fake_name("$aspdir/$file");
      $bubble_info[$i] =~ s/:/\\:/g;
    }

    # print STDERR "16495 detail-graph-cgi.pl after asp_fake_name from $aspdir/$file got \$fake_name $fake_name \$bubble_info[$i] $bubble_info[$i]\n";

    $lpar_space      = $fake_name;
    $lpar_space_proc = $fake_name;

    # to keep same count of characters
    $lpar_space =~ s/\\:/:/g;
    $lpar_space =~ s/Parm3.mmc//;
    $lpar_space = sprintf( "%-25s", $lpar_space );
    $lpar_space =~ s/:/\\:/g;
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/Parm3.mmc//;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%'

    my $wrkdir_managedname_host_file = "$aspdir/$file";
    $wrkdir_managedname_host_file =~ s/:/\\:/g;
    my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
    $wrkdir_managedname_host_file_legend =~ s/Parm3.mmc//;
    $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

    # bulid RRDTool cmd
    if ( $item eq "dsk_svc_as" ) {
      $cmd .= " DEF:curg${i}=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
      $wrkdir_managedname_host_file_legend = ""; # do not clickable legend
    }
    elsif ( $item eq "dsk_wait_as" ) {
      $cmd .= " DEF:curg${i}=\\\"$wrkdir_managedname_host_file\\\":par7:AVERAGE";
      $wrkdir_managedname_host_file_legend = ""; # do not clickable legend
    }
    elsif ( $item eq "dsk_latency" ) {
      $cmd .= " DEF:curgsix${i}=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
      $cmd .= " DEF:curgseven${i}=\\\"$wrkdir_managedname_host_file\\\":par7:AVERAGE";
      $cmd .= " CDEF:curg${i}=curgsix${i},curgseven${i},+";
      $wrkdir_managedname_host_file_legend = ""; # do not clickable legend
    }
    else {
      # total capacity
      $cmd .= " DEF:cur${i}=\\\"$wrkdir_managedname_host_file\\\":par3:AVERAGE";
      $cmd .= " DEF:curfree${i}=\\\"$wrkdir_managedname_host_file\\\":par4:AVERAGE";
      if ( $item eq "cap_free" ) {
        $cmd .= " CDEF:curg${i}=curfree${i},1000,/";
      }
      elsif ( $item eq "cap_used" ) {
        $cmd .= " CDEF:curg${i}=cur${i},curfree${i},-,1000,/";
      }
      else {    #cap_proc
        $gtype = "LINE";
        $cmd .= " CDEF:curg${i}=cur${i},curfree${i},-,cur${i},/,100,*";
      }
    }
    $cmd .= " $gtype:curg${i}$color[$found]:\\\"$lpar_space\\\"";

    $cmd .= " GPRINT:curg${i}:AVERAGE:\\\"     %6.0lf \\\"";
    my $item_day_other = "$item-other";
    if ( "$type" =~ "d" ) {
      $item_day_other = "$item-day";
    }

    $cmd .= " PRINT:curg${i}:AVERAGE:\\\"%6.0lf $delimiter $item_day_other $delimiter $lpar_space_proc $delimiter $color[$found] $delimiter $wrkdir_managedname_host_file_legend\\\"";
    my $new_line = "";
    $new_line = "\\l" if "$type" !~ "d";
    $cmd .= " GPRINT:curg${i}:MAX:\\\" %6.0lf $new_line\\\"";
    $cmd .= " PRINT:curg${i}:MAX:\\\"%6.0lf $delimiter $bubble_info[$i] $delimiter $lpar_space_orig\\\"";
    if ( "$type" =~ "d" ) {
      $cmd .= " GPRINT:curg${i}:LAST:\\\" %6.0lf \\l\\\"";
      $cmd .= " PRINT:curg${i}:LAST:\\\"%6.0lf $delimiter\\\"";
    }

    # put carriage return after each second lpar in the legend
    if ( $j == 1 ) {
      $j = 0;
    }
    else {
      # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
      $j++;
    }
    $i++;
    $gtype = "STACK";
  } ## end foreach $file (@files)

  if ( $found_at_least_one == 0 ) {
    return 0;    # no source has been found
  }

  #  print "Updated: " . localtime($last_update) . "\n";
  #if ( "$type" =~ "d" ) {
  my $ltime = localtime($last_update);
  $ltime =~ s/:/\\\:/g;
  $t = "COMMENT:\\\"Updated\\\: $ltime\\\"";
  $cmd .= " $t";
  $t = "COMMENT:Updated\\\: $ltime";

  #}

  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph ASP Mem rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $t = "" if "$type" !~ "d";
  $png_end_heading = ret_graph_param( $ret, $t );
  return 0;
} ## end sub graph_aspmem

sub asp_fake_name {

  # from full asp filename returns fake filename, cfg text for bubble in legend
  my $full_asp_file_name = shift;
  my @name_parts         = split( "\/", $full_asp_file_name );
  my $file               = $name_parts[-1];
  my $fake_name          = $file;

  # instead of ASP001 return '*SYSTEM'
  $fake_name =~ s/ASP001/*SYSTEM/;    # if it is
  $fake_name =~ s/Parm\d.mmc//;

  my $bubble_info = "";               # legend bubble for lpar (ASP) name
  $file =~ s/Parm\d.mmc//;
  if ( $file =~ /ASP\d\d\d/ ) {
    my $file_cfg = $full_asp_file_name;
    $file_cfg =~ s/Parm\d.mmc/Parm\.cfg/;

    # print STDERR "11957 \$full_asp_file_name $full_asp_file_name \$file_cfg $file_cfg\n";
    if ( !-f "$file_cfg" ) {
      $file_cfg =~ s/Parm\.cfg/Parm1\.cfg/;    #sometimes is there
    }
    if ( -f "$file_cfg" ) {
      open( FF, "<$file_cfg" ) || error( "can't open $file_cfg: $! :" . __FILE__ . ":" . __LINE__ ) && return ( $fake_name, $bubble_info );
      $bubble_info = (<FF>);

      # print STDERR "11961 \$bubble_info $bubble_info\n";
      close(FF);

      # for ASP33 & higher use name "Device name" if exists
      # 'Resource name: IASP, Device description name: IASP, ASP usage: primary iASP,...'
      my $aspnum = $file;
      $aspnum =~ /.*(ASP\d\d\d)/;
      $aspnum = $1;
      my $aspnum_all = $aspnum;
      $aspnum =~ s/ASP//;
      if ( $aspnum > 32 ) {
        ( undef, my $device_name, undef ) = split( /,/, $bubble_info );
        ( undef, $fake_name, undef ) = split( ": ", $device_name );
        $bubble_info .= "ASP number: $aspnum";
      }
    } ## end if ( -f "$file_cfg" )
  } ## end if ( $file =~ /ASP\d\d\d/)

  # print STDERR "12760 detail-graph-cgi.pl \$full_asp_file_name $full_asp_file_name \$fake_name $fake_name \$bubble_info $bubble_info\n";
  return ( $fake_name, $bubble_info );
} ## end sub asp_fake_name

sub graph_aspcap {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $upper       = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;             # 1 do not show entitle area
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t           = "COMMENT:\" \"";
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT:\" \"";
  my $rigid       = "\n --rigid";

  # print STDERR "11921 sub graph_aspcap ,$host,$server,$lpar,$type,$item,$type_sam,$graph_hours\n";

  #  my $in_cores = 1; # graph for vmware is in cores
  my $in_cores     = 0;    # graph for vmware is in MHz
                           #my $in_cores = 2;  # graph for vmware has right axis
  my $one_core_hz  = 1;
  my $one_core_mhz = 1;
  my $one_core_ghz = 1;

  my $keep_virtual = 0;
  if ( $ENV{KEEP_VIRTUAL} ) {
    $keep_virtual = $ENV{KEEP_VIRTUAL};    # keep number of virt processors in RRD --> etc/.magic
  }
  else {
    $keep_virtual = 0;
  }

  if ( !defined $upper || $upper eq '' || $upper =~ "entitle" || $upper == 0 ) {    # entitle is badly sent param
    $upper = 0.1;
  }

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $units_exponent = "--units-exponent=1.00";
  $units_exponent = "" if $vmware;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lpar: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)

  my $header_prefix = "LPAR:";

  $lpar =~ s/&&1/\//g;
  my $rrd = "$wrkdir/$server/$host/$lpar" . "Parm3.mmc";

  # print STDERR "11969 detail-graph-cgi.pl \$rrd $rrd\n";

  ( my $fake_name, undef ) = asp_fake_name($rrd);

  # print STDERR "12023 detail-graph-cgi.pl \$fake_name $fake_name\n";

  my @lpar_path = split( "\/", $lpar );
  my $lpar_space = $lpar_path[-1];
  $lpar_space =~ s/\\:/:/g;
  $lpar_space =~ s/Parm3.mmc//;
  $lpar_space = sprintf( "%-25s", $lpar_space );
  $lpar_space =~ s/:/\\:/g;

  if ( !-f "$rrd" ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $lpar_slash = $lpar_path[-1];

  $lpar_slash =~ s/\&\&1/\//g;       # to show slash and not &&1 which is general replacement for it
  $lpar_slash =~ s/--WPAR--/\//g;    # WPAR delimiter replace
  my $lpar_slash_nmon = $lpar_slash;
  $lpar_slash_nmon =~ s/$NMON//;

  my $last_txt = " last ";
  if ( $detail == 2 ) {
    $last_txt      = " ";
    $header_prefix = "";
    $fake_name = "" if $server eq "as400";
  }

  #my $header = "$header_prefix $lpar_slash_nmon :$last_txt$text";
  my $header = "$header_prefix $fake_name :$last_txt$text";

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {

      # historical reports
      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $header = "$header_prefix $lpar_slash_nmon : $start_human - $end_human";
    } ## end if ( isdigit($start_unix...))
  } ## end if ( !$start_unix eq ''...)
  my $vcpu = 0;

  if ( ( $type =~ "d" && $graph_hours == 0 ) || $vmware ) {
    my $rrd_last = rrd_file_actual("$rrd");
    RRDp::cmd qq(last "$rrd_last");
    my $last_tt = RRDp::read;
    chomp($$last_tt);
    my $l = localtime($$last_tt);

    # following must be for RRD 1.2+
    if ( $type =~ "d" ) {
      $l =~ s/:/\\:/g;
      $t = "COMMENT:Updated\\:\" $l \"";
    }

    # get LAST value from RRD
    RRDp::cmd qq(fetch "$rrd_last" "AVERAGE" "-s $$last_tt-$step" "-e $$last_tt-$step");
    my $row = RRDp::read;
    chomp($$row);
    my @row_arr = split( /\n/, $$row );

    # print STDERR "\@row_arr @row_arr\n";
    my $m = "";
    my $i = 0;

    foreach $m (@row_arr) {
      chomp($m);
      $i++;
      if ( $i == 3 ) {
        my @m_arr = split( / /, $m );
        if ($vmware) {

          # go further only if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $utiltot_mhz = 1;
            my $utiltot_ghz = 1;
            $utiltot_mhz = sprintf( "%e", $m_arr[2] );
            $utiltot_ghz = $utiltot_mhz / 1000;
            $one_core_hz = sprintf( "%e", $m_arr[3] );
            if ($in_cores) {
              my $util = sprintf( "%5.2f", $utiltot_mhz / $one_core_hz * 1000000 );
              if ( $type !~ "d" ) {next}
              $last = "COMMENT:\"Last CPU usage in cores      $util\"";
              if ( $in_cores == 2 ) {
                my $utlt_mhz = sprintf( "%5.0f", $utiltot_mhz );
                my $utlt_ghz = sprintf( "%5.2f", $utiltot_ghz );
                $last = "COMMENT:\"Last CPU usage                $util    $utlt_ghz\"";
              }
            } ## end if ($in_cores)
            else {
              $utiltot_mhz = $utiltot_mhz / 1000;
              my $util = sprintf( "%5.2f", $utiltot_mhz );
              $last = "COMMENT:\"Last utilization            $util\"";
            }

            # get vCPU
            # print STDERR "\$m_arr[18] $m_arr[18]\n";
            $vcpu = $m_arr[18];
            $vcpu = sprintf( "%e", $vcpu );
            $vcpu = $vcpu * 1;
            if ( !defined $vcpu ) {
              $vcpu = 'not defined';
            }
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end if ($vmware)
        else {
          # go further ony if it is a digit (avoid it when NaNQ (== no data) is there)
          if ( $m_arr[1] =~ /\d/ && $m_arr[2] =~ /\d/ && $m_arr[3] =~ /\d/ && $m_arr[4] =~ /\d/ ) {

            #print "m : $m\n" if $DEBUG ;
            #print "\n$m_arr[1] $m_arr[2]" if $DEBUG ;
            my $entitled_cycles = 1;
            my $capped_cycles   = 1;
            my $uncapped_cycles = 1;
            my $curr_proc       = 1;

            if ( $keep_virtual == 1 ) {

              # when KEEP_VIRTUAL then virtual_procs is the second field
              $entitled_cycles = sprintf( "%e", $m_arr[3] );
              $capped_cycles   = sprintf( "%e", $m_arr[4] );
              $uncapped_cycles = sprintf( "%e", $m_arr[5] );
              $curr_proc       = sprintf( "%e", $m_arr[1] );
            } ## end if ( $keep_virtual == ...)
            else {
              $entitled_cycles = sprintf( "%e", $m_arr[2] );
              $capped_cycles   = sprintf( "%e", $m_arr[3] );
              $uncapped_cycles = sprintf( "%e", $m_arr[4] );
              $curr_proc       = sprintf( "%e", $m_arr[1] );
            } ## end else [ if ( $keep_virtual == ...)]
            if ( $entitled_cycles != 0 ) {
              my $util = sprintf( "%5.2f", ( $capped_cycles + $uncapped_cycles ) / $entitled_cycles ) * $curr_proc;

              #print ("\n $util $curr_proc $entitled_cycles $capped_cycles $uncapped_cycles\n");

              # $last ="COMMENT:\"Last utilization in CPU cores  $util\"";
            } ## end if ( $entitled_cycles ...)
          } ## end if ( $m_arr[1] =~ /\d/...)
        } ## end else [ if ($vmware) ]
      } ## end if ( $i == 3 )
    } ## end foreach $m (@row_arr)
  } ## end if ( ( $type =~ "d" &&...))

  #if ( $type =~ "y" ) {
  #  $step_new=86400;
  #}

  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  my $vertical_label = "--vertical-label=\"GBytes\"";
  $vertical_label = "--vertical-label=\"CPU usage in Hz\"" if $vmware && !$in_cores;
  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }

  #print "creating graph : $host:$server:$lpar_slash:$type_sam:$type:$name_out:$rrd:$entitle:$item\n" if $DEBUG ;

  my $cmd = "";

  $cmd .= "$graph_cmd \"$name_out\"";
  if ( $in_cores == 2 && $vmware ) {

    #  $cmd .= " --right-axis-label \"CPU usage in MHz\"";
    $one_core_mhz = int( $one_core_hz / 1000000 );
    $one_core_ghz = $one_core_mhz / 1000;
    if ($yes_right_axis) {
      $cmd .= " --right-axis $one_core_ghz:0";
      $cmd .= " --right-axis-format %1.2lf";
    }
  } ## end if ( $in_cores == 2 &&...)
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= " --title \"$header\"";
  $cmd .= " --start \"$start_time\"";
  $cmd .= " --end \"$end_time\"";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step_new";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper";
  $cmd .= " $vertical_label";
  $cmd .= " $units_exponent";           #--units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  my $cores_mhz = "cores";              # for vmware graph legend
  if ( $in_cores == 2 ) {
    $cores_mhz = "     ";
  }
  my $gprint_form = "8.2lf";

  my $wrkdir_managedname_host_file = "$rrd";
  $wrkdir_managedname_host_file =~ s/:/\\:/g;
  my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
  $wrkdir_managedname_host_file_legend =~ s/Parm3.mmc//;
  $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

  # bulid RRDTool cmd
  # total capacity
  $cmd .= " DEF:totalm=\\\"$wrkdir_managedname_host_file\\\":par3:AVERAGE";
  $cmd .= " CDEF:total=totalm,1000,/";
  $cmd .= " DEF:freem=\\\"$wrkdir_managedname_host_file\\\":par4:AVERAGE";
  $cmd .= " CDEF:free=freem,1000,/";
  $cmd .= " DEF:total_protm=\\\"$wrkdir_managedname_host_file\\\":par5:AVERAGE";
  $cmd .= " CDEF:total_prot=total_protm,1000,/";
  $cmd .= " DEF:free_protm=\\\"$wrkdir_managedname_host_file\\\":par6:AVERAGE";
  $cmd .= " CDEF:free_prot=free_protm,1000,/";

  $cmd .= " CDEF:used=total,free,-";
  $cmd .= " CDEF:used_prot=total_prot,free_prot,-";
  $cmd .= " CDEF:used_noprot=used,used_prot,-";
  $cmd .= " CDEF:free_noprot=free,free_prot,-";

  $cmd .= " COMMENT:\"Memory GB                  Average       Max\\n\"";
  $cmd .= " AREA:used_prot#FF0000:\\\"Used protected    \\\"";
  $cmd .= " GPRINT:used_prot:AVERAGE:\\\"     %6.0lf \\\"";
  $cmd .= " GPRINT:used_prot:MAX:\\\" %6.0lf \\n\"";

  #    if ( "$type" =~ "d" ) {
  #      $cmd .= " GPRINT:used:LAST:\\\" %6.0lf \\l\\\"";
  #    }
  $cmd .= " STACK:used_noprot#FFCC00:\\\"Used not protected\\\"";
  $cmd .= " GPRINT:used_noprot:AVERAGE:\\\"     %6.0lf \\\"";
  $cmd .= " GPRINT:used_noprot:MAX:\\\" %6.0lf \\n\"";
  $cmd .= " STACK:free_prot#009900:\\\"Free protected    \\\"";
  $cmd .= " GPRINT:free_prot:AVERAGE:\\\"     %6.0lf \\\"";
  $cmd .= " GPRINT:free_prot:MAX:\\\" %6.0lf \\n\"";
  $cmd .= " STACK:free_noprot#66FF00:\\\"Free not protected\\\"";
  $cmd .= " GPRINT:free_noprot:AVERAGE:\\\"     %6.0lf \\\"";
  $cmd .= " GPRINT:free_noprot:MAX:\\\" %6.0lf \\\"";

  $cmd .= " $t2";

  $cmd .= " $last";
  $cmd .= " $t2";
  $cmd .= " $t";
  $cmd .= " HRULE:0#000000";

  #  "VRULE:0#000000"

  $cmd =~ s/\\"/"/g;

  #print STDERR "$cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_lpar : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_aspcap

sub timing_debug
{
  my $text     = shift;

  if ( defined $ENV{LPAR2RRD_UI_TIME_DEBUG} ) {
    my $act_time = localtime();
    print STDERR "DEBUG: $0 : $$ : $act_time : $text\n";
  }
  return 1;
}

sub graph_hitachi_percentages {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t2          = "COMMENT:\\n";
  my $lpar_orig   = $lpar;
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $upper_limit = "100";
  my $cmd         = "";
  my $rrd = "";

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph hitachi percentages: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  my $vertical_label = "--vertical-label=CPU usage %";
  my $time = "last $text";

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human"
  }

  my $header = $lpar eq "pool" ? "System : $time" : "CPU %: $lpar : $time";

  if ( $detail == 1 ) {
    if ( $lpar =~ /pool/ ) {
      $header = "System: $host : $time";
    }
  }

  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && defined $start_unix && defined $end_unix
	 && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
	. " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  $cmd .= "$graph_cmd \\\"$name_out\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=$upper_limit";
  $cmd .= " \\\"$vertical_label\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";

  if ($lpar eq "pool") {
    my $cpu = "$wrkdir/$server/$host/SYS-CPU.hrm";
    my $sys1 = "$wrkdir/$server/$host/SYS1.hrm";
    my $sys2 = "$wrkdir/$server/$host/SYS2.hrm";
    my $shr_lpar = "$wrkdir/$server/$host/SHR_LPAR.hrm";
    my $ded_lpar = "$wrkdir/$server/$host/DED_LPAR.hrm";

    my $file_exist_err = 0;
    if ( !-f "$cpu" ) { error( "$cpu does not exists " . __FILE__ . ":" . __LINE__ ); $file_exist_err = 1; }
    if ( !-f "$sys1" ) { error( "$sys1 does not exists " . __FILE__ . ":" . __LINE__ ); $file_exist_err = 1; }
    if ( !-f "$sys2" ) { error( "$sys2 does not exists " . __FILE__ . ":" . __LINE__ ); $file_exist_err = 1; }
    if ( !-f "$shr_lpar" ) { error( "$shr_lpar does not exists " . __FILE__ . ":" . __LINE__ ); $file_exist_err = 1; }
    if ( !-f "$ded_lpar" ) { error( "$ded_lpar does not exists " . __FILE__ . ":" . __LINE__ ); $file_exist_err = 1; }

    if ($file_exist_err) { return 0; }
    else { $rrd = $cpu; }

    $cmd .= " DEF:cpu=\"$cpu\":cpu_usedp:AVERAGE";
    $cmd .= " CDEF:percent=cpu,100,/";
    $cmd .= " DEF:sys1=\"$sys1\":usedp:AVERAGE";
    $cmd .= " CDEF:sys1p=sys1,percent,/";
    $cmd .= " COMMENT:\"[%]                  Avrg\\n\"";
    $cmd .= " AREA:sys1p#FF0000:\" SYS1         \"";
    $cmd .= " GPRINT:sys1p:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " DEF:sys2=\"$sys2\":usedp:AVERAGE";
    $cmd .= " CDEF:sys2p=sys2,percent,/";
    $cmd .= " STACK:sys2p#00FF00:\" SYS2         \"";
    $cmd .= " GPRINT:sys2p:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " DEF:ded_lpar=\"$ded_lpar\":usedp:AVERAGE";
    $cmd .= " CDEF:ded_lparp=ded_lpar,percent,/";
    $cmd .= " STACK:ded_lparp#FF00FF:\" DED_LPAR     \"";
    $cmd .= " GPRINT:ded_lparp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " CDEF:shr_lparp=100,ded_lparp,-,sys2p,-,sys1p,-";
    $cmd .= " STACK:shr_lparp#0000FF:\" SHR_LPAR     \"";
    $cmd .= " GPRINT:shr_lparp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
  } ## end if $;[ar eq "pool"
  else {
    $rrd = "$wrkdir/$server/$host/$lpar.hlm";

    if ( !-f "$rrd" ) {
      error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
      return 0;
    }

    $cmd .= " DEF:usedp=\"$rrd\":usedp:AVERAGE";
    $cmd .= " DEF:delayp=\"$rrd\":delayp:AVERAGE";
    $cmd .= " DEF:idlep=\"$rrd\":idlep:AVERAGE";
    $cmd .= " DEF:iowp=\"$rrd\":iowp:AVERAGE";
    $cmd .= " DEF:niowp=\"$rrd\":niowp:AVERAGE";
    $cmd .= " CDEF:robp=100,usedp,-,delayp,-,idlep,-,iowp,-,niowp,-";
    $cmd .= " CDEF:robp_positive=robp,0,GT,robp,0,IF";
    $cmd .= " COMMENT:\"[%]                  Avrg\\n\"";
    $cmd .= " AREA:usedp#FF0000:\" Used        \"";
    $cmd .= " GPRINT:usedp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " STACK:robp_positive#FF00FF:\" ROB         \"";
    $cmd .= " GPRINT:robp_positive:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " STACK:iowp#00FFFF:\" IO wait     \"";
    $cmd .= " GPRINT:iowp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " STACK:niowp#0000FF:\" Non IO wait \"";
    $cmd .= " GPRINT:niowp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " STACK:delayp#FFFF00:\" Delay       \"";
    $cmd .= " GPRINT:delayp:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " STACK:idlep#00FF00:\" Idle        \"";
    $cmd .= " GPRINT:idlep:AVERAGE:\" %6.1lf\"";
    $cmd .= " $t2";
  } ## end else if $lpar eq "pool"

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt     = RRDp::read;
    my $last_update = localtime($$last_tt);

    # following must be for RRD 1.2+
    $last_update =~ s/:/\\:/g;
    $cmd .= " COMMENT:\"Updated\\\: $last_update\"";
  } ## end if ( $type =~ "d" )

  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_pool : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);

  return 0;
} #end sub_hitachi graph_percentages

sub graph_hitachi_adapters {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t2          = "COMMENT:\\n";
  my $lpar_orig   = $lpar;
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $upper_limit = "0.1";
  my $cmd         = "";

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph adapters: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $vertical_label = "--vertical-label=Read - Bytes/sec - Write";
  my $time = "last $text";
  my $rrd;
  my $header;
  my $output_byte;

  if ( $item eq "hitachi-lan" ) {
    $rrd = "$wrkdir/$server/$host/$lpar.hnm";
    $header = "LAN : $lpar : ";
    $output_byte = "sbyte";
  }
  else {
    $rrd = "$wrkdir/$server/$host/$lpar.hhm";
    $header = "SAN : $lpar : ";
    $output_byte ="wbyte";
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human"
  }

  $header .= "$time";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && defined $start_unix && defined $end_unix
         && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
        . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  if ( !-f $rrd ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " \"$vertical_label\"";
  $cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";

  $cmd .= " DEF:rkbyte=\"$rrd\":rbyte:AVERAGE";
  $cmd .= " DEF:skbyte=\"$rrd\":$output_byte:AVERAGE";
  $cmd .= " CDEF:rbyte_neg=rkbyte,-1000,*";
  $cmd .= " CDEF:sbyte=skbyte,1000,*";
  $cmd .= " CDEF:rmbyte=rkbyte,1000,/";
  $cmd .= " CDEF:smbyte=skbyte,1000,/";
  $cmd .= " COMMENT:\"[MB/sec]             Avrg\\n\"";
  $cmd .= " AREA:rbyte_neg#FF0000:\" Read        \"";
  $cmd .= " GPRINT:rmbyte:AVERAGE:\" %6.1lf\"";
  $cmd .= " $t2";
  $cmd .= " AREA:sbyte#0000FF:\" Write       \"";
  $cmd .= " GPRINT:smbyte:AVERAGE:\" %6.1lf\"";
  $cmd .= " $t2";
  $cmd .= " HRULE:0#000000";

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt     = RRDp::read;
    my $last_update = localtime($$last_tt);

    # following must be for RRD 1.2+
    $last_update =~ s/:/\\:/g;
    $cmd .= " COMMENT:\"Updated\\\: $last_update\"";
  } ## end if ( $type =~ "d" )

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_pool : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_hitachi_adapters

sub graph_hitachi_adapters_agg {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $cmd  = "";
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lparagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $is_lan = $item eq "hitachi-lan-totals";
  my $head = $is_lan ? "LAN" : "SAN";
  my $header = "$head aggregated : last $text";
  if ( $detail == 1 ) {
    $header = "$head aggregated: $host : last $text";
  }
  elsif ( $detail == 2 ) {
    $header = "LPARs aggregated : $text";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  elsif ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  elsif ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }
  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick
    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $header = "$head aggregated: $host : $start_human - $end_human";
  } ## end if ( isdigit($start_unix...))

  my $rrd_type = $is_lan ? qr/.hnm$/ : qr/.hhm$/;
  opendir( DIR, "$wrkdir/$server/$host" ) || error( "can't opendir $wrkdir/$server/$host: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files_unsorted = grep( /$rrd_type/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  closedir(DIR);

  my $vertical_label = "--vertical-label=\"Read - Bytes/sec - Write\"";

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  $cmd .= " COMMENT:\"Average [MB/sec]\\l\"";
  if ( $detail == 1 ) {
    $cmd .= " COMMENT:\"Int        READ     Avg      Max      WRITE     Avg      Max\\l\"";
  }

  my $last_update_time = 0;
  my $i    = 0;
  my $gtype            = "AREA";
  my $color_index       = 0;
  my $cmdx = "";
  my $write_var = $is_lan ? "sbyte" : "wbyte";
  foreach my $file (@files) {
    chomp($file);
    my $file_orig = $file;
    $lpar = $file;
    $lpar =~ s/$rrd_type//; # remove rrd suffix

    my $lpar_space_proc = $lpar;
    my $lpar_space = $lpar;

    # add spaces to lpar name to have 12 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    $lpar_space .= " " x (12 - length($lpar_space));

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%

    my $wrkdir_server_host_file = "$wrkdir/$server/$host/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $wrkdir_server_host_file;
    $wrkdir_server_host_file_legend =~ s/%/%%/g;

    my $rrd_update_time = ( stat($wrkdir_server_host_file) )[9];
    if ( $rrd_update_time > $last_update_time ) {
      $last_update_time = $rrd_update_time;
    }

    $cmd .= " DEF:rkbyte${i}=\"$wrkdir_server_host_file\":rbyte:AVERAGE";
    $cmd .= " CDEF:rbyte${i}_neg=rkbyte${i},-1000,*"; #1024
    $cmd .= " CDEF:rmbyte${i}=rkbyte${i},1000,/";
    $cmd .= " DEF:wkbyte${i}=\"$wrkdir_server_host_file\":$write_var:AVERAGE";
    $cmd .= " CDEF:wbyte${i}=wkbyte${i},1000,*";
    $cmd .= " CDEF:wmbyte${i}=wkbyte${i},1000,/";

    $cmd .= " COMMENT:\"$lpar_space\"";
    $cmd .= " $gtype:rbyte${i}_neg$color[$color_index]:\" \"";
    $cmd .= " GPRINT:rmbyte${i}:AVERAGE:\" %5.2lf \"";
    $cmd .= " GPRINT:rmbyte${i}:MAX:\" %5.2lf        \"";
    $cmd .= " PRINT:rmbyte${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
    $cmd .= " PRINT:rmbyte${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

    $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

    $cmdx .= " $gtype:wbyte${i}$color[$color_index]: ";
    $gtype = "STACK";
    $cmd .= " $gtype:0$color[$color_index]:\" \"";
    $cmd .= " GPRINT:wmbyte${i}:AVERAGE:\" %5.2lf \"";
    $cmd .= " GPRINT:wmbyte${i}:MAX:\" %5.2lf \"";
    $cmd .= " PRINT:wmbyte${i}:AVERAGE:\"%5.2lf $delimiter $color[$color_index]\"";
    $cmd .= " PRINT:wmbyte${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";
    $cmd .= " $t2";

    $color_index = ( $color_index + 1 ) % ( $color_max + 1 );
    $i++;
    $cmd .= " COMMENT:\"\\l\"";
  } ## end foreach $file (@files)

  my $updated_label = "";
  if ( $type =~ "d" ) {
    $last_update_time = localtime($last_update_time);
    $last_update_time =~ s/:/\\\:/g;
    $updated_label = "COMMENT:\"Updated\\\: $last_update_time\"";
    $cmd .= " $updated_label";
    $updated_label = "COMMENT:Updated\\\: $last_update_time";
  }

  $cmd .= " $cmdx";
  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  $png_end_heading = ret_graph_param($ret, $updated_label);
  return 0;
} ## end sub graph_hitachi_adapters_agg

sub graph_power_adapters2 {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $item        = shift;
  my $t2          = "COMMENT:\\n";
  my $lpar_orig   = $lpar;
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $upper_limit = "0.1";
  my $cmd         = "";
  my $sriov_log_port_list;

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph adapters: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";

  if ( $lpar =~ m/totals/ ){
    return (0);
  }
  my $data_packets;
  if ($item =~ /_data/){
    $data_packets="data";
    $item =~ s/_data//;
  }
  elsif ($item =~ /_io/){
    $data_packets = "packets";
    $item =~ s/_io//;
  }

  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $end_time   = $end_unix;
      $xgrid      = "COMMENT:\" \"";    # just a trick to place there something otherwise rrdtool complains
    }
  } ## end if ( !$start_unix eq ''...)
  my $vertical_label = "";
  if ($data_packets =~ m/data/ ){
    $vertical_label = "--vertical-label=Read - Bytes/sec - Write";
  }
  elsif ($data_packets =~ m/packets/ && ($item =~ m/power_lan/ || $item =~ m/power_sri/ || $item =~ m/power_hea/)){
    $vertical_label = "--vertical-label=Read - Packets/sec - Write";
  }
  elsif ($data_packets =~ m/packets/ && $item =~ m/power_san/){
    $vertical_label = "--vertical-label=Read - IOPS - Write";
  }
  elsif ($data_packets =~ m/packets/ && $item =~ m/power_sas/){
    $vertical_label = "--vertical-label=Read - IOPS - Write";
  }
  else{
    $vertical_label = "--vertical-label=Read - NaN - Write";
  }

  my $time = "last $text";
  my $rrd;
  my $header;
  my $output_byte;
  my $done;

  if ( $item =~ m/power_san/) {
    (my $lpar_header, undef) = split ('.r', $lpar);
    $rrd = "$wrkdir/$server/$host/adapters/$lpar";
    $header = "SAN : $lpar_header : $time";
    $output_byte = "sbyte";
  }
  elsif ( $item =~ m/power_sas/) {
    (my $lpar_header, undef) = split ('.r', $lpar);
    $rrd = "$wrkdir/$server/$host/adapters/$lpar";
    $header = "SAS : $lpar_header : $time";
    $output_byte = "sbyte";
  }
  elsif ( $item =~ m/power_lan/) {
    (my $lpar_header, undef) = split ('.r', $lpar);
    $rrd = "$wrkdir/$server/$host/adapters/$lpar";
    $header = "LAN : $lpar_header : $time";
    $output_byte = "sbyte";
  }
  elsif ( $item =~ m/power_hea/) {
    (my $lpar_header, undef) = split ('.r', $lpar);
    $rrd = "$wrkdir/$server/$host/adapters/$lpar";
    $header = "HEA : $lpar_header : $time";
    $output_byte = "sbyte";
  }
  elsif ( $item =~ m/power_sri/) {
    (my $lpar_header, undef) = split ('.r', $lpar);
    if (-e "$basedir/data/$server/$host/sriov_log_port_list.json"){
      $sriov_log_port_list = read_json("$basedir/data/$server/$host/sriov_log_port_list.json");
    }
    $rrd = "$wrkdir/$server/$host/adapters/$lpar";
    $header = "SR-IOV : $lpar_header : $time";
    $output_byte = "sbyte";
  }
  if ( !$start_unix eq '' && !$end_unix eq '' ) {
    if ( isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $start_time = $start_unix;
      $req_time   = $start_time;    # set here too
      $end_time   = $end_unix;
      $xgrid      = "";             # just a trick

      my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
      my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
      $time = "$start_human - $end_human"
    }
  } ## end if ( !$start_unix eq ''...)

  #if ( $detail == 1 ) {
  #  if ( $lpar =~ /pool/ ) {
  #    $header = "System: $host : $time";
  #  }
  #}


  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )

  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && defined $start_unix && defined $end_unix
   && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
  . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  if ( !-f $rrd ) {
    error( "$rrd does not exists " . __FILE__ . ":" . __LINE__ );
    return 0;
  }
  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " \"$vertical_label\"";
  $cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";
  if ( $data_packets =~ m/data/ ){
    my $file_pth = "$wrkdir/$server/*/adapters/$lpar";
    my ( $result_cmd, $read, $write ) = LPM_easy( $file_pth, "0", "0", "read", "write");
    $cmd .= $result_cmd;
    #$cmd .= " DEF:read=\"$rrd\":read:AVERAGE";
    #$cmd .= " DEF:write=\"$rrd\":write:AVERAGE";

    $cmd .= " CDEF:write_B=$write,1024,*";
    $cmd .= " CDEF:read_B=$read,1024,*";

    $cmd .= " CDEF:write_B_neg=write_B,-1,*";
    $cmd .= " CDEF:read_B_neg=read_B,-1,*";

    $cmd .= " CDEF:read_kB=read_B,1024,/";
    $cmd .= " CDEF:write_kB=write_B,1024,/";

    $cmd .= " CDEF:read_MB=read_kB,1024,/";
    $cmd .= " CDEF:write_MB=write_kB,1024,/";

    $cmd .= " CDEF:read_GB=read_MB,1024,/";
    $cmd .= " CDEF:write_GB=write_MB,1024,/";

    $cmd .= " COMMENT:\"[MB/sec]             Avrg      Max\\n\"";
    $cmd .= " AREA:read_B_neg#FF0000:\" Read        \"";
    $cmd .= " GPRINT:read_MB:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:read_MB:MAX:\" %6.1lf\"";
    $cmd .= " $t2";
    $cmd .= " AREA:write_B#0000FF:\" Write       \"";
    $cmd .= " GPRINT:write_MB:AVERAGE:\" %6.1lf\"";
    $cmd .= " GPRINT:write_MB:MAX:\" %6.1lf\"";
  }
  elsif ( $data_packets =~ m/packets/ ){
    my $file_pth = "$wrkdir/$server/*/adapters/$lpar";
    my ( $result_cmd, $ioread, $iowrite ) = LPM_easy( $file_pth, "0", "0", "io_read", "io_write");
    $cmd .= $result_cmd;
    #$cmd .= " DEF:io_read=\"$rrd\":io_read:AVERAGE";
    #$cmd .= " DEF:io_write=\"$rrd\":io_write:AVERAGE";
    $cmd .= " CDEF:io_read=$ioread,1,/";
    $cmd .= " CDEF:io_write=$iowrite,1,/";
    $cmd .= " CDEF:io_write_neg=$iowrite,-1,*";
    $cmd .= " CDEF:io_read_neg=$ioread,-1,*";

    if (($data_packets =~ /packets/) && ($item =~ /power_lan/ || $item =~ /power_sri/ || $item =~ /power_hea/)){
      $cmd .= " COMMENT:\"[Packets]              Avrg    Max\\n\"";
    }
    elsif ($data_packets =~ /packets/ && $item =~ /power_san/ ){
      $cmd .= " COMMENT:\"[IOPS]            Avrg    Max\\n\"";
    }
    elsif ($data_packets =~ /packets/ && $item =~ /power_sas/ ){
      $cmd .= " COMMENT:\"[IOPS]            Avrg    Max\\n\"";
    }
    $cmd .= " AREA:io_read_neg#FF0000:\" Read        \"";
    $cmd .= " GPRINT:io_read:AVERAGE:\" %.0lf\"";
    $cmd .= " GPRINT:io_read:MAX:\" %.0lf\"";
    $cmd .= " $t2";
    $cmd .= " AREA:io_write#0000FF:\" Write       \"";
    $cmd .= " GPRINT:io_write:AVERAGE:\" %.0lf\"";
    $cmd .= " GPRINT:io_write:MAX:\" %.0lf\"";
  }
  $cmd .= " $t2";
  $cmd .= " HRULE:0#000000";

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_pool : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_power_adapters2

sub graph_power_adapters_agg2 {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $item        = shift;
  my $adapter_suffix=shift;
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $done = 0;
  my $cmd  = "";
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lparagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $lan_san; my $data_packets;

  $lan_san = "LAN" if ($item =~ m/power_lan/);
  $lan_san = "HEA" if ($item =~ m/power_hea/);
  $lan_san = "SAN" if ($item =~ m/power_san/);
  $lan_san = "SAS" if ($item =~ m/power_sas/);
  $lan_san = "SRI" if ($item =~ m/power_sri/);
  my @files_aliases = <$wrkdir/$server/*\/$lan_san\_aliases.json>;
  my $interface_aliases;
  foreach my $al_file (@files_aliases){
    (my $host_tmp, undef) = split ("/$lan_san\_aliases", $al_file );
    (undef, $host_tmp) = split ("$server/", $host_tmp);
    $interface_aliases->{$host_tmp} = read_json($al_file);
  }
  $data_packets = "data" if ($item =~ m/data/);
  $data_packets = "packets" if ($item =~ m/io/);
  $item =~ s/_data//;
  $item =~ s/_io//;


  if (!defined $lan_san || !defined $data_packets || $lan_san eq "" || $data_packets eq ""){
    error ( "Not defined item or lpar parameter parsing to graph_power_adapters_agg2 subroutine\n");
    return (0);
  }

  my $head = $lan_san;
  if ($head eq "SRI"){
    $head = "SR-IOV";
  }
  my $header = "$head aggregated : $host : last $text";
  if ( $detail == 1 ) {
    $header =  "$head aggregated : $host : last $text";
  }
  elsif ( $detail == 2 ) {
    $header = "$head aggregated: $text";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  elsif ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  elsif ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }
  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick
    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $header = "$head aggregated: $host : $start_human - $end_human";
  } ## end if ( isdigit($start_unix...))

  my $rrd_type = $adapter_suffix;
  opendir( DIR, "$wrkdir/$server/$host/adapters" ) || error( "can't opendir $wrkdir/$server/$host/adapters: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files_unsorted = grep( /$rrd_type/, readdir(DIR) );
  my @files = sort { lc $a cmp lc $b } @files_unsorted;
  if (!(@files)){
    return;
  }
  closedir(DIR);

  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )
  my $vertical_label;
  if ($data_packets eq "data"){
    $vertical_label = "--vertical-label=\"Read - Bytes/sec - Write\"";
  }
  elsif($data_packets eq "packets"){
    if ($lan_san eq "LAN" || $lan_san eq "HEA" || $lan_san eq "SRI"){
      $vertical_label = "--vertical-label=\"Read - Packets/sec - Write\"";
    }
    else{
      $vertical_label = "--vertical-label=\"Read - IOPS - Write\"";
    }
  }

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  if ( $detail == 1 ) {
    $cmd .= " COMMENT:\"Int                             READ    Avg       Max           WRITE   Avg      Max\\l\"";
  }

  my $file = "";
  my $i    = 0;
  my $gtype            = "AREA";
  my $color_index       = 0;
  my $cmdx = "";
  my $write_var = $lan_san ? "sbyte" : "wbyte";
  my $sriov_aliases;
  if (-e "$basedir/data/$server/$host/sriov_aliases.json"){
    $sriov_aliases = read_json("$basedir/data/$server/$host/sriov_aliases.json");
  }
  foreach $file (@files) {
    my $file_pth = "$wrkdir/$server/*/adapters/$file";
    my ( $result_cmd,  $read,  $write, $io_read, $io_write ) = LPM_easy( $file_pth, $i, "0", "read", "write", "io_read", "io_write");
    $cmd .= $result_cmd;
    $cmd .= " CDEF:r${i}=$read";
    $cmd .= " CDEF:w${i}=$write";
    $cmd .= " CDEF:io_r${i}=$io_read";
    $cmd .= " CDEF:io_w${i}=$io_write";
    chomp($file);
    my $file_orig = $file;

    $lpar = $file;
    $lpar =~ s/$rrd_type//; # remove rrd suffix
    $lpar =~ s/\.//;

    #if ($file =~ m/-V[0-9]*-C[0-9]*/){next;} #hash this if you want Virtual Adapters with XXXXX-V*-C*-T* in name

    my $lpar_space_proc = $lpar;
    my $lpar_space = $lpar;

    # add spaces to lpar name to have 12 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    $lpar_space .= " " x (12 - length($lpar_space));

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%a
    #$lpar_space_proc = $file;
    my $new_alias = "";
    my $new_partition = "";
    if (!defined $interface_aliases->{$host}{$lpar}{alias}){
      foreach my $h (keys %{$interface_aliases}){
        $new_alias = $interface_aliases->{$h}{$lpar}{alias} if defined $interface_aliases->{$h}{$lpar}{alias};
        $new_partition = $interface_aliases->{$h}{$lpar}{partition} if defined $interface_aliases->{$h}{$lpar}{partition};
      }
    }
    else{
      $new_alias = $interface_aliases->{$host}{$lpar}{alias};
      $new_partition = $interface_aliases->{$host}{$lpar}{partition};
    }
    if (defined $lpar && $lpar ne "" && defined $new_alias && $new_alias ne ""){
      $lpar_space_proc = "$new_partition $new_alias";
    }
    else {
      $lpar_space_proc = $lpar;
    }
    my $adapter_legend_png = $lpar_space_proc;
    if (defined $sriov_aliases->{$lpar}){
      $adapter_legend_png = "$adapter_legend_png ($sriov_aliases->{$lpar})";
    }
    while (length($adapter_legend_png) < 30){
      $adapter_legend_png = "$adapter_legend_png ";
    }

    my $wrkdir_server_host_file = "$wrkdir/$server/$host/adapters/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $wrkdir_server_host_file;

    $wrkdir_server_host_file_legend =~ s/%/%%/g;
    (undef, $wrkdir_server_host_file_legend) = split ('adapters/', $wrkdir_server_host_file_legend);
    ($wrkdir_server_host_file_legend, undef) = split ('.', $wrkdir_server_host_file_legend);
    if ( $data_packets eq "data" && $lan_san ne "HEA"){

#      $cmd .= " DEF:read${i}=\"$wrkdir_server_host_file\":read:AVERAGE"; #05/06/2018 saving in kB insted of B
#      $cmd .= " DEF:write${i}=\"$wrkdir_server_host_file\":write:AVERAGE";

      $cmd .= " CDEF:write_kB${i}=w${i},1,/";
      $cmd .= " CDEF:read_kB${i}=r${i},1,/";

      $cmd .= " CDEF:write_B${i}=write_kB${i},1024,*";
      $cmd .= " CDEF:read_B${i}=read_kB${i},1024,*";

      $cmd .= " CDEF:write_MB${i}=write_kB${i},1024,/";
      $cmd .= " CDEF:read_MB${i}=read_kB${i},1024,/";

      $cmd .= " CDEF:write_B_neg${i}=write_B${i},-1,*";
      $cmd .= " CDEF:read_B_neg${i}=read_B${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_B_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:read_MB${i}:MAX:\" %5.2lf        \"";
      $cmd .= " PRINT:read_MB${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_B${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:write_MB${i}:MAX:\" %5.2lf \"";
      $cmd .= " PRINT:write_MB${i}:AVERAGE:\"%5.2lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    if ( $data_packets eq "data" && $lan_san eq "HEA"){

#      $cmd .= " DEF:read${i}=\"$wrkdir_server_host_file\":read:AVERAGE"; #05/06/2018 saving in kB insted of B
#      $cmd .= " DEF:write${i}=\"$wrkdir_server_host_file\":write:AVERAGE";

      $cmd .= " CDEF:write_kB${i}=w${i},1,/";
      $cmd .= " CDEF:read_kB${i}=r${i},1,/";

      $cmd .= " CDEF:write_B${i}=write_kB${i},1024,*";
      $cmd .= " CDEF:read_B${i}=read_kB${i},1024,*";

      $cmd .= " CDEF:write_MB${i}=write_kB${i},1024,/";
      $cmd .= " CDEF:read_MB${i}=read_kB${i},1024,/";

      $cmd .= " CDEF:write_B_neg${i}=write_B${i},-1,*";
      $cmd .= " CDEF:read_B_neg${i}=read_B${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_B_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:read_MB${i}:MAX:\" %5.2lf        \"";
      $cmd .= " PRINT:read_MB${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_B${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:write_MB${i}:MAX:\" %5.2lf \"";
      $cmd .= " PRINT:write_MB${i}:AVERAGE:\"%5.2lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && ($lan_san eq "LAN" || $lan_san eq "SRI" || $lan_san eq "HEA") ){
      #$cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
      #$cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      $cmd .= " CDEF:write_packets${i}=io_w${i},1,/";
      $cmd .= " CDEF:read_packets${i}=io_r${i},1,/";
      $cmd .= " CDEF:write_packets_neg${i}=write_packets${i},-1,*";
      $cmd .= " CDEF:read_packets_neg${i}=read_packets${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_packets_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_packets${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_packets${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_packets${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_packets${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_packets${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && ($lan_san eq "LAN" || $lan_san eq "SRI" || $lan_san eq "HEA") ){
      #$cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
      #$cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      $cmd .= " CDEF:write_packets${i}=io_w${i},1,/";
      $cmd .= " CDEF:read_packets${i}=io_r${i},1,/";
      $cmd .= " CDEF:write_packets_neg${i}=write_packets${i},-1,*";
      $cmd .= " CDEF:read_packets_neg${i}=read_packets${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_packets_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_packets${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_packets${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_packets${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_packets${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_packets${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && $lan_san eq "SAN"){
      #$cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
      #$cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      #$cmd .= " CDEF:write_iops${i}=write_io${i},$hmc_iops_divide,/";
      #$cmd .= " CDEF:read_iops${i}=read_io${i},$hmc_iops_divide,/";
      $cmd .= " CDEF:write_iops${i}=io_w${i},1,/";
      $cmd .= " CDEF:read_iops${i}=io_r${i},1,/";
      $cmd .= " CDEF:write_iops_neg${i}=write_iops${i},-1,*";
      $cmd .= " CDEF:read_iops_neg${i}=read_iops${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_iops_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_iops${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_iops${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_iops${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_iops${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_iops${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && $lan_san eq "SAS"){
#      $cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
#      $cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
#      $cmd .= " CDEF:write_iops${i}=io_w${i},$hmc_iops_divide,/";
#      $cmd .= " CDEF:read_iops${i}=io_r${i},$hmc_iops_divide,/";
      $cmd .= " CDEF:write_iops${i}=io_w${i},1,/";
      $cmd .= " CDEF:read_iops${i}=io_r${i},1,/";
      $cmd .= " CDEF:write_iops_neg${i}=write_iops${i},-1,*";
      $cmd .= " CDEF:read_iops_neg${i}=read_iops${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_iops_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_iops${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_iops${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_iops${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_iops${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_iops${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    $color_index = ( $color_index + 1 ) % ( $color_max + 1 );
    $cmd .= " COMMENT:\"\\l\"";
    $i++;
  } ## end foreach $file (@files)

  $cmd .= " $cmdx";
  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  $png_end_heading = ret_graph_param($ret,"","",$data_packets);
  return 0;
} ## end sub graph_power_adapters_agg2

# XenServer
sub graph_xenserver {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t2          = "COMMENT:\\n";
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $upper_limit = "0.1";
  my $cmd         = "";

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph XenServer stats: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # TODO convert to the new new url format (see XenServerMenu.pm)

  my $cmd_params = my $label_context = my $header = "";

  if ( $item =~ m/cpu-percent/ ) {
    $cmd_params = XenServerGraph::get_params_cpu('percent');
    $label_context = "CPU %";
  } elsif ( $item =~ m/cpu-cores/ ) {
    $cmd_params = XenServerGraph::get_params_cpu('cores');
    $label_context = "CPU cores";
  } elsif ( $item =~ m/memory/ ) {
    $cmd_params = XenServerGraph::get_params_memory();
    $label_context = "MEM";
  } elsif ( $item =~ m/vbd/ ) {
    if ( $item =~ m/vbd-iops/ ) {
      $cmd_params = XenServerGraph::get_params_storage('iops');
      $label_context = "IOPS";
    } elsif ( $item =~ m/vbd-latency/ ) {
      $cmd_params = XenServerGraph::get_params_storage('latency');
      $label_context = "Latency";
    } else {
      $cmd_params = XenServerGraph::get_params_storage();
      $label_context = "Data";
    }
  } elsif ( $item =~ m/lan/ ) {
    $cmd_params = XenServerGraph::get_params_lan();
    $label_context = "LAN";
  }

  my ($graph_info, $rrd);
  if ( $item =~ m/^xen-host/ ) {
    $label_context = "Host " . $label_context;
    if ( $item =~ m/^xen-host-cpu-percent$/ ) {
      $graph_info = XenServerGraph::graph_cpu( 'host', $server, 'percent' );
    } elsif ( $item =~ m/^xen-host-cpu-cores$/ ) {
      $graph_info = XenServerGraph::graph_cpu( 'host', $server, 'cores' );
    } elsif ( $item =~ m/^xen-host-memory$/ ) {
      $graph_info = XenServerGraph::graph_memory( 'host', $server );
    }
    $rrd    = XenServerDataWrapper::get_filepath_rrd_host( $server );
    $header = XenServerGraph::get_header( 'host', $server, $label_context );
  } elsif ( $item =~ m/^xen-disk/ ) {
    $label_context = "Disk " . $label_context;
    if ( $item =~ m/^xen-disk-vbd$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'host', $server, 'vbd', $lpar );
    } elsif ( $item =~ m/^xen-disk-vbd-iops$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'host', $server, 'iops', $lpar );
    } elsif ( $item =~ m/^xen-disk-vbd-latency$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'host', $server, 'latency', $lpar );
    }
    $rrd    = XenServerDataWrapper::get_filepath_rrd_host_disk( $server, $lpar );
    my $sr_uuid = XenServerDataWrapper::complete_sr_uuid( $lpar, $server );
    $header = XenServerGraph::get_header( 'sr', $sr_uuid, $label_context );
  } elsif ( $item =~ m/^xen-lan/ ) {
    $graph_info = XenServerGraph::graph_lan( 'host', $server, $lpar );
    $rrd    = XenServerDataWrapper::get_filepath_rrd_host_lan( $server, $lpar );
    $header = XenServerGraph::get_header( 'network', $lpar, $label_context );
  } elsif ( $item =~ m/^xen-vm/ ) {
    $label_context = "VM " . $label_context;
    if ( $item =~ m/^xen-vm-cpu-percent$/ ) {
      $graph_info = XenServerGraph::graph_cpu( 'vm', $lpar, 'percent' );
    } elsif ( $item =~ m/^xen-vm-cpu-cores$/ ) {
      $graph_info = XenServerGraph::graph_cpu( 'vm', $lpar, 'cores' );
    } elsif ( $item =~ m/^xen-vm-memory$/ ) {
      $graph_info = XenServerGraph::graph_memory( 'vm', $lpar );
    } elsif ( $item =~ m/^xen-vm-vbd$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'vm', $lpar, 'vbd' );
    } elsif ( $item =~ m/^xen-vm-vbd-iops$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'vm', $lpar, 'iops' );
    } elsif ( $item =~ m/^xen-vm-vbd-latency$/ ) {
      $graph_info = XenServerGraph::graph_storage( 'vm', $lpar, 'latency' );
    } elsif ( $item =~ m/^xen-vm-lan$/ ) {
      $graph_info = XenServerGraph::graph_lan( 'vm', $lpar );
    }
    $rrd    = XenServerDataWrapper::get_filepath_rrd_vm( $lpar );
    $header = XenServerGraph::get_header( 'vm', $lpar, $label_context );
  } else {
    error( "Unknown item $item. " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $header .= " : ";
  my $time = "last $text";

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human";
  }

  $header .= "$time";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && defined $start_unix && defined $end_unix
         && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
        . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  if ( !-f $rrd ) {
    error( "$rrd does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  # remove vertical label from Dashboard
  if ( $detail == 2 ) {
    $cmd_params =~ s/ --vertical-label=\".+?\"/ /;
  }

  my ( $cmd_def, $cmd_cdef, $cmd_legend );
  $cmd_def        = $graph_info->{cmd_def};
  $cmd_cdef       = $graph_info->{cmd_cdef};
  $cmd_legend     = $graph_info->{cmd_legend};

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --alt-y-grid";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";
  $cmd .= $cmd_params;

  $cmd .= $cmd_def;
  $cmd .= $cmd_cdef;
  $cmd .= $cmd_legend;
  $cmd .= " HRULE:0#000000";

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt     = RRDp::read;
    my $last_update = localtime($$last_tt);

    # following must be for RRD 1.2+
    $last_update =~ s/:/\\:/g;
    $cmd .= " COMMENT:\"Updated\\\: $last_update\"";
  } ## end if ( $type =~ "d" )

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_xenserver : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_xenserver

sub graph_xenserver_totals {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $t2          = "COMMENT:\\n";
  my $lpar_orig   = $lpar;
  my $req_time    = 0;
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $cmd         = "";

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph XenServer aggregated stats: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $cmd_params = my $label_context = my $header = "";

  if ( $item =~ m/cpu-percent/ ) {
    $cmd_params = XenServerGraph::get_params_cpu('percent');
    $label_context = "CPU %";
  } elsif ( $item =~ m/cpu-cores/ ) {
    $cmd_params = XenServerGraph::get_params_cpu('cores');
    $label_context = "CPU cores";
  } elsif ( $item =~ m/memory/ ) {
    $cmd_params = XenServerGraph::get_params_memory();
    $label_context = ( $item =~ m/memory-free/ ) ? "MEM free" : "MEM used";
  } elsif ( $item =~ m/vbd/ ) {
    if ( $item =~ m/vbd-iops/ ) {
      $cmd_params = XenServerGraph::get_params_storage('iops');
      $label_context = "IOPS";
    } elsif ( $item =~ m/vbd-latency/ ) {
      $cmd_params = XenServerGraph::get_params_storage('latency');
      $label_context = "Latency";
    } else {
      $cmd_params = XenServerGraph::get_params_storage();
      $label_context = "Data";
    }
  } elsif ( $item =~ m/lan/ ) {
    $cmd_params = XenServerGraph::get_params_lan();
    $label_context = "LAN aggregated";
  }

  my @uuids;
  if ( $item =~ m/^xen-host-vm-(.*)-aggr$/ ) {
    $label_context = "VM " . $label_context . " aggregated";
    @uuids = @{ XenServerDataWrapper::get_vm_under_host_list( $server ) };
    $header = XenServerGraph::get_header( 'host', $server, $label_context );
  } elsif ( $item =~ m/^xen-host-(.*)-aggr$/ ) {
    $label_context = "Hosts " . $label_context . " aggregated";
    @uuids = @{ XenServerDataWrapper::get_host_in_pool_list( $server ) };
    $header = XenServerGraph::get_header( 'pool', $server, $label_context );
  } elsif ( $item =~ m/^xen-vm-(.*)-aggr$/ ) {
    $label_context = "VM " . $label_context . " aggregated";
    @uuids = @{ XenServerDataWrapper::get_vm_in_pool_list( $server ) };
    $header = XenServerGraph::get_header( 'pool', $server, $label_context );
  } elsif ( $item =~ m/^xen-lan-(.*)-aggr$/ ) {
    @uuids = @{ XenServerDataWrapper::get_host_interface_list( $server, "lan" ) };
    $header = XenServerGraph::get_header( 'host', $server, $label_context );
  } elsif ( $item =~ m/^xen-pool-vbd(.*)-aggr$/ ) {
    $label_context = "Pool Disk " . $label_context . " aggregated";
    # when generating the graph command, for each UUID run XenServerDataWrapper::get_host_interface_list( $uuid, "storage" );
    @uuids = @{ XenServerDataWrapper::get_host_in_pool_list( $server ) };
    $header = XenServerGraph::get_header( 'pool', $server, $label_context );
  } elsif ( $item =~ m/^xen-disk-(.*)-aggr$/ ) {
    $label_context = "Disk " . $label_context . " aggregated";
    @uuids = @{ XenServerDataWrapper::get_host_interface_list( $server, "storage" ) };
    $header = XenServerGraph::get_header( 'host', $server, $label_context );
  } else {
    error( "Unknown item $item. " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $header .= " : ";
  my $time   = "last $text";

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    if ( $detail != 10 ) {
      $xgrid = "";            # just a trick
    }

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human";
  }

  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  $header .= "$time";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
     if ( $width < 700 && defined $start_unix && defined $end_unix
         && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
        . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  my $last_update_time = 0; # for the "updated" label under graph legends
  $cmd_params = my $cmd_items = my $cmd_legend0 = my $cmd_legend = my $cmd_legend2 = "";
  if ( $item =~ m/^xen-(host-vm|vm|host)-cpu-(percent|cores)-aggr$/ ) {
    my $metric_type = 'percent';
    if ( $item =~ m/cpu-cores/ ) { $metric_type = 'cores'; }
    $cmd_params  = XenServerGraph::get_params_cpu($metric_type);

    my $domain_type = ( $item =~ m/^xen-(host-vm|vm)/ ) ? "vm" : "host";
    my $group_type  = ( $item =~ m/^xen-host-vm/ )      ? "host" : "pool";

    my $legend_helper0 = ( $group_type eq 'host' ) ? "Host" : "Pool";
    my $legend_helper1 = ( $domain_type eq 'vm' )  ? "VM  " : "Host";
    $cmd_legend0  = " COMMENT:\"[%]\"";
    if ( $metric_type eq 'cores' ) { $cmd_legend0 = " COMMENT:\"[cores]\""; }
    $cmd_legend0 .= " COMMENT:\\n";
    $cmd_legend0 .= " COMMENT:\"${legend_helper0}                        ${legend_helper1}                      Avrg      Max\\n\"";

    my ( $graph_info, $filepath, $itemcolor, $itemlabel, $grouplabel );
    my $itemcount = 0;

    foreach my $this_uuid (@uuids) {
      if ( $item =~ m/^xen-(host-vm|vm)/ ) {
        $filepath = XenServerDataWrapper::get_filepath_rrd_vm( $this_uuid );
      } else {
        $filepath = XenServerDataWrapper::get_filepath_rrd_host( $this_uuid );
      }

      if ( !-f $filepath ) {
        error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
        next;
      }

      my $rrd_update_time = ( stat( $filepath ) )[9];
      if ( $rrd_update_time > $last_update_time ) {
        $last_update_time = $rrd_update_time;
      }

      $itemlabel  = XenServerDataWrapper::get_label( $domain_type, $this_uuid );
      $grouplabel = XenServerDataWrapper::get_label( $group_type, $server );
      $itemlabel =~ s/:/\\:/g;
      $grouplabel =~ s/:/\\:/g;
      $itemcolor  = $color[$itemcount % $#color];

      my $graph = XenServerGraph::graph_cpu_aggr( $domain_type, $metric_type, $filepath, $itemcount, $itemcolor, $itemlabel, $grouplabel, $group_type );
      $cmd_items  .= $graph->{cmd_def};
      $cmd_items  .= $graph->{cmd_cdef};
      $cmd_legend .= $graph->{cmd_legend};

      $itemcount++;
    }

  } elsif ( $item =~ m/^xen-(host-vm|vm|host)-memory-(used|free)-aggr$/ ) {
    my $metric_type = 'used';
    if ( $item =~ m/memory-free/ ) { $metric_type = 'free'; }
    $cmd_params     = XenServerGraph::get_params_memory();

    my $domain_type = ( $item =~ m/^xen-(host-vm|vm)/ ) ? "vm" : "host";
    my $group_type  = ( $item =~ m/^xen-host-vm/ )      ? "host" : "pool";

    my $legend_helper0 = ( $group_type eq 'host' ) ? "Host" : "Pool";
    my $legend_helper1 = ( $domain_type eq 'vm' )  ? "VM  " : "Host";
    $cmd_legend0  = " COMMENT:\"[GB]\\n\"";
    $cmd_legend0 .= " COMMENT:\\n";
    $cmd_legend0 .= " COMMENT:\"${legend_helper0}                        ${legend_helper1}                      Avrg      Max\\n\"";

    my ( $graph_info, $filepath, $itemcolor, $itemlabel, $grouplabel );
    my $itemcount = 0;

    foreach my $this_uuid (@uuids) {
      if ( $item =~ m/^xen-(host-vm|vm)/ ) {
        $filepath = XenServerDataWrapper::get_filepath_rrd_vm( $this_uuid );
      } else {
        $filepath = XenServerDataWrapper::get_filepath_rrd_host( $this_uuid );
      }

      if ( !-f $filepath ) {
        error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
        next;
      }

      my $rrd_update_time = ( stat( $filepath ) )[9];
      if ( $rrd_update_time > $last_update_time ) {
        $last_update_time = $rrd_update_time;
      }

      $itemlabel  = XenServerDataWrapper::get_label( $domain_type, $this_uuid );
      $grouplabel = XenServerDataWrapper::get_label( $group_type, $server );
      $itemlabel  =~ s/:/\\:/g;
      $grouplabel =~ s/:/\\:/g;
      $itemcolor  = $color[$itemcount % $#color];

      my $graph = XenServerGraph::graph_memory_aggr( $domain_type, $metric_type, $filepath, $itemcount, $itemcolor, $itemlabel, $grouplabel, $group_type );
      $cmd_items  .= $graph->{cmd_def};
      $cmd_items  .= $graph->{cmd_cdef};
      $cmd_legend .= $graph->{cmd_legend};

      $itemcount++;
    }

  } elsif ( $item =~ m/^xen-lan-traffic-aggr$/ ) {
    $cmd_params  = XenServerGraph::get_params_lan();

    $cmd_legend0  = " COMMENT:\"[MB/s]\\n\"";
    $cmd_legend0 .= " COMMENT:\\n";
    $cmd_legend0 .= " COMMENT:\"Host                 Interface             Read     Avrg       Max      Write    Avrg      Max\\n\"";

    my $cmd_legend_lower = my $cmd_legend_upper = "";
    my ($graph_info, $filepath, $itemcolor, $itemcolor2, $itemlabel, $grouplabel );
    my $itemcount = 0;

    foreach my $host_interface (@uuids) {
      $filepath = XenServerDataWrapper::get_filepath_rrd_host_lan( $server, $host_interface );

      if ( !-f $filepath ) {
        error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
        next;
      }

      my $rrd_update_time = ( stat( $filepath ) )[9];
      if ( $rrd_update_time > $last_update_time ) {
        $last_update_time = $rrd_update_time;
      }

      $itemlabel = $host_interface;
      $grouplabel = XenServerDataWrapper::get_label( 'host', $server );
      $itemlabel =~ s/:/\\:/g;
      $grouplabel =~ s/:/\\:/g;
      $itemcolor  = $color[$itemcount % $#color];
      $itemcolor2 = $color[( $itemcount + $#color / 2) % $#color];

      my $graph = XenServerGraph::graph_lan_aggr( 'host', $filepath, $itemcount, $itemcolor, $itemcolor2, $itemlabel, $grouplabel );
      $cmd_items  .= $graph->{cmd_def};
      $cmd_items  .= $graph->{cmd_cdef};
      $cmd_legend_lower .= $graph->{cmd_legend_lower};
      $cmd_legend_upper .= $graph->{cmd_legend_upper};

      $itemcount++;
    }

    $cmd_legend  .= $cmd_legend_lower;
    $cmd_legend2 .= $cmd_legend_upper;

  } elsif ( $item =~ m/^xen-disk-(.*)-aggr$/ || $item =~ m/^xen-pool-vbd(.*)-aggr$/ ) {
    my $metric_type = 'vbd';
    if ( $item =~ m/^xen-(pool|disk)-vbd-iops-aggr$/ ) { $metric_type = 'iops'; }
    if ( $item =~ m/^xen-(pool|disk)-vbd-latency-aggr$/ ) { $metric_type = 'latency'; }

    if ( $metric_type eq 'vbd' ) {
      $cmd_params = XenServerGraph::get_params_storage();
    } else {
      $cmd_params = XenServerGraph::get_params_storage($metric_type);
    }

    my $units = "MiB/s";
    if ( $metric_type eq 'iops' ) { $units = "iops"; }
    elsif ( $metric_type eq 'latency' ) { $units = "millisec"; }
    $cmd_legend0  = " COMMENT:\"[$units]\\n\"";
    $cmd_legend0 .= " COMMENT:\\n";
    $cmd_legend0 .= " COMMENT:\"Host                 Storage               Read     Avrg       Max      Write    Avrg      Max\\n\"";

    my $cmd_legend_lower = my $cmd_legend_upper = "";
    my ($graph_info, $filepath, $itemcolor, $itemcolor2, $itemlabel, $grouplabel );
    my $itemcount = 0;

    if ( $item =~ m/^xen-pool-vbd(.*)-aggr$/ ) {
      foreach my $host_server (@uuids) {
        my @storage_uuids = @{ XenServerDataWrapper::get_host_interface_list( $host_server, "storage" ) };

        # the following is a copy of the `else` branch, except s/$server/$host_server/; some code abstraction would be neat here
        foreach my $host_interface (@storage_uuids) {
          $filepath = XenServerDataWrapper::get_filepath_rrd_host_disk( $host_server, $host_interface );

          if ( !-f $filepath ) {
            error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
            next;
          }

          my $rrd_update_time = ( stat( $filepath ) )[9];
          if ( $rrd_update_time > $last_update_time ) {
            $last_update_time = $rrd_update_time;
          }

          my $sr_uuid = XenServerDataWrapper::complete_sr_uuid( $host_interface, $host_server );
          $itemlabel  = XenServerDataWrapper::get_label( 'sr', $sr_uuid );
          $grouplabel = XenServerDataWrapper::get_label( 'host', $host_server );
          $itemlabel =~ s/:/\\:/g;
          $grouplabel =~ s/:/\\:/g;
          $itemcolor  = $color[$itemcount % $#color];
          $itemcolor2 = $color[( $itemcount + $#color / 2) % $#color];

          my $graph = XenServerGraph::graph_storage_aggr( 'host', $metric_type, $filepath, $itemcount, $itemcolor, $itemcolor2, $itemlabel, $grouplabel );
          $cmd_items  .= $graph->{cmd_def};
          $cmd_items  .= $graph->{cmd_cdef};
          $cmd_legend_lower .= $graph->{cmd_legend_lower};
          $cmd_legend_upper .= $graph->{cmd_legend_upper};

          $itemcount++;
        }
      }
    } else {
      foreach my $host_interface (@uuids) {
        $filepath = XenServerDataWrapper::get_filepath_rrd_host_disk( $server, $host_interface );

        if ( !-f $filepath ) {
          error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
          next;
        }

        my $rrd_update_time = ( stat( $filepath ) )[9];
        if ( $rrd_update_time > $last_update_time ) {
          $last_update_time = $rrd_update_time;
        }

        my $sr_uuid = XenServerDataWrapper::complete_sr_uuid( $host_interface, $server );
        $itemlabel  = XenServerDataWrapper::get_label( 'sr', $sr_uuid );
        $grouplabel = XenServerDataWrapper::get_label( 'host', $server );
        $itemlabel =~ s/:/\\:/g;
        $grouplabel =~ s/:/\\:/g;
        $itemcolor  = $color[$itemcount % $#color];
        $itemcolor2 = $color[( $itemcount + $#color / 2) % $#color];

        my $graph = XenServerGraph::graph_storage_aggr( 'host', $metric_type, $filepath, $itemcount, $itemcolor, $itemcolor2, $itemlabel, $grouplabel );
        $cmd_items  .= $graph->{cmd_def};
        $cmd_items  .= $graph->{cmd_cdef};
        $cmd_legend_lower .= $graph->{cmd_legend_lower};
        $cmd_legend_upper .= $graph->{cmd_legend_upper};

        $itemcount++;
      }
    }

    $cmd_legend  .= $cmd_legend_lower;
    $cmd_legend2 .= $cmd_legend_upper;
  }

  # remove vertical label from Dashboard
  if ( $detail == 2 ) {
    $cmd_params =~ s/ --vertical-label=\".+?\"/ /;
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --alt-y-grid";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " $no_legend";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  if ( $detail == 2 ) {
    $cmd .= " --interlaced";    # nope string
  }
  else {
    #$cmd .= " $vertical_label";
  }

  $cmd .= " HRULE:0#000000";

  $cmd .= $cmd_params;
  $cmd .= $cmd_legend0;
  $cmd .= $cmd_items;
  $cmd .= $cmd_legend;
  $cmd .= $cmd_legend2;

  # a trick for aggregated graphs, copied from graphing function for Power
  $cmd =~ s/COMMENT: //g;
  $cmd =~ s/\\"/"/g;

  my $updated_label = "";
  if ( $type =~ "d" ) {
    $last_update_time = localtime($last_update_time);
    $last_update_time =~ s/:/\\\:/g;
    $updated_label    = "COMMENT:\"Updated\\\: $last_update_time\"";
    $cmd             .= " $updated_label";
    $updated_label    = "COMMENT:Updated\\\: $last_update_time";
  }

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Graph rrdtool error : graph_xenserver_totals : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
  $png_end_heading = ret_graph_param( $ret, $updated_label );
  return 0;
} ## end sub graph_xenserver_totals

sub graph_these_rrd_files {
  my $host        = shift;
  my $server      = shift;
  my $lpar        = shift;
  my $type        = shift;
  my $name_out    = shift;
  my $type_sam    = shift;
  my $detail      = shift;
  my $graph_hours = shift;
  my $entitle     = shift;
  my $start_unix  = shift;
  my $end_unix    = shift;
  my $item        = shift;
my $adapter_suffix= shift;
  my $files       = shift;
  my $t2          = "COMMENT:\\n";
  my $step_new    = $step;
  my $last        = "COMMENT: ";
  my $act_time    = localtime();
  my $act_time_u  = time();
  my $req_time    = 0;
  my $cmd  = "";
  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if (!defined $files){
    error ("Graph error : graph_sriovagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph_lparagg: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  my $lan_san; my $data_packets;

  $lan_san = "LAN" if ($item =~ m/lan/);
  $lan_san = "HEA" if ($item =~ m/hea/);
  $lan_san = "SAN" if ($item =~ m/san/);
  $lan_san = "SAS" if ($item =~ m/sas/);
  $lan_san = "SRI" if ($item =~ m/sri/);

  $data_packets = "data" if ($item =~ m/data/);
  $data_packets = "packets" if ($item =~ m/io/);

  $item =~ s/_data//;
  $item =~ s/_io//;

  #print STDERR "lan_san: $lan_san\tdata_packets:$data_packets\n";

  if (!defined $lan_san || !defined $data_packets || $lan_san eq "" || $data_packets eq ""){
    error ( "Not defined item or lpar parameter parsing to graph_power_adapters_agg2 subroutine\n");
    return (0);
  }

  my $head = $lan_san;
  if ($head eq "SRI"){
    $head = "SR-IOV";
  }
  my $header = "$head aggregated : $host last $text";
  if ( $detail == 1 ) {
    $header = "$head aggregated : $host : last $text";
  }
  elsif ( $detail == 2 ) {
    $header = "$head aggregated : $text";
  }

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  elsif ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  elsif ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }
  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick
    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $header = "$head aggregated: $host : $start_human - $end_human";
  } ## end if ( isdigit($start_unix...))

  my $rrd_type = $adapter_suffix;
  #opendir( DIR, "$wrkdir/$server/$host/adapters" ) || error( "can't opendir $wrkdir/$server/$host/adapters: $! :" . __FILE__ . ":" . __LINE__ ) && return 0;
  #my @files_unsorted = grep( /$rrd_type/, readdir(DIR) );
  #my @files = sort { lc $a cmp lc $b } @files_unsorted;
  if (!($files)){
    return;
  }
  #closedir(DIR);

  if ( $graph_hours > 0 ) {

    # alerting
    $start_time = "now-" . $graph_hours . "h";
    if ( $graph_hours < 12 ) {
      $xgrid = "--x-grid=MINUTE:60:HOUR:1:HOUR:1:0:%H";
    }
    else {
      if ( $graph_hours > 24 ) {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:4:0:%H";
      }
      else {
        $xgrid = "--x-grid=MINUTE:60:HOUR:2:HOUR:2:0:%H";
      }
    } ## end else [ if ( $graph_hours < 12)]
  } ## end if ( $graph_hours > 0 )
  my $vertical_label;
  if ($data_packets eq "data"){
    $vertical_label = "--vertical-label=\"Read - Bytes/sec - Write\"";
  }
  elsif($data_packets eq "packets"){
    if ($lan_san eq "LAN" || $lan_san eq "HEA" || $lan_san eq "SRI"){
      $vertical_label = "--vertical-label=\"Read - Packets/sec - Write\"";
    }
    else{
      $vertical_label = "--vertical-label=\"Read - IOPS - Write\"";
    }
  }

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }
  my $no_minor = "--no-minor";
  $no_minor = "" if $super_detail == 3;
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " $no_minor";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " $vertical_label";
  $cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";
  if ( $detail == 1 ) {
    $cmd .= " COMMENT:\"Int                             READ    Avg       Max           WRITE   Avg      Max\\l\"";
  }

  my $file = "";
  my $i    = 0;
  my $gtype            = "AREA";
  my $color_index       = 0;
  my $cmdx = "";
  my $write_var = $lan_san ? "sbyte" : "wbyte";
  #print STDERR Dumper $files;
  my $sriov_aliases;
  if (-e "$basedir/data/$server/$host/sriov_aliases.json"){
    $sriov_aliases = read_json("$basedir/data/$server/$host/sriov_aliases.json");
  }
  foreach my $menu_item (sort keys %{$files}){

  foreach $file (@{$files->{$menu_item}}) {
    chomp($file);
    if (!($lpar =~ $menu_item)){
      next;
    }
    my $file_pth = "$wrkdir/$server/*/adapters/$file";
    my ( $result_cmd, $read, $write, $io_read, $io_write ) = LPM_easy( $file_pth, $i , "0", "read", "write", "io_read", "io_write");
    $cmd .= $result_cmd;
    my $file_orig = $file;

    $lpar = $file;
    $lpar =~ s/\.$rrd_type//; # remove rrd suffix

    my $lpar_space_proc = $lpar;
    my $lpar_space = $lpar;

    # add spaces to lpar name to have 12 chars total (for formating graph legend)
    $lpar_space =~ s/\&\&1/\//g;
    $lpar_space .= " " x (12 - length($lpar_space));

    $lpar_space =~ s/:/\\:/g;    # anti ':'
    $lpar_space_proc =~ s/:/\\:/g;
    $lpar_space_proc =~ s/%/%%/g;    # anti '%
    $lpar_space_proc =~ s/\.$//g;
    my $adapter_legend_png;
    if (defined $sriov_aliases->{$lpar}){
      $adapter_legend_png = "$lpar ($sriov_aliases->{$lpar})";
    }
    else{
      $adapter_legend_png = "$lpar";
    }
    while (length($adapter_legend_png) < 30){
      $adapter_legend_png = "$adapter_legend_png ";
    }

    my $wrkdir_server_host_file = "$wrkdir/$server/$host/adapters/$file";
    $wrkdir_server_host_file =~ s/:/\\:/g;
    my $wrkdir_server_host_file_legend = $lpar_space_proc;

    $wrkdir_server_host_file_legend =~ s/%/%%/g;
    #print STDERR "$delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $item\n";
    if ( $data_packets eq "data" ){
      #$cmd .= " DEF:read${i}=\"$wrkdir_server_host_file\":read:AVERAGE"; #05/06/2018 saving in kB insted of B
      #$cmd .= " DEF:write${i}=\"$wrkdir_server_host_file\":write:AVERAGE";
      $cmd .= " CDEF:write_B${i}=$write,1024,*";
      $cmd .= " CDEF:read_B${i}=$read,1024,*";

      $cmd .= " CDEF:write_kB${i}=$write,1,*";
      $cmd .= " CDEF:read_kB${i}=$read,1,*";

      $cmd .= " CDEF:write_MB${i}=write_kB${i},1024,/";
      $cmd .= " CDEF:read_MB${i}=read_kB${i},1024,/";

      $cmd .= " CDEF:write_B_neg${i}=write_B${i},-1,*";
      $cmd .= " CDEF:read_B_neg${i}=read_B${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_B_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:read_MB${i}:MAX:\" %5.2lf        \"";
      $cmd .= " PRINT:read_MB${i}:AVERAGE:\"%5.2lf $delimiter $item $delimiter $lpar_space_proc $delimiter $lpar $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_B${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_MB${i}:AVERAGE:\" %5.2lf \"";
      $cmd .= " GPRINT:write_MB${i}:MAX:\" %5.2lf \"";
      $cmd .= " PRINT:write_MB${i}:AVERAGE:\"%5.2lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_MB${i}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && ($lan_san eq "LAN" || $lan_san eq "SRI" || $lan_san eq "HEA") ){
#      $cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
#      $cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      $cmd .= " CDEF:write_packets${i}=$io_write,1,/";
      $cmd .= " CDEF:read_packets${i}=$io_read,1,/";
      $cmd .= " CDEF:write_packets_neg${i}=write_packets${i},-1,*";
      $cmd .= " CDEF:read_packets_neg${i}=read_packets${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_packets_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_packets${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_packets${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar_space_proc $delimiter $lpar $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_packets${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_packets${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_packets${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_packets${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_packets${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && $lan_san eq "SAN"){
#      $cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
#      $cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      #$cmd .= " CDEF:write_iops${i}=write_io${i},$hmc_iops_divide,/";
      #$cmd .= " CDEF:read_iops${i}=read_io${i},$hmc_iops_divide,/";
      $cmd .= " CDEF:write_iops${i}=$io_write${i},1,/";
      $cmd .= " CDEF:read_iops${i}=$io_read${i},1,/";
      $cmd .= " CDEF:write_iops_neg${i}=write_iops${i},-1,*";
      $cmd .= " CDEF:read_iops_neg${i}=read_iops${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_iops_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_iops${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_iops${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_iops${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_iops${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_iops${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    elsif ( $data_packets eq "packets" && $lan_san eq "SAS"){
      $cmd .= " DEF:read_io${i}=\"$wrkdir_server_host_file\":io_read:AVERAGE";
      $cmd .= " DEF:write_io${i}=\"$wrkdir_server_host_file\":io_write:AVERAGE";
      $cmd .= " CDEF:write_iops${i}=write_io${i},$hmc_iops_divide,/";
      $cmd .= " CDEF:read_iops${i}=read_io${i},$hmc_iops_divide,/";
      $cmd .= " CDEF:write_iops_neg${i}=write_iops${i},-1,*";
      $cmd .= " CDEF:read_iops_neg${i}=read_iops${i},-1,*";

      $cmd .= " COMMENT:\"$adapter_legend_png\"";
      $cmd .= " $gtype:read_iops_neg${i}$color[$color_index]:\" \"";
      $cmd .= " GPRINT:read_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:read_iops${i}:MAX:\" %.0lf        \"";
      $cmd .= " PRINT:read_iops${i}:AVERAGE:\"%.0lf $delimiter $item $delimiter $lpar_space_proc $delimiter $color[$color_index] $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:read_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";

      $color_index = ( $color_index + 1 ) % ( $color_max + 1 );

      $cmdx .= " $gtype:write_iops${i}$color[$color_index]: ";
      $gtype = "STACK";
      $cmd .= " $gtype:0$color[$color_index]:\" \"";
      $cmd .= " GPRINT:write_iops${i}:AVERAGE:\" %.0lf \"";
      $cmd .= " GPRINT:write_iops${i}:MAX:\" %.0lf \"";
      $cmd .= " PRINT:write_iops${i}:AVERAGE:\"%.0lf $delimiter $color[$color_index]\"";
      $cmd .= " PRINT:write_iops${i}:MAX:\" %.0lf $delimiter $wrkdir_server_host_file_legend\"";
      $cmd .= " $t2";
    }
    $color_index = ( $color_index + 1 ) % ( $color_max + 1 );
    $i++;
    $cmd .= " COMMENT:\"\\l\"";
  } ## end foreach $file (@files)
  }

  $cmd .= " $cmdx";
  $cmd .= " HRULE:0#000000";
  $cmd =~ s/COMMENT: //g;    # a trick, there was a problem with aggregated and "COMMENT: "
  $cmd =~ s/\\"/"/g;

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  $png_end_heading = ret_graph_param($ret,"","",$data_packets);
  return 0;
} ## end sub graph_these_rrd_files

sub write_json{
  my $path = shift;
  my $config=shift;       # hash holding configuration
  # my $json = JSON->new->utf8;    # this will produce compressed JSON - shorter, better for machine reading
  my $json = JSON->new->utf8->pretty;      # this will produce human readable files
  if (!defined $config || $config eq ""){
    #print "Data missing in write_json, nothing to write\n";
    return -1;
  }

  # write to JSON file
  if ( open( CFG, ">$path" ) ) {
    eval{
      print CFG $json->encode($config);
    };
    if (@_){
      error ("Can't open $path and write config:$config");
    }
    close CFG;
  }
  else {
    # error handling
  }
  return 1;
}

sub read_json{
  my $src=shift;
  if (!defined $src || $src eq ""){
    error( "Path to json file(\"$src\") not defined, can't read it at ".__FILE__.":".__LINE__."\n");
    return -1;
  }
  my $read;
  my $rawcfg;       # helper for file reading
  # read from JSON file
  if ( open( CFG, "$src" ) ) {
    local $/ = undef;    # required for re-read of encode_json pretty output
    $rawcfg = <CFG>;
    if ($rawcfg) {
      $read = decode_json($rawcfg);
    }
    close CFG;
  }
  else {
    error("cannot open json file \"$src\" in read_json function, probably doesn't exist\n");
    # error handling
  }
  return $read;
}

# hyperv cluster servers graph
#

sub frame_multi { #$host, $server, $lpar, $time, $name_out, $type_sam, $detail, $graph_hours, $entitle, $start_unix, $end_unix
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $graph_hours  = shift;
  my $entitle    = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $t          = "COMMENT:\" \"";
  my $t2         = "COMMENT:\\n";
  my $step_new   = $step;
  my $last       = "COMMENT:\" \"";
  my $act_time_u  = time();
  my $req_time    = 0;

  my $head       = "Servers aggregated";

  # working for items: hyp-clustser, hyp_clustsercpu, hyp_clustsermem, hyp_clustservms
  $head = "VMs aggregated" if $item eq "hyp_clustservms";

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);

  if ( "$type" =~ "d" ) {
    $req_time = $act_time_u - 86400;
  }
  elsif ( "$type" =~ "w" ) {
    $req_time = $act_time_u - 604800;
  }
  elsif ( "$type" =~ "m" ) {
    $req_time = $act_time_u - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time_u - 31536000;
  }
  # every case: start & end time is unix time
  my $start_time = $req_time;
  my $end_time   = $act_time_u;

  my $vertical_label = "--vertical-label=\"CPU cores\"";
  if ($item eq "hyp_clustsermem") {
    $vertical_label = "--vertical-label=\"Memory in GiBytes\"";
    $head       = "Memory usage";
  }
  my $header = "$head : last $text";

  if ( $detail == 2 ) {
    $vertical_label = "--interlaced";    # nope string
  }
  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $req_time   = $start_time;    # set here too
    $end_time   = $end_unix;
    $xgrid      = "";             # just a trick
    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    # $header = "$head aggregated: $host : $start_human - $end_human";
    $header = "$head : $host : $start_human - $end_human";
  } ## end if ( isdigit($start_unix...))

  if ( $type =~ m/d/ ) {
    $t    = "GPRINT:unrep:LAST:\" %4.0lf\"";
    $head = "COMMENT:\"                            Average   Last\\n\"";
  }
  # print STDERR "19179 \$item $item \$type $type \$detail $detail \$xgrid $xgrid \$start_unix $start_unix \$end_unix $end_unix\n";
  if ( $type_sam =~ m/x/ ) {

    # historical reports --> no xgrid
    $xgrid = "COMMENT:\" \"";           # just a trick to place there something otherwise rrdtool complains
  }

  if ( $text eq '' ) {
    error( "Graph rrdtool error : graph frame_multi: $type " . __FILE__ . ":" . __LINE__ );
    return (0);
  }
  # print STDERR "19152 $host $server $lpar\n";
  my $delim = $host;
  $delim =~ s/^cluster_//;

  my $file_html_nodes = "$wrkdir/$server/$host/node_list.html";
   #$file_html_nodes =~ s/%3A/:/g;    # host can have port vmware:444"
  my @managednamelist = ();

  # print STDERR "detail-graphs-cgi.pl 19158 $file_html_nodes\n";
  if ( -f "$file_html_nodes" ) {
    open( FH, "< $file_html_nodes" );
    while( my $line = <FH>)  {
      # print STDERR "19164 \$delim $delim \$line $line\n";
      push @managednamelist, $line if $line =~ /$delim/; # take just server lines
    }
    close(FH);
  }
  else {
    error( "Multi graph rrdtool error, cannot read nodes from file $file_html_nodes : " . __FILE__ . ":" . __LINE__ );
    exit;
  }

  my $file = "";
  my $i    = 0;
  my $cmd  = "";
  my $cmdq = "";
  my $j    = 0;
  my $managed_ok;
  my $managedname_exl = "";
  my $managedname     = "";
  my @m_excl          = "";
  my $ind = 0;

  # my @managednamelist = ();
  my $DEBUG = 1;
  my $cluster;
  if ($item eq "hyp_clustsercpu" || $item eq "hyp_clustsermem") {
    $no_legend = "--interlaced";
    $rrd_legend = "orig";
  }

  # print STDERR "19226 creating cluster servers graph : $host:$type:@managednamelist \$no_legend $no_legend \$rrd_legend $rrd_legend\n" ;

  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) ) . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) ) if ( $width < 700 && defined $start_unix && defined $end_unix && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 );
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " $vertical_label";
  #$cmd .= " --base=1024";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  #                            $cmd .= " --alt-autoscale-max";
  #$cmd .= " --upper-limit=0.1";
  #$cmd .= " --units-exponent=1.00";
  $cmd .= " $xgrid";

  if ($item ne "hyp_clustsermem" && $item ne "hyp_clustservms") {
    $cmd .= " COMMENT:\\\"Utilization in CPU cores\\:               average   maximum\\l\\\"";
  }
  if ($item eq "hyp_clustservms") {
    $cmd .= " COMMENT:\\\"Utilization in CPU cores                       \\l\\\"";
    $cmd .= " COMMENT:\\\"  Server                            VM                                     avrg     max\\l\\\"";
  }

  my $gtype      = "AREA";
  my $color_indx = 0;

  my $lpar_color_index = -1;
  my $line_indx        = 0;        # place enter every 3rd line
  my $files_index      = -1;       # for vmware it is color index

  foreach my $line (@managednamelist) {
    # <TR> <TD><B>MSNET-HVCL</B></TD> <TD align="center">ad.xorux.com</TD> <TD align="center">HVNODE01</TD> <TD align="center">UnDeFiNeD</TD> <TD align="center">UnDeFiNeD</TD></TR>
    chomp($line);
    my @atoms = split ("<\/TD>",$line);
    (undef, my $domain) = split ( ">",$atoms[1]);
    (undef, my $server) = split ( ">",$atoms[2]);
    # print STDERR "19235 \$domain $domain \$server $server\n";

    if ( $line =~ /No results were found/ ) {
      print STDERR "hyperv cluster servers : $host does not contain any managed system\n" if $DEBUG;
      return 0;
    }

    $managedname = $line;

    $managed_ok = 1;
    #if ( $managed_systems_exclude ne '' ) {
    #  @m_excl = split( /:/, $managed_systems_exclude );
    #  foreach $managedname_exl (@m_excl) {
    #    chomp($managedname_exl);
    #    if ( $managedname =~ m/^$managedname_exl$/ ) {
    #      $managed_ok = 0;
    #    }
    #  } ## end foreach $managedname_exl (@m_excl)
    #} ## end if ( $managed_systems_exclude...)
    if ( $managed_ok == 0 ) {
      next;
    }

    $file = "pool.rrm";

    my $wrkdir_managedname_host_file = "$wrkdir/windows/domain_$domain/$server/$file";
    # print STDERR "19286  creating CMD for $wrkdir_managedname_host_file\n";

    if ( -f "$wrkdir_managedname_host_file" ) {

      # avoid old servers which do not exist in the period
      my $rrd_upd_time = ( stat("$wrkdir_managedname_host_file") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
    } ## end if ( -f "$wrkdir_managedname_host_file")
    else {
      # avoid non-existing managed systems (with no utilization data on)
      next;
    }

    my $rrd = $wrkdir_managedname_host_file;
    my $managedname_space      = $server;
    my $managedname_space_proc = $server;
    $managedname_space_proc =~ s/:/\\:/g;
    $managedname_space_proc =~ s/%/%%/g;    # anti '%

    for ( my $k = length($server); $k < 35; $k++ ) {
      $managedname_space .= " ";
    }
    $managedname_space = substr( $managedname_space, 0,34  );    # not longer

    $managedname_space =~ s/:/\\:/g;                             # anti ':'

    # print STDERR "2 creating CMD for $wrkdir_managedname_host_file\n" if $DEBUG ;

    $wrkdir_managedname_host_file =~ s/:/\\:/g;
    my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
    $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

    if ($item eq "hyp_clustservms") {
      my $hosting_file = "$wrkdir/windows/domain_$domain/$server/VM_hosting.vmh";
      my @vm_hosting   = ();
      if ( -f "$hosting_file" ) {
        open( FHC, "< $hosting_file" ) || error( "cannot open file : $hosting_file " . __FILE__ . ":" . __LINE__ ) && return 0;
        @vm_hosting = <FHC>;
        close(FHC);
      }
      #prepare VM file names
      my @files = @vm_hosting;
      # print STDERR "19356 $wrkdir/$server/$host/VM_hosting.vmh \@files @files\n";

      # my $ind = 0;
      foreach my $file (@files) {
        $files_index++;
        $files_index = $files_index % $color_max;

        chomp $file;
        my $file_orig = $file;
        $file =~ s/:.*/\.rrm/;
        # print STDERR "19359 \$hyperv $hyperv $wrkdir / $host / $server / $file \$domain $domain i $wrkdir/windows/$domain/hyperv_VMs/$file\n";
        next if !-f "$wrkdir/windows/domain_$domain/hyperv_VMs/$file";

        # avoid old lpars which do not exist in the period
        my $rrd_upd_time = ( stat("$wrkdir/windows/domain_$domain/hyperv_VMs/$file") )[9];
        next if ( $rrd_upd_time < $req_time );

        my $wrkdir_server_host_file = "$wrkdir/windows/domain_$domain/hyperv_VMs/$file";

        my $wrkdir_server_host_file_legend = "toto je legenda";

        $lpar = $file;
        $lpar =~ s/\.rrm$//;
        my $lpar_orig = human_vmware_name( $lpar, "", "domain_$domain" );
        # my $lpar_orig = human_vmware_name( $lpar, "", $managedname, $host );
        $lpar = substr( $lpar_orig, 0, 35 );

        my $lpar_space      = $lpar;
        my $lpar_space_proc = $lpar_orig;

        if ( $lpar eq '' ) {
          next;    # avoid .rrm --> just to be sure :)
        }

        # add spaces to lpar name to have 35 chars total (for formating graph legend)
        $lpar_space =~ s/\&\&1/\//g;
        for ( my $k = length($lpar_space); $k < 35; $k++ ) {
          $lpar_space .= " ";
        }
        $lpar_space = $managedname_space.$lpar_space;

        # analyse $file_orig
        # 502a854c-d67c-5015-8f48-f161d122eae0:start=1440417087:end=1440418087:start=144419087:end=1440420087:start=1440421087..."
        # for graphing -> there are $start_time and $end_time, both unix
        my @st_end = split( "start", $file_orig );

        if ( scalar @st_end < 2 ) {    # some trash -> ignore
          next;
        }
        if (( scalar @st_end == 2 && $st_end[1] !~ 'end' ) || (( scalar @st_end > 2 ) && ($st_end[-1] !~ /end/))) {    # one start, no end -> no problem || no end in last atom
          $cmd .= " DEF:utiltot_mhzt_perc${ind}=\"$wrkdir_server_host_file\":PercentTotalRunTime:AVERAGE";
          $cmd .= " DEF:utiltot_mhzt_time${ind}=\"$wrkdir_server_host_file\":Timestamp_PerfTime:AVERAGE";
          $cmd .= " DEF:utiltot_mhzt_freq${ind}=\"$wrkdir_server_host_file\":Frequency_PerfTime:AVERAGE";
          $cmd .= " DEF:utiltot_mhzt_vcpu${ind}=\"$wrkdir_server_host_file\":vCPU:AVERAGE";
          $cmd .= " CDEF:utiltot_mhzt${ind}=utiltot_mhzt_perc${ind},utiltot_mhzt_time${ind},/,utiltot_mhzt_freq${ind},*,100000,/,100,/,utiltot_mhzt_vcpu${ind},*"; #orig was 25
          $cmd .= " CDEF:utiltot${ind}=utiltot_mhzt${ind}";
          # print STDERR "19383 \$cmd $cmd\n";
        }
        else {
          next; # do not take here stopped VMs
        }

        my $index_to_display = $color_indx;
        if ( $lpar_color_index > -1 ) {
          $index_to_display = $lpar_color_index;
        }

        $cmd .= " $gtype:utiltot${ind}$color[$index_to_display]:\"$lpar_space\"";

        $cmd .= " GPRINT:utiltot${ind}:AVERAGE:\"%5.2lf \"";
        $cmd .= " GPRINT:utiltot${ind}:MAX:\" %5.2lf \\l\"";
        my $my_item = "hyp_clustservms";
        $cmd .= " PRINT:utiltot${ind}:AVERAGE:\"%5.2lf $delimiter $my_item $delimiter $managedname_space $delimiter $lpar_space_proc $delimiter $color[$index_to_display]\"";
        $cmd .= " PRINT:utiltot${ind}:AVERAGE:\"%5.2lf $delimiter $color[$index_to_display] $delimiter $wrkdir_server_host_file\"";
        $cmd .= " PRINT:utiltot${ind}:MAX:\" %5.2lf $delimiter $wrkdir_server_host_file_legend\"";

        #$cmd .= " GPRINT:utiltot${i}:MAX:\" %5.2lf \\l\"";
        # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)

        $color_indx = ( $color_indx + 1 ) % ( $color_max + 1 );

        #    $col_indx++;
        $gtype = "STACK";

        $ind++;
      }
      # return;
    }

    # bulid RRDTool cmd
    if ($item eq "hyp_clustsermem") {
      $cmd .= " DEF:free${i}=$rrd:AvailableMBytes:AVERAGE";
      $cmd .= " DEF:tot${i}=$rrd:TotalPhysMemory:AVERAGE";
      $cmd .= " DEF:cachebytes${i}=$rrd:CacheBytes:AVERAGE";
      $cmd .= " CDEF:freeg${i}=free${i},1024,/";
      $cmd .= " CDEF:totg${i}=tot${i},1024,/,1024,/,1024,/";
      $cmd .= " CDEF:cacheg${i}=cachebytes${i},1024,/,1024,/,1024,/";
      $cmd .= " CDEF:used_comp${i}=totg${i},freeg${i},-";
      $cmd .= " CDEF:used${i}=used_comp${i},0,LE,0,used_comp${i},IF";
      $cmd .= " CDEF:freegc${i}=freeg${i},cacheg${i},-";
      if ($i == 0) {
        $cmd .= " CDEF:t_cacheg${i}=cacheg${i}";
        $cmd .= " CDEF:t_used${i}=used${i}";
        $cmd .= " CDEF:t_freegc${i}=freegc${i}";
      }
      else {
        my $ci = $i - 1;
        $cmd .= " CDEF:t_cacheg${i}=cacheg${i},t_cacheg${ci},+";
        $cmd .= " CDEF:t_used${i}=used${i},t_used${ci},+";
        $cmd .= " CDEF:t_freegc${i}=freegc${i},t_freegc${ci},+";

      }
      $i++;
      if ( $color_indx > $color_max ) {
        $color_indx = 0;
      }
    }
    elsif ($item ne "hyp_clustservms"){
      $cmd .= " DEF:cpu_perc${i}=\"$rrd\":PercentTotalRunTime:AVERAGE";
      $cmd .= " DEF:cpu_time${i}=\"$rrd\":Timestamp_PerfTime:AVERAGE";
      $cmd .= " DEF:cpu_freq${i}=\"$rrd\":Frequency_PerfTime:AVERAGE";

      $cmd .= " CDEF:cpuutiltot${i}=cpu_perc${i},cpu_time${i},/,cpu_freq${i},*,100000,/,100,/";    # to be in cores

      # find out max cores of the server from first line of cpu.html im comment
      my $max_cpu_cores = 0;
      if ( open( FR, "< $wrkdir/windows/domain_$domain/$server/cpu.html" ) ) {
        my $firstLine = <FR>;    # example <BR><CENTER><TABLE class="tabconfig tablesorter"><!cores:8>
        close FR;
        ( undef, my $m_cpu_cores ) = split( "cores:", $firstLine );
        if ( defined $m_cpu_cores ) {
          $m_cpu_cores =~ s/>//;
          $max_cpu_cores = $m_cpu_cores;
          chomp $max_cpu_cores;
        }
      } ## end if ( open( FR, "< $wrkdir/windows/domain_$domain/$server/cpu.html") )

      #    $cmd .= " CDEF:ncpu${i}=cpuutiltot${i},POP,$max_cpu_cores,1,/"; # trick to save number to CDEF
      $cmd .= " CDEF:ncpu${i}=cpuutiltot${i},UN,0,$max_cpu_cores,IF"; # trick to save number to CDEF

      # if it does not exist for some time period then put 0 there
      $cmd .= " CDEF:cpu${i}=ncpu${i},UN,0,ncpu${i},IF";

      $cmd .= " PRINT:cpuutiltot${i}:AVERAGE:\"%6.2lf $delimiter multihmcframe $delimiter $managedname_space_proc $delimiter $color[$color_indx]\"";
      $cmd .= " PRINT:cpuutiltot${i}:MAX:\" %6.2lf $delimiter $wrkdir_managedname_host_file_legend\"";

      if ($item eq "hyp_clustsercpu") {
        if ($i == 0) {
          $cmd .= " CDEF:totcpu${i}=cpuutiltot${i}";
        }
        else {
          my $ci = $i-1;
          $cmd .= " CDEF:totcpu${i}=cpuutiltot${i},totcpu${ci},+";
        }
      }
      else {
        $cmdq .= " $gtype:cpuutiltot${i}$color[$color_indx++]:\\\"$managedname_space\\\"";
        $cmdq .= " GPRINT:cpuutiltot${i}:AVERAGE:\\\"%6.2lf \\\"";
        $cmdq .= " GPRINT:cpuutiltot${i}:MAX:\\\"%6.2lf \\l\\\"";
        $gtype = "STACK";
      }
      $i++;
      if ( $color_indx > $color_max ) {
        $color_indx = 0;
      }
    }
  } ## end foreach my $line (@managednamelist)

  if ( $i > 0 && $item ne "hyp_clustservms") {
    if ($item ne "hyp_clustsermem") {
      # add count of all CPU in pools
      for ( $j = 0; $j < $i; $j++ ) {
        if ( $j == 0 ) {
          $cmd .= " CDEF:tcpu${j}=cpu${j}";
        }
        else {
          my $k = $j - 1;
          $cmd .= " CDEF:tcpu${j}=tcpu${k},cpu${j},+";
        } ## end else [ if ( $j == 0 ) ]
      } ## end for ( $j = 0; $j < $i; ...)
      if ( $j > 0 ) {
        $j--;
      }
      my $cpu_pool_total = "CPUs total available in pools";
      for ( my $k = length($cpu_pool_total); $k < 35; $k++ ) {
        $cpu_pool_total .= " ";
      }
      $cmd .= " CDEF:tcpun${j}=tcpu${j},0,EQ,UNKN,tcpu${j},IF";
      $cmd .= " LINE2:tcpun${j}#888888:\\\"$cpu_pool_total\\\"";

      # $cmd .= " GPRINT:tcpu${j}:AVERAGE:\\\"%6.2lf \\\"";
      # excluded as it is a bit misleading there there is any even small data gap
      $cmd .= " GPRINT:tcpun${j}:MAX:\\\"         %6.2lf \\l\\\"";
      $cmd .= $cmdq;
      $cmd .= " PRINT:tcpun${j}:MAX:\\\"         %6.2lf MAXTCPU $delimiter\\\"";
    }
    if ($item eq "hyp_clustsercpu") {
    #  $j = $i - 1;
      $cmd .= " LINE1:totcpu${j}#FF0000:\\\"Utilisation                        \\\"";
      $cmd .= " GPRINT:totcpu${j}:AVERAGE:\\\"%6.2lf \\\"";
      $cmd .= " GPRINT:totcpu${j}:MAX:\\\"%6.2lf \\l\\\"";
    }
    elsif ($item eq "hyp_clustsermem"){
      $j = $i - 1;
      $cmd .= " COMMENT:\\\"   [GiB]                Avrg     Max\\l\\\"";
      $cmd .= " AREA:t_used${j}#FF4040:\\\" Used memory  \\\"";
      $cmd .= " GPRINT:t_used${j}:AVERAGE:\\\"    %6.2lf\\\"";
      $cmd .= " GPRINT:t_used${j}:MAX:\\\" %6.2lf\\l\\\"";
      $cmd .= " $t2";
      $cmd .= " STACK:t_cacheg${j}#0080FF:\\\" Cache        \\\"";
      $cmd .= " GPRINT:t_cacheg${j}:AVERAGE:\\\"    %6.2lf\\\"";
      $cmd .= " GPRINT:t_cacheg${j}:MAX:\\\" %6.2lf\\l\\\"";
      $cmd .= " $t2";
      $cmd .= " STACK:t_freegc${j}#00FF00:\\\" Free memory    \\\"";
      $cmd .= " GPRINT:t_freegc${j}:AVERAGE:\\\"  %6.2lf\\\"";
      $cmd .= " GPRINT:t_freegc${j}:MAX:\\\" %6.2lf\\l\\\"";
    }
  }
  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;
  # print STDERR "19403 \$cmd $cmd\n";
  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$host : Multi graph rrdtool error : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub frame_multi

sub graph_ovirt {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $act_time   = localtime();
  my $act_time_u = time();

  ## command options

  my $graph_info;

  if    ( $item =~ m/^ovirt_(vm|host)_cpu_percent$/ ) {
    $graph_info = OVirtGraph::graph_cpu_percent( $1, $lpar );
  }
  elsif ( $item =~ m/^ovirt_(vm|host)_cpu_core$/ ) {
    $graph_info = OVirtGraph::graph_cpu_cores( $1, $lpar );
  }
  elsif ( $item =~ m/^ovirt_(vm|host)_mem$/ ) {
    $graph_info = OVirtGraph::graph_mem( $1, $lpar );
  }
  elsif ( $item =~ m/^ovirt_host_nic_net$/ ) {
    $graph_info = OVirtGraph::graph_host_net( $server, $lpar );
  }
  elsif ( $item =~ m/^ovirt_disk_data$/ ) {
    $graph_info = OVirtGraph::graph_disk_data( $lpar );
  }
  elsif ( $item =~ m/^ovirt_disk_latency$/ ) {
    $graph_info = OVirtGraph::graph_disk_latency( $lpar );
  }
  elsif ( $item =~ m/^ovirt_storage_domain_space$/ ) {
    $graph_info = OVirtGraph::graph_storage_domain_space( $lpar );
  }

  my $rrd    = $graph_info->{filename};
  my $header = $detail == 2 ? $graph_info->{reduced_header} : $graph_info->{header};

  if ( !-f $rrd ) {
    error( "$rrd does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  ## time settings

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set($type);
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph oVirt stats: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  $time          = $detail == 2 ? $text : "last $text";

  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $end_time   = $end_unix;
    if ( $detail != 10 ) {
      $xgrid = "";             # just a trick
    }

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human";
  }

  $header .= " : $time";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && $detail != 10 && defined $start_unix && defined $end_unix
         && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
        . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  ## graph options

  my $cmd = "";
  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --alt-y-grid";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $no_legend";
  $cmd .= " $xgrid";

  if ( $detail == 2 ) {
    $cmd .= " --interlaced";    # nope string
  }
  else {
    $cmd .= $graph_info->{cmd_vlabel};
  }

  $cmd .= " HRULE:0#000000";
  $cmd .= $graph_info->{cmd_params};
  $cmd .= $graph_info->{cmd_def};
  $cmd .= $graph_info->{cmd_cdef};
  $cmd .= $graph_info->{cmd_legend};

  if ( $type =~ "d" ) {
    RRDp::cmd qq(last "$rrd");
    my $last_tt     = RRDp::read;
    my $last_update = localtime($$last_tt);

    # following must be for RRD 1.2+
    $last_update =~ s/:/\\:/g;
    $cmd .= " COMMENT:\"Updated\\\: $last_update\"";
  } ## end if ( $type =~ "d" )

  eval {
    RRDp::cmd qq($cmd);
    $ret = RRDp::read;
  };
  if ($@) {
    error( "$lpar : Graph rrdtool error : graph_ovirt : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }

  $png_end_heading = ret_graph_param($ret);
  return 0;
} ## end sub graph_ovirt

sub graph_ovirt_totals {
  my $host       = shift;
  my $server     = shift;
  my $lpar       = shift;
  my $type       = shift;
  my $name_out   = shift;
  my $type_sam   = shift;
  my $detail     = shift;
  my $start_unix = shift;
  my $end_unix   = shift;
  my $act_time   = localtime();
  my $act_time_u = time();
  my $cmd        = "";

  # print STDERR "## $host | $server | $lpar | $item\n";

  $start_unix = "" if !defined $start_unix;
  $end_unix   = "" if !defined $end_unix;

  my $xgrid = xgrid_set( $type, $detail );
  my $text = text_set( $type );
  if ( $text eq '' || $lpar eq '' ) {
    error( "Graph rrdtool error : graph oVirt aggregated stats: ,$type, ,$text, ,$lpar, " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  my ($rrd_path, @uuids, $rrd, $time, $label, $header, $rheader);
  my $graph_type;
  $time = $detail == 2 ? "$text" : "last $text";

  if ( $item =~ /^ovirt_storage_domain_aggr_(data|latency)$/ ) {
    $graph_type = $1;
    @uuids      = @{ OVirtDataWrapper::get_arch( $lpar, 'storage_domain', 'disk' ) };
    $label      = OVirtDataWrapper::get_label( 'storage_domain', $lpar );
    $header     = $graph_type eq 'latency' ? "Disk " . ( ucfirst $1 ) . " : $label : "
      : "Disk " . ( ucfirst $1 ) . " aggregated : $label : ";
    $rheader    = ( ucfirst $1 ) . " aggr:$label:"
  }
  elsif ( $item =~ /^ovirt_vm_aggr_net$/ ) {
    $graph_type = 'vm_net';
    @uuids      = @{ OVirtDataWrapper::get_arch( $lpar, 'vm', 'nic' ) };
    $label      = OVirtDataWrapper::get_label( 'vm', $lpar );
    $header     = "VM LAN aggregated : $label : ";
    $rheader    = "LAN aggr:$label:";
  }
  elsif ( $item =~ /^ovirt_host_aggr_vm_(cpu_core|mem_used|mem_free)$/ ) {
    my $header_type = $1;

    $graph_type  = "vm_$1";
    $header_type =~ s/_/ /g;
    $header_type =~ s/(cpu|mem)/uc($1)/ge;
    @uuids       = @{ OVirtDataWrapper::get_arch( $lpar, 'host', 'vm' ) };
    $label       = OVirtDataWrapper::get_label( 'host', $lpar );
    $header      = "VM $header_type aggregated : $label : ";
    $rheader     = "VM $header_type aggr:$label";
  }
  elsif ( $item =~ /^ovirt_host_nic_aggr_net$/ ) {
    $graph_type = 'host_net';
    @uuids      = @{ OVirtDataWrapper::get_arch( $lpar, 'host', 'nic' ) };
    $label      = OVirtDataWrapper::get_label( 'host', $lpar );
    $header     = "Host LAN aggregated : $label : ";
    $rheader    = "LAN aggr:$label";
  }
  elsif ( $item =~ /^ovirt_storage_domains_total_aggr_(data|latency)$/ ) {
    $graph_type = $1;
    @uuids      = @{ OVirtDataWrapper::get_uuids( 'disk' ) };
    $header     = $graph_type eq 'latency' ? "Storage domain " . ( ucfirst $1 ) . " : "
      : "Storage domain " . ( ucfirst $1 ) . " aggregated : ";
    $rheader    = ( ucfirst $1 ) . " aggr:";
  }
  elsif ( $item =~ /^ovirt_vm_aggr_(data|latency)$/ ) {
    $graph_type = $1;
    @uuids      = @{ OVirtDataWrapper::get_arch( $lpar, 'vm', 'disk' ) };
    $label      = OVirtDataWrapper::get_label( 'vm', $lpar );
    $header     = $graph_type eq 'latency' ? "Disk " . ( ucfirst $1 ) . " : $label : "
      : "Disk " . ( ucfirst $1 ) . " aggregated : $label : ";
    $rheader    = ( ucfirst $1 ) . " aggr:$label:";
  }
  elsif ( $item =~ /^ovirt_cluster_aggr_(vm|host)_(cpu_core|cpu_percent|mem_used|mem_free|mem_used_percent)$/ ) {
    my $aggr_type = $1;
    my $metric    = $2;

    $graph_type = "$1_$2";
    @uuids      = @{ OVirtDataWrapper::get_arch( $lpar, 'cluster', $1) };

    my $header_type = $graph_type;
    $header_type =~ s/_/ /g;
    $header_type =~ s/(cpu|mem|vm)/uc($1)/ge;
    $metric      =~ s/(cpu|mem)/uc($1)/ge;
    $header_type = ucfirst $header_type;
    $label       = OVirtDataWrapper::get_label( 'cluster', $lpar );

    if ( $graph_type =~ /_percent$/ ) {
      $header  = "$header_type : $label : ";
      $rheader = "$metric:$label:";
    }
    else {
      $header = "$header_type aggregated : $label : ";
      $rheader = "$metric aggr:$label:";
    }
  }
  else {
    error( "oVirt $item aggregated graph does not exist " . __FILE__ . ":" . __LINE__ );
    return 0;
  }

  $header = $detail == 2 ? $rheader : $header;

  my $start_time = "now-1$type";
  my $end_time   = "now-1$type+1$type";
  if ( isdigit($start_unix) && isdigit($end_unix) ) {
    $start_time = $start_unix;
    $end_time   = $end_unix;
    if ( $detail != 10 ) {
      $xgrid = "";            # just a trick
    }

    my $start_human = strftime "%H:%M:%S %d.%m.%Y", localtime($start_unix);
    my $end_human   = strftime "%H:%M:%S %d.%m.%Y", localtime($end_unix);
    $time = "$start_human - $end_human";
  }

  if (($detail < 1 || $detail == 9) && $rrd_legend eq "table" ) {
    $no_legend = "--no-legend";
  }

  $header .= "$time";
  if ( $detail == 9 ) {
    $html_heading = $header;
    $header = "";
  }
  else {
    if ( $width < 700 && $detail != 10 && defined $start_unix && defined $end_unix
         && isdigit($start_unix) > 0 && isdigit($end_unix) > 0 ) {
      $header = strftime( "%H:%M:%S %d.%m.%Y", localtime($start_unix) )
        . " - " . strftime( "%H:%M:%S %d.%m.%Y", localtime($end_unix) );
    }
  }

  my $last_update_time = 0;
  my $itemcount        = 0;
  my $cmd_params       = '';
  my $vertical_label   = '';
  my $cmd_def          = '';
  my $cmd_cdef         = '';
  my $cmd_legend       = '';

  if ( $graph_type =~ /^(data|latency|vm_net|host_net)$/ ) {
    my ( $filepath, $itemlabel, $obj, $color_read, $color_write );
    my $cmd_legend_upper = my $cmd_legend_lower = "";

    if ( $graph_type eq 'data' ) {
      $vertical_label = " --vertical-label=\"Read - Bytes/sec - Write\"";
      $cmd_legend     = " COMMENT:\"[MB/sec]\"";
      $obj            = "Disk";
    }
    elsif ( $graph_type eq 'latency' ) {
      $vertical_label = " --vertical-label=\"Read - millisec - Write\"";
      $cmd_legend     = " COMMENT:\"[millisec]\"";
      $obj            = "Disk";
    }
    elsif ( $graph_type =~ /^(vm|host)_net$/ ) {
      $vertical_label = " --vertical-label=\"Read - Bytes/sec - Write\"";
      $cmd_legend     = " COMMENT:\"[MB/sec]\"";
      $obj            = "Int ";
    }

    $cmd_legend .= " COMMENT:\\n";
    $cmd_legend .= " COMMENT:\"$obj                 Read      Avrg       Max    Write      Avrg      Max\\n\"";
    $cmd_def     = $cmd_cdef = "";

    foreach my $uuid ( @uuids ) {
      $color_read  = $color[$itemcount % $#color];
      $color_write = $color[( $itemcount + 1) % $#color];

      my $graph_entry = OVirtGraph::graph_double_sided_aggr( $uuid, $lpar, $item, $itemcount,
                                                             $color_read, $color_write );
      my $filepath    = $graph_entry->{filepath};

      if ( !-f $filepath ) {
        error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
        next;
      }

      my $rrd_update_time = ( stat( $filepath ) )[9];
      if ( $rrd_update_time > $last_update_time ) {
        $last_update_time = $rrd_update_time;
      }

      $cmd_def          .= $graph_entry->{cmd_def};
      $cmd_cdef         .= $graph_entry->{cmd_cdef};
      $cmd_legend_lower .= $graph_entry->{cmd_legend_lower};
      $cmd_legend_upper .= $graph_entry->{cmd_legend_upper};

      $itemcount += 2;
    }

    $cmd_legend .= $cmd_legend_lower;
    $cmd_legend .= $cmd_legend_upper;
  }
  elsif ( $graph_type =~ /^(vm|host)_(cpu_core|cpu_percent|mem_used|mem_free|mem_used_percent)$/ ) {
    my ( $filepath, $itemlabel, $color );
    my $obj = $1 eq 'vm' ? 'VM  ' : 'Host';

    if ( $graph_type =~ /cpu_core/ ) {
      $cmd_params     = " --lower-limit=0.00";
      $cmd_params    .= " --units-exponent=1.00";
      $vertical_label = " --vertical-label=\"CPU in cores\"";
      $cmd_legend     = " COMMENT:\"[cores]\"";
    }
    elsif ( $graph_type =~ /cpu_percent/ ) {
      $cmd_params     = " --upper-limit=100.0";
      $cmd_params    .= " --lower-limit=0.00";
      $vertical_label = " --vertical-label=\"CPU load in [%]\"";
      $cmd_legend     = " COMMENT:\"[%]\"";
    }
    elsif ( $graph_type =~ /mem_used_percent/ ) {
      $cmd_params     = " --upper-limit=100.0";
      $cmd_params    .= " --lower-limit=0.00";
      $vertical_label = " --vertical-label=\"Memory in [%]\"";
      $cmd_legend     = " COMMENT:\"[%]\"";
    }
    elsif ( $graph_type =~ /mem_free|mem_used/ ) {
      $cmd_params     = " --lower-limit=0.00";
      $cmd_params    .= " --base=1024";
      $cmd_params    .= " --units-exponent=1.00";
      $vertical_label = " --vertical-label=\"Memory in GBytes\"";
      $cmd_legend     = " COMMENT:\"[GB]\"";
    }

    $cmd_legend .= " COMMENT:\\n";
    $cmd_legend .= " COMMENT:\"$obj" . ( ' ' x 27 )  . "Avrg       Max\\n\"";
    $cmd_def     = $cmd_cdef = "";

    foreach my $uuid ( @uuids ) {
      $color = $color[$itemcount % $#color];

      my $graph_entry = OVirtGraph::graph_simple_aggr( $uuid, $item, $itemcount, $color );
      my $filepath    = $graph_entry->{filepath};

      if ( !-f $filepath ) {
        error( "$filepath does not exist " . __FILE__ . ":" . __LINE__ );
        next;
      }

      my $rrd_update_time = ( stat( $filepath ) )[9];
      if ( $rrd_update_time > $last_update_time ) {
        $last_update_time = $rrd_update_time;
      }

      $cmd_def    .= $graph_entry->{cmd_def};
      $cmd_cdef   .= $graph_entry->{cmd_cdef};
      $cmd_legend .= $graph_entry->{cmd_legend};

      $itemcount++;
    }
  }

  $cmd .= "$graph_cmd \"$name_out\"";
  $cmd .= " --title \"$header\"";
  $cmd .= " --start $start_time";
  $cmd .= " --end $end_time";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag";
  $cmd .= " --slope-mode";
  $cmd .= " --width=$width";
  $cmd .= " --height=$height";
  $cmd .= " --step=$step";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --alt-y-grid";
  $cmd .= " $no_legend";
  $cmd .= " $font_def";
  $cmd .= " $font_tit";
  $cmd .= " $xgrid";

  if ( $detail == 2 ) {
    $cmd .= " --interlaced";    # nope string
  }
  else {
    $cmd .= " $vertical_label";
  }

  $cmd .= " HRULE:0#000000";
  $cmd .= $cmd_params;
  $cmd .= $cmd_def;
  $cmd .= $cmd_cdef;
  $cmd .= $cmd_legend;
  # a trick for aggregated graphs, copied from graphing function for Power
  $cmd =~ s/COMMENT: //g;
  $cmd =~ s/\\"/"/g;

  my $updated_label = "";
  if ( $type =~ "d" ) {
    $last_update_time = localtime($last_update_time);
    $last_update_time =~ s/:/\\\:/g;
    $updated_label    = "COMMENT:\"Updated\\\: $last_update_time\"";
    $cmd             .= " $updated_label";
    $updated_label    = "COMMENT:Updated\\\: $last_update_time";
  }

  eval {
    RRDp::cmd qq($cmd);
#    $RRDp::error_mode = 'catch';
    $ret = RRDp::read;
  #  print STDERR "##ERR $RRDp::error\n" if defined $RRDp::error;
  };
  if ($@) {
    error( "$lpar : Graph rrdtool error : graph_ovirt_totals : $@ " . __FILE__ . ":" . __LINE__ );
    return 1;
  }
#  print STDERR "#rrrr " . Dumper($ret) ." \n";
  $png_end_heading = ret_graph_param( $ret, $updated_label );
  return 0;
} ## end sub graph_ovirt_totals
