# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

use strict;
use Date::Parse;
use File::Compare;
use LoadDataModuleHyperV;
use File::Copy;
use POSIX qw(strftime);
use File::Glob ':glob';

use Data::Dumper;
use Time::Local;


#use lib qw (/opt/freeware/lib/perl/5.8.0);
# no longer need to use "use lib qw" as the library PATH is already in PERL5LIB env var (lpar2rrd.cfg)

# touch tmp/hyperv-debug to have debug info in load output

# set unbuffered stdout
$| = 1;

# get cmd line params
my $version = "$ENV{version}";
my $host    = "$ENV{HMC_SPACE}";         # contains username
# example: $tmpdir/HYPERV_X/Notebook   or  $tmpdir/HYPERV/Notebook
# take only basedir
$host = (split("\/",$host))[-1];

# exit if $host !~ "444"; # my home debug
my $alias;
my $username;
my $hea      = $ENV{HEA};
my $hmc_user = $ENV{HMC_USER};
my $webdir   = $ENV{WEBDIR};
my $bindir   = $ENV{BINDIR};
my $basedir  = $ENV{INPUTDIR};
my $tmpdir   = "$basedir/tmp";
if ( defined $ENV{TMPDIR_LPAR} ) {
  $tmpdir = $ENV{TMPDIR_LPAR};
}
my $rrdtool                 = $ENV{RRDTOOL};
my $DEBUG                   = $ENV{DEBUG};
  $DEBUG = 2 if ( -f "$tmpdir/hyperv-debug" ); # for extensive debug prints
  print_hyperv_debug("HYPDIR in perl $host\n") if $DEBUG == 2;

my $pic_col                 = $ENV{PICTURE_COLOR};
my $STEP                    = $ENV{SAMPLE_RATE};
my $HWINFO                  = $ENV{HWINFO};
my $CONFIG_HISTORY          = $basedir . "/data";              # do not change that as subdir tree is not created automatically here but via main loop
my $managed_systems_exclude = $ENV{MANAGED_SYSTEMS_EXCLUDE};
my $SYS_CHANGE              = $ENV{SYS_CHANGE};
my $upgrade                 = $ENV{UPGRADE};
my $SSH                     = $ENV{SSH} . " -q ";              # doubles -q from lpar2rrd.cfg, just to be sure ...

###  HYPER-V def section

my $command_date;                                              # server time when starting this script

my $et_VirtualMachine         = "VirtualMachine";
my $et_HostSystem             = "HostSystem";
my $et_Datastore              = "Datastore";
my $et_ResourcePool           = "ResourcePool";
my $et_Datacenter             = "Datacenter";
my $et_ClusterComputeResource = "ClusterComputeResource";

my $no_inserted                = 66;
my $real_sampling_period_limit = 900;
my %vm_name_uuid;
my %vm_uuid_mac;
my %vm_uuid_mem;
my %vm_name_powerstat;
my %vm_name_vcpu;
my %vm_id_path = ();    # contains vm_id : $wrkdir/$server/host/vm_uuid.rrm
my $samples_number;
my @vm_view = ();       # VM names
# longer line to omit - possible error sub hyperv_load
my $max_line_length = 10000;


# hyperv VM EnabledState
my %enabledstate = (
  1     => "OTHER",
  2     => "ON",
  3     => "OFF",
  4     => "STOPPING",
  5     => "NA",
  6     => "OFFLINE",
  7     => "TEST",
  8     => "DEFERRED",
  9     => "QUIESCE",
  32768 => "PAUSED",
  32769 => "SUSPENDED",
  32770 => "STARTING",
  32771 => "SNAPSHOTTING",
  32773 => "SAVING",
  32774 => "STOPPING",
  32776 => "PAUSINIG",
  32777 => "RESUMING"
);

my $UnDeF = "UnDeFiNeD";    # const for testing data

#   perf values Hostsystem or VM HYPER-V
my @vm_PercentTotalRunTime;
my @vm_Timestamp_PerfTime;
my @vm_Frequency_PerfTime;
my @vm_AvailableMBytes;
my @vm_CacheBytes;
my @vm_MemoryAvailable;
my @vm_TotalPhysicalMemory;
my @vm_DiskBytesPersec;
my @vm_DiskReadBytesPersec;
my @vm_DiskReadsPersec;
my @vm_DiskTransfersPersec;
my @vm_DiskWriteBytesPerse;
my @vm_DiskWritesPersec;
my @vm_BytesReceivedPersec;
my @vm_BytesSentPersec;
my @vm_BytesTotalPersec;
my @vm_PagesInputPersec;
my @vm_PagesOutputPersec;
my @vm_vCPU;

my @vm_Memory_active_KB;
my @vm_Memory_granted_KB;
my @vm_Memory_baloon_MB;
my @vm_Disk_usage_KBps;
my @vm_Disk_read_KBps;
my @vm_Disk_write_KBps;
my @vm_Network_usage_KBps;
my @vm_Network_received_KBps;
my @vm_Network_transmitted_KBps;
my @vm_Memory_swapin_KBps;
my @vm_Memory_swapout_KBps;
my @vm_Memory_compres_KBps;
my @vm_Memory_decompres_KBps;
my @vm_CPU_usage_Percent;
my @Host_memory_size;
my @vm_CPU_ready_ms;

my @arr_pointers;
my $pointer_arr = \@arr_pointers;

#   perf values cluster, some also for resourcepool
my @cl_CPU_usage_Proc;
my @cl_CPU_usage_MHz;
my @cl_CPU_reserved_MHz;
my @cl_Memory_usage_Proc;
my @cl_Memory_reserved_MB;
my @cl_Memory_granted_KB;
my @cl_Memory_active_KB;
my @cl_Memory_shared_KB;
my @cl_Memory_zero_KB;
my @cl_Memory_swap_KB;
my @cl_Memory_baloon_KB;
my @cl_Memory_consumed_KB;
my @cl_Memory_overhead_KB;
my @cl_Memory_compressed_KB;
my @cl_Memory_compression_KBps;
my @cl_Memory_decompress_KBps;
my @cl_Power_usage_Watt;
my @cl_Power_cup_Watt;
my @cl_Power_energy_usage_Joule;
my @cl_Cluster_eff_CPU_MHz;
my @cl_Cluster_eff_memory_MB;
my @cl_CPU_total_MHz;
my @cl_Memory_total_MB;

my @cl_cpu_limit;
my @cl_cpu_reservation;
my @cl_mem_limit;
my @cl_mem_reservation;
my $cluster_effectiveMemory;
my $cluster_effectiveCpu;

my $rp_cpu_reservation = 0;
my $rp_mem_reservation = 0;
my $rp_cpu_limit       = 0;
my $rp_mem_limit       = 0;

#   perf values datastore
my @ds_Datastore_freeSpace_KB;
my @ds_Datastore_used_KB;
my @ds_Datastore_provision_KB;
my @ds_Datastore_capacity_KB;
my @ds_Datastore_read_KBps;
my @ds_Datastore_write_KBps;
my @ds_Datastore_numberReadAveraged;
my @ds_Datastore_numberWriteAveraged;

my $ds_accessible;    # true/false 1/0
my $ds_freeSpace;
my $ds_used;
my $ds_provisioned;
my $ds_capacity;
my $ds_type = "";     # or 'NFS' when NFS type

my $service_instance; # vmware TOP
my $apiType_top;
my $fullName_top;
my $vmware_uuid;
my $do_fork;          # 1 means do!, 0 do not!
my @managednamelist_un;
my @rp_vm_morefs = ();

my $datacenter_list;
my $datastore_list;
my $cluster_list;
my $resourcepool_list;
my $datacenter;

my $perfmgr_view;
my $now_counters;
my $all_counters;
my $cpu_counters;
my $memory_counters;
my $disk_counters;
my $system_counters;
my $net_counters;

# selected_counters set up in sub init_perf_counter_info
my $selected_counters;    # number of counters to retrieve from entity
my $error_select_counters = 0;
my $host_hz;
my $host_cpuAlloc;
my $host_memorySize;

my $server;
my $command_unix;         # holds server UTC unix time - integer
my $vm_host;              # host hash
my $h_name;               # host name
my $fail_entity_type;     # for error printing
my $fail_entity_name;     # for error printing
my $rp_name;
my $rp_moref;
my $ds_name;
my $historical_level0 = 0;
my $numCpu;

# keeps all VMs: $wrkdir/$all_hyperv_VMs
#  NO, prepare it per domain
my $all_hyperv_VMs             = "hyperv_VMs";
my $vm_counterId_readAveraged  = "";
my $vm_counterId_writeAveraged = "";
my $vm_uuid_active             = "";
my @counter_arr_levels;
my $wrkdir_windows             = "";

# in the beginning of file can be GLOBAL INFO
# GLOBAL INFO BEGIN
# any lines of info, e.g. cluster nodes, agent mode or ...:6500
# GLOBAL INFO END
my @global_info = ();
my @local_info  = ();

# keeps server domain until end of script for possible cluster
my %server_domain = ();

# skipped computers - probably clusters
my %skipped_computers = ();

# keeps uuid{name} for all hyperv VMs
my $all_vm_uuid_names = "vm_uuid_name.txt";

# keeps all Cluster Storages UUID
my %all_clusterstorage = ();

# VM_hosting.vmh keeps presence of VMs in a Host system (esxi server)
# res_pool_name.vmr keeps presence of VMs in a ResourcePool

# function xerror if main: appends notice to following file
#                 if fork: just prints XERROR text
#        XERROR text is appended when reading forks' output
my $counters_info_file = "$basedir/logs/counter-info.txt";
my $i_am_fork          = "main";                             # in fork is 'fork'

my @counter_presence = ();                                   # indicates real counter presence

#  testing real counters
#  VirtualMachine
my @counter_vm_eng = ( 'CPU:usagemhz:MHz', 'Disk:usage:KBps', 'Disk:read:KBps', 'Disk:write:KBps', 'Network:usage:KBps', 'Network:received:KBps', 'Network:transmitted:KBps', 'Memory:active:KB', 'Memory:granted:KB', 'Memory:swapinRate:KBps', 'Memory:vmmemctl:KB', 'Memory:swapoutRate:KBps', 'Memory:decompressionRate:KBps', 'Memory:compressionRate:KBps', 'CPU:usage:Percent', 'CPU:ready:Millisecond', );

#                          'Datastore:numberReadAveraged:Number','Datastore:numberWriteAveraged:Number'
#                          );
my @counter_vm_eng6 = ( 'CPU:usagemhz:MHz', 'Disk:usage:KBps', 'Disk:read:KBps', 'Disk:write:KBps', 'Network:usage:KBps', 'Network:received:KBps', 'Network:transmitted:KBps', 'Memory:active:KB', 'Memory:granted:KB', 'Memory:swapinRate:KBps', 'Memory:vmmemctl:KB', 'Memory:swapoutRate:KBps', 'Memory:decompressionRate:KBps', 'Memory:compressionRate:KBps', 'CPU:usage:%', 'CPU:ready:ms', );
my @counter_vm_ger1 = ( 'CPU:usagemhz:MHz', 'Festplatte:usage:KB/s', 'Festplatte:read:KB/s', 'Festplatte:write:KB/s', 'Netzwerk:usage:KB/s', 'Netzwerk:received:KB/s', 'Netzwerk:transmitted:KB/s', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:swapinRate:KB/s', 'Arbeitsspeicher:vmmemctl:KB', 'Arbeitsspeicher:swapoutRate:KB/s', 'Arbeitsspeicher:decompressionRate:KB/s', 'Arbeitsspeicher:compressionRate:KB/s', 'CPU:usage:Prozent', 'CPU:ready:Millisekunde', 'Festplatte:numberRead:Anzahl', 'Festplatte:numberWrite:Anzahl' );
my @counter_vm_ger2 = ( 'CPU:usagemhz:MHz', 'Festplatte:usage:KBit/s', 'Festplatte:read:KBit/s', 'Festplatte:write:KBit/s', 'Netzwerk:usage:KBit/s', 'Netzwerk:received:KBit/s', 'Netzwerk:transmitted:KBit/s', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:swapinRate:KBit/s', 'Arbeitsspeicher:swapoutRate:KBit/s', 'Arbeitsspeicher:decompressionRate:KBit/s', 'Arbeitsspeicher:compressionRate:KBit/s' );

#  HostSystem
my @counter_hs_eng  = ( 'CPU:usagemhz:MHz', 'Disk:usage:KBps',       'Disk:read:KBps',       'Disk:write:KBps',       'Network:usage:KBps',  'Network:received:KBps',  'Network:transmitted:KBps',  'Memory:active:KB',          'Memory:granted:KB',          'Memory:swapinRate:KBps',          'Memory:vmmemctl:KB',          'Memory:swapoutRate:KBps',          'Memory:decompressionRate:KBps',          'Memory:compressionRate:KBps',          'CPU:usage:Percent', 'CPU:ready:Millisecond', );
my @counter_hs_eng6 = ( 'CPU:usagemhz:MHz', 'Disk:usage:KBps',       'Disk:read:KBps',       'Disk:write:KBps',       'Network:usage:KBps',  'Network:received:KBps',  'Network:transmitted:KBps',  'Memory:active:KB',          'Memory:granted:KB',          'Memory:swapinRate:KBps',          'Memory:vmmemctl:KB',          'Memory:swapoutRate:KBps',          'Memory:decompressionRate:KBps',          'Memory:compressionRate:KBps',          'CPU:usage:%',       'CPU:ready:ms', );
my @counter_hs_ger1 = ( 'CPU:usagemhz:MHz', 'Festplatte:usage:KB/s', 'Festplatte:read:KB/s', 'Festplatte:write:KB/s', 'Netzwerk:usage:KB/s', 'Netzwerk:received:KB/s', 'Netzwerk:transmitted:KB/s', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:swapinRate:KB/s', 'Arbeitsspeicher:vmmemctl:KB', 'Arbeitsspeicher:swapoutRate:KB/s', 'Arbeitsspeicher:decompressionRate:KB/s', 'Arbeitsspeicher:compressionRate:KB/s', 'CPU:usage:Prozent', 'CPU:ready:Millisekunde', );
my @counter_hs_ger2 = ( 'CPU:usagemhz:MHz', 'Festplatte:usage:KBit/s', 'Festplatte:read:KBit/s', 'Festplatte:write:KBit/s', 'Netzwerk:usage:KBit/s', 'Netzwerk:received:KBit/s', 'Netzwerk:transmitted:KBit/s', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:swapinRate:KBit/s', 'Arbeitsspeicher:swapoutRate:KBit/s', 'Arbeitsspeicher:decompressionRate:KBit/s', 'Arbeitsspeicher:compressionRate:KBit/s' );

my @counter_cl_eng  = ( 'CPU:usagemhz:MHz', 'CPU:usage:Percent', 'CPU:reservedCapacity:MHz', 'CPU:totalmhz:MHz', 'Cluster services:effectivecpu:MHz', 'Cluster services:effectivemem:MB', 'Memory:totalmb:MB', 'Memory:shared:KB', 'Memory:zero:KB', 'Memory:vmmemctl:KB', 'Memory:consumed:KB', 'Memory:overhead:KB', 'Memory:active:KB', 'Memory:granted:KB', 'Memory:compressed:KB', 'Memory:reservedCapacity:MB', 'Memory:swapused:KB', 'Memory:compressionRate:KBps', 'Memory:decompressionRate:KBps', 'Memory:usage:Percent', 'Power:powerCap:Watt', 'Power:power:Watt' );    # do not take 'Power:energy:Joule'
my @counter_cl_eng6 = ( 'CPU:usagemhz:MHz', 'CPU:usage:%',       'CPU:reservedCapacity:MHz', 'CPU:totalmhz:MHz', 'Cluster services:effectivecpu:MHz', 'Cluster services:effectivemem:MB', 'Memory:totalmb:MB', 'Memory:shared:KB', 'Memory:zero:KB', 'Memory:vmmemctl:KB', 'Memory:consumed:KB', 'Memory:overhead:KB', 'Memory:active:KB', 'Memory:granted:KB', 'Memory:compressed:KB', 'Memory:reservedCapacity:MB', 'Memory:swapused:KB', 'Memory:compressionRate:KBps', 'Memory:decompressionRate:KBps', 'Memory:usage:%',       'Power:powerCap:W',    'Power:power:W' );       # do not take 'Power:energy:Joule'

my @counter_cl_ger  = ( 'CPU:usagemhz:MHz', 'CPU:usage:Prozent', 'CPU:reservedCapacity:MHz', 'CPU:totalmhz:MHz', 'Clusterdienste:effectivecpu:MHz', 'Clusterdienste:effectivemem:MB', 'Arbeitsspeicher:totalmb:MB', 'Arbeitsspeicher:shared:KB', 'Arbeitsspeicher:zero:KB', 'Arbeitsspeicher:vmmemctl:KB', 'Arbeitsspeicher:consumed:KB', 'Arbeitsspeicher:overhead:KB', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:compressed:KB', 'Arbeitsspeicher:reservedCapacity:MB', 'Arbeitsspeicher:swapused:KB', 'Arbeitsspeicher:compressionRate:KB/s', 'Arbeitsspeicher:decompressionRate:KB/s', 'Arbeitsspeicher:usage:Prozent', 'Betrieb:powerCap:Watt', 'Betrieb:power:Watt' );    # do not take 'Power:energy:Joule'
my @counter_cl_ger6 = ( 'CPU:usagemhz:MHz', 'CPU:usage:%',       'CPU:reservedCapacity:MHz', 'CPU:totalmhz:MHz', 'Clusterdienste:effectivecpu:MHz', 'Clusterdienste:effectivemem:MB', 'Arbeitsspeicher:totalmb:MB', 'Arbeitsspeicher:shared:KB', 'Arbeitsspeicher:zero:KB', 'Arbeitsspeicher:vmmemctl:KB', 'Arbeitsspeicher:consumed:KB', 'Arbeitsspeicher:overhead:KB', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:compressed:KB', 'Arbeitsspeicher:reservedCapacity:MB', 'Arbeitsspeicher:swapused:KB', 'Arbeitsspeicher:compressionRate:KB/s', 'Arbeitsspeicher:decompressionRate:KB/s', 'Arbeitsspeicher:usage:%',       'Betrieb:powerCap:Watt', 'Betrieb:power:Watt' );    # do not take 'Power:energy:Joule'

my @counter_rp_eng = ( 'CPU:usagemhz:MHz', 'Memory:shared:KB',          'Memory:zero:KB',          'Memory:vmmemctl:KB',          'Memory:consumed:KB',          'Memory:overhead:KB',          'Memory:active:KB',          'Memory:granted:KB',          'Memory:compressed:KB',          'Memory:swapped:KB',          'Memory:compressionRate:KBps',          'Memory:decompressionRate:KBps' );
my @counter_rp_ger = ( 'CPU:usagemhz:MHz', 'Arbeitsspeicher:shared:KB', 'Arbeitsspeicher:zero:KB', 'Arbeitsspeicher:vmmemctl:KB', 'Arbeitsspeicher:consumed:KB', 'Arbeitsspeicher:overhead:KB', 'Arbeitsspeicher:active:KB', 'Arbeitsspeicher:granted:KB', 'Arbeitsspeicher:compressed:KB', 'Arbeitsspeicher:swapped:KB', 'Arbeitsspeicher:compressionRate:KB/s', 'Arbeitsspeicher:decompressionRate:KB/s' );
my @counter_ds_eng  = ( 'Disk:used:KB',       'Disk:provisioned:KB',       'Disk:capacity:KB',       'Datastore:read:KBps',     'Datastore:write:KBps',     'Datastore:numberReadAveraged:Number',     'Datastore:numberWriteAveraged:Number' );
my @counter_ds_eng6 = ( 'Disk:used:KB',       'Disk:provisioned:KB',       'Disk:capacity:KB',       'Datastore:read:KBps',     'Datastore:write:KBps',     'Datastore:numberReadAveraged:num',        'Datastore:numberWriteAveraged:num' );
my @counter_ds_ger  = ( 'Festplatte:used:KB', 'Festplatte:provisioned:KB', 'Festplatte:capacity:KB', 'Datenspeicher:read:KB/s', 'Datenspeicher:write:KB/s', 'Datenspeicher:numberReadAveraged:Anzahl', 'Datenspeicher:numberWriteAveraged:Anzahl' );

# 'Datastore:totalReadLatency:Millisecond','Datastore:totalWriteLatency:Millisecond',
# 'VirtualDisk:totalReadLatency:Millisecond','VirtualDisk:totalWriteLatency:Millisecond'

my $lpm            = $ENV{LPM};
my $h              = $ENV{HOSTNAME};
my $new_change     = "$tmpdir/$version-run";
my $cpu_max_filter = 100;                      # my $cpu_max_filter = 100;  # max 10k peak in % is allowed (in fact it cannot be higher than 1k now when 1 logical CPU == 0.1 entitlement)
if ( defined $ENV{CPU_MAX_FILTER} ) {
  $cpu_max_filter = $ENV{CPU_MAX_FILTER};
}
my $load_daily = 0;                            # do not load daily data from the HMC as default since 4.60
if ( defined $ENV{LOAD_DAILY} ) {
  $load_daily = $ENV{LOAD_DAILY};
}

my $delimiter = "XORUX";                       # this is for rrdtool print lines for clickable legend

#print "++ $host $hmc_user $basedir $webdir $STEP\n";
my $wrkdir = "$basedir/data";

if ( -f "$bindir/premium.pl" ) {
  require "$bindir/premium.pl";
}
else {
  require "$bindir/standard.pl";
}

# Global definitions
my $input_vm                = "";
my $input                   = "";
my $input_mem               = "";
my $input_pool              = "";
my $input_pool_mem          = "";
my $input_pool_sh           = "";
my $input_cod               = "";
my $input_shell             = "";
my $input_mem_shell         = "";
my $input_pool_shell        = "";
my $input_pool_mem_shell    = "";
my $input_pool_sh_shell     = "";
my $input_cod_shell         = "";
my $pool_list_file          = "pool_list.txt";
my $loadhours               = "";
my $loadmins                = "";
my $loadsec_vm              = "";
my $type_sam                = "";
my $managedname             = "";
my $step                    = "";
my $NO_TIME_MULTIPLY        = 18;                           # increased since 4.x?
my $no_time                 = $STEP * $NO_TIME_MULTIPLY;    # says the time interval when RRDTOOL consideres a gap in input data 18 mins now!
                                                            # looks like 60sec sample rate is stored on HMC just for last 2 days (there in no doc so far)
                                                            # INIT_LOAD_IN_HOURS_BACK should be generally high enough  (one year back =~ 9000 hours), it is in hours!!!
                                                            #my $INIT_LOAD_IN_HOURS_BACK="9000";
my $INIT_LOAD_IN_HOURS_BACK = "18";                         # not need more as 1 minute samples are no longer there, daily are not more used since 4.60
my $UPDATE_DAILY            = 86400;                        # for update config (documentation) files
my $CFG_REFRESH             = 7000;                         # update of lpar cfg

my $PARALLELIZATION = 5;

#   $PARALLELIZATION = 1;

# $UPDATE_DAILY=60;
my @pool_list = "";

# Random colors for disk charts
# my @managednamelist_un=();
my @managednamelist     = ();
my @managednamelist_vmw = ();
my $HMC                 = 1;                    # if HMC then 1, if IVM/SDMC then 0
my $SDMC                = 0;
my $IVM                 = 0;
my $FSM                 = 0;                    # so far only for setting lslparutil AMS parames
my @lpar_trans          = "";                   # lpar translation names/ids for IVM systems
my $MAX_ROWS            = 2450;                 # new HMC like 8.8.x has fix limitation 2500 rows set in hmc.properties
my $restrict_rows       = " -n $MAX_ROWS";      # workaround for HMC < 7.2.3 to do not exhaust memory
my $timerange           = " --minutes 120 ";    # it must be global var, placing there some default just to be sure ...

# last timestamp files --> must be for each load separated
my $last_file         = "last.txt";             # for ESXi server
my $last_file_pool    = "last-pool.txt";
my $last_file_mem     = "last-mem.txt";
my $last_file_sh_mem  = "last-sh-mem.txt";
my $last_file_sh_pool = "last-sh-pool.txt";
my $last_file_cod     = "last-cod.txt";

# last file for every resource pool is 'rp_name.last'
my $last_rec     = "";
my $sec          = "";
my $ivmmin       = "";
my $ivmh         = "";
my $ivmd         = "";
my $ivmm         = "";
my $ivmy         = "";
my $wday         = "";
my $yday         = "";
my $isdst        = "";
my $timeout_save = 600;    # timeout for downloading whole server/lpar cfg from the HMC (per 1 server), it prevents hanging

# my $timeout=120; # timeout for vcenter operations

my @returns;               # for forking
my $server_count = 0;
my @pid          = "";
my $cycle_count  = 1;

my @poollistagg        = "";
my $DELETED_LPARS_TIME = 8640000;    # 10 days
my @lpm_excl_vio       = "";
my $run_topten         = 0;
my $view_suff          = "view";
my $mem_params         = "";
my $idle_param         = "";

# disable Tobi's promo
# my $disable_rrdtool_tag = "COMMENT: ";
# my $disable_rrdtool_tag_agg = "COMMENT:\" \"";
my $disable_rrdtool_tag     = "--interlaced";    # just nope string, it is deprecated anyway
my $disable_rrdtool_tag_agg = "--interlaced";    # just nope string, it is deprecated anyway
my $rrd_ver                 = $RRDp::VERSION;
if ( isdigit($rrd_ver) && $rrd_ver > 1.35 ) {
  $disable_rrdtool_tag     = "--disable-rrdtool-tag";
  $disable_rrdtool_tag_agg = "--disable-rrdtool-tag";
}

# keep here green - yellow - red - blue ...
my @color = ( "#FF0000", "#0000FF", "#C0C0C0", "#FFFF00", "#00FFFF", "#FFA500", "#00FF00", "#808080", "#FF00FF", "#800080", "#FDD017", "#0000A0", "#3BB9FF", "#008000", "#800000", "#ADD8E6", "#F778A1", "#800517", "#736F6E", "#F52887", "#C11B17", "#5CB3FF", "#A52A2A", "#FF8040", "#2B60DE", "#736AFF", "#1589FF", "#98AFC7", "#8D38C9", "#307D7E", "#F6358A", "#151B54", "#6D7B8D", "#FDEEF4", "#FF0080", "#F88017", "#2554C7", "#FFF8C6", "#D4A017", "#306EFF", "#151B8D", "#9E7BFF", "#EAC117", "#E0FFFF", "#15317E", "#6C2DC7", "#FBB917", "#FCDFFF", "#15317E", "#254117", "#FAAFBE", "#357EC7", "#4AA02C", "#38ACEC" );
my $color_max = 53;    # 0 - 53 is 54 colors

my @keep_color_lpar = "";

rrdtool_graphv();

my $prem = premium();
print "LPAR2RRD $prem version $version\n" if $DEBUG;
print "Host           : $h\n"             if $DEBUG;
my $date     = "";
my $act_time = localtime();
print "Hyper-V   start: $host $act_time\n" if $DEBUG;

my %hyp_data = ();          # structure with all data of last (newer) date
my %hyp_data_one = ();      # structure with all data of last but one date

my $hyperv_host = "";

if ( !-d "$webdir" ) {
  error( " Pls set correct path to Web server pages, it does not exist here: $webdir" . __FILE__ . ":" . __LINE__ ) && return 0;
}

# cfg_config_change();

# start RRD via a pipe
use RRDp;
RRDp::start "$rrdtool";

my $rrdtool_version = 'Unknown';
$_ = `rrdtool`;
if ( /^RRDtool ([1-9]*\.[0-9]*(\.[0-9]*)?)/ ) {
    $rrdtool_version = $1;
}
print "RRDp    version: $RRDp::VERSION \n";
print "RRDtool version: $rrdtool_version\n";

print "Perl version   : $] \n";

load_hmc();

# close RRD pipe
RRDp::end;

$date = localtime();
print "date end       : $hyperv_host $date\n" if $DEBUG;

exit(0);

### ----------------------- exit main --------------------------

sub delta_get_hyp_val { # creates delta value from actual and last but one values
    my ( $class, $item, $hyp_data_comp_act, $hyp_data_comp_last, $line_contains ) = @_;

    my $values_act = get_hyp_val ( $class, $item, $hyp_data_comp_act,  $line_contains );
    my $values_one = get_hyp_val ( $class, $item, $hyp_data_comp_last, $line_contains );
    if ($item =~ "ReadBytesPersec" || $item =~ "WriteBytesPerse") {
      print "485 \$values_act $values_act \$values_one $values_one \$line_contains $line_contains\n" if $DEBUG == 2;
    }
    if ($item =~ "PercentTotalRunTime") {
      print_hyperv_debug("467000 \$values_act $values_act \$values_one $values_one\n") if $DEBUG == 2;
    }
    if ($values_act eq "UnDeFiNeD") {
      print_hyperv_debug("467001 \$class $class \$item $item \$line_contains $line_contains\n") if $DEBUG == 2;
    }
    # result is one or more values delimited by ','
    my @values_act_arr = split( /,/, $values_act );
    my @values_one_arr = split( /,/, $values_one );
    my $ret_str = "";
    print_hyperv_debug("469 hyp2rrd.pl \@values_act_arr @values_act_arr \@values_one_arr @values_one_arr\n") if $DEBUG == 2;

    my $i = 0;
    while (defined $values_act_arr[$i] && defined $values_one_arr[$i] && isdigit($values_act_arr[$i]) && isdigit($values_one_arr[$i]) ) {
      my $diff = $values_act_arr[$i] - $values_one_arr[$i];
      $diff = 0 if $diff < 0;
      if ($ret_str eq "") {
        $ret_str = $diff;
      }
      else {
        $ret_str .= ",$diff";
      }
      $i++;
    }
    if ($item =~ "PercentTotalRunTime") {
      print_hyperv_debug("487 \$ret_str $ret_str\n") if $DEBUG == 2;
    }
    return $ret_str;
}

sub get_hyp_val {
  my ( $class, $item, $ref_data, $line_contains ) = @_;

  # data example
  # 'Msvm_ComputerSystem' => [
  #                             'Caption|CreationClassName|Description|ElementName|EnabledState|HealthState|Name|NumberOfNumaNodes|OperatingStatus|Status|StatusDescriptions|Hyperv_UTC',
  #                             'Hosting Computer System|Msvm_ComputerSystem|Microsoft Hosting Computer System|HYPERV01|2|5|HYPERV01|1|0|OK|(OK)|1508229032',
  #                             'Virtual Machine|Msvm_ComputerSystem|Microsoft Virtual Machine|XoruX|3|5|143E4B15-1E45-4D7B-B352-ECE54359A778|1|0|OK|(Operating normally)|1508229032',
  #                             'Virtual Machine|Msvm_ComputerSystem|Microsoft Virtual Machine|XoruX-master|2|5|3138F59F-11AD-46FB-951C-9C147C98896C|1|0|OK|(Operating normally)|150822903
  #                             'Virtual Machine|Msvm_ComputerSystem|Microsoft Virtual Machine|WinXP|2|5|8E3AB390-A112-4DCB-AAB4-CE51042A7A4A|1|0|OK|(Operating normally)|1508229032'
  #                          ],
  # format $line_contains is either a string or special string starting with 'ITEM='
  # $line_contains = "ITEM=Name=\"$csv_number\"";

  my $line_contains_or     = "";
  my $line_contains_bslash = "";
  my $item_name    = "";
  my $item_contain = "";

  if (not defined $line_contains) {
    $line_contains = "";
  }
  elsif ($line_contains =~ "^ITEM="){
    (undef,$item_name, $item_contain) = split ("=",$line_contains);
  }
  else {
  #  # $line_contains = "\"$line_contains";
  #
    $line_contains_or     = $line_contains."\"" if $line_contains ne ""; # differ two strings having same beginning
    $line_contains_bslash = $line_contains."\\" if $line_contains ne ""; # differ two strings having same beginning
  }

  # $ref_data is a pointer to hash of computer data

  #  print "get_hyp_val, \$class $class \$line_contains $line_contains \$line_contains_bslash $line_contains_bslash\n";

  my $ret_val = $UnDeF; # return it when any error
  # print "524XX \$class ,$class,\n";

  if ( !defined $$ref_data->{$class}[0] ) {
    error("not defined \$ref_data { $class } [0]". __FILE__ . ":" . __LINE__);
    return $ret_val;
  }
  my $hyp_line = $$ref_data->{$class}[0];

  my @os_arr   = split( /,/, $hyp_line );
  my $index    = -1;                                   # the last one is Hyperv_UTC
  my $item_index = "";

  if ( $item ne "Hyperv_UTC" ) {                       # always the last
    # what is index of item?
    ($index) = grep { $os_arr[$_] eq "\"$item\"" } 0 .. $#os_arr;
    if ( !defined $index ) {
      error("not defined \$item $item in \$hyp_line $hyp_line". __FILE__ . ":" . __LINE__);
      return $ret_val;
    }
    if ($item_name ne "") {
      ($item_index) = grep { $os_arr[$_] eq "\"$item_name\"" } 0 .. $#os_arr;
      if ( !defined $item_index ) {
        error("not defined \$item_name $item_name in \$hyp_line $hyp_line". __FILE__ . ":" . __LINE__);
        return $ret_val;
      }
    }
  } ## end if ( $item ne "Hyperv_UTC")

  # print "\$index $index\n";
  $ret_val = "";
  my $line_x = 1;
  my $coma   = "";
  while ( defined $$ref_data->{$class}[$line_x] ) {
    $hyp_line = $$ref_data->{$class}[$line_x];
    $line_x++;
    # print_hyperv_debug("531 \$hyp_line $hyp_line \$line_contains $line_contains \$item_name $item_name \$item_index $item_index\n"); # if $DEBUG == 2;
    if ($item_name eq "") {
      next if $line_contains ne "" && index($hyp_line, $line_contains_or) == -1 && index($hyp_line, $line_contains_bslash) == -1 && index($hyp_line, $line_contains.":") == -1 && index($hyp_line, $line_contains.".") == -1 && index($hyp_line, $line_contains."_") == -1; # exact test
    }
    else {
      next if ( split( /,/, $hyp_line ) )[$item_index] ne $item_contain;
    }
    $ret_val .= $coma;
    my $atom = ( split( /,/, $hyp_line ) )[$index];
    # remove quotes in the beginning and end
    $atom =~ s/^\"//;
    $atom =~ s/\"$//;
    $ret_val .= $atom;
    $coma = ",";
  } ## end while ( defined $hyp_data...)
  $ret_val = $UnDeF if $ret_val eq "";
  return $ret_val;
} ## end sub get_hyp_val

sub load_hmc {
  # take sorted <<files>> from dir $tmpdir/HYPERV/$HYPDIR
  # for every two timely consecutive files call load_hmc_one
  # finished file mv to dir lpar2rrd/tmp/HYPERV/$HYPDIR so that there are always two latest files

  $hyperv_host = $host;
  $host =~ s/\//&&1/g;    # general replacement for slash
  my $comp_uuid = $host;

  my $pth = "$tmpdir/HYPERV/$host"; # real data
  # my $pth = "$tmpdir/HYPERV_X/$host"; # debug data
  # my $pth = $host;
  $pth=~ s/\&\&1/\//g;
  # reading & working with data txt files in HYPERV
  # print "615 \$pth $pth\n";

  if (! opendir( DIR, "$pth" )) {
    error("cannot open dir $pth, exiting");
    return;
  }
   my @files_iostats = grep /\d{10}.txt$/, (grep !/^\.\.?$/, readdir(DIR));
  closedir(DIR);
  my @sorted_files = sort @files_iostats;
  print "\@sorted_files  : @sorted_files\n";
  return if scalar @sorted_files < 2; # at least two files
  my $indx = 1;
  while (defined $sorted_files[$indx]) {
    # 1504012098 1504012998 limit max 3600 sec
    my $old_time = $sorted_files[$indx-1];
    $old_time =~ s/\.txt$//;
    #$old_time =~ /_(\d+)\.prf/;
    #$old_time =$1;
    my $new_time = $sorted_files[$indx];
    $new_time =~ s/\.txt$//;
    #$new_time =~ /_(\d+)\.prf/;
    #$new_time =$1;

    if (($new_time - $old_time) < 3601) { # only when lower time diff
      load_hmc_all_comps("$pth/$sorted_files[$indx]", "$pth/$sorted_files[$indx-1]");
    }

    # save files to lpar2rrd/tmp/hyperv_$host_[last & act]
    my $file_last = $sorted_files[$indx-1];
    $file_last =~ s/$old_time.txt/hyperv_/;
    $file_last = "$file_last$comp_uuid"."_last";
    my $file_act = $sorted_files[$indx];
    $file_act =~ s/$new_time.txt/hyperv_/;
    $file_act = "$file_act$comp_uuid"."_act";
    print "598 move $pth/$sorted_files[$indx-1] to $tmpdir/$file_last & copy $pth/$sorted_files[$indx] to $tmpdir/$file_act\n" if $DEBUG == 2;
    if (! move "$pth/$sorted_files[$indx-1]", "$tmpdir/$file_last") {
      error("cannot move $pth/$sorted_files[$indx-1] to $wrkdir/$tmpdir/$file_last");
    }
    if (! copy "$pth/$sorted_files[$indx]", "$tmpdir/$file_act") {
      error("cannot copy $pth/$sorted_files[$indx] to $wrkdir/$tmpdir/$file_act");
    }
    $indx++;
  }

  return;
}

sub load_hmc_all_comps {
  my $file_act = shift;
  my $file_last_but_one = shift;
  print_hyperv_debug("641 $file_act $file_last_but_one\n") if $DEBUG == 2;

  hyperv_load($file_act, $file_last_but_one);

  # print "647 result from hyperv_load.pl\n";
  # print Dumper (\%hyp_data);

  # cycle on comp in actual data hash
  # if the same comp in last but one hash then load data

  foreach my $comp (keys %hyp_data) {
    if (exists $hyp_data_one { $comp }) {
      print_hyperv_debug("653 loading data for computer $comp\n") if $DEBUG == 2;
      load_hmc_one_comp (\$hyp_data{$comp}, \$hyp_data_one{$comp} );
    }
  }
  if (!(!keys %all_clusterstorage)) {
    print "cluster storage:\n";
    print Dumper(\%all_clusterstorage);
  }
  if (!(!keys %skipped_computers)) {
    # print "skipped comps  :\n";
    # print Dumper(\%skipped_computers);

    # try to find nodes of possible cluster
    # cluster storage:
    # $VAR1 = {
    #           'HVNODE01' => {
    #                          '?Volume{b994eb5b-844f-45a7-86ea-6116e3175653}' => 'ahojte',
    #                          '?Volume{1d2ac989-641d-4fa0-9ee0-6a391c8be0da}' => 'ahojte'
    #                         },
    #           'HVNODE02' => {
    #                           '?Volume{1d2ac989-641d-4fa0-9ee0-6a391c8be0da}' => 'ahojte',
    #                           '?Volume{b994eb5b-844f-45a7-86ea-6116e3175653}' => 'ahojte'
    #                         }
    #         };
    # skipped comps  :
    # $VAR1 = {
    #           'HVNODE01' => 'MSNET-HVCL'
    #         };
    # looking for cluster nodes
    # $VAR1 = {
    #          'HVNODE01' => 'cluster_node',
    #          'HVNODE02' => 'cluster_node'
    #         };

    if (!(!keys %all_clusterstorage)) {
      # seems there is cluster here
      foreach my $comp (keys %skipped_computers) {
        my $cluster_name = $skipped_computers{$comp};
        my %cluster_nodes = ();
        if (exists $all_clusterstorage{$comp}) {
          $cluster_nodes{$comp} = "cluster_node";
          foreach my $storage (keys %{$all_clusterstorage{$comp}} ) {
            foreach my $cmp (keys %all_clusterstorage) {
              if (exists $all_clusterstorage{$cmp}{$storage}) {
                $cluster_nodes{$cmp} = "cluster_node";
              }
            }
          }
          print "cluster nodes  : for cluster $cluster_name\n";
          print Dumper(\%cluster_nodes);

          # prepare table cluster nodes  !!!! see MSCluster node to node
          my @node_list = ();
          foreach (keys %cluster_nodes) {
            my $node_name = $_;
            print_hyperv_debug("745 \$node_name $node_name\n") if $DEBUG == 2;
            my $RoleOfNode  = "UnDeFiNeD";
            my $StateOfNode = "UnDeFiNeD";
            my $domain_name = "UnDeFiNeD";
            if (exists $server_domain{$node_name}) {
              $domain_name = $server_domain{$node_name};
            }

            my $line = $cluster_name .",". $domain_name .",". $node_name .",". $RoleOfNode .",". $StateOfNode . "\n";
            push @node_list, $line;
          }

          # if does not exist cluster dir, prepare it
          my $cluster_dir = "$wrkdir_windows/cluster_$cluster_name";

          if (! -d "$cluster_dir" ) {
            print "mkdir          : $cluster_dir\n";
            mkdir( "$cluster_dir", 0755 ) || error( " Cannot mkdir $cluster_dir: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
          }

          #prepare node_list table
          my $res_ret = FormatResults(@node_list);

          open FH, '>:encoding(UTF-8)', "$cluster_dir/node_list.html" or error( "can't open $cluster_dir/node_list.html: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
          binmode( FH, ":utf8" );

          print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
          print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">CLUSTER&nbsp;&nbsp;&nbsp;&nbsp;</TH>
            <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;DOMAIN&nbsp;&nbsp;&nbsp;</TH>
            <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Node&nbsp;&nbsp;&nbsp;</TH>
            <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;RoleOfNode&nbsp;&nbsp;&nbsp;</TH>
  			    <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;StateOfNode&nbsp;&nbsp;&nbsp;</TH>
            </TR></thead><tbody>\n";
          print FH "$res_ret";

          print FH "</tbody></TABLE></CENTER>\n";
          close FH;
        }
        else {
          print "skipped comp   : $comp $cluster_name\n";
        }
      }
    }
    else {
      print "skipped comps  :\n";
      print Dumper(\%skipped_computers);
    }
  }
}

sub load_hmc_one_comp {
  my $hyp_data_comp_act  = shift; # hash pointer
  my $hyp_data_comp_last = shift; # hash pointer

  # get system time

  $command_date = get_hyp_val( "Win32_OperatingSystem", "LocalDateTime", $hyp_data_comp_act );
  chomp($command_date);

  $command_unix = get_hyp_val( "Win32_OperatingSystem", "Hyperv_UTC", $hyp_data_comp_act );

  # print "system UTC     : $command_date : $command_unix";

  # fetch server SW Type
  $apiType_top = get_hyp_val( "Win32_OperatingSystem", "Caption", $hyp_data_comp_act );
  $fullName_top = "";
  print "system      UTC: $apiType_top $fullName_top UTC:$command_date : $command_unix\n";

  # system SW type test
  # if ( $apiType_top ne "Microsoft Windows Server 2012 R2 Datacenter Evaluation" ) {
  #  error("Unknown server SW type $apiType_top\n");

    #    return
  # }

  # fetch HostSystem   'Win32_ComputerSystemProduct' UUID  'Win32_OperatingSystem' CSName
  my %pole = ();
  $pole{'name'} = get_hyp_val( 'Win32_OperatingSystem',       'CSName', $hyp_data_comp_act );
  $pole{'UUID'} = get_hyp_val( 'Win32_ComputerSystemProduct', 'UUID', $hyp_data_comp_act );
  my @spole = ();
  $spole[0]              = \%pole;
  $managednamelist_un[0] = \@spole;

  if ( !defined $managednamelist_un[0] || $managednamelist_un[0] eq '' ) {
    error("hyperv name: $host either has not been resolved or ssh key based access is not allowed or other communication error");
    exit(1);
  }
  if ( !defined $managednamelist_un[0] ) {
    error("hyperv name: $host has not array of hosts ?!?");
    print Dumper( $managednamelist_un[0] );
    exit(1);
  }

  if ( $managednamelist_un[0] =~ "no address associated with hostname" || $managednamelist_un[0] =~ "Could not resolve hostname" ) {
    error("vmware : $managednamelist_un[0]");
    exit(1);
  }

  hostsystem_perf( $hyp_data_comp_act, $hyp_data_comp_last);    # Host and all Virtual Machines

  return;

#
#---------------------------------------------------------------------------------------------------------------------------------------
#

  # fetch clusters if exist (only for vcenter)
  if ( $apiType_top =~ "VirtualCenter" ) {
    eval {
      # get cluster info
      $cluster_list = Vim::find_entity_views(
        view_type  => "ClusterComputeResource",
        properties => [ 'name', 'summary' ]
      );
      if ( !defined $cluster_list || ( $cluster_list eq "" ) ) {
        error( "Undefined cluster in vmware $host: " . __FILE__ . ":" . __LINE__ ) && exit(1);
      }
    };
    if ($@) {
      error( "vmw2rrd failed: $@ " . __FILE__ . ":" . __LINE__ );
      exit(1);
    }

    cluster_perf();
  } ## end if ( $apiType_top =~ "VirtualCenter")

  # fetch datacenters only for VC version higher than 4
  # if ($apiType_top =~ 'VirtualCenter') curly
  # print "\$fullName_top ,$fullName_top,\n";
  if ( $apiType_top =~ 'xxxHostAgent' ) {
    $datacenter_list = Vim::find_entity_views( view_type => "Datacenter", properties => ['name'] );    # ['datastore']);
    if ( !defined $datacenter_list || ( $datacenter_list eq "" ) ) {
      error( "Undefined datacenter_list in vmware $host: " . __FILE__ . ":" . __LINE__ ) && exit(1);
    }

    #print Dumper ($datacenter_list);
    foreach $datacenter (@$datacenter_list) {

      # print Dumper($datacenter);

      # fetch datastores
      $datastore_list = Vim::find_entity_views( view_type => "Datastore", properties => [ 'name', 'summary', 'vm', 'info' ], begin_entity => $datacenter );
      if ( !defined $datastore_list || ( $datastore_list eq "" ) ) {
        error( "Undefined datastore_list in vmware $host:datacenter $datacenter: " . __FILE__ . ":" . __LINE__ ) && next;
      }
      print Dumper($datastore_list);

      # $datastore_list = $datastore_list->datastore; # this is the difference from vCenter
      if ( !defined $datastore_list || ( $datastore_list eq "" ) ) {
        error( "Undefined datastore_list in vmware $host:datacenter $datacenter: " . __FILE__ . ":" . __LINE__ ) && next;
      }
      my $datacenter_moref = $datacenter->{'mo_ref'}->value;

      # my $denter_moref = $dcl->{'mo_ref'}->value;
      print "\$datacenter_moref $datacenter_moref\n";

      # my $dlist = $dcl->datastore;
      # print Dumper($dlist);
      foreach my $datastore (@$datastore_list) {
        my $ds_name = $datastore->name;
        print "----------- \$ds_name $ds_name \$managedname $managedname\n";
      }
    } ## end foreach $datacenter (@$datacenter_list)
  } ## end if ( $apiType_top =~ 'xxxHostAgent')

  if ( $fullName_top =~ 'vCenter Server 5' || $fullName_top =~ 'vCenter Server 6' || $apiType_top =~ 'HostAgent' ) {
    $datacenter_list = Vim::find_entity_views(
      view_type  => "Datacenter",
      properties => ['name']
    );

    if ( !defined $datacenter_list || ( $datacenter_list eq "" ) ) {
      error( "Undefined datacenter_list in vmware $host: " . __FILE__ . ":" . __LINE__ ) && exit(1);
    }

    if ( $apiType_top =~ "VirtualCenter" ) {
      $managedname = "vmware_$vmware_uuid";
    }
    else {    # for non vcenter should be already set up
              # $managedname = "";
    }

    # until 4.81-004 datacenter dir had own user name, but came problem with Marne-La-Vallée
    # since then datacenter dir name = mo_ref and inside is touched file <datacenter name>.dcname

    foreach $datacenter (@$datacenter_list) {

      # print Dumper($datacenter);

      # fetch datastores
      $datastore_list = Vim::find_entity_views( view_type => "Datastore", properties => [ 'name', 'summary', 'vm', 'info' ], begin_entity => $datacenter );
      if ( !defined $datastore_list || ( $datastore_list eq "" ) ) {
        error( "Undefined datastore_list in vmware $host:datacenter $datacenter: " . __FILE__ . ":" . __LINE__ ) && next;
      }

      my $datacenter_moref = $datacenter->{'mo_ref'}->value;
      $h_name = "datastore_" . $datacenter->name;    # cus contains only datastores

      # new solution
      my $datacenter_name = $h_name;
      my $moref_name      = "datastore_" . $datacenter_moref;
      if ( -d "$wrkdir/$managedname/$h_name" ) {

        # existing must be renamed
        move( "$wrkdir/$managedname/$h_name", "$wrkdir/$managedname/$moref_name" ) || error( " Cannot move $wrkdir/$managedname/$h_name to $moref_name: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      $h_name = $moref_name;

      print "retrieving perf: datacenter $datacenter_name mo_ref $h_name for $managedname\n";

      #  if ($apiType_top =~ 'HostAgent') {
      #      foreach my $datastore (@$datastore_list) {
      #        my $ds_name        = $datastore->name;
      #        print "----------- \$ds_name $ds_name\n";
      #      }
      #      next;
      #  }
      # prepare dir datacenter

      if ( !-d "$wrkdir/$managedname/$h_name" ) {

        # in case there is no cluster create also $managedname dir
        if ( !-d "$wrkdir/$managedname" ) {
          print "mkdir          : $h_name:$managedname $wrkdir/$managedname\n" if $DEBUG;
          LoadDataModuleHyperV::touch("$wrkdir/$managedname");
          mkdir( "$wrkdir/$managedname", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
        print "mkdir          : $h_name:$managedname $wrkdir/$managedname/$h_name\n" if $DEBUG;
        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name");
        mkdir( "$wrkdir/$managedname/$h_name", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname/$h_name: $!" . __FILE__ . ":" . __LINE__ ) && next;
      } ## end if ( !-d "$wrkdir/$managedname/$h_name")
      if ( $apiType_top !~ 'HostAgent' ) {
        if ( !-f "$wrkdir/$managedname/$h_name/vcenter" ) {
          `touch "$wrkdir/$managedname/$h_name/vcenter"`;    # say install_html.sh that it is vcenter
          LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/vcenter");
        }
      } ## end if ( $apiType_top !~ 'HostAgent')
      if ( $apiType_top !~ 'HostAgent' ) {
        my @files = glob("$wrkdir/$managedname/$h_name/vcenter_name_*");
        if ( !defined $files[0] || scalar @files > 1 || $files[0] ne "$wrkdir/$managedname/$h_name/vcenter_name_$host" ) {
          `rm -f "$wrkdir/$managedname/$h_name"/vcenter_name_*`;        # in case there are more names
          `touch "$wrkdir/$managedname/$h_name/vcenter_name_$host"`;    # say install_html.sh that it is vcenter name
          LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/vcenter_name_$host");
        }
      } ## end if ( $apiType_top !~ 'HostAgent')

      # new solution after 4.81-004
      # touch user datacenter name
      my $datacenter_name_file = "$datacenter_name.dcname";
      if ( !-f "$wrkdir/$managedname/$h_name/$datacenter_name_file" ) {
        `rm -f "$wrkdir/$managedname/$h_name"/*.dcname`;                 # in case there are more names
        `touch "$wrkdir/$managedname/$h_name/$datacenter_name_file"`;    # save user datacenter name
        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/$datacenter_name_file");
      }

      my $host_orig = $host;
      $host = $h_name;
      datastore_perf();
      $host = $host_orig;
    } ## end foreach $datacenter (@$datacenter_list)
  } ## end if ( $fullName_top =~ ...)

  return;                                                                # from load_hmc
} ## end sub load_hmc

#### ----- #########

sub cluster_perf {
  return if ( ( !defined $cluster_list ) || ( $cluster_list eq "" ) );

  $do_fork = "0";
  my $Resources_once = 0;                                                # only once print in menu.txt for Unregistered VMs

  # $host_orig keeps vmware connect name
  my $host_orig = $host;                                                 # necessary to know for host in cluster

  foreach my $cluster (@$cluster_list) {
    my $cluster_moref      = $cluster->{'mo_ref'}->value;
    my $cluster_moref_name = "cluster_" . $cluster_moref;
    $cluster_effectiveMemory = $cluster->summary->effectiveMemory;
    $cluster_effectiveCpu    = $cluster->summary->effectiveCpu;

    # print "\$cluster_effectiveMemory $cluster_effectiveMemory\n";
    $h_name           = "cluster_" . $cluster->name;
    $fail_entity_name = $h_name;
    $fail_entity_type = "cluster";

    $managedname = "vmware_$vmware_uuid";

    # $host = $h_name ;  # new host as a second dir name under data/

    # since >4.81-004 to enable renaming of clusters
    $host = $cluster_moref_name;
    print "retrieving perf: cluster $h_name mo_ref $cluster_moref for $managedname\n";

    if ( !-d "$wrkdir" ) {
      print "mkdir          : $host:$managedname $wrkdir\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$host:$managedname $wrkdir");
      mkdir( "$wrkdir", 0755 ) || error( " Cannot mkdir $wrkdir: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    if ( !-d "$wrkdir/$managedname" ) {
      print "mkdir          : $host:$managedname $wrkdir/$managedname\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$wrkdir/$managedname");
      mkdir( "$wrkdir/$managedname", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    # vcenter must know its alias
    my $my_alias_file_name = "$wrkdir/$managedname/vmware_alias_name";
    open FH, ">$my_alias_file_name" or error( "can't open $my_alias_file_name: $!" . __FILE__ . ":" . __LINE__ );
    print FH "$h_name|$alias\n";    # save cluster name and alias, ! when there are more clusters, you store the last $h_name
    close FH;

    if ( !-d "$wrkdir/$managedname/$host" && -d "$wrkdir/$managedname/$h_name" ) {    # rename from name to moref (new system)
      print "rename  cluster: $wrkdir/$managedname/$h_name to $wrkdir/$managedname/$host\n";
      move( "$wrkdir/$managedname/$h_name", "$wrkdir/$managedname/$host" ) || error( " Cannot move $wrkdir/$managedname/$h_name to $cluster_moref: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }
    if ( !-d "$wrkdir/$managedname/$host" ) {                                         # new cluster
      print "mkdir          : $host:$managedname $wrkdir/$managedname/$host\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host");
      mkdir( "$wrkdir/$managedname/$host", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname/$host: $!" . __FILE__ . ":" . __LINE__ ) && next;

      if ( !-f "$wrkdir/$managedname/$host/vcenter" ) {
        `touch "$wrkdir/$managedname/$host/vcenter"`;                                 # say install_html.sh that it is vcenter
      }
    } ## end if ( !-d "$wrkdir/$managedname/$host")
    if ( !-f "$wrkdir/$managedname/$host/vcenter" ) {
      `touch "$wrkdir/$managedname/$host/vcenter"`;                                   # say install_html.sh that it is vcenter
    }
    my @files_vname = glob("$wrkdir/$managedname/$h_name/vcenter_name_*");
    if ( !defined $files_vname[0] || scalar @files_vname > 1 || $files_vname[0] ne "$wrkdir/$managedname/$host/vcenter_name_$host_orig" ) {
      `rm -f "$wrkdir/$managedname/$host"/vcenter_name_*`;                            # in case there are more names
      `touch "$wrkdir/$managedname/$host/vcenter_name_$host_orig"`;                   # say install_html.sh vcenter name
    }
    if ( !-f "$wrkdir/$managedname/$host/cluster_name_$h_name" ) {
      `rm -f "$wrkdir/$managedname/$host"/cluster_name_*`;                            # in case there are more names
      `touch "$wrkdir/$managedname/$host/cluster_name_$h_name"`;                      # say install_html.sh cluster name
    }
    my $vmware_signal_file = "$wrkdir/$managedname/$host/vmware.txt";
    if ( !-f "$vmware_signal_file" ) {
      `touch "$vmware_signal_file"`;                                                  # say install_html.sh that it is vmware
    }

    prepare_last_time( $cluster, $et_ClusterComputeResource );

    # print "# prepare list of Hosts in this cluster\n";

    my @manag_list       = ();
    my $hosts_in_cluster = Vim::find_entity_views(
      view_type  => "HostSystem",
      properties => [ 'name', 'parent' ]
    );

    # print Dumper($hosts_in_cluster);

    # print "# prepare for frame_multi & multiview_hmc\n";
    foreach my $host_c (@$hosts_in_cluster) {
      my $host_parent_moref = $host_c->get_property('parent.value');
      my $host_name         = $host_c->get_property('name');

      if ( $cluster_moref eq $host_parent_moref ) {
        my $items_path = "$host_name" . "XORUX" . "$host_orig";    # original $host
        push @manag_list, $items_path;

        # server must know its cluster and alias
        my $my_cluster_file_name = "$wrkdir/$host_name/$host_orig/my_cluster_name";
        my $cluster_name         = "";
        if ( -f $my_cluster_file_name ) {
          open FH, "$my_cluster_file_name" or error( "can't open $my_cluster_file_name: $!" . __FILE__ . ":" . __LINE__ ) && next;
          $cluster_name = <FH>;
          chomp $cluster_name;
          close FH;
        } ## end if ( -f $my_cluster_file_name)
        if ( $cluster_name eq "" || $cluster_name ne "$h_name|$alias" ) {
          open FH, ">$my_cluster_file_name" or error( "can't open $my_cluster_file_name: $!" . __FILE__ . ":" . __LINE__ ) && next;

          # print FH "$host|$alias\n"; # save host name and alias
          print FH "$h_name|$alias\n";    # save host name and alias
          close FH;
          LoadDataModuleHyperV::touch("$my_cluster_file_name");
        } ## end if ( $cluster_name eq ...)
        unlink "$wrkdir/$host_name/$host_orig/im_in_cluster";    # this esxi is in cluster
      } ## end if ( $cluster_moref eq...)
    } ## end foreach my $host_c (@$hosts_in_cluster)

    # print "hosts in cluster $cluster_moref\n";
    # print Dumper(@manag_list);
    # for event debug > save it

    my $hosts_in_cluster_file = "$wrkdir/$managedname/$host/hosts_in_cluster";
    open FH, ">$hosts_in_cluster_file" or error( "can't open $hosts_in_cluster_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
    foreach my $new_line (@manag_list) {
      chomp($new_line);
      print FH $new_line . "\n";
    }
    close FH;
    print "writing hosts  : $hosts_in_cluster_file\n";

    #    $host = $ENV{HMC};
    @managednamelist_vmw = @manag_list;
    make_cmd_frame_multiview( $managedname, $host, $et_ClusterComputeResource );

    #
    ###  resourcepools
    #

    $resourcepool_list = Vim::find_entity_views(
      view_type    => "ResourcePool",
      properties   => [ 'name', 'vm', 'config' ],
      begin_entity => $cluster
    );

    #       properties => ['name','vm','config.cpuAllocation.reservation','config.memoryAllocation.reservation',
    #                                  'config.cpuAllocation.limit','config.memoryAllocation.limit' ],begin_entity=>$cluster);

    if ( !defined $resourcepool_list || ( $resourcepool_list eq "" ) ) {
      error( "Undefined resourcepool list in vmware $host:cluster $cluster " . __FILE__ . ":" . __LINE__ ) && next;
    }

    $managedname = "vmware_$vmware_uuid";

    $do_fork = "0";
    my @all_paths      = ();
    my $all_paths_file = "$wrkdir/$managedname/$host/active_rp_paths.txt";

    # print Dumper (%vm_id_path);
    foreach my $resourcepool (@$resourcepool_list) {
      $rp_name = $resourcepool->name;

      # resourcepool does not have uuid  --  !!!!!! ???? !!!!!
      $rp_moref = $resourcepool->{'mo_ref'}->value;

      # print "starting retrieving rp $rp_name $rp_moref ---------------------\n";
      if ( !defined $rp_name || ( $rp_name eq "" ) ) {
        error( "not defined or empty name of resourcepool in cluster $h_name  " . __FILE__ . ":" . __LINE__ ) && next;
      }

      if ( ( $rp_name eq "Resources" ) && ($Resources_once) ) {next}
      ;    # can be only once
      $Resources_once++;

      $rp_cpu_reservation = $resourcepool->get_property('config.cpuAllocation.reservation');
      $rp_mem_reservation = $resourcepool->get_property('config.memoryAllocation.reservation');
      $rp_cpu_limit       = $resourcepool->get_property('config.cpuAllocation.limit');
      $rp_mem_limit       = $resourcepool->get_property('config.memoryAllocation.limit');

      # print "::::::::::::::: cpu_res $rp_cpu_reservation mem_res $rp_mem_reservation cpu_lim $rp_cpu_limit mem_lim $rp_mem_limit\n";
      $rp_cpu_reservation = 0 if $rp_cpu_reservation == -1;
      $rp_mem_reservation = 0 if $rp_mem_reservation == -1;
      $rp_cpu_limit       = 0 if $rp_cpu_limit == -1;
      $rp_mem_limit       = 0 if $rp_mem_limit == -1;

      $fail_entity_name = $rp_name;
      $fail_entity_type = "resourcepool";

      @rp_vm_morefs = ();
      my @rp_vm_morefs_old = ();

      $managedname = "vmware_$vmware_uuid";

      my $rp_vm_morefs_file = "$wrkdir/$managedname/$host/rp__vmid__$rp_moref.txt";

      # since > 4.81-004
      if ( -f "$wrkdir/$managedname/$host/rp__vmid__$rp_name.txt" ) {    # old name
        print "rename respool : $wrkdir/$managedname/$host/rp__vmid__$rp_name.txt to $rp_vm_morefs_file\n";
        move( "$wrkdir/$managedname/$host/rp__vmid__$rp_name.txt", "$rp_vm_morefs_file" ) || error( " Cannot move rp__vmid__$rp_name.txt to $rp_vm_morefs_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }

      # print "looking for file $rp_vm_morefs_file\n";
      if ( -f "$rp_vm_morefs_file" ) {
        open FH, "$rp_vm_morefs_file" or error( "can't open $rp_vm_morefs_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
        @rp_vm_morefs_old = <FH>;
        close FH;
      }

      # new system of tracking VM presence in hostsystem
      # for every hostsystem ( resourcepool too)
      # - open VM hosting file
      # - track every VM - active and also non-active
      # - save Vm hosting file
      # during cycle hold hosting info in an array

      my @hosting_arr = ();

      # since > 4.81-004
      my $vmr_file = "$wrkdir/$managedname/$host/$rp_moref.vmr";
      if ( -f "$wrkdir/$managedname/$host/$rp_name.vmr" ) {    # old name
        print "rename respool : $wrkdir/$managedname/$host/$rp_name.vmr to $vmr_file\n";
        move( "$wrkdir/$managedname/$host/$rp_name.vmr", "$vmr_file" ) || error( " Cannot move $rp_name.vmr to $vmr_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      VM_hosting_read( \@hosting_arr, "$vmr_file" );

      my $vm = $resourcepool->vm;

      foreach my $vmsp (@$vm) {
        my $vm_mo_ref_id = $vmsp->value;

        # print "VM value $vm_mo_ref_id";
        if ( ( defined $vm_mo_ref_id ) && ( $vm_mo_ref_id ne "" ) ) {

          # get VM_name
          my $vm_uuid = $vm_id_path{"$vm_mo_ref_id"};    # is already prepared
          if ( !defined $vm_uuid ) {
            error( "vm_id_path does not exist for vm mo_ref $vm_mo_ref_id $fail_entity_type $fail_entity_name " . __FILE__ . ":" . __LINE__ ) && next;
          }
          $vm_uuid = basename($vm_uuid);
          $vm_uuid =~ s/\.rrm$//;
          if ( !uuid_check($vm_uuid) ) {
            error( "cannot find uuid for mo_ref $vm_mo_ref_id " . __FILE__ . ":" . __LINE__ ) && next;
          }

          push @rp_vm_morefs, $vm_mo_ref_id . "\n";
          VM_hosting_update( \@hosting_arr, $vm_uuid, $command_unix );

          #  print "\@hosting_arr @hosting_arr\n";
        } ## end if ( ( defined $vm_mo_ref_id...))
        else {
          error( "not defined vm moref in $fail_entity_type $fail_entity_name in cluster $h_name  " . __FILE__ . ":" . __LINE__ ) && next;
        }
      } ## end foreach my $vmsp (@$vm)

      VM_hosting_write( \@hosting_arr, "$vmr_file", $command_unix );

      # print "@rp_vm_morefs\n";
      push @all_paths, $rp_vm_morefs_file;

      open FH, ">$rp_vm_morefs_file" or error( "can't open $rp_vm_morefs_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
      foreach my $new_line (@rp_vm_morefs) {
        chomp($new_line);
        print FH $new_line . "\n";
      }
      close FH;
      print "writing respool: $rp_vm_morefs_file\n";

      # get perf
      # existing must be renamed
      my $rp_path = "$wrkdir/$managedname/$h_name";
      if ( -f "$rp_path/$rp_name.rrc" ) {
        print "rename respool : $rp_path/$rp_name.rrc to $rp_path/$rp_moref.rrc\n";
        move( "$rp_path/$rp_name.rrc", "$rp_path/$rp_moref.rrc" ) || error( " Cannot move $rp_path/$rp_name.rrc to $rp_path/$rp_moref.rrc: $!" . __FILE__ . ":" . __LINE__ ) && next;

        if ( -f "$rp_path/$rp_name.last" ) {
          print "rename respool : $rp_path/$rp_name.last to $rp_path/$rp_moref.last\n";
          move( "$rp_path/$rp_name.last", "$rp_path/$rp_moref.last" ) || error( " Cannot move $rp_path/$rp_name.last to $rp_path/$rp_moref.last: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
      } ## end if ( -f "$rp_path/$rp_name.rrc")

      # print "----- before \$h_name $h_name \$host $host\n";
      prepare_last_time( $resourcepool, $et_ResourcePool, $rp_name, $rp_moref );    # hash, type, name, moref
                                                                                    # print "----- after \$h_name $h_name \$host $host\n";

      # touch user resourcepool name <user name>.uuid
      my $rp_name_file = "$rp_name.$rp_moref";
      if ( !-f "$wrkdir/$managedname/$host/$rp_name_file" ) {
        `rm -f "$wrkdir/$managedname/$host"/*."$rp_moref"`;                         # in case there are more names
        `touch "$wrkdir/$managedname/$host/$rp_name_file"`;                         # save user rp name
        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host/$rp_name_file");
      }
      if ( !-f "$wrkdir/$managedname/$host/$rp_moref.rrc" ) {                       # if resourcepool has no data
        `touch "$wrkdir/$managedname/$host/$rp_moref.rrc"`;

        # print "signal file : $wrkdir/$managedname/$host/$rp_moref.rrc touched as resourcepool\n";
        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host/$rp_moref.rrc");

        # just to say install-html.sh to create item in menu.txt
        # signal file will be overwritten by creating real rrd file in LoadDataModuleVmware.pm after data coming
      } ## end if ( !-f "$wrkdir/$managedname/$host/$rp_moref.rrc")

      make_cmd_frame_multiview( $managedname, $host, $et_ResourcePool );
    } ## end foreach my $resourcepool (@$resourcepool_list)
    open FH, ">$all_paths_file" or error( "can't open $all_paths_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
    foreach my $new_line (@all_paths) {
      chomp($new_line);
      print FH $new_line . "\n";
    }
    close FH;
    print "writing respool: $all_paths_file\n";
  } ## end foreach my $cluster (@$cluster_list)

  # finish all clusters
  # if there file im_in_cluster left, must reload menu
  #my $file_pth = "$wrkdir/*/$h_name/im_in_cluster"; # path to find files
  #$file_pth =~ s/ /\\ /g;
  #my $no_name = "";
  #my @files = (<$file_pth$no_name>); # unsorted, workaround for space in names
  my @files = glob("$wrkdir/*/$h_name/im_in_cluster");
  if ( @files > 0 ) {
    LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/im_in_cluster");
  }

  $host = $host_orig;
} ## end sub cluster_perf

sub datastore_perf {

  # global var
  # $h_name  datastore_'datacenter name'
  # $datastore_list
  # dir for saving perf is $wrkdir/$managedname/$h_name
  my @datastore_VM = ();    # active VM list

  $do_fork = "1";

  #$do_fork = "0";
  @returns = ();

  # until 4.81-004 datastore files had own user name, but problem with renaming
  # since then datastore files are uuid and there is touched file <datastore name>.uuid

  foreach my $datastore (@$datastore_list) {
    $ds_type        = "";
    $ds_name        = $datastore->name;
    $ds_accessible  = $datastore->summary->accessible;    # 1=true/0=false
                                                          # print "datastore \$ds_name $ds_name \$ds_accessible ,$ds_accessible,\n";
    $ds_freeSpace   = $datastore->summary->freeSpace;
    $ds_freeSpace   = $ds_freeSpace / 1024;               # to be in KB
    $ds_capacity    = $datastore->summary->capacity;
    $ds_capacity    = $ds_capacity / 1024;                # to be in KB
    $ds_used        = $ds_capacity - $ds_freeSpace;
    $ds_provisioned = $datastore->summary->uncommitted;
    if ( !defined $ds_provisioned ) { $ds_provisioned = 0 }
    $ds_provisioned = $ds_provisioned / 1024;             # to be in KB
    $ds_provisioned = $ds_used + $ds_provisioned;
    my $ds_uuid;

    # check if it is usual VMFS datastore
    if ( exists $datastore->info->{"vmfs"} ) {
      $ds_uuid = $datastore->info->vmfs->uuid;
    }

    # check if it is NFS
    elsif ( exists $datastore->info->{"nas"} ) {

      # print Dumper ($datastore->info);
      if ( $datastore->info->{"nas"}->type ne "NFS" || !exists $datastore->info->{"url"} ) {
        error( "bad NFS datastore info $ds_name " . __FILE__ . ":" . __LINE__ );
        print Dumper ( $datastore->info );
      }
      else {
        my $url = $datastore->info->{"url"};

        # 'url' => 'ds:///vmfs/volumes/5e336b5d-d04e4379/'
        # print "\$datastore info- url $url\n";
        ( undef, $url ) = split( "volume", $url );    # intentionaly not volumes
        ( undef, $url, undef ) = split( "\/", $url );

        # print "\$datastore info- url $url\n";
        $ds_uuid = $url;
        $ds_type = "NFS";
      } ## end else [ if ( $datastore->info->...)]
    } ## end elsif ( exists $datastore...)
    else {
      # check if it is VSAN
      if ( exists $datastore->info->{"url"} ) {

        # 'url' => 'ds:///vmfs/volumes/vsan:5255e2a818d36446-b66c11540e875700/',
        my $url = $datastore->info->{"url"};

        # print "\$datastore info- url $url\n";
        ( undef, $url ) = split( "vsan:", $url );    # intentionaly not volumes
        ( $url, undef ) = split( "\/", $url );

        # print "\$datastore info- url $url\n";
        $ds_uuid = $url;
        $ds_type = "VSAN";
      } ## end if ( exists $datastore...)
      else {
        print Dumper ( $datastore->info );
        print STDERR Dumper( $datastore->info );
        error( "cannot find uuid for datastore (no type vmfs nor nfs nor vsan) $ds_name " . __FILE__ . ":" . __LINE__ ) && next;
      }
    } ## end else [ if ( exists $datastore...)]

    # print "\$ds_freeSpace $ds_freeSpace \$ds_capacity $ds_capacity \$ds_used $ds_used \$ds_provisioned $ds_provisioned\n" if ($apiType_top =~ "HostAgent" );

    if ( !defined $ds_uuid || ( !uuid_check_ds($ds_uuid) && !uuid_check_ds_nfs($ds_uuid) && !uuid_check_ds_vsan($ds_uuid) ) ) {
      error( "cannot find uuid for datastore $ds_name " . __FILE__ . ":" . __LINE__ ) && next;
    }

    print "retrieving perf: datastore $ds_name (F$server_count)\n";
    $fail_entity_name = $ds_name;
    $fail_entity_type = $et_Datastore;

    # existing must be renamed
    my $ds_path = "$wrkdir/$managedname/$h_name";
    if ( -f "$ds_path/$ds_name.rrt" ) {
      move( "$ds_path/$ds_name.rrt", "$ds_path/$ds_uuid.rrt" ) || error( " Cannot move $ds_path/$ds_name.rrt to $ds_path/$ds_uuid.rrt: $!" . __FILE__ . ":" . __LINE__ ) && next;

      if ( -f "$ds_path/$ds_name.rrs" ) {
        move( "$ds_path/$ds_name.rrs", "$ds_path/$ds_uuid.rrs" ) || error( " Cannot move $ds_path/$ds_name.rrs to $ds_path/$ds_uuid.rrs: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      if ( -f "$ds_path/$ds_name.last" ) {
        move( "$ds_path/$ds_name.last", "$ds_path/$ds_uuid.last" ) || error( " Cannot move $ds_path/$ds_name.last to $ds_path/$ds_uuid.last: $!" . __FILE__ . ":" . __LINE__ );
      }
      if ( -f "$ds_path/$ds_name.html" ) {
        move( "$ds_path/$ds_name.html", "$ds_path/$ds_uuid.html" ) || error( " Cannot move $ds_path/$ds_name.html to $ds_path/$ds_uuid.html: $!" . __FILE__ . ":" . __LINE__ );
      }
      print "renamed d-stor : $ds_path/$ds_name to $ds_path/$ds_uuid\n";
    } ## end if ( -f "$ds_path/$ds_name.rrt")

    # prepare_last_time($datastore,$et_Datastore,$ds_name);

    # new solution after 4.81-004
    #    prepare_last_time($datastore,$et_Datastore,$ds_name,$ds_uuid);

    # touch user datastore name <user name>.uuid
    my $datastore_name_file = "$ds_name.$ds_uuid";
    if ( !-f "$wrkdir/$managedname/$h_name/$datastore_name_file" ) {
      `rm -f "$wrkdir/$managedname/$h_name"/*."$ds_uuid"`;            # in case there are more names
      `touch "$wrkdir/$managedname/$h_name/$datastore_name_file"`;    # save user datacenter name
      LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/$datastore_name_file");
    }

    if ( !-f "$wrkdir/$managedname/$h_name/$ds_uuid.rrt" ) {          # if datastore has no data
      `touch "$wrkdir/$managedname/$h_name/$ds_uuid.rrt"`;

      # print "signal file : $wrkdir/$managedname/$h_name/$ds_uuid.rrt touched\n";
      LoadDataModuleHyperV::touch("$wrkdir/$managedname/$h_name/$ds_uuid.rrt");

      # just to say install-html.sh to create item in menu.txt
      # signal file will be overwritten by creating real rrd file in LoadDataModuleVmware.pm after data coming
    } ## end if ( !-f "$wrkdir/$managedname/$h_name/$ds_uuid.rrt")

    # create html list of active VMs
    # print "--------------------- creating html list of VMs\n";
    my @datastore_VM = ();                                                                                         # active VM list
    my $vms = Vim::get_views( mo_ref_array => $datastore->vm, properties => [ 'name', 'config.instanceUuid' ] );

    # print Dumper ($vms);
    foreach (@$vms) {
      my $vm_mo_ref_id = $_->{'mo_ref'}->value;

      # print "VM: ".$_->{'name'}."\t".$_->{'config.instanceUuid'}."\n";
      push @datastore_VM, $_->{'name'};
    } ## end foreach (@$vms)
    my $res_ret = FormatResults(@datastore_VM);
    open FH, '>:encoding(UTF-8)', "$wrkdir/$managedname/$h_name/$ds_uuid.html" or error( "can't open '$wrkdir/$managedname/$h_name/$ds_uuid.html': $!" . __FILE__ . ":" . __LINE__ ) && next;
    print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">\n";
    print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">VM&nbsp;&nbsp;&nbsp;&nbsp;</TH>
			</TR></thead><tbody>\n";
    $res_ret =~ s/center/right/g;
    print FH "$res_ret";
    print FH "</tbody></TABLE></CENTER><BR><BR>\n";
    close FH;
    my $vm_count = scalar @datastore_VM;
    print "VM list        : created $wrkdir/$managedname/$h_name/$ds_uuid.html with $vm_count VMs\n";

    prepare_last_time( $datastore, $et_Datastore, $ds_name, $ds_uuid );
  } ## end foreach my $datastore (@$datastore_list)

  foreach my $fh (@returns) {
    while (<$fh>) {
      print $_;
    }
    close($fh);
  } ## end foreach my $fh (@returns)
} ## end sub datastore_perf

sub hostsystem_perf {
  my $hyp_data_comp_act  = shift; # hash pointer
  my $hyp_data_comp_last = shift; # hash pointer

  $do_fork = "0";
  @returns = ();
  my $model  = "";
  my $serial = "";
  my $line   = "";
  my $hmcv   = "";

  # sorting non case sensitive - not for vmware
  #  @managednamelist = sort { lc($a) cmp lc($b) } @managednamelist_un;
  my $managednamelist = $managednamelist_un[0];

  my $managed_ok;
  my $managedname_exl = "";
  my @m_excl          = "";
  my $once            = 0;
  my $hmcv_num        = "";

  # print Dumper ($managednamelist);
  # print Dumper (\$managednamelist);

  #
###     cycle on $managedname_list (on servers)
  #       get data for all managed systems which are connected

  # 2018-07-20 servers organized under domain incl VMs

  foreach my $vm_host_tmp ( @{ $managednamelist || [] } ) {
    $vm_host = $vm_host_tmp;

    my $change_vm_uuid_names = 0;    # if changes must be saved

    my $part_of_domain = get_hyp_val( "Win32_ComputerSystem", "PartOfDomain", $hyp_data_comp_act );

#    if ($domain eq "False") {
#      $domain = "domain_no_domain";
#    }
#    else {
    $h_name = $$vm_host{'name'};
    my $domain = get_hyp_val( "Win32_ComputerSystem", "Domain", $hyp_data_comp_act );
    if ($domain eq $UnDeF) {
      error( "Detected domain name $domain, skipping this computer $h_name " . __FILE__ . ":" . __LINE__ ) && next;
    }
    my $serv_dom = $domain;
    $domain = "domain_$domain";
#    }
    $wrkdir = "$basedir/data";
    my $server_path = "$wrkdir/windows/$domain";
    my %vm_uuid_names      = ();
    my $vm_uuid_names_file = "$server_path/$all_hyperv_VMs/$all_vm_uuid_names";
    if ( -f "$vm_uuid_names_file" ) {
      open FH, "$vm_uuid_names_file" or error( "can't open $vm_uuid_names_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      while ( my $line = <FH> ) {
        chomp $line;
        ( my $word1, my $word2 ) = split /,/, $line, 2;
        $vm_uuid_names{$word1} = $word2;
      }
      close FH;
    } ## end if ( -f "$vm_uuid_names_file")

    # keep orig for tracking if there is a change
    my %vm_uuid_names_orig = %vm_uuid_names;

    # print Dumper ($vm_host);
    # print Dumper (\$vm_host);
    # $h_name = $$vm_host{'name'};
    print_hyperv_debug("\$h_name $h_name \$vm_host ,$vm_host, pid $$\n") if $DEBUG == 2;
    $fail_entity_name = $h_name;
    $fail_entity_type = "Host_System";

    $server_domain{$h_name} = $serv_dom;

    push @managednamelist_vmw, $h_name;

    #    my $vmx_view = Vim::find_entity_view(
    #      view_type => 'HostSystem',
    #      filter => { 'name' => "$h_name" },
    #      properties => [ 'hardware.systemInfo.uuid',
    #                      'hardware.memorySize',
    #                      'hardware.cpuInfo.hz',
    #                      'hardware.systemInfo.vendor',
    #                      'hardware.systemInfo.model',
    #                      'summary.hardware.cpuModel',
    #                      'summary.hardware.numCpuCores',
    #                      'summary.hardware.numCpuPkgs',
    #                      'systemResources.config.cpuAllocation.reservation',
    #                      'systemResources.config.cpuAllocation.shares.shares','parent',
    #                      'systemResources.config.memoryAllocation.reservation',
    #                      'summary.config.product']
    #    );

    my $o_uuid = $$vm_host{'UUID'};
    $host_memorySize = get_hyp_val( 'Win32_ComputerSystem', 'TotalPhysicalMemory', $hyp_data_comp_act );
    $host_memorySize = 0 if !isdigit($host_memorySize);

    $vm_TotalPhysicalMemory[0] = $host_memorySize;

    $vm_Frequency_PerfTime[0] = $UnDeF;
    if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor'} ) {
      $vm_Frequency_PerfTime[0] = get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor', 'Frequency_PerfTime', $hyp_data_comp_act, '_Total' );
    }
    if ($vm_Frequency_PerfTime[0] eq $UnDeF) {
      if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_PerfOS_Processor'}) {
        $vm_Frequency_PerfTime[0] = get_hyp_val( 'Win32_PerfRawData_PerfOS_Processor', 'Frequency_PerfTime', $hyp_data_comp_act, '_Total' );
      }
      else {
        # not sufficient data
        print_hyperv_debug("1471 no data for Win32_PerfRawData_PerfOS_Processor\n") if $DEBUG == 2;
        return;
      }
    }
    $vm_Frequency_PerfTime[0] = 1 if !isdigit( $vm_Frequency_PerfTime[0] );

    # print "\$vm_Frequency_PerfTime[0] $vm_Frequency_PerfTime[0]\n";
    #my $host_parent = $vmx_view->get_property('parent');
    my $host_parent = "";

    #my $host_memAlloc = $vmx_view->get_property('systemResources.config.memoryAllocation.reservation');

    #my $hw_vendor      = $vmx_view->get_property('hardware.systemInfo.vendor');
    #my $hw_model       = $vmx_view->get_property('hardware.systemInfo.model');
    #my $hw_numCpuPkgs  = $vmx_view->get_property('summary.hardware.numCpuPkgs');
    #my $hw_cpuModel    = $vmx_view->get_property('summary.hardware.cpuModel');
    #my $hw_numCpuCores = $vmx_view->get_property('summary.hardware.numCpuCores');
    #my $sw_apiType     = $vmx_view->get_property('summary.config.product.apiType');
    #my $sw_apiVersion  = $vmx_view->get_property('summary.config.product.apiVersion');
    #my $sw_build       = $vmx_view->get_property('summary.config.product.build');
    #my $sw_fullName    = $vmx_view->get_property('summary.config.product.fullName');
    #my $sw_localeBuild = $vmx_view->get_property('summary.config.product.localeBuild');
    #my $sw_localeVersion = $vmx_view->get_property('summary.config.product.localeVersion');

    # print "\$hw_vendor $hw_vendor \$hw_model $hw_model \$hw_cpuModel $hw_cpuModel \$hw_numCpuCores $hw_numCpuCores \$hw_numCpuPkgs $hw_numCpuPkgs \n";
    # print "\$o_uuid $o_uuid \$host_hz $host_hz\n";

    $line = $h_name . "," . $h_name . "," . $o_uuid;

    chomp($line);

    if ( $line =~ m/Error:/ || $line =~ m/Permission denied/ || $line =~ m/undefined/ ) {
      error( "problem connecting to $vm_host : $line " . __FILE__ . ":" . __LINE__ );
      next;
    }

    if ( $line !~ ".*,.*,.*" ) {

      # it must look like : PWR6A-9117-MMA-SN103A4B0,9117-MMA,103A4B0, --> exclude banners
      next;
    }

    if ( $line =~ /No results were found/ ) {
      print "$host does not contain any managed system\n" if $DEBUG;
      return 0;
    }
    ( $managedname, $model, $serial ) = split( /,/, $line );

    # if ( is_IP($managedname) == 1 ) {
    #      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    # }

    #    print "managed system : $host:$managedname (serial : $serial) ,$sw_apiType ,$sw_apiVersion ,$sw_build ,$sw_fullName ,$sw_localeBuild ,$sw_localeVersion\n" if $DEBUG ;
    print "managed system : $host:$managedname (serial : $serial) \n" if $DEBUG;

    #  rename_server($host,$managedname,$model,$serial);  # this is for IBM servers
    #rename_server( $host, $managedname, $serial );    # this is for hyper_v servers, serial is uuid

    # create sym link serial for recognizing of renamed managed systems
    # it must be here due to skipping some server (exclude, not running util collection) and saving cfg
    if ( !-d "$wrkdir" ) {
      print "mkdir          : $host:$managedname $wrkdir\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$host:$managedname $wrkdir");
      mkdir( "$wrkdir", 0755 ) || error( " Cannot mkdir $wrkdir: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    #
    ### starting here : change of $wrkdir (only 1st time)
    #
    if ($wrkdir !~ /windows$/) {
      $wrkdir = "$wrkdir/windows";
    }
    $wrkdir_windows = $wrkdir;

    if ( !-d "$wrkdir" ) {
      print "mkdir          : $host:$managedname $wrkdir\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$host:$managedname $wrkdir");
      mkdir( "$wrkdir", 0755 ) || error( " Cannot mkdir $wrkdir: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    $wrkdir = "$wrkdir/$domain";

    if ( !-d "$wrkdir" ) {
      print "mkdir          : $host:$managedname $wrkdir\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$host:$managedname $wrkdir");
      mkdir( "$wrkdir", 0755 ) || error( " Cannot mkdir $wrkdir: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    # new system all VMs in domain dir
    # it also holds file uuid->name of VMs

    if ( !-d "$wrkdir/$all_hyperv_VMs" ) {
      print "mkdir          : $host: $wrkdir/$all_hyperv_VMs\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$wrkdir/$all_hyperv_VMs");
      mkdir( "$wrkdir/$all_hyperv_VMs", 0755 ) || error( " Cannot mkdir $wrkdir/$all_hyperv_VMs: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

    if ( !-d "$wrkdir/$managedname" ) {
      print "mkdir          : $host:$managedname $wrkdir/$managedname\n" if $DEBUG;
      LoadDataModuleHyperV::touch("$wrkdir/$managedname");
      mkdir( "$wrkdir/$managedname", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
    }

#    if ( !-l "$wrkdir/$serial" ) {    # uuid only
#      print "ln -s          : $host:$managedname $wrkdir/$managedname $wrkdir/$serial \n" if $DEBUG;
#      LoadDataModuleHyperV::touch("$wrkdir/$serial");
#      symlink( "$wrkdir/$managedname", "$wrkdir/$serial" ) || error( " Cannot ln -s $wrkdir/$managedname $wrkdir/$serial: $!" . __FILE__ . ":" . __LINE__ ) && next;
#    }

 # tohle asi nhradime jen touchnutim  jmena hosta, to je jen smluvene jmeno na posilani dat z windows
 #
    #if ( !-d "$wrkdir/$managedname/$host" ) {
    #  print "mkdir          : $host:$managedname $wrkdir/$managedname/$host\n" if $DEBUG;
    #  LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host");
    #  mkdir( "$wrkdir/$managedname/$host", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname/$host: $!" . __FILE__ . ":" . __LINE__ ) && next;
    #}

    #    unlink "$wrkdir/$managedname/$host/im_in_cluster"; # if left from last load

    # server must know its alias if non vCenter

#uvidime co bude dale
#    if ( $apiType_top !~ "VirtualCenter" ) {
#      my $my_alias_file_name = "$wrkdir/$managedname/$host/hyperv_alias_name";
#      my $alias_name         = "";
#      if ( -f $my_alias_file_name ) {
#        open FH, "$my_alias_file_name" or error( "can't open $my_alias_file_name: $!" . __FILE__ . ":" . __LINE__ );
#        $alias_name = <FH>;
#        chomp $alias_name;
#        close FH;
#      } ## end if ( -f $my_alias_file_name)
#      if ( $alias_name eq "" || $alias_name ne "$host|$alias" ) {
#        open FH, ">$my_alias_file_name" or error( "can't open $my_alias_file_name: $!" . __FILE__ . ":" . __LINE__ );
#        print FH "$host|$alias\n";    # save host name and alias
#        close FH;
#        LoadDataModuleHyperV::touch("$my_alias_file_name");
#      } ## end if ( $alias_name eq ""...)
#
#      # this is very improbable but ...
#      if ( -f "$wrkdir/$managedname/$host/my_hyperv_name" ) {
#        unlink "$wrkdir/$managedname/$host/my_hyperv_name";
#        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host/my_hyperv_name");
#      }
#      if ( -f "$wrkdir/$managedname/$host/my_cluster_name" ) {
#        unlink "$wrkdir/$managedname/$host/my_cluster_name";
#        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host/my_cluster_name");
#      }
#    } ## end if ( $apiType_top !~ "VirtualCenter")
#    else {
#      # server must know its vcenter name
#      my $my_hyperv_file_name = "$wrkdir/$managedname/$host/my_hyperv_name";
#      my $hyperv_name         = "";
#      if ( -f $my_hyperv_file_name ) {
#        open FH, "$my_hyperv_file_name" or error( "can't open $my_hyperv_file_name: $!" . __FILE__ . ":" . __LINE__ );
#        $hyperv_name = <FH>;
#        chomp $hyperv_name;
#        close FH;
#      } ## end if ( -f $my_hyperv_file_name)
#      if ( $hyperv_name eq "" || $hyperv_name ne "$host|$alias" ) {
#        open FH, ">$my_hyperv_file_name" or error( "can't open $my_hyperv_file_name: $!" . __FILE__ . ":" . __LINE__ );
#        print FH "$host|$alias\n";    # save host name and alias
#        close FH;
#        LoadDataModuleHyperV::touch("$my_hyperv_file_name");
#      } ## end if ( $hyperv_name eq ""...)
#
#      # this is very improbable but ...
#      if ( -f "$wrkdir/$managedname/$host/hyperv_alias_name" ) {
#        unlink "$wrkdir/$managedname/$host/hyperv_alias_name";
#        LoadDataModuleHyperV::touch("$wrkdir/$managedname/$host/hyperv_alias_name");
#      }
#
#      # prepare for situation when ESXi is removed from cluster
#      if ( -f "$wrkdir/$managedname/$host/my_cluster_name" ) {
#        `touch "$wrkdir/$managedname/$host/im_in_cluster"`;
#      }
#    } ## end else [ if ( $apiType_top !~ "VirtualCenter")]

    $managed_ok = 1;
    @pool_list  = "";    # clean pool_list for each managed name
    if ( $managed_systems_exclude ne '' ) {
      @m_excl = split( /:/, $managed_systems_exclude );
      foreach $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok = 0;
        }
      } ## end foreach $managedname_exl (@m_excl)
    } ## end if ( $managed_systems_exclude...)
    if ( $managed_ok == 0 ) {
      print "managed system : $host:$managedname is excluded in load.sh, continuing with the others ...\n" if $DEBUG;
      save_cfg_data( $managedname, localtime(), $upgrade );    # it is necessary to have all server in cfg page
      next;
    }
    my $vm_names = "";
    if (exists $$hyp_data_comp_act->{'Msvm_ComputerSystem'}) {
      $vm_names = get_hyp_val( 'Msvm_ComputerSystem', 'ElementName', $hyp_data_comp_act, 'Virtual Machine' );
      print_hyperv_debug("---------- \$vm_names $vm_names\n") if $DEBUG == 2;
    }

    @vm_view = split( ",", $vm_names );

    # get uuid, name ond other config params of VM for lpar_trans.txt
    #    my $vm_view = Vim::find_entity_views(view_type=>'VirtualMachine',
    #       properties=>['name',
    #                    'summary.config.instanceUuid',
    #                    'summary.config.guestFullName',
    #                    'summary.storage.committed',
    #                    'summary.storage.uncommitted',
    #                    'summary.config.numCpu',
    #                    'runtime.powerState',
    #                    'config.cpuAllocation.reservation',
    #                    'config.cpuAllocation.shares.shares',
    #                    'config.cpuAllocation.shares.level',
    #                    'guest.toolsRunningStatus'
    #                    ],begin_entity=>$vm_host);

    my @lpar_trans        = ();    # original array
    my @lpar_trans_new    = ();    # if there is new VM
    my @lpar_trans_renew  = ();    # if there is change in orig arr
    my $lpar_trans_change = 0;

    # lpar_trans.txt keeps names of all VMS that anytime has/had been registered under this server
    my $lpar_trans_name = "$wrkdir/$managedname/lpar_trans.txt";
    if ( -f "$lpar_trans_name" ) {
      open FH, "$lpar_trans_name" or error( "can't open $lpar_trans_name: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      @lpar_trans = <FH>;
      close FH;
    }

    # new system of tracking VM presence in hostsystem
    # for every hostsystem ( resourcepool too)
    # - open VM hosting file
    # - track every VM - active and also non-active
    # - save Vm hosting file
    # during cycle hold hosting info in an array

    my @hosting_arr = ();
    VM_hosting_read( \@hosting_arr, "$wrkdir/$managedname/VM_hosting.vmh" );

    %vm_name_uuid      = ();
    %vm_uuid_mac       = ();
    %vm_name_powerstat = ();
    %vm_uuid_mem       = ();
    %vm_name_vcpu      = ();
    my @cpu_cfg  = ();
    my @disk_cfg = ();
    my @vhd_list = ();

    foreach my $each_vm (@vm_view) {

      next if $each_vm eq $UnDeF;

      #  new lpar will be added to lpar_trans.txt
      my $test_atom = "ITEM=ElementName="."\"".$each_vm."\"";
      my $each_vm_uuid = get_hyp_val( 'Msvm_ComputerSystem', 'Name', $hyp_data_comp_act, $test_atom );
      # print "1877 \$each_vm_uuid $each_vm_uuid \$each_vm $each_vm\n";
      if (index($each_vm_uuid, ",") != -1) {
        ($each_vm_uuid, undef) = split (",",$each_vm_uuid); # in case 2 or more uuids returned
      }
      my $each_vm_name = $each_vm;
      my $each_vm_url  = $each_vm_name;

      if ( !defined $each_vm_uuid || $each_vm_uuid eq "" || !uuid_check($each_vm_uuid) ) {
        error( "Bad VM uuid in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      if ( !defined $each_vm_name || $each_vm_name eq "" ) {
        error( "Bad VM name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }

      # for new system, do not test possible uuid collision
      # prepare url name for install-html.sh
      $each_vm_url =~ s/([^A-Za-z0-9\+-_])/sprintf("%%%02X", ord($1))/seg;
      my $new_item = "$each_vm_name" . "," . "$each_vm_url";
      if ( !defined $vm_uuid_names{$each_vm_uuid} || $vm_uuid_names{$each_vm_uuid} ne $new_item ) {
        $vm_uuid_names{$each_vm_uuid} = $new_item;
        $change_vm_uuid_names++;
      }

      VM_hosting_update( \@hosting_arr, $each_vm_uuid, $command_unix );

      #  print "\@hosting_arr @hosting_arr\n";

      my $line_test = $each_vm_uuid;    # do not know if with or without name

      if ( ( defined $line_test ) && ( !uuid_check($line_test) ) ) {
        if ( defined $each_vm_name ) {
          error( "Bad VM uuid for VM:$each_vm_name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
        else {
          error( "Bad VM uuid, undef Name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
      } ## end if ( ( defined $line_test...))
      if ( !defined $line_test ) {
        if ( defined $each_vm_name ) {
          error( "Undefined VM uuid for VM:$each_vm_name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
        else {
          error( "Undefined VM uuid nor Name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
        }
      } ## end if ( !defined $line_test)
      if ( !defined $each_vm_name ) {
        error( "Undefined VM name in $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && next;
      }
      my $vm_uuid  = $line_test;
      my $line_upd = $line_test . "," . $each_vm_name;

      # there is no moref in hyperv
      my $vm_mo_ref_id = 'moref_' . $each_vm_name;
      if ( !defined $vm_mo_ref_id ) {
        error( "VM change ID   : not defined mo_ref for VM $each_vm_name " . __FILE__ . ":" . __LINE__ );
        next;
      }
      my $match = grep {/$line_test/} @lpar_trans;
      my ($arr_index) = grep { $lpar_trans[$_] =~ $line_test } 0 .. $#lpar_trans;
      if ( $match < 1 ) {
        push @lpar_trans_new, "$line_upd" . "," . $vm_mo_ref_id . "\n";
        print "VM change ID   : new registration in lpar_trans for $line_upd,$vm_mo_ref_id\n";
      }
      else {
        # print "VM test ID     : VM found index $arr_index in arr lpar_trans\n";
        if ( !defined $lpar_trans[$arr_index] ) {
          error( "VM change ID   : not defined $arr_index in arr lpar_trans, if persist, contact lpar2rrd support " . __FILE__ . ":" . __LINE__ );
        }
        else {
          my $lpar_trans_item = $lpar_trans[$arr_index];
          chomp $lpar_trans_item;
          ( my $uuid, undef, my $old_mo_ref_id ) = split( ",", $lpar_trans_item );
          if ( !defined $old_mo_ref_id ) {
            $old_mo_ref_id = "undefined";
          }
          if ( ( $old_mo_ref_id eq "undefined" ) || ( $old_mo_ref_id ne $vm_mo_ref_id ) ) {
            print "VM change ID   : VM $lpar_trans[$arr_index] changed (or had not set) mo_ref_Id to $vm_mo_ref_id\n";

            # remove all VM-uuid items
            @lpar_trans_renew = grep { $_ !~ $uuid } @lpar_trans;
            @lpar_trans = @lpar_trans_renew;
            push @lpar_trans, "$line_upd" . "," . $vm_mo_ref_id . "\n";
            $lpar_trans_change++;
          } ## end if ( ( $old_mo_ref_id ...))
        } ## end else [ if ( !defined $lpar_trans...)]
      } ## end else [ if ( $match < 1 ) ]

      $vm_name_uuid{"$each_vm_name"} = "$line_test";

      my $powerstate = get_hyp_val( 'Msvm_ComputerSystem', 'EnabledState', $hyp_data_comp_act, $test_atom );
      if ( exists $enabledstate{$powerstate} ) {
        $powerstate = $enabledstate{$powerstate};    # from system table
      }
      else {
        $powerstate = "UNKNOWN";
      }
      $vm_name_powerstat{$each_vm} = $powerstate;

      # CLASS: Msvm_ProcessorSettingData
      # InstanceID|Limit|Reservation|VirtualQuantity|Weight
      # Microsoft:09672167-2837-4FFE-97D7-8163745E8AB3\b637f346-6a0e-4dec-af52-bd70cb80a21d\0|100000|0|1|100

      my $max_limit = "U";
      $max_limit = get_hyp_val( 'Msvm_ProcessorSettingData', 'Limit', $hyp_data_comp_act, $each_vm_uuid );
      if ($max_limit ne $UnDeF) {
        if (index( $max_limit, ",") != -1 ) {
          error( "more data in \$max_limit $max_limit " . __FILE__ . ":" . __LINE__ );
          ($max_limit, undef) = split(",",$max_limit);
        }
        $max_limit /= 1000;
      }

      my $numCpu = get_hyp_val( 'Msvm_ProcessorSettingData', 'VirtualQuantity', $hyp_data_comp_act, $each_vm_uuid );
      $vm_name_vcpu{$each_vm_name} = $numCpu;
      print_hyperv_debug("1799 \$each_vm_uuid $each_vm_uuid \$numCpu $numCpu\n") if $DEBUG == 2;

      my $cpuAllocation_reservation = get_hyp_val( 'Msvm_ProcessorSettingData', 'Reservation', $hyp_data_comp_act, $each_vm_uuid );
      $cpuAllocation_reservation /= 1000 if $cpuAllocation_reservation ne $UnDeF;

      my $cpu_weight = get_hyp_val( 'Msvm_ProcessorSettingData', 'Weight', $hyp_data_comp_act, $each_vm_uuid );

      # looking for MAC addr -> PermanentAddress
      # CLASS: Msvm_SyntheticEthernetPort
      # CreationClassName|DeviceID|Name|PermanentAddress|SystemCreationClassName|SystemName
      # Msvm_SyntheticEthernetPort|Microsoft:DD639889-6BC7-451E-AF48-FBAD2418F7E6|Ethernet Port|00155D01BB01|Msvm_ComputerSystem|3138F59F-11AD-46FB-951C-9C147C98896C

      $vm_uuid_mac{$each_vm_uuid} = $UnDeF;

      if (exists $$hyp_data_comp_act->{'Msvm_SyntheticEthernetPort'}) {
        $vm_uuid_mac{$each_vm_uuid} = get_hyp_val( 'Msvm_SyntheticEthernetPort', 'PermanentAddress', $hyp_data_comp_act, $each_vm_uuid );
      }

      my $dynamic_mem = get_hyp_val( 'Msvm_MemorySettingData', 'DynamicMemoryEnabled', $hyp_data_comp_act, $each_vm_uuid );

      my $heartbeat   = $UnDeF;
      if (exists $$hyp_data_comp_act->{'Msvm_HeartbeatComponent'}) {
        $heartbeat   = get_hyp_val( 'Msvm_HeartbeatComponent', 'StatusDescriptions', $hyp_data_comp_act, $each_vm_uuid );
      }
      my $timesync    = $UnDeF;
      if (exists $$hyp_data_comp_act->{'Msvm_TimeSyncComponent'}) {
        $timesync    = get_hyp_val( 'Msvm_TimeSyncComponent', 'StatusDescriptions', $hyp_data_comp_act, $each_vm_uuid );
      }
      my $vss_status  = $UnDeF;
      if (exists $$hyp_data_comp_act->{'Msvm_VssComponent'}) {
        $vss_status  = get_hyp_val( 'Msvm_VssComponent', 'StatusDescriptions', $hyp_data_comp_act, $each_vm_uuid );
      }
      $heartbeat  =~ s/,/ /g;
      $timesync   =~ s/,/ /g;
      $vss_status =~ s/,/ /g;

      my $guestFullName      = "";
      my $toolsRunningStatus = "";

      my $line = $each_vm_name .",". $numCpu .",". $cpuAllocation_reservation .",". $max_limit .",". $cpu_weight .",". $powerstate .",". $heartbeat .",". $vm_uuid_mac{$each_vm_uuid} .",". $dynamic_mem .",". $timesync .",". $vss_status . "\n";
      push @cpu_cfg, $line;

      # CLASS: Msvm_Memory
      # "SystemName","BlockSize","NumberOfBlocks","Name"
      # "71ACA269-B7E0-42F7-BCB6-558E042B0CCB","1048576","8192","71ACA269-B7E0-42F7-BCB6-558E042B0CCB"
      # "71ACA269-B7E0-42F7-BCB6-558E042B0CCB","1048576","4096","71ACA269-B7E0-42F7-BCB6-558E042B0CCB"
      # "71ACA269-B7E0-42F7-BCB6-558E042B0CCB","1048576","4096","71ACA269-B7E0-42F7-BCB6-558E042B0CCB"
      # sometimes can be more than 1 line , in this case choose the biggest number

      my $BlockSize      = get_hyp_val( 'Msvm_Memory', 'BlockSize',      $hyp_data_comp_act, $each_vm_uuid );
      my $NumberOfBlocks = get_hyp_val( 'Msvm_Memory', 'NumberOfBlocks', $hyp_data_comp_act, $each_vm_uuid );
      if (index ($BlockSize, ",") ne -1) {
        my $max;
        my $index = 0;
        my $max_index = 0;
        my @array = split(",",$NumberOfBlocks);
        for (@array) {
          if (!$max || $_ > $max) {
            $max = $_;
            $max_index = $index;
          }
          $index++;
        };
        # print "max: $max index:$max_index $array[$max_index]\n";
        $NumberOfBlocks = $max;
        $BlockSize      = (split(",",$BlockSize))[$max_index ];
        print "1907 \$NumberOfBlocks $NumberOfBlocks \$BlockSize $BlockSize \$each_vm_uuid $each_vm_uuid\n" if $DEBUG == 2;
      }
      my $storage_com    = 0;
      if ( $BlockSize ne $UnDeF && $NumberOfBlocks ne $UnDeF ) {
        $storage_com = $BlockSize * $NumberOfBlocks / 1024 / 1024;
      }
      $vm_uuid_mem{$each_vm_uuid} = $storage_com;
      my $storage_total = $storage_com;
      $line = "$each_vm_name,$storage_total,$storage_com\n";
      push @disk_cfg, $line;

      # prepare hash for easy VM pick up (resourcepool)
      $vm_id_path{"$vm_mo_ref_id"} = "$wrkdir/$h_name/$vm_uuid.rrm";
      # print "filling vm_id_path{\"$vm_mo_ref_id\"} = \"$wrkdir/$h_name/$host/$vm_uuid.rrm\n";

      my $vhd_path = get_hyp_val( 'Msvm_StorageAllocationSettingData', 'HostResource', $hyp_data_comp_act, $each_vm_uuid );
      my @vhd_path_array = split( /\)/,$vhd_path);

      #                                   HostResource                                    InstanceID
      # raw line example: Hard Disk Image|(E:\Export\XoruX\Virtual Hard Disks\XoruX.vhdx)|Microsoft:143E4B15-1E45-4D7B-B352-ECE54359A778\83F8638B-8DCA-4152-9EDA-2CA8B33039B4\0\0\L
      my $instance_vhd = get_hyp_val( 'Msvm_StorageAllocationSettingData', 'InstanceId', $hyp_data_comp_act, $each_vm_uuid );
      print_hyperv_debug("1787 hyp2rrd.pl \$instance_vhd $instance_vhd\n") if $DEBUG == 2;
      my @instance_vhd_array = split /,/,$instance_vhd;
      # can be more than 1 e.g.
      # (C:\Users\netadmin\Desktop\XP\Virtual Hard Disks\WinXP.vhdx)   (C:\Windows\system32\vmguest.iso)

      my $vhd_inx = 0;

      foreach (@vhd_path_array) {
        my $vhd_path_line = $_;
        # $vhd_path_line .= ")";
        $vhd_path_line =~ s/^\s+//;
        $vhd_path_line =~ s/^,//; # second and other atoms have it

        # find controller type
        my $class = 'Msvm_IDEController';
        my $i = 0;
        my $controller = "";
        my $location   = " ";
        print_hyperv_debug("1804 go for IDEController\n") if $DEBUG == 2;
        # Microsoft:83F8638B-8DCA-4152-9EDA-2CA8B33039B4\0,IDE Controller 0,5,37,3138F59F-11AD-46FB-951C-9C147C98896C,1525161659
        while ( defined $$hyp_data_comp_act->{$class}[$i] ) {
          my $hyp_line = $$hyp_data_comp_act->{$class}[$i];
          $hyp_line =~ s/\"//g;
          $i++;
          next if $hyp_line !~ "$each_vm_uuid";
          print_hyperv_debug("1809 hyp2rrd.pl $hyp_line\n") if $DEBUG == 2;
          my @item_arr = split( /,/, $hyp_line );
          my $item_ide = $item_arr[0];
          $item_ide = (split(/:/,$item_ide))[1];
          print_hyperv_debug("1817 ,,,,,,,,,,,,,,,,, \$instance_vhd_array[$vhd_inx] $instance_vhd_array[$vhd_inx] \$item_ide $item_ide\n") if $DEBUG == 2;
          if ($instance_vhd_array[$vhd_inx] =~ /\Q$item_ide\E/) {
            $controller = $item_arr[1];
            $location = (split /\\/, $instance_vhd_array[$vhd_inx])[-2];
          }
        }

        # if not IDE controller try SCSI
        if ($controller eq "") {
          my $class = 'Msvm_SCSIProtocolController';
          my $i = 0;
          print_hyperv_debug("1829 go for SCSI Controller\n") if $DEBUG == 2;
          # "Microsoft:C3A13785-9C34-449F-AFDF-3F5174FFB28B\0","SCSI Controller","5","3138F59F-11AD-46FB-951C-9C147C98896C,1525161659"
          while ( defined $$hyp_data_comp_act->{$class}[$i] ) {
            my $hyp_line = $$hyp_data_comp_act->{$class}[$i];
            $hyp_line =~ s/\"//g;
            $i++;
            next if $hyp_line !~ "$each_vm_uuid";
            print_hyperv_debug("1835 hyp2rrd.pl $hyp_line\n") if $DEBUG == 2;
            my @item_arr = split( /,/, $hyp_line );
            my $item_ide = $item_arr[0];
            $item_ide = (split(/:/,$item_ide))[1];
            print_hyperv_debug("1839 ,,,,,,,,,,,,,,,,, \$instance_vhd_array[$vhd_inx] $instance_vhd_array[$vhd_inx] \$item_ide $item_ide\n") if $DEBUG == 2;
            if ($instance_vhd_array[$vhd_inx] =~ /\Q$item_ide\E/) {
              $controller = $item_arr[1];
              $location = (split /\\/, $instance_vhd_array[$vhd_inx])[-2];
            }
          }
        }

        print_hyperv_debug("1846 hyp2rrd.pl $instance_vhd_array[$vhd_inx]\n") if $DEBUG == 2;
        $line = $each_vm_name .",". $controller .",". $location .",". $vhd_path_line . "\n";
        push @vhd_list, $line;
        $vhd_inx++;
      }

    } ## end foreach my $each_vm (@vm_view)

    # for new system, if change in VM then save
    if ($change_vm_uuid_names) {
      open FH, ">$vm_uuid_names_file" or error( "can't open $vm_uuid_names_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );
      my ( $k, $v );

      # Write key/value pairs from %vm_uuid_names to file, joined by ':'
      while ( ( $k, $v ) = each %vm_uuid_names ) {
        print FH "$k" . "," . "$v\n";
      }
      close FH;
      print "all_vm_uuid    : changed content has been written\n";
    } ## end if ($change_vm_uuid_names)

    VM_hosting_write( \@hosting_arr, "$wrkdir/$managedname/VM_hosting.vmh", $command_unix );

    # there can be new VM, or VM can possibly change ID

    if ($lpar_trans_change) {    # changed Id > rewrite file

      open FH, ">$lpar_trans_name" or error( "can't open $lpar_trans_name: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      foreach my $new_line (@lpar_trans) {
        chomp($new_line);
        print FH $new_line . "\n";
      }
      close FH;
    } ## end if ($lpar_trans_change)

    if ( @lpar_trans_new > 0 ) {    # just add new VM
      open FH, ">>$lpar_trans_name" or error( "can't open $lpar_trans_name: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      foreach my $new_line (@lpar_trans_new) {
        chomp($new_line);
        print FH $new_line . "\n";
      }
      close FH;
    } ## end if ( @lpar_trans_new >...)

    my $cpu_cores = $UnDeF;
    if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor'}) {
      $cpu_cores = get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor', 'Name', $hyp_data_comp_act );    # example Hv LP 3,Hv LP 2,Hv LP 1,Hv LP 0,_Total
    }
    my $c_cores   = $cpu_cores =~ tr/,//;
    # print  "1959 \$cpu_cores $cpu_cores\n";

    # in case of standalone windows
    if ($cpu_cores eq $UnDeF) {
      $c_cores = get_hyp_val( 'Win32_ComputerSystem', 'NumberOfProcessors', $hyp_data_comp_act );
      #  print "2766 \$vm_vCPU[0] $vm_vCPU[0] standalone windows\n";
    }

    my $res_ret   = FormatResults(@cpu_cfg);
    # print STDERR "1714 hyp2rrd.pl \$res_ret $res_ret \@cpu_cfg @cpu_cfg\n";

    # print VMs info if there are some VMs
    my $cpu_file = "$wrkdir/$managedname/cpu.html";
#    if (scalar @cpu_cfg) {
      open FH, '>:encoding(UTF-8)', "$cpu_file" or error( "can't open $cpu_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      #            <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</TH>
      #            <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;toolsStatus&nbsp;&nbsp;&nbsp;</TH>

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\"><!cores:$c_cores>\n";    # max CPU cores hidden here
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">VM&nbsp;&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;vCPU&nbsp;&nbsp;</TH>
			  <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Reservation %&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Max Limit %&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Weight (0-10000)&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;EnabledState&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;HeartBeat&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;MAC addr&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;DynamicMem&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Time Sync&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Vss status&nbsp;&nbsp;</TH>
  			</TR></thead><tbody>\n";
    if (scalar @cpu_cfg) {
      print FH "$res_ret";
    }
      print FH "</tbody></TABLE></CENTER><BR><BR>\n";
      close FH;
#    }
#    else {
#      unlink $cpu_file if (-f $cpu_file);
#    }

     # CLASS: Win32_NetworkAdapterConfiguration where IPEnabled='True'
     # "Description","DHCPEnabled","DHCPServer","DNSHostName","IPAddress","MACAddress","ServiceName","SettingID"
     # "Microsoft Failover Cluster Virtual Adapter","False",,"hvnode01","169.254.2.246 fe80::fd1f:7e9e:5894:87e","02:66:3A:56:2D:D5","Netft","{06DF9D69-2654-44C3-B5D3-DE25FD09D96F}"
     # "Hyper-V Virtual Ethernet Adapter","True","10.22.88.10","hvnode01","10.22.88.102 10.22.88.100 fe80::70df:4be7:df12:2595","00:50:56:9C:79:E0","VMSMP","{510E9C58-FEDD-436C-A3ED-001433970ABC}"
    if (exists $$hyp_data_comp_act->{"Win32_NetworkAdapterConfiguration where IPEnabled='True'"}) {
      # print "2059 Win32_NetworkAdapterConfiguration\n";
      # problem: if there are more adapters, test of MACAddress, how many of them
      my $mac_adrs  = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'MACAddress', $hyp_data_comp_act );
      my @mac_adrs_list = split(",",$mac_adrs);
      @cpu_cfg = ();
      foreach my $MACAddress (@mac_adrs_list) {
        my $adapter     = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'Description', $hyp_data_comp_act, $MACAddress );
        my $DHCPServer  = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'DHCPServer', $hyp_data_comp_act, $MACAddress );
        my $DNSHostName = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'DNSHostName', $hyp_data_comp_act, $MACAddress );
        my $IPAddress   = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'IPAddress', $hyp_data_comp_act, $MACAddress );
        # my $MACAddress  = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'MACAddress', $hyp_data_comp_act );
        print_hyperv_debug("2065 \$MACAddress $MACAddress \$adapter $adapter\n") if $DEBUG == 2;
        my $ServiceName = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'ServiceName', $hyp_data_comp_act, $MACAddress );
        my $SettingID   = get_hyp_val( "Win32_NetworkAdapterConfiguration where IPEnabled='True'", 'SettingID', $hyp_data_comp_act, $MACAddress );

        my $line = $adapter .",". $DHCPServer .",". $DNSHostName .",". $IPAddress .",". $MACAddress .",". $ServiceName .",". $SettingID . "\n";
        push @cpu_cfg,$line;
      }
      $res_ret   = FormatResults(@cpu_cfg);
      my $file_name = "$wrkdir/$managedname/NetworkAdapterConfiguration.html";
      open FH, '>:encoding(UTF-8)', "$file_name" or error( "can't open $file_name : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">Description&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;DHCPServer&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;DNSHostName&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;IPAddress&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;MACAddress&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;ServiceName&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;SettingID&nbsp;&nbsp;</TH>
  			</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER><BR><BR>\n";
      close FH;
    }

    # licensing
    # CLASS: SoftwareLicensingProduct where ApplicationID='55c92734-d682-4d71-983e-d6ec3f16059f' AND PartialProductKey IS NOT NULL
    # "Description","LicenseStatus","PartialProductKey"
    # "Windows(R) Operating System, RETAIL channel","1","8HVX7"

    my $licensestatus = "";
    if (exists $$hyp_data_comp_act->{"SoftwareLicensingProduct where ApplicationID='55c92734-d682-4d71-983e-d6ec3f16059f' AND PartialProductKey IS NOT NULL"}) {
      $licensestatus = get_hyp_val( "SoftwareLicensingProduct where ApplicationID='55c92734-d682-4d71-983e-d6ec3f16059f' AND PartialProductKey IS NOT NULL", 'LicenseStatus', $hyp_data_comp_act );
    }
    my %lic_statuses = (
    "0"  => "Unlicensed",
    "1"  => "Licensed",
    "2"  => "Out-Of-Box Grace Period",
    "3"  => "Out-Of-Tolerance Grace Period",
    "4"  => "Non-Genuine Grace Period",
    "5"  => "Notification",
    "6"  => "Extended Grace"
    );
    if (exists $lic_statuses{$licensestatus}) {
      $licensestatus = $lic_statuses{$licensestatus};
    }
    else {
      $licensestatus = "unknown";
    }

    ### not for standalone workstation
    if (exists $$hyp_data_comp_act->{'Msvm_VirtualSystemManagementServiceSettingData'}) {
      my $InstanceID = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'InstanceID', $hyp_data_comp_act );
      my $CurrentWWNN = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'CurrentWWNNAddress', $hyp_data_comp_act );
      my $MinMacAddress = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'MinimumMacAddress', $hyp_data_comp_act );
      my $MaxMacAddress = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'MaximumMacAddress', $hyp_data_comp_act );
      my $MinWWPNAddress = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'MinimumWWPNAddress', $hyp_data_comp_act );
      my $MaxWWPNAddress = get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'MaximumWWPNAddress', $hyp_data_comp_act );
      my $NumaSpanningEnabled= get_hyp_val( 'Msvm_VirtualSystemManagementServiceSettingData', 'NumaSpanningEnabled', $hyp_data_comp_act );
      $InstanceID =~ s/Microsoft://;

      my $cs_caption = get_hyp_val( 'Win32_OperatingSystem', 'Caption', $hyp_data_comp_act );
      my $cs_version = get_hyp_val( 'Win32_OperatingSystem', 'Version', $hyp_data_comp_act );

      my $line_server = $InstanceID .",". $cs_caption .",". $cs_version .",". $licensestatus .",". $c_cores .",". $CurrentWWNN .",". $MinMacAddress .",". $MaxMacAddress .",". $MinWWPNAddress .",". $MaxWWPNAddress .",". $NumaSpanningEnabled . "\n";
      @cpu_cfg = ();
      push @cpu_cfg,$line_server;
      $res_ret   = FormatResults(@cpu_cfg);

      open FH, '>:encoding(UTF-8)', "$wrkdir/$managedname/server.html" or error( "can't open '$wrkdir/$managedname/server.html': $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">InstanceID&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;OS-version&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;License&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Cores&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;CurrentWWNN&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Min MacAddress&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Max MacAddress&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Min WWPNAddress&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Max WWPNAddress&nbsp;&nbsp;</TH>
		  	<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;NumaSpanningEnabled&nbsp;&nbsp;</TH>
  			</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER><BR><BR>\n";
      close FH;
    }
    else { ### for standalone workstation
      my $InstanceID = get_hyp_val( 'Win32_ComputerSystem', 'Name', $hyp_data_comp_act );
      my $log_proc   = get_hyp_val( 'Win32_ComputerSystem', 'NumberOfLogicalProcessors', $hyp_data_comp_act );
      my $proc       = get_hyp_val( 'Win32_ComputerSystem', 'NumberOfProcessors', $hyp_data_comp_act );
      my $status     = get_hyp_val( 'Win32_ComputerSystem', 'Status', $hyp_data_comp_act );
       # CLASS: Win32_OperatingSystem
       # "CSName","Name","LocalDateTime","Caption","OtherTypeDescription","CSDVersion","Version"
       # "DESKTOP-O79DK45","Microsoft Windows 10 Home|C:\WINDOWS|\Device\Harddisk0\Partition5","20180917082509.846000+120","Microsoft Windows 10 Home",,,"10.0.16299"
      my $cs_caption = get_hyp_val( 'Win32_OperatingSystem', 'Caption', $hyp_data_comp_act );
      my $cs_version = get_hyp_val( 'Win32_OperatingSystem', 'Version', $hyp_data_comp_act );

      my $line_server = $InstanceID .",". $cs_caption .",". $cs_version .",". $licensestatus .",". $proc .",". $log_proc .",". $status . "\n";
      @cpu_cfg = ();
      push @cpu_cfg,$line_server;
      $res_ret   = FormatResults(@cpu_cfg);

      open FH, '>:encoding(UTF-8)', "$wrkdir/$managedname/server.html" or error( "can't open '$wrkdir/$managedname/server.html': $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">InstanceID&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;OS-version&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;License&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Processors&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Log.processors&nbsp;&nbsp;</TH>
	  		<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Proc.Status&nbsp;&nbsp;</TH>
  			</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER><BR><BR>\n";
      close FH;
    }

    #prepare VHD_list table if there are some VMs
    my $vhd_file = "$wrkdir/$managedname/vhd_list.html";
    if (scalar @vhd_list) {
      $res_ret = FormatResults(@vhd_list);

      open FH, '>:encoding(UTF-8)', "$vhd_file" or error( "can't open $vhd_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">VM&nbsp;&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;ControllerType&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Location&nbsp;&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Path&nbsp;&nbsp;&nbsp;</TH>
	  		</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER><BR><BR>\n";
      close FH;
    }
    else {
      unlink $vhd_file if (-f $vhd_file);
    }

    #prepare disk config file for host config
    $res_ret = FormatResults(@disk_cfg);
    open FH, '>:encoding(UTF-8)', "$wrkdir/$managedname/disk.html" or error( "can't open '$wrkdir/$managedname/disk.html': $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">\n";
    print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">VM&nbsp;&nbsp;&nbsp;&nbsp;</TH>
						<TH align=\"center\" class=\"sortable\" valign=\"right\">&nbsp;&nbsp;&nbsp;Provisioned Space GB&nbsp;&nbsp;&nbsp;</TH>
						<TH align=\"center\" class=\"sortable\" valign=\"right\">&nbsp;&nbsp;&nbsp;Used Space GB&nbsp;&nbsp;&nbsp;</TH>
			</TR></thead><tbody>\n";
    $res_ret =~ s/center/right/g;
    print FH "$res_ret";

    # print STDERR "1727 hyp2rrd.pl \$res_ret $res_ret \@disk_cfg @disk_cfg\n";
    print FH "</tbody></TABLE></CENTER><BR><BR>\n";
    close FH;

    #prepare host config file
    #for Host CPU
    my $host_cpu_shares = "";
    my $host_cpuAlloc   = "";
    my $host_memAlloc   = "";

    my $cpu_res    = $host_cpuAlloc;
    my $cpu_shares = $host_cpu_shares;
    my $comp_res   = $host_parent;

    #print Dumper($comp_res);

    #    my $comp_res_w = Vim::find_entity_view(view_type => 'ComputeResource',begin_entity=>$comp_res,properties=>['resourcePool']);
    # print Dumper($comp_res_w);
    #    my $res_pool = $comp_res_w->resourcePool;
    #print Dumper($res_pool);

    #    my $res_pool_w = Vim::find_entity_view(view_type => 'ResourcePool',begin_entity=>$res_pool,properties=>['runtime']);
    # print Dumper($res_pool_w);
    #    my $maxUsage             = $res_pool_w->runtime->cpu->maxUsage;
    #    my $reservationUsed      = $res_pool_w->runtime->cpu->reservationUsed;
    #    my $unreservedForPool    = $res_pool_w->runtime->cpu->unreservedForPool;
    #    my $reservationUsedForVm = $res_pool_w->runtime->cpu->reservationUsedForVm;
    #    my $unreservedForVm      = $res_pool_w->runtime->cpu->unreservedForVm;

    my $l_info1 = "InfO maxUsage reservationUsed unreservedForPool reservationUsedForVm unreservedForVm cpu_res cpu_shares\n";

    #    my $l_info2 = "CPU $maxUsage $reservationUsed $unreservedForPool $reservationUsedForVm $unreservedForVm $cpu_res $cpu_shares\n";
    my $l_info2 = "\n";

    # print "\$maxUsage $maxUsage $reservationUsed $unreservedForPool\n";
    # print "\$refreshRate $refreshRate \$cpu_res $cpu_res \$cpu_shares $cpu_shares\n";

    #for Host MEM
    my $mem_res = $host_memAlloc;

    my $maxUsage             = 0;    #$res_pool_w->runtime->memory->maxUsage/1024/1024;
    my $reservationUsed      = 0;    #$res_pool_w->runtime->memory->reservationUsed/1024/1024;
    my $unreservedForPool    = 0;    #$res_pool_w->runtime->memory->unreservedForPool/1024/1024;
    my $reservationUsedForVm = 0;    #$res_pool_w->runtime->memory->reservationUsedForVm/1024/1024;
    my $unreservedForVm      = 0;    #$res_pool_w->runtime->memory->unreservedForVm/1024/1024;

    my $l_info3 = "MEM $maxUsage $reservationUsed $unreservedForPool $reservationUsedForVm $unreservedForVm\n";

    # print "\$maxUsage $maxUsage $reservationUsed $unreservedForPool\n";
    # print "\$refreshRate $refreshRate \$cpu_res $cpu_res \$mem_res $mem_res\n";
    #print "after writing lpar_trans.txt $step != $STEP\n";

    # following has sense only for vmware = only one ESXi
    open FH, ">$wrkdir/$managedname/host.cfg" or error( "can't open '$wrkdir/$managedname/host.cfg': $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    print FH "$l_info1 $l_info2 $l_info3";
    close FH;

    $step = $STEP;    # do not know if vmware will have sometimes other value

    #LoadDataModuleHyperV::smdc_touch ($SDMC,$wrkdir,$managedname,$host,$act_time);

    $type_sam = "m";

    $host = ""; # working directly with domain name

    prepare_last_time( $hyp_data_comp_act, $hyp_data_comp_last, $managedname, $et_HostSystem );

    #print "prepare_last_time($managedname,$et_HostSystem);\n";

    #
    ### what about cluster
    #

    # CLASS: MSCluster_Cluster
    # "Dedicated","Fqdn","MaxNumberOfNodes","Name","SharedVolumesRoot","Status"
    # ,"MSNET-HVCL.ad.xorux.com","64","MSNET-HVCL","C:\ClusterStorage",

    if (exists $$hyp_data_comp_act->{'MSCluster_Cluster'}) {
      my $cluster_name = get_hyp_val( 'MSCluster_Cluster', 'Name', $hyp_data_comp_act );
      if (index($cluster_name,",") != -1) { # just to be sure, it shoudnt be
        print "this is probably ERROR \$cluster_name $cluster_name\n";
        $cluster_name = (split (",",$cluster_name))[0];
      }
      $cluster_name =~ s/\"//g;
      my $dedicated   = get_hyp_val( 'MSCluster_Cluster', 'Dedicated', $hyp_data_comp_act );
      my $fqdn        = get_hyp_val( 'MSCluster_Cluster', 'Fqdn', $hyp_data_comp_act );
      my $max_nodex   = get_hyp_val( 'MSCluster_Cluster', 'MaxNumberOfNodes', $hyp_data_comp_act );
      my $sv_root     = get_hyp_val( 'MSCluster_Cluster', 'SharedVolumesRoot', $hyp_data_comp_act );
      my $status   = get_hyp_val( 'MSCluster_Cluster', 'Status', $hyp_data_comp_act );

      my $line = $cluster_name .",". $fqdn .",". $dedicated .",". $max_nodex .",". $sv_root .",". $status . "\n";
      my @cluster_list = ();
      push @cluster_list, $line;

      print_hyperv_debug("2323 \$cluster_name $cluster_name \$line $line\n") if $DEBUG == 2;

      # if does not exist cluster dir, prepare it
      my $cluster_dir = "$wrkdir_windows/cluster_$cluster_name";

      if (! -d "$cluster_dir" ) {
        print "mkdir          : $cluster_dir\n";
        mkdir( "$cluster_dir", 0755 ) || error( " Cannot mkdir $cluster_dir: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }

      #prepare cluster table
      $res_ret = FormatResults(@cluster_list);

      open FH, '>:encoding(UTF-8)', "$cluster_dir/cluster_list.html" or error( "can't open $cluster_dir/cluster_list.html: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">CLUSTER&nbsp;&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;FQDN&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Dedicated&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;MaxNumberOfNodes&nbsp;&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;SharedVolumesRoot&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Status&nbsp;&nbsp;&nbsp;</TH>
	  		</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER>\n";
      close FH;
    }

    # CLASS: MSCluster_ClusterToNode
    # "Antecedent","Dependent","RoleOfNode","StateOfNode"
    # "MSCluster_Cluster.Name="MSNET-HVCL"","MSCluster_Node.Name="HVNODE02"",,
    # "MSCluster_Cluster.Name="MSNET-HVCL"","MSCluster_Node.Name="HVNODE01"",,

    if (exists $$hyp_data_comp_act->{'MSCluster_ClusterToNode'}) {
      my $cluster_name = get_hyp_val( 'MSCluster_ClusterToNode', 'Antecedent', $hyp_data_comp_act );
      # MSCluster_Cluster.Name="MSNET-HVCL",MSCluster_Cluster.Name="MSNET-HVCL"
      $cluster_name = (split (",",$cluster_name))[0];
      $cluster_name = (split ("=",$cluster_name))[1];
      $cluster_name =~ s/\"//g;
      my $node_names   = get_hyp_val( 'MSCluster_ClusterToNode', 'Dependent', $hyp_data_comp_act );
      # MSCluster_Node.Name="HVNODE02",MSCluster_Node.Name="HVNODE01"
      my @nodes = split(",",$node_names);
      my @node_list = ();
      my $domain_name = $domain;
      $domain_name =~ s/^domain_//;
      foreach (@nodes) {
        my $node_name = (split("=",$_))[1];
        $node_name =~ s/\"//g;
        print_hyperv_debug("2197 \$node_name $node_name\n") if $DEBUG == 2;
        my $RoleOfNode = get_hyp_val( 'MSCluster_ClusterToNode', 'RoleOfNode', $hyp_data_comp_act, $node_name );
        my $StateOfNode= get_hyp_val( 'MSCluster_ClusterToNode', 'StateOfNode', $hyp_data_comp_act, $node_name );

        my $line = $cluster_name .",". $domain_name .",". $node_name .",". $RoleOfNode .",". $StateOfNode . "\n";
        push @node_list, $line;
      }
      print_hyperv_debug("2188 \$cluster_name $cluster_name \$node_names $node_names\n") if $DEBUG == 2;

      # if does not exist cluster dir, prepare it
      my $cluster_dir = "$wrkdir_windows/cluster_$cluster_name";

      if (! -d "$cluster_dir" ) {
        print "mkdir          : $cluster_dir\n";
        mkdir( "$cluster_dir", 0755 ) || error( " Cannot mkdir $cluster_dir: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }

      #prepare node_list table
      $res_ret = FormatResults(@node_list);

      open FH, '>:encoding(UTF-8)', "$cluster_dir/node_list.html" or error( "can't open $cluster_dir/node_list.html: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      binmode( FH, ":utf8" );

      print FH "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">";
      print FH "<thead><TR> <TH class=\"sortable\" valign=\"center\">CLUSTER&nbsp;&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;DOMAIN&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Node&nbsp;&nbsp;&nbsp;</TH>
        <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;RoleOfNode&nbsp;&nbsp;&nbsp;</TH>
  			<TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;StateOfNode&nbsp;&nbsp;&nbsp;</TH>
	  		</TR></thead><tbody>\n";
      print FH "$res_ret";

      print FH "</tbody></TABLE></CENTER>\n";
      close FH;

    }

  }    # end of cycle on $managedname_list (on servers)

  open my $FH, ">>$counters_info_file" or error( "can't open $counters_info_file: $!" . __FILE__ . ":" . __LINE__ );

  # print output of all forks

  foreach my $fh (@returns) {
    while (<$fh>) {
      if ( $_ =~ 'XERROR' ) {
        ( undef, my $text ) = split( ":", $_ );
        print $FH "$text\n";
      }
      else {
        print $_;
      }
    } ## end while (<$fh>)
    close($fh);
  } ## end foreach my $fh (@returns)

  close $FH;

  print "All chld finish: $host \n" if $DEBUG;

#  waitpid( -1, "WNOHANG" );    # take stats of all forks

  # Include HW info
  # not forhyperv

  #  make_cmd_frame_multiview();
  # print "make_cmd_frame_multiview();\n";
  return 0;
} ## end sub hostsystem_perf

sub VM_hosting_read {
  my $a_ref        = shift;
  my $hosting_file = shift;

  if ( -f "$hosting_file" ) {
    open FH, "$hosting_file" or error( "can't open $hosting_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    @$a_ref = <FH>;
    close FH;
  }
} ## end sub VM_hosting_read

sub VM_hosting_update {
  my $a_ref    = shift;
  my $vm_uuid  = shift;
  my $cmd_time = shift;

  #  if ($vm_uuid eq "522368c5-61e5-a937-b4e7-06bd2f1a3603") {
  #	return
  #  }

  my $found = 0;
  for ( my $i = 0; $i < scalar @$a_ref; $i++ ) {
    my $pos = rindex( @$a_ref[$i], ':' );
    next if $pos < 0;                                         # some trash
    next if ( substr( @$a_ref[$i], 0, 36 ) ne "$vm_uuid" );
    if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "sta" ) {      # keep item
      substr( @$a_ref[$i], length( @$a_ref[$i] ) - 1, 5 ) = ":act\n";
      $found++;
      last;
    }
    if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "end" ) {      # restart item
      substr( @$a_ref[$i], length( @$a_ref[$i] ) - 1, 22 ) = ":start=$cmd_time:new\n";
      $found++;
      last;
    }
  } ## end for ( my $i = 0; $i < scalar...)
  if ( !$found ) {                                            # new item
    push @$a_ref, "$vm_uuid" . ":start=$cmd_time" . ":new\n";
  }
} ## end sub VM_hosting_update

sub VM_hosting_write {
  my $a_ref        = shift;
  my $hosting_file = shift;
  my $cmd_time     = shift;

  # preparing info
  my $change = 0;                                             # write file only when is change

  for ( my $i = 0; $i < scalar @$a_ref; $i++ ) {
    my $pos = rindex( @$a_ref[$i], ':' );
    next if $pos < 0;                                          # some trash
    next if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "end" );
    if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "act" ) {
      substr( @$a_ref[$i], $pos, 4 ) = "";
    }
    if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "new" ) {
      substr( @$a_ref[$i], $pos, 4 ) = "";
      $change++;
    }
    if ( substr( @$a_ref[$i], $pos + 1, 3 ) eq "sta" ) {       # not registered anymore
      substr( @$a_ref[$i], length( @$a_ref[$i] ) - 1, 16 ) = ":end=$cmd_time\n";
      $change++;
    }

  } ## end for ( my $i = 0; $i < scalar...)

  if ($change) {
    open FH, ">$hosting_file" or error( "can't open $hosting_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    print FH @$a_ref;
    close FH;
  }
} ## end sub VM_hosting_write

sub make_cmd_frame_multiview {
  my $cl_managedname = shift;
  my $cl_host        = shift;
  my $et_type        = shift;

  my $time = time();

  # Total CPU per HMC --> must be exactly here
  #  @managednamelist = sort { lc($a) cmp lc($b) } @managednamelist_un; # just to be sure!!!
  # against: semi-panic: attempt to dup freed string at ....

  # it creates cmd files only

  if ( !( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) ) {
    frame_multi( "day",   "m", "d", "MINUTE:60:HOUR:2:HOUR:4:0:%H", $cl_managedname, $cl_host );
    frame_multi( "week",  "m", "w", "HOUR:8:DAY:1:DAY:1:86400:%a",  $cl_managedname, $cl_host );
    frame_multi( "month", "m", "m", "DAY:1:DAY:2:DAY:2:0:%d",       $cl_managedname, $cl_host );
    frame_multi( "year",  "m", "y", "MONTH:1:MONTH:1:MONTH:1:0:%b", $cl_managedname, $cl_host );
  } ## end if ( !( ( defined $et_type...)))

  multiview_hmc( $host, "lpar-multi", "d", "m", $time, "day",   "MINUTE:60:HOUR:2:HOUR:4:0:%H", $et_type, $cl_managedname, $cl_host );
  multiview_hmc( $host, "lpar-multi", "w", "m", $time, "week",  "HOUR:8:DAY:1:DAY:1:86400:%a",  $et_type, $cl_managedname, $cl_host );
  multiview_hmc( $host, "lpar-multi", "m", "m", $time, "month", "DAY:1:DAY:2:DAY:2:0:%d",       $et_type, $cl_managedname, $cl_host );
  multiview_hmc( $host, "lpar-multi", "y", "m", $time, "year",  "MONTH:1:MONTH:1:MONTH:1:0:%b", $et_type, $cl_managedname, $cl_host );

  return 0;
} ## end sub make_cmd_frame_multiview

sub prepare_last_time {
  my $hyp_data_comp_act  = shift; # hash pointer
  my $hyp_data_comp_last = shift; # hash pointer

  my $entity      = shift;
  my $entity_type = shift;
  my $entity_name = shift;
  my $entity_uuid = shift;

  # if ($entity_name eq "CUSTOMER2") { return };

  $step    = $STEP;                       # do not know if vmware will have sometimes other value
  $no_time = $step * $NO_TIME_MULTIPLY;

  $loadhours = 0;                         # must be here before rrd_check

  if ( $entity_type eq $et_ResourcePool ) {

    # print "----------------------------------- getting perf for RP $entity,\$host $host,\$managedname $managedname,\$wrkdir $wrkdir\n";
    $last_file = "$entity_uuid.last";     # RP name
  }
  if ( $entity_type eq $et_Datastore ) {

    # print "----------------------------------- getting perf for datastore $entity,\$host $host,\$managedname $managedname,\$wrkdir $wrkdir\n";
    $last_file = "$entity_uuid.last";     # DS name
  }
  if ( $entity_type eq $et_ClusterComputeResource ) {
    $last_file = "last.txt";
  }

  rrd_check($managedname);                # testing if first load

  print "sample rate    : $host:$managedname $step seconds\n" if $DEBUG;

  #my $t=str2time($date);
  my $t     = $command_unix;
  my $t_int = int($t);

  # print "DATE: $t -- $t_int -- $date\n" ;
  # my $date = $command_date; # can be diff time zone for W10
  my $date = $command_unix;

  my $time_act = strftime "%d/%m/%y %H:%M:%S", localtime( time() );
  print "HyperV date    : $host:$managedname $date (local time: $time_act) \n" if $DEBUG;

  my $last_rec_file = "";

  my $where = "file";
  if ( !$loadhours ) {    # all except the initial load --> check rrd_check
    print_hyperv_debug("checking last file $wrkdir/$managedname/$host/$last_file\n") if $DEBUG == 2;
    if ( -f "$wrkdir/$managedname/$host/$last_file" ) {
      $where = "$last_file";

      # read timestamp of last record
      # this is main loop how to get corectly timestamp of last record!!!

      open( FHLT, "< $wrkdir/$managedname/$host/$last_file" ) || error( " Can't open $wrkdir/$managedname/$host/$last_file: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      foreach my $line1 (<FHLT>) {
        chomp($line1);
        $last_rec_file = $line1;
      }
      close(FHLT);
      print "last rec 1     : $host:$managedname $last_rec_file $wrkdir/$managedname/$host/$last_file\n";

      my $ret = substr( $last_rec_file, 0, 1 );
      if ( $last_rec_file eq '' || $ret =~ /\D/ ) {

        # in case of an issue with last file, remove it and use default 60 min? for further run ...
        error("Wrong input data, deleting file : $wrkdir/$managedname/$host/$last_file : $last_rec_file");
        unlink("$wrkdir/$managedname/$host/$last_file");

        # place there last 1h when an issue with last.txt
        $loadhours = 1;
        $loadmins  = 60;
        $last_rec  = $t - 3600;
      } ## end if ( $last_rec_file eq...)
      else {
        $last_rec = str2time($last_rec_file);
        $loadhours = sprintf( "%.0f", ( $t - $last_rec ) / 3600 );
        $loadhours++;
        $loadmins   = sprintf( "%.0f", ( $t - $last_rec ) / 60 );    # nothing more
        $loadsec_vm = sprintf( "%.0f", ( $t - $last_rec ) );         # for vmware
      } ## end else [ if ( $last_rec_file eq...)]
    } ## end if ( -f "$wrkdir/$managedname/$host/$last_file")
    else {
      # old not accurate way how to get last time stamp, keeping it here as backup if above temp fails for any reason
      if ( -f "$wrkdir/$managedname/$host/mem.rr$type_sam" ) {
        $where = "mem.rr$type_sam";

        # find out last record in the db (hourly)
        RRDp::cmd qq(last "$wrkdir/$managedname/$host/mem.rr$type_sam" );
        my $last_rec_raw = RRDp::read;
        chomp($$last_rec_raw);
        $last_rec = $$last_rec_raw;
        $loadhours = sprintf( "%.0f", ( $t - $last_rec ) / 3600 );
        $loadhours++;
        $loadmins = sprintf( "%.0f", ( $t - $last_rec ) / 60 + 5 - 5 );    # +5mins to be sure, not for vmware
        ( my $sec, $ivmmin, $ivmh, $ivmd, $ivmm, $ivmy, my $wday, my $yday, my $isdst ) = localtime($last_rec);
        $ivmy += 1900;
        $ivmm += 1;

        if ( $loadhours < 0 ) {                                            # Do not know why, but it sometimes happens!!!
          if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam" ) {

            # find out last record in the db (hourly)
            RRDp::cmd qq(last "$wrkdir/$managedname/$host/pool.rr$type_sam" );
            my $last_rec_raw = RRDp::read;
            chomp($$last_rec_raw);
            $last_rec = $$last_rec_raw;
            $loadhours = sprintf( "%.0f", ( $t - $last_rec ) / 3600 );
            $loadhours++;
            $loadmins   = sprintf( "%.0f", ( $t - $last_rec ) / 60 + 5 - 5 );    # +5mins to be sure, not for vmware
            $loadsec_vm = sprintf( "%.0f", ( $t - $last_rec ) );                 # # for vmware
            error("++2 $loadhours -- $last_rec -- $t");
          } ## end if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam")
        } ## end if ( $loadhours < 0 )
      } ## end if ( -f "$wrkdir/$managedname/$host/mem.rr$type_sam")
      else {
        if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam" ) {
          $where = "pool.rr$type_sam";

          # find out last record in the db (hourly)
          RRDp::cmd qq(last "$wrkdir/$managedname/$host/pool.rr$type_sam" );
          my $last_rec_raw = RRDp::read;
          chomp($$last_rec_raw);
          $last_rec = $$last_rec_raw;
          $loadhours = sprintf( "%.0f", ( $t - $last_rec ) / 3600 );
          $loadhours++;
          $loadmins = sprintf( "%.0f", ( $t - $last_rec ) / 60 + 5 );    # +5mins to be sure
        } ## end if ( -f "$wrkdir/$managedname/$host/pool.rr$type_sam")
        else {
          $where     = "init";
          $loadmins  = $INIT_LOAD_IN_HOURS_BACK * 60;
          $loadhours = $INIT_LOAD_IN_HOURS_BACK;
        }
      } ## end else [ if ( -f "$wrkdir/$managedname/$host/mem.rr$type_sam")]
    } ## end else [ if ( -f "$wrkdir/$managedname/$host/$last_file")]
  } ## end if ( !$loadhours )
  else {
    $where = "init";
    my $loadsecs = $INIT_LOAD_IN_HOURS_BACK * 3600;
    $last_rec = $t - $loadsecs;
  }

  if ( $loadhours <= 0 || $loadmins <= 0 ) {    # something wrong is here
    error("Last rec issue: $last_file:  $loadhours - $loadmins -  $last_rec -- $last_rec_file : $date : $t : 01");

    # place some reasonable defaults
    $loadhours = 1;
    $loadmins  = 60;
    $last_rec  = time();
    $last_rec  = $last_rec - 3600;
  } ## end if ( $loadhours <= 0 ||...)

  ( $sec, $ivmmin, $ivmh, $ivmd, $ivmm, $ivmy, $wday, $yday, $isdst ) = localtime($last_rec);
  $ivmy += 1900;
  $ivmm += 1;
  print "last rec 2     : $host:$managedname min:$loadmins , hour:$loadhours, $ivmm/$ivmd/$ivmy $ivmh:$ivmmin : $where\n" if $DEBUG;

  lpm_exclude_vio( $host, $managedname, $wrkdir );

  hmc_load_data($hyp_data_comp_act, $hyp_data_comp_last, $t, $managedname, $host, $last_rec, $t, $entity, $entity_type, $entity_uuid );
  $date = localtime();
  print "date load      : $host:$managedname $date\n" if $DEBUG;
} ## end sub prepare_last_time

# provides easy uuid check, pays for 1 arg only
sub uuid_check {

  return ( $_[0] =~ m{.{8}-.{4}-.{4}-.{4}-.{12}} );
}

# provides easy uuid check, pays for 1 arg only, for datastore uuid
sub uuid_check_ds {

  return ( $_[0] =~ m{.{8}-.{8}-.{4}-.{12}} );
}

sub uuid_check_ds_nfs {

  return ( $_[0] =~ m{.{8}-.{8}} );
}

sub uuid_check_ds_vsan {

  return ( $_[0] =~ m{.{16}-.{16}} );
}

sub hmc_load_data {
  my $hyp_data_comp_act  = shift; # hash pointer
  my $hyp_data_comp_last = shift; # hash pointer

  my $hmc_utime = shift;

  #my $loadhours = shift; # it must be GLOBAL variable
  my $managedname = shift;
  my $host        = shift;
  my $last_rec    = shift;
  my $t           = shift;
  my $entity      = shift;
  my $entity_type = shift;
  my $entity_uuid = shift;

  print_hyperv_debug("1905 hyp2rrd.pl \$hmc_utime $hmc_utime \$managedname $managedname \$host $host \$last_rec $last_rec \$t $t \$vm_host ,$vm_host,\n") if $DEBUG == 2;

  if ( $loadhours == $INIT_LOAD_IN_HOURS_BACK ) {

    # just to be sure ....
    $loadmins = $INIT_LOAD_IN_HOURS_BACK * 60;
  }

  if ( $loadhours <= 0 || $loadmins <= 0 ) {    # workaround as this sometimes is negative , need to check it out ...
    if ( !$last_rec eq '' ) {
      error("$act_time: time issue 1   : $host:$managedname hours:$loadhours mins:$loadmins Last saved record (vmware time) : $last_rec ; vmware time : $t");
    }
    $loadhours = 1;
    $loadmins  = 60;
  } ## end if ( $loadhours <= 0 ||...)

  my $loadsecs        = $loadmins * 60;
  my $hmc_start_utime = $hmc_utime - $loadsecs;

  if ( $loadmins > 0 ) {

    #  if ( $loadmins  > 60 ) { $loadmins = 60 };

    if ( $loadhours != $INIT_LOAD_IN_HOURS_BACK ) {
      print "download data  : $host:$managedname last $loadmins  minute(s) ($loadhours hours) ($loadsec_vm sec VM)\n" if $DEBUG;
    }

    my $pef_time_sec = $loadsec_vm;
    if ( $loadsec_vm eq "" ) {
      $pef_time_sec = $loadmins * 60;
    }

    ###   forking

    if ( !$do_fork || $cycle_count == $PARALLELIZATION ) {
      print "No fork        : $host:$managedname : $server_count\n" if $DEBUG;
      load_data_and_graph( $hyp_data_comp_act, $hyp_data_comp_last, $pef_time_sec, $entity, $entity_type, $entity_uuid );
      $cycle_count = 0;
    }
    else {

      local *FH;
      $pid[$server_count] = open( FH, "-|" );

      #      $pid[$server_count] = fork();
      if ( not defined $pid[$server_count] ) {
        error("$host:$managedname could not fork");
      }
      elsif ( $pid[$server_count] == 0 ) {
        print "Fork           : $host:$managedname : $server_count child pid $$\n" if $DEBUG;
        my $i_am_fork = "fork";
        RRDp::end;
        RRDp::start "$rrdtool";

        eval { Util::connect(); };
        if ($@) {
          my $ret = $@;
          chomp($ret);
          error( "vmw2rrd failed: $ret " . __FILE__ . ":" . __LINE__ );
          exit(1);
        } ## end if ($@)

        # locale for english
        my $serviceContent = Vim::get_service_content();
        my $sessionManager = Vim::get_view( mo_ref => $serviceContent->sessionManager );
        $sessionManager->SetLocale( locale => "en" );

        #        $sessionManager->SetLocale(locale => "de");

        Opts::assert_usage( defined($sessionManager), "No sessionManager." );

        load_data_and_graph( $hyp_data_comp_act, $hyp_data_comp_last, $pef_time_sec, $entity, $entity_type, $entity_uuid );
        print "Fork exit      : $host:$managedname : $server_count\n" if $DEBUG;
        RRDp::end;
        Util::disconnect();
        exit(0);
      } ## end elsif ( $pid[$server_count...])
      print "Parent continue: $host:$managedname $pid[$server_count ] parent pid $$\n";
      $server_count++;

      push @returns, *FH;

    } ## end else [ if ( !$do_fork || $cycle_count...)]
    $cycle_count++;
  } ## end if ( $loadmins > 0 )
  else {
    my $t1 = localtime($last_rec);
    my $t2 = localtime($t);
    error("$act_time: time issue 2   : $host:$managedname hours:$loadhours mins:$loadmins Last saved record (HMC lslparutil time) : $last_rec ; HMC time : $t - $t1 - $t2");
  }
} ## end sub hmc_load_data

sub load_data_and_graph {
  my $hyp_data_comp_act  = shift; # hash pointer
  my $hyp_data_comp_last = shift; # hash pointer

  my $pef_time_sec = shift;
  my $entity       = shift;
  my $entity_type  = shift;
  my $entity_uuid  = shift;

  if ( !defined $entity_type ) {
    $entity_type = "";
  }

  my $st_date;
  my $end_date;

  #$perfmgr_view = Vim::get_view(mo_ref => Vim::get_service_content()->perfManager);

  $error_select_counters = 0;    # next function could set it
                                 #init_perf_counter_info($entity_type);

  ( $st_date, $end_date, $pef_time_sec ) = get_last_date_range( $pef_time_sec, $entity_type );
  print "dates          : \$st_date $st_date \$end_date $end_date total sec $pef_time_sec (F$server_count)\n";

  # print "$entity_type, $st_date, $end_date, $pef_time_sec, $entity_uuid\n";

  if ( $entity_type eq $et_ClusterComputeResource ) {
    get_entity_perf( $entity, $entity_type, $st_date, $end_date, $pef_time_sec );
    return;
  }

  if ( $entity_type eq $et_ResourcePool ) {
    get_entity_perf( $entity, $entity_type, $st_date, $end_date, $pef_time_sec, $entity_uuid );
    return;
  }

  if ( $entity_type eq $et_Datastore ) {
    get_entity_perf( $entity, $entity_type, $st_date, $end_date, $pef_time_sec, $entity_uuid );
    return;
  }

  $entity_type = $et_HostSystem;
  my $entity_host = $vm_host;
  get_entity_perf( $hyp_data_comp_act, $hyp_data_comp_last, $entity_host, $entity_type, $st_date, $end_date, $pef_time_sec );

  $entity_type = $et_VirtualMachine;

  foreach my $entity ( sort { $a cmp $b } @vm_view ) {

    #check if the vm is on to collect stats

    next if $entity eq $UnDeF; # server has no VMs

    if ( $vm_name_powerstat{$entity} ne "ON" ) {
      print "fetching VM    : " . $entity . " is powered " . $vm_name_powerstat{$entity} . " No stats available. (F$server_count)\n" if $DEBUG;
      next;
    }
    $fail_entity_name = $entity;
    $fail_entity_type = "VM";

    get_entity_perf( $hyp_data_comp_act, $hyp_data_comp_last, $entity, $entity_type, $st_date, $end_date, $pef_time_sec );
  } ## end foreach my $entity ( sort {...})
} ## end sub load_data_and_graph

sub get_entity_perf {
  my ( $hyp_data_comp_act, $hyp_data_comp_last, $entity, $entity_type, $st_date, $end_date, $pef_time_sec, $entity_uuid ) = @_;

  my $entity_nick = "VM    : ";
  $entity_nick = "HS    : " if $entity_type eq $et_HostSystem;
  $entity_nick = "clust : " if $entity_type eq $et_ClusterComputeResource;
  $entity_nick = "RP    : " if $entity_type eq $et_ResourcePool;
  $entity_nick = "DS    : " if $entity_type eq $et_Datastore;

  print_hyperv_debug("in sub get_entity_perf \$entity_type $entity_type \$entity ,$entity, \$vm_host ,$vm_host, \$st_date $st_date \$end_date $end_date \$pef_time_sec $pef_time_sec\n") if $DEBUG == 2;

  # return if $entity_type eq "VirtualMachine";

  my $intervals;
  my $refreshRate = "-1";    # for any case

  my @fake_arr = ();         # just for DS ESXi using
  $fake_arr[0]{'sample'} = 'ahoj';
  my $perf_metric_ids = \@fake_arr;

  my $intervalId;

  my $maxsample;
  my $perf_query_spec;

  my $real_sampling_period = "Nothing";
  my $time_stamps;
  my $perf_data;

  my $spec_routine = 0;

  $vm_vCPU[0]                = "U"; # in any case
  $vm_MemoryAvailable[0]     = "U"; # in any case, only for VMs under 2016

  if ( $entity_type eq $et_VirtualMachine ) {

    # number of vcpu is in $vm_name_vcpu{$entity};

    # print STDERR "2443 hyp2rrd.pl computing for $entity\n";

    #return if $entity eq "CentOS01";
    #return if $entity eq "WinXP";
    #return if $entity eq "XoruX-image";
    #return if $entity eq "XoruX-master";

    $vm_uuid_active = $vm_name_uuid{$entity};
    $last_file      = "$vm_uuid_active.last";
    if ( !-f "$wrkdir/$all_hyperv_VMs/$last_file" ) {
      `touch "$wrkdir/$all_hyperv_VMs/$last_file"`;
    }
    #
    ### data digging
    #

    # Memory
    #
    $vm_TotalPhysicalMemory[0] = $vm_uuid_mem{$vm_uuid_active};

    # there are no these counters for VM
    $vm_AvailableMBytes[0]  = "U";
    $vm_CacheBytes[0]       = "U";
    $vm_PagesInputPersec[0] = "U";
    $vm_PagesOutputPersec[0]= "U";

    $vm_MemoryAvailable[0] = "U";

    if (exists $$hyp_data_comp_act->{'Msvm_SummaryInformation'}) {
      $vm_MemoryAvailable[0] = get_hyp_val( 'Msvm_SummaryInformation', 'MemoryAvailable', $hyp_data_comp_act, $vm_uuid_active ); # not name but UUID
      # sometimes happens, should be 0-100 as it is %
      if ( $vm_MemoryAvailable[0] eq $UnDeF || $vm_MemoryAvailable[0] > 100) {
        $vm_MemoryAvailable[0] = "U";
      }
    }

    # Disk
    #
    # CLASS: Win32_PerfRawData_Counters_HyperVVirtualStorageDevice
    # Frequency_PerfTime|Name|ReadBytesPersec|Timestamp_PerfTime|WriteBytesPersec
    # 3117912|C:-Users-Public-Documents-Hyper-V-Virtual hard disks-XoruX-master.vhdx|200728064|6790638553965|883835904

    $vm_DiskBytesPersec[0]     = "U";
    $vm_DiskReadBytesPersec[0] = delta_get_hyp_val( 'Win32_PerfRawData_Counters_HyperVVirtualStorageDevice', 'ReadBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
    print "2965 \$vm_DiskReadBytesPersec[0] $vm_DiskReadBytesPersec[0]\n" if $DEBUG == 2;
    if ($vm_DiskReadBytesPersec[0] eq "") {
      $vm_DiskReadBytesPersec[0] = "U";
    }
    else { # sum if more numbers like "0,1,2"
      my @narr = split( ",",$vm_DiskReadBytesPersec[0]);
      $vm_DiskReadBytesPersec[0] = 0;
      foreach (@narr) { $vm_DiskReadBytesPersec[0] += $_; }
    }
    # print "2972 \$entity $entity \$vm_DiskReadBytesPersec[0] $vm_DiskReadBytesPersec[0]\n";

    $vm_DiskReadsPersec[0]     = "U";
    $vm_DiskTransfersPersec[0] = "U";
    $vm_DiskWriteBytesPerse[0] = delta_get_hyp_val( 'Win32_PerfRawData_Counters_HyperVVirtualStorageDevice', 'WriteBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
    print "2979 \$vm_DiskWriteBytesPerse[0] $vm_DiskWriteBytesPerse[0]\n" if $DEBUG == 2;
    if ($vm_DiskWriteBytesPerse[0] eq "") {
      $vm_DiskWriteBytesPerse[0] = "U";
    }
    else { # sum if more numbers like "0,1,2"
      my @narr = split( ",",$vm_DiskWriteBytesPerse[0]);
      $vm_DiskWriteBytesPerse[0] = 0;
      foreach (@narr) { $vm_DiskWriteBytesPerse[0] += $_; }
    }
     print "2986 \$entity $entity \$vm_DiskReadBytesPersec[0] $vm_DiskReadBytesPersec[0] \$vm_DiskWriteBytesPerse[0] $vm_DiskWriteBytesPerse[0]\n" if $DEBUG == 2;

    # Net

    # CLASS: Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter
    # "Name","BytesPersec","Timestamp_PerfTime","Frequency_PerfTime","BytesReceivedPersec","BytesSentPersec"
    # "hvlinux02_Network Adapter_61538264-853b-43a0-99f5-dcfe083864c0--d599bf62-f009-4ef2-add6-1d5892df4f3b","60712415","29903862368223","2994772","53888216","6824199"
    # "hvlinux01_Network Adapter_cbd9d469-a221-4228-816f-3860110150ad--77392ce0-a661-4e13-8249-60c765b43746","66790581","29903862368223","2994772","60484772","6305809"
    # "Intel[R] 82574L Gigabit Network Connection__DEVICE_{BFEF4FE9-43CC-47AC-8097-E0B4BD66CC80}","239408559995","29903862368223","2994772","181198964451","58209595544"
    # "Virtual Switch_D576DFFB-56B1-4997-897E-0942AEBF39BB","238969981569","29903862368223","2994772","57784427572","181185553997"

    # there are two possibilities to find NET for VM
    $vm_BytesReceivedPersec[0] = "U";
    $vm_BytesSentPersec[0]     = "U";
    $vm_BytesTotalPersec[0]    = "U";

    # prepare time coeff, HMM do not know if it would be more precise to get it directly from Timestamp_PerfTime ?
    my $vm_time_step        = delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'Hyperv_UTC', $hyp_data_comp_act, $hyp_data_comp_last );
    my $vm_time_coefficient = $vm_time_step/300; #  adjusted to 5 minutes base
    if ($vm_time_coefficient < 1) { # can happen when another (debug) load from cmd line or so
      $vm_time_coefficient = 1;
    }

    if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter'}) {
      # one VM can have more adapters, add counters to one sum
      # CLASS: Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter
      # "Name","BytesPersec","Timestamp_PerfTime","Frequency_PerfTime","BytesReceivedPersec","BytesSentPersec"
      # "SRV-SQL-DEV-N02_SRV-SQL-DEV-N02_b053d289-110c-499e-98a4-6dff50bc93bd--c14392fb-3d89-440e-a56b-44a7e072a3cd","89636251572","37027116702875","2539067","89544667302","91584270"
      # "SRV-SQL-DEV-N02_SRV-SQL-DEV-N02_b053d289-110c-499e-98a4-6dff50bc93bd--11ca85b9-839d-4368-b9c6-eb7998d2d7d9","628113439","37027116702875","2539067","549465020","78648419"

      $vm_BytesReceivedPersec[0] = delta_get_hyp_val( 'Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter', 'BytesReceivedPersec', $hyp_data_comp_act, $hyp_data_comp_last, $entity);
       my @narr = split( ",",$vm_BytesReceivedPersec[0]);
       $vm_BytesReceivedPersec[0] = 0;
       foreach (@narr) { $vm_BytesReceivedPersec[0] += $_; }
      $vm_BytesReceivedPersec[0] /= $vm_time_coefficient;

      $vm_BytesSentPersec[0]     = delta_get_hyp_val( 'Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter', 'BytesSentPersec', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
       @narr = ();
       @narr = split( ",",$vm_BytesSentPersec[0]);
       $vm_BytesSentPersec[0] = 0;
       foreach (@narr) { $vm_BytesSentPersec[0] += $_; }
      $vm_BytesSentPersec[0]     /= $vm_time_coefficient;

      $vm_BytesTotalPersec[0]    = delta_get_hyp_val( 'Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter', 'BytesPersec', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
       @narr = ();
       @narr = split( ",",$vm_BytesTotalPersec[0]);
       $vm_BytesTotalPersec[0] = 0;
       foreach (@narr) { $vm_BytesTotalPersec[0] += $_; }
      $vm_BytesTotalPersec[0]    /= $vm_time_coefficient;
      # print "3032 $vm_BytesReceivedPersec[0] $vm_BytesSentPersec[0] $vm_BytesTotalPersec[0]\n";
    }
    elsif (exists $$hyp_data_comp_act->{'Win32_PerfRawData_NvspPortStats_HyperVVirtualSwitchPort'} && exists $$hyp_data_comp_act->{'Msvm_ElementSettingData'}) {

      $entity_uuid = get_hyp_val( 'Msvm_ComputerSystem', 'Name', $hyp_data_comp_act, $entity );
      print "Note           : (F$server_count) no data from Win32_PerfRawData_NvspNicStats_HyperVVirtualNetworkAdapter for entity $entity $entity_uuid\n";

      # way to identify switch port to VM, which is using it
      # CLASS: Msvm_ElementSettingData
      # "ManagedElement","SettingData"
      # "\\HVNODE01\root\virtualization\v2:Msvm_EthernetSwitchPort.CreationClassName="Msvm_EthernetSwitchPort",DeviceID="Microsoft:D71D793B-0D88-43E9-A66A-453C98082258",SystemCreationClassName="Msvm_VirtualEthernetSwitch",SystemName="6EB18D68-71D3-46AD-B6FF-CF1FD5256D8E"","\\HVNODE01\root\virtualization\v2:Msvm_EthernetPortAllocationSettingData.InstanceID="Microsoft:61538264-853B-43A0-99F5-DCFE083864C0\\d599bf62-f009-4ef2-add6-1d5892df4f3b\\C""
      # "\\HVNODE01\root\virtualization\v2:Msvm_EthernetSwitchPort.CreationClassName="Msvm_EthernetSwitchPort",DeviceID="Microsoft:D266985C-51AC-4C00-81E9-91A17B7E08AC",SystemCreationClassName="Msvm_VirtualEthernetSwitch",SystemName="6EB18D68-71D3-46AD-B6FF-CF1FD5256D8E"","\\HVNODE01\root\virtualization\v2:Msvm_EthernetPortAllocationSettingData.InstanceID="Microsoft:CBD9D469-A221-4228-816F-3860110150AD\\77392ce0-a661-4e13-8249-60c765b43746\\C""

      # CLASS: Win32_PerfRawData_NvspPortStats_HyperVVirtualSwitchPort
      # "Name","BytesPersec","Timestamp_PerfTime","Frequency_PerfTime","BytesReceivedPersec","BytesSentPersec"
      # "6EB18D68-71D3-46AD-B6FF-CF1FD5256D8E_D71D793B-0D88-43E9-A66A-453C98082258","23934152268","61482940656805","2994772","101419968","23832732300"
      # "6EB18D68-71D3-46AD-B6FF-CF1FD5256D8E_D266985C-51AC-4C00-81E9-91A17B7E08AC","170350118","61482940656805","2994772","13672846","156677272"

      # CLASS: Msvm_ComputerSystem
      # "Name","Caption","Description","ElementName","EnabledState","HealthState","NumberOfNumaNodes","OperatingStatus","Status","StatusDescriptions"
      # "HVNODE01","Hosting Computer System","Microsoft Hosting Computer System","HVNODE01","2","5","1",,"OK","OK"
      # "61538264-853B-43A0-99F5-DCFE083864C0","Virtual Machine","Microsoft Virtual Machine","hvlinux02","2","5","1",,"OK","Operating normally"
      # "CBD9D469-A221-4228-816F-3860110150AD","Virtual Machine","Microsoft Virtual Machine","hvlinux01","2","5","1",,"OK","Operating normally"

      my @vm_ports_m_e = split (",", get_hyp_val( 'Msvm_ElementSettingData', 'ManagedElement', $hyp_data_comp_act,'Msvm_EthernetSwitchPort' ));
      my @vm_ports_s_d = split (",", get_hyp_val( 'Msvm_ElementSettingData', 'SettingData', $hyp_data_comp_act,'Msvm_EthernetSwitchPort' ));
      # print "3057 \@vm_ports_s_d       : (F$server_count) @vm_ports_s_d\n @vm_ports_m_e\n";
      my ($index) = grep { $vm_ports_s_d[$_] =~ $entity_uuid } 0 .. $#vm_ports_s_d;    # is it  ?
       print "3059 \$index $index found $entity_uuid\n";
      my $vm_port = $vm_ports_m_e[$index];
      print "3061 \$vm_port $vm_port\n";
      $vm_port =~ s/.*.DeviceID="Microsoft://;
      $vm_port =~ s/".SystemCreation.*//;
      print "3064 \$vm_port $vm_port\n";
      $vm_BytesReceivedPersec[0] = delta_get_hyp_val( 'Win32_PerfRawData_NvspPortStats_HyperVVirtualSwitchPort', 'BytesSentPersec', $hyp_data_comp_act, $hyp_data_comp_last, $vm_port);
      $vm_BytesReceivedPersec[0] /= $vm_time_coefficient;
      $vm_BytesSentPersec[0]     = delta_get_hyp_val( 'Win32_PerfRawData_NvspPortStats_HyperVVirtualSwitchPort', 'BytesReceivedPersec', $hyp_data_comp_act, $hyp_data_comp_last, $vm_port );
      $vm_BytesSentPersec[0]     /= $vm_time_coefficient;
      $vm_BytesTotalPersec[0]    = $vm_BytesReceivedPersec[0] + $vm_BytesSentPersec[0];
      # print "3068 $vm_BytesReceivedPersec[0] $vm_BytesSentPersec[0] $vm_BytesTotalPersec[0]\n";

    }
    else {
      print "Note           : (F$server_count) no data from Win32_PerfRawData_NvspPortStats_HyperVVirtualSwitchPort or Msvm_ElementSettingData\n";
    }

    $vm_vCPU[0]                = "$vm_name_vcpu{$entity}";

    # CPU
    #
    # class: Win32_PerfRawData_HvStats_HyperVHypervisorVirtualProcessor
    # Frequency_PerfTime|Name|PercentTotalRunTime|Timestamp_PerfTime
    # 3117912|WinXP:Hv VP 1|3272009389048|6220968390660
    # 3117912|WinXP:Hv VP 0|3283077517400|6220968390660
    # 3117912|XoruX-master:Hv VP 1|9806837924|6220968390660
    # 3117912|XoruX-master:Hv VP 0|14282384764|6220968390660
    # 3117912|CentOS01:Hv VP 0|57059017497|6220968390660
    # 3117912|_Total|6636235146633|6220968390660

    my $values = delta_get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorVirtualProcessor', 'PercentTotalRunTime', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
    my @value_arr = split( /,/, $values );

    my $summ = 0;
    print_hyperv_debug("2731 hyp2rrd.pl \@value_arr @value_arr \$values $values \$vm_name_vcpu{$entity} $vm_name_vcpu{$entity}\n") if $DEBUG == 2;
    if ($vm_name_vcpu{$entity} ne $UnDeF) {
      for ( my $i = 0; $i < $vm_name_vcpu{$entity}; $i++ ) {
        $summ += $value_arr[$i] if defined $value_arr[$i];
      }
      $vm_PercentTotalRunTime[0] = int( $summ / $vm_name_vcpu{$entity} );    # in case of 2012
                                                                           # $vm_PercentTotalRunTime[0] = int ($summ);
    }

    $values = delta_get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorVirtualProcessor', 'Timestamp_PerfTime', $hyp_data_comp_act, $hyp_data_comp_last, $entity );
    @value_arr = split( /,/, $values );
    $summ = 0;
    if ($vm_name_vcpu{$entity} ne $UnDeF) {
      for ( my $i = 0; $i < $vm_name_vcpu{$entity}; $i++ ) {
        $summ += $value_arr[$i] if defined $value_arr[$i];
      }
    }

    # $vm_Timestamp_PerfTime[0] = int ($summ / $vm_name_vcpu{$entity});
    # $vm_Timestamp_PerfTime[0] = int($summ);  # no, you cannot make summ here, get only one time difference e.g. 1st
    $vm_Timestamp_PerfTime[0] = $value_arr[0];

    # print "\$entity $entity \$vm_PercentTotalRunTime[0] $vm_PercentTotalRunTime[0] \$vm_Timestamp_PerfTime[0] $vm_Timestamp_PerfTime[0]\n";
    # return;
  } ## end if ( $entity_type eq $et_virtualmachine)
  else {    # host_system
    #
    ### data digging
    #

    # in case of standalone windows
    if ($vm_vCPU[0] eq "U") {
      $vm_vCPU[0] = get_hyp_val( 'Win32_ComputerSystem', 'NumberOfProcessors', $hyp_data_comp_act );
      print_hyperv_debug("2766 \$vm_vCPU[0] $vm_vCPU[0] standalone windows ?\n") if $DEBUG == 2;
    }

    $vm_PercentTotalRunTime[0] = "";
    if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor'}) {
      $vm_PercentTotalRunTime[0] = delta_get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor', 'PercentTotalRunTime', $hyp_data_comp_act, $hyp_data_comp_last, '_Total' );
    }
    if ($vm_PercentTotalRunTime[0] eq "") {
      # probably standalone windows
      if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_PerfOS_Processor'} && exists $$hyp_data_comp_last->{'Win32_PerfRawData_PerfOS_Processor'}) {
        $vm_PercentTotalRunTime[0] = delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Processor','PercentUserTime', $hyp_data_comp_act, $hyp_data_comp_last, '_Total' );
      }
      else {
        # not sufficient data
        print_hyperv_debug("3086 no data for Win32_PerfRawData_PerfOS_Processor\n") if $DEBUG == 2;
        return;
      }
    }
    $vm_Timestamp_PerfTime[0]  = "";
    if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor'}) {
      $vm_Timestamp_PerfTime[0]  = delta_get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor', 'Timestamp_PerfTime', $hyp_data_comp_act, $hyp_data_comp_last,  '_Total' );
    }
    if ($vm_Timestamp_PerfTime[0] eq "") {
      if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_PerfOS_Processor'} && exists $$hyp_data_comp_last->{'Win32_PerfRawData_PerfOS_Processor'}) {
        $vm_Timestamp_PerfTime[0]  = delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Processor','Timestamp_PerfTime', $hyp_data_comp_act, $hyp_data_comp_last,  '_Total' );
      }
      else {
        print_hyperv_debug("3099 no data for Win32_PerfRawData_PerfOS_Processor\n") if $DEBUG == 2;
        return;
      }
    }

    $vm_AvailableMBytes[0]  = get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'AvailableMbytes', $hyp_data_comp_act );
    $vm_CacheBytes[0]       = get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'CacheBytes', $hyp_data_comp_act );
    my $vm_time_step        = delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'Hyperv_UTC', $hyp_data_comp_act, $hyp_data_comp_last );
    my $vm_time_coefficient = $vm_time_step/300; #  adjusted to 5 minutes base
    if ($vm_time_coefficient < 1) { # can happen when another (debug) load from cmd line or so
      $vm_time_coefficient = 1;
    }
    print_hyperv_debug("3012 \$vm_AvailableMBytes[0] $vm_AvailableMBytes[0] \$vm_CacheBytes[0] $vm_CacheBytes[0] \$vm_time_step $vm_time_step\n") if $DEBUG == 2;
    $vm_PagesInputPersec[0] = delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'PagesInputPersec', $hyp_data_comp_act, $hyp_data_comp_last );
    $vm_PagesInputPersec[0] /= $vm_time_coefficient;
    $vm_PagesOutputPersec[0]= delta_get_hyp_val( 'Win32_PerfRawData_PerfOS_Memory', 'PagesOutputPersec', $hyp_data_comp_act, $hyp_data_comp_last );
    $vm_PagesOutputPersec[0] /=$vm_time_coefficient;

    # CLASS: Win32_PerfRawData_PerfDisk_PhysicalDisk

#    $vm_DiskBytesPersec[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');
    $vm_DiskReadBytesPersec[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskReadBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');
    $vm_DiskReadsPersec[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskReadsPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');
#    $vm_DiskTransfersPersec[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskTransfersPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');
    $vm_DiskWriteBytesPerse[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskWriteBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');
    $vm_DiskWritesPersec[0] = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskWritesPersec', $hyp_data_comp_act, $hyp_data_comp_last,'_Total');

    # CLASS: Win32_PerfRawData_Tcpip_NetworkInterface

    my @values = split(",",delta_get_hyp_val('Win32_PerfRawData_Tcpip_NetworkInterface', 'BytesReceivedPersec', $hyp_data_comp_act, $hyp_data_comp_last));

    # prepare summ
    # example 16038813212 0 501059 0 0 0
    # print stderr "2452, hyp2rrd.pl \@values @values\n";
    $vm_BytesReceivedPersec[0] = eval join '+', @values;
    @values = split(",",delta_get_hyp_val('Win32_PerfRawData_Tcpip_NetworkInterface', 'BytesSentPersec', $hyp_data_comp_act, $hyp_data_comp_last));
    $vm_BytesSentPersec[0] = eval join '+', @values;
    @values = split(",",delta_get_hyp_val('Win32_PerfRawData_Tcpip_NetworkInterface', 'BytesTotalPersec', $hyp_data_comp_act, $hyp_data_comp_last));
    $vm_BytesTotalPersec[0] = eval join '+', @values;

  } ## end else [ if ( $entity_type eq $et_VirtualMachine)]

  my $samples_number_must = 1;

  my $values;

  $samples_number = 1;

  print "fetching " . $entity_nick . $entity . " refreshRate=$refreshRate real_sampling_period=$real_sampling_period samples_expected X real=$samples_number_must X $samples_number (F$server_count)\n" if $DEBUG;

  my $first_time_stamp_unix = $UnDeF;
  if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor'} ) {
    $first_time_stamp_unix = get_hyp_val( 'Win32_PerfRawData_HvStats_HyperVHypervisorLogicalProcessor', 'Hyperv_UTC', $hyp_data_comp_act, '_Total' );
  }
  if ( !defined $first_time_stamp_unix || $first_time_stamp_unix eq $UnDeF ) {
    # error( "(F$server_count) not valid data timestamp $first_time_stamp_unix, use \$command_unix $command_unix instead " . __FILE__ . ":" . __LINE__ ) ; # && return 0;
    print "Note           : (F$server_count) not valid data timestamp $first_time_stamp_unix, use \$command_unix $command_unix instead\n";
    # could be stand alone windows > use original machine time
    $first_time_stamp_unix = $command_unix
  }

  my @vm_time_stamps = ();
  $vm_time_stamps[0] = $first_time_stamp_unix;

  # prepare other metrics array with 'U'

  if ( ( $entity_type eq $et_HostSystem ) || ( $entity_type eq $et_VirtualMachine ) ) {

    #          @vm_Frequency_PerfTime       = ($host_hz)x $samples_number;
    #          @vm_PercentTotalRunTime      = ($PercentTotalRunTime)x $samples_number;
    #          @vm_Timestamp_PerfTime       = ($Timestamp_PerfTime)x $samples_number;
    #          @vm_Memory_active_KB         = ('U')x $samples_number;
    #          @vm_Memory_granted_KB        = ('U')x $samples_number;
    #          @vm_Memory_baloon_MB         = ('U')x $samples_number;
    #          @vm_Disk_usage_KBps          = ('U')x $samples_number;
    #          @vm_Disk_read_KBps           = ('U')x $samples_number;
    #          @vm_Disk_write_KBps          = ('U')x $samples_number;
    #          @vm_Network_usage_KBps       = ('U')x $samples_number;
    #          @vm_Network_received_KBps    = ('U')x $samples_number;
    #          @vm_Network_transmitted_KBps = ('U')x $samples_number;
    #          @vm_Memory_swapin_KBps       = ('U')x $samples_number;
    #          @vm_Memory_swapout_KBps      = ('U')x $samples_number;
    #          @vm_Memory_compres_KBps      = ('U')x $samples_number;
    #          @vm_Memory_decompres_KBps    = ('U')x $samples_number;
    #          @vm_CPU_usage_Percent        = ('U')x $samples_number;
    #          @vm_CPU_ready_ms             = ('U')x $samples_number;

    if ( $entity_type eq $et_HostSystem ) {

      #             @Host_memory_size         = ($host_memorySize)x $samples_number;
    }

    # rank is according to @counter_hsvm_eng(ger1,ger2) used in sub prepare_vm_metric(), see global definitions
    #          $arr_pointers[0]  = \@vm_CPU_usage_MHz;
    $arr_pointers[1]  = \@vm_Disk_usage_KBps;
    $arr_pointers[2]  = \@vm_Disk_read_KBps;
    $arr_pointers[3]  = \@vm_Disk_write_KBps;
    $arr_pointers[4]  = \@vm_Network_usage_KBps;
    $arr_pointers[5]  = \@vm_Network_received_KBps;
    $arr_pointers[6]  = \@vm_Network_transmitted_KBps;
    $arr_pointers[7]  = \@vm_Memory_active_KB;
    $arr_pointers[8]  = \@vm_Memory_granted_KB;
    $arr_pointers[9]  = \@vm_Memory_swapin_KBps;
    $arr_pointers[11] = \@vm_Memory_swapout_KBps;
    $arr_pointers[12] = \@vm_Memory_decompres_KBps;
    $arr_pointers[13] = \@vm_Memory_compres_KBps;
    $arr_pointers[10] = \@vm_Memory_baloon_MB;
    $arr_pointers[14] = \@vm_CPU_usage_Percent;
    $arr_pointers[15] = \@vm_CPU_ready_ms;
  } ## end if ( ( $entity_type eq...))
  elsif ( $entity_type eq $et_ClusterComputeResource ) {
    @cl_CPU_usage_Proc          = ('U') x $samples_number;
    @cl_CPU_usage_MHz           = ('U') x $samples_number;
    @cl_CPU_reserved_MHz        = ('U') x $samples_number;
    @cl_Memory_usage_Proc       = ('U') x $samples_number;
    @cl_Memory_reserved_MB      = ('U') x $samples_number;
    @cl_Memory_granted_KB       = ('U') x $samples_number;
    @cl_Memory_active_KB        = ('U') x $samples_number;
    @cl_Memory_shared_KB        = ('U') x $samples_number;
    @cl_Memory_zero_KB          = ('U') x $samples_number;
    @cl_Memory_swap_KB          = ('U') x $samples_number;
    @cl_Memory_baloon_KB        = ('U') x $samples_number;
    @cl_Memory_consumed_KB      = ('U') x $samples_number;
    @cl_Memory_overhead_KB      = ('U') x $samples_number;
    @cl_Memory_compressed_KB    = ('U') x $samples_number;
    @cl_Memory_compression_KBps = ('U') x $samples_number;
    @cl_Memory_decompress_KBps  = ('U') x $samples_number;
    @cl_Power_usage_Watt        = ('U') x $samples_number;
    @cl_Power_cup_Watt          = ('U') x $samples_number;
    @cl_Cluster_eff_CPU_MHz     = ('U') x $samples_number;
    @cl_Cluster_eff_memory_MB   = ('U') x $samples_number;
    @cl_CPU_total_MHz           = ($cluster_effectiveCpu) x $samples_number;
    @cl_Memory_total_MB         = ($cluster_effectiveMemory) x $samples_number;

    # rank is according to @counter_cl_eng(ger1,ger2) used in sub prepare_vm_metric(), see global definitions
    $arr_pointers[0]  = \@cl_CPU_usage_MHz;
    $arr_pointers[1]  = \@cl_CPU_usage_Proc;
    $arr_pointers[2]  = \@cl_CPU_reserved_MHz;
    $arr_pointers[3]  = \@cl_CPU_total_MHz;
    $arr_pointers[4]  = \@cl_Cluster_eff_CPU_MHz;
    $arr_pointers[5]  = \@cl_Cluster_eff_memory_MB;
    $arr_pointers[6]  = \@cl_Memory_total_MB;
    $arr_pointers[7]  = \@cl_Memory_shared_KB;
    $arr_pointers[8]  = \@cl_Memory_zero_KB;
    $arr_pointers[9]  = \@cl_Memory_baloon_KB;
    $arr_pointers[10] = \@cl_Memory_consumed_KB;
    $arr_pointers[11] = \@cl_Memory_overhead_KB;
    $arr_pointers[12] = \@cl_Memory_active_KB;
    $arr_pointers[13] = \@cl_Memory_granted_KB;
    $arr_pointers[14] = \@cl_Memory_compressed_KB;
    $arr_pointers[15] = \@cl_Memory_reserved_MB;
    $arr_pointers[16] = \@cl_Memory_swap_KB;
    $arr_pointers[17] = \@cl_Memory_compression_KBps;
    $arr_pointers[18] = \@cl_Memory_decompress_KBps;
    $arr_pointers[19] = \@cl_Memory_usage_Proc;
    $arr_pointers[20] = \@cl_Power_cup_Watt;
    $arr_pointers[21] = \@cl_Power_usage_Watt;
  } ## end elsif ( $entity_type eq $et_ClusterComputeResource)
  elsif ( $entity_type eq $et_ResourcePool ) {
    @cl_CPU_usage_MHz           = ('U') x $samples_number;
    @cl_Memory_granted_KB       = ('U') x $samples_number;
    @cl_Memory_active_KB        = ('U') x $samples_number;
    @cl_Memory_shared_KB        = ('U') x $samples_number;
    @cl_Memory_zero_KB          = ('U') x $samples_number;
    @cl_Memory_swap_KB          = ('U') x $samples_number;
    @cl_Memory_baloon_KB        = ('U') x $samples_number;
    @cl_Memory_consumed_KB      = ('U') x $samples_number;
    @cl_Memory_overhead_KB      = ('U') x $samples_number;
    @cl_Memory_compressed_KB    = ('U') x $samples_number;
    @cl_Memory_compression_KBps = ('U') x $samples_number;
    @cl_Memory_decompress_KBps  = ('U') x $samples_number;
    @cl_cpu_limit               = ($rp_cpu_limit) x $samples_number;
    @cl_cpu_reservation         = ($rp_cpu_reservation) x $samples_number;
    @cl_mem_limit               = ($rp_mem_limit) x $samples_number;
    @cl_mem_reservation         = ($rp_mem_reservation) x $samples_number;

    # rank is according to @counter_rp_eng(ger1,ger2) used in sub prepare_vm_metric(), see global definitions
    $arr_pointers[0]  = \@cl_CPU_usage_MHz;
    $arr_pointers[1]  = \@cl_Memory_shared_KB;
    $arr_pointers[2]  = \@cl_Memory_zero_KB;
    $arr_pointers[3]  = \@cl_Memory_baloon_KB;
    $arr_pointers[4]  = \@cl_Memory_consumed_KB;
    $arr_pointers[5]  = \@cl_Memory_overhead_KB;
    $arr_pointers[6]  = \@cl_Memory_active_KB;
    $arr_pointers[7]  = \@cl_Memory_granted_KB;
    $arr_pointers[8]  = \@cl_Memory_compressed_KB;
    $arr_pointers[9]  = \@cl_Memory_swap_KB;
    $arr_pointers[10] = \@cl_Memory_compression_KBps;
    $arr_pointers[11] = \@cl_Memory_decompress_KBps;
    $arr_pointers[12] = \@cl_cpu_limit;
    $arr_pointers[13] = \@cl_cpu_reservation;
    $arr_pointers[14] = \@cl_mem_limit;
    $arr_pointers[15] = \@cl_mem_reservation;
  } ## end elsif ( $entity_type eq $et_ResourcePool)
  elsif ( $entity_type eq $et_Datastore ) {
    @ds_Datastore_freeSpace_KB        = ($ds_freeSpace) x $samples_number;
    @ds_Datastore_used_KB             = ($ds_used) x $samples_number;
    @ds_Datastore_provision_KB        = ($ds_provisioned) x $samples_number;
    @ds_Datastore_capacity_KB         = ($ds_capacity) x $samples_number;
    @ds_Datastore_read_KBps           = ('U') x $samples_number;
    @ds_Datastore_write_KBps          = ('U') x $samples_number;
    @ds_Datastore_numberReadAveraged  = ('U') x $samples_number;
    @ds_Datastore_numberWriteAveraged = ('U') x $samples_number;

    # rank is according to @counter_ds_eng(ger1,ger2) used in sub prepare_vm_metric(), see global definitions
    $arr_pointers[0] = \@ds_Datastore_used_KB;
    $arr_pointers[1] = \@ds_Datastore_provision_KB;
    $arr_pointers[2] = \@ds_Datastore_capacity_KB;
    $arr_pointers[3] = \@ds_Datastore_read_KBps;
    $arr_pointers[4] = \@ds_Datastore_write_KBps;
    $arr_pointers[5] = \@ds_Datastore_numberReadAveraged;
    $arr_pointers[6] = \@ds_Datastore_numberWriteAveraged;
  } ## end elsif ( $entity_type eq $et_Datastore)
  else {
    error( "(F$server_count) unknown entity_type $entity_type " . __FILE__ . ":" . __LINE__ ) && exit 0;
  }
  my $ts_size = $samples_number;

  # print "Number of samples : $ts_size \n";
  # print "time stamps array @vm_time_stamps\n\n";

  @counter_presence = ();
  my $success_metric = 0;

  # if ($entity_type eq $et_Datastore) {
  #  	print Dumper(@$values);
  #}

  $samples_number = 1;
  my $first_update_time = $vm_time_stamps[0];
  my $last_update_time  = $vm_time_stamps[ $samples_number - 1 ];

  my $update_string     = "";
  my $two_update_string = "";
  my $one_update;
  my $two_update;
  for ( my $i = 0; $i < $samples_number; $i++ ) {
    $update_string .= "$vm_time_stamps[$i],";

    #          $two_update_string .= "$vm_time_stamps[$i],";

    my $diff_item = "U";
    if ( ( $entity_type eq $et_HostSystem ) || ( $entity_type eq $et_VirtualMachine ) ) {

      #            if ($entity_type eq $et_HostSystem) {
      #              $diff_item = $Host_memory_size[$i];
      #            };
      #            if ($entity_type eq $et_VirtualMachine) {
      #              $diff_item = $numCpu;
      #            };
       $one_update = "$vm_PercentTotalRunTime[$i],$vm_Timestamp_PerfTime[$i],$vm_Frequency_PerfTime[$i],$vm_TotalPhysicalMemory[$i],$vm_AvailableMBytes[$i],$vm_MemoryAvailable[$i],$vm_CacheBytes[$i],";
      # $one_update .= "$vm_DiskBytesPersec[$i],$vm_DiskReadBytesPersec[$i],$vm_DiskReadsPersec[$i],$vm_DiskTransfersPersec[$i],";
      $one_update .= "U,$vm_DiskReadBytesPersec[$i],$vm_DiskReadsPersec[$i],U,";
      $one_update .= "$vm_DiskWriteBytesPerse[$i],$vm_DiskWritesPersec[$i],";
      $one_update .= "$vm_BytesReceivedPersec[$i],$vm_BytesSentPersec[$i],$vm_BytesTotalPersec[$i],$vm_PagesInputPersec[$i],$vm_PagesOutputPersec[$i],$vm_vCPU[$i]";

      #            $one_update .= "$vm_Memory_active_KB[$i],$vm_Memory_granted_KB[$i],$vm_Memory_baloon_MB[$i],";
      #            $one_update .= "$vm_Disk_usage_KBps[$i],$vm_Disk_read_KBps[$i],$vm_Disk_write_KBps[$i],";
      #            $one_update .= "$vm_Network_usage_KBps[$i],$vm_Network_received_KBps[$i],$vm_Network_transmitted_KBps[$i],";
      #            $one_update .= "$vm_Memory_swapin_KBps[$i],$vm_Memory_swapout_KBps[$i],";
      #            $one_update .= "$vm_Memory_compres_KBps[$i],$vm_Memory_decompres_KBps[$i],$vm_CPU_usage_Percent[$i],$diff_item,$vm_CPU_ready_ms[$i] ";
      $update_string .= "$one_update";
    } ## end if ( ( $entity_type eq...))
    elsif ( $entity_type eq $et_ClusterComputeResource ) {
      $one_update = "$cl_CPU_usage_MHz[$i],$cl_CPU_usage_Proc[$i],$cl_CPU_reserved_MHz[$i],";
      $one_update    .= "$cl_CPU_total_MHz[$i],$cl_Cluster_eff_CPU_MHz[$i],$cl_Cluster_eff_memory_MB[$i],";
      $one_update    .= "$cl_Memory_total_MB[$i],$cl_Memory_shared_KB[$i],$cl_Memory_zero_KB[$i],";
      $one_update    .= "$cl_Memory_baloon_KB[$i],$cl_Memory_consumed_KB[$i],$cl_Memory_overhead_KB[$i],";
      $one_update    .= "$cl_Memory_active_KB[$i],$cl_Memory_granted_KB[$i],$cl_Memory_compressed_KB[$i],";
      $one_update    .= "$cl_Memory_reserved_MB[$i],$cl_Memory_swap_KB[$i],$cl_Memory_compression_KBps[$i],";
      $one_update    .= "$cl_Memory_decompress_KBps[$i],$cl_Memory_usage_Proc[$i],";
      $one_update    .= "$cl_Power_cup_Watt[$i],$cl_Power_usage_Watt[$i] ";
      $update_string .= "$one_update";
    }    # do not take $cl_Power_energy_usage_Joule[$i]
    elsif ( $entity_type eq $et_ResourcePool ) {
      $one_update = "$cl_CPU_usage_MHz[$i],";
      $one_update    .= "$cl_Memory_shared_KB[$i],$cl_Memory_zero_KB[$i],";
      $one_update    .= "$cl_Memory_baloon_KB[$i],$cl_Memory_consumed_KB[$i],$cl_Memory_overhead_KB[$i],";
      $one_update    .= "$cl_Memory_active_KB[$i],$cl_Memory_granted_KB[$i],$cl_Memory_compressed_KB[$i],";
      $one_update    .= "$cl_Memory_swap_KB[$i],$cl_Memory_compression_KBps[$i],";
      $one_update    .= "$cl_Memory_decompress_KBps[$i],$cl_cpu_limit[$i],$cl_cpu_reservation[$i],";
      $one_update    .= "$cl_mem_limit[$i],$cl_mem_reservation[$i],'U' ";                                     # U for added CPU proc
      $update_string .= "$one_update";
    } ## end elsif ( $entity_type eq $et_ResourcePool)
    elsif ( $entity_type eq $et_Datastore ) {

      # cus 30 minutes getting data for used/provisioned/capacity/(freeSpace) there are two data files
      # datastore.rrs - for mentioned above
      # datastore.rrt - for the rest ds's (regular update)
      # example for load time = 10 minutes
      # 1442308800,3697908121600,-1,-1,-1,559,2570,34,116 1442309100,3697908121600,1757208576,4664769308,5368446976,501,4631,36,121
      # 1442326200,2587793293312,U,U,U,43,2736,4,293 1442326500,2587793293312,U,U,U,156,2499,4,270

      $one_update = "$ds_Datastore_read_KBps[$i],$ds_Datastore_write_KBps[$i],";
      $one_update    .= "$ds_Datastore_numberReadAveraged[$i],$ds_Datastore_numberWriteAveraged[$i] ";
      $update_string .= "$one_update";

      $two_update = "$ds_Datastore_freeSpace_KB[$i],$ds_Datastore_used_KB[$i],";
      $two_update .= "$ds_Datastore_provision_KB[$i],$ds_Datastore_capacity_KB[$i] ";
      if ( ( index( $two_update, '-1,-1,-1' ) < 0 ) && ( index( $two_update, 'U,U,U' ) < 0 ) ) {
        $two_update_string .= "$vm_time_stamps[$i],$two_update";
      }
    } ## end elsif ( $entity_type eq $et_Datastore)
    else {
      error( "(F$server_count) unknown entity type $entity_type $fail_entity_type: $fail_entity_name " . __FILE__ . ":" . __LINE__ ) && exit 0;
    }
  } ## end for ( my $i = 0; $i < $samples_number...)
  print_hyperv_debug("string for RRD file update is:\n$update_string\n---------------------------------------------------\n\n") if $DEBUG == 2;

  my $input_vm_uuid = $vm_name_uuid{$entity};

  if ( $entity_type eq 'HostSystem' ) {
    $input_vm_uuid = 'pool';
  }
  if ( $entity_type eq $et_ClusterComputeResource ) {
    $input_vm_uuid = 'cluster';
    $type_sam      = "c";
  }
  if ( $entity_type eq $et_ResourcePool ) {
    $input_vm_uuid = $entity_uuid;
    $type_sam      = "c";
  }
  if ( $entity_type eq $et_Datastore ) {
    $input_vm_uuid = $entity_uuid;
    $type_sam      = "t";            # regular update
  }

  $SSH = "";

  my $managedname_save = $managedname;
  my $host_save        = $host;
  if ( $entity_type eq $et_VirtualMachine ) {
    $managedname_save = "hyperv_VMs";
    $host_save        = "";
  }

  if ( $entity_type ne $et_Datastore ) {
    my $res_update = LoadDataModuleHyperV::load_data( $managedname_save, $host_save, $wrkdir, \$update_string, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, \@lpar_trans, $last_file, $no_time, $SSH, $hmc_user, $input_vm_uuid, $entity_type, "(F$server_count)" );

    #          return
  }

  # here only datastore
  #        if (($entity_type eq $et_Datastore) && ($apiType_top =~ "HostAgent")) { # only for DA by ESXi 4
  #          $type_sam = "s";
  #          $update_string = $two_update_string;
  #          my $res_update = LoadDataModuleVMWare::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time*5,$SSH,$hmc_user,$input_vm_uuid,$entity_type,"(F$server_count)");
  #        }
  #        else {
  #          if (($entity_type eq $et_Datastore) && ($two_update_string ne "")) {
  #            # print "result update string $update_string 2 upd $two_update_string\n";
  #            my $res_update = LoadDataModuleVMWare::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user,$input_vm_uuid,$entity_type,"(F$server_count)");
  #            if ($res_update != $no_inserted ) { # go on only when success
  #              $type_sam = "s";  # irregular update - usually once in 30 mins
  #              # using very long heartbeat time !!!
  #              $update_string = $two_update_string;
  #              $res_update = LoadDataModuleVMWare::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time*5,$SSH,$hmc_user,$input_vm_uuid,$entity_type,"(F$server_count)");
  #            }
  #          }
  #        }

   print_hyperv_debug("2885 LoadDataModuleHyperV::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user,$input_vm_uuid,$entity_type,(F$server_count));\n") if $DEBUG == 2;

  if ( $entity_type eq 'HostSystem' ) { # go for fixed disks
    # CLASS: Win32_LogicalDisk
     # "Description","DeviceID","FreeSpace","Size"
     # "3 1/2 Inch Floppy Drive","A:",,
     # "Local Fixed Disk","C:","26691719168","42423283712"
     # "CD-ROM Disc","D:",,
     # "Local Fixed Disk","E:","10691956736","10737414144"
     # "Local Fixed Disk","F:","10555826176","10601099264"
     #
     #  other languages
     # "Místní pevný disk","C:","809388072960","955295649792"
     #
    #CLASS: Win32_PerfRawData_PerfDisk_LogicalDisk
     #"Name","Timestamp_PerfTime","Frequency_PerfTime","DiskReadBytesPersec","DiskWriteBytesPersec","DiskReadsPersec","DiskWritesPersec",
     #                                                 "AvgDisksecPerRead","AvgDisksecPerRead_Base","AvgDisksecPerWrite","AvgDisksecPerWrite_Base"
     #"E:","46731001377229","3117921","85475017216","118810357760","310433","1252304","1725749165","310433","804897803","1252304"
     #"HarddiskVolume3","46731001377229","3117921","0","0","0","0","0","0","0","0"
     #"C:","46731001377229","3117921","417952291328","715197595648","8757488","52200515","2497936692","8757488","499778004","52200515"
         # this is special case not in Win32_LogicalDisk
     # "C:\Users\jihampl","52696736742916","3117921","3332608","119705088","207","7148","16523598","207","2179586129","7148"
         # but in Win32_Volume
         # "C:\Users\jihampl\","3","21472735232","21327249408","User Disk"
     #
     #"_Total","46731001377229","3117921","503427308544","834007953408","9067921","53452819","4223685857","9067921","1304675807","53452819"
    # CLASS: Win32_Volume
     # "Name","DriveType","Capacity","FreeSpace","Label"
     # "\\?\Volume{e57611ae-0000-0000-0000-100000000000}\","3","524283904","177479680","System Reserved"
     # "E:\","3","10737414144","10691956736","FreeNAS 1st half"
     # "F:\","3","10601099264","10555826176","FreeNAS 2nd half"
     # "C:\","3","42423283712","26691719168",
     # "D:\","5",,,

    # my $ref_data = \%hyp_data;
    my $class = 'Win32_LogicalDisk';
    my $i = 0;
    print_hyperv_debug("3296 go for fixed disks\n") if $DEBUG == 2;
    while ( defined $$hyp_data_comp_act->{$class}[$i] ) {
      my $hyp_line = $$hyp_data_comp_act->{$class}[$i];
      $i++;
      next if $hyp_line !~ "\"Local Fixed Disk\"" && $hyp_line !~ "\"Místní pevný disk\"";
      print_hyperv_debug("3301 hyp2rrd.pl $hyp_line\n") if $DEBUG == 2;
      $hyp_line =~ s/\"//g;
      my @disk_arr = split( /,/, $hyp_line );
      $update_string = "$first_update_time,$disk_arr[-3],$disk_arr[-2]";
      $input_vm_uuid = "Local_Fixed_Disk_$disk_arr[-4]";
      $type_sam = "m";
      $last_file = "";
      my ($Timestamp_PerfTime,$Frequency_PerfTime,$DiskReadBytesPersec,$DiskWriteBytesPerse,$DiskReadsPersec,$DiskWritesPersec,$AvgDisksecPerRead,$AvgDisksecPerReadB,$AvgDisksecPerWrite,$AvgDisksecPerWriteB) = (0) x 10;
      if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_PerfDisk_LogicalDisk'} && exists $$hyp_data_comp_last->{'Win32_PerfRawData_PerfDisk_LogicalDisk'}) {
        $Timestamp_PerfTime  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','Timestamp_PerfTime', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $Timestamp_PerfTime  = "U" if ! defined $Timestamp_PerfTime or $Timestamp_PerfTime eq $UnDeF;
        ($Timestamp_PerfTime, undef) = split(",",$Timestamp_PerfTime); # in case more values -> take only the 1st
        $Frequency_PerfTime  = get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','Frequency_PerfTime', $hyp_data_comp_act, $disk_arr[-4]);
        $Frequency_PerfTime  = "U" if ! defined $Frequency_PerfTime or $Frequency_PerfTime eq $UnDeF;
        ($Frequency_PerfTime, undef) = split(",",$Frequency_PerfTime); # in case more values -> take only the 1st
        $DiskReadBytesPersec = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','DiskReadBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $DiskReadBytesPersec = "U" if ! defined $DiskReadBytesPersec or $DiskReadBytesPersec eq $UnDeF;
        ($DiskReadBytesPersec, undef) = split(",",$DiskReadBytesPersec); # in case more values -> take only the 1st
        $DiskWriteBytesPerse = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','DiskWriteBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $DiskWriteBytesPerse = "U" if ! defined $DiskWriteBytesPerse or $DiskWriteBytesPerse eq $UnDeF;
        ($DiskWriteBytesPerse, undef) = split(",",$DiskWriteBytesPerse); # in case more values -> take only the 1st
        $DiskReadsPersec     = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','DiskReadsPersec', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $DiskReadsPersec     = "U" if ! defined $DiskReadsPersec or $DiskReadsPersec eq $UnDeF;
        ($DiskReadsPersec, undef) = split(",",$DiskReadsPersec); # in case more values -> take only the 1st
        $DiskWritesPersec    = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','DiskWritesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $DiskWritesPersec    = "U" if ! defined $DiskWritesPersec or $DiskWritesPersec eq $UnDeF;
        ($DiskWritesPersec, undef) = split(",",$DiskWritesPersec); # in case more values -> take only the 1st
        $AvgDisksecPerRead   = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','AvgDisksecPerRead', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $AvgDisksecPerRead   = "U" if ! defined $AvgDisksecPerRead or $AvgDisksecPerRead eq $UnDeF;
        ($AvgDisksecPerRead, undef) = split(",",$AvgDisksecPerRead); # in case more values -> take only the 1st
        $AvgDisksecPerReadB  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','AvgDisksecPerRead_Base', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $AvgDisksecPerReadB  = "U" if ! defined $AvgDisksecPerReadB or $AvgDisksecPerReadB eq $UnDeF;
        ($AvgDisksecPerReadB, undef) = split(",",$AvgDisksecPerReadB); # in case more values -> take only the 1st
        $AvgDisksecPerWrite  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','AvgDisksecPerWrite', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $AvgDisksecPerWrite  = "U" if ! defined $AvgDisksecPerWrite or $AvgDisksecPerWrite eq $UnDeF;
        ($AvgDisksecPerWrite, undef) = split(",",$AvgDisksecPerWrite); # in case more values -> take only the 1st
        $AvgDisksecPerWriteB = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_LogicalDisk','AvgDisksecPerWrite_Base', $hyp_data_comp_act, $hyp_data_comp_last,$disk_arr[-4]);
        $AvgDisksecPerWriteB = "U" if ! defined $AvgDisksecPerWriteB or $AvgDisksecPerWriteB eq $UnDeF;
        ($AvgDisksecPerWriteB, undef) = split(",",$AvgDisksecPerWriteB); # in case more values -> take only the 1st
      }
      print_hyperv_debug("3331 $Timestamp_PerfTime,$Frequency_PerfTime,$DiskReadBytesPersec $DiskWriteBytesPerse $DiskReadsPersec $DiskWritesPersec $AvgDisksecPerRead $AvgDisksecPerReadB $AvgDisksecPerWrite $AvgDisksecPerWriteB\n") if $DEBUG == 2;
      $update_string .= ",$Timestamp_PerfTime,$Frequency_PerfTime,$DiskReadBytesPersec,$DiskWriteBytesPerse,$DiskReadsPersec,$DiskWritesPersec,$AvgDisksecPerRead,$AvgDisksecPerReadB,$AvgDisksecPerWrite,$AvgDisksecPerWriteB";
      print_hyperv_debug("3332 LoadDataModuleHyperV::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user,$input_vm_uuid,$entity_type,(F$server_count)); \$update_string $update_string\n") if $DEBUG == 2;
      my $res_update = LoadDataModuleHyperV::load_data( $managedname_save, $host_save, $wrkdir, \$update_string, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, \@lpar_trans, $last_file, $no_time, $SSH, $hmc_user, $input_vm_uuid, $entity_type, "(F$server_count)" );
    }

    # get Cluster Shared Storage if exists, it is in nodes, in cluster is same data as in coordinating node, do not take this data
     # Get-wmiobject -Query select Name,DriveType,Capacity,FreeSpace,Label from Win32_Volume -cn CN=HVNODE02,CN=Computers,DC=ad,DC=xorux,DC=com.Name -EA silentlyContinue |Select-Object Name DriveType Capacity FreeSpace Label
     # CLASS: Win32_Volume
     # "Name","DriveType","Capacity","FreeSpace","Label","DeviceID"
     # "\\?\Volume{22ef4fe2-0000-0000-0000-100000000000}\","3","524283904","162037760","System Reserved","\\?\Volume{22ef4fe2-0000-0000-0000-100000000000}\"
     # "C:\","3","42423283712","34396155904",,"\\?\Volume{22ef4fe2-0000-0000-0000-501f00000000}\"
     # "C:\ClusterStorage\Volume2\","3","32209104896","32110993408","CLVOL02","\\?\Volume{b994eb5b-844f-45a7-86ea-6116e3175653}\"
     # "C:\ClusterStorage\Volume1\","3","69790068736","63375728640","CLVOL01","\\?\Volume{1d2ac989-641d-4fa0-9ee0-6a391c8be0da}\"
     # "D:\","5",,,,"\\?\Volume{770da79b-8ac9-11e8-9d58-806e6f6e6963}\"

    # nonitoring data for CSV is in _PhysicalDisk under Name "1" resp. "2" and so on
     # CLASS: Win32_PerfRawData_PerfDisk_PhysicalDisk
     # "Name","Timestamp_PerfTime","Frequency_PerfTime","DiskReadBytesPersec","DiskWriteBytesPersec","DiskReadsPersec","DiskWritesPersec",
     #                                                  "AvgDisksecPerRead","AvgDisksecPerRead_Base","AvgDisksecPerWrite","AvgDisksecPerWrite_Base"
     # "0 C:","11479970095690","2994772","9944602112","44117474816","203855","3165451","193855528","203855","1699987602","3165451"
     # "1","11479970095690","2994772","96223707648","107430872064","2949239","371297","3609052217","2949239","3791159777","371297"
     # "2","11479970095690","2994772","49052672","68931584","1391","901","2442905","1391","2112507","901"
     # "_Total","11479970095690","2994772","106217362432","151617278464","3154485","3537649","3805350650","3154485","1198292590","3537649"

    # $DEBUG = 2;
    my %name_hash = ();
    $class = 'Win32_Volume';
    $i = 0;
    my $freespace = "\"FreeSpace\"";
    my $capacity = "\"Capacity\"";
    my $name = "\"Name\"";
    my $deviceid = "\"DeviceID\"";
    print_hyperv_debug("3345 go for cluster shared volumes\n") if $DEBUG == 2;
    while ( defined $$hyp_data_comp_act->{$class}[$i] ) {
      my $hyp_line = $$hyp_data_comp_act->{$class}[$i];
      if ($hyp_line =~ /Name/ && $hyp_line =~ /DeviceID/) {
        # this is the 1st line "Name","DriveType","Capacity","FreeSpace","Label","DeviceID", prepare hash with indexes
        my @name_arr = split(",",$hyp_line);
        my $name_ind = 0;
        while ( defined $name_arr[ $name_ind ]) {
          $name_hash{$name_arr[ $name_ind ]} = $name_ind;
          $name_ind++;
        }
      }
      # print "3717 from class $class ---------------------\n";
      # print Dumper (%name_hash);
      $i++;
      next if $hyp_line !~ "ClusterStorage";
      next if !keys %name_hash; # older agent does not send DeviceID

      print_hyperv_debug("3350 hyp2rrd.pl $hyp_line\n") if $DEBUG == 2;
      $hyp_line =~ s/\"//g;
      $hyp_line =~ s/\\//g;
      my @disk_arr = split( /,/, $hyp_line );
      #$update_string = "$first_update_time,$disk_arr[-4],$disk_arr[-5]";
      $update_string = "$first_update_time,$disk_arr[$name_hash{$freespace}],$disk_arr[$name_hash{$capacity}]";
      # $input_vm_uuid = "Local_Fixed_Disk_$disk_arr[-5]";
      $input_vm_uuid = "$disk_arr[$name_hash{$name}]";

      # keeps all cluster storages
      $all_clusterstorage{$h_name}{$disk_arr[$name_hash{$deviceid}]} = "ahojte"; #computer name
      (undef, $input_vm_uuid) = split ( "ClusterStorage", $input_vm_uuid);
      if ($input_vm_uuid eq "") {
        error( "can't find ClusterStorage name in line $hyp_line " . __FILE__ . ":" . __LINE__ ) && next;
      }
      my $csv_number = $input_vm_uuid;
      $input_vm_uuid = "Cluster_Storage_$input_vm_uuid";
      $type_sam = "m";
      $last_file = "";
      $csv_number =~ s/Volume//;
      my ($Timestamp_PerfTime,$Frequency_PerfTime,$DiskReadBytesPersec,$DiskWriteBytesPerse,$DiskReadsPersec,$DiskWritesPersec,$AvgDisksecPerRead,$AvgDisksecPerReadB,$AvgDisksecPerWrite,$AvgDisksecPerWriteB) = (0) x 10;
      if (isdigit($csv_number) && $csv_number ne "") {
        my $item_contains = "ITEM=Name=\"$csv_number\"";

        # work out data for cluster shared volumes
        if (exists $$hyp_data_comp_act->{'Win32_PerfRawData_PerfDisk_PhysicalDisk'} && exists $$hyp_data_comp_last->{'Win32_PerfRawData_PerfDisk_PhysicalDisk'}) {
          $Timestamp_PerfTime  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','Timestamp_PerfTime', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $Timestamp_PerfTime  = "U" if ! defined $Timestamp_PerfTime or $Timestamp_PerfTime eq $UnDeF;
          $Frequency_PerfTime  = get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','Frequency_PerfTime', $hyp_data_comp_act, $item_contains);
          $Frequency_PerfTime  = "U" if ! defined $Frequency_PerfTime or $Frequency_PerfTime eq $UnDeF;
          $DiskReadBytesPersec = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskReadBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $DiskReadBytesPersec = "U" if ! defined $DiskReadBytesPersec or $DiskReadBytesPersec eq $UnDeF;
          $DiskWriteBytesPerse = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskWriteBytesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $DiskWriteBytesPerse = "U" if ! defined $DiskWriteBytesPerse or $DiskWriteBytesPerse eq $UnDeF;
          $DiskReadsPersec     = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskReadsPersec', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $DiskReadsPersec     = "U" if ! defined $DiskReadsPersec or $DiskReadsPersec eq $UnDeF;
          $DiskWritesPersec    = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','DiskWritesPersec', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $DiskWritesPersec    = "U" if ! defined $DiskWritesPersec or $DiskWritesPersec eq $UnDeF;
          $AvgDisksecPerRead   = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','AvgDisksecPerRead', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $AvgDisksecPerRead   = "U" if ! defined $AvgDisksecPerRead or $AvgDisksecPerRead eq $UnDeF;
          $AvgDisksecPerReadB  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','AvgDisksecPerRead_Base', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $AvgDisksecPerReadB  = "U" if ! defined $AvgDisksecPerReadB or $AvgDisksecPerReadB eq $UnDeF;
          $AvgDisksecPerWrite  = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','AvgDisksecPerWrite', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $AvgDisksecPerWrite  = "U" if ! defined $AvgDisksecPerWrite or $AvgDisksecPerWrite eq $UnDeF;
          $AvgDisksecPerWriteB = delta_get_hyp_val('Win32_PerfRawData_PerfDisk_PhysicalDisk','AvgDisksecPerWrite_Base', $hyp_data_comp_act, $hyp_data_comp_last,$item_contains);
          $AvgDisksecPerWriteB = "U" if ! defined $AvgDisksecPerWriteB or $AvgDisksecPerWriteB eq $UnDeF;
        }
      }
      else {
        print "3387 not number for Cluster Shared Volume ,$csv_number,\n";
      }
      print_hyperv_debug("3404 $DiskReadBytesPersec $DiskWriteBytesPerse $DiskReadsPersec $DiskWritesPersec $AvgDisksecPerRead $AvgDisksecPerReadB $AvgDisksecPerWrite $AvgDisksecPerWriteB\n") if $DEBUG == 2;
      $update_string .= ",$Timestamp_PerfTime,$Frequency_PerfTime,$DiskReadBytesPersec,$DiskWriteBytesPerse,$DiskReadsPersec,$DiskWritesPersec,$AvgDisksecPerRead,$AvgDisksecPerReadB,$AvgDisksecPerWrite,$AvgDisksecPerWriteB";
      print_hyperv_debug("3406 LoadDataModuleHyperV::load_data ($managedname_save,$host_save,$wrkdir,\$update_string,$type_sam,$act_time,$HMC,$IVM,$SDMC,$step,$DEBUG,\@lpar_trans,$last_file,$no_time,$SSH,$hmc_user,$input_vm_uuid,$entity_type,(F$server_count));$update_string\n") if $DEBUG == 2;
      my $res_update = LoadDataModuleHyperV::load_data( $managedname_save, $host_save, $wrkdir, \$update_string, $type_sam, $act_time, $HMC, $IVM, $SDMC, $step, $DEBUG, \@lpar_trans, $last_file, $no_time, $SSH, $hmc_user, $input_vm_uuid, $entity_type, "(F$server_count)" );
    }

    print_hyperv_debug("3410 finito end of cluster shared volumes\n") if $DEBUG == 2;
    #my @host_disks = ();
  }
  print_hyperv_debug("3413 finito tuti of Host system\n") if $DEBUG == 2;
} ## end sub get_entity_perf

sub cluster_active_VMs {
  my $wrkdir       = shift;
  my $cluster_path = shift;
  my $VMs_list     = shift;    # external arr to fill

  my $hosts_in_cluster_file = "$wrkdir/$cluster_path/hosts_in_cluster";
  return if !-f $hosts_in_cluster_file;

  open FH, "<$hosts_in_cluster_file" or error( "can't open $hosts_in_cluster_file: $!" . __FILE__ . ":" . __LINE__ ) && return;
  my @hosts_in_cluster = <FH>;
  close FH;

  foreach (@hosts_in_cluster) {
    my $host = $_;
    chomp $host;
    ( my $server, undef ) = split( "XORUX", $host );
    ( my $vm_hosting ) = glob "$wrkdir/$server/*/VM_hosting.vmh";

    # print "\$host $host \$server $server \$vm_hosting $vm_hosting\n";
    open FH, "<$vm_hosting" or error( "can't open $vm_hosting: $!" . __FILE__ . ":" . __LINE__ ) && next;
    foreach (<FH>) {
      my $line = $_;
      chomp $line;
      if ( $line =~ /start=\d\d\d\d\d\d\d\d\d\d$/ ) {
        ( $line, undef ) = split( ":", $line );
        push @$VMs_list, "$wrkdir/vmware_VMs/$line" . ".rrm";
      }
    } ## end foreach (<FH>)
    close FH;

    # print "\@$VMs_list @$VMs_list\n";
  } ## end foreach (@hosts_in_cluster)
  return 0;
} ## end sub cluster_active_VMs

sub get_last_date_range {
  my ( $pef_time_sec, $entity_type ) = @_;

  return ( "vcera", "dnes", 3600 );

  #return ($st_date,$end_date,$pef_time_sec);

  # ServiceInstance current server time looks like e.g. 2015-04-16T11:12:28.296812Z
  #  my $server_time  = $service_instance->CurrentTime;
  my $server_time   = $command_date;
  my $apiType       = $service_instance->content->about->apiType;
  my $localeBuild   = $service_instance->content->about->localeBuild;
  my $localeVersion = $service_instance->content->about->localeVersion;
  my $fullName      = $service_instance->content->about->fullName;
  my $instanceUuid  = $service_instance->content->about->instanceUuid;
  print "apiType        : $apiType $fullName";
  if ( defined $localeBuild )   { print " locale $localeBuild" }
  if ( defined $localeVersion ) { print " localeversion $localeVersion" }
  if ( defined $instanceUuid )  { print " instanceUuid $instanceUuid" }
  print " (F$server_count)\n";

  if ( $apiType eq "HostAgent" ) {
    if ( $pef_time_sec > 3600 ) {
      $pef_time_sec = 3600 * 18;
    }
  }
  else {
    if ( $apiType ne "VirtualCenter" ) {
      error( "(F$server_count) unknown apiType $apiType " . $service_instance->about->fullName . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    if ( $pef_time_sec > 3600 * 18 ) {
      $pef_time_sec = 3600 * 18;
    }
  } ## end else [ if ( $apiType eq "HostAgent")]

  my $unix_time = str2time($server_time);

  if ( $entity_type eq $et_ResourcePool || $entity_type eq $et_Datastore || $entity_type eq $et_ClusterComputeResource ) {

    # 23/12/2015 try round low times to dividable 300
    $unix_time    = $unix_time -    ( $unix_time % 300 );
    $pef_time_sec = $pef_time_sec - ( $pef_time_sec % 300 );
  } ## end if ( $entity_type eq $et_ResourcePool...)

  #   if ($pef_time_sec > 3600) {
  #     $pef_time_sec = 3500;
  #   }

  # move all 5 minutes back so vCenter has time to get values, does not help
  # $unix_time -= 300;

  my ( $gsec, $gmin, $ghour, $gday, $gmonth, $gyear, $gwday, $gyday, $gisdst ) = gmtime($unix_time);
  $gyear += 1900;
  $gmonth++;
  $gmonth = "0" . $gmonth if $gmonth < 10;
  $gday   = "0" . $gday   if $gday < 10;
  $ghour  = "0" . $ghour  if $ghour < 10;
  $gmin   = "0" . $gmin   if $gmin < 10;
  $gsec   = "0" . $gsec   if $gsec < 10;

  my $end_date = $gyear . "-" . $gmonth . "-" . $gday . "T" . $ghour . ":" . $gmin . ":" . $gsec;

  my $end_time = timelocal( $gsec, $gmin, $ghour, $gday, $gmonth - 1, $gyear - 1900 );
  my $st_time = $end_time - ( $pef_time_sec - 1 + 1 );    #+ 1;  # not now 23/12/2015
                                                          # print "\$end_time $end_time \$st_time $st_time\n";

  my ( $sec, $min, $hour, $day, $month, $year, $wday, $yday, $isdst ) = localtime($st_time);
  $year += 1900;
  $month++;
  $month = "0" . $month if $month < 10;
  $day   = "0" . $day   if $day < 10;
  $hour  = "0" . $hour  if $hour < 10;
  $min   = "0" . $min   if $min < 10;
  $sec   = "0" . $sec   if $sec < 10;

  my $st_date = $year . "-" . $month . "-" . $day . "T" . $hour . ":" . $min . ":" . $sec;

  #print "\$end_date $end_date \$st_date $st_date\n";

  return ( $st_date, $end_date, $pef_time_sec );
} ## end sub get_last_date_range

sub prepare_vm_metric {
  my $entity_type = shift;
  my $perf_metric = shift;

  my $perf_values = $perf_metric->value;

  # print "\$perf_values $perf_values\n";
  my $counter = $all_counters->{ $perf_metric->id->counterId };

  if ( !defined $counter ) {
    print "asking ";
    $counter = $perf_metric->id->counterId;
    $counter = $perfmgr_view->QueryPerfCounter( counterId => $counter );
    print Dumper($counter);

    $counter = $counter->[0];
  } ## end if ( !defined $counter)

  # print Dumper("2831", $counter);
  my $group_info = $counter->groupInfo->label;
  my $nameInfo   = $counter->nameInfo->key;
  my $unitInfo   = $counter->unitInfo->label;
  my $rollupType = $counter->rollupType->val;

  if ( ( !defined $group_info ) || ( !defined $nameInfo ) || ( !defined $unitInfo ) ) {
    print "counter items \$group_info or \$nameInfo or \$unitInfo not defined (F$server_count)\n";
    return 0;
  }

  if ( !defined $rollupType ) {
    print "counter $group_info $nameInfo $unitInfo : not defined rollupType (F$server_count)\n";
    return 0;
  }

  # for the case of all real counters
  if ( $entity_type ne $et_Datastore ) {
    if ( $rollupType eq 'summation' && ( $group_info eq 'CPU' || $group_info eq 'disk' ) ) {

      # let it go
    }
    elsif ( $rollupType ne 'average' ) { return 0 }
  } ## end if ( $entity_type ne $et_Datastore)

  #  print "counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_Datastore;
  #       print "2424 counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_VirtualMachine;
  #       print "2425 counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_HostSystem;
  #       print "2735 counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_Datastore;
  #       print "2427 counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_ResourcePool;
  #       print "2428 counter: " . $counter->key . " $group_info $nameInfo $unitInfo $rollupType\n" if $entity_type eq $et_ClusterComputeResource;

  # my $factor = 1;   #only used for percent

  my $rep = $samples_number - 1;

  # test if all numbers are OK
  # sometimes, some values are not defined   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,-1,-1,-1,-1,-1
  # instead -1 prepare U
  # if all items are U then print err, do not update
  my $res = ( $perf_values =~ /([0-9]+,){$rep}[0-9]+/ );

  # print "\$rep $rep \$res $res \n";
  if ( $res != 1 ) {
    my $info_value = "length of value=" . length($perf_values) . " " . $perf_values;
    $perf_values =~ s/\-1$/U/;
    $perf_values =~ s/\-1,/U,/g;
    $res = ( $perf_values =~ /(U,){$rep}U/ );

    #error ( "probably -1 ".$perf_values."\n");
    my $res1 = ( $perf_values =~ /([0-9]+,|U,){$rep}[0-9]+|U/ );
    if ( $res == 1 || $res1 != 1 ) {
      error( "(F$server_count) error: not numbers for counter: $group_info $nameInfo $unitInfo $info_value $fail_entity_type:$fail_entity_name " . __FILE__ . ":" . __LINE__ ) && return 0;
    }
  } ## end if ( $res != 1 )

  # print Dumper($counter);

  my $test_item = $group_info . ":" . $nameInfo . ":" . $unitInfo;

  if ( $entity_type eq $et_VirtualMachine ) {

    # print "......... $test_item\n";
    my ($index) = grep { $counter_vm_eng[$_] eq $test_item } 0 .. $#counter_vm_eng;    # is it needed counter ?

    if ( !defined $index ) {                                                           # try version 6
      ($index) = grep { $counter_vm_eng6[$_] eq $test_item } 0 .. $#counter_vm_eng6;
    }
    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_vm_ger1[$_] eq $test_item } 0 .. $#counter_vm_ger1;
    }

    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_vm_ger2[$_] eq $test_item } 0 .. $#counter_vm_ger2;
    }

    if ( defined $index ) {

      #         if ($index == 3) { # as if 'Disk:write:KBps' not present
      #			return;
      #		 }

      # not here - must be in sub init_perf_counter_info
      # special case
      # 'Datastore:numberReadAveraged:Number','Datastore:numberWriteAveraged:Number'
      #print "2893 \$test_item $test_item\n";
      #if ($test_item eq 'Datastore:numberReadAveraged:Number') {
      #   $vm_counterId_readAveraged = "$counter->key";
      #   print "2895 vmw2rrd.pl \$vm_counterId_readAveraged $vm_counterId_readAveraged for VM $vm_uuid_active\n";
      #}
      #if ($test_item eq 'Datastore:numberWriteAveraged:Number') {
      #   $vm_counterId_writeAveraged = "$counter->key";
      #}
      my $po_po = $$pointer_arr[$index];
      @$po_po = split( ',', $perf_values );
      $counter_presence[$index] = 1;
      return 1;
    } ## end if ( defined $index )
  } ## end if ( $entity_type eq $et_VirtualMachine)
  if ( $entity_type eq $et_HostSystem ) {

    # print "......... $test_item\n";
    my ($index) = grep { $counter_hs_eng[$_] eq $test_item } 0 .. $#counter_hs_eng;    # is it needed counter ?

    if ( !defined $index ) {                                                           # try version 6
      ($index) = grep { $counter_hs_eng6[$_] eq $test_item } 0 .. $#counter_hs_eng6;
    }
    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_hs_ger1[$_] eq $test_item } 0 .. $#counter_hs_ger1;
    }

    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_hs_ger2[$_] eq $test_item } 0 .. $#counter_hs_ger2;
    }

    if ( defined $index ) {

      #         if ($index == 3) { # as if 'Disk:write:KBps' not present
      #			return;
      #		 }
      #         if (($entity_type eq "HostSystem") && ($index == 10)) { # do not write balloon for HostSystem
      #		   return 1;
      #	     }

      my $po_po = $$pointer_arr[$index];
      @$po_po = split( ',', $perf_values );
      $counter_presence[$index] = 1;
      return 1;
    } ## end if ( defined $index )
  } ## end if ( $entity_type eq $et_HostSystem)

  if ( $entity_type eq $et_ClusterComputeResource ) {
    my ($index) = grep { $counter_cl_eng[$_] eq $test_item } 0 .. $#counter_cl_eng;    # is it needed counter ?

    if ( !defined $index ) {                                                           # try version 6
      ($index) = grep { $counter_cl_eng6[$_] eq $test_item } 0 .. $#counter_cl_eng6;
    }
    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_cl_ger[$_] eq $test_item } 0 .. $#counter_cl_ger;
    }
    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_cl_ger6[$_] eq $test_item } 0 .. $#counter_cl_ger6;
    }

    # print "cluster counter\n";
    if ( defined $index ) {
      my $po_po = $$pointer_arr[$index];
      @$po_po = split( ',', $perf_values );

      # want to print individual counter ?
      # if ($test_item eq "Memory:totalmb:MB") { print Dumper($perf_values); }
      $counter_presence[$index] = 1;
      return 1;
    } ## end if ( defined $index )
  } ## end if ( $entity_type eq $et_ClusterComputeResource)
  if ( $entity_type eq $et_ResourcePool ) {
    my ($index) = grep { $counter_rp_eng[$_] eq $test_item } 0 .. $#counter_rp_eng;    # is it needed counter ?

    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_rp_ger[$_] eq $test_item } 0 .. $#counter_rp_ger;
    }

    # print "resourcepool counter\n";
    if ( defined $index ) {
      my $po_po = $$pointer_arr[$index];
      @$po_po = split( ',', $perf_values );

      # want to print individual counter ?
      # if ($test_item eq "Memory:totalmb:MB") { print Dumper($perf_values); }
      $counter_presence[$index] = 1;
      return 1;
    } ## end if ( defined $index )
  } ## end if ( $entity_type eq $et_ResourcePool)
  if ( $entity_type eq $et_Datastore ) {
    my ($index) = grep { $counter_ds_eng[$_] eq $test_item } 0 .. $#counter_ds_eng;    # is it needed counter ?

    if ( !defined $index ) {                                                           # try version 6
      ($index) = grep { $counter_ds_eng6[$_] eq $test_item } 0 .. $#counter_ds_eng6;
    }
    if ( !defined $index ) {                                                           # try other languages
      ($index) = grep { $counter_ds_ger[$_] eq $test_item } 0 .. $#counter_ds_ger;
    }

    # print "datastore counter\n";
    if ( defined $index ) {
      my $po_po = $$pointer_arr[$index];
      @$po_po = split( ',', $perf_values );

      # want to print individual counter ?
      # if ($test_item eq "Memory:totalmb:MB") { print Dumper($perf_values); }
      $counter_presence[$index] = 1;
      return 1;
    } ## end if ( defined $index )
  } ## end if ( $entity_type eq $et_Datastore)

  error( "(F$server_count) unknown counter: ,$group_info, ,$nameInfo, ,$unitInfo, " . __FILE__ . ":" . __LINE__ ) && return 0;
  return 0;
} ## end sub prepare_vm_metric

sub init_perf_counter_info {
  my $entity_type = shift;

  %{$now_counters} = ();    #%{$hash} = ()
  %{$all_counters} = ();

  my $perfCounterInfo = $perfmgr_view->perfCounter;
  my $ci              = 0;                            # how many counters chosen

  # prepare test array of counters for VM
  my @counter_arr = ( 'cpu:usagemhz', 'disk:usage', 'disk:read', 'disk:write', 'net:usage', 'net:received', 'net:transmitted', 'mem:active', 'mem:granted', 'mem:swapinRate', 'mem:vmmemctl', 'mem:swapoutRate', 'mem:decompressionRate', 'mem:compressionRate', 'cpu:usage', 'cpu:ready' );

  #                      'datastore:numberReadAveraged','datastore:numberWriteAveraged'
  #                     );
  #    'disk:numberRead','disk:numberWrite'
  #   );
  $selected_counters = 16;

  # want to print info $host ?
  # print "init_perf_counter_info \$host $host\n";

  # prepare test array of counters for HS
  if ( $entity_type eq $et_HostSystem ) {
    @counter_arr = ( 'cpu:usagemhz', 'disk:usage', 'disk:read', 'disk:write', 'net:usage', 'net:received', 'net:transmitted', 'mem:active', 'mem:granted', 'mem:swapinRate', 'mem:vmmemctl', 'mem:swapoutRate', 'mem:decompressionRate', 'mem:compressionRate', 'cpu:usage', 'cpu:ready' );
    $selected_counters = 16;
  }

  if ( $entity_type eq $et_ClusterComputeResource ) {

    # do not take 'power:energy'
    # vmmemctl = balloon

    # since 4.92 only 18 counters
    @counter_arr = ( 'cpu:usagemhz', 'cpu:usage', 'cpu:reservedCapacity', 'cpu:totalmhz', 'mem:totalmb', 'mem:shared', 'mem:zero', 'mem:vmmemctl', 'mem:consumed', 'mem:active', 'mem:granted', 'mem:compressed', 'mem:reservedCapacity', 'mem:swapused', 'mem:compressionRate', 'mem:decompressionRate', 'power:powerCap', 'power:power' );
    $selected_counters = 18;

    # @counter_arr = ('cpu:usagemhz','cpu:usage','cpu:reservedCapacity','cpu:totalmhz',
    #               'clusterServices:effectivecpu','clusterServices:effectivemem',
    #               'mem:totalmb','mem:shared','mem:zero','mem:vmmemctl','mem:consumed','mem:overhead',
    #               'mem:active','mem:granted','mem:compressed','mem:reservedCapacity',
    #               'mem:swapused','mem:compressionRate','mem:decompressionRate','mem:usage',
    #		           'power:powerCap','power:power'
    #                );
    #$selected_counters = 22;

    # if ( $fullName_top =~ 'vCenter Server 6') {
    #   @counter_arr = ('cpu:usagemhz','cpu:usage','cpu:reservedCapacity',
    #                'mem:shared','mem:zero','mem:consumed','mem:overhead',
    #                'mem:active','mem:granted','mem:compressed','mem:reservedCapacity',
    # 			          'mem:decompressionRate','mem:compressionRate','mem:vmmemctl',
    # 		            'mem:usage',
    #			          'power:power','power:powerCap'
    #                );
    #  $selected_counters = 17;
    # 'cpu:totalmhz',
    # 'clusterServices:effectivecpu','clusterServices:effectivemem',
    # 'mem:totalmb',
    # 'mem:swapused'
    # }
  } ## end if ( $entity_type eq $et_ClusterComputeResource)

  if ( $entity_type eq $et_ResourcePool ) {

    # do not take 'power:energy'
    # vmmemctl = balloon
    @counter_arr = ( 'cpu:usagemhz', 'mem:shared', 'mem:zero', 'mem:vmmemctl', 'mem:consumed', 'mem:overhead', 'mem:active', 'mem:granted', 'mem:compressed', 'mem:swapped', 'mem:compressionRate', 'mem:decompressionRate' );
    $selected_counters = 12;

    #if ( $fullName_top =~ 'vCenter Server 6') {
    #  @counter_arr = (
    #                 'mem:shared','mem:zero','mem:consumed','mem:overhead',
    #                 'mem:active','mem:granted','mem:compressed',
    #                 'mem:swapped','mem:decompressionRate','mem:compressionRate','mem:vmmemctl'
    #                );
    #  $selected_counters = 11;
    # }
  } ## end if ( $entity_type eq $et_ResourcePool)

  if ( $entity_type eq $et_Datastore ) {

    #@counter_arr = ('disk:capacity','disk:provisioned','disk:used',
    #                'datastore:read','datastore:write',
    #                'datastore:numberReadAveraged','datastore:numberWriteAveraged'
    #               );
    # 'virtualDisk:totalReadLatency','virtualDisk:totalWriteLatency'
    #$selected_counters = 7;
    @counter_arr = ( 'datastore:read', 'datastore:write', 'datastore:numberReadAveraged', 'datastore:numberWriteAveraged' );

    # 'virtualDisk:totalReadLatency','virtualDisk:totalWriteLatency'
    $selected_counters = 4;

    if ( $ds_type eq "NFS" ) {    # NFS does not provide Averaged
      @counter_arr = ( 'datastore:read', 'datastore:write', );
      $selected_counters = 2;
    }
  } ## end if ( $entity_type eq $et_Datastore)

  my @signal_arr = (0) x $selected_counters;

  @counter_arr_levels = ('empty') x $selected_counters;

  foreach (@$perfCounterInfo) {

    # print $i++.","; # there exists about 7 hundred counters
    #if ($entity_type eq $et_Datastore) {
    # if ($entity_type eq $et_VirtualMachine) { # this is not possible take next line
    if ( $entity_type eq '' ) {
      print Dumper( "2998", $_ );
    }

    #if ($entity_type eq $et_Datastore) {
    # no test yet
    #}
    #else {
    if ( $_->rollupType->val =~ /summation/ && ( ( $_->groupInfo->key =~ /cpu/ && $_->nameInfo->key =~ /ready/ ) || ( $_->groupInfo->key =~ /disk/ && $_->nameInfo->key =~ /number/ ) ) ) {

      # print "--------- cpu or disk summation\n";
      # let it go
    }
    elsif ( $_->rollupType->val !~ /average/ ) {
      next;
    }

    # }
    my $key        = $_->key;
    my $group_info = $_->groupInfo;
    my $name_info  = $_->nameInfo;
    my $unitInfo   = $_->unitInfo;

    my $test_item = $group_info->key . ":" . $name_info->key;

    #     if ($test_item eq 'cpu:usagemhz') {  # debug test as if counter not presented
    #         xerror ("(F$server_count) --- counter cpu:usagemhz ---  ".__FILE__.":".__LINE__);
    #        print " --- counter cpu:usagemhz ---\n";
    #        print Dumper ($_);
    #     }

    #      if ($test_item eq 'net:received') {  # debug test as if counter not presented
    #       next;
    #       }

    #        print "2959 \$test_item ,$test_item, " . $unitInfo->key ." \@counter_arr @counter_arr\n" if $entity_type eq $et_ClusterComputeResource;
    #       print "\$test_item ,$test_item, " . $unitInfo->key ." \@counter_arr @counter_arr\n" if $entity_type eq $et_Datastore;
    #       print "\$test_item ,$test_item, " . $unitInfo->key ." \@counter_arr @counter_arr\n" if $entity_type eq $et_VirtualMachine; # do not work !
    my ($index) = grep { $counter_arr[$_] eq $test_item } 0 .. $#counter_arr;    # is it needed counter ?
    if ( !defined $index ) {

      # print "not defined for $test_item\n";
      next;
    }

    #     if ($group_info->key eq 'mem' && $unitInfo->key eq 'percent') {
    # if ($group_info->key eq 'cpu' && $unitInfo->key eq 'percent') { # test for counted % for CPU_usage_percent
    #    next;
    # }

    # print "3154 \$test_item $test_item\n";
    if ( $test_item eq 'datastore:numberReadAveraged' ) {
      $vm_counterId_readAveraged = $key;

      # print "3157 vmw2rrd.pl \$vm_counterId_readAveraged $vm_counterId_readAveraged for VM ,$vm_uuid_active,\n";
    }
    if ( $test_item eq 'datastore:numberWriteAveraged' ) {
      $vm_counterId_writeAveraged = $key;
    }

    $signal_arr[$index]++;
    $now_counters->{$key} = $_;
    $all_counters->{$key} = $_;
    my $level_info = ",,";
    if ( defined $_->level ) { $level_info = $_->level }
    my $perdevicelevel_info = ",,";
    if ( defined $_->perDeviceLevel ) { $perdevicelevel_info = $_->perDeviceLevel }
    $counter_arr_levels[$index] = "$level_info $perdevicelevel_info";
    $ci++;
  } ## end foreach (@$perfCounterInfo)

  # print Dumper('3195',@counter_arr_levels);
  my $count = keys %$now_counters;
  if ( $count != $selected_counters ) {
    xerror( "(F$server_count) counter items  :$count ?? should be $selected_counters host: $h_name,$entity_type  " . __FILE__ . ":" . __LINE__ );

    # print Dumper($now_counters);
    # print Dumper(@counter_arr);
    $error_select_counters++;
  } ## end if ( $count != $selected_counters)

  # are there more similar counters ?
  if ( $count != $ci ) {
    xerror( "(F$server_count) more counters  :chosen than expected $ci > $count host: $h_name,$entity_type " . __FILE__ . ":" . __LINE__ );
    $error_select_counters++;
  }

  # each counter must be exactly 1 times
  for ( my $i = 0; $i < $selected_counters; $i++ ) {
    if ( $signal_arr[$i] == 0 ) {
      xerror( "(F$server_count) counter problem  :$counter_arr[$i] not presented host: $h_name,$entity_type " . __FILE__ . ":" . __LINE__ );
      $error_select_counters++;
    }
    if ( $signal_arr[$i] > 1 ) {
      xerror( "(F$server_count) counter problem  :$counter_arr[$i] presented $signal_arr[$i] times host: $h_name,$entity_type " . __FILE__ . ":" . __LINE__ );
      $error_select_counters++;
    }
  } ## end for ( my $i = 0; $i < $selected_counters...)

  # print "3183 vmw2rrd.pl ".Dumper($all_counters); # how many is selected?
} ## end sub init_perf_counter_info

sub filter_metric_ids {
  my ( $perf_metric_ids, $entity_type ) = @_;

  # print Dumper($perf_metric_ids);
  if ( !$now_counters ) {
    error( "(F$server_count) unknown \$now_counters originally called for unknown init() $fail_entity_type:$fail_entity_name " . __FILE__ . ":" . __LINE__ );
  }
  my $counters = $now_counters;

  my @filtered_list;

  #    if ($entity_type eq $et_VirtualMachine) {
  #      print Dumper("3088 $et_VirtualMachine", $perf_metric_ids);
  #    }

  my $metric_count = 0;
  foreach (@$perf_metric_ids) {

    # if errors in counters, let test all possible real metrics
    #     if ( $error_select_counters ) {
    #       next if $_->instance ne '';
    #       push @filtered_list, $_;
    #	   $metric_count++;
    #     }
    #	 else {
    #       print "3100 testing ".$_->counterId." ,".$_->instance.",\n";
    if ( exists $counters->{ $_->counterId } ) {

      #          if ($_->counterId == 6) {
      #			print "counter 6 set up\n";
      #            $metric_count++;
      #            next;
      #		  }
      #          next if $_->instance ne '';
      #         if (($_->instance ne '') && ($entity_type eq $et_Datastore || $entity_type eq $et_VirtualMachine)) {
      #           print "3108 $entity_type".$_->counterId." ".$_->instance."\n";
      #         }
      next if ( ( $_->instance ne '' ) && !( $entity_type eq $et_Datastore || $entity_type eq $et_VirtualMachine ) );

      # filtr all counters for res-pool except 213, 214
      #          next if (($entity_type eq $et_ResourcePool) && !($_->counterId == 213 || $_->counterId == 214 || $_->counterId == 6 || $_->counterId == 102 || $_->counterId == 98 || $_->counterId == 29 || $_->counterId == 33 || $_->counterId == 37 || $_->counterId == 41 || $_->counterId == 70 || $_->counterId == 90 || $_->counterId == 157 || $_->counterId == 159));

      $metric_count++;

      # my $metric = PerfMetricId->new (counterId => $_->counterId,
      #                                instance => '');
      push @filtered_list, $_;
    } ## end if ( exists $counters->...)

    #     }
  } ## end foreach (@$perf_metric_ids)

  # print "\@filtered_list @filtered_list\n";
  if ( ( $metric_count != $selected_counters ) && $error_select_counters ) {
    xerror( "(F$server_count) selected counters number differs from real $selected_counters ? $metric_count $fail_entity_type:$fail_entity_name " . __FILE__ . ":" . __LINE__ );
  }
  return \@filtered_list;
} ## end sub filter_metric_ids

sub get_available_intervals {
  my %args = @_;

  my $perfmgr_view = $args{perfmgr_view};
  my $entity       = $args{host};
  my $entity_type  = $args{entity_type};

  my $historical_intervals = $perfmgr_view->historicalInterval;

  # print Dumper($historical_intervals);
  my @intervals;

  if ( $entity_type eq $et_Datastore && $apiType_top !~ "VirtualCenter" ) {

    # do nothing
  }
  else {
    my $provider_summary = $perfmgr_view->QueryPerfProviderSummary( entity => $entity );
    if ( $provider_summary->refreshRate ) {
      push @intervals, $provider_summary->refreshRate;

      # print "provider_summary->refreshRate ".$provider_summary->refreshRate . "\n";
    }
  } ## end else [ if ( $entity_type eq $et_Datastore...)]
  foreach (@$historical_intervals) {
    push @intervals, $_->samplingPeriod;
  }
  return \@intervals;
} ## end sub get_available_intervals

sub rrd_check {
  my $managedname = shift;
  print_hyperv_debug("------------- rrd_check $wrkdir/$managedname/$host\n") if $DEBUG == 2;
  # Check whether do initial or normal load
  opendir( DIR, "$wrkdir/$managedname/$host" ) || error( " directory does not exists : $wrkdir/$managedname/$host" . __FILE__ . ":" . __LINE__ ) && return 0;
  my @files          = ();
  my @files_unsorted = ();

  if ( $type_sam =~ "m" ) {
    @files_unsorted = grep( /\.rrm$/, readdir(DIR) );
    @files = sort { lc $a cmp lc $b } @files_unsorted;
  }
  if ( scalar @files > 0 ) {
    closedir(DIR);
    return;
  }
  rewinddir(DIR);    # for cluster

  @files_unsorted = grep( /cluster\.rrc$/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;

  if ( scalar @files_unsorted > 0 ) {
    closedir(DIR);
    return;
  }
  rewinddir(DIR);    # for resourcepools

  @files_unsorted = grep( /\.rrc$/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;

  if ( scalar @files_unsorted > 0 ) {
    closedir(DIR);
    return;
  }

  rewinddir(DIR);    # for datastore

  @files_unsorted = grep( /\.rrs$/, readdir(DIR) );
  @files = sort { lc $a cmp lc $b } @files_unsorted;

  if ( scalar @files_unsorted > 0 ) {
    closedir(DIR);
    return;
  }

  closedir(DIR);

  print "There is no RRD: $host:$managedname attempting to do initial load, be patient, it might take some time\n" if $DEBUG;

  # get last 62ays (HMC keeps hourly history data just for 2 months, not more)
  # no idea for shother sampe rates, it is not documented yet
  # it is for initial load
  # daily data it keeps for last 2years and monthly for last 10years
  # let load far enough backward for initial load (value in hours or in days)
  $loadhours = $INIT_LOAD_IN_HOURS_BACK;
  $loadmins  = $INIT_LOAD_IN_HOURS_BACK * 60;

  return 0;
} ## end sub rrd_check

sub save_config {
  my $managedname = shift;
  my $date        = shift;
  my $upgrade     = shift;

  my $out_file           = "$CONFIG_HISTORY/$managedname/$host/config.cfg";
  my $out_file_tmp       = "$CONFIG_HISTORY/$managedname/$host/config.cfg-tmp";
  my $out_file_shell     = $out_file;
  my $out_file_tmp_shell = $out_file_tmp;
  $out_file_tmp_shell =~ s/ /\\ /g;
  $out_file_shell =~ s/ /\\ /g;
  my $lpar        = "";
  my $hw_cfg_lpar = "$bindir/hw_cfg_lpar.sh";
  my $hw_cfg_sys  = "$bindir/hw_cfg_sys.sh";

  #if ( ! -f "$hw_cfg_lpar" ) {
  #  $hw_cfg_lpar="true"
  #}
  #if ( ! -f "hw_cfg_sys" ) {
  #  $hw_cfg_lpar="true"
  #}

  # do not update cfg file if less than 1 day
  if ( -f "$out_file" && $upgrade == 0 ) {
    my $png_time = ( stat("$out_file") )[9];
    ( my $sec, my $min, my $h, my $aday, my $m, my $y, my $wday, my $yday, my $isdst ) = localtime( time() );
    ( $sec, $min, $h, my $png_day, $m, $y, $wday, $yday, $isdst ) = localtime($png_time);
    if ( $aday == $png_day ) {
      return 0;
    }
    else {
      print "cfg refresh    : $aday != $png_day\n" if $DEBUG;
    }
  } ## end if ( -f "$out_file" &&...)
  print "config save    : $host:$managedname once a day ... \n" if $DEBUG;
  print "cfg save sys   : $host:$managedname \n"                if $DEBUG;

  if ( $IVM == 1 ) {

    # save CPU speed for CPU workload estimator
    `$SSH $hmc_user\@$host "ioscli lsdev -dev proc0 -attr frequency|tail -1" > $wrkdir/$managedname/$host/cpu_speed.txt`;
  }

  LoadDataModuleHyperV::touch("config save");

  # read hw_cfg_sys.sh
  open( FHS, "< $hw_cfg_sys" ) || error( " Cannot open  $hw_cfg_sys: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my $cmd_sys = "";
  while ( my $line = <FHS> ) {
    $cmd_sys = $cmd_sys . $line;
  }
  close(FHS);
  $cmd_sys =~ s/\$managedname/$managedname/g;

  #$cmd_sys =~ s/\$managedname/\\"$managedname\\"/g;
  # with above uncomented do not work systems with space inside
  #print "$cmd_sys\n";

  # read hw_cfg_lpar.sh
  open( FHL, "< $hw_cfg_lpar" ) || error( " Cannot open  $hw_cfg_lpar: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  my $cmd_lpar     = "";
  my $cmd_lpar_glb = "";
  while ( my $line = <FHL> ) {
    $cmd_lpar_glb = $cmd_lpar_glb . $line;
  }
  close(FHL);
  $cmd_lpar_glb =~ s/\$managedname/$managedname/g;

  #$cmd_lpar_glb =~ s/\$managedname/\\"$managedname\\"/g;
  # with above uncomented do not work systems with space inside
  #print "$cmd_lpar_glb\n";

  # get list of lpars
  my @lparlist;
  if ( $IVM == 1 ) {

    # IVM does not support "sort"
    @lparlist = `$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F name|sed 's/ /%20/g'"`;
  }
  else {
    @lparlist = `$SSH $hmc_user\@$host "lssyscfg -r lpar -m \\"$managedname\\" -F name|sort -f|sed 's/ /%20/g'"`;
  }

  `echo "<HTML> <HEAD> <TITLE>LPAR2RRD old GUI </TITLE> </HEAD> \
<BODY BGCOLOR=#D3D2D2 TEXT=#000000 LINK=#0000FF VLINK= #000080 ALINK=#FF0000 > \
<PRE>" > $out_file_tmp_shell`;

  #`ssh $hmc_user\@$host ". $hw_cfg_sys $managedname" |
  # sed -e 's/sys_time=[0-9][0-9]\\\/[0-9][0-9]\\\/[0-9][0-9][0-9][0-9] [0-9][0-9]:[0-9][0-9]:[0-9][0-9],//g' \
  if ( $IVM == 1 ) {

    # IVM does not support "sort"
    $cmd_sys =~ s/|sort -nk 2 -t=//g;
    $cmd_sys =~ s/|sort -f//g;
    $cmd_sys =~ s/|sort//g;
  } ## end if ( $IVM == 1 )
  `$SSH $hmc_user\@$host "$cmd_sys"  >> $out_file_tmp_shell`;

  foreach $lpar (@lparlist) {
    chomp($lpar);
    $cmd_lpar = $cmd_lpar_glb;

    # workaround for lpars which have a space within the name
    $cmd_lpar =~ s/\$lpar_space/$lpar/g;
    $lpar =~ s/%20/ /g;
    $lpar =~ s/\&\&1/\//g;    # for those which have a slash
    $cmd_lpar =~ s/\$lpar/$lpar/g;
    if ( $IVM == 1 ) {

      # IVM does not support "sort"
      $cmd_lpar =~ s/|sort -nk 2 -t=//g;
      $cmd_lpar =~ s/|sort -f//g;
      $cmd_lpar =~ s/|sort//g;
    } ## end if ( $IVM == 1 )
    print "cfg save       : $host:$managedname:$lpar\n";

    #in 1st echo line is necessary to back-slash all ';'
    #echo "</PRE><A NAME="aix4%20test%20/%#;%20/#"></A><HR><CENTER><B> LPAR : "aix4 test /%#; /#" </B></CENTER><HR><PRE>"
    #
    ( my $cmd_lpar_preface,         my $cmd_lpar_main )             = split( '</PRE><A NAME=',         $cmd_lpar );
    ( my $cmd_lpar_text_to_replace, my $cmd_lpar_main_not_replace ) = split( '</B></CENTER><HR><PRE>', $cmd_lpar_main );
    $cmd_lpar_text_to_replace =~ s/;/\\;/g;
    $cmd_lpar = $cmd_lpar_preface . '</PRE><A NAME=';
    $cmd_lpar .= $cmd_lpar_text_to_replace . '</B></CENTER><HR><PRE>' . $cmd_lpar_main_not_replace;

    `$SSH $hmc_user\@$host "$cmd_lpar" 2>/dev/null| \
sed -e 's/^lpar_name=.*,lpar_id=[1-9],//g'  -e 's/^lpar_name=.*,lpar_id=[1-9][0-9],//g'  | \
sed -e 's/,lpar_name=.*,lpar_id=[1-9],/,/g' -e 's/,lpar_name=.*,lpar_id=[1-9][0-9],/,/g' >> $out_file_tmp_shell`;

    # must be 2 x sed, 4 x "-e" is does not work !!!
  } ## end foreach $lpar (@lparlist)
  `echo "</PRE></BODY></HTML>" >> $out_file_tmp_shell`;

  print "cfg save form  : $host:$managedname \n" if $DEBUG;

  # copy $out_file_tmp to $out_file and devide some very long rows into more rows to be better readable in the web
  open( FHR, "< $out_file_tmp" ) || error( " Cannot open  $out_file_tmp: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  open( FHW, "> $out_file" )     || error( " Cannot open  $out_file: $!" . __FILE__ . ":" . __LINE__ )     && return 0;
  my $count           = 0;
  my $count_prof      = 0;
  my $line_prev       = "";
  my $count_no_result = 0;

  while ( my $line = <FHR> ) {
    chomp($line);

    # count == 2 is only for LPAR profiles  & CPU pools where can be more than 1 profile and it should be devided into more rows
    if ( $count == 2 ) {
      if ( $line =~ m/^name=/ ) {
        if ($count_prof) {
          print FHW "\n\n";
        }
      }
      $count_prof++;
    } ## end if ( $count == 2 )
    if ( $line =~ "LPAR profiles:" ) {
      $count      = 2;
      $count_prof = 0;
      print FHW "$line\n";
      next;
    } ## end if ( $line =~ "LPAR profiles:")

    if ( $line =~ "Memory - AMS:" || $line =~ "Memory - AMS - paging:" || $line =~ "Virtual VASI:" || $line =~ "Virtual slots per lpar:" || $line =~ "Physical IO per io pool level sys:" || $line =~ "Physical IO per io pool level pool:" || $line =~ "Ethernet:" || $line =~ "HEA logical:" || $line =~ "HEA physical per system:" || $line =~ "Logical HEA per system:" || $line =~ "Virtual SCSI:" || $line =~ "Virtual Ethernet:" || $line =~ "Logical HEA:" || $line =~ "CPU pools:" || $line =~ "Physical adapters:" || $line =~ "Physical IO per slot children:" || $line =~ "Physical IO per taggedio:" || $line =~ "HEA physical per port:" || $line =~ "HEA physical per port group:" || $line =~ "HEA logical per port:" || $line =~ "HCA adapters:" || $line =~ "SNI adapters:" || $line =~ "Virtual OptiConnec:" || $line_prev =~ "Firmware:" ) {
      $line_prev       = $line;
      $count_no_result = 1;
      next;
    }

    if ($count_no_result) {
      if ( $line =~ "HSCL" || $line =~ "VIOSE0" || $line =~ "The managed system does not support hardware discovery." || $line =~ "The managed system does not support multiple shared processor pools." || $line =~ "No results were found." || $line =~ m/The command entered is either missing/ ) {
        $count_no_result = 0;
        next;
      }
      print FHW "$line_prev\n";
      $count_no_result = 0;
      if ( $line_prev =~ "Memory - AMS:" || $line_prev =~ "Memory - AMS - paging:" || $line_prev =~ "Virtual OptiConnec:" || $line_prev =~ "Virtual slots per lpar:" || $line_prev =~ "Physical IO per io pool level sys:" || $line_prev =~ "<B>Ethernet:" || $line_prev =~ "CPU pools:" || $line_prev =~ "CPU pool:" || $line_prev =~ "System overview:" || $line_prev =~ "CPU resources:" || $line_prev =~ "LPAR config:" || $line_prev =~ "LPAR profiles:" || $line_prev =~ "CPU resources:" || $line_prev =~ "CPU totally:" || $line_prev =~ "Memory resources " || $line_prev =~ "Memory:" || $line_prev =~ "Firmware:" ) {
        $count = 1;
      }
      if ( $line_prev =~ "CPU pools:" ) {
        $count      = 2;
        $count_prof = 1;
      }
    } ## end if ($count_no_result)

    if ( $line =~ "Memory - AMS:" || $line =~ "Memory - AMS - paging:" || $line =~ "Virtual OptiConnec:" || $line =~ "Virtual slots per lpar:" || $line =~ "Physical IO per io pool level sys:" || $line =~ "<B>Ethernet:" || $line =~ "CPU pools:" || $line =~ "CPU pool:" || $line =~ "System overview:" || $line =~ "CPU resources:" || $line =~ "LPAR config:" || $line =~ "LPAR profiles:" || $line =~ "CPU resources:" || $line =~ "CPU totally:" || $line =~ "Memory resources " || $line =~ "Memory:" || $line =~ "Firmware:" ) {
      $count = 1;
      print FHW "$line\n";
      next;
    }

    # Formating output
    if ( $count > 0 ) {
      my @list      = split( /,/, $line );
      my $comma     = 0;
      my $enter     = 0;
      my $end_comma = 0;
      foreach my $item (@list) {
        chomp($item);
        my $last = substr( $item, 1, 1 );
        if ( ( $item =~ "=" ) && ( $item !~ m/^"/ ) && ( $item !~ m/"$/ ) ) {
          ( my $left_c, my $right_c ) = split( /=/, $item );
          my $result = "";
          if ( ( $count == 2 ) && ( $left_c =~ m/^name=/ ) ) {
            $result = sprintf( "%-36s = </PRE><B>%s</B><PRE>\n", $left_c, $right_c );
          }
          else {
            $result = sprintf( "%-36s = %s\n", $left_c, $right_c );
          }
          if ( $result =~ m/"$/ ) {
            $comma = 0;
            $enter = 1;
          }
          $result =~ s/=/ /g;
          if ( $enter == 1 ) {
            $enter = 0;
            print FHW "$result";
          }
          else {
            print FHW "$result";
          }
          next;
        } ## end if ( ( $item =~ "=" ) ...)
        else {
          if ( ( $comma == 0 ) && ( $item =~ "^\"" ) ) {
            if ( $item =~ "^\"" ) {
              $comma = 1;
            }
            if ( $item =~ "=" ) {
              ( my $left_c, my $right_c ) = split( /=/, $item );
              my $result = sprintf( "%-36s = %s", $left_c, $right_c );
              if ( $result =~ m/"$/ ) {
                $comma = 0;
                $enter = 1;
              }
              $result =~ s/^"//g;
              $result =~ s/"$//g;
              $result =~ s/ =/   /g;
              $result =~ s/=/ /g;
              if ( $enter == 1 ) {
                $enter = 0;
                print FHW "$result\n";
              }
              else {
                print FHW "$result";
              }
              next;
            } ## end if ( $item =~ "=" )
            else {
              my $result = sprintf( "%-36s", $item );
              if ( $result =~ m/"$/ ) {
                $comma = 0;
                $enter = 1;
              }
              $result =~ s/^"//g;
              $result =~ s/"$//g;
              $result =~ s/ =/  =/g;
              if ( $enter == 1 ) {
                $enter = 0;
                print FHW "$result\n";
              }
              else {
                print FHW "$result";
              }
              next;
            } ## end else [ if ( $item =~ "=" ) ]
          } ## end if ( ( $comma == 0 ) &&...)
          else {
            if ( ( $comma == 1 ) && ( ( $item !~ m/^"/ ) || ( $item =~ m/""$/ ) || ( $item =~ m/""""$/ ) ) && ( $item !~ m/=/ ) ) {
              if ( $item =~ m/"$/ ) {
                $comma = 0;
                $enter = 1;
              }
              while ( $item =~ m/^"/ ) {
                $item =~ s/^"//g;
              }
              while ( $item =~ m/"$/ ) {
                $item =~ s/"$//g;
              }
              $item =~ s/ =/  =/g;
              if ( $enter == 1 ) {
                $enter = 0;
                print FHW "\n                                       $item\n";
              }
              else {
                print FHW "\n                                       $item";
              }
              next;
            } ## end if ( ( $comma == 1 ) &&...)
            else {
              if ( ( $comma == 1 ) && ( $item =~ m/^"/ ) ) {
                if ( ( $item =~ m/^""/ ) || ( $item =~ m/^""""/ ) ) {
                  if ( $item =~ m/"$/ ) {
                    $comma = 0;
                    $enter = 1;
                  }
                  $item =~ s/=/ /g;
                  if ( $enter == 1 ) {
                    $enter = 0;
                    print FHW "$item\n";
                  }
                  else {
                    print FHW "$item";
                  }
                  next;
                } ## end if ( ( $item =~ m/^""/...))
                ( my $left_c, my $right_c ) = split( /=/, $item );
                my $result = sprintf( "\n%-36s = %s", $left_c, $right_c );
                if ( $result =~ m/"$/ ) {
                  $comma = 0;
                  $enter = 1;
                }
                $result =~ s/^"//g;
                $result =~ s/"$//g;
                $result =~ s/ =/   /g;
                $result =~ s/=/ /g;
                if ( $enter == 1 ) {
                  $enter = 0;
                  print FHW "$result\n";
                }
                else {
                  print FHW "$result";
                }
                next;
              } ## end if ( ( $comma == 1 ) &&...)
              else {
                if ( $item =~ m/"$/ ) {
                  $comma = 0;
                  $enter = 1;
                }
                $item =~ s/^"//g;
                $item =~ s/"$//g;
                $item =~ s/ =/   /g;
                $item =~ s/=/ /g;
                if ( $enter == 1 ) {
                  $enter = 0;
                  print FHW "$item\n";
                }
                else {
                  print FHW "$item";
                }
                print FHW "$item\n";
                next;
              } ## end else [ if ( ( $comma == 1 ) &&...)]
            } ## end else [ if ( ( $comma == 1 ) &&...)]
          } ## end else [ if ( ( $comma == 0 ) &&...)]
          if ( $item =~ m/"$/ ) {
            $comma = 0;
            $enter = 1;
            $item =~ s/^"//g;
            $item =~ s/"$//g;
            $item =~ s/ =/   /g;
            $item =~ s/=/ /g;
            if ( $enter == 1 ) {
              $enter = 0;
              print FHW "$item\n";
            }
            else {
              print FHW "$item";
            }
            next;
          } ## end if ( $item =~ m/"$/ )
        } ## end else [ if ( ( $item =~ "=" ) ...)]
      } ## end foreach my $item (@list)
      if ( $comma == 1 ) {
        print FHW "\n";
      }
      if ( $count != 2 ) {
        print FHW "\n";
      }
      if ( $count == 2 ) {
        next;
      }
      $count = 0;
    } ## end if ( $count > 0 )
    else {
      $line =~ s/,/ /g;
      print FHW "$line\n";
    }
  } ## end while ( my $line = <FHR> )
  print FHW "\n</PRE><A HREF=\"http://www.lpar2rrd.com\">Created by LPAR2RRD $version</A><PRE>\n";
  print FHW "$date\n";
  close(FHW);
  close(FHR);
  unlink("$out_file_tmp") || error( " Cannot rm $out_file_tmp : $!" . __FILE__ . ":" . __LINE__ ) && return 0;

  return 1;
} ## end sub save_config

sub save_cpu_cfg_global {
  my $managedname = shift;
  my $time        = shift;
  my $upgrade     = shift;
  my $cpu_cfg     = "$wrkdir/$managedname/$host/cpu.html";
  my $mem_cfg     = "$wrkdir/$managedname/$host/mem.html";
  my $lpm_excl    = "$wrkdir/$managedname/$host/lpm-exclude.txt";
  my $act_time    = time();
  my @results     = "";

  if ( !-d "$wrkdir/$managedname/$host" ) {
    error("$wrkdir/$managedname/$host does not exist");
    return 0;
  }

  # do not update chart every run
  if ( -f "$cpu_cfg" && $upgrade == 0 ) {
    my $png_time = ( stat("$cpu_cfg") )[9];
    ( my $sec, my $min, my $h, my $aday, my $m, my $y, my $wday, my $yday, my $isdst ) = localtime( time() );
    ( $sec, $min, $h, my $png_day, $m, $y, $wday, $yday, $isdst ) = localtime($png_time);
    if ( $aday == $png_day ) {
      return 0;
    }
    else {
      print "cfg cpu refresh: $aday != $png_day\n" if $DEBUG;
    }
  } ## end if ( -f "$cpu_cfg" && ...)

  print "fetching HMC   : $host:$managedname CPU/MEM global config\n" if $DEBUG;
  if ( -f "$mem_cfg" ) {

    # backup old one, crate a new one a do diff, if there is any then --> install_html.sh
    if ( -f "$mem_cfg-old" ) {
      unlink("$mem_cfg-old");
    }
    rename( "$mem_cfg", "$mem_cfg-old" );
  } ## end if ( -f "$mem_cfg" )

  if ( -f "$cpu_cfg" ) {

    # backup old one, crate a new one a do diff, if there is any then --> install_html.sh
    if ( -f "$cpu_cfg-old" ) {
      unlink("$cpu_cfg-old");
    }
    rename( "$cpu_cfg", "$cpu_cfg-old" );
  } ## end if ( -f "$cpu_cfg" )

  if ( $lpm == 1 ) {

    # creates file with list of VIO servers, they have to be excluded from LPM
    open( FHW, "> $lpm_excl" ) || error( " Cannot open  $lpm_excl: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    @results = `$SSH $hmc_user\@$host "export LANG=en_US; lssyscfg -m \\"$managedname\\" -r lpar -F name,lpar_env"`;
    foreach my $lpm_line (@results) {
      ( my $lpar_name, my $lpar_env ) = split( /,/, $lpm_line );
      if ( !$lpar_env eq '' && $lpar_env =~ m/vioserver/ ) {
        print FHW "$lpar_name\n";
      }
    } ## end foreach my $lpm_line (@results)
    close(FHW);
  } ## end if ( $lpm == 1 )

  if ( $IVM == 1 ) {

    # does not support curr_shared_proc_pool_name
    @results = `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight"`;
  }
  else {
    @results = `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight,curr_shared_proc_pool_name"`;
  }

  if ( $results[0] =~ "HSCL" || $results[0] =~ "VIOSE0" ) {

    # something wrong with the input data
    error("$host:$managedname : $results[0] : lshwres -m $managedname -r proc --level lpar -F lpar_name,cur ...");
    return 1;
  } ## end if ( $results[0] =~ "HSCL"...)

  my $cpu_pool_err = 0;
  if ( $results[0] =~ /An invalid attribute was entered/ ) {

    # POWER5 do not support curr_shared_proc_pool_name
    #print "ERROR with $managedname : curr_shared_proc_pool_name is not valid for this frame.\n";
    $cpu_pool_err = 1;
    @results      = `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar -F lpar_name,curr_proc_mode,curr_min_proc_units,curr_proc_units,curr_max_proc_units,curr_procs,curr_sharing_mode,curr_uncap_weight"`;
  } ## end if ( $results[0] =~ /An invalid attribute was entered/)
  my $report = FormatResults(@results);
  open( FHW, "> $cpu_cfg" ) || error( " Cannot open  $cpu_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;

  #print FHW "<BR><CENTER><TABLE border=\"0\" cellpadding=\"0\" cellspacing=\"0\">\n";
  print FHW "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">\n";
  if ( $cpu_pool_err == 1 ) {
    print FHW "<thead><TR> <TH class=\"sortable\" valign=\"center\">LPAR&nbsp;&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Mode&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Virtual&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Sharing&nbsp;&nbsp;&nbsp;<br>mode</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Uncap&nbsp;&nbsp;&nbsp;<br>weight</TH><TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</TH></TR>\n</thead><tbody>\n";
  }
  else {
    print FHW "<thead><TR> <TH class=\"sortable\" valign=\"center\">LPAR&nbsp;&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Mode&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Virtual&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Sharing&nbsp;&nbsp;&nbsp;<br>mode</TH> <TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Uncap&nbsp;&nbsp;&nbsp;<br>weight</TH><TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;Pool&nbsp;&nbsp;&nbsp;</TH><TH align=\"center\" class=\"sortable\" valign=\"center\">&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;OS&nbsp;&nbsp;&nbsp;</TH></TR>\n</thead><tbody>\n";
  }

  # Check whether AIX or another OS due to "grep -p" which supports only AIX
  my $uname = `uname -a`;
  chomp($uname);
  my $aix = 0;
  my @u_a = split( / /, $uname );
  foreach my $u (@u_a) {
    if ( $u =~ "AIX" ) {
      $aix = 1;
    }
  }

  # Loop for adding OS
  foreach my $line ( split( /\n/, $report ) ) {
    my $os   = "";
    my $lpar = "";
    chomp($line);
    $line =~ s/0.0.0.0.0.0//g;
    if ( -f "$basedir/data/$managedname/$host/config.cfg" ) {
      ( my $trash, $lpar ) = split( /<TD/, $line );
      $lpar =~ s/<B>//;
      $lpar =~ s/<\/B><\/TD> //;
      $lpar =~ s/^>//;
      if ( $aix == 1 ) {

        # AIX with grep -p support
        $os = `egrep -p \"^name\.\*    $lpar\" $basedir/data/"$managedname"/"$host"/config.cfg|egrep \"^os_vers\"|head -1`;
      }
      else {
        #        $os = `egrep \"^name\.\*    $lpar\" $basedir/data/"$managedname"/"$host"/config.cfg|egrep \"^os_vers\"|head -1`;
        $os = `grep -A 30 "^name.*    $lpar" $basedir/data/"$managedname"/"$host"/config.cfg | grep "os_version"|head -1`;
      }
      chomp($os);
      $os =~ s/^os_version//g;
      $os =~ s/  //g;
      $os =~ s/ /&nbsp;/g;
    } ## end if ( -f "$basedir/data/$managedname/$host/config.cfg")
    else {
      $os = "";
    }
    $os =~ s/Unknown//;

    #print "$basedir/data/$managedname/$host/config.cfg -- $line ++++  $lpar $os $aix -- \n";
    $line =~ s/<\/TR>//;
    print FHW "$line <TD align=\"center\" nowrap>&nbsp;$os&nbsp;</TD></TR>\n";
  } ## end foreach my $line ( split( /\n/...))
  print FHW "</tbody></TABLE></CENTER><BR><BR>\n";
  close(FHW);

  # Memory
  @results = `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r mem --level lpar -F lpar_name,curr_min_mem,curr_mem,curr_max_mem,run_mem"`;
  $report  = FormatResults(@results);
  open( FHW, "> $mem_cfg" ) || error( " Cannot open  $mem_cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHW "<BR><CENTER><TABLE class=\"tabconfig tablesorter\">\n";
  print FHW "<thead><TR> <TH class=\"sortable\">LPAR&nbsp;&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;Min&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;Assigned&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;Max&nbsp;&nbsp;&nbsp;</TH> <TH align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;Running&nbsp;&nbsp;&nbsp;</TH></TR>\n</thead><tbody>\n";
  print FHW "$report\n";
  print FHW "</tbody></TABLE></CENTER><br>(all in MB)<BR>\n";
  close(FHW);

  # if any change then  run install_html.sh
  if ( -f "$mem_cfg-old" ) {
    if ( compare( "$mem_cfg", "$mem_cfg-old" ) != 0 ) {
      LoadDataModuleHyperV::touch("$mem_cfg-old");
    }
  }
  else {
    LoadDataModuleHyperV::touch("$mem_cfg-old 1");
  }

  if ( -f "$cpu_cfg-old" ) {
    if ( compare( "$cpu_cfg", "$cpu_cfg-old" ) != 0 ) {
      LoadDataModuleHyperV::touch("$cpu_cfg-old");
    }
  }
  else {
    LoadDataModuleHyperV::touch("$cpu_cfg-old 1");
  }

} ## end sub save_cpu_cfg_global

sub FormatResults {
  my @results_unsort = @_;
  my $line           = "";
  my $formated       = "";
  my @items1         = "";
  my $item           = "";

  my @results = sort { lc $a cmp lc $b } @results_unsort;
  foreach $line (@results) {
    chomp $line;
    @items1 = split /,/, $line;
    $formated = $formated . "<TR>";
    my $col = 0;
    foreach $item (@items1) {
      if ( $col == 0 ) {
        $formated = sprintf( "%s <TD><B>%s</B></TD>", $formated, $item );
      }
      else {
        $formated = sprintf( "%s <TD align=\"center\">%s</TD>", $formated, $item );
      }
      $col++;
    } ## end foreach $item (@items1)
    $formated = $formated . "</TR>\n";
  } ## end foreach $line (@results)
  return $formated;
} ## end sub FormatResults

# function saves CPU config of each lpar, this is then shown in the main panel of each lpar
# it save also lpar cfg for all lpars into lpar.cfg
sub save_cpu_cfg {
  my $managedname    = shift;
  my $time           = shift;
  my $upgrade        = shift;
  my $cfg            = "$wrkdir/$managedname/$host/cpu.cfg";
  my $cfg_lpar       = "$wrkdir/$managedname/$host/lpar.cfg";
  my $cfg_shell      = $cfg;
  my $cfg_lpar_shell = $cfg_lpar;
  $cfg_shell =~ s/ /\\ /g;
  $cfg_lpar_shell =~ s/ /\\ /g;
  my $act_time = time();
  my $serial   = "";

  if ( !-d "$wrkdir/$managedname/$host" ) {
    error("$wrkdir/$managedname/$host does not exist");
    return 0;
  }

  # --> IT must not be here!!! lpar2rrd needs to have actuall data every run!!!
  # do not update chart every run
  #if ( -f "$cfg" ) {
  #  my $png_time = (stat("$cfg"))[9];
  #  if ( ($act_time - $png_time) < $CFG_REFRESH ) {
  #    #print "creating graph : $host:$managedname:$type:$adapter_id:$port no upd\n" if $DEBUG ;
  #    return 0;
  #  }
  #}

  my @lparlist = `$SSH $hmc_user\@$host "lssyscfg -m \\"$managedname\\" -r lpar -F name,state,logical_serial_num" 2>&1`;
  if ( $lparlist[0] =~ /Standby or Operating state/ ) {
    error("$host:$managedname : $lparlist[0] : server looks like switched off");
    return 2;
  }

  if ( $lparlist[0] =~ /HSCL/ || $lparlist[0] =~ /VIOSE0/ ) {
    error("$host:$managedname : $lparlist[0] : lssyscfg -m $managedname  -r lpar -F name,state");
    return 1;
  }

  print "fetching HMC   : $host:$managedname CPU lpar config\n" if $DEBUG;
  `$SSH $hmc_user\@$host "export LANG=en_US; lshwres -m \\"$managedname\\" -r proc --level lpar" > $cfg_shell`;
  `$SSH $hmc_user\@$host "export LANG=en_US; lssyscfg -m \\"$managedname\\" -r lpar " > $cfg_lpar_shell`;

  open( FHR, "< $cfg" ) || error( " Cannot open  $cfg: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  while ( my $line = <FHR> ) {
    chomp($line);
    if ( $line =~ /Standby or Operating state/ ) {
      error("$host:$managedname : $line : server looks like switched off");
      return 2;
    }
    if ( $line =~ /HSCL/ || $line =~ /VIOSE0/ ) {
      error("$host:$managedname : $line : lshwres -m $managedname -r proc --level lpar");
      return 1;
    }

    if ( $line =~ "lpar_name=" ) {
      my @list  = split( /,/, $line );
      my $first = 0;
      my $l     = "";
      my $state = "";
      foreach my $item (@list) {
        chomp($item);
        $first++;
        if ( $first == 1 ) {
          ( my $name, my $lpar ) = split( /=/, $item );
          foreach $l (@lparlist) {
            ( my $name, my $state_a, my $serial_new ) = split( /,/, $l );
            if ( $name =~ m/^$lpar$/ ) {
              $state  = $state_a;
              $serial = $serial_new;
              last;
            }
          } ## end foreach $l (@lparlist)

          #print "$lpar $state\n";
          $lpar =~ s/\//\&\&1/g;
          open( FHW, "> $cfg-$lpar" ) || error( " Cannot open $cfg-$lpar: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
          print FHW "<B>Current CPU configuration:</B><TABLE class=\"tabconfig\">\n";
          my $result = sprintf( "<TR><TD><font size=\"-1\">state</font></TD><TD><font size=\"-1\">&nbsp;&nbsp;%s</font></TD></TR>\n", $state );
          print FHW "$result";
          chomp($serial);
          $result = sprintf( "<TR><TD><font size=\"-1\">logical_serial_num</font></TD><TD><font size=\"-1\">%s</font></TD></TR>\n", $serial );
          print FHW "$result";
          next;
        } ## end if ( $first == 1 )
        if ( $item =~ m/^curr_/ && ( $item !~ "_max" ) && ( $item !~ "_min" ) ) {
          ( my $left_c, my $right_c ) = split( /=/, $item );

          #my $result = sprintf ("%-36s = %s",$left_c,$right_cp;
          my $result = sprintf( "<TR><TD><font size=\"-1\">%s</font></TD><TD><font size=\"-1\">&nbsp;&nbsp;%s</font></TD></TR>\n", $left_c, $right_c );
          print FHW "$result";
        } ## end if ( $item =~ m/^curr_/...)
      } ## end foreach my $item (@list)

      #print FHW "<TR><TD><font size=\"-1\">$time</font></TD></TR></TABLE>\n<BR>\n";
      print FHW "</TABLE>\n<font size=\"-1\">HMC time : $time</font><BR><BR>\n";
      close(FHW);
    } ## end if ( $line =~ "lpar_name=")
  } ## end while ( my $line = <FHR> )
  close(FHR);
  return 0;
} ## end sub save_cpu_cfg

sub sys_change {
  my $managedname = shift;
  my $date        = shift;

  #my $cmd = "export LANG=en_US; lslparutil -r all -s h  -m \"$managedname\" --filter \"event_types=config_change,state_change,utility_cod_proc_usage\" -n $SYS_CHANGE";

  if ( !-d "$wrkdir/$managedname/$host" ) {
    error("$wrkdir/$managedname/$host does not exist");
    return 0;
  }
  my $gui_out    = "$wrkdir/$managedname/$host/gui-change-state.txt";
  my $head_state = "<CENTER><B>Changes in LPAR state for last $SYS_CHANGE days</B> <font size=-1>(Updated on: $date)</font><br>\
	<TABLE class=\"tabconfig tablesorter\"> \
	<thead><tr><th align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;Time&nbsp;&nbsp;&nbsp;</th><th align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;State&nbsp;&nbsp;&nbsp;</th><th align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;LPAR&nbsp;&nbsp;&nbsp;</th><th align=\"center\" class=\"sortable\">&nbsp;&nbsp;&nbsp;ID&nbsp;&nbsp;&nbsp;</th></tr></thead><tbody>\n";

  print "sys change     : $host:$managedname:$SYS_CHANGE\n" if $DEBUG;
  my @state = `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all -s h  -m \\"$managedname\\" --filter \"event_types=state_change\" -d $SYS_CHANGE"`;

  open( FHLG, "> $gui_out" ) || error( " Can't open $gui_out : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLG "$head_state\n";

  my $line_out = "";
  foreach my $line (@state) {

    #time=04/03/2014 13:26:01,event_type=state_change,resource_type=lpar,primary_state=Started,detailed_state=None,lpar_name=VIOS,lpar_id=1
    #time=05/20/2014 12:57:32,event_type=state_change,resource_type=lpar,state=Running,lpar_name=BSRV21LPAR3,lpar_id=3

    chomp($line);
    if ( $line eq '' || $line !~ m/,/ || $line !~ m/event_type=state_change/ ) {
      next;
    }
    $line =~ s/event_type=state_change,//;
    $line =~ s/,detailed_state=.*,lpar_name=/,lpar_name=/;    # to filter SDMC detailed_state=None
                                                              #$line =~ s/,,/,/g;
                                                              #print "007 $line\n";
                                                              #time=06/29/2014 22:45:24,resource_type=lpar,primary_state=Started,lpar_id=16
    if ( $line =~ m/resource_type=lpar/ ) {
      ( my $time, my $res, my $stat, my $lpar, my $id ) = split( /,/, $line );

      my $time_data = "";
      if ( defined $time && !$time eq '' ) {
        ( my $tr, $time_data ) = split( /=/, $time );
      }

      my $stat_data = "";
      if ( defined $stat && !$stat eq '' ) {
        ( my $tr, $stat_data ) = split( /=/, $stat );
      }

      my $lpar_data = "";
      if ( defined $lpar && !$lpar eq '' ) {
        ( my $tr, $lpar_data ) = split( /=/, $lpar );
      }

      my $id_data = "";
      if ( defined $id && !$id eq '' ) {
        ( my $tr, $id_data ) = split( /=/, $id );
      }

      #print "008 <tr><td>$time_data</td><td>$res_data</td><td>$stat_data</td><td>$lpar_data</td><td>$id_data</td></tr>\n";
      $line_out .= "<tr><td align=\"center\" nowrap>$time_data</td><td align=\"center\" nowrap>$stat_data</td><td align=\"center\" nowrap>$lpar_data</td><td align=\"center\" nowrap>$id_data</td></tr>\n";
    } ## end if ( $line =~ m/resource_type=lpar/)

    #if ( $line =~ m/resource_type=sys/ ) {
    # time=03/12/2014 13:54:50,resource_type=sys,primary_state=Started,detailed_state=None
    #}
  } ## end foreach my $line (@state)

  print FHLG "$line_out</tbody></centre></table>\n";
  close(FHLG);

  my $gui_out1        = "$wrkdir/$managedname/$host/gui-change-config.txt";
  my $head_config_gui = "<CENTER><B>Changes in LPAR configuration for last $SYS_CHANGE days</B> <font size=-1>(Updated on: $date)</font><br>\
	<TABLE class=\"tabconfig tablesorter\">";
  my @config = `$SSH $hmc_user\@$host "export LANG=en_US; lslparutil -r all -s h  -m \\"$managedname\\" --filter \"event_types=config_change\" -d $SYS_CHANGE"`;

  open( FHLG, "> $gui_out1" ) || error( " Can't open $gui_out1 : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FHLG "$head_config_gui\n";

  $line_out = "";
  my $line_out_head = "<tr>";
  my $head          = 0;
  foreach my $line (@config) {
    chomp($line);
    $line =~ s/event_type=config_change//;
    if ( $line eq '' || $line !~ m/,/ || $line !~ m/resource_type=lpar/ ) {
      next;
    }
    my @line_items = split( /,/, $line );
    $line_out .= "<tr>";
    my $head_ok = 0;
    foreach my $item (@line_items) {
      if ( $item eq '' || $item =~ m/resource_type=/ ) {
        next;
      }
      if ( $head == 0 ) {
        ( my $item_head, my $tr1 ) = split( /=/, $item );
        $line_out_head .= "<th align=\"center\" class=\"sortable\">$item_head&nbsp;&nbsp;&nbsp;</th>";
        $head_ok = 1;
      }
      ( my $tr, my $item_data ) = split( /=/, $item );
      if ( $item_data eq '' ) {
        next;
      }
      if ( $tr =~ m/lpar_name/ ) {
        $line_out .= "<td nowrap>&nbsp;&nbsp;$item_data</td>";
      }
      else {
        $line_out .= "<td align=\"center\" nowrap>$item_data</td>";
      }
    } ## end foreach my $item (@line_items)
    if ( $head_ok == 1 ) {
      $head = 1;
    }
    $line_out .= "</tr>\n";
  } ## end foreach my $line (@config)
  $line_out_head .= "</tr>";
  print FHLG "<thead>$line_out_head\n</thead><tbody>\n$line_out\n</tbody>\n";

  print FHLG "</table></centre>\n";
  close(FHLG);
} ## end sub sys_change

sub cfg_config_change {

  # If any change in lpar2rrd.cfg  then must be run install-html.sh
  my $cfg_update = "$tmpdir/cfg_update";

  if ( -f "$cfg_update" ) {
    my $last_line = "";
    open( FHLT, "< $cfg_update" ) || error( " Can't open $cfg_update: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    foreach my $line (<FHLT>) {
      $last_line = $line;
    }
    close(FHLT);
    my $png_time = ( stat("$basedir/etc/lpar2rrd.cfg") )[9];
    if ( $last_line < $png_time ) {
      LoadDataModuleHyperV::touch("cfg_config_change: $last_line < $png_time");
      open( FHLT, "> $cfg_update" ) || error( " Can't open $cfg_update: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      my $act_time = time();
      print FHLT "$act_time";
      close(FHLT);
    } ## end if ( $last_line < $png_time)
  } ## end if ( -f "$cfg_update" )
  else {
    LoadDataModuleHyperV::touch("cfg_config_change: $cfg_update");
    my $png_time = ( stat("$basedir/etc/lpar2rrd.cfg") )[9];
    open( FHLT, "> $cfg_update" ) || error( " Can't open $cfg_update: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    my $act_time = time();
    print FHLT "$act_time";
    close(FHLT);

    #print "$cfg_update $basedir/etc/lpar2rrd.cfg $png_time\n";
  } ## end else [ if ( -f "$cfg_update" )]

  return 1;

} ## end sub cfg_config_change

# open top10 files
sub open_topten_files {
  my $managedname = shift;
  my $host        = shift;
  my $act_time    = shift;
  my $run_topten  = 0;       # run top10 only once a day

  if ( !-f "tmp/$managedname-$host-d" ) {
    print "TOP10 refresh  : initial : tmp/$managedname-$host-d\n" if $DEBUG;
    $run_topten = 1;
  }
  else {
    my $png_time = ( stat("tmp/$managedname-$host-d") )[9];
    ( my $sec, my $min, my $h, my $aday, my $m, my $y, my $wday, my $yday, my $isdst ) = localtime( time() );
    ( $sec, $min, $h, my $png_day, $m, $y, $wday, $yday, $isdst ) = localtime($png_time);
    if ( $aday != $png_day ) {
      print "TOP10 refresh  : $aday != $png_day\n" if $DEBUG;
      $run_topten = 1;
    }
  } ## end else [ if ( !-f "tmp/$managedname-$host-d")]

  if ( $run_topten == 1 ) {
    open( TOPD, "> tmp/$managedname-$host-d" )     || error( " Can't open tmp/$managedname-$host-d: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    open( TOPW, "> tmp/$managedname-$host-w-tmp" ) || error( " Can't open tmp/$managedname-$host-w: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    open( TOPM, "> tmp/$managedname-$host-m-tmp" ) || error( " Can't open tmp/$managedname-$host-m: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    open( TOPY, "> tmp/$managedname-$host-y-tmp" ) || error( " Can't open tmp/$managedname-$host-y: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    LoadDataModuleHyperV::touch("top10 run");
  } ## end if ( $run_topten == 1 )

  return $run_topten;
} ## end sub open_topten_files

sub close_topten_files {
  $run_topten = shift;

  if ( $run_topten == 1 ) {
    close(TOPD);
    close(TOPW);
    close(TOPM);
    close(TOPY);
  } ## end if ( $run_topten == 1 )

  return 0;
} ## end sub close_topten_files

# create server/lpar cfg high level page
#sub get_cfg_server
# not for vmware

# had a problem with filling that in var in LoadDataModule.pm tehrefore it is here
sub procpoolagg {
  print "procpoolagg    :   sub cau cau nothing when vmware\n";
  return 1;
  my $host        = shift;
  my $managedname = shift;

  @poollistagg = `$SSH $hmc_user\@$host "lshwres -r procpool -m \\"$managedname\\" -F shared_proc_pool_id,lpar_names"`;

  return 1;
} ## end sub procpoolagg

# error handling
sub error {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  print "ERROR          : $text : $!\n";
  print STDERR "$act_time: $text : $!\n";

  return 1;
} ## end sub error

sub xerror {
  my $text     = shift;
  my $act_time = localtime();
  chomp($text);

  if ( $i_am_fork eq 'fork' ) {
    print "XERROR         : $act_time: $text\n";
  }
  else {
    open my $FH, ">>$counters_info_file" or error( "can't open $counters_info_file: $!" . __FILE__ . ":" . __LINE__ );
    print $FH "$act_time: $text\n";
    close $FH;
  }
  return 1;
} ## end sub xerror

sub save_cfg_data {
  my $managedname = shift;
  my $date        = shift;
  my $upgrade     = shift;
  my $ret         = 0;

  # print "save_cfg_data  :   sub cau cau nothing when hyperv\n";
  return 0;

  $ret = save_cpu_cfg( $managedname, $date, $upgrade );
  if ( $ret > 0 ) {
    return $ret;
  }

  $ret = save_cpu_cfg_global( $managedname, $date, $upgrade );
  if ( $ret == 1 ) {
    return 1;
  }

  # Include HW info
  if ($HWINFO) {
    save_config( $managedname, $date, $upgrade );
  }

  if ( $HMC == 1 || $SDMC == 1 ) {    # IVM does not support it
                                      # Include system changes
    if ($SYS_CHANGE) {
      sys_change( $managedname, $date );
    }
  } ## end if ( $HMC == 1 || $SDMC...)

  return 0;
} ## end sub save_cfg_data

sub rename_server {
  my $host        = shift;
  my $managedname = shift;
  my $model       = shift;
  my $serial      = shift;

  # since 4.8 can work for both IBM/VMWARE

  my $mod_ser = $model;
  if ( defined $serial ) {    # in case vmware, there is only uuid
    $mod_ser .= $serial;
  }

  if ( !-d "$wrkdir/$managedname" ) {

    # when managed system is renamed then find the original nale per a sym link with model*serial
    #   and rename it in lpar2rrd as well
    if ( -l "$wrkdir/$mod_ser" ) {
      my $link = readlink("$wrkdir/$mod_ser");

      #my $base = basename($link);
      # basename without direct function
      my @link_l = split( /\//, $link );
      my $base = "";
      foreach my $m (@link_l) {
        $base = $m;
      }

      print "system renamed : $host:$managedname from $base to $managedname, behave as upgrade \n" if $DEBUG;
      if ( -f "$link" ) {
        rename( "$link", "$wrkdir/$managedname" ) || error( " Cannot mv $link $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      }
      unlink("$wrkdir/$mod_ser") || error( " Cannot rm $wrkdir/$mod_ser: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
      $upgrade = 1;    # must be like upgrade behave due to views
    } ## end if ( -l "$wrkdir/$mod_ser")
    else {
      print "mkdir          : $host:$managedname $wrkdir/$managedname\n" if $DEBUG;
      mkdir( "$wrkdir/$managedname", 0755 ) || error( " Cannot mkdir $wrkdir/$managedname: $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    LoadDataModuleHyperV::touch("$wrkdir/$managedname");    #must be at the end due to renaming servers
  } ## end if ( !-d "$wrkdir/$managedname")

  # check wherher the symlink is linked to the right targed
  # there could be an issue with symlink prior 3.37 ($managedname dirs could be created from HEA stuff without care about renaming)
  my $managedname_linked = "";
  my $link               = "";
  my $link_expected      = "$wrkdir/$mod_ser";
  if ( -l "$link_expected" ) {
    $link = readlink("$link_expected");

    # basename without direct function
    my @link_l = split( /\//, $link );
    my $managedname_linked = "";
    foreach my $m (@link_l) {
      $managedname_linked = $m;
    }
    if ( $managedname =~ m/^$managedname_linked$/ ) {

      # ok, symlink target is properly linked
      return 1;
    }
    else {
      print "symlink correct: $host:$managedname : $link : $link_expected\n" if $DEBUG;
      unlink($link_expected);
    }
  } ## end if ( -l "$link_expected")

  return 1;
} ## end sub rename_server

# fill in @lpm_excl_vio for server
sub lpm_exclude_vio {
  my $host        = shift;
  my $managedname = shift;
  my $wrkdir      = shift;
  my $lpm_excl    = "$wrkdir/$managedname/$host/lpm-exclude.txt";

  if ( $lpm == 0 ) {
    return 0;    # LPM is switched off
  }
  open( FH, "< $lpm_excl" ) || return 1;
  @lpm_excl_vio = <FH>;
  close(FH);
  return 0;
} ## end sub lpm_exclude_vio

sub lpar_details {
  my $managedname = shift;
  my $host        = shift;
  my $cvs         = shift;

  if ( !-f "$wrkdir/$managedname/$host/config.cfg" ) {
    error("does not exist $wrkdir/$managedname/$host/config.cfg");
    return 1;
  }

  open( FHC, ">> $cvs" ) || error( " Can't open $cvs" . __FILE__ . ":" . __LINE__ ) && return 0;    # detailed cfg in CSV format

  open( FCFG, "< $wrkdir/$managedname/$host/config.cfg" ) || error( " Can't open $wrkdir/$managedname/$host/config.cfg" . __FILE__ . ":" . __LINE__ ) && return 0;

  my $csv_lpar_name           = "na";
  my $csv_lpar_id             = "na";
  my $csv_default_profile     = "na";
  my $csv_min_proc_units      = "na";
  my $csv_desired_proc_units  = "na";
  my $csv_max_proc_units      = "na";
  my $csv_curr_min_proc_units = "na";
  my $csv_curr_proc_units     = "na";
  my $csv_curr_max_proc_units = "na";
  my $csv_min_procs           = "na";
  my $csv_desired_procs       = "na";
  my $csv_max_procs           = "na";
  my $csv_curr_min_procs      = "na";
  my $csv_curr_procs          = "na";
  my $csv_curr_max_procs      = "na";
  my $csv_min_mem             = "na";
  my $csv_desired_mem         = "na";
  my $csv_max_mem             = "na";
  my $csv_curr_min_mem        = "na";
  my $csv_curr_mem            = "na";
  my $csv_curr_max_mem        = "na";
  my $csv_state               = "na";
  my $csv_lpar_env            = "na";
  my $csv_os_version          = "na";
  my $pool_name               = "na";
  my $cpu_mode1               = "na";
  my $phys_cores              = "na";
  my $logical                 = "na";
  my $cpu_mode2               = "na";
  my $cpu_weight              = "na";
  my $cpu_ded                 = "na";

  my $item    = "";
  my $right_c = "";
  my $first   = 0;

  # select data only from actual active profile
  my $curr_profile  = "";
  my $right_profile = 0;
  my $profile       = 0;

  #print "000 $wrkdir/$managedname/$host/config.cfg\n";
  foreach my $linel (<FCFG>) {
    chomp($linel);
    if ( $linel eq '' ) {
      next;
    }

    if ( $linel =~ "LPAR config:" ) {
      $first = 1;
      if ( $csv_lpar_name !~ m/^na$/ ) {
        print FHC "$host;$managedname;$csv_lpar_name;$pool_name;$cpu_mode1;$phys_cores;$logical;$cpu_mode2;$cpu_weight;$csv_lpar_id;$csv_default_profile;$csv_min_proc_units;$csv_desired_proc_units;$csv_max_proc_units;$csv_curr_min_proc_units;$csv_curr_max_proc_units;$csv_min_procs;$csv_desired_procs;$csv_max_procs;$csv_curr_min_procs;$csv_curr_max_procs;$csv_min_mem;$csv_desired_mem;$csv_max_mem;$csv_curr_min_mem;$csv_curr_mem;$csv_curr_max_mem;$csv_state;$csv_lpar_env;$csv_os_version\n";
      }

      #print "001 $csv_lpar_name\n";

      $csv_lpar_name           = "na";
      $csv_lpar_id             = "na";
      $csv_default_profile     = "na";
      $csv_min_proc_units      = "na";
      $csv_desired_proc_units  = "na";
      $csv_max_proc_units      = "na";
      $csv_curr_min_proc_units = "na";
      $csv_curr_proc_units     = "na";
      $csv_curr_max_proc_units = "na";
      $csv_min_procs           = "na";
      $csv_desired_procs       = "na";
      $csv_max_procs           = "na";
      $csv_curr_min_procs      = "na";
      $csv_curr_procs          = "na";
      $csv_curr_max_procs      = "na";
      $csv_min_mem             = "na";
      $csv_desired_mem         = "na";
      $csv_max_mem             = "na";
      $csv_curr_min_mem        = "na";
      $csv_curr_mem            = "na";
      $csv_curr_max_mem        = "na";
      $csv_state               = "na";
      $csv_lpar_env            = "na";
      $csv_os_version          = "na";
      $pool_name               = "na";
      $cpu_mode1               = "na";
      $phys_cores              = "na";
      $logical                 = "na";
      $cpu_mode2               = "na";
      $cpu_weight              = "na";
      $cpu_ded                 = "na";
      $curr_profile            = "";
      $right_profile           = 0;
      $profile                 = 0;
      next;
    } ## end if ( $linel =~ "LPAR config:")

    if ( $first == 0 ) {
      next;
    }

    $item    = "";
    $right_c = "na";

    #print "003 $linel \n";

    if ( $linel !~ m/ / ) {
      next;
    }

    ( $item, $right_c ) = split( /\s{1,}/, $linel );    # if is there one and more spaces or tabelators

    if ( $item eq '' ) {
      next;
    }
    if ( $right_c eq '' ) {
      next;
    }

    # --> this is due to parameter with a space inside like pool name, status ...
    my $line2 = $linel;
    $line2 =~ s/^.*$right_c / /;
    if ( length($linel) != length($line2) ) {
      $right_c .= $line2;
    }

    #print "004 $linel : $item - $right_c : profile=$profile curr_profile=$curr_profile \n";

    if ( $linel =~ m/LPAR profiles:/ ) {
      $profile = 1;
      next;
    }

    if ( $item =~ m/^curr_profile/ ) {
      $curr_profile = $right_c;
      next;
    }
    if ( $profile == 1 && $item =~ m/^name/ ) {
      $right_profile = 0;
      if ( $curr_profile =~ m/^$right_c$/ ) {
        $right_profile = 1;
      }
      next;
    } ## end if ( $profile == 1 && ...)
    if ( $item =~ m/^Memory resources / ) {
      $profile = 0;
    }

    #print "004 $linel : $item - $right_c : profile=$profile curr_profile=$curr_profile right_profile=$right_profile\n";

    if ( $item =~ m/^curr_shared_proc_pool_name/ ) {
      if ( $pool_name !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $pool_name = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_shared_proc_pool_name/)
    if ( $item =~ m/^curr_proc_mode/ ) {
      if ( $cpu_mode1 !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $cpu_mode1 = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_proc_mode/)
    if ( $item =~ m/^curr_proc_units/ ) {
      if ( $phys_cores !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $phys_cores = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_proc_units/)
    if ( $item =~ m/^curr_procs/ ) {
      if ( $logical !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $logical = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_procs/)
    if ( $item =~ m/^curr_sharing_mode/ ) {
      if ( $cpu_mode2 !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $cpu_mode2 = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_sharing_mode/)
    if ( $item =~ m/^curr_uncap_weight/ ) {
      if ( $cpu_weight !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $cpu_weight = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_uncap_weight/)
    if ( $item =~ m/^name/ ) {
      if ( $csv_lpar_name !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_lpar_name = $right_c;
      next;
    } ## end if ( $item =~ m/^name/)
    if ( $item =~ m/^lpar_id/ ) {
      if ( $csv_lpar_id !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_lpar_id = $right_c;
      next;
    } ## end if ( $item =~ m/^lpar_id/)
    if ( $item =~ m/^default_profile/ ) {
      if ( $csv_default_profile !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_default_profile = $right_c;
      next;
    } ## end if ( $item =~ m/^default_profile/)
    if ( $item =~ m/^min_proc_units/ ) {
      if ( $csv_min_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_min_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^min_proc_units/)
    if ( $item =~ m/^desired_proc_units/ ) {
      if ( $csv_desired_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_desired_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^desired_proc_units/)
    if ( $item =~ m/^max_proc_units/ ) {
      if ( $csv_max_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_max_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^max_proc_units/)
    if ( $item =~ m/^curr_min_proc_units/ ) {
      if ( $csv_curr_min_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_min_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_min_proc_units/)
    if ( $item =~ m/^curr_proc_units/ ) {
      if ( $csv_curr_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_proc_units/)
    if ( $item =~ m/^curr_max_proc_units/ ) {
      if ( $csv_curr_max_proc_units !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_max_proc_units = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_max_proc_units/)
    if ( $item =~ m/^min_procs/ ) {
      if ( $csv_min_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_min_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^min_procs/)
    if ( $item =~ m/^desired_procs/ ) {
      if ( $csv_desired_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_desired_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^desired_procs/)
    if ( $item =~ m/^max_procs/ ) {
      if ( $csv_max_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_max_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^max_procs/)
    if ( $item =~ m/^curr_min_procs/ ) {
      if ( $csv_curr_min_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_min_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_min_procs/)
    if ( $item =~ m/^curr_procs/ ) {
      if ( $csv_curr_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_procs/)
    if ( $item =~ m/^curr_max_procs/ ) {
      if ( $csv_curr_max_procs !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_max_procs = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_max_procs/)
    if ( $item =~ m/^min_mem/ ) {
      if ( $csv_min_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_min_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^min_mem/)
    if ( $item =~ m/^desired_mem/ ) {
      if ( $csv_desired_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_desired_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^desired_mem/)
    if ( $item =~ m/^max_mem/ ) {
      if ( $csv_max_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_max_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^max_mem/)
    if ( $item =~ m/^curr_min_mem/ ) {
      if ( $csv_curr_min_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_min_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_min_mem/)
    if ( $item =~ m/^curr_mem/ ) {
      if ( $csv_curr_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_mem/)
    if ( $item =~ m/^curr_max_mem/ ) {
      if ( $csv_curr_max_mem !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_curr_max_mem = $right_c;
      next;
    } ## end if ( $item =~ m/^curr_max_mem/)
    if ( $item =~ m/^state/ ) {
      if ( $csv_state !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_state = $right_c;
      next;
    } ## end if ( $item =~ m/^state/)
    if ( $item =~ m/^lpar_env/ ) {
      if ( $csv_lpar_env !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_lpar_env = $right_c;
      next;
    } ## end if ( $item =~ m/^lpar_env/)
    if ( $item =~ m/^os_version/ ) {
      if ( $csv_os_version !~ m/^na$/ ) {
        if ( $right_profile == 0 ) {
          next;
        }
      }
      $csv_os_version = $right_c;
      next;
    } ## end if ( $item =~ m/^os_version/)
  } ## end foreach my $linel (<FCFG>)

  # print the last lpar
  if ( $csv_lpar_name !~ m/^na$/ ) {
    print FHC "$host;$managedname;$csv_lpar_name;$pool_name;$cpu_mode1;$phys_cores;$logical;$cpu_mode2;$cpu_weight;$csv_lpar_id;$csv_default_profile;$csv_min_proc_units;$csv_desired_proc_units;$csv_max_proc_units;$csv_curr_min_proc_units;$csv_curr_max_proc_units;$csv_min_procs;$csv_desired_procs;$csv_max_procs;$csv_curr_min_procs;$csv_curr_max_procs;$csv_min_mem;$csv_desired_mem;$csv_max_mem;$csv_curr_min_mem;$csv_curr_mem;$csv_curr_max_mem;$csv_state;$csv_lpar_env;$csv_os_version\n";
  }

  close(FHC);
  close(FCFG);

  return 0;
} ## end sub lpar_details

# return 1 if the argument is valid IP, otherwise 0
sub is_IP {
  my $ip = shift;

  if ( $ip =~ /^(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})$/ && ( ( $1 <= 255 && $2 <= 255 && $3 <= 255 && $4 <= 255 ) ) ) {
    return 1;
  }
  else {
    return 0;
  }
} ## end sub is_IP

sub isdigit {
  my $digit = shift;
  my $text  = shift;

  my $digit_work = $digit;
  $digit_work =~ s/[0-9]//g;
  $digit_work =~ s/\.//;

  if ( length($digit_work) == 0 ) {

    # is a number
    return 1;
  }

  #if (($digit * 1) eq $digit){
  #  # is a number
  #  return 1;
  #}

  # NOT a number
  return 0;
} ## end sub isdigit

# it checks if rrdtool supports graphv --> then zoom is supported
# it checks if rrdtool supports --right-axis (used for vmware cpu graphing - since 1.2015)
sub rrdtool_graphv {
  my $graph_cmd   = "graph";
  my $graphv_file = "$tmpdir/graphv";

  my $ansx = `$rrdtool`;

  if ( index( $ansx, 'graphv' ) != -1 ) {

    # graphv exists, create a file to pass it to cgi-bin commands
    if ( !-f $graphv_file ) {
      `touch $graphv_file`;
    }
  } ## end if ( index( $ansx, 'graphv'...))
  else {
    if ( -f $graphv_file ) {
      unlink($graphv_file);
    }
  }

  $graph_cmd   = "--right-axis";
  $graphv_file = "$tmpdir/graph-right-axis";
  $ansx        = `$rrdtool graph $graph_cmd 2>&1`;

  if ( index( $ansx, "$graph_cmd" ) == -1 ) {    # OK when doesn't contain
                                                 # right-axis exists, create a file to pass it to cgi-bin commands
    if ( !-f $graphv_file ) {
      `touch $graphv_file`;
    }
  } ## end if ( index( $ansx, "$graph_cmd"...))
  else {
    if ( -f $graphv_file ) {
      unlink($graphv_file);
    }
  }

  return 0;
} ## end sub rrdtool_graphv

sub frame_multi {
  my ( $text, $type_sam, $type, $xgrid, $cl_managedname, $cl_host ) = @_;

  my $file = "";
  my $i    = 0;
  my $cmd  = "";
  my $cmdq = "";
  my $j    = 0;
  my $managed_ok;
  my $managedname_exl = "";
  my $managedname     = "";
  my @m_excl          = "";
  my $req_time        = 0;
  my $act_time        = time();
  my $name            = "$webdir/$host/pool-multi-$type";    # not really used, changed in detail-graph-cgi.pl
  $step            = $STEP;
  @managednamelist = @managednamelist_vmw;
  my $DEBUG = 1;
  my $cluster;

  print "creating m_hmc : $host:$type\n" if $DEBUG;

  # my $header = "$host aggregated : last $text";
  my $header = "Servers aggregated : last $text";

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag_agg";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$step";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU cores\\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU cores\\:               average   maximum\\l\\\"";

  my $local_host = $host;
  my $gtype      = "AREA";
  my $color_indx = 0;
  foreach my $line (@managednamelist) {
    chomp($line);

    $cluster = "";
    ( my $part1, my $part2 ) = split( 'XORUX', $line );
    if ( defined $part2 && ( $part2 ne "" ) ) {
      $line       = $part1;
      $cluster    = $part2;
      $local_host = $part2;
    }

    if ( $line =~ /No results were found/ ) {
      print "HMC : $host does not contain any managed system\n" if $DEBUG;
      return 0;
    }

    #    ($managedname, my $model, my $serial) = split(/,/,$line);
    $managedname = $line;

    #    if ( is_IP($managedname) == 1 ) {
    #      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    #    }

    $managed_ok = 1;
    if ( $managed_systems_exclude ne '' ) {
      @m_excl = split( /:/, $managed_systems_exclude );
      foreach $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok = 0;
        }
      } ## end foreach $managedname_exl (@m_excl)
    } ## end if ( $managed_systems_exclude...)
    if ( $managed_ok == 0 ) {
      next;
    }

    $file = "pool.rrm";

    # create path for cluster if it is
    #    if ($cluster ne "") {
    #	  $managedname = $cluster;
    #	  $file = "cluster.rrc";
    #    }

    if ( $type_sam =~ "d" ) {
      $req_time = $act_time - 86400;
    }
    if ( "$type" =~ "w" ) {
      $req_time = $act_time - 604800;
    }
    if ( "$type" =~ "m" ) {
      $req_time = $act_time - 2764800;
    }

    # print "1 creating CMD for $wrkdir/$managedname/$local_host/$file\n" if $DEBUG ;

    if ( -e "$wrkdir/$managedname/$local_host/$file" ) {

      # avoid old servers which do not exist in the period
      my $rrd_upd_time = ( stat("$wrkdir/$managedname/$local_host/$file") )[9];
      if ( $rrd_upd_time < $req_time ) {
        next;
      }
    } ## end if ( -e "$wrkdir/$managedname/$local_host/$file")
    else {
      # avoid non-existing managed systems (with no utilization data on)
      next;
    }

    my $managedname_space      = $managedname;
    my $managedname_space_proc = $managedname;
    $managedname_space_proc =~ s/:/\\:/g;
    $managedname_space_proc =~ s/%/%%/g;    # anti '%

    for ( my $k = length($managedname); $k < 35; $k++ ) {
      $managedname_space .= " ";
    }
    $managedname_space = substr( $managedname_space, 0, 34 );    # not longer

    $managedname_space =~ s/:/\\:/g;                             # anti ':'
    my $wrkdir_managedname_host_file = "$wrkdir/$managedname/$local_host/$file";

    # print "2 creating CMD for $wrkdir_managedname_host_file\n" if $DEBUG ;

    $wrkdir_managedname_host_file =~ s/:/\\:/g;
    my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
    $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

    # my in_cores = 1;
    # bulid RRDTool cmd
    $cmd .= " DEF:cpu_entitl_mhz${i}=\\\"$wrkdir_managedname_host_file\\\":CPU_Alloc:AVERAGE";
    $cmd .= " DEF:utiltot_mhz${i}=\\\"$wrkdir_managedname_host_file\\\":CPU_usage:AVERAGE";
    $cmd .= " DEF:one_core_hz${i}=\\\"$wrkdir_managedname_host_file\\\":host_hz:AVERAGE";

    #    $cmd .= " COMMENT:\"   Average                   cores      Ghz (right axis)\\n\"";
    #    if ($in_cores) left_curly
    $cmd .= " CDEF:cpuutiltot${i}=utiltot_mhz${i},one_core_hz${i},/,1000000,*";
    $cmd .= " CDEF:ncpu${i}=cpu_entitl_mhz${i},one_core_hz${i},/,1000000,*";
    $cmd .= " CDEF:cpu_entitl_ghz${i}=cpu_entitl_mhz${i},1000,/";
    $cmd .= " CDEF:utiltot_ghz${i}=utiltot_mhz${i},1000,/";

    #    $cmd .= " DEF:totcyc${i}=\\\"$wrkdir_managedname_host_file\\\":total_pool_cycles:AVERAGE";
    #    $cmd .= " DEF:uticyc${i}=\\\"$wrkdir_managedname_host_file\\\":utilized_pool_cyc:AVERAGE";
    #    $cmd .= " DEF:ncpu${i}=\\\"$wrkdir_managedname_host_file\\\":conf_proc_units:AVERAGE";
    #    $cmd .= " DEF:ncpubor${i}=\\\"$wrkdir_managedname_host_file\\\":bor_proc_units:AVERAGE";
    # if it does not exist for some time period then put 0 there
    $cmd .= " CDEF:cpu${i}=ncpu${i},UN,0,ncpu${i},IF";

    #    $cmd .= " CDEF:cpubor${i}=ncpubor${i},UN,0,ncpubor${i},IF";
    #    $cmd .= " CDEF:totcpu${i}=cpu${i},cpubor${i},+";
    #    $cmd .= " CDEF:cpuutil${i}=uticyc${i},totcyc${i},GT,UNKN,uticyc${i},totcyc${i},/,IF";
    #    $cmd .= " CDEF:cpuutiltot${i}=cpuutil${i},totcpu${i},*";
    #    $cmd .= " CDEF:utilisa${i}=cpuutil${i},100,*";
    $cmd .= " PRINT:cpuutiltot${i}:AVERAGE:\"%6.2lf $delimiter multihmcframe $delimiter $managedname_space_proc $delimiter $color[$color_indx] $delimiter $wrkdir_managedname_host_file_legend\"";
    $cmd .= " PRINT:cpuutiltot${i}:MAX:\" %6.2lf $delimiter\"";

    $cmdq .= " $gtype:cpuutiltot${i}$color[$color_indx++]:\\\"$managedname_space\\\"";
    $cmdq .= " GPRINT:cpuutiltot${i}:AVERAGE:\\\"%6.2lf \\\"";
    $cmdq .= " GPRINT:cpuutiltot${i}:MAX:\\\"%6.2lf \\l\\\"";
    $gtype = "STACK";
    $i++;
    if ( $color_indx > $color_max ) {
      $color_indx = 0;
    }
  } ## end foreach my $line (@managednamelist)

  if ( $i == 0 ) {

    # no available managed system
    return 1;
  }

  # add count of all CPU in pools
  for ( $j = 0; $j < $i; $j++ ) {
    if ( $j == 0 ) {
      $cmd .= " CDEF:tcpu${j}=cpu${j}";
    }
    else {
      my $k = $j - 1;

      #      $cmd .= " CDEF:tcpu_tmp${j}=cpu${j},cpubor${j},+";
      $cmd .= " CDEF:tcpu${j}=tcpu${k},cpu${j},+";
    } ## end else [ if ( $j == 0 ) ]
  } ## end for ( $j = 0; $j < $i; ...)
  if ( $j > 0 ) {
    $j--;
  }
  my $cpu_pool_total = "CPUs total available in pools";
  for ( my $k = length($cpu_pool_total); $k < 35; $k++ ) {
    $cpu_pool_total .= " ";
  }
  $cmd .= " CDEF:tcpun${j}=tcpu${j},0,EQ,UNKN,tcpu${j},IF";
  $cmd .= " LINE2:tcpun${j}#888888:\\\"$cpu_pool_total\\\"";

  #$cmd .= " GPRINT:tcpu${j}:AVERAGE:\\\"%6.2lf \\\"";
  # excluded as it is a bit misleading there there is any even small data gap
  $cmd .= " GPRINT:tcpun${j}:MAX:\\\"         %6.2lf \\l\\\"";
  $cmd .= $cmdq;
  $cmd .= " PRINT:tcpun${j}:MAX:\\\"         %6.2lf MAXTCPU $delimiter\\\"";
  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  #  my $FH;
  my $tmp_file = "$tmpdir/multi-hmc-$host-$type.cmd";
  if ( $cluster ne "" ) {
    $tmp_file = "$tmpdir/multi-hmc-$cl_managedname-$cl_host-$type.cmd";
  }

  # print "CMD is in file $tmp_file\n" if $DEBUG;
  open( FH, "> $tmp_file" ) || error( " Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  # print "3 CMD is in file $tmp_file\n" if $DEBUG ;

  return 1;    # do not execute the cmd itself

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < "$tmp_file" 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error("$host: Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      ( my $err, my $file, my $txt ) = split( /'/, $ret );
      error("Removing as it seems to be corrupted: $file");
      unlink("$file") || error( "Cannot rm $file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    else {
      error("$host: $cmd: Pool multi graph rrdtool error : $ret");
    }
  } ## end if ( $ret =~ "ERROR" )

  #unlink ("$tmp_file"); --> do not remove, it is used for details

  return 0;
} ## end sub frame_multi

# LPARs aggregated per a HMC
sub multiview_hmc {
  my ( $host, $name_part, $type, $type_sam, $act_time, $text, $xgrid, $et_type, $cl_managedname, $cl_host ) = @_;
  my $req_time = 0;
  my $file     = "";
  my $i        = 0;
  my $lpar     = "";
  my $cmd      = "";
  my $j        = 0;
  my $name     = "$webdir/$host/$name_part-$type";    # out graph file name, not really used, changed in detail-graph-cgi.pl

  my $DEBUG = 1;

  print "creating l_hmc : $host:$type - LPARs aggregated per a HMC\n" if $DEBUG;

  #my $header = "LPARs aggregated per $host: last $text";
  my $header = "VMs aggregated : last $text";
  if ( "$type" =~ "d" ) {
    $req_time = $act_time - 86400;
  }
  if ( "$type" =~ "w" ) {
    $req_time = $act_time - 604800;
  }
  if ( "$type" =~ "m" ) {
    $req_time = $act_time - 2764800;
  }

  # for daily, must be even type as for IVM/SDMC it is not in $type_sam =~ "d"
  if ( $type_sam =~ "d" || "$type" =~ "y" ) {
    $req_time = $act_time - 31536000;
  }

  my $gtype = "AREA";

  my $cluster;
  @managednamelist = @managednamelist_vmw;

  # for ResourcePool -> server is not necessary
  if ( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) {
    @managednamelist = ("resourcepool");

    # $header = "LPARs aggregated per $rp_name: last $text";
  }

  $cmd .= "graph \\\"$name.png\\\"";
  $cmd .= " --title \\\"$header\\\"";
  $cmd .= " --start now-1$type";
  $cmd .= " --end now-1$type+1$type";
  $cmd .= " --imgformat PNG";
  $cmd .= " $disable_rrdtool_tag_agg";
  $cmd .= " --slope-mode";
  $cmd .= " --width=400";
  $cmd .= " --height=150";
  $cmd .= " --step=$STEP";
  $cmd .= " --lower-limit=0.00";
  $cmd .= " --color=BACK#$pic_col";
  $cmd .= " --color=SHADEA#$pic_col";
  $cmd .= " --color=SHADEB#$pic_col";
  $cmd .= " --color=CANVAS#$pic_col";
  $cmd .= " --alt-autoscale-max";
  $cmd .= " --upper-limit=0.1";
  $cmd .= " --vertical-label=\\\"CPU GHz  \\\"";
  $cmd .= " --units-exponent=1.00";
  $cmd .= " --alt-y-grid";
  $cmd .= " --x-grid=$xgrid";
  $cmd .= " COMMENT:\\\"Utilization in CPU GHz                         \\l\\\"";
  $cmd .= " COMMENT:\\\"  Server                       VM                                avrg     max\\l\\\"";

  foreach my $line (@managednamelist) {

    # goes through all servers of each HMC
    chomp($line);
    $cluster = "";
    ( my $part1, my $part2 ) = split( 'XORUX', $line );
    if ( defined $part2 && ( $part2 ne "" ) ) {
      $line    = $part1;
      $cluster = $part2;
      $host    = $part2;    #change of $host
    }

    if ( !defined $cluster ) {    # could happen when no counters
      next;
    }

    if ( $line =~ /No results were found/ ) {
      print "HMC : $host does not contain any managed system\n" if $DEBUG;
      return 0;
    }

    #    ($managedname, my $model, my $serial) = split(/,/,$line);
    $managedname = $line;

    #    if ( is_IP($managedname) == 1 ) {
    #      next; # wrong entry from the HMC, a problem of the HMC or unconfigured server yet
    #    }

    my $managed_ok = 1;
    if ( $managed_systems_exclude ne '' ) {
      my @m_excl = split( /:/, $managed_systems_exclude );
      foreach my $managedname_exl (@m_excl) {
        chomp($managedname_exl);
        if ( $managedname =~ m/^$managedname_exl$/ ) {
          $managed_ok = 0;
        }
      } ## end foreach my $managedname_exl...
    } ## end if ( $managed_systems_exclude...)
    if ( $managed_ok == 0 ) {
      next;
    }

    my @files = ();

    if ( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) {

      # $vm_id_path{"$vm_mo_ref_id"} = "$wrkdir/$h_name/$host/$vm_uuid.rrm";  hash contains path to VM data
      # push @rp_vm_morefs,$vm_mo_ref_id."\n";                                arr contains moref to hash

      @keep_color_lpar = ();    # do not take previous colors (e.g. from cluster)

      foreach (@rp_vm_morefs) {

        #        if (! -f "$vm_id_path{$_}") {  # when not exist
        #		   next;
        #		}
        push @files, $vm_id_path{"$_"};
      } ## end foreach (@rp_vm_morefs)

      #print "rp $rp_name has these vms @files\n";
    } ## end if ( ( defined $et_type...))
    else {
      # prepare list of active registered VMs on server
      my $registered_VM_file = "$wrkdir/$managedname/$host/VM_hosting.vmh";
      next if ( !-f $registered_VM_file );    # ESXi has no VM

      open( my $fh, "$registered_VM_file" ) or error( "can't open $registered_VM_file: $!" . __FILE__ . ":" . __LINE__ ) && next;
      while ( my $row = <$fh> ) {
        ( my $vm, undef ) = split( ":", $row );
        $vm .= ".rrm";
        my $pos = rindex( $row, ':' );
        next if $pos < 0;                     # some trash
        if ( substr( $row, $pos + 1, 5 ) eq "start" ) {    # keep item
          push @files, $vm;
        }
      } ## end while ( my $row = <$fh> )
      close $fh;
    } ## end else [ if ( ( defined $et_type...))]

    #      opendir(DIR, "$wrkdir/$managedname/$host") || error(" directory does not exists : $wrkdir/$managedname/$host ".__FILE__.":".__LINE__) && return 0;
    #      if ( $type_sam =~ "m" ) {
    #        my @files_unsorted = grep(/\.rrm$/,readdir(DIR));
    #        @files = sort { lc $a cmp lc $b } @files_unsorted;
    #      }
    #      if ( $type_sam =~ "h" ) {
    #        my @files_unsorted = grep(/\.rrh$/,readdir(DIR));
    #        @files = sort { lc $a cmp lc $b } @files_unsorted;
    #      }
    #      if ( $type_sam =~ "d" ) {
    #        my @files_unsorted = grep(/\.rrd$/,readdir(DIR));
    #        @files = sort { lc $a cmp lc $b } @files_unsorted;
    #        $req_time = $act_time - 31536000;
    #      }
    #      closedir(DIR);
    #    }

    my @color_save = "";
    my $found      = -1;

    foreach $file (@files) {

      # goes through all lpars of each server
      chomp($file);

      if ( $file =~ m/^\.rr/ ) {
        next;    # avoid .rrm
      }

      # prepare $managedname/$host/$file from path
      if ( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) {
        $managedname = ( split( /\//, $file ) )[-3];
        $host        = ( split( /\//, $file ) )[-2];
        $file        = ( split( /\//, $file ) )[-1];

        $found++;
      } ## end if ( ( defined $et_type...))

      # avoid non existed - suspended or poweredoff VMs - no data saved yet
      next if !-f "$wrkdir/$all_hyperv_VMs/$file";

      # avoid old lpars which do not exist in the period
      my $rrd_upd_time = ( stat("$wrkdir/$all_hyperv_VMs/$file") )[9];    # you can add also ! defined
      if ( $rrd_upd_time < $req_time ) {
        next;
      }

      $lpar = $file;
      $lpar =~ s/.rrh//;
      $lpar =~ s/.rrm//;
      $lpar =~ s/.rrd//;

      $server = $managedname;

      my $lpar_orig = human_vmware_name( $lpar, "", $managedname, $host );
      $lpar = substr( $lpar_orig, 0, 35 );

      my $lpar_space      = $lpar;
      my $lpar_space_proc = $lpar_orig;

      if ( $lpar eq '' ) {
        next;    # avoid .rrm --> just to be sure :)
      }

      # add spaces to lpar name to have 35 chars total (for formating graph legend)
      $lpar_space =~ s/\&\&1/\//g;
      for ( my $k = length($lpar_space); $k < 35; $k++ ) {
        $lpar_space .= " ";
      }

      # Exclude pools and memory
      if ( $lpar =~ m/^cod$/ || $lpar =~ m/^pool$/ || $lpar =~ m/^mem$/ || $lpar =~ /^SharedPool[0-9]$/ || $lpar =~ m/^SharedPool[1-9][0-9]$/ || $lpar =~ m/^mem-pool$/ ) {
        next;
      }

      # Assure that lpar colors is same for each lpar through daily - yearly charts
      my $l;
      my $l_count = 0;

      if ( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) {

        # nothing yet
      }
      else {
        $found = -1;
        foreach $l (@keep_color_lpar) {
          if ( $l eq '' || $l =~ m/^ $/ ) {
            next;
          }

          #        if ( $l =~ m/^$lpar$/ ) {
          if ( $l eq "$lpar" ) {
            $found = $l_count;
            last;
          }
          $l_count++;
        } ## end foreach $l (@keep_color_lpar)
      } ## end else [ if ( ( defined $et_type...))]
      if ( $found == -1 ) {
        $keep_color_lpar[$l_count] = $lpar;
        $found = $l_count;
      }

      # take color from 0 to 53 -- division modulo
      $found = $found % $color_max;

      #print "$found $lpar $color[$found]\n";
      print "$wrkdir/$managedname/$host/$file $rrd_upd_time $req_time $act_time\n" if ( $DEBUG == 3 );

      my $server_name = $managedname;
      $server_name =~ s/\&\&1/\//g;
      my $server_name_space = $server_name;
      for ( my $k = length($server_name_space); $k < 25; $k++ ) {
        $server_name_space .= " ";
      }
      $server_name =~ s/:/\\:/g;
      $server_name =~ s/%/%%/g;

      $lpar_space = $server_name_space . "    " . $lpar_space;

      $lpar_space =~ s/:/\\:/g;    # anti ':'
      $lpar_space_proc =~ s/:/\\:/g;
      $lpar_space_proc =~ s/%/%%/g;    # anti '%

      my $wrkdir_managedname_host_file = "$wrkdir/$managedname/$host/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;
      my $wrkdir_managedname_host_file_legend = $wrkdir_managedname_host_file;
      $wrkdir_managedname_host_file_legend =~ s/%/%%/g;

      # new system
      $wrkdir_managedname_host_file = "$wrkdir/$all_hyperv_VMs/$file";
      $wrkdir_managedname_host_file =~ s/:/\\:/g;

      # bulid RRDTool cmd
      # $cmd .= " DEF:cpu_entitl_mhz${i}=\\\"$wrkdir_managedname_host_file\\\":CPU_Alloc:AVERAGE";
      $cmd .= " DEF:utiltot_mhz${i}=\\\"$wrkdir_managedname_host_file\\\":CPU_usage:AVERAGE";

      # $cmd .= " DEF:one_core_hz${i}=\\\"$wrkdir_managedname_host_file\\\":host_hz:AVERAGE";
      #    $cmd .= " COMMENT:\"   Average                   cores      Ghz (right axis)\\n\"";
      #    if ($in_cores) left_curly
      # $cmd .= " CDEF:utiltotu${i}=utiltot_mhz${i},one_core_hz${i},/,1000000,*";
      # $cmd .= " CDEF:ncpu${i}=cpu_entitl_mhz${i},one_core_hz${i},/,1000000,*";
      # $cmd .= " CDEF:cpu_entitl_ghz${i}=cpu_entitl_mhz${i},1000,/";
      # $cmd .= " CDEF:utiltot_ghz${i}=utiltot_mhz${i},1000,/";
      $cmd .= " CDEF:utiltot_ghz${i}=utiltot_mhz${i},1000,/";
      $cmd .= " CDEF:utiltot${i}=utiltot_mhz${i},1000,/";       # since 4.74- (u)

      # next line not necessary when you start with AREA
      #  $cmd .= " CDEF:utiltot${i}=utiltotu${i},UN,0,utiltotu${i},IF";
      $cmd .= " $gtype:utiltot${i}$color[$found]:\\\"$lpar_space\\\"";

      push @color_save, $lpar_orig;

      $cmd .= " PRINT:utiltot${i}:AVERAGE:\"%3.2lf $delimiter multihmclpar $delimiter $server_name $delimiter $lpar_space_proc\"";
      $cmd .= " PRINT:utiltot${i}:AVERAGE:\\\"%3.2lf $delimiter $color[$found] $delimiter $wrkdir_managedname_host_file_legend\\\"";
      $cmd .= " PRINT:utiltot${i}:MAX:\" %3.2lf $delimiter\"";

      $cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\\"";
      $cmd .= " GPRINT:utiltot${i}:MAX:\\\" %3.2lf \\l\\\"";

      # put carriage return after each second lpar in the legend
      if ( $j == 1 ) {
        $j = 0;
      }
      else {
        #$cmd .= " GPRINT:utiltot${i}:AVERAGE:\\\"%3.2lf \\t\\\"";
        # --> it does not work ideally with newer RRDTOOL (1.2.30 --> it needs to be separated by cariage return here)
        $j++;
      }
      $gtype = "STACK";
      $i++;
    } ## end foreach $file (@files)

    # write colors into a file for detail-graph-cgi.pl
    if ( ( "$type" =~ "y" ) && ( !defined $et_type ) ) {    # not for resourcepool
      open( FHC, "> $wrkdir/$managedname/$host/lpars.col" ) || error( "file does not exists : $wrkdir/$managedname/$host/lpars.col " . __FILE__ . ":" . __LINE__ ) && return 0;
      my $num_col_lpar = 0;
      foreach my $line_cs (@color_save) {
        chomp($line_cs);                                    # it must be there, somehow appear there \n ...
        if ( $line_cs eq '' ) {
          next;
        }
        my $num_col_lpar_write = $num_col_lpar % $color_max;
        print FHC "$num_col_lpar_write:$line_cs\n";
        $num_col_lpar++;
      } ## end foreach my $line_cs (@color_save)
      close(FHC);
    } ## end if ( ( "$type" =~ "y" ...))

    # colours

  } ## end foreach my $line (@managednamelist)
  if ( "$type" =~ "d" ) {
    if ( $j == 1 ) {
      $cmd .= " COMMENT:\\\" \\l\\\"";
    }

    # not for vmware $cmd .= " COMMENT:\\\"\(Note that for CPU dedicated LPARs is always shown their whole entitlement\)\\\"";
  } ## end if ( "$type" =~ "d" )
  $cmd .= " HRULE:0#000000";

  # $cmd .= " VRULE:0#000000";  --> it is causing sigsegv on linuxeS
  $cmd =~ s/\\"/"/g;

  #  my  $FH;
  my $tmp_file = "$tmpdir/multi-hmc-lpar-$host-$type.cmd";

  if ( defined $cluster && $cluster ne "" ) {
    $tmp_file = "$tmpdir/multi-hmc-lpar-$cl_managedname-$cl_host-$type.cmd";
  }

  if ( ( defined $et_type ) && ( $et_type eq $et_ResourcePool ) ) {
    $tmp_file = "$tmpdir/multi-hmc-lpar-$cl_managedname-$rp_moref-$type.cmd";
  }

  # print "multihmclpar CMD is in file $tmp_file\n" if $DEBUG;

  open( FH, "> $tmp_file" ) || error( " Can't open $tmp_file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
  print FH "$cmd\n";
  close(FH);

  return 1;    # do not execute the cmd itself

  # execute rrdtool, it is not possible to use RRDp Perl due to the syntax issues therefore use not nice direct rrdtool way
  my $ret = `$rrdtool - < "$tmp_file" 2>&1`;

  #my $ret  = `echo  "$cmd" | $rrdtool - 2>&1`;
  if ( $ret =~ "ERROR" ) {
    error("$host : Multi graph rrdtool error : $ret");
    if ( $ret =~ "is not an RRD file" ) {
      ( my $err, my $file, my $txt ) = split( /'/, $ret );
      error("Removing as it seems to be corrupted: $file");
      unlink("$file") || error( "Cannot rm $file : $!" . __FILE__ . ":" . __LINE__ ) && return 0;
    }
    else {
      error("$host: $cmd : Multi graph rrdtool error : $ret");
    }
  } ## end if ( $ret =~ "ERROR" )

  #unlink ("$tmp_file"); --> do not remove!!! used for details

  return 0;
} ## end sub multiview_hmc

sub human_vmware_name {
  my $lpar         = shift;
  my $arrow        = shift;
  my $local_server = shift;
  my $local_host   = shift;

  #  if (!$vmware) { return "$lpar" };  # only for vmware

  # read file and find human lpar name from uuid or
  # if 'neg' then find uuid from name

  if ( !defined($local_server) || "$local_server" eq "" ) {
    $local_server = $server;
  }
  if ( !defined($local_host) || "$local_host" eq "" ) {
    $local_host = $host;
  }

  # new system
  my $lpar_trans_file = "$wrkdir/$all_hyperv_VMs/$all_vm_uuid_names";

  if ( !-f $lpar_trans_file ) {
    $lpar_trans_file = "$wrkdir/$local_server/$local_host/lpar_trans.txt";
  }

  if ( -f "$lpar_trans_file" ) {
    open( FR, "< $lpar_trans_file" ) || error( " Can't open $lpar_trans_file : $!" . __FILE__ . ":" . __LINE__ ) && return "$lpar";
    foreach my $linep (<FR>) {
      chomp($linep);
      ( my $id, my $name, undef ) = split( /,/, $linep );
      if ( defined($arrow) && "$arrow" eq "neg" ) {
        ( $name, $id, undef ) = split( /,/, $linep );
      }
      if ( "$id" eq "$lpar" ) {
        $lpar = "$name";
        last;
      }
    } ## end foreach my $linep (<FR>)
    close(FR);
  } ## end if ( -f "$lpar_trans_file")
  return "$lpar";    #human name - if found, or original
} ## end sub human_vmware_name

sub hyperv_load
{
my $file_act = shift;
my $file_last_but_one = shift;

# analyse of hyperv results actual and last_but_one file
# - filling global %hyp_data structure with all info from actual file
# - filling global %hyp_data_one structure with all info from last but one file

  # load actual data
  hyp_load_file($file_act, \%hyp_data);
#  print Dumper ("--------------6049----------", \%hyp_data );

  # load last but one data
  hyp_load_file($file_last_but_one, \%hyp_data_one);
#  print Dumper ("--------------6053----------", \%hyp_data_one );

}

sub hyp_load_file
{
  my $filename = shift;
  my $ref_data = shift;

  my $tool_time       = "";    # this tool time updated before every wmic cmd
  my $hyperv_time     = "";    # from hyperv server
  my $tool_start_time = 0;     # this tool time when hyperv_time is read
  my $delta_time      = "";    # difference = tool_time - $tool_start_time

  my $num_of_keys  = 0;
  my $num_of_lines = 0;
  my $num_of_comps = 0;
  my $hyperv_comp  = "";
  my $hyperv_comp_wmi = "";

  my $index      = 0;
  my $time_index = "";
  my $class;
  my $global_info_is_read = 1;
  my $local_info_is_read  = 1;

  open FH, "$filename" or error( "can't open $filename: $!" . __FILE__ . ":" . __LINE__ ) && return;

  while ( my $line = <FH> ) {
    chomp $line;
    $line =~ s/\r$//; # there is one at the end of file
#    print "6074 \$line $line\n";
    next if $line eq "";
    if ( length($line) > $max_line_length ) {
      error( "avoid line longer than limit $max_line_length (see 1st 1000 chars) : ", substr( $line, 0, 1000 ) );
      next;
    }
    if ( $line =~ /^GLOBAL INFO BEGIN/) {
      $global_info_is_read = 0;
      next;
    }
    if ( $line =~ /^GLOBAL INFO END/) {
      $global_info_is_read = 1;
      next;
    }
    if ( ! $global_info_is_read) {
      push @global_info, $line;
      next;
    }
    if ( $line =~ /^LOCAL INFO BEGIN/) {
      $local_info_is_read = 0;
      next;
    }
    if ( $line =~ /^LOCAL INFO END/) {
      $local_info_is_read = 1;
      next;
    }
    if ( ! $local_info_is_read) {
      push @local_info, $line;
      next;
    }

    if ( $line =~ /^date / ) {
      ( undef, my $r_time ) = split( " Unix ", $line );
      chomp $r_time;
      $tool_time = $r_time if $r_time =~ /\d\d\d\d\d\d\d\d\d\d/;
      next;
    }

    if ( $line =~ /^Get-wmiobject / ) {
      if ( $line =~ /Win32_OperatingSystem/ ) { # test if computer is in domain
        $hyperv_comp_wmi = "not in domain";
        if ( $line =~ /-cn CN=/ ) {
          (undef, $hyperv_comp_wmi) = split ("-cn CN=", $line);
          ($hyperv_comp_wmi, undef) = split (",", $hyperv_comp_wmi);
          print_hyperv_debug("6353 \$hyperv_comp_wmi $hyperv_comp_wmi\n") if $DEBUG == 2;
        }
      }
      next;
    }

    if ($hyperv_comp_wmi eq "skip_this_comp") { # this is set if cluster
      next;
    }

    if ($line =~ /^CLASS: /) {
      ( undef, $class ) = split( "CLASS: ", $line );
      $index = 0;
      next if $class eq "";
      chomp $class;
      next if $class !~ "Win32_OperatingSystem";

       # here $class =~ "Win32_OperatingSystem"
      my $hyp_line_names  = <FH> ;
      my $hyp_line_values = <FH> ;
       # e.g.:
       # "PSComputerName","__GENUS","__CLASS","__SUPERCLASS","__DYNASTY","__RELPATH","__PROPERTY_COUNT","__DERIVATION","__SERVER","__NAMESPACE","__PATH","Caption","CSDVersion","CSName","LocalDateTime","Name","OtherTypeDescription","Version"
       #,"2","Win32_OperatingSystem",,,"Win32_OperatingSystem=@","7","System.String[]",,,,"Microsoft Windows Server 2016 Standard",,"HYPERV","20180412153005.744000+120","Microsoft Windows Server 2016 Standard|C:\Windows|\Device\Harddisk0\Partition4",,"10.0.14393"
       ## later on: char "|" is used as delimiter of atoms so it is necessary:
       ## "|" replace with "!" it looks very similar :)
       ## "," or ", or ," replace with | or | or | taking care because "," can be used inside of atom
       ## and then of course remove the " in the beginning and in the end of data line
      chomp $hyp_line_names;
      $hyp_line_names =~ s/\r$//; # there is one at the end of file, or you get debug files from upload
      chomp $hyp_line_values;
      $hyp_line_values =~ s/\r$//;
#      print "6113 \$hyp_line_names $hyp_line_names\n";
#      print "6114 \$hyp_line_values $hyp_line_values\n";
      next if $hyp_line_names eq "";
      next if $hyp_line_names =~ /ERROR/;

      $tool_start_time = $tool_time; # set in Win32_OperatingSystem as this is beginning of computer data

       # this  $class =~ "Win32_OperatingSystem" starts new computer, $hyperv_comp & $hyperv_time vars must be set
       # retrieve hyperv server time immediately and save as last item with every line
       # retrieve computer name and test against name in Get-wmiobject line, this is different for Cluster
      if ( $hyp_line_names =~ /LocalDateTime/ && $hyp_line_names =~ /CSName/) {
        my @os_arr = split( /,/, $hyp_line_names );
        ($time_index)  = grep { $os_arr[$_] eq "\"LocalDateTime\"" } 0 .. $#os_arr;
        ($hyperv_comp) = grep { $os_arr[$_] eq "\"CSName\"" } 0 .. $#os_arr;
#        print "6112 \$time_index $time_index \$hyperv_comp index $hyperv_comp\n";
        @os_arr = split( /,/, $hyp_line_values );
        $hyperv_comp = $os_arr[$hyperv_comp];
        $hyperv_time = $os_arr[$time_index];
        print_hyperv_debug("6395 \$hyperv_time $hyperv_time comp name $hyperv_comp $hyperv_comp_wmi \$class ,$class,\n") if $DEBUG == 2;
        if (($hyperv_comp_wmi ne "not in domain") && ($hyperv_comp ne "\"$hyperv_comp_wmi\"")) {
          # skipped computers can be clusters
          $hyperv_comp =~ s/\"//g;
          $skipped_computers{$hyperv_comp} = $hyperv_comp_wmi;
          $hyperv_comp_wmi = "skip_this_comp";
          next;
        }

         # write $class info to hash
        $index = 0;
        $delta_time = $tool_time - $tool_start_time;
        # comma_to_pipe (\$hyp_line_names);
        $ref_data->{$hyperv_comp}{$class}[ $index++ ] = $hyp_line_names . ",Hyperv_UTC";
        $num_of_keys++;
#        print "6138 ".substr( $hyperv_time, 1, 8 ) . "T" . substr( $hyperv_time, 9, 6 )."\n";
        $hyperv_time = str2time( substr( $hyperv_time, 1, 8 ) . "T" . substr( $hyperv_time, 9, 6 ) );
#        print "6139 \$hyperv_time $hyperv_time\n";
        # my $hyperv_time_to_save = $hyperv_time + $delta_time;
        # w10 sometimes gives time in diff zones "20180918095509.693000+120" or "20180918094009.309000+060", use direct unix time sent from windows
        my $hyperv_time_to_save = $tool_time;
        $hyp_line_values .= ",$hyperv_time_to_save";
        # comma_to_pipe (\$hyp_line_values);
        $ref_data->{$hyperv_comp}{$class}[ $index++ ] = $hyp_line_values;
        # print "6593 \$hyperv_time_to_save $hyperv_time_to_save \$tool_time $tool_time\n";
        $num_of_lines++;
        next;
      }
      else {
        print "can not find LocalDateTime & CSName in Win32_OperatingSystem <$hyp_line_names>\n";
        $hyperv_comp  = "";
        $index        = 0;
        $time_index   = "";
        next;
      }
    } ## end ($line =~ /^CLASS: /)
    $delta_time = $tool_time - $tool_start_time;
    #    print "6157 \$tool_time $tool_time \$tool_start_time $tool_start_time\n";
    if ( !defined $hyperv_time ) {
      $hyperv_time = 0;
      print "not defined hyperv_time in CLASS $class \$line $line\n";
      next;
    }
    if ( $index == 0 ) {
      next if $hyperv_comp eq "" || $class eq "" ;
      $line .= ",Hyperv_UTC";
      # comma_to_pipe (\$line);
      print_hyperv_debug("6716 \$hyperv_time $hyperv_time comp name $hyperv_comp $hyperv_comp_wmi \$class ,$class,\n") if $DEBUG == 2;
      $ref_data->{$hyperv_comp}{$class}[ $index++ ] = $line;
      $num_of_keys++;
    }
    else {
      #my $hyperv_time_to_save = $hyperv_time + $delta_time;
      # w10 sometimes gives time in diff zones "20180918095509.693000+120" or "20180918094009.309000+060", use direct unix time sent from windows
      my $hyperv_time_to_save = $tool_time;
      # print "6620 \$hyperv_time_to_save $hyperv_time_to_save \$tool_time $tool_time\n";
      $line .= ",$hyperv_time_to_save";
      # comma_to_pipe (\$line);
      $line =~ s/,(?!(?:[^"]*"[^"]*")*[^"]*$)/\./g; # in case "," is inside of atom e.g. "Windows(R) Operating System, RETAIL channel","1","8HVX7"
                                                    # see demo https://regex101.com/r/t5Euq1/1
      $ref_data->{$hyperv_comp}{$class}[ $index++ ] = $line;
      $num_of_lines++;
    }
  } ## end while ( my $line = <FH> )
  close FH;

  #print Dumper ( \%hyp_data );
  print "Hyper_v data   : hash has keys: $num_of_keys lines: $num_of_lines from file $filename\n";
}

sub comma_to_pipe
{
  my $ref_str = shift;
  $$ref_str =~ s/\|/!/g;
  $$ref_str =~ s/\"\,\"/\|/g;
  $$ref_str =~ s/\"\,/\|/g;
  $$ref_str =~ s/\,\"/\|/g;
  $$ref_str =~ s/^\"//;
  $$ref_str =~ s/\$\"//;
  $$ref_str =~ s/\|\,/\|\|/g;
  return
}

sub print_hyperv_debug {
  my $text = shift;
  print "$text";
} ## end sub print_hyperv_debug

